library slu_defs.async;

import 'dart:async';
import 'dart:typed_data';
import 'package:dispatched_worker/dispatched_worker.dart';

import 'slu_defs.dart'
    show ColPerm, Trans, colPermMap, transMap, errorMap, SuperLUException;

String _command(bool complex) => complex ? 'zlinsol' : 'dlinsol';

class SuperLU {
  final DispatchedWorker _worker;
  final bool _complex;

  SuperLU(this._complex, String scriptUrl)
      : _worker = new DispatchedWorker(scriptUrl);

  Future<AsyncFactors> factor(
      Float64List values, Int32List rowind, Int32List colptr,
      {ColPerm permc_spec: ColPerm.COLAMD}) {
    var completer = new Completer<AsyncFactors>();
    _worker
        .send([
      '${_command(_complex)}.factor',
      values,
      rowind,
      colptr,
      colPermMap[permc_spec]
    ])
        .then((pp_factors) {
      completer
          .complete(new AsyncFactors._internal(_worker, pp_factors, _complex));
    }).catchError((err) {
      if (errorMap.containsKey(err)) {
        throw new ArgumentError(errorMap[err]);
      } else {
        throw new SuperLUException(err);
      }
    });
    return completer.future;
  }
}

class AsyncFactors {
  final DispatchedWorker _worker;
  final num _pp_factors;
  final String _cmd;

  AsyncFactors._internal(this._worker, this._pp_factors, bool complex)
      : _cmd = _command(complex);

  /// [ldb] is the leading dimension of [rhs] (typically = [n]).
  Future<Float64List> solve(int n, int nrhs, Float64List rhs,
      {int ldb, Trans trans: Trans.NOTRANS}) {
    var completer = new Completer<Float64List>();
    _worker
        .send(['$_cmd.solve', _pp_factors, n, nrhs, rhs, ldb, transMap[trans]])
        .then((x) {
      completer.complete(x);
    }).catchError((err) {
      if (errorMap.containsKey(err)) {
        throw new ArgumentError(errorMap[err]);
      } else {
        throw new SuperLUException(err);
      }
    });
    return completer.future;
  }

  Future free() {
    var completer = new Completer();
    _worker.send(['$_cmd.free', _pp_factors]).then((_) {
      completer.complete();
    }).catchError((err) {
      if (errorMap.containsKey(err)) {
        throw new ArgumentError(errorMap[err]);
      } else {
        throw new SuperLUException(err);
      }
    });
    return completer.future;
  }
}
