library slu.test;

import 'dart:typed_data';

import 'package:test/test.dart';

import 'package:superlu/slu_ddefs.dart' as ddefs;
import 'package:superlu/slu_zdefs.dart' as zdefs;
import 'package:superlu/async/slu_ddefs.dart' as addefs;
import 'package:superlu/async/slu_zdefs.dart' as azdefs;

sluTest() {
  group('superlu', () {
    var n, x;
    Float64List a, rhs, a_z, rhs_z;
    Int32List asub, xa;
    const delta = 1e-6;

    setUp(() {
      n = 5;
      var _a = [2.0, 3.0, 3.0, -1.0, 4.0, 4.0, -3.0, 1.0, 2.0, 2.0, 6.0, 1.0];
      var _asub = [0, 1, 0, 2, 4, 1, 2, 3, 4, 2, 1, 4];
      var _xa = [0, 2, 5, 9, 10, 12];
      var _rhs = [8.0, 45.0, -3.0, 3.0, 19.0];
      x = [1, 2, 3, 4, 5];

      a = new Float64List.fromList(_a);
      a_z = new Float64List.fromList(_complex(_a));
      asub = new Int32List.fromList(_asub);
      xa = new Int32List.fromList(_xa);
      rhs = new Float64List.fromList(_rhs);
      rhs_z = new Float64List.fromList(_complex(_rhs));
    });

    group('sync', () {
      test('double', () {
        var factors =
            new ddefs.Factors(a, asub, xa, permc_spec: ddefs.ColPerm.NATURAL);
        factors.solve(n, 1, rhs);

        for (var i = 0; i < n; i++) {
          expect(rhs[i], closeTo(x[i], 1e-9));
        }

        factors.free();
      });

      test('complex', () {
        var factors =
            new zdefs.Factors(a_z, asub, xa, permc_spec: zdefs.ColPerm.NATURAL);
        factors.solve(2 * n, 1, rhs_z);

        for (var i = 0; i < n; i++) {
          expect(rhs_z[2 * i], closeTo(x[i], delta));
        }

        factors.free();
      });
    });

    group('async', () {
      test('double', () async {
        var superlu = new addefs.SuperLU();
        var factors = await superlu.factor(a, asub, xa,
            permc_spec: addefs.ColPerm.NATURAL);
        rhs = await factors.solve(n, 1, rhs);

        for (var i = 0; i < n; i++) {
          expect(rhs[i], closeTo(x[i], 1e-9));
        }

        await factors.free();
      });

      test('complex', () async {
        var superlu = new azdefs.SuperLU();
        var factors = await superlu.factor(a_z, asub, xa,
            permc_spec: azdefs.ColPerm.NATURAL);
        rhs_z = await factors.solve(2 * n, 1, rhs_z);

        for (var i = 0; i < n; i++) {
          expect(rhs_z[2 * i], closeTo(x[i], delta));
        }

        await factors.free();
      });
    });
  });
}

/// Inserts a zero valued imaginary part after each element.
List<num> _complex(List<num> l) {
  var values = new List.from(l);
  for (int i = values.length; i > 0; i--) {
    values.insert(i, 0.0);
  }
  return values;
}

main() {
  sluTest();
}
