var SuperLU = {};


// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof SuperLU !== 'undefined' ? SuperLU : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_WEB = typeof window === 'object';
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function' && !ENVIRONMENT_IS_WEB;
// Three configurations we can be running in:
// 1) We could be the application main() thread running in the main JS UI thread. (ENVIRONMENT_IS_WORKER == false and ENVIRONMENT_IS_PTHREAD == false)
// 2) We could be the application main() thread proxied to worker. (with Emscripten -s PROXY_TO_WORKER=1) (ENVIRONMENT_IS_WORKER == true, ENVIRONMENT_IS_PTHREAD == false)
// 3) We could be an application pthread running in a worker. (ENVIRONMENT_IS_WORKER == true and ENVIRONMENT_IS_PTHREAD == true)
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) { return Module['read'](filename, true) };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  if (!Module['thisProgram']) {
    if (process['argv'].length > 1) {
      Module['thisProgram'] = process['argv'][1].replace(/\\/g, '/');
    } else {
      Module['thisProgram'] = 'unknown-program';
    }
  }

  Module['arguments'] = process['argv'].slice(2);

  if (typeof module !== 'undefined') {
    module['exports'] = Module;
  }

  process['on']('uncaughtException', function(ex) {
    // suppress ExitStatus exceptions from showing an error
    if (!(ex instanceof ExitStatus)) {
      throw ex;
    }
  });

  Module['inspect'] = function () { return '[Emscripten Module object]'; };
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    if (typeof readbuffer === 'function') {
      return new Uint8Array(readbuffer(f));
    }
    var data = read(f, 'binary');
    assert(typeof data === 'object');
    return data;
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WORKER) {
    Module['load'] = importScripts;
  }

  if (typeof Module['setWindowTitle'] === 'undefined') {
    Module['setWindowTitle'] = function(title) { document.title = title };
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
if (!Module['thisProgram']) {
  Module['thisProgram'] = './this.program';
}

// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Preamble library stuff ===

// Documentation for the public APIs defined in this file must be updated in: 
//    site/source/docs/api_reference/preamble.js.rst
// A prebuilt local version of the documentation is available at: 
//    site/build/text/docs/api_reference/preamble.js.txt
// You can also build docs locally as HTML or other formats in site/
// An online HTML version (which may be of a different version of Emscripten)
//    is up at http://kripken.github.io/emscripten-site/docs/api_reference/preamble.js.html

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  setTempRet0: function (value) {
    tempRet0 = value;
  },
  getTempRet0: function () {
    return tempRet0;
  },
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  STACK_ALIGN: 16,
  prepVararg: function (ptr, type) {
    if (type === 'double' || type === 'i64') {
      // move so the load is aligned
      if (ptr & 7) {
        assert((ptr & 7) === 4);
        ptr += 4;
      }
    } else {
      assert((ptr & 3) === 0);
    }
    return ptr;
  },
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      assert(args.length == sig.length-1);
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      assert(sig.length == 1);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[sig]) {
      Runtime.funcWrappers[sig] = {};
    }
    var sigCache = Runtime.funcWrappers[sig];
    if (!sigCache[func]) {
      sigCache[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return sigCache[func];
  },
  getCompilerSetting: function (name) {
    throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+15)&-16);(assert((((STACKTOP|0) < (STACK_MAX|0))|0))|0); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + (assert(!staticSealed),size))|0;STATICTOP = (((STATICTOP)+15)&-16); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + (assert(DYNAMICTOP > 0),size))|0;DYNAMICTOP = (((DYNAMICTOP)+15)&-16); if (DYNAMICTOP >= TOTAL_MEMORY) { var success = enlargeMemory(); if (!success) { DYNAMICTOP = ret; return 0; } }; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 16))*(quantum ? quantum : 16); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}


Module['Runtime'] = Runtime;



//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  var func = Module['_' + ident]; // closure exported function
  if (!func) {
    try {
      func = eval('_' + ident); // explicit lookup
    } catch(e) {}
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

var cwrap, ccall;
(function(){
  var JSfuncs = {
    // Helpers for cwrap -- it can't refer to Runtime directly because it might
    // be renamed by closure, instead it calls JSfuncs['stackSave'].body to find
    // out what the minified function name is.
    'stackSave': function() {
      Runtime.stackSave()
    },
    'stackRestore': function() {
      Runtime.stackRestore()
    },
    // type conversion from js to c
    'arrayToC' : function(arr) {
      var ret = Runtime.stackAlloc(arr.length);
      writeArrayToMemory(arr, ret);
      return ret;
    },
    'stringToC' : function(str) {
      var ret = 0;
      if (str !== null && str !== undefined && str !== 0) { // null string
        // at most 4 bytes per UTF-8 code point, +1 for the trailing '\0'
        ret = Runtime.stackAlloc((str.length << 2) + 1);
        writeStringToMemory(str, ret);
      }
      return ret;
    }
  };
  // For fast lookup of conversion functions
  var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

  // C calling interface. 
  ccall = function ccallFunc(ident, returnType, argTypes, args, opts) {
    var func = getCFunc(ident);
    var cArgs = [];
    var stack = 0;
    assert(returnType !== 'array', 'Return type should not be "array".');
    if (args) {
      for (var i = 0; i < args.length; i++) {
        var converter = toC[argTypes[i]];
        if (converter) {
          if (stack === 0) stack = Runtime.stackSave();
          cArgs[i] = converter(args[i]);
        } else {
          cArgs[i] = args[i];
        }
      }
    }
    var ret = func.apply(null, cArgs);
    if ((!opts || !opts.async) && typeof EmterpreterAsync === 'object') {
      assert(!EmterpreterAsync.state, 'cannot start async op with normal JS calling ccall');
    }
    if (opts && opts.async) assert(!returnType, 'async ccalls cannot return values');
    if (returnType === 'string') ret = Pointer_stringify(ret);
    if (stack !== 0) {
      if (opts && opts.async) {
        EmterpreterAsync.asyncFinalizers.push(function() {
          Runtime.stackRestore(stack);
        });
        return;
      }
      Runtime.stackRestore(stack);
    }
    return ret;
  }

  var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
  function parseJSFunc(jsfunc) {
    // Match the body and the return value of a javascript function source
    var parsed = jsfunc.toString().match(sourceRegex).slice(1);
    return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
  }
  var JSsource = {};
  for (var fun in JSfuncs) {
    if (JSfuncs.hasOwnProperty(fun)) {
      // Elements of toCsource are arrays of three items:
      // the code, and the return value
      JSsource[fun] = parseJSFunc(JSfuncs[fun]);
    }
  }

  
  cwrap = function cwrap(ident, returnType, argTypes) {
    argTypes = argTypes || [];
    var cfunc = getCFunc(ident);
    // When the function takes numbers and returns a number, we can just return
    // the original function
    var numericArgs = argTypes.every(function(type){ return type === 'number'});
    var numericRet = (returnType !== 'string');
    if ( numericRet && numericArgs) {
      return cfunc;
    }
    // Creation of the arguments list (["$1","$2",...,"$nargs"])
    var argNames = argTypes.map(function(x,i){return '$'+i});
    var funcstr = "(function(" + argNames.join(',') + ") {";
    var nargs = argTypes.length;
    if (!numericArgs) {
      // Generate the code needed to convert the arguments from javascript
      // values to pointers
      funcstr += 'var stack = ' + JSsource['stackSave'].body + ';';
      for (var i = 0; i < nargs; i++) {
        var arg = argNames[i], type = argTypes[i];
        if (type === 'number') continue;
        var convertCode = JSsource[type + 'ToC']; // [code, return]
        funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
        funcstr += convertCode.body + ';';
        funcstr += arg + '=' + convertCode.returnValue + ';';
      }
    }

    // When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
    var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
    // Call the function
    funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
    if (!numericRet) { // Return type can only by 'string' or 'number'
      // Convert the result to a string
      var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
      funcstr += 'ret = ' + strgfy + '(ret);';
    }
    funcstr += "if (typeof EmterpreterAsync === 'object') { assert(!EmterpreterAsync.state, 'cannot start async op with normal JS calling cwrap') }";
    if (!numericArgs) {
      // If we had a stack, restore it
      funcstr += JSsource['stackRestore'].body.replace('()', '(stack)') + ';';
    }
    funcstr += 'return ret})';
    return eval(funcstr);
  };
})();
Module["cwrap"] = cwrap;
Module["ccall"] = ccall;


function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[((ptr)>>0)]=value; break;
      case 'i8': HEAP8[((ptr)>>0)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module['setValue'] = setValue;


function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[((ptr)>>0)];
      case 'i8': return HEAP8[((ptr)>>0)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module['getValue'] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module['ALLOC_NORMAL'] = ALLOC_NORMAL;
Module['ALLOC_STACK'] = ALLOC_STACK;
Module['ALLOC_STATIC'] = ALLOC_STATIC;
Module['ALLOC_DYNAMIC'] = ALLOC_DYNAMIC;
Module['ALLOC_NONE'] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)>>0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }
    assert(type, 'Must know what type to store in allocate!');

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module['allocate'] = allocate;

// Allocate memory during any stage of startup - static memory early on, dynamic memory later, malloc when ready
function getMemory(size) {
  if (!staticSealed) return Runtime.staticAlloc(size);
  if ((typeof _sbrk !== 'undefined' && !_sbrk.called) || !runtimeInitialized) return Runtime.dynamicAlloc(size);
  return _malloc(size);
}
Module['getMemory'] = getMemory;

function Pointer_stringify(ptr, /* optional */ length) {
  if (length === 0 || !ptr) return '';
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = 0;
  var t;
  var i = 0;
  while (1) {
    assert(ptr + i < TOTAL_MEMORY);
    t = HEAPU8[(((ptr)+(i))>>0)];
    hasUtf |= t;
    if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (hasUtf < 128) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }
  return Module['UTF8ToString'](ptr);
}
Module['Pointer_stringify'] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated ASCII-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function AsciiToString(ptr) {
  var str = '';
  while (1) {
    var ch = HEAP8[((ptr++)>>0)];
    if (!ch) return str;
    str += String.fromCharCode(ch);
  }
}
Module['AsciiToString'] = AsciiToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in ASCII form. The copy will require at most str.length+1 bytes of space in the HEAP.

function stringToAscii(str, outPtr) {
  return writeAsciiToMemory(str, outPtr, false);
}
Module['stringToAscii'] = stringToAscii;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the given array that contains uint8 values, returns
// a copy of that string as a Javascript String object.

function UTF8ArrayToString(u8Array, idx) {
  var u0, u1, u2, u3, u4, u5;

  var str = '';
  while (1) {
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    u0 = u8Array[idx++];
    if (!u0) return str;
    if (!(u0 & 0x80)) { str += String.fromCharCode(u0); continue; }
    u1 = u8Array[idx++] & 63;
    if ((u0 & 0xE0) == 0xC0) { str += String.fromCharCode(((u0 & 31) << 6) | u1); continue; }
    u2 = u8Array[idx++] & 63;
    if ((u0 & 0xF0) == 0xE0) {
      u0 = ((u0 & 15) << 12) | (u1 << 6) | u2;
    } else {
      u3 = u8Array[idx++] & 63;
      if ((u0 & 0xF8) == 0xF0) {
        u0 = ((u0 & 7) << 18) | (u1 << 12) | (u2 << 6) | u3;
      } else {
        u4 = u8Array[idx++] & 63;
        if ((u0 & 0xFC) == 0xF8) {
          u0 = ((u0 & 3) << 24) | (u1 << 18) | (u2 << 12) | (u3 << 6) | u4;
        } else {
          u5 = u8Array[idx++] & 63;
          u0 = ((u0 & 1) << 30) | (u1 << 24) | (u2 << 18) | (u3 << 12) | (u4 << 6) | u5;
        }
      }
    }
    if (u0 < 0x10000) {
      str += String.fromCharCode(u0);
    } else {
      var ch = u0 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    }
  }
}
Module['UTF8ArrayToString'] = UTF8ArrayToString;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF8ToString(ptr) {
  return UTF8ArrayToString(HEAPU8, ptr);
}
Module['UTF8ToString'] = UTF8ToString;

// Copies the given Javascript String object 'str' to the given byte array at address 'outIdx',
// encoded in UTF8 form and null-terminated. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outU8Array: the array to copy to. Each index in this array is assumed to be one 8-byte element.
//   outIdx: The starting offset in the array to begin the copying.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=1, only the null terminator will be written and nothing else.
//                    maxBytesToWrite=0 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8Array(str, outU8Array, outIdx, maxBytesToWrite) {
  if (!(maxBytesToWrite > 0)) // Parameter maxBytesToWrite is not optional. Negative values, 0, null, undefined and false each don't write out any bytes.
    return 0;

  var startIdx = outIdx;
  var endIdx = outIdx + maxBytesToWrite - 1; // -1 for string null terminator.
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      if (outIdx >= endIdx) break;
      outU8Array[outIdx++] = u;
    } else if (u <= 0x7FF) {
      if (outIdx + 1 >= endIdx) break;
      outU8Array[outIdx++] = 0xC0 | (u >> 6);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0xFFFF) {
      if (outIdx + 2 >= endIdx) break;
      outU8Array[outIdx++] = 0xE0 | (u >> 12);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x1FFFFF) {
      if (outIdx + 3 >= endIdx) break;
      outU8Array[outIdx++] = 0xF0 | (u >> 18);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x3FFFFFF) {
      if (outIdx + 4 >= endIdx) break;
      outU8Array[outIdx++] = 0xF8 | (u >> 24);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else {
      if (outIdx + 5 >= endIdx) break;
      outU8Array[outIdx++] = 0xFC | (u >> 30);
      outU8Array[outIdx++] = 0x80 | ((u >> 24) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    }
  }
  // Null-terminate the pointer to the buffer.
  outU8Array[outIdx] = 0;
  return outIdx - startIdx;
}
Module['stringToUTF8Array'] = stringToUTF8Array;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF8 form. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8(str, outPtr, maxBytesToWrite) {
  assert(typeof maxBytesToWrite == 'number', 'stringToUTF8(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
  return stringToUTF8Array(str, HEAPU8, outPtr, maxBytesToWrite);
}
Module['stringToUTF8'] = stringToUTF8;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF8 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF8(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      ++len;
    } else if (u <= 0x7FF) {
      len += 2;
    } else if (u <= 0xFFFF) {
      len += 3;
    } else if (u <= 0x1FFFFF) {
      len += 4;
    } else if (u <= 0x3FFFFFF) {
      len += 5;
    } else {
      len += 6;
    }
  }
  return len;
}
Module['lengthBytesUTF8'] = lengthBytesUTF8;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module['UTF16ToString'] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16 form. The copy will require at most str.length*4+2 bytes of space in the HEAP.
// Use the function lengthBytesUTF16() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=2, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<2 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF16(str, outPtr, maxBytesToWrite) {
  assert(typeof maxBytesToWrite == 'number', 'stringToUTF16(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 2) return 0;
  maxBytesToWrite -= 2; // Null terminator.
  var startPtr = outPtr;
  var numCharsToWrite = (maxBytesToWrite < str.length*2) ? (maxBytesToWrite / 2) : str.length;
  for (var i = 0; i < numCharsToWrite; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[((outPtr)>>1)]=codeUnit;
    outPtr += 2;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[((outPtr)>>1)]=0;
  return outPtr - startPtr;
}
Module['stringToUTF16'] = stringToUTF16;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF16(str) {
  return str.length*2;
}
Module['lengthBytesUTF16'] = lengthBytesUTF16;

function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module['UTF32ToString'] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32 form. The copy will require at most str.length*4+4 bytes of space in the HEAP.
// Use the function lengthBytesUTF32() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=4, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<4 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF32(str, outPtr, maxBytesToWrite) {
  assert(typeof maxBytesToWrite == 'number', 'stringToUTF32(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 4) return 0;
  var startPtr = outPtr;
  var endPtr = startPtr + maxBytesToWrite - 4;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++i);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[((outPtr)>>2)]=codeUnit;
    outPtr += 4;
    if (outPtr + 4 > endPtr) break;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[((outPtr)>>2)]=0;
  return outPtr - startPtr;
}
Module['stringToUTF32'] = stringToUTF32;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF32(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i);
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) ++i; // possibly a lead surrogate, so skip over the tail surrogate.
    len += 4;
  }

  return len;
}
Module['lengthBytesUTF32'] = lengthBytesUTF32;

function demangle(func) {
  var hasLibcxxabi = !!Module['___cxa_demangle'];
  if (hasLibcxxabi) {
    try {
      var buf = _malloc(func.length);
      writeStringToMemory(func.substr(1), buf);
      var status = _malloc(4);
      var ret = Module['___cxa_demangle'](buf, 0, 0, status);
      if (getValue(status, 'i32') === 0 && ret) {
        return Pointer_stringify(ret);
      }
      // otherwise, libcxxabi failed, we can try ours which may return a partial result
    } catch(e) {
      // failure when using libcxxabi, we can try ours which may return a partial result
    } finally {
      if (buf) _free(buf);
      if (status) _free(status);
      if (ret) _free(ret);
    }
  }
  var i = 3;
  // params, etc.
  var basicTypes = {
    'v': 'void',
    'b': 'bool',
    'c': 'char',
    's': 'short',
    'i': 'int',
    'l': 'long',
    'f': 'float',
    'd': 'double',
    'w': 'wchar_t',
    'a': 'signed char',
    'h': 'unsigned char',
    't': 'unsigned short',
    'j': 'unsigned int',
    'm': 'unsigned long',
    'x': 'long long',
    'y': 'unsigned long long',
    'z': '...'
  };
  var subs = [];
  var first = true;
  function dump(x) {
    //return;
    if (x) Module.print(x);
    Module.print(func);
    var pre = '';
    for (var a = 0; a < i; a++) pre += ' ';
    Module.print (pre + '^');
  }
  function parseNested() {
    i++;
    if (func[i] === 'K') i++; // ignore const
    var parts = [];
    while (func[i] !== 'E') {
      if (func[i] === 'S') { // substitution
        i++;
        var next = func.indexOf('_', i);
        var num = func.substring(i, next) || 0;
        parts.push(subs[num] || '?');
        i = next+1;
        continue;
      }
      if (func[i] === 'C') { // constructor
        parts.push(parts[parts.length-1]);
        i += 2;
        continue;
      }
      var size = parseInt(func.substr(i));
      var pre = size.toString().length;
      if (!size || !pre) { i--; break; } // counter i++ below us
      var curr = func.substr(i + pre, size);
      parts.push(curr);
      subs.push(curr);
      i += pre + size;
    }
    i++; // skip E
    return parts;
  }
  function parse(rawList, limit, allowVoid) { // main parser
    limit = limit || Infinity;
    var ret = '', list = [];
    function flushList() {
      return '(' + list.join(', ') + ')';
    }
    var name;
    if (func[i] === 'N') {
      // namespaced N-E
      name = parseNested().join('::');
      limit--;
      if (limit === 0) return rawList ? [name] : name;
    } else {
      // not namespaced
      if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
      var size = parseInt(func.substr(i));
      if (size) {
        var pre = size.toString().length;
        name = func.substr(i + pre, size);
        i += pre + size;
      }
    }
    first = false;
    if (func[i] === 'I') {
      i++;
      var iList = parse(true);
      var iRet = parse(true, 1, true);
      ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
    } else {
      ret = name;
    }
    paramLoop: while (i < func.length && limit-- > 0) {
      //dump('paramLoop');
      var c = func[i++];
      if (c in basicTypes) {
        list.push(basicTypes[c]);
      } else {
        switch (c) {
          case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
          case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
          case 'L': { // literal
            i++; // skip basic type
            var end = func.indexOf('E', i);
            var size = end - i;
            list.push(func.substr(i, size));
            i += size + 2; // size + 'EE'
            break;
          }
          case 'A': { // array
            var size = parseInt(func.substr(i));
            i += size.toString().length;
            if (func[i] !== '_') throw '?';
            i++; // skip _
            list.push(parse(true, 1, true)[0] + ' [' + size + ']');
            break;
          }
          case 'E': break paramLoop;
          default: ret += '?' + c; break paramLoop;
        }
      }
    }
    if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
    if (rawList) {
      if (ret) {
        list.push(ret + '?');
      }
      return list;
    } else {
      return ret + flushList();
    }
  }
  var parsed = func;
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    parsed = parse();
  } catch(e) {
    parsed += '?';
  }
  if (parsed.indexOf('?') >= 0 && !hasLibcxxabi) {
    Runtime.warnOnce('warning: a problem occurred in builtin C++ name demangling; build with  -s DEMANGLE_SUPPORT=1  to link in libcxxabi demangling');
  }
  return parsed;
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function jsStackTrace() {
  var err = new Error();
  if (!err.stack) {
    // IE10+ special cases: It does have callstack info, but it is only populated if an Error object is thrown,
    // so try that as a special-case.
    try {
      throw new Error(0);
    } catch(e) {
      err = e;
    }
    if (!err.stack) {
      return '(no stack trace available)';
    }
  }
  return err.stack.toString();
}

function stackTrace() {
  return demangleAll(jsStackTrace());
}
Module['stackTrace'] = stackTrace;

// Memory management

var PAGE_SIZE = 4096;

function alignMemoryPage(x) {
  if (x % 4096 > 0) {
    x += (4096 - (x % 4096));
  }
  return x;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk


function enlargeMemory() {
  abort('Cannot enlarge memory arrays. Either (1) compile with -s TOTAL_MEMORY=X with X higher than the current value ' + TOTAL_MEMORY + ', (2) compile with ALLOW_MEMORY_GROWTH which adjusts the size at runtime but prevents some optimizations, or (3) set Module.TOTAL_MEMORY before the program runs.');
}


var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;

var totalMemory = 64*1024;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  Module.printErr('increasing TOTAL_MEMORY to ' + totalMemory + ' to be compliant with the asm.js spec (and given that TOTAL_STACK=' + TOTAL_STACK + ')');
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'JS engine does not provide full typed array support');

var buffer;
buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);

// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['buffer'] = buffer;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;
var runtimeExited = false;


function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  callRuntimeCallbacks(__ATEXIT__);
  runtimeExited = true;
}

function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module['addOnPreRun'] = Module.addOnPreRun = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module['addOnInit'] = Module.addOnInit = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module['addOnPreMain'] = Module.addOnPreMain = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module['addOnExit'] = Module.addOnExit = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module['addOnPostRun'] = Module.addOnPostRun = addOnPostRun;

// Tools


function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var len = length > 0 ? length : lengthBytesUTF8(stringy)+1;
  var u8array = new Array(len);
  var numBytesWritten = stringToUTF8Array(stringy, u8array, 0, u8array.length);
  if (dontAddNull) u8array.length = numBytesWritten;
  return u8array;
}
Module['intArrayFromString'] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
        assert(false, 'Character code ' + chr + ' (' + String.fromCharCode(chr) + ')  at offset ' + i + ' not in 0x00-0xFF.');
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module['intArrayToString'] = intArrayToString;

function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))>>0)]=chr;
    i = i + 1;
  }
}
Module['writeStringToMemory'] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[((buffer++)>>0)]=array[i];
  }
}
Module['writeArrayToMemory'] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; ++i) {
    assert(str.charCodeAt(i) === str.charCodeAt(i)&0xff);
    HEAP8[((buffer++)>>0)]=str.charCodeAt(i);
  }
  // Null-terminate the pointer to the HEAP.
  if (!dontAddNull) HEAP8[((buffer)>>0)]=0;
}
Module['writeAsciiToMemory'] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}


// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


if (!Math['clz32']) Math['clz32'] = function(x) {
  x = x >>> 0;
  for (var i = 0; i < 32; i++) {
    if (x & (1 << (31 - i))) return i;
  }
  return 32;
};
Math.clz32 = Math['clz32']

var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;
var Math_clz32 = Math.clz32;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled
var runDependencyTracking = {};

function getUniqueRunDependency(id) {
  var orig = id;
  while (1) {
    if (!runDependencyTracking[id]) return id;
    id = orig + Math.random();
  }
  return id;
}

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(!runDependencyTracking[id]);
    runDependencyTracking[id] = 1;
    if (runDependencyWatcher === null && typeof setInterval !== 'undefined') {
      // Check for missing dependencies every few seconds
      runDependencyWatcher = setInterval(function() {
        if (ABORT) {
          clearInterval(runDependencyWatcher);
          runDependencyWatcher = null;
          return;
        }
        var shown = false;
        for (var dep in runDependencyTracking) {
          if (!shown) {
            shown = true;
            Module.printErr('still waiting on run dependencies:');
          }
          Module.printErr('dependency: ' + dep);
        }
        if (shown) {
          Module.printErr('(end of list)');
        }
      }, 10000);
    }
  } else {
    Module.printErr('warning: run dependency added without ID');
  }
}
Module['addRunDependency'] = addRunDependency;
function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(runDependencyTracking[id]);
    delete runDependencyTracking[id];
  } else {
    Module.printErr('warning: run dependency removed without ID');
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module['removeRunDependency'] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data



var memoryInitializer = null;



// === Body ===

var ASM_CONSTS = [];




STATIC_BASE = 8;

STATICTOP = STATIC_BASE + 4688;
  /* global initializers */  __ATINIT__.push();
  

/* memory initializer */ allocate([83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,115,116,97,116,45,62,117,116,105,109,101,0,0,0,0,117,116,105,108,46,99,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,115,116,97,116,45,62,111,112,115,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,98,117,102,32,105,110,32,105,110,116,77,97,108,108,111,99,40,41,0,0,0,0,0,109,101,109,111,114,121,46,99,0,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,98,117,102,32,105,110,32,105,110,116,67,97,108,108,111,99,40,41,0,0,0,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,65,91,93,0,0,0,0,103,101,116,95,112,101,114,109,95,99,46,99,0,0,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,112,91,93,0,0,0,0,67,79,76,65,77,68,32,102,97,105,108,101,100,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,109,97,114,107,101,114,91,93,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,116,95,99,111,108,112,116,114,91,93,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,116,95,114,111,119,105,110,100,91,93,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,97,116,97,95,99,111,108,112,116,114,91,93,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,97,116,97,95,114,111,119,105,110,100,91,93,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,116,95,99,111,108,112,116,114,91,93,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,116,95,114,111,119,105,110,100,91,93,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,98,95,99,111,108,112,116,114,91,93,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,98,95,114,111,119,105,110,100,91,93,0,0,0,0,0,77,97,116,114,105,120,32,105,115,32,110,111,116,32,115,113,117,97,114,101,0,0,0,0,73,110,118,97,108,105,100,32,73,83,80,69,67,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,105,110,118,112,46,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,100,104,101,97,100,46,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,113,115,105,122,101,46,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,108,108,105,115,116,46,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,109,97,114,107,101,114,46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,98,117,102,32,105,110,32,109,120,67,97,108,108,111,99,73,110,116,40,41,0,0,0,115,112,95,99,111,108,101,116,114,101,101,46,99,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,65,67,115,116,111,114,101,0,0,0,0,0,0,0,0,115,112,95,112,114,101,111,114,100,101,114,46,99,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,65,67,115,116,111,114,101,45,62,99,111,108,98,101,103,0,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,65,67,115,116,111,114,101,45,62,99,111,108,101,110,100,0,0,0,0,0,0,0,0,115,112,95,105,101,110,118,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,105,119,111,114,107,91,93,0,0,0,0,0,0,0,0,104,101,97,112,95,114,101,108,97,120,95,115,110,111,100,101,46,99,0,0,0,0,0,0,42,42,32,79,110,32,101,110,116,114,121,32,116,111,32,37,54,115,44,32,112,97,114,97,109,101,116,101,114,32,110,117,109,98,101,114,32,37,50,100,32,104,97,100,32,97,110,32,105,108,108,101,103,97,108,32,118,97,108,117,101,10,0,0,115,112,95,100,116,114,115,118,0,0,0,0,0,0,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,119,111,114,107,32,105,110,32,115,112,95,100,116,114,115,118,40,41,46,0,0,0,0,100,115,112,95,98,108,97,115,50,46,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,103,115,116,114,115,0,0,100,103,115,116,114,115,46,99,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,109,101,109,111,114,121,46,99,0,0,0,0,0,0,0,100,76,85,87,111,114,107,73,110,105,116,58,32,109,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,108,111,99,97,108,32,105,119,111,114,107,112,116,114,91,93,10,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,101,100,32,102,111,114,32,98,117,102,32,105,110,32,100,111,117,98,108,101,77,97,108,108,111,99,40,41,10,0,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,101,100,32,102,111,114,32,98,117,102,32,105,110,32,100,111,117,98,108,101,67,97,108,108,111,99,40,41,10,0,0,0,0,0,0,0,0,100,117,116,105,108,46,99,0,100,103,115,116,114,102,40,41,32,101,114,114,111,114,32,114,101,116,117,114,110,115,32,73,78,70,79,61,32,37,100,10,0,0,0,0,0,0,0,0,100,103,115,116,114,115,40,41,32,101,114,114,111,114,32,114,101,116,117,114,110,115,32,73,78,70,79,61,32,37,100,10,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,105,111,112,116,61,37,100,32,112,97,115,115,101,100,32,116,111,32,100,108,105,110,115,111,108,40,41,10,0,0,0,0,122,95,100,105,118,46,99,58,32,100,105,118,105,115,105,111,110,32,98,121,32,122,101,114,111,10,0,0,0,0,0,0,115,112,95,122,116,114,115,118,0,0,0,0,0,0,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,119,111,114,107,32,105,110,32,115,112,95,122,116,114,115,118,40,41,46,0,0,0,0,122,115,112,95,98,108,97,115,50,46,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,122,103,115,116,114,115,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,108,111,99,97,108,32,119,111,114,107,91,93,46,0,0,122,103,115,116,114,115,46,99,0,0,0,0,0,0,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,108,111,99,97,108,32,115,111,108,110,91,93,46,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,101,120,112,97,110,100,101,114,115,0,0,0,0,0,0,122,109,101,109,111,114,121,46,99,0,0,0,0,0,0,0,78,111,116,32,101,110,111,117,103,104,32,109,101,109,111,114,121,32,116,111,32,112,101,114,102,111,114,109,32,102,97,99,116,111,114,105,122,97,116,105,111,110,46,10,0,0,0,0,122,76,85,87,111,114,107,73,110,105,116,58,32,109,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,108,111,99,97,108,32,105,119,111,114,107,112,116,114,91,93,10,0,109,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,108,111,99,97,108,32,100,119,111,114,107,112,116,114,91,93,46,0,0,0,0,0,0,67,97,110,39,116,32,101,120,112,97,110,100,32,77,101,109,84,121,112,101,32,37,100,58,32,106,99,111,108,32,37,100,10,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,101,100,32,102,111,114,32,98,117,102,32,105,110,32,100,111,117,98,108,101,99,111,109,112,108,101,120,77,97,108,108,111,99,40,41,10,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,101,100,32,102,111,114,32,98,117,102,32,105,110,32,100,111,117,98,108,101,99,111,109,112,108,101,120,67,97,108,108,111,99,40,41,10,0,37,115,32,97,116,32,108,105,110,101,32,37,100,32,105,110,32,102,105,108,101,32,37,115,10,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,65,45,62,83,116,111,114,101,0,0,0,0,0,0,0,122,117,116,105,108,46,99,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,88,45,62,83,116,111,114,101,0,0,0,0,0,0,0,83,85,80,69,82,76,85,95,77,65,76,76,79,67,32,102,97,105,108,115,32,102,111,114,32,76,45,62,83,116,111,114,101,0,0,0,0,0,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,118,97,108,117,101,115,91,93,46,10,0,0,0,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,112,101,114,109,95,114,91,93,46,10,0,0,0,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,112,101,114,109,95,99,91,93,46,10,0,0,0,0,0,77,97,108,108,111,99,32,102,97,105,108,115,32,102,111,114,32,101,116,114,101,101,91,93,46,10,0,0,0,0,0,0,122,103,115,116,114,102,40,41,32,101,114,114,111,114,32,114,101,116,117,114,110,115,32,73,78,70,79,61,32,37,100,10,0,0,0,0,0,0,0,0,76,92,85,32,77,66,32,37,46,51,102,9,116,111,116,97,108,32,77,66,32,110,101,101,100,101,100,32,37,46,51,102,10,0,0,0,0,0,0,0,122,103,115,116,114,115,40,41,32,101,114,114,111,114,32,114,101,116,117,114,110,115,32,73,78,70,79,61,32,37,100,10,0,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,105,111,112,116,61,37,100,32,112,97,115,115,101,100,32,116,111,32,122,108,105,110,115,111,108,40,41,10,0,0,0,0,0,0,0,0,0,0,0,0,68,84,82,83,86,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,85,0,0,0,0,0,0,0,76,0,0,0,0,0,0,0,78,0,0,0,0,0,0,0,84,0,0,0,0,0,0,0,67,0,0,0,0,0,0,0,90,84,82,83,86,32,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,10,0,17,17,17,0,0,0,0,5,0,0,0,0,0,0,9,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,15,10,17,17,17,3,10,7,0,1,19,9,11,11,0,0,9,6,11,0,0,11,0,6,17,0,0,0,17,17,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,10,10,17,17,17,0,10,0,0,2,0,9,11,0,0,0,9,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,4,13,0,0,0,0,9,14,0,0,0,0,0,14,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,0,15,0,0,0,0,9,16,0,0,0,0,0,16,0,0,16,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,10,0,0,0,0,9,11,0,0,0,0,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,45,43,32,32,32,48,88,48,120,0,0,0,0,0,0,0,40,110,117,108,108,41,0,0,45,48,88,43,48,88,32,48,88,45,48,120,43,48,120,32,48,120,0,0,0,0,0,0,105,110,102,0,0,0,0,0,73,78,70,0,0,0,0,0,110,97,110,0,0,0,0,0,78,65,78,0,0,0,0,0,46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);





/* no memory initializer */
var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}

// {{PRE_LIBRARY}}


  var _BDtoIHigh=true;

   
  Module["_i64Subtract"] = _i64Subtract;

   
  Module["_i64Add"] = _i64Add;

   
  Module["_memset"] = _memset;

  var _BDtoILow=true;

   
  Module["_bitshift64Lshr"] = _bitshift64Lshr;

  
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  
  var ___errno_state=0;function ___setErrNo(value) {
      // For convenient setting and returning of errno.
      HEAP32[((___errno_state)>>2)]=value;
      return value;
    }function _strerror_r(errnum, strerrbuf, buflen) {
      if (errnum in ERRNO_MESSAGES) {
        if (ERRNO_MESSAGES[errnum].length > buflen - 1) {
          return ___setErrNo(ERRNO_CODES.ERANGE);
        } else {
          var msg = ERRNO_MESSAGES[errnum];
          writeAsciiToMemory(msg, strerrbuf);
          return 0;
        }
      } else {
        return ___setErrNo(ERRNO_CODES.EINVAL);
      }
    }function _strerror(errnum) {
      if (!_strerror.buffer) _strerror.buffer = _malloc(256);
      _strerror_r(errnum, _strerror.buffer, 256);
      return _strerror.buffer;
    }

   
  Module["_bitshift64Shl"] = _bitshift64Shl;

  function _abort() {
      Module['abort']();
    }

  
  
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            return ''; // an invalid portion invalidates the whole thing
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          stream.tty.ops.flush(stream.tty);
        },flush:function (stream) {
          stream.tty.ops.flush(stream.tty);
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              // we will read data by chunks of BUFSIZE
              var BUFSIZE = 256;
              var buf = new Buffer(BUFSIZE);
              var bytesRead = 0;
  
              var fd = process.stdin.fd;
              // Linux and Mac cannot use process.stdin.fd (which isn't set up as sync)
              var usingDevice = false;
              try {
                fd = fs.openSync('/dev/stdin', 'r');
                usingDevice = true;
              } catch (e) {}
  
              bytesRead = fs.readSync(fd, buf, 0, BUFSIZE, null);
  
              if (usingDevice) { fs.closeSync(fd); }
              if (bytesRead > 0) {
                result = buf.slice(0, bytesRead).toString('utf-8');
              } else {
                result = null;
              }
  
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val); // val == 0 would cut text output off in the middle.
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val);
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }}};
  
  var MEMFS={ops_table:null,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap,
                msync: MEMFS.stream_ops.msync
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            }
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
          // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
          // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
          // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
          node.contents = null; 
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },getFileDataAsRegularArray:function (node) {
        if (node.contents && node.contents.subarray) {
          var arr = [];
          for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
          return arr; // Returns a copy of the original data.
        }
        return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
      },getFileDataAsTypedArray:function (node) {
        if (!node.contents) return new Uint8Array;
        if (node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
        return new Uint8Array(node.contents);
      },expandFileStorage:function (node, newCapacity) {
        // If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
        // instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
        // increase the size.
        if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
          node.contents = MEMFS.getFileDataAsRegularArray(node);
          node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
        }
  
        if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
          var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
          if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
          // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
          // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
          // avoid overshooting the allocation cap by a very large margin.
          var CAPACITY_DOUBLING_MAX = 1024 * 1024;
          newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
          if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
          var oldContents = node.contents;
          node.contents = new Uint8Array(newCapacity); // Allocate new storage.
          if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
          return;
        }
        // Not using a typed array to back the file storage. Use a standard JS array instead.
        if (!node.contents && newCapacity > 0) node.contents = [];
        while (node.contents.length < newCapacity) node.contents.push(0);
      },resizeFileStorage:function (node, newSize) {
        if (node.usedBytes == newSize) return;
        if (newSize == 0) {
          node.contents = null; // Fully decommit when requesting a resize to zero.
          node.usedBytes = 0;
          return;
        }
        if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
          var oldContents = node.contents;
          node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
          if (oldContents) {
            node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
          }
          node.usedBytes = newSize;
          return;
        }
        // Backing with a JS array.
        if (!node.contents) node.contents = [];
        if (node.contents.length > newSize) node.contents.length = newSize;
        else while (node.contents.length < newSize) node.contents.push(0);
        node.usedBytes = newSize;
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes) return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else {
            for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          if (!length) return 0;
          var node = stream.node;
          node.timestamp = Date.now();
  
          if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
            if (canOwn) { // Can we just reuse the buffer we are given?
              assert(position === 0, 'canOwn must imply no weird position inside the file');
              node.contents = buffer.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
              node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
              node.contents.set(buffer.subarray(offset, offset + length), position);
              return length;
            }
          }
  
          // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
          MEMFS.expandFileStorage(node, position+length);
          if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
          else {
            for (var i = 0; i < length; i++) {
             node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
            }
          }
          node.usedBytes = Math.max(node.usedBytes, position+length);
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < stream.node.usedBytes) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        },msync:function (stream, buffer, offset, length, mmapFlags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          if (mmapFlags & 2) {
            // MAP_PRIVATE calls need not to be synced back to underlying fs
            return 0;
          }
  
          var bytesWritten = MEMFS.stream_ops.write(stream, buffer, 0, length, offset, false);
          // should we check if bytesWritten and length are the same?
          return 0;
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        if (typeof indexedDB !== 'undefined') return indexedDB;
        var ret = null;
        if (typeof window === 'object') ret = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
        assert(ret, 'IDBFS used, but indexedDB not supported');
        return ret;
      },DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        // reuse all of the core MEMFS functionality
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },getDB:function (name, callback) {
        // check the cache first
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
  
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return callback(e);
        }
        req.onupgradeneeded = function(e) {
          var db = e.target.result;
          var transaction = e.target.transaction;
  
          var fileStore;
  
          if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
            fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
          } else {
            fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
          }
  
          if (!fileStore.indexNames.contains('timestamp')) {
            fileStore.createIndex('timestamp', 'timestamp', { unique: false });
          }
        };
        req.onsuccess = function() {
          db = req.result;
  
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },getLocalSet:function (mount, callback) {
        var entries = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat;
  
          try {
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
          }
  
          entries[path] = { timestamp: stat.mtime };
        }
  
        return callback(null, { type: 'local', entries: entries });
      },getRemoteSet:function (mount, callback) {
        var entries = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function(e) {
            callback(this.error);
            e.preventDefault();
          };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          var index = store.index('timestamp');
  
          index.openKeyCursor().onsuccess = function(event) {
            var cursor = event.target.result;
  
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, entries: entries });
            }
  
            entries[cursor.primaryKey] = { timestamp: cursor.key };
  
            cursor.continue();
          };
        });
      },loadLocalEntry:function (path, callback) {
        var stat, node;
  
        try {
          var lookup = FS.lookupPath(path);
          node = lookup.node;
          stat = FS.stat(path);
        } catch (e) {
          return callback(e);
        }
  
        if (FS.isDir(stat.mode)) {
          return callback(null, { timestamp: stat.mtime, mode: stat.mode });
        } else if (FS.isFile(stat.mode)) {
          // Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
          // Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
          node.contents = MEMFS.getFileDataAsTypedArray(node);
          return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
        } else {
          return callback(new Error('node type not supported'));
        }
      },storeLocalEntry:function (path, entry, callback) {
        try {
          if (FS.isDir(entry.mode)) {
            FS.mkdir(path, entry.mode);
          } else if (FS.isFile(entry.mode)) {
            FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
          } else {
            return callback(new Error('node type not supported'));
          }
  
          FS.chmod(path, entry.mode);
          FS.utime(path, entry.timestamp, entry.timestamp);
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },removeLocalEntry:function (path, callback) {
        try {
          var lookup = FS.lookupPath(path);
          var stat = FS.stat(path);
  
          if (FS.isDir(stat.mode)) {
            FS.rmdir(path);
          } else if (FS.isFile(stat.mode)) {
            FS.unlink(path);
          }
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },loadRemoteEntry:function (store, path, callback) {
        var req = store.get(path);
        req.onsuccess = function(event) { callback(null, event.target.result); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },storeRemoteEntry:function (store, path, entry, callback) {
        var req = store.put(entry, path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },removeRemoteEntry:function (store, path, callback) {
        var req = store.delete(path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = [];
        Object.keys(src.entries).forEach(function (key) {
          var e = src.entries[key];
          var e2 = dst.entries[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create.push(key);
            total++;
          }
        });
  
        var remove = [];
        Object.keys(dst.entries).forEach(function (key) {
          var e = dst.entries[key];
          var e2 = src.entries[key];
          if (!e2) {
            remove.push(key);
            total++;
          }
        });
  
        if (!total) {
          return callback(null);
        }
  
        var errored = false;
        var completed = 0;
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        transaction.onerror = function(e) {
          done(this.error);
          e.preventDefault();
        };
  
        // sort paths in ascending order so directory entries are created
        // before the files inside them
        create.sort().forEach(function (path) {
          if (dst.type === 'local') {
            IDBFS.loadRemoteEntry(store, path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeLocalEntry(path, entry, done);
            });
          } else {
            IDBFS.loadLocalEntry(path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeRemoteEntry(store, path, entry, done);
            });
          }
        });
  
        // sort paths in descending order so files are deleted before their
        // parent directories
        remove.sort().reverse().forEach(function(path) {
          if (dst.type === 'local') {
            IDBFS.removeLocalEntry(path, done);
          } else {
            IDBFS.removeRemoteEntry(store, path, done);
          }
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          return flags;
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            path = fs.readlinkSync(path);
            path = NODEJS_PATH.relative(NODEJS_PATH.resolve(node.mount.opts.root), path);
            return path;
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          if (length === 0) return 0; // node errors on 0 length reads
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          return position;
        }}};
  
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);
  
  function _fflush(stream) {
      // int fflush(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fflush.html
  
      /*
      // Disabled, see https://github.com/kripken/emscripten/issues/2770
      stream = FS.getStreamFromPtr(stream);
      if (stream.stream_ops.flush) {
        stream.stream_ops.flush(stream);
      }
      */
    }var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || {};
  
        if (!path) return { path: '', node: null };
  
        var defaults = {
          follow_mount: true,
          recurse_count: 0
        };
        for (var key in defaults) {
          if (opts[key] === undefined) {
            opts[key] = defaults[key];
          }
        }
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            if (!islast || (islast && opts.follow_mount)) {
              current = current.mounted.root;
            }
          }
  
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
  
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err, parent);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            this.mounted = null;
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
          };
  
          FS.FSNode.prototype = {};
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); }
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); }
            }
          });
        }
  
        var node = new FS.FSNode(parent, name, mode, rdev);
  
        FS.hashAddNode(node);
  
        return node;
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return !!node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var accmode = flag & 2097155;
        var perms = ['r', 'w', 'rw'][accmode];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        var err = FS.nodePermissions(dir, 'x');
        if (err) return err;
        if (!dir.node_ops.lookup) return ERRNO_CODES.EACCES;
        return 0;
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        // clone it, so we can return an instance of FSStream
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },getStreamFromPtr:function (ptr) {
        return FS.streams[ptr - 1];
      },getPtrForStream:function (stream) {
        return stream ? stream.fd + 1 : 0;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },getMounts:function (mount) {
        var mounts = [];
        var check = [mount];
  
        while (check.length) {
          var m = check.pop();
  
          mounts.push(m);
  
          check.push.apply(check, m.mounts);
        }
  
        return mounts;
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= mounts.length) {
            callback(null);
          }
        };
  
        // sync all mounts
        mounts.forEach(function (mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },mount:function (type, opts, mountpoint) {
        var root = mountpoint === '/';
        var pseudo = !mountpoint;
        var node;
  
        if (root && FS.root) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
          mountpoint = lookup.path;  // use the absolute path
          node = lookup.node;
  
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
          }
  
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
          }
        }
  
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          mounts: []
        };
  
        // create a root node for the fs
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
  
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          // set as a mountpoint
          node.mounted = mount;
  
          // add the new mount to the current mount's children
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
  
        return mountRoot;
      },unmount:function (mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
  
        // destroy the nodes for this mount, and all its child mounts
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
  
        Object.keys(FS.nameTable).forEach(function (hash) {
          var current = FS.nameTable[hash];
  
          while (current) {
            var next = current.name_next;
  
            if (mounts.indexOf(current.mount) !== -1) {
              FS.destroyNode(current);
            }
  
            current = next;
          }
        });
  
        // no longer a mountpoint
        node.mounted = null;
  
        // remove this mount from the child mounts
        var idx = node.mount.mounts.indexOf(mount);
        assert(idx !== -1);
        node.mount.mounts.splice(idx, 1);
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        if (!name || name === '.' || name === '..') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 438 /* 0666 */;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 511 /* 0777 */;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 438 /* 0666 */;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        if (!PATH.resolve(oldpath)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        if (!parent) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        if (!old_dir || !new_dir) throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        try {
          if (FS.trackingDelegate['willMovePath']) {
            FS.trackingDelegate['willMovePath'](old_path, new_path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
        } catch(e) {
          console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readlink:function (path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return PATH.resolve(FS.getPath(lookup.node.parent), link.node_ops.readlink(link));
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        var created = false;
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
            created = true;
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // check permissions, if this is not a file we just created now (it is ok to
        // create and write to a file with read-only permissions; it is read-only
        // for later use)
        if (!created) {
          var err = FS.mayOpen(node, flags);
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        try {
          if (FS.trackingDelegate['onOpenFile']) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate['onOpenFile'](path, trackingFlags);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
        }
        return stream;
      },close:function (stream) {
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        stream.position = stream.stream_ops.llseek(stream, offset, whence);
        stream.ungotten = [];
        return stream.position;
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
        } catch(e) {
          console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
        }
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },msync:function (stream, buffer, offset, length, mmapFlags) {
        if (!stream || !stream.stream_ops.msync) {
          return 0;
        }
        return stream.stream_ops.msync(stream, buffer, offset, length, mmapFlags);
      },munmap:function (stream) {
        return 0;
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = UTF8ArrayToString(buf, 0);
        } else if (opts.encoding === 'binary') {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var buf = new Uint8Array(lengthBytesUTF8(data)+1);
          var actualNumBytes = stringToUTF8Array(data, buf, 0, buf.length);
          FS.write(stream, buf, 0, actualNumBytes, 0, opts.canOwn);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0, opts.canOwn);
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
        FS.mkdir('/home');
        FS.mkdir('/home/web_user');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function(stream, buffer, offset, length, pos) { return length; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // setup /dev/[u]random
        var random_device;
        if (typeof crypto !== 'undefined') {
          // for modern web browsers
          var randomBuffer = new Uint8Array(1);
          random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
        } else if (ENVIRONMENT_IS_NODE) {
          // for nodejs
          random_device = function() { return require('crypto').randomBytes(1)[0]; };
        } else {
          // default for ES5 platforms
          random_device = function() { return (Math.random()*256)|0; };
        }
        FS.createDevice('/dev', 'random', random_device);
        FS.createDevice('/dev', 'urandom', random_device);
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        HEAP32[((_stdin)>>2)]=FS.getPtrForStream(stdin);
        assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        HEAP32[((_stdout)>>2)]=FS.getPtrForStream(stdout);
        assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        HEAP32[((_stderr)>>2)]=FS.getPtrForStream(stderr);
        assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno, node) {
          this.node = node;
          this.setErrno = function(errno) {
            this.errno = errno;
            for (var key in ERRNO_CODES) {
              if (ERRNO_CODES[key] === errno) {
                this.code = key;
                break;
              }
            }
          };
          this.setErrno(errno);
          this.message = ERRNO_MESSAGES[errno];
          if (this.stack) this.stack = demangleAll(this.stack);
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
        function LazyUint8Array() {
          this.lengthKnown = false;
          this.chunks = []; // Loaded chunks. Index is the chunk number
        }
        LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
          if (idx > this.length-1 || idx < 0) {
            return undefined;
          }
          var chunkOffset = idx % this.chunkSize;
          var chunkNum = (idx / this.chunkSize)|0;
          return this.getter(chunkNum)[chunkOffset];
        }
        LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
          this.getter = getter;
        }
        LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
          // Find length
          var xhr = new XMLHttpRequest();
          xhr.open('HEAD', url, false);
          xhr.send(null);
          if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
          var datalength = Number(xhr.getResponseHeader("Content-length"));
          var header;
          var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
          var chunkSize = 1024*1024; // Chunk size in bytes
  
          if (!hasByteServing) chunkSize = datalength;
  
          // Function to get a range from the remote URL.
          var doXHR = (function(from, to) {
            if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
            if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
            // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
            var xhr = new XMLHttpRequest();
            xhr.open('GET', url, false);
            if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
            // Some hints to the browser that we want binary data.
            if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
            if (xhr.overrideMimeType) {
              xhr.overrideMimeType('text/plain; charset=x-user-defined');
            }
  
            xhr.send(null);
            if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
            if (xhr.response !== undefined) {
              return new Uint8Array(xhr.response || []);
            } else {
              return intArrayFromString(xhr.responseText || '', true);
            }
          });
          var lazyArray = this;
          lazyArray.setDataGetter(function(chunkNum) {
            var start = chunkNum * chunkSize;
            var end = (chunkNum+1) * chunkSize - 1; // including this byte
            end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
              lazyArray.chunks[chunkNum] = doXHR(start, end);
            }
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
            return lazyArray.chunks[chunkNum];
          });
  
          this._length = datalength;
          this._chunkSize = chunkSize;
          this.lengthKnown = true;
        }
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // Add a function that defers querying the file size until it is asked the first time.
        Object.defineProperty(node, "usedBytes", {
            get: function() { return this.contents.length; }
        });
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn, preFinish) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        var dep = getUniqueRunDependency('cp ' + fullname); // might have several active requests for the same fullname
        function processData(byteArray) {
          function finish(byteArray) {
            if (preFinish) preFinish();
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency(dep);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency(dep);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency(dep);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};
  
  
  
  
  function _mkport() { throw 'TODO' }var SOCKFS={mount:function (mount) {
        // If Module['websocket'] has already been defined (e.g. for configuring
        // the subprotocol/url) use that, if not initialise it to a new object.
        Module['websocket'] = (Module['websocket'] && 
                               ('object' === typeof Module['websocket'])) ? Module['websocket'] : {};
  
        // Add the Event registration mechanism to the exported websocket configuration
        // object so we can register network callbacks from native JavaScript too.
        // For more documentation see system/include/emscripten/emscripten.h
        Module['websocket']._callbacks = {};
        Module['websocket']['on'] = function(event, callback) {
  	    if ('function' === typeof callback) {
  		  this._callbacks[event] = callback;
          }
  	    return this;
        };
  
        Module['websocket'].emit = function(event, param) {
  	    if ('function' === typeof this._callbacks[event]) {
  		  this._callbacks[event].call(this, param);
          }
        };
  
        // If debug is enabled register simple default logging callbacks for each Event.
  
        return FS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createSocket:function (family, type, protocol) {
        var streaming = type == 1;
        if (protocol) {
          assert(streaming == (protocol == 6)); // if SOCK_STREAM, must be tcp
        }
  
        // create our internal socket structure
        var sock = {
          family: family,
          type: type,
          protocol: protocol,
          server: null,
          error: null, // Used in getsockopt for SOL_SOCKET/SO_ERROR test
          peers: {},
          pending: [],
          recv_queue: [],
          sock_ops: SOCKFS.websocket_sock_ops
        };
  
        // create the filesystem node to store the socket structure
        var name = SOCKFS.nextname();
        var node = FS.createNode(SOCKFS.root, name, 49152, 0);
        node.sock = sock;
  
        // and the wrapping stream that enables library functions such
        // as read and write to indirectly interact with the socket
        var stream = FS.createStream({
          path: name,
          node: node,
          flags: FS.modeStringToFlags('r+'),
          seekable: false,
          stream_ops: SOCKFS.stream_ops
        });
  
        // map the new stream to the socket structure (sockets have a 1:1
        // relationship with a stream)
        sock.stream = stream;
  
        return sock;
      },getSocket:function (fd) {
        var stream = FS.getStream(fd);
        if (!stream || !FS.isSocket(stream.node.mode)) {
          return null;
        }
        return stream.node.sock;
      },stream_ops:{poll:function (stream) {
          var sock = stream.node.sock;
          return sock.sock_ops.poll(sock);
        },ioctl:function (stream, request, varargs) {
          var sock = stream.node.sock;
          return sock.sock_ops.ioctl(sock, request, varargs);
        },read:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          var msg = sock.sock_ops.recvmsg(sock, length);
          if (!msg) {
            // socket is closed
            return 0;
          }
          buffer.set(msg.buffer, offset);
          return msg.buffer.length;
        },write:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          return sock.sock_ops.sendmsg(sock, buffer, offset, length);
        },close:function (stream) {
          var sock = stream.node.sock;
          sock.sock_ops.close(sock);
        }},nextname:function () {
        if (!SOCKFS.nextname.current) {
          SOCKFS.nextname.current = 0;
        }
        return 'socket[' + (SOCKFS.nextname.current++) + ']';
      },websocket_sock_ops:{createPeer:function (sock, addr, port) {
          var ws;
  
          if (typeof addr === 'object') {
            ws = addr;
            addr = null;
            port = null;
          }
  
          if (ws) {
            // for sockets that've already connected (e.g. we're the server)
            // we can inspect the _socket property for the address
            if (ws._socket) {
              addr = ws._socket.remoteAddress;
              port = ws._socket.remotePort;
            }
            // if we're just now initializing a connection to the remote,
            // inspect the url property
            else {
              var result = /ws[s]?:\/\/([^:]+):(\d+)/.exec(ws.url);
              if (!result) {
                throw new Error('WebSocket URL must be in the format ws(s)://address:port');
              }
              addr = result[1];
              port = parseInt(result[2], 10);
            }
          } else {
            // create the actual websocket object and connect
            try {
              // runtimeConfig gets set to true if WebSocket runtime configuration is available.
              var runtimeConfig = (Module['websocket'] && ('object' === typeof Module['websocket']));
  
              // The default value is 'ws://' the replace is needed because the compiler replaces '//' comments with '#'
              // comments without checking context, so we'd end up with ws:#, the replace swaps the '#' for '//' again.
              var url = 'ws:#'.replace('#', '//');
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['url']) {
                  url = Module['websocket']['url']; // Fetch runtime WebSocket URL config.
                }
              }
  
              if (url === 'ws://' || url === 'wss://') { // Is the supplied URL config just a prefix, if so complete it.
                var parts = addr.split('/');
                url = url + parts[0] + ":" + port + "/" + parts.slice(1).join('/');
              }
  
              // Make the WebSocket subprotocol (Sec-WebSocket-Protocol) default to binary if no configuration is set.
              var subProtocols = 'binary'; // The default value is 'binary'
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['subprotocol']) {
                  subProtocols = Module['websocket']['subprotocol']; // Fetch runtime WebSocket subprotocol config.
                }
              }
  
              // The regex trims the string (removes spaces at the beginning and end, then splits the string by
              // <any space>,<any space> into an Array. Whitespace removal is important for Websockify and ws.
              subProtocols = subProtocols.replace(/^ +| +$/g,"").split(/ *, */);
  
              // The node ws library API for specifying optional subprotocol is slightly different than the browser's.
              var opts = ENVIRONMENT_IS_NODE ? {'protocol': subProtocols.toString()} : subProtocols;
  
              // If node we use the ws library.
              var WebSocket = ENVIRONMENT_IS_NODE ? require('ws') : window['WebSocket'];
              ws = new WebSocket(url, opts);
              ws.binaryType = 'arraybuffer';
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EHOSTUNREACH);
            }
          }
  
  
          var peer = {
            addr: addr,
            port: port,
            socket: ws,
            dgram_send_queue: []
          };
  
          SOCKFS.websocket_sock_ops.addPeer(sock, peer);
          SOCKFS.websocket_sock_ops.handlePeerEvents(sock, peer);
  
          // if this is a bound dgram socket, send the port number first to allow
          // us to override the ephemeral port reported to us by remotePort on the
          // remote end.
          if (sock.type === 2 && typeof sock.sport !== 'undefined') {
            peer.dgram_send_queue.push(new Uint8Array([
                255, 255, 255, 255,
                'p'.charCodeAt(0), 'o'.charCodeAt(0), 'r'.charCodeAt(0), 't'.charCodeAt(0),
                ((sock.sport & 0xff00) >> 8) , (sock.sport & 0xff)
            ]));
          }
  
          return peer;
        },getPeer:function (sock, addr, port) {
          return sock.peers[addr + ':' + port];
        },addPeer:function (sock, peer) {
          sock.peers[peer.addr + ':' + peer.port] = peer;
        },removePeer:function (sock, peer) {
          delete sock.peers[peer.addr + ':' + peer.port];
        },handlePeerEvents:function (sock, peer) {
          var first = true;
  
          var handleOpen = function () {
  
            Module['websocket'].emit('open', sock.stream.fd);
  
            try {
              var queued = peer.dgram_send_queue.shift();
              while (queued) {
                peer.socket.send(queued);
                queued = peer.dgram_send_queue.shift();
              }
            } catch (e) {
              // not much we can do here in the way of proper error handling as we've already
              // lied and said this data was sent. shut it down.
              peer.socket.close();
            }
          };
  
          function handleMessage(data) {
            assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
            data = new Uint8Array(data);  // make a typed array view on the array buffer
  
  
            // if this is the port message, override the peer's port with it
            var wasfirst = first;
            first = false;
            if (wasfirst &&
                data.length === 10 &&
                data[0] === 255 && data[1] === 255 && data[2] === 255 && data[3] === 255 &&
                data[4] === 'p'.charCodeAt(0) && data[5] === 'o'.charCodeAt(0) && data[6] === 'r'.charCodeAt(0) && data[7] === 't'.charCodeAt(0)) {
              // update the peer's port and it's key in the peer map
              var newport = ((data[8] << 8) | data[9]);
              SOCKFS.websocket_sock_ops.removePeer(sock, peer);
              peer.port = newport;
              SOCKFS.websocket_sock_ops.addPeer(sock, peer);
              return;
            }
  
            sock.recv_queue.push({ addr: peer.addr, port: peer.port, data: data });
            Module['websocket'].emit('message', sock.stream.fd);
          };
  
          if (ENVIRONMENT_IS_NODE) {
            peer.socket.on('open', handleOpen);
            peer.socket.on('message', function(data, flags) {
              if (!flags.binary) {
                return;
              }
              handleMessage((new Uint8Array(data)).buffer);  // copy from node Buffer -> ArrayBuffer
            });
            peer.socket.on('close', function() {
              Module['websocket'].emit('close', sock.stream.fd);
            });
            peer.socket.on('error', function(error) {
              // Although the ws library may pass errors that may be more descriptive than
              // ECONNREFUSED they are not necessarily the expected error code e.g. 
              // ENOTFOUND on getaddrinfo seems to be node.js specific, so using ECONNREFUSED
              // is still probably the most useful thing to do.
              sock.error = ERRNO_CODES.ECONNREFUSED; // Used in getsockopt for SOL_SOCKET/SO_ERROR test.
              Module['websocket'].emit('error', [sock.stream.fd, sock.error, 'ECONNREFUSED: Connection refused']);
              // don't throw
            });
          } else {
            peer.socket.onopen = handleOpen;
            peer.socket.onclose = function() {
              Module['websocket'].emit('close', sock.stream.fd);
            };
            peer.socket.onmessage = function peer_socket_onmessage(event) {
              handleMessage(event.data);
            };
            peer.socket.onerror = function(error) {
              // The WebSocket spec only allows a 'simple event' to be thrown on error,
              // so we only really know as much as ECONNREFUSED.
              sock.error = ERRNO_CODES.ECONNREFUSED; // Used in getsockopt for SOL_SOCKET/SO_ERROR test.
              Module['websocket'].emit('error', [sock.stream.fd, sock.error, 'ECONNREFUSED: Connection refused']);
            };
          }
        },poll:function (sock) {
          if (sock.type === 1 && sock.server) {
            // listen sockets should only say they're available for reading
            // if there are pending clients.
            return sock.pending.length ? (64 | 1) : 0;
          }
  
          var mask = 0;
          var dest = sock.type === 1 ?  // we only care about the socket state for connection-based sockets
            SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport) :
            null;
  
          if (sock.recv_queue.length ||
              !dest ||  // connection-less sockets are always ready to read
              (dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {  // let recv return 0 once closed
            mask |= (64 | 1);
          }
  
          if (!dest ||  // connection-less sockets are always ready to write
              (dest && dest.socket.readyState === dest.socket.OPEN)) {
            mask |= 4;
          }
  
          if ((dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {
            mask |= 16;
          }
  
          return mask;
        },ioctl:function (sock, request, arg) {
          switch (request) {
            case 21531:
              var bytes = 0;
              if (sock.recv_queue.length) {
                bytes = sock.recv_queue[0].data.length;
              }
              HEAP32[((arg)>>2)]=bytes;
              return 0;
            default:
              return ERRNO_CODES.EINVAL;
          }
        },close:function (sock) {
          // if we've spawned a listen server, close it
          if (sock.server) {
            try {
              sock.server.close();
            } catch (e) {
            }
            sock.server = null;
          }
          // close any peer connections
          var peers = Object.keys(sock.peers);
          for (var i = 0; i < peers.length; i++) {
            var peer = sock.peers[peers[i]];
            try {
              peer.socket.close();
            } catch (e) {
            }
            SOCKFS.websocket_sock_ops.removePeer(sock, peer);
          }
          return 0;
        },bind:function (sock, addr, port) {
          if (typeof sock.saddr !== 'undefined' || typeof sock.sport !== 'undefined') {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already bound
          }
          sock.saddr = addr;
          sock.sport = port || _mkport();
          // in order to emulate dgram sockets, we need to launch a listen server when
          // binding on a connection-less socket
          // note: this is only required on the server side
          if (sock.type === 2) {
            // close the existing server if it exists
            if (sock.server) {
              sock.server.close();
              sock.server = null;
            }
            // swallow error operation not supported error that occurs when binding in the
            // browser where this isn't supported
            try {
              sock.sock_ops.listen(sock, 0);
            } catch (e) {
              if (!(e instanceof FS.ErrnoError)) throw e;
              if (e.errno !== ERRNO_CODES.EOPNOTSUPP) throw e;
            }
          }
        },connect:function (sock, addr, port) {
          if (sock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
  
          // TODO autobind
          // if (!sock.addr && sock.type == 2) {
          // }
  
          // early out if we're already connected / in the middle of connecting
          if (typeof sock.daddr !== 'undefined' && typeof sock.dport !== 'undefined') {
            var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
            if (dest) {
              if (dest.socket.readyState === dest.socket.CONNECTING) {
                throw new FS.ErrnoError(ERRNO_CODES.EALREADY);
              } else {
                throw new FS.ErrnoError(ERRNO_CODES.EISCONN);
              }
            }
          }
  
          // add the socket to our peer list and set our
          // destination address / port to match
          var peer = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
          sock.daddr = peer.addr;
          sock.dport = peer.port;
  
          // always "fail" in non-blocking mode
          throw new FS.ErrnoError(ERRNO_CODES.EINPROGRESS);
        },listen:function (sock, backlog) {
          if (!ENVIRONMENT_IS_NODE) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
          if (sock.server) {
             throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already listening
          }
          var WebSocketServer = require('ws').Server;
          var host = sock.saddr;
          sock.server = new WebSocketServer({
            host: host,
            port: sock.sport
            // TODO support backlog
          });
          Module['websocket'].emit('listen', sock.stream.fd); // Send Event with listen fd.
  
          sock.server.on('connection', function(ws) {
            if (sock.type === 1) {
              var newsock = SOCKFS.createSocket(sock.family, sock.type, sock.protocol);
  
              // create a peer on the new socket
              var peer = SOCKFS.websocket_sock_ops.createPeer(newsock, ws);
              newsock.daddr = peer.addr;
              newsock.dport = peer.port;
  
              // push to queue for accept to pick up
              sock.pending.push(newsock);
              Module['websocket'].emit('connection', newsock.stream.fd);
            } else {
              // create a peer on the listen socket so calling sendto
              // with the listen socket and an address will resolve
              // to the correct client
              SOCKFS.websocket_sock_ops.createPeer(sock, ws);
              Module['websocket'].emit('connection', sock.stream.fd);
            }
          });
          sock.server.on('closed', function() {
            Module['websocket'].emit('close', sock.stream.fd);
            sock.server = null;
          });
          sock.server.on('error', function(error) {
            // Although the ws library may pass errors that may be more descriptive than
            // ECONNREFUSED they are not necessarily the expected error code e.g. 
            // ENOTFOUND on getaddrinfo seems to be node.js specific, so using EHOSTUNREACH
            // is still probably the most useful thing to do. This error shouldn't
            // occur in a well written app as errors should get trapped in the compiled
            // app's own getaddrinfo call.
            sock.error = ERRNO_CODES.EHOSTUNREACH; // Used in getsockopt for SOL_SOCKET/SO_ERROR test.
            Module['websocket'].emit('error', [sock.stream.fd, sock.error, 'EHOSTUNREACH: Host is unreachable']);
            // don't throw
          });
        },accept:function (listensock) {
          if (!listensock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          var newsock = listensock.pending.shift();
          newsock.stream.flags = listensock.stream.flags;
          return newsock;
        },getname:function (sock, peer) {
          var addr, port;
          if (peer) {
            if (sock.daddr === undefined || sock.dport === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            }
            addr = sock.daddr;
            port = sock.dport;
          } else {
            // TODO saddr and sport will be set for bind()'d UDP sockets, but what
            // should we be returning for TCP sockets that've been connect()'d?
            addr = sock.saddr || 0;
            port = sock.sport || 0;
          }
          return { addr: addr, port: port };
        },sendmsg:function (sock, buffer, offset, length, addr, port) {
          if (sock.type === 2) {
            // connection-less sockets will honor the message address,
            // and otherwise fall back to the bound destination address
            if (addr === undefined || port === undefined) {
              addr = sock.daddr;
              port = sock.dport;
            }
            // if there was no address to fall back to, error out
            if (addr === undefined || port === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.EDESTADDRREQ);
            }
          } else {
            // connection-based sockets will only use the bound
            addr = sock.daddr;
            port = sock.dport;
          }
  
          // find the peer for the destination address
          var dest = SOCKFS.websocket_sock_ops.getPeer(sock, addr, port);
  
          // early out if not connected with a connection-based socket
          if (sock.type === 1) {
            if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            } else if (dest.socket.readyState === dest.socket.CONNECTING) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // create a copy of the incoming data to send, as the WebSocket API
          // doesn't work entirely with an ArrayBufferView, it'll just send
          // the entire underlying buffer
          var data;
          if (buffer instanceof Array || buffer instanceof ArrayBuffer) {
            data = buffer.slice(offset, offset + length);
          } else {  // ArrayBufferView
            data = buffer.buffer.slice(buffer.byteOffset + offset, buffer.byteOffset + offset + length);
          }
  
          // if we're emulating a connection-less dgram socket and don't have
          // a cached connection, queue the buffer to send upon connect and
          // lie, saying the data was sent now.
          if (sock.type === 2) {
            if (!dest || dest.socket.readyState !== dest.socket.OPEN) {
              // if we're not connected, open a new connection
              if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                dest = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
              }
              dest.dgram_send_queue.push(data);
              return length;
            }
          }
  
          try {
            // send the actual data
            dest.socket.send(data);
            return length;
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
        },recvmsg:function (sock, length) {
          // http://pubs.opengroup.org/onlinepubs/7908799/xns/recvmsg.html
          if (sock.type === 1 && sock.server) {
            // tcp servers should not be recv()'ing on the listen socket
            throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
          }
  
          var queued = sock.recv_queue.shift();
          if (!queued) {
            if (sock.type === 1) {
              var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
  
              if (!dest) {
                // if we have a destination address but are not connected, error out
                throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
              }
              else if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                // return null if the socket has closed
                return null;
              }
              else {
                // else, our socket is in a valid state but truly has nothing available
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
            } else {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // queued.data will be an ArrayBuffer if it's unadulterated, but if it's
          // requeued TCP data it'll be an ArrayBufferView
          var queuedLength = queued.data.byteLength || queued.data.length;
          var queuedOffset = queued.data.byteOffset || 0;
          var queuedBuffer = queued.data.buffer || queued.data;
          var bytesRead = Math.min(length, queuedLength);
          var res = {
            buffer: new Uint8Array(queuedBuffer, queuedOffset, bytesRead),
            addr: queued.addr,
            port: queued.port
          };
  
  
          // push back any unread data for TCP connections
          if (sock.type === 1 && bytesRead < queuedLength) {
            var bytesRemaining = queuedLength - bytesRead;
            queued.data = new Uint8Array(queuedBuffer, queuedOffset + bytesRead, bytesRemaining);
            sock.recv_queue.unshift(queued);
          }
  
          return res;
        }}};function _send(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _write(fd, buf, len);
    }
  
  function _pwrite(fildes, buf, nbyte, offset) {
      // ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _write(fildes, buf, nbyte) {
      // ssize_t write(int fildes, const void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
  
  function _fileno(stream) {
      // int fileno(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fileno.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) return -1;
      return stream.fd;
    }function _fwrite(ptr, size, nitems, stream) {
      // size_t fwrite(const void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fwrite.html
      var bytesToWrite = nitems * size;
      if (bytesToWrite == 0) return 0;
      var fd = _fileno(stream);
      var bytesWritten = _write(fd, ptr, bytesToWrite);
      if (bytesWritten == -1) {
        var streamObj = FS.getStreamFromPtr(stream);
        if (streamObj) streamObj.error = true;
        return 0;
      } else {
        return (bytesWritten / size)|0;
      }
    }
  
  
   
  Module["_strlen"] = _strlen;
  
  function __reallyNegative(x) {
      return x < 0 || (x === 0 && (1/x) === -Infinity);
    }function __formatString(format, varargs) {
      assert((varargs & 3) === 0);
      var textIndex = format;
      var argIndex = 0;
      function getNextArg(type) {
        // NOTE: Explicitly ignoring type safety. Otherwise this fails:
        //       int x = 4; printf("%c\n", (char)x);
        var ret;
        argIndex = Runtime.prepVararg(argIndex, type);
        if (type === 'double') {
          ret = (HEAP32[((tempDoublePtr)>>2)]=HEAP32[(((varargs)+(argIndex))>>2)],HEAP32[(((tempDoublePtr)+(4))>>2)]=HEAP32[(((varargs)+((argIndex)+(4)))>>2)],(+(HEAPF64[(tempDoublePtr)>>3])));
          argIndex += 8;
        } else if (type == 'i64') {
          ret = [HEAP32[(((varargs)+(argIndex))>>2)],
                 HEAP32[(((varargs)+(argIndex+4))>>2)]];
  
          argIndex += 8;
        } else {
          assert((argIndex & 3) === 0);
          type = 'i32'; // varargs are always i32, i64, or double
          ret = HEAP32[(((varargs)+(argIndex))>>2)];
          argIndex += 4;
        }
        return ret;
      }
  
      var ret = [];
      var curr, next, currArg;
      while(1) {
        var startTextIndex = textIndex;
        curr = HEAP8[((textIndex)>>0)];
        if (curr === 0) break;
        next = HEAP8[((textIndex+1)>>0)];
        if (curr == 37) {
          // Handle flags.
          var flagAlwaysSigned = false;
          var flagLeftAlign = false;
          var flagAlternative = false;
          var flagZeroPad = false;
          var flagPadSign = false;
          flagsLoop: while (1) {
            switch (next) {
              case 43:
                flagAlwaysSigned = true;
                break;
              case 45:
                flagLeftAlign = true;
                break;
              case 35:
                flagAlternative = true;
                break;
              case 48:
                if (flagZeroPad) {
                  break flagsLoop;
                } else {
                  flagZeroPad = true;
                  break;
                }
              case 32:
                flagPadSign = true;
                break;
              default:
                break flagsLoop;
            }
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          }
  
          // Handle width.
          var width = 0;
          if (next == 42) {
            width = getNextArg('i32');
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          } else {
            while (next >= 48 && next <= 57) {
              width = width * 10 + (next - 48);
              textIndex++;
              next = HEAP8[((textIndex+1)>>0)];
            }
          }
  
          // Handle precision.
          var precisionSet = false, precision = -1;
          if (next == 46) {
            precision = 0;
            precisionSet = true;
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
            if (next == 42) {
              precision = getNextArg('i32');
              textIndex++;
            } else {
              while(1) {
                var precisionChr = HEAP8[((textIndex+1)>>0)];
                if (precisionChr < 48 ||
                    precisionChr > 57) break;
                precision = precision * 10 + (precisionChr - 48);
                textIndex++;
              }
            }
            next = HEAP8[((textIndex+1)>>0)];
          }
          if (precision < 0) {
            precision = 6; // Standard default.
            precisionSet = false;
          }
  
          // Handle integer sizes. WARNING: These assume a 32-bit architecture!
          var argSize;
          switch (String.fromCharCode(next)) {
            case 'h':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 104) {
                textIndex++;
                argSize = 1; // char (actually i32 in varargs)
              } else {
                argSize = 2; // short (actually i32 in varargs)
              }
              break;
            case 'l':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 108) {
                textIndex++;
                argSize = 8; // long long
              } else {
                argSize = 4; // long
              }
              break;
            case 'L': // long long
            case 'q': // int64_t
            case 'j': // intmax_t
              argSize = 8;
              break;
            case 'z': // size_t
            case 't': // ptrdiff_t
            case 'I': // signed ptrdiff_t or unsigned size_t
              argSize = 4;
              break;
            default:
              argSize = null;
          }
          if (argSize) textIndex++;
          next = HEAP8[((textIndex+1)>>0)];
  
          // Handle type specifier.
          switch (String.fromCharCode(next)) {
            case 'd': case 'i': case 'u': case 'o': case 'x': case 'X': case 'p': {
              // Integer.
              var signed = next == 100 || next == 105;
              argSize = argSize || 4;
              var currArg = getNextArg('i' + (argSize * 8));
              var origArg = currArg;
              var argText;
              // Flatten i64-1 [low, high] into a (slightly rounded) double
              if (argSize == 8) {
                currArg = Runtime.makeBigInt(currArg[0], currArg[1], next == 117);
              }
              // Truncate to requested size.
              if (argSize <= 4) {
                var limit = Math.pow(256, argSize) - 1;
                currArg = (signed ? reSign : unSign)(currArg & limit, argSize * 8);
              }
              // Format the number.
              var currAbsArg = Math.abs(currArg);
              var prefix = '';
              if (next == 100 || next == 105) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], null); else
                argText = reSign(currArg, 8 * argSize, 1).toString(10);
              } else if (next == 117) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], true); else
                argText = unSign(currArg, 8 * argSize, 1).toString(10);
                currArg = Math.abs(currArg);
              } else if (next == 111) {
                argText = (flagAlternative ? '0' : '') + currAbsArg.toString(8);
              } else if (next == 120 || next == 88) {
                prefix = (flagAlternative && currArg != 0) ? '0x' : '';
                if (argSize == 8 && i64Math) {
                  if (origArg[1]) {
                    argText = (origArg[1]>>>0).toString(16);
                    var lower = (origArg[0]>>>0).toString(16);
                    while (lower.length < 8) lower = '0' + lower;
                    argText += lower;
                  } else {
                    argText = (origArg[0]>>>0).toString(16);
                  }
                } else
                if (currArg < 0) {
                  // Represent negative numbers in hex as 2's complement.
                  currArg = -currArg;
                  argText = (currAbsArg - 1).toString(16);
                  var buffer = [];
                  for (var i = 0; i < argText.length; i++) {
                    buffer.push((0xF - parseInt(argText[i], 16)).toString(16));
                  }
                  argText = buffer.join('');
                  while (argText.length < argSize * 2) argText = 'f' + argText;
                } else {
                  argText = currAbsArg.toString(16);
                }
                if (next == 88) {
                  prefix = prefix.toUpperCase();
                  argText = argText.toUpperCase();
                }
              } else if (next == 112) {
                if (currAbsArg === 0) {
                  argText = '(nil)';
                } else {
                  prefix = '0x';
                  argText = currAbsArg.toString(16);
                }
              }
              if (precisionSet) {
                while (argText.length < precision) {
                  argText = '0' + argText;
                }
              }
  
              // Add sign if needed
              if (currArg >= 0) {
                if (flagAlwaysSigned) {
                  prefix = '+' + prefix;
                } else if (flagPadSign) {
                  prefix = ' ' + prefix;
                }
              }
  
              // Move sign to prefix so we zero-pad after the sign
              if (argText.charAt(0) == '-') {
                prefix = '-' + prefix;
                argText = argText.substr(1);
              }
  
              // Add padding.
              while (prefix.length + argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad) {
                    argText = '0' + argText;
                  } else {
                    prefix = ' ' + prefix;
                  }
                }
              }
  
              // Insert the result into the buffer.
              argText = prefix + argText;
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 'f': case 'F': case 'e': case 'E': case 'g': case 'G': {
              // Float.
              var currArg = getNextArg('double');
              var argText;
              if (isNaN(currArg)) {
                argText = 'nan';
                flagZeroPad = false;
              } else if (!isFinite(currArg)) {
                argText = (currArg < 0 ? '-' : '') + 'inf';
                flagZeroPad = false;
              } else {
                var isGeneral = false;
                var effectivePrecision = Math.min(precision, 20);
  
                // Convert g/G to f/F or e/E, as per:
                // http://pubs.opengroup.org/onlinepubs/9699919799/functions/printf.html
                if (next == 103 || next == 71) {
                  isGeneral = true;
                  precision = precision || 1;
                  var exponent = parseInt(currArg.toExponential(effectivePrecision).split('e')[1], 10);
                  if (precision > exponent && exponent >= -4) {
                    next = ((next == 103) ? 'f' : 'F').charCodeAt(0);
                    precision -= exponent + 1;
                  } else {
                    next = ((next == 103) ? 'e' : 'E').charCodeAt(0);
                    precision--;
                  }
                  effectivePrecision = Math.min(precision, 20);
                }
  
                if (next == 101 || next == 69) {
                  argText = currArg.toExponential(effectivePrecision);
                  // Make sure the exponent has at least 2 digits.
                  if (/[eE][-+]\d$/.test(argText)) {
                    argText = argText.slice(0, -1) + '0' + argText.slice(-1);
                  }
                } else if (next == 102 || next == 70) {
                  argText = currArg.toFixed(effectivePrecision);
                  if (currArg === 0 && __reallyNegative(currArg)) {
                    argText = '-' + argText;
                  }
                }
  
                var parts = argText.split('e');
                if (isGeneral && !flagAlternative) {
                  // Discard trailing zeros and periods.
                  while (parts[0].length > 1 && parts[0].indexOf('.') != -1 &&
                         (parts[0].slice(-1) == '0' || parts[0].slice(-1) == '.')) {
                    parts[0] = parts[0].slice(0, -1);
                  }
                } else {
                  // Make sure we have a period in alternative mode.
                  if (flagAlternative && argText.indexOf('.') == -1) parts[0] += '.';
                  // Zero pad until required precision.
                  while (precision > effectivePrecision++) parts[0] += '0';
                }
                argText = parts[0] + (parts.length > 1 ? 'e' + parts[1] : '');
  
                // Capitalize 'E' if needed.
                if (next == 69) argText = argText.toUpperCase();
  
                // Add sign.
                if (currArg >= 0) {
                  if (flagAlwaysSigned) {
                    argText = '+' + argText;
                  } else if (flagPadSign) {
                    argText = ' ' + argText;
                  }
                }
              }
  
              // Add padding.
              while (argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad && (argText[0] == '-' || argText[0] == '+')) {
                    argText = argText[0] + '0' + argText.slice(1);
                  } else {
                    argText = (flagZeroPad ? '0' : ' ') + argText;
                  }
                }
              }
  
              // Adjust case.
              if (next < 97) argText = argText.toUpperCase();
  
              // Insert the result into the buffer.
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 's': {
              // String.
              var arg = getNextArg('i8*');
              var argLength = arg ? _strlen(arg) : '(null)'.length;
              if (precisionSet) argLength = Math.min(argLength, precision);
              if (!flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              if (arg) {
                for (var i = 0; i < argLength; i++) {
                  ret.push(HEAPU8[((arg++)>>0)]);
                }
              } else {
                ret = ret.concat(intArrayFromString('(null)'.substr(0, argLength), true));
              }
              if (flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              break;
            }
            case 'c': {
              // Character.
              if (flagLeftAlign) ret.push(getNextArg('i8'));
              while (--width > 0) {
                ret.push(32);
              }
              if (!flagLeftAlign) ret.push(getNextArg('i8'));
              break;
            }
            case 'n': {
              // Write the length written so far to the next parameter.
              var ptr = getNextArg('i32*');
              HEAP32[((ptr)>>2)]=ret.length;
              break;
            }
            case '%': {
              // Literal percent sign.
              ret.push(curr);
              break;
            }
            default: {
              // Unknown specifiers remain untouched.
              for (var i = startTextIndex; i < textIndex + 2; i++) {
                ret.push(HEAP8[((i)>>0)]);
              }
            }
          }
          textIndex += 2;
          // TODO: Support a/A (hex float) and m (last error) specifiers.
          // TODO: Support %1${specifier} for arg selection.
        } else {
          ret.push(curr);
          textIndex += 1;
        }
      }
      return ret;
    }function _fprintf(stream, format, varargs) {
      // int fprintf(FILE *restrict stream, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var result = __formatString(format, varargs);
      var stack = Runtime.stackSave();
      var ret = _fwrite(allocate(result, 'i8', ALLOC_STACK), 1, result.length, stream);
      Runtime.stackRestore(stack);
      return ret;
    }

  function _printf(format, varargs) {
      // int printf(const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var stdout = HEAP32[((_stdout)>>2)];
      return _fprintf(stdout, format, varargs);
    }


  
  function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
      return dest;
    } 
  Module["_memcpy"] = _memcpy;

  function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 85: return totalMemory / PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 79:
          return 0;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: {
          if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
          return 1;
        }
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

  var _fabs=Math_abs;

  function ___errno_location() {
      return ___errno_state;
    }

  var _BItoD=true;

  function _times(buffer) {
      // clock_t times(struct tms *buffer);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/times.html
      // NOTE: This is fake, since we can't calculate real CPU time usage in JS.
      if (buffer !== 0) {
        _memset(buffer, 0, 16);
      }
      return 0;
    }

  
  function __exit(status) {
      // void _exit(int status);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
      Module['exit'](status);
    }function _exit(status) {
      __exit(status);
    }

  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) {
        var success = self.alloc(bytes);
        if (!success) return -1 >>> 0; // sbrk failure code
      }
      return ret;  // Previous break location.
    }

  function _time(ptr) {
      var ret = (Date.now()/1000)|0;
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }

  
  
  function _emscripten_set_main_loop_timing(mode, value) {
      Browser.mainLoop.timingMode = mode;
      Browser.mainLoop.timingValue = value;
  
      if (!Browser.mainLoop.func) {
        console.error('emscripten_set_main_loop_timing: Cannot set timing mode for main loop since a main loop does not exist! Call emscripten_set_main_loop first to set one up.');
        return 1; // Return non-zero on failure, can't set timing mode when there is no main loop.
      }
  
      if (mode == 0 /*EM_TIMING_SETTIMEOUT*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler() {
          setTimeout(Browser.mainLoop.runner, value); // doing this each time means that on exception, we stop
        };
        Browser.mainLoop.method = 'timeout';
      } else if (mode == 1 /*EM_TIMING_RAF*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler() {
          Browser.requestAnimationFrame(Browser.mainLoop.runner);
        };
        Browser.mainLoop.method = 'rAF';
      }
      return 0;
    }function _emscripten_set_main_loop(func, fps, simulateInfiniteLoop, arg, noSetTiming) {
      Module['noExitRuntime'] = true;
  
      assert(!Browser.mainLoop.func, 'emscripten_set_main_loop: there can only be one main loop function at once: call emscripten_cancel_main_loop to cancel the previous one before setting a new one with different parameters.');
  
      Browser.mainLoop.func = func;
      Browser.mainLoop.arg = arg;
  
      var thisMainLoopId = Browser.mainLoop.currentlyRunningMainloop;
  
      Browser.mainLoop.runner = function Browser_mainLoop_runner() {
        if (ABORT) return;
        if (Browser.mainLoop.queue.length > 0) {
          var start = Date.now();
          var blocker = Browser.mainLoop.queue.shift();
          blocker.func(blocker.arg);
          if (Browser.mainLoop.remainingBlockers) {
            var remaining = Browser.mainLoop.remainingBlockers;
            var next = remaining%1 == 0 ? remaining-1 : Math.floor(remaining);
            if (blocker.counted) {
              Browser.mainLoop.remainingBlockers = next;
            } else {
              // not counted, but move the progress along a tiny bit
              next = next + 0.5; // do not steal all the next one's progress
              Browser.mainLoop.remainingBlockers = (8*remaining + next)/9;
            }
          }
          console.log('main loop blocker "' + blocker.name + '" took ' + (Date.now() - start) + ' ms'); //, left: ' + Browser.mainLoop.remainingBlockers);
          Browser.mainLoop.updateStatus();
          setTimeout(Browser.mainLoop.runner, 0);
          return;
        }
  
        // catch pauses from non-main loop sources
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Implement very basic swap interval control
        Browser.mainLoop.currentFrameNumber = Browser.mainLoop.currentFrameNumber + 1 | 0;
        if (Browser.mainLoop.timingMode == 1/*EM_TIMING_RAF*/ && Browser.mainLoop.timingValue > 1 && Browser.mainLoop.currentFrameNumber % Browser.mainLoop.timingValue != 0) {
          // Not the scheduled time to render this frame - skip.
          Browser.mainLoop.scheduler();
          return;
        }
  
        // Signal GL rendering layer that processing of a new frame is about to start. This helps it optimize
        // VBO double-buffering and reduce GPU stalls.
  
        if (Browser.mainLoop.method === 'timeout' && Module.ctx) {
          Module.printErr('Looks like you are rendering without using requestAnimationFrame for the main loop. You should use 0 for the frame rate in emscripten_set_main_loop in order to use requestAnimationFrame, as that can greatly improve your frame rates!');
          Browser.mainLoop.method = ''; // just warn once per call to set main loop
        }
  
        Browser.mainLoop.runIter(function() {
          if (typeof arg !== 'undefined') {
            Runtime.dynCall('vi', func, [arg]);
          } else {
            Runtime.dynCall('v', func);
          }
        });
  
        // catch pauses from the main loop itself
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Queue new audio data. This is important to be right after the main loop invocation, so that we will immediately be able
        // to queue the newest produced audio samples.
        // TODO: Consider adding pre- and post- rAF callbacks so that GL.newRenderingFrameStarted() and SDL.audio.queueNewAudioData()
        //       do not need to be hardcoded into this function, but can be more generic.
        if (typeof SDL === 'object' && SDL.audio && SDL.audio.queueNewAudioData) SDL.audio.queueNewAudioData();
  
        Browser.mainLoop.scheduler();
      }
  
      if (!noSetTiming) {
        if (fps && fps > 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 1000.0 / fps);
        else _emscripten_set_main_loop_timing(1/*EM_TIMING_RAF*/, 1); // Do rAF by rendering each frame (no decimating)
  
        Browser.mainLoop.scheduler();
      }
  
      if (simulateInfiniteLoop) {
        throw 'SimulateInfiniteLoop';
      }
    }var Browser={mainLoop:{scheduler:null,method:"",currentlyRunningMainloop:0,func:null,arg:0,timingMode:0,timingValue:0,currentFrameNumber:0,queue:[],pause:function () {
          Browser.mainLoop.scheduler = null;
          Browser.mainLoop.currentlyRunningMainloop++; // Incrementing this signals the previous main loop that it's now become old, and it must return.
        },resume:function () {
          Browser.mainLoop.currentlyRunningMainloop++;
          var timingMode = Browser.mainLoop.timingMode;
          var timingValue = Browser.mainLoop.timingValue;
          var func = Browser.mainLoop.func;
          Browser.mainLoop.func = null;
          _emscripten_set_main_loop(func, 0, false, Browser.mainLoop.arg, true /* do not set timing and call scheduler, we will do it on the next lines */);
          _emscripten_set_main_loop_timing(timingMode, timingValue);
          Browser.mainLoop.scheduler();
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        },runIter:function (func) {
          if (ABORT) return;
          if (Module['preMainLoop']) {
            var preRet = Module['preMainLoop']();
            if (preRet === false) {
              return; // |return false| skips a frame
            }
          }
          try {
            func();
          } catch (e) {
            if (e instanceof ExitStatus) {
              return;
            } else {
              if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
              throw e;
            }
          }
          if (Module['postMainLoop']) Module['postMainLoop']();
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          assert(typeof url == 'string', 'createObjectURL must return a url as a string');
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            assert(typeof url == 'string', 'createObjectURL must return a url as a string');
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas ||
                                document['msPointerLockElement'] === canvas;
        }
        if (canvas) {
          // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
          // Module['forcedAspectRatio'] = 4 / 3;
          
          canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                      canvas['mozRequestPointerLock'] ||
                                      canvas['webkitRequestPointerLock'] ||
                                      canvas['msRequestPointerLock'] ||
                                      function(){};
          canvas.exitPointerLock = document['exitPointerLock'] ||
                                   document['mozExitPointerLock'] ||
                                   document['webkitExitPointerLock'] ||
                                   document['msExitPointerLock'] ||
                                   function(){}; // no-op if function does not exist
          canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
  
          document.addEventListener('pointerlockchange', pointerLockChange, false);
          document.addEventListener('mozpointerlockchange', pointerLockChange, false);
          document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
          document.addEventListener('mspointerlockchange', pointerLockChange, false);
  
          if (Module['elementPointerLock']) {
            canvas.addEventListener("click", function(ev) {
              if (!Browser.pointerLock && canvas.requestPointerLock) {
                canvas.requestPointerLock();
                ev.preventDefault();
              }
            }, false);
          }
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        if (useWebGL && Module.ctx && canvas == Module.canvas) return Module.ctx; // no need to recreate GL context if it's already been created for this canvas.
  
        var ctx;
        var contextHandle;
        if (useWebGL) {
          // For GLES2/desktop GL compatibility, adjust a few defaults to be different to WebGL defaults, so that they align better with the desktop defaults.
          var contextAttributes = {
            antialias: false,
            alpha: false
          };
  
          if (webGLContextAttributes) {
            for (var attribute in webGLContextAttributes) {
              contextAttributes[attribute] = webGLContextAttributes[attribute];
            }
          }
  
          contextHandle = GL.createContext(canvas, contextAttributes);
          if (contextHandle) {
            ctx = GL.getContext(contextHandle).GLctx;
          }
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
        } else {
          ctx = canvas.getContext('2d');
        }
  
        if (!ctx) return null;
  
        if (setInModule) {
          if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
  
          Module.ctx = ctx;
          if (useWebGL) GL.makeContextCurrent(contextHandle);
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas, vrDevice) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        Browser.vrDevice = vrDevice;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
        if (typeof Browser.vrDevice === 'undefined') Browser.vrDevice = null;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          var canvasContainer = canvas.parentNode;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement'] ||
               document['msFullScreenElement'] || document['msFullscreenElement'] ||
               document['webkitCurrentFullScreenElement']) === canvasContainer) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'] ||
                                      document['msExitFullscreen'] ||
                                      document['exitFullscreen'] ||
                                      function() {};
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else {
            
            // remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
            canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
            canvasContainer.parentNode.removeChild(canvasContainer);
            
            if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
          Browser.updateCanvasDimensions(canvas);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
          document.addEventListener('MSFullscreenChange', fullScreenChange, false);
        }
  
        // create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
        var canvasContainer = document.createElement("div");
        canvas.parentNode.insertBefore(canvasContainer, canvas);
        canvasContainer.appendChild(canvas);
  
        // use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
        canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
                                            canvasContainer['mozRequestFullScreen'] ||
                                            canvasContainer['msRequestFullscreen'] ||
                                           (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
  
        if (vrDevice) {
          canvasContainer.requestFullScreen({ vrDisplay: vrDevice });
        } else {
          canvasContainer.requestFullScreen();
        }
      },nextRAF:0,fakeRequestAnimationFrame:function (func) {
        // try to keep 60fps between calls to here
        var now = Date.now();
        if (Browser.nextRAF === 0) {
          Browser.nextRAF = now + 1000/60;
        } else {
          while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
            Browser.nextRAF += 1000/60;
          }
        }
        var delay = Math.max(Browser.nextRAF - now, 0);
        setTimeout(func, delay);
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          Browser.fakeRequestAnimationFrame(func);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           Browser.fakeRequestAnimationFrame;
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },allowAsyncCallbacks:true,queuedAsyncCallbacks:[],pauseAsyncCallbacks:function () {
        Browser.allowAsyncCallbacks = false;
      },resumeAsyncCallbacks:function () { // marks future callbacks as ok to execute, and synchronously runs any remaining ones right now
        Browser.allowAsyncCallbacks = true;
        if (Browser.queuedAsyncCallbacks.length > 0) {
          var callbacks = Browser.queuedAsyncCallbacks;
          Browser.queuedAsyncCallbacks = [];
          callbacks.forEach(function(func) {
            func();
          });
        }
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } else {
            Browser.queuedAsyncCallbacks.push(func);
          }
        });
      },safeSetTimeout:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setTimeout(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } else {
            Browser.queuedAsyncCallbacks.push(func);
          }
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setInterval(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } // drop it on the floor otherwise, next interval will kick in
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },getMouseWheelDelta:function (event) {
        var delta = 0;
        switch (event.type) {
          case 'DOMMouseScroll': 
            delta = event.detail;
            break;
          case 'mousewheel': 
            delta = event.wheelDelta;
            break;
          case 'wheel': 
            delta = event['deltaY'];
            break;
          default:
            throw 'unrecognized mouse wheel event: ' + event.type;
        }
        return delta;
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
  
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
          // If this assert lands, it's likely because the browser doesn't support scrollX or pageXOffset
          // and we have no viable fallback.
          assert((typeof scrollX !== 'undefined') && (typeof scrollY !== 'undefined'), 'Unable to retrieve scroll position, mouse positions likely broken.');
  
          if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
            var touch = event.touch;
            if (touch === undefined) {
              return; // the "touch" property is only defined in SDL
  
            }
            var adjustedX = touch.pageX - (scrollX + rect.left);
            var adjustedY = touch.pageY - (scrollY + rect.top);
  
            adjustedX = adjustedX * (cw / rect.width);
            adjustedY = adjustedY * (ch / rect.height);
  
            var coords = { x: adjustedX, y: adjustedY };
            
            if (event.type === 'touchstart') {
              Browser.lastTouches[touch.identifier] = coords;
              Browser.touches[touch.identifier] = coords;
            } else if (event.type === 'touchend' || event.type === 'touchmove') {
              var last = Browser.touches[touch.identifier];
              if (!last) last = coords;
              Browser.lastTouches[touch.identifier] = last;
              Browser.touches[touch.identifier] = coords;
            } 
            return;
          }
  
          var x = event.pageX - (scrollX + rect.left);
          var y = event.pageY - (scrollY + rect.top);
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        Browser.updateCanvasDimensions(canvas, width, height);
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },updateCanvasDimensions:function (canvas, wNative, hNative) {
        if (wNative && hNative) {
          canvas.widthNative = wNative;
          canvas.heightNative = hNative;
        } else {
          wNative = canvas.widthNative;
          hNative = canvas.heightNative;
        }
        var w = wNative;
        var h = hNative;
        if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
          if (w/h < Module['forcedAspectRatio']) {
            w = Math.round(h * Module['forcedAspectRatio']);
          } else {
            h = Math.round(w / Module['forcedAspectRatio']);
          }
        }
        if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
             document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
             document['fullScreenElement'] || document['fullscreenElement'] ||
             document['msFullScreenElement'] || document['msFullscreenElement'] ||
             document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
           var factor = Math.min(screen.width / w, screen.height / h);
           w = Math.round(w * factor);
           h = Math.round(h * factor);
        }
        if (Browser.resizeCanvas) {
          if (canvas.width  != w) canvas.width  = w;
          if (canvas.height != h) canvas.height = h;
          if (typeof canvas.style != 'undefined') {
            canvas.style.removeProperty( "width");
            canvas.style.removeProperty("height");
          }
        } else {
          if (canvas.width  != wNative) canvas.width  = wNative;
          if (canvas.height != hNative) canvas.height = hNative;
          if (typeof canvas.style != 'undefined') {
            if (w != wNative || h != hNative) {
              canvas.style.setProperty( "width", w + "px", "important");
              canvas.style.setProperty("height", h + "px", "important");
            } else {
              canvas.style.removeProperty( "width");
              canvas.style.removeProperty("height");
            }
          }
        }
      },wgetRequests:{},nextWgetRequestHandle:0,getNextWgetRequestHandle:function () {
        var handle = Browser.nextWgetRequestHandle;
        Browser.nextWgetRequestHandle++;
        return handle;
      }};

___errno_state = Runtime.staticAlloc(4); HEAP32[((___errno_state)>>2)]=0;
FS.staticInit();__ATINIT__.unshift(function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() });__ATMAIN__.push(function() { FS.ignorePermissions = false });__ATEXIT__.push(function() { FS.quit() });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;
__ATINIT__.unshift(function() { TTY.init() });__ATEXIT__.push(function() { TTY.shutdown() });
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); var NODEJS_PATH = require("path"); NODEFS.staticInit(); }
__ATINIT__.push(function() { SOCKFS.root = FS.mount(SOCKFS, {}, null); });
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas, vrDevice) { Browser.requestFullScreen(lockPointer, resizeCanvas, vrDevice) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
  Module["createContext"] = function Module_createContext(canvas, useWebGL, setInModule, webGLContextAttributes) { return Browser.createContext(canvas, useWebGL, setInModule, webGLContextAttributes) }
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + TOTAL_STACK;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);


function nullFunc_iiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function invoke_iiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_iiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

Module.asmGlobalArg = { "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array, "NaN": NaN, "Infinity": Infinity };
Module.asmLibraryArg = { "abort": abort, "assert": assert, "nullFunc_iiii": nullFunc_iiii, "invoke_iiii": invoke_iiii, "_fabs": _fabs, "_send": _send, "__reallyNegative": __reallyNegative, "_fflush": _fflush, "_pwrite": _pwrite, "_strerror_r": _strerror_r, "_emscripten_set_main_loop_timing": _emscripten_set_main_loop_timing, "_sbrk": _sbrk, "_emscripten_memcpy_big": _emscripten_memcpy_big, "_fileno": _fileno, "__exit": __exit, "___setErrNo": ___setErrNo, "_mkport": _mkport, "_write": _write, "_emscripten_set_main_loop": _emscripten_set_main_loop, "___errno_location": ___errno_location, "_printf": _printf, "_sysconf": _sysconf, "_abort": _abort, "_fwrite": _fwrite, "_time": _time, "_fprintf": _fprintf, "_strerror": _strerror, "__formatString": __formatString, "_times": _times, "_exit": _exit, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8, "_stderr": _stderr };
// EMSCRIPTEN_START_ASM
var asm = (function(global, env, buffer) {
  'almost asm';
  
  var HEAP8 = new global.Int8Array(buffer);
  var HEAP16 = new global.Int16Array(buffer);
  var HEAP32 = new global.Int32Array(buffer);
  var HEAPU8 = new global.Uint8Array(buffer);
  var HEAPU16 = new global.Uint16Array(buffer);
  var HEAPU32 = new global.Uint32Array(buffer);
  var HEAPF32 = new global.Float32Array(buffer);
  var HEAPF64 = new global.Float64Array(buffer);


  var STACKTOP=env.STACKTOP|0;
  var STACK_MAX=env.STACK_MAX|0;
  var tempDoublePtr=env.tempDoublePtr|0;
  var ABORT=env.ABORT|0;
  var cttz_i8=env.cttz_i8|0;
  var _stderr=env._stderr|0;

  var __THREW__ = 0;
  var threwValue = 0;
  var setjmpId = 0;
  var undef = 0;
  var nan = global.NaN, inf = global.Infinity;
  var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;

  var tempRet0 = 0;
  var tempRet1 = 0;
  var tempRet2 = 0;
  var tempRet3 = 0;
  var tempRet4 = 0;
  var tempRet5 = 0;
  var tempRet6 = 0;
  var tempRet7 = 0;
  var tempRet8 = 0;
  var tempRet9 = 0;
  var Math_floor=global.Math.floor;
  var Math_abs=global.Math.abs;
  var Math_sqrt=global.Math.sqrt;
  var Math_pow=global.Math.pow;
  var Math_cos=global.Math.cos;
  var Math_sin=global.Math.sin;
  var Math_tan=global.Math.tan;
  var Math_acos=global.Math.acos;
  var Math_asin=global.Math.asin;
  var Math_atan=global.Math.atan;
  var Math_atan2=global.Math.atan2;
  var Math_exp=global.Math.exp;
  var Math_log=global.Math.log;
  var Math_ceil=global.Math.ceil;
  var Math_imul=global.Math.imul;
  var Math_min=global.Math.min;
  var Math_clz32=global.Math.clz32;
  var abort=env.abort;
  var assert=env.assert;
  var nullFunc_iiii=env.nullFunc_iiii;
  var invoke_iiii=env.invoke_iiii;
  var _fabs=env._fabs;
  var _send=env._send;
  var __reallyNegative=env.__reallyNegative;
  var _fflush=env._fflush;
  var _pwrite=env._pwrite;
  var _strerror_r=env._strerror_r;
  var _emscripten_set_main_loop_timing=env._emscripten_set_main_loop_timing;
  var _sbrk=env._sbrk;
  var _emscripten_memcpy_big=env._emscripten_memcpy_big;
  var _fileno=env._fileno;
  var __exit=env.__exit;
  var ___setErrNo=env.___setErrNo;
  var _mkport=env._mkport;
  var _write=env._write;
  var _emscripten_set_main_loop=env._emscripten_set_main_loop;
  var ___errno_location=env.___errno_location;
  var _printf=env._printf;
  var _sysconf=env._sysconf;
  var _abort=env._abort;
  var _fwrite=env._fwrite;
  var _time=env._time;
  var _fprintf=env._fprintf;
  var _strerror=env._strerror;
  var __formatString=env.__formatString;
  var _times=env._times;
  var _exit=env._exit;
  var tempFloat = 0.0;

// EMSCRIPTEN_START_FUNCS
function stackAlloc(size) {
  size = size|0;
  var ret = 0;
  ret = STACKTOP;
  STACKTOP = (STACKTOP + size)|0;
  STACKTOP = (STACKTOP + 15)&-16;
if ((STACKTOP|0) >= (STACK_MAX|0)) abort();

  return ret|0;
}
function stackSave() {
  return STACKTOP|0;
}
function stackRestore(top) {
  top = top|0;
  STACKTOP = top;
}
function establishStackSpace(stackBase, stackMax) {
  stackBase = stackBase|0;
  stackMax = stackMax|0;
  STACKTOP = stackBase;
  STACK_MAX = stackMax;
}

function setThrew(threw, value) {
  threw = threw|0;
  value = value|0;
  if ((__THREW__|0) == 0) {
    __THREW__ = threw;
    threwValue = value;
  }
}
function copyTempFloat(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
}
function copyTempDouble(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
  HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
  HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
  HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
  HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
}

function setTempRet0(value) {
  value = value|0;
  tempRet0 = value;
}
function getTempRet0() {
  return tempRet0|0;
}

function _SuperLU_timer_() {
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $2 = 0.0, $3 = 0, $4 = 0, $5 = 0.0, $6 = 0.0, $7 = 0.0, $8 = 0.0, $9 = 0, $clocks_per_sec = 0, $tmp = 0.0, $use = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $use = sp + 16|0;
 $0 = (_sysconf(2)|0);
 $clocks_per_sec = $0;
 (_times(($use|0))|0);
 $1 = HEAP32[$use>>2]|0;
 $2 = (+($1|0));
 $tmp = $2;
 $3 = ((($use)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (+($4|0));
 $6 = $tmp;
 $7 = $6 + $5;
 $tmp = $7;
 $8 = $tmp;
 $9 = $clocks_per_sec;
 $10 = (+($9|0));
 $11 = $8 / $10;
 STACKTOP = sp;return (+$11);
}
function _superlu_abort_and_exit($msg) {
 $msg = $msg|0;
 var $0 = 0, $1 = 0, $2 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $0 = $msg;
 $1 = HEAP32[_stderr>>2]|0;
 $2 = $0;
 (_fprintf(($1|0),($2|0),($vararg_buffer|0))|0);
 _exit(-1);
 // unreachable;
}
function _set_default_options($options) {
 $options = $options|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $options;
 $1 = $0;
 HEAP32[$1>>2] = 0;
 $2 = $0;
 $3 = ((($2)) + 4|0);
 HEAP32[$3>>2] = 1;
 $4 = $0;
 $5 = ((($4)) + 8|0);
 HEAP32[$5>>2] = 3;
 $6 = $0;
 $7 = ((($6)) + 12|0);
 HEAP32[$7>>2] = 0;
 $8 = $0;
 $9 = ((($8)) + 16|0);
 HEAP32[$9>>2] = 0;
 $10 = $0;
 $11 = ((($10)) + 24|0);
 HEAPF64[$11>>3] = 1.0;
 $12 = $0;
 $13 = ((($12)) + 32|0);
 HEAP32[$13>>2] = 0;
 $14 = $0;
 $15 = ((($14)) + 36|0);
 HEAP32[$15>>2] = 0;
 $16 = $0;
 $17 = ((($16)) + 40|0);
 HEAP32[$17>>2] = 0;
 $18 = $0;
 $19 = ((($18)) + 120|0);
 HEAP32[$19>>2] = 1;
 STACKTOP = sp;return;
}
function _Destroy_SuperMatrix_Store($A) {
 $A = $A|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $A;
 $1 = $0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 _superlu_free($3);
 STACKTOP = sp;return;
}
function _Destroy_CompCol_Matrix($A) {
 $A = $A|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $A;
 $1 = $0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 _superlu_free($5);
 $6 = $0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 _superlu_free($10);
 $11 = $0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 _superlu_free($15);
 $16 = $0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 _superlu_free($18);
 STACKTOP = sp;return;
}
function _Destroy_SuperNode_Matrix($A) {
 $A = $A|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $A;
 $1 = $0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 _superlu_free($5);
 $6 = $0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 20|0);
 $10 = HEAP32[$9>>2]|0;
 _superlu_free($10);
 $11 = $0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 _superlu_free($15);
 $16 = $0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 _superlu_free($20);
 $21 = $0;
 $22 = ((($21)) + 20|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 24|0);
 $25 = HEAP32[$24>>2]|0;
 _superlu_free($25);
 $26 = $0;
 $27 = ((($26)) + 20|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ((($28)) + 28|0);
 $30 = HEAP32[$29>>2]|0;
 _superlu_free($30);
 $31 = $0;
 $32 = ((($31)) + 20|0);
 $33 = HEAP32[$32>>2]|0;
 _superlu_free($33);
 STACKTOP = sp;return;
}
function _Destroy_CompCol_Permuted($A) {
 $A = $A|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $A;
 $1 = $0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 _superlu_free($5);
 $6 = $0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 _superlu_free($10);
 $11 = $0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 _superlu_free($13);
 STACKTOP = sp;return;
}
function _resetrep_col($nseg,$segrep,$repfnz) {
 $nseg = $nseg|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $irep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $nseg;
 $1 = $segrep;
 $2 = $repfnz;
 $i = 0;
 while(1) {
  $3 = $i;
  $4 = $0;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $i;
  $7 = $1;
  $8 = (($7) + ($6<<2)|0);
  $9 = HEAP32[$8>>2]|0;
  $irep = $9;
  $10 = $irep;
  $11 = $2;
  $12 = (($11) + ($10<<2)|0);
  HEAP32[$12>>2] = -1;
  $13 = $i;
  $14 = (($13) + 1)|0;
  $i = $14;
 }
 STACKTOP = sp;return;
}
function _countnz($n,$xprune,$nnzL,$nnzU,$Glu) {
 $n = $n|0;
 $xprune = $xprune|0;
 $nnzL = $nnzL|0;
 $nnzU = $nnzU|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $fsupc = 0, $i = 0, $irep = 0, $j = 0, $jlen = 0, $nnzL0 = 0, $nsuper = 0, $xlsub = 0, $xsup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $xprune;
 $2 = $nnzL;
 $3 = $nnzU;
 $4 = $Glu;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $xsup = $6;
 $7 = $4;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $xlsub = $9;
 $10 = $2;
 HEAP32[$10>>2] = 0;
 $11 = $0;
 $12 = $4;
 $13 = ((($12)) + 32|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<2)|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $3;
 HEAP32[$17>>2] = $16;
 $nnzL0 = 0;
 $18 = $0;
 $19 = $4;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + ($18<<2)|0);
 $23 = HEAP32[$22>>2]|0;
 $nsuper = $23;
 $24 = $0;
 $25 = ($24|0)<=(0);
 if ($25) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $26 = $i;
  $27 = $nsuper;
  $28 = ($26|0)<=($27|0);
  if (!($28)) {
   break;
  }
  $29 = $i;
  $30 = $xsup;
  $31 = (($30) + ($29<<2)|0);
  $32 = HEAP32[$31>>2]|0;
  $fsupc = $32;
  $33 = $fsupc;
  $34 = (($33) + 1)|0;
  $35 = $xlsub;
  $36 = (($35) + ($34<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $fsupc;
  $39 = $xlsub;
  $40 = (($39) + ($38<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($37) - ($41))|0;
  $jlen = $42;
  $43 = $fsupc;
  $j = $43;
  while(1) {
   $44 = $j;
   $45 = $i;
   $46 = (($45) + 1)|0;
   $47 = $xsup;
   $48 = (($47) + ($46<<2)|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ($44|0)<($49|0);
   if (!($50)) {
    break;
   }
   $51 = $jlen;
   $52 = $2;
   $53 = HEAP32[$52>>2]|0;
   $54 = (($53) + ($51))|0;
   HEAP32[$52>>2] = $54;
   $55 = $j;
   $56 = $fsupc;
   $57 = (($55) - ($56))|0;
   $58 = (($57) + 1)|0;
   $59 = $3;
   $60 = HEAP32[$59>>2]|0;
   $61 = (($60) + ($58))|0;
   HEAP32[$59>>2] = $61;
   $62 = $jlen;
   $63 = (($62) + -1)|0;
   $jlen = $63;
   $64 = $j;
   $65 = (($64) + 1)|0;
   $j = $65;
  }
  $66 = $i;
  $67 = (($66) + 1)|0;
  $68 = $xsup;
  $69 = (($68) + ($67<<2)|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (($70) - 1)|0;
  $irep = $71;
  $72 = $irep;
  $73 = $1;
  $74 = (($73) + ($72<<2)|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = $irep;
  $77 = $xlsub;
  $78 = (($77) + ($76<<2)|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (($75) - ($79))|0;
  $81 = $nnzL0;
  $82 = (($81) + ($80))|0;
  $nnzL0 = $82;
  $83 = $i;
  $84 = (($83) + 1)|0;
  $i = $84;
 }
 STACKTOP = sp;return;
}
function _fixupL($n,$perm_r,$Glu) {
 $n = $n|0;
 $perm_r = $perm_r|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $fsupc = 0, $i = 0, $j = 0;
 var $jstrt = 0, $k = 0, $lsub = 0, $nextl = 0, $nsuper = 0, $xlsub = 0, $xsup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $perm_r;
 $2 = $Glu;
 $3 = $0;
 $4 = ($3|0)<=(1);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = $2;
 $6 = HEAP32[$5>>2]|0;
 $xsup = $6;
 $7 = $2;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $lsub = $9;
 $10 = $2;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $xlsub = $12;
 $nextl = 0;
 $13 = $0;
 $14 = $2;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + ($13<<2)|0);
 $18 = HEAP32[$17>>2]|0;
 $nsuper = $18;
 $i = 0;
 while(1) {
  $19 = $i;
  $20 = $nsuper;
  $21 = ($19|0)<=($20|0);
  if (!($21)) {
   break;
  }
  $22 = $i;
  $23 = $xsup;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $fsupc = $25;
  $26 = $fsupc;
  $27 = $xlsub;
  $28 = (($27) + ($26<<2)|0);
  $29 = HEAP32[$28>>2]|0;
  $jstrt = $29;
  $30 = $nextl;
  $31 = $fsupc;
  $32 = $xlsub;
  $33 = (($32) + ($31<<2)|0);
  HEAP32[$33>>2] = $30;
  $34 = $jstrt;
  $j = $34;
  while(1) {
   $35 = $j;
   $36 = $fsupc;
   $37 = (($36) + 1)|0;
   $38 = $xlsub;
   $39 = (($38) + ($37<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($35|0)<($40|0);
   if (!($41)) {
    break;
   }
   $42 = $j;
   $43 = $lsub;
   $44 = (($43) + ($42<<2)|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = $1;
   $47 = (($46) + ($45<<2)|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = $nextl;
   $50 = $lsub;
   $51 = (($50) + ($49<<2)|0);
   HEAP32[$51>>2] = $48;
   $52 = $nextl;
   $53 = (($52) + 1)|0;
   $nextl = $53;
   $54 = $j;
   $55 = (($54) + 1)|0;
   $j = $55;
  }
  $56 = $fsupc;
  $57 = (($56) + 1)|0;
  $k = $57;
  while(1) {
   $58 = $k;
   $59 = $i;
   $60 = (($59) + 1)|0;
   $61 = $xsup;
   $62 = (($61) + ($60<<2)|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($58|0)<($63|0);
   if (!($64)) {
    break;
   }
   $65 = $nextl;
   $66 = $k;
   $67 = $xlsub;
   $68 = (($67) + ($66<<2)|0);
   HEAP32[$68>>2] = $65;
   $69 = $k;
   $70 = (($69) + 1)|0;
   $k = $70;
  }
  $71 = $i;
  $72 = (($71) + 1)|0;
  $i = $72;
 }
 $73 = $nextl;
 $74 = $0;
 $75 = $xlsub;
 $76 = (($75) + ($74<<2)|0);
 HEAP32[$76>>2] = $73;
 STACKTOP = sp;return;
}
function _StatInit($stat) {
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $msg = 0, $msg1 = 0, $panel_size = 0, $relax = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $w = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 560|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $msg = sp + 304|0;
 $msg1 = sp + 48|0;
 $0 = $stat;
 $1 = (_sp_ienv(1)|0);
 $panel_size = $1;
 $2 = (_sp_ienv(2)|0);
 $relax = $2;
 $3 = $panel_size;
 $4 = $relax;
 $5 = ($3|0)>($4|0);
 $6 = $panel_size;
 $7 = $relax;
 $8 = $5 ? $6 : $7;
 $w = $8;
 $9 = $w;
 $10 = (($9) + 1)|0;
 $11 = (_intCalloc($10)|0);
 $12 = $0;
 HEAP32[$12>>2] = $11;
 $13 = (_superlu_malloc(128)|0);
 $14 = $0;
 $15 = ((($14)) + 4|0);
 HEAP32[$15>>2] = $13;
 $16 = $0;
 $17 = ((($16)) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  HEAP32[$vararg_buffer>>2] = 8;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 305;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 48;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $20 = (_superlu_malloc(64)|0);
 $21 = $0;
 $22 = ((($21)) + 8|0);
 HEAP32[$22>>2] = $20;
 $23 = $0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  HEAP32[$vararg_buffer3>>2] = 56;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = 307;
  $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
  HEAP32[$vararg_ptr7>>2] = 48;
  (_sprintf($msg1,2864,$vararg_buffer3)|0);
  _superlu_abort_and_exit($msg1);
 }
 $i = 0;
 while(1) {
  $27 = $i;
  $28 = ($27|0)<(16);
  if (!($28)) {
   break;
  }
  $29 = $i;
  $30 = $0;
  $31 = ((($30)) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + ($29<<3)|0);
  HEAPF64[$33>>3] = 0.0;
  $34 = $i;
  $35 = $0;
  $36 = ((($35)) + 8|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<2)|0);
  HEAPF32[$38>>2] = 0.0;
  $39 = $i;
  $40 = (($39) + 1)|0;
  $i = $40;
 }
 $41 = $0;
 $42 = ((($41)) + 12|0);
 HEAP32[$42>>2] = 0;
 $43 = $0;
 $44 = ((($43)) + 16|0);
 HEAP32[$44>>2] = 0;
 $45 = $0;
 $46 = ((($45)) + 20|0);
 HEAP32[$46>>2] = 0;
 STACKTOP = sp;return;
}
function _StatFree($stat) {
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $stat;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 _superlu_free($2);
 $3 = $0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 _superlu_free($5);
 $6 = $0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 _superlu_free($8);
 STACKTOP = sp;return;
}
function _ifill($a,$alen,$ival) {
 $a = $a|0;
 $alen = $alen|0;
 $ival = $ival|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $a;
 $1 = $alen;
 $2 = $ival;
 $i = 0;
 while(1) {
  $3 = $i;
  $4 = $1;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $2;
  $7 = $i;
  $8 = $0;
  $9 = (($8) + ($7<<2)|0);
  HEAP32[$9>>2] = $6;
  $10 = $i;
  $11 = (($10) + 1)|0;
  $i = $11;
 }
 STACKTOP = sp;return;
}
function _superlu_malloc($size) {
 $size = $size|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $size;
 $1 = $0;
 $2 = (_malloc($1)|0);
 $buf = $2;
 $3 = $buf;
 STACKTOP = sp;return ($3|0);
}
function _superlu_free($addr) {
 $addr = $addr|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $addr;
 $1 = $0;
 _free($1);
 STACKTOP = sp;return;
}
function _SetIWork($m,$n,$panel_size,$iworkptr,$segrep,$parent,$xplore,$repfnz,$panel_lsub,$xprune,$marker) {
 $m = $m|0;
 $n = $n|0;
 $panel_size = $panel_size|0;
 $iworkptr = $iworkptr|0;
 $segrep = $segrep|0;
 $parent = $parent|0;
 $xplore = $xplore|0;
 $repfnz = $repfnz|0;
 $panel_lsub = $panel_lsub|0;
 $xprune = $xprune|0;
 $marker = $marker|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $m;
 $1 = $n;
 $2 = $panel_size;
 $3 = $iworkptr;
 $4 = $segrep;
 $5 = $parent;
 $6 = $xplore;
 $7 = $repfnz;
 $8 = $panel_lsub;
 $9 = $xprune;
 $10 = $marker;
 $11 = $3;
 $12 = $4;
 HEAP32[$12>>2] = $11;
 $13 = $3;
 $14 = $0;
 $15 = (($13) + ($14<<2)|0);
 $16 = $5;
 HEAP32[$16>>2] = $15;
 $17 = $5;
 $18 = HEAP32[$17>>2]|0;
 $19 = $0;
 $20 = (($18) + ($19<<2)|0);
 $21 = $6;
 HEAP32[$21>>2] = $20;
 $22 = $6;
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 $25 = (($23) + ($24<<2)|0);
 $26 = $7;
 HEAP32[$26>>2] = $25;
 $27 = $7;
 $28 = HEAP32[$27>>2]|0;
 $29 = $2;
 $30 = $0;
 $31 = Math_imul($29, $30)|0;
 $32 = (($28) + ($31<<2)|0);
 $33 = $8;
 HEAP32[$33>>2] = $32;
 $34 = $8;
 $35 = HEAP32[$34>>2]|0;
 $36 = $2;
 $37 = $0;
 $38 = Math_imul($36, $37)|0;
 $39 = (($35) + ($38<<2)|0);
 $40 = $9;
 HEAP32[$40>>2] = $39;
 $41 = $9;
 $42 = HEAP32[$41>>2]|0;
 $43 = $1;
 $44 = (($42) + ($43<<2)|0);
 $45 = $10;
 HEAP32[$45>>2] = $44;
 $46 = $7;
 $47 = HEAP32[$46>>2]|0;
 $48 = $0;
 $49 = $2;
 $50 = Math_imul($48, $49)|0;
 _ifill($47,$50,-1);
 $51 = $8;
 $52 = HEAP32[$51>>2]|0;
 $53 = $0;
 $54 = $2;
 $55 = Math_imul($53, $54)|0;
 _ifill($52,$55,-1);
 STACKTOP = sp;return;
}
function _copy_mem_int($howmany,$old,$new) {
 $howmany = $howmany|0;
 $old = $old|0;
 $new = $new|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $inew = 0, $iold = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $howmany;
 $1 = $old;
 $2 = $new;
 $3 = $1;
 $iold = $3;
 $4 = $2;
 $inew = $4;
 $i = 0;
 while(1) {
  $5 = $i;
  $6 = $0;
  $7 = ($5|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = $i;
  $9 = $iold;
  $10 = (($9) + ($8<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $i;
  $13 = $inew;
  $14 = (($13) + ($12<<2)|0);
  HEAP32[$14>>2] = $11;
  $15 = $i;
  $16 = (($15) + 1)|0;
  $i = $16;
 }
 STACKTOP = sp;return;
}
function _user_bcopy($src,$dest,$bytes) {
 $src = $src|0;
 $dest = $dest|0;
 $bytes = $bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $d_ptr = 0, $s_ptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $src;
 $1 = $dest;
 $2 = $bytes;
 $3 = $0;
 $4 = $2;
 $5 = (($3) + ($4)|0);
 $6 = ((($5)) + -1|0);
 $s_ptr = $6;
 $7 = $1;
 $8 = $2;
 $9 = (($7) + ($8)|0);
 $10 = ((($9)) + -1|0);
 $d_ptr = $10;
 while(1) {
  $11 = $d_ptr;
  $12 = $1;
  $13 = ($11>>>0)>=($12>>>0);
  if (!($13)) {
   break;
  }
  $14 = $s_ptr;
  $15 = HEAP8[$14>>0]|0;
  $16 = $d_ptr;
  HEAP8[$16>>0] = $15;
  $17 = $s_ptr;
  $18 = ((($17)) + -1|0);
  $s_ptr = $18;
  $19 = $d_ptr;
  $20 = ((($19)) + -1|0);
  $d_ptr = $20;
 }
 STACKTOP = sp;return;
}
function _intMalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 24|0;
 $0 = $n;
 $1 = $0;
 $2 = $1<<2;
 $3 = (_superlu_malloc($2)|0);
 $buf = $3;
 $4 = $buf;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = $buf;
  STACKTOP = sp;return ($6|0);
 }
 HEAP32[$vararg_buffer>>2] = 96;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 138;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = 144;
 (_sprintf($msg,2864,$vararg_buffer)|0);
 _superlu_abort_and_exit($msg);
 $6 = $buf;
 STACKTOP = sp;return ($6|0);
}
function _intCalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $i = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0;
 var $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 24|0;
 $0 = $n;
 $1 = $0;
 $2 = $1<<2;
 $3 = (_superlu_malloc($2)|0);
 $buf = $3;
 $4 = $buf;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  HEAP32[$vararg_buffer>>2] = 160;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 149;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 144;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $i = 0;
 while(1) {
  $6 = $i;
  $7 = $0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $i;
  $10 = $buf;
  $11 = (($10) + ($9<<2)|0);
  HEAP32[$11>>2] = 0;
  $12 = $i;
  $13 = (($12) + 1)|0;
  $i = $13;
 }
 $14 = $buf;
 STACKTOP = sp;return ($14|0);
}
function _get_colamd($m,$n,$nnz,$colptr,$rowind,$perm_c) {
 $m = $m|0;
 $n = $n|0;
 $nnz = $nnz|0;
 $colptr = $colptr|0;
 $rowind = $rowind|0;
 $perm_c = $perm_c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $A = 0, $Alen = 0, $i = 0, $info = 0, $knobs = 0, $msg = 0, $msg1 = 0, $msg2 = 0, $p = 0, $stats = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $vararg_ptr11 = 0;
 var $vararg_ptr12 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1104|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer8 = sp + 192|0;
 $vararg_buffer3 = sp + 176|0;
 $vararg_buffer = sp + 160|0;
 $knobs = sp;
 $stats = sp + 208|0;
 $msg = sp + 848|0;
 $msg1 = sp + 592|0;
 $msg2 = sp + 336|0;
 $0 = $m;
 $1 = $n;
 $2 = $nnz;
 $3 = $colptr;
 $4 = $rowind;
 $5 = $perm_c;
 $6 = $2;
 $7 = $0;
 $8 = $1;
 $9 = (_colamd_recommended($6,$7,$8)|0);
 $Alen = $9;
 _colamd_set_defaults($knobs);
 $10 = $Alen;
 $11 = $10<<2;
 $12 = (_superlu_malloc($11)|0);
 $A = $12;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  HEAP32[$vararg_buffer>>2] = 208;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 36;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 232;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $14 = $1;
 $15 = (($14) + 1)|0;
 $16 = $15<<2;
 $17 = (_superlu_malloc($16)|0);
 $p = $17;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  HEAP32[$vararg_buffer3>>2] = 248;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = 38;
  $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
  HEAP32[$vararg_ptr7>>2] = 232;
  (_sprintf($msg1,2864,$vararg_buffer3)|0);
  _superlu_abort_and_exit($msg1);
 }
 $i = 0;
 while(1) {
  $19 = $i;
  $20 = $1;
  $21 = ($19|0)<=($20|0);
  if (!($21)) {
   break;
  }
  $22 = $i;
  $23 = $3;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $i;
  $27 = $p;
  $28 = (($27) + ($26<<2)|0);
  HEAP32[$28>>2] = $25;
  $29 = $i;
  $30 = (($29) + 1)|0;
  $i = $30;
 }
 $i = 0;
 while(1) {
  $31 = $i;
  $32 = $2;
  $33 = ($31|0)<($32|0);
  if (!($33)) {
   break;
  }
  $34 = $i;
  $35 = $4;
  $36 = (($35) + ($34<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $i;
  $39 = $A;
  $40 = (($39) + ($38<<2)|0);
  HEAP32[$40>>2] = $37;
  $41 = $i;
  $42 = (($41) + 1)|0;
  $i = $42;
 }
 $43 = $0;
 $44 = $1;
 $45 = $Alen;
 $46 = $A;
 $47 = $p;
 $48 = (_colamd($43,$44,$45,$46,$47,$knobs,$stats)|0);
 $info = $48;
 $49 = $info;
 $50 = ($49|0)==(0);
 if ($50) {
  HEAP32[$vararg_buffer8>>2] = 272;
  $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
  HEAP32[$vararg_ptr11>>2] = 42;
  $vararg_ptr12 = ((($vararg_buffer8)) + 8|0);
  HEAP32[$vararg_ptr12>>2] = 232;
  (_sprintf($msg2,2864,$vararg_buffer8)|0);
  _superlu_abort_and_exit($msg2);
 }
 $i = 0;
 while(1) {
  $51 = $i;
  $52 = $1;
  $53 = ($51|0)<($52|0);
  if (!($53)) {
   break;
  }
  $54 = $i;
  $55 = $i;
  $56 = $p;
  $57 = (($56) + ($55<<2)|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = $5;
  $60 = (($59) + ($58<<2)|0);
  HEAP32[$60>>2] = $54;
  $61 = $i;
  $62 = (($61) + 1)|0;
  $i = $62;
 }
 $63 = $A;
 _superlu_free($63);
 $64 = $p;
 _superlu_free($64);
 STACKTOP = sp;return;
}
function _getata($m,$n,$nz,$colptr,$rowind,$atanz,$ata_colptr,$ata_rowind) {
 $m = $m|0;
 $n = $n|0;
 $nz = $nz|0;
 $colptr = $colptr|0;
 $rowind = $rowind|0;
 $atanz = $atanz|0;
 $ata_colptr = $ata_colptr|0;
 $ata_rowind = $ata_rowind|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $b_colptr = 0, $b_rowind = 0, $col = 0, $i = 0, $j = 0, $k = 0, $marker = 0, $msg = 0, $msg1 = 0, $msg2 = 0, $msg3 = 0, $msg4 = 0, $num_nz = 0, $t_colptr = 0, $t_rowind = 0, $ti = 0, $trow = 0, $vararg_buffer = 0, $vararg_buffer13 = 0;
 var $vararg_buffer18 = 0, $vararg_buffer3 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr2 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1440|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer18 = sp + 64|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $msg = sp + 1184|0;
 $msg1 = sp + 928|0;
 $msg2 = sp + 672|0;
 $msg3 = sp + 416|0;
 $msg4 = sp + 160|0;
 $0 = $m;
 $1 = $n;
 $2 = $nz;
 $3 = $colptr;
 $4 = $rowind;
 $5 = $atanz;
 $6 = $ata_colptr;
 $7 = $ata_rowind;
 $8 = $0;
 $9 = $1;
 $10 = ($8|0)>($9|0);
 $11 = $0;
 $12 = $1;
 $13 = $10 ? $11 : $12;
 $14 = (($13) + 1)|0;
 $15 = $14<<2;
 $16 = (_superlu_malloc($15)|0);
 $marker = $16;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  HEAP32[$vararg_buffer>>2] = 288;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 88;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 232;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $18 = $0;
 $19 = (($18) + 1)|0;
 $20 = $19<<2;
 $21 = (_superlu_malloc($20)|0);
 $t_colptr = $21;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  HEAP32[$vararg_buffer3>>2] = 328;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = 90;
  $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
  HEAP32[$vararg_ptr7>>2] = 232;
  (_sprintf($msg1,2864,$vararg_buffer3)|0);
  _superlu_abort_and_exit($msg1);
 }
 $23 = $2;
 $24 = $23<<2;
 $25 = (_superlu_malloc($24)|0);
 $t_rowind = $25;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  HEAP32[$vararg_buffer8>>2] = 360;
  $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
  HEAP32[$vararg_ptr11>>2] = 92;
  $vararg_ptr12 = ((($vararg_buffer8)) + 8|0);
  HEAP32[$vararg_ptr12>>2] = 232;
  (_sprintf($msg2,2864,$vararg_buffer8)|0);
  _superlu_abort_and_exit($msg2);
 }
 $i = 0;
 while(1) {
  $27 = $i;
  $28 = $0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = $i;
  $31 = $marker;
  $32 = (($31) + ($30<<2)|0);
  HEAP32[$32>>2] = 0;
  $33 = $i;
  $34 = (($33) + 1)|0;
  $i = $34;
 }
 $j = 0;
 while(1) {
  $35 = $j;
  $36 = $1;
  $37 = ($35|0)<($36|0);
  if (!($37)) {
   break;
  }
  $38 = $j;
  $39 = $3;
  $40 = (($39) + ($38<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $i = $41;
  while(1) {
   $42 = $i;
   $43 = $j;
   $44 = (($43) + 1)|0;
   $45 = $3;
   $46 = (($45) + ($44<<2)|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ($42|0)<($47|0);
   if (!($48)) {
    break;
   }
   $49 = $i;
   $50 = $4;
   $51 = (($50) + ($49<<2)|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = $marker;
   $54 = (($53) + ($52<<2)|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$54>>2] = $56;
   $57 = $i;
   $58 = (($57) + 1)|0;
   $i = $58;
  }
  $59 = $j;
  $60 = (($59) + 1)|0;
  $j = $60;
 }
 $61 = $t_colptr;
 HEAP32[$61>>2] = 0;
 $i = 0;
 while(1) {
  $62 = $i;
  $63 = $0;
  $64 = ($62|0)<($63|0);
  if (!($64)) {
   break;
  }
  $65 = $i;
  $66 = $t_colptr;
  $67 = (($66) + ($65<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = $i;
  $70 = $marker;
  $71 = (($70) + ($69<<2)|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($68) + ($72))|0;
  $74 = $i;
  $75 = (($74) + 1)|0;
  $76 = $t_colptr;
  $77 = (($76) + ($75<<2)|0);
  HEAP32[$77>>2] = $73;
  $78 = $i;
  $79 = $t_colptr;
  $80 = (($79) + ($78<<2)|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = $i;
  $83 = $marker;
  $84 = (($83) + ($82<<2)|0);
  HEAP32[$84>>2] = $81;
  $85 = $i;
  $86 = (($85) + 1)|0;
  $i = $86;
 }
 $j = 0;
 while(1) {
  $87 = $j;
  $88 = $1;
  $89 = ($87|0)<($88|0);
  if (!($89)) {
   break;
  }
  $90 = $j;
  $91 = $3;
  $92 = (($91) + ($90<<2)|0);
  $93 = HEAP32[$92>>2]|0;
  $i = $93;
  while(1) {
   $94 = $i;
   $95 = $j;
   $96 = (($95) + 1)|0;
   $97 = $3;
   $98 = (($97) + ($96<<2)|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = ($94|0)<($99|0);
   if (!($100)) {
    break;
   }
   $101 = $i;
   $102 = $4;
   $103 = (($102) + ($101<<2)|0);
   $104 = HEAP32[$103>>2]|0;
   $col = $104;
   $105 = $j;
   $106 = $col;
   $107 = $marker;
   $108 = (($107) + ($106<<2)|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = $t_rowind;
   $111 = (($110) + ($109<<2)|0);
   HEAP32[$111>>2] = $105;
   $112 = $col;
   $113 = $marker;
   $114 = (($113) + ($112<<2)|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = (($115) + 1)|0;
   HEAP32[$114>>2] = $116;
   $117 = $i;
   $118 = (($117) + 1)|0;
   $i = $118;
  }
  $119 = $j;
  $120 = (($119) + 1)|0;
  $j = $120;
 }
 $i = 0;
 while(1) {
  $121 = $i;
  $122 = $1;
  $123 = ($121|0)<($122|0);
  if (!($123)) {
   break;
  }
  $124 = $i;
  $125 = $marker;
  $126 = (($125) + ($124<<2)|0);
  HEAP32[$126>>2] = -1;
  $127 = $i;
  $128 = (($127) + 1)|0;
  $i = $128;
 }
 $num_nz = 0;
 $j = 0;
 while(1) {
  $129 = $j;
  $130 = $1;
  $131 = ($129|0)<($130|0);
  if (!($131)) {
   break;
  }
  $132 = $j;
  $133 = $j;
  $134 = $marker;
  $135 = (($134) + ($133<<2)|0);
  HEAP32[$135>>2] = $132;
  $136 = $j;
  $137 = $3;
  $138 = (($137) + ($136<<2)|0);
  $139 = HEAP32[$138>>2]|0;
  $i = $139;
  while(1) {
   $140 = $i;
   $141 = $j;
   $142 = (($141) + 1)|0;
   $143 = $3;
   $144 = (($143) + ($142<<2)|0);
   $145 = HEAP32[$144>>2]|0;
   $146 = ($140|0)<($145|0);
   if (!($146)) {
    break;
   }
   $147 = $i;
   $148 = $4;
   $149 = (($148) + ($147<<2)|0);
   $150 = HEAP32[$149>>2]|0;
   $k = $150;
   $151 = $k;
   $152 = $t_colptr;
   $153 = (($152) + ($151<<2)|0);
   $154 = HEAP32[$153>>2]|0;
   $ti = $154;
   while(1) {
    $155 = $ti;
    $156 = $k;
    $157 = (($156) + 1)|0;
    $158 = $t_colptr;
    $159 = (($158) + ($157<<2)|0);
    $160 = HEAP32[$159>>2]|0;
    $161 = ($155|0)<($160|0);
    if (!($161)) {
     break;
    }
    $162 = $ti;
    $163 = $t_rowind;
    $164 = (($163) + ($162<<2)|0);
    $165 = HEAP32[$164>>2]|0;
    $trow = $165;
    $166 = $trow;
    $167 = $marker;
    $168 = (($167) + ($166<<2)|0);
    $169 = HEAP32[$168>>2]|0;
    $170 = $j;
    $171 = ($169|0)!=($170|0);
    if ($171) {
     $172 = $j;
     $173 = $trow;
     $174 = $marker;
     $175 = (($174) + ($173<<2)|0);
     HEAP32[$175>>2] = $172;
     $176 = $num_nz;
     $177 = (($176) + 1)|0;
     $num_nz = $177;
    }
    $178 = $ti;
    $179 = (($178) + 1)|0;
    $ti = $179;
   }
   $180 = $i;
   $181 = (($180) + 1)|0;
   $i = $181;
  }
  $182 = $j;
  $183 = (($182) + 1)|0;
  $j = $183;
 }
 $184 = $num_nz;
 $185 = $5;
 HEAP32[$185>>2] = $184;
 $186 = $1;
 $187 = (($186) + 1)|0;
 $188 = $187<<2;
 $189 = (_superlu_malloc($188)|0);
 $190 = $6;
 HEAP32[$190>>2] = $189;
 $191 = ($189|0)!=(0|0);
 if (!($191)) {
  HEAP32[$vararg_buffer13>>2] = 400;
  $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
  HEAP32[$vararg_ptr16>>2] = 154;
  $vararg_ptr17 = ((($vararg_buffer13)) + 8|0);
  HEAP32[$vararg_ptr17>>2] = 232;
  (_sprintf($msg3,2864,$vararg_buffer13)|0);
  _superlu_abort_and_exit($msg3);
 }
 $192 = $5;
 $193 = HEAP32[$192>>2]|0;
 $194 = ($193|0)!=(0);
 if ($194) {
  $195 = $5;
  $196 = HEAP32[$195>>2]|0;
  $197 = $196<<2;
  $198 = (_superlu_malloc($197)|0);
  $199 = $7;
  HEAP32[$199>>2] = $198;
  $200 = ($198|0)!=(0|0);
  if (!($200)) {
   HEAP32[$vararg_buffer18>>2] = 440;
   $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
   HEAP32[$vararg_ptr21>>2] = 157;
   $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
   HEAP32[$vararg_ptr22>>2] = 232;
   (_sprintf($msg4,2864,$vararg_buffer18)|0);
   _superlu_abort_and_exit($msg4);
  }
 }
 $201 = $6;
 $202 = HEAP32[$201>>2]|0;
 $b_colptr = $202;
 $203 = $7;
 $204 = HEAP32[$203>>2]|0;
 $b_rowind = $204;
 $i = 0;
 while(1) {
  $205 = $i;
  $206 = $1;
  $207 = ($205|0)<($206|0);
  if (!($207)) {
   break;
  }
  $208 = $i;
  $209 = $marker;
  $210 = (($209) + ($208<<2)|0);
  HEAP32[$210>>2] = -1;
  $211 = $i;
  $212 = (($211) + 1)|0;
  $i = $212;
 }
 $num_nz = 0;
 $j = 0;
 while(1) {
  $213 = $j;
  $214 = $1;
  $215 = ($213|0)<($214|0);
  $216 = $num_nz;
  if (!($215)) {
   break;
  }
  $217 = $j;
  $218 = $b_colptr;
  $219 = (($218) + ($217<<2)|0);
  HEAP32[$219>>2] = $216;
  $220 = $j;
  $221 = $j;
  $222 = $marker;
  $223 = (($222) + ($221<<2)|0);
  HEAP32[$223>>2] = $220;
  $224 = $j;
  $225 = $3;
  $226 = (($225) + ($224<<2)|0);
  $227 = HEAP32[$226>>2]|0;
  $i = $227;
  while(1) {
   $228 = $i;
   $229 = $j;
   $230 = (($229) + 1)|0;
   $231 = $3;
   $232 = (($231) + ($230<<2)|0);
   $233 = HEAP32[$232>>2]|0;
   $234 = ($228|0)<($233|0);
   if (!($234)) {
    break;
   }
   $235 = $i;
   $236 = $4;
   $237 = (($236) + ($235<<2)|0);
   $238 = HEAP32[$237>>2]|0;
   $k = $238;
   $239 = $k;
   $240 = $t_colptr;
   $241 = (($240) + ($239<<2)|0);
   $242 = HEAP32[$241>>2]|0;
   $ti = $242;
   while(1) {
    $243 = $ti;
    $244 = $k;
    $245 = (($244) + 1)|0;
    $246 = $t_colptr;
    $247 = (($246) + ($245<<2)|0);
    $248 = HEAP32[$247>>2]|0;
    $249 = ($243|0)<($248|0);
    if (!($249)) {
     break;
    }
    $250 = $ti;
    $251 = $t_rowind;
    $252 = (($251) + ($250<<2)|0);
    $253 = HEAP32[$252>>2]|0;
    $trow = $253;
    $254 = $trow;
    $255 = $marker;
    $256 = (($255) + ($254<<2)|0);
    $257 = HEAP32[$256>>2]|0;
    $258 = $j;
    $259 = ($257|0)!=($258|0);
    if ($259) {
     $260 = $j;
     $261 = $trow;
     $262 = $marker;
     $263 = (($262) + ($261<<2)|0);
     HEAP32[$263>>2] = $260;
     $264 = $trow;
     $265 = $num_nz;
     $266 = (($265) + 1)|0;
     $num_nz = $266;
     $267 = $b_rowind;
     $268 = (($267) + ($265<<2)|0);
     HEAP32[$268>>2] = $264;
    }
    $269 = $ti;
    $270 = (($269) + 1)|0;
    $ti = $270;
   }
   $271 = $i;
   $272 = (($271) + 1)|0;
   $i = $272;
  }
  $273 = $j;
  $274 = (($273) + 1)|0;
  $j = $274;
 }
 $275 = $1;
 $276 = $b_colptr;
 $277 = (($276) + ($275<<2)|0);
 HEAP32[$277>>2] = $216;
 $278 = $marker;
 _superlu_free($278);
 $279 = $t_colptr;
 _superlu_free($279);
 $280 = $t_rowind;
 _superlu_free($280);
 STACKTOP = sp;return;
}
function _at_plus_a($n,$nz,$colptr,$rowind,$bnz,$b_colptr,$b_rowind) {
 $n = $n|0;
 $nz = $nz|0;
 $colptr = $colptr|0;
 $rowind = $rowind|0;
 $bnz = $bnz|0;
 $b_colptr = $b_colptr|0;
 $b_rowind = $b_rowind|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $col = 0, $i = 0, $j = 0, $k = 0, $marker = 0, $msg = 0, $msg1 = 0, $msg2 = 0, $msg3 = 0, $msg4 = 0, $num_nz = 0, $t_colptr = 0, $t_rowind = 0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer18 = 0, $vararg_buffer3 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0;
 var $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr2 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1424|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer18 = sp + 64|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $msg = sp + 1160|0;
 $msg1 = sp + 904|0;
 $msg2 = sp + 648|0;
 $msg3 = sp + 392|0;
 $msg4 = sp + 136|0;
 $0 = $n;
 $1 = $nz;
 $2 = $colptr;
 $3 = $rowind;
 $4 = $bnz;
 $5 = $b_colptr;
 $6 = $b_rowind;
 $7 = $0;
 $8 = $7<<2;
 $9 = (_superlu_malloc($8)|0);
 $marker = $9;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  HEAP32[$vararg_buffer>>2] = 288;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 222;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 232;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $11 = $0;
 $12 = (($11) + 1)|0;
 $13 = $12<<2;
 $14 = (_superlu_malloc($13)|0);
 $t_colptr = $14;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  HEAP32[$vararg_buffer3>>2] = 480;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = 224;
  $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
  HEAP32[$vararg_ptr7>>2] = 232;
  (_sprintf($msg1,2864,$vararg_buffer3)|0);
  _superlu_abort_and_exit($msg1);
 }
 $16 = $1;
 $17 = $16<<2;
 $18 = (_superlu_malloc($17)|0);
 $t_rowind = $18;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  HEAP32[$vararg_buffer8>>2] = 520;
  $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
  HEAP32[$vararg_ptr11>>2] = 226;
  $vararg_ptr12 = ((($vararg_buffer8)) + 8|0);
  HEAP32[$vararg_ptr12>>2] = 232;
  (_sprintf($msg2,2864,$vararg_buffer8)|0);
  _superlu_abort_and_exit($msg2);
 }
 $i = 0;
 while(1) {
  $20 = $i;
  $21 = $0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = $i;
  $24 = $marker;
  $25 = (($24) + ($23<<2)|0);
  HEAP32[$25>>2] = 0;
  $26 = $i;
  $27 = (($26) + 1)|0;
  $i = $27;
 }
 $j = 0;
 while(1) {
  $28 = $j;
  $29 = $0;
  $30 = ($28|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = $j;
  $32 = $2;
  $33 = (($32) + ($31<<2)|0);
  $34 = HEAP32[$33>>2]|0;
  $i = $34;
  while(1) {
   $35 = $i;
   $36 = $j;
   $37 = (($36) + 1)|0;
   $38 = $2;
   $39 = (($38) + ($37<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($35|0)<($40|0);
   if (!($41)) {
    break;
   }
   $42 = $i;
   $43 = $3;
   $44 = (($43) + ($42<<2)|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = $marker;
   $47 = (($46) + ($45<<2)|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + 1)|0;
   HEAP32[$47>>2] = $49;
   $50 = $i;
   $51 = (($50) + 1)|0;
   $i = $51;
  }
  $52 = $j;
  $53 = (($52) + 1)|0;
  $j = $53;
 }
 $54 = $t_colptr;
 HEAP32[$54>>2] = 0;
 $i = 0;
 while(1) {
  $55 = $i;
  $56 = $0;
  $57 = ($55|0)<($56|0);
  if (!($57)) {
   break;
  }
  $58 = $i;
  $59 = $t_colptr;
  $60 = (($59) + ($58<<2)|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = $i;
  $63 = $marker;
  $64 = (($63) + ($62<<2)|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (($61) + ($65))|0;
  $67 = $i;
  $68 = (($67) + 1)|0;
  $69 = $t_colptr;
  $70 = (($69) + ($68<<2)|0);
  HEAP32[$70>>2] = $66;
  $71 = $i;
  $72 = $t_colptr;
  $73 = (($72) + ($71<<2)|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $i;
  $76 = $marker;
  $77 = (($76) + ($75<<2)|0);
  HEAP32[$77>>2] = $74;
  $78 = $i;
  $79 = (($78) + 1)|0;
  $i = $79;
 }
 $j = 0;
 while(1) {
  $80 = $j;
  $81 = $0;
  $82 = ($80|0)<($81|0);
  if (!($82)) {
   break;
  }
  $83 = $j;
  $84 = $2;
  $85 = (($84) + ($83<<2)|0);
  $86 = HEAP32[$85>>2]|0;
  $i = $86;
  while(1) {
   $87 = $i;
   $88 = $j;
   $89 = (($88) + 1)|0;
   $90 = $2;
   $91 = (($90) + ($89<<2)|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = ($87|0)<($92|0);
   if (!($93)) {
    break;
   }
   $94 = $i;
   $95 = $3;
   $96 = (($95) + ($94<<2)|0);
   $97 = HEAP32[$96>>2]|0;
   $col = $97;
   $98 = $j;
   $99 = $col;
   $100 = $marker;
   $101 = (($100) + ($99<<2)|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = $t_rowind;
   $104 = (($103) + ($102<<2)|0);
   HEAP32[$104>>2] = $98;
   $105 = $col;
   $106 = $marker;
   $107 = (($106) + ($105<<2)|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (($108) + 1)|0;
   HEAP32[$107>>2] = $109;
   $110 = $i;
   $111 = (($110) + 1)|0;
   $i = $111;
  }
  $112 = $j;
  $113 = (($112) + 1)|0;
  $j = $113;
 }
 $i = 0;
 while(1) {
  $114 = $i;
  $115 = $0;
  $116 = ($114|0)<($115|0);
  if (!($116)) {
   break;
  }
  $117 = $i;
  $118 = $marker;
  $119 = (($118) + ($117<<2)|0);
  HEAP32[$119>>2] = -1;
  $120 = $i;
  $121 = (($120) + 1)|0;
  $i = $121;
 }
 $num_nz = 0;
 $j = 0;
 while(1) {
  $122 = $j;
  $123 = $0;
  $124 = ($122|0)<($123|0);
  if (!($124)) {
   break;
  }
  $125 = $j;
  $126 = $j;
  $127 = $marker;
  $128 = (($127) + ($126<<2)|0);
  HEAP32[$128>>2] = $125;
  $129 = $j;
  $130 = $2;
  $131 = (($130) + ($129<<2)|0);
  $132 = HEAP32[$131>>2]|0;
  $i = $132;
  while(1) {
   $133 = $i;
   $134 = $j;
   $135 = (($134) + 1)|0;
   $136 = $2;
   $137 = (($136) + ($135<<2)|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = ($133|0)<($138|0);
   if (!($139)) {
    break;
   }
   $140 = $i;
   $141 = $3;
   $142 = (($141) + ($140<<2)|0);
   $143 = HEAP32[$142>>2]|0;
   $k = $143;
   $144 = $k;
   $145 = $marker;
   $146 = (($145) + ($144<<2)|0);
   $147 = HEAP32[$146>>2]|0;
   $148 = $j;
   $149 = ($147|0)!=($148|0);
   if ($149) {
    $150 = $j;
    $151 = $k;
    $152 = $marker;
    $153 = (($152) + ($151<<2)|0);
    HEAP32[$153>>2] = $150;
    $154 = $num_nz;
    $155 = (($154) + 1)|0;
    $num_nz = $155;
   }
   $156 = $i;
   $157 = (($156) + 1)|0;
   $i = $157;
  }
  $158 = $j;
  $159 = $t_colptr;
  $160 = (($159) + ($158<<2)|0);
  $161 = HEAP32[$160>>2]|0;
  $i = $161;
  while(1) {
   $162 = $i;
   $163 = $j;
   $164 = (($163) + 1)|0;
   $165 = $t_colptr;
   $166 = (($165) + ($164<<2)|0);
   $167 = HEAP32[$166>>2]|0;
   $168 = ($162|0)<($167|0);
   if (!($168)) {
    break;
   }
   $169 = $i;
   $170 = $t_rowind;
   $171 = (($170) + ($169<<2)|0);
   $172 = HEAP32[$171>>2]|0;
   $k = $172;
   $173 = $k;
   $174 = $marker;
   $175 = (($174) + ($173<<2)|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = $j;
   $178 = ($176|0)!=($177|0);
   if ($178) {
    $179 = $j;
    $180 = $k;
    $181 = $marker;
    $182 = (($181) + ($180<<2)|0);
    HEAP32[$182>>2] = $179;
    $183 = $num_nz;
    $184 = (($183) + 1)|0;
    $num_nz = $184;
   }
   $185 = $i;
   $186 = (($185) + 1)|0;
   $i = $186;
  }
  $187 = $j;
  $188 = (($187) + 1)|0;
  $j = $188;
 }
 $189 = $num_nz;
 $190 = $4;
 HEAP32[$190>>2] = $189;
 $191 = $0;
 $192 = (($191) + 1)|0;
 $193 = $192<<2;
 $194 = (_superlu_malloc($193)|0);
 $195 = $5;
 HEAP32[$195>>2] = $194;
 $196 = ($194|0)!=(0|0);
 if (!($196)) {
  HEAP32[$vararg_buffer13>>2] = 552;
  $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
  HEAP32[$vararg_ptr16>>2] = 289;
  $vararg_ptr17 = ((($vararg_buffer13)) + 8|0);
  HEAP32[$vararg_ptr17>>2] = 232;
  (_sprintf($msg3,2864,$vararg_buffer13)|0);
  _superlu_abort_and_exit($msg3);
 }
 $197 = $4;
 $198 = HEAP32[$197>>2]|0;
 $199 = ($198|0)!=(0);
 if ($199) {
  $200 = $4;
  $201 = HEAP32[$200>>2]|0;
  $202 = $201<<2;
  $203 = (_superlu_malloc($202)|0);
  $204 = $6;
  HEAP32[$204>>2] = $203;
  $205 = ($203|0)!=(0|0);
  if (!($205)) {
   HEAP32[$vararg_buffer18>>2] = 592;
   $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
   HEAP32[$vararg_ptr21>>2] = 292;
   $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
   HEAP32[$vararg_ptr22>>2] = 232;
   (_sprintf($msg4,2864,$vararg_buffer18)|0);
   _superlu_abort_and_exit($msg4);
  }
 }
 $i = 0;
 while(1) {
  $206 = $i;
  $207 = $0;
  $208 = ($206|0)<($207|0);
  if (!($208)) {
   break;
  }
  $209 = $i;
  $210 = $marker;
  $211 = (($210) + ($209<<2)|0);
  HEAP32[$211>>2] = -1;
  $212 = $i;
  $213 = (($212) + 1)|0;
  $i = $213;
 }
 $num_nz = 0;
 $j = 0;
 while(1) {
  $214 = $j;
  $215 = $0;
  $216 = ($214|0)<($215|0);
  $217 = $num_nz;
  if (!($216)) {
   break;
  }
  $218 = $j;
  $219 = $5;
  $220 = HEAP32[$219>>2]|0;
  $221 = (($220) + ($218<<2)|0);
  HEAP32[$221>>2] = $217;
  $222 = $j;
  $223 = $j;
  $224 = $marker;
  $225 = (($224) + ($223<<2)|0);
  HEAP32[$225>>2] = $222;
  $226 = $j;
  $227 = $2;
  $228 = (($227) + ($226<<2)|0);
  $229 = HEAP32[$228>>2]|0;
  $i = $229;
  while(1) {
   $230 = $i;
   $231 = $j;
   $232 = (($231) + 1)|0;
   $233 = $2;
   $234 = (($233) + ($232<<2)|0);
   $235 = HEAP32[$234>>2]|0;
   $236 = ($230|0)<($235|0);
   if (!($236)) {
    break;
   }
   $237 = $i;
   $238 = $3;
   $239 = (($238) + ($237<<2)|0);
   $240 = HEAP32[$239>>2]|0;
   $k = $240;
   $241 = $k;
   $242 = $marker;
   $243 = (($242) + ($241<<2)|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = $j;
   $246 = ($244|0)!=($245|0);
   if ($246) {
    $247 = $j;
    $248 = $k;
    $249 = $marker;
    $250 = (($249) + ($248<<2)|0);
    HEAP32[$250>>2] = $247;
    $251 = $k;
    $252 = $num_nz;
    $253 = (($252) + 1)|0;
    $num_nz = $253;
    $254 = $6;
    $255 = HEAP32[$254>>2]|0;
    $256 = (($255) + ($252<<2)|0);
    HEAP32[$256>>2] = $251;
   }
   $257 = $i;
   $258 = (($257) + 1)|0;
   $i = $258;
  }
  $259 = $j;
  $260 = $t_colptr;
  $261 = (($260) + ($259<<2)|0);
  $262 = HEAP32[$261>>2]|0;
  $i = $262;
  while(1) {
   $263 = $i;
   $264 = $j;
   $265 = (($264) + 1)|0;
   $266 = $t_colptr;
   $267 = (($266) + ($265<<2)|0);
   $268 = HEAP32[$267>>2]|0;
   $269 = ($263|0)<($268|0);
   if (!($269)) {
    break;
   }
   $270 = $i;
   $271 = $t_rowind;
   $272 = (($271) + ($270<<2)|0);
   $273 = HEAP32[$272>>2]|0;
   $k = $273;
   $274 = $k;
   $275 = $marker;
   $276 = (($275) + ($274<<2)|0);
   $277 = HEAP32[$276>>2]|0;
   $278 = $j;
   $279 = ($277|0)!=($278|0);
   if ($279) {
    $280 = $j;
    $281 = $k;
    $282 = $marker;
    $283 = (($282) + ($281<<2)|0);
    HEAP32[$283>>2] = $280;
    $284 = $k;
    $285 = $num_nz;
    $286 = (($285) + 1)|0;
    $num_nz = $286;
    $287 = $6;
    $288 = HEAP32[$287>>2]|0;
    $289 = (($288) + ($285<<2)|0);
    HEAP32[$289>>2] = $284;
   }
   $290 = $i;
   $291 = (($290) + 1)|0;
   $i = $291;
  }
  $292 = $j;
  $293 = (($292) + 1)|0;
  $j = $293;
 }
 $294 = $0;
 $295 = $5;
 $296 = HEAP32[$295>>2]|0;
 $297 = (($296) + ($294<<2)|0);
 HEAP32[$297>>2] = $217;
 $298 = $marker;
 _superlu_free($298);
 $299 = $t_colptr;
 _superlu_free($299);
 $300 = $t_rowind;
 _superlu_free($300);
 STACKTOP = sp;return;
}
function _get_perm_c($ispec,$A,$perm_c) {
 $ispec = $ispec|0;
 $A = $A|0;
 $perm_c = $perm_c|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0.0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0.0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0.0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Astore = 0, $b_colptr = 0, $b_rowind = 0, $bnz = 0, $delta = 0, $dhead = 0, $i = 0;
 var $invp = 0, $llist = 0, $m = 0, $marker = 0, $maxint = 0, $msg = 0, $msg1 = 0, $msg2 = 0, $msg3 = 0, $msg4 = 0, $msg5 = 0, $msg6 = 0, $n = 0, $nofsub = 0, $qsize = 0, $t = 0.0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer18 = 0, $vararg_buffer23 = 0;
 var $vararg_buffer28 = 0, $vararg_buffer3 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr2 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $vararg_ptr27 = 0, $vararg_ptr31 = 0, $vararg_ptr32 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1984|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer28 = sp + 104|0;
 $vararg_buffer23 = sp + 88|0;
 $vararg_buffer18 = sp + 72|0;
 $vararg_buffer13 = sp + 56|0;
 $vararg_buffer8 = sp + 40|0;
 $vararg_buffer3 = sp + 24|0;
 $vararg_buffer = sp + 8|0;
 $n = sp + 164|0;
 $bnz = sp + 160|0;
 $b_colptr = sp + 156|0;
 $delta = sp + 148|0;
 $maxint = sp + 144|0;
 $nofsub = sp + 140|0;
 $b_rowind = sp + 132|0;
 $msg = sp + 1728|0;
 $msg1 = sp + 1472|0;
 $msg2 = sp + 1216|0;
 $msg3 = sp + 960|0;
 $msg4 = sp + 704|0;
 $msg5 = sp + 448|0;
 $msg6 = sp + 192|0;
 $0 = $ispec;
 $1 = $A;
 $2 = $perm_c;
 $3 = $1;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 $Astore = $5;
 HEAP32[$bnz>>2] = 0;
 $6 = $1;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $m = $8;
 $9 = $1;
 $10 = ((($9)) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$n>>2] = $11;
 $12 = (+_SuperLU_timer_());
 $t = $12;
 $13 = $0;
 if ((($13|0) == 3)) {
  $51 = $m;
  $52 = HEAP32[$n>>2]|0;
  $53 = $Astore;
  $54 = HEAP32[$53>>2]|0;
  $55 = $Astore;
  $56 = ((($55)) + 12|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $Astore;
  $59 = ((($58)) + 8|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = $2;
  _get_colamd($51,$52,$54,$57,$60,$61);
  STACKTOP = sp;return;
 } else if ((($13|0) == 1)) {
  $23 = $m;
  $24 = HEAP32[$n>>2]|0;
  $25 = $Astore;
  $26 = HEAP32[$25>>2]|0;
  $27 = $Astore;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $Astore;
  $31 = ((($30)) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  _getata($23,$24,$26,$29,$32,$bnz,$b_colptr,$b_rowind);
  $33 = (+_SuperLU_timer_());
  $34 = $t;
  $35 = $33 - $34;
  $t = $35;
 } else if ((($13|0) == 0)) {
  $i = 0;
  while(1) {
   $14 = $i;
   $15 = HEAP32[$n>>2]|0;
   $16 = ($14|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = $i;
   $18 = $i;
   $19 = $2;
   $20 = (($19) + ($18<<2)|0);
   HEAP32[$20>>2] = $17;
   $21 = $i;
   $22 = (($21) + 1)|0;
   $i = $22;
  }
  STACKTOP = sp;return;
 } else if ((($13|0) == 2)) {
  $36 = $m;
  $37 = HEAP32[$n>>2]|0;
  $38 = ($36|0)!=($37|0);
  if ($38) {
   HEAP32[$vararg_buffer>>2] = 632;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = 395;
   $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr2>>2] = 232;
   (_sprintf($msg,2864,$vararg_buffer)|0);
   _superlu_abort_and_exit($msg);
  }
  $39 = HEAP32[$n>>2]|0;
  $40 = $Astore;
  $41 = HEAP32[$40>>2]|0;
  $42 = $Astore;
  $43 = ((($42)) + 12|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = $Astore;
  $46 = ((($45)) + 8|0);
  $47 = HEAP32[$46>>2]|0;
  _at_plus_a($39,$41,$44,$47,$bnz,$b_colptr,$b_rowind);
  $48 = (+_SuperLU_timer_());
  $49 = $t;
  $50 = $48 - $49;
  $t = $50;
 } else {
  HEAP32[$vararg_buffer3>>2] = 656;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = 411;
  $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
  HEAP32[$vararg_ptr7>>2] = 232;
  (_sprintf($msg1,2864,$vararg_buffer3)|0);
  _superlu_abort_and_exit($msg1);
 }
 $62 = HEAP32[$bnz>>2]|0;
 $63 = ($62|0)!=(0);
 L16: do {
  if ($63) {
   $64 = (+_SuperLU_timer_());
   $t = $64;
   HEAP32[$delta>>2] = 0;
   HEAP32[$maxint>>2] = 2147483647;
   $65 = HEAP32[$n>>2]|0;
   $66 = HEAP32[$delta>>2]|0;
   $67 = (($65) + ($66))|0;
   $68 = $67<<2;
   $69 = (_superlu_malloc($68)|0);
   $invp = $69;
   $70 = $invp;
   $71 = ($70|0)!=(0|0);
   if (!($71)) {
    HEAP32[$vararg_buffer8>>2] = 672;
    $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
    HEAP32[$vararg_ptr11>>2] = 422;
    $vararg_ptr12 = ((($vararg_buffer8)) + 8|0);
    HEAP32[$vararg_ptr12>>2] = 232;
    (_sprintf($msg2,2864,$vararg_buffer8)|0);
    _superlu_abort_and_exit($msg2);
   }
   $72 = HEAP32[$n>>2]|0;
   $73 = HEAP32[$delta>>2]|0;
   $74 = (($72) + ($73))|0;
   $75 = $74<<2;
   $76 = (_superlu_malloc($75)|0);
   $dhead = $76;
   $77 = $dhead;
   $78 = ($77|0)!=(0|0);
   if (!($78)) {
    HEAP32[$vararg_buffer13>>2] = 704;
    $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
    HEAP32[$vararg_ptr16>>2] = 424;
    $vararg_ptr17 = ((($vararg_buffer13)) + 8|0);
    HEAP32[$vararg_ptr17>>2] = 232;
    (_sprintf($msg3,2864,$vararg_buffer13)|0);
    _superlu_abort_and_exit($msg3);
   }
   $79 = HEAP32[$n>>2]|0;
   $80 = HEAP32[$delta>>2]|0;
   $81 = (($79) + ($80))|0;
   $82 = $81<<2;
   $83 = (_superlu_malloc($82)|0);
   $qsize = $83;
   $84 = $qsize;
   $85 = ($84|0)!=(0|0);
   if (!($85)) {
    HEAP32[$vararg_buffer18>>2] = 736;
    $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
    HEAP32[$vararg_ptr21>>2] = 426;
    $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
    HEAP32[$vararg_ptr22>>2] = 232;
    (_sprintf($msg4,2864,$vararg_buffer18)|0);
    _superlu_abort_and_exit($msg4);
   }
   $86 = HEAP32[$n>>2]|0;
   $87 = $86<<2;
   $88 = (_superlu_malloc($87)|0);
   $llist = $88;
   $89 = $llist;
   $90 = ($89|0)!=(0|0);
   if (!($90)) {
    HEAP32[$vararg_buffer23>>2] = 768;
    $vararg_ptr26 = ((($vararg_buffer23)) + 4|0);
    HEAP32[$vararg_ptr26>>2] = 428;
    $vararg_ptr27 = ((($vararg_buffer23)) + 8|0);
    HEAP32[$vararg_ptr27>>2] = 232;
    (_sprintf($msg5,2864,$vararg_buffer23)|0);
    _superlu_abort_and_exit($msg5);
   }
   $91 = HEAP32[$n>>2]|0;
   $92 = $91<<2;
   $93 = (_superlu_malloc($92)|0);
   $marker = $93;
   $94 = $marker;
   $95 = ($94|0)!=(0|0);
   if (!($95)) {
    HEAP32[$vararg_buffer28>>2] = 800;
    $vararg_ptr31 = ((($vararg_buffer28)) + 4|0);
    HEAP32[$vararg_ptr31>>2] = 430;
    $vararg_ptr32 = ((($vararg_buffer28)) + 8|0);
    HEAP32[$vararg_ptr32>>2] = 232;
    (_sprintf($msg6,2864,$vararg_buffer28)|0);
    _superlu_abort_and_exit($msg6);
   }
   $i = 0;
   while(1) {
    $96 = $i;
    $97 = HEAP32[$n>>2]|0;
    $98 = ($96|0)<=($97|0);
    if (!($98)) {
     break;
    }
    $99 = $i;
    $100 = HEAP32[$b_colptr>>2]|0;
    $101 = (($100) + ($99<<2)|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = (($102) + 1)|0;
    HEAP32[$101>>2] = $103;
    $104 = $i;
    $105 = (($104) + 1)|0;
    $i = $105;
   }
   $i = 0;
   while(1) {
    $106 = $i;
    $107 = HEAP32[$bnz>>2]|0;
    $108 = ($106|0)<($107|0);
    if (!($108)) {
     break;
    }
    $109 = $i;
    $110 = HEAP32[$b_rowind>>2]|0;
    $111 = (($110) + ($109<<2)|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = (($112) + 1)|0;
    HEAP32[$111>>2] = $113;
    $114 = $i;
    $115 = (($114) + 1)|0;
    $i = $115;
   }
   $116 = HEAP32[$b_colptr>>2]|0;
   $117 = HEAP32[$b_rowind>>2]|0;
   $118 = $2;
   $119 = $invp;
   $120 = $dhead;
   $121 = $qsize;
   $122 = $llist;
   $123 = $marker;
   (_genmmd_($n,$116,$117,$118,$119,$delta,$120,$121,$122,$123,$maxint,$nofsub)|0);
   $i = 0;
   while(1) {
    $124 = $i;
    $125 = HEAP32[$n>>2]|0;
    $126 = ($124|0)<($125|0);
    if (!($126)) {
     break;
    }
    $127 = $i;
    $128 = $2;
    $129 = (($128) + ($127<<2)|0);
    $130 = HEAP32[$129>>2]|0;
    $131 = (($130) + -1)|0;
    HEAP32[$129>>2] = $131;
    $132 = $i;
    $133 = (($132) + 1)|0;
    $i = $133;
   }
   $134 = $invp;
   _superlu_free($134);
   $135 = $dhead;
   _superlu_free($135);
   $136 = $qsize;
   _superlu_free($136);
   $137 = $llist;
   _superlu_free($137);
   $138 = $marker;
   _superlu_free($138);
   $139 = HEAP32[$b_rowind>>2]|0;
   _superlu_free($139);
   $140 = (+_SuperLU_timer_());
   $141 = $t;
   $142 = $140 - $141;
   $t = $142;
  } else {
   $i = 0;
   while(1) {
    $143 = $i;
    $144 = HEAP32[$n>>2]|0;
    $145 = ($143|0)<($144|0);
    if (!($145)) {
     break L16;
    }
    $146 = $i;
    $147 = $i;
    $148 = $2;
    $149 = (($148) + ($147<<2)|0);
    HEAP32[$149>>2] = $146;
    $150 = $i;
    $151 = (($150) + 1)|0;
    $i = $151;
   }
  }
 } while(0);
 $152 = HEAP32[$b_colptr>>2]|0;
 _superlu_free($152);
 STACKTOP = sp;return;
}
function _genmmd_($neqns,$xadj,$adjncy,$invp,$perm,$delta,$dhead,$qsize,$llist,$marker,$maxint,$nofsub) {
 $neqns = $neqns|0;
 $xadj = $xadj|0;
 $adjncy = $adjncy|0;
 $invp = $invp|0;
 $perm = $perm|0;
 $delta = $delta|0;
 $dhead = $dhead|0;
 $qsize = $qsize|0;
 $llist = $llist|0;
 $marker = $marker|0;
 $maxint = $maxint|0;
 $nofsub = $nofsub|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $neqns;
 $2 = $xadj;
 $3 = $adjncy;
 $4 = $invp;
 $5 = $perm;
 $6 = $delta;
 $7 = $dhead;
 $8 = $qsize;
 $9 = $llist;
 $10 = $marker;
 $11 = $maxint;
 $12 = $nofsub;
 $13 = $10;
 $14 = ((($13)) + -4|0);
 $10 = $14;
 $15 = $9;
 $16 = ((($15)) + -4|0);
 $9 = $16;
 $17 = $8;
 $18 = ((($17)) + -4|0);
 $8 = $18;
 $19 = $7;
 $20 = ((($19)) + -4|0);
 $7 = $20;
 $21 = $5;
 $22 = ((($21)) + -4|0);
 $5 = $22;
 $23 = $4;
 $24 = ((($23)) + -4|0);
 $4 = $24;
 $25 = $3;
 $26 = ((($25)) + -4|0);
 $3 = $26;
 $27 = $2;
 $28 = ((($27)) + -4|0);
 $2 = $28;
 $29 = $1;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)<=(0);
 if ($31) {
  $0 = 0;
  $223 = $0;
  STACKTOP = sp;return ($223|0);
 }
 $32 = $12;
 HEAP32[$32>>2] = 0;
 $33 = $1;
 $34 = $2;
 $35 = ((($34)) + 4|0);
 $36 = $3;
 $37 = ((($36)) + 4|0);
 $38 = $7;
 $39 = ((($38)) + 4|0);
 $40 = $4;
 $41 = ((($40)) + 4|0);
 $42 = $5;
 $43 = ((($42)) + 4|0);
 $44 = $8;
 $45 = ((($44)) + 4|0);
 $46 = $9;
 $47 = ((($46)) + 4|0);
 $48 = $10;
 $49 = ((($48)) + 4|0);
 (_mmdint_($33,$35,$37,$39,$41,$43,$45,$47,$49)|0);
 HEAP32[840>>2] = 1;
 $50 = $7;
 $51 = ((($50)) + 4|0);
 $52 = HEAP32[$51>>2]|0;
 HEAP32[848>>2] = $52;
 while(1) {
  $53 = HEAP32[848>>2]|0;
  $54 = ($53|0)<=(0);
  if ($54) {
   break;
  }
  $55 = HEAP32[848>>2]|0;
  HEAP32[856>>2] = $55;
  $56 = HEAP32[856>>2]|0;
  $57 = $4;
  $58 = (($57) + ($56<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[848>>2] = $59;
  $60 = $11;
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[856>>2]|0;
  $63 = $10;
  $64 = (($63) + ($62<<2)|0);
  HEAP32[$64>>2] = $61;
  $65 = HEAP32[840>>2]|0;
  $66 = (0 - ($65))|0;
  $67 = HEAP32[856>>2]|0;
  $68 = $4;
  $69 = (($68) + ($67<<2)|0);
  HEAP32[$69>>2] = $66;
  $70 = HEAP32[840>>2]|0;
  $71 = (($70) + 1)|0;
  HEAP32[840>>2] = $71;
 }
 $72 = HEAP32[840>>2]|0;
 $73 = $1;
 $74 = HEAP32[$73>>2]|0;
 $75 = ($72|0)>($74|0);
 L9: do {
  if (!($75)) {
   HEAP32[864>>2] = 1;
   $76 = $7;
   $77 = ((($76)) + 4|0);
   HEAP32[$77>>2] = 0;
   HEAP32[872>>2] = 2;
   while(1) {
    $78 = HEAP32[872>>2]|0;
    $79 = $7;
    $80 = (($79) + ($78<<2)|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = ($81|0)>(0);
    $83 = HEAP32[872>>2]|0;
    if (!($82)) {
     $84 = (($83) + 1)|0;
     HEAP32[872>>2] = $84;
     continue;
    }
    $85 = $6;
    $86 = HEAP32[$85>>2]|0;
    $87 = (($83) + ($86))|0;
    HEAP32[880>>2] = $87;
    HEAP32[888>>2] = 0;
    while(1) {
     $88 = HEAP32[872>>2]|0;
     $89 = $7;
     $90 = (($89) + ($88<<2)|0);
     $91 = HEAP32[$90>>2]|0;
     HEAP32[856>>2] = $91;
     $92 = HEAP32[856>>2]|0;
     $93 = ($92|0)>(0);
     if (!($93)) {
      $94 = HEAP32[872>>2]|0;
      $95 = (($94) + 1)|0;
      HEAP32[872>>2] = $95;
      $96 = HEAP32[872>>2]|0;
      $97 = HEAP32[880>>2]|0;
      $98 = ($96|0)>($97|0);
      if ($98) {
       break;
      } else {
       continue;
      }
     }
     $99 = HEAP32[856>>2]|0;
     $100 = $4;
     $101 = (($100) + ($99<<2)|0);
     $102 = HEAP32[$101>>2]|0;
     HEAP32[848>>2] = $102;
     $103 = HEAP32[848>>2]|0;
     $104 = HEAP32[872>>2]|0;
     $105 = $7;
     $106 = (($105) + ($104<<2)|0);
     HEAP32[$106>>2] = $103;
     $107 = HEAP32[848>>2]|0;
     $108 = ($107|0)>(0);
     if ($108) {
      $109 = HEAP32[872>>2]|0;
      $110 = (0 - ($109))|0;
      $111 = HEAP32[848>>2]|0;
      $112 = $5;
      $113 = (($112) + ($111<<2)|0);
      HEAP32[$113>>2] = $110;
     }
     $114 = HEAP32[840>>2]|0;
     $115 = (0 - ($114))|0;
     $116 = HEAP32[856>>2]|0;
     $117 = $4;
     $118 = (($117) + ($116<<2)|0);
     HEAP32[$118>>2] = $115;
     $119 = $12;
     $120 = HEAP32[$119>>2]|0;
     $121 = HEAP32[872>>2]|0;
     $122 = (($120) + ($121))|0;
     $123 = HEAP32[856>>2]|0;
     $124 = $8;
     $125 = (($124) + ($123<<2)|0);
     $126 = HEAP32[$125>>2]|0;
     $127 = (($122) + ($126))|0;
     $128 = (($127) - 2)|0;
     $129 = $12;
     HEAP32[$129>>2] = $128;
     $130 = HEAP32[840>>2]|0;
     $131 = HEAP32[856>>2]|0;
     $132 = $8;
     $133 = (($132) + ($131<<2)|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = (($130) + ($134))|0;
     $136 = $1;
     $137 = HEAP32[$136>>2]|0;
     $138 = ($135|0)>($137|0);
     if ($138) {
      break L9;
     }
     $139 = HEAP32[864>>2]|0;
     $140 = (($139) + 1)|0;
     HEAP32[864>>2] = $140;
     $141 = HEAP32[864>>2]|0;
     $142 = $11;
     $143 = HEAP32[$142>>2]|0;
     $144 = ($141|0)<($143|0);
     L25: do {
      if (!($144)) {
       HEAP32[864>>2] = 1;
       $145 = $1;
       $146 = HEAP32[$145>>2]|0;
       $i__1 = $146;
       HEAP32[896>>2] = 1;
       while(1) {
        $147 = HEAP32[896>>2]|0;
        $148 = $i__1;
        $149 = ($147|0)<=($148|0);
        if (!($149)) {
         break L25;
        }
        $150 = HEAP32[896>>2]|0;
        $151 = $10;
        $152 = (($151) + ($150<<2)|0);
        $153 = HEAP32[$152>>2]|0;
        $154 = $11;
        $155 = HEAP32[$154>>2]|0;
        $156 = ($153|0)<($155|0);
        if ($156) {
         $157 = HEAP32[896>>2]|0;
         $158 = $10;
         $159 = (($158) + ($157<<2)|0);
         HEAP32[$159>>2] = 0;
        }
        $160 = HEAP32[896>>2]|0;
        $161 = (($160) + 1)|0;
        HEAP32[896>>2] = $161;
       }
      }
     } while(0);
     $162 = $2;
     $163 = ((($162)) + 4|0);
     $164 = $3;
     $165 = ((($164)) + 4|0);
     $166 = $7;
     $167 = ((($166)) + 4|0);
     $168 = $4;
     $169 = ((($168)) + 4|0);
     $170 = $5;
     $171 = ((($170)) + 4|0);
     $172 = $8;
     $173 = ((($172)) + 4|0);
     $174 = $9;
     $175 = ((($174)) + 4|0);
     $176 = $10;
     $177 = ((($176)) + 4|0);
     $178 = $11;
     (_mmdelm_(856,$163,$165,$167,$169,$171,$173,$175,$177,$178,864)|0);
     $179 = HEAP32[856>>2]|0;
     $180 = $8;
     $181 = (($180) + ($179<<2)|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = HEAP32[840>>2]|0;
     $184 = (($183) + ($182))|0;
     HEAP32[840>>2] = $184;
     $185 = HEAP32[888>>2]|0;
     $186 = HEAP32[856>>2]|0;
     $187 = $9;
     $188 = (($187) + ($186<<2)|0);
     HEAP32[$188>>2] = $185;
     $189 = HEAP32[856>>2]|0;
     HEAP32[888>>2] = $189;
     $190 = $6;
     $191 = HEAP32[$190>>2]|0;
     $192 = ($191|0)>=(0);
     if (!($192)) {
      break;
     }
    }
    $193 = HEAP32[840>>2]|0;
    $194 = $1;
    $195 = HEAP32[$194>>2]|0;
    $196 = ($193|0)>($195|0);
    if ($196) {
     break L9;
    }
    $197 = $1;
    $198 = $2;
    $199 = ((($198)) + 4|0);
    $200 = $3;
    $201 = ((($200)) + 4|0);
    $202 = $6;
    $203 = $7;
    $204 = ((($203)) + 4|0);
    $205 = $4;
    $206 = ((($205)) + 4|0);
    $207 = $5;
    $208 = ((($207)) + 4|0);
    $209 = $8;
    $210 = ((($209)) + 4|0);
    $211 = $9;
    $212 = ((($211)) + 4|0);
    $213 = $10;
    $214 = ((($213)) + 4|0);
    $215 = $11;
    (_mmdupd_(888,$197,$199,$201,$202,872,$204,$206,$208,$210,$212,$214,$215,864)|0);
   }
  }
 } while(0);
 $216 = $1;
 $217 = $5;
 $218 = ((($217)) + 4|0);
 $219 = $4;
 $220 = ((($219)) + 4|0);
 $221 = $8;
 $222 = ((($221)) + 4|0);
 (_mmdnum_($216,$218,$220,$222)|0);
 $0 = 0;
 $223 = $0;
 STACKTOP = sp;return ($223|0);
}
function _mmdint_($neqns,$xadj,$adjncy,$dhead,$dforw,$dbakw,$qsize,$llist,$marker) {
 $neqns = $neqns|0;
 $xadj = $xadj|0;
 $adjncy = $adjncy|0;
 $dhead = $dhead|0;
 $dforw = $dforw|0;
 $dbakw = $dbakw|0;
 $qsize = $qsize|0;
 $llist = $llist|0;
 $marker = $marker|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $neqns;
 $1 = $xadj;
 $2 = $adjncy;
 $3 = $dhead;
 $4 = $dforw;
 $5 = $dbakw;
 $6 = $qsize;
 $7 = $llist;
 $8 = $marker;
 $9 = $8;
 $10 = ((($9)) + -4|0);
 $8 = $10;
 $11 = $7;
 $12 = ((($11)) + -4|0);
 $7 = $12;
 $13 = $6;
 $14 = ((($13)) + -4|0);
 $6 = $14;
 $15 = $5;
 $16 = ((($15)) + -4|0);
 $5 = $16;
 $17 = $4;
 $18 = ((($17)) + -4|0);
 $4 = $18;
 $19 = $3;
 $20 = ((($19)) + -4|0);
 $3 = $20;
 $21 = $2;
 $22 = ((($21)) + -4|0);
 $2 = $22;
 $23 = $1;
 $24 = ((($23)) + -4|0);
 $1 = $24;
 $25 = $0;
 $26 = HEAP32[$25>>2]|0;
 $i__1 = $26;
 HEAP32[904>>2] = 1;
 while(1) {
  $27 = HEAP32[904>>2]|0;
  $28 = $i__1;
  $29 = ($27|0)<=($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[904>>2]|0;
  $31 = $3;
  $32 = (($31) + ($30<<2)|0);
  HEAP32[$32>>2] = 0;
  $33 = HEAP32[904>>2]|0;
  $34 = $6;
  $35 = (($34) + ($33<<2)|0);
  HEAP32[$35>>2] = 1;
  $36 = HEAP32[904>>2]|0;
  $37 = $8;
  $38 = (($37) + ($36<<2)|0);
  HEAP32[$38>>2] = 0;
  $39 = HEAP32[904>>2]|0;
  $40 = $7;
  $41 = (($40) + ($39<<2)|0);
  HEAP32[$41>>2] = 0;
  $42 = HEAP32[904>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[904>>2] = $43;
 }
 $44 = $0;
 $45 = HEAP32[$44>>2]|0;
 $i__1 = $45;
 HEAP32[904>>2] = 1;
 while(1) {
  $46 = HEAP32[904>>2]|0;
  $47 = $i__1;
  $48 = ($46|0)<=($47|0);
  if (!($48)) {
   break;
  }
  $49 = HEAP32[904>>2]|0;
  $50 = (($49) + 1)|0;
  $51 = $1;
  $52 = (($51) + ($50<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[904>>2]|0;
  $55 = $1;
  $56 = (($55) + ($54<<2)|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($53) - ($57))|0;
  $59 = (($58) + 1)|0;
  HEAP32[912>>2] = $59;
  $60 = HEAP32[912>>2]|0;
  $61 = $3;
  $62 = (($61) + ($60<<2)|0);
  $63 = HEAP32[$62>>2]|0;
  HEAP32[920>>2] = $63;
  $64 = HEAP32[920>>2]|0;
  $65 = HEAP32[904>>2]|0;
  $66 = $4;
  $67 = (($66) + ($65<<2)|0);
  HEAP32[$67>>2] = $64;
  $68 = HEAP32[904>>2]|0;
  $69 = HEAP32[912>>2]|0;
  $70 = $3;
  $71 = (($70) + ($69<<2)|0);
  HEAP32[$71>>2] = $68;
  $72 = HEAP32[920>>2]|0;
  $73 = ($72|0)>(0);
  if ($73) {
   $74 = HEAP32[904>>2]|0;
   $75 = HEAP32[920>>2]|0;
   $76 = $5;
   $77 = (($76) + ($75<<2)|0);
   HEAP32[$77>>2] = $74;
  }
  $78 = HEAP32[912>>2]|0;
  $79 = (0 - ($78))|0;
  $80 = HEAP32[904>>2]|0;
  $81 = $5;
  $82 = (($81) + ($80<<2)|0);
  HEAP32[$82>>2] = $79;
  $83 = HEAP32[904>>2]|0;
  $84 = (($83) + 1)|0;
  HEAP32[904>>2] = $84;
 }
 STACKTOP = sp;return 0;
}
function _mmdelm_($mdnode,$xadj,$adjncy,$dhead,$dforw,$dbakw,$qsize,$llist,$marker,$maxint,$tag) {
 $mdnode = $mdnode|0;
 $xadj = $xadj|0;
 $adjncy = $adjncy|0;
 $dhead = $dhead|0;
 $dforw = $dforw|0;
 $dbakw = $dbakw|0;
 $qsize = $qsize|0;
 $llist = $llist|0;
 $marker = $marker|0;
 $maxint = $maxint|0;
 $tag = $tag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, $i__2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $mdnode;
 $1 = $xadj;
 $2 = $adjncy;
 $3 = $dhead;
 $4 = $dforw;
 $5 = $dbakw;
 $6 = $qsize;
 $7 = $llist;
 $8 = $marker;
 $9 = $maxint;
 $10 = $tag;
 $11 = $8;
 $12 = ((($11)) + -4|0);
 $8 = $12;
 $13 = $7;
 $14 = ((($13)) + -4|0);
 $7 = $14;
 $15 = $6;
 $16 = ((($15)) + -4|0);
 $6 = $16;
 $17 = $5;
 $18 = ((($17)) + -4|0);
 $5 = $18;
 $19 = $4;
 $20 = ((($19)) + -4|0);
 $4 = $20;
 $21 = $3;
 $22 = ((($21)) + -4|0);
 $3 = $22;
 $23 = $2;
 $24 = ((($23)) + -4|0);
 $2 = $24;
 $25 = $1;
 $26 = ((($25)) + -4|0);
 $1 = $26;
 $27 = $10;
 $28 = HEAP32[$27>>2]|0;
 $29 = $0;
 $30 = HEAP32[$29>>2]|0;
 $31 = $8;
 $32 = (($31) + ($30<<2)|0);
 HEAP32[$32>>2] = $28;
 $33 = $0;
 $34 = HEAP32[$33>>2]|0;
 $35 = $1;
 $36 = (($35) + ($34<<2)|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[928>>2] = $37;
 $38 = $0;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($39) + 1)|0;
 $41 = $1;
 $42 = (($41) + ($40<<2)|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) - 1)|0;
 HEAP32[936>>2] = $44;
 HEAP32[944>>2] = 0;
 $45 = HEAP32[928>>2]|0;
 HEAP32[952>>2] = $45;
 $46 = HEAP32[936>>2]|0;
 HEAP32[960>>2] = $46;
 $47 = HEAP32[936>>2]|0;
 $i__1 = $47;
 $48 = HEAP32[928>>2]|0;
 HEAP32[968>>2] = $48;
 while(1) {
  $49 = HEAP32[968>>2]|0;
  $50 = $i__1;
  $51 = ($49|0)<=($50|0);
  if (!($51)) {
   break;
  }
  $52 = HEAP32[968>>2]|0;
  $53 = $2;
  $54 = (($53) + ($52<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  HEAP32[976>>2] = $55;
  $56 = HEAP32[976>>2]|0;
  $57 = ($56|0)==(0);
  if ($57) {
   break;
  }
  $58 = HEAP32[976>>2]|0;
  $59 = $8;
  $60 = (($59) + ($58<<2)|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = $10;
  $63 = HEAP32[$62>>2]|0;
  $64 = ($61|0)>=($63|0);
  do {
   if (!($64)) {
    $65 = $10;
    $66 = HEAP32[$65>>2]|0;
    $67 = HEAP32[976>>2]|0;
    $68 = $8;
    $69 = (($68) + ($67<<2)|0);
    HEAP32[$69>>2] = $66;
    $70 = HEAP32[976>>2]|0;
    $71 = $4;
    $72 = (($71) + ($70<<2)|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = ($73|0)<(0);
    if ($74) {
     $81 = HEAP32[944>>2]|0;
     $82 = HEAP32[976>>2]|0;
     $83 = $7;
     $84 = (($83) + ($82<<2)|0);
     HEAP32[$84>>2] = $81;
     $85 = HEAP32[976>>2]|0;
     HEAP32[944>>2] = $85;
     break;
    } else {
     $75 = HEAP32[976>>2]|0;
     $76 = HEAP32[952>>2]|0;
     $77 = $2;
     $78 = (($77) + ($76<<2)|0);
     HEAP32[$78>>2] = $75;
     $79 = HEAP32[952>>2]|0;
     $80 = (($79) + 1)|0;
     HEAP32[952>>2] = $80;
     break;
    }
   }
  } while(0);
  $86 = HEAP32[968>>2]|0;
  $87 = (($86) + 1)|0;
  HEAP32[968>>2] = $87;
 }
 while(1) {
  $88 = HEAP32[944>>2]|0;
  $89 = ($88|0)<=(0);
  if ($89) {
   break;
  }
  $90 = HEAP32[944>>2]|0;
  $91 = (0 - ($90))|0;
  $92 = HEAP32[960>>2]|0;
  $93 = $2;
  $94 = (($93) + ($92<<2)|0);
  HEAP32[$94>>2] = $91;
  $95 = HEAP32[944>>2]|0;
  HEAP32[984>>2] = $95;
  L14: while(1) {
   $96 = HEAP32[984>>2]|0;
   $97 = $1;
   $98 = (($97) + ($96<<2)|0);
   $99 = HEAP32[$98>>2]|0;
   HEAP32[992>>2] = $99;
   $100 = HEAP32[984>>2]|0;
   $101 = (($100) + 1)|0;
   $102 = $1;
   $103 = (($102) + ($101<<2)|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = (($104) - 1)|0;
   HEAP32[1000>>2] = $105;
   $106 = HEAP32[1000>>2]|0;
   $i__1 = $106;
   $107 = HEAP32[992>>2]|0;
   HEAP32[1008>>2] = $107;
   while(1) {
    $108 = HEAP32[1008>>2]|0;
    $109 = $i__1;
    $110 = ($108|0)<=($109|0);
    if (!($110)) {
     break L14;
    }
    $111 = HEAP32[1008>>2]|0;
    $112 = $2;
    $113 = (($112) + ($111<<2)|0);
    $114 = HEAP32[$113>>2]|0;
    HEAP32[1016>>2] = $114;
    $115 = HEAP32[1016>>2]|0;
    $116 = (0 - ($115))|0;
    HEAP32[984>>2] = $116;
    $117 = HEAP32[1016>>2]|0;
    $118 = ($117|0)<(0);
    if ($118) {
     continue L14;
    }
    $119 = HEAP32[1016>>2]|0;
    $120 = ($119|0)==(0);
    if ($120) {
     break L14;
    }
    $121 = HEAP32[1016>>2]|0;
    $122 = $8;
    $123 = (($122) + ($121<<2)|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = $10;
    $126 = HEAP32[$125>>2]|0;
    $127 = ($124|0)>=($126|0);
    if (!($127)) {
     $128 = HEAP32[1016>>2]|0;
     $129 = $4;
     $130 = (($129) + ($128<<2)|0);
     $131 = HEAP32[$130>>2]|0;
     $132 = ($131|0)<(0);
     if (!($132)) {
      $133 = $10;
      $134 = HEAP32[$133>>2]|0;
      $135 = HEAP32[1016>>2]|0;
      $136 = $8;
      $137 = (($136) + ($135<<2)|0);
      HEAP32[$137>>2] = $134;
      while(1) {
       $138 = HEAP32[952>>2]|0;
       $139 = HEAP32[960>>2]|0;
       $140 = ($138|0)<($139|0);
       if ($140) {
        break;
       }
       $141 = HEAP32[960>>2]|0;
       $142 = $2;
       $143 = (($142) + ($141<<2)|0);
       $144 = HEAP32[$143>>2]|0;
       $145 = (0 - ($144))|0;
       HEAP32[984>>2] = $145;
       $146 = HEAP32[984>>2]|0;
       $147 = $1;
       $148 = (($147) + ($146<<2)|0);
       $149 = HEAP32[$148>>2]|0;
       HEAP32[952>>2] = $149;
       $150 = HEAP32[984>>2]|0;
       $151 = (($150) + 1)|0;
       $152 = $1;
       $153 = (($152) + ($151<<2)|0);
       $154 = HEAP32[$153>>2]|0;
       $155 = (($154) - 1)|0;
       HEAP32[960>>2] = $155;
      }
      $156 = HEAP32[1016>>2]|0;
      $157 = HEAP32[952>>2]|0;
      $158 = $2;
      $159 = (($158) + ($157<<2)|0);
      HEAP32[$159>>2] = $156;
      $160 = HEAP32[952>>2]|0;
      $161 = (($160) + 1)|0;
      HEAP32[952>>2] = $161;
     }
    }
    $162 = HEAP32[1008>>2]|0;
    $163 = (($162) + 1)|0;
    HEAP32[1008>>2] = $163;
   }
  }
  $164 = HEAP32[944>>2]|0;
  $165 = $7;
  $166 = (($165) + ($164<<2)|0);
  $167 = HEAP32[$166>>2]|0;
  HEAP32[944>>2] = $167;
 }
 $168 = HEAP32[952>>2]|0;
 $169 = HEAP32[960>>2]|0;
 $170 = ($168|0)<=($169|0);
 if ($170) {
  $171 = HEAP32[952>>2]|0;
  $172 = $2;
  $173 = (($172) + ($171<<2)|0);
  HEAP32[$173>>2] = 0;
 }
 $174 = $0;
 $175 = HEAP32[$174>>2]|0;
 HEAP32[984>>2] = $175;
 L34: while(1) {
  $176 = HEAP32[984>>2]|0;
  $177 = $1;
  $178 = (($177) + ($176<<2)|0);
  $179 = HEAP32[$178>>2]|0;
  HEAP32[928>>2] = $179;
  $180 = HEAP32[984>>2]|0;
  $181 = (($180) + 1)|0;
  $182 = $1;
  $183 = (($182) + ($181<<2)|0);
  $184 = HEAP32[$183>>2]|0;
  $185 = (($184) - 1)|0;
  HEAP32[936>>2] = $185;
  $186 = HEAP32[936>>2]|0;
  $i__1 = $186;
  $187 = HEAP32[928>>2]|0;
  HEAP32[968>>2] = $187;
  while(1) {
   $188 = HEAP32[968>>2]|0;
   $189 = $i__1;
   $190 = ($188|0)<=($189|0);
   if (!($190)) {
    label = 49;
    break L34;
   }
   $191 = HEAP32[968>>2]|0;
   $192 = $2;
   $193 = (($192) + ($191<<2)|0);
   $194 = HEAP32[$193>>2]|0;
   HEAP32[1024>>2] = $194;
   $195 = HEAP32[1024>>2]|0;
   $196 = (0 - ($195))|0;
   HEAP32[984>>2] = $196;
   $197 = HEAP32[1024>>2]|0;
   $198 = ($197|0)<(0);
   if ($198) {
    continue L34;
   }
   $199 = HEAP32[1024>>2]|0;
   $200 = ($199|0)==(0);
   if ($200) {
    label = 49;
    break L34;
   }
   $201 = HEAP32[1024>>2]|0;
   $202 = $5;
   $203 = (($202) + ($201<<2)|0);
   $204 = HEAP32[$203>>2]|0;
   HEAP32[1032>>2] = $204;
   $205 = HEAP32[1032>>2]|0;
   $206 = ($205|0)==(0);
   if (!($206)) {
    $207 = HEAP32[1032>>2]|0;
    $208 = $9;
    $209 = HEAP32[$208>>2]|0;
    $210 = (0 - ($209))|0;
    $211 = ($207|0)==($210|0);
    if (!($211)) {
     $212 = HEAP32[1024>>2]|0;
     $213 = $4;
     $214 = (($213) + ($212<<2)|0);
     $215 = HEAP32[$214>>2]|0;
     HEAP32[1040>>2] = $215;
     $216 = HEAP32[1040>>2]|0;
     $217 = ($216|0)>(0);
     if ($217) {
      $218 = HEAP32[1032>>2]|0;
      $219 = HEAP32[1040>>2]|0;
      $220 = $5;
      $221 = (($220) + ($219<<2)|0);
      HEAP32[$221>>2] = $218;
     }
     $222 = HEAP32[1032>>2]|0;
     $223 = ($222|0)>(0);
     if ($223) {
      $224 = HEAP32[1040>>2]|0;
      $225 = HEAP32[1032>>2]|0;
      $226 = $4;
      $227 = (($226) + ($225<<2)|0);
      HEAP32[$227>>2] = $224;
     }
     $228 = HEAP32[1032>>2]|0;
     $229 = (0 - ($228))|0;
     HEAP32[1048>>2] = $229;
     $230 = HEAP32[1032>>2]|0;
     $231 = ($230|0)<(0);
     if ($231) {
      $232 = HEAP32[1040>>2]|0;
      $233 = HEAP32[1048>>2]|0;
      $234 = $3;
      $235 = (($234) + ($233<<2)|0);
      HEAP32[$235>>2] = $232;
     }
    }
   }
   $236 = HEAP32[1024>>2]|0;
   $237 = $1;
   $238 = (($237) + ($236<<2)|0);
   $239 = HEAP32[$238>>2]|0;
   HEAP32[992>>2] = $239;
   $240 = HEAP32[1024>>2]|0;
   $241 = (($240) + 1)|0;
   $242 = $1;
   $243 = (($242) + ($241<<2)|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = (($244) - 1)|0;
   HEAP32[1000>>2] = $245;
   $246 = HEAP32[992>>2]|0;
   HEAP32[1056>>2] = $246;
   $247 = HEAP32[1000>>2]|0;
   $i__2 = $247;
   $248 = HEAP32[992>>2]|0;
   HEAP32[1008>>2] = $248;
   while(1) {
    $249 = HEAP32[1008>>2]|0;
    $250 = $i__2;
    $251 = ($249|0)<=($250|0);
    if (!($251)) {
     break;
    }
    $252 = HEAP32[1008>>2]|0;
    $253 = $2;
    $254 = (($253) + ($252<<2)|0);
    $255 = HEAP32[$254>>2]|0;
    HEAP32[976>>2] = $255;
    $256 = HEAP32[976>>2]|0;
    $257 = ($256|0)==(0);
    if ($257) {
     break;
    }
    $258 = HEAP32[976>>2]|0;
    $259 = $8;
    $260 = (($259) + ($258<<2)|0);
    $261 = HEAP32[$260>>2]|0;
    $262 = $10;
    $263 = HEAP32[$262>>2]|0;
    $264 = ($261|0)>=($263|0);
    if (!($264)) {
     $265 = HEAP32[976>>2]|0;
     $266 = HEAP32[1056>>2]|0;
     $267 = $2;
     $268 = (($267) + ($266<<2)|0);
     HEAP32[$268>>2] = $265;
     $269 = HEAP32[1056>>2]|0;
     $270 = (($269) + 1)|0;
     HEAP32[1056>>2] = $270;
    }
    $271 = HEAP32[1008>>2]|0;
    $272 = (($271) + 1)|0;
    HEAP32[1008>>2] = $272;
   }
   $273 = HEAP32[1056>>2]|0;
   $274 = HEAP32[992>>2]|0;
   $275 = (($273) - ($274))|0;
   HEAP32[1064>>2] = $275;
   $276 = HEAP32[1064>>2]|0;
   $277 = ($276|0)>(0);
   if ($277) {
    $308 = HEAP32[1064>>2]|0;
    $309 = (($308) + 1)|0;
    $310 = HEAP32[1024>>2]|0;
    $311 = $4;
    $312 = (($311) + ($310<<2)|0);
    HEAP32[$312>>2] = $309;
    $313 = HEAP32[1024>>2]|0;
    $314 = $5;
    $315 = (($314) + ($313<<2)|0);
    HEAP32[$315>>2] = 0;
    $316 = $0;
    $317 = HEAP32[$316>>2]|0;
    $318 = HEAP32[1056>>2]|0;
    $319 = $2;
    $320 = (($319) + ($318<<2)|0);
    HEAP32[$320>>2] = $317;
    $321 = HEAP32[1056>>2]|0;
    $322 = (($321) + 1)|0;
    HEAP32[1056>>2] = $322;
    $323 = HEAP32[1056>>2]|0;
    $324 = HEAP32[1000>>2]|0;
    $325 = ($323|0)<=($324|0);
    if ($325) {
     $326 = HEAP32[1056>>2]|0;
     $327 = $2;
     $328 = (($327) + ($326<<2)|0);
     HEAP32[$328>>2] = 0;
    }
   } else {
    $278 = HEAP32[1024>>2]|0;
    $279 = $6;
    $280 = (($279) + ($278<<2)|0);
    $281 = HEAP32[$280>>2]|0;
    $282 = $0;
    $283 = HEAP32[$282>>2]|0;
    $284 = $6;
    $285 = (($284) + ($283<<2)|0);
    $286 = HEAP32[$285>>2]|0;
    $287 = (($286) + ($281))|0;
    HEAP32[$285>>2] = $287;
    $288 = HEAP32[1024>>2]|0;
    $289 = $6;
    $290 = (($289) + ($288<<2)|0);
    HEAP32[$290>>2] = 0;
    $291 = $9;
    $292 = HEAP32[$291>>2]|0;
    $293 = HEAP32[1024>>2]|0;
    $294 = $8;
    $295 = (($294) + ($293<<2)|0);
    HEAP32[$295>>2] = $292;
    $296 = $0;
    $297 = HEAP32[$296>>2]|0;
    $298 = (0 - ($297))|0;
    $299 = HEAP32[1024>>2]|0;
    $300 = $4;
    $301 = (($300) + ($299<<2)|0);
    HEAP32[$301>>2] = $298;
    $302 = $9;
    $303 = HEAP32[$302>>2]|0;
    $304 = (0 - ($303))|0;
    $305 = HEAP32[1024>>2]|0;
    $306 = $5;
    $307 = (($306) + ($305<<2)|0);
    HEAP32[$307>>2] = $304;
   }
   $329 = HEAP32[968>>2]|0;
   $330 = (($329) + 1)|0;
   HEAP32[968>>2] = $330;
  }
 }
 if ((label|0) == 49) {
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _mmdupd_($ehead,$neqns,$xadj,$adjncy,$delta,$mdeg,$dhead,$dforw,$dbakw,$qsize,$llist,$marker,$maxint,$tag) {
 $ehead = $ehead|0;
 $neqns = $neqns|0;
 $xadj = $xadj|0;
 $adjncy = $adjncy|0;
 $delta = $delta|0;
 $mdeg = $mdeg|0;
 $dhead = $dhead|0;
 $dforw = $dforw|0;
 $dbakw = $dbakw|0;
 $qsize = $qsize|0;
 $llist = $llist|0;
 $marker = $marker|0;
 $maxint = $maxint|0;
 $tag = $tag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, $i__2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $ehead;
 $1 = $neqns;
 $2 = $xadj;
 $3 = $adjncy;
 $4 = $delta;
 $5 = $mdeg;
 $6 = $dhead;
 $7 = $dforw;
 $8 = $dbakw;
 $9 = $qsize;
 $10 = $llist;
 $11 = $marker;
 $12 = $maxint;
 $13 = $tag;
 $14 = $11;
 $15 = ((($14)) + -4|0);
 $11 = $15;
 $16 = $10;
 $17 = ((($16)) + -4|0);
 $10 = $17;
 $18 = $9;
 $19 = ((($18)) + -4|0);
 $9 = $19;
 $20 = $8;
 $21 = ((($20)) + -4|0);
 $8 = $21;
 $22 = $7;
 $23 = ((($22)) + -4|0);
 $7 = $23;
 $24 = $6;
 $25 = ((($24)) + -4|0);
 $6 = $25;
 $26 = $3;
 $27 = ((($26)) + -4|0);
 $3 = $27;
 $28 = $2;
 $29 = ((($28)) + -4|0);
 $2 = $29;
 $30 = $5;
 $31 = HEAP32[$30>>2]|0;
 $32 = $4;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + ($33))|0;
 HEAP32[1072>>2] = $34;
 $35 = $0;
 $36 = HEAP32[$35>>2]|0;
 HEAP32[1080>>2] = $36;
 while(1) {
  $37 = HEAP32[1080>>2]|0;
  $38 = ($37|0)<=(0);
  if ($38) {
   break;
  }
  $39 = $13;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[1072>>2]|0;
  $42 = (($40) + ($41))|0;
  HEAP32[1088>>2] = $42;
  $43 = HEAP32[1088>>2]|0;
  $44 = $12;
  $45 = HEAP32[$44>>2]|0;
  $46 = ($43|0)<($45|0);
  if (!($46)) {
   $47 = $13;
   HEAP32[$47>>2] = 1;
   $48 = $1;
   $49 = HEAP32[$48>>2]|0;
   $i__1 = $49;
   HEAP32[1096>>2] = 1;
   while(1) {
    $50 = HEAP32[1096>>2]|0;
    $51 = $i__1;
    $52 = ($50|0)<=($51|0);
    if (!($52)) {
     break;
    }
    $53 = HEAP32[1096>>2]|0;
    $54 = $11;
    $55 = (($54) + ($53<<2)|0);
    $56 = HEAP32[$55>>2]|0;
    $57 = $12;
    $58 = HEAP32[$57>>2]|0;
    $59 = ($56|0)<($58|0);
    if ($59) {
     $60 = HEAP32[1096>>2]|0;
     $61 = $11;
     $62 = (($61) + ($60<<2)|0);
     HEAP32[$62>>2] = 0;
    }
    $63 = HEAP32[1096>>2]|0;
    $64 = (($63) + 1)|0;
    HEAP32[1096>>2] = $64;
   }
   $65 = $13;
   $66 = HEAP32[$65>>2]|0;
   $67 = HEAP32[1072>>2]|0;
   $68 = (($66) + ($67))|0;
   HEAP32[1088>>2] = $68;
  }
  HEAP32[1104>>2] = 0;
  HEAP32[1112>>2] = 0;
  HEAP32[1120>>2] = 0;
  $69 = HEAP32[1080>>2]|0;
  HEAP32[1128>>2] = $69;
  L14: while(1) {
   $70 = HEAP32[1128>>2]|0;
   $71 = $2;
   $72 = (($71) + ($70<<2)|0);
   $73 = HEAP32[$72>>2]|0;
   HEAP32[1136>>2] = $73;
   $74 = HEAP32[1128>>2]|0;
   $75 = (($74) + 1)|0;
   $76 = $2;
   $77 = (($76) + ($75<<2)|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = (($78) - 1)|0;
   HEAP32[1144>>2] = $79;
   $80 = HEAP32[1144>>2]|0;
   $i__1 = $80;
   $81 = HEAP32[1136>>2]|0;
   HEAP32[1096>>2] = $81;
   while(1) {
    $82 = HEAP32[1096>>2]|0;
    $83 = $i__1;
    $84 = ($82|0)<=($83|0);
    if (!($84)) {
     break L14;
    }
    $85 = HEAP32[1096>>2]|0;
    $86 = $3;
    $87 = (($86) + ($85<<2)|0);
    $88 = HEAP32[$87>>2]|0;
    HEAP32[1152>>2] = $88;
    $89 = HEAP32[1152>>2]|0;
    $90 = (0 - ($89))|0;
    HEAP32[1128>>2] = $90;
    $91 = HEAP32[1152>>2]|0;
    $92 = ($91|0)<(0);
    if ($92) {
     continue L14;
    }
    $93 = HEAP32[1152>>2]|0;
    $94 = ($93|0)==(0);
    if ($94) {
     break L14;
    }
    $95 = HEAP32[1152>>2]|0;
    $96 = $9;
    $97 = (($96) + ($95<<2)|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)==(0);
    do {
     if (!($99)) {
      $100 = HEAP32[1152>>2]|0;
      $101 = $9;
      $102 = (($101) + ($100<<2)|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = HEAP32[1120>>2]|0;
      $105 = (($104) + ($103))|0;
      HEAP32[1120>>2] = $105;
      $106 = HEAP32[1088>>2]|0;
      $107 = HEAP32[1152>>2]|0;
      $108 = $11;
      $109 = (($108) + ($107<<2)|0);
      HEAP32[$109>>2] = $106;
      $110 = HEAP32[1152>>2]|0;
      $111 = $8;
      $112 = (($111) + ($110<<2)|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = ($113|0)!=(0);
      if (!($114)) {
       $115 = HEAP32[1152>>2]|0;
       $116 = $7;
       $117 = (($116) + ($115<<2)|0);
       $118 = HEAP32[$117>>2]|0;
       $119 = ($118|0)==(2);
       if ($119) {
        $125 = HEAP32[1104>>2]|0;
        $126 = HEAP32[1152>>2]|0;
        $127 = $10;
        $128 = (($127) + ($126<<2)|0);
        HEAP32[$128>>2] = $125;
        $129 = HEAP32[1152>>2]|0;
        HEAP32[1104>>2] = $129;
        break;
       } else {
        $120 = HEAP32[1112>>2]|0;
        $121 = HEAP32[1152>>2]|0;
        $122 = $10;
        $123 = (($122) + ($121<<2)|0);
        HEAP32[$123>>2] = $120;
        $124 = HEAP32[1152>>2]|0;
        HEAP32[1112>>2] = $124;
        break;
       }
      }
     }
    } while(0);
    $130 = HEAP32[1096>>2]|0;
    $131 = (($130) + 1)|0;
    HEAP32[1096>>2] = $131;
   }
  }
  $132 = HEAP32[1104>>2]|0;
  HEAP32[1152>>2] = $132;
  HEAP32[1160>>2] = 1;
  L29: while(1) {
   $133 = HEAP32[1152>>2]|0;
   $134 = ($133|0)<=(0);
   L31: do {
    if ($134) {
     $273 = HEAP32[1112>>2]|0;
     HEAP32[1152>>2] = $273;
     HEAP32[1160>>2] = 0;
     label = 44;
    } else {
     $135 = HEAP32[1152>>2]|0;
     $136 = $8;
     $137 = (($136) + ($135<<2)|0);
     $138 = HEAP32[$137>>2]|0;
     $139 = ($138|0)!=(0);
     if ($139) {
      label = 64;
     } else {
      $140 = $13;
      $141 = HEAP32[$140>>2]|0;
      $142 = (($141) + 1)|0;
      HEAP32[$140>>2] = $142;
      $143 = HEAP32[1120>>2]|0;
      HEAP32[1168>>2] = $143;
      $144 = HEAP32[1152>>2]|0;
      $145 = $2;
      $146 = (($145) + ($144<<2)|0);
      $147 = HEAP32[$146>>2]|0;
      HEAP32[1136>>2] = $147;
      $148 = HEAP32[1136>>2]|0;
      $149 = $3;
      $150 = (($149) + ($148<<2)|0);
      $151 = HEAP32[$150>>2]|0;
      HEAP32[1176>>2] = $151;
      $152 = HEAP32[1176>>2]|0;
      $153 = HEAP32[1080>>2]|0;
      $154 = ($152|0)==($153|0);
      if ($154) {
       $155 = HEAP32[1136>>2]|0;
       $156 = (($155) + 1)|0;
       $157 = $3;
       $158 = (($157) + ($156<<2)|0);
       $159 = HEAP32[$158>>2]|0;
       HEAP32[1176>>2] = $159;
      }
      $160 = HEAP32[1176>>2]|0;
      HEAP32[1128>>2] = $160;
      $161 = HEAP32[1176>>2]|0;
      $162 = $7;
      $163 = (($162) + ($161<<2)|0);
      $164 = HEAP32[$163>>2]|0;
      $165 = ($164|0)<(0);
      if (!($165)) {
       $166 = HEAP32[1176>>2]|0;
       $167 = $9;
       $168 = (($167) + ($166<<2)|0);
       $169 = HEAP32[$168>>2]|0;
       $170 = HEAP32[1168>>2]|0;
       $171 = (($170) + ($169))|0;
       HEAP32[1168>>2] = $171;
       label = 60;
       break;
      }
      L39: while(1) {
       $172 = HEAP32[1128>>2]|0;
       $173 = $2;
       $174 = (($173) + ($172<<2)|0);
       $175 = HEAP32[$174>>2]|0;
       HEAP32[1136>>2] = $175;
       $176 = HEAP32[1128>>2]|0;
       $177 = (($176) + 1)|0;
       $178 = $2;
       $179 = (($178) + ($177<<2)|0);
       $180 = HEAP32[$179>>2]|0;
       $181 = (($180) - 1)|0;
       HEAP32[1144>>2] = $181;
       $182 = HEAP32[1144>>2]|0;
       $i__1 = $182;
       $183 = HEAP32[1136>>2]|0;
       HEAP32[1096>>2] = $183;
       while(1) {
        $184 = HEAP32[1096>>2]|0;
        $185 = $i__1;
        $186 = ($184|0)<=($185|0);
        if (!($186)) {
         label = 60;
         break L31;
        }
        $187 = HEAP32[1096>>2]|0;
        $188 = $3;
        $189 = (($188) + ($187<<2)|0);
        $190 = HEAP32[$189>>2]|0;
        HEAP32[1184>>2] = $190;
        $191 = HEAP32[1184>>2]|0;
        $192 = (0 - ($191))|0;
        HEAP32[1128>>2] = $192;
        $193 = HEAP32[1184>>2]|0;
        $194 = HEAP32[1152>>2]|0;
        $195 = ($193|0)==($194|0);
        do {
         if (!($195)) {
          $196 = HEAP32[1184>>2]|0;
          $197 = ($196|0)<(0);
          if ($197) {
           continue L39;
          }
          $198 = HEAP32[1184>>2]|0;
          $199 = ($198|0)==(0);
          if ($199) {
           label = 60;
           break L31;
          }
          $200 = HEAP32[1184>>2]|0;
          $201 = $9;
          $202 = (($201) + ($200<<2)|0);
          $203 = HEAP32[$202>>2]|0;
          $204 = ($203|0)==(0);
          if (!($204)) {
           $205 = HEAP32[1184>>2]|0;
           $206 = $11;
           $207 = (($206) + ($205<<2)|0);
           $208 = HEAP32[$207>>2]|0;
           $209 = $13;
           $210 = HEAP32[$209>>2]|0;
           $211 = ($208|0)>=($210|0);
           if (!($211)) {
            $212 = $13;
            $213 = HEAP32[$212>>2]|0;
            $214 = HEAP32[1184>>2]|0;
            $215 = $11;
            $216 = (($215) + ($214<<2)|0);
            HEAP32[$216>>2] = $213;
            $217 = HEAP32[1184>>2]|0;
            $218 = $9;
            $219 = (($218) + ($217<<2)|0);
            $220 = HEAP32[$219>>2]|0;
            $221 = HEAP32[1168>>2]|0;
            $222 = (($221) + ($220))|0;
            HEAP32[1168>>2] = $222;
            break;
           }
           $223 = HEAP32[1184>>2]|0;
           $224 = $8;
           $225 = (($224) + ($223<<2)|0);
           $226 = HEAP32[$225>>2]|0;
           $227 = ($226|0)!=(0);
           if (!($227)) {
            $228 = HEAP32[1184>>2]|0;
            $229 = $7;
            $230 = (($229) + ($228<<2)|0);
            $231 = HEAP32[$230>>2]|0;
            $232 = ($231|0)!=(2);
            $233 = HEAP32[1184>>2]|0;
            if (!($232)) {
             $234 = $9;
             $235 = (($234) + ($233<<2)|0);
             $236 = HEAP32[$235>>2]|0;
             $237 = HEAP32[1152>>2]|0;
             $238 = $9;
             $239 = (($238) + ($237<<2)|0);
             $240 = HEAP32[$239>>2]|0;
             $241 = (($240) + ($236))|0;
             HEAP32[$239>>2] = $241;
             $242 = HEAP32[1184>>2]|0;
             $243 = $9;
             $244 = (($243) + ($242<<2)|0);
             HEAP32[$244>>2] = 0;
             $245 = $12;
             $246 = HEAP32[$245>>2]|0;
             $247 = HEAP32[1184>>2]|0;
             $248 = $11;
             $249 = (($248) + ($247<<2)|0);
             HEAP32[$249>>2] = $246;
             $250 = HEAP32[1152>>2]|0;
             $251 = (0 - ($250))|0;
             $252 = HEAP32[1184>>2]|0;
             $253 = $7;
             $254 = (($253) + ($252<<2)|0);
             HEAP32[$254>>2] = $251;
             $255 = $12;
             $256 = HEAP32[$255>>2]|0;
             $257 = (0 - ($256))|0;
             $258 = HEAP32[1184>>2]|0;
             $259 = $8;
             $260 = (($259) + ($258<<2)|0);
             HEAP32[$260>>2] = $257;
             break;
            }
            $261 = $8;
            $262 = (($261) + ($233<<2)|0);
            $263 = HEAP32[$262>>2]|0;
            $264 = ($263|0)==(0);
            if (!($264)) {
             break;
            }
            $265 = $12;
            $266 = HEAP32[$265>>2]|0;
            $267 = (0 - ($266))|0;
            $268 = HEAP32[1184>>2]|0;
            $269 = $8;
            $270 = (($269) + ($268<<2)|0);
            HEAP32[$270>>2] = $267;
           }
          }
         }
        } while(0);
        $271 = HEAP32[1096>>2]|0;
        $272 = (($271) + 1)|0;
        HEAP32[1096>>2] = $272;
       }
      }
     }
    }
   } while(0);
   L59: while(1) {
    if ((label|0) == 44) {
     label = 0;
     $274 = HEAP32[1152>>2]|0;
     $275 = ($274|0)<=(0);
     if ($275) {
      break L29;
     }
     $276 = HEAP32[1152>>2]|0;
     $277 = $8;
     $278 = (($277) + ($276<<2)|0);
     $279 = HEAP32[$278>>2]|0;
     $280 = ($279|0)!=(0);
     if ($280) {
      label = 64;
      continue;
     }
     $281 = $13;
     $282 = HEAP32[$281>>2]|0;
     $283 = (($282) + 1)|0;
     HEAP32[$281>>2] = $283;
     $284 = HEAP32[1120>>2]|0;
     HEAP32[1168>>2] = $284;
     $285 = HEAP32[1152>>2]|0;
     $286 = $2;
     $287 = (($286) + ($285<<2)|0);
     $288 = HEAP32[$287>>2]|0;
     HEAP32[1136>>2] = $288;
     $289 = HEAP32[1152>>2]|0;
     $290 = (($289) + 1)|0;
     $291 = $2;
     $292 = (($291) + ($290<<2)|0);
     $293 = HEAP32[$292>>2]|0;
     $294 = (($293) - 1)|0;
     HEAP32[1144>>2] = $294;
     $295 = HEAP32[1144>>2]|0;
     $i__1 = $295;
     $296 = HEAP32[1136>>2]|0;
     HEAP32[1096>>2] = $296;
     while(1) {
      $297 = HEAP32[1096>>2]|0;
      $298 = $i__1;
      $299 = ($297|0)<=($298|0);
      if (!($299)) {
       label = 60;
       continue L59;
      }
      $300 = HEAP32[1096>>2]|0;
      $301 = $3;
      $302 = (($301) + ($300<<2)|0);
      $303 = HEAP32[$302>>2]|0;
      HEAP32[1176>>2] = $303;
      $304 = HEAP32[1176>>2]|0;
      $305 = ($304|0)==(0);
      if ($305) {
       label = 60;
       continue L59;
      }
      $306 = HEAP32[1176>>2]|0;
      $307 = $11;
      $308 = (($307) + ($306<<2)|0);
      $309 = HEAP32[$308>>2]|0;
      $310 = $13;
      $311 = HEAP32[$310>>2]|0;
      $312 = ($309|0)>=($311|0);
      L68: do {
       if (!($312)) {
        $313 = $13;
        $314 = HEAP32[$313>>2]|0;
        $315 = HEAP32[1176>>2]|0;
        $316 = $11;
        $317 = (($316) + ($315<<2)|0);
        HEAP32[$317>>2] = $314;
        $318 = HEAP32[1176>>2]|0;
        HEAP32[1128>>2] = $318;
        $319 = HEAP32[1176>>2]|0;
        $320 = $7;
        $321 = (($320) + ($319<<2)|0);
        $322 = HEAP32[$321>>2]|0;
        $323 = ($322|0)<(0);
        if (!($323)) {
         $324 = HEAP32[1176>>2]|0;
         $325 = $9;
         $326 = (($325) + ($324<<2)|0);
         $327 = HEAP32[$326>>2]|0;
         $328 = HEAP32[1168>>2]|0;
         $329 = (($328) + ($327))|0;
         HEAP32[1168>>2] = $329;
         break;
        }
        L72: while(1) {
         $330 = HEAP32[1128>>2]|0;
         $331 = $2;
         $332 = (($331) + ($330<<2)|0);
         $333 = HEAP32[$332>>2]|0;
         HEAP32[1192>>2] = $333;
         $334 = HEAP32[1128>>2]|0;
         $335 = (($334) + 1)|0;
         $336 = $2;
         $337 = (($336) + ($335<<2)|0);
         $338 = HEAP32[$337>>2]|0;
         $339 = (($338) - 1)|0;
         HEAP32[1200>>2] = $339;
         $340 = HEAP32[1200>>2]|0;
         $i__2 = $340;
         $341 = HEAP32[1192>>2]|0;
         HEAP32[1208>>2] = $341;
         while(1) {
          $342 = HEAP32[1208>>2]|0;
          $343 = $i__2;
          $344 = ($342|0)<=($343|0);
          if (!($344)) {
           break L68;
          }
          $345 = HEAP32[1208>>2]|0;
          $346 = $3;
          $347 = (($346) + ($345<<2)|0);
          $348 = HEAP32[$347>>2]|0;
          HEAP32[1184>>2] = $348;
          $349 = HEAP32[1184>>2]|0;
          $350 = (0 - ($349))|0;
          HEAP32[1128>>2] = $350;
          $351 = HEAP32[1184>>2]|0;
          $352 = ($351|0)<(0);
          if ($352) {
           continue L72;
          }
          $353 = HEAP32[1184>>2]|0;
          $354 = ($353|0)==(0);
          if ($354) {
           break L68;
          }
          $355 = HEAP32[1184>>2]|0;
          $356 = $11;
          $357 = (($356) + ($355<<2)|0);
          $358 = HEAP32[$357>>2]|0;
          $359 = $13;
          $360 = HEAP32[$359>>2]|0;
          $361 = ($358|0)>=($360|0);
          if (!($361)) {
           $362 = $13;
           $363 = HEAP32[$362>>2]|0;
           $364 = HEAP32[1184>>2]|0;
           $365 = $11;
           $366 = (($365) + ($364<<2)|0);
           HEAP32[$366>>2] = $363;
           $367 = HEAP32[1184>>2]|0;
           $368 = $9;
           $369 = (($368) + ($367<<2)|0);
           $370 = HEAP32[$369>>2]|0;
           $371 = HEAP32[1168>>2]|0;
           $372 = (($371) + ($370))|0;
           HEAP32[1168>>2] = $372;
          }
          $373 = HEAP32[1208>>2]|0;
          $374 = (($373) + 1)|0;
          HEAP32[1208>>2] = $374;
         }
        }
       }
      } while(0);
      $375 = HEAP32[1096>>2]|0;
      $376 = (($375) + 1)|0;
      HEAP32[1096>>2] = $376;
     }
    }
    else if ((label|0) == 60) {
     label = 0;
     $377 = HEAP32[1168>>2]|0;
     $378 = HEAP32[1152>>2]|0;
     $379 = $9;
     $380 = (($379) + ($378<<2)|0);
     $381 = HEAP32[$380>>2]|0;
     $382 = (($377) - ($381))|0;
     $383 = (($382) + 1)|0;
     HEAP32[1168>>2] = $383;
     $384 = HEAP32[1168>>2]|0;
     $385 = $6;
     $386 = (($385) + ($384<<2)|0);
     $387 = HEAP32[$386>>2]|0;
     HEAP32[1216>>2] = $387;
     $388 = HEAP32[1216>>2]|0;
     $389 = HEAP32[1152>>2]|0;
     $390 = $7;
     $391 = (($390) + ($389<<2)|0);
     HEAP32[$391>>2] = $388;
     $392 = HEAP32[1168>>2]|0;
     $393 = (0 - ($392))|0;
     $394 = HEAP32[1152>>2]|0;
     $395 = $8;
     $396 = (($395) + ($394<<2)|0);
     HEAP32[$396>>2] = $393;
     $397 = HEAP32[1216>>2]|0;
     $398 = ($397|0)>(0);
     if ($398) {
      $399 = HEAP32[1152>>2]|0;
      $400 = HEAP32[1216>>2]|0;
      $401 = $8;
      $402 = (($401) + ($400<<2)|0);
      HEAP32[$402>>2] = $399;
     }
     $403 = HEAP32[1152>>2]|0;
     $404 = HEAP32[1168>>2]|0;
     $405 = $6;
     $406 = (($405) + ($404<<2)|0);
     HEAP32[$406>>2] = $403;
     $407 = HEAP32[1168>>2]|0;
     $408 = $5;
     $409 = HEAP32[$408>>2]|0;
     $410 = ($407|0)<($409|0);
     if (!($410)) {
      label = 64;
      continue;
     }
     $411 = HEAP32[1168>>2]|0;
     $412 = $5;
     HEAP32[$412>>2] = $411;
     label = 64;
     continue;
    }
    else if ((label|0) == 64) {
     label = 0;
     $413 = HEAP32[1152>>2]|0;
     $414 = $10;
     $415 = (($414) + ($413<<2)|0);
     $416 = HEAP32[$415>>2]|0;
     HEAP32[1152>>2] = $416;
     $417 = HEAP32[1160>>2]|0;
     $418 = ($417|0)==(1);
     if ($418) {
      continue L29;
     } else {
      label = 44;
      continue;
     }
    }
   }
  }
  $419 = HEAP32[1088>>2]|0;
  $420 = $13;
  HEAP32[$420>>2] = $419;
  $421 = HEAP32[1080>>2]|0;
  $422 = $10;
  $423 = (($422) + ($421<<2)|0);
  $424 = HEAP32[$423>>2]|0;
  HEAP32[1080>>2] = $424;
 }
 STACKTOP = sp;return 0;
}
function _mmdnum_($neqns,$perm,$invp,$qsize) {
 $neqns = $neqns|0;
 $perm = $perm|0;
 $invp = $invp|0;
 $qsize = $qsize|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $neqns;
 $1 = $perm;
 $2 = $invp;
 $3 = $qsize;
 $4 = $3;
 $5 = ((($4)) + -4|0);
 $3 = $5;
 $6 = $2;
 $7 = ((($6)) + -4|0);
 $2 = $7;
 $8 = $1;
 $9 = ((($8)) + -4|0);
 $1 = $9;
 $10 = $0;
 $11 = HEAP32[$10>>2]|0;
 $i__1 = $11;
 HEAP32[1224>>2] = 1;
 while(1) {
  $12 = HEAP32[1224>>2]|0;
  $13 = $i__1;
  $14 = ($12|0)<=($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[1224>>2]|0;
  $16 = $3;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[1232>>2] = $18;
  $19 = HEAP32[1232>>2]|0;
  $20 = ($19|0)<=(0);
  if ($20) {
   $21 = HEAP32[1224>>2]|0;
   $22 = $2;
   $23 = (($22) + ($21<<2)|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[1224>>2]|0;
   $26 = $1;
   $27 = (($26) + ($25<<2)|0);
   HEAP32[$27>>2] = $24;
  }
  $28 = HEAP32[1232>>2]|0;
  $29 = ($28|0)>(0);
  if ($29) {
   $30 = HEAP32[1224>>2]|0;
   $31 = $2;
   $32 = (($31) + ($30<<2)|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (0 - ($33))|0;
   $35 = HEAP32[1224>>2]|0;
   $36 = $1;
   $37 = (($36) + ($35<<2)|0);
   HEAP32[$37>>2] = $34;
  }
  $38 = HEAP32[1224>>2]|0;
  $39 = (($38) + 1)|0;
  HEAP32[1224>>2] = $39;
 }
 $40 = $0;
 $41 = HEAP32[$40>>2]|0;
 $i__1 = $41;
 HEAP32[1224>>2] = 1;
 while(1) {
  $42 = HEAP32[1224>>2]|0;
  $43 = $i__1;
  $44 = ($42|0)<=($43|0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[1224>>2]|0;
  $46 = $1;
  $47 = (($46) + ($45<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($48|0)>(0);
  L14: do {
   if (!($49)) {
    $50 = HEAP32[1224>>2]|0;
    HEAP32[1240>>2] = $50;
    while(1) {
     $51 = HEAP32[1240>>2]|0;
     $52 = $1;
     $53 = (($52) + ($51<<2)|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = ($54|0)>(0);
     $56 = HEAP32[1240>>2]|0;
     if ($55) {
      break;
     }
     $57 = $1;
     $58 = (($57) + ($56<<2)|0);
     $59 = HEAP32[$58>>2]|0;
     $60 = (0 - ($59))|0;
     HEAP32[1240>>2] = $60;
    }
    HEAP32[1248>>2] = $56;
    $61 = HEAP32[1248>>2]|0;
    $62 = $1;
    $63 = (($62) + ($61<<2)|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = (($64) + 1)|0;
    HEAP32[1256>>2] = $65;
    $66 = HEAP32[1256>>2]|0;
    $67 = (0 - ($66))|0;
    $68 = HEAP32[1224>>2]|0;
    $69 = $2;
    $70 = (($69) + ($68<<2)|0);
    HEAP32[$70>>2] = $67;
    $71 = HEAP32[1256>>2]|0;
    $72 = HEAP32[1248>>2]|0;
    $73 = $1;
    $74 = (($73) + ($72<<2)|0);
    HEAP32[$74>>2] = $71;
    $75 = HEAP32[1224>>2]|0;
    HEAP32[1240>>2] = $75;
    while(1) {
     $76 = HEAP32[1240>>2]|0;
     $77 = $1;
     $78 = (($77) + ($76<<2)|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = (0 - ($79))|0;
     HEAP32[1264>>2] = $80;
     $81 = HEAP32[1264>>2]|0;
     $82 = ($81|0)<=(0);
     if ($82) {
      break L14;
     }
     $83 = HEAP32[1248>>2]|0;
     $84 = (0 - ($83))|0;
     $85 = HEAP32[1240>>2]|0;
     $86 = $1;
     $87 = (($86) + ($85<<2)|0);
     HEAP32[$87>>2] = $84;
     $88 = HEAP32[1264>>2]|0;
     HEAP32[1240>>2] = $88;
    }
   }
  } while(0);
  $89 = HEAP32[1224>>2]|0;
  $90 = (($89) + 1)|0;
  HEAP32[1224>>2] = $90;
 }
 $91 = $0;
 $92 = HEAP32[$91>>2]|0;
 $i__1 = $92;
 HEAP32[1224>>2] = 1;
 while(1) {
  $93 = HEAP32[1224>>2]|0;
  $94 = $i__1;
  $95 = ($93|0)<=($94|0);
  if (!($95)) {
   break;
  }
  $96 = HEAP32[1224>>2]|0;
  $97 = $2;
  $98 = (($97) + ($96<<2)|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = (0 - ($99))|0;
  HEAP32[1256>>2] = $100;
  $101 = HEAP32[1256>>2]|0;
  $102 = HEAP32[1224>>2]|0;
  $103 = $2;
  $104 = (($103) + ($102<<2)|0);
  HEAP32[$104>>2] = $101;
  $105 = HEAP32[1224>>2]|0;
  $106 = HEAP32[1256>>2]|0;
  $107 = $1;
  $108 = (($107) + ($106<<2)|0);
  HEAP32[$108>>2] = $105;
  $109 = HEAP32[1224>>2]|0;
  $110 = (($109) + 1)|0;
  HEAP32[1224>>2] = $110;
 }
 STACKTOP = sp;return 0;
}
function _sp_coletree($acolst,$acolend,$arow,$nr,$nc,$parent) {
 $acolst = $acolst|0;
 $acolend = $acolend|0;
 $arow = $arow|0;
 $nr = $nr|0;
 $nc = $nc|0;
 $parent = $parent|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $col = 0, $cset = 0;
 var $firstcol = 0, $p = 0, $pp = 0, $root = 0, $row = 0, $rroot = 0, $rset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $pp = sp;
 $0 = $acolst;
 $1 = $acolend;
 $2 = $arow;
 $3 = $nr;
 $4 = $nc;
 $5 = $parent;
 $6 = $4;
 $7 = (_mxCallocInt($6)|0);
 $root = $7;
 $8 = $4;
 _initialize_disjoint_sets($8,$pp);
 $9 = $3;
 $10 = (_mxCallocInt($9)|0);
 $firstcol = $10;
 $row = 0;
 while(1) {
  $11 = $row;
  $12 = $3;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $4;
  $15 = $row;
  $16 = (($15) + 1)|0;
  $row = $16;
  $17 = $firstcol;
  $18 = (($17) + ($15<<2)|0);
  HEAP32[$18>>2] = $14;
 }
 $col = 0;
 while(1) {
  $19 = $col;
  $20 = $4;
  $21 = ($19|0)<($20|0);
  if (!($21)) {
   break;
  }
  $22 = $col;
  $23 = $0;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $p = $25;
  while(1) {
   $26 = $p;
   $27 = $col;
   $28 = $1;
   $29 = (($28) + ($27<<2)|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($26|0)<($30|0);
   if (!($31)) {
    break;
   }
   $32 = $p;
   $33 = $2;
   $34 = (($33) + ($32<<2)|0);
   $35 = HEAP32[$34>>2]|0;
   $row = $35;
   $36 = $row;
   $37 = $firstcol;
   $38 = (($37) + ($36<<2)|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = $col;
   $41 = ($39|0)<($40|0);
   if ($41) {
    $42 = $row;
    $43 = $firstcol;
    $44 = (($43) + ($42<<2)|0);
    $45 = HEAP32[$44>>2]|0;
    $50 = $45;
   } else {
    $46 = $col;
    $50 = $46;
   }
   $47 = $row;
   $48 = $firstcol;
   $49 = (($48) + ($47<<2)|0);
   HEAP32[$49>>2] = $50;
   $51 = $p;
   $52 = (($51) + 1)|0;
   $p = $52;
  }
  $53 = $col;
  $54 = (($53) + 1)|0;
  $col = $54;
 }
 $col = 0;
 while(1) {
  $55 = $col;
  $56 = $4;
  $57 = ($55|0)<($56|0);
  if (!($57)) {
   break;
  }
  $58 = $col;
  $59 = HEAP32[$pp>>2]|0;
  $60 = (_make_set($58,$59)|0);
  $cset = $60;
  $61 = $col;
  $62 = $cset;
  $63 = $root;
  $64 = (($63) + ($62<<2)|0);
  HEAP32[$64>>2] = $61;
  $65 = $4;
  $66 = $col;
  $67 = $5;
  $68 = (($67) + ($66<<2)|0);
  HEAP32[$68>>2] = $65;
  $69 = $col;
  $70 = $0;
  $71 = (($70) + ($69<<2)|0);
  $72 = HEAP32[$71>>2]|0;
  $p = $72;
  while(1) {
   $73 = $p;
   $74 = $col;
   $75 = $1;
   $76 = (($75) + ($74<<2)|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = ($73|0)<($77|0);
   if (!($78)) {
    break;
   }
   $79 = $p;
   $80 = $2;
   $81 = (($80) + ($79<<2)|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = $firstcol;
   $84 = (($83) + ($82<<2)|0);
   $85 = HEAP32[$84>>2]|0;
   $row = $85;
   $86 = $row;
   $87 = $col;
   $88 = ($86|0)>=($87|0);
   if (!($88)) {
    $89 = $row;
    $90 = HEAP32[$pp>>2]|0;
    $91 = (_find($89,$90)|0);
    $rset = $91;
    $92 = $rset;
    $93 = $root;
    $94 = (($93) + ($92<<2)|0);
    $95 = HEAP32[$94>>2]|0;
    $rroot = $95;
    $96 = $rroot;
    $97 = $col;
    $98 = ($96|0)!=($97|0);
    if ($98) {
     $99 = $col;
     $100 = $rroot;
     $101 = $5;
     $102 = (($101) + ($100<<2)|0);
     HEAP32[$102>>2] = $99;
     $103 = $cset;
     $104 = $rset;
     $105 = HEAP32[$pp>>2]|0;
     $106 = (_link($103,$104,$105)|0);
     $cset = $106;
     $107 = $col;
     $108 = $cset;
     $109 = $root;
     $110 = (($109) + ($108<<2)|0);
     HEAP32[$110>>2] = $107;
    }
   }
   $111 = $p;
   $112 = (($111) + 1)|0;
   $p = $112;
  }
  $113 = $col;
  $114 = (($113) + 1)|0;
  $col = $114;
 }
 $115 = $root;
 _superlu_free($115);
 $116 = $firstcol;
 _superlu_free($116);
 $117 = HEAP32[$pp>>2]|0;
 _finalize_disjoint_sets($117);
 STACKTOP = sp;return 0;
}
function _TreePostorder($n,$parent) {
 $n = $n|0;
 $parent = $parent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dad = 0, $first_kid = 0, $next_kid = 0, $post = 0, $postnum = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $parent;
 $2 = $0;
 $3 = (($2) + 1)|0;
 $4 = (_mxCallocInt($3)|0);
 $first_kid = $4;
 $5 = $0;
 $6 = (($5) + 1)|0;
 $7 = (_mxCallocInt($6)|0);
 $next_kid = $7;
 $8 = $0;
 $9 = (($8) + 1)|0;
 $10 = (_mxCallocInt($9)|0);
 $post = $10;
 $v = 0;
 while(1) {
  $11 = $v;
  $12 = $0;
  $13 = ($11|0)<=($12|0);
  if (!($13)) {
   break;
  }
  $14 = $v;
  $15 = (($14) + 1)|0;
  $v = $15;
  $16 = $first_kid;
  $17 = (($16) + ($14<<2)|0);
  HEAP32[$17>>2] = -1;
 }
 $18 = $0;
 $19 = (($18) - 1)|0;
 $v = $19;
 while(1) {
  $20 = $v;
  $21 = ($20|0)>=(0);
  if (!($21)) {
   break;
  }
  $22 = $v;
  $23 = $1;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $dad = $25;
  $26 = $dad;
  $27 = $first_kid;
  $28 = (($27) + ($26<<2)|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $v;
  $31 = $next_kid;
  $32 = (($31) + ($30<<2)|0);
  HEAP32[$32>>2] = $29;
  $33 = $v;
  $34 = $dad;
  $35 = $first_kid;
  $36 = (($35) + ($34<<2)|0);
  HEAP32[$36>>2] = $33;
  $37 = $v;
  $38 = (($37) + -1)|0;
  $v = $38;
 }
 $postnum = 0;
 $39 = $0;
 $40 = $1;
 $41 = $first_kid;
 $42 = $next_kid;
 $43 = $post;
 $44 = $postnum;
 _nr_etdfs($39,$40,$41,$42,$43,$44);
 $45 = $first_kid;
 _superlu_free($45);
 $46 = $next_kid;
 _superlu_free($46);
 $47 = $post;
 STACKTOP = sp;return ($47|0);
}
function _mxCallocInt($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $i = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0;
 var $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 24|0;
 $0 = $n;
 $1 = $0;
 $2 = $1<<2;
 $3 = (_superlu_malloc($2)|0);
 $buf = $3;
 $4 = $buf;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  HEAP32[$vararg_buffer>>2] = 1272;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 57;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 1320;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $i = 0;
 while(1) {
  $6 = $i;
  $7 = $0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $i;
  $10 = $buf;
  $11 = (($10) + ($9<<2)|0);
  HEAP32[$11>>2] = 0;
  $12 = $i;
  $13 = (($12) + 1)|0;
  $i = $13;
 }
 $14 = $buf;
 STACKTOP = sp;return ($14|0);
}
function _initialize_disjoint_sets($n,$pp) {
 $n = $n|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $pp;
 $2 = $0;
 $3 = (_mxCallocInt($2)|0);
 $4 = $1;
 HEAP32[$4>>2] = $3;
 STACKTOP = sp;return;
}
function _make_set($i,$pp) {
 $i = $i|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $i;
 $1 = $pp;
 $2 = $0;
 $3 = $0;
 $4 = $1;
 $5 = (($4) + ($3<<2)|0);
 HEAP32[$5>>2] = $2;
 $6 = $0;
 STACKTOP = sp;return ($6|0);
}
function _find($i,$pp) {
 $i = $i|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $gp = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $i;
 $1 = $pp;
 $2 = $0;
 $3 = $1;
 $4 = (($3) + ($2<<2)|0);
 $5 = HEAP32[$4>>2]|0;
 $p = $5;
 $6 = $p;
 $7 = $1;
 $8 = (($7) + ($6<<2)|0);
 $9 = HEAP32[$8>>2]|0;
 $gp = $9;
 while(1) {
  $10 = $gp;
  $11 = $p;
  $12 = ($10|0)!=($11|0);
  if (!($12)) {
   break;
  }
  $13 = $gp;
  $14 = $0;
  $15 = $1;
  $16 = (($15) + ($14<<2)|0);
  HEAP32[$16>>2] = $13;
  $17 = $gp;
  $0 = $17;
  $18 = $0;
  $19 = $1;
  $20 = (($19) + ($18<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $p = $21;
  $22 = $p;
  $23 = $1;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $gp = $25;
 }
 $26 = $p;
 STACKTOP = sp;return ($26|0);
}
function _link($s,$t,$pp) {
 $s = $s|0;
 $t = $t|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $s;
 $1 = $t;
 $2 = $pp;
 $3 = $1;
 $4 = $0;
 $5 = $2;
 $6 = (($5) + ($4<<2)|0);
 HEAP32[$6>>2] = $3;
 $7 = $1;
 STACKTOP = sp;return ($7|0);
}
function _finalize_disjoint_sets($pp) {
 $pp = $pp|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $pp;
 $1 = $0;
 _superlu_free($1);
 STACKTOP = sp;return;
}
function _nr_etdfs($n,$parent,$first_kid,$next_kid,$post,$postnum) {
 $n = $n|0;
 $parent = $parent|0;
 $first_kid = $first_kid|0;
 $next_kid = $next_kid|0;
 $post = $post|0;
 $postnum = $postnum|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $current = 0, $first = 0, $next = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $parent;
 $2 = $first_kid;
 $3 = $next_kid;
 $4 = $post;
 $5 = $postnum;
 $6 = $0;
 $current = $6;
 while(1) {
  $7 = $5;
  $8 = $0;
  $9 = ($7|0)!=($8|0);
  if (!($9)) {
   label = 10;
   break;
  }
  $10 = $current;
  $11 = $2;
  $12 = (($11) + ($10<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $first = $13;
  $14 = $first;
  $15 = ($14|0)==(-1);
  if (!($15)) {
   $45 = $first;
   $current = $45;
   continue;
  }
  $16 = $5;
  $17 = (($16) + 1)|0;
  $5 = $17;
  $18 = $current;
  $19 = $4;
  $20 = (($19) + ($18<<2)|0);
  HEAP32[$20>>2] = $16;
  $21 = $current;
  $22 = $3;
  $23 = (($22) + ($21<<2)|0);
  $24 = HEAP32[$23>>2]|0;
  $next = $24;
  while(1) {
   $25 = $next;
   $26 = ($25|0)==(-1);
   if (!($26)) {
    break;
   }
   $27 = $current;
   $28 = $1;
   $29 = (($28) + ($27<<2)|0);
   $30 = HEAP32[$29>>2]|0;
   $current = $30;
   $31 = $5;
   $32 = (($31) + 1)|0;
   $5 = $32;
   $33 = $current;
   $34 = $4;
   $35 = (($34) + ($33<<2)|0);
   HEAP32[$35>>2] = $31;
   $36 = $current;
   $37 = $3;
   $38 = (($37) + ($36<<2)|0);
   $39 = HEAP32[$38>>2]|0;
   $next = $39;
  }
  $40 = $5;
  $41 = $0;
  $42 = (($41) + 1)|0;
  $43 = ($40|0)==($42|0);
  if ($43) {
   label = 10;
   break;
  }
  $44 = $next;
  $current = $44;
 }
 if ((label|0) == 10) {
  STACKTOP = sp;return;
 }
}
function _sp_preorder($options,$A,$perm_c,$etree,$AC) {
 $options = $options|0;
 $A = $A|0;
 $perm_c = $perm_c|0;
 $etree = $etree|0;
 $AC = $AC|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ACstore = 0, $Astore = 0, $i = 0, $iwork = 0, $msg = 0;
 var $msg1 = 0, $msg2 = 0, $msg3 = 0, $n = 0, $post = 0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer3 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1136|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $msg = sp + 872|0;
 $msg1 = sp + 616|0;
 $msg2 = sp + 360|0;
 $msg3 = sp + 104|0;
 $0 = $options;
 $1 = $A;
 $2 = $perm_c;
 $3 = $etree;
 $4 = $AC;
 $5 = $1;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $n = $7;
 $8 = $4;
 HEAP32[$8>>2] = 1;
 $9 = $1;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $4;
 $13 = ((($12)) + 4|0);
 HEAP32[$13>>2] = $11;
 $14 = $1;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $4;
 $18 = ((($17)) + 8|0);
 HEAP32[$18>>2] = $16;
 $19 = $1;
 $20 = ((($19)) + 12|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = $4;
 $23 = ((($22)) + 12|0);
 HEAP32[$23>>2] = $21;
 $24 = $1;
 $25 = ((($24)) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $4;
 $28 = ((($27)) + 16|0);
 HEAP32[$28>>2] = $26;
 $29 = $1;
 $30 = ((($29)) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 $Astore = $31;
 $32 = (_superlu_malloc(20)|0);
 $33 = $4;
 $34 = ((($33)) + 20|0);
 HEAP32[$34>>2] = $32;
 $ACstore = $32;
 $35 = $ACstore;
 $36 = ($35|0)!=(0|0);
 if (!($36)) {
  HEAP32[$vararg_buffer>>2] = 1336;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 81;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 1376;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $37 = $Astore;
 $38 = HEAP32[$37>>2]|0;
 $39 = $ACstore;
 HEAP32[$39>>2] = $38;
 $40 = $Astore;
 $41 = ((($40)) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $ACstore;
 $44 = ((($43)) + 4|0);
 HEAP32[$44>>2] = $42;
 $45 = $Astore;
 $46 = ((($45)) + 8|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = $ACstore;
 $49 = ((($48)) + 8|0);
 HEAP32[$49>>2] = $47;
 $50 = $n;
 $51 = $50<<2;
 $52 = (_superlu_malloc($51)|0);
 $53 = $ACstore;
 $54 = ((($53)) + 12|0);
 HEAP32[$54>>2] = $52;
 $55 = $ACstore;
 $56 = ((($55)) + 12|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = ($57|0)!=(0|0);
 if (!($58)) {
  HEAP32[$vararg_buffer3>>2] = 1392;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = 86;
  $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
  HEAP32[$vararg_ptr7>>2] = 1376;
  (_sprintf($msg1,2864,$vararg_buffer3)|0);
  _superlu_abort_and_exit($msg1);
 }
 $59 = $n;
 $60 = $59<<2;
 $61 = (_superlu_malloc($60)|0);
 $62 = $ACstore;
 $63 = ((($62)) + 16|0);
 HEAP32[$63>>2] = $61;
 $64 = $ACstore;
 $65 = ((($64)) + 16|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($66|0)!=(0|0);
 if (!($67)) {
  HEAP32[$vararg_buffer8>>2] = 1440;
  $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
  HEAP32[$vararg_ptr11>>2] = 88;
  $vararg_ptr12 = ((($vararg_buffer8)) + 8|0);
  HEAP32[$vararg_ptr12>>2] = 1376;
  (_sprintf($msg2,2864,$vararg_buffer8)|0);
  _superlu_abort_and_exit($msg2);
 }
 $i = 0;
 while(1) {
  $68 = $i;
  $69 = $n;
  $70 = ($68|0)<($69|0);
  if (!($70)) {
   break;
  }
  $71 = $i;
  $72 = $Astore;
  $73 = ((($72)) + 12|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = (($74) + ($71<<2)|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = $i;
  $78 = $2;
  $79 = (($78) + ($77<<2)|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = $ACstore;
  $82 = ((($81)) + 12|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = (($83) + ($80<<2)|0);
  HEAP32[$84>>2] = $76;
  $85 = $i;
  $86 = (($85) + 1)|0;
  $87 = $Astore;
  $88 = ((($87)) + 12|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = (($89) + ($86<<2)|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = $i;
  $93 = $2;
  $94 = (($93) + ($92<<2)|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = $ACstore;
  $97 = ((($96)) + 16|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = (($98) + ($95<<2)|0);
  HEAP32[$99>>2] = $91;
  $100 = $i;
  $101 = (($100) + 1)|0;
  $i = $101;
 }
 $102 = $0;
 $103 = HEAP32[$102>>2]|0;
 $104 = ($103|0)==(0);
 if (!($104)) {
  STACKTOP = sp;return;
 }
 $105 = $ACstore;
 $106 = ((($105)) + 12|0);
 $107 = HEAP32[$106>>2]|0;
 $108 = $ACstore;
 $109 = ((($108)) + 16|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = $ACstore;
 $112 = ((($111)) + 8|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = $1;
 $115 = ((($114)) + 12|0);
 $116 = HEAP32[$115>>2]|0;
 $117 = $1;
 $118 = ((($117)) + 16|0);
 $119 = HEAP32[$118>>2]|0;
 $120 = $3;
 (_sp_coletree($107,$110,$113,$116,$119,$120)|0);
 $121 = $0;
 $122 = ((($121)) + 32|0);
 $123 = HEAP32[$122>>2]|0;
 $124 = ($123|0)==(0);
 if (!($124)) {
  STACKTOP = sp;return;
 }
 $125 = $n;
 $126 = $3;
 $127 = (_TreePostorder($125,$126)|0);
 $post = $127;
 $128 = $n;
 $129 = (($128) + 1)|0;
 $130 = $129<<2;
 $131 = (_superlu_malloc($130)|0);
 $iwork = $131;
 $132 = $iwork;
 $133 = ($132|0)!=(0|0);
 if (!($133)) {
  HEAP32[$vararg_buffer13>>2] = 1496;
  $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
  HEAP32[$vararg_ptr16>>2] = 159;
  $vararg_ptr17 = ((($vararg_buffer13)) + 8|0);
  HEAP32[$vararg_ptr17>>2] = 1376;
  (_sprintf($msg3,2864,$vararg_buffer13)|0);
  _superlu_abort_and_exit($msg3);
 }
 $i = 0;
 while(1) {
  $134 = $i;
  $135 = $n;
  $136 = ($134|0)<($135|0);
  if (!($136)) {
   break;
  }
  $137 = $i;
  $138 = $3;
  $139 = (($138) + ($137<<2)|0);
  $140 = HEAP32[$139>>2]|0;
  $141 = $post;
  $142 = (($141) + ($140<<2)|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = $i;
  $145 = $post;
  $146 = (($145) + ($144<<2)|0);
  $147 = HEAP32[$146>>2]|0;
  $148 = $iwork;
  $149 = (($148) + ($147<<2)|0);
  HEAP32[$149>>2] = $143;
  $150 = $i;
  $151 = (($150) + 1)|0;
  $i = $151;
 }
 $i = 0;
 while(1) {
  $152 = $i;
  $153 = $n;
  $154 = ($152|0)<($153|0);
  if (!($154)) {
   break;
  }
  $155 = $i;
  $156 = $iwork;
  $157 = (($156) + ($155<<2)|0);
  $158 = HEAP32[$157>>2]|0;
  $159 = $i;
  $160 = $3;
  $161 = (($160) + ($159<<2)|0);
  HEAP32[$161>>2] = $158;
  $162 = $i;
  $163 = (($162) + 1)|0;
  $i = $163;
 }
 $i = 0;
 while(1) {
  $164 = $i;
  $165 = $n;
  $166 = ($164|0)<($165|0);
  if (!($166)) {
   break;
  }
  $167 = $i;
  $168 = $ACstore;
  $169 = ((($168)) + 12|0);
  $170 = HEAP32[$169>>2]|0;
  $171 = (($170) + ($167<<2)|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = $i;
  $174 = $post;
  $175 = (($174) + ($173<<2)|0);
  $176 = HEAP32[$175>>2]|0;
  $177 = $iwork;
  $178 = (($177) + ($176<<2)|0);
  HEAP32[$178>>2] = $172;
  $179 = $i;
  $180 = (($179) + 1)|0;
  $i = $180;
 }
 $i = 0;
 while(1) {
  $181 = $i;
  $182 = $n;
  $183 = ($181|0)<($182|0);
  if (!($183)) {
   break;
  }
  $184 = $i;
  $185 = $iwork;
  $186 = (($185) + ($184<<2)|0);
  $187 = HEAP32[$186>>2]|0;
  $188 = $i;
  $189 = $ACstore;
  $190 = ((($189)) + 12|0);
  $191 = HEAP32[$190>>2]|0;
  $192 = (($191) + ($188<<2)|0);
  HEAP32[$192>>2] = $187;
  $193 = $i;
  $194 = (($193) + 1)|0;
  $i = $194;
 }
 $i = 0;
 while(1) {
  $195 = $i;
  $196 = $n;
  $197 = ($195|0)<($196|0);
  if (!($197)) {
   break;
  }
  $198 = $i;
  $199 = $ACstore;
  $200 = ((($199)) + 16|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = (($201) + ($198<<2)|0);
  $203 = HEAP32[$202>>2]|0;
  $204 = $i;
  $205 = $post;
  $206 = (($205) + ($204<<2)|0);
  $207 = HEAP32[$206>>2]|0;
  $208 = $iwork;
  $209 = (($208) + ($207<<2)|0);
  HEAP32[$209>>2] = $203;
  $210 = $i;
  $211 = (($210) + 1)|0;
  $i = $211;
 }
 $i = 0;
 while(1) {
  $212 = $i;
  $213 = $n;
  $214 = ($212|0)<($213|0);
  if (!($214)) {
   break;
  }
  $215 = $i;
  $216 = $iwork;
  $217 = (($216) + ($215<<2)|0);
  $218 = HEAP32[$217>>2]|0;
  $219 = $i;
  $220 = $ACstore;
  $221 = ((($220)) + 16|0);
  $222 = HEAP32[$221>>2]|0;
  $223 = (($222) + ($219<<2)|0);
  HEAP32[$223>>2] = $218;
  $224 = $i;
  $225 = (($224) + 1)|0;
  $i = $225;
 }
 $i = 0;
 while(1) {
  $226 = $i;
  $227 = $n;
  $228 = ($226|0)<($227|0);
  if (!($228)) {
   break;
  }
  $229 = $i;
  $230 = $2;
  $231 = (($230) + ($229<<2)|0);
  $232 = HEAP32[$231>>2]|0;
  $233 = $post;
  $234 = (($233) + ($232<<2)|0);
  $235 = HEAP32[$234>>2]|0;
  $236 = $i;
  $237 = $iwork;
  $238 = (($237) + ($236<<2)|0);
  HEAP32[$238>>2] = $235;
  $239 = $i;
  $240 = (($239) + 1)|0;
  $i = $240;
 }
 $i = 0;
 while(1) {
  $241 = $i;
  $242 = $n;
  $243 = ($241|0)<($242|0);
  if (!($243)) {
   break;
  }
  $244 = $i;
  $245 = $iwork;
  $246 = (($245) + ($244<<2)|0);
  $247 = HEAP32[$246>>2]|0;
  $248 = $i;
  $249 = $2;
  $250 = (($249) + ($248<<2)|0);
  HEAP32[$250>>2] = $247;
  $251 = $i;
  $252 = (($251) + 1)|0;
  $i = $252;
 }
 $253 = $post;
 _superlu_free($253);
 $254 = $iwork;
 _superlu_free($254);
 STACKTOP = sp;return;
}
function _sp_ienv($ispec) {
 $ispec = $ispec|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i = sp;
 $1 = $ispec;
 $2 = $1;
 switch ($2|0) {
 case 6:  {
  $0 = 20;
  break;
 }
 case 7:  {
  $0 = 10;
  break;
 }
 case 2:  {
  $0 = 6;
  break;
 }
 case 3:  {
  $0 = 100;
  break;
 }
 case 1:  {
  $0 = 12;
  break;
 }
 case 4:  {
  $0 = 200;
  break;
 }
 case 5:  {
  $0 = 60;
  break;
 }
 default: {
  HEAP32[$i>>2] = 1;
  (_xerbla_(1488,$i)|0);
  $0 = 0;
 }
 }
 $3 = $0;
 STACKTOP = sp;return ($3|0);
}
function _relax_snode($n,$et,$relax_columns,$descendants,$relax_end) {
 $n = $n|0;
 $et = $et|0;
 $relax_columns = $relax_columns|0;
 $descendants = $descendants|0;
 $relax_end = $relax_end|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $j = 0, $parent = 0, $snode_start = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $et;
 $2 = $relax_columns;
 $3 = $descendants;
 $4 = $relax_end;
 $5 = $4;
 $6 = $0;
 _ifill($5,$6,-1);
 $j = 0;
 while(1) {
  $7 = $j;
  $8 = $0;
  $9 = ($7|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = $j;
  $11 = $3;
  $12 = (($11) + ($10<<2)|0);
  HEAP32[$12>>2] = 0;
  $13 = $j;
  $14 = (($13) + 1)|0;
  $j = $14;
 }
 $j = 0;
 while(1) {
  $15 = $j;
  $16 = $0;
  $17 = ($15|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = $j;
  $19 = $1;
  $20 = (($19) + ($18<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $parent = $21;
  $22 = $parent;
  $23 = $0;
  $24 = ($22|0)!=($23|0);
  if ($24) {
   $25 = $j;
   $26 = $3;
   $27 = (($26) + ($25<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 1)|0;
   $30 = $parent;
   $31 = $3;
   $32 = (($31) + ($30<<2)|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($33) + ($29))|0;
   HEAP32[$32>>2] = $34;
  }
  $35 = $j;
  $36 = (($35) + 1)|0;
  $j = $36;
 }
 $j = 0;
 L12: while(1) {
  $37 = $j;
  $38 = $0;
  $39 = ($37|0)<($38|0);
  if (!($39)) {
   break;
  }
  $40 = $j;
  $41 = $1;
  $42 = (($41) + ($40<<2)|0);
  $43 = HEAP32[$42>>2]|0;
  $parent = $43;
  $44 = $j;
  $snode_start = $44;
  while(1) {
   $45 = $parent;
   $46 = $0;
   $47 = ($45|0)!=($46|0);
   if (!($47)) {
    break;
   }
   $48 = $parent;
   $49 = $3;
   $50 = (($49) + ($48<<2)|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = $2;
   $53 = ($51|0)<($52|0);
   if (!($53)) {
    break;
   }
   $54 = $parent;
   $j = $54;
   $55 = $j;
   $56 = $1;
   $57 = (($56) + ($55<<2)|0);
   $58 = HEAP32[$57>>2]|0;
   $parent = $58;
  }
  $59 = $j;
  $60 = $snode_start;
  $61 = $4;
  $62 = (($61) + ($60<<2)|0);
  HEAP32[$62>>2] = $59;
  $63 = $j;
  $64 = (($63) + 1)|0;
  $j = $64;
  while(1) {
   $65 = $j;
   $66 = $3;
   $67 = (($66) + ($65<<2)|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = ($68|0)!=(0);
   if (!($69)) {
    continue L12;
   }
   $70 = $j;
   $71 = $0;
   $72 = ($70|0)<($71|0);
   if (!($72)) {
    continue L12;
   }
   $73 = $j;
   $74 = (($73) + 1)|0;
   $j = $74;
  }
 }
 STACKTOP = sp;return;
}
function _heap_relax_snode($n,$et,$relax_columns,$descendants,$relax_end) {
 $n = $n|0;
 $et = $et|0;
 $relax_columns = $relax_columns|0;
 $descendants = $descendants|0;
 $relax_end = $relax_end|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $et_save = 0, $i = 0, $inv_post = 0, $iwork = 0, $j = 0, $k = 0, $l = 0, $msg = 0, $nsuper_et = 0;
 var $nsuper_et_post = 0, $parent = 0, $post = 0, $snode_start = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 80|0;
 $0 = $n;
 $1 = $et;
 $2 = $relax_columns;
 $3 = $descendants;
 $4 = $relax_end;
 $nsuper_et = 0;
 $nsuper_et_post = 0;
 $5 = $0;
 $6 = ($5*3)|0;
 $7 = (($6) + 2)|0;
 $8 = (_intMalloc($7)|0);
 $iwork = $8;
 $9 = $iwork;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  HEAP32[$vararg_buffer>>2] = 1496;
  $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr2>>2] = 54;
  $vararg_ptr3 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr3>>2] = 1536;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $11 = $iwork;
 $12 = $0;
 $13 = (($11) + ($12<<2)|0);
 $14 = ((($13)) + 4|0);
 $inv_post = $14;
 $15 = $inv_post;
 $16 = $0;
 $17 = (($15) + ($16<<2)|0);
 $18 = ((($17)) + 4|0);
 $et_save = $18;
 $19 = $0;
 $20 = $1;
 $21 = (_TreePostorder($19,$20)|0);
 $post = $21;
 $i = 0;
 while(1) {
  $22 = $i;
  $23 = $0;
  $24 = (($23) + 1)|0;
  $25 = ($22|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = $i;
  $27 = $i;
  $28 = $post;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $inv_post;
  $32 = (($31) + ($30<<2)|0);
  HEAP32[$32>>2] = $26;
  $33 = $i;
  $34 = (($33) + 1)|0;
  $i = $34;
 }
 $i = 0;
 while(1) {
  $35 = $i;
  $36 = $0;
  $37 = ($35|0)<($36|0);
  if (!($37)) {
   break;
  }
  $38 = $i;
  $39 = $1;
  $40 = (($39) + ($38<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = $post;
  $43 = (($42) + ($41<<2)|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = $i;
  $46 = $post;
  $47 = (($46) + ($45<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = $iwork;
  $50 = (($49) + ($48<<2)|0);
  HEAP32[$50>>2] = $44;
  $51 = $i;
  $52 = $1;
  $53 = (($52) + ($51<<2)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = $i;
  $56 = $et_save;
  $57 = (($56) + ($55<<2)|0);
  HEAP32[$57>>2] = $54;
  $58 = $i;
  $59 = (($58) + 1)|0;
  $i = $59;
 }
 $i = 0;
 while(1) {
  $60 = $i;
  $61 = $0;
  $62 = ($60|0)<($61|0);
  if (!($62)) {
   break;
  }
  $63 = $i;
  $64 = $iwork;
  $65 = (($64) + ($63<<2)|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = $i;
  $68 = $1;
  $69 = (($68) + ($67<<2)|0);
  HEAP32[$69>>2] = $66;
  $70 = $i;
  $71 = (($70) + 1)|0;
  $i = $71;
 }
 $72 = $4;
 $73 = $0;
 _ifill($72,$73,-1);
 $j = 0;
 while(1) {
  $74 = $j;
  $75 = $0;
  $76 = ($74|0)<($75|0);
  if (!($76)) {
   break;
  }
  $77 = $j;
  $78 = $3;
  $79 = (($78) + ($77<<2)|0);
  HEAP32[$79>>2] = 0;
  $80 = $j;
  $81 = (($80) + 1)|0;
  $j = $81;
 }
 $j = 0;
 while(1) {
  $82 = $j;
  $83 = $0;
  $84 = ($82|0)<($83|0);
  if (!($84)) {
   break;
  }
  $85 = $j;
  $86 = $1;
  $87 = (($86) + ($85<<2)|0);
  $88 = HEAP32[$87>>2]|0;
  $parent = $88;
  $89 = $parent;
  $90 = $0;
  $91 = ($89|0)!=($90|0);
  if ($91) {
   $92 = $j;
   $93 = $3;
   $94 = (($93) + ($92<<2)|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = (($95) + 1)|0;
   $97 = $parent;
   $98 = $3;
   $99 = (($98) + ($97<<2)|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = (($100) + ($96))|0;
   HEAP32[$99>>2] = $101;
  }
  $102 = $j;
  $103 = (($102) + 1)|0;
  $j = $103;
 }
 $j = 0;
 L27: while(1) {
  $104 = $j;
  $105 = $0;
  $106 = ($104|0)<($105|0);
  if (!($106)) {
   break;
  }
  $107 = $j;
  $108 = $1;
  $109 = (($108) + ($107<<2)|0);
  $110 = HEAP32[$109>>2]|0;
  $parent = $110;
  $111 = $j;
  $snode_start = $111;
  while(1) {
   $112 = $parent;
   $113 = $0;
   $114 = ($112|0)!=($113|0);
   if (!($114)) {
    break;
   }
   $115 = $parent;
   $116 = $3;
   $117 = (($116) + ($115<<2)|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = $2;
   $120 = ($118|0)<($119|0);
   if (!($120)) {
    break;
   }
   $121 = $parent;
   $j = $121;
   $122 = $j;
   $123 = $1;
   $124 = (($123) + ($122<<2)|0);
   $125 = HEAP32[$124>>2]|0;
   $parent = $125;
  }
  $126 = $nsuper_et_post;
  $127 = (($126) + 1)|0;
  $nsuper_et_post = $127;
  $128 = $0;
  $k = $128;
  $129 = $snode_start;
  $i = $129;
  while(1) {
   $130 = $i;
   $131 = $j;
   $132 = ($130|0)<=($131|0);
   if (!($132)) {
    break;
   }
   $133 = $k;
   $134 = $i;
   $135 = $inv_post;
   $136 = (($135) + ($134<<2)|0);
   $137 = HEAP32[$136>>2]|0;
   $138 = ($133|0)<($137|0);
   if ($138) {
    $139 = $k;
    $144 = $139;
   } else {
    $140 = $i;
    $141 = $inv_post;
    $142 = (($141) + ($140<<2)|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = $143;
   }
   $k = $144;
   $145 = $i;
   $146 = (($145) + 1)|0;
   $i = $146;
  }
  $147 = $j;
  $148 = $inv_post;
  $149 = (($148) + ($147<<2)|0);
  $150 = HEAP32[$149>>2]|0;
  $l = $150;
  $151 = $l;
  $152 = $k;
  $153 = (($151) - ($152))|0;
  $154 = $j;
  $155 = $snode_start;
  $156 = (($154) - ($155))|0;
  $157 = ($153|0)==($156|0);
  L43: do {
   if ($157) {
    $158 = $l;
    $159 = $k;
    $160 = $4;
    $161 = (($160) + ($159<<2)|0);
    HEAP32[$161>>2] = $158;
    $162 = $nsuper_et;
    $163 = (($162) + 1)|0;
    $nsuper_et = $163;
   } else {
    $164 = $snode_start;
    $i = $164;
    while(1) {
     $165 = $i;
     $166 = $j;
     $167 = ($165|0)<=($166|0);
     if (!($167)) {
      break L43;
     }
     $168 = $i;
     $169 = $inv_post;
     $170 = (($169) + ($168<<2)|0);
     $171 = HEAP32[$170>>2]|0;
     $l = $171;
     $172 = $i;
     $173 = $3;
     $174 = (($173) + ($172<<2)|0);
     $175 = HEAP32[$174>>2]|0;
     $176 = ($175|0)==(0);
     if ($176) {
      $177 = $l;
      $178 = $l;
      $179 = $4;
      $180 = (($179) + ($178<<2)|0);
      HEAP32[$180>>2] = $177;
      $181 = $nsuper_et;
      $182 = (($181) + 1)|0;
      $nsuper_et = $182;
     }
     $183 = $i;
     $184 = (($183) + 1)|0;
     $i = $184;
    }
   }
  } while(0);
  $185 = $j;
  $186 = (($185) + 1)|0;
  $j = $186;
  while(1) {
   $187 = $j;
   $188 = $3;
   $189 = (($188) + ($187<<2)|0);
   $190 = HEAP32[$189>>2]|0;
   $191 = ($190|0)!=(0);
   if (!($191)) {
    continue L27;
   }
   $192 = $j;
   $193 = $0;
   $194 = ($192|0)<($193|0);
   if (!($194)) {
    continue L27;
   }
   $195 = $j;
   $196 = (($195) + 1)|0;
   $j = $196;
  }
 }
 $i = 0;
 while(1) {
  $197 = $i;
  $198 = $0;
  $199 = ($197|0)<($198|0);
  if (!($199)) {
   break;
  }
  $200 = $i;
  $201 = $et_save;
  $202 = (($201) + ($200<<2)|0);
  $203 = HEAP32[$202>>2]|0;
  $204 = $i;
  $205 = $1;
  $206 = (($205) + ($204<<2)|0);
  HEAP32[$206>>2] = $203;
  $207 = $i;
  $208 = (($207) + 1)|0;
  $i = $208;
 }
 $209 = $post;
 _superlu_free($209);
 $210 = $iwork;
 _superlu_free($210);
 STACKTOP = sp;return;
}
function _colamd_recommended($nnz,$n_row,$n_col) {
 $nnz = $nnz|0;
 $n_row = $n_row|0;
 $n_col = $n_col|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $nnz;
 $1 = $n_row;
 $2 = $n_col;
 $3 = $0;
 $4 = ($3|0)<(0);
 $5 = $1;
 $6 = ($5|0)<(0);
 $or$cond = $4 | $6;
 $7 = $2;
 $8 = ($7|0)<(0);
 $or$cond3 = $or$cond | $8;
 if ($or$cond3) {
  $26 = -1;
  STACKTOP = sp;return ($26|0);
 }
 $9 = $0;
 $10 = $9<<1;
 $11 = $2;
 $12 = (($11) + 1)|0;
 $13 = ($12*24)|0;
 $14 = (($13>>>0) / 4)&-1;
 $15 = (($10) + ($14))|0;
 $16 = $1;
 $17 = (($16) + 1)|0;
 $18 = $17<<4;
 $19 = (($18>>>0) / 4)&-1;
 $20 = (($15) + ($19))|0;
 $21 = $2;
 $22 = (($20) + ($21))|0;
 $23 = $0;
 $24 = (($23|0) / 5)&-1;
 $25 = (($22) + ($24))|0;
 $26 = $25;
 STACKTOP = sp;return ($26|0);
}
function _colamd_set_defaults($knobs) {
 $knobs = $knobs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $knobs;
 $1 = $0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $3 = $i;
  $4 = ($3|0)<(20);
  if (!($4)) {
   break;
  }
  $5 = $i;
  $6 = $0;
  $7 = (($6) + ($5<<3)|0);
  HEAPF64[$7>>3] = 0.0;
  $8 = $i;
  $9 = (($8) + 1)|0;
  $i = $9;
 }
 $10 = $0;
 HEAPF64[$10>>3] = 0.5;
 $11 = $0;
 $12 = ((($11)) + 8|0);
 HEAPF64[$12>>3] = 0.5;
 STACKTOP = sp;return;
}
function _colamd($n_row,$n_col,$Alen,$A,$p,$knobs,$stats) {
 $n_row = $n_row|0;
 $n_col = $n_col|0;
 $Alen = $Alen|0;
 $A = $A|0;
 $p = $p|0;
 $knobs = $knobs|0;
 $stats = $stats|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Col = 0, $Col_size = 0, $Row = 0, $Row_size = 0, $default_knobs = 0, $i = 0, $max_deg = 0, $n_col2 = 0, $n_row2 = 0;
 var $need = 0, $ngarbage = 0, $nnz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $n_col2 = sp + 172|0;
 $n_row2 = sp + 168|0;
 $max_deg = sp + 160|0;
 $default_knobs = sp;
 $1 = $n_row;
 $2 = $n_col;
 $3 = $Alen;
 $4 = $A;
 $5 = $p;
 $6 = $knobs;
 $7 = $stats;
 $8 = $7;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  $0 = 0;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $i = 0;
 while(1) {
  $10 = $i;
  $11 = ($10|0)<(20);
  if (!($11)) {
   break;
  }
  $12 = $i;
  $13 = $7;
  $14 = (($13) + ($12<<2)|0);
  HEAP32[$14>>2] = 0;
  $15 = $i;
  $16 = (($15) + 1)|0;
  $i = $16;
 }
 $17 = $7;
 $18 = ((($17)) + 12|0);
 HEAP32[$18>>2] = 0;
 $19 = $7;
 $20 = ((($19)) + 16|0);
 HEAP32[$20>>2] = -1;
 $21 = $7;
 $22 = ((($21)) + 20|0);
 HEAP32[$22>>2] = -1;
 $23 = $4;
 $24 = ($23|0)!=(0|0);
 if (!($24)) {
  $25 = $7;
  $26 = ((($25)) + 12|0);
  HEAP32[$26>>2] = -1;
  $0 = 0;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $27 = $5;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  $29 = $7;
  $30 = ((($29)) + 12|0);
  HEAP32[$30>>2] = -2;
  $0 = 0;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $31 = $1;
 $32 = ($31|0)<(0);
 if ($32) {
  $33 = $7;
  $34 = ((($33)) + 12|0);
  HEAP32[$34>>2] = -3;
  $35 = $1;
  $36 = $7;
  $37 = ((($36)) + 16|0);
  HEAP32[$37>>2] = $35;
  $0 = 0;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $38 = $2;
 $39 = ($38|0)<(0);
 if ($39) {
  $40 = $7;
  $41 = ((($40)) + 12|0);
  HEAP32[$41>>2] = -4;
  $42 = $2;
  $43 = $7;
  $44 = ((($43)) + 16|0);
  HEAP32[$44>>2] = $42;
  $0 = 0;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $45 = $2;
 $46 = $5;
 $47 = (($46) + ($45<<2)|0);
 $48 = HEAP32[$47>>2]|0;
 $nnz = $48;
 $49 = $nnz;
 $50 = ($49|0)<(0);
 if ($50) {
  $51 = $7;
  $52 = ((($51)) + 12|0);
  HEAP32[$52>>2] = -5;
  $53 = $nnz;
  $54 = $7;
  $55 = ((($54)) + 16|0);
  HEAP32[$55>>2] = $53;
  $0 = 0;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $56 = $5;
 $57 = HEAP32[$56>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  $59 = $7;
  $60 = ((($59)) + 12|0);
  HEAP32[$60>>2] = -6;
  $61 = $5;
  $62 = HEAP32[$61>>2]|0;
  $63 = $7;
  $64 = ((($63)) + 16|0);
  HEAP32[$64>>2] = $62;
  $0 = 0;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $65 = $6;
 $66 = ($65|0)!=(0|0);
 if (!($66)) {
  _colamd_set_defaults($default_knobs);
  $6 = $default_knobs;
 }
 $67 = $2;
 $68 = (($67) + 1)|0;
 $69 = ($68*24)|0;
 $70 = (($69>>>0) / 4)&-1;
 $Col_size = $70;
 $71 = $1;
 $72 = (($71) + 1)|0;
 $73 = $72<<4;
 $74 = (($73>>>0) / 4)&-1;
 $Row_size = $74;
 $75 = $nnz;
 $76 = $75<<1;
 $77 = $2;
 $78 = (($76) + ($77))|0;
 $79 = $Col_size;
 $80 = (($78) + ($79))|0;
 $81 = $Row_size;
 $82 = (($80) + ($81))|0;
 $need = $82;
 $83 = $need;
 $84 = $3;
 $85 = ($83|0)>($84|0);
 if ($85) {
  $86 = $7;
  $87 = ((($86)) + 12|0);
  HEAP32[$87>>2] = -7;
  $88 = $need;
  $89 = $7;
  $90 = ((($89)) + 16|0);
  HEAP32[$90>>2] = $88;
  $91 = $3;
  $92 = $7;
  $93 = ((($92)) + 20|0);
  HEAP32[$93>>2] = $91;
  $0 = 0;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 $94 = $Col_size;
 $95 = $Row_size;
 $96 = (($94) + ($95))|0;
 $97 = $3;
 $98 = (($97) - ($96))|0;
 $3 = $98;
 $99 = $3;
 $100 = $4;
 $101 = (($100) + ($99<<2)|0);
 $Col = $101;
 $102 = $3;
 $103 = $Col_size;
 $104 = (($102) + ($103))|0;
 $105 = $4;
 $106 = (($105) + ($104<<2)|0);
 $Row = $106;
 $107 = $1;
 $108 = $2;
 $109 = $Row;
 $110 = $Col;
 $111 = $4;
 $112 = $5;
 $113 = $7;
 $114 = (_init_rows_cols($107,$108,$109,$110,$111,$112,$113)|0);
 $115 = ($114|0)!=(0);
 if ($115) {
  $116 = $1;
  $117 = $2;
  $118 = $Row;
  $119 = $Col;
  $120 = $4;
  $121 = $5;
  $122 = $6;
  _init_scoring($116,$117,$118,$119,$120,$121,$122,$n_row2,$n_col2,$max_deg);
  $123 = $1;
  $124 = $2;
  $125 = $3;
  $126 = $Row;
  $127 = $Col;
  $128 = $4;
  $129 = $5;
  $130 = HEAP32[$n_col2>>2]|0;
  $131 = HEAP32[$max_deg>>2]|0;
  $132 = $nnz;
  $133 = $132<<1;
  $134 = (_find_ordering($123,$124,$125,$126,$127,$128,$129,$130,$131,$133)|0);
  $ngarbage = $134;
  $135 = $2;
  $136 = $Col;
  $137 = $5;
  _order_children($135,$136,$137);
  $138 = $1;
  $139 = HEAP32[$n_row2>>2]|0;
  $140 = (($138) - ($139))|0;
  $141 = $7;
  HEAP32[$141>>2] = $140;
  $142 = $2;
  $143 = HEAP32[$n_col2>>2]|0;
  $144 = (($142) - ($143))|0;
  $145 = $7;
  $146 = ((($145)) + 4|0);
  HEAP32[$146>>2] = $144;
  $147 = $ngarbage;
  $148 = $7;
  $149 = ((($148)) + 8|0);
  HEAP32[$149>>2] = $147;
  $0 = 1;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 } else {
  $0 = 0;
  $150 = $0;
  STACKTOP = sp;return ($150|0);
 }
 return (0)|0;
}
function _init_rows_cols($n_row,$n_col,$Row,$Col,$A,$p,$stats) {
 $n_row = $n_row|0;
 $n_col = $n_col|0;
 $Row = $Row|0;
 $Col = $Col|0;
 $A = $A|0;
 $p = $p|0;
 $stats = $stats|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $col = 0, $cp = 0, $cp_end = 0;
 var $last_row = 0, $row = 0, $rp = 0, $rp_end = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n_row;
 $2 = $n_col;
 $3 = $Row;
 $4 = $Col;
 $5 = $A;
 $6 = $p;
 $7 = $stats;
 $col = 0;
 while(1) {
  $8 = $col;
  $9 = $2;
  $10 = ($8|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $col;
  $12 = $6;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $col;
  $16 = $4;
  $17 = (($16) + (($15*24)|0)|0);
  HEAP32[$17>>2] = $14;
  $18 = $col;
  $19 = (($18) + 1)|0;
  $20 = $6;
  $21 = (($20) + ($19<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $col;
  $24 = $6;
  $25 = (($24) + ($23<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (($22) - ($26))|0;
  $28 = $col;
  $29 = $4;
  $30 = (($29) + (($28*24)|0)|0);
  $31 = ((($30)) + 4|0);
  HEAP32[$31>>2] = $27;
  $32 = $col;
  $33 = $4;
  $34 = (($33) + (($32*24)|0)|0);
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)<(0);
  if ($37) {
   label = 4;
   break;
  }
  $50 = $col;
  $51 = $4;
  $52 = (($51) + (($50*24)|0)|0);
  $53 = ((($52)) + 8|0);
  HEAP32[$53>>2] = 1;
  $54 = $col;
  $55 = $4;
  $56 = (($55) + (($54*24)|0)|0);
  $57 = ((($56)) + 12|0);
  HEAP32[$57>>2] = 0;
  $58 = $col;
  $59 = $4;
  $60 = (($59) + (($58*24)|0)|0);
  $61 = ((($60)) + 16|0);
  HEAP32[$61>>2] = -1;
  $62 = $col;
  $63 = $4;
  $64 = (($63) + (($62*24)|0)|0);
  $65 = ((($64)) + 20|0);
  HEAP32[$65>>2] = -1;
  $66 = $col;
  $67 = (($66) + 1)|0;
  $col = $67;
 }
 if ((label|0) == 4) {
  $38 = $7;
  $39 = ((($38)) + 12|0);
  HEAP32[$39>>2] = -8;
  $40 = $col;
  $41 = $7;
  $42 = ((($41)) + 16|0);
  HEAP32[$42>>2] = $40;
  $43 = $col;
  $44 = $4;
  $45 = (($44) + (($43*24)|0)|0);
  $46 = ((($45)) + 4|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = $7;
  $49 = ((($48)) + 20|0);
  HEAP32[$49>>2] = $47;
  $0 = 0;
  $376 = $0;
  STACKTOP = sp;return ($376|0);
 }
 $68 = $7;
 $69 = ((($68)) + 24|0);
 HEAP32[$69>>2] = 0;
 $row = 0;
 while(1) {
  $70 = $row;
  $71 = $1;
  $72 = ($70|0)<($71|0);
  if (!($72)) {
   break;
  }
  $73 = $row;
  $74 = $3;
  $75 = (($74) + ($73<<4)|0);
  $76 = ((($75)) + 4|0);
  HEAP32[$76>>2] = 0;
  $77 = $row;
  $78 = $3;
  $79 = (($78) + ($77<<4)|0);
  $80 = ((($79)) + 12|0);
  HEAP32[$80>>2] = -1;
  $81 = $row;
  $82 = (($81) + 1)|0;
  $row = $82;
 }
 $col = 0;
 L13: while(1) {
  $83 = $col;
  $84 = $2;
  $85 = ($83|0)<($84|0);
  if (!($85)) {
   break;
  }
  $last_row = -1;
  $86 = $col;
  $87 = $6;
  $88 = (($87) + ($86<<2)|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = $5;
  $91 = (($90) + ($89<<2)|0);
  $cp = $91;
  $92 = $col;
  $93 = (($92) + 1)|0;
  $94 = $6;
  $95 = (($94) + ($93<<2)|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = $5;
  $98 = (($97) + ($96<<2)|0);
  $cp_end = $98;
  while(1) {
   $99 = $cp;
   $100 = $cp_end;
   $101 = ($99>>>0)<($100>>>0);
   if (!($101)) {
    break;
   }
   $102 = $cp;
   $103 = ((($102)) + 4|0);
   $cp = $103;
   $104 = HEAP32[$102>>2]|0;
   $row = $104;
   $105 = $row;
   $106 = ($105|0)<(0);
   if ($106) {
    label = 15;
    break L13;
   }
   $107 = $row;
   $108 = $1;
   $109 = ($107|0)>=($108|0);
   if ($109) {
    label = 15;
    break L13;
   }
   $121 = $row;
   $122 = $last_row;
   $123 = ($121|0)<=($122|0);
   if ($123) {
    label = 18;
   } else {
    $124 = $row;
    $125 = $3;
    $126 = (($125) + ($124<<4)|0);
    $127 = ((($126)) + 12|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = $col;
    $130 = ($128|0)==($129|0);
    if ($130) {
     label = 18;
    }
   }
   if ((label|0) == 18) {
    label = 0;
    $131 = $7;
    $132 = ((($131)) + 12|0);
    HEAP32[$132>>2] = 1;
    $133 = $col;
    $134 = $7;
    $135 = ((($134)) + 16|0);
    HEAP32[$135>>2] = $133;
    $136 = $row;
    $137 = $7;
    $138 = ((($137)) + 20|0);
    HEAP32[$138>>2] = $136;
    $139 = $7;
    $140 = ((($139)) + 24|0);
    $141 = HEAP32[$140>>2]|0;
    $142 = (($141) + 1)|0;
    HEAP32[$140>>2] = $142;
   }
   $143 = $row;
   $144 = $3;
   $145 = (($144) + ($143<<4)|0);
   $146 = ((($145)) + 12|0);
   $147 = HEAP32[$146>>2]|0;
   $148 = $col;
   $149 = ($147|0)!=($148|0);
   if ($149) {
    $150 = $row;
    $151 = $3;
    $152 = (($151) + ($150<<4)|0);
    $153 = ((($152)) + 4|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = (($154) + 1)|0;
    HEAP32[$153>>2] = $155;
   } else {
    $156 = $col;
    $157 = $4;
    $158 = (($157) + (($156*24)|0)|0);
    $159 = ((($158)) + 4|0);
    $160 = HEAP32[$159>>2]|0;
    $161 = (($160) + -1)|0;
    HEAP32[$159>>2] = $161;
   }
   $162 = $col;
   $163 = $row;
   $164 = $3;
   $165 = (($164) + ($163<<4)|0);
   $166 = ((($165)) + 12|0);
   HEAP32[$166>>2] = $162;
   $167 = $row;
   $last_row = $167;
  }
  $168 = $col;
  $169 = (($168) + 1)|0;
  $col = $169;
 }
 if ((label|0) == 15) {
  $110 = $7;
  $111 = ((($110)) + 12|0);
  HEAP32[$111>>2] = -9;
  $112 = $col;
  $113 = $7;
  $114 = ((($113)) + 16|0);
  HEAP32[$114>>2] = $112;
  $115 = $row;
  $116 = $7;
  $117 = ((($116)) + 20|0);
  HEAP32[$117>>2] = $115;
  $118 = $1;
  $119 = $7;
  $120 = ((($119)) + 24|0);
  HEAP32[$120>>2] = $118;
  $0 = 0;
  $376 = $0;
  STACKTOP = sp;return ($376|0);
 }
 $170 = $2;
 $171 = $6;
 $172 = (($171) + ($170<<2)|0);
 $173 = HEAP32[$172>>2]|0;
 $174 = $3;
 HEAP32[$174>>2] = $173;
 $175 = $3;
 $176 = HEAP32[$175>>2]|0;
 $177 = $3;
 $178 = ((($177)) + 8|0);
 HEAP32[$178>>2] = $176;
 $179 = $3;
 $180 = ((($179)) + 12|0);
 HEAP32[$180>>2] = -1;
 $row = 1;
 while(1) {
  $181 = $row;
  $182 = $1;
  $183 = ($181|0)<($182|0);
  if (!($183)) {
   break;
  }
  $184 = $row;
  $185 = (($184) - 1)|0;
  $186 = $3;
  $187 = (($186) + ($185<<4)|0);
  $188 = HEAP32[$187>>2]|0;
  $189 = $row;
  $190 = (($189) - 1)|0;
  $191 = $3;
  $192 = (($191) + ($190<<4)|0);
  $193 = ((($192)) + 4|0);
  $194 = HEAP32[$193>>2]|0;
  $195 = (($188) + ($194))|0;
  $196 = $row;
  $197 = $3;
  $198 = (($197) + ($196<<4)|0);
  HEAP32[$198>>2] = $195;
  $199 = $row;
  $200 = $3;
  $201 = (($200) + ($199<<4)|0);
  $202 = HEAP32[$201>>2]|0;
  $203 = $row;
  $204 = $3;
  $205 = (($204) + ($203<<4)|0);
  $206 = ((($205)) + 8|0);
  HEAP32[$206>>2] = $202;
  $207 = $row;
  $208 = $3;
  $209 = (($208) + ($207<<4)|0);
  $210 = ((($209)) + 12|0);
  HEAP32[$210>>2] = -1;
  $211 = $row;
  $212 = (($211) + 1)|0;
  $row = $212;
 }
 $213 = $7;
 $214 = ((($213)) + 12|0);
 $215 = HEAP32[$214>>2]|0;
 $216 = ($215|0)==(1);
 $col = 0;
 L39: do {
  if ($216) {
   while(1) {
    $217 = $col;
    $218 = $2;
    $219 = ($217|0)<($218|0);
    if (!($219)) {
     break L39;
    }
    $220 = $col;
    $221 = $6;
    $222 = (($221) + ($220<<2)|0);
    $223 = HEAP32[$222>>2]|0;
    $224 = $5;
    $225 = (($224) + ($223<<2)|0);
    $cp = $225;
    $226 = $col;
    $227 = (($226) + 1)|0;
    $228 = $6;
    $229 = (($228) + ($227<<2)|0);
    $230 = HEAP32[$229>>2]|0;
    $231 = $5;
    $232 = (($231) + ($230<<2)|0);
    $cp_end = $232;
    while(1) {
     $233 = $cp;
     $234 = $cp_end;
     $235 = ($233>>>0)<($234>>>0);
     if (!($235)) {
      break;
     }
     $236 = $cp;
     $237 = ((($236)) + 4|0);
     $cp = $237;
     $238 = HEAP32[$236>>2]|0;
     $row = $238;
     $239 = $row;
     $240 = $3;
     $241 = (($240) + ($239<<4)|0);
     $242 = ((($241)) + 12|0);
     $243 = HEAP32[$242>>2]|0;
     $244 = $col;
     $245 = ($243|0)!=($244|0);
     if (!($245)) {
      continue;
     }
     $246 = $col;
     $247 = $row;
     $248 = $3;
     $249 = (($248) + ($247<<4)|0);
     $250 = ((($249)) + 8|0);
     $251 = HEAP32[$250>>2]|0;
     $252 = (($251) + 1)|0;
     HEAP32[$250>>2] = $252;
     $253 = $5;
     $254 = (($253) + ($251<<2)|0);
     HEAP32[$254>>2] = $246;
     $255 = $col;
     $256 = $row;
     $257 = $3;
     $258 = (($257) + ($256<<4)|0);
     $259 = ((($258)) + 12|0);
     HEAP32[$259>>2] = $255;
    }
    $260 = $col;
    $261 = (($260) + 1)|0;
    $col = $261;
   }
  } else {
   while(1) {
    $262 = $col;
    $263 = $2;
    $264 = ($262|0)<($263|0);
    if (!($264)) {
     break L39;
    }
    $265 = $col;
    $266 = $6;
    $267 = (($266) + ($265<<2)|0);
    $268 = HEAP32[$267>>2]|0;
    $269 = $5;
    $270 = (($269) + ($268<<2)|0);
    $cp = $270;
    $271 = $col;
    $272 = (($271) + 1)|0;
    $273 = $6;
    $274 = (($273) + ($272<<2)|0);
    $275 = HEAP32[$274>>2]|0;
    $276 = $5;
    $277 = (($276) + ($275<<2)|0);
    $cp_end = $277;
    while(1) {
     $278 = $cp;
     $279 = $cp_end;
     $280 = ($278>>>0)<($279>>>0);
     $281 = $col;
     if (!($280)) {
      break;
     }
     $282 = $cp;
     $283 = ((($282)) + 4|0);
     $cp = $283;
     $284 = HEAP32[$282>>2]|0;
     $285 = $3;
     $286 = (($285) + ($284<<4)|0);
     $287 = ((($286)) + 8|0);
     $288 = HEAP32[$287>>2]|0;
     $289 = (($288) + 1)|0;
     HEAP32[$287>>2] = $289;
     $290 = $5;
     $291 = (($290) + ($288<<2)|0);
     HEAP32[$291>>2] = $281;
    }
    $292 = (($281) + 1)|0;
    $col = $292;
   }
  }
 } while(0);
 $row = 0;
 while(1) {
  $293 = $row;
  $294 = $1;
  $295 = ($293|0)<($294|0);
  if (!($295)) {
   break;
  }
  $296 = $row;
  $297 = $3;
  $298 = (($297) + ($296<<4)|0);
  $299 = ((($298)) + 12|0);
  HEAP32[$299>>2] = 0;
  $300 = $row;
  $301 = $3;
  $302 = (($301) + ($300<<4)|0);
  $303 = ((($302)) + 4|0);
  $304 = HEAP32[$303>>2]|0;
  $305 = $row;
  $306 = $3;
  $307 = (($306) + ($305<<4)|0);
  $308 = ((($307)) + 8|0);
  HEAP32[$308>>2] = $304;
  $309 = $row;
  $310 = (($309) + 1)|0;
  $row = $310;
 }
 $311 = $7;
 $312 = ((($311)) + 12|0);
 $313 = HEAP32[$312>>2]|0;
 $314 = ($313|0)==(1);
 L60: do {
  if ($314) {
   $315 = $4;
   HEAP32[$315>>2] = 0;
   $316 = $4;
   $317 = HEAP32[$316>>2]|0;
   $318 = $6;
   HEAP32[$318>>2] = $317;
   $col = 1;
   while(1) {
    $319 = $col;
    $320 = $2;
    $321 = ($319|0)<($320|0);
    if (!($321)) {
     break;
    }
    $322 = $col;
    $323 = (($322) - 1)|0;
    $324 = $4;
    $325 = (($324) + (($323*24)|0)|0);
    $326 = HEAP32[$325>>2]|0;
    $327 = $col;
    $328 = (($327) - 1)|0;
    $329 = $4;
    $330 = (($329) + (($328*24)|0)|0);
    $331 = ((($330)) + 4|0);
    $332 = HEAP32[$331>>2]|0;
    $333 = (($326) + ($332))|0;
    $334 = $col;
    $335 = $4;
    $336 = (($335) + (($334*24)|0)|0);
    HEAP32[$336>>2] = $333;
    $337 = $col;
    $338 = $4;
    $339 = (($338) + (($337*24)|0)|0);
    $340 = HEAP32[$339>>2]|0;
    $341 = $col;
    $342 = $6;
    $343 = (($342) + ($341<<2)|0);
    HEAP32[$343>>2] = $340;
    $344 = $col;
    $345 = (($344) + 1)|0;
    $col = $345;
   }
   $row = 0;
   while(1) {
    $346 = $row;
    $347 = $1;
    $348 = ($346|0)<($347|0);
    if (!($348)) {
     break L60;
    }
    $349 = $row;
    $350 = $3;
    $351 = (($350) + ($349<<4)|0);
    $352 = HEAP32[$351>>2]|0;
    $353 = $5;
    $354 = (($353) + ($352<<2)|0);
    $rp = $354;
    $355 = $rp;
    $356 = $row;
    $357 = $3;
    $358 = (($357) + ($356<<4)|0);
    $359 = ((($358)) + 4|0);
    $360 = HEAP32[$359>>2]|0;
    $361 = (($355) + ($360<<2)|0);
    $rp_end = $361;
    while(1) {
     $362 = $rp;
     $363 = $rp_end;
     $364 = ($362>>>0)<($363>>>0);
     $365 = $row;
     if (!($364)) {
      break;
     }
     $366 = $rp;
     $367 = ((($366)) + 4|0);
     $rp = $367;
     $368 = HEAP32[$366>>2]|0;
     $369 = $6;
     $370 = (($369) + ($368<<2)|0);
     $371 = HEAP32[$370>>2]|0;
     $372 = (($371) + 1)|0;
     HEAP32[$370>>2] = $372;
     $373 = $5;
     $374 = (($373) + ($371<<2)|0);
     HEAP32[$374>>2] = $365;
    }
    $375 = (($365) + 1)|0;
    $row = $375;
   }
  }
 } while(0);
 $0 = 1;
 $376 = $0;
 STACKTOP = sp;return ($376|0);
}
function _init_scoring($n_row,$n_col,$Row,$Col,$A,$head,$knobs,$p_n_row2,$p_n_col2,$p_max_deg) {
 $n_row = $n_row|0;
 $n_col = $n_col|0;
 $Row = $Row|0;
 $Col = $Col|0;
 $A = $A|0;
 $head = $head|0;
 $knobs = $knobs|0;
 $p_n_row2 = $p_n_row2|0;
 $p_n_col2 = $p_n_col2|0;
 $p_max_deg = $p_max_deg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0.0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0.0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0.0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0.0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0.0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0.0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0.0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0.0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0.0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0.0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0.0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0.0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0.0, $330 = 0, $331 = 0;
 var $34 = 0, $35 = 0, $36 = 0.0, $37 = 0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0, $41 = 0.0, $42 = 0, $43 = 0.0, $44 = 0, $45 = 0, $46 = 0.0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0.0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0.0, $56 = 0, $57 = 0.0, $58 = 0.0, $59 = 0, $6 = 0, $60 = 0.0, $61 = 0, $62 = 0.0, $63 = 0, $64 = 0, $65 = 0.0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0;
 var $70 = 0.0, $71 = 0, $72 = 0, $73 = 0, $74 = 0.0, $75 = 0, $76 = 0.0, $77 = 0.0, $78 = 0, $79 = 0.0, $8 = 0, $80 = 0, $81 = 0.0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $col_length = 0, $cp = 0, $cp_end = 0, $deg = 0, $dense_col_count = 0, $dense_row_count = 0, $max_deg = 0;
 var $min_score = 0, $n_col2 = 0, $n_row2 = 0, $new_cp = 0, $next_col = 0, $or$cond = 0, $r = 0, $row = 0, $score = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n_row;
 $1 = $n_col;
 $2 = $Row;
 $3 = $Col;
 $4 = $A;
 $5 = $head;
 $6 = $knobs;
 $7 = $p_n_row2;
 $8 = $p_n_col2;
 $9 = $p_max_deg;
 $10 = $6;
 $11 = +HEAPF64[$10>>3];
 $12 = $1;
 $13 = (+($12|0));
 $14 = $11 * $13;
 $15 = $1;
 $16 = (+($15|0));
 $17 = $14 < $16;
 if ($17) {
  $18 = $6;
  $19 = +HEAPF64[$18>>3];
  $20 = $1;
  $21 = (+($20|0));
  $22 = $19 * $21;
  $26 = $22;
 } else {
  $23 = $1;
  $24 = (+($23|0));
  $26 = $24;
 }
 $25 = 0.0 > $26;
 do {
  if ($25) {
   $43 = 0.0;
  } else {
   $27 = $6;
   $28 = +HEAPF64[$27>>3];
   $29 = $1;
   $30 = (+($29|0));
   $31 = $28 * $30;
   $32 = $1;
   $33 = (+($32|0));
   $34 = $31 < $33;
   if ($34) {
    $35 = $6;
    $36 = +HEAPF64[$35>>3];
    $37 = $1;
    $38 = (+($37|0));
    $39 = $36 * $38;
    $43 = $39;
    break;
   } else {
    $40 = $1;
    $41 = (+($40|0));
    $43 = $41;
    break;
   }
  }
 } while(0);
 $42 = (~~(($43)));
 $dense_row_count = $42;
 $44 = $6;
 $45 = ((($44)) + 8|0);
 $46 = +HEAPF64[$45>>3];
 $47 = $0;
 $48 = (+($47|0));
 $49 = $46 * $48;
 $50 = $0;
 $51 = (+($50|0));
 $52 = $49 < $51;
 if ($52) {
  $53 = $6;
  $54 = ((($53)) + 8|0);
  $55 = +HEAPF64[$54>>3];
  $56 = $0;
  $57 = (+($56|0));
  $58 = $55 * $57;
  $62 = $58;
 } else {
  $59 = $0;
  $60 = (+($59|0));
  $62 = $60;
 }
 $61 = 0.0 > $62;
 do {
  if ($61) {
   $81 = 0.0;
  } else {
   $63 = $6;
   $64 = ((($63)) + 8|0);
   $65 = +HEAPF64[$64>>3];
   $66 = $0;
   $67 = (+($66|0));
   $68 = $65 * $67;
   $69 = $0;
   $70 = (+($69|0));
   $71 = $68 < $70;
   if ($71) {
    $72 = $6;
    $73 = ((($72)) + 8|0);
    $74 = +HEAPF64[$73>>3];
    $75 = $0;
    $76 = (+($75|0));
    $77 = $74 * $76;
    $81 = $77;
    break;
   } else {
    $78 = $0;
    $79 = (+($78|0));
    $81 = $79;
    break;
   }
  }
 } while(0);
 $80 = (~~(($81)));
 $dense_col_count = $80;
 $max_deg = 0;
 $82 = $1;
 $n_col2 = $82;
 $83 = $0;
 $n_row2 = $83;
 $84 = $1;
 $85 = (($84) - 1)|0;
 $c = $85;
 while(1) {
  $86 = $c;
  $87 = ($86|0)>=(0);
  if (!($87)) {
   break;
  }
  $88 = $c;
  $89 = $3;
  $90 = (($89) + (($88*24)|0)|0);
  $91 = ((($90)) + 4|0);
  $92 = HEAP32[$91>>2]|0;
  $deg = $92;
  $93 = $deg;
  $94 = ($93|0)==(0);
  if ($94) {
   $95 = $n_col2;
   $96 = (($95) + -1)|0;
   $n_col2 = $96;
   $97 = $c;
   $98 = $3;
   $99 = (($98) + (($97*24)|0)|0);
   $100 = ((($99)) + 12|0);
   HEAP32[$100>>2] = $96;
   $101 = $c;
   $102 = $3;
   $103 = (($102) + (($101*24)|0)|0);
   HEAP32[$103>>2] = -1;
  }
  $104 = $c;
  $105 = (($104) + -1)|0;
  $c = $105;
 }
 $106 = $1;
 $107 = (($106) - 1)|0;
 $c = $107;
 while(1) {
  $108 = $c;
  $109 = ($108|0)>=(0);
  if (!($109)) {
   break;
  }
  $110 = $c;
  $111 = $3;
  $112 = (($111) + (($110*24)|0)|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ($113|0)<(0);
  if (!($114)) {
   $115 = $c;
   $116 = $3;
   $117 = (($116) + (($115*24)|0)|0);
   $118 = ((($117)) + 4|0);
   $119 = HEAP32[$118>>2]|0;
   $deg = $119;
   $120 = $deg;
   $121 = $dense_col_count;
   $122 = ($120|0)>($121|0);
   if ($122) {
    $123 = $n_col2;
    $124 = (($123) + -1)|0;
    $n_col2 = $124;
    $125 = $c;
    $126 = $3;
    $127 = (($126) + (($125*24)|0)|0);
    $128 = ((($127)) + 12|0);
    HEAP32[$128>>2] = $124;
    $129 = $c;
    $130 = $3;
    $131 = (($130) + (($129*24)|0)|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = $4;
    $134 = (($133) + ($132<<2)|0);
    $cp = $134;
    $135 = $cp;
    $136 = $c;
    $137 = $3;
    $138 = (($137) + (($136*24)|0)|0);
    $139 = ((($138)) + 4|0);
    $140 = HEAP32[$139>>2]|0;
    $141 = (($135) + ($140<<2)|0);
    $cp_end = $141;
    while(1) {
     $142 = $cp;
     $143 = $cp_end;
     $144 = ($142>>>0)<($143>>>0);
     if (!($144)) {
      break;
     }
     $145 = $cp;
     $146 = ((($145)) + 4|0);
     $cp = $146;
     $147 = HEAP32[$145>>2]|0;
     $148 = $2;
     $149 = (($148) + ($147<<4)|0);
     $150 = ((($149)) + 8|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = (($151) + -1)|0;
     HEAP32[$150>>2] = $152;
    }
    $153 = $c;
    $154 = $3;
    $155 = (($154) + (($153*24)|0)|0);
    HEAP32[$155>>2] = -1;
   }
  }
  $156 = $c;
  $157 = (($156) + -1)|0;
  $c = $157;
 }
 $r = 0;
 while(1) {
  $158 = $r;
  $159 = $0;
  $160 = ($158|0)<($159|0);
  if (!($160)) {
   break;
  }
  $161 = $r;
  $162 = $2;
  $163 = (($162) + ($161<<4)|0);
  $164 = ((($163)) + 8|0);
  $165 = HEAP32[$164>>2]|0;
  $deg = $165;
  $166 = $deg;
  $167 = $dense_row_count;
  $168 = ($166|0)>($167|0);
  $169 = $deg;
  $170 = ($169|0)==(0);
  $or$cond = $168 | $170;
  if ($or$cond) {
   $171 = $r;
   $172 = $2;
   $173 = (($172) + ($171<<4)|0);
   $174 = ((($173)) + 12|0);
   HEAP32[$174>>2] = -1;
   $175 = $n_row2;
   $176 = (($175) + -1)|0;
   $n_row2 = $176;
  } else {
   $177 = $max_deg;
   $178 = $deg;
   $179 = ($177|0)>($178|0);
   $180 = $max_deg;
   $181 = $deg;
   $182 = $179 ? $180 : $181;
   $max_deg = $182;
  }
  $183 = $r;
  $184 = (($183) + 1)|0;
  $r = $184;
 }
 $185 = $1;
 $186 = (($185) - 1)|0;
 $c = $186;
 while(1) {
  $187 = $c;
  $188 = ($187|0)>=(0);
  if (!($188)) {
   break;
  }
  $189 = $c;
  $190 = $3;
  $191 = (($190) + (($189*24)|0)|0);
  $192 = HEAP32[$191>>2]|0;
  $193 = ($192|0)<(0);
  do {
   if (!($193)) {
    $score = 0;
    $194 = $c;
    $195 = $3;
    $196 = (($195) + (($194*24)|0)|0);
    $197 = HEAP32[$196>>2]|0;
    $198 = $4;
    $199 = (($198) + ($197<<2)|0);
    $cp = $199;
    $200 = $cp;
    $new_cp = $200;
    $201 = $cp;
    $202 = $c;
    $203 = $3;
    $204 = (($203) + (($202*24)|0)|0);
    $205 = ((($204)) + 4|0);
    $206 = HEAP32[$205>>2]|0;
    $207 = (($201) + ($206<<2)|0);
    $cp_end = $207;
    while(1) {
     $208 = $cp;
     $209 = $cp_end;
     $210 = ($208>>>0)<($209>>>0);
     if (!($210)) {
      break;
     }
     $211 = $cp;
     $212 = ((($211)) + 4|0);
     $cp = $212;
     $213 = HEAP32[$211>>2]|0;
     $row = $213;
     $214 = $row;
     $215 = $2;
     $216 = (($215) + ($214<<4)|0);
     $217 = ((($216)) + 12|0);
     $218 = HEAP32[$217>>2]|0;
     $219 = ($218|0)<(0);
     if ($219) {
      continue;
     }
     $220 = $row;
     $221 = $new_cp;
     $222 = ((($221)) + 4|0);
     $new_cp = $222;
     HEAP32[$221>>2] = $220;
     $223 = $row;
     $224 = $2;
     $225 = (($224) + ($223<<4)|0);
     $226 = ((($225)) + 8|0);
     $227 = HEAP32[$226>>2]|0;
     $228 = (($227) - 1)|0;
     $229 = $score;
     $230 = (($229) + ($228))|0;
     $score = $230;
     $231 = $score;
     $232 = $1;
     $233 = ($231|0)<($232|0);
     $234 = $score;
     $235 = $1;
     $236 = $233 ? $234 : $235;
     $score = $236;
    }
    $237 = $new_cp;
    $238 = $c;
    $239 = $3;
    $240 = (($239) + (($238*24)|0)|0);
    $241 = HEAP32[$240>>2]|0;
    $242 = $4;
    $243 = (($242) + ($241<<2)|0);
    $244 = $237;
    $245 = $243;
    $246 = (($244) - ($245))|0;
    $247 = (($246|0) / 4)&-1;
    $col_length = $247;
    $248 = $col_length;
    $249 = ($248|0)==(0);
    if ($249) {
     $250 = $n_col2;
     $251 = (($250) + -1)|0;
     $n_col2 = $251;
     $252 = $c;
     $253 = $3;
     $254 = (($253) + (($252*24)|0)|0);
     $255 = ((($254)) + 12|0);
     HEAP32[$255>>2] = $251;
     $256 = $c;
     $257 = $3;
     $258 = (($257) + (($256*24)|0)|0);
     HEAP32[$258>>2] = -1;
     break;
    } else {
     $259 = $col_length;
     $260 = $c;
     $261 = $3;
     $262 = (($261) + (($260*24)|0)|0);
     $263 = ((($262)) + 4|0);
     HEAP32[$263>>2] = $259;
     $264 = $score;
     $265 = $c;
     $266 = $3;
     $267 = (($266) + (($265*24)|0)|0);
     $268 = ((($267)) + 12|0);
     HEAP32[$268>>2] = $264;
     break;
    }
   }
  } while(0);
  $269 = $c;
  $270 = (($269) + -1)|0;
  $c = $270;
 }
 $c = 0;
 while(1) {
  $271 = $c;
  $272 = $1;
  $273 = ($271|0)<=($272|0);
  if (!($273)) {
   break;
  }
  $274 = $c;
  $275 = $5;
  $276 = (($275) + ($274<<2)|0);
  HEAP32[$276>>2] = -1;
  $277 = $c;
  $278 = (($277) + 1)|0;
  $c = $278;
 }
 $279 = $1;
 $min_score = $279;
 $280 = $1;
 $281 = (($280) - 1)|0;
 $c = $281;
 while(1) {
  $282 = $c;
  $283 = ($282|0)>=(0);
  if (!($283)) {
   break;
  }
  $284 = $c;
  $285 = $3;
  $286 = (($285) + (($284*24)|0)|0);
  $287 = HEAP32[$286>>2]|0;
  $288 = ($287|0)>=(0);
  if ($288) {
   $289 = $c;
   $290 = $3;
   $291 = (($290) + (($289*24)|0)|0);
   $292 = ((($291)) + 12|0);
   $293 = HEAP32[$292>>2]|0;
   $score = $293;
   $294 = $score;
   $295 = $5;
   $296 = (($295) + ($294<<2)|0);
   $297 = HEAP32[$296>>2]|0;
   $next_col = $297;
   $298 = $c;
   $299 = $3;
   $300 = (($299) + (($298*24)|0)|0);
   $301 = ((($300)) + 16|0);
   HEAP32[$301>>2] = -1;
   $302 = $next_col;
   $303 = $c;
   $304 = $3;
   $305 = (($304) + (($303*24)|0)|0);
   $306 = ((($305)) + 20|0);
   HEAP32[$306>>2] = $302;
   $307 = $next_col;
   $308 = ($307|0)!=(-1);
   if ($308) {
    $309 = $c;
    $310 = $next_col;
    $311 = $3;
    $312 = (($311) + (($310*24)|0)|0);
    $313 = ((($312)) + 16|0);
    HEAP32[$313>>2] = $309;
   }
   $314 = $c;
   $315 = $score;
   $316 = $5;
   $317 = (($316) + ($315<<2)|0);
   HEAP32[$317>>2] = $314;
   $318 = $min_score;
   $319 = $score;
   $320 = ($318|0)<($319|0);
   $321 = $min_score;
   $322 = $score;
   $323 = $320 ? $321 : $322;
   $min_score = $323;
  }
  $324 = $c;
  $325 = (($324) + -1)|0;
  $c = $325;
 }
 $326 = $n_col2;
 $327 = $8;
 HEAP32[$327>>2] = $326;
 $328 = $n_row2;
 $329 = $7;
 HEAP32[$329>>2] = $328;
 $330 = $max_deg;
 $331 = $9;
 HEAP32[$331>>2] = $330;
 STACKTOP = sp;return;
}
function _find_ordering($n_row,$n_col,$Alen,$Row,$Col,$A,$head,$n_col2,$max_deg,$pfree) {
 $n_row = $n_row|0;
 $n_col = $n_col|0;
 $Alen = $Alen|0;
 $Row = $Row|0;
 $Col = $Col|0;
 $A = $A|0;
 $head = $head|0;
 $n_col2 = $n_col2|0;
 $max_deg = $max_deg|0;
 $pfree = $pfree|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $col = 0, $col_thickness = 0, $cp = 0, $cp_end = 0, $cur_score = 0, $first_col = 0, $hash = 0, $head_column = 0, $k = 0;
 var $max_mark = 0, $max_score = 0, $min_score = 0, $needed_memory = 0, $new_cp = 0, $new_rp = 0, $next_col = 0, $ngarbage = 0, $pivot_col = 0, $pivot_col_score = 0, $pivot_col_thickness = 0, $pivot_row = 0, $pivot_row_degree = 0, $pivot_row_length = 0, $pivot_row_start = 0, $prev_col = 0, $row = 0, $row_mark = 0, $rp = 0, $rp_end = 0;
 var $set_difference = 0, $tag_mark = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n_row;
 $1 = $n_col;
 $2 = $Alen;
 $3 = $Row;
 $4 = $Col;
 $5 = $A;
 $6 = $head;
 $7 = $n_col2;
 $8 = $max_deg;
 $9 = $pfree;
 $10 = $1;
 $11 = (2147483647 - ($10))|0;
 $max_mark = $11;
 $12 = $0;
 $13 = $3;
 $14 = (_clear_mark($12,$13)|0);
 $tag_mark = $14;
 $min_score = 0;
 $ngarbage = 0;
 $k = 0;
 while(1) {
  $15 = $k;
  $16 = $7;
  $17 = ($15|0)<($16|0);
  if (!($17)) {
   break;
  }
  while(1) {
   $18 = $min_score;
   $19 = $6;
   $20 = (($19) + ($18<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)==(-1);
   if ($22) {
    $23 = $min_score;
    $24 = $1;
    $25 = ($23|0)<($24|0);
    $610 = $25;
   } else {
    $610 = 0;
   }
   $26 = $min_score;
   if (!($610)) {
    break;
   }
   $27 = (($26) + 1)|0;
   $min_score = $27;
  }
  $28 = $6;
  $29 = (($28) + ($26<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $pivot_col = $30;
  $31 = $pivot_col;
  $32 = $4;
  $33 = (($32) + (($31*24)|0)|0);
  $34 = ((($33)) + 20|0);
  $35 = HEAP32[$34>>2]|0;
  $next_col = $35;
  $36 = $next_col;
  $37 = $min_score;
  $38 = $6;
  $39 = (($38) + ($37<<2)|0);
  HEAP32[$39>>2] = $36;
  $40 = $next_col;
  $41 = ($40|0)!=(-1);
  if ($41) {
   $42 = $next_col;
   $43 = $4;
   $44 = (($43) + (($42*24)|0)|0);
   $45 = ((($44)) + 16|0);
   HEAP32[$45>>2] = -1;
  }
  $46 = $pivot_col;
  $47 = $4;
  $48 = (($47) + (($46*24)|0)|0);
  $49 = ((($48)) + 12|0);
  $50 = HEAP32[$49>>2]|0;
  $pivot_col_score = $50;
  $51 = $k;
  $52 = $pivot_col;
  $53 = $4;
  $54 = (($53) + (($52*24)|0)|0);
  $55 = ((($54)) + 12|0);
  HEAP32[$55>>2] = $51;
  $56 = $pivot_col;
  $57 = $4;
  $58 = (($57) + (($56*24)|0)|0);
  $59 = ((($58)) + 8|0);
  $60 = HEAP32[$59>>2]|0;
  $pivot_col_thickness = $60;
  $61 = $pivot_col_thickness;
  $62 = $k;
  $63 = (($62) + ($61))|0;
  $k = $63;
  $64 = $pivot_col_score;
  $65 = $1;
  $66 = $k;
  $67 = (($65) - ($66))|0;
  $68 = ($64|0)<($67|0);
  if ($68) {
   $69 = $pivot_col_score;
   $73 = $69;
  } else {
   $70 = $1;
   $71 = $k;
   $72 = (($70) - ($71))|0;
   $73 = $72;
  }
  $needed_memory = $73;
  $74 = $9;
  $75 = $needed_memory;
  $76 = (($74) + ($75))|0;
  $77 = $2;
  $78 = ($76|0)>=($77|0);
  if ($78) {
   $79 = $0;
   $80 = $1;
   $81 = $3;
   $82 = $4;
   $83 = $5;
   $84 = $9;
   $85 = $5;
   $86 = (($85) + ($84<<2)|0);
   $87 = (_garbage_collection($79,$80,$81,$82,$83,$86)|0);
   $9 = $87;
   $88 = $ngarbage;
   $89 = (($88) + 1)|0;
   $ngarbage = $89;
   $90 = $0;
   $91 = $3;
   $92 = (_clear_mark($90,$91)|0);
   $tag_mark = $92;
  }
  $93 = $9;
  $pivot_row_start = $93;
  $pivot_row_degree = 0;
  $94 = $pivot_col_thickness;
  $95 = (0 - ($94))|0;
  $96 = $pivot_col;
  $97 = $4;
  $98 = (($97) + (($96*24)|0)|0);
  $99 = ((($98)) + 8|0);
  HEAP32[$99>>2] = $95;
  $100 = $pivot_col;
  $101 = $4;
  $102 = (($101) + (($100*24)|0)|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = $5;
  $105 = (($104) + ($103<<2)|0);
  $cp = $105;
  $106 = $cp;
  $107 = $pivot_col;
  $108 = $4;
  $109 = (($108) + (($107*24)|0)|0);
  $110 = ((($109)) + 4|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = (($106) + ($111<<2)|0);
  $cp_end = $112;
  L20: while(1) {
   $113 = $cp;
   $114 = $cp_end;
   $115 = ($113>>>0)<($114>>>0);
   if (!($115)) {
    break;
   }
   $116 = $cp;
   $117 = ((($116)) + 4|0);
   $cp = $117;
   $118 = HEAP32[$116>>2]|0;
   $row = $118;
   $119 = $row;
   $120 = $3;
   $121 = (($120) + ($119<<4)|0);
   $122 = ((($121)) + 12|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = ($123|0)<(0);
   if ($124) {
    continue;
   }
   $125 = $row;
   $126 = $3;
   $127 = (($126) + ($125<<4)|0);
   $128 = HEAP32[$127>>2]|0;
   $129 = $5;
   $130 = (($129) + ($128<<2)|0);
   $rp = $130;
   $131 = $rp;
   $132 = $row;
   $133 = $3;
   $134 = (($133) + ($132<<4)|0);
   $135 = ((($134)) + 4|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = (($131) + ($136<<2)|0);
   $rp_end = $137;
   while(1) {
    $138 = $rp;
    $139 = $rp_end;
    $140 = ($138>>>0)<($139>>>0);
    if (!($140)) {
     continue L20;
    }
    $141 = $rp;
    $142 = ((($141)) + 4|0);
    $rp = $142;
    $143 = HEAP32[$141>>2]|0;
    $col = $143;
    $144 = $col;
    $145 = $4;
    $146 = (($145) + (($144*24)|0)|0);
    $147 = ((($146)) + 8|0);
    $148 = HEAP32[$147>>2]|0;
    $col_thickness = $148;
    $149 = $col_thickness;
    $150 = ($149|0)>(0);
    if (!($150)) {
     continue;
    }
    $151 = $col;
    $152 = $4;
    $153 = (($152) + (($151*24)|0)|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = ($154|0)>=(0);
    if (!($155)) {
     continue;
    }
    $156 = $col_thickness;
    $157 = (0 - ($156))|0;
    $158 = $col;
    $159 = $4;
    $160 = (($159) + (($158*24)|0)|0);
    $161 = ((($160)) + 8|0);
    HEAP32[$161>>2] = $157;
    $162 = $col;
    $163 = $9;
    $164 = (($163) + 1)|0;
    $9 = $164;
    $165 = $5;
    $166 = (($165) + ($163<<2)|0);
    HEAP32[$166>>2] = $162;
    $167 = $col_thickness;
    $168 = $pivot_row_degree;
    $169 = (($168) + ($167))|0;
    $pivot_row_degree = $169;
   }
  }
  $170 = $pivot_col_thickness;
  $171 = $pivot_col;
  $172 = $4;
  $173 = (($172) + (($171*24)|0)|0);
  $174 = ((($173)) + 8|0);
  HEAP32[$174>>2] = $170;
  $175 = $8;
  $176 = $pivot_row_degree;
  $177 = ($175|0)>($176|0);
  $178 = $8;
  $179 = $pivot_row_degree;
  $180 = $177 ? $178 : $179;
  $8 = $180;
  $181 = $pivot_col;
  $182 = $4;
  $183 = (($182) + (($181*24)|0)|0);
  $184 = HEAP32[$183>>2]|0;
  $185 = $5;
  $186 = (($185) + ($184<<2)|0);
  $cp = $186;
  $187 = $cp;
  $188 = $pivot_col;
  $189 = $4;
  $190 = (($189) + (($188*24)|0)|0);
  $191 = ((($190)) + 4|0);
  $192 = HEAP32[$191>>2]|0;
  $193 = (($187) + ($192<<2)|0);
  $cp_end = $193;
  while(1) {
   $194 = $cp;
   $195 = $cp_end;
   $196 = ($194>>>0)<($195>>>0);
   if (!($196)) {
    break;
   }
   $197 = $cp;
   $198 = ((($197)) + 4|0);
   $cp = $198;
   $199 = HEAP32[$197>>2]|0;
   $row = $199;
   $200 = $row;
   $201 = $3;
   $202 = (($201) + ($200<<4)|0);
   $203 = ((($202)) + 12|0);
   HEAP32[$203>>2] = -1;
  }
  $204 = $9;
  $205 = $pivot_row_start;
  $206 = (($204) - ($205))|0;
  $pivot_row_length = $206;
  $207 = $pivot_row_length;
  $208 = ($207|0)>(0);
  if ($208) {
   $209 = $pivot_col;
   $210 = $4;
   $211 = (($210) + (($209*24)|0)|0);
   $212 = HEAP32[$211>>2]|0;
   $213 = $5;
   $214 = (($213) + ($212<<2)|0);
   $215 = HEAP32[$214>>2]|0;
   $pivot_row = $215;
  } else {
   $pivot_row = -1;
  }
  $216 = $pivot_row_start;
  $217 = $5;
  $218 = (($217) + ($216<<2)|0);
  $rp = $218;
  $219 = $rp;
  $220 = $pivot_row_length;
  $221 = (($219) + ($220<<2)|0);
  $rp_end = $221;
  L38: while(1) {
   $222 = $rp;
   $223 = $rp_end;
   $224 = ($222>>>0)<($223>>>0);
   if (!($224)) {
    break;
   }
   $225 = $rp;
   $226 = ((($225)) + 4|0);
   $rp = $226;
   $227 = HEAP32[$225>>2]|0;
   $col = $227;
   $228 = $col;
   $229 = $4;
   $230 = (($229) + (($228*24)|0)|0);
   $231 = ((($230)) + 8|0);
   $232 = HEAP32[$231>>2]|0;
   $233 = (0 - ($232))|0;
   $col_thickness = $233;
   $234 = $col_thickness;
   $235 = $col;
   $236 = $4;
   $237 = (($236) + (($235*24)|0)|0);
   $238 = ((($237)) + 8|0);
   HEAP32[$238>>2] = $234;
   $239 = $col;
   $240 = $4;
   $241 = (($240) + (($239*24)|0)|0);
   $242 = ((($241)) + 12|0);
   $243 = HEAP32[$242>>2]|0;
   $cur_score = $243;
   $244 = $col;
   $245 = $4;
   $246 = (($245) + (($244*24)|0)|0);
   $247 = ((($246)) + 16|0);
   $248 = HEAP32[$247>>2]|0;
   $prev_col = $248;
   $249 = $col;
   $250 = $4;
   $251 = (($250) + (($249*24)|0)|0);
   $252 = ((($251)) + 20|0);
   $253 = HEAP32[$252>>2]|0;
   $next_col = $253;
   $254 = $prev_col;
   $255 = ($254|0)==(-1);
   $256 = $next_col;
   if ($255) {
    $257 = $cur_score;
    $258 = $6;
    $259 = (($258) + ($257<<2)|0);
    HEAP32[$259>>2] = $256;
   } else {
    $260 = $prev_col;
    $261 = $4;
    $262 = (($261) + (($260*24)|0)|0);
    $263 = ((($262)) + 20|0);
    HEAP32[$263>>2] = $256;
   }
   $264 = $next_col;
   $265 = ($264|0)!=(-1);
   if ($265) {
    $266 = $prev_col;
    $267 = $next_col;
    $268 = $4;
    $269 = (($268) + (($267*24)|0)|0);
    $270 = ((($269)) + 16|0);
    HEAP32[$270>>2] = $266;
   }
   $271 = $col;
   $272 = $4;
   $273 = (($272) + (($271*24)|0)|0);
   $274 = HEAP32[$273>>2]|0;
   $275 = $5;
   $276 = (($275) + ($274<<2)|0);
   $cp = $276;
   $277 = $cp;
   $278 = $col;
   $279 = $4;
   $280 = (($279) + (($278*24)|0)|0);
   $281 = ((($280)) + 4|0);
   $282 = HEAP32[$281>>2]|0;
   $283 = (($277) + ($282<<2)|0);
   $cp_end = $283;
   while(1) {
    $284 = $cp;
    $285 = $cp_end;
    $286 = ($284>>>0)<($285>>>0);
    if (!($286)) {
     continue L38;
    }
    $287 = $cp;
    $288 = ((($287)) + 4|0);
    $cp = $288;
    $289 = HEAP32[$287>>2]|0;
    $row = $289;
    $290 = $row;
    $291 = $3;
    $292 = (($291) + ($290<<4)|0);
    $293 = ((($292)) + 12|0);
    $294 = HEAP32[$293>>2]|0;
    $row_mark = $294;
    $295 = $row_mark;
    $296 = ($295|0)<(0);
    if ($296) {
     continue;
    }
    $297 = $row_mark;
    $298 = $tag_mark;
    $299 = (($297) - ($298))|0;
    $set_difference = $299;
    $300 = $set_difference;
    $301 = ($300|0)<(0);
    if ($301) {
     $302 = $row;
     $303 = $3;
     $304 = (($303) + ($302<<4)|0);
     $305 = ((($304)) + 8|0);
     $306 = HEAP32[$305>>2]|0;
     $set_difference = $306;
    }
    $307 = $col_thickness;
    $308 = $set_difference;
    $309 = (($308) - ($307))|0;
    $set_difference = $309;
    $310 = $set_difference;
    $311 = ($310|0)==(0);
    if ($311) {
     $312 = $row;
     $313 = $3;
     $314 = (($313) + ($312<<4)|0);
     $315 = ((($314)) + 12|0);
     HEAP32[$315>>2] = -1;
     continue;
    } else {
     $316 = $set_difference;
     $317 = $tag_mark;
     $318 = (($316) + ($317))|0;
     $319 = $row;
     $320 = $3;
     $321 = (($320) + ($319<<4)|0);
     $322 = ((($321)) + 12|0);
     HEAP32[$322>>2] = $318;
     continue;
    }
   }
  }
  $323 = $pivot_row_start;
  $324 = $5;
  $325 = (($324) + ($323<<2)|0);
  $rp = $325;
  $326 = $rp;
  $327 = $pivot_row_length;
  $328 = (($326) + ($327<<2)|0);
  $rp_end = $328;
  while(1) {
   $329 = $rp;
   $330 = $rp_end;
   $331 = ($329>>>0)<($330>>>0);
   if (!($331)) {
    break;
   }
   $332 = $rp;
   $333 = ((($332)) + 4|0);
   $rp = $333;
   $334 = HEAP32[$332>>2]|0;
   $col = $334;
   $hash = 0;
   $cur_score = 0;
   $335 = $col;
   $336 = $4;
   $337 = (($336) + (($335*24)|0)|0);
   $338 = HEAP32[$337>>2]|0;
   $339 = $5;
   $340 = (($339) + ($338<<2)|0);
   $cp = $340;
   $341 = $cp;
   $new_cp = $341;
   $342 = $cp;
   $343 = $col;
   $344 = $4;
   $345 = (($344) + (($343*24)|0)|0);
   $346 = ((($345)) + 4|0);
   $347 = HEAP32[$346>>2]|0;
   $348 = (($342) + ($347<<2)|0);
   $cp_end = $348;
   while(1) {
    $349 = $cp;
    $350 = $cp_end;
    $351 = ($349>>>0)<($350>>>0);
    if (!($351)) {
     break;
    }
    $352 = $cp;
    $353 = ((($352)) + 4|0);
    $cp = $353;
    $354 = HEAP32[$352>>2]|0;
    $row = $354;
    $355 = $row;
    $356 = $3;
    $357 = (($356) + ($355<<4)|0);
    $358 = ((($357)) + 12|0);
    $359 = HEAP32[$358>>2]|0;
    $row_mark = $359;
    $360 = $row_mark;
    $361 = ($360|0)<(0);
    if ($361) {
     continue;
    }
    $362 = $row;
    $363 = $new_cp;
    $364 = ((($363)) + 4|0);
    $new_cp = $364;
    HEAP32[$363>>2] = $362;
    $365 = $row;
    $366 = $hash;
    $367 = (($366) + ($365))|0;
    $hash = $367;
    $368 = $row_mark;
    $369 = $tag_mark;
    $370 = (($368) - ($369))|0;
    $371 = $cur_score;
    $372 = (($371) + ($370))|0;
    $cur_score = $372;
    $373 = $cur_score;
    $374 = $1;
    $375 = ($373|0)<($374|0);
    $376 = $cur_score;
    $377 = $1;
    $378 = $375 ? $376 : $377;
    $cur_score = $378;
   }
   $379 = $new_cp;
   $380 = $col;
   $381 = $4;
   $382 = (($381) + (($380*24)|0)|0);
   $383 = HEAP32[$382>>2]|0;
   $384 = $5;
   $385 = (($384) + ($383<<2)|0);
   $386 = $379;
   $387 = $385;
   $388 = (($386) - ($387))|0;
   $389 = (($388|0) / 4)&-1;
   $390 = $col;
   $391 = $4;
   $392 = (($391) + (($390*24)|0)|0);
   $393 = ((($392)) + 4|0);
   HEAP32[$393>>2] = $389;
   $394 = $col;
   $395 = $4;
   $396 = (($395) + (($394*24)|0)|0);
   $397 = ((($396)) + 4|0);
   $398 = HEAP32[$397>>2]|0;
   $399 = ($398|0)==(0);
   if ($399) {
    $400 = $col;
    $401 = $4;
    $402 = (($401) + (($400*24)|0)|0);
    HEAP32[$402>>2] = -1;
    $403 = $col;
    $404 = $4;
    $405 = (($404) + (($403*24)|0)|0);
    $406 = ((($405)) + 8|0);
    $407 = HEAP32[$406>>2]|0;
    $408 = $pivot_row_degree;
    $409 = (($408) - ($407))|0;
    $pivot_row_degree = $409;
    $410 = $k;
    $411 = $col;
    $412 = $4;
    $413 = (($412) + (($411*24)|0)|0);
    $414 = ((($413)) + 12|0);
    HEAP32[$414>>2] = $410;
    $415 = $col;
    $416 = $4;
    $417 = (($416) + (($415*24)|0)|0);
    $418 = ((($417)) + 8|0);
    $419 = HEAP32[$418>>2]|0;
    $420 = $k;
    $421 = (($420) + ($419))|0;
    $k = $421;
    continue;
   }
   $422 = $cur_score;
   $423 = $col;
   $424 = $4;
   $425 = (($424) + (($423*24)|0)|0);
   $426 = ((($425)) + 12|0);
   HEAP32[$426>>2] = $422;
   $427 = $1;
   $428 = (($427) + 1)|0;
   $429 = $hash;
   $430 = (($429>>>0) % ($428>>>0))&-1;
   $hash = $430;
   $431 = $hash;
   $432 = $6;
   $433 = (($432) + ($431<<2)|0);
   $434 = HEAP32[$433>>2]|0;
   $head_column = $434;
   $435 = $head_column;
   $436 = ($435|0)>(-1);
   $437 = $head_column;
   if ($436) {
    $438 = $4;
    $439 = (($438) + (($437*24)|0)|0);
    $440 = ((($439)) + 16|0);
    $441 = HEAP32[$440>>2]|0;
    $first_col = $441;
    $442 = $col;
    $443 = $head_column;
    $444 = $4;
    $445 = (($444) + (($443*24)|0)|0);
    $446 = ((($445)) + 16|0);
    HEAP32[$446>>2] = $442;
   } else {
    $447 = (($437) + 2)|0;
    $448 = (0 - ($447))|0;
    $first_col = $448;
    $449 = $col;
    $450 = (($449) + 2)|0;
    $451 = (0 - ($450))|0;
    $452 = $hash;
    $453 = $6;
    $454 = (($453) + ($452<<2)|0);
    HEAP32[$454>>2] = $451;
   }
   $455 = $first_col;
   $456 = $col;
   $457 = $4;
   $458 = (($457) + (($456*24)|0)|0);
   $459 = ((($458)) + 20|0);
   HEAP32[$459>>2] = $455;
   $460 = $hash;
   $461 = $col;
   $462 = $4;
   $463 = (($462) + (($461*24)|0)|0);
   $464 = ((($463)) + 16|0);
   HEAP32[$464>>2] = $460;
  }
  $465 = $4;
  $466 = $5;
  $467 = $6;
  $468 = $pivot_row_start;
  $469 = $pivot_row_length;
  _detect_super_cols($465,$466,$467,$468,$469);
  $470 = $pivot_col;
  $471 = $4;
  $472 = (($471) + (($470*24)|0)|0);
  HEAP32[$472>>2] = -1;
  $473 = $8;
  $474 = (($473) + 1)|0;
  $475 = $tag_mark;
  $476 = (($475) + ($474))|0;
  $tag_mark = $476;
  $477 = $tag_mark;
  $478 = $max_mark;
  $479 = ($477|0)>=($478|0);
  if ($479) {
   $480 = $0;
   $481 = $3;
   $482 = (_clear_mark($480,$481)|0);
   $tag_mark = $482;
  }
  $483 = $pivot_row_start;
  $484 = $5;
  $485 = (($484) + ($483<<2)|0);
  $rp = $485;
  $486 = $rp;
  $new_rp = $486;
  $487 = $rp;
  $488 = $pivot_row_length;
  $489 = (($487) + ($488<<2)|0);
  $rp_end = $489;
  while(1) {
   $490 = $rp;
   $491 = $rp_end;
   $492 = ($490>>>0)<($491>>>0);
   if (!($492)) {
    break;
   }
   $493 = $rp;
   $494 = ((($493)) + 4|0);
   $rp = $494;
   $495 = HEAP32[$493>>2]|0;
   $col = $495;
   $496 = $col;
   $497 = $4;
   $498 = (($497) + (($496*24)|0)|0);
   $499 = HEAP32[$498>>2]|0;
   $500 = ($499|0)<(0);
   if ($500) {
    continue;
   }
   $501 = $col;
   $502 = $new_rp;
   $503 = ((($502)) + 4|0);
   $new_rp = $503;
   HEAP32[$502>>2] = $501;
   $504 = $pivot_row;
   $505 = $col;
   $506 = $4;
   $507 = (($506) + (($505*24)|0)|0);
   $508 = HEAP32[$507>>2]|0;
   $509 = $col;
   $510 = $4;
   $511 = (($510) + (($509*24)|0)|0);
   $512 = ((($511)) + 4|0);
   $513 = HEAP32[$512>>2]|0;
   $514 = (($513) + 1)|0;
   HEAP32[$512>>2] = $514;
   $515 = (($508) + ($513))|0;
   $516 = $5;
   $517 = (($516) + ($515<<2)|0);
   HEAP32[$517>>2] = $504;
   $518 = $col;
   $519 = $4;
   $520 = (($519) + (($518*24)|0)|0);
   $521 = ((($520)) + 12|0);
   $522 = HEAP32[$521>>2]|0;
   $523 = $pivot_row_degree;
   $524 = (($522) + ($523))|0;
   $cur_score = $524;
   $525 = $1;
   $526 = $k;
   $527 = (($525) - ($526))|0;
   $528 = $col;
   $529 = $4;
   $530 = (($529) + (($528*24)|0)|0);
   $531 = ((($530)) + 8|0);
   $532 = HEAP32[$531>>2]|0;
   $533 = (($527) - ($532))|0;
   $max_score = $533;
   $534 = $col;
   $535 = $4;
   $536 = (($535) + (($534*24)|0)|0);
   $537 = ((($536)) + 8|0);
   $538 = HEAP32[$537>>2]|0;
   $539 = $cur_score;
   $540 = (($539) - ($538))|0;
   $cur_score = $540;
   $541 = $cur_score;
   $542 = $max_score;
   $543 = ($541|0)<($542|0);
   $544 = $cur_score;
   $545 = $max_score;
   $546 = $543 ? $544 : $545;
   $cur_score = $546;
   $547 = $cur_score;
   $548 = $col;
   $549 = $4;
   $550 = (($549) + (($548*24)|0)|0);
   $551 = ((($550)) + 12|0);
   HEAP32[$551>>2] = $547;
   $552 = $cur_score;
   $553 = $6;
   $554 = (($553) + ($552<<2)|0);
   $555 = HEAP32[$554>>2]|0;
   $next_col = $555;
   $556 = $next_col;
   $557 = $col;
   $558 = $4;
   $559 = (($558) + (($557*24)|0)|0);
   $560 = ((($559)) + 20|0);
   HEAP32[$560>>2] = $556;
   $561 = $col;
   $562 = $4;
   $563 = (($562) + (($561*24)|0)|0);
   $564 = ((($563)) + 16|0);
   HEAP32[$564>>2] = -1;
   $565 = $next_col;
   $566 = ($565|0)!=(-1);
   if ($566) {
    $567 = $col;
    $568 = $next_col;
    $569 = $4;
    $570 = (($569) + (($568*24)|0)|0);
    $571 = ((($570)) + 16|0);
    HEAP32[$571>>2] = $567;
   }
   $572 = $col;
   $573 = $cur_score;
   $574 = $6;
   $575 = (($574) + ($573<<2)|0);
   HEAP32[$575>>2] = $572;
   $576 = $min_score;
   $577 = $cur_score;
   $578 = ($576|0)<($577|0);
   $579 = $min_score;
   $580 = $cur_score;
   $581 = $578 ? $579 : $580;
   $min_score = $581;
  }
  $582 = $pivot_row_degree;
  $583 = ($582|0)>(0);
  if (!($583)) {
   continue;
  }
  $584 = $pivot_row_start;
  $585 = $pivot_row;
  $586 = $3;
  $587 = (($586) + ($585<<4)|0);
  HEAP32[$587>>2] = $584;
  $588 = $new_rp;
  $589 = $pivot_row_start;
  $590 = $5;
  $591 = (($590) + ($589<<2)|0);
  $592 = $588;
  $593 = $591;
  $594 = (($592) - ($593))|0;
  $595 = (($594|0) / 4)&-1;
  $596 = $pivot_row;
  $597 = $3;
  $598 = (($597) + ($596<<4)|0);
  $599 = ((($598)) + 4|0);
  HEAP32[$599>>2] = $595;
  $600 = $pivot_row_degree;
  $601 = $pivot_row;
  $602 = $3;
  $603 = (($602) + ($601<<4)|0);
  $604 = ((($603)) + 8|0);
  HEAP32[$604>>2] = $600;
  $605 = $pivot_row;
  $606 = $3;
  $607 = (($606) + ($605<<4)|0);
  $608 = ((($607)) + 12|0);
  HEAP32[$608>>2] = 0;
 }
 $609 = $ngarbage;
 STACKTOP = sp;return ($609|0);
}
function _order_children($n_col,$Col,$p) {
 $n_col = $n_col|0;
 $Col = $Col|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $c = 0, $i = 0, $order = 0;
 var $parent = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n_col;
 $1 = $Col;
 $2 = $p;
 $i = 0;
 while(1) {
  $3 = $i;
  $4 = $0;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $i;
  $7 = $1;
  $8 = (($7) + (($6*24)|0)|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(-1);
  if (!($10)) {
   $11 = $i;
   $12 = $1;
   $13 = (($12) + (($11*24)|0)|0);
   $14 = ((($13)) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)==(-1);
   if ($16) {
    $17 = $i;
    $parent = $17;
    while(1) {
     $18 = $parent;
     $19 = $1;
     $20 = (($19) + (($18*24)|0)|0);
     $21 = ((($20)) + 8|0);
     $22 = HEAP32[$21>>2]|0;
     $parent = $22;
     $23 = $parent;
     $24 = $1;
     $25 = (($24) + (($23*24)|0)|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==(-1);
     $28 = $27 ^ 1;
     if (!($28)) {
      break;
     }
    }
    $29 = $i;
    $c = $29;
    $30 = $parent;
    $31 = $1;
    $32 = (($31) + (($30*24)|0)|0);
    $33 = ((($32)) + 12|0);
    $34 = HEAP32[$33>>2]|0;
    $order = $34;
    while(1) {
     $35 = $order;
     $36 = (($35) + 1)|0;
     $order = $36;
     $37 = $c;
     $38 = $1;
     $39 = (($38) + (($37*24)|0)|0);
     $40 = ((($39)) + 12|0);
     HEAP32[$40>>2] = $35;
     $41 = $parent;
     $42 = $c;
     $43 = $1;
     $44 = (($43) + (($42*24)|0)|0);
     $45 = ((($44)) + 8|0);
     HEAP32[$45>>2] = $41;
     $46 = $c;
     $47 = $1;
     $48 = (($47) + (($46*24)|0)|0);
     $49 = ((($48)) + 8|0);
     $50 = HEAP32[$49>>2]|0;
     $c = $50;
     $51 = $c;
     $52 = $1;
     $53 = (($52) + (($51*24)|0)|0);
     $54 = ((($53)) + 12|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)==(-1);
     if (!($56)) {
      break;
     }
    }
    $57 = $order;
    $58 = $parent;
    $59 = $1;
    $60 = (($59) + (($58*24)|0)|0);
    $61 = ((($60)) + 12|0);
    HEAP32[$61>>2] = $57;
   }
  }
  $62 = $i;
  $63 = (($62) + 1)|0;
  $i = $63;
 }
 $c = 0;
 while(1) {
  $64 = $c;
  $65 = $0;
  $66 = ($64|0)<($65|0);
  if (!($66)) {
   break;
  }
  $67 = $c;
  $68 = $c;
  $69 = $1;
  $70 = (($69) + (($68*24)|0)|0);
  $71 = ((($70)) + 12|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = $2;
  $74 = (($73) + ($72<<2)|0);
  HEAP32[$74>>2] = $67;
  $75 = $c;
  $76 = (($75) + 1)|0;
  $c = $76;
 }
 STACKTOP = sp;return;
}
function _clear_mark($n_row,$Row) {
 $n_row = $n_row|0;
 $Row = $Row|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n_row;
 $1 = $Row;
 $r = 0;
 while(1) {
  $2 = $r;
  $3 = $0;
  $4 = ($2|0)<($3|0);
  if (!($4)) {
   break;
  }
  $5 = $r;
  $6 = $1;
  $7 = (($6) + ($5<<4)|0);
  $8 = ((($7)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)>=(0);
  if ($10) {
   $11 = $r;
   $12 = $1;
   $13 = (($12) + ($11<<4)|0);
   $14 = ((($13)) + 12|0);
   HEAP32[$14>>2] = 0;
  }
  $15 = $r;
  $16 = (($15) + 1)|0;
  $r = $16;
 }
 STACKTOP = sp;return 1;
}
function _garbage_collection($n_row,$n_col,$Row,$Col,$A,$pfree) {
 $n_row = $n_row|0;
 $n_col = $n_col|0;
 $Row = $Row|0;
 $Col = $Col|0;
 $A = $A|0;
 $pfree = $pfree|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $j = 0, $length = 0, $pdest = 0, $psrc = 0, $r = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n_row;
 $1 = $n_col;
 $2 = $Row;
 $3 = $Col;
 $4 = $A;
 $5 = $pfree;
 $6 = $4;
 $pdest = $6;
 $c = 0;
 while(1) {
  $7 = $c;
  $8 = $1;
  $9 = ($7|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = $c;
  $11 = $3;
  $12 = (($11) + (($10*24)|0)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)>=(0);
  if ($14) {
   $15 = $c;
   $16 = $3;
   $17 = (($16) + (($15*24)|0)|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = $4;
   $20 = (($19) + ($18<<2)|0);
   $psrc = $20;
   $21 = $pdest;
   $22 = $4;
   $23 = $21;
   $24 = $22;
   $25 = (($23) - ($24))|0;
   $26 = (($25|0) / 4)&-1;
   $27 = $c;
   $28 = $3;
   $29 = (($28) + (($27*24)|0)|0);
   HEAP32[$29>>2] = $26;
   $30 = $c;
   $31 = $3;
   $32 = (($31) + (($30*24)|0)|0);
   $33 = ((($32)) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $length = $34;
   $j = 0;
   while(1) {
    $35 = $j;
    $36 = $length;
    $37 = ($35|0)<($36|0);
    if (!($37)) {
     break;
    }
    $38 = $psrc;
    $39 = ((($38)) + 4|0);
    $psrc = $39;
    $40 = HEAP32[$38>>2]|0;
    $r = $40;
    $41 = $r;
    $42 = $2;
    $43 = (($42) + ($41<<4)|0);
    $44 = ((($43)) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)>=(0);
    if ($46) {
     $47 = $r;
     $48 = $pdest;
     $49 = ((($48)) + 4|0);
     $pdest = $49;
     HEAP32[$48>>2] = $47;
    }
    $50 = $j;
    $51 = (($50) + 1)|0;
    $j = $51;
   }
   $52 = $pdest;
   $53 = $c;
   $54 = $3;
   $55 = (($54) + (($53*24)|0)|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = $4;
   $58 = (($57) + ($56<<2)|0);
   $59 = $52;
   $60 = $58;
   $61 = (($59) - ($60))|0;
   $62 = (($61|0) / 4)&-1;
   $63 = $c;
   $64 = $3;
   $65 = (($64) + (($63*24)|0)|0);
   $66 = ((($65)) + 4|0);
   HEAP32[$66>>2] = $62;
  }
  $67 = $c;
  $68 = (($67) + 1)|0;
  $c = $68;
 }
 $r = 0;
 while(1) {
  $69 = $r;
  $70 = $0;
  $71 = ($69|0)<($70|0);
  if (!($71)) {
   break;
  }
  $72 = $r;
  $73 = $2;
  $74 = (($73) + ($72<<4)|0);
  $75 = ((($74)) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = ($76|0)>=(0);
  do {
   if ($77) {
    $78 = $r;
    $79 = $2;
    $80 = (($79) + ($78<<4)|0);
    $81 = ((($80)) + 4|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = ($82|0)==(0);
    $84 = $r;
    $85 = $2;
    $86 = (($85) + ($84<<4)|0);
    if ($83) {
     $87 = ((($86)) + 12|0);
     HEAP32[$87>>2] = -1;
     break;
    } else {
     $88 = HEAP32[$86>>2]|0;
     $89 = $4;
     $90 = (($89) + ($88<<2)|0);
     $psrc = $90;
     $91 = $psrc;
     $92 = HEAP32[$91>>2]|0;
     $93 = $r;
     $94 = $2;
     $95 = (($94) + ($93<<4)|0);
     $96 = ((($95)) + 12|0);
     HEAP32[$96>>2] = $92;
     $97 = $r;
     $98 = (0 - ($97))|0;
     $99 = (($98) - 1)|0;
     $100 = $psrc;
     HEAP32[$100>>2] = $99;
     break;
    }
   }
  } while(0);
  $101 = $r;
  $102 = (($101) + 1)|0;
  $r = $102;
 }
 $103 = $pdest;
 $psrc = $103;
 while(1) {
  $104 = $psrc;
  $105 = $5;
  $106 = ($104>>>0)<($105>>>0);
  if (!($106)) {
   break;
  }
  $107 = $psrc;
  $108 = ((($107)) + 4|0);
  $psrc = $108;
  $109 = HEAP32[$107>>2]|0;
  $110 = ($109|0)<(0);
  if (!($110)) {
   continue;
  }
  $111 = $psrc;
  $112 = ((($111)) + -4|0);
  $psrc = $112;
  $113 = $psrc;
  $114 = HEAP32[$113>>2]|0;
  $115 = (0 - ($114))|0;
  $116 = (($115) - 1)|0;
  $r = $116;
  $117 = $r;
  $118 = $2;
  $119 = (($118) + ($117<<4)|0);
  $120 = ((($119)) + 12|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = $psrc;
  HEAP32[$122>>2] = $121;
  $123 = $pdest;
  $124 = $4;
  $125 = $123;
  $126 = $124;
  $127 = (($125) - ($126))|0;
  $128 = (($127|0) / 4)&-1;
  $129 = $r;
  $130 = $2;
  $131 = (($130) + ($129<<4)|0);
  HEAP32[$131>>2] = $128;
  $132 = $r;
  $133 = $2;
  $134 = (($133) + ($132<<4)|0);
  $135 = ((($134)) + 4|0);
  $136 = HEAP32[$135>>2]|0;
  $length = $136;
  $j = 0;
  while(1) {
   $137 = $j;
   $138 = $length;
   $139 = ($137|0)<($138|0);
   if (!($139)) {
    break;
   }
   $140 = $psrc;
   $141 = ((($140)) + 4|0);
   $psrc = $141;
   $142 = HEAP32[$140>>2]|0;
   $c = $142;
   $143 = $c;
   $144 = $3;
   $145 = (($144) + (($143*24)|0)|0);
   $146 = HEAP32[$145>>2]|0;
   $147 = ($146|0)>=(0);
   if ($147) {
    $148 = $c;
    $149 = $pdest;
    $150 = ((($149)) + 4|0);
    $pdest = $150;
    HEAP32[$149>>2] = $148;
   }
   $151 = $j;
   $152 = (($151) + 1)|0;
   $j = $152;
  }
  $153 = $pdest;
  $154 = $r;
  $155 = $2;
  $156 = (($155) + ($154<<4)|0);
  $157 = HEAP32[$156>>2]|0;
  $158 = $4;
  $159 = (($158) + ($157<<2)|0);
  $160 = $153;
  $161 = $159;
  $162 = (($160) - ($161))|0;
  $163 = (($162|0) / 4)&-1;
  $164 = $r;
  $165 = $2;
  $166 = (($165) + ($164<<4)|0);
  $167 = ((($166)) + 4|0);
  HEAP32[$167>>2] = $163;
 }
 $168 = $pdest;
 $169 = $4;
 $170 = $168;
 $171 = $169;
 $172 = (($170) - ($171))|0;
 $173 = (($172|0) / 4)&-1;
 STACKTOP = sp;return ($173|0);
}
function _detect_super_cols($Col,$A,$head,$row_start,$row_length) {
 $Col = $Col|0;
 $A = $A|0;
 $head = $head|0;
 $row_start = $row_start|0;
 $row_length = $row_length|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $col = 0, $cp1 = 0, $cp2 = 0, $first_col = 0, $hash = 0, $head_column = 0;
 var $i = 0, $length = 0, $prev_c = 0, $rp = 0, $rp_end = 0, $super_c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $Col;
 $1 = $A;
 $2 = $head;
 $3 = $row_start;
 $4 = $row_length;
 $5 = $3;
 $6 = $1;
 $7 = (($6) + ($5<<2)|0);
 $rp = $7;
 $8 = $rp;
 $9 = $4;
 $10 = (($8) + ($9<<2)|0);
 $rp_end = $10;
 while(1) {
  $11 = $rp;
  $12 = $rp_end;
  $13 = ($11>>>0)<($12>>>0);
  if (!($13)) {
   break;
  }
  $14 = $rp;
  $15 = ((($14)) + 4|0);
  $rp = $15;
  $16 = HEAP32[$14>>2]|0;
  $col = $16;
  $17 = $col;
  $18 = $0;
  $19 = (($18) + (($17*24)|0)|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)<(0);
  if ($21) {
   continue;
  }
  $22 = $col;
  $23 = $0;
  $24 = (($23) + (($22*24)|0)|0);
  $25 = ((($24)) + 16|0);
  $26 = HEAP32[$25>>2]|0;
  $hash = $26;
  $27 = $hash;
  $28 = $2;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $head_column = $30;
  $31 = $head_column;
  $32 = ($31|0)>(-1);
  $33 = $head_column;
  if ($32) {
   $34 = $0;
   $35 = (($34) + (($33*24)|0)|0);
   $36 = ((($35)) + 16|0);
   $37 = HEAP32[$36>>2]|0;
   $first_col = $37;
  } else {
   $38 = (($33) + 2)|0;
   $39 = (0 - ($38))|0;
   $first_col = $39;
  }
  $40 = $first_col;
  $super_c = $40;
  while(1) {
   $41 = $super_c;
   $42 = ($41|0)!=(-1);
   if (!($42)) {
    break;
   }
   $43 = $super_c;
   $44 = $0;
   $45 = (($44) + (($43*24)|0)|0);
   $46 = ((($45)) + 4|0);
   $47 = HEAP32[$46>>2]|0;
   $length = $47;
   $48 = $super_c;
   $prev_c = $48;
   $49 = $super_c;
   $50 = $0;
   $51 = (($50) + (($49*24)|0)|0);
   $52 = ((($51)) + 20|0);
   $53 = HEAP32[$52>>2]|0;
   $c = $53;
   while(1) {
    $54 = $c;
    $55 = ($54|0)!=(-1);
    if (!($55)) {
     break;
    }
    $56 = $c;
    $57 = $0;
    $58 = (($57) + (($56*24)|0)|0);
    $59 = ((($58)) + 4|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = $length;
    $62 = ($60|0)!=($61|0);
    do {
     if ($62) {
      label = 13;
     } else {
      $63 = $c;
      $64 = $0;
      $65 = (($64) + (($63*24)|0)|0);
      $66 = ((($65)) + 12|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = $super_c;
      $69 = $0;
      $70 = (($69) + (($68*24)|0)|0);
      $71 = ((($70)) + 12|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = ($67|0)!=($72|0);
      if ($73) {
       label = 13;
      } else {
       $75 = $super_c;
       $76 = $0;
       $77 = (($76) + (($75*24)|0)|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = $1;
       $80 = (($79) + ($78<<2)|0);
       $cp1 = $80;
       $81 = $c;
       $82 = $0;
       $83 = (($82) + (($81*24)|0)|0);
       $84 = HEAP32[$83>>2]|0;
       $85 = $1;
       $86 = (($85) + ($84<<2)|0);
       $cp2 = $86;
       $i = 0;
       while(1) {
        $87 = $i;
        $88 = $length;
        $89 = ($87|0)<($88|0);
        if (!($89)) {
         break;
        }
        $90 = $cp1;
        $91 = ((($90)) + 4|0);
        $cp1 = $91;
        $92 = HEAP32[$90>>2]|0;
        $93 = $cp2;
        $94 = ((($93)) + 4|0);
        $cp2 = $94;
        $95 = HEAP32[$93>>2]|0;
        $96 = ($92|0)!=($95|0);
        if ($96) {
         break;
        }
        $97 = $i;
        $98 = (($97) + 1)|0;
        $i = $98;
       }
       $99 = $i;
       $100 = $length;
       $101 = ($99|0)!=($100|0);
       $102 = $c;
       if ($101) {
        $prev_c = $102;
        break;
       } else {
        $103 = $0;
        $104 = (($103) + (($102*24)|0)|0);
        $105 = ((($104)) + 8|0);
        $106 = HEAP32[$105>>2]|0;
        $107 = $super_c;
        $108 = $0;
        $109 = (($108) + (($107*24)|0)|0);
        $110 = ((($109)) + 8|0);
        $111 = HEAP32[$110>>2]|0;
        $112 = (($111) + ($106))|0;
        HEAP32[$110>>2] = $112;
        $113 = $super_c;
        $114 = $c;
        $115 = $0;
        $116 = (($115) + (($114*24)|0)|0);
        $117 = ((($116)) + 8|0);
        HEAP32[$117>>2] = $113;
        $118 = $c;
        $119 = $0;
        $120 = (($119) + (($118*24)|0)|0);
        HEAP32[$120>>2] = -2;
        $121 = $c;
        $122 = $0;
        $123 = (($122) + (($121*24)|0)|0);
        $124 = ((($123)) + 12|0);
        HEAP32[$124>>2] = -1;
        $125 = $c;
        $126 = $0;
        $127 = (($126) + (($125*24)|0)|0);
        $128 = ((($127)) + 20|0);
        $129 = HEAP32[$128>>2]|0;
        $130 = $prev_c;
        $131 = $0;
        $132 = (($131) + (($130*24)|0)|0);
        $133 = ((($132)) + 20|0);
        HEAP32[$133>>2] = $129;
        break;
       }
      }
     }
    } while(0);
    if ((label|0) == 13) {
     label = 0;
     $74 = $c;
     $prev_c = $74;
    }
    $134 = $c;
    $135 = $0;
    $136 = (($135) + (($134*24)|0)|0);
    $137 = ((($136)) + 20|0);
    $138 = HEAP32[$137>>2]|0;
    $c = $138;
   }
   $139 = $super_c;
   $140 = $0;
   $141 = (($140) + (($139*24)|0)|0);
   $142 = ((($141)) + 20|0);
   $143 = HEAP32[$142>>2]|0;
   $super_c = $143;
  }
  $144 = $head_column;
  $145 = ($144|0)>(-1);
  if ($145) {
   $146 = $head_column;
   $147 = $0;
   $148 = (($147) + (($146*24)|0)|0);
   $149 = ((($148)) + 16|0);
   HEAP32[$149>>2] = -1;
   continue;
  } else {
   $150 = $hash;
   $151 = $2;
   $152 = (($151) + ($150<<2)|0);
   HEAP32[$152>>2] = -1;
   continue;
  }
 }
 STACKTOP = sp;return;
}
function _lsame_($ca,$cb) {
 $ca = $ca|0;
 $cb = $cb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $inta = 0, $intb = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond19 = 0, $or$cond21 = 0, $or$cond23 = 0;
 var $or$cond25 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $ret_val = 0, $zcode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $ca;
 $2 = $cb;
 $3 = $1;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $2;
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($5|0)==($8|0);
 $10 = $9&1;
 $ret_val = $10;
 $11 = $ret_val;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = $ret_val;
  $0 = $13;
  $89 = $0;
  STACKTOP = sp;return ($89|0);
 }
 $zcode = 90;
 $14 = $1;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $inta = $16;
 $17 = $2;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $intb = $19;
 $20 = $zcode;
 $21 = ($20|0)==(90);
 $22 = $zcode;
 $23 = ($22|0)==(122);
 $or$cond = $21 | $23;
 do {
  if ($or$cond) {
   $24 = $inta;
   $25 = ($24|0)>=(97);
   $26 = $inta;
   $27 = ($26|0)<=(122);
   $or$cond3 = $25 & $27;
   if ($or$cond3) {
    $28 = $inta;
    $29 = (($28) + -32)|0;
    $inta = $29;
   }
   $30 = $intb;
   $31 = ($30|0)>=(97);
   $32 = $intb;
   $33 = ($32|0)<=(122);
   $or$cond5 = $31 & $33;
   if ($or$cond5) {
    $34 = $intb;
    $35 = (($34) + -32)|0;
    $intb = $35;
   }
  } else {
   $36 = $zcode;
   $37 = ($36|0)==(233);
   $38 = $zcode;
   $39 = ($38|0)==(169);
   $or$cond7 = $37 | $39;
   if (!($or$cond7)) {
    $68 = $zcode;
    $69 = ($68|0)==(218);
    $70 = $zcode;
    $71 = ($70|0)==(250);
    $or$cond21 = $69 | $71;
    if (!($or$cond21)) {
     break;
    }
    $72 = $inta;
    $73 = ($72|0)>=(225);
    $74 = $inta;
    $75 = ($74|0)<=(250);
    $or$cond23 = $73 & $75;
    if ($or$cond23) {
     $76 = $inta;
     $77 = (($76) + -32)|0;
     $inta = $77;
    }
    $78 = $intb;
    $79 = ($78|0)>=(225);
    $80 = $intb;
    $81 = ($80|0)<=(250);
    $or$cond25 = $79 & $81;
    if (!($or$cond25)) {
     break;
    }
    $82 = $intb;
    $83 = (($82) + -32)|0;
    $intb = $83;
    break;
   }
   $40 = $inta;
   $41 = ($40|0)>=(129);
   $42 = $inta;
   $43 = ($42|0)<=(137);
   $or$cond9 = $41 & $43;
   if ($or$cond9) {
    label = 12;
   } else {
    $44 = $inta;
    $45 = ($44|0)>=(145);
    $46 = $inta;
    $47 = ($46|0)<=(153);
    $or$cond11 = $45 & $47;
    if ($or$cond11) {
     label = 12;
    } else {
     $48 = $inta;
     $49 = ($48|0)>=(162);
     $50 = $inta;
     $51 = ($50|0)<=(169);
     $or$cond13 = $49 & $51;
     if ($or$cond13) {
      label = 12;
     }
    }
   }
   if ((label|0) == 12) {
    $52 = $inta;
    $53 = (($52) + 64)|0;
    $inta = $53;
   }
   $54 = $intb;
   $55 = ($54|0)>=(129);
   $56 = $intb;
   $57 = ($56|0)<=(137);
   $or$cond15 = $55 & $57;
   if (!($or$cond15)) {
    $58 = $intb;
    $59 = ($58|0)>=(145);
    $60 = $intb;
    $61 = ($60|0)<=(153);
    $or$cond17 = $59 & $61;
    if (!($or$cond17)) {
     $62 = $intb;
     $63 = ($62|0)>=(162);
     $64 = $intb;
     $65 = ($64|0)<=(169);
     $or$cond19 = $63 & $65;
     if (!($or$cond19)) {
      break;
     }
    }
   }
   $66 = $intb;
   $67 = (($66) + 64)|0;
   $intb = $67;
  }
 } while(0);
 $84 = $inta;
 $85 = $intb;
 $86 = ($84|0)==($85|0);
 $87 = $86&1;
 $ret_val = $87;
 $88 = $ret_val;
 $0 = $88;
 $89 = $0;
 STACKTOP = sp;return ($89|0);
}
function _xerbla_($srname,$info) {
 $srname = $srname|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $0 = $srname;
 $1 = $info;
 $2 = $0;
 $3 = $1;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $2;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $4;
 (_printf((1560|0),($vararg_buffer|0))|0);
 STACKTOP = sp;return 0;
}
function _sp_dtrsv($uplo,$trans,$diag,$L,$U,$x,$stat,$info) {
 $uplo = $uplo|0;
 $trans = $trans|0;
 $diag = $diag|0;
 $L = $L|0;
 $U = $U|0;
 $x = $x|0;
 $stat = $stat|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0.0, $147 = 0.0, $148 = 0.0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0.0, $174 = 0, $175 = 0, $176 = 0, $177 = 0.0, $178 = 0.0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0.0, $183 = 0.0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0.0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0.0, $228 = 0.0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0.0, $287 = 0.0, $288 = 0.0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0.0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0.0, $325 = 0, $326 = 0, $327 = 0, $328 = 0.0, $329 = 0.0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0.0, $334 = 0.0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0.0, $371 = 0.0, $372 = 0.0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0.0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0.0, $402 = 0.0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0.0, $407 = 0.0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0.0, $466 = 0.0, $467 = 0.0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0.0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0.0;
 var $512 = 0.0, $513 = 0, $514 = 0, $515 = 0, $516 = 0.0, $517 = 0.0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0.0, $531 = 0.0, $532 = 0.0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0.0, $611 = 0.0, $612 = 0.0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0.0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0.0, $642 = 0.0, $643 = 0, $644 = 0, $645 = 0, $646 = 0.0, $647 = 0.0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0.0;
 var $657 = 0.0, $658 = 0.0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0.0, $665 = 0, $666 = 0, $667 = 0, $668 = 0.0, $669 = 0.0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0.0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0.0, $681 = 0.0, $682 = 0, $683 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Lstore = 0, $Lval = 0, $Ustore = 0, $Uval = 0, $alpha = 0.0, $beta = 0.0, $fsupc = 0, $i = 0, $incx = 0, $incy = 0, $iptr = 0, $irow = 0, $istart = 0, $jcol = 0, $k = 0, $luptr = 0;
 var $msg = 0, $nrow = 0, $nsupc = 0, $nsupr = 0, $solve_ops = 0.0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $work = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 400|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp + 16|0;
 $incx = sp + 84|0;
 $nsupr = sp + 68|0;
 $nsupc = sp + 64|0;
 $i = sp + 48|0;
 $msg = sp + 144|0;
 $1 = $uplo;
 $2 = $trans;
 $3 = $diag;
 $4 = $L;
 $5 = $U;
 $6 = $x;
 $7 = $stat;
 $8 = $info;
 HEAP32[$incx>>2] = 1;
 $incy = 1;
 $alpha = 1.0;
 $beta = 1.0;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 $10 = $1;
 $11 = (_lsame_($10,3400)|0);
 $12 = ($11|0)!=(0);
 if ($12) {
  label = 4;
 } else {
  $13 = $1;
  $14 = (_lsame_($13,3392)|0);
  $15 = ($14|0)!=(0);
  if ($15) {
   label = 4;
  } else {
   $16 = $8;
   HEAP32[$16>>2] = -1;
  }
 }
 do {
  if ((label|0) == 4) {
   $17 = $2;
   $18 = (_lsame_($17,3408)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = $2;
    $21 = (_lsame_($20,3416)|0);
    $22 = ($21|0)!=(0);
    if (!($22)) {
     $23 = $2;
     $24 = (_lsame_($23,3424)|0);
     $25 = ($24|0)!=(0);
     if (!($25)) {
      $26 = $8;
      HEAP32[$26>>2] = -2;
      break;
     }
    }
   }
   $27 = $3;
   $28 = (_lsame_($27,3392)|0);
   $29 = ($28|0)!=(0);
   if (!($29)) {
    $30 = $3;
    $31 = (_lsame_($30,3408)|0);
    $32 = ($31|0)!=(0);
    if (!($32)) {
     $33 = $8;
     HEAP32[$33>>2] = -3;
     break;
    }
   }
   $34 = $4;
   $35 = ((($34)) + 12|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = $4;
   $38 = ((($37)) + 16|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($36|0)!=($39|0);
   if (!($40)) {
    $41 = $4;
    $42 = ((($41)) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)<(0);
    if (!($44)) {
     $46 = $5;
     $47 = ((($46)) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = $5;
     $50 = ((($49)) + 16|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = ($48|0)!=($51|0);
     if (!($52)) {
      $53 = $5;
      $54 = ((($53)) + 12|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = ($55|0)<(0);
      if (!($56)) {
       break;
      }
     }
     $57 = $8;
     HEAP32[$57>>2] = -5;
     break;
    }
   }
   $45 = $8;
   HEAP32[$45>>2] = -4;
  }
 } while(0);
 $58 = $8;
 $59 = HEAP32[$58>>2]|0;
 $60 = ($59|0)!=(0);
 if ($60) {
  $61 = $8;
  $62 = HEAP32[$61>>2]|0;
  $63 = (0 - ($62))|0;
  HEAP32[$i>>2] = $63;
  (_xerbla_(1624,$i)|0);
  $0 = 0;
  $683 = $0;
  STACKTOP = sp;return ($683|0);
 }
 $64 = $4;
 $65 = ((($64)) + 20|0);
 $66 = HEAP32[$65>>2]|0;
 $Lstore = $66;
 $67 = $Lstore;
 $68 = ((($67)) + 8|0);
 $69 = HEAP32[$68>>2]|0;
 $Lval = $69;
 $70 = $5;
 $71 = ((($70)) + 20|0);
 $72 = HEAP32[$71>>2]|0;
 $Ustore = $72;
 $73 = $Ustore;
 $74 = ((($73)) + 4|0);
 $75 = HEAP32[$74>>2]|0;
 $Uval = $75;
 $solve_ops = 0.0;
 $76 = $4;
 $77 = ((($76)) + 12|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = (_doubleCalloc($78)|0);
 $work = $79;
 $80 = ($79|0)!=(0|0);
 if (!($80)) {
  HEAP32[$vararg_buffer>>2] = 1640;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 123;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 1680;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $81 = $2;
 $82 = (_lsame_($81,3408)|0);
 $83 = ($82|0)!=(0);
 $84 = $1;
 $85 = (_lsame_($84,3400)|0);
 $86 = ($85|0)!=(0);
 L30: do {
  if ($83) {
   if ($86) {
    $87 = $4;
    $88 = ((($87)) + 12|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = ($89|0)==(0);
    if ($90) {
     $0 = 0;
     $683 = $0;
     STACKTOP = sp;return ($683|0);
    }
    $k = 0;
    while(1) {
     $91 = $k;
     $92 = $Lstore;
     $93 = ((($92)) + 4|0);
     $94 = HEAP32[$93>>2]|0;
     $95 = ($91|0)<=($94|0);
     if (!($95)) {
      break L30;
     }
     $96 = $k;
     $97 = $Lstore;
     $98 = ((($97)) + 28|0);
     $99 = HEAP32[$98>>2]|0;
     $100 = (($99) + ($96<<2)|0);
     $101 = HEAP32[$100>>2]|0;
     $fsupc = $101;
     $102 = $fsupc;
     $103 = $Lstore;
     $104 = ((($103)) + 20|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (($105) + ($102<<2)|0);
     $107 = HEAP32[$106>>2]|0;
     $istart = $107;
     $108 = $fsupc;
     $109 = (($108) + 1)|0;
     $110 = $Lstore;
     $111 = ((($110)) + 20|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = (($112) + ($109<<2)|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = $istart;
     $116 = (($114) - ($115))|0;
     HEAP32[$nsupr>>2] = $116;
     $117 = $k;
     $118 = (($117) + 1)|0;
     $119 = $Lstore;
     $120 = ((($119)) + 28|0);
     $121 = HEAP32[$120>>2]|0;
     $122 = (($121) + ($118<<2)|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = $fsupc;
     $125 = (($123) - ($124))|0;
     HEAP32[$nsupc>>2] = $125;
     $126 = $fsupc;
     $127 = $Lstore;
     $128 = ((($127)) + 12|0);
     $129 = HEAP32[$128>>2]|0;
     $130 = (($129) + ($126<<2)|0);
     $131 = HEAP32[$130>>2]|0;
     $luptr = $131;
     $132 = HEAP32[$nsupr>>2]|0;
     $133 = HEAP32[$nsupc>>2]|0;
     $134 = (($132) - ($133))|0;
     $nrow = $134;
     $135 = HEAP32[$nsupc>>2]|0;
     $136 = HEAP32[$nsupc>>2]|0;
     $137 = (($136) - 1)|0;
     $138 = Math_imul($135, $137)|0;
     $139 = (+($138|0));
     $140 = $solve_ops;
     $141 = $140 + $139;
     $solve_ops = $141;
     $142 = $nrow;
     $143 = $142<<1;
     $144 = HEAP32[$nsupc>>2]|0;
     $145 = Math_imul($143, $144)|0;
     $146 = (+($145|0));
     $147 = $solve_ops;
     $148 = $147 + $146;
     $solve_ops = $148;
     $149 = HEAP32[$nsupc>>2]|0;
     $150 = ($149|0)==(1);
     L82: do {
      if ($150) {
       $151 = $istart;
       $152 = (($151) + 1)|0;
       $iptr = $152;
       while(1) {
        $153 = $iptr;
        $154 = $fsupc;
        $155 = (($154) + 1)|0;
        $156 = $Lstore;
        $157 = ((($156)) + 20|0);
        $158 = HEAP32[$157>>2]|0;
        $159 = (($158) + ($155<<2)|0);
        $160 = HEAP32[$159>>2]|0;
        $161 = ($153|0)<($160|0);
        if (!($161)) {
         break L82;
        }
        $162 = $iptr;
        $163 = $Lstore;
        $164 = ((($163)) + 16|0);
        $165 = HEAP32[$164>>2]|0;
        $166 = (($165) + ($162<<2)|0);
        $167 = HEAP32[$166>>2]|0;
        $irow = $167;
        $168 = $luptr;
        $169 = (($168) + 1)|0;
        $luptr = $169;
        $170 = $fsupc;
        $171 = $6;
        $172 = (($171) + ($170<<3)|0);
        $173 = +HEAPF64[$172>>3];
        $174 = $luptr;
        $175 = $Lval;
        $176 = (($175) + ($174<<3)|0);
        $177 = +HEAPF64[$176>>3];
        $178 = $173 * $177;
        $179 = $irow;
        $180 = $6;
        $181 = (($180) + ($179<<3)|0);
        $182 = +HEAPF64[$181>>3];
        $183 = $182 - $178;
        HEAPF64[$181>>3] = $183;
        $184 = $iptr;
        $185 = (($184) + 1)|0;
        $iptr = $185;
       }
      } else {
       $186 = HEAP32[$nsupr>>2]|0;
       $187 = HEAP32[$nsupc>>2]|0;
       $188 = $luptr;
       $189 = $Lval;
       $190 = (($189) + ($188<<3)|0);
       $191 = $fsupc;
       $192 = $6;
       $193 = (($192) + ($191<<3)|0);
       _dlsolve($186,$187,$190,$193);
       $194 = HEAP32[$nsupr>>2]|0;
       $195 = HEAP32[$nsupr>>2]|0;
       $196 = HEAP32[$nsupc>>2]|0;
       $197 = (($195) - ($196))|0;
       $198 = HEAP32[$nsupc>>2]|0;
       $199 = $luptr;
       $200 = HEAP32[$nsupc>>2]|0;
       $201 = (($199) + ($200))|0;
       $202 = $Lval;
       $203 = (($202) + ($201<<3)|0);
       $204 = $fsupc;
       $205 = $6;
       $206 = (($205) + ($204<<3)|0);
       $207 = $work;
       _dmatvec($194,$197,$198,$203,$206,$207);
       $208 = $istart;
       $209 = HEAP32[$nsupc>>2]|0;
       $210 = (($208) + ($209))|0;
       $iptr = $210;
       HEAP32[$i>>2] = 0;
       while(1) {
        $211 = HEAP32[$i>>2]|0;
        $212 = $nrow;
        $213 = ($211|0)<($212|0);
        if (!($213)) {
         break L82;
        }
        $214 = $iptr;
        $215 = $Lstore;
        $216 = ((($215)) + 16|0);
        $217 = HEAP32[$216>>2]|0;
        $218 = (($217) + ($214<<2)|0);
        $219 = HEAP32[$218>>2]|0;
        $irow = $219;
        $220 = HEAP32[$i>>2]|0;
        $221 = $work;
        $222 = (($221) + ($220<<3)|0);
        $223 = +HEAPF64[$222>>3];
        $224 = $irow;
        $225 = $6;
        $226 = (($225) + ($224<<3)|0);
        $227 = +HEAPF64[$226>>3];
        $228 = $227 - $223;
        HEAPF64[$226>>3] = $228;
        $229 = HEAP32[$i>>2]|0;
        $230 = $work;
        $231 = (($230) + ($229<<3)|0);
        HEAPF64[$231>>3] = 0.0;
        $232 = HEAP32[$i>>2]|0;
        $233 = (($232) + 1)|0;
        HEAP32[$i>>2] = $233;
        $234 = $iptr;
        $235 = (($234) + 1)|0;
        $iptr = $235;
       }
      }
     } while(0);
     $236 = $k;
     $237 = (($236) + 1)|0;
     $k = $237;
    }
   }
   $238 = $5;
   $239 = ((($238)) + 12|0);
   $240 = HEAP32[$239>>2]|0;
   $241 = ($240|0)==(0);
   if ($241) {
    $0 = 0;
    $683 = $0;
    STACKTOP = sp;return ($683|0);
   }
   $242 = $Lstore;
   $243 = ((($242)) + 4|0);
   $244 = HEAP32[$243>>2]|0;
   $k = $244;
   while(1) {
    $245 = $k;
    $246 = ($245|0)>=(0);
    if (!($246)) {
     break L30;
    }
    $247 = $k;
    $248 = $Lstore;
    $249 = ((($248)) + 28|0);
    $250 = HEAP32[$249>>2]|0;
    $251 = (($250) + ($247<<2)|0);
    $252 = HEAP32[$251>>2]|0;
    $fsupc = $252;
    $253 = $fsupc;
    $254 = (($253) + 1)|0;
    $255 = $Lstore;
    $256 = ((($255)) + 20|0);
    $257 = HEAP32[$256>>2]|0;
    $258 = (($257) + ($254<<2)|0);
    $259 = HEAP32[$258>>2]|0;
    $260 = $fsupc;
    $261 = $Lstore;
    $262 = ((($261)) + 20|0);
    $263 = HEAP32[$262>>2]|0;
    $264 = (($263) + ($260<<2)|0);
    $265 = HEAP32[$264>>2]|0;
    $266 = (($259) - ($265))|0;
    HEAP32[$nsupr>>2] = $266;
    $267 = $k;
    $268 = (($267) + 1)|0;
    $269 = $Lstore;
    $270 = ((($269)) + 28|0);
    $271 = HEAP32[$270>>2]|0;
    $272 = (($271) + ($268<<2)|0);
    $273 = HEAP32[$272>>2]|0;
    $274 = $fsupc;
    $275 = (($273) - ($274))|0;
    HEAP32[$nsupc>>2] = $275;
    $276 = $fsupc;
    $277 = $Lstore;
    $278 = ((($277)) + 12|0);
    $279 = HEAP32[$278>>2]|0;
    $280 = (($279) + ($276<<2)|0);
    $281 = HEAP32[$280>>2]|0;
    $luptr = $281;
    $282 = HEAP32[$nsupc>>2]|0;
    $283 = HEAP32[$nsupc>>2]|0;
    $284 = (($283) + 1)|0;
    $285 = Math_imul($282, $284)|0;
    $286 = (+($285|0));
    $287 = $solve_ops;
    $288 = $287 + $286;
    $solve_ops = $288;
    $289 = HEAP32[$nsupc>>2]|0;
    $290 = ($289|0)==(1);
    L100: do {
     if ($290) {
      $291 = $luptr;
      $292 = $Lval;
      $293 = (($292) + ($291<<3)|0);
      $294 = +HEAPF64[$293>>3];
      $295 = $fsupc;
      $296 = $6;
      $297 = (($296) + ($295<<3)|0);
      $298 = +HEAPF64[$297>>3];
      $299 = $298 / $294;
      HEAPF64[$297>>3] = $299;
      $300 = $fsupc;
      $301 = $Ustore;
      $302 = ((($301)) + 12|0);
      $303 = HEAP32[$302>>2]|0;
      $304 = (($303) + ($300<<2)|0);
      $305 = HEAP32[$304>>2]|0;
      HEAP32[$i>>2] = $305;
      while(1) {
       $306 = HEAP32[$i>>2]|0;
       $307 = $fsupc;
       $308 = (($307) + 1)|0;
       $309 = $Ustore;
       $310 = ((($309)) + 12|0);
       $311 = HEAP32[$310>>2]|0;
       $312 = (($311) + ($308<<2)|0);
       $313 = HEAP32[$312>>2]|0;
       $314 = ($306|0)<($313|0);
       if (!($314)) {
        break L100;
       }
       $315 = HEAP32[$i>>2]|0;
       $316 = $Ustore;
       $317 = ((($316)) + 8|0);
       $318 = HEAP32[$317>>2]|0;
       $319 = (($318) + ($315<<2)|0);
       $320 = HEAP32[$319>>2]|0;
       $irow = $320;
       $321 = $fsupc;
       $322 = $6;
       $323 = (($322) + ($321<<3)|0);
       $324 = +HEAPF64[$323>>3];
       $325 = HEAP32[$i>>2]|0;
       $326 = $Uval;
       $327 = (($326) + ($325<<3)|0);
       $328 = +HEAPF64[$327>>3];
       $329 = $324 * $328;
       $330 = $irow;
       $331 = $6;
       $332 = (($331) + ($330<<3)|0);
       $333 = +HEAPF64[$332>>3];
       $334 = $333 - $329;
       HEAPF64[$332>>3] = $334;
       $335 = HEAP32[$i>>2]|0;
       $336 = (($335) + 1)|0;
       HEAP32[$i>>2] = $336;
      }
     } else {
      $337 = HEAP32[$nsupr>>2]|0;
      $338 = HEAP32[$nsupc>>2]|0;
      $339 = $luptr;
      $340 = $Lval;
      $341 = (($340) + ($339<<3)|0);
      $342 = $fsupc;
      $343 = $6;
      $344 = (($343) + ($342<<3)|0);
      _dusolve($337,$338,$341,$344);
      $345 = $fsupc;
      $jcol = $345;
      while(1) {
       $346 = $jcol;
       $347 = $k;
       $348 = (($347) + 1)|0;
       $349 = $Lstore;
       $350 = ((($349)) + 28|0);
       $351 = HEAP32[$350>>2]|0;
       $352 = (($351) + ($348<<2)|0);
       $353 = HEAP32[$352>>2]|0;
       $354 = ($346|0)<($353|0);
       if (!($354)) {
        break L100;
       }
       $355 = $jcol;
       $356 = (($355) + 1)|0;
       $357 = $Ustore;
       $358 = ((($357)) + 12|0);
       $359 = HEAP32[$358>>2]|0;
       $360 = (($359) + ($356<<2)|0);
       $361 = HEAP32[$360>>2]|0;
       $362 = $jcol;
       $363 = $Ustore;
       $364 = ((($363)) + 12|0);
       $365 = HEAP32[$364>>2]|0;
       $366 = (($365) + ($362<<2)|0);
       $367 = HEAP32[$366>>2]|0;
       $368 = (($361) - ($367))|0;
       $369 = $368<<1;
       $370 = (+($369|0));
       $371 = $solve_ops;
       $372 = $371 + $370;
       $solve_ops = $372;
       $373 = $jcol;
       $374 = $Ustore;
       $375 = ((($374)) + 12|0);
       $376 = HEAP32[$375>>2]|0;
       $377 = (($376) + ($373<<2)|0);
       $378 = HEAP32[$377>>2]|0;
       HEAP32[$i>>2] = $378;
       while(1) {
        $379 = HEAP32[$i>>2]|0;
        $380 = $jcol;
        $381 = (($380) + 1)|0;
        $382 = $Ustore;
        $383 = ((($382)) + 12|0);
        $384 = HEAP32[$383>>2]|0;
        $385 = (($384) + ($381<<2)|0);
        $386 = HEAP32[$385>>2]|0;
        $387 = ($379|0)<($386|0);
        if (!($387)) {
         break;
        }
        $388 = HEAP32[$i>>2]|0;
        $389 = $Ustore;
        $390 = ((($389)) + 8|0);
        $391 = HEAP32[$390>>2]|0;
        $392 = (($391) + ($388<<2)|0);
        $393 = HEAP32[$392>>2]|0;
        $irow = $393;
        $394 = $jcol;
        $395 = $6;
        $396 = (($395) + ($394<<3)|0);
        $397 = +HEAPF64[$396>>3];
        $398 = HEAP32[$i>>2]|0;
        $399 = $Uval;
        $400 = (($399) + ($398<<3)|0);
        $401 = +HEAPF64[$400>>3];
        $402 = $397 * $401;
        $403 = $irow;
        $404 = $6;
        $405 = (($404) + ($403<<3)|0);
        $406 = +HEAPF64[$405>>3];
        $407 = $406 - $402;
        HEAPF64[$405>>3] = $407;
        $408 = HEAP32[$i>>2]|0;
        $409 = (($408) + 1)|0;
        HEAP32[$i>>2] = $409;
       }
       $410 = $jcol;
       $411 = (($410) + 1)|0;
       $jcol = $411;
      }
     }
    } while(0);
    $412 = $k;
    $413 = (($412) + -1)|0;
    $k = $413;
   }
  } else {
   if ($86) {
    $414 = $4;
    $415 = ((($414)) + 12|0);
    $416 = HEAP32[$415>>2]|0;
    $417 = ($416|0)==(0);
    if ($417) {
     $0 = 0;
     $683 = $0;
     STACKTOP = sp;return ($683|0);
    }
    $418 = $Lstore;
    $419 = ((($418)) + 4|0);
    $420 = HEAP32[$419>>2]|0;
    $k = $420;
    while(1) {
     $421 = $k;
     $422 = ($421|0)>=(0);
     if (!($422)) {
      break L30;
     }
     $423 = $k;
     $424 = $Lstore;
     $425 = ((($424)) + 28|0);
     $426 = HEAP32[$425>>2]|0;
     $427 = (($426) + ($423<<2)|0);
     $428 = HEAP32[$427>>2]|0;
     $fsupc = $428;
     $429 = $fsupc;
     $430 = $Lstore;
     $431 = ((($430)) + 20|0);
     $432 = HEAP32[$431>>2]|0;
     $433 = (($432) + ($429<<2)|0);
     $434 = HEAP32[$433>>2]|0;
     $istart = $434;
     $435 = $fsupc;
     $436 = (($435) + 1)|0;
     $437 = $Lstore;
     $438 = ((($437)) + 20|0);
     $439 = HEAP32[$438>>2]|0;
     $440 = (($439) + ($436<<2)|0);
     $441 = HEAP32[$440>>2]|0;
     $442 = $istart;
     $443 = (($441) - ($442))|0;
     HEAP32[$nsupr>>2] = $443;
     $444 = $k;
     $445 = (($444) + 1)|0;
     $446 = $Lstore;
     $447 = ((($446)) + 28|0);
     $448 = HEAP32[$447>>2]|0;
     $449 = (($448) + ($445<<2)|0);
     $450 = HEAP32[$449>>2]|0;
     $451 = $fsupc;
     $452 = (($450) - ($451))|0;
     HEAP32[$nsupc>>2] = $452;
     $453 = $fsupc;
     $454 = $Lstore;
     $455 = ((($454)) + 12|0);
     $456 = HEAP32[$455>>2]|0;
     $457 = (($456) + ($453<<2)|0);
     $458 = HEAP32[$457>>2]|0;
     $luptr = $458;
     $459 = HEAP32[$nsupr>>2]|0;
     $460 = HEAP32[$nsupc>>2]|0;
     $461 = (($459) - ($460))|0;
     $462 = $461<<1;
     $463 = HEAP32[$nsupc>>2]|0;
     $464 = Math_imul($462, $463)|0;
     $465 = (+($464|0));
     $466 = $solve_ops;
     $467 = $466 + $465;
     $solve_ops = $467;
     $468 = $fsupc;
     $jcol = $468;
     while(1) {
      $469 = $jcol;
      $470 = $k;
      $471 = (($470) + 1)|0;
      $472 = $Lstore;
      $473 = ((($472)) + 28|0);
      $474 = HEAP32[$473>>2]|0;
      $475 = (($474) + ($471<<2)|0);
      $476 = HEAP32[$475>>2]|0;
      $477 = ($469|0)<($476|0);
      if (!($477)) {
       break;
      }
      $478 = $istart;
      $479 = HEAP32[$nsupc>>2]|0;
      $480 = (($478) + ($479))|0;
      $iptr = $480;
      $481 = $jcol;
      $482 = $Lstore;
      $483 = ((($482)) + 12|0);
      $484 = HEAP32[$483>>2]|0;
      $485 = (($484) + ($481<<2)|0);
      $486 = HEAP32[$485>>2]|0;
      $487 = HEAP32[$nsupc>>2]|0;
      $488 = (($486) + ($487))|0;
      HEAP32[$i>>2] = $488;
      while(1) {
       $489 = HEAP32[$i>>2]|0;
       $490 = $jcol;
       $491 = (($490) + 1)|0;
       $492 = $Lstore;
       $493 = ((($492)) + 12|0);
       $494 = HEAP32[$493>>2]|0;
       $495 = (($494) + ($491<<2)|0);
       $496 = HEAP32[$495>>2]|0;
       $497 = ($489|0)<($496|0);
       if (!($497)) {
        break;
       }
       $498 = $iptr;
       $499 = $Lstore;
       $500 = ((($499)) + 16|0);
       $501 = HEAP32[$500>>2]|0;
       $502 = (($501) + ($498<<2)|0);
       $503 = HEAP32[$502>>2]|0;
       $irow = $503;
       $504 = $irow;
       $505 = $6;
       $506 = (($505) + ($504<<3)|0);
       $507 = +HEAPF64[$506>>3];
       $508 = HEAP32[$i>>2]|0;
       $509 = $Lval;
       $510 = (($509) + ($508<<3)|0);
       $511 = +HEAPF64[$510>>3];
       $512 = $507 * $511;
       $513 = $jcol;
       $514 = $6;
       $515 = (($514) + ($513<<3)|0);
       $516 = +HEAPF64[$515>>3];
       $517 = $516 - $512;
       HEAPF64[$515>>3] = $517;
       $518 = $iptr;
       $519 = (($518) + 1)|0;
       $iptr = $519;
       $520 = HEAP32[$i>>2]|0;
       $521 = (($520) + 1)|0;
       HEAP32[$i>>2] = $521;
      }
      $522 = $jcol;
      $523 = (($522) + 1)|0;
      $jcol = $523;
     }
     $524 = HEAP32[$nsupc>>2]|0;
     $525 = ($524|0)>(1);
     if ($525) {
      $526 = HEAP32[$nsupc>>2]|0;
      $527 = HEAP32[$nsupc>>2]|0;
      $528 = (($527) - 1)|0;
      $529 = Math_imul($526, $528)|0;
      $530 = (+($529|0));
      $531 = $solve_ops;
      $532 = $531 + $530;
      $solve_ops = $532;
      $533 = $luptr;
      $534 = $Lval;
      $535 = (($534) + ($533<<3)|0);
      $536 = $fsupc;
      $537 = $6;
      $538 = (($537) + ($536<<3)|0);
      (_dtrsv_(3400,3416,3392,$nsupc,$535,$nsupr,$538,$incx)|0);
     }
     $539 = $k;
     $540 = (($539) + -1)|0;
     $k = $540;
    }
   }
   $541 = $5;
   $542 = ((($541)) + 12|0);
   $543 = HEAP32[$542>>2]|0;
   $544 = ($543|0)==(0);
   if ($544) {
    $0 = 0;
    $683 = $0;
    STACKTOP = sp;return ($683|0);
   }
   $k = 0;
   while(1) {
    $545 = $k;
    $546 = $Lstore;
    $547 = ((($546)) + 4|0);
    $548 = HEAP32[$547>>2]|0;
    $549 = ($545|0)<=($548|0);
    if (!($549)) {
     break L30;
    }
    $550 = $k;
    $551 = $Lstore;
    $552 = ((($551)) + 28|0);
    $553 = HEAP32[$552>>2]|0;
    $554 = (($553) + ($550<<2)|0);
    $555 = HEAP32[$554>>2]|0;
    $fsupc = $555;
    $556 = $fsupc;
    $557 = (($556) + 1)|0;
    $558 = $Lstore;
    $559 = ((($558)) + 20|0);
    $560 = HEAP32[$559>>2]|0;
    $561 = (($560) + ($557<<2)|0);
    $562 = HEAP32[$561>>2]|0;
    $563 = $fsupc;
    $564 = $Lstore;
    $565 = ((($564)) + 20|0);
    $566 = HEAP32[$565>>2]|0;
    $567 = (($566) + ($563<<2)|0);
    $568 = HEAP32[$567>>2]|0;
    $569 = (($562) - ($568))|0;
    HEAP32[$nsupr>>2] = $569;
    $570 = $k;
    $571 = (($570) + 1)|0;
    $572 = $Lstore;
    $573 = ((($572)) + 28|0);
    $574 = HEAP32[$573>>2]|0;
    $575 = (($574) + ($571<<2)|0);
    $576 = HEAP32[$575>>2]|0;
    $577 = $fsupc;
    $578 = (($576) - ($577))|0;
    HEAP32[$nsupc>>2] = $578;
    $579 = $fsupc;
    $580 = $Lstore;
    $581 = ((($580)) + 12|0);
    $582 = HEAP32[$581>>2]|0;
    $583 = (($582) + ($579<<2)|0);
    $584 = HEAP32[$583>>2]|0;
    $luptr = $584;
    $585 = $fsupc;
    $jcol = $585;
    while(1) {
     $586 = $jcol;
     $587 = $k;
     $588 = (($587) + 1)|0;
     $589 = $Lstore;
     $590 = ((($589)) + 28|0);
     $591 = HEAP32[$590>>2]|0;
     $592 = (($591) + ($588<<2)|0);
     $593 = HEAP32[$592>>2]|0;
     $594 = ($586|0)<($593|0);
     if (!($594)) {
      break;
     }
     $595 = $jcol;
     $596 = (($595) + 1)|0;
     $597 = $Ustore;
     $598 = ((($597)) + 12|0);
     $599 = HEAP32[$598>>2]|0;
     $600 = (($599) + ($596<<2)|0);
     $601 = HEAP32[$600>>2]|0;
     $602 = $jcol;
     $603 = $Ustore;
     $604 = ((($603)) + 12|0);
     $605 = HEAP32[$604>>2]|0;
     $606 = (($605) + ($602<<2)|0);
     $607 = HEAP32[$606>>2]|0;
     $608 = (($601) - ($607))|0;
     $609 = $608<<1;
     $610 = (+($609|0));
     $611 = $solve_ops;
     $612 = $611 + $610;
     $solve_ops = $612;
     $613 = $jcol;
     $614 = $Ustore;
     $615 = ((($614)) + 12|0);
     $616 = HEAP32[$615>>2]|0;
     $617 = (($616) + ($613<<2)|0);
     $618 = HEAP32[$617>>2]|0;
     HEAP32[$i>>2] = $618;
     while(1) {
      $619 = HEAP32[$i>>2]|0;
      $620 = $jcol;
      $621 = (($620) + 1)|0;
      $622 = $Ustore;
      $623 = ((($622)) + 12|0);
      $624 = HEAP32[$623>>2]|0;
      $625 = (($624) + ($621<<2)|0);
      $626 = HEAP32[$625>>2]|0;
      $627 = ($619|0)<($626|0);
      if (!($627)) {
       break;
      }
      $628 = HEAP32[$i>>2]|0;
      $629 = $Ustore;
      $630 = ((($629)) + 8|0);
      $631 = HEAP32[$630>>2]|0;
      $632 = (($631) + ($628<<2)|0);
      $633 = HEAP32[$632>>2]|0;
      $irow = $633;
      $634 = $irow;
      $635 = $6;
      $636 = (($635) + ($634<<3)|0);
      $637 = +HEAPF64[$636>>3];
      $638 = HEAP32[$i>>2]|0;
      $639 = $Uval;
      $640 = (($639) + ($638<<3)|0);
      $641 = +HEAPF64[$640>>3];
      $642 = $637 * $641;
      $643 = $jcol;
      $644 = $6;
      $645 = (($644) + ($643<<3)|0);
      $646 = +HEAPF64[$645>>3];
      $647 = $646 - $642;
      HEAPF64[$645>>3] = $647;
      $648 = HEAP32[$i>>2]|0;
      $649 = (($648) + 1)|0;
      HEAP32[$i>>2] = $649;
     }
     $650 = $jcol;
     $651 = (($650) + 1)|0;
     $jcol = $651;
    }
    $652 = HEAP32[$nsupc>>2]|0;
    $653 = HEAP32[$nsupc>>2]|0;
    $654 = (($653) + 1)|0;
    $655 = Math_imul($652, $654)|0;
    $656 = (+($655|0));
    $657 = $solve_ops;
    $658 = $657 + $656;
    $solve_ops = $658;
    $659 = HEAP32[$nsupc>>2]|0;
    $660 = ($659|0)==(1);
    $661 = $luptr;
    $662 = $Lval;
    $663 = (($662) + ($661<<3)|0);
    if ($660) {
     $664 = +HEAPF64[$663>>3];
     $665 = $fsupc;
     $666 = $6;
     $667 = (($666) + ($665<<3)|0);
     $668 = +HEAPF64[$667>>3];
     $669 = $668 / $664;
     HEAPF64[$667>>3] = $669;
    } else {
     $670 = $fsupc;
     $671 = $6;
     $672 = (($671) + ($670<<3)|0);
     (_dtrsv_(3392,3416,3408,$nsupc,$663,$nsupr,$672,$incx)|0);
    }
    $673 = $k;
    $674 = (($673) + 1)|0;
    $k = $674;
   }
  }
 } while(0);
 $675 = $solve_ops;
 $676 = $7;
 $677 = ((($676)) + 8|0);
 $678 = HEAP32[$677>>2]|0;
 $679 = ((($678)) + 44|0);
 $680 = +HEAPF32[$679>>2];
 $681 = $680 + $675;
 HEAPF32[$679>>2] = $681;
 $682 = $work;
 _superlu_free($682);
 $0 = 0;
 $683 = $0;
 STACKTOP = sp;return ($683|0);
}
function _dgstrf($options,$A,$relax,$panel_size,$etree,$work,$lwork,$perm_c,$perm_r,$L,$U,$stat,$info) {
 $options = $options|0;
 $A = $A|0;
 $relax = $relax|0;
 $panel_size = $panel_size|0;
 $etree = $etree|0;
 $work = $work|0;
 $lwork = $lwork|0;
 $perm_c = $perm_c|0;
 $perm_r = $perm_r|0;
 $L = $L|0;
 $U = $U|0;
 $stat = $stat|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0.0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0.0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0.0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0.0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0.0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0.0, $513 = 0, $514 = 0, $515 = 0.0, $516 = 0.0, $517 = 0, $518 = 0, $519 = 0.0, $52 = 0, $520 = 0.0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0.0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Astore = 0, $a = 0, $asub = 0, $dense = 0, $diag_pivot_thresh = 0.0, $dwork = 0, $fact = 0, $fill_ratio = 0.0, $fsupc = 0, $i = 0;
 var $icol = 0, $iinfo = 0, $iperm_c = 0, $iperm_r = 0, $iperm_r_allocated = 0, $iwork = 0, $jcol = 0, $jj = 0, $jsupno = 0, $k = 0, $kcol = 0, $m = 0, $marker = 0, $min_mn = 0, $n = 0, $new_next = 0, $nextlu = 0, $nextu = 0, $nnzL = 0, $nnzU = 0;
 var $nseg = 0, $nseg1 = 0, $nzlumax = 0, $ops = 0, $or$cond = 0, $or$cond3 = 0, $panel_histo = 0, $panel_lsub = 0, $parent = 0, $pivrow = 0, $relax_end = 0, $repfnz = 0, $segrep = 0, $supno = 0, $tempv = 0, $usepr = 0, $w_def = 0, $xa_begin = 0, $xa_end = 0, $xlsub = 0;
 var $xlusup = 0, $xplore = 0, $xprune = 0, $xsup = 0, $xusub = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 272|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $iwork = sp + 204|0;
 $dwork = sp + 200|0;
 $segrep = sp + 196|0;
 $repfnz = sp + 192|0;
 $parent = sp + 188|0;
 $xplore = sp + 184|0;
 $panel_lsub = sp + 180|0;
 $xprune = sp + 176|0;
 $marker = sp + 172|0;
 $dense = sp + 168|0;
 $tempv = sp + 164|0;
 $nzlumax = sp + 120|0;
 $pivrow = sp + 112|0;
 $nseg1 = sp + 108|0;
 $nseg = sp + 104|0;
 $usepr = sp + 36|0;
 $nnzL = sp + 28|0;
 $nnzU = sp + 24|0;
 $0 = $options;
 $1 = $A;
 $2 = $relax;
 $3 = $panel_size;
 $4 = $etree;
 $5 = $work;
 $6 = $lwork;
 $7 = $perm_c;
 $8 = $perm_r;
 $9 = $L;
 $10 = $U;
 $11 = $stat;
 $12 = $info;
 $iperm_r = 0;
 $13 = (_sp_ienv(6)|0);
 $14 = (+($13|0));
 $fill_ratio = $14;
 $15 = $0;
 $16 = HEAP32[$15>>2]|0;
 $fact = $16;
 $17 = $0;
 $18 = ((($17)) + 24|0);
 $19 = +HEAPF64[$18>>3];
 $diag_pivot_thresh = $19;
 $iperm_r_allocated = 0;
 $20 = $11;
 $21 = HEAP32[$20>>2]|0;
 $panel_histo = $21;
 $22 = $11;
 $23 = ((($22)) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $ops = $24;
 $iinfo = 0;
 $25 = $1;
 $26 = ((($25)) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 $m = $27;
 $28 = $1;
 $29 = ((($28)) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $n = $30;
 $31 = $m;
 $32 = $n;
 $33 = ($31|0)<($32|0);
 $34 = $m;
 $35 = $n;
 $36 = $33 ? $34 : $35;
 $min_mn = $36;
 $37 = $1;
 $38 = ((($37)) + 20|0);
 $39 = HEAP32[$38>>2]|0;
 $Astore = $39;
 $40 = $Astore;
 $41 = ((($40)) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $a = $42;
 $43 = $Astore;
 $44 = ((($43)) + 8|0);
 $45 = HEAP32[$44>>2]|0;
 $asub = $45;
 $46 = $Astore;
 $47 = ((($46)) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $xa_begin = $48;
 $49 = $Astore;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $xa_end = $51;
 $52 = $fact;
 $53 = $5;
 $54 = $6;
 $55 = $m;
 $56 = $n;
 $57 = $Astore;
 $58 = HEAP32[$57>>2]|0;
 $59 = $3;
 $60 = $fill_ratio;
 $61 = $9;
 $62 = $10;
 $63 = (_dLUMemInit($52,$53,$54,$55,$56,$58,$59,$60,$61,$62,1696,$iwork,$dwork)|0);
 $64 = $12;
 HEAP32[$64>>2] = $63;
 $65 = $12;
 $66 = HEAP32[$65>>2]|0;
 $67 = ($66|0)!=(0);
 if ($67) {
  STACKTOP = sp;return;
 }
 $68 = HEAP32[1696>>2]|0;
 $xsup = $68;
 $69 = HEAP32[(1700)>>2]|0;
 $supno = $69;
 $70 = HEAP32[(1708)>>2]|0;
 $xlsub = $70;
 $71 = HEAP32[(1716)>>2]|0;
 $xlusup = $71;
 $72 = HEAP32[(1728)>>2]|0;
 $xusub = $72;
 $73 = $m;
 $74 = $n;
 $75 = $3;
 $76 = HEAP32[$iwork>>2]|0;
 _SetIWork($73,$74,$75,$76,$segrep,$parent,$xplore,$repfnz,$panel_lsub,$xprune,$marker);
 $77 = $m;
 $78 = $3;
 $79 = HEAP32[$dwork>>2]|0;
 _dSetRWork($77,$78,$79,$dense,$tempv);
 $80 = $fact;
 $81 = ($80|0)==(2);
 $82 = $81&1;
 HEAP32[$usepr>>2] = $82;
 $83 = HEAP32[$usepr>>2]|0;
 $84 = ($83|0)!=(0);
 if ($84) {
  $85 = $m;
  $86 = (_intMalloc($85)|0);
  $iperm_r = $86;
  $k = 0;
  while(1) {
   $87 = $k;
   $88 = $m;
   $89 = ($87|0)<($88|0);
   if (!($89)) {
    break;
   }
   $90 = $k;
   $91 = $k;
   $92 = $8;
   $93 = (($92) + ($91<<2)|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = $iperm_r;
   $96 = (($95) + ($94<<2)|0);
   HEAP32[$96>>2] = $90;
   $97 = $k;
   $98 = (($97) + 1)|0;
   $k = $98;
  }
  $iperm_r_allocated = 1;
 }
 $99 = $n;
 $100 = (_intMalloc($99)|0);
 $iperm_c = $100;
 $k = 0;
 while(1) {
  $101 = $k;
  $102 = $n;
  $103 = ($101|0)<($102|0);
  if (!($103)) {
   break;
  }
  $104 = $k;
  $105 = $k;
  $106 = $7;
  $107 = (($106) + ($105<<2)|0);
  $108 = HEAP32[$107>>2]|0;
  $109 = $iperm_c;
  $110 = (($109) + ($108<<2)|0);
  HEAP32[$110>>2] = $104;
  $111 = $k;
  $112 = (($111) + 1)|0;
  $k = $112;
 }
 $113 = $n;
 $114 = (_intMalloc($113)|0);
 $relax_end = $114;
 $115 = $0;
 $116 = ((($115)) + 32|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = ($117|0)==(1);
 $119 = $n;
 $120 = $4;
 $121 = $2;
 $122 = HEAP32[$marker>>2]|0;
 $123 = $relax_end;
 if ($118) {
  _heap_relax_snode($119,$120,$121,$122,$123);
 } else {
  _relax_snode($119,$120,$121,$122,$123);
 }
 $124 = $8;
 $125 = $m;
 _ifill($124,$125,-1);
 $126 = HEAP32[$marker>>2]|0;
 $127 = $m;
 $128 = ($127*3)|0;
 _ifill($126,$128,-1);
 $129 = $supno;
 HEAP32[$129>>2] = -1;
 $130 = $xlusup;
 HEAP32[$130>>2] = 0;
 $131 = $xusub;
 HEAP32[$131>>2] = 0;
 $132 = $xlsub;
 HEAP32[$132>>2] = 0;
 $133 = $xsup;
 HEAP32[$133>>2] = 0;
 $134 = $3;
 $w_def = $134;
 $jcol = 0;
 L19: while(1) {
  $135 = $jcol;
  $136 = $min_mn;
  $137 = ($135|0)<($136|0);
  if (!($137)) {
   label = 48;
   break;
  }
  $138 = $jcol;
  $139 = $relax_end;
  $140 = (($139) + ($138<<2)|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = ($141|0)!=(-1);
  if ($142) {
   $143 = $jcol;
   $144 = $relax_end;
   $145 = (($144) + ($143<<2)|0);
   $146 = HEAP32[$145>>2]|0;
   $kcol = $146;
   $147 = $kcol;
   $148 = $jcol;
   $149 = (($147) - ($148))|0;
   $150 = (($149) + 1)|0;
   $151 = $panel_histo;
   $152 = (($151) + ($150<<2)|0);
   $153 = HEAP32[$152>>2]|0;
   $154 = (($153) + 1)|0;
   HEAP32[$152>>2] = $154;
   $155 = $jcol;
   $156 = $kcol;
   $157 = $asub;
   $158 = $xa_begin;
   $159 = $xa_end;
   $160 = HEAP32[$xprune>>2]|0;
   $161 = HEAP32[$marker>>2]|0;
   $162 = (_dsnode_dfs($155,$156,$157,$158,$159,$160,$161,1696)|0);
   $163 = $12;
   HEAP32[$163>>2] = $162;
   $164 = ($162|0)!=(0);
   if ($164) {
    label = 60;
    break;
   }
   $165 = $jcol;
   $166 = $xusub;
   $167 = (($166) + ($165<<2)|0);
   $168 = HEAP32[$167>>2]|0;
   $nextu = $168;
   $169 = $jcol;
   $170 = $xlusup;
   $171 = (($170) + ($169<<2)|0);
   $172 = HEAP32[$171>>2]|0;
   $nextlu = $172;
   $173 = $jcol;
   $174 = $supno;
   $175 = (($174) + ($173<<2)|0);
   $176 = HEAP32[$175>>2]|0;
   $jsupno = $176;
   $177 = $jsupno;
   $178 = $xsup;
   $179 = (($178) + ($177<<2)|0);
   $180 = HEAP32[$179>>2]|0;
   $fsupc = $180;
   $181 = $nextlu;
   $182 = $fsupc;
   $183 = (($182) + 1)|0;
   $184 = $xlsub;
   $185 = (($184) + ($183<<2)|0);
   $186 = HEAP32[$185>>2]|0;
   $187 = $fsupc;
   $188 = $xlsub;
   $189 = (($188) + ($187<<2)|0);
   $190 = HEAP32[$189>>2]|0;
   $191 = (($186) - ($190))|0;
   $192 = $kcol;
   $193 = $jcol;
   $194 = (($192) - ($193))|0;
   $195 = (($194) + 1)|0;
   $196 = Math_imul($191, $195)|0;
   $197 = (($181) + ($196))|0;
   $new_next = $197;
   $198 = HEAP32[(1740)>>2]|0;
   HEAP32[$nzlumax>>2] = $198;
   while(1) {
    $199 = $new_next;
    $200 = HEAP32[$nzlumax>>2]|0;
    $201 = ($199|0)>($200|0);
    $202 = $jcol;
    if (!($201)) {
     break;
    }
    $203 = $nextlu;
    $204 = (_dLUMemXpand($202,$203,0,$nzlumax,1696)|0);
    $205 = $12;
    HEAP32[$205>>2] = $204;
    $206 = ($204|0)!=(0);
    if ($206) {
     label = 60;
     break L19;
    }
   }
   $icol = $202;
   while(1) {
    $207 = $icol;
    $208 = $kcol;
    $209 = ($207|0)<=($208|0);
    if (!($209)) {
     break;
    }
    $210 = $nextu;
    $211 = $icol;
    $212 = (($211) + 1)|0;
    $213 = $xusub;
    $214 = (($213) + ($212<<2)|0);
    HEAP32[$214>>2] = $210;
    $215 = $icol;
    $216 = $xa_begin;
    $217 = (($216) + ($215<<2)|0);
    $218 = HEAP32[$217>>2]|0;
    $k = $218;
    while(1) {
     $219 = $k;
     $220 = $icol;
     $221 = $xa_end;
     $222 = (($221) + ($220<<2)|0);
     $223 = HEAP32[$222>>2]|0;
     $224 = ($219|0)<($223|0);
     if (!($224)) {
      break;
     }
     $225 = $k;
     $226 = $a;
     $227 = (($226) + ($225<<3)|0);
     $228 = +HEAPF64[$227>>3];
     $229 = $k;
     $230 = $asub;
     $231 = (($230) + ($229<<2)|0);
     $232 = HEAP32[$231>>2]|0;
     $233 = HEAP32[$dense>>2]|0;
     $234 = (($233) + ($232<<3)|0);
     HEAPF64[$234>>3] = $228;
     $235 = $k;
     $236 = (($235) + 1)|0;
     $k = $236;
    }
    $237 = $icol;
    $238 = $jsupno;
    $239 = $fsupc;
    $240 = HEAP32[$dense>>2]|0;
    $241 = HEAP32[$tempv>>2]|0;
    $242 = $11;
    (_dsnode_bmod($237,$238,$239,$240,$241,1696,$242)|0);
    $243 = $icol;
    $244 = $diag_pivot_thresh;
    $245 = $8;
    $246 = $iperm_r;
    $247 = $iperm_c;
    $248 = $11;
    $249 = (_dpivotL($243,$244,$usepr,$245,$246,$247,$pivrow,1696,$248)|0);
    $250 = $12;
    HEAP32[$250>>2] = $249;
    $251 = ($249|0)!=(0);
    $252 = $iinfo;
    $253 = ($252|0)==(0);
    $or$cond = $251 & $253;
    if ($or$cond) {
     $254 = $12;
     $255 = HEAP32[$254>>2]|0;
     $iinfo = $255;
    }
    $256 = $icol;
    $257 = (($256) + 1)|0;
    $icol = $257;
   }
   $258 = $icol;
   $jcol = $258;
   continue;
  }
  $259 = $w_def;
  $3 = $259;
  $260 = $jcol;
  $261 = (($260) + 1)|0;
  $k = $261;
  while(1) {
   $262 = $k;
   $263 = $jcol;
   $264 = $3;
   $265 = (($263) + ($264))|0;
   $266 = $min_mn;
   $267 = ($265|0)<($266|0);
   if ($267) {
    $268 = $jcol;
    $269 = $3;
    $270 = (($268) + ($269))|0;
    $273 = $270;
   } else {
    $271 = $min_mn;
    $273 = $271;
   }
   $272 = ($262|0)<($273|0);
   if (!($272)) {
    break;
   }
   $274 = $k;
   $275 = $relax_end;
   $276 = (($275) + ($274<<2)|0);
   $277 = HEAP32[$276>>2]|0;
   $278 = ($277|0)!=(-1);
   $279 = $k;
   if ($278) {
    label = 35;
    break;
   }
   $282 = (($279) + 1)|0;
   $k = $282;
  }
  if ((label|0) == 35) {
   label = 0;
   $280 = $jcol;
   $281 = (($279) - ($280))|0;
   $3 = $281;
  }
  $283 = $k;
  $284 = $min_mn;
  $285 = ($283|0)==($284|0);
  if ($285) {
   $286 = $min_mn;
   $287 = $jcol;
   $288 = (($286) - ($287))|0;
   $3 = $288;
  }
  $289 = $3;
  $290 = $panel_histo;
  $291 = (($290) + ($289<<2)|0);
  $292 = HEAP32[$291>>2]|0;
  $293 = (($292) + 1)|0;
  HEAP32[$291>>2] = $293;
  $294 = $m;
  $295 = $3;
  $296 = $jcol;
  $297 = $1;
  $298 = $8;
  $299 = HEAP32[$dense>>2]|0;
  $300 = HEAP32[$panel_lsub>>2]|0;
  $301 = HEAP32[$segrep>>2]|0;
  $302 = HEAP32[$repfnz>>2]|0;
  $303 = HEAP32[$xprune>>2]|0;
  $304 = HEAP32[$marker>>2]|0;
  $305 = HEAP32[$parent>>2]|0;
  $306 = HEAP32[$xplore>>2]|0;
  _dpanel_dfs($294,$295,$296,$297,$298,$nseg1,$299,$300,$301,$302,$303,$304,$305,$306,1696);
  $307 = $m;
  $308 = $3;
  $309 = $jcol;
  $310 = HEAP32[$nseg1>>2]|0;
  $311 = HEAP32[$dense>>2]|0;
  $312 = HEAP32[$tempv>>2]|0;
  $313 = HEAP32[$segrep>>2]|0;
  $314 = HEAP32[$repfnz>>2]|0;
  $315 = $11;
  _dpanel_bmod($307,$308,$309,$310,$311,$312,$313,$314,1696,$315);
  $316 = $jcol;
  $jj = $316;
  while(1) {
   $317 = $jj;
   $318 = $jcol;
   $319 = $3;
   $320 = (($318) + ($319))|0;
   $321 = ($317|0)<($320|0);
   if (!($321)) {
    break;
   }
   $322 = $jj;
   $323 = $jcol;
   $324 = (($322) - ($323))|0;
   $325 = $m;
   $326 = Math_imul($324, $325)|0;
   $k = $326;
   $327 = HEAP32[$nseg1>>2]|0;
   HEAP32[$nseg>>2] = $327;
   $328 = $m;
   $329 = $jj;
   $330 = $8;
   $331 = $k;
   $332 = HEAP32[$panel_lsub>>2]|0;
   $333 = (($332) + ($331<<2)|0);
   $334 = HEAP32[$segrep>>2]|0;
   $335 = $k;
   $336 = HEAP32[$repfnz>>2]|0;
   $337 = (($336) + ($335<<2)|0);
   $338 = HEAP32[$xprune>>2]|0;
   $339 = HEAP32[$marker>>2]|0;
   $340 = HEAP32[$parent>>2]|0;
   $341 = HEAP32[$xplore>>2]|0;
   $342 = (_dcolumn_dfs($328,$329,$330,$nseg,$333,$334,$337,$338,$339,$340,$341,1696)|0);
   $343 = $12;
   HEAP32[$343>>2] = $342;
   $344 = ($342|0)!=(0);
   if ($344) {
    label = 60;
    break L19;
   }
   $345 = $jj;
   $346 = HEAP32[$nseg>>2]|0;
   $347 = HEAP32[$nseg1>>2]|0;
   $348 = (($346) - ($347))|0;
   $349 = $k;
   $350 = HEAP32[$dense>>2]|0;
   $351 = (($350) + ($349<<3)|0);
   $352 = HEAP32[$tempv>>2]|0;
   $353 = HEAP32[$nseg1>>2]|0;
   $354 = HEAP32[$segrep>>2]|0;
   $355 = (($354) + ($353<<2)|0);
   $356 = $k;
   $357 = HEAP32[$repfnz>>2]|0;
   $358 = (($357) + ($356<<2)|0);
   $359 = $jcol;
   $360 = $11;
   $361 = (_dcolumn_bmod($345,$348,$351,$352,$355,$358,$359,1696,$360)|0);
   $362 = $12;
   HEAP32[$362>>2] = $361;
   $363 = ($361|0)!=(0);
   if ($363) {
    label = 60;
    break L19;
   }
   $364 = $jj;
   $365 = HEAP32[$nseg>>2]|0;
   $366 = HEAP32[$segrep>>2]|0;
   $367 = $k;
   $368 = HEAP32[$repfnz>>2]|0;
   $369 = (($368) + ($367<<2)|0);
   $370 = $8;
   $371 = $k;
   $372 = HEAP32[$dense>>2]|0;
   $373 = (($372) + ($371<<3)|0);
   $374 = (_dcopy_to_ucol($364,$365,$366,$369,$370,$373,1696)|0);
   $375 = $12;
   HEAP32[$375>>2] = $374;
   $376 = ($374|0)!=(0);
   if ($376) {
    label = 60;
    break L19;
   }
   $377 = $jj;
   $378 = $diag_pivot_thresh;
   $379 = $8;
   $380 = $iperm_r;
   $381 = $iperm_c;
   $382 = $11;
   $383 = (_dpivotL($377,$378,$usepr,$379,$380,$381,$pivrow,1696,$382)|0);
   $384 = $12;
   HEAP32[$384>>2] = $383;
   $385 = ($383|0)!=(0);
   $386 = $iinfo;
   $387 = ($386|0)==(0);
   $or$cond3 = $385 & $387;
   if ($or$cond3) {
    $388 = $12;
    $389 = HEAP32[$388>>2]|0;
    $iinfo = $389;
   }
   $390 = $jj;
   $391 = $8;
   $392 = HEAP32[$pivrow>>2]|0;
   $393 = HEAP32[$nseg>>2]|0;
   $394 = HEAP32[$segrep>>2]|0;
   $395 = $k;
   $396 = HEAP32[$repfnz>>2]|0;
   $397 = (($396) + ($395<<2)|0);
   $398 = HEAP32[$xprune>>2]|0;
   _dpruneL($390,$391,$392,$393,$394,$397,$398,1696);
   $399 = HEAP32[$nseg>>2]|0;
   $400 = HEAP32[$segrep>>2]|0;
   $401 = $k;
   $402 = HEAP32[$repfnz>>2]|0;
   $403 = (($402) + ($401<<2)|0);
   _resetrep_col($399,$400,$403);
   $404 = $jj;
   $405 = (($404) + 1)|0;
   $jj = $405;
  }
  $406 = $3;
  $407 = $jcol;
  $408 = (($407) + ($406))|0;
  $jcol = $408;
 }
 if ((label|0) == 48) {
  $409 = $iinfo;
  $410 = $12;
  HEAP32[$410>>2] = $409;
  $411 = $m;
  $412 = $n;
  $413 = ($411|0)>($412|0);
  L67: do {
   if ($413) {
    $k = 0;
    $i = 0;
    while(1) {
     $414 = $i;
     $415 = $m;
     $416 = ($414|0)<($415|0);
     if (!($416)) {
      break L67;
     }
     $417 = $i;
     $418 = $8;
     $419 = (($418) + ($417<<2)|0);
     $420 = HEAP32[$419>>2]|0;
     $421 = ($420|0)==(-1);
     if ($421) {
      $422 = $n;
      $423 = $k;
      $424 = (($422) + ($423))|0;
      $425 = $i;
      $426 = $8;
      $427 = (($426) + ($425<<2)|0);
      HEAP32[$427>>2] = $424;
      $428 = $k;
      $429 = (($428) + 1)|0;
      $k = $429;
     }
     $430 = $i;
     $431 = (($430) + 1)|0;
     $i = $431;
    }
   }
  } while(0);
  $432 = $min_mn;
  $433 = HEAP32[$xprune>>2]|0;
  _countnz($432,$433,$nnzL,$nnzU,1696);
  $434 = $min_mn;
  $435 = $8;
  _fixupL($434,$435,1696);
  $436 = HEAP32[$iwork>>2]|0;
  $437 = HEAP32[$dwork>>2]|0;
  _dLUWorkFree($436,$437,1696);
  $438 = $fact;
  $439 = ($438|0)==(2);
  if ($439) {
   $440 = HEAP32[$nnzL>>2]|0;
   $441 = $9;
   $442 = ((($441)) + 20|0);
   $443 = HEAP32[$442>>2]|0;
   HEAP32[$443>>2] = $440;
   $444 = $n;
   $445 = HEAP32[(1700)>>2]|0;
   $446 = (($445) + ($444<<2)|0);
   $447 = HEAP32[$446>>2]|0;
   $448 = $9;
   $449 = ((($448)) + 20|0);
   $450 = HEAP32[$449>>2]|0;
   $451 = ((($450)) + 4|0);
   HEAP32[$451>>2] = $447;
   $452 = HEAP32[(1712)>>2]|0;
   $453 = $9;
   $454 = ((($453)) + 20|0);
   $455 = HEAP32[$454>>2]|0;
   $456 = ((($455)) + 8|0);
   HEAP32[$456>>2] = $452;
   $457 = HEAP32[(1716)>>2]|0;
   $458 = $9;
   $459 = ((($458)) + 20|0);
   $460 = HEAP32[$459>>2]|0;
   $461 = ((($460)) + 12|0);
   HEAP32[$461>>2] = $457;
   $462 = HEAP32[(1704)>>2]|0;
   $463 = $9;
   $464 = ((($463)) + 20|0);
   $465 = HEAP32[$464>>2]|0;
   $466 = ((($465)) + 16|0);
   HEAP32[$466>>2] = $462;
   $467 = HEAP32[(1708)>>2]|0;
   $468 = $9;
   $469 = ((($468)) + 20|0);
   $470 = HEAP32[$469>>2]|0;
   $471 = ((($470)) + 20|0);
   HEAP32[$471>>2] = $467;
   $472 = HEAP32[$nnzU>>2]|0;
   $473 = $10;
   $474 = ((($473)) + 20|0);
   $475 = HEAP32[$474>>2]|0;
   HEAP32[$475>>2] = $472;
   $476 = HEAP32[(1720)>>2]|0;
   $477 = $10;
   $478 = ((($477)) + 20|0);
   $479 = HEAP32[$478>>2]|0;
   $480 = ((($479)) + 4|0);
   HEAP32[$480>>2] = $476;
   $481 = HEAP32[(1724)>>2]|0;
   $482 = $10;
   $483 = ((($482)) + 20|0);
   $484 = HEAP32[$483>>2]|0;
   $485 = ((($484)) + 8|0);
   HEAP32[$485>>2] = $481;
   $486 = HEAP32[(1728)>>2]|0;
   $487 = $10;
   $488 = ((($487)) + 20|0);
   $489 = HEAP32[$488>>2]|0;
   $490 = ((($489)) + 12|0);
   HEAP32[$490>>2] = $486;
  } else {
   $491 = $9;
   $492 = $1;
   $493 = ((($492)) + 12|0);
   $494 = HEAP32[$493>>2]|0;
   $495 = $min_mn;
   $496 = HEAP32[$nnzL>>2]|0;
   $497 = HEAP32[(1712)>>2]|0;
   $498 = HEAP32[(1716)>>2]|0;
   $499 = HEAP32[(1704)>>2]|0;
   $500 = HEAP32[(1708)>>2]|0;
   $501 = HEAP32[(1700)>>2]|0;
   $502 = HEAP32[1696>>2]|0;
   _dCreate_SuperNode_Matrix($491,$494,$495,$496,$497,$498,$499,$500,$501,$502,3,1,1);
   $503 = $10;
   $504 = $min_mn;
   $505 = $min_mn;
   $506 = HEAP32[$nnzU>>2]|0;
   $507 = HEAP32[(1720)>>2]|0;
   $508 = HEAP32[(1724)>>2]|0;
   $509 = HEAP32[(1728)>>2]|0;
   _dCreate_CompCol_Matrix($503,$504,$505,$506,$507,$508,$509,0,1,4);
  }
  $510 = $ops;
  $511 = ((($510)) + 52|0);
  $512 = +HEAPF32[$511>>2];
  $513 = $ops;
  $514 = ((($513)) + 56|0);
  $515 = +HEAPF32[$514>>2];
  $516 = $512 + $515;
  $517 = $ops;
  $518 = ((($517)) + 28|0);
  $519 = +HEAPF32[$518>>2];
  $520 = $519 + $516;
  HEAPF32[$518>>2] = $520;
  $521 = HEAP32[(1752)>>2]|0;
  $522 = (($521) + -1)|0;
  HEAP32[(1752)>>2] = $522;
  $523 = $11;
  $524 = ((($523)) + 20|0);
  HEAP32[$524>>2] = $522;
  $525 = $iperm_r_allocated;
  $526 = ($525|0)!=(0);
  if ($526) {
   $527 = $iperm_r;
   _superlu_free($527);
  }
  $528 = $iperm_c;
  _superlu_free($528);
  $529 = $relax_end;
  _superlu_free($529);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 60) {
  STACKTOP = sp;return;
 }
}
function _dgstrs($trans,$L,$U,$perm_c,$perm_r,$B,$stat,$info) {
 $trans = $trans|0;
 $L = $L|0;
 $U = $U|0;
 $perm_c = $perm_c|0;
 $perm_r = $perm_r|0;
 $B = $B|0;
 $stat = $stat|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0.0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0.0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0.0, $21 = 0, $210 = 0.0, $211 = 0.0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0.0, $219 = 0.0, $22 = 0, $220 = 0.0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0.0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0.0, $264 = 0.0, $265 = 0, $266 = 0, $267 = 0, $268 = 0.0, $269 = 0.0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0.0, $324 = 0, $325 = 0, $326 = 0, $327 = 0.0, $328 = 0.0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0.0, $388 = 0.0, $389 = 0.0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0.0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0.0;
 var $404 = 0.0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0.0, $457 = 0.0, $458 = 0.0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0.0, $484 = 0, $485 = 0, $486 = 0, $487 = 0.0, $488 = 0.0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0.0, $493 = 0.0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0.0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0.0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0.0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0.0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0.0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0.0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0.0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Bmat = 0, $Bstore = 0, $Lstore = 0, $Lval = 0, $Ustore = 0, $Uval = 0, $fsupc = 0, $i = 0, $incx = 0, $incy = 0, $iptr = 0, $irow = 0, $istart = 0;
 var $j = 0, $jcol = 0, $k = 0, $ldb = 0, $luptr = 0, $msg = 0, $msg1 = 0, $n = 0, $nrhs = 0, $nrow = 0, $nsupc = 0, $nsupr = 0, $or$cond = 0, $or$cond3 = 0, $rhs_work = 0, $soln = 0, $solve_ops = 0.0, $vararg_buffer = 0, $vararg_buffer6 = 0, $vararg_ptr10 = 0;
 var $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $work = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 688|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer = sp;
 $i = sp + 72|0;
 $msg = sp + 424|0;
 $msg1 = sp + 168|0;
 $0 = $trans;
 $1 = $L;
 $2 = $U;
 $3 = $perm_c;
 $4 = $perm_r;
 $5 = $B;
 $6 = $stat;
 $7 = $info;
 $incx = 1;
 $incy = 1;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = $5;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 $Bstore = $11;
 $12 = $Bstore;
 $13 = HEAP32[$12>>2]|0;
 $ldb = $13;
 $14 = $5;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $nrhs = $16;
 $17 = $0;
 $18 = ($17|0)!=(0);
 $19 = $0;
 $20 = ($19|0)!=(1);
 $or$cond = $18 & $20;
 $21 = $0;
 $22 = ($21|0)!=(2);
 $or$cond3 = $or$cond & $22;
 do {
  if ($or$cond3) {
   $23 = $7;
   HEAP32[$23>>2] = -1;
  } else {
   $24 = $1;
   $25 = ((($24)) + 12|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = $1;
   $28 = ((($27)) + 16|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($26|0)!=($29|0);
   if (!($30)) {
    $31 = $1;
    $32 = ((($31)) + 12|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($33|0)<(0);
    if (!($34)) {
     $35 = $1;
     $36 = HEAP32[$35>>2]|0;
     $37 = ($36|0)!=(3);
     if (!($37)) {
      $38 = $1;
      $39 = ((($38)) + 4|0);
      $40 = HEAP32[$39>>2]|0;
      $41 = ($40|0)!=(1);
      if (!($41)) {
       $42 = $1;
       $43 = ((($42)) + 8|0);
       $44 = HEAP32[$43>>2]|0;
       $45 = ($44|0)!=(1);
       if (!($45)) {
        $47 = $2;
        $48 = ((($47)) + 12|0);
        $49 = HEAP32[$48>>2]|0;
        $50 = $2;
        $51 = ((($50)) + 16|0);
        $52 = HEAP32[$51>>2]|0;
        $53 = ($49|0)!=($52|0);
        if (!($53)) {
         $54 = $2;
         $55 = ((($54)) + 12|0);
         $56 = HEAP32[$55>>2]|0;
         $57 = ($56|0)<(0);
         if (!($57)) {
          $58 = $2;
          $59 = HEAP32[$58>>2]|0;
          $60 = ($59|0)!=(0);
          if (!($60)) {
           $61 = $2;
           $62 = ((($61)) + 4|0);
           $63 = HEAP32[$62>>2]|0;
           $64 = ($63|0)!=(1);
           if (!($64)) {
            $65 = $2;
            $66 = ((($65)) + 8|0);
            $67 = HEAP32[$66>>2]|0;
            $68 = ($67|0)!=(4);
            if (!($68)) {
             $70 = $ldb;
             $71 = $1;
             $72 = ((($71)) + 12|0);
             $73 = HEAP32[$72>>2]|0;
             $74 = (0)>($73|0);
             if ($74) {
              $79 = 0;
             } else {
              $75 = $1;
              $76 = ((($75)) + 12|0);
              $77 = HEAP32[$76>>2]|0;
              $79 = $77;
             }
             $78 = ($70|0)<($79|0);
             if (!($78)) {
              $80 = $5;
              $81 = HEAP32[$80>>2]|0;
              $82 = ($81|0)!=(6);
              if (!($82)) {
               $83 = $5;
               $84 = ((($83)) + 4|0);
               $85 = HEAP32[$84>>2]|0;
               $86 = ($85|0)!=(1);
               if (!($86)) {
                $87 = $5;
                $88 = ((($87)) + 8|0);
                $89 = HEAP32[$88>>2]|0;
                $90 = ($89|0)!=(0);
                if (!($90)) {
                 break;
                }
               }
              }
             }
             $91 = $7;
             HEAP32[$91>>2] = -6;
             break;
            }
           }
          }
         }
        }
        $69 = $7;
        HEAP32[$69>>2] = -3;
        break;
       }
      }
     }
    }
   }
   $46 = $7;
   HEAP32[$46>>2] = -2;
  }
 } while(0);
 $92 = $7;
 $93 = HEAP32[$92>>2]|0;
 $94 = ($93|0)!=(0);
 if ($94) {
  $95 = $7;
  $96 = HEAP32[$95>>2]|0;
  $97 = (0 - ($96))|0;
  HEAP32[$i>>2] = $97;
  (_xerbla_(1784,$i)|0);
  STACKTOP = sp;return;
 }
 $98 = $1;
 $99 = ((($98)) + 12|0);
 $100 = HEAP32[$99>>2]|0;
 $n = $100;
 $101 = $n;
 $102 = $nrhs;
 $103 = Math_imul($101, $102)|0;
 $104 = (_doubleCalloc($103)|0);
 $work = $104;
 $105 = $work;
 $106 = ($105|0)!=(0|0);
 if (!($106)) {
  HEAP32[$vararg_buffer>>2] = 2360;
  $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 137;
  $vararg_ptr5 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr5>>2] = 1792;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $107 = $n;
 $108 = (_doubleMalloc($107)|0);
 $soln = $108;
 $109 = $soln;
 $110 = ($109|0)!=(0|0);
 if (!($110)) {
  HEAP32[$vararg_buffer6>>2] = 2408;
  $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
  HEAP32[$vararg_ptr9>>2] = 139;
  $vararg_ptr10 = ((($vararg_buffer6)) + 8|0);
  HEAP32[$vararg_ptr10>>2] = 1792;
  (_sprintf($msg1,2864,$vararg_buffer6)|0);
  _superlu_abort_and_exit($msg1);
 }
 $111 = $Bstore;
 $112 = ((($111)) + 4|0);
 $113 = HEAP32[$112>>2]|0;
 $Bmat = $113;
 $114 = $1;
 $115 = ((($114)) + 20|0);
 $116 = HEAP32[$115>>2]|0;
 $Lstore = $116;
 $117 = $Lstore;
 $118 = ((($117)) + 8|0);
 $119 = HEAP32[$118>>2]|0;
 $Lval = $119;
 $120 = $2;
 $121 = ((($120)) + 20|0);
 $122 = HEAP32[$121>>2]|0;
 $Ustore = $122;
 $123 = $Ustore;
 $124 = ((($123)) + 4|0);
 $125 = HEAP32[$124>>2]|0;
 $Uval = $125;
 $solve_ops = 0.0;
 $126 = $0;
 $127 = ($126|0)==(0);
 HEAP32[$i>>2] = 0;
 L37: do {
  if ($127) {
   while(1) {
    $128 = HEAP32[$i>>2]|0;
    $129 = $nrhs;
    $130 = ($128|0)<($129|0);
    if (!($130)) {
     break;
    }
    $131 = HEAP32[$i>>2]|0;
    $132 = $ldb;
    $133 = Math_imul($131, $132)|0;
    $134 = $Bmat;
    $135 = (($134) + ($133<<3)|0);
    $rhs_work = $135;
    $k = 0;
    while(1) {
     $136 = $k;
     $137 = $n;
     $138 = ($136|0)<($137|0);
     if (!($138)) {
      break;
     }
     $139 = $k;
     $140 = $rhs_work;
     $141 = (($140) + ($139<<3)|0);
     $142 = +HEAPF64[$141>>3];
     $143 = $k;
     $144 = $4;
     $145 = (($144) + ($143<<2)|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = $soln;
     $148 = (($147) + ($146<<3)|0);
     HEAPF64[$148>>3] = $142;
     $149 = $k;
     $150 = (($149) + 1)|0;
     $k = $150;
    }
    $k = 0;
    while(1) {
     $151 = $k;
     $152 = $n;
     $153 = ($151|0)<($152|0);
     if (!($153)) {
      break;
     }
     $154 = $k;
     $155 = $soln;
     $156 = (($155) + ($154<<3)|0);
     $157 = +HEAPF64[$156>>3];
     $158 = $k;
     $159 = $rhs_work;
     $160 = (($159) + ($158<<3)|0);
     HEAPF64[$160>>3] = $157;
     $161 = $k;
     $162 = (($161) + 1)|0;
     $k = $162;
    }
    $163 = HEAP32[$i>>2]|0;
    $164 = (($163) + 1)|0;
    HEAP32[$i>>2] = $164;
   }
   $k = 0;
   while(1) {
    $165 = $k;
    $166 = $Lstore;
    $167 = ((($166)) + 4|0);
    $168 = HEAP32[$167>>2]|0;
    $169 = ($165|0)<=($168|0);
    if (!($169)) {
     break;
    }
    $170 = $k;
    $171 = $Lstore;
    $172 = ((($171)) + 28|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = (($173) + ($170<<2)|0);
    $175 = HEAP32[$174>>2]|0;
    $fsupc = $175;
    $176 = $fsupc;
    $177 = $Lstore;
    $178 = ((($177)) + 20|0);
    $179 = HEAP32[$178>>2]|0;
    $180 = (($179) + ($176<<2)|0);
    $181 = HEAP32[$180>>2]|0;
    $istart = $181;
    $182 = $fsupc;
    $183 = (($182) + 1)|0;
    $184 = $Lstore;
    $185 = ((($184)) + 20|0);
    $186 = HEAP32[$185>>2]|0;
    $187 = (($186) + ($183<<2)|0);
    $188 = HEAP32[$187>>2]|0;
    $189 = $istart;
    $190 = (($188) - ($189))|0;
    $nsupr = $190;
    $191 = $k;
    $192 = (($191) + 1)|0;
    $193 = $Lstore;
    $194 = ((($193)) + 28|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = (($195) + ($192<<2)|0);
    $197 = HEAP32[$196>>2]|0;
    $198 = $fsupc;
    $199 = (($197) - ($198))|0;
    $nsupc = $199;
    $200 = $nsupr;
    $201 = $nsupc;
    $202 = (($200) - ($201))|0;
    $nrow = $202;
    $203 = $nsupc;
    $204 = $nsupc;
    $205 = (($204) - 1)|0;
    $206 = Math_imul($203, $205)|0;
    $207 = $nrhs;
    $208 = Math_imul($206, $207)|0;
    $209 = (+($208|0));
    $210 = $solve_ops;
    $211 = $210 + $209;
    $solve_ops = $211;
    $212 = $nrow;
    $213 = $212<<1;
    $214 = $nsupc;
    $215 = Math_imul($213, $214)|0;
    $216 = $nrhs;
    $217 = Math_imul($215, $216)|0;
    $218 = (+($217|0));
    $219 = $solve_ops;
    $220 = $219 + $218;
    $solve_ops = $220;
    $221 = $nsupc;
    $222 = ($221|0)==(1);
    L80: do {
     if ($222) {
      $j = 0;
      while(1) {
       $223 = $j;
       $224 = $nrhs;
       $225 = ($223|0)<($224|0);
       if (!($225)) {
        break L80;
       }
       $226 = $j;
       $227 = $ldb;
       $228 = Math_imul($226, $227)|0;
       $229 = $Bmat;
       $230 = (($229) + ($228<<3)|0);
       $rhs_work = $230;
       $231 = $fsupc;
       $232 = $Lstore;
       $233 = ((($232)) + 12|0);
       $234 = HEAP32[$233>>2]|0;
       $235 = (($234) + ($231<<2)|0);
       $236 = HEAP32[$235>>2]|0;
       $luptr = $236;
       $237 = $istart;
       $238 = (($237) + 1)|0;
       $iptr = $238;
       while(1) {
        $239 = $iptr;
        $240 = $fsupc;
        $241 = (($240) + 1)|0;
        $242 = $Lstore;
        $243 = ((($242)) + 20|0);
        $244 = HEAP32[$243>>2]|0;
        $245 = (($244) + ($241<<2)|0);
        $246 = HEAP32[$245>>2]|0;
        $247 = ($239|0)<($246|0);
        if (!($247)) {
         break;
        }
        $248 = $iptr;
        $249 = $Lstore;
        $250 = ((($249)) + 16|0);
        $251 = HEAP32[$250>>2]|0;
        $252 = (($251) + ($248<<2)|0);
        $253 = HEAP32[$252>>2]|0;
        $irow = $253;
        $254 = $luptr;
        $255 = (($254) + 1)|0;
        $luptr = $255;
        $256 = $fsupc;
        $257 = $rhs_work;
        $258 = (($257) + ($256<<3)|0);
        $259 = +HEAPF64[$258>>3];
        $260 = $luptr;
        $261 = $Lval;
        $262 = (($261) + ($260<<3)|0);
        $263 = +HEAPF64[$262>>3];
        $264 = $259 * $263;
        $265 = $irow;
        $266 = $rhs_work;
        $267 = (($266) + ($265<<3)|0);
        $268 = +HEAPF64[$267>>3];
        $269 = $268 - $264;
        HEAPF64[$267>>3] = $269;
        $270 = $iptr;
        $271 = (($270) + 1)|0;
        $iptr = $271;
       }
       $272 = $j;
       $273 = (($272) + 1)|0;
       $j = $273;
      }
     } else {
      $274 = $fsupc;
      $275 = $Lstore;
      $276 = ((($275)) + 12|0);
      $277 = HEAP32[$276>>2]|0;
      $278 = (($277) + ($274<<2)|0);
      $279 = HEAP32[$278>>2]|0;
      $luptr = $279;
      $j = 0;
      while(1) {
       $280 = $j;
       $281 = $nrhs;
       $282 = ($280|0)<($281|0);
       if (!($282)) {
        break L80;
       }
       $283 = $j;
       $284 = $ldb;
       $285 = Math_imul($283, $284)|0;
       $286 = $Bmat;
       $287 = (($286) + ($285<<3)|0);
       $rhs_work = $287;
       $288 = $nsupr;
       $289 = $nsupc;
       $290 = $luptr;
       $291 = $Lval;
       $292 = (($291) + ($290<<3)|0);
       $293 = $fsupc;
       $294 = $rhs_work;
       $295 = (($294) + ($293<<3)|0);
       _dlsolve($288,$289,$292,$295);
       $296 = $nsupr;
       $297 = $nrow;
       $298 = $nsupc;
       $299 = $luptr;
       $300 = $nsupc;
       $301 = (($299) + ($300))|0;
       $302 = $Lval;
       $303 = (($302) + ($301<<3)|0);
       $304 = $fsupc;
       $305 = $rhs_work;
       $306 = (($305) + ($304<<3)|0);
       $307 = $work;
       _dmatvec($296,$297,$298,$303,$306,$307);
       $308 = $istart;
       $309 = $nsupc;
       $310 = (($308) + ($309))|0;
       $iptr = $310;
       HEAP32[$i>>2] = 0;
       while(1) {
        $311 = HEAP32[$i>>2]|0;
        $312 = $nrow;
        $313 = ($311|0)<($312|0);
        if (!($313)) {
         break;
        }
        $314 = $iptr;
        $315 = $Lstore;
        $316 = ((($315)) + 16|0);
        $317 = HEAP32[$316>>2]|0;
        $318 = (($317) + ($314<<2)|0);
        $319 = HEAP32[$318>>2]|0;
        $irow = $319;
        $320 = HEAP32[$i>>2]|0;
        $321 = $work;
        $322 = (($321) + ($320<<3)|0);
        $323 = +HEAPF64[$322>>3];
        $324 = $irow;
        $325 = $rhs_work;
        $326 = (($325) + ($324<<3)|0);
        $327 = +HEAPF64[$326>>3];
        $328 = $327 - $323;
        HEAPF64[$326>>3] = $328;
        $329 = HEAP32[$i>>2]|0;
        $330 = $work;
        $331 = (($330) + ($329<<3)|0);
        HEAPF64[$331>>3] = 0.0;
        $332 = $iptr;
        $333 = (($332) + 1)|0;
        $iptr = $333;
        $334 = HEAP32[$i>>2]|0;
        $335 = (($334) + 1)|0;
        HEAP32[$i>>2] = $335;
       }
       $336 = $j;
       $337 = (($336) + 1)|0;
       $j = $337;
      }
     }
    } while(0);
    $338 = $k;
    $339 = (($338) + 1)|0;
    $k = $339;
   }
   $340 = $Lstore;
   $341 = ((($340)) + 4|0);
   $342 = HEAP32[$341>>2]|0;
   $k = $342;
   while(1) {
    $343 = $k;
    $344 = ($343|0)>=(0);
    if (!($344)) {
     break;
    }
    $345 = $k;
    $346 = $Lstore;
    $347 = ((($346)) + 28|0);
    $348 = HEAP32[$347>>2]|0;
    $349 = (($348) + ($345<<2)|0);
    $350 = HEAP32[$349>>2]|0;
    $fsupc = $350;
    $351 = $fsupc;
    $352 = $Lstore;
    $353 = ((($352)) + 20|0);
    $354 = HEAP32[$353>>2]|0;
    $355 = (($354) + ($351<<2)|0);
    $356 = HEAP32[$355>>2]|0;
    $istart = $356;
    $357 = $fsupc;
    $358 = (($357) + 1)|0;
    $359 = $Lstore;
    $360 = ((($359)) + 20|0);
    $361 = HEAP32[$360>>2]|0;
    $362 = (($361) + ($358<<2)|0);
    $363 = HEAP32[$362>>2]|0;
    $364 = $istart;
    $365 = (($363) - ($364))|0;
    $nsupr = $365;
    $366 = $k;
    $367 = (($366) + 1)|0;
    $368 = $Lstore;
    $369 = ((($368)) + 28|0);
    $370 = HEAP32[$369>>2]|0;
    $371 = (($370) + ($367<<2)|0);
    $372 = HEAP32[$371>>2]|0;
    $373 = $fsupc;
    $374 = (($372) - ($373))|0;
    $nsupc = $374;
    $375 = $fsupc;
    $376 = $Lstore;
    $377 = ((($376)) + 12|0);
    $378 = HEAP32[$377>>2]|0;
    $379 = (($378) + ($375<<2)|0);
    $380 = HEAP32[$379>>2]|0;
    $luptr = $380;
    $381 = $nsupc;
    $382 = $nsupc;
    $383 = (($382) + 1)|0;
    $384 = Math_imul($381, $383)|0;
    $385 = $nrhs;
    $386 = Math_imul($384, $385)|0;
    $387 = (+($386|0));
    $388 = $solve_ops;
    $389 = $388 + $387;
    $solve_ops = $389;
    $390 = $nsupc;
    $391 = ($390|0)==(1);
    L102: do {
     if ($391) {
      $392 = $Bmat;
      $rhs_work = $392;
      $j = 0;
      while(1) {
       $393 = $j;
       $394 = $nrhs;
       $395 = ($393|0)<($394|0);
       if (!($395)) {
        break L102;
       }
       $396 = $luptr;
       $397 = $Lval;
       $398 = (($397) + ($396<<3)|0);
       $399 = +HEAPF64[$398>>3];
       $400 = $fsupc;
       $401 = $rhs_work;
       $402 = (($401) + ($400<<3)|0);
       $403 = +HEAPF64[$402>>3];
       $404 = $403 / $399;
       HEAPF64[$402>>3] = $404;
       $405 = $ldb;
       $406 = $rhs_work;
       $407 = (($406) + ($405<<3)|0);
       $rhs_work = $407;
       $408 = $j;
       $409 = (($408) + 1)|0;
       $j = $409;
      }
     } else {
      $j = 0;
      while(1) {
       $410 = $j;
       $411 = $nrhs;
       $412 = ($410|0)<($411|0);
       if (!($412)) {
        break L102;
       }
       $413 = $nsupr;
       $414 = $nsupc;
       $415 = $luptr;
       $416 = $Lval;
       $417 = (($416) + ($415<<3)|0);
       $418 = $fsupc;
       $419 = $j;
       $420 = $ldb;
       $421 = Math_imul($419, $420)|0;
       $422 = (($418) + ($421))|0;
       $423 = $Bmat;
       $424 = (($423) + ($422<<3)|0);
       _dusolve($413,$414,$417,$424);
       $425 = $j;
       $426 = (($425) + 1)|0;
       $j = $426;
      }
     }
    } while(0);
    $j = 0;
    while(1) {
     $427 = $j;
     $428 = $nrhs;
     $429 = ($427|0)<($428|0);
     if (!($429)) {
      break;
     }
     $430 = $j;
     $431 = $ldb;
     $432 = Math_imul($430, $431)|0;
     $433 = $Bmat;
     $434 = (($433) + ($432<<3)|0);
     $rhs_work = $434;
     $435 = $fsupc;
     $jcol = $435;
     while(1) {
      $436 = $jcol;
      $437 = $fsupc;
      $438 = $nsupc;
      $439 = (($437) + ($438))|0;
      $440 = ($436|0)<($439|0);
      if (!($440)) {
       break;
      }
      $441 = $jcol;
      $442 = (($441) + 1)|0;
      $443 = $Ustore;
      $444 = ((($443)) + 12|0);
      $445 = HEAP32[$444>>2]|0;
      $446 = (($445) + ($442<<2)|0);
      $447 = HEAP32[$446>>2]|0;
      $448 = $jcol;
      $449 = $Ustore;
      $450 = ((($449)) + 12|0);
      $451 = HEAP32[$450>>2]|0;
      $452 = (($451) + ($448<<2)|0);
      $453 = HEAP32[$452>>2]|0;
      $454 = (($447) - ($453))|0;
      $455 = $454<<1;
      $456 = (+($455|0));
      $457 = $solve_ops;
      $458 = $457 + $456;
      $solve_ops = $458;
      $459 = $jcol;
      $460 = $Ustore;
      $461 = ((($460)) + 12|0);
      $462 = HEAP32[$461>>2]|0;
      $463 = (($462) + ($459<<2)|0);
      $464 = HEAP32[$463>>2]|0;
      HEAP32[$i>>2] = $464;
      while(1) {
       $465 = HEAP32[$i>>2]|0;
       $466 = $jcol;
       $467 = (($466) + 1)|0;
       $468 = $Ustore;
       $469 = ((($468)) + 12|0);
       $470 = HEAP32[$469>>2]|0;
       $471 = (($470) + ($467<<2)|0);
       $472 = HEAP32[$471>>2]|0;
       $473 = ($465|0)<($472|0);
       if (!($473)) {
        break;
       }
       $474 = HEAP32[$i>>2]|0;
       $475 = $Ustore;
       $476 = ((($475)) + 8|0);
       $477 = HEAP32[$476>>2]|0;
       $478 = (($477) + ($474<<2)|0);
       $479 = HEAP32[$478>>2]|0;
       $irow = $479;
       $480 = $jcol;
       $481 = $rhs_work;
       $482 = (($481) + ($480<<3)|0);
       $483 = +HEAPF64[$482>>3];
       $484 = HEAP32[$i>>2]|0;
       $485 = $Uval;
       $486 = (($485) + ($484<<3)|0);
       $487 = +HEAPF64[$486>>3];
       $488 = $483 * $487;
       $489 = $irow;
       $490 = $rhs_work;
       $491 = (($490) + ($489<<3)|0);
       $492 = +HEAPF64[$491>>3];
       $493 = $492 - $488;
       HEAPF64[$491>>3] = $493;
       $494 = HEAP32[$i>>2]|0;
       $495 = (($494) + 1)|0;
       HEAP32[$i>>2] = $495;
      }
      $496 = $jcol;
      $497 = (($496) + 1)|0;
      $jcol = $497;
     }
     $498 = $j;
     $499 = (($498) + 1)|0;
     $j = $499;
    }
    $500 = $k;
    $501 = (($500) + -1)|0;
    $k = $501;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $502 = HEAP32[$i>>2]|0;
    $503 = $nrhs;
    $504 = ($502|0)<($503|0);
    if (!($504)) {
     break;
    }
    $505 = HEAP32[$i>>2]|0;
    $506 = $ldb;
    $507 = Math_imul($505, $506)|0;
    $508 = $Bmat;
    $509 = (($508) + ($507<<3)|0);
    $rhs_work = $509;
    $k = 0;
    while(1) {
     $510 = $k;
     $511 = $n;
     $512 = ($510|0)<($511|0);
     if (!($512)) {
      break;
     }
     $513 = $k;
     $514 = $3;
     $515 = (($514) + ($513<<2)|0);
     $516 = HEAP32[$515>>2]|0;
     $517 = $rhs_work;
     $518 = (($517) + ($516<<3)|0);
     $519 = +HEAPF64[$518>>3];
     $520 = $k;
     $521 = $soln;
     $522 = (($521) + ($520<<3)|0);
     HEAPF64[$522>>3] = $519;
     $523 = $k;
     $524 = (($523) + 1)|0;
     $k = $524;
    }
    $k = 0;
    while(1) {
     $525 = $k;
     $526 = $n;
     $527 = ($525|0)<($526|0);
     if (!($527)) {
      break;
     }
     $528 = $k;
     $529 = $soln;
     $530 = (($529) + ($528<<3)|0);
     $531 = +HEAPF64[$530>>3];
     $532 = $k;
     $533 = $rhs_work;
     $534 = (($533) + ($532<<3)|0);
     HEAPF64[$534>>3] = $531;
     $535 = $k;
     $536 = (($535) + 1)|0;
     $k = $536;
    }
    $537 = HEAP32[$i>>2]|0;
    $538 = (($537) + 1)|0;
    HEAP32[$i>>2] = $538;
   }
   $539 = $solve_ops;
   $540 = $6;
   $541 = ((($540)) + 8|0);
   $542 = HEAP32[$541>>2]|0;
   $543 = ((($542)) + 44|0);
   HEAPF32[$543>>2] = $539;
  } else {
   while(1) {
    $544 = HEAP32[$i>>2]|0;
    $545 = $nrhs;
    $546 = ($544|0)<($545|0);
    if (!($546)) {
     break;
    }
    $547 = HEAP32[$i>>2]|0;
    $548 = $ldb;
    $549 = Math_imul($547, $548)|0;
    $550 = $Bmat;
    $551 = (($550) + ($549<<3)|0);
    $rhs_work = $551;
    $k = 0;
    while(1) {
     $552 = $k;
     $553 = $n;
     $554 = ($552|0)<($553|0);
     if (!($554)) {
      break;
     }
     $555 = $k;
     $556 = $rhs_work;
     $557 = (($556) + ($555<<3)|0);
     $558 = +HEAPF64[$557>>3];
     $559 = $k;
     $560 = $3;
     $561 = (($560) + ($559<<2)|0);
     $562 = HEAP32[$561>>2]|0;
     $563 = $soln;
     $564 = (($563) + ($562<<3)|0);
     HEAPF64[$564>>3] = $558;
     $565 = $k;
     $566 = (($565) + 1)|0;
     $k = $566;
    }
    $k = 0;
    while(1) {
     $567 = $k;
     $568 = $n;
     $569 = ($567|0)<($568|0);
     if (!($569)) {
      break;
     }
     $570 = $k;
     $571 = $soln;
     $572 = (($571) + ($570<<3)|0);
     $573 = +HEAPF64[$572>>3];
     $574 = $k;
     $575 = $rhs_work;
     $576 = (($575) + ($574<<3)|0);
     HEAPF64[$576>>3] = $573;
     $577 = $k;
     $578 = (($577) + 1)|0;
     $k = $578;
    }
    $579 = HEAP32[$i>>2]|0;
    $580 = (($579) + 1)|0;
    HEAP32[$i>>2] = $580;
   }
   $581 = $6;
   $582 = ((($581)) + 8|0);
   $583 = HEAP32[$582>>2]|0;
   $584 = ((($583)) + 44|0);
   HEAPF32[$584>>2] = 0.0;
   $k = 0;
   while(1) {
    $585 = $k;
    $586 = $nrhs;
    $587 = ($585|0)<($586|0);
    if (!($587)) {
     break;
    }
    $588 = $1;
    $589 = $2;
    $590 = $k;
    $591 = $ldb;
    $592 = Math_imul($590, $591)|0;
    $593 = $Bmat;
    $594 = (($593) + ($592<<3)|0);
    $595 = $6;
    $596 = $7;
    (_sp_dtrsv(3392,3416,3408,$588,$589,$594,$595,$596)|0);
    $597 = $1;
    $598 = $2;
    $599 = $k;
    $600 = $ldb;
    $601 = Math_imul($599, $600)|0;
    $602 = $Bmat;
    $603 = (($602) + ($601<<3)|0);
    $604 = $6;
    $605 = $7;
    (_sp_dtrsv(3400,3416,3392,$597,$598,$603,$604,$605)|0);
    $606 = $k;
    $607 = (($606) + 1)|0;
    $k = $607;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $608 = HEAP32[$i>>2]|0;
    $609 = $nrhs;
    $610 = ($608|0)<($609|0);
    if (!($610)) {
     break L37;
    }
    $611 = HEAP32[$i>>2]|0;
    $612 = $ldb;
    $613 = Math_imul($611, $612)|0;
    $614 = $Bmat;
    $615 = (($614) + ($613<<3)|0);
    $rhs_work = $615;
    $k = 0;
    while(1) {
     $616 = $k;
     $617 = $n;
     $618 = ($616|0)<($617|0);
     if (!($618)) {
      break;
     }
     $619 = $k;
     $620 = $4;
     $621 = (($620) + ($619<<2)|0);
     $622 = HEAP32[$621>>2]|0;
     $623 = $rhs_work;
     $624 = (($623) + ($622<<3)|0);
     $625 = +HEAPF64[$624>>3];
     $626 = $k;
     $627 = $soln;
     $628 = (($627) + ($626<<3)|0);
     HEAPF64[$628>>3] = $625;
     $629 = $k;
     $630 = (($629) + 1)|0;
     $k = $630;
    }
    $k = 0;
    while(1) {
     $631 = $k;
     $632 = $n;
     $633 = ($631|0)<($632|0);
     if (!($633)) {
      break;
     }
     $634 = $k;
     $635 = $soln;
     $636 = (($635) + ($634<<3)|0);
     $637 = +HEAPF64[$636>>3];
     $638 = $k;
     $639 = $rhs_work;
     $640 = (($639) + ($638<<3)|0);
     HEAPF64[$640>>3] = $637;
     $641 = $k;
     $642 = (($641) + 1)|0;
     $k = $642;
    }
    $643 = HEAP32[$i>>2]|0;
    $644 = (($643) + 1)|0;
    HEAP32[$i>>2] = $644;
   }
  }
 } while(0);
 $645 = $work;
 _superlu_free($645);
 $646 = $soln;
 _superlu_free($646);
 STACKTOP = sp;return;
}
function _dcopy_to_ucol($jcol,$nseg,$segrep,$repfnz,$perm_r,$dense,$Glu) {
 $jcol = $jcol|0;
 $nseg = $nseg|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $perm_r = $perm_r|0;
 $dense = $dense|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0.0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $fsupc = 0, $i = 0, $irow = 0, $isub = 0, $jsupno = 0, $k = 0, $kfnz = 0, $krep = 0, $ksub = 0, $ksupno = 0, $lsub = 0, $mem_error = 0, $new_next = 0, $nextu = 0, $nzumax = 0, $segsze = 0;
 var $supno = 0, $ucol = 0, $usub = 0, $xlsub = 0, $xsup = 0, $xusub = 0, $zero = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $nzumax = sp + 8|0;
 $1 = $jcol;
 $2 = $nseg;
 $3 = $segrep;
 $4 = $repfnz;
 $5 = $perm_r;
 $6 = $dense;
 $7 = $Glu;
 $zero = 0.0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $xsup = $9;
 $10 = $7;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $supno = $12;
 $13 = $7;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $lsub = $15;
 $16 = $7;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $xlsub = $18;
 $19 = $7;
 $20 = ((($19)) + 24|0);
 $21 = HEAP32[$20>>2]|0;
 $ucol = $21;
 $22 = $7;
 $23 = ((($22)) + 28|0);
 $24 = HEAP32[$23>>2]|0;
 $usub = $24;
 $25 = $7;
 $26 = ((($25)) + 32|0);
 $27 = HEAP32[$26>>2]|0;
 $xusub = $27;
 $28 = $7;
 $29 = ((($28)) + 40|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$nzumax>>2] = $30;
 $31 = $1;
 $32 = $supno;
 $33 = (($32) + ($31<<2)|0);
 $34 = HEAP32[$33>>2]|0;
 $jsupno = $34;
 $35 = $1;
 $36 = $xusub;
 $37 = (($36) + ($35<<2)|0);
 $38 = HEAP32[$37>>2]|0;
 $nextu = $38;
 $39 = $2;
 $40 = (($39) - 1)|0;
 $k = $40;
 $ksub = 0;
 L1: while(1) {
  $41 = $ksub;
  $42 = $2;
  $43 = ($41|0)<($42|0);
  if (!($43)) {
   label = 16;
   break;
  }
  $44 = $k;
  $45 = (($44) + -1)|0;
  $k = $45;
  $46 = $3;
  $47 = (($46) + ($44<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $krep = $48;
  $49 = $krep;
  $50 = $supno;
  $51 = (($50) + ($49<<2)|0);
  $52 = HEAP32[$51>>2]|0;
  $ksupno = $52;
  $53 = $ksupno;
  $54 = $jsupno;
  $55 = ($53|0)!=($54|0);
  L4: do {
   if ($55) {
    $56 = $krep;
    $57 = $4;
    $58 = (($57) + ($56<<2)|0);
    $59 = HEAP32[$58>>2]|0;
    $kfnz = $59;
    $60 = $kfnz;
    $61 = ($60|0)!=(-1);
    if ($61) {
     $62 = $ksupno;
     $63 = $xsup;
     $64 = (($63) + ($62<<2)|0);
     $65 = HEAP32[$64>>2]|0;
     $fsupc = $65;
     $66 = $fsupc;
     $67 = $xlsub;
     $68 = (($67) + ($66<<2)|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = $kfnz;
     $71 = (($69) + ($70))|0;
     $72 = $fsupc;
     $73 = (($71) - ($72))|0;
     $isub = $73;
     $74 = $krep;
     $75 = $kfnz;
     $76 = (($74) - ($75))|0;
     $77 = (($76) + 1)|0;
     $segsze = $77;
     $78 = $nextu;
     $79 = $segsze;
     $80 = (($78) + ($79))|0;
     $new_next = $80;
     while(1) {
      $81 = $new_next;
      $82 = HEAP32[$nzumax>>2]|0;
      $83 = ($81|0)>($82|0);
      if (!($83)) {
       break;
      }
      $84 = $1;
      $85 = $nextu;
      $86 = $7;
      $87 = (_dLUMemXpand($84,$85,1,$nzumax,$86)|0);
      $mem_error = $87;
      $88 = ($87|0)!=(0);
      if ($88) {
       label = 8;
       break L1;
      }
      $90 = $7;
      $91 = ((($90)) + 24|0);
      $92 = HEAP32[$91>>2]|0;
      $ucol = $92;
      $93 = $1;
      $94 = $nextu;
      $95 = $7;
      $96 = (_dLUMemXpand($93,$94,3,$nzumax,$95)|0);
      $mem_error = $96;
      $97 = ($96|0)!=(0);
      if ($97) {
       label = 10;
       break L1;
      }
      $99 = $7;
      $100 = ((($99)) + 28|0);
      $101 = HEAP32[$100>>2]|0;
      $usub = $101;
      $102 = $7;
      $103 = ((($102)) + 8|0);
      $104 = HEAP32[$103>>2]|0;
      $lsub = $104;
     }
     $i = 0;
     while(1) {
      $105 = $i;
      $106 = $segsze;
      $107 = ($105|0)<($106|0);
      if (!($107)) {
       break L4;
      }
      $108 = $isub;
      $109 = $lsub;
      $110 = (($109) + ($108<<2)|0);
      $111 = HEAP32[$110>>2]|0;
      $irow = $111;
      $112 = $irow;
      $113 = $5;
      $114 = (($113) + ($112<<2)|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = $nextu;
      $117 = $usub;
      $118 = (($117) + ($116<<2)|0);
      HEAP32[$118>>2] = $115;
      $119 = $irow;
      $120 = $6;
      $121 = (($120) + ($119<<3)|0);
      $122 = +HEAPF64[$121>>3];
      $123 = $nextu;
      $124 = $ucol;
      $125 = (($124) + ($123<<3)|0);
      HEAPF64[$125>>3] = $122;
      $126 = $zero;
      $127 = $irow;
      $128 = $6;
      $129 = (($128) + ($127<<3)|0);
      HEAPF64[$129>>3] = $126;
      $130 = $nextu;
      $131 = (($130) + 1)|0;
      $nextu = $131;
      $132 = $isub;
      $133 = (($132) + 1)|0;
      $isub = $133;
      $134 = $i;
      $135 = (($134) + 1)|0;
      $i = $135;
     }
    }
   }
  } while(0);
  $136 = $ksub;
  $137 = (($136) + 1)|0;
  $ksub = $137;
 }
 if ((label|0) == 8) {
  $89 = $mem_error;
  $0 = $89;
  $143 = $0;
  STACKTOP = sp;return ($143|0);
 }
 else if ((label|0) == 10) {
  $98 = $mem_error;
  $0 = $98;
  $143 = $0;
  STACKTOP = sp;return ($143|0);
 }
 else if ((label|0) == 16) {
  $138 = $nextu;
  $139 = $1;
  $140 = (($139) + 1)|0;
  $141 = $xusub;
  $142 = (($141) + ($140<<2)|0);
  HEAP32[$142>>2] = $138;
  $0 = 0;
  $143 = $0;
  STACKTOP = sp;return ($143|0);
 }
 return (0)|0;
}
function _dsnode_dfs($jcol,$kcol,$asub,$xa_begin,$xa_end,$xprune,$marker,$Glu) {
 $jcol = $jcol|0;
 $kcol = $kcol|0;
 $asub = $asub|0;
 $xa_begin = $xa_begin|0;
 $xa_end = $xa_end|0;
 $xprune = $xprune|0;
 $marker = $marker|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0;
 var $ifrom = 0, $ito = 0, $k = 0, $kmark = 0, $krow = 0, $lsub = 0, $mem_error = 0, $new_next = 0, $nextl = 0, $nsuper = 0, $nzlmax = 0, $supno = 0, $xlsub = 0, $xsup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $nzlmax = sp;
 $1 = $jcol;
 $2 = $kcol;
 $3 = $asub;
 $4 = $xa_begin;
 $5 = $xa_end;
 $6 = $xprune;
 $7 = $marker;
 $8 = $Glu;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $xsup = $10;
 $11 = $8;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $supno = $13;
 $14 = $8;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $lsub = $16;
 $17 = $8;
 $18 = ((($17)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $xlsub = $19;
 $20 = $8;
 $21 = ((($20)) + 36|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$nzlmax>>2] = $22;
 $23 = $1;
 $24 = $supno;
 $25 = (($24) + ($23<<2)|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + 1)|0;
 HEAP32[$25>>2] = $27;
 $nsuper = $27;
 $28 = $1;
 $29 = $xlsub;
 $30 = (($29) + ($28<<2)|0);
 $31 = HEAP32[$30>>2]|0;
 $nextl = $31;
 $32 = $1;
 $i = $32;
 L1: while(1) {
  $33 = $i;
  $34 = $2;
  $35 = ($33|0)<=($34|0);
  if (!($35)) {
   break;
  }
  $36 = $i;
  $37 = $4;
  $38 = (($37) + ($36<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $k = $39;
  while(1) {
   $40 = $k;
   $41 = $i;
   $42 = $5;
   $43 = (($42) + ($41<<2)|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($40|0)<($44|0);
   if (!($45)) {
    break;
   }
   $46 = $k;
   $47 = $3;
   $48 = (($47) + ($46<<2)|0);
   $49 = HEAP32[$48>>2]|0;
   $krow = $49;
   $50 = $krow;
   $51 = $7;
   $52 = (($51) + ($50<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $kmark = $53;
   $54 = $kmark;
   $55 = $2;
   $56 = ($54|0)!=($55|0);
   if ($56) {
    $57 = $2;
    $58 = $krow;
    $59 = $7;
    $60 = (($59) + ($58<<2)|0);
    HEAP32[$60>>2] = $57;
    $61 = $krow;
    $62 = $nextl;
    $63 = (($62) + 1)|0;
    $nextl = $63;
    $64 = $lsub;
    $65 = (($64) + ($62<<2)|0);
    HEAP32[$65>>2] = $61;
    $66 = $nextl;
    $67 = HEAP32[$nzlmax>>2]|0;
    $68 = ($66|0)>=($67|0);
    if ($68) {
     $69 = $1;
     $70 = $nextl;
     $71 = $8;
     $72 = (_dLUMemXpand($69,$70,2,$nzlmax,$71)|0);
     $mem_error = $72;
     $73 = ($72|0)!=(0);
     if ($73) {
      label = 8;
      break L1;
     }
     $75 = $8;
     $76 = ((($75)) + 8|0);
     $77 = HEAP32[$76>>2]|0;
     $lsub = $77;
    }
   }
   $78 = $k;
   $79 = (($78) + 1)|0;
   $k = $79;
  }
  $80 = $nsuper;
  $81 = $i;
  $82 = $supno;
  $83 = (($82) + ($81<<2)|0);
  HEAP32[$83>>2] = $80;
  $84 = $i;
  $85 = (($84) + 1)|0;
  $i = $85;
 }
 if ((label|0) == 8) {
  $74 = $mem_error;
  $0 = $74;
  $158 = $0;
  STACKTOP = sp;return ($158|0);
 }
 $86 = $1;
 $87 = $2;
 $88 = ($86|0)<($87|0);
 if ($88) {
  $89 = $nextl;
  $90 = $nextl;
  $91 = $1;
  $92 = $xlsub;
  $93 = (($92) + ($91<<2)|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (($90) - ($94))|0;
  $96 = (($89) + ($95))|0;
  $new_next = $96;
  while(1) {
   $97 = $new_next;
   $98 = HEAP32[$nzlmax>>2]|0;
   $99 = ($97|0)>($98|0);
   if (!($99)) {
    break;
   }
   $100 = $1;
   $101 = $nextl;
   $102 = $8;
   $103 = (_dLUMemXpand($100,$101,2,$nzlmax,$102)|0);
   $mem_error = $103;
   $104 = ($103|0)!=(0);
   if ($104) {
    label = 16;
    break;
   }
   $106 = $8;
   $107 = ((($106)) + 8|0);
   $108 = HEAP32[$107>>2]|0;
   $lsub = $108;
  }
  if ((label|0) == 16) {
   $105 = $mem_error;
   $0 = $105;
   $158 = $0;
   STACKTOP = sp;return ($158|0);
  }
  $109 = $nextl;
  $ito = $109;
  $110 = $1;
  $111 = $xlsub;
  $112 = (($111) + ($110<<2)|0);
  $113 = HEAP32[$112>>2]|0;
  $ifrom = $113;
  while(1) {
   $114 = $ifrom;
   $115 = $nextl;
   $116 = ($114|0)<($115|0);
   if (!($116)) {
    break;
   }
   $117 = $ifrom;
   $118 = (($117) + 1)|0;
   $ifrom = $118;
   $119 = $lsub;
   $120 = (($119) + ($117<<2)|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = $ito;
   $123 = (($122) + 1)|0;
   $ito = $123;
   $124 = $lsub;
   $125 = (($124) + ($122<<2)|0);
   HEAP32[$125>>2] = $121;
  }
  $126 = $1;
  $127 = (($126) + 1)|0;
  $i = $127;
  while(1) {
   $128 = $i;
   $129 = $2;
   $130 = ($128|0)<=($129|0);
   if (!($130)) {
    break;
   }
   $131 = $nextl;
   $132 = $i;
   $133 = $xlsub;
   $134 = (($133) + ($132<<2)|0);
   HEAP32[$134>>2] = $131;
   $135 = $i;
   $136 = (($135) + 1)|0;
   $i = $136;
  }
  $137 = $ito;
  $nextl = $137;
 }
 $138 = $2;
 $139 = (($138) + 1)|0;
 $140 = $nsuper;
 $141 = (($140) + 1)|0;
 $142 = $xsup;
 $143 = (($142) + ($141<<2)|0);
 HEAP32[$143>>2] = $139;
 $144 = $nsuper;
 $145 = $2;
 $146 = (($145) + 1)|0;
 $147 = $supno;
 $148 = (($147) + ($146<<2)|0);
 HEAP32[$148>>2] = $144;
 $149 = $nextl;
 $150 = $2;
 $151 = $6;
 $152 = (($151) + ($150<<2)|0);
 HEAP32[$152>>2] = $149;
 $153 = $nextl;
 $154 = $2;
 $155 = (($154) + 1)|0;
 $156 = $xlsub;
 $157 = (($156) + ($155<<2)|0);
 HEAP32[$157>>2] = $153;
 $0 = 0;
 $158 = $0;
 STACKTOP = sp;return ($158|0);
}
function _dsnode_bmod($jcol,$jsupno,$fsupc,$dense,$tempv,$Glu,$stat) {
 $jcol = $jcol|0;
 $jsupno = $jsupno|0;
 $fsupc = $fsupc|0;
 $dense = $dense|0;
 $tempv = $tempv|0;
 $Glu = $Glu|0;
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0.0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0.0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0.0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0, $93 = 0;
 var $94 = 0.0, $95 = 0.0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iptr = 0, $irow = 0, $isub = 0, $lsub = 0, $luptr = 0, $lusup = 0, $nextlu = 0, $nrow = 0, $nsupc = 0, $nsupr = 0, $ops = 0, $ufirst = 0, $xlsub = 0;
 var $xlusup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $jcol;
 $1 = $jsupno;
 $2 = $fsupc;
 $3 = $dense;
 $4 = $tempv;
 $5 = $Glu;
 $6 = $stat;
 $7 = $6;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $ops = $9;
 $10 = $5;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $lsub = $12;
 $13 = $5;
 $14 = ((($13)) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $xlsub = $15;
 $16 = $5;
 $17 = ((($16)) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $lusup = $18;
 $19 = $5;
 $20 = ((($19)) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $xlusup = $21;
 $22 = $0;
 $23 = $xlusup;
 $24 = (($23) + ($22<<2)|0);
 $25 = HEAP32[$24>>2]|0;
 $nextlu = $25;
 $26 = $2;
 $27 = $xlsub;
 $28 = (($27) + ($26<<2)|0);
 $29 = HEAP32[$28>>2]|0;
 $isub = $29;
 while(1) {
  $30 = $isub;
  $31 = $2;
  $32 = (($31) + 1)|0;
  $33 = $xlsub;
  $34 = (($33) + ($32<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($30|0)<($35|0);
  if (!($36)) {
   break;
  }
  $37 = $isub;
  $38 = $lsub;
  $39 = (($38) + ($37<<2)|0);
  $40 = HEAP32[$39>>2]|0;
  $irow = $40;
  $41 = $irow;
  $42 = $3;
  $43 = (($42) + ($41<<3)|0);
  $44 = +HEAPF64[$43>>3];
  $45 = $nextlu;
  $46 = $lusup;
  $47 = (($46) + ($45<<3)|0);
  HEAPF64[$47>>3] = $44;
  $48 = $irow;
  $49 = $3;
  $50 = (($49) + ($48<<3)|0);
  HEAPF64[$50>>3] = 0.0;
  $51 = $nextlu;
  $52 = (($51) + 1)|0;
  $nextlu = $52;
  $53 = $isub;
  $54 = (($53) + 1)|0;
  $isub = $54;
 }
 $55 = $nextlu;
 $56 = $0;
 $57 = (($56) + 1)|0;
 $58 = $xlusup;
 $59 = (($58) + ($57<<2)|0);
 HEAP32[$59>>2] = $55;
 $60 = $2;
 $61 = $0;
 $62 = ($60|0)<($61|0);
 if (!($62)) {
  STACKTOP = sp;return 0;
 }
 $63 = $2;
 $64 = $xlusup;
 $65 = (($64) + ($63<<2)|0);
 $66 = HEAP32[$65>>2]|0;
 $luptr = $66;
 $67 = $2;
 $68 = (($67) + 1)|0;
 $69 = $xlsub;
 $70 = (($69) + ($68<<2)|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = $2;
 $73 = $xlsub;
 $74 = (($73) + ($72<<2)|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = (($71) - ($75))|0;
 $nsupr = $76;
 $77 = $0;
 $78 = $2;
 $79 = (($77) - ($78))|0;
 $nsupc = $79;
 $80 = $0;
 $81 = $xlusup;
 $82 = (($81) + ($80<<2)|0);
 $83 = HEAP32[$82>>2]|0;
 $ufirst = $83;
 $84 = $nsupr;
 $85 = $nsupc;
 $86 = (($84) - ($85))|0;
 $nrow = $86;
 $87 = $nsupc;
 $88 = $nsupc;
 $89 = (($88) - 1)|0;
 $90 = Math_imul($87, $89)|0;
 $91 = (+($90|0));
 $92 = $ops;
 $93 = ((($92)) + 52|0);
 $94 = +HEAPF32[$93>>2];
 $95 = $94 + $91;
 HEAPF32[$93>>2] = $95;
 $96 = $nrow;
 $97 = $96<<1;
 $98 = $nsupc;
 $99 = Math_imul($97, $98)|0;
 $100 = (+($99|0));
 $101 = $ops;
 $102 = ((($101)) + 56|0);
 $103 = +HEAPF32[$102>>2];
 $104 = $103 + $100;
 HEAPF32[$102>>2] = $104;
 $105 = $nsupr;
 $106 = $nsupc;
 $107 = $luptr;
 $108 = $lusup;
 $109 = (($108) + ($107<<3)|0);
 $110 = $ufirst;
 $111 = $lusup;
 $112 = (($111) + ($110<<3)|0);
 _dlsolve($105,$106,$109,$112);
 $113 = $nsupr;
 $114 = $nrow;
 $115 = $nsupc;
 $116 = $luptr;
 $117 = $nsupc;
 $118 = (($116) + ($117))|0;
 $119 = $lusup;
 $120 = (($119) + ($118<<3)|0);
 $121 = $ufirst;
 $122 = $lusup;
 $123 = (($122) + ($121<<3)|0);
 $124 = $4;
 _dmatvec($113,$114,$115,$120,$123,$124);
 $125 = $ufirst;
 $126 = $nsupc;
 $127 = (($125) + ($126))|0;
 $iptr = $127;
 $i = 0;
 while(1) {
  $128 = $i;
  $129 = $nrow;
  $130 = ($128|0)<($129|0);
  if (!($130)) {
   break;
  }
  $131 = $i;
  $132 = $4;
  $133 = (($132) + ($131<<3)|0);
  $134 = +HEAPF64[$133>>3];
  $135 = $iptr;
  $136 = (($135) + 1)|0;
  $iptr = $136;
  $137 = $lusup;
  $138 = (($137) + ($135<<3)|0);
  $139 = +HEAPF64[$138>>3];
  $140 = $139 - $134;
  HEAPF64[$138>>3] = $140;
  $141 = $i;
  $142 = $4;
  $143 = (($142) + ($141<<3)|0);
  HEAPF64[$143>>3] = 0.0;
  $144 = $i;
  $145 = (($144) + 1)|0;
  $i = $145;
 }
 STACKTOP = sp;return 0;
}
function _dpanel_dfs($m,$w,$jcol,$A,$perm_r,$nseg,$dense,$panel_lsub,$segrep,$repfnz,$xprune,$marker,$parent,$xplore,$Glu) {
 $m = $m|0;
 $w = $w|0;
 $jcol = $jcol|0;
 $A = $A|0;
 $perm_r = $perm_r|0;
 $nseg = $nseg|0;
 $dense = $dense|0;
 $panel_lsub = $panel_lsub|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $xprune = $xprune|0;
 $marker = $marker|0;
 $parent = $parent|0;
 $xplore = $xplore|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Astore = 0;
 var $a = 0, $asub = 0, $chmark = 0, $chperm = 0, $chrep = 0, $dense_col = 0, $jj = 0, $k = 0, $kchild = 0, $kmark = 0, $kpar = 0, $kperm = 0, $krep = 0, $krow = 0, $lsub = 0, $marker1 = 0, $maxdfs = 0, $myfnz = 0, $nextl_col = 0, $oldrep = 0;
 var $repfnz_col = 0, $supno = 0, $xa_begin = 0, $xa_end = 0, $xdfs = 0, $xlsub = 0, $xsup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $m;
 $1 = $w;
 $2 = $jcol;
 $3 = $A;
 $4 = $perm_r;
 $5 = $nseg;
 $6 = $dense;
 $7 = $panel_lsub;
 $8 = $segrep;
 $9 = $repfnz;
 $10 = $xprune;
 $11 = $marker;
 $12 = $parent;
 $13 = $xplore;
 $14 = $Glu;
 $15 = $3;
 $16 = ((($15)) + 20|0);
 $17 = HEAP32[$16>>2]|0;
 $Astore = $17;
 $18 = $Astore;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $a = $20;
 $21 = $Astore;
 $22 = ((($21)) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 $asub = $23;
 $24 = $Astore;
 $25 = ((($24)) + 12|0);
 $26 = HEAP32[$25>>2]|0;
 $xa_begin = $26;
 $27 = $Astore;
 $28 = ((($27)) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 $xa_end = $29;
 $30 = $11;
 $31 = $0;
 $32 = (($30) + ($31<<2)|0);
 $marker1 = $32;
 $33 = $9;
 $repfnz_col = $33;
 $34 = $6;
 $dense_col = $34;
 $35 = $5;
 HEAP32[$35>>2] = 0;
 $36 = $14;
 $37 = HEAP32[$36>>2]|0;
 $xsup = $37;
 $38 = $14;
 $39 = ((($38)) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $supno = $40;
 $41 = $14;
 $42 = ((($41)) + 8|0);
 $43 = HEAP32[$42>>2]|0;
 $lsub = $43;
 $44 = $14;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $xlsub = $46;
 $47 = $2;
 $jj = $47;
 while(1) {
  $48 = $jj;
  $49 = $2;
  $50 = $1;
  $51 = (($49) + ($50))|0;
  $52 = ($48|0)<($51|0);
  if (!($52)) {
   break;
  }
  $53 = $jj;
  $54 = $2;
  $55 = (($53) - ($54))|0;
  $56 = $0;
  $57 = Math_imul($55, $56)|0;
  $nextl_col = $57;
  $58 = $jj;
  $59 = $xa_begin;
  $60 = (($59) + ($58<<2)|0);
  $61 = HEAP32[$60>>2]|0;
  $k = $61;
  while(1) {
   $62 = $k;
   $63 = $jj;
   $64 = $xa_end;
   $65 = (($64) + ($63<<2)|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = ($62|0)<($66|0);
   if (!($67)) {
    break;
   }
   $68 = $k;
   $69 = $asub;
   $70 = (($69) + ($68<<2)|0);
   $71 = HEAP32[$70>>2]|0;
   $krow = $71;
   $72 = $k;
   $73 = $a;
   $74 = (($73) + ($72<<3)|0);
   $75 = +HEAPF64[$74>>3];
   $76 = $krow;
   $77 = $dense_col;
   $78 = (($77) + ($76<<3)|0);
   HEAPF64[$78>>3] = $75;
   $79 = $krow;
   $80 = $11;
   $81 = (($80) + ($79<<2)|0);
   $82 = HEAP32[$81>>2]|0;
   $kmark = $82;
   $83 = $kmark;
   $84 = $jj;
   $85 = ($83|0)==($84|0);
   L7: do {
    if (!($85)) {
     $86 = $jj;
     $87 = $krow;
     $88 = $11;
     $89 = (($88) + ($87<<2)|0);
     HEAP32[$89>>2] = $86;
     $90 = $krow;
     $91 = $4;
     $92 = (($91) + ($90<<2)|0);
     $93 = HEAP32[$92>>2]|0;
     $kperm = $93;
     $94 = $kperm;
     $95 = ($94|0)==(-1);
     if ($95) {
      $96 = $krow;
      $97 = $nextl_col;
      $98 = (($97) + 1)|0;
      $nextl_col = $98;
      $99 = $7;
      $100 = (($99) + ($97<<2)|0);
      HEAP32[$100>>2] = $96;
      break;
     }
     $101 = $kperm;
     $102 = $supno;
     $103 = (($102) + ($101<<2)|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = (($104) + 1)|0;
     $106 = $xsup;
     $107 = (($106) + ($105<<2)|0);
     $108 = HEAP32[$107>>2]|0;
     $109 = (($108) - 1)|0;
     $krep = $109;
     $110 = $krep;
     $111 = $repfnz_col;
     $112 = (($111) + ($110<<2)|0);
     $113 = HEAP32[$112>>2]|0;
     $myfnz = $113;
     $114 = $myfnz;
     $115 = ($114|0)!=(-1);
     if ($115) {
      $116 = $myfnz;
      $117 = $kperm;
      $118 = ($116|0)>($117|0);
      if (!($118)) {
       break;
      }
      $119 = $kperm;
      $120 = $krep;
      $121 = $repfnz_col;
      $122 = (($121) + ($120<<2)|0);
      HEAP32[$122>>2] = $119;
      break;
     }
     $oldrep = -1;
     $123 = $oldrep;
     $124 = $krep;
     $125 = $12;
     $126 = (($125) + ($124<<2)|0);
     HEAP32[$126>>2] = $123;
     $127 = $kperm;
     $128 = $krep;
     $129 = $repfnz_col;
     $130 = (($129) + ($128<<2)|0);
     HEAP32[$130>>2] = $127;
     $131 = $krep;
     $132 = $xlsub;
     $133 = (($132) + ($131<<2)|0);
     $134 = HEAP32[$133>>2]|0;
     $xdfs = $134;
     $135 = $krep;
     $136 = $10;
     $137 = (($136) + ($135<<2)|0);
     $138 = HEAP32[$137>>2]|0;
     $maxdfs = $138;
     while(1) {
      $139 = $xdfs;
      $140 = $maxdfs;
      $141 = ($139|0)<($140|0);
      if (!($141)) {
       $214 = $krep;
       $215 = $marker1;
       $216 = (($215) + ($214<<2)|0);
       $217 = HEAP32[$216>>2]|0;
       $218 = $2;
       $219 = ($217|0)<($218|0);
       if ($219) {
        $220 = $krep;
        $221 = $5;
        $222 = HEAP32[$221>>2]|0;
        $223 = $8;
        $224 = (($223) + ($222<<2)|0);
        HEAP32[$224>>2] = $220;
        $225 = $5;
        $226 = HEAP32[$225>>2]|0;
        $227 = (($226) + 1)|0;
        HEAP32[$225>>2] = $227;
        $228 = $jj;
        $229 = $krep;
        $230 = $marker1;
        $231 = (($230) + ($229<<2)|0);
        HEAP32[$231>>2] = $228;
       }
       $232 = $krep;
       $233 = $12;
       $234 = (($233) + ($232<<2)|0);
       $235 = HEAP32[$234>>2]|0;
       $kpar = $235;
       $236 = $kpar;
       $237 = ($236|0)==(-1);
       if ($237) {
        break L7;
       }
       $238 = $kpar;
       $krep = $238;
       $239 = $krep;
       $240 = $13;
       $241 = (($240) + ($239<<2)|0);
       $242 = HEAP32[$241>>2]|0;
       $xdfs = $242;
       $243 = $krep;
       $244 = $10;
       $245 = (($244) + ($243<<2)|0);
       $246 = HEAP32[$245>>2]|0;
       $maxdfs = $246;
       $247 = $kpar;
       $248 = ($247|0)!=(-1);
       if ($248) {
        continue;
       } else {
        break;
       }
      }
      $142 = $xdfs;
      $143 = $lsub;
      $144 = (($143) + ($142<<2)|0);
      $145 = HEAP32[$144>>2]|0;
      $kchild = $145;
      $146 = $xdfs;
      $147 = (($146) + 1)|0;
      $xdfs = $147;
      $148 = $kchild;
      $149 = $11;
      $150 = (($149) + ($148<<2)|0);
      $151 = HEAP32[$150>>2]|0;
      $chmark = $151;
      $152 = $chmark;
      $153 = $jj;
      $154 = ($152|0)!=($153|0);
      if (!($154)) {
       continue;
      }
      $155 = $jj;
      $156 = $kchild;
      $157 = $11;
      $158 = (($157) + ($156<<2)|0);
      HEAP32[$158>>2] = $155;
      $159 = $kchild;
      $160 = $4;
      $161 = (($160) + ($159<<2)|0);
      $162 = HEAP32[$161>>2]|0;
      $chperm = $162;
      $163 = $chperm;
      $164 = ($163|0)==(-1);
      if ($164) {
       $165 = $kchild;
       $166 = $nextl_col;
       $167 = (($166) + 1)|0;
       $nextl_col = $167;
       $168 = $7;
       $169 = (($168) + ($166<<2)|0);
       HEAP32[$169>>2] = $165;
       continue;
      }
      $170 = $chperm;
      $171 = $supno;
      $172 = (($171) + ($170<<2)|0);
      $173 = HEAP32[$172>>2]|0;
      $174 = (($173) + 1)|0;
      $175 = $xsup;
      $176 = (($175) + ($174<<2)|0);
      $177 = HEAP32[$176>>2]|0;
      $178 = (($177) - 1)|0;
      $chrep = $178;
      $179 = $chrep;
      $180 = $repfnz_col;
      $181 = (($180) + ($179<<2)|0);
      $182 = HEAP32[$181>>2]|0;
      $myfnz = $182;
      $183 = $myfnz;
      $184 = ($183|0)!=(-1);
      if (!($184)) {
       $192 = $xdfs;
       $193 = $krep;
       $194 = $13;
       $195 = (($194) + ($193<<2)|0);
       HEAP32[$195>>2] = $192;
       $196 = $krep;
       $oldrep = $196;
       $197 = $chrep;
       $krep = $197;
       $198 = $oldrep;
       $199 = $krep;
       $200 = $12;
       $201 = (($200) + ($199<<2)|0);
       HEAP32[$201>>2] = $198;
       $202 = $chperm;
       $203 = $krep;
       $204 = $repfnz_col;
       $205 = (($204) + ($203<<2)|0);
       HEAP32[$205>>2] = $202;
       $206 = $krep;
       $207 = $xlsub;
       $208 = (($207) + ($206<<2)|0);
       $209 = HEAP32[$208>>2]|0;
       $xdfs = $209;
       $210 = $krep;
       $211 = $10;
       $212 = (($211) + ($210<<2)|0);
       $213 = HEAP32[$212>>2]|0;
       $maxdfs = $213;
       continue;
      }
      $185 = $myfnz;
      $186 = $chperm;
      $187 = ($185|0)>($186|0);
      if (!($187)) {
       continue;
      }
      $188 = $chperm;
      $189 = $chrep;
      $190 = $repfnz_col;
      $191 = (($190) + ($189<<2)|0);
      HEAP32[$191>>2] = $188;
     }
    }
   } while(0);
   $249 = $k;
   $250 = (($249) + 1)|0;
   $k = $250;
  }
  $251 = $0;
  $252 = $repfnz_col;
  $253 = (($252) + ($251<<2)|0);
  $repfnz_col = $253;
  $254 = $0;
  $255 = $dense_col;
  $256 = (($255) + ($254<<3)|0);
  $dense_col = $256;
  $257 = $jj;
  $258 = (($257) + 1)|0;
  $jj = $258;
 }
 STACKTOP = sp;return;
}
function _dpanel_bmod($m,$w,$jcol,$nseg,$dense,$tempv,$segrep,$repfnz,$Glu,$stat) {
 $m = $m|0;
 $w = $w|0;
 $jcol = $jcol|0;
 $nseg = $nseg|0;
 $dense = $dense|0;
 $tempv = $tempv|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $Glu = $Glu|0;
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0.0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0.0, $164 = 0, $165 = 0, $166 = 0, $167 = 0.0, $168 = 0.0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0.0, $173 = 0.0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0.0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0.0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0.0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0.0, $214 = 0.0, $215 = 0.0, $216 = 0.0, $217 = 0.0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0.0, $243 = 0, $244 = 0, $245 = 0, $246 = 0.0, $247 = 0.0, $248 = 0.0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0.0, $253 = 0.0, $254 = 0.0, $255 = 0, $256 = 0, $257 = 0, $258 = 0.0, $259 = 0.0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0.0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0.0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0.0;
 var $279 = 0.0, $28 = 0, $280 = 0.0, $281 = 0.0, $282 = 0.0, $283 = 0.0, $284 = 0, $285 = 0, $286 = 0, $287 = 0.0, $288 = 0.0, $289 = 0.0, $29 = 0, $290 = 0.0, $291 = 0, $292 = 0, $293 = 0, $294 = 0.0, $295 = 0.0, $296 = 0.0;
 var $297 = 0.0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0.0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0.0, $333 = 0, $334 = 0, $335 = 0, $336 = 0.0, $337 = 0.0, $338 = 0.0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0.0, $343 = 0.0, $344 = 0.0, $345 = 0.0, $346 = 0, $347 = 0, $348 = 0, $349 = 0.0, $35 = 0;
 var $350 = 0.0, $351 = 0.0, $352 = 0, $353 = 0, $354 = 0, $355 = 0.0, $356 = 0.0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0.0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0.0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0.0, $499 = 0.0, $5 = 0, $50 = 0, $500 = 0.0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0.0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0.0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0.0, $607 = 0, $608 = 0, $609 = 0.0, $61 = 0, $610 = 0.0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0.0, $616 = 0, $617 = 0, $618 = 0.0, $619 = 0.0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0.0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0.0, $652 = 0, $653 = 0, $654 = 0, $655 = 0.0, $656 = 0.0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0.0, $661 = 0.0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0.0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0.0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0.0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0.0, $702 = 0.0, $703 = 0.0, $704 = 0.0, $705 = 0.0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0.0, $731 = 0, $732 = 0, $733 = 0, $734 = 0.0, $735 = 0.0, $736 = 0.0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0.0, $741 = 0.0, $742 = 0.0, $743 = 0, $744 = 0, $745 = 0, $746 = 0.0;
 var $747 = 0.0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0.0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0.0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0.0, $767 = 0.0, $768 = 0.0, $769 = 0.0, $77 = 0, $770 = 0.0, $771 = 0.0, $772 = 0, $773 = 0, $774 = 0, $775 = 0.0, $776 = 0.0, $777 = 0.0, $778 = 0.0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0.0;
 var $783 = 0.0, $784 = 0.0, $785 = 0.0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0.0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0.0, $821 = 0, $822 = 0, $823 = 0, $824 = 0.0, $825 = 0.0, $826 = 0.0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0.0, $831 = 0.0, $832 = 0.0, $833 = 0.0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0.0, $838 = 0.0, $839 = 0.0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0.0, $844 = 0.0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0.0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0;
 var $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0;
 var $909 = 0, $91 = 0, $910 = 0, $911 = 0.0, $912 = 0, $913 = 0, $914 = 0, $915 = 0.0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0;
 var $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0.0, $934 = 0, $935 = 0, $936 = 0, $937 = 0.0, $938 = 0.0, $939 = 0.0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0;
 var $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $MatvecTmp = 0, $TriTmp = 0, $block_nrow = 0;
 var $dense_col = 0, $fsupc = 0, $i = 0, $irow = 0, $isub = 0, $isub1 = 0, $jj = 0, $k = 0, $kfnz = 0, $krep = 0, $krep_ind = 0, $ksub = 0, $ldaTmp = 0, $lptr = 0, $lsub = 0, $luptr = 0, $luptr1 = 0, $luptr2 = 0, $lusup = 0, $no_zeros = 0;
 var $nrow = 0, $nsupc = 0, $nsupr = 0, $one = 0.0, $ops = 0, $r_hi = 0, $r_ind = 0, $repfnz_col = 0, $segsze = 0, $supno = 0, $tempv1 = 0, $ukj = 0.0, $ukj1 = 0.0, $ukj2 = 0.0, $xlsub = 0, $xlusup = 0, $xsup = 0, $zero = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $m;
 $1 = $w;
 $2 = $jcol;
 $3 = $nseg;
 $4 = $dense;
 $5 = $tempv;
 $6 = $segrep;
 $7 = $repfnz;
 $8 = $Glu;
 $9 = $stat;
 $zero = 0.0;
 $one = 1.0;
 $10 = $9;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $ops = $12;
 $13 = $8;
 $14 = HEAP32[$13>>2]|0;
 $xsup = $14;
 $15 = $8;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $supno = $17;
 $18 = $8;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $lsub = $20;
 $21 = $8;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $xlsub = $23;
 $24 = $8;
 $25 = ((($24)) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $lusup = $26;
 $27 = $8;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 $xlusup = $29;
 $30 = HEAP32[1808>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = (_sp_ienv(3)|0);
  $33 = (_sp_ienv(7)|0);
  $34 = ($32|0)>($33|0);
  if ($34) {
   $35 = (_sp_ienv(3)|0);
   $37 = $35;
  } else {
   $36 = (_sp_ienv(7)|0);
   $37 = $36;
  }
  HEAP32[1816>>2] = $37;
  $38 = (_sp_ienv(4)|0);
  HEAP32[1824>>2] = $38;
  $39 = (_sp_ienv(5)|0);
  HEAP32[1832>>2] = $39;
  HEAP32[1808>>2] = 0;
 }
 $40 = HEAP32[1816>>2]|0;
 $41 = HEAP32[1824>>2]|0;
 $42 = (($40) + ($41))|0;
 $ldaTmp = $42;
 $43 = $3;
 $44 = (($43) - 1)|0;
 $k = $44;
 $ksub = 0;
 while(1) {
  $45 = $ksub;
  $46 = $3;
  $47 = ($45|0)<($46|0);
  if (!($47)) {
   break;
  }
  $48 = $k;
  $49 = (($48) + -1)|0;
  $k = $49;
  $50 = $6;
  $51 = (($50) + ($48<<2)|0);
  $52 = HEAP32[$51>>2]|0;
  $krep = $52;
  $53 = $krep;
  $54 = $supno;
  $55 = (($54) + ($53<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $xsup;
  $58 = (($57) + ($56<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  $fsupc = $59;
  $60 = $krep;
  $61 = $fsupc;
  $62 = (($60) - ($61))|0;
  $63 = (($62) + 1)|0;
  $nsupc = $63;
  $64 = $fsupc;
  $65 = (($64) + 1)|0;
  $66 = $xlsub;
  $67 = (($66) + ($65<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = $fsupc;
  $70 = $xlsub;
  $71 = (($70) + ($69<<2)|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($68) - ($72))|0;
  $nsupr = $73;
  $74 = $nsupr;
  $75 = $nsupc;
  $76 = (($74) - ($75))|0;
  $nrow = $76;
  $77 = $fsupc;
  $78 = $xlsub;
  $79 = (($78) + ($77<<2)|0);
  $80 = HEAP32[$79>>2]|0;
  $lptr = $80;
  $81 = $lptr;
  $82 = $nsupc;
  $83 = (($81) + ($82))|0;
  $84 = (($83) - 1)|0;
  $krep_ind = $84;
  $85 = $7;
  $repfnz_col = $85;
  $86 = $4;
  $dense_col = $86;
  $87 = $nsupc;
  $88 = HEAP32[1832>>2]|0;
  $89 = ($87|0)>=($88|0);
  L11: do {
   if ($89) {
    $90 = $nrow;
    $91 = HEAP32[1824>>2]|0;
    $92 = ($90|0)>($91|0);
    if ($92) {
     $93 = $5;
     $TriTmp = $93;
     $94 = $2;
     $jj = $94;
     while(1) {
      $95 = $jj;
      $96 = $2;
      $97 = $1;
      $98 = (($96) + ($97))|0;
      $99 = ($95|0)<($98|0);
      if (!($99)) {
       break;
      }
      $100 = $krep;
      $101 = $repfnz_col;
      $102 = (($101) + ($100<<2)|0);
      $103 = HEAP32[$102>>2]|0;
      $kfnz = $103;
      $104 = $kfnz;
      $105 = ($104|0)==(-1);
      L17: do {
       if (!($105)) {
        $106 = $krep;
        $107 = $kfnz;
        $108 = (($106) - ($107))|0;
        $109 = (($108) + 1)|0;
        $segsze = $109;
        $110 = $fsupc;
        $111 = $xlusup;
        $112 = (($111) + ($110<<2)|0);
        $113 = HEAP32[$112>>2]|0;
        $luptr = $113;
        $114 = $segsze;
        $115 = $segsze;
        $116 = (($115) - 1)|0;
        $117 = Math_imul($114, $116)|0;
        $118 = (+($117|0));
        $119 = $ops;
        $120 = ((($119)) + 52|0);
        $121 = +HEAPF32[$120>>2];
        $122 = $121 + $118;
        HEAPF32[$120>>2] = $122;
        $123 = $nrow;
        $124 = $123<<1;
        $125 = $segsze;
        $126 = Math_imul($124, $125)|0;
        $127 = (+($126|0));
        $128 = $ops;
        $129 = ((($128)) + 56|0);
        $130 = +HEAPF32[$129>>2];
        $131 = $130 + $127;
        HEAPF32[$129>>2] = $131;
        $132 = $segsze;
        $133 = ($132|0)==(1);
        if ($133) {
         $134 = $krep_ind;
         $135 = $lsub;
         $136 = (($135) + ($134<<2)|0);
         $137 = HEAP32[$136>>2]|0;
         $138 = $dense_col;
         $139 = (($138) + ($137<<3)|0);
         $140 = +HEAPF64[$139>>3];
         $ukj = $140;
         $141 = $nsupr;
         $142 = $nsupc;
         $143 = (($142) - 1)|0;
         $144 = Math_imul($141, $143)|0;
         $145 = $nsupc;
         $146 = (($144) + ($145))|0;
         $147 = $luptr;
         $148 = (($147) + ($146))|0;
         $luptr = $148;
         $149 = $lptr;
         $150 = $nsupc;
         $151 = (($149) + ($150))|0;
         $i = $151;
         while(1) {
          $152 = $i;
          $153 = $fsupc;
          $154 = (($153) + 1)|0;
          $155 = $xlsub;
          $156 = (($155) + ($154<<2)|0);
          $157 = HEAP32[$156>>2]|0;
          $158 = ($152|0)<($157|0);
          if (!($158)) {
           break L17;
          }
          $159 = $i;
          $160 = $lsub;
          $161 = (($160) + ($159<<2)|0);
          $162 = HEAP32[$161>>2]|0;
          $irow = $162;
          $163 = $ukj;
          $164 = $luptr;
          $165 = $lusup;
          $166 = (($165) + ($164<<3)|0);
          $167 = +HEAPF64[$166>>3];
          $168 = $163 * $167;
          $169 = $irow;
          $170 = $dense_col;
          $171 = (($170) + ($169<<3)|0);
          $172 = +HEAPF64[$171>>3];
          $173 = $172 - $168;
          HEAPF64[$171>>3] = $173;
          $174 = $luptr;
          $175 = (($174) + 1)|0;
          $luptr = $175;
          $176 = $i;
          $177 = (($176) + 1)|0;
          $i = $177;
         }
        }
        $178 = $segsze;
        $179 = ($178|0)<=(3);
        if (!($179)) {
         $359 = $kfnz;
         $360 = $fsupc;
         $361 = (($359) - ($360))|0;
         $no_zeros = $361;
         $362 = $lptr;
         $363 = $no_zeros;
         $364 = (($362) + ($363))|0;
         $isub = $364;
         $i = 0;
         while(1) {
          $365 = $i;
          $366 = $segsze;
          $367 = ($365|0)<($366|0);
          if (!($367)) {
           break;
          }
          $368 = $isub;
          $369 = $lsub;
          $370 = (($369) + ($368<<2)|0);
          $371 = HEAP32[$370>>2]|0;
          $irow = $371;
          $372 = $irow;
          $373 = $dense_col;
          $374 = (($373) + ($372<<3)|0);
          $375 = +HEAPF64[$374>>3];
          $376 = $i;
          $377 = $TriTmp;
          $378 = (($377) + ($376<<3)|0);
          HEAPF64[$378>>3] = $375;
          $379 = $isub;
          $380 = (($379) + 1)|0;
          $isub = $380;
          $381 = $i;
          $382 = (($381) + 1)|0;
          $i = $382;
         }
         $383 = $nsupr;
         $384 = $no_zeros;
         $385 = Math_imul($383, $384)|0;
         $386 = $no_zeros;
         $387 = (($385) + ($386))|0;
         $388 = $luptr;
         $389 = (($388) + ($387))|0;
         $luptr = $389;
         $390 = $nsupr;
         $391 = $segsze;
         $392 = $luptr;
         $393 = $lusup;
         $394 = (($393) + ($392<<3)|0);
         $395 = $TriTmp;
         _dlsolve($390,$391,$394,$395);
         break;
        }
        $180 = $krep_ind;
        $181 = $lsub;
        $182 = (($181) + ($180<<2)|0);
        $183 = HEAP32[$182>>2]|0;
        $184 = $dense_col;
        $185 = (($184) + ($183<<3)|0);
        $186 = +HEAPF64[$185>>3];
        $ukj = $186;
        $187 = $krep_ind;
        $188 = (($187) - 1)|0;
        $189 = $lsub;
        $190 = (($189) + ($188<<2)|0);
        $191 = HEAP32[$190>>2]|0;
        $192 = $dense_col;
        $193 = (($192) + ($191<<3)|0);
        $194 = +HEAPF64[$193>>3];
        $ukj1 = $194;
        $195 = $nsupr;
        $196 = $nsupc;
        $197 = (($196) - 1)|0;
        $198 = Math_imul($195, $197)|0;
        $199 = $nsupc;
        $200 = (($198) + ($199))|0;
        $201 = (($200) - 1)|0;
        $202 = $luptr;
        $203 = (($202) + ($201))|0;
        $luptr = $203;
        $204 = $luptr;
        $205 = $nsupr;
        $206 = (($204) - ($205))|0;
        $luptr1 = $206;
        $207 = $segsze;
        $208 = ($207|0)==(2);
        if ($208) {
         $209 = $ukj1;
         $210 = $luptr1;
         $211 = $lusup;
         $212 = (($211) + ($210<<3)|0);
         $213 = +HEAPF64[$212>>3];
         $214 = $209 * $213;
         $215 = $ukj;
         $216 = $215 - $214;
         $ukj = $216;
         $217 = $ukj;
         $218 = $krep_ind;
         $219 = $lsub;
         $220 = (($219) + ($218<<2)|0);
         $221 = HEAP32[$220>>2]|0;
         $222 = $dense_col;
         $223 = (($222) + ($221<<3)|0);
         HEAPF64[$223>>3] = $217;
         $224 = $lptr;
         $225 = $nsupc;
         $226 = (($224) + ($225))|0;
         $i = $226;
         while(1) {
          $227 = $i;
          $228 = $fsupc;
          $229 = (($228) + 1)|0;
          $230 = $xlsub;
          $231 = (($230) + ($229<<2)|0);
          $232 = HEAP32[$231>>2]|0;
          $233 = ($227|0)<($232|0);
          if (!($233)) {
           break L17;
          }
          $234 = $i;
          $235 = $lsub;
          $236 = (($235) + ($234<<2)|0);
          $237 = HEAP32[$236>>2]|0;
          $irow = $237;
          $238 = $luptr;
          $239 = (($238) + 1)|0;
          $luptr = $239;
          $240 = $luptr1;
          $241 = (($240) + 1)|0;
          $luptr1 = $241;
          $242 = $ukj;
          $243 = $luptr;
          $244 = $lusup;
          $245 = (($244) + ($243<<3)|0);
          $246 = +HEAPF64[$245>>3];
          $247 = $242 * $246;
          $248 = $ukj1;
          $249 = $luptr1;
          $250 = $lusup;
          $251 = (($250) + ($249<<3)|0);
          $252 = +HEAPF64[$251>>3];
          $253 = $248 * $252;
          $254 = $247 + $253;
          $255 = $irow;
          $256 = $dense_col;
          $257 = (($256) + ($255<<3)|0);
          $258 = +HEAPF64[$257>>3];
          $259 = $258 - $254;
          HEAPF64[$257>>3] = $259;
          $260 = $i;
          $261 = (($260) + 1)|0;
          $i = $261;
         }
        } else {
         $262 = $krep_ind;
         $263 = (($262) - 2)|0;
         $264 = $lsub;
         $265 = (($264) + ($263<<2)|0);
         $266 = HEAP32[$265>>2]|0;
         $267 = $dense_col;
         $268 = (($267) + ($266<<3)|0);
         $269 = +HEAPF64[$268>>3];
         $ukj2 = $269;
         $270 = $luptr1;
         $271 = $nsupr;
         $272 = (($270) - ($271))|0;
         $luptr2 = $272;
         $273 = $ukj2;
         $274 = $luptr2;
         $275 = (($274) - 1)|0;
         $276 = $lusup;
         $277 = (($276) + ($275<<3)|0);
         $278 = +HEAPF64[$277>>3];
         $279 = $273 * $278;
         $280 = $ukj1;
         $281 = $280 - $279;
         $ukj1 = $281;
         $282 = $ukj;
         $283 = $ukj1;
         $284 = $luptr1;
         $285 = $lusup;
         $286 = (($285) + ($284<<3)|0);
         $287 = +HEAPF64[$286>>3];
         $288 = $283 * $287;
         $289 = $282 - $288;
         $290 = $ukj2;
         $291 = $luptr2;
         $292 = $lusup;
         $293 = (($292) + ($291<<3)|0);
         $294 = +HEAPF64[$293>>3];
         $295 = $290 * $294;
         $296 = $289 - $295;
         $ukj = $296;
         $297 = $ukj;
         $298 = $krep_ind;
         $299 = $lsub;
         $300 = (($299) + ($298<<2)|0);
         $301 = HEAP32[$300>>2]|0;
         $302 = $dense_col;
         $303 = (($302) + ($301<<3)|0);
         HEAPF64[$303>>3] = $297;
         $304 = $ukj1;
         $305 = $krep_ind;
         $306 = (($305) - 1)|0;
         $307 = $lsub;
         $308 = (($307) + ($306<<2)|0);
         $309 = HEAP32[$308>>2]|0;
         $310 = $dense_col;
         $311 = (($310) + ($309<<3)|0);
         HEAPF64[$311>>3] = $304;
         $312 = $lptr;
         $313 = $nsupc;
         $314 = (($312) + ($313))|0;
         $i = $314;
         while(1) {
          $315 = $i;
          $316 = $fsupc;
          $317 = (($316) + 1)|0;
          $318 = $xlsub;
          $319 = (($318) + ($317<<2)|0);
          $320 = HEAP32[$319>>2]|0;
          $321 = ($315|0)<($320|0);
          if (!($321)) {
           break L17;
          }
          $322 = $i;
          $323 = $lsub;
          $324 = (($323) + ($322<<2)|0);
          $325 = HEAP32[$324>>2]|0;
          $irow = $325;
          $326 = $luptr;
          $327 = (($326) + 1)|0;
          $luptr = $327;
          $328 = $luptr1;
          $329 = (($328) + 1)|0;
          $luptr1 = $329;
          $330 = $luptr2;
          $331 = (($330) + 1)|0;
          $luptr2 = $331;
          $332 = $ukj;
          $333 = $luptr;
          $334 = $lusup;
          $335 = (($334) + ($333<<3)|0);
          $336 = +HEAPF64[$335>>3];
          $337 = $332 * $336;
          $338 = $ukj1;
          $339 = $luptr1;
          $340 = $lusup;
          $341 = (($340) + ($339<<3)|0);
          $342 = +HEAPF64[$341>>3];
          $343 = $338 * $342;
          $344 = $337 + $343;
          $345 = $ukj2;
          $346 = $luptr2;
          $347 = $lusup;
          $348 = (($347) + ($346<<3)|0);
          $349 = +HEAPF64[$348>>3];
          $350 = $345 * $349;
          $351 = $344 + $350;
          $352 = $irow;
          $353 = $dense_col;
          $354 = (($353) + ($352<<3)|0);
          $355 = +HEAPF64[$354>>3];
          $356 = $355 - $351;
          HEAPF64[$354>>3] = $356;
          $357 = $i;
          $358 = (($357) + 1)|0;
          $i = $358;
         }
        }
       }
      } while(0);
      $396 = $jj;
      $397 = (($396) + 1)|0;
      $jj = $397;
      $398 = $0;
      $399 = $repfnz_col;
      $400 = (($399) + ($398<<2)|0);
      $repfnz_col = $400;
      $401 = $0;
      $402 = $dense_col;
      $403 = (($402) + ($401<<3)|0);
      $dense_col = $403;
      $404 = $ldaTmp;
      $405 = $TriTmp;
      $406 = (($405) + ($404<<3)|0);
      $TriTmp = $406;
     }
     $r_ind = 0;
     while(1) {
      $407 = $r_ind;
      $408 = $nrow;
      $409 = ($407|0)<($408|0);
      if (!($409)) {
       break;
      }
      $410 = $nrow;
      $411 = $r_ind;
      $412 = HEAP32[1824>>2]|0;
      $413 = (($411) + ($412))|0;
      $414 = ($410|0)<($413|0);
      if ($414) {
       $415 = $nrow;
       $419 = $415;
      } else {
       $416 = $r_ind;
       $417 = HEAP32[1824>>2]|0;
       $418 = (($416) + ($417))|0;
       $419 = $418;
      }
      $r_hi = $419;
      $420 = HEAP32[1824>>2]|0;
      $421 = $r_hi;
      $422 = $r_ind;
      $423 = (($421) - ($422))|0;
      $424 = ($420|0)<($423|0);
      if ($424) {
       $425 = HEAP32[1824>>2]|0;
       $429 = $425;
      } else {
       $426 = $r_hi;
       $427 = $r_ind;
       $428 = (($426) - ($427))|0;
       $429 = $428;
      }
      $block_nrow = $429;
      $430 = $fsupc;
      $431 = $xlusup;
      $432 = (($431) + ($430<<2)|0);
      $433 = HEAP32[$432>>2]|0;
      $434 = $nsupc;
      $435 = (($433) + ($434))|0;
      $436 = $r_ind;
      $437 = (($435) + ($436))|0;
      $luptr = $437;
      $438 = $lptr;
      $439 = $nsupc;
      $440 = (($438) + ($439))|0;
      $441 = $r_ind;
      $442 = (($440) + ($441))|0;
      $isub1 = $442;
      $443 = $7;
      $repfnz_col = $443;
      $444 = $5;
      $TriTmp = $444;
      $445 = $4;
      $dense_col = $445;
      $446 = $2;
      $jj = $446;
      while(1) {
       $447 = $jj;
       $448 = $2;
       $449 = $1;
       $450 = (($448) + ($449))|0;
       $451 = ($447|0)<($450|0);
       if (!($451)) {
        break;
       }
       $452 = $krep;
       $453 = $repfnz_col;
       $454 = (($453) + ($452<<2)|0);
       $455 = HEAP32[$454>>2]|0;
       $kfnz = $455;
       $456 = $kfnz;
       $457 = ($456|0)==(-1);
       L57: do {
        if (!($457)) {
         $458 = $krep;
         $459 = $kfnz;
         $460 = (($458) - ($459))|0;
         $461 = (($460) + 1)|0;
         $segsze = $461;
         $462 = $segsze;
         $463 = ($462|0)<=(3);
         if (!($463)) {
          $464 = $kfnz;
          $465 = $fsupc;
          $466 = (($464) - ($465))|0;
          $no_zeros = $466;
          $467 = $luptr;
          $468 = $nsupr;
          $469 = $no_zeros;
          $470 = Math_imul($468, $469)|0;
          $471 = (($467) + ($470))|0;
          $luptr1 = $471;
          $472 = HEAP32[1816>>2]|0;
          $473 = $TriTmp;
          $474 = (($473) + ($472<<3)|0);
          $MatvecTmp = $474;
          $475 = $nsupr;
          $476 = $block_nrow;
          $477 = $segsze;
          $478 = $luptr1;
          $479 = $lusup;
          $480 = (($479) + ($478<<3)|0);
          $481 = $TriTmp;
          $482 = $MatvecTmp;
          _dmatvec($475,$476,$477,$480,$481,$482);
          $483 = $isub1;
          $isub = $483;
          $i = 0;
          while(1) {
           $484 = $i;
           $485 = $block_nrow;
           $486 = ($484|0)<($485|0);
           if (!($486)) {
            break L57;
           }
           $487 = $isub;
           $488 = $lsub;
           $489 = (($488) + ($487<<2)|0);
           $490 = HEAP32[$489>>2]|0;
           $irow = $490;
           $491 = $i;
           $492 = $MatvecTmp;
           $493 = (($492) + ($491<<3)|0);
           $494 = +HEAPF64[$493>>3];
           $495 = $irow;
           $496 = $dense_col;
           $497 = (($496) + ($495<<3)|0);
           $498 = +HEAPF64[$497>>3];
           $499 = $498 - $494;
           HEAPF64[$497>>3] = $499;
           $500 = $zero;
           $501 = $i;
           $502 = $MatvecTmp;
           $503 = (($502) + ($501<<3)|0);
           HEAPF64[$503>>3] = $500;
           $504 = $isub;
           $505 = (($504) + 1)|0;
           $isub = $505;
           $506 = $i;
           $507 = (($506) + 1)|0;
           $i = $507;
          }
         }
        }
       } while(0);
       $508 = $jj;
       $509 = (($508) + 1)|0;
       $jj = $509;
       $510 = $0;
       $511 = $repfnz_col;
       $512 = (($511) + ($510<<2)|0);
       $repfnz_col = $512;
       $513 = $0;
       $514 = $dense_col;
       $515 = (($514) + ($513<<3)|0);
       $dense_col = $515;
       $516 = $ldaTmp;
       $517 = $TriTmp;
       $518 = (($517) + ($516<<3)|0);
       $TriTmp = $518;
      }
      $519 = HEAP32[1824>>2]|0;
      $520 = $r_ind;
      $521 = (($520) + ($519))|0;
      $r_ind = $521;
     }
     $522 = $7;
     $repfnz_col = $522;
     $523 = $5;
     $TriTmp = $523;
     $524 = $4;
     $dense_col = $524;
     $525 = $2;
     $jj = $525;
     while(1) {
      $526 = $jj;
      $527 = $2;
      $528 = $1;
      $529 = (($527) + ($528))|0;
      $530 = ($526|0)<($529|0);
      if (!($530)) {
       break L11;
      }
      $531 = $krep;
      $532 = $repfnz_col;
      $533 = (($532) + ($531<<2)|0);
      $534 = HEAP32[$533>>2]|0;
      $kfnz = $534;
      $535 = $kfnz;
      $536 = ($535|0)==(-1);
      L69: do {
       if (!($536)) {
        $537 = $krep;
        $538 = $kfnz;
        $539 = (($537) - ($538))|0;
        $540 = (($539) + 1)|0;
        $segsze = $540;
        $541 = $segsze;
        $542 = ($541|0)<=(3);
        if (!($542)) {
         $543 = $kfnz;
         $544 = $fsupc;
         $545 = (($543) - ($544))|0;
         $no_zeros = $545;
         $546 = $lptr;
         $547 = $no_zeros;
         $548 = (($546) + ($547))|0;
         $isub = $548;
         $i = 0;
         while(1) {
          $549 = $i;
          $550 = $segsze;
          $551 = ($549|0)<($550|0);
          if (!($551)) {
           break L69;
          }
          $552 = $isub;
          $553 = $lsub;
          $554 = (($553) + ($552<<2)|0);
          $555 = HEAP32[$554>>2]|0;
          $irow = $555;
          $556 = $i;
          $557 = $TriTmp;
          $558 = (($557) + ($556<<3)|0);
          $559 = +HEAPF64[$558>>3];
          $560 = $irow;
          $561 = $dense_col;
          $562 = (($561) + ($560<<3)|0);
          HEAPF64[$562>>3] = $559;
          $563 = $zero;
          $564 = $i;
          $565 = $TriTmp;
          $566 = (($565) + ($564<<3)|0);
          HEAPF64[$566>>3] = $563;
          $567 = $isub;
          $568 = (($567) + 1)|0;
          $isub = $568;
          $569 = $i;
          $570 = (($569) + 1)|0;
          $i = $570;
         }
        }
       }
      } while(0);
      $571 = $jj;
      $572 = (($571) + 1)|0;
      $jj = $572;
      $573 = $0;
      $574 = $repfnz_col;
      $575 = (($574) + ($573<<2)|0);
      $repfnz_col = $575;
      $576 = $0;
      $577 = $dense_col;
      $578 = (($577) + ($576<<3)|0);
      $dense_col = $578;
      $579 = $ldaTmp;
      $580 = $TriTmp;
      $581 = (($580) + ($579<<3)|0);
      $TriTmp = $581;
     }
    } else {
     label = 55;
    }
   } else {
    label = 55;
   }
  } while(0);
  L76: do {
   if ((label|0) == 55) {
    label = 0;
    $582 = $2;
    $jj = $582;
    while(1) {
     $583 = $jj;
     $584 = $2;
     $585 = $1;
     $586 = (($584) + ($585))|0;
     $587 = ($583|0)<($586|0);
     if (!($587)) {
      break L76;
     }
     $588 = $krep;
     $589 = $repfnz_col;
     $590 = (($589) + ($588<<2)|0);
     $591 = HEAP32[$590>>2]|0;
     $kfnz = $591;
     $592 = $kfnz;
     $593 = ($592|0)==(-1);
     L81: do {
      if (!($593)) {
       $594 = $krep;
       $595 = $kfnz;
       $596 = (($594) - ($595))|0;
       $597 = (($596) + 1)|0;
       $segsze = $597;
       $598 = $fsupc;
       $599 = $xlusup;
       $600 = (($599) + ($598<<2)|0);
       $601 = HEAP32[$600>>2]|0;
       $luptr = $601;
       $602 = $segsze;
       $603 = $segsze;
       $604 = (($603) - 1)|0;
       $605 = Math_imul($602, $604)|0;
       $606 = (+($605|0));
       $607 = $ops;
       $608 = ((($607)) + 52|0);
       $609 = +HEAPF32[$608>>2];
       $610 = $609 + $606;
       HEAPF32[$608>>2] = $610;
       $611 = $nrow;
       $612 = $611<<1;
       $613 = $segsze;
       $614 = Math_imul($612, $613)|0;
       $615 = (+($614|0));
       $616 = $ops;
       $617 = ((($616)) + 56|0);
       $618 = +HEAPF32[$617>>2];
       $619 = $618 + $615;
       HEAPF32[$617>>2] = $619;
       $620 = $segsze;
       $621 = ($620|0)==(1);
       if ($621) {
        $622 = $krep_ind;
        $623 = $lsub;
        $624 = (($623) + ($622<<2)|0);
        $625 = HEAP32[$624>>2]|0;
        $626 = $dense_col;
        $627 = (($626) + ($625<<3)|0);
        $628 = +HEAPF64[$627>>3];
        $ukj = $628;
        $629 = $nsupr;
        $630 = $nsupc;
        $631 = (($630) - 1)|0;
        $632 = Math_imul($629, $631)|0;
        $633 = $nsupc;
        $634 = (($632) + ($633))|0;
        $635 = $luptr;
        $636 = (($635) + ($634))|0;
        $luptr = $636;
        $637 = $lptr;
        $638 = $nsupc;
        $639 = (($637) + ($638))|0;
        $i = $639;
        while(1) {
         $640 = $i;
         $641 = $fsupc;
         $642 = (($641) + 1)|0;
         $643 = $xlsub;
         $644 = (($643) + ($642<<2)|0);
         $645 = HEAP32[$644>>2]|0;
         $646 = ($640|0)<($645|0);
         if (!($646)) {
          break L81;
         }
         $647 = $i;
         $648 = $lsub;
         $649 = (($648) + ($647<<2)|0);
         $650 = HEAP32[$649>>2]|0;
         $irow = $650;
         $651 = $ukj;
         $652 = $luptr;
         $653 = $lusup;
         $654 = (($653) + ($652<<3)|0);
         $655 = +HEAPF64[$654>>3];
         $656 = $651 * $655;
         $657 = $irow;
         $658 = $dense_col;
         $659 = (($658) + ($657<<3)|0);
         $660 = +HEAPF64[$659>>3];
         $661 = $660 - $656;
         HEAPF64[$659>>3] = $661;
         $662 = $luptr;
         $663 = (($662) + 1)|0;
         $luptr = $663;
         $664 = $i;
         $665 = (($664) + 1)|0;
         $i = $665;
        }
       }
       $666 = $segsze;
       $667 = ($666|0)<=(3);
       if ($667) {
        $668 = $krep_ind;
        $669 = $lsub;
        $670 = (($669) + ($668<<2)|0);
        $671 = HEAP32[$670>>2]|0;
        $672 = $dense_col;
        $673 = (($672) + ($671<<3)|0);
        $674 = +HEAPF64[$673>>3];
        $ukj = $674;
        $675 = $nsupr;
        $676 = $nsupc;
        $677 = (($676) - 1)|0;
        $678 = Math_imul($675, $677)|0;
        $679 = $nsupc;
        $680 = (($678) + ($679))|0;
        $681 = (($680) - 1)|0;
        $682 = $luptr;
        $683 = (($682) + ($681))|0;
        $luptr = $683;
        $684 = $krep_ind;
        $685 = (($684) - 1)|0;
        $686 = $lsub;
        $687 = (($686) + ($685<<2)|0);
        $688 = HEAP32[$687>>2]|0;
        $689 = $dense_col;
        $690 = (($689) + ($688<<3)|0);
        $691 = +HEAPF64[$690>>3];
        $ukj1 = $691;
        $692 = $luptr;
        $693 = $nsupr;
        $694 = (($692) - ($693))|0;
        $luptr1 = $694;
        $695 = $segsze;
        $696 = ($695|0)==(2);
        if ($696) {
         $697 = $ukj1;
         $698 = $luptr1;
         $699 = $lusup;
         $700 = (($699) + ($698<<3)|0);
         $701 = +HEAPF64[$700>>3];
         $702 = $697 * $701;
         $703 = $ukj;
         $704 = $703 - $702;
         $ukj = $704;
         $705 = $ukj;
         $706 = $krep_ind;
         $707 = $lsub;
         $708 = (($707) + ($706<<2)|0);
         $709 = HEAP32[$708>>2]|0;
         $710 = $dense_col;
         $711 = (($710) + ($709<<3)|0);
         HEAPF64[$711>>3] = $705;
         $712 = $lptr;
         $713 = $nsupc;
         $714 = (($712) + ($713))|0;
         $i = $714;
         while(1) {
          $715 = $i;
          $716 = $fsupc;
          $717 = (($716) + 1)|0;
          $718 = $xlsub;
          $719 = (($718) + ($717<<2)|0);
          $720 = HEAP32[$719>>2]|0;
          $721 = ($715|0)<($720|0);
          if (!($721)) {
           break L81;
          }
          $722 = $i;
          $723 = $lsub;
          $724 = (($723) + ($722<<2)|0);
          $725 = HEAP32[$724>>2]|0;
          $irow = $725;
          $726 = $luptr;
          $727 = (($726) + 1)|0;
          $luptr = $727;
          $728 = $luptr1;
          $729 = (($728) + 1)|0;
          $luptr1 = $729;
          $730 = $ukj;
          $731 = $luptr;
          $732 = $lusup;
          $733 = (($732) + ($731<<3)|0);
          $734 = +HEAPF64[$733>>3];
          $735 = $730 * $734;
          $736 = $ukj1;
          $737 = $luptr1;
          $738 = $lusup;
          $739 = (($738) + ($737<<3)|0);
          $740 = +HEAPF64[$739>>3];
          $741 = $736 * $740;
          $742 = $735 + $741;
          $743 = $irow;
          $744 = $dense_col;
          $745 = (($744) + ($743<<3)|0);
          $746 = +HEAPF64[$745>>3];
          $747 = $746 - $742;
          HEAPF64[$745>>3] = $747;
          $748 = $i;
          $749 = (($748) + 1)|0;
          $i = $749;
         }
        } else {
         $750 = $krep_ind;
         $751 = (($750) - 2)|0;
         $752 = $lsub;
         $753 = (($752) + ($751<<2)|0);
         $754 = HEAP32[$753>>2]|0;
         $755 = $dense_col;
         $756 = (($755) + ($754<<3)|0);
         $757 = +HEAPF64[$756>>3];
         $ukj2 = $757;
         $758 = $luptr1;
         $759 = $nsupr;
         $760 = (($758) - ($759))|0;
         $luptr2 = $760;
         $761 = $ukj2;
         $762 = $luptr2;
         $763 = (($762) - 1)|0;
         $764 = $lusup;
         $765 = (($764) + ($763<<3)|0);
         $766 = +HEAPF64[$765>>3];
         $767 = $761 * $766;
         $768 = $ukj1;
         $769 = $768 - $767;
         $ukj1 = $769;
         $770 = $ukj;
         $771 = $ukj1;
         $772 = $luptr1;
         $773 = $lusup;
         $774 = (($773) + ($772<<3)|0);
         $775 = +HEAPF64[$774>>3];
         $776 = $771 * $775;
         $777 = $770 - $776;
         $778 = $ukj2;
         $779 = $luptr2;
         $780 = $lusup;
         $781 = (($780) + ($779<<3)|0);
         $782 = +HEAPF64[$781>>3];
         $783 = $778 * $782;
         $784 = $777 - $783;
         $ukj = $784;
         $785 = $ukj;
         $786 = $krep_ind;
         $787 = $lsub;
         $788 = (($787) + ($786<<2)|0);
         $789 = HEAP32[$788>>2]|0;
         $790 = $dense_col;
         $791 = (($790) + ($789<<3)|0);
         HEAPF64[$791>>3] = $785;
         $792 = $ukj1;
         $793 = $krep_ind;
         $794 = (($793) - 1)|0;
         $795 = $lsub;
         $796 = (($795) + ($794<<2)|0);
         $797 = HEAP32[$796>>2]|0;
         $798 = $dense_col;
         $799 = (($798) + ($797<<3)|0);
         HEAPF64[$799>>3] = $792;
         $800 = $lptr;
         $801 = $nsupc;
         $802 = (($800) + ($801))|0;
         $i = $802;
         while(1) {
          $803 = $i;
          $804 = $fsupc;
          $805 = (($804) + 1)|0;
          $806 = $xlsub;
          $807 = (($806) + ($805<<2)|0);
          $808 = HEAP32[$807>>2]|0;
          $809 = ($803|0)<($808|0);
          if (!($809)) {
           break L81;
          }
          $810 = $i;
          $811 = $lsub;
          $812 = (($811) + ($810<<2)|0);
          $813 = HEAP32[$812>>2]|0;
          $irow = $813;
          $814 = $luptr;
          $815 = (($814) + 1)|0;
          $luptr = $815;
          $816 = $luptr1;
          $817 = (($816) + 1)|0;
          $luptr1 = $817;
          $818 = $luptr2;
          $819 = (($818) + 1)|0;
          $luptr2 = $819;
          $820 = $ukj;
          $821 = $luptr;
          $822 = $lusup;
          $823 = (($822) + ($821<<3)|0);
          $824 = +HEAPF64[$823>>3];
          $825 = $820 * $824;
          $826 = $ukj1;
          $827 = $luptr1;
          $828 = $lusup;
          $829 = (($828) + ($827<<3)|0);
          $830 = +HEAPF64[$829>>3];
          $831 = $826 * $830;
          $832 = $825 + $831;
          $833 = $ukj2;
          $834 = $luptr2;
          $835 = $lusup;
          $836 = (($835) + ($834<<3)|0);
          $837 = +HEAPF64[$836>>3];
          $838 = $833 * $837;
          $839 = $832 + $838;
          $840 = $irow;
          $841 = $dense_col;
          $842 = (($841) + ($840<<3)|0);
          $843 = +HEAPF64[$842>>3];
          $844 = $843 - $839;
          HEAPF64[$842>>3] = $844;
          $845 = $i;
          $846 = (($845) + 1)|0;
          $i = $846;
         }
        }
       }
       $847 = $kfnz;
       $848 = $fsupc;
       $849 = (($847) - ($848))|0;
       $no_zeros = $849;
       $850 = $lptr;
       $851 = $no_zeros;
       $852 = (($850) + ($851))|0;
       $isub = $852;
       $i = 0;
       while(1) {
        $853 = $i;
        $854 = $segsze;
        $855 = ($853|0)<($854|0);
        if (!($855)) {
         break;
        }
        $856 = $isub;
        $857 = $lsub;
        $858 = (($857) + ($856<<2)|0);
        $859 = HEAP32[$858>>2]|0;
        $irow = $859;
        $860 = $irow;
        $861 = $dense_col;
        $862 = (($861) + ($860<<3)|0);
        $863 = +HEAPF64[$862>>3];
        $864 = $i;
        $865 = $5;
        $866 = (($865) + ($864<<3)|0);
        HEAPF64[$866>>3] = $863;
        $867 = $isub;
        $868 = (($867) + 1)|0;
        $isub = $868;
        $869 = $i;
        $870 = (($869) + 1)|0;
        $i = $870;
       }
       $871 = $nsupr;
       $872 = $no_zeros;
       $873 = Math_imul($871, $872)|0;
       $874 = $no_zeros;
       $875 = (($873) + ($874))|0;
       $876 = $luptr;
       $877 = (($876) + ($875))|0;
       $luptr = $877;
       $878 = $nsupr;
       $879 = $segsze;
       $880 = $luptr;
       $881 = $lusup;
       $882 = (($881) + ($880<<3)|0);
       $883 = $5;
       _dlsolve($878,$879,$882,$883);
       $884 = $segsze;
       $885 = $luptr;
       $886 = (($885) + ($884))|0;
       $luptr = $886;
       $887 = $segsze;
       $888 = $5;
       $889 = (($888) + ($887<<3)|0);
       $tempv1 = $889;
       $890 = $nsupr;
       $891 = $nrow;
       $892 = $segsze;
       $893 = $luptr;
       $894 = $lusup;
       $895 = (($894) + ($893<<3)|0);
       $896 = $5;
       $897 = $tempv1;
       _dmatvec($890,$891,$892,$895,$896,$897);
       $898 = $lptr;
       $899 = $no_zeros;
       $900 = (($898) + ($899))|0;
       $isub = $900;
       $i = 0;
       while(1) {
        $901 = $i;
        $902 = $segsze;
        $903 = ($901|0)<($902|0);
        if (!($903)) {
         break;
        }
        $904 = $isub;
        $905 = $lsub;
        $906 = (($905) + ($904<<2)|0);
        $907 = HEAP32[$906>>2]|0;
        $irow = $907;
        $908 = $i;
        $909 = $5;
        $910 = (($909) + ($908<<3)|0);
        $911 = +HEAPF64[$910>>3];
        $912 = $irow;
        $913 = $dense_col;
        $914 = (($913) + ($912<<3)|0);
        HEAPF64[$914>>3] = $911;
        $915 = $zero;
        $916 = $i;
        $917 = $5;
        $918 = (($917) + ($916<<3)|0);
        HEAPF64[$918>>3] = $915;
        $919 = $isub;
        $920 = (($919) + 1)|0;
        $isub = $920;
        $921 = $i;
        $922 = (($921) + 1)|0;
        $i = $922;
       }
       $i = 0;
       while(1) {
        $923 = $i;
        $924 = $nrow;
        $925 = ($923|0)<($924|0);
        if (!($925)) {
         break L81;
        }
        $926 = $isub;
        $927 = $lsub;
        $928 = (($927) + ($926<<2)|0);
        $929 = HEAP32[$928>>2]|0;
        $irow = $929;
        $930 = $i;
        $931 = $tempv1;
        $932 = (($931) + ($930<<3)|0);
        $933 = +HEAPF64[$932>>3];
        $934 = $irow;
        $935 = $dense_col;
        $936 = (($935) + ($934<<3)|0);
        $937 = +HEAPF64[$936>>3];
        $938 = $937 - $933;
        HEAPF64[$936>>3] = $938;
        $939 = $zero;
        $940 = $i;
        $941 = $tempv1;
        $942 = (($941) + ($940<<3)|0);
        HEAPF64[$942>>3] = $939;
        $943 = $isub;
        $944 = (($943) + 1)|0;
        $isub = $944;
        $945 = $i;
        $946 = (($945) + 1)|0;
        $i = $946;
       }
      }
     } while(0);
     $947 = $jj;
     $948 = (($947) + 1)|0;
     $jj = $948;
     $949 = $0;
     $950 = $repfnz_col;
     $951 = (($950) + ($949<<2)|0);
     $repfnz_col = $951;
     $952 = $0;
     $953 = $dense_col;
     $954 = (($953) + ($952<<3)|0);
     $dense_col = $954;
    }
   }
  } while(0);
  $955 = $ksub;
  $956 = (($955) + 1)|0;
  $ksub = $956;
 }
 STACKTOP = sp;return;
}
function _dcolumn_dfs($m,$jcol,$perm_r,$nseg,$lsub_col,$segrep,$repfnz,$xprune,$marker,$parent,$xplore,$Glu) {
 $m = $m|0;
 $jcol = $jcol|0;
 $perm_r = $perm_r|0;
 $nseg = $nseg|0;
 $lsub_col = $lsub_col|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $xprune = $xprune|0;
 $marker = $marker|0;
 $parent = $parent|0;
 $xplore = $xplore|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $chmark = 0, $chperm = 0, $chrep = 0, $fsupc = 0, $ifrom = 0, $istop = 0, $ito = 0, $jcolm1 = 0, $jcolp1 = 0, $jm1ptr = 0, $jptr = 0, $jsuper = 0, $k = 0;
 var $kchild = 0, $kmark = 0, $kpar = 0, $kperm = 0, $krep = 0, $krow = 0, $lsub = 0, $marker2 = 0, $maxdfs = 0, $mem_error = 0, $myfnz = 0, $nextl = 0, $nsuper = 0, $nzlmax = 0, $oldrep = 0, $supno = 0, $xdfs = 0, $xlsub = 0, $xsup = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $nzlmax = sp;
 $1 = $m;
 $2 = $jcol;
 $3 = $perm_r;
 $4 = $nseg;
 $5 = $lsub_col;
 $6 = $segrep;
 $7 = $repfnz;
 $8 = $xprune;
 $9 = $marker;
 $10 = $parent;
 $11 = $xplore;
 $12 = $Glu;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $xsup = $14;
 $15 = $12;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $supno = $17;
 $18 = $12;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $lsub = $20;
 $21 = $12;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $xlsub = $23;
 $24 = $12;
 $25 = ((($24)) + 36|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$nzlmax>>2] = $26;
 $27 = HEAP32[1840>>2]|0;
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = (_sp_ienv(3)|0);
  HEAP32[1848>>2] = $29;
  HEAP32[1840>>2] = 0;
 }
 $30 = $2;
 $31 = (($30) + 1)|0;
 $jcolp1 = $31;
 $32 = $2;
 $33 = (($32) - 1)|0;
 $jcolm1 = $33;
 $34 = $2;
 $35 = $supno;
 $36 = (($35) + ($34<<2)|0);
 $37 = HEAP32[$36>>2]|0;
 $nsuper = $37;
 $38 = $nsuper;
 $jsuper = $38;
 $39 = $2;
 $40 = $xlsub;
 $41 = (($40) + ($39<<2)|0);
 $42 = HEAP32[$41>>2]|0;
 $nextl = $42;
 $43 = $1;
 $44 = $43<<1;
 $45 = $9;
 $46 = (($45) + ($44<<2)|0);
 $marker2 = $46;
 $k = 0;
 L4: while(1) {
  $47 = $k;
  $48 = $5;
  $49 = (($48) + ($47<<2)|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = ($50|0)!=(-1);
  if (!($51)) {
   label = 33;
   break;
  }
  $52 = $k;
  $53 = $5;
  $54 = (($53) + ($52<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $krow = $55;
  $56 = $k;
  $57 = $5;
  $58 = (($57) + ($56<<2)|0);
  HEAP32[$58>>2] = -1;
  $59 = $krow;
  $60 = $marker2;
  $61 = (($60) + ($59<<2)|0);
  $62 = HEAP32[$61>>2]|0;
  $kmark = $62;
  $63 = $kmark;
  $64 = $2;
  $65 = ($63|0)==($64|0);
  L7: do {
   if (!($65)) {
    $66 = $2;
    $67 = $krow;
    $68 = $marker2;
    $69 = (($68) + ($67<<2)|0);
    HEAP32[$69>>2] = $66;
    $70 = $krow;
    $71 = $3;
    $72 = (($71) + ($70<<2)|0);
    $73 = HEAP32[$72>>2]|0;
    $kperm = $73;
    $74 = $kperm;
    $75 = ($74|0)==(-1);
    if ($75) {
     $76 = $krow;
     $77 = $nextl;
     $78 = (($77) + 1)|0;
     $nextl = $78;
     $79 = $lsub;
     $80 = (($79) + ($77<<2)|0);
     HEAP32[$80>>2] = $76;
     $81 = $nextl;
     $82 = HEAP32[$nzlmax>>2]|0;
     $83 = ($81|0)>=($82|0);
     if ($83) {
      $84 = $2;
      $85 = $nextl;
      $86 = $12;
      $87 = (_dLUMemXpand($84,$85,2,$nzlmax,$86)|0);
      $mem_error = $87;
      $88 = ($87|0)!=(0);
      if ($88) {
       label = 9;
       break L4;
      }
      $90 = $12;
      $91 = ((($90)) + 8|0);
      $92 = HEAP32[$91>>2]|0;
      $lsub = $92;
     }
     $93 = $kmark;
     $94 = $jcolm1;
     $95 = ($93|0)!=($94|0);
     if (!($95)) {
      break;
     }
     $jsuper = -1;
     break;
    }
    $96 = $kperm;
    $97 = $supno;
    $98 = (($97) + ($96<<2)|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = (($99) + 1)|0;
    $101 = $xsup;
    $102 = (($101) + ($100<<2)|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (($103) - 1)|0;
    $krep = $104;
    $105 = $krep;
    $106 = $7;
    $107 = (($106) + ($105<<2)|0);
    $108 = HEAP32[$107>>2]|0;
    $myfnz = $108;
    $109 = $myfnz;
    $110 = ($109|0)!=(-1);
    if ($110) {
     $111 = $myfnz;
     $112 = $kperm;
     $113 = ($111|0)>($112|0);
     if (!($113)) {
      break;
     }
     $114 = $kperm;
     $115 = $krep;
     $116 = $7;
     $117 = (($116) + ($115<<2)|0);
     HEAP32[$117>>2] = $114;
     break;
    }
    $oldrep = -1;
    $118 = $oldrep;
    $119 = $krep;
    $120 = $10;
    $121 = (($120) + ($119<<2)|0);
    HEAP32[$121>>2] = $118;
    $122 = $kperm;
    $123 = $krep;
    $124 = $7;
    $125 = (($124) + ($123<<2)|0);
    HEAP32[$125>>2] = $122;
    $126 = $krep;
    $127 = $xlsub;
    $128 = (($127) + ($126<<2)|0);
    $129 = HEAP32[$128>>2]|0;
    $xdfs = $129;
    $130 = $krep;
    $131 = $8;
    $132 = (($131) + ($130<<2)|0);
    $133 = HEAP32[$132>>2]|0;
    $maxdfs = $133;
    while(1) {
     $134 = $xdfs;
     $135 = $maxdfs;
     $136 = ($134|0)<($135|0);
     if (!($136)) {
      $224 = $krep;
      $225 = $4;
      $226 = HEAP32[$225>>2]|0;
      $227 = $6;
      $228 = (($227) + ($226<<2)|0);
      HEAP32[$228>>2] = $224;
      $229 = $4;
      $230 = HEAP32[$229>>2]|0;
      $231 = (($230) + 1)|0;
      HEAP32[$229>>2] = $231;
      $232 = $krep;
      $233 = $10;
      $234 = (($233) + ($232<<2)|0);
      $235 = HEAP32[$234>>2]|0;
      $kpar = $235;
      $236 = $kpar;
      $237 = ($236|0)==(-1);
      if ($237) {
       break L7;
      }
      $238 = $kpar;
      $krep = $238;
      $239 = $krep;
      $240 = $11;
      $241 = (($240) + ($239<<2)|0);
      $242 = HEAP32[$241>>2]|0;
      $xdfs = $242;
      $243 = $krep;
      $244 = $8;
      $245 = (($244) + ($243<<2)|0);
      $246 = HEAP32[$245>>2]|0;
      $maxdfs = $246;
      $247 = $kpar;
      $248 = ($247|0)!=(-1);
      if ($248) {
       continue;
      } else {
       break;
      }
     }
     $137 = $xdfs;
     $138 = $lsub;
     $139 = (($138) + ($137<<2)|0);
     $140 = HEAP32[$139>>2]|0;
     $kchild = $140;
     $141 = $xdfs;
     $142 = (($141) + 1)|0;
     $xdfs = $142;
     $143 = $kchild;
     $144 = $marker2;
     $145 = (($144) + ($143<<2)|0);
     $146 = HEAP32[$145>>2]|0;
     $chmark = $146;
     $147 = $chmark;
     $148 = $2;
     $149 = ($147|0)!=($148|0);
     if (!($149)) {
      continue;
     }
     $150 = $2;
     $151 = $kchild;
     $152 = $marker2;
     $153 = (($152) + ($151<<2)|0);
     HEAP32[$153>>2] = $150;
     $154 = $kchild;
     $155 = $3;
     $156 = (($155) + ($154<<2)|0);
     $157 = HEAP32[$156>>2]|0;
     $chperm = $157;
     $158 = $chperm;
     $159 = ($158|0)==(-1);
     if (!($159)) {
      $180 = $chperm;
      $181 = $supno;
      $182 = (($181) + ($180<<2)|0);
      $183 = HEAP32[$182>>2]|0;
      $184 = (($183) + 1)|0;
      $185 = $xsup;
      $186 = (($185) + ($184<<2)|0);
      $187 = HEAP32[$186>>2]|0;
      $188 = (($187) - 1)|0;
      $chrep = $188;
      $189 = $chrep;
      $190 = $7;
      $191 = (($190) + ($189<<2)|0);
      $192 = HEAP32[$191>>2]|0;
      $myfnz = $192;
      $193 = $myfnz;
      $194 = ($193|0)!=(-1);
      if (!($194)) {
       $202 = $xdfs;
       $203 = $krep;
       $204 = $11;
       $205 = (($204) + ($203<<2)|0);
       HEAP32[$205>>2] = $202;
       $206 = $krep;
       $oldrep = $206;
       $207 = $chrep;
       $krep = $207;
       $208 = $oldrep;
       $209 = $krep;
       $210 = $10;
       $211 = (($210) + ($209<<2)|0);
       HEAP32[$211>>2] = $208;
       $212 = $chperm;
       $213 = $krep;
       $214 = $7;
       $215 = (($214) + ($213<<2)|0);
       HEAP32[$215>>2] = $212;
       $216 = $krep;
       $217 = $xlsub;
       $218 = (($217) + ($216<<2)|0);
       $219 = HEAP32[$218>>2]|0;
       $xdfs = $219;
       $220 = $krep;
       $221 = $8;
       $222 = (($221) + ($220<<2)|0);
       $223 = HEAP32[$222>>2]|0;
       $maxdfs = $223;
       continue;
      }
      $195 = $myfnz;
      $196 = $chperm;
      $197 = ($195|0)>($196|0);
      if (!($197)) {
       continue;
      }
      $198 = $chperm;
      $199 = $chrep;
      $200 = $7;
      $201 = (($200) + ($199<<2)|0);
      HEAP32[$201>>2] = $198;
      continue;
     }
     $160 = $kchild;
     $161 = $nextl;
     $162 = (($161) + 1)|0;
     $nextl = $162;
     $163 = $lsub;
     $164 = (($163) + ($161<<2)|0);
     HEAP32[$164>>2] = $160;
     $165 = $nextl;
     $166 = HEAP32[$nzlmax>>2]|0;
     $167 = ($165|0)>=($166|0);
     if ($167) {
      $168 = $2;
      $169 = $nextl;
      $170 = $12;
      $171 = (_dLUMemXpand($168,$169,2,$nzlmax,$170)|0);
      $mem_error = $171;
      $172 = ($171|0)!=(0);
      if ($172) {
       label = 22;
       break L4;
      }
      $174 = $12;
      $175 = ((($174)) + 8|0);
      $176 = HEAP32[$175>>2]|0;
      $lsub = $176;
     }
     $177 = $chmark;
     $178 = $jcolm1;
     $179 = ($177|0)!=($178|0);
     if (!($179)) {
      continue;
     }
     $jsuper = -1;
    }
   }
  } while(0);
  $249 = $k;
  $250 = (($249) + 1)|0;
  $k = $250;
 }
 if ((label|0) == 9) {
  $89 = $mem_error;
  $0 = $89;
  $346 = $0;
  STACKTOP = sp;return ($346|0);
 }
 else if ((label|0) == 22) {
  $173 = $mem_error;
  $0 = $173;
  $346 = $0;
  STACKTOP = sp;return ($346|0);
 }
 else if ((label|0) == 33) {
  $251 = $2;
  $252 = ($251|0)==(0);
  if ($252) {
   $253 = $supno;
   HEAP32[$253>>2] = 0;
   $nsuper = 0;
  } else {
   $254 = $nsuper;
   $255 = $xsup;
   $256 = (($255) + ($254<<2)|0);
   $257 = HEAP32[$256>>2]|0;
   $fsupc = $257;
   $258 = $2;
   $259 = $xlsub;
   $260 = (($259) + ($258<<2)|0);
   $261 = HEAP32[$260>>2]|0;
   $jptr = $261;
   $262 = $jcolm1;
   $263 = $xlsub;
   $264 = (($263) + ($262<<2)|0);
   $265 = HEAP32[$264>>2]|0;
   $jm1ptr = $265;
   $266 = $nextl;
   $267 = $jptr;
   $268 = (($266) - ($267))|0;
   $269 = $jptr;
   $270 = $jm1ptr;
   $271 = (($269) - ($270))|0;
   $272 = (($271) - 1)|0;
   $273 = ($268|0)!=($272|0);
   if ($273) {
    $jsuper = -1;
   }
   $274 = $2;
   $275 = $fsupc;
   $276 = (($274) - ($275))|0;
   $277 = HEAP32[1848>>2]|0;
   $278 = ($276|0)>=($277|0);
   if ($278) {
    $jsuper = -1;
   }
   $279 = $jsuper;
   $280 = ($279|0)==(-1);
   if ($280) {
    $281 = $fsupc;
    $282 = $jcolm1;
    $283 = (($282) - 1)|0;
    $284 = ($281|0)<($283|0);
    if ($284) {
     $285 = $fsupc;
     $286 = (($285) + 1)|0;
     $287 = $xlsub;
     $288 = (($287) + ($286<<2)|0);
     $289 = HEAP32[$288>>2]|0;
     $ito = $289;
     $290 = $ito;
     $291 = $jcolm1;
     $292 = $xlsub;
     $293 = (($292) + ($291<<2)|0);
     HEAP32[$293>>2] = $290;
     $294 = $ito;
     $295 = $jptr;
     $296 = (($294) + ($295))|0;
     $297 = $jm1ptr;
     $298 = (($296) - ($297))|0;
     $istop = $298;
     $299 = $istop;
     $300 = $jcolm1;
     $301 = $8;
     $302 = (($301) + ($300<<2)|0);
     HEAP32[$302>>2] = $299;
     $303 = $istop;
     $304 = $2;
     $305 = $xlsub;
     $306 = (($305) + ($304<<2)|0);
     HEAP32[$306>>2] = $303;
     $307 = $jm1ptr;
     $ifrom = $307;
     while(1) {
      $308 = $ifrom;
      $309 = $nextl;
      $310 = ($308|0)<($309|0);
      if (!($310)) {
       break;
      }
      $311 = $ifrom;
      $312 = $lsub;
      $313 = (($312) + ($311<<2)|0);
      $314 = HEAP32[$313>>2]|0;
      $315 = $ito;
      $316 = $lsub;
      $317 = (($316) + ($315<<2)|0);
      HEAP32[$317>>2] = $314;
      $318 = $ifrom;
      $319 = (($318) + 1)|0;
      $ifrom = $319;
      $320 = $ito;
      $321 = (($320) + 1)|0;
      $ito = $321;
     }
     $322 = $ito;
     $nextl = $322;
    }
    $323 = $nsuper;
    $324 = (($323) + 1)|0;
    $nsuper = $324;
    $325 = $nsuper;
    $326 = $2;
    $327 = $supno;
    $328 = (($327) + ($326<<2)|0);
    HEAP32[$328>>2] = $325;
   }
  }
  $329 = $jcolp1;
  $330 = $nsuper;
  $331 = (($330) + 1)|0;
  $332 = $xsup;
  $333 = (($332) + ($331<<2)|0);
  HEAP32[$333>>2] = $329;
  $334 = $nsuper;
  $335 = $jcolp1;
  $336 = $supno;
  $337 = (($336) + ($335<<2)|0);
  HEAP32[$337>>2] = $334;
  $338 = $nextl;
  $339 = $2;
  $340 = $8;
  $341 = (($340) + ($339<<2)|0);
  HEAP32[$341>>2] = $338;
  $342 = $nextl;
  $343 = $jcolp1;
  $344 = $xlsub;
  $345 = (($344) + ($343<<2)|0);
  HEAP32[$345>>2] = $342;
  $0 = 0;
  $346 = $0;
  STACKTOP = sp;return ($346|0);
 }
 return (0)|0;
}
function _dcolumn_bmod($jcol,$nseg,$dense,$tempv,$segrep,$repfnz,$fpanelc,$Glu,$stat) {
 $jcol = $jcol|0;
 $nseg = $nseg|0;
 $dense = $dense|0;
 $tempv = $tempv|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $fpanelc = $fpanelc|0;
 $Glu = $Glu|0;
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0.0, $124 = 0, $125 = 0, $126 = 0.0, $127 = 0.0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0.0, $133 = 0;
 var $134 = 0, $135 = 0.0, $136 = 0.0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0.0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0.0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0.0, $173 = 0.0, $174 = 0, $175 = 0, $176 = 0, $177 = 0.0, $178 = 0.0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0.0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0.0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0.0, $215 = 0, $216 = 0, $217 = 0, $218 = 0.0, $219 = 0.0, $22 = 0, $220 = 0.0, $221 = 0.0, $222 = 0.0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0.0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0.0, $252 = 0.0, $253 = 0.0, $254 = 0, $255 = 0, $256 = 0, $257 = 0.0, $258 = 0.0, $259 = 0.0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0.0, $264 = 0.0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0.0, $275 = 0, $276 = 0, $277 = 0, $278 = 0.0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0.0, $284 = 0.0, $285 = 0.0, $286 = 0.0, $287 = 0.0, $288 = 0.0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0.0, $293 = 0.0, $294 = 0.0, $295 = 0.0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0.0, $302 = 0.0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0.0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0.0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0.0, $342 = 0.0, $343 = 0.0, $344 = 0, $345 = 0, $346 = 0, $347 = 0.0, $348 = 0.0, $349 = 0.0, $35 = 0;
 var $350 = 0.0, $351 = 0, $352 = 0, $353 = 0, $354 = 0.0, $355 = 0.0, $356 = 0.0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0.0, $361 = 0.0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0.0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0.0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0.0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0.0, $451 = 0, $452 = 0, $453 = 0, $454 = 0.0, $455 = 0.0, $456 = 0.0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0.0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0.0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0.0, $588 = 0, $589 = 0, $59 = 0, $590 = 0.0, $591 = 0.0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0.0, $597 = 0, $598 = 0, $599 = 0.0, $6 = 0, $60 = 0, $600 = 0.0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0.0, $631 = 0, $632 = 0, $633 = 0, $634 = 0.0, $635 = 0.0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d_fsupc = 0, $fst_col = 0, $fsupc = 0, $i = 0, $incx = 0, $incy = 0, $irow = 0, $isub = 0, $jcolp1 = 0, $jsupno = 0, $k = 0, $kfnz = 0, $krep = 0, $krep_ind = 0, $ksub = 0, $ksupno = 0;
 var $lptr = 0, $lsub = 0, $luptr = 0, $luptr1 = 0, $luptr2 = 0, $lusup = 0, $mem_error = 0, $new_next = 0, $nextlu = 0, $no_zeros = 0, $none = 0.0, $nrow = 0, $nsupc = 0, $nsupr = 0, $nzlumax = 0, $one = 0.0, $ops = 0, $segsze = 0, $supno = 0, $tempv1 = 0;
 var $ufirst = 0, $ukj = 0.0, $ukj1 = 0.0, $ukj2 = 0.0, $xlsub = 0, $xlusup = 0, $xsup = 0, $zero = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $nzlumax = sp + 60|0;
 $1 = $jcol;
 $2 = $nseg;
 $3 = $dense;
 $4 = $tempv;
 $5 = $segrep;
 $6 = $repfnz;
 $7 = $fpanelc;
 $8 = $Glu;
 $9 = $stat;
 $incx = 1;
 $incy = 1;
 $zero = 0.0;
 $one = 1.0;
 $none = -1.0;
 $10 = $9;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $ops = $12;
 $13 = $8;
 $14 = HEAP32[$13>>2]|0;
 $xsup = $14;
 $15 = $8;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $supno = $17;
 $18 = $8;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $lsub = $20;
 $21 = $8;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $xlsub = $23;
 $24 = $8;
 $25 = ((($24)) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $lusup = $26;
 $27 = $8;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 $xlusup = $29;
 $30 = $8;
 $31 = ((($30)) + 44|0);
 $32 = HEAP32[$31>>2]|0;
 HEAP32[$nzlumax>>2] = $32;
 $33 = $1;
 $34 = (($33) + 1)|0;
 $jcolp1 = $34;
 $35 = $1;
 $36 = $supno;
 $37 = (($36) + ($35<<2)|0);
 $38 = HEAP32[$37>>2]|0;
 $jsupno = $38;
 $39 = $2;
 $40 = (($39) - 1)|0;
 $k = $40;
 $ksub = 0;
 while(1) {
  $41 = $ksub;
  $42 = $2;
  $43 = ($41|0)<($42|0);
  if (!($43)) {
   break;
  }
  $44 = $k;
  $45 = $5;
  $46 = (($45) + ($44<<2)|0);
  $47 = HEAP32[$46>>2]|0;
  $krep = $47;
  $48 = $k;
  $49 = (($48) + -1)|0;
  $k = $49;
  $50 = $krep;
  $51 = $supno;
  $52 = (($51) + ($50<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  $ksupno = $53;
  $54 = $jsupno;
  $55 = $ksupno;
  $56 = ($54|0)!=($55|0);
  L4: do {
   if ($56) {
    $57 = $ksupno;
    $58 = $xsup;
    $59 = (($58) + ($57<<2)|0);
    $60 = HEAP32[$59>>2]|0;
    $fsupc = $60;
    $61 = $fsupc;
    $62 = $7;
    $63 = ($61|0)>($62|0);
    $64 = $fsupc;
    $65 = $7;
    $66 = $63 ? $64 : $65;
    $fst_col = $66;
    $67 = $fst_col;
    $68 = $fsupc;
    $69 = (($67) - ($68))|0;
    $d_fsupc = $69;
    $70 = $fst_col;
    $71 = $xlusup;
    $72 = (($71) + ($70<<2)|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = $d_fsupc;
    $75 = (($73) + ($74))|0;
    $luptr = $75;
    $76 = $fsupc;
    $77 = $xlsub;
    $78 = (($77) + ($76<<2)|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = $d_fsupc;
    $81 = (($79) + ($80))|0;
    $lptr = $81;
    $82 = $krep;
    $83 = $6;
    $84 = (($83) + ($82<<2)|0);
    $85 = HEAP32[$84>>2]|0;
    $kfnz = $85;
    $86 = $kfnz;
    $87 = $7;
    $88 = ($86|0)>($87|0);
    $89 = $kfnz;
    $90 = $7;
    $91 = $88 ? $89 : $90;
    $kfnz = $91;
    $92 = $krep;
    $93 = $kfnz;
    $94 = (($92) - ($93))|0;
    $95 = (($94) + 1)|0;
    $segsze = $95;
    $96 = $krep;
    $97 = $fst_col;
    $98 = (($96) - ($97))|0;
    $99 = (($98) + 1)|0;
    $nsupc = $99;
    $100 = $fsupc;
    $101 = (($100) + 1)|0;
    $102 = $xlsub;
    $103 = (($102) + ($101<<2)|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = $fsupc;
    $106 = $xlsub;
    $107 = (($106) + ($105<<2)|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = (($104) - ($108))|0;
    $nsupr = $109;
    $110 = $nsupr;
    $111 = $d_fsupc;
    $112 = (($110) - ($111))|0;
    $113 = $nsupc;
    $114 = (($112) - ($113))|0;
    $nrow = $114;
    $115 = $lptr;
    $116 = $nsupc;
    $117 = (($115) + ($116))|0;
    $118 = (($117) - 1)|0;
    $krep_ind = $118;
    $119 = $segsze;
    $120 = $segsze;
    $121 = (($120) - 1)|0;
    $122 = Math_imul($119, $121)|0;
    $123 = (+($122|0));
    $124 = $ops;
    $125 = ((($124)) + 52|0);
    $126 = +HEAPF32[$125>>2];
    $127 = $126 + $123;
    HEAPF32[$125>>2] = $127;
    $128 = $nrow;
    $129 = $128<<1;
    $130 = $segsze;
    $131 = Math_imul($129, $130)|0;
    $132 = (+($131|0));
    $133 = $ops;
    $134 = ((($133)) + 56|0);
    $135 = +HEAPF32[$134>>2];
    $136 = $135 + $132;
    HEAPF32[$134>>2] = $136;
    $137 = $segsze;
    $138 = ($137|0)==(1);
    if ($138) {
     $139 = $krep_ind;
     $140 = $lsub;
     $141 = (($140) + ($139<<2)|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = $3;
     $144 = (($143) + ($142<<3)|0);
     $145 = +HEAPF64[$144>>3];
     $ukj = $145;
     $146 = $nsupr;
     $147 = $nsupc;
     $148 = (($147) - 1)|0;
     $149 = Math_imul($146, $148)|0;
     $150 = $nsupc;
     $151 = (($149) + ($150))|0;
     $152 = $luptr;
     $153 = (($152) + ($151))|0;
     $luptr = $153;
     $154 = $lptr;
     $155 = $nsupc;
     $156 = (($154) + ($155))|0;
     $i = $156;
     while(1) {
      $157 = $i;
      $158 = $fsupc;
      $159 = (($158) + 1)|0;
      $160 = $xlsub;
      $161 = (($160) + ($159<<2)|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = ($157|0)<($162|0);
      if (!($163)) {
       break L4;
      }
      $164 = $i;
      $165 = $lsub;
      $166 = (($165) + ($164<<2)|0);
      $167 = HEAP32[$166>>2]|0;
      $irow = $167;
      $168 = $ukj;
      $169 = $luptr;
      $170 = $lusup;
      $171 = (($170) + ($169<<3)|0);
      $172 = +HEAPF64[$171>>3];
      $173 = $168 * $172;
      $174 = $irow;
      $175 = $3;
      $176 = (($175) + ($174<<3)|0);
      $177 = +HEAPF64[$176>>3];
      $178 = $177 - $173;
      HEAPF64[$176>>3] = $178;
      $179 = $luptr;
      $180 = (($179) + 1)|0;
      $luptr = $180;
      $181 = $i;
      $182 = (($181) + 1)|0;
      $i = $182;
     }
    }
    $183 = $segsze;
    $184 = ($183|0)<=(3);
    if ($184) {
     $185 = $krep_ind;
     $186 = $lsub;
     $187 = (($186) + ($185<<2)|0);
     $188 = HEAP32[$187>>2]|0;
     $189 = $3;
     $190 = (($189) + ($188<<3)|0);
     $191 = +HEAPF64[$190>>3];
     $ukj = $191;
     $192 = $nsupr;
     $193 = $nsupc;
     $194 = (($193) - 1)|0;
     $195 = Math_imul($192, $194)|0;
     $196 = $nsupc;
     $197 = (($195) + ($196))|0;
     $198 = (($197) - 1)|0;
     $199 = $luptr;
     $200 = (($199) + ($198))|0;
     $luptr = $200;
     $201 = $krep_ind;
     $202 = (($201) - 1)|0;
     $203 = $lsub;
     $204 = (($203) + ($202<<2)|0);
     $205 = HEAP32[$204>>2]|0;
     $206 = $3;
     $207 = (($206) + ($205<<3)|0);
     $208 = +HEAPF64[$207>>3];
     $ukj1 = $208;
     $209 = $luptr;
     $210 = $nsupr;
     $211 = (($209) - ($210))|0;
     $luptr1 = $211;
     $212 = $segsze;
     $213 = ($212|0)==(2);
     if ($213) {
      $214 = $ukj1;
      $215 = $luptr1;
      $216 = $lusup;
      $217 = (($216) + ($215<<3)|0);
      $218 = +HEAPF64[$217>>3];
      $219 = $214 * $218;
      $220 = $ukj;
      $221 = $220 - $219;
      $ukj = $221;
      $222 = $ukj;
      $223 = $krep_ind;
      $224 = $lsub;
      $225 = (($224) + ($223<<2)|0);
      $226 = HEAP32[$225>>2]|0;
      $227 = $3;
      $228 = (($227) + ($226<<3)|0);
      HEAPF64[$228>>3] = $222;
      $229 = $lptr;
      $230 = $nsupc;
      $231 = (($229) + ($230))|0;
      $i = $231;
      while(1) {
       $232 = $i;
       $233 = $fsupc;
       $234 = (($233) + 1)|0;
       $235 = $xlsub;
       $236 = (($235) + ($234<<2)|0);
       $237 = HEAP32[$236>>2]|0;
       $238 = ($232|0)<($237|0);
       if (!($238)) {
        break L4;
       }
       $239 = $i;
       $240 = $lsub;
       $241 = (($240) + ($239<<2)|0);
       $242 = HEAP32[$241>>2]|0;
       $irow = $242;
       $243 = $luptr;
       $244 = (($243) + 1)|0;
       $luptr = $244;
       $245 = $luptr1;
       $246 = (($245) + 1)|0;
       $luptr1 = $246;
       $247 = $ukj;
       $248 = $luptr;
       $249 = $lusup;
       $250 = (($249) + ($248<<3)|0);
       $251 = +HEAPF64[$250>>3];
       $252 = $247 * $251;
       $253 = $ukj1;
       $254 = $luptr1;
       $255 = $lusup;
       $256 = (($255) + ($254<<3)|0);
       $257 = +HEAPF64[$256>>3];
       $258 = $253 * $257;
       $259 = $252 + $258;
       $260 = $irow;
       $261 = $3;
       $262 = (($261) + ($260<<3)|0);
       $263 = +HEAPF64[$262>>3];
       $264 = $263 - $259;
       HEAPF64[$262>>3] = $264;
       $265 = $i;
       $266 = (($265) + 1)|0;
       $i = $266;
      }
     } else {
      $267 = $krep_ind;
      $268 = (($267) - 2)|0;
      $269 = $lsub;
      $270 = (($269) + ($268<<2)|0);
      $271 = HEAP32[$270>>2]|0;
      $272 = $3;
      $273 = (($272) + ($271<<3)|0);
      $274 = +HEAPF64[$273>>3];
      $ukj2 = $274;
      $275 = $luptr1;
      $276 = $nsupr;
      $277 = (($275) - ($276))|0;
      $luptr2 = $277;
      $278 = $ukj2;
      $279 = $luptr2;
      $280 = (($279) - 1)|0;
      $281 = $lusup;
      $282 = (($281) + ($280<<3)|0);
      $283 = +HEAPF64[$282>>3];
      $284 = $278 * $283;
      $285 = $ukj1;
      $286 = $285 - $284;
      $ukj1 = $286;
      $287 = $ukj;
      $288 = $ukj1;
      $289 = $luptr1;
      $290 = $lusup;
      $291 = (($290) + ($289<<3)|0);
      $292 = +HEAPF64[$291>>3];
      $293 = $288 * $292;
      $294 = $287 - $293;
      $295 = $ukj2;
      $296 = $luptr2;
      $297 = $lusup;
      $298 = (($297) + ($296<<3)|0);
      $299 = +HEAPF64[$298>>3];
      $300 = $295 * $299;
      $301 = $294 - $300;
      $ukj = $301;
      $302 = $ukj;
      $303 = $krep_ind;
      $304 = $lsub;
      $305 = (($304) + ($303<<2)|0);
      $306 = HEAP32[$305>>2]|0;
      $307 = $3;
      $308 = (($307) + ($306<<3)|0);
      HEAPF64[$308>>3] = $302;
      $309 = $ukj1;
      $310 = $krep_ind;
      $311 = (($310) - 1)|0;
      $312 = $lsub;
      $313 = (($312) + ($311<<2)|0);
      $314 = HEAP32[$313>>2]|0;
      $315 = $3;
      $316 = (($315) + ($314<<3)|0);
      HEAPF64[$316>>3] = $309;
      $317 = $lptr;
      $318 = $nsupc;
      $319 = (($317) + ($318))|0;
      $i = $319;
      while(1) {
       $320 = $i;
       $321 = $fsupc;
       $322 = (($321) + 1)|0;
       $323 = $xlsub;
       $324 = (($323) + ($322<<2)|0);
       $325 = HEAP32[$324>>2]|0;
       $326 = ($320|0)<($325|0);
       if (!($326)) {
        break L4;
       }
       $327 = $i;
       $328 = $lsub;
       $329 = (($328) + ($327<<2)|0);
       $330 = HEAP32[$329>>2]|0;
       $irow = $330;
       $331 = $luptr;
       $332 = (($331) + 1)|0;
       $luptr = $332;
       $333 = $luptr1;
       $334 = (($333) + 1)|0;
       $luptr1 = $334;
       $335 = $luptr2;
       $336 = (($335) + 1)|0;
       $luptr2 = $336;
       $337 = $ukj;
       $338 = $luptr;
       $339 = $lusup;
       $340 = (($339) + ($338<<3)|0);
       $341 = +HEAPF64[$340>>3];
       $342 = $337 * $341;
       $343 = $ukj1;
       $344 = $luptr1;
       $345 = $lusup;
       $346 = (($345) + ($344<<3)|0);
       $347 = +HEAPF64[$346>>3];
       $348 = $343 * $347;
       $349 = $342 + $348;
       $350 = $ukj2;
       $351 = $luptr2;
       $352 = $lusup;
       $353 = (($352) + ($351<<3)|0);
       $354 = +HEAPF64[$353>>3];
       $355 = $350 * $354;
       $356 = $349 + $355;
       $357 = $irow;
       $358 = $3;
       $359 = (($358) + ($357<<3)|0);
       $360 = +HEAPF64[$359>>3];
       $361 = $360 - $356;
       HEAPF64[$359>>3] = $361;
       $362 = $i;
       $363 = (($362) + 1)|0;
       $i = $363;
      }
     }
    }
    $364 = $kfnz;
    $365 = $fst_col;
    $366 = (($364) - ($365))|0;
    $no_zeros = $366;
    $367 = $lptr;
    $368 = $no_zeros;
    $369 = (($367) + ($368))|0;
    $isub = $369;
    $i = 0;
    while(1) {
     $370 = $i;
     $371 = $segsze;
     $372 = ($370|0)<($371|0);
     if (!($372)) {
      break;
     }
     $373 = $isub;
     $374 = $lsub;
     $375 = (($374) + ($373<<2)|0);
     $376 = HEAP32[$375>>2]|0;
     $irow = $376;
     $377 = $irow;
     $378 = $3;
     $379 = (($378) + ($377<<3)|0);
     $380 = +HEAPF64[$379>>3];
     $381 = $i;
     $382 = $4;
     $383 = (($382) + ($381<<3)|0);
     HEAPF64[$383>>3] = $380;
     $384 = $isub;
     $385 = (($384) + 1)|0;
     $isub = $385;
     $386 = $i;
     $387 = (($386) + 1)|0;
     $i = $387;
    }
    $388 = $nsupr;
    $389 = $no_zeros;
    $390 = Math_imul($388, $389)|0;
    $391 = $no_zeros;
    $392 = (($390) + ($391))|0;
    $393 = $luptr;
    $394 = (($393) + ($392))|0;
    $luptr = $394;
    $395 = $nsupr;
    $396 = $segsze;
    $397 = $luptr;
    $398 = $lusup;
    $399 = (($398) + ($397<<3)|0);
    $400 = $4;
    _dlsolve($395,$396,$399,$400);
    $401 = $segsze;
    $402 = $luptr;
    $403 = (($402) + ($401))|0;
    $luptr = $403;
    $404 = $segsze;
    $405 = $4;
    $406 = (($405) + ($404<<3)|0);
    $tempv1 = $406;
    $407 = $nsupr;
    $408 = $nrow;
    $409 = $segsze;
    $410 = $luptr;
    $411 = $lusup;
    $412 = (($411) + ($410<<3)|0);
    $413 = $4;
    $414 = $tempv1;
    _dmatvec($407,$408,$409,$412,$413,$414);
    $415 = $lptr;
    $416 = $no_zeros;
    $417 = (($415) + ($416))|0;
    $isub = $417;
    $i = 0;
    while(1) {
     $418 = $i;
     $419 = $segsze;
     $420 = ($418|0)<($419|0);
     if (!($420)) {
      break;
     }
     $421 = $isub;
     $422 = $lsub;
     $423 = (($422) + ($421<<2)|0);
     $424 = HEAP32[$423>>2]|0;
     $irow = $424;
     $425 = $i;
     $426 = $4;
     $427 = (($426) + ($425<<3)|0);
     $428 = +HEAPF64[$427>>3];
     $429 = $irow;
     $430 = $3;
     $431 = (($430) + ($429<<3)|0);
     HEAPF64[$431>>3] = $428;
     $432 = $zero;
     $433 = $i;
     $434 = $4;
     $435 = (($434) + ($433<<3)|0);
     HEAPF64[$435>>3] = $432;
     $436 = $isub;
     $437 = (($436) + 1)|0;
     $isub = $437;
     $438 = $i;
     $439 = (($438) + 1)|0;
     $i = $439;
    }
    $i = 0;
    while(1) {
     $440 = $i;
     $441 = $nrow;
     $442 = ($440|0)<($441|0);
     if (!($442)) {
      break L4;
     }
     $443 = $isub;
     $444 = $lsub;
     $445 = (($444) + ($443<<2)|0);
     $446 = HEAP32[$445>>2]|0;
     $irow = $446;
     $447 = $i;
     $448 = $tempv1;
     $449 = (($448) + ($447<<3)|0);
     $450 = +HEAPF64[$449>>3];
     $451 = $irow;
     $452 = $3;
     $453 = (($452) + ($451<<3)|0);
     $454 = +HEAPF64[$453>>3];
     $455 = $454 - $450;
     HEAPF64[$453>>3] = $455;
     $456 = $zero;
     $457 = $i;
     $458 = $tempv1;
     $459 = (($458) + ($457<<3)|0);
     HEAPF64[$459>>3] = $456;
     $460 = $isub;
     $461 = (($460) + 1)|0;
     $isub = $461;
     $462 = $i;
     $463 = (($462) + 1)|0;
     $i = $463;
    }
   }
  } while(0);
  $464 = $ksub;
  $465 = (($464) + 1)|0;
  $ksub = $465;
 }
 $466 = $1;
 $467 = $xlusup;
 $468 = (($467) + ($466<<2)|0);
 $469 = HEAP32[$468>>2]|0;
 $nextlu = $469;
 $470 = $jsupno;
 $471 = $xsup;
 $472 = (($471) + ($470<<2)|0);
 $473 = HEAP32[$472>>2]|0;
 $fsupc = $473;
 $474 = $nextlu;
 $475 = $fsupc;
 $476 = (($475) + 1)|0;
 $477 = $xlsub;
 $478 = (($477) + ($476<<2)|0);
 $479 = HEAP32[$478>>2]|0;
 $480 = (($474) + ($479))|0;
 $481 = $fsupc;
 $482 = $xlsub;
 $483 = (($482) + ($481<<2)|0);
 $484 = HEAP32[$483>>2]|0;
 $485 = (($480) - ($484))|0;
 $new_next = $485;
 while(1) {
  $486 = $new_next;
  $487 = HEAP32[$nzlumax>>2]|0;
  $488 = ($486|0)>($487|0);
  if (!($488)) {
   break;
  }
  $489 = $1;
  $490 = $nextlu;
  $491 = $8;
  $492 = (_dLUMemXpand($489,$490,0,$nzlumax,$491)|0);
  $mem_error = $492;
  $493 = ($492|0)!=(0);
  if ($493) {
   label = 29;
   break;
  }
  $495 = $8;
  $496 = ((($495)) + 16|0);
  $497 = HEAP32[$496>>2]|0;
  $lusup = $497;
  $498 = $8;
  $499 = ((($498)) + 8|0);
  $500 = HEAP32[$499>>2]|0;
  $lsub = $500;
 }
 if ((label|0) == 29) {
  $494 = $mem_error;
  $0 = $494;
  $643 = $0;
  STACKTOP = sp;return ($643|0);
 }
 $501 = $fsupc;
 $502 = $xlsub;
 $503 = (($502) + ($501<<2)|0);
 $504 = HEAP32[$503>>2]|0;
 $isub = $504;
 while(1) {
  $505 = $isub;
  $506 = $fsupc;
  $507 = (($506) + 1)|0;
  $508 = $xlsub;
  $509 = (($508) + ($507<<2)|0);
  $510 = HEAP32[$509>>2]|0;
  $511 = ($505|0)<($510|0);
  if (!($511)) {
   break;
  }
  $512 = $isub;
  $513 = $lsub;
  $514 = (($513) + ($512<<2)|0);
  $515 = HEAP32[$514>>2]|0;
  $irow = $515;
  $516 = $irow;
  $517 = $3;
  $518 = (($517) + ($516<<3)|0);
  $519 = +HEAPF64[$518>>3];
  $520 = $nextlu;
  $521 = $lusup;
  $522 = (($521) + ($520<<3)|0);
  HEAPF64[$522>>3] = $519;
  $523 = $zero;
  $524 = $irow;
  $525 = $3;
  $526 = (($525) + ($524<<3)|0);
  HEAPF64[$526>>3] = $523;
  $527 = $nextlu;
  $528 = (($527) + 1)|0;
  $nextlu = $528;
  $529 = $isub;
  $530 = (($529) + 1)|0;
  $isub = $530;
 }
 $531 = $nextlu;
 $532 = $jcolp1;
 $533 = $xlusup;
 $534 = (($533) + ($532<<2)|0);
 HEAP32[$534>>2] = $531;
 $535 = $fsupc;
 $536 = $7;
 $537 = ($535|0)>($536|0);
 $538 = $fsupc;
 $539 = $7;
 $540 = $537 ? $538 : $539;
 $fst_col = $540;
 $541 = $fst_col;
 $542 = $1;
 $543 = ($541|0)<($542|0);
 L49: do {
  if ($543) {
   $544 = $fst_col;
   $545 = $fsupc;
   $546 = (($544) - ($545))|0;
   $d_fsupc = $546;
   $547 = $fsupc;
   $548 = $xlsub;
   $549 = (($548) + ($547<<2)|0);
   $550 = HEAP32[$549>>2]|0;
   $551 = $d_fsupc;
   $552 = (($550) + ($551))|0;
   $lptr = $552;
   $553 = $fst_col;
   $554 = $xlusup;
   $555 = (($554) + ($553<<2)|0);
   $556 = HEAP32[$555>>2]|0;
   $557 = $d_fsupc;
   $558 = (($556) + ($557))|0;
   $luptr = $558;
   $559 = $fsupc;
   $560 = (($559) + 1)|0;
   $561 = $xlsub;
   $562 = (($561) + ($560<<2)|0);
   $563 = HEAP32[$562>>2]|0;
   $564 = $fsupc;
   $565 = $xlsub;
   $566 = (($565) + ($564<<2)|0);
   $567 = HEAP32[$566>>2]|0;
   $568 = (($563) - ($567))|0;
   $nsupr = $568;
   $569 = $1;
   $570 = $fst_col;
   $571 = (($569) - ($570))|0;
   $nsupc = $571;
   $572 = $nsupr;
   $573 = $d_fsupc;
   $574 = (($572) - ($573))|0;
   $575 = $nsupc;
   $576 = (($574) - ($575))|0;
   $nrow = $576;
   $577 = $1;
   $578 = $xlusup;
   $579 = (($578) + ($577<<2)|0);
   $580 = HEAP32[$579>>2]|0;
   $581 = $d_fsupc;
   $582 = (($580) + ($581))|0;
   $ufirst = $582;
   $583 = $nsupc;
   $584 = $nsupc;
   $585 = (($584) - 1)|0;
   $586 = Math_imul($583, $585)|0;
   $587 = (+($586|0));
   $588 = $ops;
   $589 = ((($588)) + 52|0);
   $590 = +HEAPF32[$589>>2];
   $591 = $590 + $587;
   HEAPF32[$589>>2] = $591;
   $592 = $nrow;
   $593 = $592<<1;
   $594 = $nsupc;
   $595 = Math_imul($593, $594)|0;
   $596 = (+($595|0));
   $597 = $ops;
   $598 = ((($597)) + 56|0);
   $599 = +HEAPF32[$598>>2];
   $600 = $599 + $596;
   HEAPF32[$598>>2] = $600;
   $601 = $nsupr;
   $602 = $nsupc;
   $603 = $luptr;
   $604 = $lusup;
   $605 = (($604) + ($603<<3)|0);
   $606 = $ufirst;
   $607 = $lusup;
   $608 = (($607) + ($606<<3)|0);
   _dlsolve($601,$602,$605,$608);
   $609 = $nsupr;
   $610 = $nrow;
   $611 = $nsupc;
   $612 = $luptr;
   $613 = $nsupc;
   $614 = (($612) + ($613))|0;
   $615 = $lusup;
   $616 = (($615) + ($614<<3)|0);
   $617 = $ufirst;
   $618 = $lusup;
   $619 = (($618) + ($617<<3)|0);
   $620 = $4;
   _dmatvec($609,$610,$611,$616,$619,$620);
   $621 = $ufirst;
   $622 = $nsupc;
   $623 = (($621) + ($622))|0;
   $isub = $623;
   $i = 0;
   while(1) {
    $624 = $i;
    $625 = $nrow;
    $626 = ($624|0)<($625|0);
    if (!($626)) {
     break L49;
    }
    $627 = $i;
    $628 = $4;
    $629 = (($628) + ($627<<3)|0);
    $630 = +HEAPF64[$629>>3];
    $631 = $isub;
    $632 = $lusup;
    $633 = (($632) + ($631<<3)|0);
    $634 = +HEAPF64[$633>>3];
    $635 = $634 - $630;
    HEAPF64[$633>>3] = $635;
    $636 = $i;
    $637 = $4;
    $638 = (($637) + ($636<<3)|0);
    HEAPF64[$638>>3] = 0.0;
    $639 = $isub;
    $640 = (($639) + 1)|0;
    $isub = $640;
    $641 = $i;
    $642 = (($641) + 1)|0;
    $i = $642;
   }
  }
 } while(0);
 $0 = 0;
 $643 = $0;
 STACKTOP = sp;return ($643|0);
}
function _dpivotL($jcol,$u,$usepr,$perm_r,$iperm_r,$iperm_c,$pivrow,$Glu,$stat) {
 $jcol = $jcol|0;
 $u = +$u;
 $usepr = $usepr|0;
 $perm_r = $perm_r|0;
 $iperm_r = $iperm_r|0;
 $iperm_c = $iperm_c|0;
 $pivrow = $pivrow|0;
 $Glu = $Glu|0;
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0.0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0.0, $137 = 0.0, $138 = 0.0, $139 = 0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0.0, $154 = 0.0, $155 = 0.0, $156 = 0, $157 = 0.0, $158 = 0.0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0.0, $20 = 0, $200 = 0.0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0.0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0.0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0.0, $226 = 0, $227 = 0, $228 = 0.0, $229 = 0.0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0.0, $234 = 0.0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0.0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0.0, $245 = 0.0, $246 = 0, $247 = 0, $248 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0.0, $89 = 0, $9 = 0, $90 = 0.0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $diag = 0, $diagind = 0, $fsupc = 0, $icol = 0, $isub = 0, $itemp = 0, $k = 0, $lptr = 0, $lsub = 0, $lsub_ptr = 0, $lu_col_ptr = 0;
 var $lu_sup_ptr = 0, $lusup = 0, $nsupc = 0, $nsupr = 0, $old_pivptr = 0, $ops = 0, $pivmax = 0.0, $pivptr = 0, $rtemp = 0.0, $temp = 0.0, $thresh = 0.0, $xlsub = 0, $xlusup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $jcol;
 $2 = $u;
 $3 = $usepr;
 $4 = $perm_r;
 $5 = $iperm_r;
 $6 = $iperm_c;
 $7 = $pivrow;
 $8 = $Glu;
 $9 = $stat;
 $10 = $9;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $ops = $12;
 $13 = $8;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $lsub = $15;
 $16 = $8;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $xlsub = $18;
 $19 = $8;
 $20 = ((($19)) + 16|0);
 $21 = HEAP32[$20>>2]|0;
 $lusup = $21;
 $22 = $8;
 $23 = ((($22)) + 20|0);
 $24 = HEAP32[$23>>2]|0;
 $xlusup = $24;
 $25 = $1;
 $26 = $8;
 $27 = ((($26)) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($25<<2)|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $8;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + ($30<<2)|0);
 $34 = HEAP32[$33>>2]|0;
 $fsupc = $34;
 $35 = $1;
 $36 = $fsupc;
 $37 = (($35) - ($36))|0;
 $nsupc = $37;
 $38 = $fsupc;
 $39 = $xlsub;
 $40 = (($39) + ($38<<2)|0);
 $41 = HEAP32[$40>>2]|0;
 $lptr = $41;
 $42 = $fsupc;
 $43 = (($42) + 1)|0;
 $44 = $xlsub;
 $45 = (($44) + ($43<<2)|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = $lptr;
 $48 = (($46) - ($47))|0;
 $nsupr = $48;
 $49 = $fsupc;
 $50 = $xlusup;
 $51 = (($50) + ($49<<2)|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = $lusup;
 $54 = (($53) + ($52<<3)|0);
 $lu_sup_ptr = $54;
 $55 = $1;
 $56 = $xlusup;
 $57 = (($56) + ($55<<2)|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $lusup;
 $60 = (($59) + ($58<<3)|0);
 $lu_col_ptr = $60;
 $61 = $lptr;
 $62 = $lsub;
 $63 = (($62) + ($61<<2)|0);
 $lsub_ptr = $63;
 $64 = $3;
 $65 = HEAP32[$64>>2]|0;
 $66 = ($65|0)!=(0);
 if ($66) {
  $67 = $1;
  $68 = $5;
  $69 = (($68) + ($67<<2)|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $7;
  HEAP32[$71>>2] = $70;
 }
 $72 = $1;
 $73 = $6;
 $74 = (($73) + ($72<<2)|0);
 $75 = HEAP32[$74>>2]|0;
 $diagind = $75;
 $pivmax = 0.0;
 $76 = $nsupc;
 $pivptr = $76;
 $diag = -1;
 $77 = $nsupc;
 $old_pivptr = $77;
 $78 = $nsupc;
 $isub = $78;
 while(1) {
  $79 = $isub;
  $80 = $nsupr;
  $81 = ($79|0)<($80|0);
  if (!($81)) {
   break;
  }
  $82 = $isub;
  $83 = $lu_col_ptr;
  $84 = (($83) + ($82<<3)|0);
  $85 = +HEAPF64[$84>>3];
  $86 = (+Math_abs((+$85)));
  $rtemp = $86;
  $87 = $rtemp;
  $88 = $pivmax;
  $89 = $87 > $88;
  if ($89) {
   $90 = $rtemp;
   $pivmax = $90;
   $91 = $isub;
   $pivptr = $91;
  }
  $92 = $3;
  $93 = HEAP32[$92>>2]|0;
  $94 = ($93|0)!=(0);
  if ($94) {
   $95 = $isub;
   $96 = $lsub_ptr;
   $97 = (($96) + ($95<<2)|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = $7;
   $100 = HEAP32[$99>>2]|0;
   $101 = ($98|0)==($100|0);
   if ($101) {
    $102 = $isub;
    $old_pivptr = $102;
   }
  }
  $103 = $isub;
  $104 = $lsub_ptr;
  $105 = (($104) + ($103<<2)|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = $diagind;
  $108 = ($106|0)==($107|0);
  if ($108) {
   $109 = $isub;
   $diag = $109;
  }
  $110 = $isub;
  $111 = (($110) + 1)|0;
  $isub = $111;
 }
 $112 = $pivmax;
 $113 = $112 == 0.0;
 if ($113) {
  $114 = $pivptr;
  $115 = $lsub_ptr;
  $116 = (($115) + ($114<<2)|0);
  $117 = HEAP32[$116>>2]|0;
  $118 = $7;
  HEAP32[$118>>2] = $117;
  $119 = $1;
  $120 = $7;
  $121 = HEAP32[$120>>2]|0;
  $122 = $4;
  $123 = (($122) + ($121<<2)|0);
  HEAP32[$123>>2] = $119;
  $124 = $3;
  HEAP32[$124>>2] = 0;
  $125 = $1;
  $126 = (($125) + 1)|0;
  $0 = $126;
  $248 = $0;
  STACKTOP = sp;return ($248|0);
 }
 $127 = $2;
 $128 = $pivmax;
 $129 = $127 * $128;
 $thresh = $129;
 $130 = $3;
 $131 = HEAP32[$130>>2]|0;
 $132 = ($131|0)!=(0);
 do {
  if ($132) {
   $133 = $old_pivptr;
   $134 = $lu_col_ptr;
   $135 = (($134) + ($133<<3)|0);
   $136 = +HEAPF64[$135>>3];
   $137 = (+Math_abs((+$136)));
   $rtemp = $137;
   $138 = $rtemp;
   $139 = $138 != 0.0;
   if ($139) {
    $140 = $rtemp;
    $141 = $thresh;
    $142 = $140 >= $141;
    if ($142) {
     $143 = $old_pivptr;
     $pivptr = $143;
     break;
    }
   }
   $144 = $3;
   HEAP32[$144>>2] = 0;
  }
 } while(0);
 $145 = $3;
 $146 = HEAP32[$145>>2]|0;
 $147 = ($146|0)==(0);
 if ($147) {
  $148 = $diag;
  $149 = ($148|0)>=(0);
  if ($149) {
   $150 = $diag;
   $151 = $lu_col_ptr;
   $152 = (($151) + ($150<<3)|0);
   $153 = +HEAPF64[$152>>3];
   $154 = (+Math_abs((+$153)));
   $rtemp = $154;
   $155 = $rtemp;
   $156 = $155 != 0.0;
   if ($156) {
    $157 = $rtemp;
    $158 = $thresh;
    $159 = $157 >= $158;
    if ($159) {
     $160 = $diag;
     $pivptr = $160;
    }
   }
  }
  $161 = $pivptr;
  $162 = $lsub_ptr;
  $163 = (($162) + ($161<<2)|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = $7;
  HEAP32[$165>>2] = $164;
 }
 $166 = $1;
 $167 = $7;
 $168 = HEAP32[$167>>2]|0;
 $169 = $4;
 $170 = (($169) + ($168<<2)|0);
 HEAP32[$170>>2] = $166;
 $171 = $pivptr;
 $172 = $nsupc;
 $173 = ($171|0)!=($172|0);
 L37: do {
  if ($173) {
   $174 = $pivptr;
   $175 = $lsub_ptr;
   $176 = (($175) + ($174<<2)|0);
   $177 = HEAP32[$176>>2]|0;
   $itemp = $177;
   $178 = $nsupc;
   $179 = $lsub_ptr;
   $180 = (($179) + ($178<<2)|0);
   $181 = HEAP32[$180>>2]|0;
   $182 = $pivptr;
   $183 = $lsub_ptr;
   $184 = (($183) + ($182<<2)|0);
   HEAP32[$184>>2] = $181;
   $185 = $itemp;
   $186 = $nsupc;
   $187 = $lsub_ptr;
   $188 = (($187) + ($186<<2)|0);
   HEAP32[$188>>2] = $185;
   $icol = 0;
   while(1) {
    $189 = $icol;
    $190 = $nsupc;
    $191 = ($189|0)<=($190|0);
    if (!($191)) {
     break L37;
    }
    $192 = $pivptr;
    $193 = $icol;
    $194 = $nsupr;
    $195 = Math_imul($193, $194)|0;
    $196 = (($192) + ($195))|0;
    $itemp = $196;
    $197 = $itemp;
    $198 = $lu_sup_ptr;
    $199 = (($198) + ($197<<3)|0);
    $200 = +HEAPF64[$199>>3];
    $temp = $200;
    $201 = $nsupc;
    $202 = $icol;
    $203 = $nsupr;
    $204 = Math_imul($202, $203)|0;
    $205 = (($201) + ($204))|0;
    $206 = $lu_sup_ptr;
    $207 = (($206) + ($205<<3)|0);
    $208 = +HEAPF64[$207>>3];
    $209 = $itemp;
    $210 = $lu_sup_ptr;
    $211 = (($210) + ($209<<3)|0);
    HEAPF64[$211>>3] = $208;
    $212 = $temp;
    $213 = $nsupc;
    $214 = $icol;
    $215 = $nsupr;
    $216 = Math_imul($214, $215)|0;
    $217 = (($213) + ($216))|0;
    $218 = $lu_sup_ptr;
    $219 = (($218) + ($217<<3)|0);
    HEAPF64[$219>>3] = $212;
    $220 = $icol;
    $221 = (($220) + 1)|0;
    $icol = $221;
   }
  }
 } while(0);
 $222 = $nsupr;
 $223 = $nsupc;
 $224 = (($222) - ($223))|0;
 $225 = (+($224|0));
 $226 = $ops;
 $227 = ((($226)) + 28|0);
 $228 = +HEAPF32[$227>>2];
 $229 = $228 + $225;
 HEAPF32[$227>>2] = $229;
 $230 = $nsupc;
 $231 = $lu_col_ptr;
 $232 = (($231) + ($230<<3)|0);
 $233 = +HEAPF64[$232>>3];
 $234 = 1.0 / $233;
 $temp = $234;
 $235 = $nsupc;
 $236 = (($235) + 1)|0;
 $k = $236;
 while(1) {
  $237 = $k;
  $238 = $nsupr;
  $239 = ($237|0)<($238|0);
  if (!($239)) {
   break;
  }
  $240 = $temp;
  $241 = $k;
  $242 = $lu_col_ptr;
  $243 = (($242) + ($241<<3)|0);
  $244 = +HEAPF64[$243>>3];
  $245 = $244 * $240;
  HEAPF64[$243>>3] = $245;
  $246 = $k;
  $247 = (($246) + 1)|0;
  $k = $247;
 }
 $0 = 0;
 $248 = $0;
 STACKTOP = sp;return ($248|0);
}
function _dpruneL($jcol,$perm_r,$pivrow,$nseg,$segrep,$repfnz,$xprune,$Glu) {
 $jcol = $jcol|0;
 $perm_r = $perm_r|0;
 $pivrow = $pivrow|0;
 $nseg = $nseg|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $xprune = $xprune|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0.0, $164 = 0, $165 = 0, $166 = 0, $167 = 0.0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0.0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $do_prune = 0, $i = 0, $irep = 0, $irep1 = 0, $jsupno = 0, $kmax = 0, $kmin = 0, $krow = 0, $ktemp = 0, $lsub = 0, $lusup = 0, $maxloc = 0, $minloc = 0, $movnum = 0, $supno = 0;
 var $utemp = 0.0, $xlsub = 0, $xlusup = 0, $xsup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $jcol;
 $1 = $perm_r;
 $2 = $pivrow;
 $3 = $nseg;
 $4 = $segrep;
 $5 = $repfnz;
 $6 = $xprune;
 $7 = $Glu;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $xsup = $9;
 $10 = $7;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $supno = $12;
 $13 = $7;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $lsub = $15;
 $16 = $7;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $xlsub = $18;
 $19 = $7;
 $20 = ((($19)) + 16|0);
 $21 = HEAP32[$20>>2]|0;
 $lusup = $21;
 $22 = $7;
 $23 = ((($22)) + 20|0);
 $24 = HEAP32[$23>>2]|0;
 $xlusup = $24;
 $25 = $0;
 $26 = $supno;
 $27 = (($26) + ($25<<2)|0);
 $28 = HEAP32[$27>>2]|0;
 $jsupno = $28;
 $i = 0;
 while(1) {
  $29 = $i;
  $30 = $3;
  $31 = ($29|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = $i;
  $33 = $4;
  $34 = (($33) + ($32<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $irep = $35;
  $36 = $irep;
  $37 = (($36) + 1)|0;
  $irep1 = $37;
  $do_prune = 0;
  $38 = $irep;
  $39 = $5;
  $40 = (($39) + ($38<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ($41|0)==(-1);
  if (!($42)) {
   $43 = $irep;
   $44 = $supno;
   $45 = (($44) + ($43<<2)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = $irep1;
   $48 = $supno;
   $49 = (($48) + ($47<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = ($46|0)==($50|0);
   if (!($51)) {
    $52 = $irep;
    $53 = $supno;
    $54 = (($53) + ($52<<2)|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = $jsupno;
    $57 = ($55|0)!=($56|0);
    if ($57) {
     $58 = $irep;
     $59 = $6;
     $60 = (($59) + ($58<<2)|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = $irep1;
     $63 = $xlsub;
     $64 = (($63) + ($62<<2)|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = ($61|0)>=($65|0);
     L8: do {
      if ($66) {
       $67 = $irep;
       $68 = $xlsub;
       $69 = (($68) + ($67<<2)|0);
       $70 = HEAP32[$69>>2]|0;
       $kmin = $70;
       $71 = $irep1;
       $72 = $xlsub;
       $73 = (($72) + ($71<<2)|0);
       $74 = HEAP32[$73>>2]|0;
       $75 = (($74) - 1)|0;
       $kmax = $75;
       $76 = $kmin;
       $krow = $76;
       while(1) {
        $77 = $krow;
        $78 = $kmax;
        $79 = ($77|0)<=($78|0);
        if (!($79)) {
         break L8;
        }
        $80 = $krow;
        $81 = $lsub;
        $82 = (($81) + ($80<<2)|0);
        $83 = HEAP32[$82>>2]|0;
        $84 = $2;
        $85 = ($83|0)==($84|0);
        if ($85) {
         break;
        }
        $86 = $krow;
        $87 = (($86) + 1)|0;
        $krow = $87;
       }
       $do_prune = 1;
      }
     } while(0);
     $88 = $do_prune;
     $89 = ($88|0)!=(0);
     if ($89) {
      $movnum = 0;
      $90 = $irep;
      $91 = $irep;
      $92 = $supno;
      $93 = (($92) + ($91<<2)|0);
      $94 = HEAP32[$93>>2]|0;
      $95 = $xsup;
      $96 = (($95) + ($94<<2)|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = ($90|0)==($97|0);
      if ($98) {
       $movnum = 1;
      }
      while(1) {
       $99 = $kmin;
       $100 = $kmax;
       $101 = ($99|0)<=($100|0);
       if (!($101)) {
        break;
       }
       $102 = $kmax;
       $103 = $lsub;
       $104 = (($103) + ($102<<2)|0);
       $105 = HEAP32[$104>>2]|0;
       $106 = $1;
       $107 = (($106) + ($105<<2)|0);
       $108 = HEAP32[$107>>2]|0;
       $109 = ($108|0)==(-1);
       if ($109) {
        $110 = $kmax;
        $111 = (($110) + -1)|0;
        $kmax = $111;
        continue;
       }
       $112 = $kmin;
       $113 = $lsub;
       $114 = (($113) + ($112<<2)|0);
       $115 = HEAP32[$114>>2]|0;
       $116 = $1;
       $117 = (($116) + ($115<<2)|0);
       $118 = HEAP32[$117>>2]|0;
       $119 = ($118|0)!=(-1);
       $120 = $kmin;
       if ($119) {
        $121 = (($120) + 1)|0;
        $kmin = $121;
        continue;
       }
       $122 = $lsub;
       $123 = (($122) + ($120<<2)|0);
       $124 = HEAP32[$123>>2]|0;
       $ktemp = $124;
       $125 = $kmax;
       $126 = $lsub;
       $127 = (($126) + ($125<<2)|0);
       $128 = HEAP32[$127>>2]|0;
       $129 = $kmin;
       $130 = $lsub;
       $131 = (($130) + ($129<<2)|0);
       HEAP32[$131>>2] = $128;
       $132 = $ktemp;
       $133 = $kmax;
       $134 = $lsub;
       $135 = (($134) + ($133<<2)|0);
       HEAP32[$135>>2] = $132;
       $136 = $movnum;
       $137 = ($136|0)!=(0);
       if ($137) {
        $138 = $irep;
        $139 = $xlusup;
        $140 = (($139) + ($138<<2)|0);
        $141 = HEAP32[$140>>2]|0;
        $142 = $kmin;
        $143 = $irep;
        $144 = $xlsub;
        $145 = (($144) + ($143<<2)|0);
        $146 = HEAP32[$145>>2]|0;
        $147 = (($142) - ($146))|0;
        $148 = (($141) + ($147))|0;
        $minloc = $148;
        $149 = $irep;
        $150 = $xlusup;
        $151 = (($150) + ($149<<2)|0);
        $152 = HEAP32[$151>>2]|0;
        $153 = $kmax;
        $154 = $irep;
        $155 = $xlsub;
        $156 = (($155) + ($154<<2)|0);
        $157 = HEAP32[$156>>2]|0;
        $158 = (($153) - ($157))|0;
        $159 = (($152) + ($158))|0;
        $maxloc = $159;
        $160 = $minloc;
        $161 = $lusup;
        $162 = (($161) + ($160<<3)|0);
        $163 = +HEAPF64[$162>>3];
        $utemp = $163;
        $164 = $maxloc;
        $165 = $lusup;
        $166 = (($165) + ($164<<3)|0);
        $167 = +HEAPF64[$166>>3];
        $168 = $minloc;
        $169 = $lusup;
        $170 = (($169) + ($168<<3)|0);
        HEAPF64[$170>>3] = $167;
        $171 = $utemp;
        $172 = $maxloc;
        $173 = $lusup;
        $174 = (($173) + ($172<<3)|0);
        HEAPF64[$174>>3] = $171;
       }
       $175 = $kmin;
       $176 = (($175) + 1)|0;
       $kmin = $176;
       $177 = $kmax;
       $178 = (($177) + -1)|0;
       $kmax = $178;
      }
      $179 = $kmin;
      $180 = $irep;
      $181 = $6;
      $182 = (($181) + ($180<<2)|0);
      HEAP32[$182>>2] = $179;
     }
    }
   }
  }
  $183 = $i;
  $184 = (($183) + 1)|0;
  $i = $184;
 }
 STACKTOP = sp;return;
}
function _dSetupSpace($work,$lwork,$Glu) {
 $work = $work|0;
 $lwork = $lwork|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $work;
 $1 = $lwork;
 $2 = $Glu;
 $3 = $1;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = $2;
  $6 = ((($5)) + 52|0);
  HEAP32[$6>>2] = 0;
  STACKTOP = sp;return;
 }
 $7 = $1;
 $8 = ($7|0)>(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = $2;
 $10 = ((($9)) + 52|0);
 HEAP32[$10>>2] = 1;
 $11 = $2;
 $12 = ((($11)) + 64|0);
 $13 = ((($12)) + 4|0);
 HEAP32[$13>>2] = 0;
 $14 = $2;
 $15 = ((($14)) + 64|0);
 $16 = ((($15)) + 8|0);
 HEAP32[$16>>2] = 0;
 $17 = $1;
 $18 = (($17|0) / 4)&-1;
 $19 = $18<<2;
 $20 = $2;
 $21 = ((($20)) + 64|0);
 $22 = ((($21)) + 12|0);
 HEAP32[$22>>2] = $19;
 $23 = $2;
 $24 = ((($23)) + 64|0);
 $25 = ((($24)) + 12|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $2;
 $28 = ((($27)) + 64|0);
 HEAP32[$28>>2] = $26;
 $29 = $0;
 $30 = $2;
 $31 = ((($30)) + 64|0);
 $32 = ((($31)) + 16|0);
 HEAP32[$32>>2] = $29;
 STACKTOP = sp;return;
}
function _duser_malloc($bytes,$which_end,$Glu) {
 $bytes = $bytes|0;
 $which_end = $which_end|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $bytes;
 $2 = $which_end;
 $3 = $Glu;
 $4 = $1;
 $5 = $3;
 $6 = ((($5)) + 64|0);
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($4) + ($8))|0;
 $10 = $3;
 $11 = ((($10)) + 64|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($9|0)>=($12|0);
 if ($13) {
  $0 = 0;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 }
 $14 = $2;
 $15 = ($14|0)==(0);
 if ($15) {
  $16 = $3;
  $17 = ((($16)) + 64|0);
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $3;
  $21 = ((($20)) + 64|0);
  $22 = ((($21)) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($19) + ($23)|0);
  $buf = $24;
  $25 = $1;
  $26 = $3;
  $27 = ((($26)) + 64|0);
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($25))|0;
  HEAP32[$28>>2] = $30;
 } else {
  $31 = $1;
  $32 = $3;
  $33 = ((($32)) + 64|0);
  $34 = ((($33)) + 12|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) - ($31))|0;
  HEAP32[$34>>2] = $36;
  $37 = $3;
  $38 = ((($37)) + 64|0);
  $39 = ((($38)) + 16|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = $3;
  $42 = ((($41)) + 64|0);
  $43 = ((($42)) + 12|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($40) + ($44)|0);
  $buf = $45;
 }
 $46 = $1;
 $47 = $3;
 $48 = ((($47)) + 64|0);
 $49 = ((($48)) + 4|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = (($50) + ($46))|0;
 HEAP32[$49>>2] = $51;
 $52 = $buf;
 $0 = $52;
 $53 = $0;
 STACKTOP = sp;return ($53|0);
}
function _duser_free($bytes,$which_end,$Glu) {
 $bytes = $bytes|0;
 $which_end = $which_end|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $bytes;
 $1 = $which_end;
 $2 = $Glu;
 $3 = $1;
 $4 = ($3|0)==(0);
 $5 = $0;
 $6 = $2;
 $7 = ((($6)) + 64|0);
 if ($4) {
  $8 = ((($7)) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) - ($5))|0;
  HEAP32[$8>>2] = $10;
 } else {
  $11 = ((($7)) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($5))|0;
  HEAP32[$11>>2] = $13;
 }
 $14 = $0;
 $15 = $2;
 $16 = ((($15)) + 64|0);
 $17 = ((($16)) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) - ($14))|0;
 HEAP32[$17>>2] = $19;
 STACKTOP = sp;return;
}
function _dQuerySpace($L,$U,$mem_usage) {
 $L = $L|0;
 $U = $U|0;
 $mem_usage = $mem_usage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0, $42 = 0, $43 = 0.0, $44 = 0.0;
 var $45 = 0, $46 = 0.0, $47 = 0.0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0, $62 = 0.0;
 var $63 = 0.0, $64 = 0, $65 = 0.0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0.0, $73 = 0.0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0;
 var $81 = 0.0, $82 = 0.0, $83 = 0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0.0, $89 = 0, $9 = 0, $90 = 0, $Lstore = 0, $Ustore = 0, $dword = 0, $iword = 0, $n = 0, $panel_size = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $L;
 $1 = $U;
 $2 = $mem_usage;
 $3 = (_sp_ienv(1)|0);
 $panel_size = $3;
 $4 = $0;
 $5 = ((($4)) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $Lstore = $6;
 $7 = $1;
 $8 = ((($7)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 $Ustore = $9;
 $10 = $0;
 $11 = ((($10)) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $n = $12;
 $iword = 4;
 $dword = 8;
 $13 = $n;
 $14 = (+($13|0));
 $15 = 4.0 * $14;
 $16 = $15 + 3.0;
 $17 = $iword;
 $18 = (+($17|0));
 $19 = $16 * $18;
 $20 = $n;
 $21 = $Lstore;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20<<2)|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $dword;
 $27 = Math_imul($25, $26)|0;
 $28 = (+($27|0));
 $29 = $19 + $28;
 $30 = $n;
 $31 = $Lstore;
 $32 = ((($31)) + 20|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + ($30<<2)|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = $iword;
 $37 = Math_imul($35, $36)|0;
 $38 = (+($37|0));
 $39 = $29 + $38;
 $40 = $39;
 $41 = $2;
 HEAPF32[$41>>2] = $40;
 $42 = $n;
 $43 = (+($42|0));
 $44 = $43 + 1.0;
 $45 = $iword;
 $46 = (+($45|0));
 $47 = $44 * $46;
 $48 = $n;
 $49 = $Ustore;
 $50 = ((($49)) + 12|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($48<<2)|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = $dword;
 $55 = $iword;
 $56 = (($54) + ($55))|0;
 $57 = Math_imul($53, $56)|0;
 $58 = (+($57|0));
 $59 = $47 + $58;
 $60 = $59;
 $61 = $2;
 $62 = +HEAPF32[$61>>2];
 $63 = $62 + $60;
 HEAPF32[$61>>2] = $63;
 $64 = $2;
 $65 = +HEAPF32[$64>>2];
 $66 = $panel_size;
 $67 = (+($66|0));
 $68 = 2.0 * $67;
 $69 = $68 + 4.0;
 $70 = $69 + 3.0;
 $71 = $n;
 $72 = (+($71|0));
 $73 = $70 * $72;
 $74 = $iword;
 $75 = (+($74|0));
 $76 = $73 * $75;
 $77 = $panel_size;
 $78 = (+($77|0));
 $79 = $78 + 1.0;
 $80 = $n;
 $81 = (+($80|0));
 $82 = $79 * $81;
 $83 = $dword;
 $84 = (+($83|0));
 $85 = $82 * $84;
 $86 = $76 + $85;
 $87 = $86;
 $88 = $65 + $87;
 $89 = $2;
 $90 = ((($89)) + 4|0);
 HEAPF32[$90>>2] = $88;
 STACKTOP = sp;return 0;
}
function _dLUMemInit($fact,$work,$lwork,$m,$n,$annz,$panel_size,$fill_ratio,$L,$U,$Glu,$iwork,$dwork) {
 $fact = $fact|0;
 $work = $work|0;
 $lwork = $lwork|0;
 $m = $m|0;
 $n = $n|0;
 $annz = $annz|0;
 $panel_size = $panel_size|0;
 $fill_ratio = +$fill_ratio;
 $L = $L|0;
 $U = $U|0;
 $Glu = $Glu|0;
 $iwork = $iwork|0;
 $dwork = $dwork|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0.0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0.0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0.0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0.0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0.0, $380 = 0, $381 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0;
 var $43 = 0, $44 = 0.0, $45 = 0.0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0.0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $Lstore = 0, $Ustore = 0, $dword = 0, $info = 0, $iword = 0, $lsub = 0, $lusup = 0, $msg = 0, $nzlmax = 0, $nzlumax = 0, $nzumax = 0, $or$cond = 0, $or$cond3 = 0, $supno = 0, $ucol = 0, $usub = 0, $vararg_buffer = 0, $vararg_buffer6 = 0;
 var $vararg_ptr4 = 0, $vararg_ptr5 = 0, $xlsub = 0, $xlusup = 0, $xsup = 0, $xusub = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 416|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer = sp + 8|0;
 $nzlmax = sp + 36|0;
 $nzumax = sp + 32|0;
 $nzlumax = sp + 28|0;
 $msg = sp + 152|0;
 $1 = $fact;
 $2 = $work;
 $3 = $lwork;
 $4 = $m;
 $5 = $n;
 $6 = $annz;
 $7 = $panel_size;
 $8 = $fill_ratio;
 $9 = $L;
 $10 = $U;
 $11 = $Glu;
 $12 = $iwork;
 $13 = $dwork;
 $iword = 4;
 $dword = 8;
 $14 = $5;
 $15 = $11;
 $16 = ((($15)) + 48|0);
 HEAP32[$16>>2] = $14;
 $17 = $11;
 $18 = ((($17)) + 56|0);
 HEAP32[$18>>2] = 0;
 $19 = $11;
 $20 = ((($19)) + 60|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  $23 = (_superlu_malloc(32)|0);
  $24 = $11;
  $25 = ((($24)) + 60|0);
  HEAP32[$25>>2] = $23;
 }
 $26 = $11;
 $27 = ((($26)) + 60|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(0|0);
 if (!($29)) {
  HEAP32[$vararg_buffer>>2] = 2520;
  $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 203;
  $vararg_ptr5 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr5>>2] = 1856;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $30 = $1;
 $31 = ($30|0)!=(2);
 L7: do {
  if ($31) {
   $32 = $8;
   $33 = $6;
   $34 = (+($33|0));
   $35 = $32 * $34;
   $36 = (~~(($35)));
   HEAP32[$nzlumax>>2] = $36;
   HEAP32[$nzumax>>2] = $36;
   $37 = $8;
   $38 = $37 / 4.0;
   $39 = 1.0 > $38;
   $40 = $8;
   $41 = $40 / 4.0;
   $42 = $39 ? 1.0 : $41;
   $43 = $6;
   $44 = (+($43|0));
   $45 = $42 * $44;
   $46 = (~~(($45)));
   HEAP32[$nzlmax>>2] = $46;
   $47 = $3;
   $48 = ($47|0)==(-1);
   if ($48) {
    $49 = $5;
    $50 = ($49*5)|0;
    $51 = (($50) + 5)|0;
    $52 = $iword;
    $53 = Math_imul($51, $52)|0;
    $54 = $7;
    $55 = $54<<1;
    $56 = (($55) + 4)|0;
    $57 = (($56) + 3)|0;
    $58 = $4;
    $59 = Math_imul($57, $58)|0;
    $60 = $59<<2;
    $61 = $7;
    $62 = (($61) + 1)|0;
    $63 = $4;
    $64 = Math_imul($62, $63)|0;
    $65 = $64<<3;
    $66 = (($60) + ($65))|0;
    $67 = (($53) + ($66))|0;
    $68 = HEAP32[$nzlmax>>2]|0;
    $69 = HEAP32[$nzumax>>2]|0;
    $70 = (($68) + ($69))|0;
    $71 = $iword;
    $72 = Math_imul($70, $71)|0;
    $73 = (($67) + ($72))|0;
    $74 = HEAP32[$nzlumax>>2]|0;
    $75 = HEAP32[$nzumax>>2]|0;
    $76 = (($74) + ($75))|0;
    $77 = $dword;
    $78 = Math_imul($76, $77)|0;
    $79 = (($73) + ($78))|0;
    $80 = $5;
    $81 = (($79) + ($80))|0;
    $0 = $81;
    $381 = $0;
    STACKTOP = sp;return ($381|0);
   }
   $82 = $2;
   $83 = $3;
   $84 = $11;
   _dSetupSpace($82,$83,$84);
   $85 = $11;
   $86 = ((($85)) + 52|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = ($87|0)==(0);
   $89 = $5;
   $90 = (($89) + 1)|0;
   if ($88) {
    $91 = (_intMalloc($90)|0);
    $xsup = $91;
    $92 = $5;
    $93 = (($92) + 1)|0;
    $94 = (_intMalloc($93)|0);
    $supno = $94;
    $95 = $5;
    $96 = (($95) + 1)|0;
    $97 = (_intMalloc($96)|0);
    $xlsub = $97;
    $98 = $5;
    $99 = (($98) + 1)|0;
    $100 = (_intMalloc($99)|0);
    $xlusup = $100;
    $101 = $5;
    $102 = (($101) + 1)|0;
    $103 = (_intMalloc($102)|0);
    $xusub = $103;
   } else {
    $104 = $iword;
    $105 = Math_imul($90, $104)|0;
    $106 = $11;
    $107 = (_duser_malloc($105,0,$106)|0);
    $xsup = $107;
    $108 = $5;
    $109 = (($108) + 1)|0;
    $110 = $iword;
    $111 = Math_imul($109, $110)|0;
    $112 = $11;
    $113 = (_duser_malloc($111,0,$112)|0);
    $supno = $113;
    $114 = $5;
    $115 = (($114) + 1)|0;
    $116 = $iword;
    $117 = Math_imul($115, $116)|0;
    $118 = $11;
    $119 = (_duser_malloc($117,0,$118)|0);
    $xlsub = $119;
    $120 = $5;
    $121 = (($120) + 1)|0;
    $122 = $iword;
    $123 = Math_imul($121, $122)|0;
    $124 = $11;
    $125 = (_duser_malloc($123,0,$124)|0);
    $xlusup = $125;
    $126 = $5;
    $127 = (($126) + 1)|0;
    $128 = $iword;
    $129 = Math_imul($127, $128)|0;
    $130 = $11;
    $131 = (_duser_malloc($129,0,$130)|0);
    $xusub = $131;
   }
   $132 = $11;
   $133 = (_dexpand($nzlumax,0,0,0,$132)|0);
   $lusup = $133;
   $134 = $11;
   $135 = (_dexpand($nzumax,1,0,0,$134)|0);
   $ucol = $135;
   $136 = $11;
   $137 = (_dexpand($nzlmax,2,0,0,$136)|0);
   $lsub = $137;
   $138 = $11;
   $139 = (_dexpand($nzumax,3,0,1,$138)|0);
   $usub = $139;
   while(1) {
    $140 = $lusup;
    $141 = ($140|0)!=(0|0);
    $142 = $ucol;
    $143 = ($142|0)!=(0|0);
    $or$cond = $141 & $143;
    $144 = $lsub;
    $145 = ($144|0)!=(0|0);
    $or$cond3 = $or$cond & $145;
    if ($or$cond3) {
     $146 = $usub;
     $147 = ($146|0)!=(0|0);
     $148 = $147 ^ 1;
     if (!($148)) {
      break L7;
     }
    }
    $149 = $11;
    $150 = ((($149)) + 52|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = ($151|0)==(0);
    if ($152) {
     $153 = $lusup;
     _superlu_free($153);
     $154 = $ucol;
     _superlu_free($154);
     $155 = $lsub;
     _superlu_free($155);
     $156 = $usub;
     _superlu_free($156);
    } else {
     $157 = HEAP32[$nzlumax>>2]|0;
     $158 = HEAP32[$nzumax>>2]|0;
     $159 = (($157) + ($158))|0;
     $160 = $dword;
     $161 = Math_imul($159, $160)|0;
     $162 = HEAP32[$nzlmax>>2]|0;
     $163 = HEAP32[$nzumax>>2]|0;
     $164 = (($162) + ($163))|0;
     $165 = $iword;
     $166 = Math_imul($164, $165)|0;
     $167 = (($161) + ($166))|0;
     $168 = $11;
     _duser_free($167,0,$168);
    }
    $169 = HEAP32[$nzlumax>>2]|0;
    $170 = (($169|0) / 2)&-1;
    HEAP32[$nzlumax>>2] = $170;
    $171 = HEAP32[$nzumax>>2]|0;
    $172 = (($171|0) / 2)&-1;
    HEAP32[$nzumax>>2] = $172;
    $173 = HEAP32[$nzlmax>>2]|0;
    $174 = (($173|0) / 2)&-1;
    HEAP32[$nzlmax>>2] = $174;
    $175 = HEAP32[$nzlumax>>2]|0;
    $176 = $6;
    $177 = ($175|0)<($176|0);
    if ($177) {
     break;
    }
    $185 = $11;
    $186 = (_dexpand($nzlumax,0,0,0,$185)|0);
    $lusup = $186;
    $187 = $11;
    $188 = (_dexpand($nzumax,1,0,0,$187)|0);
    $ucol = $188;
    $189 = $11;
    $190 = (_dexpand($nzlmax,2,0,0,$189)|0);
    $lsub = $190;
    $191 = $11;
    $192 = (_dexpand($nzumax,3,0,1,$191)|0);
    $usub = $192;
   }
   (_printf((2576|0),($vararg_buffer6|0))|0);
   $178 = HEAP32[$nzlmax>>2]|0;
   $179 = HEAP32[$nzumax>>2]|0;
   $180 = HEAP32[$nzlumax>>2]|0;
   $181 = $5;
   $182 = (_dmemory_usage($178,$179,$180,$181)|0);
   $183 = $5;
   $184 = (($182) + ($183))|0;
   $0 = $184;
   $381 = $0;
   STACKTOP = sp;return ($381|0);
  } else {
   $193 = $9;
   $194 = ((($193)) + 20|0);
   $195 = HEAP32[$194>>2]|0;
   $Lstore = $195;
   $196 = $10;
   $197 = ((($196)) + 20|0);
   $198 = HEAP32[$197>>2]|0;
   $Ustore = $198;
   $199 = $Lstore;
   $200 = ((($199)) + 28|0);
   $201 = HEAP32[$200>>2]|0;
   $xsup = $201;
   $202 = $Lstore;
   $203 = ((($202)) + 24|0);
   $204 = HEAP32[$203>>2]|0;
   $supno = $204;
   $205 = $Lstore;
   $206 = ((($205)) + 20|0);
   $207 = HEAP32[$206>>2]|0;
   $xlsub = $207;
   $208 = $Lstore;
   $209 = ((($208)) + 12|0);
   $210 = HEAP32[$209>>2]|0;
   $xlusup = $210;
   $211 = $Ustore;
   $212 = ((($211)) + 12|0);
   $213 = HEAP32[$212>>2]|0;
   $xusub = $213;
   $214 = $11;
   $215 = ((($214)) + 36|0);
   $216 = HEAP32[$215>>2]|0;
   HEAP32[$nzlmax>>2] = $216;
   $217 = $11;
   $218 = ((($217)) + 40|0);
   $219 = HEAP32[$218>>2]|0;
   HEAP32[$nzumax>>2] = $219;
   $220 = $11;
   $221 = ((($220)) + 44|0);
   $222 = HEAP32[$221>>2]|0;
   HEAP32[$nzlumax>>2] = $222;
   $223 = $3;
   $224 = ($223|0)==(-1);
   if ($224) {
    $225 = $5;
    $226 = ($225*5)|0;
    $227 = (($226) + 5)|0;
    $228 = $iword;
    $229 = Math_imul($227, $228)|0;
    $230 = $7;
    $231 = $230<<1;
    $232 = (($231) + 4)|0;
    $233 = (($232) + 3)|0;
    $234 = $4;
    $235 = Math_imul($233, $234)|0;
    $236 = $235<<2;
    $237 = $7;
    $238 = (($237) + 1)|0;
    $239 = $4;
    $240 = Math_imul($238, $239)|0;
    $241 = $240<<3;
    $242 = (($236) + ($241))|0;
    $243 = (($229) + ($242))|0;
    $244 = HEAP32[$nzlmax>>2]|0;
    $245 = HEAP32[$nzumax>>2]|0;
    $246 = (($244) + ($245))|0;
    $247 = $iword;
    $248 = Math_imul($246, $247)|0;
    $249 = (($243) + ($248))|0;
    $250 = HEAP32[$nzlumax>>2]|0;
    $251 = HEAP32[$nzumax>>2]|0;
    $252 = (($250) + ($251))|0;
    $253 = $dword;
    $254 = Math_imul($252, $253)|0;
    $255 = (($249) + ($254))|0;
    $256 = $5;
    $257 = (($255) + ($256))|0;
    $0 = $257;
    $381 = $0;
    STACKTOP = sp;return ($381|0);
   }
   $258 = $3;
   $259 = ($258|0)==(0);
   $260 = $11;
   $261 = ((($260)) + 52|0);
   if ($259) {
    HEAP32[$261>>2] = 0;
   } else {
    HEAP32[$261>>2] = 1;
    $262 = $3;
    $263 = (($262|0) / 4)&-1;
    $264 = $263<<2;
    $265 = $11;
    $266 = ((($265)) + 64|0);
    $267 = ((($266)) + 12|0);
    HEAP32[$267>>2] = $264;
    $268 = $11;
    $269 = ((($268)) + 64|0);
    $270 = ((($269)) + 12|0);
    $271 = HEAP32[$270>>2]|0;
    $272 = $11;
    $273 = ((($272)) + 64|0);
    HEAP32[$273>>2] = $271;
   }
   $274 = $Lstore;
   $275 = ((($274)) + 16|0);
   $276 = HEAP32[$275>>2]|0;
   $277 = $11;
   $278 = ((($277)) + 60|0);
   $279 = HEAP32[$278>>2]|0;
   $280 = ((($279)) + 16|0);
   $281 = ((($280)) + 4|0);
   HEAP32[$281>>2] = $276;
   $lsub = $276;
   $282 = $Lstore;
   $283 = ((($282)) + 8|0);
   $284 = HEAP32[$283>>2]|0;
   $285 = $11;
   $286 = ((($285)) + 60|0);
   $287 = HEAP32[$286>>2]|0;
   $288 = ((($287)) + 4|0);
   HEAP32[$288>>2] = $284;
   $lusup = $284;
   $289 = $Ustore;
   $290 = ((($289)) + 8|0);
   $291 = HEAP32[$290>>2]|0;
   $292 = $11;
   $293 = ((($292)) + 60|0);
   $294 = HEAP32[$293>>2]|0;
   $295 = ((($294)) + 24|0);
   $296 = ((($295)) + 4|0);
   HEAP32[$296>>2] = $291;
   $usub = $291;
   $297 = $Ustore;
   $298 = ((($297)) + 4|0);
   $299 = HEAP32[$298>>2]|0;
   $300 = $11;
   $301 = ((($300)) + 60|0);
   $302 = HEAP32[$301>>2]|0;
   $303 = ((($302)) + 8|0);
   $304 = ((($303)) + 4|0);
   HEAP32[$304>>2] = $299;
   $ucol = $299;
   $305 = HEAP32[$nzlmax>>2]|0;
   $306 = $11;
   $307 = ((($306)) + 60|0);
   $308 = HEAP32[$307>>2]|0;
   $309 = ((($308)) + 16|0);
   HEAP32[$309>>2] = $305;
   $310 = HEAP32[$nzlumax>>2]|0;
   $311 = $11;
   $312 = ((($311)) + 60|0);
   $313 = HEAP32[$312>>2]|0;
   HEAP32[$313>>2] = $310;
   $314 = HEAP32[$nzumax>>2]|0;
   $315 = $11;
   $316 = ((($315)) + 60|0);
   $317 = HEAP32[$316>>2]|0;
   $318 = ((($317)) + 24|0);
   HEAP32[$318>>2] = $314;
   $319 = HEAP32[$nzumax>>2]|0;
   $320 = $11;
   $321 = ((($320)) + 60|0);
   $322 = HEAP32[$321>>2]|0;
   $323 = ((($322)) + 8|0);
   HEAP32[$323>>2] = $319;
  }
 } while(0);
 $324 = $xsup;
 $325 = $11;
 HEAP32[$325>>2] = $324;
 $326 = $supno;
 $327 = $11;
 $328 = ((($327)) + 4|0);
 HEAP32[$328>>2] = $326;
 $329 = $lsub;
 $330 = $11;
 $331 = ((($330)) + 8|0);
 HEAP32[$331>>2] = $329;
 $332 = $xlsub;
 $333 = $11;
 $334 = ((($333)) + 12|0);
 HEAP32[$334>>2] = $332;
 $335 = $lusup;
 $336 = $11;
 $337 = ((($336)) + 16|0);
 HEAP32[$337>>2] = $335;
 $338 = $xlusup;
 $339 = $11;
 $340 = ((($339)) + 20|0);
 HEAP32[$340>>2] = $338;
 $341 = $ucol;
 $342 = $11;
 $343 = ((($342)) + 24|0);
 HEAP32[$343>>2] = $341;
 $344 = $usub;
 $345 = $11;
 $346 = ((($345)) + 28|0);
 HEAP32[$346>>2] = $344;
 $347 = $xusub;
 $348 = $11;
 $349 = ((($348)) + 32|0);
 HEAP32[$349>>2] = $347;
 $350 = HEAP32[$nzlmax>>2]|0;
 $351 = $11;
 $352 = ((($351)) + 36|0);
 HEAP32[$352>>2] = $350;
 $353 = HEAP32[$nzumax>>2]|0;
 $354 = $11;
 $355 = ((($354)) + 40|0);
 HEAP32[$355>>2] = $353;
 $356 = HEAP32[$nzlumax>>2]|0;
 $357 = $11;
 $358 = ((($357)) + 44|0);
 HEAP32[$358>>2] = $356;
 $359 = $4;
 $360 = $5;
 $361 = $7;
 $362 = $12;
 $363 = $13;
 $364 = $11;
 $365 = (_dLUWorkInit($359,$360,$361,$362,$363,$364)|0);
 $info = $365;
 $366 = $info;
 $367 = ($366|0)!=(0);
 if ($367) {
  $368 = $info;
  $369 = HEAP32[$nzlmax>>2]|0;
  $370 = HEAP32[$nzumax>>2]|0;
  $371 = HEAP32[$nzlumax>>2]|0;
  $372 = $5;
  $373 = (_dmemory_usage($369,$370,$371,$372)|0);
  $374 = (($368) + ($373))|0;
  $375 = $5;
  $376 = (($374) + ($375))|0;
  $0 = $376;
  $381 = $0;
  STACKTOP = sp;return ($381|0);
 } else {
  $377 = $11;
  $378 = ((($377)) + 56|0);
  $379 = HEAP32[$378>>2]|0;
  $380 = (($379) + 1)|0;
  HEAP32[$378>>2] = $380;
  $0 = 0;
  $381 = $0;
  STACKTOP = sp;return ($381|0);
 }
 return (0)|0;
}
function _dexpand($prev_len,$type,$len_to_copy,$keep_prev,$Glu) {
 $prev_len = $prev_len|0;
 $type = $type|0;
 $len_to_copy = $len_to_copy|0;
 $keep_prev = $keep_prev|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0.0, $161 = 0.0, $162 = 0.0, $163 = 0.0, $164 = 0, $165 = 0, $166 = 0.0, $167 = 0.0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0.0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0.0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0.0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0.0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $EXPAND = 0.0, $alpha = 0.0, $bytes_to_copy = 0, $expanders = 0;
 var $extra = 0, $lword = 0, $new_len = 0, $new_mem = 0, $old_mem = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $tries = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $prev_len;
 $2 = $type;
 $3 = $len_to_copy;
 $4 = $keep_prev;
 $5 = $Glu;
 $EXPAND = 1.5;
 $6 = $5;
 $7 = ((($6)) + 60|0);
 $8 = HEAP32[$7>>2]|0;
 $expanders = $8;
 $9 = $EXPAND;
 $alpha = $9;
 $10 = $5;
 $11 = ((($10)) + 56|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 $14 = $4;
 $15 = ($14|0)!=(0);
 $or$cond = $13 | $15;
 if ($or$cond) {
  $16 = $1;
  $17 = HEAP32[$16>>2]|0;
  $new_len = $17;
 } else {
  $18 = $alpha;
  $19 = $1;
  $20 = HEAP32[$19>>2]|0;
  $21 = (+($20|0));
  $22 = $18 * $21;
  $23 = (~~(($22)));
  $new_len = $23;
 }
 $24 = $2;
 $25 = ($24|0)==(2);
 $26 = $2;
 $27 = ($26|0)==(3);
 $or$cond3 = $25 | $27;
 if ($or$cond3) {
  $lword = 4;
 } else {
  $lword = 8;
 }
 $28 = $5;
 $29 = ((($28)) + 52|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $new_len;
   $33 = $lword;
   $34 = Math_imul($32, $33)|0;
   $35 = (_superlu_malloc($34)|0);
   $new_mem = $35;
   $36 = $5;
   $37 = ((($36)) + 56|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ($38|0)!=(0);
   if ($39) {
    $tries = 0;
    $40 = $4;
    $41 = ($40|0)!=(0);
    L13: do {
     if ($41) {
      $42 = $new_mem;
      $43 = ($42|0)!=(0|0);
      if (!($43)) {
       $0 = 0;
       $295 = $0;
       STACKTOP = sp;return ($295|0);
      }
     } else {
      while(1) {
       $44 = $new_mem;
       $45 = ($44|0)!=(0|0);
       $46 = $45 ^ 1;
       if (!($46)) {
        break L13;
       }
       $47 = $tries;
       $48 = (($47) + 1)|0;
       $tries = $48;
       $49 = ($48|0)>(10);
       if ($49) {
        break;
       }
       $50 = $alpha;
       $51 = $50 + 1.0;
       $52 = $51 / 2.0;
       $alpha = $52;
       $53 = $alpha;
       $54 = $1;
       $55 = HEAP32[$54>>2]|0;
       $56 = (+($55|0));
       $57 = $53 * $56;
       $58 = (~~(($57)));
       $new_len = $58;
       $59 = $new_len;
       $60 = $lword;
       $61 = Math_imul($59, $60)|0;
       $62 = (_superlu_malloc($61)|0);
       $new_mem = $62;
      }
      $0 = 0;
      $295 = $0;
      STACKTOP = sp;return ($295|0);
     }
    } while(0);
    $63 = $2;
    $64 = ($63|0)==(2);
    $65 = $2;
    $66 = ($65|0)==(3);
    $or$cond5 = $64 | $66;
    $67 = $3;
    $68 = $2;
    $69 = $expanders;
    $70 = (($69) + ($68<<3)|0);
    $71 = ((($70)) + 4|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = $new_mem;
    if ($or$cond5) {
     _copy_mem_int($67,$72,$73);
    } else {
     _copy_mem_double($67,$72,$73);
    }
    $74 = $2;
    $75 = $expanders;
    $76 = (($75) + ($74<<3)|0);
    $77 = ((($76)) + 4|0);
    $78 = HEAP32[$77>>2]|0;
    _superlu_free($78);
   }
   $79 = $new_mem;
   $80 = $2;
   $81 = $expanders;
   $82 = (($81) + ($80<<3)|0);
   $83 = ((($82)) + 4|0);
   HEAP32[$83>>2] = $79;
  } else {
   $84 = $5;
   $85 = ((($84)) + 56|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = ($86|0)==(0);
   if ($87) {
    $88 = $new_len;
    $89 = $lword;
    $90 = Math_imul($88, $89)|0;
    $91 = $5;
    $92 = (_duser_malloc($90,0,$91)|0);
    $new_mem = $92;
    $93 = $new_mem;
    $94 = $93;
    $95 = $94 & 7;
    $96 = ($95|0)!=(0);
    if ($96) {
     $97 = $2;
     $98 = ($97|0)==(0);
     $99 = $2;
     $100 = ($99|0)==(1);
     $or$cond7 = $98 | $100;
     if ($or$cond7) {
      $101 = $new_mem;
      $old_mem = $101;
      $102 = $new_mem;
      $103 = $102;
      $104 = (($103) + 7)|0;
      $105 = $104 & -8;
      $106 = $105;
      $new_mem = $106;
      $107 = $new_mem;
      $108 = $old_mem;
      $109 = $107;
      $110 = $108;
      $111 = (($109) - ($110))|0;
      $extra = $111;
      $112 = $extra;
      $113 = $5;
      $114 = ((($113)) + 64|0);
      $115 = ((($114)) + 8|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = (($116) + ($112))|0;
      HEAP32[$115>>2] = $117;
      $118 = $extra;
      $119 = $5;
      $120 = ((($119)) + 64|0);
      $121 = ((($120)) + 4|0);
      $122 = HEAP32[$121>>2]|0;
      $123 = (($122) + ($118))|0;
      HEAP32[$121>>2] = $123;
     }
    }
    $124 = $new_mem;
    $125 = $2;
    $126 = $expanders;
    $127 = (($126) + ($125<<3)|0);
    $128 = ((($127)) + 4|0);
    HEAP32[$128>>2] = $124;
    break;
   }
   $tries = 0;
   $129 = $new_len;
   $130 = $1;
   $131 = HEAP32[$130>>2]|0;
   $132 = (($129) - ($131))|0;
   $133 = $lword;
   $134 = Math_imul($132, $133)|0;
   $extra = $134;
   $135 = $4;
   $136 = ($135|0)!=(0);
   L37: do {
    if ($136) {
     $137 = $extra;
     $138 = $5;
     $139 = ((($138)) + 64|0);
     $140 = ((($139)) + 4|0);
     $141 = HEAP32[$140>>2]|0;
     $142 = (($137) + ($141))|0;
     $143 = $5;
     $144 = ((($143)) + 64|0);
     $145 = HEAP32[$144>>2]|0;
     $146 = ($142|0)>=($145|0);
     if ($146) {
      $0 = 0;
      $295 = $0;
      STACKTOP = sp;return ($295|0);
     }
    } else {
     while(1) {
      $147 = $extra;
      $148 = $5;
      $149 = ((($148)) + 64|0);
      $150 = ((($149)) + 4|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = (($147) + ($151))|0;
      $153 = $5;
      $154 = ((($153)) + 64|0);
      $155 = HEAP32[$154>>2]|0;
      $156 = ($152|0)>=($155|0);
      if (!($156)) {
       break L37;
      }
      $157 = $tries;
      $158 = (($157) + 1)|0;
      $tries = $158;
      $159 = ($158|0)>(10);
      if ($159) {
       break;
      }
      $160 = $alpha;
      $161 = $160 + 1.0;
      $162 = $161 / 2.0;
      $alpha = $162;
      $163 = $alpha;
      $164 = $1;
      $165 = HEAP32[$164>>2]|0;
      $166 = (+($165|0));
      $167 = $163 * $166;
      $168 = (~~(($167)));
      $new_len = $168;
      $169 = $new_len;
      $170 = $1;
      $171 = HEAP32[$170>>2]|0;
      $172 = (($169) - ($171))|0;
      $173 = $lword;
      $174 = Math_imul($172, $173)|0;
      $extra = $174;
     }
     $0 = 0;
     $295 = $0;
     STACKTOP = sp;return ($295|0);
    }
   } while(0);
   $175 = $2;
   $176 = ($175|0)!=(3);
   if ($176) {
    $177 = $2;
    $178 = (($177) + 1)|0;
    $179 = $expanders;
    $180 = (($179) + ($178<<3)|0);
    $181 = ((($180)) + 4|0);
    $182 = HEAP32[$181>>2]|0;
    $183 = $extra;
    $184 = (($182) + ($183)|0);
    $new_mem = $184;
    $185 = $5;
    $186 = ((($185)) + 64|0);
    $187 = ((($186)) + 16|0);
    $188 = HEAP32[$187>>2]|0;
    $189 = $5;
    $190 = ((($189)) + 64|0);
    $191 = ((($190)) + 8|0);
    $192 = HEAP32[$191>>2]|0;
    $193 = (($188) + ($192)|0);
    $194 = $2;
    $195 = (($194) + 1)|0;
    $196 = $expanders;
    $197 = (($196) + ($195<<3)|0);
    $198 = ((($197)) + 4|0);
    $199 = HEAP32[$198>>2]|0;
    $200 = $193;
    $201 = $199;
    $202 = (($200) - ($201))|0;
    $bytes_to_copy = $202;
    $203 = $2;
    $204 = (($203) + 1)|0;
    $205 = $expanders;
    $206 = (($205) + ($204<<3)|0);
    $207 = ((($206)) + 4|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = $new_mem;
    $210 = $bytes_to_copy;
    _user_bcopy($208,$209,$210);
    $211 = $2;
    $212 = ($211>>>0)<(3);
    if ($212) {
     $213 = $expanders;
     $214 = ((($213)) + 24|0);
     $215 = ((($214)) + 4|0);
     $216 = HEAP32[$215>>2]|0;
     $217 = $extra;
     $218 = (($216) + ($217)|0);
     $219 = $expanders;
     $220 = ((($219)) + 24|0);
     $221 = ((($220)) + 4|0);
     HEAP32[$221>>2] = $218;
     $222 = $5;
     $223 = ((($222)) + 28|0);
     HEAP32[$223>>2] = $218;
    }
    $224 = $2;
    $225 = ($224>>>0)<(2);
    if ($225) {
     $226 = $expanders;
     $227 = ((($226)) + 16|0);
     $228 = ((($227)) + 4|0);
     $229 = HEAP32[$228>>2]|0;
     $230 = $extra;
     $231 = (($229) + ($230)|0);
     $232 = $expanders;
     $233 = ((($232)) + 16|0);
     $234 = ((($233)) + 4|0);
     HEAP32[$234>>2] = $231;
     $235 = $5;
     $236 = ((($235)) + 8|0);
     HEAP32[$236>>2] = $231;
    }
    $237 = $2;
    $238 = ($237>>>0)<(1);
    if ($238) {
     $239 = $expanders;
     $240 = ((($239)) + 8|0);
     $241 = ((($240)) + 4|0);
     $242 = HEAP32[$241>>2]|0;
     $243 = $extra;
     $244 = (($242) + ($243)|0);
     $245 = $expanders;
     $246 = ((($245)) + 8|0);
     $247 = ((($246)) + 4|0);
     HEAP32[$247>>2] = $244;
     $248 = $5;
     $249 = ((($248)) + 24|0);
     HEAP32[$249>>2] = $244;
    }
    $250 = $extra;
    $251 = $5;
    $252 = ((($251)) + 64|0);
    $253 = ((($252)) + 8|0);
    $254 = HEAP32[$253>>2]|0;
    $255 = (($254) + ($250))|0;
    HEAP32[$253>>2] = $255;
    $256 = $extra;
    $257 = $5;
    $258 = ((($257)) + 64|0);
    $259 = ((($258)) + 4|0);
    $260 = HEAP32[$259>>2]|0;
    $261 = (($260) + ($256))|0;
    HEAP32[$259>>2] = $261;
    $262 = $2;
    $263 = ($262|0)==(1);
    if ($263) {
     $264 = $extra;
     $265 = $5;
     $266 = ((($265)) + 64|0);
     $267 = ((($266)) + 8|0);
     $268 = HEAP32[$267>>2]|0;
     $269 = (($268) + ($264))|0;
     HEAP32[$267>>2] = $269;
     $270 = $extra;
     $271 = $5;
     $272 = ((($271)) + 64|0);
     $273 = ((($272)) + 4|0);
     $274 = HEAP32[$273>>2]|0;
     $275 = (($274) + ($270))|0;
     HEAP32[$273>>2] = $275;
    }
   }
  }
 } while(0);
 $276 = $new_len;
 $277 = $2;
 $278 = $expanders;
 $279 = (($278) + ($277<<3)|0);
 HEAP32[$279>>2] = $276;
 $280 = $new_len;
 $281 = $1;
 HEAP32[$281>>2] = $280;
 $282 = $5;
 $283 = ((($282)) + 56|0);
 $284 = HEAP32[$283>>2]|0;
 $285 = ($284|0)!=(0);
 if ($285) {
  $286 = $5;
  $287 = ((($286)) + 56|0);
  $288 = HEAP32[$287>>2]|0;
  $289 = (($288) + 1)|0;
  HEAP32[$287>>2] = $289;
 }
 $290 = $2;
 $291 = $expanders;
 $292 = (($291) + ($290<<3)|0);
 $293 = ((($292)) + 4|0);
 $294 = HEAP32[$293>>2]|0;
 $0 = $294;
 $295 = $0;
 STACKTOP = sp;return ($295|0);
}
function _dmemory_usage($nzlmax,$nzumax,$nzlumax,$n) {
 $nzlmax = $nzlmax|0;
 $nzumax = $nzumax|0;
 $nzlumax = $nzlumax|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $dword = 0, $iword = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $nzlmax;
 $1 = $nzumax;
 $2 = $nzlumax;
 $3 = $n;
 $iword = 4;
 $dword = 8;
 $4 = $3;
 $5 = ($4*10)|0;
 $6 = $iword;
 $7 = Math_imul($5, $6)|0;
 $8 = $0;
 $9 = $iword;
 $10 = Math_imul($8, $9)|0;
 $11 = (($7) + ($10))|0;
 $12 = $1;
 $13 = $iword;
 $14 = $dword;
 $15 = (($13) + ($14))|0;
 $16 = Math_imul($12, $15)|0;
 $17 = (($11) + ($16))|0;
 $18 = $2;
 $19 = $dword;
 $20 = Math_imul($18, $19)|0;
 $21 = (($17) + ($20))|0;
 STACKTOP = sp;return ($21|0);
}
function _dLUWorkInit($m,$n,$panel_size,$iworkptr,$dworkptr,$Glu) {
 $m = $m|0;
 $n = $n|0;
 $panel_size = $panel_size|0;
 $iworkptr = $iworkptr|0;
 $dworkptr = $dworkptr|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dsize = 0, $extra = 0, $isize = 0, $maxsuper = 0;
 var $old_ptr = 0, $rowblk = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $m;
 $2 = $n;
 $3 = $panel_size;
 $4 = $iworkptr;
 $5 = $dworkptr;
 $6 = $Glu;
 $7 = (_sp_ienv(3)|0);
 $8 = (_sp_ienv(7)|0);
 $9 = ($7|0)>($8|0);
 if ($9) {
  $10 = (_sp_ienv(3)|0);
  $12 = $10;
 } else {
  $11 = (_sp_ienv(7)|0);
  $12 = $11;
 }
 $maxsuper = $12;
 $13 = (_sp_ienv(4)|0);
 $rowblk = $13;
 $14 = $3;
 $15 = $14<<1;
 $16 = (($15) + 3)|0;
 $17 = (($16) + 3)|0;
 $18 = $1;
 $19 = Math_imul($17, $18)|0;
 $20 = $2;
 $21 = (($19) + ($20))|0;
 $22 = $21<<2;
 $isize = $22;
 $23 = $1;
 $24 = $3;
 $25 = Math_imul($23, $24)|0;
 $26 = $1;
 $27 = $maxsuper;
 $28 = $rowblk;
 $29 = (($27) + ($28))|0;
 $30 = $3;
 $31 = Math_imul($29, $30)|0;
 $32 = ($26|0)>($31|0);
 if ($32) {
  $33 = $1;
  $40 = $33;
 } else {
  $34 = $maxsuper;
  $35 = $rowblk;
  $36 = (($34) + ($35))|0;
  $37 = $3;
  $38 = Math_imul($36, $37)|0;
  $40 = $38;
 }
 $39 = (($25) + ($40))|0;
 $41 = $39<<3;
 $dsize = $41;
 $42 = $6;
 $43 = ((($42)) + 52|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)==(0);
 $46 = $isize;
 if ($45) {
  $47 = (($46>>>0) / 4)&-1;
  $48 = (_intCalloc($47)|0);
  $49 = $4;
  HEAP32[$49>>2] = $48;
 } else {
  $50 = $6;
  $51 = (_duser_malloc($46,1,$50)|0);
  $52 = $4;
  HEAP32[$52>>2] = $51;
 }
 $53 = $4;
 $54 = HEAP32[$53>>2]|0;
 $55 = ($54|0)!=(0|0);
 if (!($55)) {
  $56 = HEAP32[_stderr>>2]|0;
  (_fprintf(($56|0),(1872|0),($vararg_buffer|0))|0);
  $57 = $isize;
  $58 = $2;
  $59 = (($57) + ($58))|0;
  $0 = $59;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 $60 = $6;
 $61 = ((($60)) + 52|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = ($62|0)==(0);
 $64 = $dsize;
 if ($63) {
  $65 = (_superlu_malloc($64)|0);
  $66 = $5;
  HEAP32[$66>>2] = $65;
 } else {
  $67 = $6;
  $68 = (_duser_malloc($64,1,$67)|0);
  $69 = $5;
  HEAP32[$69>>2] = $68;
  $70 = $5;
  $71 = HEAP32[$70>>2]|0;
  $72 = $71;
  $73 = $72 & 7;
  $74 = ($73|0)!=(0);
  if ($74) {
   $75 = $5;
   $76 = HEAP32[$75>>2]|0;
   $old_ptr = $76;
   $77 = $5;
   $78 = HEAP32[$77>>2]|0;
   $79 = $78;
   $80 = (($79) + 7)|0;
   $81 = $80 & -8;
   $82 = $81;
   $83 = $5;
   HEAP32[$83>>2] = $82;
   $84 = $5;
   $85 = HEAP32[$84>>2]|0;
   $86 = ((($85)) + -8|0);
   $87 = $5;
   HEAP32[$87>>2] = $86;
   $88 = $old_ptr;
   $89 = $5;
   $90 = HEAP32[$89>>2]|0;
   $91 = $88;
   $92 = $90;
   $93 = (($91) - ($92))|0;
   $extra = $93;
   $94 = $extra;
   $95 = $6;
   $96 = ((($95)) + 64|0);
   $97 = ((($96)) + 12|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = (($98) - ($94))|0;
   HEAP32[$97>>2] = $99;
   $100 = $extra;
   $101 = $6;
   $102 = ((($101)) + 64|0);
   $103 = ((($102)) + 4|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = (($104) + ($100))|0;
   HEAP32[$103>>2] = $105;
  }
 }
 $106 = $5;
 $107 = HEAP32[$106>>2]|0;
 $108 = ($107|0)!=(0|0);
 if ($108) {
  $0 = 0;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 } else {
  $109 = HEAP32[_stderr>>2]|0;
  (_fprintf(($109|0),(2672|0),($vararg_buffer1|0))|0);
  $110 = $isize;
  $111 = $dsize;
  $112 = (($110) + ($111))|0;
  $113 = $2;
  $114 = (($112) + ($113))|0;
  $0 = $114;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 return (0)|0;
}
function _dSetRWork($m,$panel_size,$dworkptr,$dense,$tempv) {
 $m = $m|0;
 $panel_size = $panel_size|0;
 $dworkptr = $dworkptr|0;
 $dense = $dense|0;
 $tempv = $tempv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $maxsuper = 0, $rowblk = 0, $zero = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $m;
 $1 = $panel_size;
 $2 = $dworkptr;
 $3 = $dense;
 $4 = $tempv;
 $zero = 0.0;
 $5 = (_sp_ienv(3)|0);
 $6 = (_sp_ienv(7)|0);
 $7 = ($5|0)>($6|0);
 if ($7) {
  $8 = (_sp_ienv(3)|0);
  $10 = $8;
 } else {
  $9 = (_sp_ienv(7)|0);
  $10 = $9;
 }
 $maxsuper = $10;
 $11 = (_sp_ienv(4)|0);
 $rowblk = $11;
 $12 = $2;
 $13 = $3;
 HEAP32[$13>>2] = $12;
 $14 = $3;
 $15 = HEAP32[$14>>2]|0;
 $16 = $1;
 $17 = $0;
 $18 = Math_imul($16, $17)|0;
 $19 = (($15) + ($18<<3)|0);
 $20 = $4;
 HEAP32[$20>>2] = $19;
 $21 = $3;
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = $1;
 $25 = Math_imul($23, $24)|0;
 $26 = $zero;
 _dfill($22,$25,$26);
 $27 = $4;
 $28 = HEAP32[$27>>2]|0;
 $29 = $0;
 $30 = $maxsuper;
 $31 = $rowblk;
 $32 = (($30) + ($31))|0;
 $33 = $1;
 $34 = Math_imul($32, $33)|0;
 $35 = ($29|0)>($34|0);
 if ($35) {
  $36 = $0;
  $43 = $36;
  $42 = $zero;
  _dfill($28,$43,$42);
  STACKTOP = sp;return;
 } else {
  $37 = $maxsuper;
  $38 = $rowblk;
  $39 = (($37) + ($38))|0;
  $40 = $1;
  $41 = Math_imul($39, $40)|0;
  $43 = $41;
  $42 = $zero;
  _dfill($28,$43,$42);
  STACKTOP = sp;return;
 }
}
function _dLUWorkFree($iwork,$dwork,$Glu) {
 $iwork = $iwork|0;
 $dwork = $dwork|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $iwork;
 $1 = $dwork;
 $2 = $Glu;
 $3 = $2;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = $0;
  _superlu_free($7);
  $8 = $1;
  _superlu_free($8);
 } else {
  $9 = $2;
  $10 = ((($9)) + 64|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $2;
  $13 = ((($12)) + 64|0);
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($11) - ($15))|0;
  $17 = $2;
  $18 = ((($17)) + 64|0);
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) - ($16))|0;
  HEAP32[$19>>2] = $21;
  $22 = $2;
  $23 = ((($22)) + 64|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $2;
  $26 = ((($25)) + 64|0);
  $27 = ((($26)) + 12|0);
  HEAP32[$27>>2] = $24;
 }
 $28 = $2;
 $29 = ((($28)) + 60|0);
 $30 = HEAP32[$29>>2]|0;
 _superlu_free($30);
 $31 = $2;
 $32 = ((($31)) + 60|0);
 HEAP32[$32>>2] = 0;
 STACKTOP = sp;return;
}
function _dLUMemXpand($jcol,$next,$mem_type,$maxlen,$Glu) {
 $jcol = $jcol|0;
 $next = $next|0;
 $mem_type = $mem_type|0;
 $maxlen = $maxlen|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $new_mem = 0, $nzlmax = 0, $nzlumax = 0, $nzumax = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $1 = $jcol;
 $2 = $next;
 $3 = $mem_type;
 $4 = $maxlen;
 $5 = $Glu;
 $6 = $3;
 $7 = ($6|0)==(3);
 $8 = $4;
 $9 = $3;
 $10 = $2;
 $11 = $5;
 if ($7) {
  $12 = (_dexpand($8,$9,$10,1,$11)|0);
  $new_mem = $12;
 } else {
  $13 = (_dexpand($8,$9,$10,0,$11)|0);
  $new_mem = $13;
 }
 $14 = $new_mem;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $16 = $5;
  $17 = ((($16)) + 36|0);
  $18 = HEAP32[$17>>2]|0;
  $nzlmax = $18;
  $19 = $5;
  $20 = ((($19)) + 40|0);
  $21 = HEAP32[$20>>2]|0;
  $nzumax = $21;
  $22 = $5;
  $23 = ((($22)) + 44|0);
  $24 = HEAP32[$23>>2]|0;
  $nzlumax = $24;
  $25 = HEAP32[_stderr>>2]|0;
  $26 = $3;
  $27 = $1;
  HEAP32[$vararg_buffer>>2] = $26;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $27;
  (_fprintf(($25|0),(2712|0),($vararg_buffer|0))|0);
  $28 = $nzlmax;
  $29 = $nzumax;
  $30 = $nzlumax;
  $31 = $5;
  $32 = ((($31)) + 48|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (_dmemory_usage($28,$29,$30,$33)|0);
  $35 = $5;
  $36 = ((($35)) + 48|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($34) + ($37))|0;
  $0 = $38;
  $68 = $0;
  STACKTOP = sp;return ($68|0);
 }
 $39 = $3;
 if ((($39|0) == 3)) {
  $61 = $new_mem;
  $62 = $5;
  $63 = ((($62)) + 28|0);
  HEAP32[$63>>2] = $61;
  $64 = $4;
  $65 = HEAP32[$64>>2]|0;
  $66 = $5;
  $67 = ((($66)) + 40|0);
  HEAP32[$67>>2] = $65;
 } else if ((($39|0) == 0)) {
  $40 = $new_mem;
  $41 = $5;
  $42 = ((($41)) + 16|0);
  HEAP32[$42>>2] = $40;
  $43 = $4;
  $44 = HEAP32[$43>>2]|0;
  $45 = $5;
  $46 = ((($45)) + 44|0);
  HEAP32[$46>>2] = $44;
 } else if ((($39|0) == 1)) {
  $47 = $new_mem;
  $48 = $5;
  $49 = ((($48)) + 24|0);
  HEAP32[$49>>2] = $47;
  $50 = $4;
  $51 = HEAP32[$50>>2]|0;
  $52 = $5;
  $53 = ((($52)) + 40|0);
  HEAP32[$53>>2] = $51;
 } else if ((($39|0) == 2)) {
  $54 = $new_mem;
  $55 = $5;
  $56 = ((($55)) + 8|0);
  HEAP32[$56>>2] = $54;
  $57 = $4;
  $58 = HEAP32[$57>>2]|0;
  $59 = $5;
  $60 = ((($59)) + 36|0);
  HEAP32[$60>>2] = $58;
 }
 $0 = 0;
 $68 = $0;
 STACKTOP = sp;return ($68|0);
}
function _copy_mem_double($howmany,$old,$new) {
 $howmany = $howmany|0;
 $old = $old|0;
 $new = $new|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dnew = 0, $dold = 0, $i = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $howmany;
 $1 = $old;
 $2 = $new;
 $3 = $1;
 $dold = $3;
 $4 = $2;
 $dnew = $4;
 $i = 0;
 while(1) {
  $5 = $i;
  $6 = $0;
  $7 = ($5|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = $i;
  $9 = $dold;
  $10 = (($9) + ($8<<3)|0);
  $11 = +HEAPF64[$10>>3];
  $12 = $i;
  $13 = $dnew;
  $14 = (($13) + ($12<<3)|0);
  HEAPF64[$14>>3] = $11;
  $15 = $i;
  $16 = (($15) + 1)|0;
  $i = $16;
 }
 STACKTOP = sp;return;
}
function _doubleMalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 24|0;
 $0 = $n;
 $1 = $0;
 $2 = $1<<3;
 $3 = (_superlu_malloc($2)|0);
 $buf = $3;
 $4 = $buf;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = $buf;
  STACKTOP = sp;return ($6|0);
 }
 HEAP32[$vararg_buffer>>2] = 1920;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 671;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = 1856;
 (_sprintf($msg,2864,$vararg_buffer)|0);
 _superlu_abort_and_exit($msg);
 $6 = $buf;
 STACKTOP = sp;return ($6|0);
}
function _doubleCalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0.0, $buf = 0, $i = 0, $msg = 0, $vararg_buffer = 0;
 var $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zero = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp + 8|0;
 $msg = sp + 32|0;
 $0 = $n;
 $zero = 0.0;
 $1 = $0;
 $2 = $1<<3;
 $3 = (_superlu_malloc($2)|0);
 $buf = $3;
 $4 = $buf;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  HEAP32[$vararg_buffer>>2] = 1976;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 683;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 1856;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $i = 0;
 while(1) {
  $6 = $i;
  $7 = $0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $zero;
  $10 = $i;
  $11 = $buf;
  $12 = (($11) + ($10<<3)|0);
  HEAPF64[$12>>3] = $9;
  $13 = $i;
  $14 = (($13) + 1)|0;
  $i = $14;
 }
 $15 = $buf;
 STACKTOP = sp;return ($15|0);
}
function _dCreate_CompCol_Matrix($A,$m,$n,$nnz,$nzval,$rowind,$colptr,$stype,$dtype,$mtype) {
 $A = $A|0;
 $m = $m|0;
 $n = $n|0;
 $nnz = $nnz|0;
 $nzval = $nzval|0;
 $rowind = $rowind|0;
 $colptr = $colptr|0;
 $stype = $stype|0;
 $dtype = $dtype|0;
 $mtype = $mtype|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $Astore = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 320|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 56|0;
 $0 = $A;
 $1 = $m;
 $2 = $n;
 $3 = $nnz;
 $4 = $nzval;
 $5 = $rowind;
 $6 = $colptr;
 $7 = $stype;
 $8 = $dtype;
 $9 = $mtype;
 $10 = $7;
 $11 = $0;
 HEAP32[$11>>2] = $10;
 $12 = $8;
 $13 = $0;
 $14 = ((($13)) + 4|0);
 HEAP32[$14>>2] = $12;
 $15 = $9;
 $16 = $0;
 $17 = ((($16)) + 8|0);
 HEAP32[$17>>2] = $15;
 $18 = $1;
 $19 = $0;
 $20 = ((($19)) + 12|0);
 HEAP32[$20>>2] = $18;
 $21 = $2;
 $22 = $0;
 $23 = ((($22)) + 16|0);
 HEAP32[$23>>2] = $21;
 $24 = (_superlu_malloc(16)|0);
 $25 = $0;
 $26 = ((($25)) + 20|0);
 HEAP32[$26>>2] = $24;
 $27 = $0;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 if (!($30)) {
  HEAP32[$vararg_buffer>>2] = 2896;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 41;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2032;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $31 = $0;
 $32 = ((($31)) + 20|0);
 $33 = HEAP32[$32>>2]|0;
 $Astore = $33;
 $34 = $3;
 $35 = $Astore;
 HEAP32[$35>>2] = $34;
 $36 = $4;
 $37 = $Astore;
 $38 = ((($37)) + 4|0);
 HEAP32[$38>>2] = $36;
 $39 = $5;
 $40 = $Astore;
 $41 = ((($40)) + 8|0);
 HEAP32[$41>>2] = $39;
 $42 = $6;
 $43 = $Astore;
 $44 = ((($43)) + 12|0);
 HEAP32[$44>>2] = $42;
 STACKTOP = sp;return;
}
function _dCreate_Dense_Matrix($X,$m,$n,$x,$ldx,$stype,$dtype,$mtype) {
 $X = $X|0;
 $m = $m|0;
 $n = $n|0;
 $x = $x|0;
 $ldx = $ldx|0;
 $stype = $stype|0;
 $dtype = $dtype|0;
 $mtype = $mtype|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $Xstore = 0, $msg = 0, $vararg_buffer = 0;
 var $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 48|0;
 $0 = $X;
 $1 = $m;
 $2 = $n;
 $3 = $x;
 $4 = $ldx;
 $5 = $stype;
 $6 = $dtype;
 $7 = $mtype;
 $8 = $5;
 $9 = $0;
 HEAP32[$9>>2] = $8;
 $10 = $6;
 $11 = $0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = $7;
 $14 = $0;
 $15 = ((($14)) + 8|0);
 HEAP32[$15>>2] = $13;
 $16 = $1;
 $17 = $0;
 $18 = ((($17)) + 12|0);
 HEAP32[$18>>2] = $16;
 $19 = $2;
 $20 = $0;
 $21 = ((($20)) + 16|0);
 HEAP32[$21>>2] = $19;
 $22 = (_superlu_malloc(8)|0);
 $23 = $0;
 $24 = ((($23)) + 20|0);
 HEAP32[$24>>2] = $22;
 $25 = $0;
 $26 = ((($25)) + 20|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  HEAP32[$vararg_buffer>>2] = 2944;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 104;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2032;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $29 = $0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 $Xstore = $31;
 $32 = $4;
 $33 = $Xstore;
 HEAP32[$33>>2] = $32;
 $34 = $3;
 $35 = $Xstore;
 $36 = ((($35)) + 4|0);
 HEAP32[$36>>2] = $34;
 STACKTOP = sp;return;
}
function _dCreate_SuperNode_Matrix($L,$m,$n,$nnz,$nzval,$nzval_colptr,$rowind,$rowind_colptr,$col_to_sup,$sup_to_col,$stype,$dtype,$mtype) {
 $L = $L|0;
 $m = $m|0;
 $n = $n|0;
 $nnz = $nnz|0;
 $nzval = $nzval|0;
 $nzval_colptr = $nzval_colptr|0;
 $rowind = $rowind|0;
 $rowind_colptr = $rowind_colptr|0;
 $col_to_sup = $col_to_sup|0;
 $sup_to_col = $sup_to_col|0;
 $stype = $stype|0;
 $dtype = $dtype|0;
 $mtype = $mtype|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $Lstore = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 72|0;
 $0 = $L;
 $1 = $m;
 $2 = $n;
 $3 = $nnz;
 $4 = $nzval;
 $5 = $nzval_colptr;
 $6 = $rowind;
 $7 = $rowind_colptr;
 $8 = $col_to_sup;
 $9 = $sup_to_col;
 $10 = $stype;
 $11 = $dtype;
 $12 = $mtype;
 $13 = $10;
 $14 = $0;
 HEAP32[$14>>2] = $13;
 $15 = $11;
 $16 = $0;
 $17 = ((($16)) + 4|0);
 HEAP32[$17>>2] = $15;
 $18 = $12;
 $19 = $0;
 $20 = ((($19)) + 8|0);
 HEAP32[$20>>2] = $18;
 $21 = $1;
 $22 = $0;
 $23 = ((($22)) + 12|0);
 HEAP32[$23>>2] = $21;
 $24 = $2;
 $25 = $0;
 $26 = ((($25)) + 16|0);
 HEAP32[$26>>2] = $24;
 $27 = (_superlu_malloc(32)|0);
 $28 = $0;
 $29 = ((($28)) + 20|0);
 HEAP32[$29>>2] = $27;
 $30 = $0;
 $31 = ((($30)) + 20|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)!=(0|0);
 if (!($33)) {
  HEAP32[$vararg_buffer>>2] = 2984;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 137;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2032;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $34 = $0;
 $35 = ((($34)) + 20|0);
 $36 = HEAP32[$35>>2]|0;
 $Lstore = $36;
 $37 = $3;
 $38 = $Lstore;
 HEAP32[$38>>2] = $37;
 $39 = $2;
 $40 = $8;
 $41 = (($40) + ($39<<2)|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $Lstore;
 $44 = ((($43)) + 4|0);
 HEAP32[$44>>2] = $42;
 $45 = $4;
 $46 = $Lstore;
 $47 = ((($46)) + 8|0);
 HEAP32[$47>>2] = $45;
 $48 = $5;
 $49 = $Lstore;
 $50 = ((($49)) + 12|0);
 HEAP32[$50>>2] = $48;
 $51 = $6;
 $52 = $Lstore;
 $53 = ((($52)) + 16|0);
 HEAP32[$53>>2] = $51;
 $54 = $7;
 $55 = $Lstore;
 $56 = ((($55)) + 20|0);
 HEAP32[$56>>2] = $54;
 $57 = $8;
 $58 = $Lstore;
 $59 = ((($58)) + 24|0);
 HEAP32[$59>>2] = $57;
 $60 = $9;
 $61 = $Lstore;
 $62 = ((($61)) + 28|0);
 HEAP32[$62>>2] = $60;
 STACKTOP = sp;return;
}
function _dfill($a,$alen,$dval) {
 $a = $a|0;
 $alen = $alen|0;
 $dval = +$dval;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0.0, $3 = 0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $a;
 $1 = $alen;
 $2 = $dval;
 $i = 0;
 while(1) {
  $3 = $i;
  $4 = $1;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $2;
  $7 = $i;
  $8 = $0;
  $9 = (($8) + ($7<<3)|0);
  HEAPF64[$9>>3] = $6;
  $10 = $i;
  $11 = (($10) + 1)|0;
  $i = $11;
 }
 STACKTOP = sp;return;
}
function _dlsolve($ldm,$ncol,$M,$rhs) {
 $ldm = $ldm|0;
 $ncol = $ncol|0;
 $M = $M|0;
 $rhs = $rhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0.0, $104 = 0.0, $105 = 0.0, $106 = 0, $107 = 0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0.0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0.0, $129 = 0, $13 = 0, $130 = 0, $131 = 0.0, $132 = 0.0, $133 = 0.0;
 var $134 = 0.0, $135 = 0, $136 = 0, $137 = 0.0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0, $142 = 0, $143 = 0.0, $144 = 0.0, $145 = 0.0, $146 = 0.0, $147 = 0, $148 = 0, $149 = 0.0, $15 = 0, $150 = 0.0, $151 = 0.0;
 var $152 = 0.0, $153 = 0, $154 = 0, $155 = 0.0, $156 = 0.0, $157 = 0.0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0.0, $163 = 0.0, $164 = 0, $165 = 0, $166 = 0.0, $167 = 0.0, $168 = 0.0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0.0, $173 = 0.0, $174 = 0.0, $175 = 0.0, $176 = 0, $177 = 0, $178 = 0.0, $179 = 0.0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0, $183 = 0, $184 = 0.0, $185 = 0.0, $186 = 0.0, $187 = 0.0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0, $195 = 0, $196 = 0.0, $197 = 0.0, $198 = 0.0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0.0, $204 = 0.0, $205 = 0;
 var $206 = 0, $207 = 0.0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0.0, $211 = 0, $212 = 0, $213 = 0.0, $214 = 0.0, $215 = 0.0, $216 = 0.0, $217 = 0, $218 = 0, $219 = 0.0, $22 = 0, $220 = 0.0, $221 = 0.0, $222 = 0.0, $223 = 0;
 var $224 = 0, $225 = 0.0, $226 = 0.0, $227 = 0.0, $228 = 0.0, $229 = 0, $23 = 0, $230 = 0, $231 = 0.0, $232 = 0.0, $233 = 0.0, $234 = 0.0, $235 = 0, $236 = 0, $237 = 0.0, $238 = 0.0, $239 = 0.0, $24 = 0, $240 = 0.0, $241 = 0;
 var $242 = 0, $243 = 0.0, $244 = 0.0, $245 = 0.0, $246 = 0.0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0.0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0.0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0.0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0.0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0.0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0.0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0.0, $291 = 0.0, $292 = 0, $293 = 0, $294 = 0.0, $295 = 0.0, $296 = 0.0;
 var $297 = 0.0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0.0, $302 = 0.0, $303 = 0.0, $304 = 0, $305 = 0, $306 = 0.0, $307 = 0.0, $308 = 0.0, $309 = 0.0, $31 = 0, $310 = 0, $311 = 0, $312 = 0.0, $313 = 0.0;
 var $314 = 0.0, $315 = 0.0, $316 = 0, $317 = 0, $318 = 0.0, $319 = 0.0, $32 = 0, $320 = 0.0, $321 = 0.0, $322 = 0, $323 = 0, $324 = 0.0, $325 = 0.0, $326 = 0.0, $327 = 0.0, $328 = 0, $329 = 0, $33 = 0, $330 = 0.0, $331 = 0.0;
 var $332 = 0.0, $333 = 0.0, $334 = 0, $335 = 0, $336 = 0.0, $337 = 0.0, $338 = 0.0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0.0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0.0, $376 = 0.0, $377 = 0, $378 = 0, $379 = 0.0, $38 = 0, $380 = 0.0, $381 = 0.0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0.0;
 var $387 = 0.0, $388 = 0, $389 = 0, $39 = 0, $390 = 0.0, $391 = 0.0, $392 = 0.0, $393 = 0.0, $394 = 0, $395 = 0, $396 = 0.0, $397 = 0.0, $398 = 0.0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0.0;
 var $404 = 0.0, $405 = 0, $406 = 0, $407 = 0.0, $408 = 0.0, $409 = 0.0, $41 = 0, $410 = 0.0, $411 = 0, $412 = 0, $413 = 0.0, $414 = 0.0, $415 = 0.0, $416 = 0.0, $417 = 0, $418 = 0, $419 = 0.0, $42 = 0.0, $420 = 0.0, $421 = 0.0;
 var $422 = 0.0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0.0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0.0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0.0, $447 = 0.0, $448 = 0, $449 = 0, $45 = 0, $450 = 0.0, $451 = 0.0, $452 = 0.0, $453 = 0.0, $454 = 0, $455 = 0, $456 = 0.0, $457 = 0.0, $458 = 0.0;
 var $459 = 0.0, $46 = 0, $460 = 0, $461 = 0, $462 = 0.0, $463 = 0.0, $464 = 0.0, $465 = 0.0, $466 = 0, $467 = 0, $468 = 0.0, $469 = 0.0, $47 = 0.0, $470 = 0.0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0.0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0.0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0.0, $5 = 0, $50 = 0, $500 = 0.0, $501 = 0, $502 = 0, $503 = 0.0, $504 = 0.0, $505 = 0.0, $506 = 0.0, $507 = 0, $508 = 0, $509 = 0, $51 = 0.0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0.0, $520 = 0.0, $521 = 0.0, $522 = 0, $523 = 0, $524 = 0.0, $525 = 0.0, $526 = 0.0, $527 = 0.0, $528 = 0, $529 = 0, $53 = 0.0;
 var $530 = 0.0, $531 = 0.0, $532 = 0.0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0, $62 = 0.0, $63 = 0.0, $64 = 0.0;
 var $65 = 0.0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0, $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0.0;
 var $83 = 0, $84 = 0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0.0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0.0, $93 = 0.0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0.0, $99 = 0.0, $M0 = 0, $Mki0 = 0;
 var $Mki1 = 0, $Mki2 = 0, $Mki3 = 0, $Mki4 = 0, $Mki5 = 0, $Mki6 = 0, $Mki7 = 0, $firstcol = 0, $k = 0, $x0 = 0.0, $x1 = 0.0, $x2 = 0.0, $x3 = 0.0, $x4 = 0.0, $x5 = 0.0, $x6 = 0.0, $x7 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $ldm;
 $1 = $ncol;
 $2 = $M;
 $3 = $rhs;
 $firstcol = 0;
 $4 = $2;
 $M0 = $4;
 while(1) {
  $5 = $firstcol;
  $6 = $1;
  $7 = (($6) - 7)|0;
  $8 = ($5|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $M0;
  $10 = ((($9)) + 8|0);
  $Mki0 = $10;
  $11 = $Mki0;
  $12 = $0;
  $13 = (($11) + ($12<<3)|0);
  $14 = ((($13)) + 8|0);
  $Mki1 = $14;
  $15 = $Mki1;
  $16 = $0;
  $17 = (($15) + ($16<<3)|0);
  $18 = ((($17)) + 8|0);
  $Mki2 = $18;
  $19 = $Mki2;
  $20 = $0;
  $21 = (($19) + ($20<<3)|0);
  $22 = ((($21)) + 8|0);
  $Mki3 = $22;
  $23 = $Mki3;
  $24 = $0;
  $25 = (($23) + ($24<<3)|0);
  $26 = ((($25)) + 8|0);
  $Mki4 = $26;
  $27 = $Mki4;
  $28 = $0;
  $29 = (($27) + ($28<<3)|0);
  $30 = ((($29)) + 8|0);
  $Mki5 = $30;
  $31 = $Mki5;
  $32 = $0;
  $33 = (($31) + ($32<<3)|0);
  $34 = ((($33)) + 8|0);
  $Mki6 = $34;
  $35 = $Mki6;
  $36 = $0;
  $37 = (($35) + ($36<<3)|0);
  $38 = ((($37)) + 8|0);
  $Mki7 = $38;
  $39 = $firstcol;
  $40 = $3;
  $41 = (($40) + ($39<<3)|0);
  $42 = +HEAPF64[$41>>3];
  $x0 = $42;
  $43 = $firstcol;
  $44 = (($43) + 1)|0;
  $45 = $3;
  $46 = (($45) + ($44<<3)|0);
  $47 = +HEAPF64[$46>>3];
  $48 = $x0;
  $49 = $Mki0;
  $50 = ((($49)) + 8|0);
  $Mki0 = $50;
  $51 = +HEAPF64[$49>>3];
  $52 = $48 * $51;
  $53 = $47 - $52;
  $x1 = $53;
  $54 = $firstcol;
  $55 = (($54) + 2)|0;
  $56 = $3;
  $57 = (($56) + ($55<<3)|0);
  $58 = +HEAPF64[$57>>3];
  $59 = $x0;
  $60 = $Mki0;
  $61 = ((($60)) + 8|0);
  $Mki0 = $61;
  $62 = +HEAPF64[$60>>3];
  $63 = $59 * $62;
  $64 = $58 - $63;
  $65 = $x1;
  $66 = $Mki1;
  $67 = ((($66)) + 8|0);
  $Mki1 = $67;
  $68 = +HEAPF64[$66>>3];
  $69 = $65 * $68;
  $70 = $64 - $69;
  $x2 = $70;
  $71 = $firstcol;
  $72 = (($71) + 3)|0;
  $73 = $3;
  $74 = (($73) + ($72<<3)|0);
  $75 = +HEAPF64[$74>>3];
  $76 = $x0;
  $77 = $Mki0;
  $78 = ((($77)) + 8|0);
  $Mki0 = $78;
  $79 = +HEAPF64[$77>>3];
  $80 = $76 * $79;
  $81 = $75 - $80;
  $82 = $x1;
  $83 = $Mki1;
  $84 = ((($83)) + 8|0);
  $Mki1 = $84;
  $85 = +HEAPF64[$83>>3];
  $86 = $82 * $85;
  $87 = $81 - $86;
  $88 = $x2;
  $89 = $Mki2;
  $90 = ((($89)) + 8|0);
  $Mki2 = $90;
  $91 = +HEAPF64[$89>>3];
  $92 = $88 * $91;
  $93 = $87 - $92;
  $x3 = $93;
  $94 = $firstcol;
  $95 = (($94) + 4)|0;
  $96 = $3;
  $97 = (($96) + ($95<<3)|0);
  $98 = +HEAPF64[$97>>3];
  $99 = $x0;
  $100 = $Mki0;
  $101 = ((($100)) + 8|0);
  $Mki0 = $101;
  $102 = +HEAPF64[$100>>3];
  $103 = $99 * $102;
  $104 = $98 - $103;
  $105 = $x1;
  $106 = $Mki1;
  $107 = ((($106)) + 8|0);
  $Mki1 = $107;
  $108 = +HEAPF64[$106>>3];
  $109 = $105 * $108;
  $110 = $104 - $109;
  $111 = $x2;
  $112 = $Mki2;
  $113 = ((($112)) + 8|0);
  $Mki2 = $113;
  $114 = +HEAPF64[$112>>3];
  $115 = $111 * $114;
  $116 = $110 - $115;
  $117 = $x3;
  $118 = $Mki3;
  $119 = ((($118)) + 8|0);
  $Mki3 = $119;
  $120 = +HEAPF64[$118>>3];
  $121 = $117 * $120;
  $122 = $116 - $121;
  $x4 = $122;
  $123 = $firstcol;
  $124 = (($123) + 5)|0;
  $125 = $3;
  $126 = (($125) + ($124<<3)|0);
  $127 = +HEAPF64[$126>>3];
  $128 = $x0;
  $129 = $Mki0;
  $130 = ((($129)) + 8|0);
  $Mki0 = $130;
  $131 = +HEAPF64[$129>>3];
  $132 = $128 * $131;
  $133 = $127 - $132;
  $134 = $x1;
  $135 = $Mki1;
  $136 = ((($135)) + 8|0);
  $Mki1 = $136;
  $137 = +HEAPF64[$135>>3];
  $138 = $134 * $137;
  $139 = $133 - $138;
  $140 = $x2;
  $141 = $Mki2;
  $142 = ((($141)) + 8|0);
  $Mki2 = $142;
  $143 = +HEAPF64[$141>>3];
  $144 = $140 * $143;
  $145 = $139 - $144;
  $146 = $x3;
  $147 = $Mki3;
  $148 = ((($147)) + 8|0);
  $Mki3 = $148;
  $149 = +HEAPF64[$147>>3];
  $150 = $146 * $149;
  $151 = $145 - $150;
  $152 = $x4;
  $153 = $Mki4;
  $154 = ((($153)) + 8|0);
  $Mki4 = $154;
  $155 = +HEAPF64[$153>>3];
  $156 = $152 * $155;
  $157 = $151 - $156;
  $x5 = $157;
  $158 = $firstcol;
  $159 = (($158) + 6)|0;
  $160 = $3;
  $161 = (($160) + ($159<<3)|0);
  $162 = +HEAPF64[$161>>3];
  $163 = $x0;
  $164 = $Mki0;
  $165 = ((($164)) + 8|0);
  $Mki0 = $165;
  $166 = +HEAPF64[$164>>3];
  $167 = $163 * $166;
  $168 = $162 - $167;
  $169 = $x1;
  $170 = $Mki1;
  $171 = ((($170)) + 8|0);
  $Mki1 = $171;
  $172 = +HEAPF64[$170>>3];
  $173 = $169 * $172;
  $174 = $168 - $173;
  $175 = $x2;
  $176 = $Mki2;
  $177 = ((($176)) + 8|0);
  $Mki2 = $177;
  $178 = +HEAPF64[$176>>3];
  $179 = $175 * $178;
  $180 = $174 - $179;
  $181 = $x3;
  $182 = $Mki3;
  $183 = ((($182)) + 8|0);
  $Mki3 = $183;
  $184 = +HEAPF64[$182>>3];
  $185 = $181 * $184;
  $186 = $180 - $185;
  $187 = $x4;
  $188 = $Mki4;
  $189 = ((($188)) + 8|0);
  $Mki4 = $189;
  $190 = +HEAPF64[$188>>3];
  $191 = $187 * $190;
  $192 = $186 - $191;
  $193 = $x5;
  $194 = $Mki5;
  $195 = ((($194)) + 8|0);
  $Mki5 = $195;
  $196 = +HEAPF64[$194>>3];
  $197 = $193 * $196;
  $198 = $192 - $197;
  $x6 = $198;
  $199 = $firstcol;
  $200 = (($199) + 7)|0;
  $201 = $3;
  $202 = (($201) + ($200<<3)|0);
  $203 = +HEAPF64[$202>>3];
  $204 = $x0;
  $205 = $Mki0;
  $206 = ((($205)) + 8|0);
  $Mki0 = $206;
  $207 = +HEAPF64[$205>>3];
  $208 = $204 * $207;
  $209 = $203 - $208;
  $210 = $x1;
  $211 = $Mki1;
  $212 = ((($211)) + 8|0);
  $Mki1 = $212;
  $213 = +HEAPF64[$211>>3];
  $214 = $210 * $213;
  $215 = $209 - $214;
  $216 = $x2;
  $217 = $Mki2;
  $218 = ((($217)) + 8|0);
  $Mki2 = $218;
  $219 = +HEAPF64[$217>>3];
  $220 = $216 * $219;
  $221 = $215 - $220;
  $222 = $x3;
  $223 = $Mki3;
  $224 = ((($223)) + 8|0);
  $Mki3 = $224;
  $225 = +HEAPF64[$223>>3];
  $226 = $222 * $225;
  $227 = $221 - $226;
  $228 = $x4;
  $229 = $Mki4;
  $230 = ((($229)) + 8|0);
  $Mki4 = $230;
  $231 = +HEAPF64[$229>>3];
  $232 = $228 * $231;
  $233 = $227 - $232;
  $234 = $x5;
  $235 = $Mki5;
  $236 = ((($235)) + 8|0);
  $Mki5 = $236;
  $237 = +HEAPF64[$235>>3];
  $238 = $234 * $237;
  $239 = $233 - $238;
  $240 = $x6;
  $241 = $Mki6;
  $242 = ((($241)) + 8|0);
  $Mki6 = $242;
  $243 = +HEAPF64[$241>>3];
  $244 = $240 * $243;
  $245 = $239 - $244;
  $x7 = $245;
  $246 = $x1;
  $247 = $firstcol;
  $248 = (($247) + 1)|0;
  $firstcol = $248;
  $249 = $3;
  $250 = (($249) + ($248<<3)|0);
  HEAPF64[$250>>3] = $246;
  $251 = $x2;
  $252 = $firstcol;
  $253 = (($252) + 1)|0;
  $firstcol = $253;
  $254 = $3;
  $255 = (($254) + ($253<<3)|0);
  HEAPF64[$255>>3] = $251;
  $256 = $x3;
  $257 = $firstcol;
  $258 = (($257) + 1)|0;
  $firstcol = $258;
  $259 = $3;
  $260 = (($259) + ($258<<3)|0);
  HEAPF64[$260>>3] = $256;
  $261 = $x4;
  $262 = $firstcol;
  $263 = (($262) + 1)|0;
  $firstcol = $263;
  $264 = $3;
  $265 = (($264) + ($263<<3)|0);
  HEAPF64[$265>>3] = $261;
  $266 = $x5;
  $267 = $firstcol;
  $268 = (($267) + 1)|0;
  $firstcol = $268;
  $269 = $3;
  $270 = (($269) + ($268<<3)|0);
  HEAPF64[$270>>3] = $266;
  $271 = $x6;
  $272 = $firstcol;
  $273 = (($272) + 1)|0;
  $firstcol = $273;
  $274 = $3;
  $275 = (($274) + ($273<<3)|0);
  HEAPF64[$275>>3] = $271;
  $276 = $x7;
  $277 = $firstcol;
  $278 = (($277) + 1)|0;
  $firstcol = $278;
  $279 = $3;
  $280 = (($279) + ($278<<3)|0);
  HEAPF64[$280>>3] = $276;
  $281 = $firstcol;
  $282 = (($281) + 1)|0;
  $firstcol = $282;
  $283 = $firstcol;
  $k = $283;
  while(1) {
   $284 = $k;
   $285 = $1;
   $286 = ($284|0)<($285|0);
   if (!($286)) {
    break;
   }
   $287 = $k;
   $288 = $3;
   $289 = (($288) + ($287<<3)|0);
   $290 = +HEAPF64[$289>>3];
   $291 = $x0;
   $292 = $Mki0;
   $293 = ((($292)) + 8|0);
   $Mki0 = $293;
   $294 = +HEAPF64[$292>>3];
   $295 = $291 * $294;
   $296 = $290 - $295;
   $297 = $x1;
   $298 = $Mki1;
   $299 = ((($298)) + 8|0);
   $Mki1 = $299;
   $300 = +HEAPF64[$298>>3];
   $301 = $297 * $300;
   $302 = $296 - $301;
   $303 = $x2;
   $304 = $Mki2;
   $305 = ((($304)) + 8|0);
   $Mki2 = $305;
   $306 = +HEAPF64[$304>>3];
   $307 = $303 * $306;
   $308 = $302 - $307;
   $309 = $x3;
   $310 = $Mki3;
   $311 = ((($310)) + 8|0);
   $Mki3 = $311;
   $312 = +HEAPF64[$310>>3];
   $313 = $309 * $312;
   $314 = $308 - $313;
   $315 = $x4;
   $316 = $Mki4;
   $317 = ((($316)) + 8|0);
   $Mki4 = $317;
   $318 = +HEAPF64[$316>>3];
   $319 = $315 * $318;
   $320 = $314 - $319;
   $321 = $x5;
   $322 = $Mki5;
   $323 = ((($322)) + 8|0);
   $Mki5 = $323;
   $324 = +HEAPF64[$322>>3];
   $325 = $321 * $324;
   $326 = $320 - $325;
   $327 = $x6;
   $328 = $Mki6;
   $329 = ((($328)) + 8|0);
   $Mki6 = $329;
   $330 = +HEAPF64[$328>>3];
   $331 = $327 * $330;
   $332 = $326 - $331;
   $333 = $x7;
   $334 = $Mki7;
   $335 = ((($334)) + 8|0);
   $Mki7 = $335;
   $336 = +HEAPF64[$334>>3];
   $337 = $333 * $336;
   $338 = $332 - $337;
   $339 = $k;
   $340 = $3;
   $341 = (($340) + ($339<<3)|0);
   HEAPF64[$341>>3] = $338;
   $342 = $k;
   $343 = (($342) + 1)|0;
   $k = $343;
  }
  $344 = $0;
  $345 = $344<<3;
  $346 = (($345) + 8)|0;
  $347 = $M0;
  $348 = (($347) + ($346<<3)|0);
  $M0 = $348;
 }
 while(1) {
  $349 = $firstcol;
  $350 = $1;
  $351 = (($350) - 3)|0;
  $352 = ($349|0)<($351|0);
  if (!($352)) {
   break;
  }
  $353 = $M0;
  $354 = ((($353)) + 8|0);
  $Mki0 = $354;
  $355 = $Mki0;
  $356 = $0;
  $357 = (($355) + ($356<<3)|0);
  $358 = ((($357)) + 8|0);
  $Mki1 = $358;
  $359 = $Mki1;
  $360 = $0;
  $361 = (($359) + ($360<<3)|0);
  $362 = ((($361)) + 8|0);
  $Mki2 = $362;
  $363 = $Mki2;
  $364 = $0;
  $365 = (($363) + ($364<<3)|0);
  $366 = ((($365)) + 8|0);
  $Mki3 = $366;
  $367 = $firstcol;
  $368 = $3;
  $369 = (($368) + ($367<<3)|0);
  $370 = +HEAPF64[$369>>3];
  $x0 = $370;
  $371 = $firstcol;
  $372 = (($371) + 1)|0;
  $373 = $3;
  $374 = (($373) + ($372<<3)|0);
  $375 = +HEAPF64[$374>>3];
  $376 = $x0;
  $377 = $Mki0;
  $378 = ((($377)) + 8|0);
  $Mki0 = $378;
  $379 = +HEAPF64[$377>>3];
  $380 = $376 * $379;
  $381 = $375 - $380;
  $x1 = $381;
  $382 = $firstcol;
  $383 = (($382) + 2)|0;
  $384 = $3;
  $385 = (($384) + ($383<<3)|0);
  $386 = +HEAPF64[$385>>3];
  $387 = $x0;
  $388 = $Mki0;
  $389 = ((($388)) + 8|0);
  $Mki0 = $389;
  $390 = +HEAPF64[$388>>3];
  $391 = $387 * $390;
  $392 = $386 - $391;
  $393 = $x1;
  $394 = $Mki1;
  $395 = ((($394)) + 8|0);
  $Mki1 = $395;
  $396 = +HEAPF64[$394>>3];
  $397 = $393 * $396;
  $398 = $392 - $397;
  $x2 = $398;
  $399 = $firstcol;
  $400 = (($399) + 3)|0;
  $401 = $3;
  $402 = (($401) + ($400<<3)|0);
  $403 = +HEAPF64[$402>>3];
  $404 = $x0;
  $405 = $Mki0;
  $406 = ((($405)) + 8|0);
  $Mki0 = $406;
  $407 = +HEAPF64[$405>>3];
  $408 = $404 * $407;
  $409 = $403 - $408;
  $410 = $x1;
  $411 = $Mki1;
  $412 = ((($411)) + 8|0);
  $Mki1 = $412;
  $413 = +HEAPF64[$411>>3];
  $414 = $410 * $413;
  $415 = $409 - $414;
  $416 = $x2;
  $417 = $Mki2;
  $418 = ((($417)) + 8|0);
  $Mki2 = $418;
  $419 = +HEAPF64[$417>>3];
  $420 = $416 * $419;
  $421 = $415 - $420;
  $x3 = $421;
  $422 = $x1;
  $423 = $firstcol;
  $424 = (($423) + 1)|0;
  $firstcol = $424;
  $425 = $3;
  $426 = (($425) + ($424<<3)|0);
  HEAPF64[$426>>3] = $422;
  $427 = $x2;
  $428 = $firstcol;
  $429 = (($428) + 1)|0;
  $firstcol = $429;
  $430 = $3;
  $431 = (($430) + ($429<<3)|0);
  HEAPF64[$431>>3] = $427;
  $432 = $x3;
  $433 = $firstcol;
  $434 = (($433) + 1)|0;
  $firstcol = $434;
  $435 = $3;
  $436 = (($435) + ($434<<3)|0);
  HEAPF64[$436>>3] = $432;
  $437 = $firstcol;
  $438 = (($437) + 1)|0;
  $firstcol = $438;
  $439 = $firstcol;
  $k = $439;
  while(1) {
   $440 = $k;
   $441 = $1;
   $442 = ($440|0)<($441|0);
   if (!($442)) {
    break;
   }
   $443 = $k;
   $444 = $3;
   $445 = (($444) + ($443<<3)|0);
   $446 = +HEAPF64[$445>>3];
   $447 = $x0;
   $448 = $Mki0;
   $449 = ((($448)) + 8|0);
   $Mki0 = $449;
   $450 = +HEAPF64[$448>>3];
   $451 = $447 * $450;
   $452 = $446 - $451;
   $453 = $x1;
   $454 = $Mki1;
   $455 = ((($454)) + 8|0);
   $Mki1 = $455;
   $456 = +HEAPF64[$454>>3];
   $457 = $453 * $456;
   $458 = $452 - $457;
   $459 = $x2;
   $460 = $Mki2;
   $461 = ((($460)) + 8|0);
   $Mki2 = $461;
   $462 = +HEAPF64[$460>>3];
   $463 = $459 * $462;
   $464 = $458 - $463;
   $465 = $x3;
   $466 = $Mki3;
   $467 = ((($466)) + 8|0);
   $Mki3 = $467;
   $468 = +HEAPF64[$466>>3];
   $469 = $465 * $468;
   $470 = $464 - $469;
   $471 = $k;
   $472 = $3;
   $473 = (($472) + ($471<<3)|0);
   HEAPF64[$473>>3] = $470;
   $474 = $k;
   $475 = (($474) + 1)|0;
   $k = $475;
  }
  $476 = $0;
  $477 = $476<<2;
  $478 = (($477) + 4)|0;
  $479 = $M0;
  $480 = (($479) + ($478<<3)|0);
  $M0 = $480;
 }
 $481 = $firstcol;
 $482 = $1;
 $483 = (($482) - 1)|0;
 $484 = ($481|0)<($483|0);
 if (!($484)) {
  STACKTOP = sp;return;
 }
 $485 = $M0;
 $486 = ((($485)) + 8|0);
 $Mki0 = $486;
 $487 = $Mki0;
 $488 = $0;
 $489 = (($487) + ($488<<3)|0);
 $490 = ((($489)) + 8|0);
 $Mki1 = $490;
 $491 = $firstcol;
 $492 = $3;
 $493 = (($492) + ($491<<3)|0);
 $494 = +HEAPF64[$493>>3];
 $x0 = $494;
 $495 = $firstcol;
 $496 = (($495) + 1)|0;
 $497 = $3;
 $498 = (($497) + ($496<<3)|0);
 $499 = +HEAPF64[$498>>3];
 $500 = $x0;
 $501 = $Mki0;
 $502 = ((($501)) + 8|0);
 $Mki0 = $502;
 $503 = +HEAPF64[$501>>3];
 $504 = $500 * $503;
 $505 = $499 - $504;
 $x1 = $505;
 $506 = $x1;
 $507 = $firstcol;
 $508 = (($507) + 1)|0;
 $firstcol = $508;
 $509 = $3;
 $510 = (($509) + ($508<<3)|0);
 HEAPF64[$510>>3] = $506;
 $511 = $firstcol;
 $512 = (($511) + 1)|0;
 $firstcol = $512;
 $513 = $firstcol;
 $k = $513;
 while(1) {
  $514 = $k;
  $515 = $1;
  $516 = ($514|0)<($515|0);
  if (!($516)) {
   break;
  }
  $517 = $k;
  $518 = $3;
  $519 = (($518) + ($517<<3)|0);
  $520 = +HEAPF64[$519>>3];
  $521 = $x0;
  $522 = $Mki0;
  $523 = ((($522)) + 8|0);
  $Mki0 = $523;
  $524 = +HEAPF64[$522>>3];
  $525 = $521 * $524;
  $526 = $520 - $525;
  $527 = $x1;
  $528 = $Mki1;
  $529 = ((($528)) + 8|0);
  $Mki1 = $529;
  $530 = +HEAPF64[$528>>3];
  $531 = $527 * $530;
  $532 = $526 - $531;
  $533 = $k;
  $534 = $3;
  $535 = (($534) + ($533<<3)|0);
  HEAPF64[$535>>3] = $532;
  $536 = $k;
  $537 = (($536) + 1)|0;
  $k = $537;
 }
 STACKTOP = sp;return;
}
function _dusolve($ldm,$ncol,$M,$rhs) {
 $ldm = $ldm|0;
 $ncol = $ncol|0;
 $M = $M|0;
 $rhs = $rhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0.0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0.0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $irow = 0, $j = 0, $jcol = 0, $xj = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $ldm;
 $1 = $ncol;
 $2 = $M;
 $3 = $rhs;
 $4 = $1;
 $5 = (($4) - 1)|0;
 $jcol = $5;
 $j = 0;
 while(1) {
  $6 = $j;
  $7 = $1;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $jcol;
  $10 = $3;
  $11 = (($10) + ($9<<3)|0);
  $12 = +HEAPF64[$11>>3];
  $13 = $jcol;
  $14 = $jcol;
  $15 = $0;
  $16 = Math_imul($14, $15)|0;
  $17 = (($13) + ($16))|0;
  $18 = $2;
  $19 = (($18) + ($17<<3)|0);
  $20 = +HEAPF64[$19>>3];
  $21 = $12 / $20;
  $xj = $21;
  $22 = $xj;
  $23 = $jcol;
  $24 = $3;
  $25 = (($24) + ($23<<3)|0);
  HEAPF64[$25>>3] = $22;
  $irow = 0;
  while(1) {
   $26 = $irow;
   $27 = $jcol;
   $28 = ($26|0)<($27|0);
   if (!($28)) {
    break;
   }
   $29 = $xj;
   $30 = $irow;
   $31 = $jcol;
   $32 = $0;
   $33 = Math_imul($31, $32)|0;
   $34 = (($30) + ($33))|0;
   $35 = $2;
   $36 = (($35) + ($34<<3)|0);
   $37 = +HEAPF64[$36>>3];
   $38 = $29 * $37;
   $39 = $irow;
   $40 = $3;
   $41 = (($40) + ($39<<3)|0);
   $42 = +HEAPF64[$41>>3];
   $43 = $42 - $38;
   HEAPF64[$41>>3] = $43;
   $44 = $irow;
   $45 = (($44) + 1)|0;
   $irow = $45;
  }
  $46 = $jcol;
  $47 = (($46) + -1)|0;
  $jcol = $47;
  $48 = $j;
  $49 = (($48) + 1)|0;
  $j = $49;
 }
 STACKTOP = sp;return;
}
function _dmatvec($ldm,$nrow,$ncol,$M,$vec,$Mxvec) {
 $ldm = $ldm|0;
 $nrow = $nrow|0;
 $ncol = $ncol|0;
 $M = $M|0;
 $vec = $vec|0;
 $Mxvec = $Mxvec|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0.0, $104 = 0.0, $105 = 0.0, $106 = 0, $107 = 0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0.0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0.0, $127 = 0.0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0.0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0.0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0.0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0.0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0.0, $172 = 0, $173 = 0, $174 = 0.0, $175 = 0.0, $176 = 0.0, $177 = 0, $178 = 0, $179 = 0.0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0.0, $183 = 0, $184 = 0, $185 = 0.0, $186 = 0.0, $187 = 0.0, $188 = 0.0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0, $195 = 0, $196 = 0, $197 = 0.0, $198 = 0.0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0.0, $214 = 0, $215 = 0, $216 = 0, $217 = 0.0, $218 = 0, $219 = 0, $22 = 0, $220 = 0.0, $221 = 0.0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0.0, $226 = 0.0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0.0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0.0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0.0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0.0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0.0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0.0, $73 = 0, $74 = 0, $75 = 0, $76 = 0.0, $77 = 0, $78 = 0, $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0, $83 = 0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0.0, $91 = 0.0, $92 = 0.0, $93 = 0.0, $94 = 0, $95 = 0, $96 = 0.0, $97 = 0.0, $98 = 0.0, $99 = 0.0, $M0 = 0, $Mki0 = 0, $Mki1 = 0, $Mki2 = 0, $Mki3 = 0, $Mki4 = 0, $Mki5 = 0, $Mki6 = 0;
 var $Mki7 = 0, $firstcol = 0, $k = 0, $vi0 = 0.0, $vi1 = 0.0, $vi2 = 0.0, $vi3 = 0.0, $vi4 = 0.0, $vi5 = 0.0, $vi6 = 0.0, $vi7 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $ldm;
 $1 = $nrow;
 $2 = $ncol;
 $3 = $M;
 $4 = $vec;
 $5 = $Mxvec;
 $firstcol = 0;
 $6 = $3;
 $M0 = $6;
 while(1) {
  $7 = $firstcol;
  $8 = $2;
  $9 = (($8) - 7)|0;
  $10 = ($7|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $M0;
  $Mki0 = $11;
  $12 = $Mki0;
  $13 = $0;
  $14 = (($12) + ($13<<3)|0);
  $Mki1 = $14;
  $15 = $Mki1;
  $16 = $0;
  $17 = (($15) + ($16<<3)|0);
  $Mki2 = $17;
  $18 = $Mki2;
  $19 = $0;
  $20 = (($18) + ($19<<3)|0);
  $Mki3 = $20;
  $21 = $Mki3;
  $22 = $0;
  $23 = (($21) + ($22<<3)|0);
  $Mki4 = $23;
  $24 = $Mki4;
  $25 = $0;
  $26 = (($24) + ($25<<3)|0);
  $Mki5 = $26;
  $27 = $Mki5;
  $28 = $0;
  $29 = (($27) + ($28<<3)|0);
  $Mki6 = $29;
  $30 = $Mki6;
  $31 = $0;
  $32 = (($30) + ($31<<3)|0);
  $Mki7 = $32;
  $33 = $firstcol;
  $34 = (($33) + 1)|0;
  $firstcol = $34;
  $35 = $4;
  $36 = (($35) + ($33<<3)|0);
  $37 = +HEAPF64[$36>>3];
  $vi0 = $37;
  $38 = $firstcol;
  $39 = (($38) + 1)|0;
  $firstcol = $39;
  $40 = $4;
  $41 = (($40) + ($38<<3)|0);
  $42 = +HEAPF64[$41>>3];
  $vi1 = $42;
  $43 = $firstcol;
  $44 = (($43) + 1)|0;
  $firstcol = $44;
  $45 = $4;
  $46 = (($45) + ($43<<3)|0);
  $47 = +HEAPF64[$46>>3];
  $vi2 = $47;
  $48 = $firstcol;
  $49 = (($48) + 1)|0;
  $firstcol = $49;
  $50 = $4;
  $51 = (($50) + ($48<<3)|0);
  $52 = +HEAPF64[$51>>3];
  $vi3 = $52;
  $53 = $firstcol;
  $54 = (($53) + 1)|0;
  $firstcol = $54;
  $55 = $4;
  $56 = (($55) + ($53<<3)|0);
  $57 = +HEAPF64[$56>>3];
  $vi4 = $57;
  $58 = $firstcol;
  $59 = (($58) + 1)|0;
  $firstcol = $59;
  $60 = $4;
  $61 = (($60) + ($58<<3)|0);
  $62 = +HEAPF64[$61>>3];
  $vi5 = $62;
  $63 = $firstcol;
  $64 = (($63) + 1)|0;
  $firstcol = $64;
  $65 = $4;
  $66 = (($65) + ($63<<3)|0);
  $67 = +HEAPF64[$66>>3];
  $vi6 = $67;
  $68 = $firstcol;
  $69 = (($68) + 1)|0;
  $firstcol = $69;
  $70 = $4;
  $71 = (($70) + ($68<<3)|0);
  $72 = +HEAPF64[$71>>3];
  $vi7 = $72;
  $k = 0;
  while(1) {
   $73 = $k;
   $74 = $1;
   $75 = ($73|0)<($74|0);
   if (!($75)) {
    break;
   }
   $76 = $vi0;
   $77 = $Mki0;
   $78 = ((($77)) + 8|0);
   $Mki0 = $78;
   $79 = +HEAPF64[$77>>3];
   $80 = $76 * $79;
   $81 = $vi1;
   $82 = $Mki1;
   $83 = ((($82)) + 8|0);
   $Mki1 = $83;
   $84 = +HEAPF64[$82>>3];
   $85 = $81 * $84;
   $86 = $80 + $85;
   $87 = $vi2;
   $88 = $Mki2;
   $89 = ((($88)) + 8|0);
   $Mki2 = $89;
   $90 = +HEAPF64[$88>>3];
   $91 = $87 * $90;
   $92 = $86 + $91;
   $93 = $vi3;
   $94 = $Mki3;
   $95 = ((($94)) + 8|0);
   $Mki3 = $95;
   $96 = +HEAPF64[$94>>3];
   $97 = $93 * $96;
   $98 = $92 + $97;
   $99 = $vi4;
   $100 = $Mki4;
   $101 = ((($100)) + 8|0);
   $Mki4 = $101;
   $102 = +HEAPF64[$100>>3];
   $103 = $99 * $102;
   $104 = $98 + $103;
   $105 = $vi5;
   $106 = $Mki5;
   $107 = ((($106)) + 8|0);
   $Mki5 = $107;
   $108 = +HEAPF64[$106>>3];
   $109 = $105 * $108;
   $110 = $104 + $109;
   $111 = $vi6;
   $112 = $Mki6;
   $113 = ((($112)) + 8|0);
   $Mki6 = $113;
   $114 = +HEAPF64[$112>>3];
   $115 = $111 * $114;
   $116 = $110 + $115;
   $117 = $vi7;
   $118 = $Mki7;
   $119 = ((($118)) + 8|0);
   $Mki7 = $119;
   $120 = +HEAPF64[$118>>3];
   $121 = $117 * $120;
   $122 = $116 + $121;
   $123 = $k;
   $124 = $5;
   $125 = (($124) + ($123<<3)|0);
   $126 = +HEAPF64[$125>>3];
   $127 = $126 + $122;
   HEAPF64[$125>>3] = $127;
   $128 = $k;
   $129 = (($128) + 1)|0;
   $k = $129;
  }
  $130 = $0;
  $131 = $130<<3;
  $132 = $M0;
  $133 = (($132) + ($131<<3)|0);
  $M0 = $133;
 }
 while(1) {
  $134 = $firstcol;
  $135 = $2;
  $136 = (($135) - 3)|0;
  $137 = ($134|0)<($136|0);
  if (!($137)) {
   break;
  }
  $138 = $M0;
  $Mki0 = $138;
  $139 = $Mki0;
  $140 = $0;
  $141 = (($139) + ($140<<3)|0);
  $Mki1 = $141;
  $142 = $Mki1;
  $143 = $0;
  $144 = (($142) + ($143<<3)|0);
  $Mki2 = $144;
  $145 = $Mki2;
  $146 = $0;
  $147 = (($145) + ($146<<3)|0);
  $Mki3 = $147;
  $148 = $firstcol;
  $149 = (($148) + 1)|0;
  $firstcol = $149;
  $150 = $4;
  $151 = (($150) + ($148<<3)|0);
  $152 = +HEAPF64[$151>>3];
  $vi0 = $152;
  $153 = $firstcol;
  $154 = (($153) + 1)|0;
  $firstcol = $154;
  $155 = $4;
  $156 = (($155) + ($153<<3)|0);
  $157 = +HEAPF64[$156>>3];
  $vi1 = $157;
  $158 = $firstcol;
  $159 = (($158) + 1)|0;
  $firstcol = $159;
  $160 = $4;
  $161 = (($160) + ($158<<3)|0);
  $162 = +HEAPF64[$161>>3];
  $vi2 = $162;
  $163 = $firstcol;
  $164 = (($163) + 1)|0;
  $firstcol = $164;
  $165 = $4;
  $166 = (($165) + ($163<<3)|0);
  $167 = +HEAPF64[$166>>3];
  $vi3 = $167;
  $k = 0;
  while(1) {
   $168 = $k;
   $169 = $1;
   $170 = ($168|0)<($169|0);
   if (!($170)) {
    break;
   }
   $171 = $vi0;
   $172 = $Mki0;
   $173 = ((($172)) + 8|0);
   $Mki0 = $173;
   $174 = +HEAPF64[$172>>3];
   $175 = $171 * $174;
   $176 = $vi1;
   $177 = $Mki1;
   $178 = ((($177)) + 8|0);
   $Mki1 = $178;
   $179 = +HEAPF64[$177>>3];
   $180 = $176 * $179;
   $181 = $175 + $180;
   $182 = $vi2;
   $183 = $Mki2;
   $184 = ((($183)) + 8|0);
   $Mki2 = $184;
   $185 = +HEAPF64[$183>>3];
   $186 = $182 * $185;
   $187 = $181 + $186;
   $188 = $vi3;
   $189 = $Mki3;
   $190 = ((($189)) + 8|0);
   $Mki3 = $190;
   $191 = +HEAPF64[$189>>3];
   $192 = $188 * $191;
   $193 = $187 + $192;
   $194 = $k;
   $195 = $5;
   $196 = (($195) + ($194<<3)|0);
   $197 = +HEAPF64[$196>>3];
   $198 = $197 + $193;
   HEAPF64[$196>>3] = $198;
   $199 = $k;
   $200 = (($199) + 1)|0;
   $k = $200;
  }
  $201 = $0;
  $202 = $201<<2;
  $203 = $M0;
  $204 = (($203) + ($202<<3)|0);
  $M0 = $204;
 }
 while(1) {
  $205 = $firstcol;
  $206 = $2;
  $207 = ($205|0)<($206|0);
  if (!($207)) {
   break;
  }
  $208 = $M0;
  $Mki0 = $208;
  $209 = $firstcol;
  $210 = (($209) + 1)|0;
  $firstcol = $210;
  $211 = $4;
  $212 = (($211) + ($209<<3)|0);
  $213 = +HEAPF64[$212>>3];
  $vi0 = $213;
  $k = 0;
  while(1) {
   $214 = $k;
   $215 = $1;
   $216 = ($214|0)<($215|0);
   if (!($216)) {
    break;
   }
   $217 = $vi0;
   $218 = $Mki0;
   $219 = ((($218)) + 8|0);
   $Mki0 = $219;
   $220 = +HEAPF64[$218>>3];
   $221 = $217 * $220;
   $222 = $k;
   $223 = $5;
   $224 = (($223) + ($222<<3)|0);
   $225 = +HEAPF64[$224>>3];
   $226 = $225 + $221;
   HEAPF64[$224>>3] = $226;
   $227 = $k;
   $228 = (($227) + 1)|0;
   $k = $228;
  }
  $229 = $0;
  $230 = $M0;
  $231 = (($230) + ($229<<3)|0);
  $M0 = $231;
 }
 STACKTOP = sp;return;
}
function _dlinsol($iopt,$n,$nnz,$nrhs,$values,$rowind,$colptr,$b,$ldb,$f_factors,$permc_spec,$trans,$info) {
 $iopt = $iopt|0;
 $n = $n|0;
 $nnz = $nnz|0;
 $nrhs = $nrhs|0;
 $values = $values|0;
 $rowind = $rowind|0;
 $colptr = $colptr|0;
 $b = $b|0;
 $ldb = $ldb|0;
 $f_factors = $f_factors|0;
 $permc_spec = $permc_spec|0;
 $trans = $trans|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0.0, $113 = 0.0, $114 = 0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $A = 0, $AC = 0, $B = 0, $L = 0, $LUfactors = 0, $U = 0, $etree = 0, $mem_usage = 0, $options = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0;
 var $or$cond17 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $panel_size = 0, $perm_c = 0, $perm_r = 0, $relax = 0, $stat = 0, $vararg_buffer = 0, $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer22 = 0, $vararg_buffer24 = 0, $vararg_buffer28 = 0, $vararg_buffer31 = 0, $vararg_ptr27 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 400|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer31 = sp + 200|0;
 $vararg_buffer28 = sp + 192|0;
 $vararg_buffer24 = sp + 176|0;
 $vararg_buffer22 = sp + 168|0;
 $vararg_buffer20 = sp + 160|0;
 $vararg_buffer18 = sp + 152|0;
 $vararg_buffer = sp + 144|0;
 $A = sp + 320|0;
 $AC = sp + 296|0;
 $B = sp + 272|0;
 $mem_usage = sp + 232|0;
 $options = sp;
 $stat = sp + 208|0;
 $0 = $iopt;
 $1 = $n;
 $2 = $nnz;
 $3 = $nrhs;
 $4 = $values;
 $5 = $rowind;
 $6 = $colptr;
 $7 = $b;
 $8 = $ldb;
 $9 = $f_factors;
 $10 = $permc_spec;
 $11 = $trans;
 $12 = $info;
 $13 = $0;
 $14 = ($13|0)!=(3);
 $15 = $1;
 $16 = ($15|0)<=(0);
 $or$cond = $14 & $16;
 if ($or$cond) {
  $17 = $12;
  HEAP32[$17>>2] = -2;
  STACKTOP = sp;return;
 }
 $18 = $0;
 $19 = ($18|0)==(1);
 $20 = $2;
 $21 = ($20|0)<(0);
 $or$cond3 = $19 & $21;
 if ($or$cond3) {
  $22 = $12;
  HEAP32[$22>>2] = -3;
  STACKTOP = sp;return;
 }
 $23 = $0;
 $24 = ($23|0)==(2);
 $25 = $3;
 $26 = ($25|0)<(0);
 $or$cond5 = $24 & $26;
 if ($or$cond5) {
  $27 = $12;
  HEAP32[$27>>2] = -4;
  STACKTOP = sp;return;
 }
 $28 = $0;
 $29 = ($28|0)==(1);
 $30 = $4;
 $31 = ($30|0)==(0|0);
 $or$cond7 = $29 & $31;
 if ($or$cond7) {
  $32 = $12;
  HEAP32[$32>>2] = -5;
  STACKTOP = sp;return;
 }
 $33 = $0;
 $34 = ($33|0)==(1);
 $35 = $5;
 $36 = ($35|0)==(0|0);
 $or$cond9 = $34 & $36;
 if ($or$cond9) {
  $37 = $12;
  HEAP32[$37>>2] = -6;
  STACKTOP = sp;return;
 }
 $38 = $0;
 $39 = ($38|0)==(1);
 $40 = $6;
 $41 = ($40|0)==(0|0);
 $or$cond11 = $39 & $41;
 if ($or$cond11) {
  $42 = $12;
  HEAP32[$42>>2] = -7;
  STACKTOP = sp;return;
 }
 $43 = $0;
 $44 = ($43|0)==(1);
 if ($44) {
  $45 = $6;
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)!=(0);
  if ($47) {
   $48 = $12;
   HEAP32[$48>>2] = -7;
   STACKTOP = sp;return;
  }
 }
 $49 = $0;
 $50 = ($49|0)==(2);
 $51 = $7;
 $52 = ($51|0)==(0|0);
 $or$cond13 = $50 & $52;
 if ($or$cond13) {
  $53 = $12;
  HEAP32[$53>>2] = -8;
  STACKTOP = sp;return;
 }
 $54 = $0;
 $55 = ($54|0)==(2);
 $56 = $8;
 $57 = ($56|0)<=(0);
 $or$cond15 = $55 & $57;
 if ($or$cond15) {
  $58 = $12;
  HEAP32[$58>>2] = -9;
  STACKTOP = sp;return;
 }
 $59 = $0;
 $60 = ($59|0)==(2);
 $61 = $9;
 $62 = ($61|0)==(0|0);
 $or$cond17 = $60 & $62;
 if ($or$cond17) {
  $63 = $12;
  HEAP32[$63>>2] = -10;
  STACKTOP = sp;return;
 }
 $64 = $0;
 $65 = ($64|0)==(1);
 if (!($65)) {
  $133 = $0;
  $134 = ($133|0)==(2);
  if ($134) {
   _StatInit($stat);
   $135 = $9;
   $136 = HEAP32[$135>>2]|0;
   $LUfactors = $136;
   $137 = $LUfactors;
   $138 = HEAP32[$137>>2]|0;
   $L = $138;
   $139 = $LUfactors;
   $140 = ((($139)) + 4|0);
   $141 = HEAP32[$140>>2]|0;
   $U = $141;
   $142 = $LUfactors;
   $143 = ((($142)) + 8|0);
   $144 = HEAP32[$143>>2]|0;
   $perm_c = $144;
   $145 = $LUfactors;
   $146 = ((($145)) + 12|0);
   $147 = HEAP32[$146>>2]|0;
   $perm_r = $147;
   $148 = $1;
   $149 = $3;
   $150 = $7;
   $151 = $8;
   _dCreate_Dense_Matrix($B,$148,$149,$150,$151,6,1,0);
   $152 = $11;
   $153 = $L;
   $154 = $U;
   $155 = $perm_c;
   $156 = $perm_r;
   $157 = $12;
   _dgstrs($152,$153,$154,$155,$156,$B,$stat,$157);
   $158 = $12;
   $159 = HEAP32[$158>>2]|0;
   $160 = ($159|0)!=(0);
   if ($160) {
    $161 = $12;
    $162 = HEAP32[$161>>2]|0;
    HEAP32[$vararg_buffer28>>2] = $162;
    (_printf((2080|0),($vararg_buffer28|0))|0);
   }
   _Destroy_SuperMatrix_Store($B);
   _StatFree($stat);
   STACKTOP = sp;return;
  }
  $163 = $0;
  $164 = ($163|0)==(3);
  if ($164) {
   $165 = $9;
   $166 = HEAP32[$165>>2]|0;
   $LUfactors = $166;
   $167 = $LUfactors;
   $168 = ((($167)) + 12|0);
   $169 = HEAP32[$168>>2]|0;
   _superlu_free($169);
   $170 = $LUfactors;
   $171 = ((($170)) + 8|0);
   $172 = HEAP32[$171>>2]|0;
   _superlu_free($172);
   $173 = $LUfactors;
   $174 = HEAP32[$173>>2]|0;
   _Destroy_SuperNode_Matrix($174);
   $175 = $LUfactors;
   $176 = ((($175)) + 4|0);
   $177 = HEAP32[$176>>2]|0;
   _Destroy_CompCol_Matrix($177);
   $178 = $LUfactors;
   $179 = HEAP32[$178>>2]|0;
   _superlu_free($179);
   $180 = $LUfactors;
   $181 = ((($180)) + 4|0);
   $182 = HEAP32[$181>>2]|0;
   _superlu_free($182);
   $183 = $LUfactors;
   _superlu_free($183);
   STACKTOP = sp;return;
  } else {
   $184 = HEAP32[_stderr>>2]|0;
   $185 = $0;
   HEAP32[$vararg_buffer31>>2] = $185;
   (_fprintf(($184|0),(2120|0),($vararg_buffer31|0))|0);
   $186 = $12;
   HEAP32[$186>>2] = -1;
   STACKTOP = sp;return;
  }
 }
 _set_default_options($options);
 _StatInit($stat);
 $66 = $1;
 $67 = $1;
 $68 = $2;
 $69 = $4;
 $70 = $5;
 $71 = $6;
 _dCreate_CompCol_Matrix($A,$66,$67,$68,$69,$70,$71,0,1,0);
 $72 = (_superlu_malloc(24)|0);
 $L = $72;
 $73 = (_superlu_malloc(24)|0);
 $U = $73;
 $74 = $1;
 $75 = (_intMalloc($74)|0);
 $perm_r = $75;
 $76 = ($75|0)!=(0|0);
 if (!($76)) {
  $77 = HEAP32[_stderr>>2]|0;
  (_fprintf(($77|0),(3056|0),($vararg_buffer|0))|0);
  STACKTOP = sp;return;
 }
 $78 = $1;
 $79 = (_intMalloc($78)|0);
 $perm_c = $79;
 $80 = ($79|0)!=(0|0);
 if (!($80)) {
  $81 = HEAP32[_stderr>>2]|0;
  (_fprintf(($81|0),(3088|0),($vararg_buffer18|0))|0);
  STACKTOP = sp;return;
 }
 $82 = $1;
 $83 = (_intMalloc($82)|0);
 $etree = $83;
 $84 = ($83|0)!=(0|0);
 if (!($84)) {
  $85 = HEAP32[_stderr>>2]|0;
  (_fprintf(($85|0),(3120|0),($vararg_buffer20|0))|0);
  STACKTOP = sp;return;
 }
 $86 = $10;
 $87 = $perm_c;
 _get_perm_c($86,$A,$87);
 $88 = $perm_c;
 $89 = $etree;
 _sp_preorder($options,$A,$88,$89,$AC);
 $90 = (_sp_ienv(1)|0);
 $panel_size = $90;
 $91 = (_sp_ienv(2)|0);
 $relax = $91;
 $92 = $relax;
 $93 = $panel_size;
 $94 = $etree;
 $95 = $perm_c;
 $96 = $perm_r;
 $97 = $L;
 $98 = $U;
 $99 = $12;
 _dgstrf($options,$AC,$92,$93,$94,0,0,$95,$96,$97,$98,$stat,$99);
 $100 = $12;
 $101 = HEAP32[$100>>2]|0;
 $102 = ($101|0)==(0);
 if (!($102)) {
  $103 = $12;
  $104 = HEAP32[$103>>2]|0;
  HEAP32[$vararg_buffer22>>2] = $104;
  (_printf((2040|0),($vararg_buffer22|0))|0);
  $105 = $12;
  $106 = HEAP32[$105>>2]|0;
  $107 = $1;
  $108 = ($106|0)<=($107|0);
  if ($108) {
   $109 = $L;
   $110 = $U;
   (_dQuerySpace($109,$110,$mem_usage)|0);
   $111 = +HEAPF32[$mem_usage>>2];
   $112 = $111;
   $113 = $112 / 1.0E+6;
   $114 = ((($mem_usage)) + 4|0);
   $115 = +HEAPF32[$114>>2];
   $116 = $115;
   $117 = $116 / 1.0E+6;
   HEAPF64[$vararg_buffer24>>3] = $113;
   $vararg_ptr27 = ((($vararg_buffer24)) + 8|0);
   HEAPF64[$vararg_ptr27>>3] = $117;
   (_printf((3192|0),($vararg_buffer24|0))|0);
  }
 }
 $118 = (_superlu_malloc(16)|0);
 $LUfactors = $118;
 $119 = $L;
 $120 = $LUfactors;
 HEAP32[$120>>2] = $119;
 $121 = $U;
 $122 = $LUfactors;
 $123 = ((($122)) + 4|0);
 HEAP32[$123>>2] = $121;
 $124 = $perm_c;
 $125 = $LUfactors;
 $126 = ((($125)) + 8|0);
 HEAP32[$126>>2] = $124;
 $127 = $perm_r;
 $128 = $LUfactors;
 $129 = ((($128)) + 12|0);
 HEAP32[$129>>2] = $127;
 $130 = $LUfactors;
 $131 = $9;
 HEAP32[$131>>2] = $130;
 $132 = $etree;
 _superlu_free($132);
 _Destroy_SuperMatrix_Store($A);
 _Destroy_CompCol_Permuted($AC);
 _StatFree($stat);
 STACKTOP = sp;return;
}
function _z_div($c,$a,$b) {
 $c = $c|0;
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0.0, $25 = 0.0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0, $35 = 0.0, $36 = 0.0, $37 = 0.0, $38 = 0, $39 = 0, $4 = 0.0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0.0, $44 = 0;
 var $45 = 0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0.0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0.0, $60 = 0, $61 = 0.0, $62 = 0.0;
 var $63 = 0.0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0, $68 = 0.0, $69 = 0, $7 = 0.0, $70 = 0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0.0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0, $79 = 0.0, $8 = 0, $80 = 0;
 var $81 = 0.0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $abi = 0.0, $abr = 0.0, $ci = 0.0, $cr = 0.0, $den = 0.0, $ratio = 0.0, $vararg_buffer = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp + 48|0;
 $0 = $c;
 $1 = $a;
 $2 = $b;
 $3 = $2;
 $4 = +HEAPF64[$3>>3];
 $abr = $4;
 $5 = $4 < 0.0;
 if ($5) {
  $6 = $abr;
  $7 = -$6;
  $abr = $7;
 }
 $8 = $2;
 $9 = ((($8)) + 8|0);
 $10 = +HEAPF64[$9>>3];
 $abi = $10;
 $11 = $10 < 0.0;
 if ($11) {
  $12 = $abi;
  $13 = -$12;
  $abi = $13;
 }
 $14 = $abr;
 $15 = $abi;
 $16 = $14 <= $15;
 if (!($16)) {
  $54 = $2;
  $55 = ((($54)) + 8|0);
  $56 = +HEAPF64[$55>>3];
  $57 = $2;
  $58 = +HEAPF64[$57>>3];
  $59 = $56 / $58;
  $ratio = $59;
  $60 = $2;
  $61 = +HEAPF64[$60>>3];
  $62 = $ratio;
  $63 = $ratio;
  $64 = $62 * $63;
  $65 = 1.0 + $64;
  $66 = $61 * $65;
  $den = $66;
  $67 = $1;
  $68 = +HEAPF64[$67>>3];
  $69 = $1;
  $70 = ((($69)) + 8|0);
  $71 = +HEAPF64[$70>>3];
  $72 = $ratio;
  $73 = $71 * $72;
  $74 = $68 + $73;
  $75 = $den;
  $76 = $74 / $75;
  $cr = $76;
  $77 = $1;
  $78 = ((($77)) + 8|0);
  $79 = +HEAPF64[$78>>3];
  $80 = $1;
  $81 = +HEAPF64[$80>>3];
  $82 = $ratio;
  $83 = $81 * $82;
  $84 = $79 - $83;
  $85 = $den;
  $86 = $84 / $85;
  $ci = $86;
  $87 = $cr;
  $88 = $0;
  HEAPF64[$88>>3] = $87;
  $89 = $ci;
  $90 = $0;
  $91 = ((($90)) + 8|0);
  HEAPF64[$91>>3] = $89;
  STACKTOP = sp;return;
 }
 $17 = $abi;
 $18 = $17 == 0.0;
 if ($18) {
  $19 = HEAP32[_stderr>>2]|0;
  (_fprintf(($19|0),(2160|0),($vararg_buffer|0))|0);
  _exit(-1);
  // unreachable;
 }
 $20 = $2;
 $21 = +HEAPF64[$20>>3];
 $22 = $2;
 $23 = ((($22)) + 8|0);
 $24 = +HEAPF64[$23>>3];
 $25 = $21 / $24;
 $ratio = $25;
 $26 = $2;
 $27 = ((($26)) + 8|0);
 $28 = +HEAPF64[$27>>3];
 $29 = $ratio;
 $30 = $ratio;
 $31 = $29 * $30;
 $32 = 1.0 + $31;
 $33 = $28 * $32;
 $den = $33;
 $34 = $1;
 $35 = +HEAPF64[$34>>3];
 $36 = $ratio;
 $37 = $35 * $36;
 $38 = $1;
 $39 = ((($38)) + 8|0);
 $40 = +HEAPF64[$39>>3];
 $41 = $37 + $40;
 $42 = $den;
 $43 = $41 / $42;
 $cr = $43;
 $44 = $1;
 $45 = ((($44)) + 8|0);
 $46 = +HEAPF64[$45>>3];
 $47 = $ratio;
 $48 = $46 * $47;
 $49 = $1;
 $50 = +HEAPF64[$49>>3];
 $51 = $48 - $50;
 $52 = $den;
 $53 = $51 / $52;
 $ci = $53;
 $87 = $cr;
 $88 = $0;
 HEAPF64[$88>>3] = $87;
 $89 = $ci;
 $90 = $0;
 $91 = ((($90)) + 8|0);
 HEAPF64[$91>>3] = $89;
 STACKTOP = sp;return;
}
function _z_abs1($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $2 = 0.0, $3 = 0, $4 = 0, $5 = 0.0, $6 = 0.0, $7 = 0, $8 = 0.0, $9 = 0.0, $imag = 0.0, $real = 0.0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $z;
 $1 = $0;
 $2 = +HEAPF64[$1>>3];
 $real = $2;
 $3 = $0;
 $4 = ((($3)) + 8|0);
 $5 = +HEAPF64[$4>>3];
 $imag = $5;
 $6 = $real;
 $7 = $6 < 0.0;
 if ($7) {
  $8 = $real;
  $9 = -$8;
  $real = $9;
 }
 $10 = $imag;
 $11 = $10 < 0.0;
 if ($11) {
  $12 = $imag;
  $13 = -$12;
  $imag = $13;
 }
 $14 = $real;
 $15 = $imag;
 $16 = $14 + $15;
 STACKTOP = sp;return (+$16);
}
function _d_cnjg($r,$z) {
 $r = $r|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0.0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0, $8 = 0.0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $r;
 $1 = $z;
 $2 = $1;
 $3 = +HEAPF64[$2>>3];
 $4 = $0;
 HEAPF64[$4>>3] = $3;
 $5 = $1;
 $6 = ((($5)) + 8|0);
 $7 = +HEAPF64[$6>>3];
 $8 = -$7;
 $9 = $0;
 $10 = ((($9)) + 8|0);
 HEAPF64[$10>>3] = $8;
 STACKTOP = sp;return;
}
function _sp_ztrsv($uplo,$trans,$diag,$L,$U,$x,$stat,$info) {
 $uplo = $uplo|0;
 $trans = $trans|0;
 $diag = $diag|0;
 $L = $L|0;
 $U = $U|0;
 $x = $x|0;
 $stat = $stat|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0.0, $1027 = 0.0, $1028 = 0.0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0.0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0.0, $1074 = 0.0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0.0, $108 = 0, $1080 = 0.0, $1081 = 0.0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0.0, $1087 = 0;
 var $1088 = 0.0, $1089 = 0.0, $109 = 0, $1090 = 0.0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0.0, $1096 = 0.0, $1097 = 0.0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0.0, $1102 = 0, $1103 = 0.0, $1104 = 0.0;
 var $1105 = 0.0, $1106 = 0.0, $1107 = 0.0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0.0, $1113 = 0.0, $1114 = 0.0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0.0;
 var $1123 = 0, $1124 = 0.0, $1125 = 0.0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0.0, $1144 = 0.0, $1145 = 0.0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0.0, $1225 = 0.0, $1226 = 0.0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0.0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0.0, $1257 = 0.0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0.0, $1263 = 0.0, $1264 = 0.0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0.0, $127 = 0, $1270 = 0, $1271 = 0.0, $1272 = 0.0, $1273 = 0.0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0.0, $1279 = 0.0, $128 = 0, $1280 = 0.0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0.0, $1285 = 0;
 var $1286 = 0.0, $1287 = 0.0, $1288 = 0.0, $1289 = 0.0, $129 = 0, $1290 = 0.0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0.0, $1296 = 0.0, $1297 = 0.0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0.0, $1306 = 0, $1307 = 0.0, $1308 = 0.0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0.0, $1326 = 0.0, $1327 = 0.0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0.0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0.0, $1339 = 0.0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0.0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0.0, $1362 = 0.0, $1363 = 0, $1364 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0.0, $144 = 0.0, $145 = 0.0, $146 = 0, $147 = 0;
 var $148 = 0, $149 = 0, $15 = 0, $150 = 0.0, $151 = 0.0, $152 = 0.0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0;
 var $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0.0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0.0, $182 = 0.0, $183 = 0;
 var $184 = 0, $185 = 0, $186 = 0, $187 = 0.0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0.0, $193 = 0.0, $194 = 0.0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0;
 var $201 = 0, $202 = 0, $203 = 0.0, $204 = 0.0, $205 = 0, $206 = 0, $207 = 0, $208 = 0.0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0.0, $214 = 0.0, $215 = 0.0, $216 = 0.0, $217 = 0.0, $218 = 0, $219 = 0;
 var $22 = 0, $220 = 0, $221 = 0, $222 = 0.0, $223 = 0.0, $224 = 0.0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0.0, $233 = 0, $234 = 0.0, $235 = 0.0, $236 = 0, $237 = 0;
 var $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0;
 var $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0;
 var $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0.0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0.0, $284 = 0.0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0;
 var $292 = 0.0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0.0, $298 = 0.0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0;
 var $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0;
 var $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0;
 var $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0;
 var $364 = 0.0, $365 = 0.0, $366 = 0.0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0;
 var $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0;
 var $40 = 0, $400 = 0, $401 = 0, $402 = 0.0, $403 = 0, $404 = 0, $405 = 0, $406 = 0.0, $407 = 0.0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0.0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0.0;
 var $418 = 0.0, $419 = 0.0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0.0, $425 = 0, $426 = 0, $427 = 0, $428 = 0.0, $429 = 0.0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0.0, $434 = 0, $435 = 0;
 var $436 = 0, $437 = 0, $438 = 0.0, $439 = 0.0, $44 = 0, $440 = 0.0, $441 = 0.0, $442 = 0.0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0.0, $448 = 0.0, $449 = 0.0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0;
 var $454 = 0, $455 = 0, $456 = 0, $457 = 0.0, $458 = 0, $459 = 0.0, $46 = 0, $460 = 0.0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0;
 var $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0;
 var $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0.0, $501 = 0.0, $502 = 0.0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0;
 var $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0;
 var $526 = 0, $527 = 0.0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0.0, $532 = 0.0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0.0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0.0, $543 = 0.0;
 var $544 = 0.0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0.0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0.0, $554 = 0.0, $555 = 0, $556 = 0, $557 = 0, $558 = 0.0, $559 = 0, $56 = 0, $560 = 0, $561 = 0;
 var $562 = 0, $563 = 0.0, $564 = 0.0, $565 = 0.0, $566 = 0.0, $567 = 0.0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0.0, $573 = 0.0, $574 = 0.0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0;
 var $580 = 0, $581 = 0, $582 = 0.0, $583 = 0, $584 = 0.0, $585 = 0.0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0;
 var $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0;
 var $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0;
 var $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0;
 var $652 = 0, $653 = 0.0, $654 = 0.0, $655 = 0.0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0;
 var $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0;
 var $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0.0, $696 = 0, $697 = 0, $698 = 0, $699 = 0.0, $7 = 0, $70 = 0, $700 = 0.0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0.0;
 var $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0.0, $711 = 0.0, $712 = 0.0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0.0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0.0, $722 = 0.0, $723 = 0;
 var $724 = 0, $725 = 0, $726 = 0.0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0.0, $732 = 0.0, $733 = 0.0, $734 = 0.0, $735 = 0.0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0.0, $741 = 0.0;
 var $742 = 0.0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0.0, $751 = 0, $752 = 0.0, $753 = 0.0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0;
 var $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0.0, $772 = 0.0, $773 = 0.0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0;
 var $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0;
 var $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0;
 var $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0;
 var $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0;
 var $850 = 0, $851 = 0.0, $852 = 0.0, $853 = 0.0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0;
 var $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0.0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0.0, $883 = 0.0, $884 = 0, $885 = 0, $886 = 0;
 var $887 = 0, $888 = 0.0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0.0, $894 = 0.0, $895 = 0.0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0.0, $901 = 0, $902 = 0, $903 = 0;
 var $904 = 0.0, $905 = 0.0, $906 = 0, $907 = 0, $908 = 0, $909 = 0.0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0.0, $915 = 0.0, $916 = 0.0, $917 = 0.0, $918 = 0.0, $919 = 0, $92 = 0, $920 = 0, $921 = 0;
 var $922 = 0, $923 = 0.0, $924 = 0.0, $925 = 0.0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0.0, $934 = 0, $935 = 0.0, $936 = 0.0, $937 = 0, $938 = 0, $939 = 0, $94 = 0;
 var $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0.0, $954 = 0.0, $955 = 0.0, $956 = 0, $957 = 0, $958 = 0;
 var $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0;
 var $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0;
 var $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $Lstore = 0, $Lval = 0, $Ustore = 0, $Uval = 0, $alpha = 0, $beta = 0, $ci = 0.0, $ci10 = 0.0, $ci12 = 0.0, $ci2 = 0.0, $ci4 = 0.0, $ci6 = 0.0, $ci8 = 0.0, $comp_zero = 0, $cr = 0.0;
 var $cr1 = 0.0, $cr11 = 0.0, $cr3 = 0.0, $cr5 = 0.0, $cr7 = 0.0, $cr9 = 0.0, $fsupc = 0, $i = 0, $incx = 0, $incy = 0, $iptr = 0, $irow = 0, $istart = 0, $jcol = 0, $k = 0, $luptr = 0, $msg = 0, $nrow = 0, $nsupc = 0, $nsupr = 0;
 var $solve_ops = 0.0, $temp = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $work = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 560|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp + 176|0;
 $incx = sp + 244|0;
 $temp = sp + 160|0;
 $alpha = sp + 144|0;
 $beta = sp + 128|0;
 $comp_zero = sp + 112|0;
 $nsupr = sp + 228|0;
 $nsupc = sp + 224|0;
 $i = sp + 208|0;
 $msg = sp + 304|0;
 $1 = $uplo;
 $2 = $trans;
 $3 = $diag;
 $4 = $L;
 $5 = $U;
 $6 = $x;
 $7 = $stat;
 $8 = $info;
 HEAP32[$incx>>2] = 1;
 $incy = 1;
 ;HEAP32[$alpha>>2]=HEAP32[2504>>2]|0;HEAP32[$alpha+4>>2]=HEAP32[2504+4>>2]|0;HEAP32[$alpha+8>>2]=HEAP32[2504+8>>2]|0;HEAP32[$alpha+12>>2]=HEAP32[2504+12>>2]|0;
 ;HEAP32[$beta>>2]=HEAP32[2504>>2]|0;HEAP32[$beta+4>>2]=HEAP32[2504+4>>2]|0;HEAP32[$beta+8>>2]=HEAP32[2504+8>>2]|0;HEAP32[$beta+12>>2]=HEAP32[2504+12>>2]|0;
 ;HEAP32[$comp_zero>>2]=0|0;HEAP32[$comp_zero+4>>2]=0|0;HEAP32[$comp_zero+8>>2]=0|0;HEAP32[$comp_zero+12>>2]=0|0;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 $10 = $1;
 $11 = (_lsame_($10,3400)|0);
 $12 = ($11|0)!=(0);
 if ($12) {
  label = 4;
 } else {
  $13 = $1;
  $14 = (_lsame_($13,3392)|0);
  $15 = ($14|0)!=(0);
  if ($15) {
   label = 4;
  } else {
   $16 = $8;
   HEAP32[$16>>2] = -1;
  }
 }
 do {
  if ((label|0) == 4) {
   $17 = $2;
   $18 = (_lsame_($17,3408)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = $2;
    $21 = (_lsame_($20,3416)|0);
    $22 = ($21|0)!=(0);
    if (!($22)) {
     $23 = $2;
     $24 = (_lsame_($23,3424)|0);
     $25 = ($24|0)!=(0);
     if (!($25)) {
      $26 = $8;
      HEAP32[$26>>2] = -2;
      break;
     }
    }
   }
   $27 = $3;
   $28 = (_lsame_($27,3392)|0);
   $29 = ($28|0)!=(0);
   if (!($29)) {
    $30 = $3;
    $31 = (_lsame_($30,3408)|0);
    $32 = ($31|0)!=(0);
    if (!($32)) {
     $33 = $8;
     HEAP32[$33>>2] = -3;
     break;
    }
   }
   $34 = $4;
   $35 = ((($34)) + 12|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = $4;
   $38 = ((($37)) + 16|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($36|0)!=($39|0);
   if (!($40)) {
    $41 = $4;
    $42 = ((($41)) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)<(0);
    if (!($44)) {
     $46 = $5;
     $47 = ((($46)) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = $5;
     $50 = ((($49)) + 16|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = ($48|0)!=($51|0);
     if (!($52)) {
      $53 = $5;
      $54 = ((($53)) + 12|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = ($55|0)<(0);
      if (!($56)) {
       break;
      }
     }
     $57 = $8;
     HEAP32[$57>>2] = -5;
     break;
    }
   }
   $45 = $8;
   HEAP32[$45>>2] = -4;
  }
 } while(0);
 $58 = $8;
 $59 = HEAP32[$58>>2]|0;
 $60 = ($59|0)!=(0);
 if ($60) {
  $61 = $8;
  $62 = HEAP32[$61>>2]|0;
  $63 = (0 - ($62))|0;
  HEAP32[$i>>2] = $63;
  (_xerbla_(2192,$i)|0);
  $0 = 0;
  $1364 = $0;
  STACKTOP = sp;return ($1364|0);
 }
 $64 = $4;
 $65 = ((($64)) + 20|0);
 $66 = HEAP32[$65>>2]|0;
 $Lstore = $66;
 $67 = $Lstore;
 $68 = ((($67)) + 8|0);
 $69 = HEAP32[$68>>2]|0;
 $Lval = $69;
 $70 = $5;
 $71 = ((($70)) + 20|0);
 $72 = HEAP32[$71>>2]|0;
 $Ustore = $72;
 $73 = $Ustore;
 $74 = ((($73)) + 4|0);
 $75 = HEAP32[$74>>2]|0;
 $Uval = $75;
 $solve_ops = 0.0;
 $76 = $4;
 $77 = ((($76)) + 12|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = (_doublecomplexCalloc($78)|0);
 $work = $79;
 $80 = ($79|0)!=(0|0);
 if (!($80)) {
  HEAP32[$vararg_buffer>>2] = 2208;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 125;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2248;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $81 = $2;
 $82 = (_lsame_($81,3408)|0);
 $83 = ($82|0)!=(0);
 L30: do {
  if ($83) {
   $84 = $1;
   $85 = (_lsame_($84,3400)|0);
   $86 = ($85|0)!=(0);
   if ($86) {
    $87 = $4;
    $88 = ((($87)) + 12|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = ($89|0)==(0);
    if ($90) {
     $0 = 0;
     $1364 = $0;
     STACKTOP = sp;return ($1364|0);
    }
    $k = 0;
    while(1) {
     $91 = $k;
     $92 = $Lstore;
     $93 = ((($92)) + 4|0);
     $94 = HEAP32[$93>>2]|0;
     $95 = ($91|0)<=($94|0);
     if (!($95)) {
      break L30;
     }
     $96 = $k;
     $97 = $Lstore;
     $98 = ((($97)) + 28|0);
     $99 = HEAP32[$98>>2]|0;
     $100 = (($99) + ($96<<2)|0);
     $101 = HEAP32[$100>>2]|0;
     $fsupc = $101;
     $102 = $fsupc;
     $103 = $Lstore;
     $104 = ((($103)) + 20|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (($105) + ($102<<2)|0);
     $107 = HEAP32[$106>>2]|0;
     $istart = $107;
     $108 = $fsupc;
     $109 = (($108) + 1)|0;
     $110 = $Lstore;
     $111 = ((($110)) + 20|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = (($112) + ($109<<2)|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = $istart;
     $116 = (($114) - ($115))|0;
     HEAP32[$nsupr>>2] = $116;
     $117 = $k;
     $118 = (($117) + 1)|0;
     $119 = $Lstore;
     $120 = ((($119)) + 28|0);
     $121 = HEAP32[$120>>2]|0;
     $122 = (($121) + ($118<<2)|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = $fsupc;
     $125 = (($123) - ($124))|0;
     HEAP32[$nsupc>>2] = $125;
     $126 = $fsupc;
     $127 = $Lstore;
     $128 = ((($127)) + 12|0);
     $129 = HEAP32[$128>>2]|0;
     $130 = (($129) + ($126<<2)|0);
     $131 = HEAP32[$130>>2]|0;
     $luptr = $131;
     $132 = HEAP32[$nsupr>>2]|0;
     $133 = HEAP32[$nsupc>>2]|0;
     $134 = (($132) - ($133))|0;
     $nrow = $134;
     $135 = HEAP32[$nsupc>>2]|0;
     $136 = $135<<2;
     $137 = HEAP32[$nsupc>>2]|0;
     $138 = (($137) - 1)|0;
     $139 = Math_imul($136, $138)|0;
     $140 = HEAP32[$nsupc>>2]|0;
     $141 = ($140*10)|0;
     $142 = (($139) + ($141))|0;
     $143 = (+($142|0));
     $144 = $solve_ops;
     $145 = $144 + $143;
     $solve_ops = $145;
     $146 = $nrow;
     $147 = $146<<3;
     $148 = HEAP32[$nsupc>>2]|0;
     $149 = Math_imul($147, $148)|0;
     $150 = (+($149|0));
     $151 = $solve_ops;
     $152 = $151 + $150;
     $solve_ops = $152;
     $153 = HEAP32[$nsupc>>2]|0;
     $154 = ($153|0)==(1);
     L41: do {
      if ($154) {
       $155 = $istart;
       $156 = (($155) + 1)|0;
       $iptr = $156;
       while(1) {
        $157 = $iptr;
        $158 = $fsupc;
        $159 = (($158) + 1)|0;
        $160 = $Lstore;
        $161 = ((($160)) + 20|0);
        $162 = HEAP32[$161>>2]|0;
        $163 = (($162) + ($159<<2)|0);
        $164 = HEAP32[$163>>2]|0;
        $165 = ($157|0)<($164|0);
        if (!($165)) {
         break L41;
        }
        $166 = $iptr;
        $167 = $Lstore;
        $168 = ((($167)) + 16|0);
        $169 = HEAP32[$168>>2]|0;
        $170 = (($169) + ($166<<2)|0);
        $171 = HEAP32[$170>>2]|0;
        $irow = $171;
        $172 = $luptr;
        $173 = (($172) + 1)|0;
        $luptr = $173;
        $174 = $fsupc;
        $175 = $6;
        $176 = (($175) + ($174<<4)|0);
        $177 = +HEAPF64[$176>>3];
        $178 = $luptr;
        $179 = $Lval;
        $180 = (($179) + ($178<<4)|0);
        $181 = +HEAPF64[$180>>3];
        $182 = $177 * $181;
        $183 = $fsupc;
        $184 = $6;
        $185 = (($184) + ($183<<4)|0);
        $186 = ((($185)) + 8|0);
        $187 = +HEAPF64[$186>>3];
        $188 = $luptr;
        $189 = $Lval;
        $190 = (($189) + ($188<<4)|0);
        $191 = ((($190)) + 8|0);
        $192 = +HEAPF64[$191>>3];
        $193 = $187 * $192;
        $194 = $182 - $193;
        $cr = $194;
        $195 = $fsupc;
        $196 = $6;
        $197 = (($196) + ($195<<4)|0);
        $198 = ((($197)) + 8|0);
        $199 = +HEAPF64[$198>>3];
        $200 = $luptr;
        $201 = $Lval;
        $202 = (($201) + ($200<<4)|0);
        $203 = +HEAPF64[$202>>3];
        $204 = $199 * $203;
        $205 = $fsupc;
        $206 = $6;
        $207 = (($206) + ($205<<4)|0);
        $208 = +HEAPF64[$207>>3];
        $209 = $luptr;
        $210 = $Lval;
        $211 = (($210) + ($209<<4)|0);
        $212 = ((($211)) + 8|0);
        $213 = +HEAPF64[$212>>3];
        $214 = $208 * $213;
        $215 = $204 + $214;
        $ci = $215;
        $216 = $cr;
        HEAPF64[$comp_zero>>3] = $216;
        $217 = $ci;
        $218 = ((($comp_zero)) + 8|0);
        HEAPF64[$218>>3] = $217;
        $219 = $irow;
        $220 = $6;
        $221 = (($220) + ($219<<4)|0);
        $222 = +HEAPF64[$221>>3];
        $223 = +HEAPF64[$comp_zero>>3];
        $224 = $222 - $223;
        $225 = $irow;
        $226 = $6;
        $227 = (($226) + ($225<<4)|0);
        HEAPF64[$227>>3] = $224;
        $228 = $irow;
        $229 = $6;
        $230 = (($229) + ($228<<4)|0);
        $231 = ((($230)) + 8|0);
        $232 = +HEAPF64[$231>>3];
        $233 = ((($comp_zero)) + 8|0);
        $234 = +HEAPF64[$233>>3];
        $235 = $232 - $234;
        $236 = $irow;
        $237 = $6;
        $238 = (($237) + ($236<<4)|0);
        $239 = ((($238)) + 8|0);
        HEAPF64[$239>>3] = $235;
        $240 = $iptr;
        $241 = (($240) + 1)|0;
        $iptr = $241;
       }
      } else {
       $242 = HEAP32[$nsupr>>2]|0;
       $243 = HEAP32[$nsupc>>2]|0;
       $244 = $luptr;
       $245 = $Lval;
       $246 = (($245) + ($244<<4)|0);
       $247 = $fsupc;
       $248 = $6;
       $249 = (($248) + ($247<<4)|0);
       _zlsolve($242,$243,$246,$249);
       $250 = HEAP32[$nsupr>>2]|0;
       $251 = HEAP32[$nsupr>>2]|0;
       $252 = HEAP32[$nsupc>>2]|0;
       $253 = (($251) - ($252))|0;
       $254 = HEAP32[$nsupc>>2]|0;
       $255 = $luptr;
       $256 = HEAP32[$nsupc>>2]|0;
       $257 = (($255) + ($256))|0;
       $258 = $Lval;
       $259 = (($258) + ($257<<4)|0);
       $260 = $fsupc;
       $261 = $6;
       $262 = (($261) + ($260<<4)|0);
       $263 = $work;
       _zmatvec($250,$253,$254,$259,$262,$263);
       $264 = $istart;
       $265 = HEAP32[$nsupc>>2]|0;
       $266 = (($264) + ($265))|0;
       $iptr = $266;
       HEAP32[$i>>2] = 0;
       while(1) {
        $267 = HEAP32[$i>>2]|0;
        $268 = $nrow;
        $269 = ($267|0)<($268|0);
        if (!($269)) {
         break L41;
        }
        $270 = $iptr;
        $271 = $Lstore;
        $272 = ((($271)) + 16|0);
        $273 = HEAP32[$272>>2]|0;
        $274 = (($273) + ($270<<2)|0);
        $275 = HEAP32[$274>>2]|0;
        $irow = $275;
        $276 = $irow;
        $277 = $6;
        $278 = (($277) + ($276<<4)|0);
        $279 = +HEAPF64[$278>>3];
        $280 = HEAP32[$i>>2]|0;
        $281 = $work;
        $282 = (($281) + ($280<<4)|0);
        $283 = +HEAPF64[$282>>3];
        $284 = $279 - $283;
        $285 = $irow;
        $286 = $6;
        $287 = (($286) + ($285<<4)|0);
        HEAPF64[$287>>3] = $284;
        $288 = $irow;
        $289 = $6;
        $290 = (($289) + ($288<<4)|0);
        $291 = ((($290)) + 8|0);
        $292 = +HEAPF64[$291>>3];
        $293 = HEAP32[$i>>2]|0;
        $294 = $work;
        $295 = (($294) + ($293<<4)|0);
        $296 = ((($295)) + 8|0);
        $297 = +HEAPF64[$296>>3];
        $298 = $292 - $297;
        $299 = $irow;
        $300 = $6;
        $301 = (($300) + ($299<<4)|0);
        $302 = ((($301)) + 8|0);
        HEAPF64[$302>>3] = $298;
        $303 = HEAP32[$i>>2]|0;
        $304 = $work;
        $305 = (($304) + ($303<<4)|0);
        ;HEAP32[$305>>2]=HEAP32[$comp_zero>>2]|0;HEAP32[$305+4>>2]=HEAP32[$comp_zero+4>>2]|0;HEAP32[$305+8>>2]=HEAP32[$comp_zero+8>>2]|0;HEAP32[$305+12>>2]=HEAP32[$comp_zero+12>>2]|0;
        $306 = HEAP32[$i>>2]|0;
        $307 = (($306) + 1)|0;
        HEAP32[$i>>2] = $307;
        $308 = $iptr;
        $309 = (($308) + 1)|0;
        $iptr = $309;
       }
      }
     } while(0);
     $310 = $k;
     $311 = (($310) + 1)|0;
     $k = $311;
    }
   }
   $312 = $5;
   $313 = ((($312)) + 12|0);
   $314 = HEAP32[$313>>2]|0;
   $315 = ($314|0)==(0);
   if ($315) {
    $0 = 0;
    $1364 = $0;
    STACKTOP = sp;return ($1364|0);
   }
   $316 = $Lstore;
   $317 = ((($316)) + 4|0);
   $318 = HEAP32[$317>>2]|0;
   $k = $318;
   while(1) {
    $319 = $k;
    $320 = ($319|0)>=(0);
    if (!($320)) {
     break L30;
    }
    $321 = $k;
    $322 = $Lstore;
    $323 = ((($322)) + 28|0);
    $324 = HEAP32[$323>>2]|0;
    $325 = (($324) + ($321<<2)|0);
    $326 = HEAP32[$325>>2]|0;
    $fsupc = $326;
    $327 = $fsupc;
    $328 = (($327) + 1)|0;
    $329 = $Lstore;
    $330 = ((($329)) + 20|0);
    $331 = HEAP32[$330>>2]|0;
    $332 = (($331) + ($328<<2)|0);
    $333 = HEAP32[$332>>2]|0;
    $334 = $fsupc;
    $335 = $Lstore;
    $336 = ((($335)) + 20|0);
    $337 = HEAP32[$336>>2]|0;
    $338 = (($337) + ($334<<2)|0);
    $339 = HEAP32[$338>>2]|0;
    $340 = (($333) - ($339))|0;
    HEAP32[$nsupr>>2] = $340;
    $341 = $k;
    $342 = (($341) + 1)|0;
    $343 = $Lstore;
    $344 = ((($343)) + 28|0);
    $345 = HEAP32[$344>>2]|0;
    $346 = (($345) + ($342<<2)|0);
    $347 = HEAP32[$346>>2]|0;
    $348 = $fsupc;
    $349 = (($347) - ($348))|0;
    HEAP32[$nsupc>>2] = $349;
    $350 = $fsupc;
    $351 = $Lstore;
    $352 = ((($351)) + 12|0);
    $353 = HEAP32[$352>>2]|0;
    $354 = (($353) + ($350<<2)|0);
    $355 = HEAP32[$354>>2]|0;
    $luptr = $355;
    $356 = HEAP32[$nsupc>>2]|0;
    $357 = $356<<2;
    $358 = HEAP32[$nsupc>>2]|0;
    $359 = (($358) + 1)|0;
    $360 = Math_imul($357, $359)|0;
    $361 = HEAP32[$nsupc>>2]|0;
    $362 = ($361*10)|0;
    $363 = (($360) + ($362))|0;
    $364 = (+($363|0));
    $365 = $solve_ops;
    $366 = $365 + $364;
    $solve_ops = $366;
    $367 = HEAP32[$nsupc>>2]|0;
    $368 = ($367|0)==(1);
    L59: do {
     if ($368) {
      $369 = $fsupc;
      $370 = $6;
      $371 = (($370) + ($369<<4)|0);
      $372 = $fsupc;
      $373 = $6;
      $374 = (($373) + ($372<<4)|0);
      $375 = $luptr;
      $376 = $Lval;
      $377 = (($376) + ($375<<4)|0);
      _z_div($371,$374,$377);
      $378 = $fsupc;
      $379 = $Ustore;
      $380 = ((($379)) + 12|0);
      $381 = HEAP32[$380>>2]|0;
      $382 = (($381) + ($378<<2)|0);
      $383 = HEAP32[$382>>2]|0;
      HEAP32[$i>>2] = $383;
      while(1) {
       $384 = HEAP32[$i>>2]|0;
       $385 = $fsupc;
       $386 = (($385) + 1)|0;
       $387 = $Ustore;
       $388 = ((($387)) + 12|0);
       $389 = HEAP32[$388>>2]|0;
       $390 = (($389) + ($386<<2)|0);
       $391 = HEAP32[$390>>2]|0;
       $392 = ($384|0)<($391|0);
       if (!($392)) {
        break L59;
       }
       $393 = HEAP32[$i>>2]|0;
       $394 = $Ustore;
       $395 = ((($394)) + 8|0);
       $396 = HEAP32[$395>>2]|0;
       $397 = (($396) + ($393<<2)|0);
       $398 = HEAP32[$397>>2]|0;
       $irow = $398;
       $399 = $fsupc;
       $400 = $6;
       $401 = (($400) + ($399<<4)|0);
       $402 = +HEAPF64[$401>>3];
       $403 = HEAP32[$i>>2]|0;
       $404 = $Uval;
       $405 = (($404) + ($403<<4)|0);
       $406 = +HEAPF64[$405>>3];
       $407 = $402 * $406;
       $408 = $fsupc;
       $409 = $6;
       $410 = (($409) + ($408<<4)|0);
       $411 = ((($410)) + 8|0);
       $412 = +HEAPF64[$411>>3];
       $413 = HEAP32[$i>>2]|0;
       $414 = $Uval;
       $415 = (($414) + ($413<<4)|0);
       $416 = ((($415)) + 8|0);
       $417 = +HEAPF64[$416>>3];
       $418 = $412 * $417;
       $419 = $407 - $418;
       $cr1 = $419;
       $420 = $fsupc;
       $421 = $6;
       $422 = (($421) + ($420<<4)|0);
       $423 = ((($422)) + 8|0);
       $424 = +HEAPF64[$423>>3];
       $425 = HEAP32[$i>>2]|0;
       $426 = $Uval;
       $427 = (($426) + ($425<<4)|0);
       $428 = +HEAPF64[$427>>3];
       $429 = $424 * $428;
       $430 = $fsupc;
       $431 = $6;
       $432 = (($431) + ($430<<4)|0);
       $433 = +HEAPF64[$432>>3];
       $434 = HEAP32[$i>>2]|0;
       $435 = $Uval;
       $436 = (($435) + ($434<<4)|0);
       $437 = ((($436)) + 8|0);
       $438 = +HEAPF64[$437>>3];
       $439 = $433 * $438;
       $440 = $429 + $439;
       $ci2 = $440;
       $441 = $cr1;
       HEAPF64[$comp_zero>>3] = $441;
       $442 = $ci2;
       $443 = ((($comp_zero)) + 8|0);
       HEAPF64[$443>>3] = $442;
       $444 = $irow;
       $445 = $6;
       $446 = (($445) + ($444<<4)|0);
       $447 = +HEAPF64[$446>>3];
       $448 = +HEAPF64[$comp_zero>>3];
       $449 = $447 - $448;
       $450 = $irow;
       $451 = $6;
       $452 = (($451) + ($450<<4)|0);
       HEAPF64[$452>>3] = $449;
       $453 = $irow;
       $454 = $6;
       $455 = (($454) + ($453<<4)|0);
       $456 = ((($455)) + 8|0);
       $457 = +HEAPF64[$456>>3];
       $458 = ((($comp_zero)) + 8|0);
       $459 = +HEAPF64[$458>>3];
       $460 = $457 - $459;
       $461 = $irow;
       $462 = $6;
       $463 = (($462) + ($461<<4)|0);
       $464 = ((($463)) + 8|0);
       HEAPF64[$464>>3] = $460;
       $465 = HEAP32[$i>>2]|0;
       $466 = (($465) + 1)|0;
       HEAP32[$i>>2] = $466;
      }
     } else {
      $467 = HEAP32[$nsupr>>2]|0;
      $468 = HEAP32[$nsupc>>2]|0;
      $469 = $luptr;
      $470 = $Lval;
      $471 = (($470) + ($469<<4)|0);
      $472 = $fsupc;
      $473 = $6;
      $474 = (($473) + ($472<<4)|0);
      _zusolve($467,$468,$471,$474);
      $475 = $fsupc;
      $jcol = $475;
      while(1) {
       $476 = $jcol;
       $477 = $k;
       $478 = (($477) + 1)|0;
       $479 = $Lstore;
       $480 = ((($479)) + 28|0);
       $481 = HEAP32[$480>>2]|0;
       $482 = (($481) + ($478<<2)|0);
       $483 = HEAP32[$482>>2]|0;
       $484 = ($476|0)<($483|0);
       if (!($484)) {
        break L59;
       }
       $485 = $jcol;
       $486 = (($485) + 1)|0;
       $487 = $Ustore;
       $488 = ((($487)) + 12|0);
       $489 = HEAP32[$488>>2]|0;
       $490 = (($489) + ($486<<2)|0);
       $491 = HEAP32[$490>>2]|0;
       $492 = $jcol;
       $493 = $Ustore;
       $494 = ((($493)) + 12|0);
       $495 = HEAP32[$494>>2]|0;
       $496 = (($495) + ($492<<2)|0);
       $497 = HEAP32[$496>>2]|0;
       $498 = (($491) - ($497))|0;
       $499 = $498<<3;
       $500 = (+($499|0));
       $501 = $solve_ops;
       $502 = $501 + $500;
       $solve_ops = $502;
       $503 = $jcol;
       $504 = $Ustore;
       $505 = ((($504)) + 12|0);
       $506 = HEAP32[$505>>2]|0;
       $507 = (($506) + ($503<<2)|0);
       $508 = HEAP32[$507>>2]|0;
       HEAP32[$i>>2] = $508;
       while(1) {
        $509 = HEAP32[$i>>2]|0;
        $510 = $jcol;
        $511 = (($510) + 1)|0;
        $512 = $Ustore;
        $513 = ((($512)) + 12|0);
        $514 = HEAP32[$513>>2]|0;
        $515 = (($514) + ($511<<2)|0);
        $516 = HEAP32[$515>>2]|0;
        $517 = ($509|0)<($516|0);
        if (!($517)) {
         break;
        }
        $518 = HEAP32[$i>>2]|0;
        $519 = $Ustore;
        $520 = ((($519)) + 8|0);
        $521 = HEAP32[$520>>2]|0;
        $522 = (($521) + ($518<<2)|0);
        $523 = HEAP32[$522>>2]|0;
        $irow = $523;
        $524 = $jcol;
        $525 = $6;
        $526 = (($525) + ($524<<4)|0);
        $527 = +HEAPF64[$526>>3];
        $528 = HEAP32[$i>>2]|0;
        $529 = $Uval;
        $530 = (($529) + ($528<<4)|0);
        $531 = +HEAPF64[$530>>3];
        $532 = $527 * $531;
        $533 = $jcol;
        $534 = $6;
        $535 = (($534) + ($533<<4)|0);
        $536 = ((($535)) + 8|0);
        $537 = +HEAPF64[$536>>3];
        $538 = HEAP32[$i>>2]|0;
        $539 = $Uval;
        $540 = (($539) + ($538<<4)|0);
        $541 = ((($540)) + 8|0);
        $542 = +HEAPF64[$541>>3];
        $543 = $537 * $542;
        $544 = $532 - $543;
        $cr3 = $544;
        $545 = $jcol;
        $546 = $6;
        $547 = (($546) + ($545<<4)|0);
        $548 = ((($547)) + 8|0);
        $549 = +HEAPF64[$548>>3];
        $550 = HEAP32[$i>>2]|0;
        $551 = $Uval;
        $552 = (($551) + ($550<<4)|0);
        $553 = +HEAPF64[$552>>3];
        $554 = $549 * $553;
        $555 = $jcol;
        $556 = $6;
        $557 = (($556) + ($555<<4)|0);
        $558 = +HEAPF64[$557>>3];
        $559 = HEAP32[$i>>2]|0;
        $560 = $Uval;
        $561 = (($560) + ($559<<4)|0);
        $562 = ((($561)) + 8|0);
        $563 = +HEAPF64[$562>>3];
        $564 = $558 * $563;
        $565 = $554 + $564;
        $ci4 = $565;
        $566 = $cr3;
        HEAPF64[$comp_zero>>3] = $566;
        $567 = $ci4;
        $568 = ((($comp_zero)) + 8|0);
        HEAPF64[$568>>3] = $567;
        $569 = $irow;
        $570 = $6;
        $571 = (($570) + ($569<<4)|0);
        $572 = +HEAPF64[$571>>3];
        $573 = +HEAPF64[$comp_zero>>3];
        $574 = $572 - $573;
        $575 = $irow;
        $576 = $6;
        $577 = (($576) + ($575<<4)|0);
        HEAPF64[$577>>3] = $574;
        $578 = $irow;
        $579 = $6;
        $580 = (($579) + ($578<<4)|0);
        $581 = ((($580)) + 8|0);
        $582 = +HEAPF64[$581>>3];
        $583 = ((($comp_zero)) + 8|0);
        $584 = +HEAPF64[$583>>3];
        $585 = $582 - $584;
        $586 = $irow;
        $587 = $6;
        $588 = (($587) + ($586<<4)|0);
        $589 = ((($588)) + 8|0);
        HEAPF64[$589>>3] = $585;
        $590 = HEAP32[$i>>2]|0;
        $591 = (($590) + 1)|0;
        HEAP32[$i>>2] = $591;
       }
       $592 = $jcol;
       $593 = (($592) + 1)|0;
       $jcol = $593;
      }
     }
    } while(0);
    $594 = $k;
    $595 = (($594) + -1)|0;
    $k = $595;
   }
  } else {
   $596 = $2;
   $597 = (_lsame_($596,3416)|0);
   $598 = ($597|0)!=(0);
   $599 = $1;
   $600 = (_lsame_($599,3400)|0);
   $601 = ($600|0)!=(0);
   if ($598) {
    if ($601) {
     $602 = $4;
     $603 = ((($602)) + 12|0);
     $604 = HEAP32[$603>>2]|0;
     $605 = ($604|0)==(0);
     if ($605) {
      $0 = 0;
      $1364 = $0;
      STACKTOP = sp;return ($1364|0);
     }
     $606 = $Lstore;
     $607 = ((($606)) + 4|0);
     $608 = HEAP32[$607>>2]|0;
     $k = $608;
     while(1) {
      $609 = $k;
      $610 = ($609|0)>=(0);
      if (!($610)) {
       break L30;
      }
      $611 = $k;
      $612 = $Lstore;
      $613 = ((($612)) + 28|0);
      $614 = HEAP32[$613>>2]|0;
      $615 = (($614) + ($611<<2)|0);
      $616 = HEAP32[$615>>2]|0;
      $fsupc = $616;
      $617 = $fsupc;
      $618 = $Lstore;
      $619 = ((($618)) + 20|0);
      $620 = HEAP32[$619>>2]|0;
      $621 = (($620) + ($617<<2)|0);
      $622 = HEAP32[$621>>2]|0;
      $istart = $622;
      $623 = $fsupc;
      $624 = (($623) + 1)|0;
      $625 = $Lstore;
      $626 = ((($625)) + 20|0);
      $627 = HEAP32[$626>>2]|0;
      $628 = (($627) + ($624<<2)|0);
      $629 = HEAP32[$628>>2]|0;
      $630 = $istart;
      $631 = (($629) - ($630))|0;
      HEAP32[$nsupr>>2] = $631;
      $632 = $k;
      $633 = (($632) + 1)|0;
      $634 = $Lstore;
      $635 = ((($634)) + 28|0);
      $636 = HEAP32[$635>>2]|0;
      $637 = (($636) + ($633<<2)|0);
      $638 = HEAP32[$637>>2]|0;
      $639 = $fsupc;
      $640 = (($638) - ($639))|0;
      HEAP32[$nsupc>>2] = $640;
      $641 = $fsupc;
      $642 = $Lstore;
      $643 = ((($642)) + 12|0);
      $644 = HEAP32[$643>>2]|0;
      $645 = (($644) + ($641<<2)|0);
      $646 = HEAP32[$645>>2]|0;
      $luptr = $646;
      $647 = HEAP32[$nsupr>>2]|0;
      $648 = HEAP32[$nsupc>>2]|0;
      $649 = (($647) - ($648))|0;
      $650 = $649<<3;
      $651 = HEAP32[$nsupc>>2]|0;
      $652 = Math_imul($650, $651)|0;
      $653 = (+($652|0));
      $654 = $solve_ops;
      $655 = $654 + $653;
      $solve_ops = $655;
      $656 = $fsupc;
      $jcol = $656;
      while(1) {
       $657 = $jcol;
       $658 = $k;
       $659 = (($658) + 1)|0;
       $660 = $Lstore;
       $661 = ((($660)) + 28|0);
       $662 = HEAP32[$661>>2]|0;
       $663 = (($662) + ($659<<2)|0);
       $664 = HEAP32[$663>>2]|0;
       $665 = ($657|0)<($664|0);
       if (!($665)) {
        break;
       }
       $666 = $istart;
       $667 = HEAP32[$nsupc>>2]|0;
       $668 = (($666) + ($667))|0;
       $iptr = $668;
       $669 = $jcol;
       $670 = $Lstore;
       $671 = ((($670)) + 12|0);
       $672 = HEAP32[$671>>2]|0;
       $673 = (($672) + ($669<<2)|0);
       $674 = HEAP32[$673>>2]|0;
       $675 = HEAP32[$nsupc>>2]|0;
       $676 = (($674) + ($675))|0;
       HEAP32[$i>>2] = $676;
       while(1) {
        $677 = HEAP32[$i>>2]|0;
        $678 = $jcol;
        $679 = (($678) + 1)|0;
        $680 = $Lstore;
        $681 = ((($680)) + 12|0);
        $682 = HEAP32[$681>>2]|0;
        $683 = (($682) + ($679<<2)|0);
        $684 = HEAP32[$683>>2]|0;
        $685 = ($677|0)<($684|0);
        if (!($685)) {
         break;
        }
        $686 = $iptr;
        $687 = $Lstore;
        $688 = ((($687)) + 16|0);
        $689 = HEAP32[$688>>2]|0;
        $690 = (($689) + ($686<<2)|0);
        $691 = HEAP32[$690>>2]|0;
        $irow = $691;
        $692 = $irow;
        $693 = $6;
        $694 = (($693) + ($692<<4)|0);
        $695 = +HEAPF64[$694>>3];
        $696 = HEAP32[$i>>2]|0;
        $697 = $Lval;
        $698 = (($697) + ($696<<4)|0);
        $699 = +HEAPF64[$698>>3];
        $700 = $695 * $699;
        $701 = $irow;
        $702 = $6;
        $703 = (($702) + ($701<<4)|0);
        $704 = ((($703)) + 8|0);
        $705 = +HEAPF64[$704>>3];
        $706 = HEAP32[$i>>2]|0;
        $707 = $Lval;
        $708 = (($707) + ($706<<4)|0);
        $709 = ((($708)) + 8|0);
        $710 = +HEAPF64[$709>>3];
        $711 = $705 * $710;
        $712 = $700 - $711;
        $cr5 = $712;
        $713 = $irow;
        $714 = $6;
        $715 = (($714) + ($713<<4)|0);
        $716 = ((($715)) + 8|0);
        $717 = +HEAPF64[$716>>3];
        $718 = HEAP32[$i>>2]|0;
        $719 = $Lval;
        $720 = (($719) + ($718<<4)|0);
        $721 = +HEAPF64[$720>>3];
        $722 = $717 * $721;
        $723 = $irow;
        $724 = $6;
        $725 = (($724) + ($723<<4)|0);
        $726 = +HEAPF64[$725>>3];
        $727 = HEAP32[$i>>2]|0;
        $728 = $Lval;
        $729 = (($728) + ($727<<4)|0);
        $730 = ((($729)) + 8|0);
        $731 = +HEAPF64[$730>>3];
        $732 = $726 * $731;
        $733 = $722 + $732;
        $ci6 = $733;
        $734 = $cr5;
        HEAPF64[$comp_zero>>3] = $734;
        $735 = $ci6;
        $736 = ((($comp_zero)) + 8|0);
        HEAPF64[$736>>3] = $735;
        $737 = $jcol;
        $738 = $6;
        $739 = (($738) + ($737<<4)|0);
        $740 = +HEAPF64[$739>>3];
        $741 = +HEAPF64[$comp_zero>>3];
        $742 = $740 - $741;
        $743 = $jcol;
        $744 = $6;
        $745 = (($744) + ($743<<4)|0);
        HEAPF64[$745>>3] = $742;
        $746 = $jcol;
        $747 = $6;
        $748 = (($747) + ($746<<4)|0);
        $749 = ((($748)) + 8|0);
        $750 = +HEAPF64[$749>>3];
        $751 = ((($comp_zero)) + 8|0);
        $752 = +HEAPF64[$751>>3];
        $753 = $750 - $752;
        $754 = $jcol;
        $755 = $6;
        $756 = (($755) + ($754<<4)|0);
        $757 = ((($756)) + 8|0);
        HEAPF64[$757>>3] = $753;
        $758 = $iptr;
        $759 = (($758) + 1)|0;
        $iptr = $759;
        $760 = HEAP32[$i>>2]|0;
        $761 = (($760) + 1)|0;
        HEAP32[$i>>2] = $761;
       }
       $762 = $jcol;
       $763 = (($762) + 1)|0;
       $jcol = $763;
      }
      $764 = HEAP32[$nsupc>>2]|0;
      $765 = ($764|0)>(1);
      if ($765) {
       $766 = HEAP32[$nsupc>>2]|0;
       $767 = $766<<2;
       $768 = HEAP32[$nsupc>>2]|0;
       $769 = (($768) - 1)|0;
       $770 = Math_imul($767, $769)|0;
       $771 = (+($770|0));
       $772 = $solve_ops;
       $773 = $772 + $771;
       $solve_ops = $773;
       $774 = $luptr;
       $775 = $Lval;
       $776 = (($775) + ($774<<4)|0);
       $777 = $fsupc;
       $778 = $6;
       $779 = (($778) + ($777<<4)|0);
       (_ztrsv_(3400,3416,3392,$nsupc,$776,$nsupr,$779,$incx)|0);
      }
      $780 = $k;
      $781 = (($780) + -1)|0;
      $k = $781;
     }
    }
    $782 = $5;
    $783 = ((($782)) + 12|0);
    $784 = HEAP32[$783>>2]|0;
    $785 = ($784|0)==(0);
    if ($785) {
     $0 = 0;
     $1364 = $0;
     STACKTOP = sp;return ($1364|0);
    }
    $k = 0;
    while(1) {
     $786 = $k;
     $787 = $Lstore;
     $788 = ((($787)) + 4|0);
     $789 = HEAP32[$788>>2]|0;
     $790 = ($786|0)<=($789|0);
     if (!($790)) {
      break L30;
     }
     $791 = $k;
     $792 = $Lstore;
     $793 = ((($792)) + 28|0);
     $794 = HEAP32[$793>>2]|0;
     $795 = (($794) + ($791<<2)|0);
     $796 = HEAP32[$795>>2]|0;
     $fsupc = $796;
     $797 = $fsupc;
     $798 = (($797) + 1)|0;
     $799 = $Lstore;
     $800 = ((($799)) + 20|0);
     $801 = HEAP32[$800>>2]|0;
     $802 = (($801) + ($798<<2)|0);
     $803 = HEAP32[$802>>2]|0;
     $804 = $fsupc;
     $805 = $Lstore;
     $806 = ((($805)) + 20|0);
     $807 = HEAP32[$806>>2]|0;
     $808 = (($807) + ($804<<2)|0);
     $809 = HEAP32[$808>>2]|0;
     $810 = (($803) - ($809))|0;
     HEAP32[$nsupr>>2] = $810;
     $811 = $k;
     $812 = (($811) + 1)|0;
     $813 = $Lstore;
     $814 = ((($813)) + 28|0);
     $815 = HEAP32[$814>>2]|0;
     $816 = (($815) + ($812<<2)|0);
     $817 = HEAP32[$816>>2]|0;
     $818 = $fsupc;
     $819 = (($817) - ($818))|0;
     HEAP32[$nsupc>>2] = $819;
     $820 = $fsupc;
     $821 = $Lstore;
     $822 = ((($821)) + 12|0);
     $823 = HEAP32[$822>>2]|0;
     $824 = (($823) + ($820<<2)|0);
     $825 = HEAP32[$824>>2]|0;
     $luptr = $825;
     $826 = $fsupc;
     $jcol = $826;
     while(1) {
      $827 = $jcol;
      $828 = $k;
      $829 = (($828) + 1)|0;
      $830 = $Lstore;
      $831 = ((($830)) + 28|0);
      $832 = HEAP32[$831>>2]|0;
      $833 = (($832) + ($829<<2)|0);
      $834 = HEAP32[$833>>2]|0;
      $835 = ($827|0)<($834|0);
      if (!($835)) {
       break;
      }
      $836 = $jcol;
      $837 = (($836) + 1)|0;
      $838 = $Ustore;
      $839 = ((($838)) + 12|0);
      $840 = HEAP32[$839>>2]|0;
      $841 = (($840) + ($837<<2)|0);
      $842 = HEAP32[$841>>2]|0;
      $843 = $jcol;
      $844 = $Ustore;
      $845 = ((($844)) + 12|0);
      $846 = HEAP32[$845>>2]|0;
      $847 = (($846) + ($843<<2)|0);
      $848 = HEAP32[$847>>2]|0;
      $849 = (($842) - ($848))|0;
      $850 = $849<<3;
      $851 = (+($850|0));
      $852 = $solve_ops;
      $853 = $852 + $851;
      $solve_ops = $853;
      $854 = $jcol;
      $855 = $Ustore;
      $856 = ((($855)) + 12|0);
      $857 = HEAP32[$856>>2]|0;
      $858 = (($857) + ($854<<2)|0);
      $859 = HEAP32[$858>>2]|0;
      HEAP32[$i>>2] = $859;
      while(1) {
       $860 = HEAP32[$i>>2]|0;
       $861 = $jcol;
       $862 = (($861) + 1)|0;
       $863 = $Ustore;
       $864 = ((($863)) + 12|0);
       $865 = HEAP32[$864>>2]|0;
       $866 = (($865) + ($862<<2)|0);
       $867 = HEAP32[$866>>2]|0;
       $868 = ($860|0)<($867|0);
       if (!($868)) {
        break;
       }
       $869 = HEAP32[$i>>2]|0;
       $870 = $Ustore;
       $871 = ((($870)) + 8|0);
       $872 = HEAP32[$871>>2]|0;
       $873 = (($872) + ($869<<2)|0);
       $874 = HEAP32[$873>>2]|0;
       $irow = $874;
       $875 = $irow;
       $876 = $6;
       $877 = (($876) + ($875<<4)|0);
       $878 = +HEAPF64[$877>>3];
       $879 = HEAP32[$i>>2]|0;
       $880 = $Uval;
       $881 = (($880) + ($879<<4)|0);
       $882 = +HEAPF64[$881>>3];
       $883 = $878 * $882;
       $884 = $irow;
       $885 = $6;
       $886 = (($885) + ($884<<4)|0);
       $887 = ((($886)) + 8|0);
       $888 = +HEAPF64[$887>>3];
       $889 = HEAP32[$i>>2]|0;
       $890 = $Uval;
       $891 = (($890) + ($889<<4)|0);
       $892 = ((($891)) + 8|0);
       $893 = +HEAPF64[$892>>3];
       $894 = $888 * $893;
       $895 = $883 - $894;
       $cr7 = $895;
       $896 = $irow;
       $897 = $6;
       $898 = (($897) + ($896<<4)|0);
       $899 = ((($898)) + 8|0);
       $900 = +HEAPF64[$899>>3];
       $901 = HEAP32[$i>>2]|0;
       $902 = $Uval;
       $903 = (($902) + ($901<<4)|0);
       $904 = +HEAPF64[$903>>3];
       $905 = $900 * $904;
       $906 = $irow;
       $907 = $6;
       $908 = (($907) + ($906<<4)|0);
       $909 = +HEAPF64[$908>>3];
       $910 = HEAP32[$i>>2]|0;
       $911 = $Uval;
       $912 = (($911) + ($910<<4)|0);
       $913 = ((($912)) + 8|0);
       $914 = +HEAPF64[$913>>3];
       $915 = $909 * $914;
       $916 = $905 + $915;
       $ci8 = $916;
       $917 = $cr7;
       HEAPF64[$comp_zero>>3] = $917;
       $918 = $ci8;
       $919 = ((($comp_zero)) + 8|0);
       HEAPF64[$919>>3] = $918;
       $920 = $jcol;
       $921 = $6;
       $922 = (($921) + ($920<<4)|0);
       $923 = +HEAPF64[$922>>3];
       $924 = +HEAPF64[$comp_zero>>3];
       $925 = $923 - $924;
       $926 = $jcol;
       $927 = $6;
       $928 = (($927) + ($926<<4)|0);
       HEAPF64[$928>>3] = $925;
       $929 = $jcol;
       $930 = $6;
       $931 = (($930) + ($929<<4)|0);
       $932 = ((($931)) + 8|0);
       $933 = +HEAPF64[$932>>3];
       $934 = ((($comp_zero)) + 8|0);
       $935 = +HEAPF64[$934>>3];
       $936 = $933 - $935;
       $937 = $jcol;
       $938 = $6;
       $939 = (($938) + ($937<<4)|0);
       $940 = ((($939)) + 8|0);
       HEAPF64[$940>>3] = $936;
       $941 = HEAP32[$i>>2]|0;
       $942 = (($941) + 1)|0;
       HEAP32[$i>>2] = $942;
      }
      $943 = $jcol;
      $944 = (($943) + 1)|0;
      $jcol = $944;
     }
     $945 = HEAP32[$nsupc>>2]|0;
     $946 = $945<<2;
     $947 = HEAP32[$nsupc>>2]|0;
     $948 = (($947) + 1)|0;
     $949 = Math_imul($946, $948)|0;
     $950 = HEAP32[$nsupc>>2]|0;
     $951 = ($950*10)|0;
     $952 = (($949) + ($951))|0;
     $953 = (+($952|0));
     $954 = $solve_ops;
     $955 = $954 + $953;
     $solve_ops = $955;
     $956 = HEAP32[$nsupc>>2]|0;
     $957 = ($956|0)==(1);
     if ($957) {
      $958 = $fsupc;
      $959 = $6;
      $960 = (($959) + ($958<<4)|0);
      $961 = $fsupc;
      $962 = $6;
      $963 = (($962) + ($961<<4)|0);
      $964 = $luptr;
      $965 = $Lval;
      $966 = (($965) + ($964<<4)|0);
      _z_div($960,$963,$966);
     } else {
      $967 = $luptr;
      $968 = $Lval;
      $969 = (($968) + ($967<<4)|0);
      $970 = $fsupc;
      $971 = $6;
      $972 = (($971) + ($970<<4)|0);
      (_ztrsv_(3392,3416,3408,$nsupc,$969,$nsupr,$972,$incx)|0);
     }
     $973 = $k;
     $974 = (($973) + 1)|0;
     $k = $974;
    }
   } else {
    if ($601) {
     $975 = $4;
     $976 = ((($975)) + 12|0);
     $977 = HEAP32[$976>>2]|0;
     $978 = ($977|0)==(0);
     if ($978) {
      $0 = 0;
      $1364 = $0;
      STACKTOP = sp;return ($1364|0);
     }
     $979 = $Lstore;
     $980 = ((($979)) + 4|0);
     $981 = HEAP32[$980>>2]|0;
     $k = $981;
     while(1) {
      $982 = $k;
      $983 = ($982|0)>=(0);
      if (!($983)) {
       break L30;
      }
      $984 = $k;
      $985 = $Lstore;
      $986 = ((($985)) + 28|0);
      $987 = HEAP32[$986>>2]|0;
      $988 = (($987) + ($984<<2)|0);
      $989 = HEAP32[$988>>2]|0;
      $fsupc = $989;
      $990 = $fsupc;
      $991 = $Lstore;
      $992 = ((($991)) + 20|0);
      $993 = HEAP32[$992>>2]|0;
      $994 = (($993) + ($990<<2)|0);
      $995 = HEAP32[$994>>2]|0;
      $istart = $995;
      $996 = $fsupc;
      $997 = (($996) + 1)|0;
      $998 = $Lstore;
      $999 = ((($998)) + 20|0);
      $1000 = HEAP32[$999>>2]|0;
      $1001 = (($1000) + ($997<<2)|0);
      $1002 = HEAP32[$1001>>2]|0;
      $1003 = $istart;
      $1004 = (($1002) - ($1003))|0;
      HEAP32[$nsupr>>2] = $1004;
      $1005 = $k;
      $1006 = (($1005) + 1)|0;
      $1007 = $Lstore;
      $1008 = ((($1007)) + 28|0);
      $1009 = HEAP32[$1008>>2]|0;
      $1010 = (($1009) + ($1006<<2)|0);
      $1011 = HEAP32[$1010>>2]|0;
      $1012 = $fsupc;
      $1013 = (($1011) - ($1012))|0;
      HEAP32[$nsupc>>2] = $1013;
      $1014 = $fsupc;
      $1015 = $Lstore;
      $1016 = ((($1015)) + 12|0);
      $1017 = HEAP32[$1016>>2]|0;
      $1018 = (($1017) + ($1014<<2)|0);
      $1019 = HEAP32[$1018>>2]|0;
      $luptr = $1019;
      $1020 = HEAP32[$nsupr>>2]|0;
      $1021 = HEAP32[$nsupc>>2]|0;
      $1022 = (($1020) - ($1021))|0;
      $1023 = $1022<<3;
      $1024 = HEAP32[$nsupc>>2]|0;
      $1025 = Math_imul($1023, $1024)|0;
      $1026 = (+($1025|0));
      $1027 = $solve_ops;
      $1028 = $1027 + $1026;
      $solve_ops = $1028;
      $1029 = $fsupc;
      $jcol = $1029;
      while(1) {
       $1030 = $jcol;
       $1031 = $k;
       $1032 = (($1031) + 1)|0;
       $1033 = $Lstore;
       $1034 = ((($1033)) + 28|0);
       $1035 = HEAP32[$1034>>2]|0;
       $1036 = (($1035) + ($1032<<2)|0);
       $1037 = HEAP32[$1036>>2]|0;
       $1038 = ($1030|0)<($1037|0);
       if (!($1038)) {
        break;
       }
       $1039 = $istart;
       $1040 = HEAP32[$nsupc>>2]|0;
       $1041 = (($1039) + ($1040))|0;
       $iptr = $1041;
       $1042 = $jcol;
       $1043 = $Lstore;
       $1044 = ((($1043)) + 12|0);
       $1045 = HEAP32[$1044>>2]|0;
       $1046 = (($1045) + ($1042<<2)|0);
       $1047 = HEAP32[$1046>>2]|0;
       $1048 = HEAP32[$nsupc>>2]|0;
       $1049 = (($1047) + ($1048))|0;
       HEAP32[$i>>2] = $1049;
       while(1) {
        $1050 = HEAP32[$i>>2]|0;
        $1051 = $jcol;
        $1052 = (($1051) + 1)|0;
        $1053 = $Lstore;
        $1054 = ((($1053)) + 12|0);
        $1055 = HEAP32[$1054>>2]|0;
        $1056 = (($1055) + ($1052<<2)|0);
        $1057 = HEAP32[$1056>>2]|0;
        $1058 = ($1050|0)<($1057|0);
        if (!($1058)) {
         break;
        }
        $1059 = $iptr;
        $1060 = $Lstore;
        $1061 = ((($1060)) + 16|0);
        $1062 = HEAP32[$1061>>2]|0;
        $1063 = (($1062) + ($1059<<2)|0);
        $1064 = HEAP32[$1063>>2]|0;
        $irow = $1064;
        $1065 = HEAP32[$i>>2]|0;
        $1066 = $Lval;
        $1067 = (($1066) + ($1065<<4)|0);
        $1068 = +HEAPF64[$1067>>3];
        HEAPF64[$temp>>3] = $1068;
        $1069 = HEAP32[$i>>2]|0;
        $1070 = $Lval;
        $1071 = (($1070) + ($1069<<4)|0);
        $1072 = ((($1071)) + 8|0);
        $1073 = +HEAPF64[$1072>>3];
        $1074 = -$1073;
        $1075 = ((($temp)) + 8|0);
        HEAPF64[$1075>>3] = $1074;
        $1076 = $irow;
        $1077 = $6;
        $1078 = (($1077) + ($1076<<4)|0);
        $1079 = +HEAPF64[$1078>>3];
        $1080 = +HEAPF64[$temp>>3];
        $1081 = $1079 * $1080;
        $1082 = $irow;
        $1083 = $6;
        $1084 = (($1083) + ($1082<<4)|0);
        $1085 = ((($1084)) + 8|0);
        $1086 = +HEAPF64[$1085>>3];
        $1087 = ((($temp)) + 8|0);
        $1088 = +HEAPF64[$1087>>3];
        $1089 = $1086 * $1088;
        $1090 = $1081 - $1089;
        $cr9 = $1090;
        $1091 = $irow;
        $1092 = $6;
        $1093 = (($1092) + ($1091<<4)|0);
        $1094 = ((($1093)) + 8|0);
        $1095 = +HEAPF64[$1094>>3];
        $1096 = +HEAPF64[$temp>>3];
        $1097 = $1095 * $1096;
        $1098 = $irow;
        $1099 = $6;
        $1100 = (($1099) + ($1098<<4)|0);
        $1101 = +HEAPF64[$1100>>3];
        $1102 = ((($temp)) + 8|0);
        $1103 = +HEAPF64[$1102>>3];
        $1104 = $1101 * $1103;
        $1105 = $1097 + $1104;
        $ci10 = $1105;
        $1106 = $cr9;
        HEAPF64[$comp_zero>>3] = $1106;
        $1107 = $ci10;
        $1108 = ((($comp_zero)) + 8|0);
        HEAPF64[$1108>>3] = $1107;
        $1109 = $jcol;
        $1110 = $6;
        $1111 = (($1110) + ($1109<<4)|0);
        $1112 = +HEAPF64[$1111>>3];
        $1113 = +HEAPF64[$comp_zero>>3];
        $1114 = $1112 - $1113;
        $1115 = $jcol;
        $1116 = $6;
        $1117 = (($1116) + ($1115<<4)|0);
        HEAPF64[$1117>>3] = $1114;
        $1118 = $jcol;
        $1119 = $6;
        $1120 = (($1119) + ($1118<<4)|0);
        $1121 = ((($1120)) + 8|0);
        $1122 = +HEAPF64[$1121>>3];
        $1123 = ((($comp_zero)) + 8|0);
        $1124 = +HEAPF64[$1123>>3];
        $1125 = $1122 - $1124;
        $1126 = $jcol;
        $1127 = $6;
        $1128 = (($1127) + ($1126<<4)|0);
        $1129 = ((($1128)) + 8|0);
        HEAPF64[$1129>>3] = $1125;
        $1130 = $iptr;
        $1131 = (($1130) + 1)|0;
        $iptr = $1131;
        $1132 = HEAP32[$i>>2]|0;
        $1133 = (($1132) + 1)|0;
        HEAP32[$i>>2] = $1133;
       }
       $1134 = $jcol;
       $1135 = (($1134) + 1)|0;
       $jcol = $1135;
      }
      $1136 = HEAP32[$nsupc>>2]|0;
      $1137 = ($1136|0)>(1);
      if ($1137) {
       $1138 = HEAP32[$nsupc>>2]|0;
       $1139 = $1138<<2;
       $1140 = HEAP32[$nsupc>>2]|0;
       $1141 = (($1140) - 1)|0;
       $1142 = Math_imul($1139, $1141)|0;
       $1143 = (+($1142|0));
       $1144 = $solve_ops;
       $1145 = $1144 + $1143;
       $solve_ops = $1145;
       $1146 = $2;
       $1147 = $luptr;
       $1148 = $Lval;
       $1149 = (($1148) + ($1147<<4)|0);
       $1150 = $fsupc;
       $1151 = $6;
       $1152 = (($1151) + ($1150<<4)|0);
       (_ztrsv_(3400,$1146,3392,$nsupc,$1149,$nsupr,$1152,$incx)|0);
      }
      $1153 = $k;
      $1154 = (($1153) + -1)|0;
      $k = $1154;
     }
    }
    $1155 = $5;
    $1156 = ((($1155)) + 12|0);
    $1157 = HEAP32[$1156>>2]|0;
    $1158 = ($1157|0)==(0);
    if ($1158) {
     $0 = 0;
     $1364 = $0;
     STACKTOP = sp;return ($1364|0);
    }
    $k = 0;
    while(1) {
     $1159 = $k;
     $1160 = $Lstore;
     $1161 = ((($1160)) + 4|0);
     $1162 = HEAP32[$1161>>2]|0;
     $1163 = ($1159|0)<=($1162|0);
     if (!($1163)) {
      break L30;
     }
     $1164 = $k;
     $1165 = $Lstore;
     $1166 = ((($1165)) + 28|0);
     $1167 = HEAP32[$1166>>2]|0;
     $1168 = (($1167) + ($1164<<2)|0);
     $1169 = HEAP32[$1168>>2]|0;
     $fsupc = $1169;
     $1170 = $fsupc;
     $1171 = (($1170) + 1)|0;
     $1172 = $Lstore;
     $1173 = ((($1172)) + 20|0);
     $1174 = HEAP32[$1173>>2]|0;
     $1175 = (($1174) + ($1171<<2)|0);
     $1176 = HEAP32[$1175>>2]|0;
     $1177 = $fsupc;
     $1178 = $Lstore;
     $1179 = ((($1178)) + 20|0);
     $1180 = HEAP32[$1179>>2]|0;
     $1181 = (($1180) + ($1177<<2)|0);
     $1182 = HEAP32[$1181>>2]|0;
     $1183 = (($1176) - ($1182))|0;
     HEAP32[$nsupr>>2] = $1183;
     $1184 = $k;
     $1185 = (($1184) + 1)|0;
     $1186 = $Lstore;
     $1187 = ((($1186)) + 28|0);
     $1188 = HEAP32[$1187>>2]|0;
     $1189 = (($1188) + ($1185<<2)|0);
     $1190 = HEAP32[$1189>>2]|0;
     $1191 = $fsupc;
     $1192 = (($1190) - ($1191))|0;
     HEAP32[$nsupc>>2] = $1192;
     $1193 = $fsupc;
     $1194 = $Lstore;
     $1195 = ((($1194)) + 12|0);
     $1196 = HEAP32[$1195>>2]|0;
     $1197 = (($1196) + ($1193<<2)|0);
     $1198 = HEAP32[$1197>>2]|0;
     $luptr = $1198;
     $1199 = $fsupc;
     $jcol = $1199;
     while(1) {
      $1200 = $jcol;
      $1201 = $k;
      $1202 = (($1201) + 1)|0;
      $1203 = $Lstore;
      $1204 = ((($1203)) + 28|0);
      $1205 = HEAP32[$1204>>2]|0;
      $1206 = (($1205) + ($1202<<2)|0);
      $1207 = HEAP32[$1206>>2]|0;
      $1208 = ($1200|0)<($1207|0);
      if (!($1208)) {
       break;
      }
      $1209 = $jcol;
      $1210 = (($1209) + 1)|0;
      $1211 = $Ustore;
      $1212 = ((($1211)) + 12|0);
      $1213 = HEAP32[$1212>>2]|0;
      $1214 = (($1213) + ($1210<<2)|0);
      $1215 = HEAP32[$1214>>2]|0;
      $1216 = $jcol;
      $1217 = $Ustore;
      $1218 = ((($1217)) + 12|0);
      $1219 = HEAP32[$1218>>2]|0;
      $1220 = (($1219) + ($1216<<2)|0);
      $1221 = HEAP32[$1220>>2]|0;
      $1222 = (($1215) - ($1221))|0;
      $1223 = $1222<<3;
      $1224 = (+($1223|0));
      $1225 = $solve_ops;
      $1226 = $1225 + $1224;
      $solve_ops = $1226;
      $1227 = $jcol;
      $1228 = $Ustore;
      $1229 = ((($1228)) + 12|0);
      $1230 = HEAP32[$1229>>2]|0;
      $1231 = (($1230) + ($1227<<2)|0);
      $1232 = HEAP32[$1231>>2]|0;
      HEAP32[$i>>2] = $1232;
      while(1) {
       $1233 = HEAP32[$i>>2]|0;
       $1234 = $jcol;
       $1235 = (($1234) + 1)|0;
       $1236 = $Ustore;
       $1237 = ((($1236)) + 12|0);
       $1238 = HEAP32[$1237>>2]|0;
       $1239 = (($1238) + ($1235<<2)|0);
       $1240 = HEAP32[$1239>>2]|0;
       $1241 = ($1233|0)<($1240|0);
       if (!($1241)) {
        break;
       }
       $1242 = HEAP32[$i>>2]|0;
       $1243 = $Ustore;
       $1244 = ((($1243)) + 8|0);
       $1245 = HEAP32[$1244>>2]|0;
       $1246 = (($1245) + ($1242<<2)|0);
       $1247 = HEAP32[$1246>>2]|0;
       $irow = $1247;
       $1248 = HEAP32[$i>>2]|0;
       $1249 = $Uval;
       $1250 = (($1249) + ($1248<<4)|0);
       $1251 = +HEAPF64[$1250>>3];
       HEAPF64[$temp>>3] = $1251;
       $1252 = HEAP32[$i>>2]|0;
       $1253 = $Uval;
       $1254 = (($1253) + ($1252<<4)|0);
       $1255 = ((($1254)) + 8|0);
       $1256 = +HEAPF64[$1255>>3];
       $1257 = -$1256;
       $1258 = ((($temp)) + 8|0);
       HEAPF64[$1258>>3] = $1257;
       $1259 = $irow;
       $1260 = $6;
       $1261 = (($1260) + ($1259<<4)|0);
       $1262 = +HEAPF64[$1261>>3];
       $1263 = +HEAPF64[$temp>>3];
       $1264 = $1262 * $1263;
       $1265 = $irow;
       $1266 = $6;
       $1267 = (($1266) + ($1265<<4)|0);
       $1268 = ((($1267)) + 8|0);
       $1269 = +HEAPF64[$1268>>3];
       $1270 = ((($temp)) + 8|0);
       $1271 = +HEAPF64[$1270>>3];
       $1272 = $1269 * $1271;
       $1273 = $1264 - $1272;
       $cr11 = $1273;
       $1274 = $irow;
       $1275 = $6;
       $1276 = (($1275) + ($1274<<4)|0);
       $1277 = ((($1276)) + 8|0);
       $1278 = +HEAPF64[$1277>>3];
       $1279 = +HEAPF64[$temp>>3];
       $1280 = $1278 * $1279;
       $1281 = $irow;
       $1282 = $6;
       $1283 = (($1282) + ($1281<<4)|0);
       $1284 = +HEAPF64[$1283>>3];
       $1285 = ((($temp)) + 8|0);
       $1286 = +HEAPF64[$1285>>3];
       $1287 = $1284 * $1286;
       $1288 = $1280 + $1287;
       $ci12 = $1288;
       $1289 = $cr11;
       HEAPF64[$comp_zero>>3] = $1289;
       $1290 = $ci12;
       $1291 = ((($comp_zero)) + 8|0);
       HEAPF64[$1291>>3] = $1290;
       $1292 = $jcol;
       $1293 = $6;
       $1294 = (($1293) + ($1292<<4)|0);
       $1295 = +HEAPF64[$1294>>3];
       $1296 = +HEAPF64[$comp_zero>>3];
       $1297 = $1295 - $1296;
       $1298 = $jcol;
       $1299 = $6;
       $1300 = (($1299) + ($1298<<4)|0);
       HEAPF64[$1300>>3] = $1297;
       $1301 = $jcol;
       $1302 = $6;
       $1303 = (($1302) + ($1301<<4)|0);
       $1304 = ((($1303)) + 8|0);
       $1305 = +HEAPF64[$1304>>3];
       $1306 = ((($comp_zero)) + 8|0);
       $1307 = +HEAPF64[$1306>>3];
       $1308 = $1305 - $1307;
       $1309 = $jcol;
       $1310 = $6;
       $1311 = (($1310) + ($1309<<4)|0);
       $1312 = ((($1311)) + 8|0);
       HEAPF64[$1312>>3] = $1308;
       $1313 = HEAP32[$i>>2]|0;
       $1314 = (($1313) + 1)|0;
       HEAP32[$i>>2] = $1314;
      }
      $1315 = $jcol;
      $1316 = (($1315) + 1)|0;
      $jcol = $1316;
     }
     $1317 = HEAP32[$nsupc>>2]|0;
     $1318 = $1317<<2;
     $1319 = HEAP32[$nsupc>>2]|0;
     $1320 = (($1319) + 1)|0;
     $1321 = Math_imul($1318, $1320)|0;
     $1322 = HEAP32[$nsupc>>2]|0;
     $1323 = ($1322*10)|0;
     $1324 = (($1321) + ($1323))|0;
     $1325 = (+($1324|0));
     $1326 = $solve_ops;
     $1327 = $1326 + $1325;
     $solve_ops = $1327;
     $1328 = HEAP32[$nsupc>>2]|0;
     $1329 = ($1328|0)==(1);
     if ($1329) {
      $1330 = $luptr;
      $1331 = $Lval;
      $1332 = (($1331) + ($1330<<4)|0);
      $1333 = +HEAPF64[$1332>>3];
      HEAPF64[$temp>>3] = $1333;
      $1334 = $luptr;
      $1335 = $Lval;
      $1336 = (($1335) + ($1334<<4)|0);
      $1337 = ((($1336)) + 8|0);
      $1338 = +HEAPF64[$1337>>3];
      $1339 = -$1338;
      $1340 = ((($temp)) + 8|0);
      HEAPF64[$1340>>3] = $1339;
      $1341 = $fsupc;
      $1342 = $6;
      $1343 = (($1342) + ($1341<<4)|0);
      $1344 = $fsupc;
      $1345 = $6;
      $1346 = (($1345) + ($1344<<4)|0);
      _z_div($1343,$1346,$temp);
     } else {
      $1347 = $2;
      $1348 = $luptr;
      $1349 = $Lval;
      $1350 = (($1349) + ($1348<<4)|0);
      $1351 = $fsupc;
      $1352 = $6;
      $1353 = (($1352) + ($1351<<4)|0);
      (_ztrsv_(3392,$1347,3408,$nsupc,$1350,$nsupr,$1353,$incx)|0);
     }
     $1354 = $k;
     $1355 = (($1354) + 1)|0;
     $k = $1355;
    }
   }
  }
 } while(0);
 $1356 = $solve_ops;
 $1357 = $7;
 $1358 = ((($1357)) + 8|0);
 $1359 = HEAP32[$1358>>2]|0;
 $1360 = ((($1359)) + 44|0);
 $1361 = +HEAPF32[$1360>>2];
 $1362 = $1361 + $1356;
 HEAPF32[$1360>>2] = $1362;
 $1363 = $work;
 _superlu_free($1363);
 $0 = 0;
 $1364 = $0;
 STACKTOP = sp;return ($1364|0);
}
function _zgstrf($options,$A,$relax,$panel_size,$etree,$work,$lwork,$perm_c,$perm_r,$L,$U,$stat,$info) {
 $options = $options|0;
 $A = $A|0;
 $relax = $relax|0;
 $panel_size = $panel_size|0;
 $etree = $etree|0;
 $work = $work|0;
 $lwork = $lwork|0;
 $perm_c = $perm_c|0;
 $perm_r = $perm_r|0;
 $L = $L|0;
 $U = $U|0;
 $stat = $stat|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0.0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0.0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0.0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0.0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0.0;
 var $512 = 0, $513 = 0, $514 = 0.0, $515 = 0.0, $516 = 0, $517 = 0, $518 = 0.0, $519 = 0.0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0.0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Astore = 0, $a = 0, $asub = 0, $dense = 0, $diag_pivot_thresh = 0.0, $fact = 0, $fill_ratio = 0.0, $fsupc = 0, $i = 0, $icol = 0, $iinfo = 0;
 var $iperm_c = 0, $iperm_r = 0, $iperm_r_allocated = 0, $iwork = 0, $jcol = 0, $jj = 0, $jsupno = 0, $k = 0, $kcol = 0, $m = 0, $marker = 0, $min_mn = 0, $n = 0, $new_next = 0, $nextlu = 0, $nextu = 0, $nnzL = 0, $nnzU = 0, $nseg = 0, $nseg1 = 0;
 var $nzlumax = 0, $ops = 0, $or$cond = 0, $or$cond3 = 0, $panel_histo = 0, $panel_lsub = 0, $parent = 0, $pivrow = 0, $relax_end = 0, $repfnz = 0, $segrep = 0, $supno = 0, $tempv = 0, $usepr = 0, $w_def = 0, $xa_begin = 0, $xa_end = 0, $xlsub = 0, $xlusup = 0, $xplore = 0;
 var $xprune = 0, $xsup = 0, $xusub = 0, $zwork = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 272|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $iwork = sp + 204|0;
 $zwork = sp + 200|0;
 $segrep = sp + 196|0;
 $repfnz = sp + 192|0;
 $parent = sp + 188|0;
 $xplore = sp + 184|0;
 $panel_lsub = sp + 180|0;
 $xprune = sp + 176|0;
 $marker = sp + 172|0;
 $dense = sp + 168|0;
 $tempv = sp + 164|0;
 $nzlumax = sp + 120|0;
 $pivrow = sp + 112|0;
 $nseg1 = sp + 108|0;
 $nseg = sp + 104|0;
 $usepr = sp + 36|0;
 $nnzL = sp + 28|0;
 $nnzU = sp + 24|0;
 $0 = $options;
 $1 = $A;
 $2 = $relax;
 $3 = $panel_size;
 $4 = $etree;
 $5 = $work;
 $6 = $lwork;
 $7 = $perm_c;
 $8 = $perm_r;
 $9 = $L;
 $10 = $U;
 $11 = $stat;
 $12 = $info;
 $iperm_r = 0;
 $13 = (_sp_ienv(6)|0);
 $14 = (+($13|0));
 $fill_ratio = $14;
 $15 = $0;
 $16 = HEAP32[$15>>2]|0;
 $fact = $16;
 $17 = $0;
 $18 = ((($17)) + 24|0);
 $19 = +HEAPF64[$18>>3];
 $diag_pivot_thresh = $19;
 $iperm_r_allocated = 0;
 $20 = $11;
 $21 = HEAP32[$20>>2]|0;
 $panel_histo = $21;
 $22 = $11;
 $23 = ((($22)) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $ops = $24;
 $iinfo = 0;
 $25 = $1;
 $26 = ((($25)) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 $m = $27;
 $28 = $1;
 $29 = ((($28)) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $n = $30;
 $31 = $m;
 $32 = $n;
 $33 = ($31|0)<($32|0);
 $34 = $m;
 $35 = $n;
 $36 = $33 ? $34 : $35;
 $min_mn = $36;
 $37 = $1;
 $38 = ((($37)) + 20|0);
 $39 = HEAP32[$38>>2]|0;
 $Astore = $39;
 $40 = $Astore;
 $41 = ((($40)) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $a = $42;
 $43 = $Astore;
 $44 = ((($43)) + 8|0);
 $45 = HEAP32[$44>>2]|0;
 $asub = $45;
 $46 = $Astore;
 $47 = ((($46)) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $xa_begin = $48;
 $49 = $Astore;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $xa_end = $51;
 $52 = $fact;
 $53 = $5;
 $54 = $6;
 $55 = $m;
 $56 = $n;
 $57 = $Astore;
 $58 = HEAP32[$57>>2]|0;
 $59 = $3;
 $60 = $fill_ratio;
 $61 = $9;
 $62 = $10;
 $63 = (_zLUMemInit($52,$53,$54,$55,$56,$58,$59,$60,$61,$62,2264,$iwork,$zwork)|0);
 $64 = $12;
 HEAP32[$64>>2] = $63;
 $65 = $12;
 $66 = HEAP32[$65>>2]|0;
 $67 = ($66|0)!=(0);
 if ($67) {
  STACKTOP = sp;return;
 }
 $68 = HEAP32[2264>>2]|0;
 $xsup = $68;
 $69 = HEAP32[(2268)>>2]|0;
 $supno = $69;
 $70 = HEAP32[(2276)>>2]|0;
 $xlsub = $70;
 $71 = HEAP32[(2284)>>2]|0;
 $xlusup = $71;
 $72 = HEAP32[(2296)>>2]|0;
 $xusub = $72;
 $73 = $m;
 $74 = $n;
 $75 = $3;
 $76 = HEAP32[$iwork>>2]|0;
 _SetIWork($73,$74,$75,$76,$segrep,$parent,$xplore,$repfnz,$panel_lsub,$xprune,$marker);
 $77 = $m;
 $78 = $3;
 $79 = HEAP32[$zwork>>2]|0;
 _zSetRWork($77,$78,$79,$dense,$tempv);
 $80 = $fact;
 $81 = ($80|0)==(2);
 $82 = $81&1;
 HEAP32[$usepr>>2] = $82;
 $83 = HEAP32[$usepr>>2]|0;
 $84 = ($83|0)!=(0);
 if ($84) {
  $85 = $m;
  $86 = (_intMalloc($85)|0);
  $iperm_r = $86;
  $k = 0;
  while(1) {
   $87 = $k;
   $88 = $m;
   $89 = ($87|0)<($88|0);
   if (!($89)) {
    break;
   }
   $90 = $k;
   $91 = $k;
   $92 = $8;
   $93 = (($92) + ($91<<2)|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = $iperm_r;
   $96 = (($95) + ($94<<2)|0);
   HEAP32[$96>>2] = $90;
   $97 = $k;
   $98 = (($97) + 1)|0;
   $k = $98;
  }
  $iperm_r_allocated = 1;
 }
 $99 = $n;
 $100 = (_intMalloc($99)|0);
 $iperm_c = $100;
 $k = 0;
 while(1) {
  $101 = $k;
  $102 = $n;
  $103 = ($101|0)<($102|0);
  if (!($103)) {
   break;
  }
  $104 = $k;
  $105 = $k;
  $106 = $7;
  $107 = (($106) + ($105<<2)|0);
  $108 = HEAP32[$107>>2]|0;
  $109 = $iperm_c;
  $110 = (($109) + ($108<<2)|0);
  HEAP32[$110>>2] = $104;
  $111 = $k;
  $112 = (($111) + 1)|0;
  $k = $112;
 }
 $113 = $n;
 $114 = (_intMalloc($113)|0);
 $relax_end = $114;
 $115 = $0;
 $116 = ((($115)) + 32|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = ($117|0)==(1);
 $119 = $n;
 $120 = $4;
 $121 = $2;
 $122 = HEAP32[$marker>>2]|0;
 $123 = $relax_end;
 if ($118) {
  _heap_relax_snode($119,$120,$121,$122,$123);
 } else {
  _relax_snode($119,$120,$121,$122,$123);
 }
 $124 = $8;
 $125 = $m;
 _ifill($124,$125,-1);
 $126 = HEAP32[$marker>>2]|0;
 $127 = $m;
 $128 = ($127*3)|0;
 _ifill($126,$128,-1);
 $129 = $supno;
 HEAP32[$129>>2] = -1;
 $130 = $xlusup;
 HEAP32[$130>>2] = 0;
 $131 = $xusub;
 HEAP32[$131>>2] = 0;
 $132 = $xlsub;
 HEAP32[$132>>2] = 0;
 $133 = $xsup;
 HEAP32[$133>>2] = 0;
 $134 = $3;
 $w_def = $134;
 $jcol = 0;
 L19: while(1) {
  $135 = $jcol;
  $136 = $min_mn;
  $137 = ($135|0)<($136|0);
  if (!($137)) {
   label = 48;
   break;
  }
  $138 = $jcol;
  $139 = $relax_end;
  $140 = (($139) + ($138<<2)|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = ($141|0)!=(-1);
  if ($142) {
   $143 = $jcol;
   $144 = $relax_end;
   $145 = (($144) + ($143<<2)|0);
   $146 = HEAP32[$145>>2]|0;
   $kcol = $146;
   $147 = $kcol;
   $148 = $jcol;
   $149 = (($147) - ($148))|0;
   $150 = (($149) + 1)|0;
   $151 = $panel_histo;
   $152 = (($151) + ($150<<2)|0);
   $153 = HEAP32[$152>>2]|0;
   $154 = (($153) + 1)|0;
   HEAP32[$152>>2] = $154;
   $155 = $jcol;
   $156 = $kcol;
   $157 = $asub;
   $158 = $xa_begin;
   $159 = $xa_end;
   $160 = HEAP32[$xprune>>2]|0;
   $161 = HEAP32[$marker>>2]|0;
   $162 = (_zsnode_dfs($155,$156,$157,$158,$159,$160,$161,2264)|0);
   $163 = $12;
   HEAP32[$163>>2] = $162;
   $164 = ($162|0)!=(0);
   if ($164) {
    label = 60;
    break;
   }
   $165 = $jcol;
   $166 = $xusub;
   $167 = (($166) + ($165<<2)|0);
   $168 = HEAP32[$167>>2]|0;
   $nextu = $168;
   $169 = $jcol;
   $170 = $xlusup;
   $171 = (($170) + ($169<<2)|0);
   $172 = HEAP32[$171>>2]|0;
   $nextlu = $172;
   $173 = $jcol;
   $174 = $supno;
   $175 = (($174) + ($173<<2)|0);
   $176 = HEAP32[$175>>2]|0;
   $jsupno = $176;
   $177 = $jsupno;
   $178 = $xsup;
   $179 = (($178) + ($177<<2)|0);
   $180 = HEAP32[$179>>2]|0;
   $fsupc = $180;
   $181 = $nextlu;
   $182 = $fsupc;
   $183 = (($182) + 1)|0;
   $184 = $xlsub;
   $185 = (($184) + ($183<<2)|0);
   $186 = HEAP32[$185>>2]|0;
   $187 = $fsupc;
   $188 = $xlsub;
   $189 = (($188) + ($187<<2)|0);
   $190 = HEAP32[$189>>2]|0;
   $191 = (($186) - ($190))|0;
   $192 = $kcol;
   $193 = $jcol;
   $194 = (($192) - ($193))|0;
   $195 = (($194) + 1)|0;
   $196 = Math_imul($191, $195)|0;
   $197 = (($181) + ($196))|0;
   $new_next = $197;
   $198 = HEAP32[(2308)>>2]|0;
   HEAP32[$nzlumax>>2] = $198;
   while(1) {
    $199 = $new_next;
    $200 = HEAP32[$nzlumax>>2]|0;
    $201 = ($199|0)>($200|0);
    $202 = $jcol;
    if (!($201)) {
     break;
    }
    $203 = $nextlu;
    $204 = (_zLUMemXpand($202,$203,0,$nzlumax,2264)|0);
    $205 = $12;
    HEAP32[$205>>2] = $204;
    $206 = ($204|0)!=(0);
    if ($206) {
     label = 60;
     break L19;
    }
   }
   $icol = $202;
   while(1) {
    $207 = $icol;
    $208 = $kcol;
    $209 = ($207|0)<=($208|0);
    if (!($209)) {
     break;
    }
    $210 = $nextu;
    $211 = $icol;
    $212 = (($211) + 1)|0;
    $213 = $xusub;
    $214 = (($213) + ($212<<2)|0);
    HEAP32[$214>>2] = $210;
    $215 = $icol;
    $216 = $xa_begin;
    $217 = (($216) + ($215<<2)|0);
    $218 = HEAP32[$217>>2]|0;
    $k = $218;
    while(1) {
     $219 = $k;
     $220 = $icol;
     $221 = $xa_end;
     $222 = (($221) + ($220<<2)|0);
     $223 = HEAP32[$222>>2]|0;
     $224 = ($219|0)<($223|0);
     if (!($224)) {
      break;
     }
     $225 = $k;
     $226 = $asub;
     $227 = (($226) + ($225<<2)|0);
     $228 = HEAP32[$227>>2]|0;
     $229 = HEAP32[$dense>>2]|0;
     $230 = (($229) + ($228<<4)|0);
     $231 = $k;
     $232 = $a;
     $233 = (($232) + ($231<<4)|0);
     ;HEAP32[$230>>2]=HEAP32[$233>>2]|0;HEAP32[$230+4>>2]=HEAP32[$233+4>>2]|0;HEAP32[$230+8>>2]=HEAP32[$233+8>>2]|0;HEAP32[$230+12>>2]=HEAP32[$233+12>>2]|0;
     $234 = $k;
     $235 = (($234) + 1)|0;
     $k = $235;
    }
    $236 = $icol;
    $237 = $jsupno;
    $238 = $fsupc;
    $239 = HEAP32[$dense>>2]|0;
    $240 = HEAP32[$tempv>>2]|0;
    $241 = $11;
    (_zsnode_bmod($236,$237,$238,$239,$240,2264,$241)|0);
    $242 = $icol;
    $243 = $diag_pivot_thresh;
    $244 = $8;
    $245 = $iperm_r;
    $246 = $iperm_c;
    $247 = $11;
    $248 = (_zpivotL($242,$243,$usepr,$244,$245,$246,$pivrow,2264,$247)|0);
    $249 = $12;
    HEAP32[$249>>2] = $248;
    $250 = ($248|0)!=(0);
    $251 = $iinfo;
    $252 = ($251|0)==(0);
    $or$cond = $250 & $252;
    if ($or$cond) {
     $253 = $12;
     $254 = HEAP32[$253>>2]|0;
     $iinfo = $254;
    }
    $255 = $icol;
    $256 = (($255) + 1)|0;
    $icol = $256;
   }
   $257 = $icol;
   $jcol = $257;
   continue;
  }
  $258 = $w_def;
  $3 = $258;
  $259 = $jcol;
  $260 = (($259) + 1)|0;
  $k = $260;
  while(1) {
   $261 = $k;
   $262 = $jcol;
   $263 = $3;
   $264 = (($262) + ($263))|0;
   $265 = $min_mn;
   $266 = ($264|0)<($265|0);
   if ($266) {
    $267 = $jcol;
    $268 = $3;
    $269 = (($267) + ($268))|0;
    $272 = $269;
   } else {
    $270 = $min_mn;
    $272 = $270;
   }
   $271 = ($261|0)<($272|0);
   if (!($271)) {
    break;
   }
   $273 = $k;
   $274 = $relax_end;
   $275 = (($274) + ($273<<2)|0);
   $276 = HEAP32[$275>>2]|0;
   $277 = ($276|0)!=(-1);
   $278 = $k;
   if ($277) {
    label = 35;
    break;
   }
   $281 = (($278) + 1)|0;
   $k = $281;
  }
  if ((label|0) == 35) {
   label = 0;
   $279 = $jcol;
   $280 = (($278) - ($279))|0;
   $3 = $280;
  }
  $282 = $k;
  $283 = $min_mn;
  $284 = ($282|0)==($283|0);
  if ($284) {
   $285 = $min_mn;
   $286 = $jcol;
   $287 = (($285) - ($286))|0;
   $3 = $287;
  }
  $288 = $3;
  $289 = $panel_histo;
  $290 = (($289) + ($288<<2)|0);
  $291 = HEAP32[$290>>2]|0;
  $292 = (($291) + 1)|0;
  HEAP32[$290>>2] = $292;
  $293 = $m;
  $294 = $3;
  $295 = $jcol;
  $296 = $1;
  $297 = $8;
  $298 = HEAP32[$dense>>2]|0;
  $299 = HEAP32[$panel_lsub>>2]|0;
  $300 = HEAP32[$segrep>>2]|0;
  $301 = HEAP32[$repfnz>>2]|0;
  $302 = HEAP32[$xprune>>2]|0;
  $303 = HEAP32[$marker>>2]|0;
  $304 = HEAP32[$parent>>2]|0;
  $305 = HEAP32[$xplore>>2]|0;
  _zpanel_dfs($293,$294,$295,$296,$297,$nseg1,$298,$299,$300,$301,$302,$303,$304,$305,2264);
  $306 = $m;
  $307 = $3;
  $308 = $jcol;
  $309 = HEAP32[$nseg1>>2]|0;
  $310 = HEAP32[$dense>>2]|0;
  $311 = HEAP32[$tempv>>2]|0;
  $312 = HEAP32[$segrep>>2]|0;
  $313 = HEAP32[$repfnz>>2]|0;
  $314 = $11;
  _zpanel_bmod($306,$307,$308,$309,$310,$311,$312,$313,2264,$314);
  $315 = $jcol;
  $jj = $315;
  while(1) {
   $316 = $jj;
   $317 = $jcol;
   $318 = $3;
   $319 = (($317) + ($318))|0;
   $320 = ($316|0)<($319|0);
   if (!($320)) {
    break;
   }
   $321 = $jj;
   $322 = $jcol;
   $323 = (($321) - ($322))|0;
   $324 = $m;
   $325 = Math_imul($323, $324)|0;
   $k = $325;
   $326 = HEAP32[$nseg1>>2]|0;
   HEAP32[$nseg>>2] = $326;
   $327 = $m;
   $328 = $jj;
   $329 = $8;
   $330 = $k;
   $331 = HEAP32[$panel_lsub>>2]|0;
   $332 = (($331) + ($330<<2)|0);
   $333 = HEAP32[$segrep>>2]|0;
   $334 = $k;
   $335 = HEAP32[$repfnz>>2]|0;
   $336 = (($335) + ($334<<2)|0);
   $337 = HEAP32[$xprune>>2]|0;
   $338 = HEAP32[$marker>>2]|0;
   $339 = HEAP32[$parent>>2]|0;
   $340 = HEAP32[$xplore>>2]|0;
   $341 = (_zcolumn_dfs($327,$328,$329,$nseg,$332,$333,$336,$337,$338,$339,$340,2264)|0);
   $342 = $12;
   HEAP32[$342>>2] = $341;
   $343 = ($341|0)!=(0);
   if ($343) {
    label = 60;
    break L19;
   }
   $344 = $jj;
   $345 = HEAP32[$nseg>>2]|0;
   $346 = HEAP32[$nseg1>>2]|0;
   $347 = (($345) - ($346))|0;
   $348 = $k;
   $349 = HEAP32[$dense>>2]|0;
   $350 = (($349) + ($348<<4)|0);
   $351 = HEAP32[$tempv>>2]|0;
   $352 = HEAP32[$nseg1>>2]|0;
   $353 = HEAP32[$segrep>>2]|0;
   $354 = (($353) + ($352<<2)|0);
   $355 = $k;
   $356 = HEAP32[$repfnz>>2]|0;
   $357 = (($356) + ($355<<2)|0);
   $358 = $jcol;
   $359 = $11;
   $360 = (_zcolumn_bmod($344,$347,$350,$351,$354,$357,$358,2264,$359)|0);
   $361 = $12;
   HEAP32[$361>>2] = $360;
   $362 = ($360|0)!=(0);
   if ($362) {
    label = 60;
    break L19;
   }
   $363 = $jj;
   $364 = HEAP32[$nseg>>2]|0;
   $365 = HEAP32[$segrep>>2]|0;
   $366 = $k;
   $367 = HEAP32[$repfnz>>2]|0;
   $368 = (($367) + ($366<<2)|0);
   $369 = $8;
   $370 = $k;
   $371 = HEAP32[$dense>>2]|0;
   $372 = (($371) + ($370<<4)|0);
   $373 = (_zcopy_to_ucol($363,$364,$365,$368,$369,$372,2264)|0);
   $374 = $12;
   HEAP32[$374>>2] = $373;
   $375 = ($373|0)!=(0);
   if ($375) {
    label = 60;
    break L19;
   }
   $376 = $jj;
   $377 = $diag_pivot_thresh;
   $378 = $8;
   $379 = $iperm_r;
   $380 = $iperm_c;
   $381 = $11;
   $382 = (_zpivotL($376,$377,$usepr,$378,$379,$380,$pivrow,2264,$381)|0);
   $383 = $12;
   HEAP32[$383>>2] = $382;
   $384 = ($382|0)!=(0);
   $385 = $iinfo;
   $386 = ($385|0)==(0);
   $or$cond3 = $384 & $386;
   if ($or$cond3) {
    $387 = $12;
    $388 = HEAP32[$387>>2]|0;
    $iinfo = $388;
   }
   $389 = $jj;
   $390 = $8;
   $391 = HEAP32[$pivrow>>2]|0;
   $392 = HEAP32[$nseg>>2]|0;
   $393 = HEAP32[$segrep>>2]|0;
   $394 = $k;
   $395 = HEAP32[$repfnz>>2]|0;
   $396 = (($395) + ($394<<2)|0);
   $397 = HEAP32[$xprune>>2]|0;
   _zpruneL($389,$390,$391,$392,$393,$396,$397,2264);
   $398 = HEAP32[$nseg>>2]|0;
   $399 = HEAP32[$segrep>>2]|0;
   $400 = $k;
   $401 = HEAP32[$repfnz>>2]|0;
   $402 = (($401) + ($400<<2)|0);
   _resetrep_col($398,$399,$402);
   $403 = $jj;
   $404 = (($403) + 1)|0;
   $jj = $404;
  }
  $405 = $3;
  $406 = $jcol;
  $407 = (($406) + ($405))|0;
  $jcol = $407;
 }
 if ((label|0) == 48) {
  $408 = $iinfo;
  $409 = $12;
  HEAP32[$409>>2] = $408;
  $410 = $m;
  $411 = $n;
  $412 = ($410|0)>($411|0);
  L70: do {
   if ($412) {
    $k = 0;
    $i = 0;
    while(1) {
     $413 = $i;
     $414 = $m;
     $415 = ($413|0)<($414|0);
     if (!($415)) {
      break L70;
     }
     $416 = $i;
     $417 = $8;
     $418 = (($417) + ($416<<2)|0);
     $419 = HEAP32[$418>>2]|0;
     $420 = ($419|0)==(-1);
     if ($420) {
      $421 = $n;
      $422 = $k;
      $423 = (($421) + ($422))|0;
      $424 = $i;
      $425 = $8;
      $426 = (($425) + ($424<<2)|0);
      HEAP32[$426>>2] = $423;
      $427 = $k;
      $428 = (($427) + 1)|0;
      $k = $428;
     }
     $429 = $i;
     $430 = (($429) + 1)|0;
     $i = $430;
    }
   }
  } while(0);
  $431 = $min_mn;
  $432 = HEAP32[$xprune>>2]|0;
  _countnz($431,$432,$nnzL,$nnzU,2264);
  $433 = $min_mn;
  $434 = $8;
  _fixupL($433,$434,2264);
  $435 = HEAP32[$iwork>>2]|0;
  $436 = HEAP32[$zwork>>2]|0;
  _zLUWorkFree($435,$436,2264);
  $437 = $fact;
  $438 = ($437|0)==(2);
  if ($438) {
   $439 = HEAP32[$nnzL>>2]|0;
   $440 = $9;
   $441 = ((($440)) + 20|0);
   $442 = HEAP32[$441>>2]|0;
   HEAP32[$442>>2] = $439;
   $443 = $n;
   $444 = HEAP32[(2268)>>2]|0;
   $445 = (($444) + ($443<<2)|0);
   $446 = HEAP32[$445>>2]|0;
   $447 = $9;
   $448 = ((($447)) + 20|0);
   $449 = HEAP32[$448>>2]|0;
   $450 = ((($449)) + 4|0);
   HEAP32[$450>>2] = $446;
   $451 = HEAP32[(2280)>>2]|0;
   $452 = $9;
   $453 = ((($452)) + 20|0);
   $454 = HEAP32[$453>>2]|0;
   $455 = ((($454)) + 8|0);
   HEAP32[$455>>2] = $451;
   $456 = HEAP32[(2284)>>2]|0;
   $457 = $9;
   $458 = ((($457)) + 20|0);
   $459 = HEAP32[$458>>2]|0;
   $460 = ((($459)) + 12|0);
   HEAP32[$460>>2] = $456;
   $461 = HEAP32[(2272)>>2]|0;
   $462 = $9;
   $463 = ((($462)) + 20|0);
   $464 = HEAP32[$463>>2]|0;
   $465 = ((($464)) + 16|0);
   HEAP32[$465>>2] = $461;
   $466 = HEAP32[(2276)>>2]|0;
   $467 = $9;
   $468 = ((($467)) + 20|0);
   $469 = HEAP32[$468>>2]|0;
   $470 = ((($469)) + 20|0);
   HEAP32[$470>>2] = $466;
   $471 = HEAP32[$nnzU>>2]|0;
   $472 = $10;
   $473 = ((($472)) + 20|0);
   $474 = HEAP32[$473>>2]|0;
   HEAP32[$474>>2] = $471;
   $475 = HEAP32[(2288)>>2]|0;
   $476 = $10;
   $477 = ((($476)) + 20|0);
   $478 = HEAP32[$477>>2]|0;
   $479 = ((($478)) + 4|0);
   HEAP32[$479>>2] = $475;
   $480 = HEAP32[(2292)>>2]|0;
   $481 = $10;
   $482 = ((($481)) + 20|0);
   $483 = HEAP32[$482>>2]|0;
   $484 = ((($483)) + 8|0);
   HEAP32[$484>>2] = $480;
   $485 = HEAP32[(2296)>>2]|0;
   $486 = $10;
   $487 = ((($486)) + 20|0);
   $488 = HEAP32[$487>>2]|0;
   $489 = ((($488)) + 12|0);
   HEAP32[$489>>2] = $485;
  } else {
   $490 = $9;
   $491 = $1;
   $492 = ((($491)) + 12|0);
   $493 = HEAP32[$492>>2]|0;
   $494 = $min_mn;
   $495 = HEAP32[$nnzL>>2]|0;
   $496 = HEAP32[(2280)>>2]|0;
   $497 = HEAP32[(2284)>>2]|0;
   $498 = HEAP32[(2272)>>2]|0;
   $499 = HEAP32[(2276)>>2]|0;
   $500 = HEAP32[(2268)>>2]|0;
   $501 = HEAP32[2264>>2]|0;
   _zCreate_SuperNode_Matrix($490,$493,$494,$495,$496,$497,$498,$499,$500,$501,3,3,1);
   $502 = $10;
   $503 = $min_mn;
   $504 = $min_mn;
   $505 = HEAP32[$nnzU>>2]|0;
   $506 = HEAP32[(2288)>>2]|0;
   $507 = HEAP32[(2292)>>2]|0;
   $508 = HEAP32[(2296)>>2]|0;
   _zCreate_CompCol_Matrix($502,$503,$504,$505,$506,$507,$508,0,3,4);
  }
  $509 = $ops;
  $510 = ((($509)) + 52|0);
  $511 = +HEAPF32[$510>>2];
  $512 = $ops;
  $513 = ((($512)) + 56|0);
  $514 = +HEAPF32[$513>>2];
  $515 = $511 + $514;
  $516 = $ops;
  $517 = ((($516)) + 28|0);
  $518 = +HEAPF32[$517>>2];
  $519 = $518 + $515;
  HEAPF32[$517>>2] = $519;
  $520 = HEAP32[(2320)>>2]|0;
  $521 = (($520) + -1)|0;
  HEAP32[(2320)>>2] = $521;
  $522 = $11;
  $523 = ((($522)) + 20|0);
  HEAP32[$523>>2] = $521;
  $524 = $iperm_r_allocated;
  $525 = ($524|0)!=(0);
  if ($525) {
   $526 = $iperm_r;
   _superlu_free($526);
  }
  $527 = $iperm_c;
  _superlu_free($527);
  $528 = $relax_end;
  _superlu_free($528);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 60) {
  STACKTOP = sp;return;
 }
}
function _zgstrs($trans,$L,$U,$perm_c,$perm_r,$B,$stat,$info) {
 $trans = $trans|0;
 $L = $L|0;
 $U = $U|0;
 $perm_c = $perm_c|0;
 $perm_r = $perm_r|0;
 $B = $B|0;
 $stat = $stat|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0.0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0.0, $218 = 0.0, $219 = 0.0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0.0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0.0, $263 = 0.0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0.0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0.0, $274 = 0.0, $275 = 0.0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0.0, $281 = 0, $282 = 0, $283 = 0, $284 = 0.0, $285 = 0.0, $286 = 0, $287 = 0, $288 = 0, $289 = 0.0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0.0, $295 = 0.0, $296 = 0.0;
 var $297 = 0.0, $298 = 0.0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0.0, $304 = 0.0, $305 = 0.0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0.0;
 var $314 = 0, $315 = 0.0, $316 = 0.0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0.0, $375 = 0, $376 = 0, $377 = 0, $378 = 0.0, $379 = 0.0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0.0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0.0, $393 = 0.0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0.0, $462 = 0.0, $463 = 0.0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0.0, $531 = 0.0, $532 = 0.0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0.0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0.0, $562 = 0.0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0.0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0.0, $573 = 0.0, $574 = 0.0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0.0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0.0, $584 = 0.0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0.0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0.0, $594 = 0.0, $595 = 0.0, $596 = 0.0, $597 = 0.0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0.0, $603 = 0.0, $604 = 0.0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0.0, $613 = 0, $614 = 0.0, $615 = 0.0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0.0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Bmat = 0, $Bstore = 0, $Lstore = 0, $Lval = 0, $Ustore = 0, $Uval = 0, $ci = 0.0, $ci3 = 0.0;
 var $cr = 0.0, $cr2 = 0.0, $fsupc = 0, $i = 0, $incx = 0, $incy = 0, $iptr = 0, $irow = 0, $istart = 0, $j = 0, $jcol = 0, $k = 0, $ldb = 0, $luptr = 0, $msg = 0, $msg1 = 0, $n = 0, $nrhs = 0, $nrow = 0, $nsupc = 0;
 var $nsupr = 0, $or$cond = 0, $or$cond3 = 0, $rhs_work = 0, $soln = 0, $solve_ops = 0.0, $temp_comp = 0, $vararg_buffer = 0, $vararg_buffer6 = 0, $vararg_ptr10 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $work = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 736|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer6 = sp + 64|0;
 $vararg_buffer = sp + 48|0;
 $temp_comp = sp + 32|0;
 $i = sp + 120|0;
 $msg = sp + 472|0;
 $msg1 = sp + 216|0;
 $0 = $trans;
 $1 = $L;
 $2 = $U;
 $3 = $perm_c;
 $4 = $perm_r;
 $5 = $B;
 $6 = $stat;
 $7 = $info;
 $incx = 1;
 $incy = 1;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = $5;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 $Bstore = $11;
 $12 = $Bstore;
 $13 = HEAP32[$12>>2]|0;
 $ldb = $13;
 $14 = $5;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $nrhs = $16;
 $17 = $0;
 $18 = ($17|0)!=(0);
 $19 = $0;
 $20 = ($19|0)!=(1);
 $or$cond = $18 & $20;
 $21 = $0;
 $22 = ($21|0)!=(2);
 $or$cond3 = $or$cond & $22;
 do {
  if ($or$cond3) {
   $23 = $7;
   HEAP32[$23>>2] = -1;
  } else {
   $24 = $1;
   $25 = ((($24)) + 12|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = $1;
   $28 = ((($27)) + 16|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($26|0)!=($29|0);
   if (!($30)) {
    $31 = $1;
    $32 = ((($31)) + 12|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($33|0)<(0);
    if (!($34)) {
     $35 = $1;
     $36 = HEAP32[$35>>2]|0;
     $37 = ($36|0)!=(3);
     if (!($37)) {
      $38 = $1;
      $39 = ((($38)) + 4|0);
      $40 = HEAP32[$39>>2]|0;
      $41 = ($40|0)!=(3);
      if (!($41)) {
       $42 = $1;
       $43 = ((($42)) + 8|0);
       $44 = HEAP32[$43>>2]|0;
       $45 = ($44|0)!=(1);
       if (!($45)) {
        $47 = $2;
        $48 = ((($47)) + 12|0);
        $49 = HEAP32[$48>>2]|0;
        $50 = $2;
        $51 = ((($50)) + 16|0);
        $52 = HEAP32[$51>>2]|0;
        $53 = ($49|0)!=($52|0);
        if (!($53)) {
         $54 = $2;
         $55 = ((($54)) + 12|0);
         $56 = HEAP32[$55>>2]|0;
         $57 = ($56|0)<(0);
         if (!($57)) {
          $58 = $2;
          $59 = HEAP32[$58>>2]|0;
          $60 = ($59|0)!=(0);
          if (!($60)) {
           $61 = $2;
           $62 = ((($61)) + 4|0);
           $63 = HEAP32[$62>>2]|0;
           $64 = ($63|0)!=(3);
           if (!($64)) {
            $65 = $2;
            $66 = ((($65)) + 8|0);
            $67 = HEAP32[$66>>2]|0;
            $68 = ($67|0)!=(4);
            if (!($68)) {
             $70 = $ldb;
             $71 = $1;
             $72 = ((($71)) + 12|0);
             $73 = HEAP32[$72>>2]|0;
             $74 = (0)>($73|0);
             if ($74) {
              $79 = 0;
             } else {
              $75 = $1;
              $76 = ((($75)) + 12|0);
              $77 = HEAP32[$76>>2]|0;
              $79 = $77;
             }
             $78 = ($70|0)<($79|0);
             if (!($78)) {
              $80 = $5;
              $81 = HEAP32[$80>>2]|0;
              $82 = ($81|0)!=(6);
              if (!($82)) {
               $83 = $5;
               $84 = ((($83)) + 4|0);
               $85 = HEAP32[$84>>2]|0;
               $86 = ($85|0)!=(3);
               if (!($86)) {
                $87 = $5;
                $88 = ((($87)) + 8|0);
                $89 = HEAP32[$88>>2]|0;
                $90 = ($89|0)!=(0);
                if (!($90)) {
                 break;
                }
               }
              }
             }
             $91 = $7;
             HEAP32[$91>>2] = -6;
             break;
            }
           }
          }
         }
        }
        $69 = $7;
        HEAP32[$69>>2] = -3;
        break;
       }
      }
     }
    }
   }
   $46 = $7;
   HEAP32[$46>>2] = -2;
  }
 } while(0);
 $92 = $7;
 $93 = HEAP32[$92>>2]|0;
 $94 = ($93|0)!=(0);
 if ($94) {
  $95 = $7;
  $96 = HEAP32[$95>>2]|0;
  $97 = (0 - ($96))|0;
  HEAP32[$i>>2] = $97;
  (_xerbla_(2352,$i)|0);
  STACKTOP = sp;return;
 }
 $98 = $1;
 $99 = ((($98)) + 12|0);
 $100 = HEAP32[$99>>2]|0;
 $n = $100;
 $101 = $n;
 $102 = $nrhs;
 $103 = Math_imul($101, $102)|0;
 $104 = (_doublecomplexCalloc($103)|0);
 $work = $104;
 $105 = $work;
 $106 = ($105|0)!=(0|0);
 if (!($106)) {
  HEAP32[$vararg_buffer>>2] = 2360;
  $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 138;
  $vararg_ptr5 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr5>>2] = 2392;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $107 = $n;
 $108 = (_doublecomplexMalloc($107)|0);
 $soln = $108;
 $109 = $soln;
 $110 = ($109|0)!=(0|0);
 if (!($110)) {
  HEAP32[$vararg_buffer6>>2] = 2408;
  $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
  HEAP32[$vararg_ptr9>>2] = 140;
  $vararg_ptr10 = ((($vararg_buffer6)) + 8|0);
  HEAP32[$vararg_ptr10>>2] = 2392;
  (_sprintf($msg1,2864,$vararg_buffer6)|0);
  _superlu_abort_and_exit($msg1);
 }
 $111 = $Bstore;
 $112 = ((($111)) + 4|0);
 $113 = HEAP32[$112>>2]|0;
 $Bmat = $113;
 $114 = $1;
 $115 = ((($114)) + 20|0);
 $116 = HEAP32[$115>>2]|0;
 $Lstore = $116;
 $117 = $Lstore;
 $118 = ((($117)) + 8|0);
 $119 = HEAP32[$118>>2]|0;
 $Lval = $119;
 $120 = $2;
 $121 = ((($120)) + 20|0);
 $122 = HEAP32[$121>>2]|0;
 $Ustore = $122;
 $123 = $Ustore;
 $124 = ((($123)) + 4|0);
 $125 = HEAP32[$124>>2]|0;
 $Uval = $125;
 $solve_ops = 0.0;
 $126 = $0;
 $127 = ($126|0)==(0);
 HEAP32[$i>>2] = 0;
 L37: do {
  if ($127) {
   while(1) {
    $128 = HEAP32[$i>>2]|0;
    $129 = $nrhs;
    $130 = ($128|0)<($129|0);
    if (!($130)) {
     break;
    }
    $131 = HEAP32[$i>>2]|0;
    $132 = $ldb;
    $133 = Math_imul($131, $132)|0;
    $134 = $Bmat;
    $135 = (($134) + ($133<<4)|0);
    $rhs_work = $135;
    $k = 0;
    while(1) {
     $136 = $k;
     $137 = $n;
     $138 = ($136|0)<($137|0);
     if (!($138)) {
      break;
     }
     $139 = $k;
     $140 = $4;
     $141 = (($140) + ($139<<2)|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = $soln;
     $144 = (($143) + ($142<<4)|0);
     $145 = $k;
     $146 = $rhs_work;
     $147 = (($146) + ($145<<4)|0);
     ;HEAP32[$144>>2]=HEAP32[$147>>2]|0;HEAP32[$144+4>>2]=HEAP32[$147+4>>2]|0;HEAP32[$144+8>>2]=HEAP32[$147+8>>2]|0;HEAP32[$144+12>>2]=HEAP32[$147+12>>2]|0;
     $148 = $k;
     $149 = (($148) + 1)|0;
     $k = $149;
    }
    $k = 0;
    while(1) {
     $150 = $k;
     $151 = $n;
     $152 = ($150|0)<($151|0);
     if (!($152)) {
      break;
     }
     $153 = $k;
     $154 = $rhs_work;
     $155 = (($154) + ($153<<4)|0);
     $156 = $k;
     $157 = $soln;
     $158 = (($157) + ($156<<4)|0);
     ;HEAP32[$155>>2]=HEAP32[$158>>2]|0;HEAP32[$155+4>>2]=HEAP32[$158+4>>2]|0;HEAP32[$155+8>>2]=HEAP32[$158+8>>2]|0;HEAP32[$155+12>>2]=HEAP32[$158+12>>2]|0;
     $159 = $k;
     $160 = (($159) + 1)|0;
     $k = $160;
    }
    $161 = HEAP32[$i>>2]|0;
    $162 = (($161) + 1)|0;
    HEAP32[$i>>2] = $162;
   }
   $k = 0;
   while(1) {
    $163 = $k;
    $164 = $Lstore;
    $165 = ((($164)) + 4|0);
    $166 = HEAP32[$165>>2]|0;
    $167 = ($163|0)<=($166|0);
    if (!($167)) {
     break;
    }
    $168 = $k;
    $169 = $Lstore;
    $170 = ((($169)) + 28|0);
    $171 = HEAP32[$170>>2]|0;
    $172 = (($171) + ($168<<2)|0);
    $173 = HEAP32[$172>>2]|0;
    $fsupc = $173;
    $174 = $fsupc;
    $175 = $Lstore;
    $176 = ((($175)) + 20|0);
    $177 = HEAP32[$176>>2]|0;
    $178 = (($177) + ($174<<2)|0);
    $179 = HEAP32[$178>>2]|0;
    $istart = $179;
    $180 = $fsupc;
    $181 = (($180) + 1)|0;
    $182 = $Lstore;
    $183 = ((($182)) + 20|0);
    $184 = HEAP32[$183>>2]|0;
    $185 = (($184) + ($181<<2)|0);
    $186 = HEAP32[$185>>2]|0;
    $187 = $istart;
    $188 = (($186) - ($187))|0;
    $nsupr = $188;
    $189 = $k;
    $190 = (($189) + 1)|0;
    $191 = $Lstore;
    $192 = ((($191)) + 28|0);
    $193 = HEAP32[$192>>2]|0;
    $194 = (($193) + ($190<<2)|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = $fsupc;
    $197 = (($195) - ($196))|0;
    $nsupc = $197;
    $198 = $nsupr;
    $199 = $nsupc;
    $200 = (($198) - ($199))|0;
    $nrow = $200;
    $201 = $nsupc;
    $202 = $201<<2;
    $203 = $nsupc;
    $204 = (($203) - 1)|0;
    $205 = Math_imul($202, $204)|0;
    $206 = $nrhs;
    $207 = Math_imul($205, $206)|0;
    $208 = (+($207|0));
    $209 = $solve_ops;
    $210 = $209 + $208;
    $solve_ops = $210;
    $211 = $nrow;
    $212 = $211<<3;
    $213 = $nsupc;
    $214 = Math_imul($212, $213)|0;
    $215 = $nrhs;
    $216 = Math_imul($214, $215)|0;
    $217 = (+($216|0));
    $218 = $solve_ops;
    $219 = $218 + $217;
    $solve_ops = $219;
    $220 = $nsupc;
    $221 = ($220|0)==(1);
    L84: do {
     if ($221) {
      $j = 0;
      while(1) {
       $222 = $j;
       $223 = $nrhs;
       $224 = ($222|0)<($223|0);
       if (!($224)) {
        break L84;
       }
       $225 = $j;
       $226 = $ldb;
       $227 = Math_imul($225, $226)|0;
       $228 = $Bmat;
       $229 = (($228) + ($227<<4)|0);
       $rhs_work = $229;
       $230 = $fsupc;
       $231 = $Lstore;
       $232 = ((($231)) + 12|0);
       $233 = HEAP32[$232>>2]|0;
       $234 = (($233) + ($230<<2)|0);
       $235 = HEAP32[$234>>2]|0;
       $luptr = $235;
       $236 = $istart;
       $237 = (($236) + 1)|0;
       $iptr = $237;
       while(1) {
        $238 = $iptr;
        $239 = $fsupc;
        $240 = (($239) + 1)|0;
        $241 = $Lstore;
        $242 = ((($241)) + 20|0);
        $243 = HEAP32[$242>>2]|0;
        $244 = (($243) + ($240<<2)|0);
        $245 = HEAP32[$244>>2]|0;
        $246 = ($238|0)<($245|0);
        if (!($246)) {
         break;
        }
        $247 = $iptr;
        $248 = $Lstore;
        $249 = ((($248)) + 16|0);
        $250 = HEAP32[$249>>2]|0;
        $251 = (($250) + ($247<<2)|0);
        $252 = HEAP32[$251>>2]|0;
        $irow = $252;
        $253 = $luptr;
        $254 = (($253) + 1)|0;
        $luptr = $254;
        $255 = $fsupc;
        $256 = $rhs_work;
        $257 = (($256) + ($255<<4)|0);
        $258 = +HEAPF64[$257>>3];
        $259 = $luptr;
        $260 = $Lval;
        $261 = (($260) + ($259<<4)|0);
        $262 = +HEAPF64[$261>>3];
        $263 = $258 * $262;
        $264 = $fsupc;
        $265 = $rhs_work;
        $266 = (($265) + ($264<<4)|0);
        $267 = ((($266)) + 8|0);
        $268 = +HEAPF64[$267>>3];
        $269 = $luptr;
        $270 = $Lval;
        $271 = (($270) + ($269<<4)|0);
        $272 = ((($271)) + 8|0);
        $273 = +HEAPF64[$272>>3];
        $274 = $268 * $273;
        $275 = $263 - $274;
        $cr = $275;
        $276 = $fsupc;
        $277 = $rhs_work;
        $278 = (($277) + ($276<<4)|0);
        $279 = ((($278)) + 8|0);
        $280 = +HEAPF64[$279>>3];
        $281 = $luptr;
        $282 = $Lval;
        $283 = (($282) + ($281<<4)|0);
        $284 = +HEAPF64[$283>>3];
        $285 = $280 * $284;
        $286 = $fsupc;
        $287 = $rhs_work;
        $288 = (($287) + ($286<<4)|0);
        $289 = +HEAPF64[$288>>3];
        $290 = $luptr;
        $291 = $Lval;
        $292 = (($291) + ($290<<4)|0);
        $293 = ((($292)) + 8|0);
        $294 = +HEAPF64[$293>>3];
        $295 = $289 * $294;
        $296 = $285 + $295;
        $ci = $296;
        $297 = $cr;
        HEAPF64[$temp_comp>>3] = $297;
        $298 = $ci;
        $299 = ((($temp_comp)) + 8|0);
        HEAPF64[$299>>3] = $298;
        $300 = $irow;
        $301 = $rhs_work;
        $302 = (($301) + ($300<<4)|0);
        $303 = +HEAPF64[$302>>3];
        $304 = +HEAPF64[$temp_comp>>3];
        $305 = $303 - $304;
        $306 = $irow;
        $307 = $rhs_work;
        $308 = (($307) + ($306<<4)|0);
        HEAPF64[$308>>3] = $305;
        $309 = $irow;
        $310 = $rhs_work;
        $311 = (($310) + ($309<<4)|0);
        $312 = ((($311)) + 8|0);
        $313 = +HEAPF64[$312>>3];
        $314 = ((($temp_comp)) + 8|0);
        $315 = +HEAPF64[$314>>3];
        $316 = $313 - $315;
        $317 = $irow;
        $318 = $rhs_work;
        $319 = (($318) + ($317<<4)|0);
        $320 = ((($319)) + 8|0);
        HEAPF64[$320>>3] = $316;
        $321 = $iptr;
        $322 = (($321) + 1)|0;
        $iptr = $322;
       }
       $323 = $j;
       $324 = (($323) + 1)|0;
       $j = $324;
      }
     } else {
      $325 = $fsupc;
      $326 = $Lstore;
      $327 = ((($326)) + 12|0);
      $328 = HEAP32[$327>>2]|0;
      $329 = (($328) + ($325<<2)|0);
      $330 = HEAP32[$329>>2]|0;
      $luptr = $330;
      $j = 0;
      while(1) {
       $331 = $j;
       $332 = $nrhs;
       $333 = ($331|0)<($332|0);
       if (!($333)) {
        break L84;
       }
       $334 = $j;
       $335 = $ldb;
       $336 = Math_imul($334, $335)|0;
       $337 = $Bmat;
       $338 = (($337) + ($336<<4)|0);
       $rhs_work = $338;
       $339 = $nsupr;
       $340 = $nsupc;
       $341 = $luptr;
       $342 = $Lval;
       $343 = (($342) + ($341<<4)|0);
       $344 = $fsupc;
       $345 = $rhs_work;
       $346 = (($345) + ($344<<4)|0);
       _zlsolve($339,$340,$343,$346);
       $347 = $nsupr;
       $348 = $nrow;
       $349 = $nsupc;
       $350 = $luptr;
       $351 = $nsupc;
       $352 = (($350) + ($351))|0;
       $353 = $Lval;
       $354 = (($353) + ($352<<4)|0);
       $355 = $fsupc;
       $356 = $rhs_work;
       $357 = (($356) + ($355<<4)|0);
       $358 = $work;
       _zmatvec($347,$348,$349,$354,$357,$358);
       $359 = $istart;
       $360 = $nsupc;
       $361 = (($359) + ($360))|0;
       $iptr = $361;
       HEAP32[$i>>2] = 0;
       while(1) {
        $362 = HEAP32[$i>>2]|0;
        $363 = $nrow;
        $364 = ($362|0)<($363|0);
        if (!($364)) {
         break;
        }
        $365 = $iptr;
        $366 = $Lstore;
        $367 = ((($366)) + 16|0);
        $368 = HEAP32[$367>>2]|0;
        $369 = (($368) + ($365<<2)|0);
        $370 = HEAP32[$369>>2]|0;
        $irow = $370;
        $371 = $irow;
        $372 = $rhs_work;
        $373 = (($372) + ($371<<4)|0);
        $374 = +HEAPF64[$373>>3];
        $375 = HEAP32[$i>>2]|0;
        $376 = $work;
        $377 = (($376) + ($375<<4)|0);
        $378 = +HEAPF64[$377>>3];
        $379 = $374 - $378;
        $380 = $irow;
        $381 = $rhs_work;
        $382 = (($381) + ($380<<4)|0);
        HEAPF64[$382>>3] = $379;
        $383 = $irow;
        $384 = $rhs_work;
        $385 = (($384) + ($383<<4)|0);
        $386 = ((($385)) + 8|0);
        $387 = +HEAPF64[$386>>3];
        $388 = HEAP32[$i>>2]|0;
        $389 = $work;
        $390 = (($389) + ($388<<4)|0);
        $391 = ((($390)) + 8|0);
        $392 = +HEAPF64[$391>>3];
        $393 = $387 - $392;
        $394 = $irow;
        $395 = $rhs_work;
        $396 = (($395) + ($394<<4)|0);
        $397 = ((($396)) + 8|0);
        HEAPF64[$397>>3] = $393;
        $398 = HEAP32[$i>>2]|0;
        $399 = $work;
        $400 = (($399) + ($398<<4)|0);
        HEAPF64[$400>>3] = 0.0;
        $401 = HEAP32[$i>>2]|0;
        $402 = $work;
        $403 = (($402) + ($401<<4)|0);
        $404 = ((($403)) + 8|0);
        HEAPF64[$404>>3] = 0.0;
        $405 = $iptr;
        $406 = (($405) + 1)|0;
        $iptr = $406;
        $407 = HEAP32[$i>>2]|0;
        $408 = (($407) + 1)|0;
        HEAP32[$i>>2] = $408;
       }
       $409 = $j;
       $410 = (($409) + 1)|0;
       $j = $410;
      }
     }
    } while(0);
    $411 = $k;
    $412 = (($411) + 1)|0;
    $k = $412;
   }
   $413 = $Lstore;
   $414 = ((($413)) + 4|0);
   $415 = HEAP32[$414>>2]|0;
   $k = $415;
   while(1) {
    $416 = $k;
    $417 = ($416|0)>=(0);
    if (!($417)) {
     break;
    }
    $418 = $k;
    $419 = $Lstore;
    $420 = ((($419)) + 28|0);
    $421 = HEAP32[$420>>2]|0;
    $422 = (($421) + ($418<<2)|0);
    $423 = HEAP32[$422>>2]|0;
    $fsupc = $423;
    $424 = $fsupc;
    $425 = $Lstore;
    $426 = ((($425)) + 20|0);
    $427 = HEAP32[$426>>2]|0;
    $428 = (($427) + ($424<<2)|0);
    $429 = HEAP32[$428>>2]|0;
    $istart = $429;
    $430 = $fsupc;
    $431 = (($430) + 1)|0;
    $432 = $Lstore;
    $433 = ((($432)) + 20|0);
    $434 = HEAP32[$433>>2]|0;
    $435 = (($434) + ($431<<2)|0);
    $436 = HEAP32[$435>>2]|0;
    $437 = $istart;
    $438 = (($436) - ($437))|0;
    $nsupr = $438;
    $439 = $k;
    $440 = (($439) + 1)|0;
    $441 = $Lstore;
    $442 = ((($441)) + 28|0);
    $443 = HEAP32[$442>>2]|0;
    $444 = (($443) + ($440<<2)|0);
    $445 = HEAP32[$444>>2]|0;
    $446 = $fsupc;
    $447 = (($445) - ($446))|0;
    $nsupc = $447;
    $448 = $fsupc;
    $449 = $Lstore;
    $450 = ((($449)) + 12|0);
    $451 = HEAP32[$450>>2]|0;
    $452 = (($451) + ($448<<2)|0);
    $453 = HEAP32[$452>>2]|0;
    $luptr = $453;
    $454 = $nsupc;
    $455 = $454<<2;
    $456 = $nsupc;
    $457 = (($456) + 1)|0;
    $458 = Math_imul($455, $457)|0;
    $459 = $nrhs;
    $460 = Math_imul($458, $459)|0;
    $461 = (+($460|0));
    $462 = $solve_ops;
    $463 = $462 + $461;
    $solve_ops = $463;
    $464 = $nsupc;
    $465 = ($464|0)==(1);
    L106: do {
     if ($465) {
      $466 = $Bmat;
      $rhs_work = $466;
      $j = 0;
      while(1) {
       $467 = $j;
       $468 = $nrhs;
       $469 = ($467|0)<($468|0);
       if (!($469)) {
        break L106;
       }
       $470 = $fsupc;
       $471 = $rhs_work;
       $472 = (($471) + ($470<<4)|0);
       $473 = $fsupc;
       $474 = $rhs_work;
       $475 = (($474) + ($473<<4)|0);
       $476 = $luptr;
       $477 = $Lval;
       $478 = (($477) + ($476<<4)|0);
       _z_div($472,$475,$478);
       $479 = $ldb;
       $480 = $rhs_work;
       $481 = (($480) + ($479<<4)|0);
       $rhs_work = $481;
       $482 = $j;
       $483 = (($482) + 1)|0;
       $j = $483;
      }
     } else {
      $j = 0;
      while(1) {
       $484 = $j;
       $485 = $nrhs;
       $486 = ($484|0)<($485|0);
       if (!($486)) {
        break L106;
       }
       $487 = $nsupr;
       $488 = $nsupc;
       $489 = $luptr;
       $490 = $Lval;
       $491 = (($490) + ($489<<4)|0);
       $492 = $fsupc;
       $493 = $j;
       $494 = $ldb;
       $495 = Math_imul($493, $494)|0;
       $496 = (($492) + ($495))|0;
       $497 = $Bmat;
       $498 = (($497) + ($496<<4)|0);
       _zusolve($487,$488,$491,$498);
       $499 = $j;
       $500 = (($499) + 1)|0;
       $j = $500;
      }
     }
    } while(0);
    $j = 0;
    while(1) {
     $501 = $j;
     $502 = $nrhs;
     $503 = ($501|0)<($502|0);
     if (!($503)) {
      break;
     }
     $504 = $j;
     $505 = $ldb;
     $506 = Math_imul($504, $505)|0;
     $507 = $Bmat;
     $508 = (($507) + ($506<<4)|0);
     $rhs_work = $508;
     $509 = $fsupc;
     $jcol = $509;
     while(1) {
      $510 = $jcol;
      $511 = $fsupc;
      $512 = $nsupc;
      $513 = (($511) + ($512))|0;
      $514 = ($510|0)<($513|0);
      if (!($514)) {
       break;
      }
      $515 = $jcol;
      $516 = (($515) + 1)|0;
      $517 = $Ustore;
      $518 = ((($517)) + 12|0);
      $519 = HEAP32[$518>>2]|0;
      $520 = (($519) + ($516<<2)|0);
      $521 = HEAP32[$520>>2]|0;
      $522 = $jcol;
      $523 = $Ustore;
      $524 = ((($523)) + 12|0);
      $525 = HEAP32[$524>>2]|0;
      $526 = (($525) + ($522<<2)|0);
      $527 = HEAP32[$526>>2]|0;
      $528 = (($521) - ($527))|0;
      $529 = $528<<3;
      $530 = (+($529|0));
      $531 = $solve_ops;
      $532 = $531 + $530;
      $solve_ops = $532;
      $533 = $jcol;
      $534 = $Ustore;
      $535 = ((($534)) + 12|0);
      $536 = HEAP32[$535>>2]|0;
      $537 = (($536) + ($533<<2)|0);
      $538 = HEAP32[$537>>2]|0;
      HEAP32[$i>>2] = $538;
      while(1) {
       $539 = HEAP32[$i>>2]|0;
       $540 = $jcol;
       $541 = (($540) + 1)|0;
       $542 = $Ustore;
       $543 = ((($542)) + 12|0);
       $544 = HEAP32[$543>>2]|0;
       $545 = (($544) + ($541<<2)|0);
       $546 = HEAP32[$545>>2]|0;
       $547 = ($539|0)<($546|0);
       if (!($547)) {
        break;
       }
       $548 = HEAP32[$i>>2]|0;
       $549 = $Ustore;
       $550 = ((($549)) + 8|0);
       $551 = HEAP32[$550>>2]|0;
       $552 = (($551) + ($548<<2)|0);
       $553 = HEAP32[$552>>2]|0;
       $irow = $553;
       $554 = $jcol;
       $555 = $rhs_work;
       $556 = (($555) + ($554<<4)|0);
       $557 = +HEAPF64[$556>>3];
       $558 = HEAP32[$i>>2]|0;
       $559 = $Uval;
       $560 = (($559) + ($558<<4)|0);
       $561 = +HEAPF64[$560>>3];
       $562 = $557 * $561;
       $563 = $jcol;
       $564 = $rhs_work;
       $565 = (($564) + ($563<<4)|0);
       $566 = ((($565)) + 8|0);
       $567 = +HEAPF64[$566>>3];
       $568 = HEAP32[$i>>2]|0;
       $569 = $Uval;
       $570 = (($569) + ($568<<4)|0);
       $571 = ((($570)) + 8|0);
       $572 = +HEAPF64[$571>>3];
       $573 = $567 * $572;
       $574 = $562 - $573;
       $cr2 = $574;
       $575 = $jcol;
       $576 = $rhs_work;
       $577 = (($576) + ($575<<4)|0);
       $578 = ((($577)) + 8|0);
       $579 = +HEAPF64[$578>>3];
       $580 = HEAP32[$i>>2]|0;
       $581 = $Uval;
       $582 = (($581) + ($580<<4)|0);
       $583 = +HEAPF64[$582>>3];
       $584 = $579 * $583;
       $585 = $jcol;
       $586 = $rhs_work;
       $587 = (($586) + ($585<<4)|0);
       $588 = +HEAPF64[$587>>3];
       $589 = HEAP32[$i>>2]|0;
       $590 = $Uval;
       $591 = (($590) + ($589<<4)|0);
       $592 = ((($591)) + 8|0);
       $593 = +HEAPF64[$592>>3];
       $594 = $588 * $593;
       $595 = $584 + $594;
       $ci3 = $595;
       $596 = $cr2;
       HEAPF64[$temp_comp>>3] = $596;
       $597 = $ci3;
       $598 = ((($temp_comp)) + 8|0);
       HEAPF64[$598>>3] = $597;
       $599 = $irow;
       $600 = $rhs_work;
       $601 = (($600) + ($599<<4)|0);
       $602 = +HEAPF64[$601>>3];
       $603 = +HEAPF64[$temp_comp>>3];
       $604 = $602 - $603;
       $605 = $irow;
       $606 = $rhs_work;
       $607 = (($606) + ($605<<4)|0);
       HEAPF64[$607>>3] = $604;
       $608 = $irow;
       $609 = $rhs_work;
       $610 = (($609) + ($608<<4)|0);
       $611 = ((($610)) + 8|0);
       $612 = +HEAPF64[$611>>3];
       $613 = ((($temp_comp)) + 8|0);
       $614 = +HEAPF64[$613>>3];
       $615 = $612 - $614;
       $616 = $irow;
       $617 = $rhs_work;
       $618 = (($617) + ($616<<4)|0);
       $619 = ((($618)) + 8|0);
       HEAPF64[$619>>3] = $615;
       $620 = HEAP32[$i>>2]|0;
       $621 = (($620) + 1)|0;
       HEAP32[$i>>2] = $621;
      }
      $622 = $jcol;
      $623 = (($622) + 1)|0;
      $jcol = $623;
     }
     $624 = $j;
     $625 = (($624) + 1)|0;
     $j = $625;
    }
    $626 = $k;
    $627 = (($626) + -1)|0;
    $k = $627;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $628 = HEAP32[$i>>2]|0;
    $629 = $nrhs;
    $630 = ($628|0)<($629|0);
    if (!($630)) {
     break;
    }
    $631 = HEAP32[$i>>2]|0;
    $632 = $ldb;
    $633 = Math_imul($631, $632)|0;
    $634 = $Bmat;
    $635 = (($634) + ($633<<4)|0);
    $rhs_work = $635;
    $k = 0;
    while(1) {
     $636 = $k;
     $637 = $n;
     $638 = ($636|0)<($637|0);
     if (!($638)) {
      break;
     }
     $639 = $k;
     $640 = $soln;
     $641 = (($640) + ($639<<4)|0);
     $642 = $k;
     $643 = $3;
     $644 = (($643) + ($642<<2)|0);
     $645 = HEAP32[$644>>2]|0;
     $646 = $rhs_work;
     $647 = (($646) + ($645<<4)|0);
     ;HEAP32[$641>>2]=HEAP32[$647>>2]|0;HEAP32[$641+4>>2]=HEAP32[$647+4>>2]|0;HEAP32[$641+8>>2]=HEAP32[$647+8>>2]|0;HEAP32[$641+12>>2]=HEAP32[$647+12>>2]|0;
     $648 = $k;
     $649 = (($648) + 1)|0;
     $k = $649;
    }
    $k = 0;
    while(1) {
     $650 = $k;
     $651 = $n;
     $652 = ($650|0)<($651|0);
     if (!($652)) {
      break;
     }
     $653 = $k;
     $654 = $rhs_work;
     $655 = (($654) + ($653<<4)|0);
     $656 = $k;
     $657 = $soln;
     $658 = (($657) + ($656<<4)|0);
     ;HEAP32[$655>>2]=HEAP32[$658>>2]|0;HEAP32[$655+4>>2]=HEAP32[$658+4>>2]|0;HEAP32[$655+8>>2]=HEAP32[$658+8>>2]|0;HEAP32[$655+12>>2]=HEAP32[$658+12>>2]|0;
     $659 = $k;
     $660 = (($659) + 1)|0;
     $k = $660;
    }
    $661 = HEAP32[$i>>2]|0;
    $662 = (($661) + 1)|0;
    HEAP32[$i>>2] = $662;
   }
   $663 = $solve_ops;
   $664 = $6;
   $665 = ((($664)) + 8|0);
   $666 = HEAP32[$665>>2]|0;
   $667 = ((($666)) + 44|0);
   HEAPF32[$667>>2] = $663;
  } else {
   while(1) {
    $668 = HEAP32[$i>>2]|0;
    $669 = $nrhs;
    $670 = ($668|0)<($669|0);
    if (!($670)) {
     break;
    }
    $671 = HEAP32[$i>>2]|0;
    $672 = $ldb;
    $673 = Math_imul($671, $672)|0;
    $674 = $Bmat;
    $675 = (($674) + ($673<<4)|0);
    $rhs_work = $675;
    $k = 0;
    while(1) {
     $676 = $k;
     $677 = $n;
     $678 = ($676|0)<($677|0);
     if (!($678)) {
      break;
     }
     $679 = $k;
     $680 = $3;
     $681 = (($680) + ($679<<2)|0);
     $682 = HEAP32[$681>>2]|0;
     $683 = $soln;
     $684 = (($683) + ($682<<4)|0);
     $685 = $k;
     $686 = $rhs_work;
     $687 = (($686) + ($685<<4)|0);
     ;HEAP32[$684>>2]=HEAP32[$687>>2]|0;HEAP32[$684+4>>2]=HEAP32[$687+4>>2]|0;HEAP32[$684+8>>2]=HEAP32[$687+8>>2]|0;HEAP32[$684+12>>2]=HEAP32[$687+12>>2]|0;
     $688 = $k;
     $689 = (($688) + 1)|0;
     $k = $689;
    }
    $k = 0;
    while(1) {
     $690 = $k;
     $691 = $n;
     $692 = ($690|0)<($691|0);
     if (!($692)) {
      break;
     }
     $693 = $k;
     $694 = $rhs_work;
     $695 = (($694) + ($693<<4)|0);
     $696 = $k;
     $697 = $soln;
     $698 = (($697) + ($696<<4)|0);
     ;HEAP32[$695>>2]=HEAP32[$698>>2]|0;HEAP32[$695+4>>2]=HEAP32[$698+4>>2]|0;HEAP32[$695+8>>2]=HEAP32[$698+8>>2]|0;HEAP32[$695+12>>2]=HEAP32[$698+12>>2]|0;
     $699 = $k;
     $700 = (($699) + 1)|0;
     $k = $700;
    }
    $701 = HEAP32[$i>>2]|0;
    $702 = (($701) + 1)|0;
    HEAP32[$i>>2] = $702;
   }
   $703 = $6;
   $704 = ((($703)) + 8|0);
   $705 = HEAP32[$704>>2]|0;
   $706 = ((($705)) + 44|0);
   HEAPF32[$706>>2] = 0.0;
   $707 = $0;
   $708 = ($707|0)==(1);
   $k = 0;
   L50: do {
    if ($708) {
     while(1) {
      $709 = $k;
      $710 = $nrhs;
      $711 = ($709|0)<($710|0);
      if (!($711)) {
       break L50;
      }
      $712 = $1;
      $713 = $2;
      $714 = $k;
      $715 = $ldb;
      $716 = Math_imul($714, $715)|0;
      $717 = $Bmat;
      $718 = (($717) + ($716<<4)|0);
      $719 = $6;
      $720 = $7;
      (_sp_ztrsv(3392,3416,3408,$712,$713,$718,$719,$720)|0);
      $721 = $1;
      $722 = $2;
      $723 = $k;
      $724 = $ldb;
      $725 = Math_imul($723, $724)|0;
      $726 = $Bmat;
      $727 = (($726) + ($725<<4)|0);
      $728 = $6;
      $729 = $7;
      (_sp_ztrsv(3400,3416,3392,$721,$722,$727,$728,$729)|0);
      $730 = $k;
      $731 = (($730) + 1)|0;
      $k = $731;
     }
    } else {
     while(1) {
      $732 = $k;
      $733 = $nrhs;
      $734 = ($732|0)<($733|0);
      if (!($734)) {
       break L50;
      }
      $735 = $1;
      $736 = $2;
      $737 = $k;
      $738 = $ldb;
      $739 = Math_imul($737, $738)|0;
      $740 = $Bmat;
      $741 = (($740) + ($739<<4)|0);
      $742 = $6;
      $743 = $7;
      (_sp_ztrsv(3392,3424,3408,$735,$736,$741,$742,$743)|0);
      $744 = $1;
      $745 = $2;
      $746 = $k;
      $747 = $ldb;
      $748 = Math_imul($746, $747)|0;
      $749 = $Bmat;
      $750 = (($749) + ($748<<4)|0);
      $751 = $6;
      $752 = $7;
      (_sp_ztrsv(3400,3424,3392,$744,$745,$750,$751,$752)|0);
      $753 = $k;
      $754 = (($753) + 1)|0;
      $k = $754;
     }
    }
   } while(0);
   HEAP32[$i>>2] = 0;
   while(1) {
    $755 = HEAP32[$i>>2]|0;
    $756 = $nrhs;
    $757 = ($755|0)<($756|0);
    if (!($757)) {
     break L37;
    }
    $758 = HEAP32[$i>>2]|0;
    $759 = $ldb;
    $760 = Math_imul($758, $759)|0;
    $761 = $Bmat;
    $762 = (($761) + ($760<<4)|0);
    $rhs_work = $762;
    $k = 0;
    while(1) {
     $763 = $k;
     $764 = $n;
     $765 = ($763|0)<($764|0);
     if (!($765)) {
      break;
     }
     $766 = $k;
     $767 = $soln;
     $768 = (($767) + ($766<<4)|0);
     $769 = $k;
     $770 = $4;
     $771 = (($770) + ($769<<2)|0);
     $772 = HEAP32[$771>>2]|0;
     $773 = $rhs_work;
     $774 = (($773) + ($772<<4)|0);
     ;HEAP32[$768>>2]=HEAP32[$774>>2]|0;HEAP32[$768+4>>2]=HEAP32[$774+4>>2]|0;HEAP32[$768+8>>2]=HEAP32[$774+8>>2]|0;HEAP32[$768+12>>2]=HEAP32[$774+12>>2]|0;
     $775 = $k;
     $776 = (($775) + 1)|0;
     $k = $776;
    }
    $k = 0;
    while(1) {
     $777 = $k;
     $778 = $n;
     $779 = ($777|0)<($778|0);
     if (!($779)) {
      break;
     }
     $780 = $k;
     $781 = $rhs_work;
     $782 = (($781) + ($780<<4)|0);
     $783 = $k;
     $784 = $soln;
     $785 = (($784) + ($783<<4)|0);
     ;HEAP32[$782>>2]=HEAP32[$785>>2]|0;HEAP32[$782+4>>2]=HEAP32[$785+4>>2]|0;HEAP32[$782+8>>2]=HEAP32[$785+8>>2]|0;HEAP32[$782+12>>2]=HEAP32[$785+12>>2]|0;
     $786 = $k;
     $787 = (($786) + 1)|0;
     $k = $787;
    }
    $788 = HEAP32[$i>>2]|0;
    $789 = (($788) + 1)|0;
    HEAP32[$i>>2] = $789;
   }
  }
 } while(0);
 $790 = $work;
 _superlu_free($790);
 $791 = $soln;
 _superlu_free($791);
 STACKTOP = sp;return;
}
function _zcopy_to_ucol($jcol,$nseg,$segrep,$repfnz,$perm_r,$dense,$Glu) {
 $jcol = $jcol|0;
 $nseg = $nseg|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $perm_r = $perm_r|0;
 $dense = $dense|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $fsupc = 0, $i = 0, $irow = 0, $isub = 0, $jsupno = 0, $k = 0, $kfnz = 0, $krep = 0, $ksub = 0, $ksupno = 0, $lsub = 0, $mem_error = 0, $new_next = 0, $nextu = 0, $nzumax = 0, $segsze = 0, $supno = 0, $ucol = 0;
 var $usub = 0, $xlsub = 0, $xsup = 0, $xusub = 0, $zero = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $nzumax = sp + 16|0;
 $zero = sp;
 $1 = $jcol;
 $2 = $nseg;
 $3 = $segrep;
 $4 = $repfnz;
 $5 = $perm_r;
 $6 = $dense;
 $7 = $Glu;
 ;HEAP32[$zero>>2]=0|0;HEAP32[$zero+4>>2]=0|0;HEAP32[$zero+8>>2]=0|0;HEAP32[$zero+12>>2]=0|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $xsup = $9;
 $10 = $7;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $supno = $12;
 $13 = $7;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $lsub = $15;
 $16 = $7;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $xlsub = $18;
 $19 = $7;
 $20 = ((($19)) + 24|0);
 $21 = HEAP32[$20>>2]|0;
 $ucol = $21;
 $22 = $7;
 $23 = ((($22)) + 28|0);
 $24 = HEAP32[$23>>2]|0;
 $usub = $24;
 $25 = $7;
 $26 = ((($25)) + 32|0);
 $27 = HEAP32[$26>>2]|0;
 $xusub = $27;
 $28 = $7;
 $29 = ((($28)) + 40|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$nzumax>>2] = $30;
 $31 = $1;
 $32 = $supno;
 $33 = (($32) + ($31<<2)|0);
 $34 = HEAP32[$33>>2]|0;
 $jsupno = $34;
 $35 = $1;
 $36 = $xusub;
 $37 = (($36) + ($35<<2)|0);
 $38 = HEAP32[$37>>2]|0;
 $nextu = $38;
 $39 = $2;
 $40 = (($39) - 1)|0;
 $k = $40;
 $ksub = 0;
 L1: while(1) {
  $41 = $ksub;
  $42 = $2;
  $43 = ($41|0)<($42|0);
  if (!($43)) {
   label = 16;
   break;
  }
  $44 = $k;
  $45 = (($44) + -1)|0;
  $k = $45;
  $46 = $3;
  $47 = (($46) + ($44<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  $krep = $48;
  $49 = $krep;
  $50 = $supno;
  $51 = (($50) + ($49<<2)|0);
  $52 = HEAP32[$51>>2]|0;
  $ksupno = $52;
  $53 = $ksupno;
  $54 = $jsupno;
  $55 = ($53|0)!=($54|0);
  L4: do {
   if ($55) {
    $56 = $krep;
    $57 = $4;
    $58 = (($57) + ($56<<2)|0);
    $59 = HEAP32[$58>>2]|0;
    $kfnz = $59;
    $60 = $kfnz;
    $61 = ($60|0)!=(-1);
    if ($61) {
     $62 = $ksupno;
     $63 = $xsup;
     $64 = (($63) + ($62<<2)|0);
     $65 = HEAP32[$64>>2]|0;
     $fsupc = $65;
     $66 = $fsupc;
     $67 = $xlsub;
     $68 = (($67) + ($66<<2)|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = $kfnz;
     $71 = (($69) + ($70))|0;
     $72 = $fsupc;
     $73 = (($71) - ($72))|0;
     $isub = $73;
     $74 = $krep;
     $75 = $kfnz;
     $76 = (($74) - ($75))|0;
     $77 = (($76) + 1)|0;
     $segsze = $77;
     $78 = $nextu;
     $79 = $segsze;
     $80 = (($78) + ($79))|0;
     $new_next = $80;
     while(1) {
      $81 = $new_next;
      $82 = HEAP32[$nzumax>>2]|0;
      $83 = ($81|0)>($82|0);
      if (!($83)) {
       break;
      }
      $84 = $1;
      $85 = $nextu;
      $86 = $7;
      $87 = (_zLUMemXpand($84,$85,1,$nzumax,$86)|0);
      $mem_error = $87;
      $88 = ($87|0)!=(0);
      if ($88) {
       label = 8;
       break L1;
      }
      $90 = $7;
      $91 = ((($90)) + 24|0);
      $92 = HEAP32[$91>>2]|0;
      $ucol = $92;
      $93 = $1;
      $94 = $nextu;
      $95 = $7;
      $96 = (_zLUMemXpand($93,$94,3,$nzumax,$95)|0);
      $mem_error = $96;
      $97 = ($96|0)!=(0);
      if ($97) {
       label = 10;
       break L1;
      }
      $99 = $7;
      $100 = ((($99)) + 28|0);
      $101 = HEAP32[$100>>2]|0;
      $usub = $101;
      $102 = $7;
      $103 = ((($102)) + 8|0);
      $104 = HEAP32[$103>>2]|0;
      $lsub = $104;
     }
     $i = 0;
     while(1) {
      $105 = $i;
      $106 = $segsze;
      $107 = ($105|0)<($106|0);
      if (!($107)) {
       break L4;
      }
      $108 = $isub;
      $109 = $lsub;
      $110 = (($109) + ($108<<2)|0);
      $111 = HEAP32[$110>>2]|0;
      $irow = $111;
      $112 = $irow;
      $113 = $5;
      $114 = (($113) + ($112<<2)|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = $nextu;
      $117 = $usub;
      $118 = (($117) + ($116<<2)|0);
      HEAP32[$118>>2] = $115;
      $119 = $nextu;
      $120 = $ucol;
      $121 = (($120) + ($119<<4)|0);
      $122 = $irow;
      $123 = $6;
      $124 = (($123) + ($122<<4)|0);
      ;HEAP32[$121>>2]=HEAP32[$124>>2]|0;HEAP32[$121+4>>2]=HEAP32[$124+4>>2]|0;HEAP32[$121+8>>2]=HEAP32[$124+8>>2]|0;HEAP32[$121+12>>2]=HEAP32[$124+12>>2]|0;
      $125 = $irow;
      $126 = $6;
      $127 = (($126) + ($125<<4)|0);
      ;HEAP32[$127>>2]=HEAP32[$zero>>2]|0;HEAP32[$127+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$127+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$127+12>>2]=HEAP32[$zero+12>>2]|0;
      $128 = $nextu;
      $129 = (($128) + 1)|0;
      $nextu = $129;
      $130 = $isub;
      $131 = (($130) + 1)|0;
      $isub = $131;
      $132 = $i;
      $133 = (($132) + 1)|0;
      $i = $133;
     }
    }
   }
  } while(0);
  $134 = $ksub;
  $135 = (($134) + 1)|0;
  $ksub = $135;
 }
 if ((label|0) == 8) {
  $89 = $mem_error;
  $0 = $89;
  $141 = $0;
  STACKTOP = sp;return ($141|0);
 }
 else if ((label|0) == 10) {
  $98 = $mem_error;
  $0 = $98;
  $141 = $0;
  STACKTOP = sp;return ($141|0);
 }
 else if ((label|0) == 16) {
  $136 = $nextu;
  $137 = $1;
  $138 = (($137) + 1)|0;
  $139 = $xusub;
  $140 = (($139) + ($138<<2)|0);
  HEAP32[$140>>2] = $136;
  $0 = 0;
  $141 = $0;
  STACKTOP = sp;return ($141|0);
 }
 return (0)|0;
}
function _zsnode_dfs($jcol,$kcol,$asub,$xa_begin,$xa_end,$xprune,$marker,$Glu) {
 $jcol = $jcol|0;
 $kcol = $kcol|0;
 $asub = $asub|0;
 $xa_begin = $xa_begin|0;
 $xa_end = $xa_end|0;
 $xprune = $xprune|0;
 $marker = $marker|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0;
 var $ifrom = 0, $ito = 0, $k = 0, $kmark = 0, $krow = 0, $lsub = 0, $mem_error = 0, $new_next = 0, $nextl = 0, $nsuper = 0, $nzlmax = 0, $supno = 0, $xlsub = 0, $xsup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $nzlmax = sp;
 $1 = $jcol;
 $2 = $kcol;
 $3 = $asub;
 $4 = $xa_begin;
 $5 = $xa_end;
 $6 = $xprune;
 $7 = $marker;
 $8 = $Glu;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $xsup = $10;
 $11 = $8;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $supno = $13;
 $14 = $8;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $lsub = $16;
 $17 = $8;
 $18 = ((($17)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $xlsub = $19;
 $20 = $8;
 $21 = ((($20)) + 36|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$nzlmax>>2] = $22;
 $23 = $1;
 $24 = $supno;
 $25 = (($24) + ($23<<2)|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + 1)|0;
 HEAP32[$25>>2] = $27;
 $nsuper = $27;
 $28 = $1;
 $29 = $xlsub;
 $30 = (($29) + ($28<<2)|0);
 $31 = HEAP32[$30>>2]|0;
 $nextl = $31;
 $32 = $1;
 $i = $32;
 L1: while(1) {
  $33 = $i;
  $34 = $2;
  $35 = ($33|0)<=($34|0);
  if (!($35)) {
   break;
  }
  $36 = $i;
  $37 = $4;
  $38 = (($37) + ($36<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $k = $39;
  while(1) {
   $40 = $k;
   $41 = $i;
   $42 = $5;
   $43 = (($42) + ($41<<2)|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($40|0)<($44|0);
   if (!($45)) {
    break;
   }
   $46 = $k;
   $47 = $3;
   $48 = (($47) + ($46<<2)|0);
   $49 = HEAP32[$48>>2]|0;
   $krow = $49;
   $50 = $krow;
   $51 = $7;
   $52 = (($51) + ($50<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $kmark = $53;
   $54 = $kmark;
   $55 = $2;
   $56 = ($54|0)!=($55|0);
   if ($56) {
    $57 = $2;
    $58 = $krow;
    $59 = $7;
    $60 = (($59) + ($58<<2)|0);
    HEAP32[$60>>2] = $57;
    $61 = $krow;
    $62 = $nextl;
    $63 = (($62) + 1)|0;
    $nextl = $63;
    $64 = $lsub;
    $65 = (($64) + ($62<<2)|0);
    HEAP32[$65>>2] = $61;
    $66 = $nextl;
    $67 = HEAP32[$nzlmax>>2]|0;
    $68 = ($66|0)>=($67|0);
    if ($68) {
     $69 = $1;
     $70 = $nextl;
     $71 = $8;
     $72 = (_zLUMemXpand($69,$70,2,$nzlmax,$71)|0);
     $mem_error = $72;
     $73 = ($72|0)!=(0);
     if ($73) {
      label = 8;
      break L1;
     }
     $75 = $8;
     $76 = ((($75)) + 8|0);
     $77 = HEAP32[$76>>2]|0;
     $lsub = $77;
    }
   }
   $78 = $k;
   $79 = (($78) + 1)|0;
   $k = $79;
  }
  $80 = $nsuper;
  $81 = $i;
  $82 = $supno;
  $83 = (($82) + ($81<<2)|0);
  HEAP32[$83>>2] = $80;
  $84 = $i;
  $85 = (($84) + 1)|0;
  $i = $85;
 }
 if ((label|0) == 8) {
  $74 = $mem_error;
  $0 = $74;
  $158 = $0;
  STACKTOP = sp;return ($158|0);
 }
 $86 = $1;
 $87 = $2;
 $88 = ($86|0)<($87|0);
 if ($88) {
  $89 = $nextl;
  $90 = $nextl;
  $91 = $1;
  $92 = $xlsub;
  $93 = (($92) + ($91<<2)|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (($90) - ($94))|0;
  $96 = (($89) + ($95))|0;
  $new_next = $96;
  while(1) {
   $97 = $new_next;
   $98 = HEAP32[$nzlmax>>2]|0;
   $99 = ($97|0)>($98|0);
   if (!($99)) {
    break;
   }
   $100 = $1;
   $101 = $nextl;
   $102 = $8;
   $103 = (_zLUMemXpand($100,$101,2,$nzlmax,$102)|0);
   $mem_error = $103;
   $104 = ($103|0)!=(0);
   if ($104) {
    label = 16;
    break;
   }
   $106 = $8;
   $107 = ((($106)) + 8|0);
   $108 = HEAP32[$107>>2]|0;
   $lsub = $108;
  }
  if ((label|0) == 16) {
   $105 = $mem_error;
   $0 = $105;
   $158 = $0;
   STACKTOP = sp;return ($158|0);
  }
  $109 = $nextl;
  $ito = $109;
  $110 = $1;
  $111 = $xlsub;
  $112 = (($111) + ($110<<2)|0);
  $113 = HEAP32[$112>>2]|0;
  $ifrom = $113;
  while(1) {
   $114 = $ifrom;
   $115 = $nextl;
   $116 = ($114|0)<($115|0);
   if (!($116)) {
    break;
   }
   $117 = $ifrom;
   $118 = (($117) + 1)|0;
   $ifrom = $118;
   $119 = $lsub;
   $120 = (($119) + ($117<<2)|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = $ito;
   $123 = (($122) + 1)|0;
   $ito = $123;
   $124 = $lsub;
   $125 = (($124) + ($122<<2)|0);
   HEAP32[$125>>2] = $121;
  }
  $126 = $1;
  $127 = (($126) + 1)|0;
  $i = $127;
  while(1) {
   $128 = $i;
   $129 = $2;
   $130 = ($128|0)<=($129|0);
   if (!($130)) {
    break;
   }
   $131 = $nextl;
   $132 = $i;
   $133 = $xlsub;
   $134 = (($133) + ($132<<2)|0);
   HEAP32[$134>>2] = $131;
   $135 = $i;
   $136 = (($135) + 1)|0;
   $i = $136;
  }
  $137 = $ito;
  $nextl = $137;
 }
 $138 = $2;
 $139 = (($138) + 1)|0;
 $140 = $nsuper;
 $141 = (($140) + 1)|0;
 $142 = $xsup;
 $143 = (($142) + ($141<<2)|0);
 HEAP32[$143>>2] = $139;
 $144 = $nsuper;
 $145 = $2;
 $146 = (($145) + 1)|0;
 $147 = $supno;
 $148 = (($147) + ($146<<2)|0);
 HEAP32[$148>>2] = $144;
 $149 = $nextl;
 $150 = $2;
 $151 = $6;
 $152 = (($151) + ($150<<2)|0);
 HEAP32[$152>>2] = $149;
 $153 = $nextl;
 $154 = $2;
 $155 = (($154) + 1)|0;
 $156 = $xlsub;
 $157 = (($156) + ($155<<2)|0);
 HEAP32[$157>>2] = $153;
 $0 = 0;
 $158 = $0;
 STACKTOP = sp;return ($158|0);
}
function _zsnode_bmod($jcol,$jsupno,$fsupc,$dense,$tempv,$Glu,$stat) {
 $jcol = $jcol|0;
 $jsupno = $jsupno|0;
 $fsupc = $fsupc|0;
 $dense = $dense|0;
 $tempv = $tempv|0;
 $Glu = $Glu|0;
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0.0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0.0, $135 = 0, $136 = 0, $137 = 0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0.0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0.0, $153 = 0.0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0, $93 = 0, $94 = 0.0;
 var $95 = 0.0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $comp_zero = 0, $i = 0, $iptr = 0, $irow = 0, $isub = 0, $lsub = 0, $luptr = 0, $lusup = 0, $nextlu = 0, $nrow = 0, $nsupc = 0, $nsupr = 0, $ops = 0, $ufirst = 0, $xlsub = 0;
 var $xlusup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $comp_zero = sp;
 $0 = $jcol;
 $1 = $jsupno;
 $2 = $fsupc;
 $3 = $dense;
 $4 = $tempv;
 $5 = $Glu;
 $6 = $stat;
 ;HEAP32[$comp_zero>>2]=0|0;HEAP32[$comp_zero+4>>2]=0|0;HEAP32[$comp_zero+8>>2]=0|0;HEAP32[$comp_zero+12>>2]=0|0;
 $7 = $6;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $ops = $9;
 $10 = $5;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $lsub = $12;
 $13 = $5;
 $14 = ((($13)) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $xlsub = $15;
 $16 = $5;
 $17 = ((($16)) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $lusup = $18;
 $19 = $5;
 $20 = ((($19)) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $xlusup = $21;
 $22 = $0;
 $23 = $xlusup;
 $24 = (($23) + ($22<<2)|0);
 $25 = HEAP32[$24>>2]|0;
 $nextlu = $25;
 $26 = $2;
 $27 = $xlsub;
 $28 = (($27) + ($26<<2)|0);
 $29 = HEAP32[$28>>2]|0;
 $isub = $29;
 while(1) {
  $30 = $isub;
  $31 = $2;
  $32 = (($31) + 1)|0;
  $33 = $xlsub;
  $34 = (($33) + ($32<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($30|0)<($35|0);
  if (!($36)) {
   break;
  }
  $37 = $isub;
  $38 = $lsub;
  $39 = (($38) + ($37<<2)|0);
  $40 = HEAP32[$39>>2]|0;
  $irow = $40;
  $41 = $nextlu;
  $42 = $lusup;
  $43 = (($42) + ($41<<4)|0);
  $44 = $irow;
  $45 = $3;
  $46 = (($45) + ($44<<4)|0);
  ;HEAP32[$43>>2]=HEAP32[$46>>2]|0;HEAP32[$43+4>>2]=HEAP32[$46+4>>2]|0;HEAP32[$43+8>>2]=HEAP32[$46+8>>2]|0;HEAP32[$43+12>>2]=HEAP32[$46+12>>2]|0;
  $47 = $irow;
  $48 = $3;
  $49 = (($48) + ($47<<4)|0);
  ;HEAP32[$49>>2]=HEAP32[$comp_zero>>2]|0;HEAP32[$49+4>>2]=HEAP32[$comp_zero+4>>2]|0;HEAP32[$49+8>>2]=HEAP32[$comp_zero+8>>2]|0;HEAP32[$49+12>>2]=HEAP32[$comp_zero+12>>2]|0;
  $50 = $nextlu;
  $51 = (($50) + 1)|0;
  $nextlu = $51;
  $52 = $isub;
  $53 = (($52) + 1)|0;
  $isub = $53;
 }
 $54 = $nextlu;
 $55 = $0;
 $56 = (($55) + 1)|0;
 $57 = $xlusup;
 $58 = (($57) + ($56<<2)|0);
 HEAP32[$58>>2] = $54;
 $59 = $2;
 $60 = $0;
 $61 = ($59|0)<($60|0);
 if (!($61)) {
  STACKTOP = sp;return 0;
 }
 $62 = $2;
 $63 = $xlusup;
 $64 = (($63) + ($62<<2)|0);
 $65 = HEAP32[$64>>2]|0;
 $luptr = $65;
 $66 = $2;
 $67 = (($66) + 1)|0;
 $68 = $xlsub;
 $69 = (($68) + ($67<<2)|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = $2;
 $72 = $xlsub;
 $73 = (($72) + ($71<<2)|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = (($70) - ($74))|0;
 $nsupr = $75;
 $76 = $0;
 $77 = $2;
 $78 = (($76) - ($77))|0;
 $nsupc = $78;
 $79 = $0;
 $80 = $xlusup;
 $81 = (($80) + ($79<<2)|0);
 $82 = HEAP32[$81>>2]|0;
 $ufirst = $82;
 $83 = $nsupr;
 $84 = $nsupc;
 $85 = (($83) - ($84))|0;
 $nrow = $85;
 $86 = $nsupc;
 $87 = $86<<2;
 $88 = $nsupc;
 $89 = (($88) - 1)|0;
 $90 = Math_imul($87, $89)|0;
 $91 = (+($90|0));
 $92 = $ops;
 $93 = ((($92)) + 52|0);
 $94 = +HEAPF32[$93>>2];
 $95 = $94 + $91;
 HEAPF32[$93>>2] = $95;
 $96 = $nrow;
 $97 = $96<<3;
 $98 = $nsupc;
 $99 = Math_imul($97, $98)|0;
 $100 = (+($99|0));
 $101 = $ops;
 $102 = ((($101)) + 56|0);
 $103 = +HEAPF32[$102>>2];
 $104 = $103 + $100;
 HEAPF32[$102>>2] = $104;
 $105 = $nsupr;
 $106 = $nsupc;
 $107 = $luptr;
 $108 = $lusup;
 $109 = (($108) + ($107<<4)|0);
 $110 = $ufirst;
 $111 = $lusup;
 $112 = (($111) + ($110<<4)|0);
 _zlsolve($105,$106,$109,$112);
 $113 = $nsupr;
 $114 = $nrow;
 $115 = $nsupc;
 $116 = $luptr;
 $117 = $nsupc;
 $118 = (($116) + ($117))|0;
 $119 = $lusup;
 $120 = (($119) + ($118<<4)|0);
 $121 = $ufirst;
 $122 = $lusup;
 $123 = (($122) + ($121<<4)|0);
 $124 = $4;
 _zmatvec($113,$114,$115,$120,$123,$124);
 $125 = $ufirst;
 $126 = $nsupc;
 $127 = (($125) + ($126))|0;
 $iptr = $127;
 $i = 0;
 while(1) {
  $128 = $i;
  $129 = $nrow;
  $130 = ($128|0)<($129|0);
  if (!($130)) {
   break;
  }
  $131 = $iptr;
  $132 = $lusup;
  $133 = (($132) + ($131<<4)|0);
  $134 = +HEAPF64[$133>>3];
  $135 = $i;
  $136 = $4;
  $137 = (($136) + ($135<<4)|0);
  $138 = +HEAPF64[$137>>3];
  $139 = $134 - $138;
  $140 = $iptr;
  $141 = $lusup;
  $142 = (($141) + ($140<<4)|0);
  HEAPF64[$142>>3] = $139;
  $143 = $iptr;
  $144 = $lusup;
  $145 = (($144) + ($143<<4)|0);
  $146 = ((($145)) + 8|0);
  $147 = +HEAPF64[$146>>3];
  $148 = $i;
  $149 = $4;
  $150 = (($149) + ($148<<4)|0);
  $151 = ((($150)) + 8|0);
  $152 = +HEAPF64[$151>>3];
  $153 = $147 - $152;
  $154 = $iptr;
  $155 = $lusup;
  $156 = (($155) + ($154<<4)|0);
  $157 = ((($156)) + 8|0);
  HEAPF64[$157>>3] = $153;
  $158 = $iptr;
  $159 = (($158) + 1)|0;
  $iptr = $159;
  $160 = $i;
  $161 = $4;
  $162 = (($161) + ($160<<4)|0);
  ;HEAP32[$162>>2]=HEAP32[$comp_zero>>2]|0;HEAP32[$162+4>>2]=HEAP32[$comp_zero+4>>2]|0;HEAP32[$162+8>>2]=HEAP32[$comp_zero+8>>2]|0;HEAP32[$162+12>>2]=HEAP32[$comp_zero+12>>2]|0;
  $163 = $i;
  $164 = (($163) + 1)|0;
  $i = $164;
 }
 STACKTOP = sp;return 0;
}
function _zpanel_dfs($m,$w,$jcol,$A,$perm_r,$nseg,$dense,$panel_lsub,$segrep,$repfnz,$xprune,$marker,$parent,$xplore,$Glu) {
 $m = $m|0;
 $w = $w|0;
 $jcol = $jcol|0;
 $A = $A|0;
 $perm_r = $perm_r|0;
 $nseg = $nseg|0;
 $dense = $dense|0;
 $panel_lsub = $panel_lsub|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $xprune = $xprune|0;
 $marker = $marker|0;
 $parent = $parent|0;
 $xplore = $xplore|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Astore = 0, $a = 0;
 var $asub = 0, $chmark = 0, $chperm = 0, $chrep = 0, $dense_col = 0, $jj = 0, $k = 0, $kchild = 0, $kmark = 0, $kpar = 0, $kperm = 0, $krep = 0, $krow = 0, $lsub = 0, $marker1 = 0, $maxdfs = 0, $myfnz = 0, $nextl_col = 0, $oldrep = 0, $repfnz_col = 0;
 var $supno = 0, $xa_begin = 0, $xa_end = 0, $xdfs = 0, $xlsub = 0, $xsup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $m;
 $1 = $w;
 $2 = $jcol;
 $3 = $A;
 $4 = $perm_r;
 $5 = $nseg;
 $6 = $dense;
 $7 = $panel_lsub;
 $8 = $segrep;
 $9 = $repfnz;
 $10 = $xprune;
 $11 = $marker;
 $12 = $parent;
 $13 = $xplore;
 $14 = $Glu;
 $15 = $3;
 $16 = ((($15)) + 20|0);
 $17 = HEAP32[$16>>2]|0;
 $Astore = $17;
 $18 = $Astore;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $a = $20;
 $21 = $Astore;
 $22 = ((($21)) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 $asub = $23;
 $24 = $Astore;
 $25 = ((($24)) + 12|0);
 $26 = HEAP32[$25>>2]|0;
 $xa_begin = $26;
 $27 = $Astore;
 $28 = ((($27)) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 $xa_end = $29;
 $30 = $11;
 $31 = $0;
 $32 = (($30) + ($31<<2)|0);
 $marker1 = $32;
 $33 = $9;
 $repfnz_col = $33;
 $34 = $6;
 $dense_col = $34;
 $35 = $5;
 HEAP32[$35>>2] = 0;
 $36 = $14;
 $37 = HEAP32[$36>>2]|0;
 $xsup = $37;
 $38 = $14;
 $39 = ((($38)) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $supno = $40;
 $41 = $14;
 $42 = ((($41)) + 8|0);
 $43 = HEAP32[$42>>2]|0;
 $lsub = $43;
 $44 = $14;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $xlsub = $46;
 $47 = $2;
 $jj = $47;
 while(1) {
  $48 = $jj;
  $49 = $2;
  $50 = $1;
  $51 = (($49) + ($50))|0;
  $52 = ($48|0)<($51|0);
  if (!($52)) {
   break;
  }
  $53 = $jj;
  $54 = $2;
  $55 = (($53) - ($54))|0;
  $56 = $0;
  $57 = Math_imul($55, $56)|0;
  $nextl_col = $57;
  $58 = $jj;
  $59 = $xa_begin;
  $60 = (($59) + ($58<<2)|0);
  $61 = HEAP32[$60>>2]|0;
  $k = $61;
  while(1) {
   $62 = $k;
   $63 = $jj;
   $64 = $xa_end;
   $65 = (($64) + ($63<<2)|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = ($62|0)<($66|0);
   if (!($67)) {
    break;
   }
   $68 = $k;
   $69 = $asub;
   $70 = (($69) + ($68<<2)|0);
   $71 = HEAP32[$70>>2]|0;
   $krow = $71;
   $72 = $krow;
   $73 = $dense_col;
   $74 = (($73) + ($72<<4)|0);
   $75 = $k;
   $76 = $a;
   $77 = (($76) + ($75<<4)|0);
   ;HEAP32[$74>>2]=HEAP32[$77>>2]|0;HEAP32[$74+4>>2]=HEAP32[$77+4>>2]|0;HEAP32[$74+8>>2]=HEAP32[$77+8>>2]|0;HEAP32[$74+12>>2]=HEAP32[$77+12>>2]|0;
   $78 = $krow;
   $79 = $11;
   $80 = (($79) + ($78<<2)|0);
   $81 = HEAP32[$80>>2]|0;
   $kmark = $81;
   $82 = $kmark;
   $83 = $jj;
   $84 = ($82|0)==($83|0);
   L7: do {
    if (!($84)) {
     $85 = $jj;
     $86 = $krow;
     $87 = $11;
     $88 = (($87) + ($86<<2)|0);
     HEAP32[$88>>2] = $85;
     $89 = $krow;
     $90 = $4;
     $91 = (($90) + ($89<<2)|0);
     $92 = HEAP32[$91>>2]|0;
     $kperm = $92;
     $93 = $kperm;
     $94 = ($93|0)==(-1);
     if ($94) {
      $95 = $krow;
      $96 = $nextl_col;
      $97 = (($96) + 1)|0;
      $nextl_col = $97;
      $98 = $7;
      $99 = (($98) + ($96<<2)|0);
      HEAP32[$99>>2] = $95;
      break;
     }
     $100 = $kperm;
     $101 = $supno;
     $102 = (($101) + ($100<<2)|0);
     $103 = HEAP32[$102>>2]|0;
     $104 = (($103) + 1)|0;
     $105 = $xsup;
     $106 = (($105) + ($104<<2)|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = (($107) - 1)|0;
     $krep = $108;
     $109 = $krep;
     $110 = $repfnz_col;
     $111 = (($110) + ($109<<2)|0);
     $112 = HEAP32[$111>>2]|0;
     $myfnz = $112;
     $113 = $myfnz;
     $114 = ($113|0)!=(-1);
     if ($114) {
      $115 = $myfnz;
      $116 = $kperm;
      $117 = ($115|0)>($116|0);
      if (!($117)) {
       break;
      }
      $118 = $kperm;
      $119 = $krep;
      $120 = $repfnz_col;
      $121 = (($120) + ($119<<2)|0);
      HEAP32[$121>>2] = $118;
      break;
     }
     $oldrep = -1;
     $122 = $oldrep;
     $123 = $krep;
     $124 = $12;
     $125 = (($124) + ($123<<2)|0);
     HEAP32[$125>>2] = $122;
     $126 = $kperm;
     $127 = $krep;
     $128 = $repfnz_col;
     $129 = (($128) + ($127<<2)|0);
     HEAP32[$129>>2] = $126;
     $130 = $krep;
     $131 = $xlsub;
     $132 = (($131) + ($130<<2)|0);
     $133 = HEAP32[$132>>2]|0;
     $xdfs = $133;
     $134 = $krep;
     $135 = $10;
     $136 = (($135) + ($134<<2)|0);
     $137 = HEAP32[$136>>2]|0;
     $maxdfs = $137;
     while(1) {
      $138 = $xdfs;
      $139 = $maxdfs;
      $140 = ($138|0)<($139|0);
      if (!($140)) {
       $213 = $krep;
       $214 = $marker1;
       $215 = (($214) + ($213<<2)|0);
       $216 = HEAP32[$215>>2]|0;
       $217 = $2;
       $218 = ($216|0)<($217|0);
       if ($218) {
        $219 = $krep;
        $220 = $5;
        $221 = HEAP32[$220>>2]|0;
        $222 = $8;
        $223 = (($222) + ($221<<2)|0);
        HEAP32[$223>>2] = $219;
        $224 = $5;
        $225 = HEAP32[$224>>2]|0;
        $226 = (($225) + 1)|0;
        HEAP32[$224>>2] = $226;
        $227 = $jj;
        $228 = $krep;
        $229 = $marker1;
        $230 = (($229) + ($228<<2)|0);
        HEAP32[$230>>2] = $227;
       }
       $231 = $krep;
       $232 = $12;
       $233 = (($232) + ($231<<2)|0);
       $234 = HEAP32[$233>>2]|0;
       $kpar = $234;
       $235 = $kpar;
       $236 = ($235|0)==(-1);
       if ($236) {
        break L7;
       }
       $237 = $kpar;
       $krep = $237;
       $238 = $krep;
       $239 = $13;
       $240 = (($239) + ($238<<2)|0);
       $241 = HEAP32[$240>>2]|0;
       $xdfs = $241;
       $242 = $krep;
       $243 = $10;
       $244 = (($243) + ($242<<2)|0);
       $245 = HEAP32[$244>>2]|0;
       $maxdfs = $245;
       $246 = $kpar;
       $247 = ($246|0)!=(-1);
       if ($247) {
        continue;
       } else {
        break;
       }
      }
      $141 = $xdfs;
      $142 = $lsub;
      $143 = (($142) + ($141<<2)|0);
      $144 = HEAP32[$143>>2]|0;
      $kchild = $144;
      $145 = $xdfs;
      $146 = (($145) + 1)|0;
      $xdfs = $146;
      $147 = $kchild;
      $148 = $11;
      $149 = (($148) + ($147<<2)|0);
      $150 = HEAP32[$149>>2]|0;
      $chmark = $150;
      $151 = $chmark;
      $152 = $jj;
      $153 = ($151|0)!=($152|0);
      if (!($153)) {
       continue;
      }
      $154 = $jj;
      $155 = $kchild;
      $156 = $11;
      $157 = (($156) + ($155<<2)|0);
      HEAP32[$157>>2] = $154;
      $158 = $kchild;
      $159 = $4;
      $160 = (($159) + ($158<<2)|0);
      $161 = HEAP32[$160>>2]|0;
      $chperm = $161;
      $162 = $chperm;
      $163 = ($162|0)==(-1);
      if ($163) {
       $164 = $kchild;
       $165 = $nextl_col;
       $166 = (($165) + 1)|0;
       $nextl_col = $166;
       $167 = $7;
       $168 = (($167) + ($165<<2)|0);
       HEAP32[$168>>2] = $164;
       continue;
      }
      $169 = $chperm;
      $170 = $supno;
      $171 = (($170) + ($169<<2)|0);
      $172 = HEAP32[$171>>2]|0;
      $173 = (($172) + 1)|0;
      $174 = $xsup;
      $175 = (($174) + ($173<<2)|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = (($176) - 1)|0;
      $chrep = $177;
      $178 = $chrep;
      $179 = $repfnz_col;
      $180 = (($179) + ($178<<2)|0);
      $181 = HEAP32[$180>>2]|0;
      $myfnz = $181;
      $182 = $myfnz;
      $183 = ($182|0)!=(-1);
      if (!($183)) {
       $191 = $xdfs;
       $192 = $krep;
       $193 = $13;
       $194 = (($193) + ($192<<2)|0);
       HEAP32[$194>>2] = $191;
       $195 = $krep;
       $oldrep = $195;
       $196 = $chrep;
       $krep = $196;
       $197 = $oldrep;
       $198 = $krep;
       $199 = $12;
       $200 = (($199) + ($198<<2)|0);
       HEAP32[$200>>2] = $197;
       $201 = $chperm;
       $202 = $krep;
       $203 = $repfnz_col;
       $204 = (($203) + ($202<<2)|0);
       HEAP32[$204>>2] = $201;
       $205 = $krep;
       $206 = $xlsub;
       $207 = (($206) + ($205<<2)|0);
       $208 = HEAP32[$207>>2]|0;
       $xdfs = $208;
       $209 = $krep;
       $210 = $10;
       $211 = (($210) + ($209<<2)|0);
       $212 = HEAP32[$211>>2]|0;
       $maxdfs = $212;
       continue;
      }
      $184 = $myfnz;
      $185 = $chperm;
      $186 = ($184|0)>($185|0);
      if (!($186)) {
       continue;
      }
      $187 = $chperm;
      $188 = $chrep;
      $189 = $repfnz_col;
      $190 = (($189) + ($188<<2)|0);
      HEAP32[$190>>2] = $187;
     }
    }
   } while(0);
   $248 = $k;
   $249 = (($248) + 1)|0;
   $k = $249;
  }
  $250 = $0;
  $251 = $repfnz_col;
  $252 = (($251) + ($250<<2)|0);
  $repfnz_col = $252;
  $253 = $0;
  $254 = $dense_col;
  $255 = (($254) + ($253<<4)|0);
  $dense_col = $255;
  $256 = $jj;
  $257 = (($256) + 1)|0;
  $jj = $257;
 }
 STACKTOP = sp;return;
}
function _zpanel_bmod($m,$w,$jcol,$nseg,$dense,$tempv,$segrep,$repfnz,$Glu,$stat) {
 $m = $m|0;
 $w = $w|0;
 $jcol = $jcol|0;
 $nseg = $nseg|0;
 $dense = $dense|0;
 $tempv = $tempv|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $Glu = $Glu|0;
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0.0, $1002 = 0.0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0.0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0.0, $1038 = 0.0, $1039 = 0, $104 = 0, $1040 = 0.0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0.0, $1046 = 0.0, $1047 = 0.0, $1048 = 0, $1049 = 0.0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0.0, $1054 = 0.0, $1055 = 0.0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0.0, $1061 = 0.0, $1062 = 0.0, $1063 = 0.0, $1064 = 0.0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0.0;
 var $107 = 0, $1070 = 0.0, $1071 = 0.0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0.0, $108 = 0, $1080 = 0, $1081 = 0.0, $1082 = 0.0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0.0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0.0, $1125 = 0.0, $1126 = 0, $1127 = 0.0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0.0, $1133 = 0.0, $1134 = 0.0, $1135 = 0, $1136 = 0.0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0.0;
 var $1141 = 0.0, $1142 = 0.0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0.0, $1148 = 0.0, $1149 = 0.0, $115 = 0, $1150 = 0.0, $1151 = 0.0, $1152 = 0, $1153 = 0.0, $1154 = 0.0, $1155 = 0.0, $1156 = 0, $1157 = 0.0, $1158 = 0, $1159 = 0.0;
 var $116 = 0, $1160 = 0.0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0.0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0.0, $1190 = 0.0, $1191 = 0.0, $1192 = 0, $1193 = 0.0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0.0, $1199 = 0.0, $12 = 0, $120 = 0, $1200 = 0.0, $1201 = 0, $1202 = 0.0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0.0, $1207 = 0.0, $1208 = 0.0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0.0, $1214 = 0.0, $1215 = 0.0, $1216 = 0.0, $1217 = 0.0, $1218 = 0, $1219 = 0.0, $122 = 0.0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0.0, $1224 = 0.0, $1225 = 0, $1226 = 0.0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0.0, $1230 = 0;
 var $1231 = 0.0, $1232 = 0.0, $1233 = 0.0, $1234 = 0, $1235 = 0.0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0.0, $124 = 0, $1240 = 0.0, $1241 = 0.0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0.0, $1247 = 0.0, $1248 = 0.0, $1249 = 0.0;
 var $125 = 0, $1250 = 0.0, $1251 = 0, $1252 = 0.0, $1253 = 0.0, $1254 = 0.0, $1255 = 0, $1256 = 0.0, $1257 = 0, $1258 = 0.0, $1259 = 0.0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0.0, $1265 = 0.0, $1266 = 0.0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0.0, $1275 = 0, $1276 = 0.0, $1277 = 0.0, $1278 = 0, $1279 = 0, $128 = 0.0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0.0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0.0, $13 = 0, $130 = 0, $1300 = 0.0, $1301 = 0, $1302 = 0.0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0.0, $1309 = 0.0, $131 = 0.0, $1310 = 0.0, $1311 = 0, $1312 = 0.0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0.0, $1318 = 0.0, $1319 = 0.0, $132 = 0.0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0.0, $1326 = 0.0, $1327 = 0.0, $1328 = 0.0, $1329 = 0.0, $133 = 0, $1330 = 0, $1331 = 0.0, $1332 = 0.0, $1333 = 0.0, $1334 = 0, $1335 = 0.0, $1336 = 0, $1337 = 0.0, $1338 = 0.0, $1339 = 0;
 var $134 = 0, $1340 = 0.0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0.0, $1345 = 0.0, $1346 = 0, $1347 = 0.0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0.0, $1353 = 0.0, $1354 = 0.0, $1355 = 0, $1356 = 0.0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0.0, $1361 = 0.0, $1362 = 0.0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0.0, $1368 = 0.0, $1369 = 0.0, $137 = 0, $1370 = 0.0, $1371 = 0.0, $1372 = 0, $1373 = 0.0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0.0, $1378 = 0.0, $1379 = 0, $138 = 0, $1380 = 0.0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0.0, $1386 = 0.0, $1387 = 0.0, $1388 = 0, $1389 = 0.0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0.0;
 var $1394 = 0.0, $1395 = 0.0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0.0, $1401 = 0.0, $1402 = 0.0, $1403 = 0.0, $1404 = 0.0, $1405 = 0, $1406 = 0.0, $1407 = 0.0, $1408 = 0.0, $1409 = 0, $141 = 0, $1410 = 0.0;
 var $1411 = 0, $1412 = 0.0, $1413 = 0.0, $1414 = 0, $1415 = 0.0, $1416 = 0.0, $1417 = 0.0, $1418 = 0, $1419 = 0.0, $142 = 0, $1420 = 0, $1421 = 0.0, $1422 = 0.0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0;
 var $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0;
 var $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0.0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0.0, $1462 = 0.0, $1463 = 0, $1464 = 0.0, $1465 = 0;
 var $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0.0, $147 = 0, $1470 = 0.0, $1471 = 0.0, $1472 = 0, $1473 = 0.0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0.0, $1478 = 0.0, $1479 = 0.0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0;
 var $1484 = 0.0, $1485 = 0.0, $1486 = 0.0, $1487 = 0.0, $1488 = 0.0, $1489 = 0, $149 = 0, $1490 = 0.0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0.0, $1495 = 0.0, $1496 = 0, $1497 = 0.0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0;
 var $1501 = 0, $1502 = 0.0, $1503 = 0.0, $1504 = 0.0, $1505 = 0, $1506 = 0.0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0.0, $1511 = 0.0, $1512 = 0.0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0.0, $1518 = 0.0, $1519 = 0.0;
 var $152 = 0, $1520 = 0.0, $1521 = 0.0, $1522 = 0, $1523 = 0.0, $1524 = 0.0, $1525 = 0.0, $1526 = 0, $1527 = 0.0, $1528 = 0, $1529 = 0.0, $153 = 0, $1530 = 0.0, $1531 = 0, $1532 = 0.0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0.0, $1537 = 0.0;
 var $1538 = 0, $1539 = 0.0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0.0, $1545 = 0.0, $1546 = 0.0, $1547 = 0, $1548 = 0.0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0.0, $1553 = 0.0, $1554 = 0.0, $1555 = 0;
 var $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0.0, $156 = 0, $1560 = 0.0, $1561 = 0.0, $1562 = 0.0, $1563 = 0.0, $1564 = 0, $1565 = 0.0, $1566 = 0.0, $1567 = 0.0, $1568 = 0, $1569 = 0.0, $157 = 0, $1570 = 0, $1571 = 0.0, $1572 = 0.0, $1573 = 0;
 var $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0.0, $1578 = 0.0, $1579 = 0.0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0.0, $1588 = 0, $1589 = 0.0, $159 = 0, $1590 = 0.0, $1591 = 0;
 var $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0;
 var $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0, $1627 = 0;
 var $1628 = 0, $1629 = 0, $163 = 0.0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0, $1645 = 0;
 var $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0;
 var $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0.0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0.0, $1680 = 0.0, $1681 = 0;
 var $1682 = 0, $1683 = 0, $1684 = 0.0, $1685 = 0.0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0.0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0.0, $1699 = 0.0, $17 = 0;
 var $170 = 0.0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0, $1717 = 0;
 var $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $173 = 0, $174 = 0, $175 = 0.0, $176 = 0.0, $177 = 0.0, $178 = 0, $179 = 0.0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0.0, $184 = 0.0, $185 = 0.0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0.0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0.0, $201 = 0.0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0.0, $21 = 0, $210 = 0, $211 = 0.0, $212 = 0.0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0.0, $251 = 0, $252 = 0, $253 = 0, $254 = 0.0, $255 = 0.0, $256 = 0, $257 = 0.0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0.0, $263 = 0.0, $264 = 0.0, $265 = 0, $266 = 0.0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0.0, $271 = 0.0, $272 = 0.0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0.0;
 var $278 = 0.0, $279 = 0.0, $28 = 0, $280 = 0.0, $281 = 0.0, $282 = 0, $283 = 0.0, $284 = 0.0, $285 = 0.0, $286 = 0, $287 = 0.0, $288 = 0, $289 = 0.0, $29 = 0, $290 = 0.0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0.0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0.0, $321 = 0.0, $322 = 0, $323 = 0.0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0.0, $329 = 0.0, $33 = 0, $330 = 0.0;
 var $331 = 0, $332 = 0.0, $333 = 0, $334 = 0, $335 = 0, $336 = 0.0, $337 = 0.0, $338 = 0.0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0.0, $344 = 0.0, $345 = 0.0, $346 = 0.0, $347 = 0.0, $348 = 0, $349 = 0.0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0.0, $354 = 0.0, $355 = 0, $356 = 0.0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0.0, $362 = 0.0, $363 = 0.0, $364 = 0, $365 = 0.0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0.0, $37 = 0, $370 = 0.0, $371 = 0.0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0.0, $377 = 0.0, $378 = 0.0, $379 = 0.0, $38 = 0, $380 = 0.0, $381 = 0, $382 = 0.0, $383 = 0.0, $384 = 0.0, $385 = 0;
 var $386 = 0.0, $387 = 0, $388 = 0.0, $389 = 0.0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0.0, $395 = 0.0, $396 = 0.0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0.0, $405 = 0, $406 = 0.0, $407 = 0.0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0.0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0.0, $43 = 0, $430 = 0.0, $431 = 0, $432 = 0.0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0.0, $439 = 0.0;
 var $44 = 0, $440 = 0.0, $441 = 0, $442 = 0.0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0.0, $448 = 0.0, $449 = 0.0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0.0, $456 = 0.0, $457 = 0.0;
 var $458 = 0.0, $459 = 0.0, $46 = 0, $460 = 0, $461 = 0.0, $462 = 0.0, $463 = 0.0, $464 = 0, $465 = 0.0, $466 = 0, $467 = 0.0, $468 = 0.0, $469 = 0, $47 = 0, $470 = 0.0, $471 = 0, $472 = 0, $473 = 0, $474 = 0.0, $475 = 0.0;
 var $476 = 0, $477 = 0.0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0.0, $483 = 0.0, $484 = 0.0, $485 = 0, $486 = 0.0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0.0, $491 = 0.0, $492 = 0.0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0.0, $498 = 0.0, $499 = 0.0, $5 = 0, $50 = 0, $500 = 0.0, $501 = 0.0, $502 = 0, $503 = 0.0, $504 = 0, $505 = 0, $506 = 0, $507 = 0.0, $508 = 0.0, $509 = 0, $51 = 0, $510 = 0.0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0.0, $516 = 0.0, $517 = 0.0, $518 = 0, $519 = 0.0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0.0, $524 = 0.0, $525 = 0.0, $526 = 0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0.0, $531 = 0.0, $532 = 0.0, $533 = 0.0, $534 = 0.0, $535 = 0, $536 = 0.0, $537 = 0.0, $538 = 0.0, $539 = 0, $54 = 0, $540 = 0.0, $541 = 0, $542 = 0.0, $543 = 0.0, $544 = 0, $545 = 0.0, $546 = 0.0, $547 = 0.0;
 var $548 = 0, $549 = 0.0, $55 = 0, $550 = 0, $551 = 0.0, $552 = 0.0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0;
 var $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0;
 var $584 = 0, $585 = 0, $586 = 0, $587 = 0.0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0.0, $592 = 0.0, $593 = 0, $594 = 0.0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0.0, $6 = 0, $60 = 0, $600 = 0.0;
 var $601 = 0.0, $602 = 0, $603 = 0.0, $604 = 0, $605 = 0, $606 = 0, $607 = 0.0, $608 = 0.0, $609 = 0.0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0.0, $615 = 0.0, $616 = 0.0, $617 = 0.0, $618 = 0.0, $619 = 0;
 var $62 = 0, $620 = 0.0, $621 = 0, $622 = 0, $623 = 0, $624 = 0.0, $625 = 0.0, $626 = 0, $627 = 0.0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0.0, $633 = 0.0, $634 = 0.0, $635 = 0, $636 = 0.0, $637 = 0;
 var $638 = 0, $639 = 0, $64 = 0, $640 = 0.0, $641 = 0.0, $642 = 0.0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0.0, $648 = 0.0, $649 = 0.0, $65 = 0, $650 = 0.0, $651 = 0.0, $652 = 0, $653 = 0.0, $654 = 0.0, $655 = 0.0;
 var $656 = 0, $657 = 0.0, $658 = 0, $659 = 0.0, $66 = 0, $660 = 0.0, $661 = 0, $662 = 0.0, $663 = 0, $664 = 0, $665 = 0, $666 = 0.0, $667 = 0.0, $668 = 0, $669 = 0.0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0;
 var $674 = 0.0, $675 = 0.0, $676 = 0.0, $677 = 0, $678 = 0.0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0.0, $683 = 0.0, $684 = 0.0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0.0, $69 = 0, $690 = 0.0, $691 = 0.0;
 var $692 = 0.0, $693 = 0.0, $694 = 0, $695 = 0.0, $696 = 0.0, $697 = 0.0, $698 = 0, $699 = 0.0, $7 = 0, $70 = 0, $700 = 0, $701 = 0.0, $702 = 0.0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0.0, $708 = 0.0, $709 = 0.0;
 var $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0.0, $718 = 0, $719 = 0.0, $72 = 0, $720 = 0.0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0;
 var $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0;
 var $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0;
 var $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0;
 var $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0;
 var $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0;
 var $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0;
 var $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0;
 var $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0.0, $862 = 0, $863 = 0, $864 = 0, $865 = 0.0, $866 = 0.0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0;
 var $872 = 0, $873 = 0, $874 = 0.0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0.0, $88 = 0, $880 = 0.0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0;
 var $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0;
 var $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0;
 var $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0;
 var $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0;
 var $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0;
 var $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0.0, $99 = 0, $990 = 0, $991 = 0, $992 = 0.0, $993 = 0.0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0.0;
 var $999 = 0, $MatvecTmp = 0, $TriTmp = 0, $block_nrow = 0, $ci = 0.0, $ci10 = 0.0, $ci12 = 0.0, $ci14 = 0.0, $ci16 = 0.0, $ci18 = 0.0, $ci2 = 0.0, $ci20 = 0.0, $ci22 = 0.0, $ci24 = 0.0, $ci26 = 0.0, $ci28 = 0.0, $ci30 = 0.0, $ci32 = 0.0, $ci34 = 0.0, $ci36 = 0.0;
 var $ci38 = 0.0, $ci4 = 0.0, $ci6 = 0.0, $ci8 = 0.0, $comp_temp = 0, $comp_temp1 = 0, $cr = 0.0, $cr1 = 0.0, $cr11 = 0.0, $cr13 = 0.0, $cr15 = 0.0, $cr17 = 0.0, $cr19 = 0.0, $cr21 = 0.0, $cr23 = 0.0, $cr25 = 0.0, $cr27 = 0.0, $cr29 = 0.0, $cr3 = 0.0, $cr31 = 0.0;
 var $cr33 = 0.0, $cr35 = 0.0, $cr37 = 0.0, $cr5 = 0.0, $cr7 = 0.0, $cr9 = 0.0, $dense_col = 0, $fsupc = 0, $i = 0, $irow = 0, $isub = 0, $isub1 = 0, $jj = 0, $k = 0, $kfnz = 0, $krep = 0, $krep_ind = 0, $ksub = 0, $ldaTmp = 0, $lptr = 0;
 var $lsub = 0, $luptr = 0, $luptr1 = 0, $luptr2 = 0, $lusup = 0, $no_zeros = 0, $nrow = 0, $nsupc = 0, $nsupr = 0, $one = 0, $ops = 0, $r_hi = 0, $r_ind = 0, $repfnz_col = 0, $segsze = 0, $supno = 0, $tempv1 = 0, $ukj = 0, $ukj1 = 0, $ukj2 = 0;
 var $xlsub = 0, $xlusup = 0, $xsup = 0, $zero = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 624|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ukj = sp + 416|0;
 $ukj1 = sp + 400|0;
 $ukj2 = sp + 384|0;
 $zero = sp + 368|0;
 $one = sp + 352|0;
 $comp_temp = sp + 336|0;
 $comp_temp1 = sp + 320|0;
 $0 = $m;
 $1 = $w;
 $2 = $jcol;
 $3 = $nseg;
 $4 = $dense;
 $5 = $tempv;
 $6 = $segrep;
 $7 = $repfnz;
 $8 = $Glu;
 $9 = $stat;
 ;HEAP32[$zero>>2]=0|0;HEAP32[$zero+4>>2]=0|0;HEAP32[$zero+8>>2]=0|0;HEAP32[$zero+12>>2]=0|0;
 ;HEAP32[$one>>2]=HEAP32[2504>>2]|0;HEAP32[$one+4>>2]=HEAP32[2504+4>>2]|0;HEAP32[$one+8>>2]=HEAP32[2504+8>>2]|0;HEAP32[$one+12>>2]=HEAP32[2504+12>>2]|0;
 $10 = $9;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $ops = $12;
 $13 = $8;
 $14 = HEAP32[$13>>2]|0;
 $xsup = $14;
 $15 = $8;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $supno = $17;
 $18 = $8;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $lsub = $20;
 $21 = $8;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $xlsub = $23;
 $24 = $8;
 $25 = ((($24)) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $lusup = $26;
 $27 = $8;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 $xlusup = $29;
 $30 = HEAP32[2440>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = (_sp_ienv(3)|0);
  $33 = (_sp_ienv(7)|0);
  $34 = ($32|0)>($33|0);
  if ($34) {
   $35 = (_sp_ienv(3)|0);
   $37 = $35;
  } else {
   $36 = (_sp_ienv(7)|0);
   $37 = $36;
  }
  HEAP32[2448>>2] = $37;
  $38 = (_sp_ienv(4)|0);
  HEAP32[2456>>2] = $38;
  $39 = (_sp_ienv(5)|0);
  HEAP32[2464>>2] = $39;
  HEAP32[2440>>2] = 0;
 }
 $40 = HEAP32[2448>>2]|0;
 $41 = HEAP32[2456>>2]|0;
 $42 = (($40) + ($41))|0;
 $ldaTmp = $42;
 $43 = $3;
 $44 = (($43) - 1)|0;
 $k = $44;
 $ksub = 0;
 while(1) {
  $45 = $ksub;
  $46 = $3;
  $47 = ($45|0)<($46|0);
  if (!($47)) {
   break;
  }
  $48 = $k;
  $49 = (($48) + -1)|0;
  $k = $49;
  $50 = $6;
  $51 = (($50) + ($48<<2)|0);
  $52 = HEAP32[$51>>2]|0;
  $krep = $52;
  $53 = $krep;
  $54 = $supno;
  $55 = (($54) + ($53<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $xsup;
  $58 = (($57) + ($56<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  $fsupc = $59;
  $60 = $krep;
  $61 = $fsupc;
  $62 = (($60) - ($61))|0;
  $63 = (($62) + 1)|0;
  $nsupc = $63;
  $64 = $fsupc;
  $65 = (($64) + 1)|0;
  $66 = $xlsub;
  $67 = (($66) + ($65<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = $fsupc;
  $70 = $xlsub;
  $71 = (($70) + ($69<<2)|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($68) - ($72))|0;
  $nsupr = $73;
  $74 = $nsupr;
  $75 = $nsupc;
  $76 = (($74) - ($75))|0;
  $nrow = $76;
  $77 = $fsupc;
  $78 = $xlsub;
  $79 = (($78) + ($77<<2)|0);
  $80 = HEAP32[$79>>2]|0;
  $lptr = $80;
  $81 = $lptr;
  $82 = $nsupc;
  $83 = (($81) + ($82))|0;
  $84 = (($83) - 1)|0;
  $krep_ind = $84;
  $85 = $7;
  $repfnz_col = $85;
  $86 = $4;
  $dense_col = $86;
  $87 = $nsupc;
  $88 = HEAP32[2464>>2]|0;
  $89 = ($87|0)>=($88|0);
  L11: do {
   if ($89) {
    $90 = $nrow;
    $91 = HEAP32[2456>>2]|0;
    $92 = ($90|0)>($91|0);
    if ($92) {
     $93 = $5;
     $TriTmp = $93;
     $94 = $2;
     $jj = $94;
     while(1) {
      $95 = $jj;
      $96 = $2;
      $97 = $1;
      $98 = (($96) + ($97))|0;
      $99 = ($95|0)<($98|0);
      if (!($99)) {
       break;
      }
      $100 = $krep;
      $101 = $repfnz_col;
      $102 = (($101) + ($100<<2)|0);
      $103 = HEAP32[$102>>2]|0;
      $kfnz = $103;
      $104 = $kfnz;
      $105 = ($104|0)==(-1);
      L17: do {
       if (!($105)) {
        $106 = $krep;
        $107 = $kfnz;
        $108 = (($106) - ($107))|0;
        $109 = (($108) + 1)|0;
        $segsze = $109;
        $110 = $fsupc;
        $111 = $xlusup;
        $112 = (($111) + ($110<<2)|0);
        $113 = HEAP32[$112>>2]|0;
        $luptr = $113;
        $114 = $segsze;
        $115 = $114<<2;
        $116 = $segsze;
        $117 = (($116) - 1)|0;
        $118 = Math_imul($115, $117)|0;
        $119 = (+($118|0));
        $120 = $ops;
        $121 = ((($120)) + 52|0);
        $122 = +HEAPF32[$121>>2];
        $123 = $122 + $119;
        HEAPF32[$121>>2] = $123;
        $124 = $nrow;
        $125 = $124<<3;
        $126 = $segsze;
        $127 = Math_imul($125, $126)|0;
        $128 = (+($127|0));
        $129 = $ops;
        $130 = ((($129)) + 56|0);
        $131 = +HEAPF32[$130>>2];
        $132 = $131 + $128;
        HEAPF32[$130>>2] = $132;
        $133 = $segsze;
        $134 = ($133|0)==(1);
        if ($134) {
         $135 = $krep_ind;
         $136 = $lsub;
         $137 = (($136) + ($135<<2)|0);
         $138 = HEAP32[$137>>2]|0;
         $139 = $dense_col;
         $140 = (($139) + ($138<<4)|0);
         ;HEAP32[$ukj>>2]=HEAP32[$140>>2]|0;HEAP32[$ukj+4>>2]=HEAP32[$140+4>>2]|0;HEAP32[$ukj+8>>2]=HEAP32[$140+8>>2]|0;HEAP32[$ukj+12>>2]=HEAP32[$140+12>>2]|0;
         $141 = $nsupr;
         $142 = $nsupc;
         $143 = (($142) - 1)|0;
         $144 = Math_imul($141, $143)|0;
         $145 = $nsupc;
         $146 = (($144) + ($145))|0;
         $147 = $luptr;
         $148 = (($147) + ($146))|0;
         $luptr = $148;
         $149 = $lptr;
         $150 = $nsupc;
         $151 = (($149) + ($150))|0;
         $i = $151;
         while(1) {
          $152 = $i;
          $153 = $fsupc;
          $154 = (($153) + 1)|0;
          $155 = $xlsub;
          $156 = (($155) + ($154<<2)|0);
          $157 = HEAP32[$156>>2]|0;
          $158 = ($152|0)<($157|0);
          if (!($158)) {
           break L17;
          }
          $159 = $i;
          $160 = $lsub;
          $161 = (($160) + ($159<<2)|0);
          $162 = HEAP32[$161>>2]|0;
          $irow = $162;
          $163 = +HEAPF64[$ukj>>3];
          $164 = $luptr;
          $165 = $lusup;
          $166 = (($165) + ($164<<4)|0);
          $167 = +HEAPF64[$166>>3];
          $168 = $163 * $167;
          $169 = ((($ukj)) + 8|0);
          $170 = +HEAPF64[$169>>3];
          $171 = $luptr;
          $172 = $lusup;
          $173 = (($172) + ($171<<4)|0);
          $174 = ((($173)) + 8|0);
          $175 = +HEAPF64[$174>>3];
          $176 = $170 * $175;
          $177 = $168 - $176;
          $cr = $177;
          $178 = ((($ukj)) + 8|0);
          $179 = +HEAPF64[$178>>3];
          $180 = $luptr;
          $181 = $lusup;
          $182 = (($181) + ($180<<4)|0);
          $183 = +HEAPF64[$182>>3];
          $184 = $179 * $183;
          $185 = +HEAPF64[$ukj>>3];
          $186 = $luptr;
          $187 = $lusup;
          $188 = (($187) + ($186<<4)|0);
          $189 = ((($188)) + 8|0);
          $190 = +HEAPF64[$189>>3];
          $191 = $185 * $190;
          $192 = $184 + $191;
          $ci = $192;
          $193 = $cr;
          HEAPF64[$comp_temp>>3] = $193;
          $194 = $ci;
          $195 = ((($comp_temp)) + 8|0);
          HEAPF64[$195>>3] = $194;
          $196 = $irow;
          $197 = $dense_col;
          $198 = (($197) + ($196<<4)|0);
          $199 = +HEAPF64[$198>>3];
          $200 = +HEAPF64[$comp_temp>>3];
          $201 = $199 - $200;
          $202 = $irow;
          $203 = $dense_col;
          $204 = (($203) + ($202<<4)|0);
          HEAPF64[$204>>3] = $201;
          $205 = $irow;
          $206 = $dense_col;
          $207 = (($206) + ($205<<4)|0);
          $208 = ((($207)) + 8|0);
          $209 = +HEAPF64[$208>>3];
          $210 = ((($comp_temp)) + 8|0);
          $211 = +HEAPF64[$210>>3];
          $212 = $209 - $211;
          $213 = $irow;
          $214 = $dense_col;
          $215 = (($214) + ($213<<4)|0);
          $216 = ((($215)) + 8|0);
          HEAPF64[$216>>3] = $212;
          $217 = $luptr;
          $218 = (($217) + 1)|0;
          $luptr = $218;
          $219 = $i;
          $220 = (($219) + 1)|0;
          $i = $220;
         }
        }
        $221 = $segsze;
        $222 = ($221|0)<=(3);
        if (!($222)) {
         $727 = $kfnz;
         $728 = $fsupc;
         $729 = (($727) - ($728))|0;
         $no_zeros = $729;
         $730 = $lptr;
         $731 = $no_zeros;
         $732 = (($730) + ($731))|0;
         $isub = $732;
         $i = 0;
         while(1) {
          $733 = $i;
          $734 = $segsze;
          $735 = ($733|0)<($734|0);
          if (!($735)) {
           break;
          }
          $736 = $isub;
          $737 = $lsub;
          $738 = (($737) + ($736<<2)|0);
          $739 = HEAP32[$738>>2]|0;
          $irow = $739;
          $740 = $i;
          $741 = $TriTmp;
          $742 = (($741) + ($740<<4)|0);
          $743 = $irow;
          $744 = $dense_col;
          $745 = (($744) + ($743<<4)|0);
          ;HEAP32[$742>>2]=HEAP32[$745>>2]|0;HEAP32[$742+4>>2]=HEAP32[$745+4>>2]|0;HEAP32[$742+8>>2]=HEAP32[$745+8>>2]|0;HEAP32[$742+12>>2]=HEAP32[$745+12>>2]|0;
          $746 = $isub;
          $747 = (($746) + 1)|0;
          $isub = $747;
          $748 = $i;
          $749 = (($748) + 1)|0;
          $i = $749;
         }
         $750 = $nsupr;
         $751 = $no_zeros;
         $752 = Math_imul($750, $751)|0;
         $753 = $no_zeros;
         $754 = (($752) + ($753))|0;
         $755 = $luptr;
         $756 = (($755) + ($754))|0;
         $luptr = $756;
         $757 = $nsupr;
         $758 = $segsze;
         $759 = $luptr;
         $760 = $lusup;
         $761 = (($760) + ($759<<4)|0);
         $762 = $TriTmp;
         _zlsolve($757,$758,$761,$762);
         break;
        }
        $223 = $krep_ind;
        $224 = $lsub;
        $225 = (($224) + ($223<<2)|0);
        $226 = HEAP32[$225>>2]|0;
        $227 = $dense_col;
        $228 = (($227) + ($226<<4)|0);
        ;HEAP32[$ukj>>2]=HEAP32[$228>>2]|0;HEAP32[$ukj+4>>2]=HEAP32[$228+4>>2]|0;HEAP32[$ukj+8>>2]=HEAP32[$228+8>>2]|0;HEAP32[$ukj+12>>2]=HEAP32[$228+12>>2]|0;
        $229 = $krep_ind;
        $230 = (($229) - 1)|0;
        $231 = $lsub;
        $232 = (($231) + ($230<<2)|0);
        $233 = HEAP32[$232>>2]|0;
        $234 = $dense_col;
        $235 = (($234) + ($233<<4)|0);
        ;HEAP32[$ukj1>>2]=HEAP32[$235>>2]|0;HEAP32[$ukj1+4>>2]=HEAP32[$235+4>>2]|0;HEAP32[$ukj1+8>>2]=HEAP32[$235+8>>2]|0;HEAP32[$ukj1+12>>2]=HEAP32[$235+12>>2]|0;
        $236 = $nsupr;
        $237 = $nsupc;
        $238 = (($237) - 1)|0;
        $239 = Math_imul($236, $238)|0;
        $240 = $nsupc;
        $241 = (($239) + ($240))|0;
        $242 = (($241) - 1)|0;
        $243 = $luptr;
        $244 = (($243) + ($242))|0;
        $luptr = $244;
        $245 = $luptr;
        $246 = $nsupr;
        $247 = (($245) - ($246))|0;
        $luptr1 = $247;
        $248 = $segsze;
        $249 = ($248|0)==(2);
        if ($249) {
         $250 = +HEAPF64[$ukj1>>3];
         $251 = $luptr1;
         $252 = $lusup;
         $253 = (($252) + ($251<<4)|0);
         $254 = +HEAPF64[$253>>3];
         $255 = $250 * $254;
         $256 = ((($ukj1)) + 8|0);
         $257 = +HEAPF64[$256>>3];
         $258 = $luptr1;
         $259 = $lusup;
         $260 = (($259) + ($258<<4)|0);
         $261 = ((($260)) + 8|0);
         $262 = +HEAPF64[$261>>3];
         $263 = $257 * $262;
         $264 = $255 - $263;
         $cr1 = $264;
         $265 = ((($ukj1)) + 8|0);
         $266 = +HEAPF64[$265>>3];
         $267 = $luptr1;
         $268 = $lusup;
         $269 = (($268) + ($267<<4)|0);
         $270 = +HEAPF64[$269>>3];
         $271 = $266 * $270;
         $272 = +HEAPF64[$ukj1>>3];
         $273 = $luptr1;
         $274 = $lusup;
         $275 = (($274) + ($273<<4)|0);
         $276 = ((($275)) + 8|0);
         $277 = +HEAPF64[$276>>3];
         $278 = $272 * $277;
         $279 = $271 + $278;
         $ci2 = $279;
         $280 = $cr1;
         HEAPF64[$comp_temp>>3] = $280;
         $281 = $ci2;
         $282 = ((($comp_temp)) + 8|0);
         HEAPF64[$282>>3] = $281;
         $283 = +HEAPF64[$ukj>>3];
         $284 = +HEAPF64[$comp_temp>>3];
         $285 = $283 - $284;
         HEAPF64[$ukj>>3] = $285;
         $286 = ((($ukj)) + 8|0);
         $287 = +HEAPF64[$286>>3];
         $288 = ((($comp_temp)) + 8|0);
         $289 = +HEAPF64[$288>>3];
         $290 = $287 - $289;
         $291 = ((($ukj)) + 8|0);
         HEAPF64[$291>>3] = $290;
         $292 = $krep_ind;
         $293 = $lsub;
         $294 = (($293) + ($292<<2)|0);
         $295 = HEAP32[$294>>2]|0;
         $296 = $dense_col;
         $297 = (($296) + ($295<<4)|0);
         ;HEAP32[$297>>2]=HEAP32[$ukj>>2]|0;HEAP32[$297+4>>2]=HEAP32[$ukj+4>>2]|0;HEAP32[$297+8>>2]=HEAP32[$ukj+8>>2]|0;HEAP32[$297+12>>2]=HEAP32[$ukj+12>>2]|0;
         $298 = $lptr;
         $299 = $nsupc;
         $300 = (($298) + ($299))|0;
         $i = $300;
         while(1) {
          $301 = $i;
          $302 = $fsupc;
          $303 = (($302) + 1)|0;
          $304 = $xlsub;
          $305 = (($304) + ($303<<2)|0);
          $306 = HEAP32[$305>>2]|0;
          $307 = ($301|0)<($306|0);
          if (!($307)) {
           break L17;
          }
          $308 = $i;
          $309 = $lsub;
          $310 = (($309) + ($308<<2)|0);
          $311 = HEAP32[$310>>2]|0;
          $irow = $311;
          $312 = $luptr;
          $313 = (($312) + 1)|0;
          $luptr = $313;
          $314 = $luptr1;
          $315 = (($314) + 1)|0;
          $luptr1 = $315;
          $316 = +HEAPF64[$ukj>>3];
          $317 = $luptr;
          $318 = $lusup;
          $319 = (($318) + ($317<<4)|0);
          $320 = +HEAPF64[$319>>3];
          $321 = $316 * $320;
          $322 = ((($ukj)) + 8|0);
          $323 = +HEAPF64[$322>>3];
          $324 = $luptr;
          $325 = $lusup;
          $326 = (($325) + ($324<<4)|0);
          $327 = ((($326)) + 8|0);
          $328 = +HEAPF64[$327>>3];
          $329 = $323 * $328;
          $330 = $321 - $329;
          $cr3 = $330;
          $331 = ((($ukj)) + 8|0);
          $332 = +HEAPF64[$331>>3];
          $333 = $luptr;
          $334 = $lusup;
          $335 = (($334) + ($333<<4)|0);
          $336 = +HEAPF64[$335>>3];
          $337 = $332 * $336;
          $338 = +HEAPF64[$ukj>>3];
          $339 = $luptr;
          $340 = $lusup;
          $341 = (($340) + ($339<<4)|0);
          $342 = ((($341)) + 8|0);
          $343 = +HEAPF64[$342>>3];
          $344 = $338 * $343;
          $345 = $337 + $344;
          $ci4 = $345;
          $346 = $cr3;
          HEAPF64[$comp_temp>>3] = $346;
          $347 = $ci4;
          $348 = ((($comp_temp)) + 8|0);
          HEAPF64[$348>>3] = $347;
          $349 = +HEAPF64[$ukj1>>3];
          $350 = $luptr1;
          $351 = $lusup;
          $352 = (($351) + ($350<<4)|0);
          $353 = +HEAPF64[$352>>3];
          $354 = $349 * $353;
          $355 = ((($ukj1)) + 8|0);
          $356 = +HEAPF64[$355>>3];
          $357 = $luptr1;
          $358 = $lusup;
          $359 = (($358) + ($357<<4)|0);
          $360 = ((($359)) + 8|0);
          $361 = +HEAPF64[$360>>3];
          $362 = $356 * $361;
          $363 = $354 - $362;
          $cr5 = $363;
          $364 = ((($ukj1)) + 8|0);
          $365 = +HEAPF64[$364>>3];
          $366 = $luptr1;
          $367 = $lusup;
          $368 = (($367) + ($366<<4)|0);
          $369 = +HEAPF64[$368>>3];
          $370 = $365 * $369;
          $371 = +HEAPF64[$ukj1>>3];
          $372 = $luptr1;
          $373 = $lusup;
          $374 = (($373) + ($372<<4)|0);
          $375 = ((($374)) + 8|0);
          $376 = +HEAPF64[$375>>3];
          $377 = $371 * $376;
          $378 = $370 + $377;
          $ci6 = $378;
          $379 = $cr5;
          HEAPF64[$comp_temp1>>3] = $379;
          $380 = $ci6;
          $381 = ((($comp_temp1)) + 8|0);
          HEAPF64[$381>>3] = $380;
          $382 = +HEAPF64[$comp_temp>>3];
          $383 = +HEAPF64[$comp_temp1>>3];
          $384 = $382 + $383;
          HEAPF64[$comp_temp>>3] = $384;
          $385 = ((($comp_temp)) + 8|0);
          $386 = +HEAPF64[$385>>3];
          $387 = ((($comp_temp1)) + 8|0);
          $388 = +HEAPF64[$387>>3];
          $389 = $386 + $388;
          $390 = ((($comp_temp)) + 8|0);
          HEAPF64[$390>>3] = $389;
          $391 = $irow;
          $392 = $dense_col;
          $393 = (($392) + ($391<<4)|0);
          $394 = +HEAPF64[$393>>3];
          $395 = +HEAPF64[$comp_temp>>3];
          $396 = $394 - $395;
          $397 = $irow;
          $398 = $dense_col;
          $399 = (($398) + ($397<<4)|0);
          HEAPF64[$399>>3] = $396;
          $400 = $irow;
          $401 = $dense_col;
          $402 = (($401) + ($400<<4)|0);
          $403 = ((($402)) + 8|0);
          $404 = +HEAPF64[$403>>3];
          $405 = ((($comp_temp)) + 8|0);
          $406 = +HEAPF64[$405>>3];
          $407 = $404 - $406;
          $408 = $irow;
          $409 = $dense_col;
          $410 = (($409) + ($408<<4)|0);
          $411 = ((($410)) + 8|0);
          HEAPF64[$411>>3] = $407;
          $412 = $i;
          $413 = (($412) + 1)|0;
          $i = $413;
         }
        } else {
         $414 = $krep_ind;
         $415 = (($414) - 2)|0;
         $416 = $lsub;
         $417 = (($416) + ($415<<2)|0);
         $418 = HEAP32[$417>>2]|0;
         $419 = $dense_col;
         $420 = (($419) + ($418<<4)|0);
         ;HEAP32[$ukj2>>2]=HEAP32[$420>>2]|0;HEAP32[$ukj2+4>>2]=HEAP32[$420+4>>2]|0;HEAP32[$ukj2+8>>2]=HEAP32[$420+8>>2]|0;HEAP32[$ukj2+12>>2]=HEAP32[$420+12>>2]|0;
         $421 = $luptr1;
         $422 = $nsupr;
         $423 = (($421) - ($422))|0;
         $luptr2 = $423;
         $424 = +HEAPF64[$ukj2>>3];
         $425 = $luptr2;
         $426 = (($425) - 1)|0;
         $427 = $lusup;
         $428 = (($427) + ($426<<4)|0);
         $429 = +HEAPF64[$428>>3];
         $430 = $424 * $429;
         $431 = ((($ukj2)) + 8|0);
         $432 = +HEAPF64[$431>>3];
         $433 = $luptr2;
         $434 = (($433) - 1)|0;
         $435 = $lusup;
         $436 = (($435) + ($434<<4)|0);
         $437 = ((($436)) + 8|0);
         $438 = +HEAPF64[$437>>3];
         $439 = $432 * $438;
         $440 = $430 - $439;
         $cr7 = $440;
         $441 = ((($ukj2)) + 8|0);
         $442 = +HEAPF64[$441>>3];
         $443 = $luptr2;
         $444 = (($443) - 1)|0;
         $445 = $lusup;
         $446 = (($445) + ($444<<4)|0);
         $447 = +HEAPF64[$446>>3];
         $448 = $442 * $447;
         $449 = +HEAPF64[$ukj2>>3];
         $450 = $luptr2;
         $451 = (($450) - 1)|0;
         $452 = $lusup;
         $453 = (($452) + ($451<<4)|0);
         $454 = ((($453)) + 8|0);
         $455 = +HEAPF64[$454>>3];
         $456 = $449 * $455;
         $457 = $448 + $456;
         $ci8 = $457;
         $458 = $cr7;
         HEAPF64[$comp_temp>>3] = $458;
         $459 = $ci8;
         $460 = ((($comp_temp)) + 8|0);
         HEAPF64[$460>>3] = $459;
         $461 = +HEAPF64[$ukj1>>3];
         $462 = +HEAPF64[$comp_temp>>3];
         $463 = $461 - $462;
         HEAPF64[$ukj1>>3] = $463;
         $464 = ((($ukj1)) + 8|0);
         $465 = +HEAPF64[$464>>3];
         $466 = ((($comp_temp)) + 8|0);
         $467 = +HEAPF64[$466>>3];
         $468 = $465 - $467;
         $469 = ((($ukj1)) + 8|0);
         HEAPF64[$469>>3] = $468;
         $470 = +HEAPF64[$ukj1>>3];
         $471 = $luptr1;
         $472 = $lusup;
         $473 = (($472) + ($471<<4)|0);
         $474 = +HEAPF64[$473>>3];
         $475 = $470 * $474;
         $476 = ((($ukj1)) + 8|0);
         $477 = +HEAPF64[$476>>3];
         $478 = $luptr1;
         $479 = $lusup;
         $480 = (($479) + ($478<<4)|0);
         $481 = ((($480)) + 8|0);
         $482 = +HEAPF64[$481>>3];
         $483 = $477 * $482;
         $484 = $475 - $483;
         $cr9 = $484;
         $485 = ((($ukj1)) + 8|0);
         $486 = +HEAPF64[$485>>3];
         $487 = $luptr1;
         $488 = $lusup;
         $489 = (($488) + ($487<<4)|0);
         $490 = +HEAPF64[$489>>3];
         $491 = $486 * $490;
         $492 = +HEAPF64[$ukj1>>3];
         $493 = $luptr1;
         $494 = $lusup;
         $495 = (($494) + ($493<<4)|0);
         $496 = ((($495)) + 8|0);
         $497 = +HEAPF64[$496>>3];
         $498 = $492 * $497;
         $499 = $491 + $498;
         $ci10 = $499;
         $500 = $cr9;
         HEAPF64[$comp_temp>>3] = $500;
         $501 = $ci10;
         $502 = ((($comp_temp)) + 8|0);
         HEAPF64[$502>>3] = $501;
         $503 = +HEAPF64[$ukj2>>3];
         $504 = $luptr2;
         $505 = $lusup;
         $506 = (($505) + ($504<<4)|0);
         $507 = +HEAPF64[$506>>3];
         $508 = $503 * $507;
         $509 = ((($ukj2)) + 8|0);
         $510 = +HEAPF64[$509>>3];
         $511 = $luptr2;
         $512 = $lusup;
         $513 = (($512) + ($511<<4)|0);
         $514 = ((($513)) + 8|0);
         $515 = +HEAPF64[$514>>3];
         $516 = $510 * $515;
         $517 = $508 - $516;
         $cr11 = $517;
         $518 = ((($ukj2)) + 8|0);
         $519 = +HEAPF64[$518>>3];
         $520 = $luptr2;
         $521 = $lusup;
         $522 = (($521) + ($520<<4)|0);
         $523 = +HEAPF64[$522>>3];
         $524 = $519 * $523;
         $525 = +HEAPF64[$ukj2>>3];
         $526 = $luptr2;
         $527 = $lusup;
         $528 = (($527) + ($526<<4)|0);
         $529 = ((($528)) + 8|0);
         $530 = +HEAPF64[$529>>3];
         $531 = $525 * $530;
         $532 = $524 + $531;
         $ci12 = $532;
         $533 = $cr11;
         HEAPF64[$comp_temp1>>3] = $533;
         $534 = $ci12;
         $535 = ((($comp_temp1)) + 8|0);
         HEAPF64[$535>>3] = $534;
         $536 = +HEAPF64[$comp_temp>>3];
         $537 = +HEAPF64[$comp_temp1>>3];
         $538 = $536 + $537;
         HEAPF64[$comp_temp>>3] = $538;
         $539 = ((($comp_temp)) + 8|0);
         $540 = +HEAPF64[$539>>3];
         $541 = ((($comp_temp1)) + 8|0);
         $542 = +HEAPF64[$541>>3];
         $543 = $540 + $542;
         $544 = ((($comp_temp)) + 8|0);
         HEAPF64[$544>>3] = $543;
         $545 = +HEAPF64[$ukj>>3];
         $546 = +HEAPF64[$comp_temp>>3];
         $547 = $545 - $546;
         HEAPF64[$ukj>>3] = $547;
         $548 = ((($ukj)) + 8|0);
         $549 = +HEAPF64[$548>>3];
         $550 = ((($comp_temp)) + 8|0);
         $551 = +HEAPF64[$550>>3];
         $552 = $549 - $551;
         $553 = ((($ukj)) + 8|0);
         HEAPF64[$553>>3] = $552;
         $554 = $krep_ind;
         $555 = $lsub;
         $556 = (($555) + ($554<<2)|0);
         $557 = HEAP32[$556>>2]|0;
         $558 = $dense_col;
         $559 = (($558) + ($557<<4)|0);
         ;HEAP32[$559>>2]=HEAP32[$ukj>>2]|0;HEAP32[$559+4>>2]=HEAP32[$ukj+4>>2]|0;HEAP32[$559+8>>2]=HEAP32[$ukj+8>>2]|0;HEAP32[$559+12>>2]=HEAP32[$ukj+12>>2]|0;
         $560 = $krep_ind;
         $561 = (($560) - 1)|0;
         $562 = $lsub;
         $563 = (($562) + ($561<<2)|0);
         $564 = HEAP32[$563>>2]|0;
         $565 = $dense_col;
         $566 = (($565) + ($564<<4)|0);
         ;HEAP32[$566>>2]=HEAP32[$ukj1>>2]|0;HEAP32[$566+4>>2]=HEAP32[$ukj1+4>>2]|0;HEAP32[$566+8>>2]=HEAP32[$ukj1+8>>2]|0;HEAP32[$566+12>>2]=HEAP32[$ukj1+12>>2]|0;
         $567 = $lptr;
         $568 = $nsupc;
         $569 = (($567) + ($568))|0;
         $i = $569;
         while(1) {
          $570 = $i;
          $571 = $fsupc;
          $572 = (($571) + 1)|0;
          $573 = $xlsub;
          $574 = (($573) + ($572<<2)|0);
          $575 = HEAP32[$574>>2]|0;
          $576 = ($570|0)<($575|0);
          if (!($576)) {
           break L17;
          }
          $577 = $i;
          $578 = $lsub;
          $579 = (($578) + ($577<<2)|0);
          $580 = HEAP32[$579>>2]|0;
          $irow = $580;
          $581 = $luptr;
          $582 = (($581) + 1)|0;
          $luptr = $582;
          $583 = $luptr1;
          $584 = (($583) + 1)|0;
          $luptr1 = $584;
          $585 = $luptr2;
          $586 = (($585) + 1)|0;
          $luptr2 = $586;
          $587 = +HEAPF64[$ukj>>3];
          $588 = $luptr;
          $589 = $lusup;
          $590 = (($589) + ($588<<4)|0);
          $591 = +HEAPF64[$590>>3];
          $592 = $587 * $591;
          $593 = ((($ukj)) + 8|0);
          $594 = +HEAPF64[$593>>3];
          $595 = $luptr;
          $596 = $lusup;
          $597 = (($596) + ($595<<4)|0);
          $598 = ((($597)) + 8|0);
          $599 = +HEAPF64[$598>>3];
          $600 = $594 * $599;
          $601 = $592 - $600;
          $cr13 = $601;
          $602 = ((($ukj)) + 8|0);
          $603 = +HEAPF64[$602>>3];
          $604 = $luptr;
          $605 = $lusup;
          $606 = (($605) + ($604<<4)|0);
          $607 = +HEAPF64[$606>>3];
          $608 = $603 * $607;
          $609 = +HEAPF64[$ukj>>3];
          $610 = $luptr;
          $611 = $lusup;
          $612 = (($611) + ($610<<4)|0);
          $613 = ((($612)) + 8|0);
          $614 = +HEAPF64[$613>>3];
          $615 = $609 * $614;
          $616 = $608 + $615;
          $ci14 = $616;
          $617 = $cr13;
          HEAPF64[$comp_temp>>3] = $617;
          $618 = $ci14;
          $619 = ((($comp_temp)) + 8|0);
          HEAPF64[$619>>3] = $618;
          $620 = +HEAPF64[$ukj1>>3];
          $621 = $luptr1;
          $622 = $lusup;
          $623 = (($622) + ($621<<4)|0);
          $624 = +HEAPF64[$623>>3];
          $625 = $620 * $624;
          $626 = ((($ukj1)) + 8|0);
          $627 = +HEAPF64[$626>>3];
          $628 = $luptr1;
          $629 = $lusup;
          $630 = (($629) + ($628<<4)|0);
          $631 = ((($630)) + 8|0);
          $632 = +HEAPF64[$631>>3];
          $633 = $627 * $632;
          $634 = $625 - $633;
          $cr15 = $634;
          $635 = ((($ukj1)) + 8|0);
          $636 = +HEAPF64[$635>>3];
          $637 = $luptr1;
          $638 = $lusup;
          $639 = (($638) + ($637<<4)|0);
          $640 = +HEAPF64[$639>>3];
          $641 = $636 * $640;
          $642 = +HEAPF64[$ukj1>>3];
          $643 = $luptr1;
          $644 = $lusup;
          $645 = (($644) + ($643<<4)|0);
          $646 = ((($645)) + 8|0);
          $647 = +HEAPF64[$646>>3];
          $648 = $642 * $647;
          $649 = $641 + $648;
          $ci16 = $649;
          $650 = $cr15;
          HEAPF64[$comp_temp1>>3] = $650;
          $651 = $ci16;
          $652 = ((($comp_temp1)) + 8|0);
          HEAPF64[$652>>3] = $651;
          $653 = +HEAPF64[$comp_temp>>3];
          $654 = +HEAPF64[$comp_temp1>>3];
          $655 = $653 + $654;
          HEAPF64[$comp_temp>>3] = $655;
          $656 = ((($comp_temp)) + 8|0);
          $657 = +HEAPF64[$656>>3];
          $658 = ((($comp_temp1)) + 8|0);
          $659 = +HEAPF64[$658>>3];
          $660 = $657 + $659;
          $661 = ((($comp_temp)) + 8|0);
          HEAPF64[$661>>3] = $660;
          $662 = +HEAPF64[$ukj2>>3];
          $663 = $luptr2;
          $664 = $lusup;
          $665 = (($664) + ($663<<4)|0);
          $666 = +HEAPF64[$665>>3];
          $667 = $662 * $666;
          $668 = ((($ukj2)) + 8|0);
          $669 = +HEAPF64[$668>>3];
          $670 = $luptr2;
          $671 = $lusup;
          $672 = (($671) + ($670<<4)|0);
          $673 = ((($672)) + 8|0);
          $674 = +HEAPF64[$673>>3];
          $675 = $669 * $674;
          $676 = $667 - $675;
          $cr17 = $676;
          $677 = ((($ukj2)) + 8|0);
          $678 = +HEAPF64[$677>>3];
          $679 = $luptr2;
          $680 = $lusup;
          $681 = (($680) + ($679<<4)|0);
          $682 = +HEAPF64[$681>>3];
          $683 = $678 * $682;
          $684 = +HEAPF64[$ukj2>>3];
          $685 = $luptr2;
          $686 = $lusup;
          $687 = (($686) + ($685<<4)|0);
          $688 = ((($687)) + 8|0);
          $689 = +HEAPF64[$688>>3];
          $690 = $684 * $689;
          $691 = $683 + $690;
          $ci18 = $691;
          $692 = $cr17;
          HEAPF64[$comp_temp1>>3] = $692;
          $693 = $ci18;
          $694 = ((($comp_temp1)) + 8|0);
          HEAPF64[$694>>3] = $693;
          $695 = +HEAPF64[$comp_temp>>3];
          $696 = +HEAPF64[$comp_temp1>>3];
          $697 = $695 + $696;
          HEAPF64[$comp_temp>>3] = $697;
          $698 = ((($comp_temp)) + 8|0);
          $699 = +HEAPF64[$698>>3];
          $700 = ((($comp_temp1)) + 8|0);
          $701 = +HEAPF64[$700>>3];
          $702 = $699 + $701;
          $703 = ((($comp_temp)) + 8|0);
          HEAPF64[$703>>3] = $702;
          $704 = $irow;
          $705 = $dense_col;
          $706 = (($705) + ($704<<4)|0);
          $707 = +HEAPF64[$706>>3];
          $708 = +HEAPF64[$comp_temp>>3];
          $709 = $707 - $708;
          $710 = $irow;
          $711 = $dense_col;
          $712 = (($711) + ($710<<4)|0);
          HEAPF64[$712>>3] = $709;
          $713 = $irow;
          $714 = $dense_col;
          $715 = (($714) + ($713<<4)|0);
          $716 = ((($715)) + 8|0);
          $717 = +HEAPF64[$716>>3];
          $718 = ((($comp_temp)) + 8|0);
          $719 = +HEAPF64[$718>>3];
          $720 = $717 - $719;
          $721 = $irow;
          $722 = $dense_col;
          $723 = (($722) + ($721<<4)|0);
          $724 = ((($723)) + 8|0);
          HEAPF64[$724>>3] = $720;
          $725 = $i;
          $726 = (($725) + 1)|0;
          $i = $726;
         }
        }
       }
      } while(0);
      $763 = $jj;
      $764 = (($763) + 1)|0;
      $jj = $764;
      $765 = $0;
      $766 = $repfnz_col;
      $767 = (($766) + ($765<<2)|0);
      $repfnz_col = $767;
      $768 = $0;
      $769 = $dense_col;
      $770 = (($769) + ($768<<4)|0);
      $dense_col = $770;
      $771 = $ldaTmp;
      $772 = $TriTmp;
      $773 = (($772) + ($771<<4)|0);
      $TriTmp = $773;
     }
     $r_ind = 0;
     while(1) {
      $774 = $r_ind;
      $775 = $nrow;
      $776 = ($774|0)<($775|0);
      if (!($776)) {
       break;
      }
      $777 = $nrow;
      $778 = $r_ind;
      $779 = HEAP32[2456>>2]|0;
      $780 = (($778) + ($779))|0;
      $781 = ($777|0)<($780|0);
      if ($781) {
       $782 = $nrow;
       $786 = $782;
      } else {
       $783 = $r_ind;
       $784 = HEAP32[2456>>2]|0;
       $785 = (($783) + ($784))|0;
       $786 = $785;
      }
      $r_hi = $786;
      $787 = HEAP32[2456>>2]|0;
      $788 = $r_hi;
      $789 = $r_ind;
      $790 = (($788) - ($789))|0;
      $791 = ($787|0)<($790|0);
      if ($791) {
       $792 = HEAP32[2456>>2]|0;
       $796 = $792;
      } else {
       $793 = $r_hi;
       $794 = $r_ind;
       $795 = (($793) - ($794))|0;
       $796 = $795;
      }
      $block_nrow = $796;
      $797 = $fsupc;
      $798 = $xlusup;
      $799 = (($798) + ($797<<2)|0);
      $800 = HEAP32[$799>>2]|0;
      $801 = $nsupc;
      $802 = (($800) + ($801))|0;
      $803 = $r_ind;
      $804 = (($802) + ($803))|0;
      $luptr = $804;
      $805 = $lptr;
      $806 = $nsupc;
      $807 = (($805) + ($806))|0;
      $808 = $r_ind;
      $809 = (($807) + ($808))|0;
      $isub1 = $809;
      $810 = $7;
      $repfnz_col = $810;
      $811 = $5;
      $TriTmp = $811;
      $812 = $4;
      $dense_col = $812;
      $813 = $2;
      $jj = $813;
      while(1) {
       $814 = $jj;
       $815 = $2;
       $816 = $1;
       $817 = (($815) + ($816))|0;
       $818 = ($814|0)<($817|0);
       if (!($818)) {
        break;
       }
       $819 = $krep;
       $820 = $repfnz_col;
       $821 = (($820) + ($819<<2)|0);
       $822 = HEAP32[$821>>2]|0;
       $kfnz = $822;
       $823 = $kfnz;
       $824 = ($823|0)==(-1);
       L57: do {
        if (!($824)) {
         $825 = $krep;
         $826 = $kfnz;
         $827 = (($825) - ($826))|0;
         $828 = (($827) + 1)|0;
         $segsze = $828;
         $829 = $segsze;
         $830 = ($829|0)<=(3);
         if (!($830)) {
          $831 = $kfnz;
          $832 = $fsupc;
          $833 = (($831) - ($832))|0;
          $no_zeros = $833;
          $834 = $luptr;
          $835 = $nsupr;
          $836 = $no_zeros;
          $837 = Math_imul($835, $836)|0;
          $838 = (($834) + ($837))|0;
          $luptr1 = $838;
          $839 = HEAP32[2448>>2]|0;
          $840 = $TriTmp;
          $841 = (($840) + ($839<<4)|0);
          $MatvecTmp = $841;
          $842 = $nsupr;
          $843 = $block_nrow;
          $844 = $segsze;
          $845 = $luptr1;
          $846 = $lusup;
          $847 = (($846) + ($845<<4)|0);
          $848 = $TriTmp;
          $849 = $MatvecTmp;
          _zmatvec($842,$843,$844,$847,$848,$849);
          $850 = $isub1;
          $isub = $850;
          $i = 0;
          while(1) {
           $851 = $i;
           $852 = $block_nrow;
           $853 = ($851|0)<($852|0);
           if (!($853)) {
            break L57;
           }
           $854 = $isub;
           $855 = $lsub;
           $856 = (($855) + ($854<<2)|0);
           $857 = HEAP32[$856>>2]|0;
           $irow = $857;
           $858 = $irow;
           $859 = $dense_col;
           $860 = (($859) + ($858<<4)|0);
           $861 = +HEAPF64[$860>>3];
           $862 = $i;
           $863 = $MatvecTmp;
           $864 = (($863) + ($862<<4)|0);
           $865 = +HEAPF64[$864>>3];
           $866 = $861 - $865;
           $867 = $irow;
           $868 = $dense_col;
           $869 = (($868) + ($867<<4)|0);
           HEAPF64[$869>>3] = $866;
           $870 = $irow;
           $871 = $dense_col;
           $872 = (($871) + ($870<<4)|0);
           $873 = ((($872)) + 8|0);
           $874 = +HEAPF64[$873>>3];
           $875 = $i;
           $876 = $MatvecTmp;
           $877 = (($876) + ($875<<4)|0);
           $878 = ((($877)) + 8|0);
           $879 = +HEAPF64[$878>>3];
           $880 = $874 - $879;
           $881 = $irow;
           $882 = $dense_col;
           $883 = (($882) + ($881<<4)|0);
           $884 = ((($883)) + 8|0);
           HEAPF64[$884>>3] = $880;
           $885 = $i;
           $886 = $MatvecTmp;
           $887 = (($886) + ($885<<4)|0);
           ;HEAP32[$887>>2]=HEAP32[$zero>>2]|0;HEAP32[$887+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$887+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$887+12>>2]=HEAP32[$zero+12>>2]|0;
           $888 = $isub;
           $889 = (($888) + 1)|0;
           $isub = $889;
           $890 = $i;
           $891 = (($890) + 1)|0;
           $i = $891;
          }
         }
        }
       } while(0);
       $892 = $jj;
       $893 = (($892) + 1)|0;
       $jj = $893;
       $894 = $0;
       $895 = $repfnz_col;
       $896 = (($895) + ($894<<2)|0);
       $repfnz_col = $896;
       $897 = $0;
       $898 = $dense_col;
       $899 = (($898) + ($897<<4)|0);
       $dense_col = $899;
       $900 = $ldaTmp;
       $901 = $TriTmp;
       $902 = (($901) + ($900<<4)|0);
       $TriTmp = $902;
      }
      $903 = HEAP32[2456>>2]|0;
      $904 = $r_ind;
      $905 = (($904) + ($903))|0;
      $r_ind = $905;
     }
     $906 = $7;
     $repfnz_col = $906;
     $907 = $5;
     $TriTmp = $907;
     $908 = $4;
     $dense_col = $908;
     $909 = $2;
     $jj = $909;
     while(1) {
      $910 = $jj;
      $911 = $2;
      $912 = $1;
      $913 = (($911) + ($912))|0;
      $914 = ($910|0)<($913|0);
      if (!($914)) {
       break L11;
      }
      $915 = $krep;
      $916 = $repfnz_col;
      $917 = (($916) + ($915<<2)|0);
      $918 = HEAP32[$917>>2]|0;
      $kfnz = $918;
      $919 = $kfnz;
      $920 = ($919|0)==(-1);
      L69: do {
       if (!($920)) {
        $921 = $krep;
        $922 = $kfnz;
        $923 = (($921) - ($922))|0;
        $924 = (($923) + 1)|0;
        $segsze = $924;
        $925 = $segsze;
        $926 = ($925|0)<=(3);
        if (!($926)) {
         $927 = $kfnz;
         $928 = $fsupc;
         $929 = (($927) - ($928))|0;
         $no_zeros = $929;
         $930 = $lptr;
         $931 = $no_zeros;
         $932 = (($930) + ($931))|0;
         $isub = $932;
         $i = 0;
         while(1) {
          $933 = $i;
          $934 = $segsze;
          $935 = ($933|0)<($934|0);
          if (!($935)) {
           break L69;
          }
          $936 = $isub;
          $937 = $lsub;
          $938 = (($937) + ($936<<2)|0);
          $939 = HEAP32[$938>>2]|0;
          $irow = $939;
          $940 = $irow;
          $941 = $dense_col;
          $942 = (($941) + ($940<<4)|0);
          $943 = $i;
          $944 = $TriTmp;
          $945 = (($944) + ($943<<4)|0);
          ;HEAP32[$942>>2]=HEAP32[$945>>2]|0;HEAP32[$942+4>>2]=HEAP32[$945+4>>2]|0;HEAP32[$942+8>>2]=HEAP32[$945+8>>2]|0;HEAP32[$942+12>>2]=HEAP32[$945+12>>2]|0;
          $946 = $i;
          $947 = $TriTmp;
          $948 = (($947) + ($946<<4)|0);
          ;HEAP32[$948>>2]=HEAP32[$zero>>2]|0;HEAP32[$948+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$948+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$948+12>>2]=HEAP32[$zero+12>>2]|0;
          $949 = $isub;
          $950 = (($949) + 1)|0;
          $isub = $950;
          $951 = $i;
          $952 = (($951) + 1)|0;
          $i = $952;
         }
        }
       }
      } while(0);
      $953 = $jj;
      $954 = (($953) + 1)|0;
      $jj = $954;
      $955 = $0;
      $956 = $repfnz_col;
      $957 = (($956) + ($955<<2)|0);
      $repfnz_col = $957;
      $958 = $0;
      $959 = $dense_col;
      $960 = (($959) + ($958<<4)|0);
      $dense_col = $960;
      $961 = $ldaTmp;
      $962 = $TriTmp;
      $963 = (($962) + ($961<<4)|0);
      $TriTmp = $963;
     }
    } else {
     label = 55;
    }
   } else {
    label = 55;
   }
  } while(0);
  L76: do {
   if ((label|0) == 55) {
    label = 0;
    $964 = $2;
    $jj = $964;
    while(1) {
     $965 = $jj;
     $966 = $2;
     $967 = $1;
     $968 = (($966) + ($967))|0;
     $969 = ($965|0)<($968|0);
     if (!($969)) {
      break L76;
     }
     $970 = $krep;
     $971 = $repfnz_col;
     $972 = (($971) + ($970<<2)|0);
     $973 = HEAP32[$972>>2]|0;
     $kfnz = $973;
     $974 = $kfnz;
     $975 = ($974|0)==(-1);
     L81: do {
      if (!($975)) {
       $976 = $krep;
       $977 = $kfnz;
       $978 = (($976) - ($977))|0;
       $979 = (($978) + 1)|0;
       $segsze = $979;
       $980 = $fsupc;
       $981 = $xlusup;
       $982 = (($981) + ($980<<2)|0);
       $983 = HEAP32[$982>>2]|0;
       $luptr = $983;
       $984 = $segsze;
       $985 = $984<<2;
       $986 = $segsze;
       $987 = (($986) - 1)|0;
       $988 = Math_imul($985, $987)|0;
       $989 = (+($988|0));
       $990 = $ops;
       $991 = ((($990)) + 52|0);
       $992 = +HEAPF32[$991>>2];
       $993 = $992 + $989;
       HEAPF32[$991>>2] = $993;
       $994 = $nrow;
       $995 = $994<<3;
       $996 = $segsze;
       $997 = Math_imul($995, $996)|0;
       $998 = (+($997|0));
       $999 = $ops;
       $1000 = ((($999)) + 56|0);
       $1001 = +HEAPF32[$1000>>2];
       $1002 = $1001 + $998;
       HEAPF32[$1000>>2] = $1002;
       $1003 = $segsze;
       $1004 = ($1003|0)==(1);
       if ($1004) {
        $1005 = $krep_ind;
        $1006 = $lsub;
        $1007 = (($1006) + ($1005<<2)|0);
        $1008 = HEAP32[$1007>>2]|0;
        $1009 = $dense_col;
        $1010 = (($1009) + ($1008<<4)|0);
        ;HEAP32[$ukj>>2]=HEAP32[$1010>>2]|0;HEAP32[$ukj+4>>2]=HEAP32[$1010+4>>2]|0;HEAP32[$ukj+8>>2]=HEAP32[$1010+8>>2]|0;HEAP32[$ukj+12>>2]=HEAP32[$1010+12>>2]|0;
        $1011 = $nsupr;
        $1012 = $nsupc;
        $1013 = (($1012) - 1)|0;
        $1014 = Math_imul($1011, $1013)|0;
        $1015 = $nsupc;
        $1016 = (($1014) + ($1015))|0;
        $1017 = $luptr;
        $1018 = (($1017) + ($1016))|0;
        $luptr = $1018;
        $1019 = $lptr;
        $1020 = $nsupc;
        $1021 = (($1019) + ($1020))|0;
        $i = $1021;
        while(1) {
         $1022 = $i;
         $1023 = $fsupc;
         $1024 = (($1023) + 1)|0;
         $1025 = $xlsub;
         $1026 = (($1025) + ($1024<<2)|0);
         $1027 = HEAP32[$1026>>2]|0;
         $1028 = ($1022|0)<($1027|0);
         if (!($1028)) {
          break L81;
         }
         $1029 = $i;
         $1030 = $lsub;
         $1031 = (($1030) + ($1029<<2)|0);
         $1032 = HEAP32[$1031>>2]|0;
         $irow = $1032;
         $1033 = +HEAPF64[$ukj>>3];
         $1034 = $luptr;
         $1035 = $lusup;
         $1036 = (($1035) + ($1034<<4)|0);
         $1037 = +HEAPF64[$1036>>3];
         $1038 = $1033 * $1037;
         $1039 = ((($ukj)) + 8|0);
         $1040 = +HEAPF64[$1039>>3];
         $1041 = $luptr;
         $1042 = $lusup;
         $1043 = (($1042) + ($1041<<4)|0);
         $1044 = ((($1043)) + 8|0);
         $1045 = +HEAPF64[$1044>>3];
         $1046 = $1040 * $1045;
         $1047 = $1038 - $1046;
         $cr19 = $1047;
         $1048 = ((($ukj)) + 8|0);
         $1049 = +HEAPF64[$1048>>3];
         $1050 = $luptr;
         $1051 = $lusup;
         $1052 = (($1051) + ($1050<<4)|0);
         $1053 = +HEAPF64[$1052>>3];
         $1054 = $1049 * $1053;
         $1055 = +HEAPF64[$ukj>>3];
         $1056 = $luptr;
         $1057 = $lusup;
         $1058 = (($1057) + ($1056<<4)|0);
         $1059 = ((($1058)) + 8|0);
         $1060 = +HEAPF64[$1059>>3];
         $1061 = $1055 * $1060;
         $1062 = $1054 + $1061;
         $ci20 = $1062;
         $1063 = $cr19;
         HEAPF64[$comp_temp>>3] = $1063;
         $1064 = $ci20;
         $1065 = ((($comp_temp)) + 8|0);
         HEAPF64[$1065>>3] = $1064;
         $1066 = $irow;
         $1067 = $dense_col;
         $1068 = (($1067) + ($1066<<4)|0);
         $1069 = +HEAPF64[$1068>>3];
         $1070 = +HEAPF64[$comp_temp>>3];
         $1071 = $1069 - $1070;
         $1072 = $irow;
         $1073 = $dense_col;
         $1074 = (($1073) + ($1072<<4)|0);
         HEAPF64[$1074>>3] = $1071;
         $1075 = $irow;
         $1076 = $dense_col;
         $1077 = (($1076) + ($1075<<4)|0);
         $1078 = ((($1077)) + 8|0);
         $1079 = +HEAPF64[$1078>>3];
         $1080 = ((($comp_temp)) + 8|0);
         $1081 = +HEAPF64[$1080>>3];
         $1082 = $1079 - $1081;
         $1083 = $irow;
         $1084 = $dense_col;
         $1085 = (($1084) + ($1083<<4)|0);
         $1086 = ((($1085)) + 8|0);
         HEAPF64[$1086>>3] = $1082;
         $1087 = $luptr;
         $1088 = (($1087) + 1)|0;
         $luptr = $1088;
         $1089 = $i;
         $1090 = (($1089) + 1)|0;
         $i = $1090;
        }
       }
       $1091 = $segsze;
       $1092 = ($1091|0)<=(3);
       if ($1092) {
        $1093 = $krep_ind;
        $1094 = $lsub;
        $1095 = (($1094) + ($1093<<2)|0);
        $1096 = HEAP32[$1095>>2]|0;
        $1097 = $dense_col;
        $1098 = (($1097) + ($1096<<4)|0);
        ;HEAP32[$ukj>>2]=HEAP32[$1098>>2]|0;HEAP32[$ukj+4>>2]=HEAP32[$1098+4>>2]|0;HEAP32[$ukj+8>>2]=HEAP32[$1098+8>>2]|0;HEAP32[$ukj+12>>2]=HEAP32[$1098+12>>2]|0;
        $1099 = $nsupr;
        $1100 = $nsupc;
        $1101 = (($1100) - 1)|0;
        $1102 = Math_imul($1099, $1101)|0;
        $1103 = $nsupc;
        $1104 = (($1102) + ($1103))|0;
        $1105 = (($1104) - 1)|0;
        $1106 = $luptr;
        $1107 = (($1106) + ($1105))|0;
        $luptr = $1107;
        $1108 = $krep_ind;
        $1109 = (($1108) - 1)|0;
        $1110 = $lsub;
        $1111 = (($1110) + ($1109<<2)|0);
        $1112 = HEAP32[$1111>>2]|0;
        $1113 = $dense_col;
        $1114 = (($1113) + ($1112<<4)|0);
        ;HEAP32[$ukj1>>2]=HEAP32[$1114>>2]|0;HEAP32[$ukj1+4>>2]=HEAP32[$1114+4>>2]|0;HEAP32[$ukj1+8>>2]=HEAP32[$1114+8>>2]|0;HEAP32[$ukj1+12>>2]=HEAP32[$1114+12>>2]|0;
        $1115 = $luptr;
        $1116 = $nsupr;
        $1117 = (($1115) - ($1116))|0;
        $luptr1 = $1117;
        $1118 = $segsze;
        $1119 = ($1118|0)==(2);
        if ($1119) {
         $1120 = +HEAPF64[$ukj1>>3];
         $1121 = $luptr1;
         $1122 = $lusup;
         $1123 = (($1122) + ($1121<<4)|0);
         $1124 = +HEAPF64[$1123>>3];
         $1125 = $1120 * $1124;
         $1126 = ((($ukj1)) + 8|0);
         $1127 = +HEAPF64[$1126>>3];
         $1128 = $luptr1;
         $1129 = $lusup;
         $1130 = (($1129) + ($1128<<4)|0);
         $1131 = ((($1130)) + 8|0);
         $1132 = +HEAPF64[$1131>>3];
         $1133 = $1127 * $1132;
         $1134 = $1125 - $1133;
         $cr21 = $1134;
         $1135 = ((($ukj1)) + 8|0);
         $1136 = +HEAPF64[$1135>>3];
         $1137 = $luptr1;
         $1138 = $lusup;
         $1139 = (($1138) + ($1137<<4)|0);
         $1140 = +HEAPF64[$1139>>3];
         $1141 = $1136 * $1140;
         $1142 = +HEAPF64[$ukj1>>3];
         $1143 = $luptr1;
         $1144 = $lusup;
         $1145 = (($1144) + ($1143<<4)|0);
         $1146 = ((($1145)) + 8|0);
         $1147 = +HEAPF64[$1146>>3];
         $1148 = $1142 * $1147;
         $1149 = $1141 + $1148;
         $ci22 = $1149;
         $1150 = $cr21;
         HEAPF64[$comp_temp>>3] = $1150;
         $1151 = $ci22;
         $1152 = ((($comp_temp)) + 8|0);
         HEAPF64[$1152>>3] = $1151;
         $1153 = +HEAPF64[$ukj>>3];
         $1154 = +HEAPF64[$comp_temp>>3];
         $1155 = $1153 - $1154;
         HEAPF64[$ukj>>3] = $1155;
         $1156 = ((($ukj)) + 8|0);
         $1157 = +HEAPF64[$1156>>3];
         $1158 = ((($comp_temp)) + 8|0);
         $1159 = +HEAPF64[$1158>>3];
         $1160 = $1157 - $1159;
         $1161 = ((($ukj)) + 8|0);
         HEAPF64[$1161>>3] = $1160;
         $1162 = $krep_ind;
         $1163 = $lsub;
         $1164 = (($1163) + ($1162<<2)|0);
         $1165 = HEAP32[$1164>>2]|0;
         $1166 = $dense_col;
         $1167 = (($1166) + ($1165<<4)|0);
         ;HEAP32[$1167>>2]=HEAP32[$ukj>>2]|0;HEAP32[$1167+4>>2]=HEAP32[$ukj+4>>2]|0;HEAP32[$1167+8>>2]=HEAP32[$ukj+8>>2]|0;HEAP32[$1167+12>>2]=HEAP32[$ukj+12>>2]|0;
         $1168 = $lptr;
         $1169 = $nsupc;
         $1170 = (($1168) + ($1169))|0;
         $i = $1170;
         while(1) {
          $1171 = $i;
          $1172 = $fsupc;
          $1173 = (($1172) + 1)|0;
          $1174 = $xlsub;
          $1175 = (($1174) + ($1173<<2)|0);
          $1176 = HEAP32[$1175>>2]|0;
          $1177 = ($1171|0)<($1176|0);
          if (!($1177)) {
           break L81;
          }
          $1178 = $i;
          $1179 = $lsub;
          $1180 = (($1179) + ($1178<<2)|0);
          $1181 = HEAP32[$1180>>2]|0;
          $irow = $1181;
          $1182 = $luptr;
          $1183 = (($1182) + 1)|0;
          $luptr = $1183;
          $1184 = $luptr1;
          $1185 = (($1184) + 1)|0;
          $luptr1 = $1185;
          $1186 = +HEAPF64[$ukj>>3];
          $1187 = $luptr;
          $1188 = $lusup;
          $1189 = (($1188) + ($1187<<4)|0);
          $1190 = +HEAPF64[$1189>>3];
          $1191 = $1186 * $1190;
          $1192 = ((($ukj)) + 8|0);
          $1193 = +HEAPF64[$1192>>3];
          $1194 = $luptr;
          $1195 = $lusup;
          $1196 = (($1195) + ($1194<<4)|0);
          $1197 = ((($1196)) + 8|0);
          $1198 = +HEAPF64[$1197>>3];
          $1199 = $1193 * $1198;
          $1200 = $1191 - $1199;
          $cr23 = $1200;
          $1201 = ((($ukj)) + 8|0);
          $1202 = +HEAPF64[$1201>>3];
          $1203 = $luptr;
          $1204 = $lusup;
          $1205 = (($1204) + ($1203<<4)|0);
          $1206 = +HEAPF64[$1205>>3];
          $1207 = $1202 * $1206;
          $1208 = +HEAPF64[$ukj>>3];
          $1209 = $luptr;
          $1210 = $lusup;
          $1211 = (($1210) + ($1209<<4)|0);
          $1212 = ((($1211)) + 8|0);
          $1213 = +HEAPF64[$1212>>3];
          $1214 = $1208 * $1213;
          $1215 = $1207 + $1214;
          $ci24 = $1215;
          $1216 = $cr23;
          HEAPF64[$comp_temp>>3] = $1216;
          $1217 = $ci24;
          $1218 = ((($comp_temp)) + 8|0);
          HEAPF64[$1218>>3] = $1217;
          $1219 = +HEAPF64[$ukj1>>3];
          $1220 = $luptr1;
          $1221 = $lusup;
          $1222 = (($1221) + ($1220<<4)|0);
          $1223 = +HEAPF64[$1222>>3];
          $1224 = $1219 * $1223;
          $1225 = ((($ukj1)) + 8|0);
          $1226 = +HEAPF64[$1225>>3];
          $1227 = $luptr1;
          $1228 = $lusup;
          $1229 = (($1228) + ($1227<<4)|0);
          $1230 = ((($1229)) + 8|0);
          $1231 = +HEAPF64[$1230>>3];
          $1232 = $1226 * $1231;
          $1233 = $1224 - $1232;
          $cr25 = $1233;
          $1234 = ((($ukj1)) + 8|0);
          $1235 = +HEAPF64[$1234>>3];
          $1236 = $luptr1;
          $1237 = $lusup;
          $1238 = (($1237) + ($1236<<4)|0);
          $1239 = +HEAPF64[$1238>>3];
          $1240 = $1235 * $1239;
          $1241 = +HEAPF64[$ukj1>>3];
          $1242 = $luptr1;
          $1243 = $lusup;
          $1244 = (($1243) + ($1242<<4)|0);
          $1245 = ((($1244)) + 8|0);
          $1246 = +HEAPF64[$1245>>3];
          $1247 = $1241 * $1246;
          $1248 = $1240 + $1247;
          $ci26 = $1248;
          $1249 = $cr25;
          HEAPF64[$comp_temp1>>3] = $1249;
          $1250 = $ci26;
          $1251 = ((($comp_temp1)) + 8|0);
          HEAPF64[$1251>>3] = $1250;
          $1252 = +HEAPF64[$comp_temp>>3];
          $1253 = +HEAPF64[$comp_temp1>>3];
          $1254 = $1252 + $1253;
          HEAPF64[$comp_temp>>3] = $1254;
          $1255 = ((($comp_temp)) + 8|0);
          $1256 = +HEAPF64[$1255>>3];
          $1257 = ((($comp_temp1)) + 8|0);
          $1258 = +HEAPF64[$1257>>3];
          $1259 = $1256 + $1258;
          $1260 = ((($comp_temp)) + 8|0);
          HEAPF64[$1260>>3] = $1259;
          $1261 = $irow;
          $1262 = $dense_col;
          $1263 = (($1262) + ($1261<<4)|0);
          $1264 = +HEAPF64[$1263>>3];
          $1265 = +HEAPF64[$comp_temp>>3];
          $1266 = $1264 - $1265;
          $1267 = $irow;
          $1268 = $dense_col;
          $1269 = (($1268) + ($1267<<4)|0);
          HEAPF64[$1269>>3] = $1266;
          $1270 = $irow;
          $1271 = $dense_col;
          $1272 = (($1271) + ($1270<<4)|0);
          $1273 = ((($1272)) + 8|0);
          $1274 = +HEAPF64[$1273>>3];
          $1275 = ((($comp_temp)) + 8|0);
          $1276 = +HEAPF64[$1275>>3];
          $1277 = $1274 - $1276;
          $1278 = $irow;
          $1279 = $dense_col;
          $1280 = (($1279) + ($1278<<4)|0);
          $1281 = ((($1280)) + 8|0);
          HEAPF64[$1281>>3] = $1277;
          $1282 = $i;
          $1283 = (($1282) + 1)|0;
          $i = $1283;
         }
        } else {
         $1284 = $krep_ind;
         $1285 = (($1284) - 2)|0;
         $1286 = $lsub;
         $1287 = (($1286) + ($1285<<2)|0);
         $1288 = HEAP32[$1287>>2]|0;
         $1289 = $dense_col;
         $1290 = (($1289) + ($1288<<4)|0);
         ;HEAP32[$ukj2>>2]=HEAP32[$1290>>2]|0;HEAP32[$ukj2+4>>2]=HEAP32[$1290+4>>2]|0;HEAP32[$ukj2+8>>2]=HEAP32[$1290+8>>2]|0;HEAP32[$ukj2+12>>2]=HEAP32[$1290+12>>2]|0;
         $1291 = $luptr1;
         $1292 = $nsupr;
         $1293 = (($1291) - ($1292))|0;
         $luptr2 = $1293;
         $1294 = +HEAPF64[$ukj2>>3];
         $1295 = $luptr2;
         $1296 = (($1295) - 1)|0;
         $1297 = $lusup;
         $1298 = (($1297) + ($1296<<4)|0);
         $1299 = +HEAPF64[$1298>>3];
         $1300 = $1294 * $1299;
         $1301 = ((($ukj2)) + 8|0);
         $1302 = +HEAPF64[$1301>>3];
         $1303 = $luptr2;
         $1304 = (($1303) - 1)|0;
         $1305 = $lusup;
         $1306 = (($1305) + ($1304<<4)|0);
         $1307 = ((($1306)) + 8|0);
         $1308 = +HEAPF64[$1307>>3];
         $1309 = $1302 * $1308;
         $1310 = $1300 - $1309;
         $cr27 = $1310;
         $1311 = ((($ukj2)) + 8|0);
         $1312 = +HEAPF64[$1311>>3];
         $1313 = $luptr2;
         $1314 = (($1313) - 1)|0;
         $1315 = $lusup;
         $1316 = (($1315) + ($1314<<4)|0);
         $1317 = +HEAPF64[$1316>>3];
         $1318 = $1312 * $1317;
         $1319 = +HEAPF64[$ukj2>>3];
         $1320 = $luptr2;
         $1321 = (($1320) - 1)|0;
         $1322 = $lusup;
         $1323 = (($1322) + ($1321<<4)|0);
         $1324 = ((($1323)) + 8|0);
         $1325 = +HEAPF64[$1324>>3];
         $1326 = $1319 * $1325;
         $1327 = $1318 + $1326;
         $ci28 = $1327;
         $1328 = $cr27;
         HEAPF64[$comp_temp>>3] = $1328;
         $1329 = $ci28;
         $1330 = ((($comp_temp)) + 8|0);
         HEAPF64[$1330>>3] = $1329;
         $1331 = +HEAPF64[$ukj1>>3];
         $1332 = +HEAPF64[$comp_temp>>3];
         $1333 = $1331 - $1332;
         HEAPF64[$ukj1>>3] = $1333;
         $1334 = ((($ukj1)) + 8|0);
         $1335 = +HEAPF64[$1334>>3];
         $1336 = ((($comp_temp)) + 8|0);
         $1337 = +HEAPF64[$1336>>3];
         $1338 = $1335 - $1337;
         $1339 = ((($ukj1)) + 8|0);
         HEAPF64[$1339>>3] = $1338;
         $1340 = +HEAPF64[$ukj1>>3];
         $1341 = $luptr1;
         $1342 = $lusup;
         $1343 = (($1342) + ($1341<<4)|0);
         $1344 = +HEAPF64[$1343>>3];
         $1345 = $1340 * $1344;
         $1346 = ((($ukj1)) + 8|0);
         $1347 = +HEAPF64[$1346>>3];
         $1348 = $luptr1;
         $1349 = $lusup;
         $1350 = (($1349) + ($1348<<4)|0);
         $1351 = ((($1350)) + 8|0);
         $1352 = +HEAPF64[$1351>>3];
         $1353 = $1347 * $1352;
         $1354 = $1345 - $1353;
         $cr29 = $1354;
         $1355 = ((($ukj1)) + 8|0);
         $1356 = +HEAPF64[$1355>>3];
         $1357 = $luptr1;
         $1358 = $lusup;
         $1359 = (($1358) + ($1357<<4)|0);
         $1360 = +HEAPF64[$1359>>3];
         $1361 = $1356 * $1360;
         $1362 = +HEAPF64[$ukj1>>3];
         $1363 = $luptr1;
         $1364 = $lusup;
         $1365 = (($1364) + ($1363<<4)|0);
         $1366 = ((($1365)) + 8|0);
         $1367 = +HEAPF64[$1366>>3];
         $1368 = $1362 * $1367;
         $1369 = $1361 + $1368;
         $ci30 = $1369;
         $1370 = $cr29;
         HEAPF64[$comp_temp>>3] = $1370;
         $1371 = $ci30;
         $1372 = ((($comp_temp)) + 8|0);
         HEAPF64[$1372>>3] = $1371;
         $1373 = +HEAPF64[$ukj2>>3];
         $1374 = $luptr2;
         $1375 = $lusup;
         $1376 = (($1375) + ($1374<<4)|0);
         $1377 = +HEAPF64[$1376>>3];
         $1378 = $1373 * $1377;
         $1379 = ((($ukj2)) + 8|0);
         $1380 = +HEAPF64[$1379>>3];
         $1381 = $luptr2;
         $1382 = $lusup;
         $1383 = (($1382) + ($1381<<4)|0);
         $1384 = ((($1383)) + 8|0);
         $1385 = +HEAPF64[$1384>>3];
         $1386 = $1380 * $1385;
         $1387 = $1378 - $1386;
         $cr31 = $1387;
         $1388 = ((($ukj2)) + 8|0);
         $1389 = +HEAPF64[$1388>>3];
         $1390 = $luptr2;
         $1391 = $lusup;
         $1392 = (($1391) + ($1390<<4)|0);
         $1393 = +HEAPF64[$1392>>3];
         $1394 = $1389 * $1393;
         $1395 = +HEAPF64[$ukj2>>3];
         $1396 = $luptr2;
         $1397 = $lusup;
         $1398 = (($1397) + ($1396<<4)|0);
         $1399 = ((($1398)) + 8|0);
         $1400 = +HEAPF64[$1399>>3];
         $1401 = $1395 * $1400;
         $1402 = $1394 + $1401;
         $ci32 = $1402;
         $1403 = $cr31;
         HEAPF64[$comp_temp1>>3] = $1403;
         $1404 = $ci32;
         $1405 = ((($comp_temp1)) + 8|0);
         HEAPF64[$1405>>3] = $1404;
         $1406 = +HEAPF64[$comp_temp>>3];
         $1407 = +HEAPF64[$comp_temp1>>3];
         $1408 = $1406 + $1407;
         HEAPF64[$comp_temp>>3] = $1408;
         $1409 = ((($comp_temp)) + 8|0);
         $1410 = +HEAPF64[$1409>>3];
         $1411 = ((($comp_temp1)) + 8|0);
         $1412 = +HEAPF64[$1411>>3];
         $1413 = $1410 + $1412;
         $1414 = ((($comp_temp)) + 8|0);
         HEAPF64[$1414>>3] = $1413;
         $1415 = +HEAPF64[$ukj>>3];
         $1416 = +HEAPF64[$comp_temp>>3];
         $1417 = $1415 - $1416;
         HEAPF64[$ukj>>3] = $1417;
         $1418 = ((($ukj)) + 8|0);
         $1419 = +HEAPF64[$1418>>3];
         $1420 = ((($comp_temp)) + 8|0);
         $1421 = +HEAPF64[$1420>>3];
         $1422 = $1419 - $1421;
         $1423 = ((($ukj)) + 8|0);
         HEAPF64[$1423>>3] = $1422;
         $1424 = $krep_ind;
         $1425 = $lsub;
         $1426 = (($1425) + ($1424<<2)|0);
         $1427 = HEAP32[$1426>>2]|0;
         $1428 = $dense_col;
         $1429 = (($1428) + ($1427<<4)|0);
         ;HEAP32[$1429>>2]=HEAP32[$ukj>>2]|0;HEAP32[$1429+4>>2]=HEAP32[$ukj+4>>2]|0;HEAP32[$1429+8>>2]=HEAP32[$ukj+8>>2]|0;HEAP32[$1429+12>>2]=HEAP32[$ukj+12>>2]|0;
         $1430 = $krep_ind;
         $1431 = (($1430) - 1)|0;
         $1432 = $lsub;
         $1433 = (($1432) + ($1431<<2)|0);
         $1434 = HEAP32[$1433>>2]|0;
         $1435 = $dense_col;
         $1436 = (($1435) + ($1434<<4)|0);
         ;HEAP32[$1436>>2]=HEAP32[$ukj1>>2]|0;HEAP32[$1436+4>>2]=HEAP32[$ukj1+4>>2]|0;HEAP32[$1436+8>>2]=HEAP32[$ukj1+8>>2]|0;HEAP32[$1436+12>>2]=HEAP32[$ukj1+12>>2]|0;
         $1437 = $lptr;
         $1438 = $nsupc;
         $1439 = (($1437) + ($1438))|0;
         $i = $1439;
         while(1) {
          $1440 = $i;
          $1441 = $fsupc;
          $1442 = (($1441) + 1)|0;
          $1443 = $xlsub;
          $1444 = (($1443) + ($1442<<2)|0);
          $1445 = HEAP32[$1444>>2]|0;
          $1446 = ($1440|0)<($1445|0);
          if (!($1446)) {
           break L81;
          }
          $1447 = $i;
          $1448 = $lsub;
          $1449 = (($1448) + ($1447<<2)|0);
          $1450 = HEAP32[$1449>>2]|0;
          $irow = $1450;
          $1451 = $luptr;
          $1452 = (($1451) + 1)|0;
          $luptr = $1452;
          $1453 = $luptr1;
          $1454 = (($1453) + 1)|0;
          $luptr1 = $1454;
          $1455 = $luptr2;
          $1456 = (($1455) + 1)|0;
          $luptr2 = $1456;
          $1457 = +HEAPF64[$ukj>>3];
          $1458 = $luptr;
          $1459 = $lusup;
          $1460 = (($1459) + ($1458<<4)|0);
          $1461 = +HEAPF64[$1460>>3];
          $1462 = $1457 * $1461;
          $1463 = ((($ukj)) + 8|0);
          $1464 = +HEAPF64[$1463>>3];
          $1465 = $luptr;
          $1466 = $lusup;
          $1467 = (($1466) + ($1465<<4)|0);
          $1468 = ((($1467)) + 8|0);
          $1469 = +HEAPF64[$1468>>3];
          $1470 = $1464 * $1469;
          $1471 = $1462 - $1470;
          $cr33 = $1471;
          $1472 = ((($ukj)) + 8|0);
          $1473 = +HEAPF64[$1472>>3];
          $1474 = $luptr;
          $1475 = $lusup;
          $1476 = (($1475) + ($1474<<4)|0);
          $1477 = +HEAPF64[$1476>>3];
          $1478 = $1473 * $1477;
          $1479 = +HEAPF64[$ukj>>3];
          $1480 = $luptr;
          $1481 = $lusup;
          $1482 = (($1481) + ($1480<<4)|0);
          $1483 = ((($1482)) + 8|0);
          $1484 = +HEAPF64[$1483>>3];
          $1485 = $1479 * $1484;
          $1486 = $1478 + $1485;
          $ci34 = $1486;
          $1487 = $cr33;
          HEAPF64[$comp_temp>>3] = $1487;
          $1488 = $ci34;
          $1489 = ((($comp_temp)) + 8|0);
          HEAPF64[$1489>>3] = $1488;
          $1490 = +HEAPF64[$ukj1>>3];
          $1491 = $luptr1;
          $1492 = $lusup;
          $1493 = (($1492) + ($1491<<4)|0);
          $1494 = +HEAPF64[$1493>>3];
          $1495 = $1490 * $1494;
          $1496 = ((($ukj1)) + 8|0);
          $1497 = +HEAPF64[$1496>>3];
          $1498 = $luptr1;
          $1499 = $lusup;
          $1500 = (($1499) + ($1498<<4)|0);
          $1501 = ((($1500)) + 8|0);
          $1502 = +HEAPF64[$1501>>3];
          $1503 = $1497 * $1502;
          $1504 = $1495 - $1503;
          $cr35 = $1504;
          $1505 = ((($ukj1)) + 8|0);
          $1506 = +HEAPF64[$1505>>3];
          $1507 = $luptr1;
          $1508 = $lusup;
          $1509 = (($1508) + ($1507<<4)|0);
          $1510 = +HEAPF64[$1509>>3];
          $1511 = $1506 * $1510;
          $1512 = +HEAPF64[$ukj1>>3];
          $1513 = $luptr1;
          $1514 = $lusup;
          $1515 = (($1514) + ($1513<<4)|0);
          $1516 = ((($1515)) + 8|0);
          $1517 = +HEAPF64[$1516>>3];
          $1518 = $1512 * $1517;
          $1519 = $1511 + $1518;
          $ci36 = $1519;
          $1520 = $cr35;
          HEAPF64[$comp_temp1>>3] = $1520;
          $1521 = $ci36;
          $1522 = ((($comp_temp1)) + 8|0);
          HEAPF64[$1522>>3] = $1521;
          $1523 = +HEAPF64[$comp_temp>>3];
          $1524 = +HEAPF64[$comp_temp1>>3];
          $1525 = $1523 + $1524;
          HEAPF64[$comp_temp>>3] = $1525;
          $1526 = ((($comp_temp)) + 8|0);
          $1527 = +HEAPF64[$1526>>3];
          $1528 = ((($comp_temp1)) + 8|0);
          $1529 = +HEAPF64[$1528>>3];
          $1530 = $1527 + $1529;
          $1531 = ((($comp_temp)) + 8|0);
          HEAPF64[$1531>>3] = $1530;
          $1532 = +HEAPF64[$ukj2>>3];
          $1533 = $luptr2;
          $1534 = $lusup;
          $1535 = (($1534) + ($1533<<4)|0);
          $1536 = +HEAPF64[$1535>>3];
          $1537 = $1532 * $1536;
          $1538 = ((($ukj2)) + 8|0);
          $1539 = +HEAPF64[$1538>>3];
          $1540 = $luptr2;
          $1541 = $lusup;
          $1542 = (($1541) + ($1540<<4)|0);
          $1543 = ((($1542)) + 8|0);
          $1544 = +HEAPF64[$1543>>3];
          $1545 = $1539 * $1544;
          $1546 = $1537 - $1545;
          $cr37 = $1546;
          $1547 = ((($ukj2)) + 8|0);
          $1548 = +HEAPF64[$1547>>3];
          $1549 = $luptr2;
          $1550 = $lusup;
          $1551 = (($1550) + ($1549<<4)|0);
          $1552 = +HEAPF64[$1551>>3];
          $1553 = $1548 * $1552;
          $1554 = +HEAPF64[$ukj2>>3];
          $1555 = $luptr2;
          $1556 = $lusup;
          $1557 = (($1556) + ($1555<<4)|0);
          $1558 = ((($1557)) + 8|0);
          $1559 = +HEAPF64[$1558>>3];
          $1560 = $1554 * $1559;
          $1561 = $1553 + $1560;
          $ci38 = $1561;
          $1562 = $cr37;
          HEAPF64[$comp_temp1>>3] = $1562;
          $1563 = $ci38;
          $1564 = ((($comp_temp1)) + 8|0);
          HEAPF64[$1564>>3] = $1563;
          $1565 = +HEAPF64[$comp_temp>>3];
          $1566 = +HEAPF64[$comp_temp1>>3];
          $1567 = $1565 + $1566;
          HEAPF64[$comp_temp>>3] = $1567;
          $1568 = ((($comp_temp)) + 8|0);
          $1569 = +HEAPF64[$1568>>3];
          $1570 = ((($comp_temp1)) + 8|0);
          $1571 = +HEAPF64[$1570>>3];
          $1572 = $1569 + $1571;
          $1573 = ((($comp_temp)) + 8|0);
          HEAPF64[$1573>>3] = $1572;
          $1574 = $irow;
          $1575 = $dense_col;
          $1576 = (($1575) + ($1574<<4)|0);
          $1577 = +HEAPF64[$1576>>3];
          $1578 = +HEAPF64[$comp_temp>>3];
          $1579 = $1577 - $1578;
          $1580 = $irow;
          $1581 = $dense_col;
          $1582 = (($1581) + ($1580<<4)|0);
          HEAPF64[$1582>>3] = $1579;
          $1583 = $irow;
          $1584 = $dense_col;
          $1585 = (($1584) + ($1583<<4)|0);
          $1586 = ((($1585)) + 8|0);
          $1587 = +HEAPF64[$1586>>3];
          $1588 = ((($comp_temp)) + 8|0);
          $1589 = +HEAPF64[$1588>>3];
          $1590 = $1587 - $1589;
          $1591 = $irow;
          $1592 = $dense_col;
          $1593 = (($1592) + ($1591<<4)|0);
          $1594 = ((($1593)) + 8|0);
          HEAPF64[$1594>>3] = $1590;
          $1595 = $i;
          $1596 = (($1595) + 1)|0;
          $i = $1596;
         }
        }
       }
       $1597 = $kfnz;
       $1598 = $fsupc;
       $1599 = (($1597) - ($1598))|0;
       $no_zeros = $1599;
       $1600 = $lptr;
       $1601 = $no_zeros;
       $1602 = (($1600) + ($1601))|0;
       $isub = $1602;
       $i = 0;
       while(1) {
        $1603 = $i;
        $1604 = $segsze;
        $1605 = ($1603|0)<($1604|0);
        if (!($1605)) {
         break;
        }
        $1606 = $isub;
        $1607 = $lsub;
        $1608 = (($1607) + ($1606<<2)|0);
        $1609 = HEAP32[$1608>>2]|0;
        $irow = $1609;
        $1610 = $i;
        $1611 = $5;
        $1612 = (($1611) + ($1610<<4)|0);
        $1613 = $irow;
        $1614 = $dense_col;
        $1615 = (($1614) + ($1613<<4)|0);
        ;HEAP32[$1612>>2]=HEAP32[$1615>>2]|0;HEAP32[$1612+4>>2]=HEAP32[$1615+4>>2]|0;HEAP32[$1612+8>>2]=HEAP32[$1615+8>>2]|0;HEAP32[$1612+12>>2]=HEAP32[$1615+12>>2]|0;
        $1616 = $isub;
        $1617 = (($1616) + 1)|0;
        $isub = $1617;
        $1618 = $i;
        $1619 = (($1618) + 1)|0;
        $i = $1619;
       }
       $1620 = $nsupr;
       $1621 = $no_zeros;
       $1622 = Math_imul($1620, $1621)|0;
       $1623 = $no_zeros;
       $1624 = (($1622) + ($1623))|0;
       $1625 = $luptr;
       $1626 = (($1625) + ($1624))|0;
       $luptr = $1626;
       $1627 = $nsupr;
       $1628 = $segsze;
       $1629 = $luptr;
       $1630 = $lusup;
       $1631 = (($1630) + ($1629<<4)|0);
       $1632 = $5;
       _zlsolve($1627,$1628,$1631,$1632);
       $1633 = $segsze;
       $1634 = $luptr;
       $1635 = (($1634) + ($1633))|0;
       $luptr = $1635;
       $1636 = $segsze;
       $1637 = $5;
       $1638 = (($1637) + ($1636<<4)|0);
       $tempv1 = $1638;
       $1639 = $nsupr;
       $1640 = $nrow;
       $1641 = $segsze;
       $1642 = $luptr;
       $1643 = $lusup;
       $1644 = (($1643) + ($1642<<4)|0);
       $1645 = $5;
       $1646 = $tempv1;
       _zmatvec($1639,$1640,$1641,$1644,$1645,$1646);
       $1647 = $lptr;
       $1648 = $no_zeros;
       $1649 = (($1647) + ($1648))|0;
       $isub = $1649;
       $i = 0;
       while(1) {
        $1650 = $i;
        $1651 = $segsze;
        $1652 = ($1650|0)<($1651|0);
        if (!($1652)) {
         break;
        }
        $1653 = $isub;
        $1654 = $lsub;
        $1655 = (($1654) + ($1653<<2)|0);
        $1656 = HEAP32[$1655>>2]|0;
        $irow = $1656;
        $1657 = $irow;
        $1658 = $dense_col;
        $1659 = (($1658) + ($1657<<4)|0);
        $1660 = $i;
        $1661 = $5;
        $1662 = (($1661) + ($1660<<4)|0);
        ;HEAP32[$1659>>2]=HEAP32[$1662>>2]|0;HEAP32[$1659+4>>2]=HEAP32[$1662+4>>2]|0;HEAP32[$1659+8>>2]=HEAP32[$1662+8>>2]|0;HEAP32[$1659+12>>2]=HEAP32[$1662+12>>2]|0;
        $1663 = $i;
        $1664 = $5;
        $1665 = (($1664) + ($1663<<4)|0);
        ;HEAP32[$1665>>2]=HEAP32[$zero>>2]|0;HEAP32[$1665+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$1665+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$1665+12>>2]=HEAP32[$zero+12>>2]|0;
        $1666 = $isub;
        $1667 = (($1666) + 1)|0;
        $isub = $1667;
        $1668 = $i;
        $1669 = (($1668) + 1)|0;
        $i = $1669;
       }
       $i = 0;
       while(1) {
        $1670 = $i;
        $1671 = $nrow;
        $1672 = ($1670|0)<($1671|0);
        if (!($1672)) {
         break L81;
        }
        $1673 = $isub;
        $1674 = $lsub;
        $1675 = (($1674) + ($1673<<2)|0);
        $1676 = HEAP32[$1675>>2]|0;
        $irow = $1676;
        $1677 = $irow;
        $1678 = $dense_col;
        $1679 = (($1678) + ($1677<<4)|0);
        $1680 = +HEAPF64[$1679>>3];
        $1681 = $i;
        $1682 = $tempv1;
        $1683 = (($1682) + ($1681<<4)|0);
        $1684 = +HEAPF64[$1683>>3];
        $1685 = $1680 - $1684;
        $1686 = $irow;
        $1687 = $dense_col;
        $1688 = (($1687) + ($1686<<4)|0);
        HEAPF64[$1688>>3] = $1685;
        $1689 = $irow;
        $1690 = $dense_col;
        $1691 = (($1690) + ($1689<<4)|0);
        $1692 = ((($1691)) + 8|0);
        $1693 = +HEAPF64[$1692>>3];
        $1694 = $i;
        $1695 = $tempv1;
        $1696 = (($1695) + ($1694<<4)|0);
        $1697 = ((($1696)) + 8|0);
        $1698 = +HEAPF64[$1697>>3];
        $1699 = $1693 - $1698;
        $1700 = $irow;
        $1701 = $dense_col;
        $1702 = (($1701) + ($1700<<4)|0);
        $1703 = ((($1702)) + 8|0);
        HEAPF64[$1703>>3] = $1699;
        $1704 = $i;
        $1705 = $tempv1;
        $1706 = (($1705) + ($1704<<4)|0);
        ;HEAP32[$1706>>2]=HEAP32[$zero>>2]|0;HEAP32[$1706+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$1706+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$1706+12>>2]=HEAP32[$zero+12>>2]|0;
        $1707 = $isub;
        $1708 = (($1707) + 1)|0;
        $isub = $1708;
        $1709 = $i;
        $1710 = (($1709) + 1)|0;
        $i = $1710;
       }
      }
     } while(0);
     $1711 = $jj;
     $1712 = (($1711) + 1)|0;
     $jj = $1712;
     $1713 = $0;
     $1714 = $repfnz_col;
     $1715 = (($1714) + ($1713<<2)|0);
     $repfnz_col = $1715;
     $1716 = $0;
     $1717 = $dense_col;
     $1718 = (($1717) + ($1716<<4)|0);
     $dense_col = $1718;
    }
   }
  } while(0);
  $1719 = $ksub;
  $1720 = (($1719) + 1)|0;
  $ksub = $1720;
 }
 STACKTOP = sp;return;
}
function _zcolumn_dfs($m,$jcol,$perm_r,$nseg,$lsub_col,$segrep,$repfnz,$xprune,$marker,$parent,$xplore,$Glu) {
 $m = $m|0;
 $jcol = $jcol|0;
 $perm_r = $perm_r|0;
 $nseg = $nseg|0;
 $lsub_col = $lsub_col|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $xprune = $xprune|0;
 $marker = $marker|0;
 $parent = $parent|0;
 $xplore = $xplore|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $chmark = 0, $chperm = 0, $chrep = 0, $fsupc = 0, $ifrom = 0, $istop = 0, $ito = 0, $jcolm1 = 0, $jcolp1 = 0, $jm1ptr = 0, $jptr = 0, $jsuper = 0, $k = 0;
 var $kchild = 0, $kmark = 0, $kpar = 0, $kperm = 0, $krep = 0, $krow = 0, $lsub = 0, $marker2 = 0, $maxdfs = 0, $mem_error = 0, $myfnz = 0, $nextl = 0, $nsuper = 0, $nzlmax = 0, $oldrep = 0, $supno = 0, $xdfs = 0, $xlsub = 0, $xsup = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $nzlmax = sp;
 $1 = $m;
 $2 = $jcol;
 $3 = $perm_r;
 $4 = $nseg;
 $5 = $lsub_col;
 $6 = $segrep;
 $7 = $repfnz;
 $8 = $xprune;
 $9 = $marker;
 $10 = $parent;
 $11 = $xplore;
 $12 = $Glu;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $xsup = $14;
 $15 = $12;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $supno = $17;
 $18 = $12;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $lsub = $20;
 $21 = $12;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $xlsub = $23;
 $24 = $12;
 $25 = ((($24)) + 36|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$nzlmax>>2] = $26;
 $27 = HEAP32[2472>>2]|0;
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = (_sp_ienv(3)|0);
  HEAP32[2480>>2] = $29;
  HEAP32[2472>>2] = 0;
 }
 $30 = $2;
 $31 = (($30) + 1)|0;
 $jcolp1 = $31;
 $32 = $2;
 $33 = (($32) - 1)|0;
 $jcolm1 = $33;
 $34 = $2;
 $35 = $supno;
 $36 = (($35) + ($34<<2)|0);
 $37 = HEAP32[$36>>2]|0;
 $nsuper = $37;
 $38 = $nsuper;
 $jsuper = $38;
 $39 = $2;
 $40 = $xlsub;
 $41 = (($40) + ($39<<2)|0);
 $42 = HEAP32[$41>>2]|0;
 $nextl = $42;
 $43 = $1;
 $44 = $43<<1;
 $45 = $9;
 $46 = (($45) + ($44<<2)|0);
 $marker2 = $46;
 $k = 0;
 L4: while(1) {
  $47 = $k;
  $48 = $5;
  $49 = (($48) + ($47<<2)|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = ($50|0)!=(-1);
  if (!($51)) {
   label = 33;
   break;
  }
  $52 = $k;
  $53 = $5;
  $54 = (($53) + ($52<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $krow = $55;
  $56 = $k;
  $57 = $5;
  $58 = (($57) + ($56<<2)|0);
  HEAP32[$58>>2] = -1;
  $59 = $krow;
  $60 = $marker2;
  $61 = (($60) + ($59<<2)|0);
  $62 = HEAP32[$61>>2]|0;
  $kmark = $62;
  $63 = $kmark;
  $64 = $2;
  $65 = ($63|0)==($64|0);
  L7: do {
   if (!($65)) {
    $66 = $2;
    $67 = $krow;
    $68 = $marker2;
    $69 = (($68) + ($67<<2)|0);
    HEAP32[$69>>2] = $66;
    $70 = $krow;
    $71 = $3;
    $72 = (($71) + ($70<<2)|0);
    $73 = HEAP32[$72>>2]|0;
    $kperm = $73;
    $74 = $kperm;
    $75 = ($74|0)==(-1);
    if ($75) {
     $76 = $krow;
     $77 = $nextl;
     $78 = (($77) + 1)|0;
     $nextl = $78;
     $79 = $lsub;
     $80 = (($79) + ($77<<2)|0);
     HEAP32[$80>>2] = $76;
     $81 = $nextl;
     $82 = HEAP32[$nzlmax>>2]|0;
     $83 = ($81|0)>=($82|0);
     if ($83) {
      $84 = $2;
      $85 = $nextl;
      $86 = $12;
      $87 = (_zLUMemXpand($84,$85,2,$nzlmax,$86)|0);
      $mem_error = $87;
      $88 = ($87|0)!=(0);
      if ($88) {
       label = 9;
       break L4;
      }
      $90 = $12;
      $91 = ((($90)) + 8|0);
      $92 = HEAP32[$91>>2]|0;
      $lsub = $92;
     }
     $93 = $kmark;
     $94 = $jcolm1;
     $95 = ($93|0)!=($94|0);
     if (!($95)) {
      break;
     }
     $jsuper = -1;
     break;
    }
    $96 = $kperm;
    $97 = $supno;
    $98 = (($97) + ($96<<2)|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = (($99) + 1)|0;
    $101 = $xsup;
    $102 = (($101) + ($100<<2)|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (($103) - 1)|0;
    $krep = $104;
    $105 = $krep;
    $106 = $7;
    $107 = (($106) + ($105<<2)|0);
    $108 = HEAP32[$107>>2]|0;
    $myfnz = $108;
    $109 = $myfnz;
    $110 = ($109|0)!=(-1);
    if ($110) {
     $111 = $myfnz;
     $112 = $kperm;
     $113 = ($111|0)>($112|0);
     if (!($113)) {
      break;
     }
     $114 = $kperm;
     $115 = $krep;
     $116 = $7;
     $117 = (($116) + ($115<<2)|0);
     HEAP32[$117>>2] = $114;
     break;
    }
    $oldrep = -1;
    $118 = $oldrep;
    $119 = $krep;
    $120 = $10;
    $121 = (($120) + ($119<<2)|0);
    HEAP32[$121>>2] = $118;
    $122 = $kperm;
    $123 = $krep;
    $124 = $7;
    $125 = (($124) + ($123<<2)|0);
    HEAP32[$125>>2] = $122;
    $126 = $krep;
    $127 = $xlsub;
    $128 = (($127) + ($126<<2)|0);
    $129 = HEAP32[$128>>2]|0;
    $xdfs = $129;
    $130 = $krep;
    $131 = $8;
    $132 = (($131) + ($130<<2)|0);
    $133 = HEAP32[$132>>2]|0;
    $maxdfs = $133;
    while(1) {
     $134 = $xdfs;
     $135 = $maxdfs;
     $136 = ($134|0)<($135|0);
     if (!($136)) {
      $224 = $krep;
      $225 = $4;
      $226 = HEAP32[$225>>2]|0;
      $227 = $6;
      $228 = (($227) + ($226<<2)|0);
      HEAP32[$228>>2] = $224;
      $229 = $4;
      $230 = HEAP32[$229>>2]|0;
      $231 = (($230) + 1)|0;
      HEAP32[$229>>2] = $231;
      $232 = $krep;
      $233 = $10;
      $234 = (($233) + ($232<<2)|0);
      $235 = HEAP32[$234>>2]|0;
      $kpar = $235;
      $236 = $kpar;
      $237 = ($236|0)==(-1);
      if ($237) {
       break L7;
      }
      $238 = $kpar;
      $krep = $238;
      $239 = $krep;
      $240 = $11;
      $241 = (($240) + ($239<<2)|0);
      $242 = HEAP32[$241>>2]|0;
      $xdfs = $242;
      $243 = $krep;
      $244 = $8;
      $245 = (($244) + ($243<<2)|0);
      $246 = HEAP32[$245>>2]|0;
      $maxdfs = $246;
      $247 = $kpar;
      $248 = ($247|0)!=(-1);
      if ($248) {
       continue;
      } else {
       break;
      }
     }
     $137 = $xdfs;
     $138 = $lsub;
     $139 = (($138) + ($137<<2)|0);
     $140 = HEAP32[$139>>2]|0;
     $kchild = $140;
     $141 = $xdfs;
     $142 = (($141) + 1)|0;
     $xdfs = $142;
     $143 = $kchild;
     $144 = $marker2;
     $145 = (($144) + ($143<<2)|0);
     $146 = HEAP32[$145>>2]|0;
     $chmark = $146;
     $147 = $chmark;
     $148 = $2;
     $149 = ($147|0)!=($148|0);
     if (!($149)) {
      continue;
     }
     $150 = $2;
     $151 = $kchild;
     $152 = $marker2;
     $153 = (($152) + ($151<<2)|0);
     HEAP32[$153>>2] = $150;
     $154 = $kchild;
     $155 = $3;
     $156 = (($155) + ($154<<2)|0);
     $157 = HEAP32[$156>>2]|0;
     $chperm = $157;
     $158 = $chperm;
     $159 = ($158|0)==(-1);
     if (!($159)) {
      $180 = $chperm;
      $181 = $supno;
      $182 = (($181) + ($180<<2)|0);
      $183 = HEAP32[$182>>2]|0;
      $184 = (($183) + 1)|0;
      $185 = $xsup;
      $186 = (($185) + ($184<<2)|0);
      $187 = HEAP32[$186>>2]|0;
      $188 = (($187) - 1)|0;
      $chrep = $188;
      $189 = $chrep;
      $190 = $7;
      $191 = (($190) + ($189<<2)|0);
      $192 = HEAP32[$191>>2]|0;
      $myfnz = $192;
      $193 = $myfnz;
      $194 = ($193|0)!=(-1);
      if (!($194)) {
       $202 = $xdfs;
       $203 = $krep;
       $204 = $11;
       $205 = (($204) + ($203<<2)|0);
       HEAP32[$205>>2] = $202;
       $206 = $krep;
       $oldrep = $206;
       $207 = $chrep;
       $krep = $207;
       $208 = $oldrep;
       $209 = $krep;
       $210 = $10;
       $211 = (($210) + ($209<<2)|0);
       HEAP32[$211>>2] = $208;
       $212 = $chperm;
       $213 = $krep;
       $214 = $7;
       $215 = (($214) + ($213<<2)|0);
       HEAP32[$215>>2] = $212;
       $216 = $krep;
       $217 = $xlsub;
       $218 = (($217) + ($216<<2)|0);
       $219 = HEAP32[$218>>2]|0;
       $xdfs = $219;
       $220 = $krep;
       $221 = $8;
       $222 = (($221) + ($220<<2)|0);
       $223 = HEAP32[$222>>2]|0;
       $maxdfs = $223;
       continue;
      }
      $195 = $myfnz;
      $196 = $chperm;
      $197 = ($195|0)>($196|0);
      if (!($197)) {
       continue;
      }
      $198 = $chperm;
      $199 = $chrep;
      $200 = $7;
      $201 = (($200) + ($199<<2)|0);
      HEAP32[$201>>2] = $198;
      continue;
     }
     $160 = $kchild;
     $161 = $nextl;
     $162 = (($161) + 1)|0;
     $nextl = $162;
     $163 = $lsub;
     $164 = (($163) + ($161<<2)|0);
     HEAP32[$164>>2] = $160;
     $165 = $nextl;
     $166 = HEAP32[$nzlmax>>2]|0;
     $167 = ($165|0)>=($166|0);
     if ($167) {
      $168 = $2;
      $169 = $nextl;
      $170 = $12;
      $171 = (_zLUMemXpand($168,$169,2,$nzlmax,$170)|0);
      $mem_error = $171;
      $172 = ($171|0)!=(0);
      if ($172) {
       label = 22;
       break L4;
      }
      $174 = $12;
      $175 = ((($174)) + 8|0);
      $176 = HEAP32[$175>>2]|0;
      $lsub = $176;
     }
     $177 = $chmark;
     $178 = $jcolm1;
     $179 = ($177|0)!=($178|0);
     if (!($179)) {
      continue;
     }
     $jsuper = -1;
    }
   }
  } while(0);
  $249 = $k;
  $250 = (($249) + 1)|0;
  $k = $250;
 }
 if ((label|0) == 9) {
  $89 = $mem_error;
  $0 = $89;
  $346 = $0;
  STACKTOP = sp;return ($346|0);
 }
 else if ((label|0) == 22) {
  $173 = $mem_error;
  $0 = $173;
  $346 = $0;
  STACKTOP = sp;return ($346|0);
 }
 else if ((label|0) == 33) {
  $251 = $2;
  $252 = ($251|0)==(0);
  if ($252) {
   $253 = $supno;
   HEAP32[$253>>2] = 0;
   $nsuper = 0;
  } else {
   $254 = $nsuper;
   $255 = $xsup;
   $256 = (($255) + ($254<<2)|0);
   $257 = HEAP32[$256>>2]|0;
   $fsupc = $257;
   $258 = $2;
   $259 = $xlsub;
   $260 = (($259) + ($258<<2)|0);
   $261 = HEAP32[$260>>2]|0;
   $jptr = $261;
   $262 = $jcolm1;
   $263 = $xlsub;
   $264 = (($263) + ($262<<2)|0);
   $265 = HEAP32[$264>>2]|0;
   $jm1ptr = $265;
   $266 = $nextl;
   $267 = $jptr;
   $268 = (($266) - ($267))|0;
   $269 = $jptr;
   $270 = $jm1ptr;
   $271 = (($269) - ($270))|0;
   $272 = (($271) - 1)|0;
   $273 = ($268|0)!=($272|0);
   if ($273) {
    $jsuper = -1;
   }
   $274 = $2;
   $275 = $fsupc;
   $276 = (($274) - ($275))|0;
   $277 = HEAP32[2480>>2]|0;
   $278 = ($276|0)>=($277|0);
   if ($278) {
    $jsuper = -1;
   }
   $279 = $jsuper;
   $280 = ($279|0)==(-1);
   if ($280) {
    $281 = $fsupc;
    $282 = $jcolm1;
    $283 = (($282) - 1)|0;
    $284 = ($281|0)<($283|0);
    if ($284) {
     $285 = $fsupc;
     $286 = (($285) + 1)|0;
     $287 = $xlsub;
     $288 = (($287) + ($286<<2)|0);
     $289 = HEAP32[$288>>2]|0;
     $ito = $289;
     $290 = $ito;
     $291 = $jcolm1;
     $292 = $xlsub;
     $293 = (($292) + ($291<<2)|0);
     HEAP32[$293>>2] = $290;
     $294 = $ito;
     $295 = $jptr;
     $296 = (($294) + ($295))|0;
     $297 = $jm1ptr;
     $298 = (($296) - ($297))|0;
     $istop = $298;
     $299 = $istop;
     $300 = $jcolm1;
     $301 = $8;
     $302 = (($301) + ($300<<2)|0);
     HEAP32[$302>>2] = $299;
     $303 = $istop;
     $304 = $2;
     $305 = $xlsub;
     $306 = (($305) + ($304<<2)|0);
     HEAP32[$306>>2] = $303;
     $307 = $jm1ptr;
     $ifrom = $307;
     while(1) {
      $308 = $ifrom;
      $309 = $nextl;
      $310 = ($308|0)<($309|0);
      if (!($310)) {
       break;
      }
      $311 = $ifrom;
      $312 = $lsub;
      $313 = (($312) + ($311<<2)|0);
      $314 = HEAP32[$313>>2]|0;
      $315 = $ito;
      $316 = $lsub;
      $317 = (($316) + ($315<<2)|0);
      HEAP32[$317>>2] = $314;
      $318 = $ifrom;
      $319 = (($318) + 1)|0;
      $ifrom = $319;
      $320 = $ito;
      $321 = (($320) + 1)|0;
      $ito = $321;
     }
     $322 = $ito;
     $nextl = $322;
    }
    $323 = $nsuper;
    $324 = (($323) + 1)|0;
    $nsuper = $324;
    $325 = $nsuper;
    $326 = $2;
    $327 = $supno;
    $328 = (($327) + ($326<<2)|0);
    HEAP32[$328>>2] = $325;
   }
  }
  $329 = $jcolp1;
  $330 = $nsuper;
  $331 = (($330) + 1)|0;
  $332 = $xsup;
  $333 = (($332) + ($331<<2)|0);
  HEAP32[$333>>2] = $329;
  $334 = $nsuper;
  $335 = $jcolp1;
  $336 = $supno;
  $337 = (($336) + ($335<<2)|0);
  HEAP32[$337>>2] = $334;
  $338 = $nextl;
  $339 = $2;
  $340 = $8;
  $341 = (($340) + ($339<<2)|0);
  HEAP32[$341>>2] = $338;
  $342 = $nextl;
  $343 = $jcolp1;
  $344 = $xlsub;
  $345 = (($344) + ($343<<2)|0);
  HEAP32[$345>>2] = $342;
  $0 = 0;
  $346 = $0;
  STACKTOP = sp;return ($346|0);
 }
 return (0)|0;
}
function _zcolumn_bmod($jcol,$nseg,$dense,$tempv,$segrep,$repfnz,$fpanelc,$Glu,$stat) {
 $jcol = $jcol|0;
 $nseg = $nseg|0;
 $dense = $dense|0;
 $tempv = $tempv|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $fpanelc = $fpanelc|0;
 $Glu = $Glu|0;
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0.0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0.0, $1016 = 0.0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0.0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0.0, $103 = 0, $1030 = 0.0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0;
 var $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0.0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0.0, $129 = 0, $13 = 0, $130 = 0;
 var $131 = 0, $132 = 0, $133 = 0.0, $134 = 0, $135 = 0, $136 = 0.0, $137 = 0.0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0;
 var $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0;
 var $168 = 0.0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0.0, $173 = 0.0, $174 = 0, $175 = 0.0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0.0, $183 = 0, $184 = 0.0, $185 = 0;
 var $186 = 0, $187 = 0, $188 = 0.0, $189 = 0.0, $19 = 0, $190 = 0.0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0.0, $196 = 0.0, $197 = 0.0, $198 = 0.0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0;
 var $203 = 0, $204 = 0.0, $205 = 0.0, $206 = 0.0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0.0, $215 = 0, $216 = 0.0, $217 = 0.0, $218 = 0, $219 = 0, $22 = 0, $220 = 0;
 var $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0;
 var $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0.0, $256 = 0, $257 = 0;
 var $258 = 0, $259 = 0.0, $26 = 0, $260 = 0.0, $261 = 0, $262 = 0.0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0.0, $268 = 0.0, $269 = 0.0, $27 = 0, $270 = 0, $271 = 0.0, $272 = 0, $273 = 0, $274 = 0, $275 = 0.0;
 var $276 = 0.0, $277 = 0.0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0.0, $283 = 0.0, $284 = 0.0, $285 = 0.0, $286 = 0.0, $287 = 0, $288 = 0.0, $289 = 0.0, $29 = 0, $290 = 0.0, $291 = 0, $292 = 0.0, $293 = 0;
 var $294 = 0.0, $295 = 0.0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0;
 var $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0.0, $322 = 0, $323 = 0, $324 = 0, $325 = 0.0, $326 = 0.0, $327 = 0, $328 = 0.0, $329 = 0;
 var $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0.0, $334 = 0.0, $335 = 0.0, $336 = 0, $337 = 0.0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0.0, $342 = 0.0, $343 = 0.0, $344 = 0, $345 = 0, $346 = 0, $347 = 0;
 var $348 = 0.0, $349 = 0.0, $35 = 0, $350 = 0.0, $351 = 0.0, $352 = 0.0, $353 = 0, $354 = 0.0, $355 = 0, $356 = 0, $357 = 0, $358 = 0.0, $359 = 0.0, $36 = 0, $360 = 0, $361 = 0.0, $362 = 0, $363 = 0, $364 = 0, $365 = 0;
 var $366 = 0.0, $367 = 0.0, $368 = 0.0, $369 = 0, $37 = 0, $370 = 0.0, $371 = 0, $372 = 0, $373 = 0, $374 = 0.0, $375 = 0.0, $376 = 0.0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0.0, $382 = 0.0, $383 = 0.0;
 var $384 = 0.0, $385 = 0.0, $386 = 0, $387 = 0.0, $388 = 0.0, $389 = 0.0, $39 = 0, $390 = 0, $391 = 0.0, $392 = 0, $393 = 0.0, $394 = 0.0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0.0, $4 = 0, $40 = 0, $400 = 0.0;
 var $401 = 0.0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0.0, $41 = 0, $410 = 0, $411 = 0.0, $412 = 0.0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0;
 var $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0.0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0.0, $435 = 0.0, $436 = 0, $437 = 0.0;
 var $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0.0, $444 = 0.0, $445 = 0.0, $446 = 0, $447 = 0.0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0.0, $453 = 0.0, $454 = 0.0, $455 = 0;
 var $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0.0, $461 = 0.0, $462 = 0.0, $463 = 0.0, $464 = 0.0, $465 = 0, $466 = 0.0, $467 = 0.0, $468 = 0.0, $469 = 0, $47 = 0, $470 = 0.0, $471 = 0, $472 = 0.0, $473 = 0.0;
 var $474 = 0, $475 = 0.0, $476 = 0, $477 = 0, $478 = 0, $479 = 0.0, $48 = 0, $480 = 0.0, $481 = 0, $482 = 0.0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0.0, $488 = 0.0, $489 = 0.0, $49 = 0, $490 = 0, $491 = 0.0;
 var $492 = 0, $493 = 0, $494 = 0, $495 = 0.0, $496 = 0.0, $497 = 0.0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0.0, $503 = 0.0, $504 = 0.0, $505 = 0.0, $506 = 0.0, $507 = 0, $508 = 0.0, $509 = 0;
 var $51 = 0, $510 = 0, $511 = 0, $512 = 0.0, $513 = 0.0, $514 = 0, $515 = 0.0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0.0, $521 = 0.0, $522 = 0.0, $523 = 0, $524 = 0.0, $525 = 0, $526 = 0, $527 = 0;
 var $528 = 0.0, $529 = 0.0, $53 = 0, $530 = 0.0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0.0, $536 = 0.0, $537 = 0.0, $538 = 0.0, $539 = 0.0, $54 = 0, $540 = 0, $541 = 0.0, $542 = 0.0, $543 = 0.0, $544 = 0, $545 = 0.0;
 var $546 = 0, $547 = 0.0, $548 = 0.0, $549 = 0, $55 = 0, $550 = 0.0, $551 = 0.0, $552 = 0.0, $553 = 0, $554 = 0.0, $555 = 0, $556 = 0.0, $557 = 0.0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0;
 var $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0;
 var $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0.0, $593 = 0, $594 = 0, $595 = 0, $596 = 0.0, $597 = 0.0, $598 = 0, $599 = 0.0, $6 = 0;
 var $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0.0, $605 = 0.0, $606 = 0.0, $607 = 0, $608 = 0.0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0.0, $613 = 0.0, $614 = 0.0, $615 = 0, $616 = 0, $617 = 0;
 var $618 = 0, $619 = 0.0, $62 = 0, $620 = 0.0, $621 = 0.0, $622 = 0.0, $623 = 0.0, $624 = 0, $625 = 0.0, $626 = 0, $627 = 0, $628 = 0, $629 = 0.0, $63 = 0, $630 = 0.0, $631 = 0, $632 = 0.0, $633 = 0, $634 = 0, $635 = 0;
 var $636 = 0, $637 = 0.0, $638 = 0.0, $639 = 0.0, $64 = 0, $640 = 0, $641 = 0.0, $642 = 0, $643 = 0, $644 = 0, $645 = 0.0, $646 = 0.0, $647 = 0.0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0.0, $653 = 0.0;
 var $654 = 0.0, $655 = 0.0, $656 = 0.0, $657 = 0, $658 = 0.0, $659 = 0.0, $66 = 0, $660 = 0.0, $661 = 0, $662 = 0.0, $663 = 0, $664 = 0.0, $665 = 0.0, $666 = 0, $667 = 0.0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0.0;
 var $672 = 0.0, $673 = 0, $674 = 0.0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0.0, $68 = 0, $680 = 0.0, $681 = 0.0, $682 = 0, $683 = 0.0, $684 = 0, $685 = 0, $686 = 0, $687 = 0.0, $688 = 0.0, $689 = 0.0, $69 = 0;
 var $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0.0, $695 = 0.0, $696 = 0.0, $697 = 0.0, $698 = 0.0, $699 = 0, $7 = 0, $70 = 0, $700 = 0.0, $701 = 0.0, $702 = 0.0, $703 = 0, $704 = 0.0, $705 = 0, $706 = 0.0, $707 = 0.0;
 var $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0.0, $713 = 0.0, $714 = 0.0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0.0, $723 = 0, $724 = 0.0, $725 = 0.0;
 var $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0;
 var $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0;
 var $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0;
 var $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0;
 var $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0.0;
 var $816 = 0, $817 = 0, $818 = 0, $819 = 0.0, $82 = 0, $820 = 0.0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0.0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0.0;
 var $834 = 0.0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0;
 var $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0;
 var $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0;
 var $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0;
 var $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0;
 var $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0;
 var $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0;
 var $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0.0, $969 = 0, $97 = 0, $970 = 0, $971 = 0.0, $972 = 0.0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0.0, $978 = 0;
 var $979 = 0, $98 = 0, $980 = 0.0, $981 = 0.0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0;
 var $997 = 0, $998 = 0, $999 = 0, $ci = 0.0, $ci10 = 0.0, $ci12 = 0.0, $ci14 = 0.0, $ci16 = 0.0, $ci18 = 0.0, $ci2 = 0.0, $ci4 = 0.0, $ci6 = 0.0, $ci8 = 0.0, $comp_temp = 0, $comp_temp1 = 0, $cr = 0.0, $cr1 = 0.0, $cr11 = 0.0, $cr13 = 0.0, $cr15 = 0.0;
 var $cr17 = 0.0, $cr3 = 0.0, $cr5 = 0.0, $cr7 = 0.0, $cr9 = 0.0, $d_fsupc = 0, $fst_col = 0, $fsupc = 0, $i = 0, $incx = 0, $incy = 0, $irow = 0, $isub = 0, $jcolp1 = 0, $jsupno = 0, $k = 0, $kfnz = 0, $krep = 0, $krep_ind = 0, $ksub = 0;
 var $ksupno = 0, $lptr = 0, $lsub = 0, $luptr = 0, $luptr1 = 0, $luptr2 = 0, $lusup = 0, $mem_error = 0, $new_next = 0, $nextlu = 0, $no_zeros = 0, $none = 0, $nrow = 0, $nsupc = 0, $nsupr = 0, $nzlumax = 0, $one = 0, $ops = 0, $segsze = 0, $supno = 0;
 var $tempv1 = 0, $ufirst = 0, $ukj = 0, $ukj1 = 0, $ukj2 = 0, $xlsub = 0, $xlusup = 0, $xsup = 0, $zero = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 480|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ukj = sp + 272|0;
 $ukj1 = sp + 256|0;
 $ukj2 = sp + 240|0;
 $nzlumax = sp + 300|0;
 $zero = sp + 224|0;
 $one = sp + 208|0;
 $none = sp + 192|0;
 $comp_temp = sp + 176|0;
 $comp_temp1 = sp + 160|0;
 $1 = $jcol;
 $2 = $nseg;
 $3 = $dense;
 $4 = $tempv;
 $5 = $segrep;
 $6 = $repfnz;
 $7 = $fpanelc;
 $8 = $Glu;
 $9 = $stat;
 $incx = 1;
 $incy = 1;
 ;HEAP32[$zero>>2]=0|0;HEAP32[$zero+4>>2]=0|0;HEAP32[$zero+8>>2]=0|0;HEAP32[$zero+12>>2]=0|0;
 ;HEAP32[$one>>2]=HEAP32[2504>>2]|0;HEAP32[$one+4>>2]=HEAP32[2504+4>>2]|0;HEAP32[$one+8>>2]=HEAP32[2504+8>>2]|0;HEAP32[$one+12>>2]=HEAP32[2504+12>>2]|0;
 ;HEAP32[$none>>2]=HEAP32[2488>>2]|0;HEAP32[$none+4>>2]=HEAP32[2488+4>>2]|0;HEAP32[$none+8>>2]=HEAP32[2488+8>>2]|0;HEAP32[$none+12>>2]=HEAP32[2488+12>>2]|0;
 $10 = $9;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $ops = $12;
 $13 = $8;
 $14 = HEAP32[$13>>2]|0;
 $xsup = $14;
 $15 = $8;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $supno = $17;
 $18 = $8;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $lsub = $20;
 $21 = $8;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $xlsub = $23;
 $24 = $8;
 $25 = ((($24)) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $lusup = $26;
 $27 = $8;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 $xlusup = $29;
 $30 = $8;
 $31 = ((($30)) + 44|0);
 $32 = HEAP32[$31>>2]|0;
 HEAP32[$nzlumax>>2] = $32;
 $33 = $1;
 $34 = (($33) + 1)|0;
 $jcolp1 = $34;
 $35 = $1;
 $36 = $supno;
 $37 = (($36) + ($35<<2)|0);
 $38 = HEAP32[$37>>2]|0;
 $jsupno = $38;
 $39 = $2;
 $40 = (($39) - 1)|0;
 $k = $40;
 $ksub = 0;
 while(1) {
  $41 = $ksub;
  $42 = $2;
  $43 = ($41|0)<($42|0);
  if (!($43)) {
   break;
  }
  $44 = $k;
  $45 = $5;
  $46 = (($45) + ($44<<2)|0);
  $47 = HEAP32[$46>>2]|0;
  $krep = $47;
  $48 = $k;
  $49 = (($48) + -1)|0;
  $k = $49;
  $50 = $krep;
  $51 = $supno;
  $52 = (($51) + ($50<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  $ksupno = $53;
  $54 = $jsupno;
  $55 = $ksupno;
  $56 = ($54|0)!=($55|0);
  L4: do {
   if ($56) {
    $57 = $ksupno;
    $58 = $xsup;
    $59 = (($58) + ($57<<2)|0);
    $60 = HEAP32[$59>>2]|0;
    $fsupc = $60;
    $61 = $fsupc;
    $62 = $7;
    $63 = ($61|0)>($62|0);
    $64 = $fsupc;
    $65 = $7;
    $66 = $63 ? $64 : $65;
    $fst_col = $66;
    $67 = $fst_col;
    $68 = $fsupc;
    $69 = (($67) - ($68))|0;
    $d_fsupc = $69;
    $70 = $fst_col;
    $71 = $xlusup;
    $72 = (($71) + ($70<<2)|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = $d_fsupc;
    $75 = (($73) + ($74))|0;
    $luptr = $75;
    $76 = $fsupc;
    $77 = $xlsub;
    $78 = (($77) + ($76<<2)|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = $d_fsupc;
    $81 = (($79) + ($80))|0;
    $lptr = $81;
    $82 = $krep;
    $83 = $6;
    $84 = (($83) + ($82<<2)|0);
    $85 = HEAP32[$84>>2]|0;
    $kfnz = $85;
    $86 = $kfnz;
    $87 = $7;
    $88 = ($86|0)>($87|0);
    $89 = $kfnz;
    $90 = $7;
    $91 = $88 ? $89 : $90;
    $kfnz = $91;
    $92 = $krep;
    $93 = $kfnz;
    $94 = (($92) - ($93))|0;
    $95 = (($94) + 1)|0;
    $segsze = $95;
    $96 = $krep;
    $97 = $fst_col;
    $98 = (($96) - ($97))|0;
    $99 = (($98) + 1)|0;
    $nsupc = $99;
    $100 = $fsupc;
    $101 = (($100) + 1)|0;
    $102 = $xlsub;
    $103 = (($102) + ($101<<2)|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = $fsupc;
    $106 = $xlsub;
    $107 = (($106) + ($105<<2)|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = (($104) - ($108))|0;
    $nsupr = $109;
    $110 = $nsupr;
    $111 = $d_fsupc;
    $112 = (($110) - ($111))|0;
    $113 = $nsupc;
    $114 = (($112) - ($113))|0;
    $nrow = $114;
    $115 = $lptr;
    $116 = $nsupc;
    $117 = (($115) + ($116))|0;
    $118 = (($117) - 1)|0;
    $krep_ind = $118;
    $119 = $segsze;
    $120 = $119<<2;
    $121 = $segsze;
    $122 = (($121) - 1)|0;
    $123 = Math_imul($120, $122)|0;
    $124 = (+($123|0));
    $125 = $ops;
    $126 = ((($125)) + 52|0);
    $127 = +HEAPF32[$126>>2];
    $128 = $127 + $124;
    HEAPF32[$126>>2] = $128;
    $129 = $nrow;
    $130 = $129<<3;
    $131 = $segsze;
    $132 = Math_imul($130, $131)|0;
    $133 = (+($132|0));
    $134 = $ops;
    $135 = ((($134)) + 56|0);
    $136 = +HEAPF32[$135>>2];
    $137 = $136 + $133;
    HEAPF32[$135>>2] = $137;
    $138 = $segsze;
    $139 = ($138|0)==(1);
    if ($139) {
     $140 = $krep_ind;
     $141 = $lsub;
     $142 = (($141) + ($140<<2)|0);
     $143 = HEAP32[$142>>2]|0;
     $144 = $3;
     $145 = (($144) + ($143<<4)|0);
     ;HEAP32[$ukj>>2]=HEAP32[$145>>2]|0;HEAP32[$ukj+4>>2]=HEAP32[$145+4>>2]|0;HEAP32[$ukj+8>>2]=HEAP32[$145+8>>2]|0;HEAP32[$ukj+12>>2]=HEAP32[$145+12>>2]|0;
     $146 = $nsupr;
     $147 = $nsupc;
     $148 = (($147) - 1)|0;
     $149 = Math_imul($146, $148)|0;
     $150 = $nsupc;
     $151 = (($149) + ($150))|0;
     $152 = $luptr;
     $153 = (($152) + ($151))|0;
     $luptr = $153;
     $154 = $lptr;
     $155 = $nsupc;
     $156 = (($154) + ($155))|0;
     $i = $156;
     while(1) {
      $157 = $i;
      $158 = $fsupc;
      $159 = (($158) + 1)|0;
      $160 = $xlsub;
      $161 = (($160) + ($159<<2)|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = ($157|0)<($162|0);
      if (!($163)) {
       break L4;
      }
      $164 = $i;
      $165 = $lsub;
      $166 = (($165) + ($164<<2)|0);
      $167 = HEAP32[$166>>2]|0;
      $irow = $167;
      $168 = +HEAPF64[$ukj>>3];
      $169 = $luptr;
      $170 = $lusup;
      $171 = (($170) + ($169<<4)|0);
      $172 = +HEAPF64[$171>>3];
      $173 = $168 * $172;
      $174 = ((($ukj)) + 8|0);
      $175 = +HEAPF64[$174>>3];
      $176 = $luptr;
      $177 = $lusup;
      $178 = (($177) + ($176<<4)|0);
      $179 = ((($178)) + 8|0);
      $180 = +HEAPF64[$179>>3];
      $181 = $175 * $180;
      $182 = $173 - $181;
      $cr = $182;
      $183 = ((($ukj)) + 8|0);
      $184 = +HEAPF64[$183>>3];
      $185 = $luptr;
      $186 = $lusup;
      $187 = (($186) + ($185<<4)|0);
      $188 = +HEAPF64[$187>>3];
      $189 = $184 * $188;
      $190 = +HEAPF64[$ukj>>3];
      $191 = $luptr;
      $192 = $lusup;
      $193 = (($192) + ($191<<4)|0);
      $194 = ((($193)) + 8|0);
      $195 = +HEAPF64[$194>>3];
      $196 = $190 * $195;
      $197 = $189 + $196;
      $ci = $197;
      $198 = $cr;
      HEAPF64[$comp_temp>>3] = $198;
      $199 = $ci;
      $200 = ((($comp_temp)) + 8|0);
      HEAPF64[$200>>3] = $199;
      $201 = $irow;
      $202 = $3;
      $203 = (($202) + ($201<<4)|0);
      $204 = +HEAPF64[$203>>3];
      $205 = +HEAPF64[$comp_temp>>3];
      $206 = $204 - $205;
      $207 = $irow;
      $208 = $3;
      $209 = (($208) + ($207<<4)|0);
      HEAPF64[$209>>3] = $206;
      $210 = $irow;
      $211 = $3;
      $212 = (($211) + ($210<<4)|0);
      $213 = ((($212)) + 8|0);
      $214 = +HEAPF64[$213>>3];
      $215 = ((($comp_temp)) + 8|0);
      $216 = +HEAPF64[$215>>3];
      $217 = $214 - $216;
      $218 = $irow;
      $219 = $3;
      $220 = (($219) + ($218<<4)|0);
      $221 = ((($220)) + 8|0);
      HEAPF64[$221>>3] = $217;
      $222 = $luptr;
      $223 = (($222) + 1)|0;
      $luptr = $223;
      $224 = $i;
      $225 = (($224) + 1)|0;
      $i = $225;
     }
    }
    $226 = $segsze;
    $227 = ($226|0)<=(3);
    if ($227) {
     $228 = $krep_ind;
     $229 = $lsub;
     $230 = (($229) + ($228<<2)|0);
     $231 = HEAP32[$230>>2]|0;
     $232 = $3;
     $233 = (($232) + ($231<<4)|0);
     ;HEAP32[$ukj>>2]=HEAP32[$233>>2]|0;HEAP32[$ukj+4>>2]=HEAP32[$233+4>>2]|0;HEAP32[$ukj+8>>2]=HEAP32[$233+8>>2]|0;HEAP32[$ukj+12>>2]=HEAP32[$233+12>>2]|0;
     $234 = $nsupr;
     $235 = $nsupc;
     $236 = (($235) - 1)|0;
     $237 = Math_imul($234, $236)|0;
     $238 = $nsupc;
     $239 = (($237) + ($238))|0;
     $240 = (($239) - 1)|0;
     $241 = $luptr;
     $242 = (($241) + ($240))|0;
     $luptr = $242;
     $243 = $krep_ind;
     $244 = (($243) - 1)|0;
     $245 = $lsub;
     $246 = (($245) + ($244<<2)|0);
     $247 = HEAP32[$246>>2]|0;
     $248 = $3;
     $249 = (($248) + ($247<<4)|0);
     ;HEAP32[$ukj1>>2]=HEAP32[$249>>2]|0;HEAP32[$ukj1+4>>2]=HEAP32[$249+4>>2]|0;HEAP32[$ukj1+8>>2]=HEAP32[$249+8>>2]|0;HEAP32[$ukj1+12>>2]=HEAP32[$249+12>>2]|0;
     $250 = $luptr;
     $251 = $nsupr;
     $252 = (($250) - ($251))|0;
     $luptr1 = $252;
     $253 = $segsze;
     $254 = ($253|0)==(2);
     if ($254) {
      $255 = +HEAPF64[$ukj1>>3];
      $256 = $luptr1;
      $257 = $lusup;
      $258 = (($257) + ($256<<4)|0);
      $259 = +HEAPF64[$258>>3];
      $260 = $255 * $259;
      $261 = ((($ukj1)) + 8|0);
      $262 = +HEAPF64[$261>>3];
      $263 = $luptr1;
      $264 = $lusup;
      $265 = (($264) + ($263<<4)|0);
      $266 = ((($265)) + 8|0);
      $267 = +HEAPF64[$266>>3];
      $268 = $262 * $267;
      $269 = $260 - $268;
      $cr1 = $269;
      $270 = ((($ukj1)) + 8|0);
      $271 = +HEAPF64[$270>>3];
      $272 = $luptr1;
      $273 = $lusup;
      $274 = (($273) + ($272<<4)|0);
      $275 = +HEAPF64[$274>>3];
      $276 = $271 * $275;
      $277 = +HEAPF64[$ukj1>>3];
      $278 = $luptr1;
      $279 = $lusup;
      $280 = (($279) + ($278<<4)|0);
      $281 = ((($280)) + 8|0);
      $282 = +HEAPF64[$281>>3];
      $283 = $277 * $282;
      $284 = $276 + $283;
      $ci2 = $284;
      $285 = $cr1;
      HEAPF64[$comp_temp>>3] = $285;
      $286 = $ci2;
      $287 = ((($comp_temp)) + 8|0);
      HEAPF64[$287>>3] = $286;
      $288 = +HEAPF64[$ukj>>3];
      $289 = +HEAPF64[$comp_temp>>3];
      $290 = $288 - $289;
      HEAPF64[$ukj>>3] = $290;
      $291 = ((($ukj)) + 8|0);
      $292 = +HEAPF64[$291>>3];
      $293 = ((($comp_temp)) + 8|0);
      $294 = +HEAPF64[$293>>3];
      $295 = $292 - $294;
      $296 = ((($ukj)) + 8|0);
      HEAPF64[$296>>3] = $295;
      $297 = $krep_ind;
      $298 = $lsub;
      $299 = (($298) + ($297<<2)|0);
      $300 = HEAP32[$299>>2]|0;
      $301 = $3;
      $302 = (($301) + ($300<<4)|0);
      ;HEAP32[$302>>2]=HEAP32[$ukj>>2]|0;HEAP32[$302+4>>2]=HEAP32[$ukj+4>>2]|0;HEAP32[$302+8>>2]=HEAP32[$ukj+8>>2]|0;HEAP32[$302+12>>2]=HEAP32[$ukj+12>>2]|0;
      $303 = $lptr;
      $304 = $nsupc;
      $305 = (($303) + ($304))|0;
      $i = $305;
      while(1) {
       $306 = $i;
       $307 = $fsupc;
       $308 = (($307) + 1)|0;
       $309 = $xlsub;
       $310 = (($309) + ($308<<2)|0);
       $311 = HEAP32[$310>>2]|0;
       $312 = ($306|0)<($311|0);
       if (!($312)) {
        break L4;
       }
       $313 = $i;
       $314 = $lsub;
       $315 = (($314) + ($313<<2)|0);
       $316 = HEAP32[$315>>2]|0;
       $irow = $316;
       $317 = $luptr;
       $318 = (($317) + 1)|0;
       $luptr = $318;
       $319 = $luptr1;
       $320 = (($319) + 1)|0;
       $luptr1 = $320;
       $321 = +HEAPF64[$ukj>>3];
       $322 = $luptr;
       $323 = $lusup;
       $324 = (($323) + ($322<<4)|0);
       $325 = +HEAPF64[$324>>3];
       $326 = $321 * $325;
       $327 = ((($ukj)) + 8|0);
       $328 = +HEAPF64[$327>>3];
       $329 = $luptr;
       $330 = $lusup;
       $331 = (($330) + ($329<<4)|0);
       $332 = ((($331)) + 8|0);
       $333 = +HEAPF64[$332>>3];
       $334 = $328 * $333;
       $335 = $326 - $334;
       $cr3 = $335;
       $336 = ((($ukj)) + 8|0);
       $337 = +HEAPF64[$336>>3];
       $338 = $luptr;
       $339 = $lusup;
       $340 = (($339) + ($338<<4)|0);
       $341 = +HEAPF64[$340>>3];
       $342 = $337 * $341;
       $343 = +HEAPF64[$ukj>>3];
       $344 = $luptr;
       $345 = $lusup;
       $346 = (($345) + ($344<<4)|0);
       $347 = ((($346)) + 8|0);
       $348 = +HEAPF64[$347>>3];
       $349 = $343 * $348;
       $350 = $342 + $349;
       $ci4 = $350;
       $351 = $cr3;
       HEAPF64[$comp_temp>>3] = $351;
       $352 = $ci4;
       $353 = ((($comp_temp)) + 8|0);
       HEAPF64[$353>>3] = $352;
       $354 = +HEAPF64[$ukj1>>3];
       $355 = $luptr1;
       $356 = $lusup;
       $357 = (($356) + ($355<<4)|0);
       $358 = +HEAPF64[$357>>3];
       $359 = $354 * $358;
       $360 = ((($ukj1)) + 8|0);
       $361 = +HEAPF64[$360>>3];
       $362 = $luptr1;
       $363 = $lusup;
       $364 = (($363) + ($362<<4)|0);
       $365 = ((($364)) + 8|0);
       $366 = +HEAPF64[$365>>3];
       $367 = $361 * $366;
       $368 = $359 - $367;
       $cr5 = $368;
       $369 = ((($ukj1)) + 8|0);
       $370 = +HEAPF64[$369>>3];
       $371 = $luptr1;
       $372 = $lusup;
       $373 = (($372) + ($371<<4)|0);
       $374 = +HEAPF64[$373>>3];
       $375 = $370 * $374;
       $376 = +HEAPF64[$ukj1>>3];
       $377 = $luptr1;
       $378 = $lusup;
       $379 = (($378) + ($377<<4)|0);
       $380 = ((($379)) + 8|0);
       $381 = +HEAPF64[$380>>3];
       $382 = $376 * $381;
       $383 = $375 + $382;
       $ci6 = $383;
       $384 = $cr5;
       HEAPF64[$comp_temp1>>3] = $384;
       $385 = $ci6;
       $386 = ((($comp_temp1)) + 8|0);
       HEAPF64[$386>>3] = $385;
       $387 = +HEAPF64[$comp_temp>>3];
       $388 = +HEAPF64[$comp_temp1>>3];
       $389 = $387 + $388;
       HEAPF64[$comp_temp>>3] = $389;
       $390 = ((($comp_temp)) + 8|0);
       $391 = +HEAPF64[$390>>3];
       $392 = ((($comp_temp1)) + 8|0);
       $393 = +HEAPF64[$392>>3];
       $394 = $391 + $393;
       $395 = ((($comp_temp)) + 8|0);
       HEAPF64[$395>>3] = $394;
       $396 = $irow;
       $397 = $3;
       $398 = (($397) + ($396<<4)|0);
       $399 = +HEAPF64[$398>>3];
       $400 = +HEAPF64[$comp_temp>>3];
       $401 = $399 - $400;
       $402 = $irow;
       $403 = $3;
       $404 = (($403) + ($402<<4)|0);
       HEAPF64[$404>>3] = $401;
       $405 = $irow;
       $406 = $3;
       $407 = (($406) + ($405<<4)|0);
       $408 = ((($407)) + 8|0);
       $409 = +HEAPF64[$408>>3];
       $410 = ((($comp_temp)) + 8|0);
       $411 = +HEAPF64[$410>>3];
       $412 = $409 - $411;
       $413 = $irow;
       $414 = $3;
       $415 = (($414) + ($413<<4)|0);
       $416 = ((($415)) + 8|0);
       HEAPF64[$416>>3] = $412;
       $417 = $i;
       $418 = (($417) + 1)|0;
       $i = $418;
      }
     } else {
      $419 = $krep_ind;
      $420 = (($419) - 2)|0;
      $421 = $lsub;
      $422 = (($421) + ($420<<2)|0);
      $423 = HEAP32[$422>>2]|0;
      $424 = $3;
      $425 = (($424) + ($423<<4)|0);
      ;HEAP32[$ukj2>>2]=HEAP32[$425>>2]|0;HEAP32[$ukj2+4>>2]=HEAP32[$425+4>>2]|0;HEAP32[$ukj2+8>>2]=HEAP32[$425+8>>2]|0;HEAP32[$ukj2+12>>2]=HEAP32[$425+12>>2]|0;
      $426 = $luptr1;
      $427 = $nsupr;
      $428 = (($426) - ($427))|0;
      $luptr2 = $428;
      $429 = +HEAPF64[$ukj2>>3];
      $430 = $luptr2;
      $431 = (($430) - 1)|0;
      $432 = $lusup;
      $433 = (($432) + ($431<<4)|0);
      $434 = +HEAPF64[$433>>3];
      $435 = $429 * $434;
      $436 = ((($ukj2)) + 8|0);
      $437 = +HEAPF64[$436>>3];
      $438 = $luptr2;
      $439 = (($438) - 1)|0;
      $440 = $lusup;
      $441 = (($440) + ($439<<4)|0);
      $442 = ((($441)) + 8|0);
      $443 = +HEAPF64[$442>>3];
      $444 = $437 * $443;
      $445 = $435 - $444;
      $cr7 = $445;
      $446 = ((($ukj2)) + 8|0);
      $447 = +HEAPF64[$446>>3];
      $448 = $luptr2;
      $449 = (($448) - 1)|0;
      $450 = $lusup;
      $451 = (($450) + ($449<<4)|0);
      $452 = +HEAPF64[$451>>3];
      $453 = $447 * $452;
      $454 = +HEAPF64[$ukj2>>3];
      $455 = $luptr2;
      $456 = (($455) - 1)|0;
      $457 = $lusup;
      $458 = (($457) + ($456<<4)|0);
      $459 = ((($458)) + 8|0);
      $460 = +HEAPF64[$459>>3];
      $461 = $454 * $460;
      $462 = $453 + $461;
      $ci8 = $462;
      $463 = $cr7;
      HEAPF64[$comp_temp>>3] = $463;
      $464 = $ci8;
      $465 = ((($comp_temp)) + 8|0);
      HEAPF64[$465>>3] = $464;
      $466 = +HEAPF64[$ukj1>>3];
      $467 = +HEAPF64[$comp_temp>>3];
      $468 = $466 - $467;
      HEAPF64[$ukj1>>3] = $468;
      $469 = ((($ukj1)) + 8|0);
      $470 = +HEAPF64[$469>>3];
      $471 = ((($comp_temp)) + 8|0);
      $472 = +HEAPF64[$471>>3];
      $473 = $470 - $472;
      $474 = ((($ukj1)) + 8|0);
      HEAPF64[$474>>3] = $473;
      $475 = +HEAPF64[$ukj1>>3];
      $476 = $luptr1;
      $477 = $lusup;
      $478 = (($477) + ($476<<4)|0);
      $479 = +HEAPF64[$478>>3];
      $480 = $475 * $479;
      $481 = ((($ukj1)) + 8|0);
      $482 = +HEAPF64[$481>>3];
      $483 = $luptr1;
      $484 = $lusup;
      $485 = (($484) + ($483<<4)|0);
      $486 = ((($485)) + 8|0);
      $487 = +HEAPF64[$486>>3];
      $488 = $482 * $487;
      $489 = $480 - $488;
      $cr9 = $489;
      $490 = ((($ukj1)) + 8|0);
      $491 = +HEAPF64[$490>>3];
      $492 = $luptr1;
      $493 = $lusup;
      $494 = (($493) + ($492<<4)|0);
      $495 = +HEAPF64[$494>>3];
      $496 = $491 * $495;
      $497 = +HEAPF64[$ukj1>>3];
      $498 = $luptr1;
      $499 = $lusup;
      $500 = (($499) + ($498<<4)|0);
      $501 = ((($500)) + 8|0);
      $502 = +HEAPF64[$501>>3];
      $503 = $497 * $502;
      $504 = $496 + $503;
      $ci10 = $504;
      $505 = $cr9;
      HEAPF64[$comp_temp>>3] = $505;
      $506 = $ci10;
      $507 = ((($comp_temp)) + 8|0);
      HEAPF64[$507>>3] = $506;
      $508 = +HEAPF64[$ukj2>>3];
      $509 = $luptr2;
      $510 = $lusup;
      $511 = (($510) + ($509<<4)|0);
      $512 = +HEAPF64[$511>>3];
      $513 = $508 * $512;
      $514 = ((($ukj2)) + 8|0);
      $515 = +HEAPF64[$514>>3];
      $516 = $luptr2;
      $517 = $lusup;
      $518 = (($517) + ($516<<4)|0);
      $519 = ((($518)) + 8|0);
      $520 = +HEAPF64[$519>>3];
      $521 = $515 * $520;
      $522 = $513 - $521;
      $cr11 = $522;
      $523 = ((($ukj2)) + 8|0);
      $524 = +HEAPF64[$523>>3];
      $525 = $luptr2;
      $526 = $lusup;
      $527 = (($526) + ($525<<4)|0);
      $528 = +HEAPF64[$527>>3];
      $529 = $524 * $528;
      $530 = +HEAPF64[$ukj2>>3];
      $531 = $luptr2;
      $532 = $lusup;
      $533 = (($532) + ($531<<4)|0);
      $534 = ((($533)) + 8|0);
      $535 = +HEAPF64[$534>>3];
      $536 = $530 * $535;
      $537 = $529 + $536;
      $ci12 = $537;
      $538 = $cr11;
      HEAPF64[$comp_temp1>>3] = $538;
      $539 = $ci12;
      $540 = ((($comp_temp1)) + 8|0);
      HEAPF64[$540>>3] = $539;
      $541 = +HEAPF64[$comp_temp>>3];
      $542 = +HEAPF64[$comp_temp1>>3];
      $543 = $541 + $542;
      HEAPF64[$comp_temp>>3] = $543;
      $544 = ((($comp_temp)) + 8|0);
      $545 = +HEAPF64[$544>>3];
      $546 = ((($comp_temp1)) + 8|0);
      $547 = +HEAPF64[$546>>3];
      $548 = $545 + $547;
      $549 = ((($comp_temp)) + 8|0);
      HEAPF64[$549>>3] = $548;
      $550 = +HEAPF64[$ukj>>3];
      $551 = +HEAPF64[$comp_temp>>3];
      $552 = $550 - $551;
      HEAPF64[$ukj>>3] = $552;
      $553 = ((($ukj)) + 8|0);
      $554 = +HEAPF64[$553>>3];
      $555 = ((($comp_temp)) + 8|0);
      $556 = +HEAPF64[$555>>3];
      $557 = $554 - $556;
      $558 = ((($ukj)) + 8|0);
      HEAPF64[$558>>3] = $557;
      $559 = $krep_ind;
      $560 = $lsub;
      $561 = (($560) + ($559<<2)|0);
      $562 = HEAP32[$561>>2]|0;
      $563 = $3;
      $564 = (($563) + ($562<<4)|0);
      ;HEAP32[$564>>2]=HEAP32[$ukj>>2]|0;HEAP32[$564+4>>2]=HEAP32[$ukj+4>>2]|0;HEAP32[$564+8>>2]=HEAP32[$ukj+8>>2]|0;HEAP32[$564+12>>2]=HEAP32[$ukj+12>>2]|0;
      $565 = $krep_ind;
      $566 = (($565) - 1)|0;
      $567 = $lsub;
      $568 = (($567) + ($566<<2)|0);
      $569 = HEAP32[$568>>2]|0;
      $570 = $3;
      $571 = (($570) + ($569<<4)|0);
      ;HEAP32[$571>>2]=HEAP32[$ukj1>>2]|0;HEAP32[$571+4>>2]=HEAP32[$ukj1+4>>2]|0;HEAP32[$571+8>>2]=HEAP32[$ukj1+8>>2]|0;HEAP32[$571+12>>2]=HEAP32[$ukj1+12>>2]|0;
      $572 = $lptr;
      $573 = $nsupc;
      $574 = (($572) + ($573))|0;
      $i = $574;
      while(1) {
       $575 = $i;
       $576 = $fsupc;
       $577 = (($576) + 1)|0;
       $578 = $xlsub;
       $579 = (($578) + ($577<<2)|0);
       $580 = HEAP32[$579>>2]|0;
       $581 = ($575|0)<($580|0);
       if (!($581)) {
        break L4;
       }
       $582 = $i;
       $583 = $lsub;
       $584 = (($583) + ($582<<2)|0);
       $585 = HEAP32[$584>>2]|0;
       $irow = $585;
       $586 = $luptr;
       $587 = (($586) + 1)|0;
       $luptr = $587;
       $588 = $luptr1;
       $589 = (($588) + 1)|0;
       $luptr1 = $589;
       $590 = $luptr2;
       $591 = (($590) + 1)|0;
       $luptr2 = $591;
       $592 = +HEAPF64[$ukj>>3];
       $593 = $luptr;
       $594 = $lusup;
       $595 = (($594) + ($593<<4)|0);
       $596 = +HEAPF64[$595>>3];
       $597 = $592 * $596;
       $598 = ((($ukj)) + 8|0);
       $599 = +HEAPF64[$598>>3];
       $600 = $luptr;
       $601 = $lusup;
       $602 = (($601) + ($600<<4)|0);
       $603 = ((($602)) + 8|0);
       $604 = +HEAPF64[$603>>3];
       $605 = $599 * $604;
       $606 = $597 - $605;
       $cr13 = $606;
       $607 = ((($ukj)) + 8|0);
       $608 = +HEAPF64[$607>>3];
       $609 = $luptr;
       $610 = $lusup;
       $611 = (($610) + ($609<<4)|0);
       $612 = +HEAPF64[$611>>3];
       $613 = $608 * $612;
       $614 = +HEAPF64[$ukj>>3];
       $615 = $luptr;
       $616 = $lusup;
       $617 = (($616) + ($615<<4)|0);
       $618 = ((($617)) + 8|0);
       $619 = +HEAPF64[$618>>3];
       $620 = $614 * $619;
       $621 = $613 + $620;
       $ci14 = $621;
       $622 = $cr13;
       HEAPF64[$comp_temp>>3] = $622;
       $623 = $ci14;
       $624 = ((($comp_temp)) + 8|0);
       HEAPF64[$624>>3] = $623;
       $625 = +HEAPF64[$ukj1>>3];
       $626 = $luptr1;
       $627 = $lusup;
       $628 = (($627) + ($626<<4)|0);
       $629 = +HEAPF64[$628>>3];
       $630 = $625 * $629;
       $631 = ((($ukj1)) + 8|0);
       $632 = +HEAPF64[$631>>3];
       $633 = $luptr1;
       $634 = $lusup;
       $635 = (($634) + ($633<<4)|0);
       $636 = ((($635)) + 8|0);
       $637 = +HEAPF64[$636>>3];
       $638 = $632 * $637;
       $639 = $630 - $638;
       $cr15 = $639;
       $640 = ((($ukj1)) + 8|0);
       $641 = +HEAPF64[$640>>3];
       $642 = $luptr1;
       $643 = $lusup;
       $644 = (($643) + ($642<<4)|0);
       $645 = +HEAPF64[$644>>3];
       $646 = $641 * $645;
       $647 = +HEAPF64[$ukj1>>3];
       $648 = $luptr1;
       $649 = $lusup;
       $650 = (($649) + ($648<<4)|0);
       $651 = ((($650)) + 8|0);
       $652 = +HEAPF64[$651>>3];
       $653 = $647 * $652;
       $654 = $646 + $653;
       $ci16 = $654;
       $655 = $cr15;
       HEAPF64[$comp_temp1>>3] = $655;
       $656 = $ci16;
       $657 = ((($comp_temp1)) + 8|0);
       HEAPF64[$657>>3] = $656;
       $658 = +HEAPF64[$comp_temp>>3];
       $659 = +HEAPF64[$comp_temp1>>3];
       $660 = $658 + $659;
       HEAPF64[$comp_temp>>3] = $660;
       $661 = ((($comp_temp)) + 8|0);
       $662 = +HEAPF64[$661>>3];
       $663 = ((($comp_temp1)) + 8|0);
       $664 = +HEAPF64[$663>>3];
       $665 = $662 + $664;
       $666 = ((($comp_temp)) + 8|0);
       HEAPF64[$666>>3] = $665;
       $667 = +HEAPF64[$ukj2>>3];
       $668 = $luptr2;
       $669 = $lusup;
       $670 = (($669) + ($668<<4)|0);
       $671 = +HEAPF64[$670>>3];
       $672 = $667 * $671;
       $673 = ((($ukj2)) + 8|0);
       $674 = +HEAPF64[$673>>3];
       $675 = $luptr2;
       $676 = $lusup;
       $677 = (($676) + ($675<<4)|0);
       $678 = ((($677)) + 8|0);
       $679 = +HEAPF64[$678>>3];
       $680 = $674 * $679;
       $681 = $672 - $680;
       $cr17 = $681;
       $682 = ((($ukj2)) + 8|0);
       $683 = +HEAPF64[$682>>3];
       $684 = $luptr2;
       $685 = $lusup;
       $686 = (($685) + ($684<<4)|0);
       $687 = +HEAPF64[$686>>3];
       $688 = $683 * $687;
       $689 = +HEAPF64[$ukj2>>3];
       $690 = $luptr2;
       $691 = $lusup;
       $692 = (($691) + ($690<<4)|0);
       $693 = ((($692)) + 8|0);
       $694 = +HEAPF64[$693>>3];
       $695 = $689 * $694;
       $696 = $688 + $695;
       $ci18 = $696;
       $697 = $cr17;
       HEAPF64[$comp_temp1>>3] = $697;
       $698 = $ci18;
       $699 = ((($comp_temp1)) + 8|0);
       HEAPF64[$699>>3] = $698;
       $700 = +HEAPF64[$comp_temp>>3];
       $701 = +HEAPF64[$comp_temp1>>3];
       $702 = $700 + $701;
       HEAPF64[$comp_temp>>3] = $702;
       $703 = ((($comp_temp)) + 8|0);
       $704 = +HEAPF64[$703>>3];
       $705 = ((($comp_temp1)) + 8|0);
       $706 = +HEAPF64[$705>>3];
       $707 = $704 + $706;
       $708 = ((($comp_temp)) + 8|0);
       HEAPF64[$708>>3] = $707;
       $709 = $irow;
       $710 = $3;
       $711 = (($710) + ($709<<4)|0);
       $712 = +HEAPF64[$711>>3];
       $713 = +HEAPF64[$comp_temp>>3];
       $714 = $712 - $713;
       $715 = $irow;
       $716 = $3;
       $717 = (($716) + ($715<<4)|0);
       HEAPF64[$717>>3] = $714;
       $718 = $irow;
       $719 = $3;
       $720 = (($719) + ($718<<4)|0);
       $721 = ((($720)) + 8|0);
       $722 = +HEAPF64[$721>>3];
       $723 = ((($comp_temp)) + 8|0);
       $724 = +HEAPF64[$723>>3];
       $725 = $722 - $724;
       $726 = $irow;
       $727 = $3;
       $728 = (($727) + ($726<<4)|0);
       $729 = ((($728)) + 8|0);
       HEAPF64[$729>>3] = $725;
       $730 = $i;
       $731 = (($730) + 1)|0;
       $i = $731;
      }
     }
    }
    $732 = $kfnz;
    $733 = $fst_col;
    $734 = (($732) - ($733))|0;
    $no_zeros = $734;
    $735 = $lptr;
    $736 = $no_zeros;
    $737 = (($735) + ($736))|0;
    $isub = $737;
    $i = 0;
    while(1) {
     $738 = $i;
     $739 = $segsze;
     $740 = ($738|0)<($739|0);
     if (!($740)) {
      break;
     }
     $741 = $isub;
     $742 = $lsub;
     $743 = (($742) + ($741<<2)|0);
     $744 = HEAP32[$743>>2]|0;
     $irow = $744;
     $745 = $i;
     $746 = $4;
     $747 = (($746) + ($745<<4)|0);
     $748 = $irow;
     $749 = $3;
     $750 = (($749) + ($748<<4)|0);
     ;HEAP32[$747>>2]=HEAP32[$750>>2]|0;HEAP32[$747+4>>2]=HEAP32[$750+4>>2]|0;HEAP32[$747+8>>2]=HEAP32[$750+8>>2]|0;HEAP32[$747+12>>2]=HEAP32[$750+12>>2]|0;
     $751 = $isub;
     $752 = (($751) + 1)|0;
     $isub = $752;
     $753 = $i;
     $754 = (($753) + 1)|0;
     $i = $754;
    }
    $755 = $nsupr;
    $756 = $no_zeros;
    $757 = Math_imul($755, $756)|0;
    $758 = $no_zeros;
    $759 = (($757) + ($758))|0;
    $760 = $luptr;
    $761 = (($760) + ($759))|0;
    $luptr = $761;
    $762 = $nsupr;
    $763 = $segsze;
    $764 = $luptr;
    $765 = $lusup;
    $766 = (($765) + ($764<<4)|0);
    $767 = $4;
    _zlsolve($762,$763,$766,$767);
    $768 = $segsze;
    $769 = $luptr;
    $770 = (($769) + ($768))|0;
    $luptr = $770;
    $771 = $segsze;
    $772 = $4;
    $773 = (($772) + ($771<<4)|0);
    $tempv1 = $773;
    $774 = $nsupr;
    $775 = $nrow;
    $776 = $segsze;
    $777 = $luptr;
    $778 = $lusup;
    $779 = (($778) + ($777<<4)|0);
    $780 = $4;
    $781 = $tempv1;
    _zmatvec($774,$775,$776,$779,$780,$781);
    $782 = $lptr;
    $783 = $no_zeros;
    $784 = (($782) + ($783))|0;
    $isub = $784;
    $i = 0;
    while(1) {
     $785 = $i;
     $786 = $segsze;
     $787 = ($785|0)<($786|0);
     if (!($787)) {
      break;
     }
     $788 = $isub;
     $789 = $lsub;
     $790 = (($789) + ($788<<2)|0);
     $791 = HEAP32[$790>>2]|0;
     $irow = $791;
     $792 = $irow;
     $793 = $3;
     $794 = (($793) + ($792<<4)|0);
     $795 = $i;
     $796 = $4;
     $797 = (($796) + ($795<<4)|0);
     ;HEAP32[$794>>2]=HEAP32[$797>>2]|0;HEAP32[$794+4>>2]=HEAP32[$797+4>>2]|0;HEAP32[$794+8>>2]=HEAP32[$797+8>>2]|0;HEAP32[$794+12>>2]=HEAP32[$797+12>>2]|0;
     $798 = $i;
     $799 = $4;
     $800 = (($799) + ($798<<4)|0);
     ;HEAP32[$800>>2]=HEAP32[$zero>>2]|0;HEAP32[$800+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$800+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$800+12>>2]=HEAP32[$zero+12>>2]|0;
     $801 = $isub;
     $802 = (($801) + 1)|0;
     $isub = $802;
     $803 = $i;
     $804 = (($803) + 1)|0;
     $i = $804;
    }
    $i = 0;
    while(1) {
     $805 = $i;
     $806 = $nrow;
     $807 = ($805|0)<($806|0);
     if (!($807)) {
      break L4;
     }
     $808 = $isub;
     $809 = $lsub;
     $810 = (($809) + ($808<<2)|0);
     $811 = HEAP32[$810>>2]|0;
     $irow = $811;
     $812 = $irow;
     $813 = $3;
     $814 = (($813) + ($812<<4)|0);
     $815 = +HEAPF64[$814>>3];
     $816 = $i;
     $817 = $tempv1;
     $818 = (($817) + ($816<<4)|0);
     $819 = +HEAPF64[$818>>3];
     $820 = $815 - $819;
     $821 = $irow;
     $822 = $3;
     $823 = (($822) + ($821<<4)|0);
     HEAPF64[$823>>3] = $820;
     $824 = $irow;
     $825 = $3;
     $826 = (($825) + ($824<<4)|0);
     $827 = ((($826)) + 8|0);
     $828 = +HEAPF64[$827>>3];
     $829 = $i;
     $830 = $tempv1;
     $831 = (($830) + ($829<<4)|0);
     $832 = ((($831)) + 8|0);
     $833 = +HEAPF64[$832>>3];
     $834 = $828 - $833;
     $835 = $irow;
     $836 = $3;
     $837 = (($836) + ($835<<4)|0);
     $838 = ((($837)) + 8|0);
     HEAPF64[$838>>3] = $834;
     $839 = $i;
     $840 = $tempv1;
     $841 = (($840) + ($839<<4)|0);
     ;HEAP32[$841>>2]=HEAP32[$zero>>2]|0;HEAP32[$841+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$841+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$841+12>>2]=HEAP32[$zero+12>>2]|0;
     $842 = $isub;
     $843 = (($842) + 1)|0;
     $isub = $843;
     $844 = $i;
     $845 = (($844) + 1)|0;
     $i = $845;
    }
   }
  } while(0);
  $846 = $ksub;
  $847 = (($846) + 1)|0;
  $ksub = $847;
 }
 $848 = $1;
 $849 = $xlusup;
 $850 = (($849) + ($848<<2)|0);
 $851 = HEAP32[$850>>2]|0;
 $nextlu = $851;
 $852 = $jsupno;
 $853 = $xsup;
 $854 = (($853) + ($852<<2)|0);
 $855 = HEAP32[$854>>2]|0;
 $fsupc = $855;
 $856 = $nextlu;
 $857 = $fsupc;
 $858 = (($857) + 1)|0;
 $859 = $xlsub;
 $860 = (($859) + ($858<<2)|0);
 $861 = HEAP32[$860>>2]|0;
 $862 = (($856) + ($861))|0;
 $863 = $fsupc;
 $864 = $xlsub;
 $865 = (($864) + ($863<<2)|0);
 $866 = HEAP32[$865>>2]|0;
 $867 = (($862) - ($866))|0;
 $new_next = $867;
 while(1) {
  $868 = $new_next;
  $869 = HEAP32[$nzlumax>>2]|0;
  $870 = ($868|0)>($869|0);
  if (!($870)) {
   break;
  }
  $871 = $1;
  $872 = $nextlu;
  $873 = $8;
  $874 = (_zLUMemXpand($871,$872,0,$nzlumax,$873)|0);
  $mem_error = $874;
  $875 = ($874|0)!=(0);
  if ($875) {
   label = 29;
   break;
  }
  $877 = $8;
  $878 = ((($877)) + 16|0);
  $879 = HEAP32[$878>>2]|0;
  $lusup = $879;
  $880 = $8;
  $881 = ((($880)) + 8|0);
  $882 = HEAP32[$881>>2]|0;
  $lsub = $882;
 }
 if ((label|0) == 29) {
  $876 = $mem_error;
  $0 = $876;
  $1042 = $0;
  STACKTOP = sp;return ($1042|0);
 }
 $883 = $fsupc;
 $884 = $xlsub;
 $885 = (($884) + ($883<<2)|0);
 $886 = HEAP32[$885>>2]|0;
 $isub = $886;
 while(1) {
  $887 = $isub;
  $888 = $fsupc;
  $889 = (($888) + 1)|0;
  $890 = $xlsub;
  $891 = (($890) + ($889<<2)|0);
  $892 = HEAP32[$891>>2]|0;
  $893 = ($887|0)<($892|0);
  if (!($893)) {
   break;
  }
  $894 = $isub;
  $895 = $lsub;
  $896 = (($895) + ($894<<2)|0);
  $897 = HEAP32[$896>>2]|0;
  $irow = $897;
  $898 = $nextlu;
  $899 = $lusup;
  $900 = (($899) + ($898<<4)|0);
  $901 = $irow;
  $902 = $3;
  $903 = (($902) + ($901<<4)|0);
  ;HEAP32[$900>>2]=HEAP32[$903>>2]|0;HEAP32[$900+4>>2]=HEAP32[$903+4>>2]|0;HEAP32[$900+8>>2]=HEAP32[$903+8>>2]|0;HEAP32[$900+12>>2]=HEAP32[$903+12>>2]|0;
  $904 = $irow;
  $905 = $3;
  $906 = (($905) + ($904<<4)|0);
  ;HEAP32[$906>>2]=HEAP32[$zero>>2]|0;HEAP32[$906+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$906+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$906+12>>2]=HEAP32[$zero+12>>2]|0;
  $907 = $nextlu;
  $908 = (($907) + 1)|0;
  $nextlu = $908;
  $909 = $isub;
  $910 = (($909) + 1)|0;
  $isub = $910;
 }
 $911 = $nextlu;
 $912 = $jcolp1;
 $913 = $xlusup;
 $914 = (($913) + ($912<<2)|0);
 HEAP32[$914>>2] = $911;
 $915 = $fsupc;
 $916 = $7;
 $917 = ($915|0)>($916|0);
 $918 = $fsupc;
 $919 = $7;
 $920 = $917 ? $918 : $919;
 $fst_col = $920;
 $921 = $fst_col;
 $922 = $1;
 $923 = ($921|0)<($922|0);
 L49: do {
  if ($923) {
   $924 = $fst_col;
   $925 = $fsupc;
   $926 = (($924) - ($925))|0;
   $d_fsupc = $926;
   $927 = $fsupc;
   $928 = $xlsub;
   $929 = (($928) + ($927<<2)|0);
   $930 = HEAP32[$929>>2]|0;
   $931 = $d_fsupc;
   $932 = (($930) + ($931))|0;
   $lptr = $932;
   $933 = $fst_col;
   $934 = $xlusup;
   $935 = (($934) + ($933<<2)|0);
   $936 = HEAP32[$935>>2]|0;
   $937 = $d_fsupc;
   $938 = (($936) + ($937))|0;
   $luptr = $938;
   $939 = $fsupc;
   $940 = (($939) + 1)|0;
   $941 = $xlsub;
   $942 = (($941) + ($940<<2)|0);
   $943 = HEAP32[$942>>2]|0;
   $944 = $fsupc;
   $945 = $xlsub;
   $946 = (($945) + ($944<<2)|0);
   $947 = HEAP32[$946>>2]|0;
   $948 = (($943) - ($947))|0;
   $nsupr = $948;
   $949 = $1;
   $950 = $fst_col;
   $951 = (($949) - ($950))|0;
   $nsupc = $951;
   $952 = $nsupr;
   $953 = $d_fsupc;
   $954 = (($952) - ($953))|0;
   $955 = $nsupc;
   $956 = (($954) - ($955))|0;
   $nrow = $956;
   $957 = $1;
   $958 = $xlusup;
   $959 = (($958) + ($957<<2)|0);
   $960 = HEAP32[$959>>2]|0;
   $961 = $d_fsupc;
   $962 = (($960) + ($961))|0;
   $ufirst = $962;
   $963 = $nsupc;
   $964 = $963<<2;
   $965 = $nsupc;
   $966 = (($965) - 1)|0;
   $967 = Math_imul($964, $966)|0;
   $968 = (+($967|0));
   $969 = $ops;
   $970 = ((($969)) + 52|0);
   $971 = +HEAPF32[$970>>2];
   $972 = $971 + $968;
   HEAPF32[$970>>2] = $972;
   $973 = $nrow;
   $974 = $973<<3;
   $975 = $nsupc;
   $976 = Math_imul($974, $975)|0;
   $977 = (+($976|0));
   $978 = $ops;
   $979 = ((($978)) + 56|0);
   $980 = +HEAPF32[$979>>2];
   $981 = $980 + $977;
   HEAPF32[$979>>2] = $981;
   $982 = $nsupr;
   $983 = $nsupc;
   $984 = $luptr;
   $985 = $lusup;
   $986 = (($985) + ($984<<4)|0);
   $987 = $ufirst;
   $988 = $lusup;
   $989 = (($988) + ($987<<4)|0);
   _zlsolve($982,$983,$986,$989);
   $990 = $nsupr;
   $991 = $nrow;
   $992 = $nsupc;
   $993 = $luptr;
   $994 = $nsupc;
   $995 = (($993) + ($994))|0;
   $996 = $lusup;
   $997 = (($996) + ($995<<4)|0);
   $998 = $ufirst;
   $999 = $lusup;
   $1000 = (($999) + ($998<<4)|0);
   $1001 = $4;
   _zmatvec($990,$991,$992,$997,$1000,$1001);
   $1002 = $ufirst;
   $1003 = $nsupc;
   $1004 = (($1002) + ($1003))|0;
   $isub = $1004;
   $i = 0;
   while(1) {
    $1005 = $i;
    $1006 = $nrow;
    $1007 = ($1005|0)<($1006|0);
    if (!($1007)) {
     break L49;
    }
    $1008 = $isub;
    $1009 = $lusup;
    $1010 = (($1009) + ($1008<<4)|0);
    $1011 = +HEAPF64[$1010>>3];
    $1012 = $i;
    $1013 = $4;
    $1014 = (($1013) + ($1012<<4)|0);
    $1015 = +HEAPF64[$1014>>3];
    $1016 = $1011 - $1015;
    $1017 = $isub;
    $1018 = $lusup;
    $1019 = (($1018) + ($1017<<4)|0);
    HEAPF64[$1019>>3] = $1016;
    $1020 = $isub;
    $1021 = $lusup;
    $1022 = (($1021) + ($1020<<4)|0);
    $1023 = ((($1022)) + 8|0);
    $1024 = +HEAPF64[$1023>>3];
    $1025 = $i;
    $1026 = $4;
    $1027 = (($1026) + ($1025<<4)|0);
    $1028 = ((($1027)) + 8|0);
    $1029 = +HEAPF64[$1028>>3];
    $1030 = $1024 - $1029;
    $1031 = $isub;
    $1032 = $lusup;
    $1033 = (($1032) + ($1031<<4)|0);
    $1034 = ((($1033)) + 8|0);
    HEAPF64[$1034>>3] = $1030;
    $1035 = $i;
    $1036 = $4;
    $1037 = (($1036) + ($1035<<4)|0);
    ;HEAP32[$1037>>2]=HEAP32[$zero>>2]|0;HEAP32[$1037+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$1037+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$1037+12>>2]=HEAP32[$zero+12>>2]|0;
    $1038 = $isub;
    $1039 = (($1038) + 1)|0;
    $isub = $1039;
    $1040 = $i;
    $1041 = (($1040) + 1)|0;
    $i = $1041;
   }
  }
 } while(0);
 $0 = 0;
 $1042 = $0;
 STACKTOP = sp;return ($1042|0);
}
function _zpivotL($jcol,$u,$usepr,$perm_r,$iperm_r,$iperm_c,$pivrow,$Glu,$stat) {
 $jcol = $jcol|0;
 $u = +$u;
 $usepr = $usepr|0;
 $perm_r = $perm_r|0;
 $iperm_r = $iperm_r|0;
 $iperm_c = $iperm_c|0;
 $pivrow = $pivrow|0;
 $Glu = $Glu|0;
 $stat = $stat|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0.0, $127 = 0.0, $128 = 0.0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0.0, $136 = 0.0, $137 = 0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0.0;
 var $152 = 0.0, $153 = 0, $154 = 0.0, $155 = 0.0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0.0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0.0, $221 = 0, $222 = 0, $223 = 0.0;
 var $224 = 0.0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0.0, $237 = 0.0, $238 = 0.0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0.0, $244 = 0, $245 = 0.0, $246 = 0.0, $247 = 0.0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0.0, $253 = 0.0, $254 = 0.0, $255 = 0, $256 = 0, $257 = 0, $258 = 0.0, $259 = 0, $26 = 0;
 var $260 = 0.0, $261 = 0.0, $262 = 0.0, $263 = 0.0, $264 = 0, $265 = 0, $266 = 0, $267 = 0.0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0.0;
 var $86 = 0.0, $87 = 0.0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ci = 0.0, $cr = 0.0, $diag = 0, $diagind = 0, $fsupc = 0;
 var $icol = 0, $isub = 0, $itemp = 0, $k = 0, $lptr = 0, $lsub = 0, $lsub_ptr = 0, $lu_col_ptr = 0, $lu_sup_ptr = 0, $lusup = 0, $nsupc = 0, $nsupr = 0, $old_pivptr = 0, $one = 0, $ops = 0, $pivmax = 0.0, $pivptr = 0, $rtemp = 0.0, $temp = 0, $thresh = 0.0;
 var $xlsub = 0, $xlusup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $one = sp + 56|0;
 $temp = sp + 16|0;
 $1 = $jcol;
 $2 = $u;
 $3 = $usepr;
 $4 = $perm_r;
 $5 = $iperm_r;
 $6 = $iperm_c;
 $7 = $pivrow;
 $8 = $Glu;
 $9 = $stat;
 ;HEAP32[$one>>2]=HEAP32[2504>>2]|0;HEAP32[$one+4>>2]=HEAP32[2504+4>>2]|0;HEAP32[$one+8>>2]=HEAP32[2504+8>>2]|0;HEAP32[$one+12>>2]=HEAP32[2504+12>>2]|0;
 $10 = $9;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $ops = $12;
 $13 = $8;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $lsub = $15;
 $16 = $8;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $xlsub = $18;
 $19 = $8;
 $20 = ((($19)) + 16|0);
 $21 = HEAP32[$20>>2]|0;
 $lusup = $21;
 $22 = $8;
 $23 = ((($22)) + 20|0);
 $24 = HEAP32[$23>>2]|0;
 $xlusup = $24;
 $25 = $1;
 $26 = $8;
 $27 = ((($26)) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($25<<2)|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $8;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + ($30<<2)|0);
 $34 = HEAP32[$33>>2]|0;
 $fsupc = $34;
 $35 = $1;
 $36 = $fsupc;
 $37 = (($35) - ($36))|0;
 $nsupc = $37;
 $38 = $fsupc;
 $39 = $xlsub;
 $40 = (($39) + ($38<<2)|0);
 $41 = HEAP32[$40>>2]|0;
 $lptr = $41;
 $42 = $fsupc;
 $43 = (($42) + 1)|0;
 $44 = $xlsub;
 $45 = (($44) + ($43<<2)|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = $lptr;
 $48 = (($46) - ($47))|0;
 $nsupr = $48;
 $49 = $fsupc;
 $50 = $xlusup;
 $51 = (($50) + ($49<<2)|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = $lusup;
 $54 = (($53) + ($52<<4)|0);
 $lu_sup_ptr = $54;
 $55 = $1;
 $56 = $xlusup;
 $57 = (($56) + ($55<<2)|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $lusup;
 $60 = (($59) + ($58<<4)|0);
 $lu_col_ptr = $60;
 $61 = $lptr;
 $62 = $lsub;
 $63 = (($62) + ($61<<2)|0);
 $lsub_ptr = $63;
 $64 = $3;
 $65 = HEAP32[$64>>2]|0;
 $66 = ($65|0)!=(0);
 if ($66) {
  $67 = $1;
  $68 = $5;
  $69 = (($68) + ($67<<2)|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $7;
  HEAP32[$71>>2] = $70;
 }
 $72 = $1;
 $73 = $6;
 $74 = (($73) + ($72<<2)|0);
 $75 = HEAP32[$74>>2]|0;
 $diagind = $75;
 $pivmax = 0.0;
 $76 = $nsupc;
 $pivptr = $76;
 $diag = -1;
 $77 = $nsupc;
 $old_pivptr = $77;
 $78 = $nsupc;
 $isub = $78;
 while(1) {
  $79 = $isub;
  $80 = $nsupr;
  $81 = ($79|0)<($80|0);
  if (!($81)) {
   break;
  }
  $82 = $isub;
  $83 = $lu_col_ptr;
  $84 = (($83) + ($82<<4)|0);
  $85 = (+_z_abs1($84));
  $rtemp = $85;
  $86 = $rtemp;
  $87 = $pivmax;
  $88 = $86 > $87;
  if ($88) {
   $89 = $rtemp;
   $pivmax = $89;
   $90 = $isub;
   $pivptr = $90;
  }
  $91 = $3;
  $92 = HEAP32[$91>>2]|0;
  $93 = ($92|0)!=(0);
  if ($93) {
   $94 = $isub;
   $95 = $lsub_ptr;
   $96 = (($95) + ($94<<2)|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = $7;
   $99 = HEAP32[$98>>2]|0;
   $100 = ($97|0)==($99|0);
   if ($100) {
    $101 = $isub;
    $old_pivptr = $101;
   }
  }
  $102 = $isub;
  $103 = $lsub_ptr;
  $104 = (($103) + ($102<<2)|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = $diagind;
  $107 = ($105|0)==($106|0);
  if ($107) {
   $108 = $isub;
   $diag = $108;
  }
  $109 = $isub;
  $110 = (($109) + 1)|0;
  $isub = $110;
 }
 $111 = $pivmax;
 $112 = $111 == 0.0;
 if ($112) {
  $113 = $pivptr;
  $114 = $lsub_ptr;
  $115 = (($114) + ($113<<2)|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = $7;
  HEAP32[$117>>2] = $116;
  $118 = $1;
  $119 = $7;
  $120 = HEAP32[$119>>2]|0;
  $121 = $4;
  $122 = (($121) + ($120<<2)|0);
  HEAP32[$122>>2] = $118;
  $123 = $3;
  HEAP32[$123>>2] = 0;
  $124 = $1;
  $125 = (($124) + 1)|0;
  $0 = $125;
  $274 = $0;
  STACKTOP = sp;return ($274|0);
 }
 $126 = $2;
 $127 = $pivmax;
 $128 = $126 * $127;
 $thresh = $128;
 $129 = $3;
 $130 = HEAP32[$129>>2]|0;
 $131 = ($130|0)!=(0);
 do {
  if ($131) {
   $132 = $old_pivptr;
   $133 = $lu_col_ptr;
   $134 = (($133) + ($132<<4)|0);
   $135 = (+_z_abs1($134));
   $rtemp = $135;
   $136 = $rtemp;
   $137 = $136 != 0.0;
   if ($137) {
    $138 = $rtemp;
    $139 = $thresh;
    $140 = $138 >= $139;
    if ($140) {
     $141 = $old_pivptr;
     $pivptr = $141;
     break;
    }
   }
   $142 = $3;
   HEAP32[$142>>2] = 0;
  }
 } while(0);
 $143 = $3;
 $144 = HEAP32[$143>>2]|0;
 $145 = ($144|0)==(0);
 if ($145) {
  $146 = $diag;
  $147 = ($146|0)>=(0);
  if ($147) {
   $148 = $diag;
   $149 = $lu_col_ptr;
   $150 = (($149) + ($148<<4)|0);
   $151 = (+_z_abs1($150));
   $rtemp = $151;
   $152 = $rtemp;
   $153 = $152 != 0.0;
   if ($153) {
    $154 = $rtemp;
    $155 = $thresh;
    $156 = $154 >= $155;
    if ($156) {
     $157 = $diag;
     $pivptr = $157;
    }
   }
  }
  $158 = $pivptr;
  $159 = $lsub_ptr;
  $160 = (($159) + ($158<<2)|0);
  $161 = HEAP32[$160>>2]|0;
  $162 = $7;
  HEAP32[$162>>2] = $161;
 }
 $163 = $1;
 $164 = $7;
 $165 = HEAP32[$164>>2]|0;
 $166 = $4;
 $167 = (($166) + ($165<<2)|0);
 HEAP32[$167>>2] = $163;
 $168 = $pivptr;
 $169 = $nsupc;
 $170 = ($168|0)!=($169|0);
 L37: do {
  if ($170) {
   $171 = $pivptr;
   $172 = $lsub_ptr;
   $173 = (($172) + ($171<<2)|0);
   $174 = HEAP32[$173>>2]|0;
   $itemp = $174;
   $175 = $nsupc;
   $176 = $lsub_ptr;
   $177 = (($176) + ($175<<2)|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = $pivptr;
   $180 = $lsub_ptr;
   $181 = (($180) + ($179<<2)|0);
   HEAP32[$181>>2] = $178;
   $182 = $itemp;
   $183 = $nsupc;
   $184 = $lsub_ptr;
   $185 = (($184) + ($183<<2)|0);
   HEAP32[$185>>2] = $182;
   $icol = 0;
   while(1) {
    $186 = $icol;
    $187 = $nsupc;
    $188 = ($186|0)<=($187|0);
    if (!($188)) {
     break L37;
    }
    $189 = $pivptr;
    $190 = $icol;
    $191 = $nsupr;
    $192 = Math_imul($190, $191)|0;
    $193 = (($189) + ($192))|0;
    $itemp = $193;
    $194 = $itemp;
    $195 = $lu_sup_ptr;
    $196 = (($195) + ($194<<4)|0);
    ;HEAP32[$temp>>2]=HEAP32[$196>>2]|0;HEAP32[$temp+4>>2]=HEAP32[$196+4>>2]|0;HEAP32[$temp+8>>2]=HEAP32[$196+8>>2]|0;HEAP32[$temp+12>>2]=HEAP32[$196+12>>2]|0;
    $197 = $itemp;
    $198 = $lu_sup_ptr;
    $199 = (($198) + ($197<<4)|0);
    $200 = $nsupc;
    $201 = $icol;
    $202 = $nsupr;
    $203 = Math_imul($201, $202)|0;
    $204 = (($200) + ($203))|0;
    $205 = $lu_sup_ptr;
    $206 = (($205) + ($204<<4)|0);
    ;HEAP32[$199>>2]=HEAP32[$206>>2]|0;HEAP32[$199+4>>2]=HEAP32[$206+4>>2]|0;HEAP32[$199+8>>2]=HEAP32[$206+8>>2]|0;HEAP32[$199+12>>2]=HEAP32[$206+12>>2]|0;
    $207 = $nsupc;
    $208 = $icol;
    $209 = $nsupr;
    $210 = Math_imul($208, $209)|0;
    $211 = (($207) + ($210))|0;
    $212 = $lu_sup_ptr;
    $213 = (($212) + ($211<<4)|0);
    ;HEAP32[$213>>2]=HEAP32[$temp>>2]|0;HEAP32[$213+4>>2]=HEAP32[$temp+4>>2]|0;HEAP32[$213+8>>2]=HEAP32[$temp+8>>2]|0;HEAP32[$213+12>>2]=HEAP32[$temp+12>>2]|0;
    $214 = $icol;
    $215 = (($214) + 1)|0;
    $icol = $215;
   }
  }
 } while(0);
 $216 = $nsupr;
 $217 = $nsupc;
 $218 = (($216) - ($217))|0;
 $219 = ($218*10)|0;
 $220 = (+($219|0));
 $221 = $ops;
 $222 = ((($221)) + 28|0);
 $223 = +HEAPF32[$222>>2];
 $224 = $223 + $220;
 HEAPF32[$222>>2] = $224;
 $225 = $nsupc;
 $226 = $lu_col_ptr;
 $227 = (($226) + ($225<<4)|0);
 _z_div($temp,$one,$227);
 $228 = $nsupc;
 $229 = (($228) + 1)|0;
 $k = $229;
 while(1) {
  $230 = $k;
  $231 = $nsupr;
  $232 = ($230|0)<($231|0);
  if (!($232)) {
   break;
  }
  $233 = $k;
  $234 = $lu_col_ptr;
  $235 = (($234) + ($233<<4)|0);
  $236 = +HEAPF64[$235>>3];
  $237 = +HEAPF64[$temp>>3];
  $238 = $236 * $237;
  $239 = $k;
  $240 = $lu_col_ptr;
  $241 = (($240) + ($239<<4)|0);
  $242 = ((($241)) + 8|0);
  $243 = +HEAPF64[$242>>3];
  $244 = ((($temp)) + 8|0);
  $245 = +HEAPF64[$244>>3];
  $246 = $243 * $245;
  $247 = $238 - $246;
  $cr = $247;
  $248 = $k;
  $249 = $lu_col_ptr;
  $250 = (($249) + ($248<<4)|0);
  $251 = ((($250)) + 8|0);
  $252 = +HEAPF64[$251>>3];
  $253 = +HEAPF64[$temp>>3];
  $254 = $252 * $253;
  $255 = $k;
  $256 = $lu_col_ptr;
  $257 = (($256) + ($255<<4)|0);
  $258 = +HEAPF64[$257>>3];
  $259 = ((($temp)) + 8|0);
  $260 = +HEAPF64[$259>>3];
  $261 = $258 * $260;
  $262 = $254 + $261;
  $ci = $262;
  $263 = $cr;
  $264 = $k;
  $265 = $lu_col_ptr;
  $266 = (($265) + ($264<<4)|0);
  HEAPF64[$266>>3] = $263;
  $267 = $ci;
  $268 = $k;
  $269 = $lu_col_ptr;
  $270 = (($269) + ($268<<4)|0);
  $271 = ((($270)) + 8|0);
  HEAPF64[$271>>3] = $267;
  $272 = $k;
  $273 = (($272) + 1)|0;
  $k = $273;
 }
 $0 = 0;
 $274 = $0;
 STACKTOP = sp;return ($274|0);
}
function _zpruneL($jcol,$perm_r,$pivrow,$nseg,$segrep,$repfnz,$xprune,$Glu) {
 $jcol = $jcol|0;
 $perm_r = $perm_r|0;
 $pivrow = $pivrow|0;
 $nseg = $nseg|0;
 $segrep = $segrep|0;
 $repfnz = $repfnz|0;
 $xprune = $xprune|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $do_prune = 0, $i = 0, $irep = 0, $irep1 = 0, $jsupno = 0, $kmax = 0, $kmin = 0, $krow = 0, $ktemp = 0, $lsub = 0, $lusup = 0, $maxloc = 0, $minloc = 0, $movnum = 0, $supno = 0, $utemp = 0, $xlsub = 0, $xlusup = 0;
 var $xsup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $utemp = sp;
 $0 = $jcol;
 $1 = $perm_r;
 $2 = $pivrow;
 $3 = $nseg;
 $4 = $segrep;
 $5 = $repfnz;
 $6 = $xprune;
 $7 = $Glu;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $xsup = $9;
 $10 = $7;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $supno = $12;
 $13 = $7;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $lsub = $15;
 $16 = $7;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $xlsub = $18;
 $19 = $7;
 $20 = ((($19)) + 16|0);
 $21 = HEAP32[$20>>2]|0;
 $lusup = $21;
 $22 = $7;
 $23 = ((($22)) + 20|0);
 $24 = HEAP32[$23>>2]|0;
 $xlusup = $24;
 $25 = $0;
 $26 = $supno;
 $27 = (($26) + ($25<<2)|0);
 $28 = HEAP32[$27>>2]|0;
 $jsupno = $28;
 $i = 0;
 while(1) {
  $29 = $i;
  $30 = $3;
  $31 = ($29|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = $i;
  $33 = $4;
  $34 = (($33) + ($32<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $irep = $35;
  $36 = $irep;
  $37 = (($36) + 1)|0;
  $irep1 = $37;
  $do_prune = 0;
  $38 = $irep;
  $39 = $5;
  $40 = (($39) + ($38<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ($41|0)==(-1);
  if (!($42)) {
   $43 = $irep;
   $44 = $supno;
   $45 = (($44) + ($43<<2)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = $irep1;
   $48 = $supno;
   $49 = (($48) + ($47<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = ($46|0)==($50|0);
   if (!($51)) {
    $52 = $irep;
    $53 = $supno;
    $54 = (($53) + ($52<<2)|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = $jsupno;
    $57 = ($55|0)!=($56|0);
    if ($57) {
     $58 = $irep;
     $59 = $6;
     $60 = (($59) + ($58<<2)|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = $irep1;
     $63 = $xlsub;
     $64 = (($63) + ($62<<2)|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = ($61|0)>=($65|0);
     L8: do {
      if ($66) {
       $67 = $irep;
       $68 = $xlsub;
       $69 = (($68) + ($67<<2)|0);
       $70 = HEAP32[$69>>2]|0;
       $kmin = $70;
       $71 = $irep1;
       $72 = $xlsub;
       $73 = (($72) + ($71<<2)|0);
       $74 = HEAP32[$73>>2]|0;
       $75 = (($74) - 1)|0;
       $kmax = $75;
       $76 = $kmin;
       $krow = $76;
       while(1) {
        $77 = $krow;
        $78 = $kmax;
        $79 = ($77|0)<=($78|0);
        if (!($79)) {
         break L8;
        }
        $80 = $krow;
        $81 = $lsub;
        $82 = (($81) + ($80<<2)|0);
        $83 = HEAP32[$82>>2]|0;
        $84 = $2;
        $85 = ($83|0)==($84|0);
        if ($85) {
         break;
        }
        $86 = $krow;
        $87 = (($86) + 1)|0;
        $krow = $87;
       }
       $do_prune = 1;
      }
     } while(0);
     $88 = $do_prune;
     $89 = ($88|0)!=(0);
     if ($89) {
      $movnum = 0;
      $90 = $irep;
      $91 = $irep;
      $92 = $supno;
      $93 = (($92) + ($91<<2)|0);
      $94 = HEAP32[$93>>2]|0;
      $95 = $xsup;
      $96 = (($95) + ($94<<2)|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = ($90|0)==($97|0);
      if ($98) {
       $movnum = 1;
      }
      while(1) {
       $99 = $kmin;
       $100 = $kmax;
       $101 = ($99|0)<=($100|0);
       if (!($101)) {
        break;
       }
       $102 = $kmax;
       $103 = $lsub;
       $104 = (($103) + ($102<<2)|0);
       $105 = HEAP32[$104>>2]|0;
       $106 = $1;
       $107 = (($106) + ($105<<2)|0);
       $108 = HEAP32[$107>>2]|0;
       $109 = ($108|0)==(-1);
       if ($109) {
        $110 = $kmax;
        $111 = (($110) + -1)|0;
        $kmax = $111;
        continue;
       }
       $112 = $kmin;
       $113 = $lsub;
       $114 = (($113) + ($112<<2)|0);
       $115 = HEAP32[$114>>2]|0;
       $116 = $1;
       $117 = (($116) + ($115<<2)|0);
       $118 = HEAP32[$117>>2]|0;
       $119 = ($118|0)!=(-1);
       $120 = $kmin;
       if ($119) {
        $121 = (($120) + 1)|0;
        $kmin = $121;
        continue;
       }
       $122 = $lsub;
       $123 = (($122) + ($120<<2)|0);
       $124 = HEAP32[$123>>2]|0;
       $ktemp = $124;
       $125 = $kmax;
       $126 = $lsub;
       $127 = (($126) + ($125<<2)|0);
       $128 = HEAP32[$127>>2]|0;
       $129 = $kmin;
       $130 = $lsub;
       $131 = (($130) + ($129<<2)|0);
       HEAP32[$131>>2] = $128;
       $132 = $ktemp;
       $133 = $kmax;
       $134 = $lsub;
       $135 = (($134) + ($133<<2)|0);
       HEAP32[$135>>2] = $132;
       $136 = $movnum;
       $137 = ($136|0)!=(0);
       if ($137) {
        $138 = $irep;
        $139 = $xlusup;
        $140 = (($139) + ($138<<2)|0);
        $141 = HEAP32[$140>>2]|0;
        $142 = $kmin;
        $143 = $irep;
        $144 = $xlsub;
        $145 = (($144) + ($143<<2)|0);
        $146 = HEAP32[$145>>2]|0;
        $147 = (($142) - ($146))|0;
        $148 = (($141) + ($147))|0;
        $minloc = $148;
        $149 = $irep;
        $150 = $xlusup;
        $151 = (($150) + ($149<<2)|0);
        $152 = HEAP32[$151>>2]|0;
        $153 = $kmax;
        $154 = $irep;
        $155 = $xlsub;
        $156 = (($155) + ($154<<2)|0);
        $157 = HEAP32[$156>>2]|0;
        $158 = (($153) - ($157))|0;
        $159 = (($152) + ($158))|0;
        $maxloc = $159;
        $160 = $minloc;
        $161 = $lusup;
        $162 = (($161) + ($160<<4)|0);
        ;HEAP32[$utemp>>2]=HEAP32[$162>>2]|0;HEAP32[$utemp+4>>2]=HEAP32[$162+4>>2]|0;HEAP32[$utemp+8>>2]=HEAP32[$162+8>>2]|0;HEAP32[$utemp+12>>2]=HEAP32[$162+12>>2]|0;
        $163 = $minloc;
        $164 = $lusup;
        $165 = (($164) + ($163<<4)|0);
        $166 = $maxloc;
        $167 = $lusup;
        $168 = (($167) + ($166<<4)|0);
        ;HEAP32[$165>>2]=HEAP32[$168>>2]|0;HEAP32[$165+4>>2]=HEAP32[$168+4>>2]|0;HEAP32[$165+8>>2]=HEAP32[$168+8>>2]|0;HEAP32[$165+12>>2]=HEAP32[$168+12>>2]|0;
        $169 = $maxloc;
        $170 = $lusup;
        $171 = (($170) + ($169<<4)|0);
        ;HEAP32[$171>>2]=HEAP32[$utemp>>2]|0;HEAP32[$171+4>>2]=HEAP32[$utemp+4>>2]|0;HEAP32[$171+8>>2]=HEAP32[$utemp+8>>2]|0;HEAP32[$171+12>>2]=HEAP32[$utemp+12>>2]|0;
       }
       $172 = $kmin;
       $173 = (($172) + 1)|0;
       $kmin = $173;
       $174 = $kmax;
       $175 = (($174) + -1)|0;
       $kmax = $175;
      }
      $176 = $kmin;
      $177 = $irep;
      $178 = $6;
      $179 = (($178) + ($177<<2)|0);
      HEAP32[$179>>2] = $176;
     }
    }
   }
  }
  $180 = $i;
  $181 = (($180) + 1)|0;
  $i = $181;
 }
 STACKTOP = sp;return;
}
function _zSetupSpace($work,$lwork,$Glu) {
 $work = $work|0;
 $lwork = $lwork|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $work;
 $1 = $lwork;
 $2 = $Glu;
 $3 = $1;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = $2;
  $6 = ((($5)) + 52|0);
  HEAP32[$6>>2] = 0;
  STACKTOP = sp;return;
 }
 $7 = $1;
 $8 = ($7|0)>(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = $2;
 $10 = ((($9)) + 52|0);
 HEAP32[$10>>2] = 1;
 $11 = $2;
 $12 = ((($11)) + 64|0);
 $13 = ((($12)) + 4|0);
 HEAP32[$13>>2] = 0;
 $14 = $2;
 $15 = ((($14)) + 64|0);
 $16 = ((($15)) + 8|0);
 HEAP32[$16>>2] = 0;
 $17 = $1;
 $18 = (($17|0) / 4)&-1;
 $19 = $18<<2;
 $20 = $2;
 $21 = ((($20)) + 64|0);
 $22 = ((($21)) + 12|0);
 HEAP32[$22>>2] = $19;
 $23 = $2;
 $24 = ((($23)) + 64|0);
 $25 = ((($24)) + 12|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $2;
 $28 = ((($27)) + 64|0);
 HEAP32[$28>>2] = $26;
 $29 = $0;
 $30 = $2;
 $31 = ((($30)) + 64|0);
 $32 = ((($31)) + 16|0);
 HEAP32[$32>>2] = $29;
 STACKTOP = sp;return;
}
function _zuser_malloc($bytes,$which_end,$Glu) {
 $bytes = $bytes|0;
 $which_end = $which_end|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $bytes;
 $2 = $which_end;
 $3 = $Glu;
 $4 = $1;
 $5 = $3;
 $6 = ((($5)) + 64|0);
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($4) + ($8))|0;
 $10 = $3;
 $11 = ((($10)) + 64|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($9|0)>=($12|0);
 if ($13) {
  $0 = 0;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 }
 $14 = $2;
 $15 = ($14|0)==(0);
 if ($15) {
  $16 = $3;
  $17 = ((($16)) + 64|0);
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $3;
  $21 = ((($20)) + 64|0);
  $22 = ((($21)) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($19) + ($23)|0);
  $buf = $24;
  $25 = $1;
  $26 = $3;
  $27 = ((($26)) + 64|0);
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($25))|0;
  HEAP32[$28>>2] = $30;
 } else {
  $31 = $1;
  $32 = $3;
  $33 = ((($32)) + 64|0);
  $34 = ((($33)) + 12|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) - ($31))|0;
  HEAP32[$34>>2] = $36;
  $37 = $3;
  $38 = ((($37)) + 64|0);
  $39 = ((($38)) + 16|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = $3;
  $42 = ((($41)) + 64|0);
  $43 = ((($42)) + 12|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($40) + ($44)|0);
  $buf = $45;
 }
 $46 = $1;
 $47 = $3;
 $48 = ((($47)) + 64|0);
 $49 = ((($48)) + 4|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = (($50) + ($46))|0;
 HEAP32[$49>>2] = $51;
 $52 = $buf;
 $0 = $52;
 $53 = $0;
 STACKTOP = sp;return ($53|0);
}
function _zuser_free($bytes,$which_end,$Glu) {
 $bytes = $bytes|0;
 $which_end = $which_end|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $bytes;
 $1 = $which_end;
 $2 = $Glu;
 $3 = $1;
 $4 = ($3|0)==(0);
 $5 = $0;
 $6 = $2;
 $7 = ((($6)) + 64|0);
 if ($4) {
  $8 = ((($7)) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) - ($5))|0;
  HEAP32[$8>>2] = $10;
 } else {
  $11 = ((($7)) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($5))|0;
  HEAP32[$11>>2] = $13;
 }
 $14 = $0;
 $15 = $2;
 $16 = ((($15)) + 64|0);
 $17 = ((($16)) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) - ($14))|0;
 HEAP32[$17>>2] = $19;
 STACKTOP = sp;return;
}
function _zQuerySpace($L,$U,$mem_usage) {
 $L = $L|0;
 $U = $U|0;
 $mem_usage = $mem_usage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0, $42 = 0, $43 = 0.0, $44 = 0.0;
 var $45 = 0, $46 = 0.0, $47 = 0.0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0, $62 = 0.0;
 var $63 = 0.0, $64 = 0, $65 = 0.0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0.0, $73 = 0.0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0;
 var $81 = 0.0, $82 = 0.0, $83 = 0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0.0, $89 = 0, $9 = 0, $90 = 0, $Lstore = 0, $Ustore = 0, $dword = 0, $iword = 0, $n = 0, $panel_size = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $L;
 $1 = $U;
 $2 = $mem_usage;
 $3 = (_sp_ienv(1)|0);
 $panel_size = $3;
 $4 = $0;
 $5 = ((($4)) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $Lstore = $6;
 $7 = $1;
 $8 = ((($7)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 $Ustore = $9;
 $10 = $0;
 $11 = ((($10)) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $n = $12;
 $iword = 4;
 $dword = 16;
 $13 = $n;
 $14 = (+($13|0));
 $15 = 4.0 * $14;
 $16 = $15 + 3.0;
 $17 = $iword;
 $18 = (+($17|0));
 $19 = $16 * $18;
 $20 = $n;
 $21 = $Lstore;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20<<2)|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $dword;
 $27 = Math_imul($25, $26)|0;
 $28 = (+($27|0));
 $29 = $19 + $28;
 $30 = $n;
 $31 = $Lstore;
 $32 = ((($31)) + 20|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + ($30<<2)|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = $iword;
 $37 = Math_imul($35, $36)|0;
 $38 = (+($37|0));
 $39 = $29 + $38;
 $40 = $39;
 $41 = $2;
 HEAPF32[$41>>2] = $40;
 $42 = $n;
 $43 = (+($42|0));
 $44 = $43 + 1.0;
 $45 = $iword;
 $46 = (+($45|0));
 $47 = $44 * $46;
 $48 = $n;
 $49 = $Ustore;
 $50 = ((($49)) + 12|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($48<<2)|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = $dword;
 $55 = $iword;
 $56 = (($54) + ($55))|0;
 $57 = Math_imul($53, $56)|0;
 $58 = (+($57|0));
 $59 = $47 + $58;
 $60 = $59;
 $61 = $2;
 $62 = +HEAPF32[$61>>2];
 $63 = $62 + $60;
 HEAPF32[$61>>2] = $63;
 $64 = $2;
 $65 = +HEAPF32[$64>>2];
 $66 = $panel_size;
 $67 = (+($66|0));
 $68 = 2.0 * $67;
 $69 = $68 + 4.0;
 $70 = $69 + 3.0;
 $71 = $n;
 $72 = (+($71|0));
 $73 = $70 * $72;
 $74 = $iword;
 $75 = (+($74|0));
 $76 = $73 * $75;
 $77 = $panel_size;
 $78 = (+($77|0));
 $79 = $78 + 1.0;
 $80 = $n;
 $81 = (+($80|0));
 $82 = $79 * $81;
 $83 = $dword;
 $84 = (+($83|0));
 $85 = $82 * $84;
 $86 = $76 + $85;
 $87 = $86;
 $88 = $65 + $87;
 $89 = $2;
 $90 = ((($89)) + 4|0);
 HEAPF32[$90>>2] = $88;
 STACKTOP = sp;return 0;
}
function _zLUMemInit($fact,$work,$lwork,$m,$n,$annz,$panel_size,$fill_ratio,$L,$U,$Glu,$iwork,$dwork) {
 $fact = $fact|0;
 $work = $work|0;
 $lwork = $lwork|0;
 $m = $m|0;
 $n = $n|0;
 $annz = $annz|0;
 $panel_size = $panel_size|0;
 $fill_ratio = +$fill_ratio;
 $L = $L|0;
 $U = $U|0;
 $Glu = $Glu|0;
 $iwork = $iwork|0;
 $dwork = $dwork|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0.0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0.0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0.0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0.0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0.0, $380 = 0, $381 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0;
 var $43 = 0, $44 = 0.0, $45 = 0.0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0.0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $Lstore = 0, $Ustore = 0, $dword = 0, $info = 0, $iword = 0, $lsub = 0, $lusup = 0, $msg = 0, $nzlmax = 0, $nzlumax = 0, $nzumax = 0, $or$cond = 0, $or$cond3 = 0, $supno = 0, $ucol = 0, $usub = 0, $vararg_buffer = 0, $vararg_buffer6 = 0;
 var $vararg_ptr4 = 0, $vararg_ptr5 = 0, $xlsub = 0, $xlusup = 0, $xsup = 0, $xusub = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 416|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer = sp + 8|0;
 $nzlmax = sp + 36|0;
 $nzumax = sp + 32|0;
 $nzlumax = sp + 28|0;
 $msg = sp + 152|0;
 $1 = $fact;
 $2 = $work;
 $3 = $lwork;
 $4 = $m;
 $5 = $n;
 $6 = $annz;
 $7 = $panel_size;
 $8 = $fill_ratio;
 $9 = $L;
 $10 = $U;
 $11 = $Glu;
 $12 = $iwork;
 $13 = $dwork;
 $iword = 4;
 $dword = 16;
 $14 = $5;
 $15 = $11;
 $16 = ((($15)) + 48|0);
 HEAP32[$16>>2] = $14;
 $17 = $11;
 $18 = ((($17)) + 56|0);
 HEAP32[$18>>2] = 0;
 $19 = $11;
 $20 = ((($19)) + 60|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  $23 = (_superlu_malloc(32)|0);
  $24 = $11;
  $25 = ((($24)) + 60|0);
  HEAP32[$25>>2] = $23;
 }
 $26 = $11;
 $27 = ((($26)) + 60|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(0|0);
 if (!($29)) {
  HEAP32[$vararg_buffer>>2] = 2520;
  $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = 203;
  $vararg_ptr5 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr5>>2] = 2560;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $30 = $1;
 $31 = ($30|0)!=(2);
 L7: do {
  if ($31) {
   $32 = $8;
   $33 = $6;
   $34 = (+($33|0));
   $35 = $32 * $34;
   $36 = (~~(($35)));
   HEAP32[$nzlumax>>2] = $36;
   HEAP32[$nzumax>>2] = $36;
   $37 = $8;
   $38 = $37 / 4.0;
   $39 = 1.0 > $38;
   $40 = $8;
   $41 = $40 / 4.0;
   $42 = $39 ? 1.0 : $41;
   $43 = $6;
   $44 = (+($43|0));
   $45 = $42 * $44;
   $46 = (~~(($45)));
   HEAP32[$nzlmax>>2] = $46;
   $47 = $3;
   $48 = ($47|0)==(-1);
   if ($48) {
    $49 = $5;
    $50 = ($49*5)|0;
    $51 = (($50) + 5)|0;
    $52 = $iword;
    $53 = Math_imul($51, $52)|0;
    $54 = $7;
    $55 = $54<<1;
    $56 = (($55) + 4)|0;
    $57 = (($56) + 3)|0;
    $58 = $4;
    $59 = Math_imul($57, $58)|0;
    $60 = $59<<2;
    $61 = $7;
    $62 = (($61) + 1)|0;
    $63 = $4;
    $64 = Math_imul($62, $63)|0;
    $65 = $64<<4;
    $66 = (($60) + ($65))|0;
    $67 = (($53) + ($66))|0;
    $68 = HEAP32[$nzlmax>>2]|0;
    $69 = HEAP32[$nzumax>>2]|0;
    $70 = (($68) + ($69))|0;
    $71 = $iword;
    $72 = Math_imul($70, $71)|0;
    $73 = (($67) + ($72))|0;
    $74 = HEAP32[$nzlumax>>2]|0;
    $75 = HEAP32[$nzumax>>2]|0;
    $76 = (($74) + ($75))|0;
    $77 = $dword;
    $78 = Math_imul($76, $77)|0;
    $79 = (($73) + ($78))|0;
    $80 = $5;
    $81 = (($79) + ($80))|0;
    $0 = $81;
    $381 = $0;
    STACKTOP = sp;return ($381|0);
   }
   $82 = $2;
   $83 = $3;
   $84 = $11;
   _zSetupSpace($82,$83,$84);
   $85 = $11;
   $86 = ((($85)) + 52|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = ($87|0)==(0);
   $89 = $5;
   $90 = (($89) + 1)|0;
   if ($88) {
    $91 = (_intMalloc($90)|0);
    $xsup = $91;
    $92 = $5;
    $93 = (($92) + 1)|0;
    $94 = (_intMalloc($93)|0);
    $supno = $94;
    $95 = $5;
    $96 = (($95) + 1)|0;
    $97 = (_intMalloc($96)|0);
    $xlsub = $97;
    $98 = $5;
    $99 = (($98) + 1)|0;
    $100 = (_intMalloc($99)|0);
    $xlusup = $100;
    $101 = $5;
    $102 = (($101) + 1)|0;
    $103 = (_intMalloc($102)|0);
    $xusub = $103;
   } else {
    $104 = $iword;
    $105 = Math_imul($90, $104)|0;
    $106 = $11;
    $107 = (_zuser_malloc($105,0,$106)|0);
    $xsup = $107;
    $108 = $5;
    $109 = (($108) + 1)|0;
    $110 = $iword;
    $111 = Math_imul($109, $110)|0;
    $112 = $11;
    $113 = (_zuser_malloc($111,0,$112)|0);
    $supno = $113;
    $114 = $5;
    $115 = (($114) + 1)|0;
    $116 = $iword;
    $117 = Math_imul($115, $116)|0;
    $118 = $11;
    $119 = (_zuser_malloc($117,0,$118)|0);
    $xlsub = $119;
    $120 = $5;
    $121 = (($120) + 1)|0;
    $122 = $iword;
    $123 = Math_imul($121, $122)|0;
    $124 = $11;
    $125 = (_zuser_malloc($123,0,$124)|0);
    $xlusup = $125;
    $126 = $5;
    $127 = (($126) + 1)|0;
    $128 = $iword;
    $129 = Math_imul($127, $128)|0;
    $130 = $11;
    $131 = (_zuser_malloc($129,0,$130)|0);
    $xusub = $131;
   }
   $132 = $11;
   $133 = (_zexpand($nzlumax,0,0,0,$132)|0);
   $lusup = $133;
   $134 = $11;
   $135 = (_zexpand($nzumax,1,0,0,$134)|0);
   $ucol = $135;
   $136 = $11;
   $137 = (_zexpand($nzlmax,2,0,0,$136)|0);
   $lsub = $137;
   $138 = $11;
   $139 = (_zexpand($nzumax,3,0,1,$138)|0);
   $usub = $139;
   while(1) {
    $140 = $lusup;
    $141 = ($140|0)!=(0|0);
    $142 = $ucol;
    $143 = ($142|0)!=(0|0);
    $or$cond = $141 & $143;
    $144 = $lsub;
    $145 = ($144|0)!=(0|0);
    $or$cond3 = $or$cond & $145;
    if ($or$cond3) {
     $146 = $usub;
     $147 = ($146|0)!=(0|0);
     $148 = $147 ^ 1;
     if (!($148)) {
      break L7;
     }
    }
    $149 = $11;
    $150 = ((($149)) + 52|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = ($151|0)==(0);
    if ($152) {
     $153 = $lusup;
     _superlu_free($153);
     $154 = $ucol;
     _superlu_free($154);
     $155 = $lsub;
     _superlu_free($155);
     $156 = $usub;
     _superlu_free($156);
    } else {
     $157 = HEAP32[$nzlumax>>2]|0;
     $158 = HEAP32[$nzumax>>2]|0;
     $159 = (($157) + ($158))|0;
     $160 = $dword;
     $161 = Math_imul($159, $160)|0;
     $162 = HEAP32[$nzlmax>>2]|0;
     $163 = HEAP32[$nzumax>>2]|0;
     $164 = (($162) + ($163))|0;
     $165 = $iword;
     $166 = Math_imul($164, $165)|0;
     $167 = (($161) + ($166))|0;
     $168 = $11;
     _zuser_free($167,0,$168);
    }
    $169 = HEAP32[$nzlumax>>2]|0;
    $170 = (($169|0) / 2)&-1;
    HEAP32[$nzlumax>>2] = $170;
    $171 = HEAP32[$nzumax>>2]|0;
    $172 = (($171|0) / 2)&-1;
    HEAP32[$nzumax>>2] = $172;
    $173 = HEAP32[$nzlmax>>2]|0;
    $174 = (($173|0) / 2)&-1;
    HEAP32[$nzlmax>>2] = $174;
    $175 = HEAP32[$nzlumax>>2]|0;
    $176 = $6;
    $177 = ($175|0)<($176|0);
    if ($177) {
     break;
    }
    $185 = $11;
    $186 = (_zexpand($nzlumax,0,0,0,$185)|0);
    $lusup = $186;
    $187 = $11;
    $188 = (_zexpand($nzumax,1,0,0,$187)|0);
    $ucol = $188;
    $189 = $11;
    $190 = (_zexpand($nzlmax,2,0,0,$189)|0);
    $lsub = $190;
    $191 = $11;
    $192 = (_zexpand($nzumax,3,0,1,$191)|0);
    $usub = $192;
   }
   (_printf((2576|0),($vararg_buffer6|0))|0);
   $178 = HEAP32[$nzlmax>>2]|0;
   $179 = HEAP32[$nzumax>>2]|0;
   $180 = HEAP32[$nzlumax>>2]|0;
   $181 = $5;
   $182 = (_zmemory_usage($178,$179,$180,$181)|0);
   $183 = $5;
   $184 = (($182) + ($183))|0;
   $0 = $184;
   $381 = $0;
   STACKTOP = sp;return ($381|0);
  } else {
   $193 = $9;
   $194 = ((($193)) + 20|0);
   $195 = HEAP32[$194>>2]|0;
   $Lstore = $195;
   $196 = $10;
   $197 = ((($196)) + 20|0);
   $198 = HEAP32[$197>>2]|0;
   $Ustore = $198;
   $199 = $Lstore;
   $200 = ((($199)) + 28|0);
   $201 = HEAP32[$200>>2]|0;
   $xsup = $201;
   $202 = $Lstore;
   $203 = ((($202)) + 24|0);
   $204 = HEAP32[$203>>2]|0;
   $supno = $204;
   $205 = $Lstore;
   $206 = ((($205)) + 20|0);
   $207 = HEAP32[$206>>2]|0;
   $xlsub = $207;
   $208 = $Lstore;
   $209 = ((($208)) + 12|0);
   $210 = HEAP32[$209>>2]|0;
   $xlusup = $210;
   $211 = $Ustore;
   $212 = ((($211)) + 12|0);
   $213 = HEAP32[$212>>2]|0;
   $xusub = $213;
   $214 = $11;
   $215 = ((($214)) + 36|0);
   $216 = HEAP32[$215>>2]|0;
   HEAP32[$nzlmax>>2] = $216;
   $217 = $11;
   $218 = ((($217)) + 40|0);
   $219 = HEAP32[$218>>2]|0;
   HEAP32[$nzumax>>2] = $219;
   $220 = $11;
   $221 = ((($220)) + 44|0);
   $222 = HEAP32[$221>>2]|0;
   HEAP32[$nzlumax>>2] = $222;
   $223 = $3;
   $224 = ($223|0)==(-1);
   if ($224) {
    $225 = $5;
    $226 = ($225*5)|0;
    $227 = (($226) + 5)|0;
    $228 = $iword;
    $229 = Math_imul($227, $228)|0;
    $230 = $7;
    $231 = $230<<1;
    $232 = (($231) + 4)|0;
    $233 = (($232) + 3)|0;
    $234 = $4;
    $235 = Math_imul($233, $234)|0;
    $236 = $235<<2;
    $237 = $7;
    $238 = (($237) + 1)|0;
    $239 = $4;
    $240 = Math_imul($238, $239)|0;
    $241 = $240<<4;
    $242 = (($236) + ($241))|0;
    $243 = (($229) + ($242))|0;
    $244 = HEAP32[$nzlmax>>2]|0;
    $245 = HEAP32[$nzumax>>2]|0;
    $246 = (($244) + ($245))|0;
    $247 = $iword;
    $248 = Math_imul($246, $247)|0;
    $249 = (($243) + ($248))|0;
    $250 = HEAP32[$nzlumax>>2]|0;
    $251 = HEAP32[$nzumax>>2]|0;
    $252 = (($250) + ($251))|0;
    $253 = $dword;
    $254 = Math_imul($252, $253)|0;
    $255 = (($249) + ($254))|0;
    $256 = $5;
    $257 = (($255) + ($256))|0;
    $0 = $257;
    $381 = $0;
    STACKTOP = sp;return ($381|0);
   }
   $258 = $3;
   $259 = ($258|0)==(0);
   $260 = $11;
   $261 = ((($260)) + 52|0);
   if ($259) {
    HEAP32[$261>>2] = 0;
   } else {
    HEAP32[$261>>2] = 1;
    $262 = $3;
    $263 = (($262|0) / 4)&-1;
    $264 = $263<<2;
    $265 = $11;
    $266 = ((($265)) + 64|0);
    $267 = ((($266)) + 12|0);
    HEAP32[$267>>2] = $264;
    $268 = $11;
    $269 = ((($268)) + 64|0);
    $270 = ((($269)) + 12|0);
    $271 = HEAP32[$270>>2]|0;
    $272 = $11;
    $273 = ((($272)) + 64|0);
    HEAP32[$273>>2] = $271;
   }
   $274 = $Lstore;
   $275 = ((($274)) + 16|0);
   $276 = HEAP32[$275>>2]|0;
   $277 = $11;
   $278 = ((($277)) + 60|0);
   $279 = HEAP32[$278>>2]|0;
   $280 = ((($279)) + 16|0);
   $281 = ((($280)) + 4|0);
   HEAP32[$281>>2] = $276;
   $lsub = $276;
   $282 = $Lstore;
   $283 = ((($282)) + 8|0);
   $284 = HEAP32[$283>>2]|0;
   $285 = $11;
   $286 = ((($285)) + 60|0);
   $287 = HEAP32[$286>>2]|0;
   $288 = ((($287)) + 4|0);
   HEAP32[$288>>2] = $284;
   $lusup = $284;
   $289 = $Ustore;
   $290 = ((($289)) + 8|0);
   $291 = HEAP32[$290>>2]|0;
   $292 = $11;
   $293 = ((($292)) + 60|0);
   $294 = HEAP32[$293>>2]|0;
   $295 = ((($294)) + 24|0);
   $296 = ((($295)) + 4|0);
   HEAP32[$296>>2] = $291;
   $usub = $291;
   $297 = $Ustore;
   $298 = ((($297)) + 4|0);
   $299 = HEAP32[$298>>2]|0;
   $300 = $11;
   $301 = ((($300)) + 60|0);
   $302 = HEAP32[$301>>2]|0;
   $303 = ((($302)) + 8|0);
   $304 = ((($303)) + 4|0);
   HEAP32[$304>>2] = $299;
   $ucol = $299;
   $305 = HEAP32[$nzlmax>>2]|0;
   $306 = $11;
   $307 = ((($306)) + 60|0);
   $308 = HEAP32[$307>>2]|0;
   $309 = ((($308)) + 16|0);
   HEAP32[$309>>2] = $305;
   $310 = HEAP32[$nzlumax>>2]|0;
   $311 = $11;
   $312 = ((($311)) + 60|0);
   $313 = HEAP32[$312>>2]|0;
   HEAP32[$313>>2] = $310;
   $314 = HEAP32[$nzumax>>2]|0;
   $315 = $11;
   $316 = ((($315)) + 60|0);
   $317 = HEAP32[$316>>2]|0;
   $318 = ((($317)) + 24|0);
   HEAP32[$318>>2] = $314;
   $319 = HEAP32[$nzumax>>2]|0;
   $320 = $11;
   $321 = ((($320)) + 60|0);
   $322 = HEAP32[$321>>2]|0;
   $323 = ((($322)) + 8|0);
   HEAP32[$323>>2] = $319;
  }
 } while(0);
 $324 = $xsup;
 $325 = $11;
 HEAP32[$325>>2] = $324;
 $326 = $supno;
 $327 = $11;
 $328 = ((($327)) + 4|0);
 HEAP32[$328>>2] = $326;
 $329 = $lsub;
 $330 = $11;
 $331 = ((($330)) + 8|0);
 HEAP32[$331>>2] = $329;
 $332 = $xlsub;
 $333 = $11;
 $334 = ((($333)) + 12|0);
 HEAP32[$334>>2] = $332;
 $335 = $lusup;
 $336 = $11;
 $337 = ((($336)) + 16|0);
 HEAP32[$337>>2] = $335;
 $338 = $xlusup;
 $339 = $11;
 $340 = ((($339)) + 20|0);
 HEAP32[$340>>2] = $338;
 $341 = $ucol;
 $342 = $11;
 $343 = ((($342)) + 24|0);
 HEAP32[$343>>2] = $341;
 $344 = $usub;
 $345 = $11;
 $346 = ((($345)) + 28|0);
 HEAP32[$346>>2] = $344;
 $347 = $xusub;
 $348 = $11;
 $349 = ((($348)) + 32|0);
 HEAP32[$349>>2] = $347;
 $350 = HEAP32[$nzlmax>>2]|0;
 $351 = $11;
 $352 = ((($351)) + 36|0);
 HEAP32[$352>>2] = $350;
 $353 = HEAP32[$nzumax>>2]|0;
 $354 = $11;
 $355 = ((($354)) + 40|0);
 HEAP32[$355>>2] = $353;
 $356 = HEAP32[$nzlumax>>2]|0;
 $357 = $11;
 $358 = ((($357)) + 44|0);
 HEAP32[$358>>2] = $356;
 $359 = $4;
 $360 = $5;
 $361 = $7;
 $362 = $12;
 $363 = $13;
 $364 = $11;
 $365 = (_zLUWorkInit($359,$360,$361,$362,$363,$364)|0);
 $info = $365;
 $366 = $info;
 $367 = ($366|0)!=(0);
 if ($367) {
  $368 = $info;
  $369 = HEAP32[$nzlmax>>2]|0;
  $370 = HEAP32[$nzumax>>2]|0;
  $371 = HEAP32[$nzlumax>>2]|0;
  $372 = $5;
  $373 = (_zmemory_usage($369,$370,$371,$372)|0);
  $374 = (($368) + ($373))|0;
  $375 = $5;
  $376 = (($374) + ($375))|0;
  $0 = $376;
  $381 = $0;
  STACKTOP = sp;return ($381|0);
 } else {
  $377 = $11;
  $378 = ((($377)) + 56|0);
  $379 = HEAP32[$378>>2]|0;
  $380 = (($379) + 1)|0;
  HEAP32[$378>>2] = $380;
  $0 = 0;
  $381 = $0;
  STACKTOP = sp;return ($381|0);
 }
 return (0)|0;
}
function _zexpand($prev_len,$type,$len_to_copy,$keep_prev,$Glu) {
 $prev_len = $prev_len|0;
 $type = $type|0;
 $len_to_copy = $len_to_copy|0;
 $keep_prev = $keep_prev|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0.0, $161 = 0.0, $162 = 0.0, $163 = 0.0, $164 = 0, $165 = 0, $166 = 0.0, $167 = 0.0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0.0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0.0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0.0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0.0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $EXPAND = 0.0, $alpha = 0.0, $bytes_to_copy = 0, $expanders = 0;
 var $extra = 0, $lword = 0, $new_len = 0, $new_mem = 0, $old_mem = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $tries = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $prev_len;
 $2 = $type;
 $3 = $len_to_copy;
 $4 = $keep_prev;
 $5 = $Glu;
 $EXPAND = 1.5;
 $6 = $5;
 $7 = ((($6)) + 60|0);
 $8 = HEAP32[$7>>2]|0;
 $expanders = $8;
 $9 = $EXPAND;
 $alpha = $9;
 $10 = $5;
 $11 = ((($10)) + 56|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 $14 = $4;
 $15 = ($14|0)!=(0);
 $or$cond = $13 | $15;
 if ($or$cond) {
  $16 = $1;
  $17 = HEAP32[$16>>2]|0;
  $new_len = $17;
 } else {
  $18 = $alpha;
  $19 = $1;
  $20 = HEAP32[$19>>2]|0;
  $21 = (+($20|0));
  $22 = $18 * $21;
  $23 = (~~(($22)));
  $new_len = $23;
 }
 $24 = $2;
 $25 = ($24|0)==(2);
 $26 = $2;
 $27 = ($26|0)==(3);
 $or$cond3 = $25 | $27;
 if ($or$cond3) {
  $lword = 4;
 } else {
  $lword = 16;
 }
 $28 = $5;
 $29 = ((($28)) + 52|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)==(0);
 do {
  if ($31) {
   $32 = $new_len;
   $33 = $lword;
   $34 = Math_imul($32, $33)|0;
   $35 = (_superlu_malloc($34)|0);
   $new_mem = $35;
   $36 = $5;
   $37 = ((($36)) + 56|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ($38|0)!=(0);
   if ($39) {
    $tries = 0;
    $40 = $4;
    $41 = ($40|0)!=(0);
    L13: do {
     if ($41) {
      $42 = $new_mem;
      $43 = ($42|0)!=(0|0);
      if (!($43)) {
       $0 = 0;
       $295 = $0;
       STACKTOP = sp;return ($295|0);
      }
     } else {
      while(1) {
       $44 = $new_mem;
       $45 = ($44|0)!=(0|0);
       $46 = $45 ^ 1;
       if (!($46)) {
        break L13;
       }
       $47 = $tries;
       $48 = (($47) + 1)|0;
       $tries = $48;
       $49 = ($48|0)>(10);
       if ($49) {
        break;
       }
       $50 = $alpha;
       $51 = $50 + 1.0;
       $52 = $51 / 2.0;
       $alpha = $52;
       $53 = $alpha;
       $54 = $1;
       $55 = HEAP32[$54>>2]|0;
       $56 = (+($55|0));
       $57 = $53 * $56;
       $58 = (~~(($57)));
       $new_len = $58;
       $59 = $new_len;
       $60 = $lword;
       $61 = Math_imul($59, $60)|0;
       $62 = (_superlu_malloc($61)|0);
       $new_mem = $62;
      }
      $0 = 0;
      $295 = $0;
      STACKTOP = sp;return ($295|0);
     }
    } while(0);
    $63 = $2;
    $64 = ($63|0)==(2);
    $65 = $2;
    $66 = ($65|0)==(3);
    $or$cond5 = $64 | $66;
    $67 = $3;
    $68 = $2;
    $69 = $expanders;
    $70 = (($69) + ($68<<3)|0);
    $71 = ((($70)) + 4|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = $new_mem;
    if ($or$cond5) {
     _copy_mem_int($67,$72,$73);
    } else {
     _copy_mem_doublecomplex($67,$72,$73);
    }
    $74 = $2;
    $75 = $expanders;
    $76 = (($75) + ($74<<3)|0);
    $77 = ((($76)) + 4|0);
    $78 = HEAP32[$77>>2]|0;
    _superlu_free($78);
   }
   $79 = $new_mem;
   $80 = $2;
   $81 = $expanders;
   $82 = (($81) + ($80<<3)|0);
   $83 = ((($82)) + 4|0);
   HEAP32[$83>>2] = $79;
  } else {
   $84 = $5;
   $85 = ((($84)) + 56|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = ($86|0)==(0);
   if ($87) {
    $88 = $new_len;
    $89 = $lword;
    $90 = Math_imul($88, $89)|0;
    $91 = $5;
    $92 = (_zuser_malloc($90,0,$91)|0);
    $new_mem = $92;
    $93 = $new_mem;
    $94 = $93;
    $95 = $94 & 7;
    $96 = ($95|0)!=(0);
    if ($96) {
     $97 = $2;
     $98 = ($97|0)==(0);
     $99 = $2;
     $100 = ($99|0)==(1);
     $or$cond7 = $98 | $100;
     if ($or$cond7) {
      $101 = $new_mem;
      $old_mem = $101;
      $102 = $new_mem;
      $103 = $102;
      $104 = (($103) + 7)|0;
      $105 = $104 & -8;
      $106 = $105;
      $new_mem = $106;
      $107 = $new_mem;
      $108 = $old_mem;
      $109 = $107;
      $110 = $108;
      $111 = (($109) - ($110))|0;
      $extra = $111;
      $112 = $extra;
      $113 = $5;
      $114 = ((($113)) + 64|0);
      $115 = ((($114)) + 8|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = (($116) + ($112))|0;
      HEAP32[$115>>2] = $117;
      $118 = $extra;
      $119 = $5;
      $120 = ((($119)) + 64|0);
      $121 = ((($120)) + 4|0);
      $122 = HEAP32[$121>>2]|0;
      $123 = (($122) + ($118))|0;
      HEAP32[$121>>2] = $123;
     }
    }
    $124 = $new_mem;
    $125 = $2;
    $126 = $expanders;
    $127 = (($126) + ($125<<3)|0);
    $128 = ((($127)) + 4|0);
    HEAP32[$128>>2] = $124;
    break;
   }
   $tries = 0;
   $129 = $new_len;
   $130 = $1;
   $131 = HEAP32[$130>>2]|0;
   $132 = (($129) - ($131))|0;
   $133 = $lword;
   $134 = Math_imul($132, $133)|0;
   $extra = $134;
   $135 = $4;
   $136 = ($135|0)!=(0);
   L37: do {
    if ($136) {
     $137 = $extra;
     $138 = $5;
     $139 = ((($138)) + 64|0);
     $140 = ((($139)) + 4|0);
     $141 = HEAP32[$140>>2]|0;
     $142 = (($137) + ($141))|0;
     $143 = $5;
     $144 = ((($143)) + 64|0);
     $145 = HEAP32[$144>>2]|0;
     $146 = ($142|0)>=($145|0);
     if ($146) {
      $0 = 0;
      $295 = $0;
      STACKTOP = sp;return ($295|0);
     }
    } else {
     while(1) {
      $147 = $extra;
      $148 = $5;
      $149 = ((($148)) + 64|0);
      $150 = ((($149)) + 4|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = (($147) + ($151))|0;
      $153 = $5;
      $154 = ((($153)) + 64|0);
      $155 = HEAP32[$154>>2]|0;
      $156 = ($152|0)>=($155|0);
      if (!($156)) {
       break L37;
      }
      $157 = $tries;
      $158 = (($157) + 1)|0;
      $tries = $158;
      $159 = ($158|0)>(10);
      if ($159) {
       break;
      }
      $160 = $alpha;
      $161 = $160 + 1.0;
      $162 = $161 / 2.0;
      $alpha = $162;
      $163 = $alpha;
      $164 = $1;
      $165 = HEAP32[$164>>2]|0;
      $166 = (+($165|0));
      $167 = $163 * $166;
      $168 = (~~(($167)));
      $new_len = $168;
      $169 = $new_len;
      $170 = $1;
      $171 = HEAP32[$170>>2]|0;
      $172 = (($169) - ($171))|0;
      $173 = $lword;
      $174 = Math_imul($172, $173)|0;
      $extra = $174;
     }
     $0 = 0;
     $295 = $0;
     STACKTOP = sp;return ($295|0);
    }
   } while(0);
   $175 = $2;
   $176 = ($175|0)!=(3);
   if ($176) {
    $177 = $2;
    $178 = (($177) + 1)|0;
    $179 = $expanders;
    $180 = (($179) + ($178<<3)|0);
    $181 = ((($180)) + 4|0);
    $182 = HEAP32[$181>>2]|0;
    $183 = $extra;
    $184 = (($182) + ($183)|0);
    $new_mem = $184;
    $185 = $5;
    $186 = ((($185)) + 64|0);
    $187 = ((($186)) + 16|0);
    $188 = HEAP32[$187>>2]|0;
    $189 = $5;
    $190 = ((($189)) + 64|0);
    $191 = ((($190)) + 8|0);
    $192 = HEAP32[$191>>2]|0;
    $193 = (($188) + ($192)|0);
    $194 = $2;
    $195 = (($194) + 1)|0;
    $196 = $expanders;
    $197 = (($196) + ($195<<3)|0);
    $198 = ((($197)) + 4|0);
    $199 = HEAP32[$198>>2]|0;
    $200 = $193;
    $201 = $199;
    $202 = (($200) - ($201))|0;
    $bytes_to_copy = $202;
    $203 = $2;
    $204 = (($203) + 1)|0;
    $205 = $expanders;
    $206 = (($205) + ($204<<3)|0);
    $207 = ((($206)) + 4|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = $new_mem;
    $210 = $bytes_to_copy;
    _user_bcopy($208,$209,$210);
    $211 = $2;
    $212 = ($211>>>0)<(3);
    if ($212) {
     $213 = $expanders;
     $214 = ((($213)) + 24|0);
     $215 = ((($214)) + 4|0);
     $216 = HEAP32[$215>>2]|0;
     $217 = $extra;
     $218 = (($216) + ($217)|0);
     $219 = $expanders;
     $220 = ((($219)) + 24|0);
     $221 = ((($220)) + 4|0);
     HEAP32[$221>>2] = $218;
     $222 = $5;
     $223 = ((($222)) + 28|0);
     HEAP32[$223>>2] = $218;
    }
    $224 = $2;
    $225 = ($224>>>0)<(2);
    if ($225) {
     $226 = $expanders;
     $227 = ((($226)) + 16|0);
     $228 = ((($227)) + 4|0);
     $229 = HEAP32[$228>>2]|0;
     $230 = $extra;
     $231 = (($229) + ($230)|0);
     $232 = $expanders;
     $233 = ((($232)) + 16|0);
     $234 = ((($233)) + 4|0);
     HEAP32[$234>>2] = $231;
     $235 = $5;
     $236 = ((($235)) + 8|0);
     HEAP32[$236>>2] = $231;
    }
    $237 = $2;
    $238 = ($237>>>0)<(1);
    if ($238) {
     $239 = $expanders;
     $240 = ((($239)) + 8|0);
     $241 = ((($240)) + 4|0);
     $242 = HEAP32[$241>>2]|0;
     $243 = $extra;
     $244 = (($242) + ($243)|0);
     $245 = $expanders;
     $246 = ((($245)) + 8|0);
     $247 = ((($246)) + 4|0);
     HEAP32[$247>>2] = $244;
     $248 = $5;
     $249 = ((($248)) + 24|0);
     HEAP32[$249>>2] = $244;
    }
    $250 = $extra;
    $251 = $5;
    $252 = ((($251)) + 64|0);
    $253 = ((($252)) + 8|0);
    $254 = HEAP32[$253>>2]|0;
    $255 = (($254) + ($250))|0;
    HEAP32[$253>>2] = $255;
    $256 = $extra;
    $257 = $5;
    $258 = ((($257)) + 64|0);
    $259 = ((($258)) + 4|0);
    $260 = HEAP32[$259>>2]|0;
    $261 = (($260) + ($256))|0;
    HEAP32[$259>>2] = $261;
    $262 = $2;
    $263 = ($262|0)==(1);
    if ($263) {
     $264 = $extra;
     $265 = $5;
     $266 = ((($265)) + 64|0);
     $267 = ((($266)) + 8|0);
     $268 = HEAP32[$267>>2]|0;
     $269 = (($268) + ($264))|0;
     HEAP32[$267>>2] = $269;
     $270 = $extra;
     $271 = $5;
     $272 = ((($271)) + 64|0);
     $273 = ((($272)) + 4|0);
     $274 = HEAP32[$273>>2]|0;
     $275 = (($274) + ($270))|0;
     HEAP32[$273>>2] = $275;
    }
   }
  }
 } while(0);
 $276 = $new_len;
 $277 = $2;
 $278 = $expanders;
 $279 = (($278) + ($277<<3)|0);
 HEAP32[$279>>2] = $276;
 $280 = $new_len;
 $281 = $1;
 HEAP32[$281>>2] = $280;
 $282 = $5;
 $283 = ((($282)) + 56|0);
 $284 = HEAP32[$283>>2]|0;
 $285 = ($284|0)!=(0);
 if ($285) {
  $286 = $5;
  $287 = ((($286)) + 56|0);
  $288 = HEAP32[$287>>2]|0;
  $289 = (($288) + 1)|0;
  HEAP32[$287>>2] = $289;
 }
 $290 = $2;
 $291 = $expanders;
 $292 = (($291) + ($290<<3)|0);
 $293 = ((($292)) + 4|0);
 $294 = HEAP32[$293>>2]|0;
 $0 = $294;
 $295 = $0;
 STACKTOP = sp;return ($295|0);
}
function _zmemory_usage($nzlmax,$nzumax,$nzlumax,$n) {
 $nzlmax = $nzlmax|0;
 $nzumax = $nzumax|0;
 $nzlumax = $nzlumax|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $dword = 0, $iword = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $nzlmax;
 $1 = $nzumax;
 $2 = $nzlumax;
 $3 = $n;
 $iword = 4;
 $dword = 16;
 $4 = $3;
 $5 = ($4*10)|0;
 $6 = $iword;
 $7 = Math_imul($5, $6)|0;
 $8 = $0;
 $9 = $iword;
 $10 = Math_imul($8, $9)|0;
 $11 = (($7) + ($10))|0;
 $12 = $1;
 $13 = $iword;
 $14 = $dword;
 $15 = (($13) + ($14))|0;
 $16 = Math_imul($12, $15)|0;
 $17 = (($11) + ($16))|0;
 $18 = $2;
 $19 = $dword;
 $20 = Math_imul($18, $19)|0;
 $21 = (($17) + ($20))|0;
 STACKTOP = sp;return ($21|0);
}
function _zLUWorkInit($m,$n,$panel_size,$iworkptr,$dworkptr,$Glu) {
 $m = $m|0;
 $n = $n|0;
 $panel_size = $panel_size|0;
 $iworkptr = $iworkptr|0;
 $dworkptr = $dworkptr|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dsize = 0, $extra = 0, $isize = 0, $maxsuper = 0;
 var $old_ptr = 0, $rowblk = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $m;
 $2 = $n;
 $3 = $panel_size;
 $4 = $iworkptr;
 $5 = $dworkptr;
 $6 = $Glu;
 $7 = (_sp_ienv(3)|0);
 $8 = (_sp_ienv(7)|0);
 $9 = ($7|0)>($8|0);
 if ($9) {
  $10 = (_sp_ienv(3)|0);
  $12 = $10;
 } else {
  $11 = (_sp_ienv(7)|0);
  $12 = $11;
 }
 $maxsuper = $12;
 $13 = (_sp_ienv(4)|0);
 $rowblk = $13;
 $14 = $3;
 $15 = $14<<1;
 $16 = (($15) + 3)|0;
 $17 = (($16) + 3)|0;
 $18 = $1;
 $19 = Math_imul($17, $18)|0;
 $20 = $2;
 $21 = (($19) + ($20))|0;
 $22 = $21<<2;
 $isize = $22;
 $23 = $1;
 $24 = $3;
 $25 = Math_imul($23, $24)|0;
 $26 = $1;
 $27 = $maxsuper;
 $28 = $rowblk;
 $29 = (($27) + ($28))|0;
 $30 = $3;
 $31 = Math_imul($29, $30)|0;
 $32 = ($26|0)>($31|0);
 if ($32) {
  $33 = $1;
  $40 = $33;
 } else {
  $34 = $maxsuper;
  $35 = $rowblk;
  $36 = (($34) + ($35))|0;
  $37 = $3;
  $38 = Math_imul($36, $37)|0;
  $40 = $38;
 }
 $39 = (($25) + ($40))|0;
 $41 = $39<<4;
 $dsize = $41;
 $42 = $6;
 $43 = ((($42)) + 52|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)==(0);
 $46 = $isize;
 if ($45) {
  $47 = (($46>>>0) / 4)&-1;
  $48 = (_intCalloc($47)|0);
  $49 = $4;
  HEAP32[$49>>2] = $48;
 } else {
  $50 = $6;
  $51 = (_zuser_malloc($46,1,$50)|0);
  $52 = $4;
  HEAP32[$52>>2] = $51;
 }
 $53 = $4;
 $54 = HEAP32[$53>>2]|0;
 $55 = ($54|0)!=(0|0);
 if (!($55)) {
  $56 = HEAP32[_stderr>>2]|0;
  (_fprintf(($56|0),(2624|0),($vararg_buffer|0))|0);
  $57 = $isize;
  $58 = $2;
  $59 = (($57) + ($58))|0;
  $0 = $59;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 $60 = $6;
 $61 = ((($60)) + 52|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = ($62|0)==(0);
 $64 = $dsize;
 if ($63) {
  $65 = (_superlu_malloc($64)|0);
  $66 = $5;
  HEAP32[$66>>2] = $65;
 } else {
  $67 = $6;
  $68 = (_zuser_malloc($64,1,$67)|0);
  $69 = $5;
  HEAP32[$69>>2] = $68;
  $70 = $5;
  $71 = HEAP32[$70>>2]|0;
  $72 = $71;
  $73 = $72 & 7;
  $74 = ($73|0)!=(0);
  if ($74) {
   $75 = $5;
   $76 = HEAP32[$75>>2]|0;
   $old_ptr = $76;
   $77 = $5;
   $78 = HEAP32[$77>>2]|0;
   $79 = $78;
   $80 = (($79) + 7)|0;
   $81 = $80 & -8;
   $82 = $81;
   $83 = $5;
   HEAP32[$83>>2] = $82;
   $84 = $5;
   $85 = HEAP32[$84>>2]|0;
   $86 = ((($85)) + -8|0);
   $87 = $5;
   HEAP32[$87>>2] = $86;
   $88 = $old_ptr;
   $89 = $5;
   $90 = HEAP32[$89>>2]|0;
   $91 = $88;
   $92 = $90;
   $93 = (($91) - ($92))|0;
   $extra = $93;
   $94 = $extra;
   $95 = $6;
   $96 = ((($95)) + 64|0);
   $97 = ((($96)) + 12|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = (($98) - ($94))|0;
   HEAP32[$97>>2] = $99;
   $100 = $extra;
   $101 = $6;
   $102 = ((($101)) + 64|0);
   $103 = ((($102)) + 4|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = (($104) + ($100))|0;
   HEAP32[$103>>2] = $105;
  }
 }
 $106 = $5;
 $107 = HEAP32[$106>>2]|0;
 $108 = ($107|0)!=(0|0);
 if ($108) {
  $0 = 0;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 } else {
  $109 = HEAP32[_stderr>>2]|0;
  (_fprintf(($109|0),(2672|0),($vararg_buffer1|0))|0);
  $110 = $isize;
  $111 = $dsize;
  $112 = (($110) + ($111))|0;
  $113 = $2;
  $114 = (($112) + ($113))|0;
  $0 = $114;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 return (0)|0;
}
function _zSetRWork($m,$panel_size,$dworkptr,$dense,$tempv) {
 $m = $m|0;
 $panel_size = $panel_size|0;
 $dworkptr = $dworkptr|0;
 $dense = $dense|0;
 $tempv = $tempv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $maxsuper = 0, $rowblk = 0, $zero = 0, $zero$byval_copy = 0, $zero$byval_copy1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $zero$byval_copy1 = sp + 32|0;
 $zero$byval_copy = sp + 16|0;
 $zero = sp;
 $0 = $m;
 $1 = $panel_size;
 $2 = $dworkptr;
 $3 = $dense;
 $4 = $tempv;
 ;HEAP32[$zero>>2]=0|0;HEAP32[$zero+4>>2]=0|0;HEAP32[$zero+8>>2]=0|0;HEAP32[$zero+12>>2]=0|0;
 $5 = (_sp_ienv(3)|0);
 $6 = (_sp_ienv(7)|0);
 $7 = ($5|0)>($6|0);
 if ($7) {
  $8 = (_sp_ienv(3)|0);
  $10 = $8;
 } else {
  $9 = (_sp_ienv(7)|0);
  $10 = $9;
 }
 $maxsuper = $10;
 $11 = (_sp_ienv(4)|0);
 $rowblk = $11;
 $12 = $2;
 $13 = $3;
 HEAP32[$13>>2] = $12;
 $14 = $3;
 $15 = HEAP32[$14>>2]|0;
 $16 = $1;
 $17 = $0;
 $18 = Math_imul($16, $17)|0;
 $19 = (($15) + ($18<<4)|0);
 $20 = $4;
 HEAP32[$20>>2] = $19;
 $21 = $3;
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = $1;
 $25 = Math_imul($23, $24)|0;
 ;HEAP32[$zero$byval_copy>>2]=HEAP32[$zero>>2]|0;HEAP32[$zero$byval_copy+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$zero$byval_copy+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$zero$byval_copy+12>>2]=HEAP32[$zero+12>>2]|0;
 _zfill($22,$25,$zero$byval_copy);
 $26 = $4;
 $27 = HEAP32[$26>>2]|0;
 $28 = $0;
 $29 = $maxsuper;
 $30 = $rowblk;
 $31 = (($29) + ($30))|0;
 $32 = $1;
 $33 = Math_imul($31, $32)|0;
 $34 = ($28|0)>($33|0);
 if ($34) {
  $35 = $0;
  $41 = $35;
  ;HEAP32[$zero$byval_copy1>>2]=HEAP32[$zero>>2]|0;HEAP32[$zero$byval_copy1+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$zero$byval_copy1+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$zero$byval_copy1+12>>2]=HEAP32[$zero+12>>2]|0;
  _zfill($27,$41,$zero$byval_copy1);
  STACKTOP = sp;return;
 } else {
  $36 = $maxsuper;
  $37 = $rowblk;
  $38 = (($36) + ($37))|0;
  $39 = $1;
  $40 = Math_imul($38, $39)|0;
  $41 = $40;
  ;HEAP32[$zero$byval_copy1>>2]=HEAP32[$zero>>2]|0;HEAP32[$zero$byval_copy1+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$zero$byval_copy1+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$zero$byval_copy1+12>>2]=HEAP32[$zero+12>>2]|0;
  _zfill($27,$41,$zero$byval_copy1);
  STACKTOP = sp;return;
 }
}
function _zLUWorkFree($iwork,$dwork,$Glu) {
 $iwork = $iwork|0;
 $dwork = $dwork|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $iwork;
 $1 = $dwork;
 $2 = $Glu;
 $3 = $2;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = $0;
  _superlu_free($7);
  $8 = $1;
  _superlu_free($8);
 } else {
  $9 = $2;
  $10 = ((($9)) + 64|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $2;
  $13 = ((($12)) + 64|0);
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($11) - ($15))|0;
  $17 = $2;
  $18 = ((($17)) + 64|0);
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) - ($16))|0;
  HEAP32[$19>>2] = $21;
  $22 = $2;
  $23 = ((($22)) + 64|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $2;
  $26 = ((($25)) + 64|0);
  $27 = ((($26)) + 12|0);
  HEAP32[$27>>2] = $24;
 }
 $28 = $2;
 $29 = ((($28)) + 60|0);
 $30 = HEAP32[$29>>2]|0;
 _superlu_free($30);
 $31 = $2;
 $32 = ((($31)) + 60|0);
 HEAP32[$32>>2] = 0;
 STACKTOP = sp;return;
}
function _zLUMemXpand($jcol,$next,$mem_type,$maxlen,$Glu) {
 $jcol = $jcol|0;
 $next = $next|0;
 $mem_type = $mem_type|0;
 $maxlen = $maxlen|0;
 $Glu = $Glu|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $new_mem = 0, $nzlmax = 0, $nzlumax = 0, $nzumax = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $1 = $jcol;
 $2 = $next;
 $3 = $mem_type;
 $4 = $maxlen;
 $5 = $Glu;
 $6 = $3;
 $7 = ($6|0)==(3);
 $8 = $4;
 $9 = $3;
 $10 = $2;
 $11 = $5;
 if ($7) {
  $12 = (_zexpand($8,$9,$10,1,$11)|0);
  $new_mem = $12;
 } else {
  $13 = (_zexpand($8,$9,$10,0,$11)|0);
  $new_mem = $13;
 }
 $14 = $new_mem;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $16 = $5;
  $17 = ((($16)) + 36|0);
  $18 = HEAP32[$17>>2]|0;
  $nzlmax = $18;
  $19 = $5;
  $20 = ((($19)) + 40|0);
  $21 = HEAP32[$20>>2]|0;
  $nzumax = $21;
  $22 = $5;
  $23 = ((($22)) + 44|0);
  $24 = HEAP32[$23>>2]|0;
  $nzlumax = $24;
  $25 = HEAP32[_stderr>>2]|0;
  $26 = $3;
  $27 = $1;
  HEAP32[$vararg_buffer>>2] = $26;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $27;
  (_fprintf(($25|0),(2712|0),($vararg_buffer|0))|0);
  $28 = $nzlmax;
  $29 = $nzumax;
  $30 = $nzlumax;
  $31 = $5;
  $32 = ((($31)) + 48|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (_zmemory_usage($28,$29,$30,$33)|0);
  $35 = $5;
  $36 = ((($35)) + 48|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($34) + ($37))|0;
  $0 = $38;
  $68 = $0;
  STACKTOP = sp;return ($68|0);
 }
 $39 = $3;
 if ((($39|0) == 0)) {
  $40 = $new_mem;
  $41 = $5;
  $42 = ((($41)) + 16|0);
  HEAP32[$42>>2] = $40;
  $43 = $4;
  $44 = HEAP32[$43>>2]|0;
  $45 = $5;
  $46 = ((($45)) + 44|0);
  HEAP32[$46>>2] = $44;
 } else if ((($39|0) == 2)) {
  $54 = $new_mem;
  $55 = $5;
  $56 = ((($55)) + 8|0);
  HEAP32[$56>>2] = $54;
  $57 = $4;
  $58 = HEAP32[$57>>2]|0;
  $59 = $5;
  $60 = ((($59)) + 36|0);
  HEAP32[$60>>2] = $58;
 } else if ((($39|0) == 3)) {
  $61 = $new_mem;
  $62 = $5;
  $63 = ((($62)) + 28|0);
  HEAP32[$63>>2] = $61;
  $64 = $4;
  $65 = HEAP32[$64>>2]|0;
  $66 = $5;
  $67 = ((($66)) + 40|0);
  HEAP32[$67>>2] = $65;
 } else if ((($39|0) == 1)) {
  $47 = $new_mem;
  $48 = $5;
  $49 = ((($48)) + 24|0);
  HEAP32[$49>>2] = $47;
  $50 = $4;
  $51 = HEAP32[$50>>2]|0;
  $52 = $5;
  $53 = ((($52)) + 40|0);
  HEAP32[$53>>2] = $51;
 }
 $0 = 0;
 $68 = $0;
 STACKTOP = sp;return ($68|0);
}
function _copy_mem_doublecomplex($howmany,$old,$new) {
 $howmany = $howmany|0;
 $old = $old|0;
 $new = $new|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dnew = 0, $dold = 0, $i = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $howmany;
 $1 = $old;
 $2 = $new;
 $3 = $1;
 $dold = $3;
 $4 = $2;
 $dnew = $4;
 $i = 0;
 while(1) {
  $5 = $i;
  $6 = $0;
  $7 = ($5|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = $i;
  $9 = $dnew;
  $10 = (($9) + ($8<<4)|0);
  $11 = $i;
  $12 = $dold;
  $13 = (($12) + ($11<<4)|0);
  ;HEAP32[$10>>2]=HEAP32[$13>>2]|0;HEAP32[$10+4>>2]=HEAP32[$13+4>>2]|0;HEAP32[$10+8>>2]=HEAP32[$13+8>>2]|0;HEAP32[$10+12>>2]=HEAP32[$13+12>>2]|0;
  $14 = $i;
  $15 = (($14) + 1)|0;
  $i = $15;
 }
 STACKTOP = sp;return;
}
function _doublecomplexMalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 24|0;
 $0 = $n;
 $1 = $0;
 $2 = $1<<4;
 $3 = (_superlu_malloc($2)|0);
 $buf = $3;
 $4 = $buf;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = $buf;
  STACKTOP = sp;return ($6|0);
 }
 HEAP32[$vararg_buffer>>2] = 2752;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 671;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = 2560;
 (_sprintf($msg,2864,$vararg_buffer)|0);
 _superlu_abort_and_exit($msg);
 $6 = $buf;
 STACKTOP = sp;return ($6|0);
}
function _doublecomplexCalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $i = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0;
 var $vararg_ptr2 = 0, $zero = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp + 16|0;
 $zero = sp;
 $msg = sp + 40|0;
 $0 = $n;
 ;HEAP32[$zero>>2]=0|0;HEAP32[$zero+4>>2]=0|0;HEAP32[$zero+8>>2]=0|0;HEAP32[$zero+12>>2]=0|0;
 $1 = $0;
 $2 = $1<<4;
 $3 = (_superlu_malloc($2)|0);
 $buf = $3;
 $4 = $buf;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  HEAP32[$vararg_buffer>>2] = 2808;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 683;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2560;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $i = 0;
 while(1) {
  $6 = $i;
  $7 = $0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $i;
  $10 = $buf;
  $11 = (($10) + ($9<<4)|0);
  ;HEAP32[$11>>2]=HEAP32[$zero>>2]|0;HEAP32[$11+4>>2]=HEAP32[$zero+4>>2]|0;HEAP32[$11+8>>2]=HEAP32[$zero+8>>2]|0;HEAP32[$11+12>>2]=HEAP32[$zero+12>>2]|0;
  $12 = $i;
  $13 = (($12) + 1)|0;
  $i = $13;
 }
 $14 = $buf;
 STACKTOP = sp;return ($14|0);
}
function _zCreate_CompCol_Matrix($A,$m,$n,$nnz,$nzval,$rowind,$colptr,$stype,$dtype,$mtype) {
 $A = $A|0;
 $m = $m|0;
 $n = $n|0;
 $nnz = $nnz|0;
 $nzval = $nzval|0;
 $rowind = $rowind|0;
 $colptr = $colptr|0;
 $stype = $stype|0;
 $dtype = $dtype|0;
 $mtype = $mtype|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $Astore = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 320|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 56|0;
 $0 = $A;
 $1 = $m;
 $2 = $n;
 $3 = $nnz;
 $4 = $nzval;
 $5 = $rowind;
 $6 = $colptr;
 $7 = $stype;
 $8 = $dtype;
 $9 = $mtype;
 $10 = $7;
 $11 = $0;
 HEAP32[$11>>2] = $10;
 $12 = $8;
 $13 = $0;
 $14 = ((($13)) + 4|0);
 HEAP32[$14>>2] = $12;
 $15 = $9;
 $16 = $0;
 $17 = ((($16)) + 8|0);
 HEAP32[$17>>2] = $15;
 $18 = $1;
 $19 = $0;
 $20 = ((($19)) + 12|0);
 HEAP32[$20>>2] = $18;
 $21 = $2;
 $22 = $0;
 $23 = ((($22)) + 16|0);
 HEAP32[$23>>2] = $21;
 $24 = (_superlu_malloc(16)|0);
 $25 = $0;
 $26 = ((($25)) + 20|0);
 HEAP32[$26>>2] = $24;
 $27 = $0;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 if (!($30)) {
  HEAP32[$vararg_buffer>>2] = 2896;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 41;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2936;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $31 = $0;
 $32 = ((($31)) + 20|0);
 $33 = HEAP32[$32>>2]|0;
 $Astore = $33;
 $34 = $3;
 $35 = $Astore;
 HEAP32[$35>>2] = $34;
 $36 = $4;
 $37 = $Astore;
 $38 = ((($37)) + 4|0);
 HEAP32[$38>>2] = $36;
 $39 = $5;
 $40 = $Astore;
 $41 = ((($40)) + 8|0);
 HEAP32[$41>>2] = $39;
 $42 = $6;
 $43 = $Astore;
 $44 = ((($43)) + 12|0);
 HEAP32[$44>>2] = $42;
 STACKTOP = sp;return;
}
function _zCreate_Dense_Matrix($X,$m,$n,$x,$ldx,$stype,$dtype,$mtype) {
 $X = $X|0;
 $m = $m|0;
 $n = $n|0;
 $x = $x|0;
 $ldx = $ldx|0;
 $stype = $stype|0;
 $dtype = $dtype|0;
 $mtype = $mtype|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $Xstore = 0, $msg = 0, $vararg_buffer = 0;
 var $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 48|0;
 $0 = $X;
 $1 = $m;
 $2 = $n;
 $3 = $x;
 $4 = $ldx;
 $5 = $stype;
 $6 = $dtype;
 $7 = $mtype;
 $8 = $5;
 $9 = $0;
 HEAP32[$9>>2] = $8;
 $10 = $6;
 $11 = $0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = $7;
 $14 = $0;
 $15 = ((($14)) + 8|0);
 HEAP32[$15>>2] = $13;
 $16 = $1;
 $17 = $0;
 $18 = ((($17)) + 12|0);
 HEAP32[$18>>2] = $16;
 $19 = $2;
 $20 = $0;
 $21 = ((($20)) + 16|0);
 HEAP32[$21>>2] = $19;
 $22 = (_superlu_malloc(8)|0);
 $23 = $0;
 $24 = ((($23)) + 20|0);
 HEAP32[$24>>2] = $22;
 $25 = $0;
 $26 = ((($25)) + 20|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  HEAP32[$vararg_buffer>>2] = 2944;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 104;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2936;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $29 = $0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 $Xstore = $31;
 $32 = $4;
 $33 = $Xstore;
 HEAP32[$33>>2] = $32;
 $34 = $3;
 $35 = $Xstore;
 $36 = ((($35)) + 4|0);
 HEAP32[$36>>2] = $34;
 STACKTOP = sp;return;
}
function _zCreate_SuperNode_Matrix($L,$m,$n,$nnz,$nzval,$nzval_colptr,$rowind,$rowind_colptr,$col_to_sup,$sup_to_col,$stype,$dtype,$mtype) {
 $L = $L|0;
 $m = $m|0;
 $n = $n|0;
 $nnz = $nnz|0;
 $nzval = $nzval|0;
 $nzval_colptr = $nzval_colptr|0;
 $rowind = $rowind|0;
 $rowind_colptr = $rowind_colptr|0;
 $col_to_sup = $col_to_sup|0;
 $sup_to_col = $sup_to_col|0;
 $stype = $stype|0;
 $dtype = $dtype|0;
 $mtype = $mtype|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $Lstore = 0, $msg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $msg = sp + 72|0;
 $0 = $L;
 $1 = $m;
 $2 = $n;
 $3 = $nnz;
 $4 = $nzval;
 $5 = $nzval_colptr;
 $6 = $rowind;
 $7 = $rowind_colptr;
 $8 = $col_to_sup;
 $9 = $sup_to_col;
 $10 = $stype;
 $11 = $dtype;
 $12 = $mtype;
 $13 = $10;
 $14 = $0;
 HEAP32[$14>>2] = $13;
 $15 = $11;
 $16 = $0;
 $17 = ((($16)) + 4|0);
 HEAP32[$17>>2] = $15;
 $18 = $12;
 $19 = $0;
 $20 = ((($19)) + 8|0);
 HEAP32[$20>>2] = $18;
 $21 = $1;
 $22 = $0;
 $23 = ((($22)) + 12|0);
 HEAP32[$23>>2] = $21;
 $24 = $2;
 $25 = $0;
 $26 = ((($25)) + 16|0);
 HEAP32[$26>>2] = $24;
 $27 = (_superlu_malloc(32)|0);
 $28 = $0;
 $29 = ((($28)) + 20|0);
 HEAP32[$29>>2] = $27;
 $30 = $0;
 $31 = ((($30)) + 20|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)!=(0|0);
 if (!($33)) {
  HEAP32[$vararg_buffer>>2] = 2984;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 137;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2936;
  (_sprintf($msg,2864,$vararg_buffer)|0);
  _superlu_abort_and_exit($msg);
 }
 $34 = $0;
 $35 = ((($34)) + 20|0);
 $36 = HEAP32[$35>>2]|0;
 $Lstore = $36;
 $37 = $3;
 $38 = $Lstore;
 HEAP32[$38>>2] = $37;
 $39 = $2;
 $40 = $8;
 $41 = (($40) + ($39<<2)|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = $Lstore;
 $44 = ((($43)) + 4|0);
 HEAP32[$44>>2] = $42;
 $45 = $4;
 $46 = $Lstore;
 $47 = ((($46)) + 8|0);
 HEAP32[$47>>2] = $45;
 $48 = $5;
 $49 = $Lstore;
 $50 = ((($49)) + 12|0);
 HEAP32[$50>>2] = $48;
 $51 = $6;
 $52 = $Lstore;
 $53 = ((($52)) + 16|0);
 HEAP32[$53>>2] = $51;
 $54 = $7;
 $55 = $Lstore;
 $56 = ((($55)) + 20|0);
 HEAP32[$56>>2] = $54;
 $57 = $8;
 $58 = $Lstore;
 $59 = ((($58)) + 24|0);
 HEAP32[$59>>2] = $57;
 $60 = $9;
 $61 = $Lstore;
 $62 = ((($61)) + 28|0);
 HEAP32[$62>>2] = $60;
 STACKTOP = sp;return;
}
function _zfill($a,$alen,$dval) {
 $a = $a|0;
 $alen = $alen|0;
 $dval = $dval|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $a;
 $1 = $alen;
 $i = 0;
 while(1) {
  $2 = $i;
  $3 = $1;
  $4 = ($2|0)<($3|0);
  if (!($4)) {
   break;
  }
  $5 = $i;
  $6 = $0;
  $7 = (($6) + ($5<<4)|0);
  ;HEAP32[$7>>2]=HEAP32[$dval>>2]|0;HEAP32[$7+4>>2]=HEAP32[$dval+4>>2]|0;HEAP32[$7+8>>2]=HEAP32[$dval+8>>2]|0;HEAP32[$7+12>>2]=HEAP32[$dval+12>>2]|0;
  $8 = $i;
  $9 = (($8) + 1)|0;
  $i = $9;
 }
 STACKTOP = sp;return;
}
function _zlsolve($ldm,$ncol,$M,$rhs) {
 $ldm = $ldm|0;
 $ncol = $ncol|0;
 $M = $M|0;
 $rhs = $rhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0.0, $102 = 0.0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0.0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0.0, $113 = 0, $114 = 0.0, $115 = 0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0, $122 = 0.0, $123 = 0.0, $124 = 0.0, $125 = 0, $126 = 0.0, $127 = 0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0, $132 = 0, $133 = 0.0;
 var $134 = 0.0, $135 = 0.0, $136 = 0.0, $137 = 0.0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0.0, $142 = 0.0, $143 = 0.0, $144 = 0, $145 = 0.0, $146 = 0, $147 = 0.0, $148 = 0.0, $149 = 0, $15 = 0, $150 = 0.0, $151 = 0;
 var $152 = 0.0, $153 = 0.0, $154 = 0, $155 = 0.0, $156 = 0, $157 = 0, $158 = 0.0, $159 = 0.0, $16 = 0, $160 = 0.0, $161 = 0, $162 = 0.0, $163 = 0, $164 = 0.0, $165 = 0.0, $166 = 0.0, $167 = 0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0.0, $171 = 0.0, $172 = 0.0, $173 = 0.0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0.0, $182 = 0.0, $183 = 0.0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0.0, $19 = 0, $190 = 0, $191 = 0.0, $192 = 0.0, $193 = 0, $194 = 0.0, $195 = 0, $196 = 0.0, $197 = 0.0, $198 = 0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0.0, $203 = 0.0, $204 = 0.0, $205 = 0;
 var $206 = 0.0, $207 = 0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0.0, $211 = 0, $212 = 0, $213 = 0.0, $214 = 0.0, $215 = 0.0, $216 = 0.0, $217 = 0.0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0.0, $222 = 0.0, $223 = 0.0;
 var $224 = 0, $225 = 0.0, $226 = 0, $227 = 0.0, $228 = 0.0, $229 = 0, $23 = 0, $230 = 0.0, $231 = 0, $232 = 0.0, $233 = 0.0, $234 = 0, $235 = 0.0, $236 = 0, $237 = 0, $238 = 0.0, $239 = 0.0, $24 = 0, $240 = 0.0, $241 = 0;
 var $242 = 0.0, $243 = 0, $244 = 0.0, $245 = 0.0, $246 = 0.0, $247 = 0, $248 = 0, $249 = 0.0, $25 = 0, $250 = 0.0, $251 = 0.0, $252 = 0.0, $253 = 0.0, $254 = 0, $255 = 0, $256 = 0, $257 = 0.0, $258 = 0.0, $259 = 0.0, $26 = 0.0;
 var $260 = 0, $261 = 0.0, $262 = 0, $263 = 0.0, $264 = 0.0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0.0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0.0, $285 = 0, $286 = 0.0, $287 = 0.0, $288 = 0, $289 = 0.0, $29 = 0.0, $290 = 0, $291 = 0, $292 = 0.0, $293 = 0.0, $294 = 0.0, $295 = 0, $296 = 0.0;
 var $297 = 0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0, $302 = 0, $303 = 0.0, $304 = 0.0, $305 = 0.0, $306 = 0.0, $307 = 0.0, $308 = 0, $309 = 0, $31 = 0.0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0.0, $315 = 0.0, $316 = 0.0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0.0, $325 = 0, $326 = 0.0, $327 = 0.0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0.0, $333 = 0, $334 = 0.0, $335 = 0.0, $336 = 0, $337 = 0.0, $338 = 0, $339 = 0, $34 = 0.0, $340 = 0.0, $341 = 0.0, $342 = 0.0, $343 = 0, $344 = 0.0, $345 = 0, $346 = 0.0, $347 = 0.0, $348 = 0.0, $349 = 0, $35 = 0.0;
 var $350 = 0, $351 = 0.0, $352 = 0.0, $353 = 0.0, $354 = 0.0, $355 = 0.0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0.0, $360 = 0, $361 = 0, $362 = 0.0, $363 = 0.0, $364 = 0.0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0.0, $373 = 0, $374 = 0.0, $375 = 0.0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0.0, $380 = 0.0, $381 = 0, $382 = 0.0, $383 = 0.0, $384 = 0, $385 = 0.0, $386 = 0;
 var $387 = 0, $388 = 0.0, $389 = 0.0, $39 = 0, $390 = 0.0, $391 = 0, $392 = 0.0, $393 = 0, $394 = 0.0, $395 = 0.0, $396 = 0.0, $397 = 0, $398 = 0, $399 = 0.0, $4 = 0, $40 = 0.0, $400 = 0.0, $401 = 0.0, $402 = 0.0, $403 = 0.0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0.0, $410 = 0.0, $411 = 0.0, $412 = 0.0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0.0, $420 = 0.0, $421 = 0;
 var $422 = 0.0, $423 = 0.0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0.0, $429 = 0, $43 = 0, $430 = 0.0, $431 = 0.0, $432 = 0, $433 = 0.0, $434 = 0, $435 = 0, $436 = 0.0, $437 = 0.0, $438 = 0.0, $439 = 0, $44 = 0;
 var $440 = 0.0, $441 = 0, $442 = 0.0, $443 = 0.0, $444 = 0.0, $445 = 0, $446 = 0, $447 = 0.0, $448 = 0.0, $449 = 0.0, $45 = 0.0, $450 = 0.0, $451 = 0.0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0.0;
 var $459 = 0.0, $46 = 0.0, $460 = 0.0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0.0, $469 = 0, $47 = 0.0, $470 = 0.0, $471 = 0.0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0.0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0.0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0.0, $497 = 0, $498 = 0.0, $499 = 0.0, $5 = 0, $50 = 0, $500 = 0, $501 = 0.0, $502 = 0, $503 = 0, $504 = 0.0, $505 = 0.0, $506 = 0.0, $507 = 0, $508 = 0.0, $509 = 0, $51 = 0, $510 = 0.0, $511 = 0.0;
 var $512 = 0.0, $513 = 0, $514 = 0, $515 = 0.0, $516 = 0.0, $517 = 0.0, $518 = 0.0, $519 = 0.0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0.0, $528 = 0.0, $529 = 0.0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0.0, $536 = 0, $537 = 0.0, $538 = 0.0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0.0, $551 = 0, $552 = 0.0, $553 = 0.0, $554 = 0, $555 = 0.0, $556 = 0, $557 = 0, $558 = 0.0, $559 = 0.0, $56 = 0, $560 = 0.0, $561 = 0, $562 = 0.0, $563 = 0, $564 = 0.0, $565 = 0.0, $566 = 0.0;
 var $567 = 0, $568 = 0, $569 = 0.0, $57 = 0.0, $570 = 0.0, $571 = 0.0, $572 = 0.0, $573 = 0.0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0.0, $580 = 0.0, $581 = 0.0, $582 = 0.0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0.0, $590 = 0.0, $591 = 0, $592 = 0.0, $593 = 0.0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0.0, $599 = 0, $6 = 0, $60 = 0, $600 = 0.0, $601 = 0.0;
 var $602 = 0, $603 = 0.0, $604 = 0, $605 = 0, $606 = 0.0, $607 = 0.0, $608 = 0.0, $609 = 0, $61 = 0, $610 = 0.0, $611 = 0, $612 = 0.0, $613 = 0.0, $614 = 0.0, $615 = 0, $616 = 0, $617 = 0.0, $618 = 0.0, $619 = 0.0, $62 = 0;
 var $620 = 0.0, $621 = 0.0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0.0, $629 = 0.0, $63 = 0, $630 = 0.0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0.0;
 var $639 = 0, $64 = 0, $640 = 0.0, $641 = 0.0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $65 = 0.0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0.0, $73 = 0.0;
 var $74 = 0, $75 = 0.0, $76 = 0, $77 = 0, $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0, $82 = 0.0, $83 = 0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0.0;
 var $92 = 0.0, $93 = 0.0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $M0 = 0, $Mki0 = 0, $Mki1 = 0, $Mki2 = 0, $Mki3 = 0, $ci = 0.0, $ci10 = 0.0, $ci12 = 0.0, $ci14 = 0.0, $ci16 = 0.0, $ci18 = 0.0, $ci2 = 0.0;
 var $ci20 = 0.0, $ci22 = 0.0, $ci24 = 0.0, $ci4 = 0.0, $ci6 = 0.0, $ci8 = 0.0, $cr = 0.0, $cr1 = 0.0, $cr11 = 0.0, $cr13 = 0.0, $cr15 = 0.0, $cr17 = 0.0, $cr19 = 0.0, $cr21 = 0.0, $cr23 = 0.0, $cr3 = 0.0, $cr5 = 0.0, $cr7 = 0.0, $cr9 = 0.0, $firstcol = 0;
 var $k = 0, $temp = 0, $x0 = 0, $x1 = 0, $x2 = 0, $x3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $x0 = sp + 272|0;
 $x1 = sp + 256|0;
 $x2 = sp + 240|0;
 $x3 = sp + 224|0;
 $temp = sp + 208|0;
 $0 = $ldm;
 $1 = $ncol;
 $2 = $M;
 $3 = $rhs;
 $firstcol = 0;
 $4 = $2;
 $M0 = $4;
 while(1) {
  $5 = $firstcol;
  $6 = $1;
  $7 = (($6) - 3)|0;
  $8 = ($5|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $M0;
  $10 = ((($9)) + 16|0);
  $Mki0 = $10;
  $11 = $Mki0;
  $12 = $0;
  $13 = (($11) + ($12<<4)|0);
  $14 = ((($13)) + 16|0);
  $Mki1 = $14;
  $15 = $Mki1;
  $16 = $0;
  $17 = (($15) + ($16<<4)|0);
  $18 = ((($17)) + 16|0);
  $Mki2 = $18;
  $19 = $Mki2;
  $20 = $0;
  $21 = (($19) + ($20<<4)|0);
  $22 = ((($21)) + 16|0);
  $Mki3 = $22;
  $23 = $firstcol;
  $24 = $3;
  $25 = (($24) + ($23<<4)|0);
  ;HEAP32[$x0>>2]=HEAP32[$25>>2]|0;HEAP32[$x0+4>>2]=HEAP32[$25+4>>2]|0;HEAP32[$x0+8>>2]=HEAP32[$25+8>>2]|0;HEAP32[$x0+12>>2]=HEAP32[$25+12>>2]|0;
  $26 = +HEAPF64[$x0>>3];
  $27 = $Mki0;
  $28 = +HEAPF64[$27>>3];
  $29 = $26 * $28;
  $30 = ((($x0)) + 8|0);
  $31 = +HEAPF64[$30>>3];
  $32 = $Mki0;
  $33 = ((($32)) + 8|0);
  $34 = +HEAPF64[$33>>3];
  $35 = $31 * $34;
  $36 = $29 - $35;
  $cr = $36;
  $37 = ((($x0)) + 8|0);
  $38 = +HEAPF64[$37>>3];
  $39 = $Mki0;
  $40 = +HEAPF64[$39>>3];
  $41 = $38 * $40;
  $42 = +HEAPF64[$x0>>3];
  $43 = $Mki0;
  $44 = ((($43)) + 8|0);
  $45 = +HEAPF64[$44>>3];
  $46 = $42 * $45;
  $47 = $41 + $46;
  $ci = $47;
  $48 = $cr;
  HEAPF64[$temp>>3] = $48;
  $49 = $ci;
  $50 = ((($temp)) + 8|0);
  HEAPF64[$50>>3] = $49;
  $51 = $Mki0;
  $52 = ((($51)) + 16|0);
  $Mki0 = $52;
  $53 = $firstcol;
  $54 = (($53) + 1)|0;
  $55 = $3;
  $56 = (($55) + ($54<<4)|0);
  $57 = +HEAPF64[$56>>3];
  $58 = +HEAPF64[$temp>>3];
  $59 = $57 - $58;
  HEAPF64[$x1>>3] = $59;
  $60 = $firstcol;
  $61 = (($60) + 1)|0;
  $62 = $3;
  $63 = (($62) + ($61<<4)|0);
  $64 = ((($63)) + 8|0);
  $65 = +HEAPF64[$64>>3];
  $66 = ((($temp)) + 8|0);
  $67 = +HEAPF64[$66>>3];
  $68 = $65 - $67;
  $69 = ((($x1)) + 8|0);
  HEAPF64[$69>>3] = $68;
  $70 = +HEAPF64[$x0>>3];
  $71 = $Mki0;
  $72 = +HEAPF64[$71>>3];
  $73 = $70 * $72;
  $74 = ((($x0)) + 8|0);
  $75 = +HEAPF64[$74>>3];
  $76 = $Mki0;
  $77 = ((($76)) + 8|0);
  $78 = +HEAPF64[$77>>3];
  $79 = $75 * $78;
  $80 = $73 - $79;
  $cr1 = $80;
  $81 = ((($x0)) + 8|0);
  $82 = +HEAPF64[$81>>3];
  $83 = $Mki0;
  $84 = +HEAPF64[$83>>3];
  $85 = $82 * $84;
  $86 = +HEAPF64[$x0>>3];
  $87 = $Mki0;
  $88 = ((($87)) + 8|0);
  $89 = +HEAPF64[$88>>3];
  $90 = $86 * $89;
  $91 = $85 + $90;
  $ci2 = $91;
  $92 = $cr1;
  HEAPF64[$temp>>3] = $92;
  $93 = $ci2;
  $94 = ((($temp)) + 8|0);
  HEAPF64[$94>>3] = $93;
  $95 = $Mki0;
  $96 = ((($95)) + 16|0);
  $Mki0 = $96;
  $97 = $firstcol;
  $98 = (($97) + 2)|0;
  $99 = $3;
  $100 = (($99) + ($98<<4)|0);
  $101 = +HEAPF64[$100>>3];
  $102 = +HEAPF64[$temp>>3];
  $103 = $101 - $102;
  HEAPF64[$x2>>3] = $103;
  $104 = $firstcol;
  $105 = (($104) + 2)|0;
  $106 = $3;
  $107 = (($106) + ($105<<4)|0);
  $108 = ((($107)) + 8|0);
  $109 = +HEAPF64[$108>>3];
  $110 = ((($temp)) + 8|0);
  $111 = +HEAPF64[$110>>3];
  $112 = $109 - $111;
  $113 = ((($x2)) + 8|0);
  HEAPF64[$113>>3] = $112;
  $114 = +HEAPF64[$x1>>3];
  $115 = $Mki1;
  $116 = +HEAPF64[$115>>3];
  $117 = $114 * $116;
  $118 = ((($x1)) + 8|0);
  $119 = +HEAPF64[$118>>3];
  $120 = $Mki1;
  $121 = ((($120)) + 8|0);
  $122 = +HEAPF64[$121>>3];
  $123 = $119 * $122;
  $124 = $117 - $123;
  $cr3 = $124;
  $125 = ((($x1)) + 8|0);
  $126 = +HEAPF64[$125>>3];
  $127 = $Mki1;
  $128 = +HEAPF64[$127>>3];
  $129 = $126 * $128;
  $130 = +HEAPF64[$x1>>3];
  $131 = $Mki1;
  $132 = ((($131)) + 8|0);
  $133 = +HEAPF64[$132>>3];
  $134 = $130 * $133;
  $135 = $129 + $134;
  $ci4 = $135;
  $136 = $cr3;
  HEAPF64[$temp>>3] = $136;
  $137 = $ci4;
  $138 = ((($temp)) + 8|0);
  HEAPF64[$138>>3] = $137;
  $139 = $Mki1;
  $140 = ((($139)) + 16|0);
  $Mki1 = $140;
  $141 = +HEAPF64[$x2>>3];
  $142 = +HEAPF64[$temp>>3];
  $143 = $141 - $142;
  HEAPF64[$x2>>3] = $143;
  $144 = ((($x2)) + 8|0);
  $145 = +HEAPF64[$144>>3];
  $146 = ((($temp)) + 8|0);
  $147 = +HEAPF64[$146>>3];
  $148 = $145 - $147;
  $149 = ((($x2)) + 8|0);
  HEAPF64[$149>>3] = $148;
  $150 = +HEAPF64[$x0>>3];
  $151 = $Mki0;
  $152 = +HEAPF64[$151>>3];
  $153 = $150 * $152;
  $154 = ((($x0)) + 8|0);
  $155 = +HEAPF64[$154>>3];
  $156 = $Mki0;
  $157 = ((($156)) + 8|0);
  $158 = +HEAPF64[$157>>3];
  $159 = $155 * $158;
  $160 = $153 - $159;
  $cr5 = $160;
  $161 = ((($x0)) + 8|0);
  $162 = +HEAPF64[$161>>3];
  $163 = $Mki0;
  $164 = +HEAPF64[$163>>3];
  $165 = $162 * $164;
  $166 = +HEAPF64[$x0>>3];
  $167 = $Mki0;
  $168 = ((($167)) + 8|0);
  $169 = +HEAPF64[$168>>3];
  $170 = $166 * $169;
  $171 = $165 + $170;
  $ci6 = $171;
  $172 = $cr5;
  HEAPF64[$temp>>3] = $172;
  $173 = $ci6;
  $174 = ((($temp)) + 8|0);
  HEAPF64[$174>>3] = $173;
  $175 = $Mki0;
  $176 = ((($175)) + 16|0);
  $Mki0 = $176;
  $177 = $firstcol;
  $178 = (($177) + 3)|0;
  $179 = $3;
  $180 = (($179) + ($178<<4)|0);
  $181 = +HEAPF64[$180>>3];
  $182 = +HEAPF64[$temp>>3];
  $183 = $181 - $182;
  HEAPF64[$x3>>3] = $183;
  $184 = $firstcol;
  $185 = (($184) + 3)|0;
  $186 = $3;
  $187 = (($186) + ($185<<4)|0);
  $188 = ((($187)) + 8|0);
  $189 = +HEAPF64[$188>>3];
  $190 = ((($temp)) + 8|0);
  $191 = +HEAPF64[$190>>3];
  $192 = $189 - $191;
  $193 = ((($x3)) + 8|0);
  HEAPF64[$193>>3] = $192;
  $194 = +HEAPF64[$x1>>3];
  $195 = $Mki1;
  $196 = +HEAPF64[$195>>3];
  $197 = $194 * $196;
  $198 = ((($x1)) + 8|0);
  $199 = +HEAPF64[$198>>3];
  $200 = $Mki1;
  $201 = ((($200)) + 8|0);
  $202 = +HEAPF64[$201>>3];
  $203 = $199 * $202;
  $204 = $197 - $203;
  $cr7 = $204;
  $205 = ((($x1)) + 8|0);
  $206 = +HEAPF64[$205>>3];
  $207 = $Mki1;
  $208 = +HEAPF64[$207>>3];
  $209 = $206 * $208;
  $210 = +HEAPF64[$x1>>3];
  $211 = $Mki1;
  $212 = ((($211)) + 8|0);
  $213 = +HEAPF64[$212>>3];
  $214 = $210 * $213;
  $215 = $209 + $214;
  $ci8 = $215;
  $216 = $cr7;
  HEAPF64[$temp>>3] = $216;
  $217 = $ci8;
  $218 = ((($temp)) + 8|0);
  HEAPF64[$218>>3] = $217;
  $219 = $Mki1;
  $220 = ((($219)) + 16|0);
  $Mki1 = $220;
  $221 = +HEAPF64[$x3>>3];
  $222 = +HEAPF64[$temp>>3];
  $223 = $221 - $222;
  HEAPF64[$x3>>3] = $223;
  $224 = ((($x3)) + 8|0);
  $225 = +HEAPF64[$224>>3];
  $226 = ((($temp)) + 8|0);
  $227 = +HEAPF64[$226>>3];
  $228 = $225 - $227;
  $229 = ((($x3)) + 8|0);
  HEAPF64[$229>>3] = $228;
  $230 = +HEAPF64[$x2>>3];
  $231 = $Mki2;
  $232 = +HEAPF64[$231>>3];
  $233 = $230 * $232;
  $234 = ((($x2)) + 8|0);
  $235 = +HEAPF64[$234>>3];
  $236 = $Mki2;
  $237 = ((($236)) + 8|0);
  $238 = +HEAPF64[$237>>3];
  $239 = $235 * $238;
  $240 = $233 - $239;
  $cr9 = $240;
  $241 = ((($x2)) + 8|0);
  $242 = +HEAPF64[$241>>3];
  $243 = $Mki2;
  $244 = +HEAPF64[$243>>3];
  $245 = $242 * $244;
  $246 = +HEAPF64[$x2>>3];
  $247 = $Mki2;
  $248 = ((($247)) + 8|0);
  $249 = +HEAPF64[$248>>3];
  $250 = $246 * $249;
  $251 = $245 + $250;
  $ci10 = $251;
  $252 = $cr9;
  HEAPF64[$temp>>3] = $252;
  $253 = $ci10;
  $254 = ((($temp)) + 8|0);
  HEAPF64[$254>>3] = $253;
  $255 = $Mki2;
  $256 = ((($255)) + 16|0);
  $Mki2 = $256;
  $257 = +HEAPF64[$x3>>3];
  $258 = +HEAPF64[$temp>>3];
  $259 = $257 - $258;
  HEAPF64[$x3>>3] = $259;
  $260 = ((($x3)) + 8|0);
  $261 = +HEAPF64[$260>>3];
  $262 = ((($temp)) + 8|0);
  $263 = +HEAPF64[$262>>3];
  $264 = $261 - $263;
  $265 = ((($x3)) + 8|0);
  HEAPF64[$265>>3] = $264;
  $266 = $firstcol;
  $267 = (($266) + 1)|0;
  $firstcol = $267;
  $268 = $3;
  $269 = (($268) + ($267<<4)|0);
  ;HEAP32[$269>>2]=HEAP32[$x1>>2]|0;HEAP32[$269+4>>2]=HEAP32[$x1+4>>2]|0;HEAP32[$269+8>>2]=HEAP32[$x1+8>>2]|0;HEAP32[$269+12>>2]=HEAP32[$x1+12>>2]|0;
  $270 = $firstcol;
  $271 = (($270) + 1)|0;
  $firstcol = $271;
  $272 = $3;
  $273 = (($272) + ($271<<4)|0);
  ;HEAP32[$273>>2]=HEAP32[$x2>>2]|0;HEAP32[$273+4>>2]=HEAP32[$x2+4>>2]|0;HEAP32[$273+8>>2]=HEAP32[$x2+8>>2]|0;HEAP32[$273+12>>2]=HEAP32[$x2+12>>2]|0;
  $274 = $firstcol;
  $275 = (($274) + 1)|0;
  $firstcol = $275;
  $276 = $3;
  $277 = (($276) + ($275<<4)|0);
  ;HEAP32[$277>>2]=HEAP32[$x3>>2]|0;HEAP32[$277+4>>2]=HEAP32[$x3+4>>2]|0;HEAP32[$277+8>>2]=HEAP32[$x3+8>>2]|0;HEAP32[$277+12>>2]=HEAP32[$x3+12>>2]|0;
  $278 = $firstcol;
  $279 = (($278) + 1)|0;
  $firstcol = $279;
  $280 = $firstcol;
  $k = $280;
  while(1) {
   $281 = $k;
   $282 = $1;
   $283 = ($281|0)<($282|0);
   if (!($283)) {
    break;
   }
   $284 = +HEAPF64[$x0>>3];
   $285 = $Mki0;
   $286 = +HEAPF64[$285>>3];
   $287 = $284 * $286;
   $288 = ((($x0)) + 8|0);
   $289 = +HEAPF64[$288>>3];
   $290 = $Mki0;
   $291 = ((($290)) + 8|0);
   $292 = +HEAPF64[$291>>3];
   $293 = $289 * $292;
   $294 = $287 - $293;
   $cr11 = $294;
   $295 = ((($x0)) + 8|0);
   $296 = +HEAPF64[$295>>3];
   $297 = $Mki0;
   $298 = +HEAPF64[$297>>3];
   $299 = $296 * $298;
   $300 = +HEAPF64[$x0>>3];
   $301 = $Mki0;
   $302 = ((($301)) + 8|0);
   $303 = +HEAPF64[$302>>3];
   $304 = $300 * $303;
   $305 = $299 + $304;
   $ci12 = $305;
   $306 = $cr11;
   HEAPF64[$temp>>3] = $306;
   $307 = $ci12;
   $308 = ((($temp)) + 8|0);
   HEAPF64[$308>>3] = $307;
   $309 = $Mki0;
   $310 = ((($309)) + 16|0);
   $Mki0 = $310;
   $311 = $k;
   $312 = $3;
   $313 = (($312) + ($311<<4)|0);
   $314 = +HEAPF64[$313>>3];
   $315 = +HEAPF64[$temp>>3];
   $316 = $314 - $315;
   $317 = $k;
   $318 = $3;
   $319 = (($318) + ($317<<4)|0);
   HEAPF64[$319>>3] = $316;
   $320 = $k;
   $321 = $3;
   $322 = (($321) + ($320<<4)|0);
   $323 = ((($322)) + 8|0);
   $324 = +HEAPF64[$323>>3];
   $325 = ((($temp)) + 8|0);
   $326 = +HEAPF64[$325>>3];
   $327 = $324 - $326;
   $328 = $k;
   $329 = $3;
   $330 = (($329) + ($328<<4)|0);
   $331 = ((($330)) + 8|0);
   HEAPF64[$331>>3] = $327;
   $332 = +HEAPF64[$x1>>3];
   $333 = $Mki1;
   $334 = +HEAPF64[$333>>3];
   $335 = $332 * $334;
   $336 = ((($x1)) + 8|0);
   $337 = +HEAPF64[$336>>3];
   $338 = $Mki1;
   $339 = ((($338)) + 8|0);
   $340 = +HEAPF64[$339>>3];
   $341 = $337 * $340;
   $342 = $335 - $341;
   $cr13 = $342;
   $343 = ((($x1)) + 8|0);
   $344 = +HEAPF64[$343>>3];
   $345 = $Mki1;
   $346 = +HEAPF64[$345>>3];
   $347 = $344 * $346;
   $348 = +HEAPF64[$x1>>3];
   $349 = $Mki1;
   $350 = ((($349)) + 8|0);
   $351 = +HEAPF64[$350>>3];
   $352 = $348 * $351;
   $353 = $347 + $352;
   $ci14 = $353;
   $354 = $cr13;
   HEAPF64[$temp>>3] = $354;
   $355 = $ci14;
   $356 = ((($temp)) + 8|0);
   HEAPF64[$356>>3] = $355;
   $357 = $Mki1;
   $358 = ((($357)) + 16|0);
   $Mki1 = $358;
   $359 = $k;
   $360 = $3;
   $361 = (($360) + ($359<<4)|0);
   $362 = +HEAPF64[$361>>3];
   $363 = +HEAPF64[$temp>>3];
   $364 = $362 - $363;
   $365 = $k;
   $366 = $3;
   $367 = (($366) + ($365<<4)|0);
   HEAPF64[$367>>3] = $364;
   $368 = $k;
   $369 = $3;
   $370 = (($369) + ($368<<4)|0);
   $371 = ((($370)) + 8|0);
   $372 = +HEAPF64[$371>>3];
   $373 = ((($temp)) + 8|0);
   $374 = +HEAPF64[$373>>3];
   $375 = $372 - $374;
   $376 = $k;
   $377 = $3;
   $378 = (($377) + ($376<<4)|0);
   $379 = ((($378)) + 8|0);
   HEAPF64[$379>>3] = $375;
   $380 = +HEAPF64[$x2>>3];
   $381 = $Mki2;
   $382 = +HEAPF64[$381>>3];
   $383 = $380 * $382;
   $384 = ((($x2)) + 8|0);
   $385 = +HEAPF64[$384>>3];
   $386 = $Mki2;
   $387 = ((($386)) + 8|0);
   $388 = +HEAPF64[$387>>3];
   $389 = $385 * $388;
   $390 = $383 - $389;
   $cr15 = $390;
   $391 = ((($x2)) + 8|0);
   $392 = +HEAPF64[$391>>3];
   $393 = $Mki2;
   $394 = +HEAPF64[$393>>3];
   $395 = $392 * $394;
   $396 = +HEAPF64[$x2>>3];
   $397 = $Mki2;
   $398 = ((($397)) + 8|0);
   $399 = +HEAPF64[$398>>3];
   $400 = $396 * $399;
   $401 = $395 + $400;
   $ci16 = $401;
   $402 = $cr15;
   HEAPF64[$temp>>3] = $402;
   $403 = $ci16;
   $404 = ((($temp)) + 8|0);
   HEAPF64[$404>>3] = $403;
   $405 = $Mki2;
   $406 = ((($405)) + 16|0);
   $Mki2 = $406;
   $407 = $k;
   $408 = $3;
   $409 = (($408) + ($407<<4)|0);
   $410 = +HEAPF64[$409>>3];
   $411 = +HEAPF64[$temp>>3];
   $412 = $410 - $411;
   $413 = $k;
   $414 = $3;
   $415 = (($414) + ($413<<4)|0);
   HEAPF64[$415>>3] = $412;
   $416 = $k;
   $417 = $3;
   $418 = (($417) + ($416<<4)|0);
   $419 = ((($418)) + 8|0);
   $420 = +HEAPF64[$419>>3];
   $421 = ((($temp)) + 8|0);
   $422 = +HEAPF64[$421>>3];
   $423 = $420 - $422;
   $424 = $k;
   $425 = $3;
   $426 = (($425) + ($424<<4)|0);
   $427 = ((($426)) + 8|0);
   HEAPF64[$427>>3] = $423;
   $428 = +HEAPF64[$x3>>3];
   $429 = $Mki3;
   $430 = +HEAPF64[$429>>3];
   $431 = $428 * $430;
   $432 = ((($x3)) + 8|0);
   $433 = +HEAPF64[$432>>3];
   $434 = $Mki3;
   $435 = ((($434)) + 8|0);
   $436 = +HEAPF64[$435>>3];
   $437 = $433 * $436;
   $438 = $431 - $437;
   $cr17 = $438;
   $439 = ((($x3)) + 8|0);
   $440 = +HEAPF64[$439>>3];
   $441 = $Mki3;
   $442 = +HEAPF64[$441>>3];
   $443 = $440 * $442;
   $444 = +HEAPF64[$x3>>3];
   $445 = $Mki3;
   $446 = ((($445)) + 8|0);
   $447 = +HEAPF64[$446>>3];
   $448 = $444 * $447;
   $449 = $443 + $448;
   $ci18 = $449;
   $450 = $cr17;
   HEAPF64[$temp>>3] = $450;
   $451 = $ci18;
   $452 = ((($temp)) + 8|0);
   HEAPF64[$452>>3] = $451;
   $453 = $Mki3;
   $454 = ((($453)) + 16|0);
   $Mki3 = $454;
   $455 = $k;
   $456 = $3;
   $457 = (($456) + ($455<<4)|0);
   $458 = +HEAPF64[$457>>3];
   $459 = +HEAPF64[$temp>>3];
   $460 = $458 - $459;
   $461 = $k;
   $462 = $3;
   $463 = (($462) + ($461<<4)|0);
   HEAPF64[$463>>3] = $460;
   $464 = $k;
   $465 = $3;
   $466 = (($465) + ($464<<4)|0);
   $467 = ((($466)) + 8|0);
   $468 = +HEAPF64[$467>>3];
   $469 = ((($temp)) + 8|0);
   $470 = +HEAPF64[$469>>3];
   $471 = $468 - $470;
   $472 = $k;
   $473 = $3;
   $474 = (($473) + ($472<<4)|0);
   $475 = ((($474)) + 8|0);
   HEAPF64[$475>>3] = $471;
   $476 = $k;
   $477 = (($476) + 1)|0;
   $k = $477;
  }
  $478 = $0;
  $479 = $478<<2;
  $480 = (($479) + 4)|0;
  $481 = $M0;
  $482 = (($481) + ($480<<4)|0);
  $M0 = $482;
 }
 $483 = $firstcol;
 $484 = $1;
 $485 = (($484) - 1)|0;
 $486 = ($483|0)<($485|0);
 if (!($486)) {
  STACKTOP = sp;return;
 }
 $487 = $M0;
 $488 = ((($487)) + 16|0);
 $Mki0 = $488;
 $489 = $Mki0;
 $490 = $0;
 $491 = (($489) + ($490<<4)|0);
 $492 = ((($491)) + 16|0);
 $Mki1 = $492;
 $493 = $firstcol;
 $494 = $3;
 $495 = (($494) + ($493<<4)|0);
 ;HEAP32[$x0>>2]=HEAP32[$495>>2]|0;HEAP32[$x0+4>>2]=HEAP32[$495+4>>2]|0;HEAP32[$x0+8>>2]=HEAP32[$495+8>>2]|0;HEAP32[$x0+12>>2]=HEAP32[$495+12>>2]|0;
 $496 = +HEAPF64[$x0>>3];
 $497 = $Mki0;
 $498 = +HEAPF64[$497>>3];
 $499 = $496 * $498;
 $500 = ((($x0)) + 8|0);
 $501 = +HEAPF64[$500>>3];
 $502 = $Mki0;
 $503 = ((($502)) + 8|0);
 $504 = +HEAPF64[$503>>3];
 $505 = $501 * $504;
 $506 = $499 - $505;
 $cr19 = $506;
 $507 = ((($x0)) + 8|0);
 $508 = +HEAPF64[$507>>3];
 $509 = $Mki0;
 $510 = +HEAPF64[$509>>3];
 $511 = $508 * $510;
 $512 = +HEAPF64[$x0>>3];
 $513 = $Mki0;
 $514 = ((($513)) + 8|0);
 $515 = +HEAPF64[$514>>3];
 $516 = $512 * $515;
 $517 = $511 + $516;
 $ci20 = $517;
 $518 = $cr19;
 HEAPF64[$temp>>3] = $518;
 $519 = $ci20;
 $520 = ((($temp)) + 8|0);
 HEAPF64[$520>>3] = $519;
 $521 = $Mki0;
 $522 = ((($521)) + 16|0);
 $Mki0 = $522;
 $523 = $firstcol;
 $524 = (($523) + 1)|0;
 $525 = $3;
 $526 = (($525) + ($524<<4)|0);
 $527 = +HEAPF64[$526>>3];
 $528 = +HEAPF64[$temp>>3];
 $529 = $527 - $528;
 HEAPF64[$x1>>3] = $529;
 $530 = $firstcol;
 $531 = (($530) + 1)|0;
 $532 = $3;
 $533 = (($532) + ($531<<4)|0);
 $534 = ((($533)) + 8|0);
 $535 = +HEAPF64[$534>>3];
 $536 = ((($temp)) + 8|0);
 $537 = +HEAPF64[$536>>3];
 $538 = $535 - $537;
 $539 = ((($x1)) + 8|0);
 HEAPF64[$539>>3] = $538;
 $540 = $firstcol;
 $541 = (($540) + 1)|0;
 $firstcol = $541;
 $542 = $3;
 $543 = (($542) + ($541<<4)|0);
 ;HEAP32[$543>>2]=HEAP32[$x1>>2]|0;HEAP32[$543+4>>2]=HEAP32[$x1+4>>2]|0;HEAP32[$543+8>>2]=HEAP32[$x1+8>>2]|0;HEAP32[$543+12>>2]=HEAP32[$x1+12>>2]|0;
 $544 = $firstcol;
 $545 = (($544) + 1)|0;
 $firstcol = $545;
 $546 = $firstcol;
 $k = $546;
 while(1) {
  $547 = $k;
  $548 = $1;
  $549 = ($547|0)<($548|0);
  if (!($549)) {
   break;
  }
  $550 = +HEAPF64[$x0>>3];
  $551 = $Mki0;
  $552 = +HEAPF64[$551>>3];
  $553 = $550 * $552;
  $554 = ((($x0)) + 8|0);
  $555 = +HEAPF64[$554>>3];
  $556 = $Mki0;
  $557 = ((($556)) + 8|0);
  $558 = +HEAPF64[$557>>3];
  $559 = $555 * $558;
  $560 = $553 - $559;
  $cr21 = $560;
  $561 = ((($x0)) + 8|0);
  $562 = +HEAPF64[$561>>3];
  $563 = $Mki0;
  $564 = +HEAPF64[$563>>3];
  $565 = $562 * $564;
  $566 = +HEAPF64[$x0>>3];
  $567 = $Mki0;
  $568 = ((($567)) + 8|0);
  $569 = +HEAPF64[$568>>3];
  $570 = $566 * $569;
  $571 = $565 + $570;
  $ci22 = $571;
  $572 = $cr21;
  HEAPF64[$temp>>3] = $572;
  $573 = $ci22;
  $574 = ((($temp)) + 8|0);
  HEAPF64[$574>>3] = $573;
  $575 = $Mki0;
  $576 = ((($575)) + 16|0);
  $Mki0 = $576;
  $577 = $k;
  $578 = $3;
  $579 = (($578) + ($577<<4)|0);
  $580 = +HEAPF64[$579>>3];
  $581 = +HEAPF64[$temp>>3];
  $582 = $580 - $581;
  $583 = $k;
  $584 = $3;
  $585 = (($584) + ($583<<4)|0);
  HEAPF64[$585>>3] = $582;
  $586 = $k;
  $587 = $3;
  $588 = (($587) + ($586<<4)|0);
  $589 = ((($588)) + 8|0);
  $590 = +HEAPF64[$589>>3];
  $591 = ((($temp)) + 8|0);
  $592 = +HEAPF64[$591>>3];
  $593 = $590 - $592;
  $594 = $k;
  $595 = $3;
  $596 = (($595) + ($594<<4)|0);
  $597 = ((($596)) + 8|0);
  HEAPF64[$597>>3] = $593;
  $598 = +HEAPF64[$x1>>3];
  $599 = $Mki1;
  $600 = +HEAPF64[$599>>3];
  $601 = $598 * $600;
  $602 = ((($x1)) + 8|0);
  $603 = +HEAPF64[$602>>3];
  $604 = $Mki1;
  $605 = ((($604)) + 8|0);
  $606 = +HEAPF64[$605>>3];
  $607 = $603 * $606;
  $608 = $601 - $607;
  $cr23 = $608;
  $609 = ((($x1)) + 8|0);
  $610 = +HEAPF64[$609>>3];
  $611 = $Mki1;
  $612 = +HEAPF64[$611>>3];
  $613 = $610 * $612;
  $614 = +HEAPF64[$x1>>3];
  $615 = $Mki1;
  $616 = ((($615)) + 8|0);
  $617 = +HEAPF64[$616>>3];
  $618 = $614 * $617;
  $619 = $613 + $618;
  $ci24 = $619;
  $620 = $cr23;
  HEAPF64[$temp>>3] = $620;
  $621 = $ci24;
  $622 = ((($temp)) + 8|0);
  HEAPF64[$622>>3] = $621;
  $623 = $Mki1;
  $624 = ((($623)) + 16|0);
  $Mki1 = $624;
  $625 = $k;
  $626 = $3;
  $627 = (($626) + ($625<<4)|0);
  $628 = +HEAPF64[$627>>3];
  $629 = +HEAPF64[$temp>>3];
  $630 = $628 - $629;
  $631 = $k;
  $632 = $3;
  $633 = (($632) + ($631<<4)|0);
  HEAPF64[$633>>3] = $630;
  $634 = $k;
  $635 = $3;
  $636 = (($635) + ($634<<4)|0);
  $637 = ((($636)) + 8|0);
  $638 = +HEAPF64[$637>>3];
  $639 = ((($temp)) + 8|0);
  $640 = +HEAPF64[$639>>3];
  $641 = $638 - $640;
  $642 = $k;
  $643 = $3;
  $644 = (($643) + ($642<<4)|0);
  $645 = ((($644)) + 8|0);
  HEAPF64[$645>>3] = $641;
  $646 = $k;
  $647 = (($646) + 1)|0;
  $k = $647;
 }
 STACKTOP = sp;return;
}
function _zusolve($ldm,$ncol,$M,$rhs) {
 $ldm = $ldm|0;
 $ncol = $ncol|0;
 $M = $M|0;
 $rhs = $rhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0.0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0.0, $34 = 0.0, $35 = 0, $36 = 0.0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0.0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0.0, $78 = 0.0, $79 = 0.0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0.0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $ci = 0.0, $cr = 0.0, $irow = 0, $j = 0, $jcol = 0, $temp = 0, $xj = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $xj = sp + 32|0;
 $temp = sp + 16|0;
 $0 = $ldm;
 $1 = $ncol;
 $2 = $M;
 $3 = $rhs;
 $4 = $1;
 $5 = (($4) - 1)|0;
 $jcol = $5;
 $j = 0;
 while(1) {
  $6 = $j;
  $7 = $1;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $jcol;
  $10 = $3;
  $11 = (($10) + ($9<<4)|0);
  $12 = $jcol;
  $13 = $jcol;
  $14 = $0;
  $15 = Math_imul($13, $14)|0;
  $16 = (($12) + ($15))|0;
  $17 = $2;
  $18 = (($17) + ($16<<4)|0);
  _z_div($xj,$11,$18);
  $19 = $jcol;
  $20 = $3;
  $21 = (($20) + ($19<<4)|0);
  ;HEAP32[$21>>2]=HEAP32[$xj>>2]|0;HEAP32[$21+4>>2]=HEAP32[$xj+4>>2]|0;HEAP32[$21+8>>2]=HEAP32[$xj+8>>2]|0;HEAP32[$21+12>>2]=HEAP32[$xj+12>>2]|0;
  $irow = 0;
  while(1) {
   $22 = $irow;
   $23 = $jcol;
   $24 = ($22|0)<($23|0);
   if (!($24)) {
    break;
   }
   $25 = +HEAPF64[$xj>>3];
   $26 = $irow;
   $27 = $jcol;
   $28 = $0;
   $29 = Math_imul($27, $28)|0;
   $30 = (($26) + ($29))|0;
   $31 = $2;
   $32 = (($31) + ($30<<4)|0);
   $33 = +HEAPF64[$32>>3];
   $34 = $25 * $33;
   $35 = ((($xj)) + 8|0);
   $36 = +HEAPF64[$35>>3];
   $37 = $irow;
   $38 = $jcol;
   $39 = $0;
   $40 = Math_imul($38, $39)|0;
   $41 = (($37) + ($40))|0;
   $42 = $2;
   $43 = (($42) + ($41<<4)|0);
   $44 = ((($43)) + 8|0);
   $45 = +HEAPF64[$44>>3];
   $46 = $36 * $45;
   $47 = $34 - $46;
   $cr = $47;
   $48 = ((($xj)) + 8|0);
   $49 = +HEAPF64[$48>>3];
   $50 = $irow;
   $51 = $jcol;
   $52 = $0;
   $53 = Math_imul($51, $52)|0;
   $54 = (($50) + ($53))|0;
   $55 = $2;
   $56 = (($55) + ($54<<4)|0);
   $57 = +HEAPF64[$56>>3];
   $58 = $49 * $57;
   $59 = +HEAPF64[$xj>>3];
   $60 = $irow;
   $61 = $jcol;
   $62 = $0;
   $63 = Math_imul($61, $62)|0;
   $64 = (($60) + ($63))|0;
   $65 = $2;
   $66 = (($65) + ($64<<4)|0);
   $67 = ((($66)) + 8|0);
   $68 = +HEAPF64[$67>>3];
   $69 = $59 * $68;
   $70 = $58 + $69;
   $ci = $70;
   $71 = $cr;
   HEAPF64[$temp>>3] = $71;
   $72 = $ci;
   $73 = ((($temp)) + 8|0);
   HEAPF64[$73>>3] = $72;
   $74 = $irow;
   $75 = $3;
   $76 = (($75) + ($74<<4)|0);
   $77 = +HEAPF64[$76>>3];
   $78 = +HEAPF64[$temp>>3];
   $79 = $77 - $78;
   $80 = $irow;
   $81 = $3;
   $82 = (($81) + ($80<<4)|0);
   HEAPF64[$82>>3] = $79;
   $83 = $irow;
   $84 = $3;
   $85 = (($84) + ($83<<4)|0);
   $86 = ((($85)) + 8|0);
   $87 = +HEAPF64[$86>>3];
   $88 = ((($temp)) + 8|0);
   $89 = +HEAPF64[$88>>3];
   $90 = $87 - $89;
   $91 = $irow;
   $92 = $3;
   $93 = (($92) + ($91<<4)|0);
   $94 = ((($93)) + 8|0);
   HEAPF64[$94>>3] = $90;
   $95 = $irow;
   $96 = (($95) + 1)|0;
   $irow = $96;
  }
  $97 = $jcol;
  $98 = (($97) + -1)|0;
  $jcol = $98;
  $99 = $j;
  $100 = (($99) + 1)|0;
  $j = $100;
 }
 STACKTOP = sp;return;
}
function _zmatvec($ldm,$nrow,$ncol,$M,$vec,$Mxvec) {
 $ldm = $ldm|0;
 $nrow = $nrow|0;
 $ncol = $ncol|0;
 $M = $M|0;
 $vec = $vec|0;
 $Mxvec = $Mxvec|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0.0, $103 = 0.0, $104 = 0.0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0.0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0.0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0.0, $137 = 0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0, $141 = 0.0, $142 = 0, $143 = 0, $144 = 0.0, $145 = 0.0, $146 = 0.0, $147 = 0, $148 = 0.0, $149 = 0, $15 = 0, $150 = 0.0, $151 = 0.0;
 var $152 = 0.0, $153 = 0, $154 = 0, $155 = 0.0, $156 = 0.0, $157 = 0.0, $158 = 0.0, $159 = 0.0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0.0, $167 = 0.0, $168 = 0.0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0.0, $177 = 0, $178 = 0.0, $179 = 0.0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0.0, $185 = 0, $186 = 0.0, $187 = 0.0, $188 = 0;
 var $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0.0, $193 = 0.0, $194 = 0.0, $195 = 0, $196 = 0.0, $197 = 0, $198 = 0.0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0.0, $201 = 0, $202 = 0, $203 = 0.0, $204 = 0.0, $205 = 0.0;
 var $206 = 0.0, $207 = 0.0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0.0, $215 = 0.0, $216 = 0.0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0.0, $225 = 0, $226 = 0.0, $227 = 0.0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0.0, $25 = 0, $250 = 0, $251 = 0.0, $252 = 0.0, $253 = 0, $254 = 0.0, $255 = 0, $256 = 0, $257 = 0.0, $258 = 0.0, $259 = 0.0, $26 = 0;
 var $260 = 0, $261 = 0.0, $262 = 0, $263 = 0.0, $264 = 0.0, $265 = 0.0, $266 = 0, $267 = 0, $268 = 0.0, $269 = 0.0, $27 = 0, $270 = 0.0, $271 = 0.0, $272 = 0.0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0.0, $28 = 0, $280 = 0.0, $281 = 0.0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0.0, $29 = 0, $290 = 0, $291 = 0.0, $292 = 0.0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0, $42 = 0.0;
 var $43 = 0.0, $44 = 0, $45 = 0.0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0.0, $6 = 0, $60 = 0.0;
 var $61 = 0.0, $62 = 0.0, $63 = 0.0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0.0, $81 = 0, $82 = 0.0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0.0, $89 = 0, $9 = 0, $90 = 0.0, $91 = 0.0, $92 = 0, $93 = 0.0, $94 = 0, $95 = 0, $96 = 0.0, $97 = 0.0;
 var $98 = 0.0, $99 = 0, $M0 = 0, $Mki0 = 0, $Mki1 = 0, $Mki2 = 0, $Mki3 = 0, $ci = 0.0, $ci2 = 0.0, $ci4 = 0.0, $ci6 = 0.0, $ci8 = 0.0, $cr = 0.0, $cr1 = 0.0, $cr3 = 0.0, $cr5 = 0.0, $cr7 = 0.0, $firstcol = 0, $k = 0, $temp = 0;
 var $vi0 = 0, $vi1 = 0, $vi2 = 0, $vi3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vi0 = sp + 144|0;
 $vi1 = sp + 128|0;
 $vi2 = sp + 112|0;
 $vi3 = sp + 96|0;
 $temp = sp + 80|0;
 $0 = $ldm;
 $1 = $nrow;
 $2 = $ncol;
 $3 = $M;
 $4 = $vec;
 $5 = $Mxvec;
 $firstcol = 0;
 $6 = $3;
 $M0 = $6;
 while(1) {
  $7 = $firstcol;
  $8 = $2;
  $9 = (($8) - 3)|0;
  $10 = ($7|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = $M0;
  $Mki0 = $11;
  $12 = $Mki0;
  $13 = $0;
  $14 = (($12) + ($13<<4)|0);
  $Mki1 = $14;
  $15 = $Mki1;
  $16 = $0;
  $17 = (($15) + ($16<<4)|0);
  $Mki2 = $17;
  $18 = $Mki2;
  $19 = $0;
  $20 = (($18) + ($19<<4)|0);
  $Mki3 = $20;
  $21 = $firstcol;
  $22 = (($21) + 1)|0;
  $firstcol = $22;
  $23 = $4;
  $24 = (($23) + ($21<<4)|0);
  ;HEAP32[$vi0>>2]=HEAP32[$24>>2]|0;HEAP32[$vi0+4>>2]=HEAP32[$24+4>>2]|0;HEAP32[$vi0+8>>2]=HEAP32[$24+8>>2]|0;HEAP32[$vi0+12>>2]=HEAP32[$24+12>>2]|0;
  $25 = $firstcol;
  $26 = (($25) + 1)|0;
  $firstcol = $26;
  $27 = $4;
  $28 = (($27) + ($25<<4)|0);
  ;HEAP32[$vi1>>2]=HEAP32[$28>>2]|0;HEAP32[$vi1+4>>2]=HEAP32[$28+4>>2]|0;HEAP32[$vi1+8>>2]=HEAP32[$28+8>>2]|0;HEAP32[$vi1+12>>2]=HEAP32[$28+12>>2]|0;
  $29 = $firstcol;
  $30 = (($29) + 1)|0;
  $firstcol = $30;
  $31 = $4;
  $32 = (($31) + ($29<<4)|0);
  ;HEAP32[$vi2>>2]=HEAP32[$32>>2]|0;HEAP32[$vi2+4>>2]=HEAP32[$32+4>>2]|0;HEAP32[$vi2+8>>2]=HEAP32[$32+8>>2]|0;HEAP32[$vi2+12>>2]=HEAP32[$32+12>>2]|0;
  $33 = $firstcol;
  $34 = (($33) + 1)|0;
  $firstcol = $34;
  $35 = $4;
  $36 = (($35) + ($33<<4)|0);
  ;HEAP32[$vi3>>2]=HEAP32[$36>>2]|0;HEAP32[$vi3+4>>2]=HEAP32[$36+4>>2]|0;HEAP32[$vi3+8>>2]=HEAP32[$36+8>>2]|0;HEAP32[$vi3+12>>2]=HEAP32[$36+12>>2]|0;
  $k = 0;
  while(1) {
   $37 = $k;
   $38 = $1;
   $39 = ($37|0)<($38|0);
   if (!($39)) {
    break;
   }
   $40 = +HEAPF64[$vi0>>3];
   $41 = $Mki0;
   $42 = +HEAPF64[$41>>3];
   $43 = $40 * $42;
   $44 = ((($vi0)) + 8|0);
   $45 = +HEAPF64[$44>>3];
   $46 = $Mki0;
   $47 = ((($46)) + 8|0);
   $48 = +HEAPF64[$47>>3];
   $49 = $45 * $48;
   $50 = $43 - $49;
   $cr = $50;
   $51 = ((($vi0)) + 8|0);
   $52 = +HEAPF64[$51>>3];
   $53 = $Mki0;
   $54 = +HEAPF64[$53>>3];
   $55 = $52 * $54;
   $56 = +HEAPF64[$vi0>>3];
   $57 = $Mki0;
   $58 = ((($57)) + 8|0);
   $59 = +HEAPF64[$58>>3];
   $60 = $56 * $59;
   $61 = $55 + $60;
   $ci = $61;
   $62 = $cr;
   HEAPF64[$temp>>3] = $62;
   $63 = $ci;
   $64 = ((($temp)) + 8|0);
   HEAPF64[$64>>3] = $63;
   $65 = $Mki0;
   $66 = ((($65)) + 16|0);
   $Mki0 = $66;
   $67 = $k;
   $68 = $5;
   $69 = (($68) + ($67<<4)|0);
   $70 = +HEAPF64[$69>>3];
   $71 = +HEAPF64[$temp>>3];
   $72 = $70 + $71;
   $73 = $k;
   $74 = $5;
   $75 = (($74) + ($73<<4)|0);
   HEAPF64[$75>>3] = $72;
   $76 = $k;
   $77 = $5;
   $78 = (($77) + ($76<<4)|0);
   $79 = ((($78)) + 8|0);
   $80 = +HEAPF64[$79>>3];
   $81 = ((($temp)) + 8|0);
   $82 = +HEAPF64[$81>>3];
   $83 = $80 + $82;
   $84 = $k;
   $85 = $5;
   $86 = (($85) + ($84<<4)|0);
   $87 = ((($86)) + 8|0);
   HEAPF64[$87>>3] = $83;
   $88 = +HEAPF64[$vi1>>3];
   $89 = $Mki1;
   $90 = +HEAPF64[$89>>3];
   $91 = $88 * $90;
   $92 = ((($vi1)) + 8|0);
   $93 = +HEAPF64[$92>>3];
   $94 = $Mki1;
   $95 = ((($94)) + 8|0);
   $96 = +HEAPF64[$95>>3];
   $97 = $93 * $96;
   $98 = $91 - $97;
   $cr1 = $98;
   $99 = ((($vi1)) + 8|0);
   $100 = +HEAPF64[$99>>3];
   $101 = $Mki1;
   $102 = +HEAPF64[$101>>3];
   $103 = $100 * $102;
   $104 = +HEAPF64[$vi1>>3];
   $105 = $Mki1;
   $106 = ((($105)) + 8|0);
   $107 = +HEAPF64[$106>>3];
   $108 = $104 * $107;
   $109 = $103 + $108;
   $ci2 = $109;
   $110 = $cr1;
   HEAPF64[$temp>>3] = $110;
   $111 = $ci2;
   $112 = ((($temp)) + 8|0);
   HEAPF64[$112>>3] = $111;
   $113 = $Mki1;
   $114 = ((($113)) + 16|0);
   $Mki1 = $114;
   $115 = $k;
   $116 = $5;
   $117 = (($116) + ($115<<4)|0);
   $118 = +HEAPF64[$117>>3];
   $119 = +HEAPF64[$temp>>3];
   $120 = $118 + $119;
   $121 = $k;
   $122 = $5;
   $123 = (($122) + ($121<<4)|0);
   HEAPF64[$123>>3] = $120;
   $124 = $k;
   $125 = $5;
   $126 = (($125) + ($124<<4)|0);
   $127 = ((($126)) + 8|0);
   $128 = +HEAPF64[$127>>3];
   $129 = ((($temp)) + 8|0);
   $130 = +HEAPF64[$129>>3];
   $131 = $128 + $130;
   $132 = $k;
   $133 = $5;
   $134 = (($133) + ($132<<4)|0);
   $135 = ((($134)) + 8|0);
   HEAPF64[$135>>3] = $131;
   $136 = +HEAPF64[$vi2>>3];
   $137 = $Mki2;
   $138 = +HEAPF64[$137>>3];
   $139 = $136 * $138;
   $140 = ((($vi2)) + 8|0);
   $141 = +HEAPF64[$140>>3];
   $142 = $Mki2;
   $143 = ((($142)) + 8|0);
   $144 = +HEAPF64[$143>>3];
   $145 = $141 * $144;
   $146 = $139 - $145;
   $cr3 = $146;
   $147 = ((($vi2)) + 8|0);
   $148 = +HEAPF64[$147>>3];
   $149 = $Mki2;
   $150 = +HEAPF64[$149>>3];
   $151 = $148 * $150;
   $152 = +HEAPF64[$vi2>>3];
   $153 = $Mki2;
   $154 = ((($153)) + 8|0);
   $155 = +HEAPF64[$154>>3];
   $156 = $152 * $155;
   $157 = $151 + $156;
   $ci4 = $157;
   $158 = $cr3;
   HEAPF64[$temp>>3] = $158;
   $159 = $ci4;
   $160 = ((($temp)) + 8|0);
   HEAPF64[$160>>3] = $159;
   $161 = $Mki2;
   $162 = ((($161)) + 16|0);
   $Mki2 = $162;
   $163 = $k;
   $164 = $5;
   $165 = (($164) + ($163<<4)|0);
   $166 = +HEAPF64[$165>>3];
   $167 = +HEAPF64[$temp>>3];
   $168 = $166 + $167;
   $169 = $k;
   $170 = $5;
   $171 = (($170) + ($169<<4)|0);
   HEAPF64[$171>>3] = $168;
   $172 = $k;
   $173 = $5;
   $174 = (($173) + ($172<<4)|0);
   $175 = ((($174)) + 8|0);
   $176 = +HEAPF64[$175>>3];
   $177 = ((($temp)) + 8|0);
   $178 = +HEAPF64[$177>>3];
   $179 = $176 + $178;
   $180 = $k;
   $181 = $5;
   $182 = (($181) + ($180<<4)|0);
   $183 = ((($182)) + 8|0);
   HEAPF64[$183>>3] = $179;
   $184 = +HEAPF64[$vi3>>3];
   $185 = $Mki3;
   $186 = +HEAPF64[$185>>3];
   $187 = $184 * $186;
   $188 = ((($vi3)) + 8|0);
   $189 = +HEAPF64[$188>>3];
   $190 = $Mki3;
   $191 = ((($190)) + 8|0);
   $192 = +HEAPF64[$191>>3];
   $193 = $189 * $192;
   $194 = $187 - $193;
   $cr5 = $194;
   $195 = ((($vi3)) + 8|0);
   $196 = +HEAPF64[$195>>3];
   $197 = $Mki3;
   $198 = +HEAPF64[$197>>3];
   $199 = $196 * $198;
   $200 = +HEAPF64[$vi3>>3];
   $201 = $Mki3;
   $202 = ((($201)) + 8|0);
   $203 = +HEAPF64[$202>>3];
   $204 = $200 * $203;
   $205 = $199 + $204;
   $ci6 = $205;
   $206 = $cr5;
   HEAPF64[$temp>>3] = $206;
   $207 = $ci6;
   $208 = ((($temp)) + 8|0);
   HEAPF64[$208>>3] = $207;
   $209 = $Mki3;
   $210 = ((($209)) + 16|0);
   $Mki3 = $210;
   $211 = $k;
   $212 = $5;
   $213 = (($212) + ($211<<4)|0);
   $214 = +HEAPF64[$213>>3];
   $215 = +HEAPF64[$temp>>3];
   $216 = $214 + $215;
   $217 = $k;
   $218 = $5;
   $219 = (($218) + ($217<<4)|0);
   HEAPF64[$219>>3] = $216;
   $220 = $k;
   $221 = $5;
   $222 = (($221) + ($220<<4)|0);
   $223 = ((($222)) + 8|0);
   $224 = +HEAPF64[$223>>3];
   $225 = ((($temp)) + 8|0);
   $226 = +HEAPF64[$225>>3];
   $227 = $224 + $226;
   $228 = $k;
   $229 = $5;
   $230 = (($229) + ($228<<4)|0);
   $231 = ((($230)) + 8|0);
   HEAPF64[$231>>3] = $227;
   $232 = $k;
   $233 = (($232) + 1)|0;
   $k = $233;
  }
  $234 = $0;
  $235 = $234<<2;
  $236 = $M0;
  $237 = (($236) + ($235<<4)|0);
  $M0 = $237;
 }
 while(1) {
  $238 = $firstcol;
  $239 = $2;
  $240 = ($238|0)<($239|0);
  if (!($240)) {
   break;
  }
  $241 = $M0;
  $Mki0 = $241;
  $242 = $firstcol;
  $243 = (($242) + 1)|0;
  $firstcol = $243;
  $244 = $4;
  $245 = (($244) + ($242<<4)|0);
  ;HEAP32[$vi0>>2]=HEAP32[$245>>2]|0;HEAP32[$vi0+4>>2]=HEAP32[$245+4>>2]|0;HEAP32[$vi0+8>>2]=HEAP32[$245+8>>2]|0;HEAP32[$vi0+12>>2]=HEAP32[$245+12>>2]|0;
  $k = 0;
  while(1) {
   $246 = $k;
   $247 = $1;
   $248 = ($246|0)<($247|0);
   if (!($248)) {
    break;
   }
   $249 = +HEAPF64[$vi0>>3];
   $250 = $Mki0;
   $251 = +HEAPF64[$250>>3];
   $252 = $249 * $251;
   $253 = ((($vi0)) + 8|0);
   $254 = +HEAPF64[$253>>3];
   $255 = $Mki0;
   $256 = ((($255)) + 8|0);
   $257 = +HEAPF64[$256>>3];
   $258 = $254 * $257;
   $259 = $252 - $258;
   $cr7 = $259;
   $260 = ((($vi0)) + 8|0);
   $261 = +HEAPF64[$260>>3];
   $262 = $Mki0;
   $263 = +HEAPF64[$262>>3];
   $264 = $261 * $263;
   $265 = +HEAPF64[$vi0>>3];
   $266 = $Mki0;
   $267 = ((($266)) + 8|0);
   $268 = +HEAPF64[$267>>3];
   $269 = $265 * $268;
   $270 = $264 + $269;
   $ci8 = $270;
   $271 = $cr7;
   HEAPF64[$temp>>3] = $271;
   $272 = $ci8;
   $273 = ((($temp)) + 8|0);
   HEAPF64[$273>>3] = $272;
   $274 = $Mki0;
   $275 = ((($274)) + 16|0);
   $Mki0 = $275;
   $276 = $k;
   $277 = $5;
   $278 = (($277) + ($276<<4)|0);
   $279 = +HEAPF64[$278>>3];
   $280 = +HEAPF64[$temp>>3];
   $281 = $279 + $280;
   $282 = $k;
   $283 = $5;
   $284 = (($283) + ($282<<4)|0);
   HEAPF64[$284>>3] = $281;
   $285 = $k;
   $286 = $5;
   $287 = (($286) + ($285<<4)|0);
   $288 = ((($287)) + 8|0);
   $289 = +HEAPF64[$288>>3];
   $290 = ((($temp)) + 8|0);
   $291 = +HEAPF64[$290>>3];
   $292 = $289 + $291;
   $293 = $k;
   $294 = $5;
   $295 = (($294) + ($293<<4)|0);
   $296 = ((($295)) + 8|0);
   HEAPF64[$296>>3] = $292;
   $297 = $k;
   $298 = (($297) + 1)|0;
   $k = $298;
  }
  $299 = $0;
  $300 = $M0;
  $301 = (($300) + ($299<<4)|0);
  $M0 = $301;
 }
 STACKTOP = sp;return;
}
function _zlinsol($iopt,$n,$nnz,$nrhs,$dvalues,$rowind,$colptr,$db,$ldb,$f_factors,$permc_spec,$trans,$info) {
 $iopt = $iopt|0;
 $n = $n|0;
 $nnz = $nnz|0;
 $nrhs = $nrhs|0;
 $dvalues = $dvalues|0;
 $rowind = $rowind|0;
 $colptr = $colptr|0;
 $db = $db|0;
 $ldb = $ldb|0;
 $f_factors = $f_factors|0;
 $permc_spec = $permc_spec|0;
 $trans = $trans|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0.0, $135 = 0.0, $136 = 0.0, $137 = 0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0.0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0.0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0.0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0.0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0.0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $A = 0, $AC = 0, $B = 0, $L = 0, $LUfactors = 0, $U = 0, $b = 0, $etree = 0, $i = 0, $mem_usage = 0, $options = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0;
 var $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $panel_size = 0, $perm_c = 0, $perm_r = 0, $relax = 0, $stat = 0, $values = 0, $vararg_buffer = 0, $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer22 = 0, $vararg_buffer24 = 0, $vararg_buffer26 = 0, $vararg_buffer30 = 0, $vararg_buffer32 = 0, $vararg_buffer35 = 0, $vararg_ptr29 = 0;
 var $z = 0, $z1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 464|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer35 = sp + 248|0;
 $vararg_buffer32 = sp + 240|0;
 $vararg_buffer30 = sp + 232|0;
 $vararg_buffer26 = sp + 216|0;
 $vararg_buffer24 = sp + 208|0;
 $vararg_buffer22 = sp + 200|0;
 $vararg_buffer20 = sp + 192|0;
 $vararg_buffer18 = sp + 184|0;
 $vararg_buffer = sp + 176|0;
 $A = sp + 376|0;
 $AC = sp + 352|0;
 $B = sp + 328|0;
 $mem_usage = sp + 288|0;
 $options = sp + 32|0;
 $stat = sp + 264|0;
 $z = sp + 16|0;
 $z1 = sp;
 $0 = $iopt;
 $1 = $n;
 $2 = $nnz;
 $3 = $nrhs;
 $4 = $dvalues;
 $5 = $rowind;
 $6 = $colptr;
 $7 = $db;
 $8 = $ldb;
 $9 = $f_factors;
 $10 = $permc_spec;
 $11 = $trans;
 $12 = $info;
 $13 = $0;
 $14 = ($13|0)!=(3);
 $15 = $1;
 $16 = ($15|0)<=(0);
 $or$cond = $14 & $16;
 if ($or$cond) {
  $17 = $12;
  HEAP32[$17>>2] = -2;
  STACKTOP = sp;return;
 }
 $18 = $0;
 $19 = ($18|0)==(1);
 $20 = $2;
 $21 = ($20|0)<(0);
 $or$cond3 = $19 & $21;
 if ($or$cond3) {
  $22 = $12;
  HEAP32[$22>>2] = -3;
  STACKTOP = sp;return;
 }
 $23 = $0;
 $24 = ($23|0)==(2);
 $25 = $3;
 $26 = ($25|0)<(0);
 $or$cond5 = $24 & $26;
 if ($or$cond5) {
  $27 = $12;
  HEAP32[$27>>2] = -4;
  STACKTOP = sp;return;
 }
 $28 = $0;
 $29 = ($28|0)==(1);
 $30 = $4;
 $31 = ($30|0)==(0|0);
 $or$cond7 = $29 & $31;
 if ($or$cond7) {
  $32 = $12;
  HEAP32[$32>>2] = -5;
  STACKTOP = sp;return;
 }
 $33 = $0;
 $34 = ($33|0)==(1);
 $35 = $5;
 $36 = ($35|0)==(0|0);
 $or$cond9 = $34 & $36;
 if ($or$cond9) {
  $37 = $12;
  HEAP32[$37>>2] = -6;
  STACKTOP = sp;return;
 }
 $38 = $0;
 $39 = ($38|0)==(1);
 $40 = $6;
 $41 = ($40|0)==(0|0);
 $or$cond11 = $39 & $41;
 if ($or$cond11) {
  $42 = $12;
  HEAP32[$42>>2] = -7;
  STACKTOP = sp;return;
 }
 $43 = $0;
 $44 = ($43|0)==(1);
 if ($44) {
  $45 = $6;
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)!=(0);
  if ($47) {
   $48 = $12;
   HEAP32[$48>>2] = -7;
   STACKTOP = sp;return;
  }
 }
 $49 = $0;
 $50 = ($49|0)==(2);
 $51 = $7;
 $52 = ($51|0)==(0|0);
 $or$cond13 = $50 & $52;
 if ($or$cond13) {
  $53 = $12;
  HEAP32[$53>>2] = -8;
  STACKTOP = sp;return;
 }
 $54 = $0;
 $55 = ($54|0)==(2);
 $56 = $8;
 $57 = ($56|0)<=(0);
 $or$cond15 = $55 & $57;
 if ($or$cond15) {
  $58 = $12;
  HEAP32[$58>>2] = -9;
  STACKTOP = sp;return;
 }
 $59 = $0;
 $60 = ($59|0)==(2);
 $61 = $9;
 $62 = ($61|0)==(0|0);
 $or$cond17 = $60 & $62;
 if ($or$cond17) {
  $63 = $12;
  HEAP32[$63>>2] = -10;
  STACKTOP = sp;return;
 }
 $64 = $0;
 $65 = ($64|0)==(1);
 if (!($65)) {
  $157 = $0;
  $158 = ($157|0)==(2);
  if (!($158)) {
   $240 = $0;
   $241 = ($240|0)==(3);
   if ($241) {
    $242 = $9;
    $243 = HEAP32[$242>>2]|0;
    $LUfactors = $243;
    $244 = $LUfactors;
    $245 = ((($244)) + 12|0);
    $246 = HEAP32[$245>>2]|0;
    _superlu_free($246);
    $247 = $LUfactors;
    $248 = ((($247)) + 8|0);
    $249 = HEAP32[$248>>2]|0;
    _superlu_free($249);
    $250 = $LUfactors;
    $251 = HEAP32[$250>>2]|0;
    _Destroy_SuperNode_Matrix($251);
    $252 = $LUfactors;
    $253 = ((($252)) + 4|0);
    $254 = HEAP32[$253>>2]|0;
    _Destroy_CompCol_Matrix($254);
    $255 = $LUfactors;
    $256 = HEAP32[$255>>2]|0;
    _superlu_free($256);
    $257 = $LUfactors;
    $258 = ((($257)) + 4|0);
    $259 = HEAP32[$258>>2]|0;
    _superlu_free($259);
    $260 = $LUfactors;
    _superlu_free($260);
    STACKTOP = sp;return;
   } else {
    $261 = HEAP32[_stderr>>2]|0;
    $262 = $0;
    HEAP32[$vararg_buffer35>>2] = $262;
    (_fprintf(($261|0),(3272|0),($vararg_buffer35|0))|0);
    $263 = $12;
    HEAP32[$263>>2] = -1;
    STACKTOP = sp;return;
   }
  }
  _StatInit($stat);
  $159 = $9;
  $160 = HEAP32[$159>>2]|0;
  $LUfactors = $160;
  $161 = $LUfactors;
  $162 = HEAP32[$161>>2]|0;
  $L = $162;
  $163 = $LUfactors;
  $164 = ((($163)) + 4|0);
  $165 = HEAP32[$164>>2]|0;
  $U = $165;
  $166 = $LUfactors;
  $167 = ((($166)) + 8|0);
  $168 = HEAP32[$167>>2]|0;
  $perm_c = $168;
  $169 = $LUfactors;
  $170 = ((($169)) + 12|0);
  $171 = HEAP32[$170>>2]|0;
  $perm_r = $171;
  $172 = $1;
  $173 = $3;
  $174 = Math_imul($172, $173)|0;
  $175 = (_doublecomplexMalloc($174)|0);
  $b = $175;
  $176 = ($175|0)!=(0|0);
  if (!($176)) {
   (_printf((3024|0),($vararg_buffer30|0))|0);
   STACKTOP = sp;return;
  }
  $i = 0;
  while(1) {
   $177 = $i;
   $178 = $1;
   $179 = $3;
   $180 = Math_imul($178, $179)|0;
   $181 = ($177|0)<($180|0);
   if (!($181)) {
    break;
   }
   $182 = $i;
   $183 = $182<<1;
   $184 = $7;
   $185 = (($184) + ($183<<3)|0);
   $186 = +HEAPF64[$185>>3];
   HEAPF64[$z1>>3] = $186;
   $187 = ((($z1)) + 8|0);
   $188 = $i;
   $189 = $188<<1;
   $190 = (($189) + 1)|0;
   $191 = $7;
   $192 = (($191) + ($190<<3)|0);
   $193 = +HEAPF64[$192>>3];
   HEAPF64[$187>>3] = $193;
   $194 = $i;
   $195 = $b;
   $196 = (($195) + ($194<<4)|0);
   ;HEAP32[$196>>2]=HEAP32[$z1>>2]|0;HEAP32[$196+4>>2]=HEAP32[$z1+4>>2]|0;HEAP32[$196+8>>2]=HEAP32[$z1+8>>2]|0;HEAP32[$196+12>>2]=HEAP32[$z1+12>>2]|0;
   $197 = $i;
   $198 = (($197) + 1)|0;
   $i = $198;
  }
  $199 = $1;
  $200 = $3;
  $201 = $b;
  $202 = $8;
  _zCreate_Dense_Matrix($B,$199,$200,$201,$202,6,3,0);
  $203 = $11;
  $204 = $L;
  $205 = $U;
  $206 = $perm_c;
  $207 = $perm_r;
  $208 = $12;
  _zgstrs($203,$204,$205,$206,$207,$B,$stat,$208);
  $209 = $12;
  $210 = HEAP32[$209>>2]|0;
  $211 = ($210|0)!=(0);
  if ($211) {
   $212 = $12;
   $213 = HEAP32[$212>>2]|0;
   HEAP32[$vararg_buffer32>>2] = $213;
   (_printf((3232|0),($vararg_buffer32|0))|0);
  }
  $i = 0;
  while(1) {
   $214 = $i;
   $215 = $1;
   $216 = $3;
   $217 = Math_imul($215, $216)|0;
   $218 = ($214|0)<($217|0);
   if (!($218)) {
    break;
   }
   $219 = $i;
   $220 = $b;
   $221 = (($220) + ($219<<4)|0);
   $222 = +HEAPF64[$221>>3];
   $223 = $i;
   $224 = $223<<1;
   $225 = $7;
   $226 = (($225) + ($224<<3)|0);
   HEAPF64[$226>>3] = $222;
   $227 = $i;
   $228 = $b;
   $229 = (($228) + ($227<<4)|0);
   $230 = ((($229)) + 8|0);
   $231 = +HEAPF64[$230>>3];
   $232 = $i;
   $233 = $232<<1;
   $234 = (($233) + 1)|0;
   $235 = $7;
   $236 = (($235) + ($234<<3)|0);
   HEAPF64[$236>>3] = $231;
   $237 = $i;
   $238 = (($237) + 1)|0;
   $i = $238;
  }
  _Destroy_SuperMatrix_Store($B);
  _StatFree($stat);
  $239 = $b;
  _superlu_free($239);
  STACKTOP = sp;return;
 }
 _set_default_options($options);
 _StatInit($stat);
 $66 = $2;
 $67 = (_doublecomplexMalloc($66)|0);
 $values = $67;
 $68 = ($67|0)!=(0|0);
 if (!($68)) {
  (_printf((3024|0),($vararg_buffer|0))|0);
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $69 = $i;
  $70 = $2;
  $71 = ($69|0)<($70|0);
  if (!($71)) {
   break;
  }
  $72 = $i;
  $73 = $72<<1;
  $74 = $4;
  $75 = (($74) + ($73<<3)|0);
  $76 = +HEAPF64[$75>>3];
  HEAPF64[$z>>3] = $76;
  $77 = ((($z)) + 8|0);
  $78 = $i;
  $79 = $78<<1;
  $80 = (($79) + 1)|0;
  $81 = $4;
  $82 = (($81) + ($80<<3)|0);
  $83 = +HEAPF64[$82>>3];
  HEAPF64[$77>>3] = $83;
  $84 = $i;
  $85 = $values;
  $86 = (($85) + ($84<<4)|0);
  ;HEAP32[$86>>2]=HEAP32[$z>>2]|0;HEAP32[$86+4>>2]=HEAP32[$z+4>>2]|0;HEAP32[$86+8>>2]=HEAP32[$z+8>>2]|0;HEAP32[$86+12>>2]=HEAP32[$z+12>>2]|0;
  $87 = $i;
  $88 = (($87) + 1)|0;
  $i = $88;
 }
 $89 = $1;
 $90 = $1;
 $91 = $2;
 $92 = $values;
 $93 = $5;
 $94 = $6;
 _zCreate_CompCol_Matrix($A,$89,$90,$91,$92,$93,$94,0,3,0);
 $95 = (_superlu_malloc(24)|0);
 $L = $95;
 $96 = (_superlu_malloc(24)|0);
 $U = $96;
 $97 = $1;
 $98 = (_intMalloc($97)|0);
 $perm_r = $98;
 $99 = ($98|0)!=(0|0);
 if (!($99)) {
  $100 = HEAP32[_stderr>>2]|0;
  (_fprintf(($100|0),(3056|0),($vararg_buffer18|0))|0);
  STACKTOP = sp;return;
 }
 $101 = $1;
 $102 = (_intMalloc($101)|0);
 $perm_c = $102;
 $103 = ($102|0)!=(0|0);
 if (!($103)) {
  $104 = HEAP32[_stderr>>2]|0;
  (_fprintf(($104|0),(3088|0),($vararg_buffer20|0))|0);
  STACKTOP = sp;return;
 }
 $105 = $1;
 $106 = (_intMalloc($105)|0);
 $etree = $106;
 $107 = ($106|0)!=(0|0);
 if (!($107)) {
  $108 = HEAP32[_stderr>>2]|0;
  (_fprintf(($108|0),(3120|0),($vararg_buffer22|0))|0);
  STACKTOP = sp;return;
 }
 $109 = $10;
 $110 = $perm_c;
 _get_perm_c($109,$A,$110);
 $111 = $perm_c;
 $112 = $etree;
 _sp_preorder($options,$A,$111,$112,$AC);
 $113 = (_sp_ienv(1)|0);
 $panel_size = $113;
 $114 = (_sp_ienv(2)|0);
 $relax = $114;
 $115 = $relax;
 $116 = $panel_size;
 $117 = $etree;
 $118 = $perm_c;
 $119 = $perm_r;
 $120 = $L;
 $121 = $U;
 $122 = $12;
 _zgstrf($options,$AC,$115,$116,$117,0,0,$118,$119,$120,$121,$stat,$122);
 $123 = $12;
 $124 = HEAP32[$123>>2]|0;
 $125 = ($124|0)==(0);
 if (!($125)) {
  $126 = $12;
  $127 = HEAP32[$126>>2]|0;
  HEAP32[$vararg_buffer24>>2] = $127;
  (_printf((3152|0),($vararg_buffer24|0))|0);
  $128 = $12;
  $129 = HEAP32[$128>>2]|0;
  $130 = $1;
  $131 = ($129|0)<=($130|0);
  if ($131) {
   $132 = $L;
   $133 = $U;
   (_zQuerySpace($132,$133,$mem_usage)|0);
   $134 = +HEAPF32[$mem_usage>>2];
   $135 = $134;
   $136 = $135 / 1.0E+6;
   $137 = ((($mem_usage)) + 4|0);
   $138 = +HEAPF32[$137>>2];
   $139 = $138;
   $140 = $139 / 1.0E+6;
   HEAPF64[$vararg_buffer26>>3] = $136;
   $vararg_ptr29 = ((($vararg_buffer26)) + 8|0);
   HEAPF64[$vararg_ptr29>>3] = $140;
   (_printf((3192|0),($vararg_buffer26|0))|0);
  }
 }
 $141 = (_superlu_malloc(16)|0);
 $LUfactors = $141;
 $142 = $L;
 $143 = $LUfactors;
 HEAP32[$143>>2] = $142;
 $144 = $U;
 $145 = $LUfactors;
 $146 = ((($145)) + 4|0);
 HEAP32[$146>>2] = $144;
 $147 = $perm_c;
 $148 = $LUfactors;
 $149 = ((($148)) + 8|0);
 HEAP32[$149>>2] = $147;
 $150 = $perm_r;
 $151 = $LUfactors;
 $152 = ((($151)) + 12|0);
 HEAP32[$152>>2] = $150;
 $153 = $LUfactors;
 $154 = $9;
 HEAP32[$154>>2] = $153;
 $155 = $etree;
 _superlu_free($155);
 _Destroy_SuperMatrix_Store($A);
 _Destroy_CompCol_Permuted($AC);
 _StatFree($stat);
 $156 = $values;
 _superlu_free($156);
 STACKTOP = sp;return;
}
function _dtrsv_($uplo,$trans,$diag,$n,$a,$lda,$x,$incx) {
 $uplo = $uplo|0;
 $trans = $trans|0;
 $diag = $diag|0;
 $n = $n|0;
 $a = $a|0;
 $lda = $lda|0;
 $x = $x|0;
 $incx = $incx|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0.0, $102 = 0.0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0.0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0.0, $124 = 0.0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0.0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0.0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0.0, $171 = 0.0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0.0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0.0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0.0, $198 = 0.0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0.0, $204 = 0.0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0.0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0.0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0.0, $243 = 0.0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0.0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0.0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0.0, $269 = 0.0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0.0, $275 = 0.0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0.0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0.0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0.0, $311 = 0.0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0.0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0.0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0.0, $342 = 0.0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0.0, $348 = 0.0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0.0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0.0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0.0, $39 = 0, $390 = 0.0, $391 = 0.0, $392 = 0.0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0.0, $408 = 0.0, $409 = 0.0, $41 = 0, $410 = 0.0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0.0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0.0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0.0, $452 = 0.0, $453 = 0.0, $454 = 0.0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0.0, $474 = 0.0, $475 = 0.0, $476 = 0.0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0.0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0.0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0.0, $52 = 0, $520 = 0.0, $521 = 0.0, $522 = 0.0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0.0, $538 = 0.0, $539 = 0.0, $54 = 0, $540 = 0.0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0.0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0.0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0.0, $588 = 0.0, $589 = 0.0, $59 = 0, $590 = 0.0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0.0, $61 = 0, $610 = 0.0, $611 = 0.0, $612 = 0.0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0.0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0.0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, $i__2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $uplo;
 $2 = $trans;
 $3 = $diag;
 $4 = $n;
 $5 = $a;
 $6 = $lda;
 $7 = $x;
 $8 = $incx;
 HEAP32[3312>>2] = 0;
 $9 = $1;
 $10 = (_lsame_($9,3392)|0);
 $11 = ($10|0)!=(0);
 if ($11) {
  label = 4;
 } else {
  $12 = $1;
  $13 = (_lsame_($12,3400)|0);
  $14 = ($13|0)!=(0);
  if ($14) {
   label = 4;
  } else {
   HEAP32[3312>>2] = 1;
  }
 }
 do {
  if ((label|0) == 4) {
   $15 = $2;
   $16 = (_lsame_($15,3408)|0);
   $17 = ($16|0)!=(0);
   if (!($17)) {
    $18 = $2;
    $19 = (_lsame_($18,3416)|0);
    $20 = ($19|0)!=(0);
    if (!($20)) {
     $21 = $2;
     $22 = (_lsame_($21,3424)|0);
     $23 = ($22|0)!=(0);
     if (!($23)) {
      HEAP32[3312>>2] = 2;
      break;
     }
    }
   }
   $24 = $3;
   $25 = (_lsame_($24,3392)|0);
   $26 = ($25|0)!=(0);
   if (!($26)) {
    $27 = $3;
    $28 = (_lsame_($27,3408)|0);
    $29 = ($28|0)!=(0);
    if (!($29)) {
     HEAP32[3312>>2] = 3;
     break;
    }
   }
   $30 = $4;
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)<(0);
   if ($32) {
    HEAP32[3312>>2] = 4;
    break;
   }
   $33 = $6;
   $34 = HEAP32[$33>>2]|0;
   $35 = $4;
   $36 = HEAP32[$35>>2]|0;
   $37 = (1)>=($36|0);
   if ($37) {
    $41 = 1;
   } else {
    $38 = $4;
    $39 = HEAP32[$38>>2]|0;
    $41 = $39;
   }
   $40 = ($34|0)<($41|0);
   if ($40) {
    HEAP32[3312>>2] = 6;
    break;
   }
   $42 = $8;
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)==(0);
   if ($44) {
    HEAP32[3312>>2] = 8;
   }
  }
 } while(0);
 $45 = HEAP32[3312>>2]|0;
 $46 = ($45|0)!=(0);
 if ($46) {
  (_xerbla_(3320,3312)|0);
  $0 = 0;
  $623 = $0;
  STACKTOP = sp;return ($623|0);
 }
 $47 = $4;
 $48 = HEAP32[$47>>2]|0;
 $49 = ($48|0)==(0);
 if ($49) {
  $0 = 0;
  $623 = $0;
  STACKTOP = sp;return ($623|0);
 }
 $50 = $3;
 $51 = (_lsame_($50,3408)|0);
 HEAP32[3328>>2] = $51;
 $52 = $8;
 $53 = HEAP32[$52>>2]|0;
 $54 = ($53|0)<=(0);
 if ($54) {
  $55 = $4;
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) - 1)|0;
  $58 = $8;
  $59 = HEAP32[$58>>2]|0;
  $60 = Math_imul($57, $59)|0;
  $61 = (1 - ($60))|0;
  HEAP32[3336>>2] = $61;
 } else {
  $62 = $8;
  $63 = HEAP32[$62>>2]|0;
  $64 = ($63|0)!=(1);
  if ($64) {
   HEAP32[3336>>2] = 1;
  }
 }
 $65 = $2;
 $66 = (_lsame_($65,3408)|0);
 $67 = ($66|0)!=(0);
 $68 = $1;
 $69 = (_lsame_($68,3392)|0);
 $70 = ($69|0)!=(0);
 $71 = $8;
 $72 = HEAP32[$71>>2]|0;
 $73 = ($72|0)==(1);
 L39: do {
  if ($67) {
   if ($70) {
    if ($73) {
     $74 = $4;
     $75 = HEAP32[$74>>2]|0;
     HEAP32[3344>>2] = $75;
     while(1) {
      $76 = HEAP32[3344>>2]|0;
      $77 = ($76|0)>=(1);
      if (!($77)) {
       break L39;
      }
      $78 = HEAP32[3344>>2]|0;
      $79 = (($78) - 1)|0;
      $80 = $7;
      $81 = (($80) + ($79<<3)|0);
      $82 = +HEAPF64[$81>>3];
      $83 = $82 != 0.0;
      L89: do {
       if ($83) {
        $84 = HEAP32[3328>>2]|0;
        $85 = ($84|0)!=(0);
        if ($85) {
         $86 = HEAP32[3344>>2]|0;
         $87 = (($86) - 1)|0;
         $88 = HEAP32[3344>>2]|0;
         $89 = (($88) - 1)|0;
         $90 = $6;
         $91 = HEAP32[$90>>2]|0;
         $92 = Math_imul($89, $91)|0;
         $93 = (($87) + ($92))|0;
         $94 = $5;
         $95 = (($94) + ($93<<3)|0);
         $96 = +HEAPF64[$95>>3];
         $97 = HEAP32[3344>>2]|0;
         $98 = (($97) - 1)|0;
         $99 = $7;
         $100 = (($99) + ($98<<3)|0);
         $101 = +HEAPF64[$100>>3];
         $102 = $101 / $96;
         HEAPF64[$100>>3] = $102;
        }
        $103 = HEAP32[3344>>2]|0;
        $104 = (($103) - 1)|0;
        $105 = $7;
        $106 = (($105) + ($104<<3)|0);
        $107 = +HEAPF64[$106>>3];
        HEAPF64[3352>>3] = $107;
        $108 = HEAP32[3344>>2]|0;
        $109 = (($108) - 1)|0;
        HEAP32[3360>>2] = $109;
        while(1) {
         $110 = HEAP32[3360>>2]|0;
         $111 = ($110|0)>=(1);
         if (!($111)) {
          break L89;
         }
         $112 = +HEAPF64[3352>>3];
         $113 = HEAP32[3360>>2]|0;
         $114 = (($113) - 1)|0;
         $115 = HEAP32[3344>>2]|0;
         $116 = (($115) - 1)|0;
         $117 = $6;
         $118 = HEAP32[$117>>2]|0;
         $119 = Math_imul($116, $118)|0;
         $120 = (($114) + ($119))|0;
         $121 = $5;
         $122 = (($121) + ($120<<3)|0);
         $123 = +HEAPF64[$122>>3];
         $124 = $112 * $123;
         $125 = HEAP32[3360>>2]|0;
         $126 = (($125) - 1)|0;
         $127 = $7;
         $128 = (($127) + ($126<<3)|0);
         $129 = +HEAPF64[$128>>3];
         $130 = $129 - $124;
         HEAPF64[$128>>3] = $130;
         $131 = HEAP32[3360>>2]|0;
         $132 = (($131) + -1)|0;
         HEAP32[3360>>2] = $132;
        }
       }
      } while(0);
      $133 = HEAP32[3344>>2]|0;
      $134 = (($133) + -1)|0;
      HEAP32[3344>>2] = $134;
     }
    } else {
     $135 = HEAP32[3336>>2]|0;
     $136 = $4;
     $137 = HEAP32[$136>>2]|0;
     $138 = (($137) - 1)|0;
     $139 = $8;
     $140 = HEAP32[$139>>2]|0;
     $141 = Math_imul($138, $140)|0;
     $142 = (($135) + ($141))|0;
     HEAP32[3368>>2] = $142;
     $143 = $4;
     $144 = HEAP32[$143>>2]|0;
     HEAP32[3344>>2] = $144;
     while(1) {
      $145 = HEAP32[3344>>2]|0;
      $146 = ($145|0)>=(1);
      if (!($146)) {
       break L39;
      }
      $147 = HEAP32[3368>>2]|0;
      $148 = (($147) - 1)|0;
      $149 = $7;
      $150 = (($149) + ($148<<3)|0);
      $151 = +HEAPF64[$150>>3];
      $152 = $151 != 0.0;
      L76: do {
       if ($152) {
        $153 = HEAP32[3328>>2]|0;
        $154 = ($153|0)!=(0);
        if ($154) {
         $155 = HEAP32[3344>>2]|0;
         $156 = (($155) - 1)|0;
         $157 = HEAP32[3344>>2]|0;
         $158 = (($157) - 1)|0;
         $159 = $6;
         $160 = HEAP32[$159>>2]|0;
         $161 = Math_imul($158, $160)|0;
         $162 = (($156) + ($161))|0;
         $163 = $5;
         $164 = (($163) + ($162<<3)|0);
         $165 = +HEAPF64[$164>>3];
         $166 = HEAP32[3368>>2]|0;
         $167 = (($166) - 1)|0;
         $168 = $7;
         $169 = (($168) + ($167<<3)|0);
         $170 = +HEAPF64[$169>>3];
         $171 = $170 / $165;
         HEAPF64[$169>>3] = $171;
        }
        $172 = HEAP32[3368>>2]|0;
        $173 = (($172) - 1)|0;
        $174 = $7;
        $175 = (($174) + ($173<<3)|0);
        $176 = +HEAPF64[$175>>3];
        HEAPF64[3352>>3] = $176;
        $177 = HEAP32[3368>>2]|0;
        HEAP32[3376>>2] = $177;
        $178 = HEAP32[3344>>2]|0;
        $179 = (($178) - 1)|0;
        HEAP32[3360>>2] = $179;
        while(1) {
         $180 = HEAP32[3360>>2]|0;
         $181 = ($180|0)>=(1);
         if (!($181)) {
          break L76;
         }
         $182 = $8;
         $183 = HEAP32[$182>>2]|0;
         $184 = HEAP32[3376>>2]|0;
         $185 = (($184) - ($183))|0;
         HEAP32[3376>>2] = $185;
         $186 = +HEAPF64[3352>>3];
         $187 = HEAP32[3360>>2]|0;
         $188 = (($187) - 1)|0;
         $189 = HEAP32[3344>>2]|0;
         $190 = (($189) - 1)|0;
         $191 = $6;
         $192 = HEAP32[$191>>2]|0;
         $193 = Math_imul($190, $192)|0;
         $194 = (($188) + ($193))|0;
         $195 = $5;
         $196 = (($195) + ($194<<3)|0);
         $197 = +HEAPF64[$196>>3];
         $198 = $186 * $197;
         $199 = HEAP32[3376>>2]|0;
         $200 = (($199) - 1)|0;
         $201 = $7;
         $202 = (($201) + ($200<<3)|0);
         $203 = +HEAPF64[$202>>3];
         $204 = $203 - $198;
         HEAPF64[$202>>3] = $204;
         $205 = HEAP32[3360>>2]|0;
         $206 = (($205) + -1)|0;
         HEAP32[3360>>2] = $206;
        }
       }
      } while(0);
      $207 = $8;
      $208 = HEAP32[$207>>2]|0;
      $209 = HEAP32[3368>>2]|0;
      $210 = (($209) - ($208))|0;
      HEAP32[3368>>2] = $210;
      $211 = HEAP32[3344>>2]|0;
      $212 = (($211) + -1)|0;
      HEAP32[3344>>2] = $212;
     }
    }
   } else {
    if ($73) {
     $213 = $4;
     $214 = HEAP32[$213>>2]|0;
     $i__1 = $214;
     HEAP32[3344>>2] = 1;
     while(1) {
      $215 = HEAP32[3344>>2]|0;
      $216 = $4;
      $217 = HEAP32[$216>>2]|0;
      $218 = ($215|0)<=($217|0);
      if (!($218)) {
       break L39;
      }
      $219 = HEAP32[3344>>2]|0;
      $220 = (($219) - 1)|0;
      $221 = $7;
      $222 = (($221) + ($220<<3)|0);
      $223 = +HEAPF64[$222>>3];
      $224 = $223 != 0.0;
      L48: do {
       if ($224) {
        $225 = HEAP32[3328>>2]|0;
        $226 = ($225|0)!=(0);
        if ($226) {
         $227 = HEAP32[3344>>2]|0;
         $228 = (($227) - 1)|0;
         $229 = HEAP32[3344>>2]|0;
         $230 = (($229) - 1)|0;
         $231 = $6;
         $232 = HEAP32[$231>>2]|0;
         $233 = Math_imul($230, $232)|0;
         $234 = (($228) + ($233))|0;
         $235 = $5;
         $236 = (($235) + ($234<<3)|0);
         $237 = +HEAPF64[$236>>3];
         $238 = HEAP32[3344>>2]|0;
         $239 = (($238) - 1)|0;
         $240 = $7;
         $241 = (($240) + ($239<<3)|0);
         $242 = +HEAPF64[$241>>3];
         $243 = $242 / $237;
         HEAPF64[$241>>3] = $243;
        }
        $244 = HEAP32[3344>>2]|0;
        $245 = (($244) - 1)|0;
        $246 = $7;
        $247 = (($246) + ($245<<3)|0);
        $248 = +HEAPF64[$247>>3];
        HEAPF64[3352>>3] = $248;
        $249 = $4;
        $250 = HEAP32[$249>>2]|0;
        $i__2 = $250;
        $251 = HEAP32[3344>>2]|0;
        $252 = (($251) + 1)|0;
        HEAP32[3360>>2] = $252;
        while(1) {
         $253 = HEAP32[3360>>2]|0;
         $254 = $4;
         $255 = HEAP32[$254>>2]|0;
         $256 = ($253|0)<=($255|0);
         if (!($256)) {
          break L48;
         }
         $257 = +HEAPF64[3352>>3];
         $258 = HEAP32[3360>>2]|0;
         $259 = (($258) - 1)|0;
         $260 = HEAP32[3344>>2]|0;
         $261 = (($260) - 1)|0;
         $262 = $6;
         $263 = HEAP32[$262>>2]|0;
         $264 = Math_imul($261, $263)|0;
         $265 = (($259) + ($264))|0;
         $266 = $5;
         $267 = (($266) + ($265<<3)|0);
         $268 = +HEAPF64[$267>>3];
         $269 = $257 * $268;
         $270 = HEAP32[3360>>2]|0;
         $271 = (($270) - 1)|0;
         $272 = $7;
         $273 = (($272) + ($271<<3)|0);
         $274 = +HEAPF64[$273>>3];
         $275 = $274 - $269;
         HEAPF64[$273>>3] = $275;
         $276 = HEAP32[3360>>2]|0;
         $277 = (($276) + 1)|0;
         HEAP32[3360>>2] = $277;
        }
       }
      } while(0);
      $278 = HEAP32[3344>>2]|0;
      $279 = (($278) + 1)|0;
      HEAP32[3344>>2] = $279;
     }
    } else {
     $280 = HEAP32[3336>>2]|0;
     HEAP32[3368>>2] = $280;
     $281 = $4;
     $282 = HEAP32[$281>>2]|0;
     $i__1 = $282;
     HEAP32[3344>>2] = 1;
     while(1) {
      $283 = HEAP32[3344>>2]|0;
      $284 = $4;
      $285 = HEAP32[$284>>2]|0;
      $286 = ($283|0)<=($285|0);
      if (!($286)) {
       break L39;
      }
      $287 = HEAP32[3368>>2]|0;
      $288 = (($287) - 1)|0;
      $289 = $7;
      $290 = (($289) + ($288<<3)|0);
      $291 = +HEAPF64[$290>>3];
      $292 = $291 != 0.0;
      L61: do {
       if ($292) {
        $293 = HEAP32[3328>>2]|0;
        $294 = ($293|0)!=(0);
        if ($294) {
         $295 = HEAP32[3344>>2]|0;
         $296 = (($295) - 1)|0;
         $297 = HEAP32[3344>>2]|0;
         $298 = (($297) - 1)|0;
         $299 = $6;
         $300 = HEAP32[$299>>2]|0;
         $301 = Math_imul($298, $300)|0;
         $302 = (($296) + ($301))|0;
         $303 = $5;
         $304 = (($303) + ($302<<3)|0);
         $305 = +HEAPF64[$304>>3];
         $306 = HEAP32[3368>>2]|0;
         $307 = (($306) - 1)|0;
         $308 = $7;
         $309 = (($308) + ($307<<3)|0);
         $310 = +HEAPF64[$309>>3];
         $311 = $310 / $305;
         HEAPF64[$309>>3] = $311;
        }
        $312 = HEAP32[3368>>2]|0;
        $313 = (($312) - 1)|0;
        $314 = $7;
        $315 = (($314) + ($313<<3)|0);
        $316 = +HEAPF64[$315>>3];
        HEAPF64[3352>>3] = $316;
        $317 = HEAP32[3368>>2]|0;
        HEAP32[3376>>2] = $317;
        $318 = $4;
        $319 = HEAP32[$318>>2]|0;
        $i__2 = $319;
        $320 = HEAP32[3344>>2]|0;
        $321 = (($320) + 1)|0;
        HEAP32[3360>>2] = $321;
        while(1) {
         $322 = HEAP32[3360>>2]|0;
         $323 = $4;
         $324 = HEAP32[$323>>2]|0;
         $325 = ($322|0)<=($324|0);
         if (!($325)) {
          break L61;
         }
         $326 = $8;
         $327 = HEAP32[$326>>2]|0;
         $328 = HEAP32[3376>>2]|0;
         $329 = (($328) + ($327))|0;
         HEAP32[3376>>2] = $329;
         $330 = +HEAPF64[3352>>3];
         $331 = HEAP32[3360>>2]|0;
         $332 = (($331) - 1)|0;
         $333 = HEAP32[3344>>2]|0;
         $334 = (($333) - 1)|0;
         $335 = $6;
         $336 = HEAP32[$335>>2]|0;
         $337 = Math_imul($334, $336)|0;
         $338 = (($332) + ($337))|0;
         $339 = $5;
         $340 = (($339) + ($338<<3)|0);
         $341 = +HEAPF64[$340>>3];
         $342 = $330 * $341;
         $343 = HEAP32[3376>>2]|0;
         $344 = (($343) - 1)|0;
         $345 = $7;
         $346 = (($345) + ($344<<3)|0);
         $347 = +HEAPF64[$346>>3];
         $348 = $347 - $342;
         HEAPF64[$346>>3] = $348;
         $349 = HEAP32[3360>>2]|0;
         $350 = (($349) + 1)|0;
         HEAP32[3360>>2] = $350;
        }
       }
      } while(0);
      $351 = $8;
      $352 = HEAP32[$351>>2]|0;
      $353 = HEAP32[3368>>2]|0;
      $354 = (($353) + ($352))|0;
      HEAP32[3368>>2] = $354;
      $355 = HEAP32[3344>>2]|0;
      $356 = (($355) + 1)|0;
      HEAP32[3344>>2] = $356;
     }
    }
   }
  } else {
   if ($70) {
    if ($73) {
     $357 = $4;
     $358 = HEAP32[$357>>2]|0;
     $i__1 = $358;
     HEAP32[3344>>2] = 1;
     while(1) {
      $359 = HEAP32[3344>>2]|0;
      $360 = $4;
      $361 = HEAP32[$360>>2]|0;
      $362 = ($359|0)<=($361|0);
      if (!($362)) {
       break L39;
      }
      $363 = HEAP32[3344>>2]|0;
      $364 = (($363) - 1)|0;
      $365 = $7;
      $366 = (($365) + ($364<<3)|0);
      $367 = +HEAPF64[$366>>3];
      HEAPF64[3352>>3] = $367;
      $368 = HEAP32[3344>>2]|0;
      $369 = (($368) - 1)|0;
      $i__2 = $369;
      HEAP32[3360>>2] = 1;
      while(1) {
       $370 = HEAP32[3360>>2]|0;
       $371 = HEAP32[3344>>2]|0;
       $372 = (($371) - 1)|0;
       $373 = ($370|0)<=($372|0);
       if (!($373)) {
        break;
       }
       $374 = HEAP32[3360>>2]|0;
       $375 = (($374) - 1)|0;
       $376 = HEAP32[3344>>2]|0;
       $377 = (($376) - 1)|0;
       $378 = $6;
       $379 = HEAP32[$378>>2]|0;
       $380 = Math_imul($377, $379)|0;
       $381 = (($375) + ($380))|0;
       $382 = $5;
       $383 = (($382) + ($381<<3)|0);
       $384 = +HEAPF64[$383>>3];
       $385 = HEAP32[3360>>2]|0;
       $386 = (($385) - 1)|0;
       $387 = $7;
       $388 = (($387) + ($386<<3)|0);
       $389 = +HEAPF64[$388>>3];
       $390 = $384 * $389;
       $391 = +HEAPF64[3352>>3];
       $392 = $391 - $390;
       HEAPF64[3352>>3] = $392;
       $393 = HEAP32[3360>>2]|0;
       $394 = (($393) + 1)|0;
       HEAP32[3360>>2] = $394;
      }
      $395 = HEAP32[3328>>2]|0;
      $396 = ($395|0)!=(0);
      if ($396) {
       $397 = HEAP32[3344>>2]|0;
       $398 = (($397) - 1)|0;
       $399 = HEAP32[3344>>2]|0;
       $400 = (($399) - 1)|0;
       $401 = $6;
       $402 = HEAP32[$401>>2]|0;
       $403 = Math_imul($400, $402)|0;
       $404 = (($398) + ($403))|0;
       $405 = $5;
       $406 = (($405) + ($404<<3)|0);
       $407 = +HEAPF64[$406>>3];
       $408 = +HEAPF64[3352>>3];
       $409 = $408 / $407;
       HEAPF64[3352>>3] = $409;
      }
      $410 = +HEAPF64[3352>>3];
      $411 = HEAP32[3344>>2]|0;
      $412 = (($411) - 1)|0;
      $413 = $7;
      $414 = (($413) + ($412<<3)|0);
      HEAPF64[$414>>3] = $410;
      $415 = HEAP32[3344>>2]|0;
      $416 = (($415) + 1)|0;
      HEAP32[3344>>2] = $416;
     }
    } else {
     $417 = HEAP32[3336>>2]|0;
     HEAP32[3368>>2] = $417;
     $418 = $4;
     $419 = HEAP32[$418>>2]|0;
     $i__1 = $419;
     HEAP32[3344>>2] = 1;
     while(1) {
      $420 = HEAP32[3344>>2]|0;
      $421 = $4;
      $422 = HEAP32[$421>>2]|0;
      $423 = ($420|0)<=($422|0);
      if (!($423)) {
       break L39;
      }
      $424 = HEAP32[3368>>2]|0;
      $425 = (($424) - 1)|0;
      $426 = $7;
      $427 = (($426) + ($425<<3)|0);
      $428 = +HEAPF64[$427>>3];
      HEAPF64[3352>>3] = $428;
      $429 = HEAP32[3336>>2]|0;
      HEAP32[3376>>2] = $429;
      $430 = HEAP32[3344>>2]|0;
      $431 = (($430) - 1)|0;
      $i__2 = $431;
      HEAP32[3360>>2] = 1;
      while(1) {
       $432 = HEAP32[3360>>2]|0;
       $433 = HEAP32[3344>>2]|0;
       $434 = (($433) - 1)|0;
       $435 = ($432|0)<=($434|0);
       if (!($435)) {
        break;
       }
       $436 = HEAP32[3360>>2]|0;
       $437 = (($436) - 1)|0;
       $438 = HEAP32[3344>>2]|0;
       $439 = (($438) - 1)|0;
       $440 = $6;
       $441 = HEAP32[$440>>2]|0;
       $442 = Math_imul($439, $441)|0;
       $443 = (($437) + ($442))|0;
       $444 = $5;
       $445 = (($444) + ($443<<3)|0);
       $446 = +HEAPF64[$445>>3];
       $447 = HEAP32[3376>>2]|0;
       $448 = (($447) - 1)|0;
       $449 = $7;
       $450 = (($449) + ($448<<3)|0);
       $451 = +HEAPF64[$450>>3];
       $452 = $446 * $451;
       $453 = +HEAPF64[3352>>3];
       $454 = $453 - $452;
       HEAPF64[3352>>3] = $454;
       $455 = $8;
       $456 = HEAP32[$455>>2]|0;
       $457 = HEAP32[3376>>2]|0;
       $458 = (($457) + ($456))|0;
       HEAP32[3376>>2] = $458;
       $459 = HEAP32[3360>>2]|0;
       $460 = (($459) + 1)|0;
       HEAP32[3360>>2] = $460;
      }
      $461 = HEAP32[3328>>2]|0;
      $462 = ($461|0)!=(0);
      if ($462) {
       $463 = HEAP32[3344>>2]|0;
       $464 = (($463) - 1)|0;
       $465 = HEAP32[3344>>2]|0;
       $466 = (($465) - 1)|0;
       $467 = $6;
       $468 = HEAP32[$467>>2]|0;
       $469 = Math_imul($466, $468)|0;
       $470 = (($464) + ($469))|0;
       $471 = $5;
       $472 = (($471) + ($470<<3)|0);
       $473 = +HEAPF64[$472>>3];
       $474 = +HEAPF64[3352>>3];
       $475 = $474 / $473;
       HEAPF64[3352>>3] = $475;
      }
      $476 = +HEAPF64[3352>>3];
      $477 = HEAP32[3368>>2]|0;
      $478 = (($477) - 1)|0;
      $479 = $7;
      $480 = (($479) + ($478<<3)|0);
      HEAPF64[$480>>3] = $476;
      $481 = $8;
      $482 = HEAP32[$481>>2]|0;
      $483 = HEAP32[3368>>2]|0;
      $484 = (($483) + ($482))|0;
      HEAP32[3368>>2] = $484;
      $485 = HEAP32[3344>>2]|0;
      $486 = (($485) + 1)|0;
      HEAP32[3344>>2] = $486;
     }
    }
   } else {
    $487 = $4;
    $488 = HEAP32[$487>>2]|0;
    if ($73) {
     HEAP32[3344>>2] = $488;
     while(1) {
      $489 = HEAP32[3344>>2]|0;
      $490 = ($489|0)>=(1);
      if (!($490)) {
       break L39;
      }
      $491 = HEAP32[3344>>2]|0;
      $492 = (($491) - 1)|0;
      $493 = $7;
      $494 = (($493) + ($492<<3)|0);
      $495 = +HEAPF64[$494>>3];
      HEAPF64[3352>>3] = $495;
      $496 = HEAP32[3344>>2]|0;
      $497 = (($496) + 1)|0;
      $i__1 = $497;
      $498 = $4;
      $499 = HEAP32[$498>>2]|0;
      HEAP32[3360>>2] = $499;
      while(1) {
       $500 = HEAP32[3360>>2]|0;
       $501 = HEAP32[3344>>2]|0;
       $502 = (($501) + 1)|0;
       $503 = ($500|0)>=($502|0);
       if (!($503)) {
        break;
       }
       $504 = HEAP32[3360>>2]|0;
       $505 = (($504) - 1)|0;
       $506 = HEAP32[3344>>2]|0;
       $507 = (($506) - 1)|0;
       $508 = $6;
       $509 = HEAP32[$508>>2]|0;
       $510 = Math_imul($507, $509)|0;
       $511 = (($505) + ($510))|0;
       $512 = $5;
       $513 = (($512) + ($511<<3)|0);
       $514 = +HEAPF64[$513>>3];
       $515 = HEAP32[3360>>2]|0;
       $516 = (($515) - 1)|0;
       $517 = $7;
       $518 = (($517) + ($516<<3)|0);
       $519 = +HEAPF64[$518>>3];
       $520 = $514 * $519;
       $521 = +HEAPF64[3352>>3];
       $522 = $521 - $520;
       HEAPF64[3352>>3] = $522;
       $523 = HEAP32[3360>>2]|0;
       $524 = (($523) + -1)|0;
       HEAP32[3360>>2] = $524;
      }
      $525 = HEAP32[3328>>2]|0;
      $526 = ($525|0)!=(0);
      if ($526) {
       $527 = HEAP32[3344>>2]|0;
       $528 = (($527) - 1)|0;
       $529 = HEAP32[3344>>2]|0;
       $530 = (($529) - 1)|0;
       $531 = $6;
       $532 = HEAP32[$531>>2]|0;
       $533 = Math_imul($530, $532)|0;
       $534 = (($528) + ($533))|0;
       $535 = $5;
       $536 = (($535) + ($534<<3)|0);
       $537 = +HEAPF64[$536>>3];
       $538 = +HEAPF64[3352>>3];
       $539 = $538 / $537;
       HEAPF64[3352>>3] = $539;
      }
      $540 = +HEAPF64[3352>>3];
      $541 = HEAP32[3344>>2]|0;
      $542 = (($541) - 1)|0;
      $543 = $7;
      $544 = (($543) + ($542<<3)|0);
      HEAPF64[$544>>3] = $540;
      $545 = HEAP32[3344>>2]|0;
      $546 = (($545) + -1)|0;
      HEAP32[3344>>2] = $546;
     }
    } else {
     $547 = (($488) - 1)|0;
     $548 = $8;
     $549 = HEAP32[$548>>2]|0;
     $550 = Math_imul($547, $549)|0;
     $551 = HEAP32[3336>>2]|0;
     $552 = (($551) + ($550))|0;
     HEAP32[3336>>2] = $552;
     $553 = HEAP32[3336>>2]|0;
     HEAP32[3368>>2] = $553;
     $554 = $4;
     $555 = HEAP32[$554>>2]|0;
     HEAP32[3344>>2] = $555;
     while(1) {
      $556 = HEAP32[3344>>2]|0;
      $557 = ($556|0)>=(1);
      if (!($557)) {
       break L39;
      }
      $558 = HEAP32[3368>>2]|0;
      $559 = (($558) - 1)|0;
      $560 = $7;
      $561 = (($560) + ($559<<3)|0);
      $562 = +HEAPF64[$561>>3];
      HEAPF64[3352>>3] = $562;
      $563 = HEAP32[3336>>2]|0;
      HEAP32[3376>>2] = $563;
      $564 = HEAP32[3344>>2]|0;
      $565 = (($564) + 1)|0;
      $i__1 = $565;
      $566 = $4;
      $567 = HEAP32[$566>>2]|0;
      HEAP32[3360>>2] = $567;
      while(1) {
       $568 = HEAP32[3360>>2]|0;
       $569 = HEAP32[3344>>2]|0;
       $570 = (($569) + 1)|0;
       $571 = ($568|0)>=($570|0);
       if (!($571)) {
        break;
       }
       $572 = HEAP32[3360>>2]|0;
       $573 = (($572) - 1)|0;
       $574 = HEAP32[3344>>2]|0;
       $575 = (($574) - 1)|0;
       $576 = $6;
       $577 = HEAP32[$576>>2]|0;
       $578 = Math_imul($575, $577)|0;
       $579 = (($573) + ($578))|0;
       $580 = $5;
       $581 = (($580) + ($579<<3)|0);
       $582 = +HEAPF64[$581>>3];
       $583 = HEAP32[3376>>2]|0;
       $584 = (($583) - 1)|0;
       $585 = $7;
       $586 = (($585) + ($584<<3)|0);
       $587 = +HEAPF64[$586>>3];
       $588 = $582 * $587;
       $589 = +HEAPF64[3352>>3];
       $590 = $589 - $588;
       HEAPF64[3352>>3] = $590;
       $591 = $8;
       $592 = HEAP32[$591>>2]|0;
       $593 = HEAP32[3376>>2]|0;
       $594 = (($593) - ($592))|0;
       HEAP32[3376>>2] = $594;
       $595 = HEAP32[3360>>2]|0;
       $596 = (($595) + -1)|0;
       HEAP32[3360>>2] = $596;
      }
      $597 = HEAP32[3328>>2]|0;
      $598 = ($597|0)!=(0);
      if ($598) {
       $599 = HEAP32[3344>>2]|0;
       $600 = (($599) - 1)|0;
       $601 = HEAP32[3344>>2]|0;
       $602 = (($601) - 1)|0;
       $603 = $6;
       $604 = HEAP32[$603>>2]|0;
       $605 = Math_imul($602, $604)|0;
       $606 = (($600) + ($605))|0;
       $607 = $5;
       $608 = (($607) + ($606<<3)|0);
       $609 = +HEAPF64[$608>>3];
       $610 = +HEAPF64[3352>>3];
       $611 = $610 / $609;
       HEAPF64[3352>>3] = $611;
      }
      $612 = +HEAPF64[3352>>3];
      $613 = HEAP32[3368>>2]|0;
      $614 = (($613) - 1)|0;
      $615 = $7;
      $616 = (($615) + ($614<<3)|0);
      HEAPF64[$616>>3] = $612;
      $617 = $8;
      $618 = HEAP32[$617>>2]|0;
      $619 = HEAP32[3368>>2]|0;
      $620 = (($619) - ($618))|0;
      HEAP32[3368>>2] = $620;
      $621 = HEAP32[3344>>2]|0;
      $622 = (($621) + -1)|0;
      HEAP32[3344>>2] = $622;
     }
    }
   }
  }
 } while(0);
 $0 = 0;
 $623 = $0;
 STACKTOP = sp;return ($623|0);
}
function _ztrsv_($uplo,$trans,$diag,$n,$a,$lda,$x,$incx) {
 $uplo = $uplo|0;
 $trans = $trans|0;
 $diag = $diag|0;
 $n = $n|0;
 $a = $a|0;
 $lda = $lda|0;
 $x = $x|0;
 $incx = $incx|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0.0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0.0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0.0, $1032 = 0.0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0.0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0.0;
 var $1051 = 0.0, $1052 = 0.0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0.0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0.0;
 var $107 = 0, $1070 = 0.0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0.0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0.0;
 var $1088 = 0.0, $1089 = 0.0, $109 = 0, $1090 = 0, $1091 = 0.0, $1092 = 0.0, $1093 = 0.0, $1094 = 0.0, $1095 = 0, $1096 = 0.0, $1097 = 0.0, $1098 = 0, $1099 = 0.0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0.0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0.0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0.0, $1121 = 0, $1122 = 0.0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0.0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0.0, $1144 = 0.0, $1145 = 0, $1146 = 0.0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0.0, $1153 = 0.0, $1154 = 0.0, $1155 = 0.0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0.0, $1162 = 0.0, $1163 = 0, $1164 = 0.0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0.0, $117 = 0.0, $1170 = 0.0, $1171 = 0.0, $1172 = 0, $1173 = 0.0, $1174 = 0.0, $1175 = 0.0, $1176 = 0.0, $1177 = 0;
 var $1178 = 0.0, $1179 = 0.0, $118 = 0, $1180 = 0, $1181 = 0.0, $1182 = 0, $1183 = 0.0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0.0, $1203 = 0, $1204 = 0.0, $1205 = 0, $1206 = 0.0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0.0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0.0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0.0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0.0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0.0, $1271 = 0.0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0.0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0.0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0.0, $129 = 0, $1290 = 0.0, $1291 = 0.0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0.0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0.0, $1309 = 0.0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0.0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0.0, $1327 = 0.0, $1328 = 0.0, $1329 = 0, $133 = 0, $1330 = 0.0, $1331 = 0.0, $1332 = 0.0, $1333 = 0.0, $1334 = 0, $1335 = 0.0, $1336 = 0.0, $1337 = 0, $1338 = 0.0, $1339 = 0;
 var $134 = 0.0, $1340 = 0.0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0.0, $1356 = 0, $1357 = 0.0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0.0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0.0, $1379 = 0.0, $138 = 0, $1380 = 0, $1381 = 0.0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0.0, $1388 = 0.0, $1389 = 0.0, $139 = 0, $1390 = 0.0, $1391 = 0, $1392 = 0, $1393 = 0;
 var $1394 = 0, $1395 = 0, $1396 = 0.0, $1397 = 0.0, $1398 = 0, $1399 = 0.0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0.0, $1405 = 0.0, $1406 = 0.0, $1407 = 0, $1408 = 0.0, $1409 = 0.0, $141 = 0, $1410 = 0.0;
 var $1411 = 0.0, $1412 = 0, $1413 = 0.0, $1414 = 0.0, $1415 = 0, $1416 = 0.0, $1417 = 0, $1418 = 0.0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0;
 var $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0.0, $1434 = 0, $1435 = 0.0, $1436 = 0, $1437 = 0.0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0.0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0;
 var $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0.0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0;
 var $1466 = 0, $1467 = 0.0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0.0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0;
 var $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0.0;
 var $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0.0, $1506 = 0.0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0.0, $1519 = 0;
 var $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0.0, $1525 = 0.0, $1526 = 0.0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0.0;
 var $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0.0, $1544 = 0.0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0;
 var $1556 = 0.0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0.0, $1562 = 0.0, $1563 = 0.0, $1564 = 0, $1565 = 0.0, $1566 = 0.0, $1567 = 0.0, $1568 = 0.0, $1569 = 0, $157 = 0.0, $1570 = 0.0, $1571 = 0.0, $1572 = 0, $1573 = 0.0;
 var $1574 = 0, $1575 = 0.0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0.0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0.0, $1590 = 0, $1591 = 0;
 var $1592 = 0, $1593 = 0, $1594 = 0.0, $1595 = 0, $1596 = 0.0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0;
 var $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0.0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0.0, $1618 = 0.0, $1619 = 0, $162 = 0, $1620 = 0.0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0.0, $1627 = 0.0;
 var $1628 = 0.0, $1629 = 0.0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0.0, $1636 = 0.0, $1637 = 0, $1638 = 0.0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0.0, $1644 = 0.0, $1645 = 0.0;
 var $1646 = 0, $1647 = 0.0, $1648 = 0.0, $1649 = 0.0, $165 = 0, $1650 = 0.0, $1651 = 0, $1652 = 0.0, $1653 = 0.0, $1654 = 0, $1655 = 0.0, $1656 = 0, $1657 = 0.0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0;
 var $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0.0, $1677 = 0, $1678 = 0.0, $1679 = 0, $168 = 0, $1680 = 0.0, $1681 = 0;
 var $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0.0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $17 = 0, $170 = 0, $171 = 0.0;
 var $172 = 0.0, $173 = 0.0, $174 = 0.0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0.0, $187 = 0.0, $188 = 0.0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0.0, $201 = 0.0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0.0;
 var $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0.0, $216 = 0, $217 = 0.0, $218 = 0.0, $219 = 0, $22 = 0, $220 = 0.0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0.0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0.0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0.0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0.0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0.0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0.0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0.0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0.0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0.0, $331 = 0.0, $332 = 0.0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0.0, $345 = 0.0, $346 = 0.0, $347 = 0.0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0.0, $36 = 0, $360 = 0.0, $361 = 0.0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0.0, $373 = 0.0, $374 = 0.0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0.0, $381 = 0.0, $382 = 0.0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0.0;
 var $389 = 0, $39 = 0, $390 = 0.0, $391 = 0.0, $392 = 0, $393 = 0.0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0.0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0.0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0.0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0.0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0.0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0.0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0.0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0.0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0;
 var $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0.0, $501 = 0.0, $502 = 0.0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0;
 var $514 = 0.0, $515 = 0.0, $516 = 0.0, $517 = 0.0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0.0, $53 = 0, $530 = 0.0, $531 = 0.0;
 var $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0.0, $543 = 0.0, $544 = 0.0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0;
 var $550 = 0.0, $551 = 0.0, $552 = 0.0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0.0, $559 = 0, $56 = 0, $560 = 0.0, $561 = 0.0, $562 = 0, $563 = 0.0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0;
 var $569 = 0.0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0;
 var $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0.0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0.0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0;
 var $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0.0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0;
 var $622 = 0, $623 = 0.0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0.0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0;
 var $640 = 0.0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0;
 var $659 = 0, $66 = 0, $660 = 0, $661 = 0.0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0.0, $673 = 0.0, $674 = 0.0, $675 = 0, $676 = 0;
 var $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0.0, $687 = 0.0, $688 = 0.0, $689 = 0.0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0;
 var $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0.0, $702 = 0.0, $703 = 0.0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0;
 var $712 = 0, $713 = 0, $714 = 0.0, $715 = 0.0, $716 = 0.0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0.0, $723 = 0.0, $724 = 0.0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0;
 var $730 = 0.0, $731 = 0, $732 = 0.0, $733 = 0.0, $734 = 0, $735 = 0.0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0.0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0;
 var $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0.0;
 var $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0.0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0;
 var $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0.0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0;
 var $802 = 0.0, $803 = 0.0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0.0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0;
 var $820 = 0, $821 = 0.0, $822 = 0.0, $823 = 0.0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0.0, $835 = 0, $836 = 0, $837 = 0, $838 = 0;
 var $839 = 0, $84 = 0, $840 = 0.0, $841 = 0.0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0.0, $850 = 0, $851 = 0, $852 = 0, $853 = 0.0, $854 = 0, $855 = 0, $856 = 0;
 var $857 = 0, $858 = 0.0, $859 = 0.0, $86 = 0, $860 = 0.0, $861 = 0, $862 = 0.0, $863 = 0.0, $864 = 0.0, $865 = 0.0, $866 = 0, $867 = 0.0, $868 = 0.0, $869 = 0, $87 = 0, $870 = 0.0, $871 = 0, $872 = 0.0, $873 = 0, $874 = 0;
 var $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0.0, $888 = 0, $889 = 0.0, $89 = 0, $890 = 0, $891 = 0, $892 = 0;
 var $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0.0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0;
 var $910 = 0.0, $911 = 0.0, $912 = 0, $913 = 0.0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0.0, $92 = 0.0, $920 = 0.0, $921 = 0.0, $922 = 0.0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0.0;
 var $929 = 0.0, $93 = 0, $930 = 0, $931 = 0.0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0.0, $937 = 0.0, $938 = 0.0, $939 = 0, $94 = 0, $940 = 0.0, $941 = 0.0, $942 = 0.0, $943 = 0.0, $944 = 0, $945 = 0.0, $946 = 0.0;
 var $947 = 0, $948 = 0.0, $949 = 0, $95 = 0, $950 = 0.0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0;
 var $965 = 0.0, $966 = 0, $967 = 0.0, $968 = 0, $969 = 0.0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0.0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0;
 var $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0.0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $a_dim1 = 0, $i__1 = 0;
 var $i__2 = 0, $i__3 = 0, $i__4 = 0, $i__5 = 0, $z__1 = 0, $z__2 = 0, $z__3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $z__1 = sp + 32|0;
 $z__2 = sp + 16|0;
 $z__3 = sp;
 $1 = $uplo;
 $2 = $trans;
 $3 = $diag;
 $4 = $n;
 $5 = $a;
 $6 = $lda;
 $7 = $x;
 $8 = $incx;
 HEAP32[3384>>2] = 0;
 $9 = $1;
 $10 = (_lsame_($9,3392)|0);
 $11 = ($10|0)!=(0);
 if ($11) {
  label = 4;
 } else {
  $12 = $1;
  $13 = (_lsame_($12,3400)|0);
  $14 = ($13|0)!=(0);
  if ($14) {
   label = 4;
  } else {
   HEAP32[3384>>2] = 1;
  }
 }
 do {
  if ((label|0) == 4) {
   $15 = $2;
   $16 = (_lsame_($15,3408)|0);
   $17 = ($16|0)!=(0);
   if (!($17)) {
    $18 = $2;
    $19 = (_lsame_($18,3416)|0);
    $20 = ($19|0)!=(0);
    if (!($20)) {
     $21 = $2;
     $22 = (_lsame_($21,3424)|0);
     $23 = ($22|0)!=(0);
     if (!($23)) {
      HEAP32[3384>>2] = 2;
      break;
     }
    }
   }
   $24 = $3;
   $25 = (_lsame_($24,3392)|0);
   $26 = ($25|0)!=(0);
   if (!($26)) {
    $27 = $3;
    $28 = (_lsame_($27,3408)|0);
    $29 = ($28|0)!=(0);
    if (!($29)) {
     HEAP32[3384>>2] = 3;
     break;
    }
   }
   $30 = $4;
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)<(0);
   if ($32) {
    HEAP32[3384>>2] = 4;
    break;
   }
   $33 = $6;
   $34 = HEAP32[$33>>2]|0;
   $35 = $4;
   $36 = HEAP32[$35>>2]|0;
   $37 = (1)>=($36|0);
   if ($37) {
    $41 = 1;
   } else {
    $38 = $4;
    $39 = HEAP32[$38>>2]|0;
    $41 = $39;
   }
   $40 = ($34|0)<($41|0);
   if ($40) {
    HEAP32[3384>>2] = 6;
    break;
   }
   $42 = $8;
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)==(0);
   if ($44) {
    HEAP32[3384>>2] = 8;
   }
  }
 } while(0);
 $45 = HEAP32[3384>>2]|0;
 $46 = ($45|0)!=(0);
 if ($46) {
  (_xerbla_(3432,3384)|0);
  $0 = 0;
  $1697 = $0;
  STACKTOP = sp;return ($1697|0);
 }
 $47 = $4;
 $48 = HEAP32[$47>>2]|0;
 $49 = ($48|0)==(0);
 if ($49) {
  $0 = 0;
  $1697 = $0;
  STACKTOP = sp;return ($1697|0);
 }
 $50 = $2;
 $51 = (_lsame_($50,3416)|0);
 HEAP32[3440>>2] = $51;
 $52 = $3;
 $53 = (_lsame_($52,3408)|0);
 HEAP32[3448>>2] = $53;
 $54 = $8;
 $55 = HEAP32[$54>>2]|0;
 $56 = ($55|0)<=(0);
 if ($56) {
  $57 = $4;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) - 1)|0;
  $60 = $8;
  $61 = HEAP32[$60>>2]|0;
  $62 = Math_imul($59, $61)|0;
  $63 = (1 - ($62))|0;
  HEAP32[3456>>2] = $63;
 } else {
  $64 = $8;
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)!=(1);
  if ($66) {
   HEAP32[3456>>2] = 1;
  }
 }
 $67 = $2;
 $68 = (_lsame_($67,3408)|0);
 $69 = ($68|0)!=(0);
 $70 = $1;
 $71 = (_lsame_($70,3392)|0);
 $72 = ($71|0)!=(0);
 $73 = $8;
 $74 = HEAP32[$73>>2]|0;
 $75 = ($74|0)==(1);
 L39: do {
  if ($69) {
   if ($72) {
    if ($75) {
     $76 = $4;
     $77 = HEAP32[$76>>2]|0;
     HEAP32[3464>>2] = $77;
     while(1) {
      $78 = HEAP32[3464>>2]|0;
      $79 = ($78|0)>=(1);
      if (!($79)) {
       break L39;
      }
      $80 = HEAP32[3464>>2]|0;
      $i__1 = $80;
      $81 = HEAP32[3464>>2]|0;
      $82 = (($81) - 1)|0;
      $83 = $7;
      $84 = (($83) + ($82<<4)|0);
      $85 = +HEAPF64[$84>>3];
      $86 = $85 != 0.0;
      if ($86) {
       label = 34;
      } else {
       $87 = HEAP32[3464>>2]|0;
       $88 = (($87) - 1)|0;
       $89 = $7;
       $90 = (($89) + ($88<<4)|0);
       $91 = ((($90)) + 8|0);
       $92 = +HEAPF64[$91>>3];
       $93 = $92 != 0.0;
       if ($93) {
        label = 34;
       }
      }
      L120: do {
       if ((label|0) == 34) {
        label = 0;
        $94 = HEAP32[3448>>2]|0;
        $95 = ($94|0)!=(0);
        if ($95) {
         $96 = HEAP32[3464>>2]|0;
         $i__1 = $96;
         $97 = HEAP32[3464>>2]|0;
         $98 = (($97) - 1)|0;
         $99 = $7;
         $100 = (($99) + ($98<<4)|0);
         $101 = HEAP32[3464>>2]|0;
         $102 = (($101) - 1)|0;
         $103 = HEAP32[3464>>2]|0;
         $104 = (($103) - 1)|0;
         $105 = $6;
         $106 = HEAP32[$105>>2]|0;
         $107 = Math_imul($104, $106)|0;
         $108 = (($102) + ($107))|0;
         $109 = $5;
         $110 = (($109) + ($108<<4)|0);
         _z_div($z__1,$100,$110);
         $111 = +HEAPF64[$z__1>>3];
         $112 = HEAP32[3464>>2]|0;
         $113 = (($112) - 1)|0;
         $114 = $7;
         $115 = (($114) + ($113<<4)|0);
         HEAPF64[$115>>3] = $111;
         $116 = ((($z__1)) + 8|0);
         $117 = +HEAPF64[$116>>3];
         $118 = HEAP32[3464>>2]|0;
         $119 = (($118) - 1)|0;
         $120 = $7;
         $121 = (($120) + ($119<<4)|0);
         $122 = ((($121)) + 8|0);
         HEAPF64[$122>>3] = $117;
        }
        $123 = HEAP32[3464>>2]|0;
        $i__1 = $123;
        $124 = HEAP32[3464>>2]|0;
        $125 = (($124) - 1)|0;
        $126 = $7;
        $127 = (($126) + ($125<<4)|0);
        $128 = +HEAPF64[$127>>3];
        HEAPF64[3472>>3] = $128;
        $129 = HEAP32[3464>>2]|0;
        $130 = (($129) - 1)|0;
        $131 = $7;
        $132 = (($131) + ($130<<4)|0);
        $133 = ((($132)) + 8|0);
        $134 = +HEAPF64[$133>>3];
        HEAPF64[(3480)>>3] = $134;
        $135 = HEAP32[3464>>2]|0;
        $136 = (($135) - 1)|0;
        HEAP32[3488>>2] = $136;
        while(1) {
         $137 = HEAP32[3488>>2]|0;
         $138 = ($137|0)>=(1);
         if (!($138)) {
          break L120;
         }
         $139 = HEAP32[3488>>2]|0;
         $i__1 = $139;
         $140 = HEAP32[3488>>2]|0;
         $i__2 = $140;
         $141 = HEAP32[3488>>2]|0;
         $142 = HEAP32[3464>>2]|0;
         $143 = $a_dim1;
         $144 = Math_imul($142, $143)|0;
         $145 = (($141) + ($144))|0;
         $i__3 = $145;
         $146 = +HEAPF64[3472>>3];
         $147 = HEAP32[3488>>2]|0;
         $148 = (($147) - 1)|0;
         $149 = HEAP32[3464>>2]|0;
         $150 = (($149) - 1)|0;
         $151 = $6;
         $152 = HEAP32[$151>>2]|0;
         $153 = Math_imul($150, $152)|0;
         $154 = (($148) + ($153))|0;
         $155 = $5;
         $156 = (($155) + ($154<<4)|0);
         $157 = +HEAPF64[$156>>3];
         $158 = $146 * $157;
         $159 = +HEAPF64[(3480)>>3];
         $160 = HEAP32[3488>>2]|0;
         $161 = (($160) - 1)|0;
         $162 = HEAP32[3464>>2]|0;
         $163 = (($162) - 1)|0;
         $164 = $6;
         $165 = HEAP32[$164>>2]|0;
         $166 = Math_imul($163, $165)|0;
         $167 = (($161) + ($166))|0;
         $168 = $5;
         $169 = (($168) + ($167<<4)|0);
         $170 = ((($169)) + 8|0);
         $171 = +HEAPF64[$170>>3];
         $172 = $159 * $171;
         $173 = $158 - $172;
         HEAPF64[$z__2>>3] = $173;
         $174 = +HEAPF64[3472>>3];
         $175 = HEAP32[3488>>2]|0;
         $176 = (($175) - 1)|0;
         $177 = HEAP32[3464>>2]|0;
         $178 = (($177) - 1)|0;
         $179 = $6;
         $180 = HEAP32[$179>>2]|0;
         $181 = Math_imul($178, $180)|0;
         $182 = (($176) + ($181))|0;
         $183 = $5;
         $184 = (($183) + ($182<<4)|0);
         $185 = ((($184)) + 8|0);
         $186 = +HEAPF64[$185>>3];
         $187 = $174 * $186;
         $188 = +HEAPF64[(3480)>>3];
         $189 = HEAP32[3488>>2]|0;
         $190 = (($189) - 1)|0;
         $191 = HEAP32[3464>>2]|0;
         $192 = (($191) - 1)|0;
         $193 = $6;
         $194 = HEAP32[$193>>2]|0;
         $195 = Math_imul($192, $194)|0;
         $196 = (($190) + ($195))|0;
         $197 = $5;
         $198 = (($197) + ($196<<4)|0);
         $199 = +HEAPF64[$198>>3];
         $200 = $188 * $199;
         $201 = $187 + $200;
         $202 = ((($z__2)) + 8|0);
         HEAPF64[$202>>3] = $201;
         $203 = HEAP32[3488>>2]|0;
         $204 = (($203) - 1)|0;
         $205 = $7;
         $206 = (($205) + ($204<<4)|0);
         $207 = +HEAPF64[$206>>3];
         $208 = +HEAPF64[$z__2>>3];
         $209 = $207 - $208;
         HEAPF64[$z__1>>3] = $209;
         $210 = HEAP32[3488>>2]|0;
         $211 = (($210) - 1)|0;
         $212 = $7;
         $213 = (($212) + ($211<<4)|0);
         $214 = ((($213)) + 8|0);
         $215 = +HEAPF64[$214>>3];
         $216 = ((($z__2)) + 8|0);
         $217 = +HEAPF64[$216>>3];
         $218 = $215 - $217;
         $219 = ((($z__1)) + 8|0);
         HEAPF64[$219>>3] = $218;
         $220 = +HEAPF64[$z__1>>3];
         $221 = HEAP32[3488>>2]|0;
         $222 = (($221) - 1)|0;
         $223 = $7;
         $224 = (($223) + ($222<<4)|0);
         HEAPF64[$224>>3] = $220;
         $225 = ((($z__1)) + 8|0);
         $226 = +HEAPF64[$225>>3];
         $227 = HEAP32[3488>>2]|0;
         $228 = (($227) - 1)|0;
         $229 = $7;
         $230 = (($229) + ($228<<4)|0);
         $231 = ((($230)) + 8|0);
         HEAPF64[$231>>3] = $226;
         $232 = HEAP32[3488>>2]|0;
         $233 = (($232) + -1)|0;
         HEAP32[3488>>2] = $233;
        }
       }
      } while(0);
      $234 = HEAP32[3464>>2]|0;
      $235 = (($234) + -1)|0;
      HEAP32[3464>>2] = $235;
     }
    } else {
     $236 = HEAP32[3456>>2]|0;
     $237 = $4;
     $238 = HEAP32[$237>>2]|0;
     $239 = (($238) - 1)|0;
     $240 = $8;
     $241 = HEAP32[$240>>2]|0;
     $242 = Math_imul($239, $241)|0;
     $243 = (($236) + ($242))|0;
     HEAP32[3496>>2] = $243;
     $244 = $4;
     $245 = HEAP32[$244>>2]|0;
     HEAP32[3464>>2] = $245;
     while(1) {
      $246 = HEAP32[3464>>2]|0;
      $247 = ($246|0)>=(1);
      if (!($247)) {
       break L39;
      }
      $248 = HEAP32[3496>>2]|0;
      $i__1 = $248;
      $249 = HEAP32[3496>>2]|0;
      $250 = (($249) - 1)|0;
      $251 = $7;
      $252 = (($251) + ($250<<4)|0);
      $253 = +HEAPF64[$252>>3];
      $254 = $253 != 0.0;
      if ($254) {
       label = 44;
      } else {
       $255 = HEAP32[3496>>2]|0;
       $256 = (($255) - 1)|0;
       $257 = $7;
       $258 = (($257) + ($256<<4)|0);
       $259 = ((($258)) + 8|0);
       $260 = +HEAPF64[$259>>3];
       $261 = $260 != 0.0;
       if ($261) {
        label = 44;
       }
      }
      L135: do {
       if ((label|0) == 44) {
        label = 0;
        $262 = HEAP32[3448>>2]|0;
        $263 = ($262|0)!=(0);
        if ($263) {
         $264 = HEAP32[3496>>2]|0;
         $i__1 = $264;
         $265 = HEAP32[3496>>2]|0;
         $266 = (($265) - 1)|0;
         $267 = $7;
         $268 = (($267) + ($266<<4)|0);
         $269 = HEAP32[3464>>2]|0;
         $270 = (($269) - 1)|0;
         $271 = HEAP32[3464>>2]|0;
         $272 = (($271) - 1)|0;
         $273 = $6;
         $274 = HEAP32[$273>>2]|0;
         $275 = Math_imul($272, $274)|0;
         $276 = (($270) + ($275))|0;
         $277 = $5;
         $278 = (($277) + ($276<<4)|0);
         _z_div($z__1,$268,$278);
         $279 = +HEAPF64[$z__1>>3];
         $280 = HEAP32[3496>>2]|0;
         $281 = (($280) - 1)|0;
         $282 = $7;
         $283 = (($282) + ($281<<4)|0);
         HEAPF64[$283>>3] = $279;
         $284 = ((($z__1)) + 8|0);
         $285 = +HEAPF64[$284>>3];
         $286 = HEAP32[3496>>2]|0;
         $287 = (($286) - 1)|0;
         $288 = $7;
         $289 = (($288) + ($287<<4)|0);
         $290 = ((($289)) + 8|0);
         HEAPF64[$290>>3] = $285;
        }
        $291 = HEAP32[3496>>2]|0;
        $i__1 = $291;
        $292 = HEAP32[3496>>2]|0;
        $293 = (($292) - 1)|0;
        $294 = $7;
        $295 = (($294) + ($293<<4)|0);
        $296 = +HEAPF64[$295>>3];
        HEAPF64[3472>>3] = $296;
        $297 = HEAP32[3496>>2]|0;
        $298 = (($297) - 1)|0;
        $299 = $7;
        $300 = (($299) + ($298<<4)|0);
        $301 = ((($300)) + 8|0);
        $302 = +HEAPF64[$301>>3];
        HEAPF64[(3480)>>3] = $302;
        $303 = HEAP32[3496>>2]|0;
        HEAP32[3504>>2] = $303;
        $304 = HEAP32[3464>>2]|0;
        $305 = (($304) - 1)|0;
        HEAP32[3488>>2] = $305;
        while(1) {
         $306 = HEAP32[3488>>2]|0;
         $307 = ($306|0)>=(1);
         if (!($307)) {
          break L135;
         }
         $308 = $8;
         $309 = HEAP32[$308>>2]|0;
         $310 = HEAP32[3504>>2]|0;
         $311 = (($310) - ($309))|0;
         HEAP32[3504>>2] = $311;
         $312 = HEAP32[3504>>2]|0;
         $i__1 = $312;
         $313 = HEAP32[3504>>2]|0;
         $i__2 = $313;
         $314 = HEAP32[3488>>2]|0;
         $315 = HEAP32[3464>>2]|0;
         $316 = $a_dim1;
         $317 = Math_imul($315, $316)|0;
         $318 = (($314) + ($317))|0;
         $i__3 = $318;
         $319 = +HEAPF64[3472>>3];
         $320 = HEAP32[3488>>2]|0;
         $321 = (($320) - 1)|0;
         $322 = HEAP32[3464>>2]|0;
         $323 = (($322) - 1)|0;
         $324 = $6;
         $325 = HEAP32[$324>>2]|0;
         $326 = Math_imul($323, $325)|0;
         $327 = (($321) + ($326))|0;
         $328 = $5;
         $329 = (($328) + ($327<<4)|0);
         $330 = +HEAPF64[$329>>3];
         $331 = $319 * $330;
         $332 = +HEAPF64[(3480)>>3];
         $333 = HEAP32[3488>>2]|0;
         $334 = (($333) - 1)|0;
         $335 = HEAP32[3464>>2]|0;
         $336 = (($335) - 1)|0;
         $337 = $6;
         $338 = HEAP32[$337>>2]|0;
         $339 = Math_imul($336, $338)|0;
         $340 = (($334) + ($339))|0;
         $341 = $5;
         $342 = (($341) + ($340<<4)|0);
         $343 = ((($342)) + 8|0);
         $344 = +HEAPF64[$343>>3];
         $345 = $332 * $344;
         $346 = $331 - $345;
         HEAPF64[$z__2>>3] = $346;
         $347 = +HEAPF64[3472>>3];
         $348 = HEAP32[3488>>2]|0;
         $349 = (($348) - 1)|0;
         $350 = HEAP32[3464>>2]|0;
         $351 = (($350) - 1)|0;
         $352 = $6;
         $353 = HEAP32[$352>>2]|0;
         $354 = Math_imul($351, $353)|0;
         $355 = (($349) + ($354))|0;
         $356 = $5;
         $357 = (($356) + ($355<<4)|0);
         $358 = ((($357)) + 8|0);
         $359 = +HEAPF64[$358>>3];
         $360 = $347 * $359;
         $361 = +HEAPF64[(3480)>>3];
         $362 = HEAP32[3488>>2]|0;
         $363 = (($362) - 1)|0;
         $364 = HEAP32[3464>>2]|0;
         $365 = (($364) - 1)|0;
         $366 = $6;
         $367 = HEAP32[$366>>2]|0;
         $368 = Math_imul($365, $367)|0;
         $369 = (($363) + ($368))|0;
         $370 = $5;
         $371 = (($370) + ($369<<4)|0);
         $372 = +HEAPF64[$371>>3];
         $373 = $361 * $372;
         $374 = $360 + $373;
         $375 = ((($z__2)) + 8|0);
         HEAPF64[$375>>3] = $374;
         $376 = HEAP32[3504>>2]|0;
         $377 = (($376) - 1)|0;
         $378 = $7;
         $379 = (($378) + ($377<<4)|0);
         $380 = +HEAPF64[$379>>3];
         $381 = +HEAPF64[$z__2>>3];
         $382 = $380 - $381;
         HEAPF64[$z__1>>3] = $382;
         $383 = HEAP32[3504>>2]|0;
         $384 = (($383) - 1)|0;
         $385 = $7;
         $386 = (($385) + ($384<<4)|0);
         $387 = ((($386)) + 8|0);
         $388 = +HEAPF64[$387>>3];
         $389 = ((($z__2)) + 8|0);
         $390 = +HEAPF64[$389>>3];
         $391 = $388 - $390;
         $392 = ((($z__1)) + 8|0);
         HEAPF64[$392>>3] = $391;
         $393 = +HEAPF64[$z__1>>3];
         $394 = HEAP32[3504>>2]|0;
         $395 = (($394) - 1)|0;
         $396 = $7;
         $397 = (($396) + ($395<<4)|0);
         HEAPF64[$397>>3] = $393;
         $398 = ((($z__1)) + 8|0);
         $399 = +HEAPF64[$398>>3];
         $400 = HEAP32[3504>>2]|0;
         $401 = (($400) - 1)|0;
         $402 = $7;
         $403 = (($402) + ($401<<4)|0);
         $404 = ((($403)) + 8|0);
         HEAPF64[$404>>3] = $399;
         $405 = HEAP32[3488>>2]|0;
         $406 = (($405) + -1)|0;
         HEAP32[3488>>2] = $406;
        }
       }
      } while(0);
      $407 = $8;
      $408 = HEAP32[$407>>2]|0;
      $409 = HEAP32[3496>>2]|0;
      $410 = (($409) - ($408))|0;
      HEAP32[3496>>2] = $410;
      $411 = HEAP32[3464>>2]|0;
      $412 = (($411) + -1)|0;
      HEAP32[3464>>2] = $412;
     }
    }
   } else {
    if ($75) {
     $413 = $4;
     $414 = HEAP32[$413>>2]|0;
     $i__1 = $414;
     HEAP32[3464>>2] = 1;
     while(1) {
      $415 = HEAP32[3464>>2]|0;
      $416 = $4;
      $417 = HEAP32[$416>>2]|0;
      $418 = ($415|0)<=($417|0);
      if (!($418)) {
       break L39;
      }
      $419 = HEAP32[3464>>2]|0;
      $i__2 = $419;
      $420 = HEAP32[3464>>2]|0;
      $421 = (($420) - 1)|0;
      $422 = $7;
      $423 = (($422) + ($421<<4)|0);
      $424 = +HEAPF64[$423>>3];
      $425 = $424 != 0.0;
      if ($425) {
       label = 55;
      } else {
       $426 = HEAP32[3464>>2]|0;
       $427 = (($426) - 1)|0;
       $428 = $7;
       $429 = (($428) + ($427<<4)|0);
       $430 = ((($429)) + 8|0);
       $431 = +HEAPF64[$430>>3];
       $432 = $431 != 0.0;
       if ($432) {
        label = 55;
       }
      }
      L167: do {
       if ((label|0) == 55) {
        label = 0;
        $433 = HEAP32[3448>>2]|0;
        $434 = ($433|0)!=(0);
        if ($434) {
         $435 = HEAP32[3464>>2]|0;
         $i__2 = $435;
         $436 = HEAP32[3464>>2]|0;
         $437 = (($436) - 1)|0;
         $438 = $7;
         $439 = (($438) + ($437<<4)|0);
         $440 = HEAP32[3464>>2]|0;
         $441 = (($440) - 1)|0;
         $442 = HEAP32[3464>>2]|0;
         $443 = (($442) - 1)|0;
         $444 = $6;
         $445 = HEAP32[$444>>2]|0;
         $446 = Math_imul($443, $445)|0;
         $447 = (($441) + ($446))|0;
         $448 = $5;
         $449 = (($448) + ($447<<4)|0);
         _z_div($z__1,$439,$449);
         $450 = +HEAPF64[$z__1>>3];
         $451 = HEAP32[3464>>2]|0;
         $452 = (($451) - 1)|0;
         $453 = $7;
         $454 = (($453) + ($452<<4)|0);
         HEAPF64[$454>>3] = $450;
         $455 = ((($z__1)) + 8|0);
         $456 = +HEAPF64[$455>>3];
         $457 = HEAP32[3464>>2]|0;
         $458 = (($457) - 1)|0;
         $459 = $7;
         $460 = (($459) + ($458<<4)|0);
         $461 = ((($460)) + 8|0);
         HEAPF64[$461>>3] = $456;
        }
        $462 = HEAP32[3464>>2]|0;
        $i__2 = $462;
        $463 = HEAP32[3464>>2]|0;
        $464 = (($463) - 1)|0;
        $465 = $7;
        $466 = (($465) + ($464<<4)|0);
        $467 = +HEAPF64[$466>>3];
        HEAPF64[3472>>3] = $467;
        $468 = HEAP32[3464>>2]|0;
        $469 = (($468) - 1)|0;
        $470 = $7;
        $471 = (($470) + ($469<<4)|0);
        $472 = ((($471)) + 8|0);
        $473 = +HEAPF64[$472>>3];
        HEAPF64[(3480)>>3] = $473;
        $474 = $4;
        $475 = HEAP32[$474>>2]|0;
        $i__2 = $475;
        $476 = HEAP32[3464>>2]|0;
        $477 = (($476) + 1)|0;
        HEAP32[3488>>2] = $477;
        while(1) {
         $478 = HEAP32[3488>>2]|0;
         $479 = $4;
         $480 = HEAP32[$479>>2]|0;
         $481 = ($478|0)<=($480|0);
         if (!($481)) {
          break L167;
         }
         $482 = HEAP32[3488>>2]|0;
         $i__3 = $482;
         $483 = HEAP32[3488>>2]|0;
         $i__4 = $483;
         $484 = HEAP32[3488>>2]|0;
         $485 = HEAP32[3464>>2]|0;
         $486 = $a_dim1;
         $487 = Math_imul($485, $486)|0;
         $488 = (($484) + ($487))|0;
         $i__5 = $488;
         $489 = +HEAPF64[3472>>3];
         $490 = HEAP32[3488>>2]|0;
         $491 = (($490) - 1)|0;
         $492 = HEAP32[3464>>2]|0;
         $493 = (($492) - 1)|0;
         $494 = $6;
         $495 = HEAP32[$494>>2]|0;
         $496 = Math_imul($493, $495)|0;
         $497 = (($491) + ($496))|0;
         $498 = $5;
         $499 = (($498) + ($497<<4)|0);
         $500 = +HEAPF64[$499>>3];
         $501 = $489 * $500;
         $502 = +HEAPF64[(3480)>>3];
         $503 = HEAP32[3488>>2]|0;
         $504 = (($503) - 1)|0;
         $505 = HEAP32[3464>>2]|0;
         $506 = (($505) - 1)|0;
         $507 = $6;
         $508 = HEAP32[$507>>2]|0;
         $509 = Math_imul($506, $508)|0;
         $510 = (($504) + ($509))|0;
         $511 = $5;
         $512 = (($511) + ($510<<4)|0);
         $513 = ((($512)) + 8|0);
         $514 = +HEAPF64[$513>>3];
         $515 = $502 * $514;
         $516 = $501 - $515;
         HEAPF64[$z__2>>3] = $516;
         $517 = +HEAPF64[3472>>3];
         $518 = HEAP32[3488>>2]|0;
         $519 = (($518) - 1)|0;
         $520 = HEAP32[3464>>2]|0;
         $521 = (($520) - 1)|0;
         $522 = $6;
         $523 = HEAP32[$522>>2]|0;
         $524 = Math_imul($521, $523)|0;
         $525 = (($519) + ($524))|0;
         $526 = $5;
         $527 = (($526) + ($525<<4)|0);
         $528 = ((($527)) + 8|0);
         $529 = +HEAPF64[$528>>3];
         $530 = $517 * $529;
         $531 = +HEAPF64[(3480)>>3];
         $532 = HEAP32[3488>>2]|0;
         $533 = (($532) - 1)|0;
         $534 = HEAP32[3464>>2]|0;
         $535 = (($534) - 1)|0;
         $536 = $6;
         $537 = HEAP32[$536>>2]|0;
         $538 = Math_imul($535, $537)|0;
         $539 = (($533) + ($538))|0;
         $540 = $5;
         $541 = (($540) + ($539<<4)|0);
         $542 = +HEAPF64[$541>>3];
         $543 = $531 * $542;
         $544 = $530 + $543;
         $545 = ((($z__2)) + 8|0);
         HEAPF64[$545>>3] = $544;
         $546 = HEAP32[3488>>2]|0;
         $547 = (($546) - 1)|0;
         $548 = $7;
         $549 = (($548) + ($547<<4)|0);
         $550 = +HEAPF64[$549>>3];
         $551 = +HEAPF64[$z__2>>3];
         $552 = $550 - $551;
         HEAPF64[$z__1>>3] = $552;
         $553 = HEAP32[3488>>2]|0;
         $554 = (($553) - 1)|0;
         $555 = $7;
         $556 = (($555) + ($554<<4)|0);
         $557 = ((($556)) + 8|0);
         $558 = +HEAPF64[$557>>3];
         $559 = ((($z__2)) + 8|0);
         $560 = +HEAPF64[$559>>3];
         $561 = $558 - $560;
         $562 = ((($z__1)) + 8|0);
         HEAPF64[$562>>3] = $561;
         $563 = +HEAPF64[$z__1>>3];
         $564 = HEAP32[3488>>2]|0;
         $565 = (($564) - 1)|0;
         $566 = $7;
         $567 = (($566) + ($565<<4)|0);
         HEAPF64[$567>>3] = $563;
         $568 = ((($z__1)) + 8|0);
         $569 = +HEAPF64[$568>>3];
         $570 = HEAP32[3488>>2]|0;
         $571 = (($570) - 1)|0;
         $572 = $7;
         $573 = (($572) + ($571<<4)|0);
         $574 = ((($573)) + 8|0);
         HEAPF64[$574>>3] = $569;
         $575 = HEAP32[3488>>2]|0;
         $576 = (($575) + 1)|0;
         HEAP32[3488>>2] = $576;
        }
       }
      } while(0);
      $577 = HEAP32[3464>>2]|0;
      $578 = (($577) + 1)|0;
      HEAP32[3464>>2] = $578;
     }
    } else {
     $579 = HEAP32[3456>>2]|0;
     HEAP32[3496>>2] = $579;
     $580 = $4;
     $581 = HEAP32[$580>>2]|0;
     $i__1 = $581;
     HEAP32[3464>>2] = 1;
     while(1) {
      $582 = HEAP32[3464>>2]|0;
      $583 = $4;
      $584 = HEAP32[$583>>2]|0;
      $585 = ($582|0)<=($584|0);
      if (!($585)) {
       break L39;
      }
      $586 = HEAP32[3496>>2]|0;
      $i__2 = $586;
      $587 = HEAP32[3496>>2]|0;
      $588 = (($587) - 1)|0;
      $589 = $7;
      $590 = (($589) + ($588<<4)|0);
      $591 = +HEAPF64[$590>>3];
      $592 = $591 != 0.0;
      if ($592) {
       label = 65;
      } else {
       $593 = HEAP32[3496>>2]|0;
       $594 = (($593) - 1)|0;
       $595 = $7;
       $596 = (($595) + ($594<<4)|0);
       $597 = ((($596)) + 8|0);
       $598 = +HEAPF64[$597>>3];
       $599 = $598 != 0.0;
       if ($599) {
        label = 65;
       }
      }
      L152: do {
       if ((label|0) == 65) {
        label = 0;
        $600 = HEAP32[3448>>2]|0;
        $601 = ($600|0)!=(0);
        if ($601) {
         $602 = HEAP32[3496>>2]|0;
         $i__2 = $602;
         $603 = HEAP32[3496>>2]|0;
         $604 = (($603) - 1)|0;
         $605 = $7;
         $606 = (($605) + ($604<<4)|0);
         $607 = HEAP32[3464>>2]|0;
         $608 = (($607) - 1)|0;
         $609 = HEAP32[3464>>2]|0;
         $610 = (($609) - 1)|0;
         $611 = $6;
         $612 = HEAP32[$611>>2]|0;
         $613 = Math_imul($610, $612)|0;
         $614 = (($608) + ($613))|0;
         $615 = $5;
         $616 = (($615) + ($614<<4)|0);
         _z_div($z__1,$606,$616);
         $617 = +HEAPF64[$z__1>>3];
         $618 = HEAP32[3496>>2]|0;
         $619 = (($618) - 1)|0;
         $620 = $7;
         $621 = (($620) + ($619<<4)|0);
         HEAPF64[$621>>3] = $617;
         $622 = ((($z__1)) + 8|0);
         $623 = +HEAPF64[$622>>3];
         $624 = HEAP32[3496>>2]|0;
         $625 = (($624) - 1)|0;
         $626 = $7;
         $627 = (($626) + ($625<<4)|0);
         $628 = ((($627)) + 8|0);
         HEAPF64[$628>>3] = $623;
        }
        $629 = HEAP32[3496>>2]|0;
        $i__2 = $629;
        $630 = HEAP32[3496>>2]|0;
        $631 = (($630) - 1)|0;
        $632 = $7;
        $633 = (($632) + ($631<<4)|0);
        $634 = +HEAPF64[$633>>3];
        HEAPF64[3472>>3] = $634;
        $635 = HEAP32[3496>>2]|0;
        $636 = (($635) - 1)|0;
        $637 = $7;
        $638 = (($637) + ($636<<4)|0);
        $639 = ((($638)) + 8|0);
        $640 = +HEAPF64[$639>>3];
        HEAPF64[(3480)>>3] = $640;
        $641 = HEAP32[3496>>2]|0;
        HEAP32[3504>>2] = $641;
        $642 = $4;
        $643 = HEAP32[$642>>2]|0;
        $i__2 = $643;
        $644 = HEAP32[3464>>2]|0;
        $645 = (($644) + 1)|0;
        HEAP32[3488>>2] = $645;
        while(1) {
         $646 = HEAP32[3488>>2]|0;
         $647 = $4;
         $648 = HEAP32[$647>>2]|0;
         $649 = ($646|0)<=($648|0);
         if (!($649)) {
          break L152;
         }
         $650 = $8;
         $651 = HEAP32[$650>>2]|0;
         $652 = HEAP32[3504>>2]|0;
         $653 = (($652) + ($651))|0;
         HEAP32[3504>>2] = $653;
         $654 = HEAP32[3504>>2]|0;
         $i__3 = $654;
         $655 = HEAP32[3504>>2]|0;
         $i__4 = $655;
         $656 = HEAP32[3488>>2]|0;
         $657 = HEAP32[3464>>2]|0;
         $658 = $a_dim1;
         $659 = Math_imul($657, $658)|0;
         $660 = (($656) + ($659))|0;
         $i__5 = $660;
         $661 = +HEAPF64[3472>>3];
         $662 = HEAP32[3488>>2]|0;
         $663 = (($662) - 1)|0;
         $664 = HEAP32[3464>>2]|0;
         $665 = (($664) - 1)|0;
         $666 = $6;
         $667 = HEAP32[$666>>2]|0;
         $668 = Math_imul($665, $667)|0;
         $669 = (($663) + ($668))|0;
         $670 = $5;
         $671 = (($670) + ($669<<4)|0);
         $672 = +HEAPF64[$671>>3];
         $673 = $661 * $672;
         $674 = +HEAPF64[(3480)>>3];
         $675 = HEAP32[3488>>2]|0;
         $676 = (($675) - 1)|0;
         $677 = HEAP32[3464>>2]|0;
         $678 = (($677) - 1)|0;
         $679 = $6;
         $680 = HEAP32[$679>>2]|0;
         $681 = Math_imul($678, $680)|0;
         $682 = (($676) + ($681))|0;
         $683 = $5;
         $684 = (($683) + ($682<<4)|0);
         $685 = ((($684)) + 8|0);
         $686 = +HEAPF64[$685>>3];
         $687 = $674 * $686;
         $688 = $673 - $687;
         HEAPF64[$z__2>>3] = $688;
         $689 = +HEAPF64[3472>>3];
         $690 = HEAP32[3488>>2]|0;
         $691 = (($690) - 1)|0;
         $692 = HEAP32[3464>>2]|0;
         $693 = (($692) - 1)|0;
         $694 = $6;
         $695 = HEAP32[$694>>2]|0;
         $696 = Math_imul($693, $695)|0;
         $697 = (($691) + ($696))|0;
         $698 = $5;
         $699 = (($698) + ($697<<4)|0);
         $700 = ((($699)) + 8|0);
         $701 = +HEAPF64[$700>>3];
         $702 = $689 * $701;
         $703 = +HEAPF64[(3480)>>3];
         $704 = HEAP32[3488>>2]|0;
         $705 = (($704) - 1)|0;
         $706 = HEAP32[3464>>2]|0;
         $707 = (($706) - 1)|0;
         $708 = $6;
         $709 = HEAP32[$708>>2]|0;
         $710 = Math_imul($707, $709)|0;
         $711 = (($705) + ($710))|0;
         $712 = $5;
         $713 = (($712) + ($711<<4)|0);
         $714 = +HEAPF64[$713>>3];
         $715 = $703 * $714;
         $716 = $702 + $715;
         $717 = ((($z__2)) + 8|0);
         HEAPF64[$717>>3] = $716;
         $718 = HEAP32[3504>>2]|0;
         $719 = (($718) - 1)|0;
         $720 = $7;
         $721 = (($720) + ($719<<4)|0);
         $722 = +HEAPF64[$721>>3];
         $723 = +HEAPF64[$z__2>>3];
         $724 = $722 - $723;
         HEAPF64[$z__1>>3] = $724;
         $725 = HEAP32[3504>>2]|0;
         $726 = (($725) - 1)|0;
         $727 = $7;
         $728 = (($727) + ($726<<4)|0);
         $729 = ((($728)) + 8|0);
         $730 = +HEAPF64[$729>>3];
         $731 = ((($z__2)) + 8|0);
         $732 = +HEAPF64[$731>>3];
         $733 = $730 - $732;
         $734 = ((($z__1)) + 8|0);
         HEAPF64[$734>>3] = $733;
         $735 = +HEAPF64[$z__1>>3];
         $736 = HEAP32[3504>>2]|0;
         $737 = (($736) - 1)|0;
         $738 = $7;
         $739 = (($738) + ($737<<4)|0);
         HEAPF64[$739>>3] = $735;
         $740 = ((($z__1)) + 8|0);
         $741 = +HEAPF64[$740>>3];
         $742 = HEAP32[3504>>2]|0;
         $743 = (($742) - 1)|0;
         $744 = $7;
         $745 = (($744) + ($743<<4)|0);
         $746 = ((($745)) + 8|0);
         HEAPF64[$746>>3] = $741;
         $747 = HEAP32[3488>>2]|0;
         $748 = (($747) + 1)|0;
         HEAP32[3488>>2] = $748;
        }
       }
      } while(0);
      $749 = $8;
      $750 = HEAP32[$749>>2]|0;
      $751 = HEAP32[3496>>2]|0;
      $752 = (($751) + ($750))|0;
      HEAP32[3496>>2] = $752;
      $753 = HEAP32[3464>>2]|0;
      $754 = (($753) + 1)|0;
      HEAP32[3464>>2] = $754;
     }
    }
   }
  } else {
   if ($72) {
    if ($75) {
     $755 = $4;
     $756 = HEAP32[$755>>2]|0;
     $i__1 = $756;
     HEAP32[3464>>2] = 1;
     while(1) {
      $757 = HEAP32[3464>>2]|0;
      $758 = $4;
      $759 = HEAP32[$758>>2]|0;
      $760 = ($757|0)<=($759|0);
      if (!($760)) {
       break L39;
      }
      $761 = HEAP32[3464>>2]|0;
      $i__2 = $761;
      $762 = HEAP32[3464>>2]|0;
      $763 = (($762) - 1)|0;
      $764 = $7;
      $765 = (($764) + ($763<<4)|0);
      $766 = +HEAPF64[$765>>3];
      HEAPF64[3472>>3] = $766;
      $767 = HEAP32[3464>>2]|0;
      $768 = (($767) - 1)|0;
      $769 = $7;
      $770 = (($769) + ($768<<4)|0);
      $771 = ((($770)) + 8|0);
      $772 = +HEAPF64[$771>>3];
      HEAPF64[(3480)>>3] = $772;
      $773 = HEAP32[3440>>2]|0;
      $774 = ($773|0)!=(0);
      $775 = HEAP32[3464>>2]|0;
      $776 = (($775) - 1)|0;
      $i__2 = $776;
      HEAP32[3488>>2] = 1;
      if ($774) {
       while(1) {
        $777 = HEAP32[3488>>2]|0;
        $778 = HEAP32[3464>>2]|0;
        $779 = (($778) - 1)|0;
        $780 = ($777|0)<=($779|0);
        if (!($780)) {
         break;
        }
        $781 = HEAP32[3488>>2]|0;
        $782 = HEAP32[3464>>2]|0;
        $783 = $a_dim1;
        $784 = Math_imul($782, $783)|0;
        $785 = (($781) + ($784))|0;
        $i__3 = $785;
        $786 = HEAP32[3488>>2]|0;
        $i__4 = $786;
        $787 = HEAP32[3488>>2]|0;
        $788 = (($787) - 1)|0;
        $789 = HEAP32[3464>>2]|0;
        $790 = (($789) - 1)|0;
        $791 = $6;
        $792 = HEAP32[$791>>2]|0;
        $793 = Math_imul($790, $792)|0;
        $794 = (($788) + ($793))|0;
        $795 = $5;
        $796 = (($795) + ($794<<4)|0);
        $797 = +HEAPF64[$796>>3];
        $798 = HEAP32[3488>>2]|0;
        $799 = (($798) - 1)|0;
        $800 = $7;
        $801 = (($800) + ($799<<4)|0);
        $802 = +HEAPF64[$801>>3];
        $803 = $797 * $802;
        $804 = HEAP32[3488>>2]|0;
        $805 = (($804) - 1)|0;
        $806 = HEAP32[3464>>2]|0;
        $807 = (($806) - 1)|0;
        $808 = $6;
        $809 = HEAP32[$808>>2]|0;
        $810 = Math_imul($807, $809)|0;
        $811 = (($805) + ($810))|0;
        $812 = $5;
        $813 = (($812) + ($811<<4)|0);
        $814 = ((($813)) + 8|0);
        $815 = +HEAPF64[$814>>3];
        $816 = HEAP32[3488>>2]|0;
        $817 = (($816) - 1)|0;
        $818 = $7;
        $819 = (($818) + ($817<<4)|0);
        $820 = ((($819)) + 8|0);
        $821 = +HEAPF64[$820>>3];
        $822 = $815 * $821;
        $823 = $803 - $822;
        HEAPF64[$z__2>>3] = $823;
        $824 = HEAP32[3488>>2]|0;
        $825 = (($824) - 1)|0;
        $826 = HEAP32[3464>>2]|0;
        $827 = (($826) - 1)|0;
        $828 = $6;
        $829 = HEAP32[$828>>2]|0;
        $830 = Math_imul($827, $829)|0;
        $831 = (($825) + ($830))|0;
        $832 = $5;
        $833 = (($832) + ($831<<4)|0);
        $834 = +HEAPF64[$833>>3];
        $835 = HEAP32[3488>>2]|0;
        $836 = (($835) - 1)|0;
        $837 = $7;
        $838 = (($837) + ($836<<4)|0);
        $839 = ((($838)) + 8|0);
        $840 = +HEAPF64[$839>>3];
        $841 = $834 * $840;
        $842 = HEAP32[3488>>2]|0;
        $843 = (($842) - 1)|0;
        $844 = HEAP32[3464>>2]|0;
        $845 = (($844) - 1)|0;
        $846 = $6;
        $847 = HEAP32[$846>>2]|0;
        $848 = Math_imul($845, $847)|0;
        $849 = (($843) + ($848))|0;
        $850 = $5;
        $851 = (($850) + ($849<<4)|0);
        $852 = ((($851)) + 8|0);
        $853 = +HEAPF64[$852>>3];
        $854 = HEAP32[3488>>2]|0;
        $855 = (($854) - 1)|0;
        $856 = $7;
        $857 = (($856) + ($855<<4)|0);
        $858 = +HEAPF64[$857>>3];
        $859 = $853 * $858;
        $860 = $841 + $859;
        $861 = ((($z__2)) + 8|0);
        HEAPF64[$861>>3] = $860;
        $862 = +HEAPF64[3472>>3];
        $863 = +HEAPF64[$z__2>>3];
        $864 = $862 - $863;
        HEAPF64[$z__1>>3] = $864;
        $865 = +HEAPF64[(3480)>>3];
        $866 = ((($z__2)) + 8|0);
        $867 = +HEAPF64[$866>>3];
        $868 = $865 - $867;
        $869 = ((($z__1)) + 8|0);
        HEAPF64[$869>>3] = $868;
        $870 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $870;
        $871 = ((($z__1)) + 8|0);
        $872 = +HEAPF64[$871>>3];
        HEAPF64[(3480)>>3] = $872;
        $873 = HEAP32[3488>>2]|0;
        $874 = (($873) + 1)|0;
        HEAP32[3488>>2] = $874;
       }
       $875 = HEAP32[3448>>2]|0;
       $876 = ($875|0)!=(0);
       if ($876) {
        $877 = HEAP32[3464>>2]|0;
        $878 = (($877) - 1)|0;
        $879 = HEAP32[3464>>2]|0;
        $880 = (($879) - 1)|0;
        $881 = $6;
        $882 = HEAP32[$881>>2]|0;
        $883 = Math_imul($880, $882)|0;
        $884 = (($878) + ($883))|0;
        $885 = $5;
        $886 = (($885) + ($884<<4)|0);
        _z_div($z__1,3472,$886);
        $887 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $887;
        $888 = ((($z__1)) + 8|0);
        $889 = +HEAPF64[$888>>3];
        HEAPF64[(3480)>>3] = $889;
       }
      } else {
       while(1) {
        $890 = HEAP32[3488>>2]|0;
        $891 = HEAP32[3464>>2]|0;
        $892 = (($891) - 1)|0;
        $893 = ($890|0)<=($892|0);
        if (!($893)) {
         break;
        }
        $894 = HEAP32[3488>>2]|0;
        $895 = (($894) - 1)|0;
        $896 = HEAP32[3464>>2]|0;
        $897 = (($896) - 1)|0;
        $898 = $6;
        $899 = HEAP32[$898>>2]|0;
        $900 = Math_imul($897, $899)|0;
        $901 = (($895) + ($900))|0;
        $902 = $5;
        $903 = (($902) + ($901<<4)|0);
        _d_cnjg($z__3,$903);
        $904 = HEAP32[3488>>2]|0;
        $i__3 = $904;
        $905 = +HEAPF64[$z__3>>3];
        $906 = HEAP32[3488>>2]|0;
        $907 = (($906) - 1)|0;
        $908 = $7;
        $909 = (($908) + ($907<<4)|0);
        $910 = +HEAPF64[$909>>3];
        $911 = $905 * $910;
        $912 = ((($z__3)) + 8|0);
        $913 = +HEAPF64[$912>>3];
        $914 = HEAP32[3488>>2]|0;
        $915 = (($914) - 1)|0;
        $916 = $7;
        $917 = (($916) + ($915<<4)|0);
        $918 = ((($917)) + 8|0);
        $919 = +HEAPF64[$918>>3];
        $920 = $913 * $919;
        $921 = $911 - $920;
        HEAPF64[$z__2>>3] = $921;
        $922 = +HEAPF64[$z__3>>3];
        $923 = HEAP32[3488>>2]|0;
        $924 = (($923) - 1)|0;
        $925 = $7;
        $926 = (($925) + ($924<<4)|0);
        $927 = ((($926)) + 8|0);
        $928 = +HEAPF64[$927>>3];
        $929 = $922 * $928;
        $930 = ((($z__3)) + 8|0);
        $931 = +HEAPF64[$930>>3];
        $932 = HEAP32[3488>>2]|0;
        $933 = (($932) - 1)|0;
        $934 = $7;
        $935 = (($934) + ($933<<4)|0);
        $936 = +HEAPF64[$935>>3];
        $937 = $931 * $936;
        $938 = $929 + $937;
        $939 = ((($z__2)) + 8|0);
        HEAPF64[$939>>3] = $938;
        $940 = +HEAPF64[3472>>3];
        $941 = +HEAPF64[$z__2>>3];
        $942 = $940 - $941;
        HEAPF64[$z__1>>3] = $942;
        $943 = +HEAPF64[(3480)>>3];
        $944 = ((($z__2)) + 8|0);
        $945 = +HEAPF64[$944>>3];
        $946 = $943 - $945;
        $947 = ((($z__1)) + 8|0);
        HEAPF64[$947>>3] = $946;
        $948 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $948;
        $949 = ((($z__1)) + 8|0);
        $950 = +HEAPF64[$949>>3];
        HEAPF64[(3480)>>3] = $950;
        $951 = HEAP32[3488>>2]|0;
        $952 = (($951) + 1)|0;
        HEAP32[3488>>2] = $952;
       }
       $953 = HEAP32[3448>>2]|0;
       $954 = ($953|0)!=(0);
       if ($954) {
        $955 = HEAP32[3464>>2]|0;
        $956 = (($955) - 1)|0;
        $957 = HEAP32[3464>>2]|0;
        $958 = (($957) - 1)|0;
        $959 = $6;
        $960 = HEAP32[$959>>2]|0;
        $961 = Math_imul($958, $960)|0;
        $962 = (($956) + ($961))|0;
        $963 = $5;
        $964 = (($963) + ($962<<4)|0);
        _d_cnjg($z__2,$964);
        _z_div($z__1,3472,$z__2);
        $965 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $965;
        $966 = ((($z__1)) + 8|0);
        $967 = +HEAPF64[$966>>3];
        HEAPF64[(3480)>>3] = $967;
       }
      }
      $968 = HEAP32[3464>>2]|0;
      $i__2 = $968;
      $969 = +HEAPF64[3472>>3];
      $970 = HEAP32[3464>>2]|0;
      $971 = (($970) - 1)|0;
      $972 = $7;
      $973 = (($972) + ($971<<4)|0);
      HEAPF64[$973>>3] = $969;
      $974 = +HEAPF64[(3480)>>3];
      $975 = HEAP32[3464>>2]|0;
      $976 = (($975) - 1)|0;
      $977 = $7;
      $978 = (($977) + ($976<<4)|0);
      $979 = ((($978)) + 8|0);
      HEAPF64[$979>>3] = $974;
      $980 = HEAP32[3464>>2]|0;
      $981 = (($980) + 1)|0;
      HEAP32[3464>>2] = $981;
     }
    } else {
     $982 = HEAP32[3456>>2]|0;
     HEAP32[3496>>2] = $982;
     $983 = $4;
     $984 = HEAP32[$983>>2]|0;
     $i__1 = $984;
     HEAP32[3464>>2] = 1;
     while(1) {
      $985 = HEAP32[3464>>2]|0;
      $986 = $4;
      $987 = HEAP32[$986>>2]|0;
      $988 = ($985|0)<=($987|0);
      if (!($988)) {
       break L39;
      }
      $989 = HEAP32[3456>>2]|0;
      HEAP32[3504>>2] = $989;
      $990 = HEAP32[3496>>2]|0;
      $i__2 = $990;
      $991 = HEAP32[3496>>2]|0;
      $992 = (($991) - 1)|0;
      $993 = $7;
      $994 = (($993) + ($992<<4)|0);
      $995 = +HEAPF64[$994>>3];
      HEAPF64[3472>>3] = $995;
      $996 = HEAP32[3496>>2]|0;
      $997 = (($996) - 1)|0;
      $998 = $7;
      $999 = (($998) + ($997<<4)|0);
      $1000 = ((($999)) + 8|0);
      $1001 = +HEAPF64[$1000>>3];
      HEAPF64[(3480)>>3] = $1001;
      $1002 = HEAP32[3440>>2]|0;
      $1003 = ($1002|0)!=(0);
      $1004 = HEAP32[3464>>2]|0;
      $1005 = (($1004) - 1)|0;
      $i__2 = $1005;
      HEAP32[3488>>2] = 1;
      if ($1003) {
       while(1) {
        $1006 = HEAP32[3488>>2]|0;
        $1007 = HEAP32[3464>>2]|0;
        $1008 = (($1007) - 1)|0;
        $1009 = ($1006|0)<=($1008|0);
        if (!($1009)) {
         break;
        }
        $1010 = HEAP32[3488>>2]|0;
        $1011 = HEAP32[3464>>2]|0;
        $1012 = $a_dim1;
        $1013 = Math_imul($1011, $1012)|0;
        $1014 = (($1010) + ($1013))|0;
        $i__3 = $1014;
        $1015 = HEAP32[3504>>2]|0;
        $i__4 = $1015;
        $1016 = HEAP32[3488>>2]|0;
        $1017 = (($1016) - 1)|0;
        $1018 = HEAP32[3464>>2]|0;
        $1019 = (($1018) - 1)|0;
        $1020 = $6;
        $1021 = HEAP32[$1020>>2]|0;
        $1022 = Math_imul($1019, $1021)|0;
        $1023 = (($1017) + ($1022))|0;
        $1024 = $5;
        $1025 = (($1024) + ($1023<<4)|0);
        $1026 = +HEAPF64[$1025>>3];
        $1027 = HEAP32[3504>>2]|0;
        $1028 = (($1027) - 1)|0;
        $1029 = $7;
        $1030 = (($1029) + ($1028<<4)|0);
        $1031 = +HEAPF64[$1030>>3];
        $1032 = $1026 * $1031;
        $1033 = HEAP32[3488>>2]|0;
        $1034 = (($1033) - 1)|0;
        $1035 = HEAP32[3464>>2]|0;
        $1036 = (($1035) - 1)|0;
        $1037 = $6;
        $1038 = HEAP32[$1037>>2]|0;
        $1039 = Math_imul($1036, $1038)|0;
        $1040 = (($1034) + ($1039))|0;
        $1041 = $5;
        $1042 = (($1041) + ($1040<<4)|0);
        $1043 = ((($1042)) + 8|0);
        $1044 = +HEAPF64[$1043>>3];
        $1045 = HEAP32[3504>>2]|0;
        $1046 = (($1045) - 1)|0;
        $1047 = $7;
        $1048 = (($1047) + ($1046<<4)|0);
        $1049 = ((($1048)) + 8|0);
        $1050 = +HEAPF64[$1049>>3];
        $1051 = $1044 * $1050;
        $1052 = $1032 - $1051;
        HEAPF64[$z__2>>3] = $1052;
        $1053 = HEAP32[3488>>2]|0;
        $1054 = (($1053) - 1)|0;
        $1055 = HEAP32[3464>>2]|0;
        $1056 = (($1055) - 1)|0;
        $1057 = $6;
        $1058 = HEAP32[$1057>>2]|0;
        $1059 = Math_imul($1056, $1058)|0;
        $1060 = (($1054) + ($1059))|0;
        $1061 = $5;
        $1062 = (($1061) + ($1060<<4)|0);
        $1063 = +HEAPF64[$1062>>3];
        $1064 = HEAP32[3504>>2]|0;
        $1065 = (($1064) - 1)|0;
        $1066 = $7;
        $1067 = (($1066) + ($1065<<4)|0);
        $1068 = ((($1067)) + 8|0);
        $1069 = +HEAPF64[$1068>>3];
        $1070 = $1063 * $1069;
        $1071 = HEAP32[3488>>2]|0;
        $1072 = (($1071) - 1)|0;
        $1073 = HEAP32[3464>>2]|0;
        $1074 = (($1073) - 1)|0;
        $1075 = $6;
        $1076 = HEAP32[$1075>>2]|0;
        $1077 = Math_imul($1074, $1076)|0;
        $1078 = (($1072) + ($1077))|0;
        $1079 = $5;
        $1080 = (($1079) + ($1078<<4)|0);
        $1081 = ((($1080)) + 8|0);
        $1082 = +HEAPF64[$1081>>3];
        $1083 = HEAP32[3504>>2]|0;
        $1084 = (($1083) - 1)|0;
        $1085 = $7;
        $1086 = (($1085) + ($1084<<4)|0);
        $1087 = +HEAPF64[$1086>>3];
        $1088 = $1082 * $1087;
        $1089 = $1070 + $1088;
        $1090 = ((($z__2)) + 8|0);
        HEAPF64[$1090>>3] = $1089;
        $1091 = +HEAPF64[3472>>3];
        $1092 = +HEAPF64[$z__2>>3];
        $1093 = $1091 - $1092;
        HEAPF64[$z__1>>3] = $1093;
        $1094 = +HEAPF64[(3480)>>3];
        $1095 = ((($z__2)) + 8|0);
        $1096 = +HEAPF64[$1095>>3];
        $1097 = $1094 - $1096;
        $1098 = ((($z__1)) + 8|0);
        HEAPF64[$1098>>3] = $1097;
        $1099 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1099;
        $1100 = ((($z__1)) + 8|0);
        $1101 = +HEAPF64[$1100>>3];
        HEAPF64[(3480)>>3] = $1101;
        $1102 = $8;
        $1103 = HEAP32[$1102>>2]|0;
        $1104 = HEAP32[3504>>2]|0;
        $1105 = (($1104) + ($1103))|0;
        HEAP32[3504>>2] = $1105;
        $1106 = HEAP32[3488>>2]|0;
        $1107 = (($1106) + 1)|0;
        HEAP32[3488>>2] = $1107;
       }
       $1108 = HEAP32[3448>>2]|0;
       $1109 = ($1108|0)!=(0);
       if ($1109) {
        $1110 = HEAP32[3464>>2]|0;
        $1111 = (($1110) - 1)|0;
        $1112 = HEAP32[3464>>2]|0;
        $1113 = (($1112) - 1)|0;
        $1114 = $6;
        $1115 = HEAP32[$1114>>2]|0;
        $1116 = Math_imul($1113, $1115)|0;
        $1117 = (($1111) + ($1116))|0;
        $1118 = $5;
        $1119 = (($1118) + ($1117<<4)|0);
        _z_div($z__1,3472,$1119);
        $1120 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1120;
        $1121 = ((($z__1)) + 8|0);
        $1122 = +HEAPF64[$1121>>3];
        HEAPF64[(3480)>>3] = $1122;
       }
      } else {
       while(1) {
        $1123 = HEAP32[3488>>2]|0;
        $1124 = HEAP32[3464>>2]|0;
        $1125 = (($1124) - 1)|0;
        $1126 = ($1123|0)<=($1125|0);
        if (!($1126)) {
         break;
        }
        $1127 = HEAP32[3488>>2]|0;
        $1128 = (($1127) - 1)|0;
        $1129 = HEAP32[3464>>2]|0;
        $1130 = (($1129) - 1)|0;
        $1131 = $6;
        $1132 = HEAP32[$1131>>2]|0;
        $1133 = Math_imul($1130, $1132)|0;
        $1134 = (($1128) + ($1133))|0;
        $1135 = $5;
        $1136 = (($1135) + ($1134<<4)|0);
        _d_cnjg($z__3,$1136);
        $1137 = HEAP32[3504>>2]|0;
        $i__3 = $1137;
        $1138 = +HEAPF64[$z__3>>3];
        $1139 = HEAP32[3504>>2]|0;
        $1140 = (($1139) - 1)|0;
        $1141 = $7;
        $1142 = (($1141) + ($1140<<4)|0);
        $1143 = +HEAPF64[$1142>>3];
        $1144 = $1138 * $1143;
        $1145 = ((($z__3)) + 8|0);
        $1146 = +HEAPF64[$1145>>3];
        $1147 = HEAP32[3504>>2]|0;
        $1148 = (($1147) - 1)|0;
        $1149 = $7;
        $1150 = (($1149) + ($1148<<4)|0);
        $1151 = ((($1150)) + 8|0);
        $1152 = +HEAPF64[$1151>>3];
        $1153 = $1146 * $1152;
        $1154 = $1144 - $1153;
        HEAPF64[$z__2>>3] = $1154;
        $1155 = +HEAPF64[$z__3>>3];
        $1156 = HEAP32[3504>>2]|0;
        $1157 = (($1156) - 1)|0;
        $1158 = $7;
        $1159 = (($1158) + ($1157<<4)|0);
        $1160 = ((($1159)) + 8|0);
        $1161 = +HEAPF64[$1160>>3];
        $1162 = $1155 * $1161;
        $1163 = ((($z__3)) + 8|0);
        $1164 = +HEAPF64[$1163>>3];
        $1165 = HEAP32[3504>>2]|0;
        $1166 = (($1165) - 1)|0;
        $1167 = $7;
        $1168 = (($1167) + ($1166<<4)|0);
        $1169 = +HEAPF64[$1168>>3];
        $1170 = $1164 * $1169;
        $1171 = $1162 + $1170;
        $1172 = ((($z__2)) + 8|0);
        HEAPF64[$1172>>3] = $1171;
        $1173 = +HEAPF64[3472>>3];
        $1174 = +HEAPF64[$z__2>>3];
        $1175 = $1173 - $1174;
        HEAPF64[$z__1>>3] = $1175;
        $1176 = +HEAPF64[(3480)>>3];
        $1177 = ((($z__2)) + 8|0);
        $1178 = +HEAPF64[$1177>>3];
        $1179 = $1176 - $1178;
        $1180 = ((($z__1)) + 8|0);
        HEAPF64[$1180>>3] = $1179;
        $1181 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1181;
        $1182 = ((($z__1)) + 8|0);
        $1183 = +HEAPF64[$1182>>3];
        HEAPF64[(3480)>>3] = $1183;
        $1184 = $8;
        $1185 = HEAP32[$1184>>2]|0;
        $1186 = HEAP32[3504>>2]|0;
        $1187 = (($1186) + ($1185))|0;
        HEAP32[3504>>2] = $1187;
        $1188 = HEAP32[3488>>2]|0;
        $1189 = (($1188) + 1)|0;
        HEAP32[3488>>2] = $1189;
       }
       $1190 = HEAP32[3448>>2]|0;
       $1191 = ($1190|0)!=(0);
       if ($1191) {
        $1192 = HEAP32[3464>>2]|0;
        $1193 = (($1192) - 1)|0;
        $1194 = HEAP32[3464>>2]|0;
        $1195 = (($1194) - 1)|0;
        $1196 = $6;
        $1197 = HEAP32[$1196>>2]|0;
        $1198 = Math_imul($1195, $1197)|0;
        $1199 = (($1193) + ($1198))|0;
        $1200 = $5;
        $1201 = (($1200) + ($1199<<4)|0);
        _d_cnjg($z__2,$1201);
        _z_div($z__1,3472,$z__2);
        $1202 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1202;
        $1203 = ((($z__1)) + 8|0);
        $1204 = +HEAPF64[$1203>>3];
        HEAPF64[(3480)>>3] = $1204;
       }
      }
      $1205 = HEAP32[3496>>2]|0;
      $i__2 = $1205;
      $1206 = +HEAPF64[3472>>3];
      $1207 = HEAP32[3496>>2]|0;
      $1208 = (($1207) - 1)|0;
      $1209 = $7;
      $1210 = (($1209) + ($1208<<4)|0);
      HEAPF64[$1210>>3] = $1206;
      $1211 = +HEAPF64[(3480)>>3];
      $1212 = HEAP32[3496>>2]|0;
      $1213 = (($1212) - 1)|0;
      $1214 = $7;
      $1215 = (($1214) + ($1213<<4)|0);
      $1216 = ((($1215)) + 8|0);
      HEAPF64[$1216>>3] = $1211;
      $1217 = $8;
      $1218 = HEAP32[$1217>>2]|0;
      $1219 = HEAP32[3496>>2]|0;
      $1220 = (($1219) + ($1218))|0;
      HEAP32[3496>>2] = $1220;
      $1221 = HEAP32[3464>>2]|0;
      $1222 = (($1221) + 1)|0;
      HEAP32[3464>>2] = $1222;
     }
    }
   } else {
    $1223 = $4;
    $1224 = HEAP32[$1223>>2]|0;
    if ($75) {
     HEAP32[3464>>2] = $1224;
     while(1) {
      $1225 = HEAP32[3464>>2]|0;
      $1226 = ($1225|0)>=(1);
      if (!($1226)) {
       break L39;
      }
      $1227 = HEAP32[3464>>2]|0;
      $i__1 = $1227;
      $1228 = HEAP32[3464>>2]|0;
      $1229 = (($1228) - 1)|0;
      $1230 = $7;
      $1231 = (($1230) + ($1229<<4)|0);
      $1232 = +HEAPF64[$1231>>3];
      HEAPF64[3472>>3] = $1232;
      $1233 = HEAP32[3464>>2]|0;
      $1234 = (($1233) - 1)|0;
      $1235 = $7;
      $1236 = (($1235) + ($1234<<4)|0);
      $1237 = ((($1236)) + 8|0);
      $1238 = +HEAPF64[$1237>>3];
      HEAPF64[(3480)>>3] = $1238;
      $1239 = HEAP32[3440>>2]|0;
      $1240 = ($1239|0)!=(0);
      $1241 = HEAP32[3464>>2]|0;
      $1242 = (($1241) + 1)|0;
      $i__1 = $1242;
      $1243 = $4;
      $1244 = HEAP32[$1243>>2]|0;
      HEAP32[3488>>2] = $1244;
      if ($1240) {
       while(1) {
        $1245 = HEAP32[3488>>2]|0;
        $1246 = HEAP32[3464>>2]|0;
        $1247 = (($1246) + 1)|0;
        $1248 = ($1245|0)>=($1247|0);
        if (!($1248)) {
         break;
        }
        $1249 = HEAP32[3488>>2]|0;
        $1250 = HEAP32[3464>>2]|0;
        $1251 = $a_dim1;
        $1252 = Math_imul($1250, $1251)|0;
        $1253 = (($1249) + ($1252))|0;
        $i__2 = $1253;
        $1254 = HEAP32[3488>>2]|0;
        $i__3 = $1254;
        $1255 = HEAP32[3488>>2]|0;
        $1256 = (($1255) - 1)|0;
        $1257 = HEAP32[3464>>2]|0;
        $1258 = (($1257) - 1)|0;
        $1259 = $6;
        $1260 = HEAP32[$1259>>2]|0;
        $1261 = Math_imul($1258, $1260)|0;
        $1262 = (($1256) + ($1261))|0;
        $1263 = $5;
        $1264 = (($1263) + ($1262<<4)|0);
        $1265 = +HEAPF64[$1264>>3];
        $1266 = HEAP32[3488>>2]|0;
        $1267 = (($1266) - 1)|0;
        $1268 = $7;
        $1269 = (($1268) + ($1267<<4)|0);
        $1270 = +HEAPF64[$1269>>3];
        $1271 = $1265 * $1270;
        $1272 = HEAP32[3488>>2]|0;
        $1273 = (($1272) - 1)|0;
        $1274 = HEAP32[3464>>2]|0;
        $1275 = (($1274) - 1)|0;
        $1276 = $6;
        $1277 = HEAP32[$1276>>2]|0;
        $1278 = Math_imul($1275, $1277)|0;
        $1279 = (($1273) + ($1278))|0;
        $1280 = $5;
        $1281 = (($1280) + ($1279<<4)|0);
        $1282 = ((($1281)) + 8|0);
        $1283 = +HEAPF64[$1282>>3];
        $1284 = HEAP32[3488>>2]|0;
        $1285 = (($1284) - 1)|0;
        $1286 = $7;
        $1287 = (($1286) + ($1285<<4)|0);
        $1288 = ((($1287)) + 8|0);
        $1289 = +HEAPF64[$1288>>3];
        $1290 = $1283 * $1289;
        $1291 = $1271 - $1290;
        HEAPF64[$z__2>>3] = $1291;
        $1292 = HEAP32[3488>>2]|0;
        $1293 = (($1292) - 1)|0;
        $1294 = HEAP32[3464>>2]|0;
        $1295 = (($1294) - 1)|0;
        $1296 = $6;
        $1297 = HEAP32[$1296>>2]|0;
        $1298 = Math_imul($1295, $1297)|0;
        $1299 = (($1293) + ($1298))|0;
        $1300 = $5;
        $1301 = (($1300) + ($1299<<4)|0);
        $1302 = +HEAPF64[$1301>>3];
        $1303 = HEAP32[3488>>2]|0;
        $1304 = (($1303) - 1)|0;
        $1305 = $7;
        $1306 = (($1305) + ($1304<<4)|0);
        $1307 = ((($1306)) + 8|0);
        $1308 = +HEAPF64[$1307>>3];
        $1309 = $1302 * $1308;
        $1310 = HEAP32[3488>>2]|0;
        $1311 = (($1310) - 1)|0;
        $1312 = HEAP32[3464>>2]|0;
        $1313 = (($1312) - 1)|0;
        $1314 = $6;
        $1315 = HEAP32[$1314>>2]|0;
        $1316 = Math_imul($1313, $1315)|0;
        $1317 = (($1311) + ($1316))|0;
        $1318 = $5;
        $1319 = (($1318) + ($1317<<4)|0);
        $1320 = ((($1319)) + 8|0);
        $1321 = +HEAPF64[$1320>>3];
        $1322 = HEAP32[3488>>2]|0;
        $1323 = (($1322) - 1)|0;
        $1324 = $7;
        $1325 = (($1324) + ($1323<<4)|0);
        $1326 = +HEAPF64[$1325>>3];
        $1327 = $1321 * $1326;
        $1328 = $1309 + $1327;
        $1329 = ((($z__2)) + 8|0);
        HEAPF64[$1329>>3] = $1328;
        $1330 = +HEAPF64[3472>>3];
        $1331 = +HEAPF64[$z__2>>3];
        $1332 = $1330 - $1331;
        HEAPF64[$z__1>>3] = $1332;
        $1333 = +HEAPF64[(3480)>>3];
        $1334 = ((($z__2)) + 8|0);
        $1335 = +HEAPF64[$1334>>3];
        $1336 = $1333 - $1335;
        $1337 = ((($z__1)) + 8|0);
        HEAPF64[$1337>>3] = $1336;
        $1338 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1338;
        $1339 = ((($z__1)) + 8|0);
        $1340 = +HEAPF64[$1339>>3];
        HEAPF64[(3480)>>3] = $1340;
        $1341 = HEAP32[3488>>2]|0;
        $1342 = (($1341) + -1)|0;
        HEAP32[3488>>2] = $1342;
       }
       $1343 = HEAP32[3448>>2]|0;
       $1344 = ($1343|0)!=(0);
       if ($1344) {
        $1345 = HEAP32[3464>>2]|0;
        $1346 = (($1345) - 1)|0;
        $1347 = HEAP32[3464>>2]|0;
        $1348 = (($1347) - 1)|0;
        $1349 = $6;
        $1350 = HEAP32[$1349>>2]|0;
        $1351 = Math_imul($1348, $1350)|0;
        $1352 = (($1346) + ($1351))|0;
        $1353 = $5;
        $1354 = (($1353) + ($1352<<4)|0);
        _z_div($z__1,3472,$1354);
        $1355 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1355;
        $1356 = ((($z__1)) + 8|0);
        $1357 = +HEAPF64[$1356>>3];
        HEAPF64[(3480)>>3] = $1357;
       }
      } else {
       while(1) {
        $1358 = HEAP32[3488>>2]|0;
        $1359 = HEAP32[3464>>2]|0;
        $1360 = (($1359) + 1)|0;
        $1361 = ($1358|0)>=($1360|0);
        if (!($1361)) {
         break;
        }
        $1362 = HEAP32[3488>>2]|0;
        $1363 = (($1362) - 1)|0;
        $1364 = HEAP32[3464>>2]|0;
        $1365 = (($1364) - 1)|0;
        $1366 = $6;
        $1367 = HEAP32[$1366>>2]|0;
        $1368 = Math_imul($1365, $1367)|0;
        $1369 = (($1363) + ($1368))|0;
        $1370 = $5;
        $1371 = (($1370) + ($1369<<4)|0);
        _d_cnjg($z__3,$1371);
        $1372 = HEAP32[3488>>2]|0;
        $i__2 = $1372;
        $1373 = +HEAPF64[$z__3>>3];
        $1374 = HEAP32[3488>>2]|0;
        $1375 = (($1374) - 1)|0;
        $1376 = $7;
        $1377 = (($1376) + ($1375<<4)|0);
        $1378 = +HEAPF64[$1377>>3];
        $1379 = $1373 * $1378;
        $1380 = ((($z__3)) + 8|0);
        $1381 = +HEAPF64[$1380>>3];
        $1382 = HEAP32[3488>>2]|0;
        $1383 = (($1382) - 1)|0;
        $1384 = $7;
        $1385 = (($1384) + ($1383<<4)|0);
        $1386 = ((($1385)) + 8|0);
        $1387 = +HEAPF64[$1386>>3];
        $1388 = $1381 * $1387;
        $1389 = $1379 - $1388;
        HEAPF64[$z__2>>3] = $1389;
        $1390 = +HEAPF64[$z__3>>3];
        $1391 = HEAP32[3488>>2]|0;
        $1392 = (($1391) - 1)|0;
        $1393 = $7;
        $1394 = (($1393) + ($1392<<4)|0);
        $1395 = ((($1394)) + 8|0);
        $1396 = +HEAPF64[$1395>>3];
        $1397 = $1390 * $1396;
        $1398 = ((($z__3)) + 8|0);
        $1399 = +HEAPF64[$1398>>3];
        $1400 = HEAP32[3488>>2]|0;
        $1401 = (($1400) - 1)|0;
        $1402 = $7;
        $1403 = (($1402) + ($1401<<4)|0);
        $1404 = +HEAPF64[$1403>>3];
        $1405 = $1399 * $1404;
        $1406 = $1397 + $1405;
        $1407 = ((($z__2)) + 8|0);
        HEAPF64[$1407>>3] = $1406;
        $1408 = +HEAPF64[3472>>3];
        $1409 = +HEAPF64[$z__2>>3];
        $1410 = $1408 - $1409;
        HEAPF64[$z__1>>3] = $1410;
        $1411 = +HEAPF64[(3480)>>3];
        $1412 = ((($z__2)) + 8|0);
        $1413 = +HEAPF64[$1412>>3];
        $1414 = $1411 - $1413;
        $1415 = ((($z__1)) + 8|0);
        HEAPF64[$1415>>3] = $1414;
        $1416 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1416;
        $1417 = ((($z__1)) + 8|0);
        $1418 = +HEAPF64[$1417>>3];
        HEAPF64[(3480)>>3] = $1418;
        $1419 = HEAP32[3488>>2]|0;
        $1420 = (($1419) + -1)|0;
        HEAP32[3488>>2] = $1420;
       }
       $1421 = HEAP32[3448>>2]|0;
       $1422 = ($1421|0)!=(0);
       if ($1422) {
        $1423 = HEAP32[3464>>2]|0;
        $1424 = (($1423) - 1)|0;
        $1425 = HEAP32[3464>>2]|0;
        $1426 = (($1425) - 1)|0;
        $1427 = $6;
        $1428 = HEAP32[$1427>>2]|0;
        $1429 = Math_imul($1426, $1428)|0;
        $1430 = (($1424) + ($1429))|0;
        $1431 = $5;
        $1432 = (($1431) + ($1430<<4)|0);
        _d_cnjg($z__2,$1432);
        _z_div($z__1,3472,$z__2);
        $1433 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1433;
        $1434 = ((($z__1)) + 8|0);
        $1435 = +HEAPF64[$1434>>3];
        HEAPF64[(3480)>>3] = $1435;
       }
      }
      $1436 = HEAP32[3464>>2]|0;
      $i__1 = $1436;
      $1437 = +HEAPF64[3472>>3];
      $1438 = HEAP32[3464>>2]|0;
      $1439 = (($1438) - 1)|0;
      $1440 = $7;
      $1441 = (($1440) + ($1439<<4)|0);
      HEAPF64[$1441>>3] = $1437;
      $1442 = +HEAPF64[(3480)>>3];
      $1443 = HEAP32[3464>>2]|0;
      $1444 = (($1443) - 1)|0;
      $1445 = $7;
      $1446 = (($1445) + ($1444<<4)|0);
      $1447 = ((($1446)) + 8|0);
      HEAPF64[$1447>>3] = $1442;
      $1448 = HEAP32[3464>>2]|0;
      $1449 = (($1448) + -1)|0;
      HEAP32[3464>>2] = $1449;
     }
    } else {
     $1450 = (($1224) - 1)|0;
     $1451 = $8;
     $1452 = HEAP32[$1451>>2]|0;
     $1453 = Math_imul($1450, $1452)|0;
     $1454 = HEAP32[3456>>2]|0;
     $1455 = (($1454) + ($1453))|0;
     HEAP32[3456>>2] = $1455;
     $1456 = HEAP32[3456>>2]|0;
     HEAP32[3496>>2] = $1456;
     $1457 = $4;
     $1458 = HEAP32[$1457>>2]|0;
     HEAP32[3464>>2] = $1458;
     while(1) {
      $1459 = HEAP32[3464>>2]|0;
      $1460 = ($1459|0)>=(1);
      if (!($1460)) {
       break L39;
      }
      $1461 = HEAP32[3456>>2]|0;
      HEAP32[3504>>2] = $1461;
      $1462 = HEAP32[3496>>2]|0;
      $i__1 = $1462;
      $1463 = HEAP32[3496>>2]|0;
      $1464 = (($1463) - 1)|0;
      $1465 = $7;
      $1466 = (($1465) + ($1464<<4)|0);
      $1467 = +HEAPF64[$1466>>3];
      HEAPF64[3472>>3] = $1467;
      $1468 = HEAP32[3496>>2]|0;
      $1469 = (($1468) - 1)|0;
      $1470 = $7;
      $1471 = (($1470) + ($1469<<4)|0);
      $1472 = ((($1471)) + 8|0);
      $1473 = +HEAPF64[$1472>>3];
      HEAPF64[(3480)>>3] = $1473;
      $1474 = HEAP32[3440>>2]|0;
      $1475 = ($1474|0)!=(0);
      $1476 = HEAP32[3464>>2]|0;
      $1477 = (($1476) + 1)|0;
      $i__1 = $1477;
      $1478 = $4;
      $1479 = HEAP32[$1478>>2]|0;
      HEAP32[3488>>2] = $1479;
      if ($1475) {
       while(1) {
        $1480 = HEAP32[3488>>2]|0;
        $1481 = HEAP32[3464>>2]|0;
        $1482 = (($1481) + 1)|0;
        $1483 = ($1480|0)>=($1482|0);
        if (!($1483)) {
         break;
        }
        $1484 = HEAP32[3488>>2]|0;
        $1485 = HEAP32[3464>>2]|0;
        $1486 = $a_dim1;
        $1487 = Math_imul($1485, $1486)|0;
        $1488 = (($1484) + ($1487))|0;
        $i__2 = $1488;
        $1489 = HEAP32[3504>>2]|0;
        $i__3 = $1489;
        $1490 = HEAP32[3488>>2]|0;
        $1491 = (($1490) - 1)|0;
        $1492 = HEAP32[3464>>2]|0;
        $1493 = (($1492) - 1)|0;
        $1494 = $6;
        $1495 = HEAP32[$1494>>2]|0;
        $1496 = Math_imul($1493, $1495)|0;
        $1497 = (($1491) + ($1496))|0;
        $1498 = $5;
        $1499 = (($1498) + ($1497<<4)|0);
        $1500 = +HEAPF64[$1499>>3];
        $1501 = HEAP32[3504>>2]|0;
        $1502 = (($1501) - 1)|0;
        $1503 = $7;
        $1504 = (($1503) + ($1502<<4)|0);
        $1505 = +HEAPF64[$1504>>3];
        $1506 = $1500 * $1505;
        $1507 = HEAP32[3488>>2]|0;
        $1508 = (($1507) - 1)|0;
        $1509 = HEAP32[3464>>2]|0;
        $1510 = (($1509) - 1)|0;
        $1511 = $6;
        $1512 = HEAP32[$1511>>2]|0;
        $1513 = Math_imul($1510, $1512)|0;
        $1514 = (($1508) + ($1513))|0;
        $1515 = $5;
        $1516 = (($1515) + ($1514<<4)|0);
        $1517 = ((($1516)) + 8|0);
        $1518 = +HEAPF64[$1517>>3];
        $1519 = HEAP32[3504>>2]|0;
        $1520 = (($1519) - 1)|0;
        $1521 = $7;
        $1522 = (($1521) + ($1520<<4)|0);
        $1523 = ((($1522)) + 8|0);
        $1524 = +HEAPF64[$1523>>3];
        $1525 = $1518 * $1524;
        $1526 = $1506 - $1525;
        HEAPF64[$z__2>>3] = $1526;
        $1527 = HEAP32[3488>>2]|0;
        $1528 = (($1527) - 1)|0;
        $1529 = HEAP32[3464>>2]|0;
        $1530 = (($1529) - 1)|0;
        $1531 = $6;
        $1532 = HEAP32[$1531>>2]|0;
        $1533 = Math_imul($1530, $1532)|0;
        $1534 = (($1528) + ($1533))|0;
        $1535 = $5;
        $1536 = (($1535) + ($1534<<4)|0);
        $1537 = +HEAPF64[$1536>>3];
        $1538 = HEAP32[3504>>2]|0;
        $1539 = (($1538) - 1)|0;
        $1540 = $7;
        $1541 = (($1540) + ($1539<<4)|0);
        $1542 = ((($1541)) + 8|0);
        $1543 = +HEAPF64[$1542>>3];
        $1544 = $1537 * $1543;
        $1545 = HEAP32[3488>>2]|0;
        $1546 = (($1545) - 1)|0;
        $1547 = HEAP32[3464>>2]|0;
        $1548 = (($1547) - 1)|0;
        $1549 = $6;
        $1550 = HEAP32[$1549>>2]|0;
        $1551 = Math_imul($1548, $1550)|0;
        $1552 = (($1546) + ($1551))|0;
        $1553 = $5;
        $1554 = (($1553) + ($1552<<4)|0);
        $1555 = ((($1554)) + 8|0);
        $1556 = +HEAPF64[$1555>>3];
        $1557 = HEAP32[3504>>2]|0;
        $1558 = (($1557) - 1)|0;
        $1559 = $7;
        $1560 = (($1559) + ($1558<<4)|0);
        $1561 = +HEAPF64[$1560>>3];
        $1562 = $1556 * $1561;
        $1563 = $1544 + $1562;
        $1564 = ((($z__2)) + 8|0);
        HEAPF64[$1564>>3] = $1563;
        $1565 = +HEAPF64[3472>>3];
        $1566 = +HEAPF64[$z__2>>3];
        $1567 = $1565 - $1566;
        HEAPF64[$z__1>>3] = $1567;
        $1568 = +HEAPF64[(3480)>>3];
        $1569 = ((($z__2)) + 8|0);
        $1570 = +HEAPF64[$1569>>3];
        $1571 = $1568 - $1570;
        $1572 = ((($z__1)) + 8|0);
        HEAPF64[$1572>>3] = $1571;
        $1573 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1573;
        $1574 = ((($z__1)) + 8|0);
        $1575 = +HEAPF64[$1574>>3];
        HEAPF64[(3480)>>3] = $1575;
        $1576 = $8;
        $1577 = HEAP32[$1576>>2]|0;
        $1578 = HEAP32[3504>>2]|0;
        $1579 = (($1578) - ($1577))|0;
        HEAP32[3504>>2] = $1579;
        $1580 = HEAP32[3488>>2]|0;
        $1581 = (($1580) + -1)|0;
        HEAP32[3488>>2] = $1581;
       }
       $1582 = HEAP32[3448>>2]|0;
       $1583 = ($1582|0)!=(0);
       if ($1583) {
        $1584 = HEAP32[3464>>2]|0;
        $1585 = (($1584) - 1)|0;
        $1586 = HEAP32[3464>>2]|0;
        $1587 = (($1586) - 1)|0;
        $1588 = $6;
        $1589 = HEAP32[$1588>>2]|0;
        $1590 = Math_imul($1587, $1589)|0;
        $1591 = (($1585) + ($1590))|0;
        $1592 = $5;
        $1593 = (($1592) + ($1591<<4)|0);
        _z_div($z__1,3472,$1593);
        $1594 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1594;
        $1595 = ((($z__1)) + 8|0);
        $1596 = +HEAPF64[$1595>>3];
        HEAPF64[(3480)>>3] = $1596;
       }
      } else {
       while(1) {
        $1597 = HEAP32[3488>>2]|0;
        $1598 = HEAP32[3464>>2]|0;
        $1599 = (($1598) + 1)|0;
        $1600 = ($1597|0)>=($1599|0);
        if (!($1600)) {
         break;
        }
        $1601 = HEAP32[3488>>2]|0;
        $1602 = (($1601) - 1)|0;
        $1603 = HEAP32[3464>>2]|0;
        $1604 = (($1603) - 1)|0;
        $1605 = $6;
        $1606 = HEAP32[$1605>>2]|0;
        $1607 = Math_imul($1604, $1606)|0;
        $1608 = (($1602) + ($1607))|0;
        $1609 = $5;
        $1610 = (($1609) + ($1608<<4)|0);
        _d_cnjg($z__3,$1610);
        $1611 = HEAP32[3504>>2]|0;
        $i__2 = $1611;
        $1612 = +HEAPF64[$z__3>>3];
        $1613 = HEAP32[3504>>2]|0;
        $1614 = (($1613) - 1)|0;
        $1615 = $7;
        $1616 = (($1615) + ($1614<<4)|0);
        $1617 = +HEAPF64[$1616>>3];
        $1618 = $1612 * $1617;
        $1619 = ((($z__3)) + 8|0);
        $1620 = +HEAPF64[$1619>>3];
        $1621 = HEAP32[3504>>2]|0;
        $1622 = (($1621) - 1)|0;
        $1623 = $7;
        $1624 = (($1623) + ($1622<<4)|0);
        $1625 = ((($1624)) + 8|0);
        $1626 = +HEAPF64[$1625>>3];
        $1627 = $1620 * $1626;
        $1628 = $1618 - $1627;
        HEAPF64[$z__2>>3] = $1628;
        $1629 = +HEAPF64[$z__3>>3];
        $1630 = HEAP32[3504>>2]|0;
        $1631 = (($1630) - 1)|0;
        $1632 = $7;
        $1633 = (($1632) + ($1631<<4)|0);
        $1634 = ((($1633)) + 8|0);
        $1635 = +HEAPF64[$1634>>3];
        $1636 = $1629 * $1635;
        $1637 = ((($z__3)) + 8|0);
        $1638 = +HEAPF64[$1637>>3];
        $1639 = HEAP32[3504>>2]|0;
        $1640 = (($1639) - 1)|0;
        $1641 = $7;
        $1642 = (($1641) + ($1640<<4)|0);
        $1643 = +HEAPF64[$1642>>3];
        $1644 = $1638 * $1643;
        $1645 = $1636 + $1644;
        $1646 = ((($z__2)) + 8|0);
        HEAPF64[$1646>>3] = $1645;
        $1647 = +HEAPF64[3472>>3];
        $1648 = +HEAPF64[$z__2>>3];
        $1649 = $1647 - $1648;
        HEAPF64[$z__1>>3] = $1649;
        $1650 = +HEAPF64[(3480)>>3];
        $1651 = ((($z__2)) + 8|0);
        $1652 = +HEAPF64[$1651>>3];
        $1653 = $1650 - $1652;
        $1654 = ((($z__1)) + 8|0);
        HEAPF64[$1654>>3] = $1653;
        $1655 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1655;
        $1656 = ((($z__1)) + 8|0);
        $1657 = +HEAPF64[$1656>>3];
        HEAPF64[(3480)>>3] = $1657;
        $1658 = $8;
        $1659 = HEAP32[$1658>>2]|0;
        $1660 = HEAP32[3504>>2]|0;
        $1661 = (($1660) - ($1659))|0;
        HEAP32[3504>>2] = $1661;
        $1662 = HEAP32[3488>>2]|0;
        $1663 = (($1662) + -1)|0;
        HEAP32[3488>>2] = $1663;
       }
       $1664 = HEAP32[3448>>2]|0;
       $1665 = ($1664|0)!=(0);
       if ($1665) {
        $1666 = HEAP32[3464>>2]|0;
        $1667 = (($1666) - 1)|0;
        $1668 = HEAP32[3464>>2]|0;
        $1669 = (($1668) - 1)|0;
        $1670 = $6;
        $1671 = HEAP32[$1670>>2]|0;
        $1672 = Math_imul($1669, $1671)|0;
        $1673 = (($1667) + ($1672))|0;
        $1674 = $5;
        $1675 = (($1674) + ($1673<<4)|0);
        _d_cnjg($z__2,$1675);
        _z_div($z__1,3472,$z__2);
        $1676 = +HEAPF64[$z__1>>3];
        HEAPF64[3472>>3] = $1676;
        $1677 = ((($z__1)) + 8|0);
        $1678 = +HEAPF64[$1677>>3];
        HEAPF64[(3480)>>3] = $1678;
       }
      }
      $1679 = HEAP32[3496>>2]|0;
      $i__1 = $1679;
      $1680 = +HEAPF64[3472>>3];
      $1681 = HEAP32[3496>>2]|0;
      $1682 = (($1681) - 1)|0;
      $1683 = $7;
      $1684 = (($1683) + ($1682<<4)|0);
      HEAPF64[$1684>>3] = $1680;
      $1685 = +HEAPF64[(3480)>>3];
      $1686 = HEAP32[3496>>2]|0;
      $1687 = (($1686) - 1)|0;
      $1688 = $7;
      $1689 = (($1688) + ($1687<<4)|0);
      $1690 = ((($1689)) + 8|0);
      HEAPF64[$1690>>3] = $1685;
      $1691 = $8;
      $1692 = HEAP32[$1691>>2]|0;
      $1693 = HEAP32[3496>>2]|0;
      $1694 = (($1693) - ($1692))|0;
      HEAP32[3496>>2] = $1694;
      $1695 = HEAP32[3464>>2]|0;
      $1696 = (($1695) + -1)|0;
      HEAP32[3464>>2] = $1696;
     }
    }
   }
  }
 } while(0);
 $0 = 0;
 $1697 = $0;
 STACKTOP = sp;return ($1697|0);
}
function _frexp($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $$0 = 0.0, $$01 = 0.0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0.0, $8 = 0, $9 = 0, $storemerge = 0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = (_bitshift64Lshr(($0|0),($1|0),52)|0);
 $3 = tempRet0;
 $4 = $2 & 2047;
 if ((($4|0) == 2047)) {
  $$0 = $x;
  return (+$$0);
 } else if ((($4|0) == 0)) {
  $5 = $x != 0.0;
  if ($5) {
   $6 = $x * 1.8446744073709552E+19;
   $7 = (+_frexp($6,$e));
   $8 = HEAP32[$e>>2]|0;
   $9 = (($8) + -64)|0;
   $$01 = $7;$storemerge = $9;
  } else {
   $$01 = $x;$storemerge = 0;
  }
  HEAP32[$e>>2] = $storemerge;
  $$0 = $$01;
  return (+$$0);
 } else {
  $10 = (($4) + -1022)|0;
  HEAP32[$e>>2] = $10;
  $11 = $1 & -2146435073;
  $12 = $11 | 1071644672;
  HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $12;$13 = +HEAPF64[tempDoublePtr>>3];
  $$0 = $13;
  return (+$$0);
 }
 return +(0.0);
}
function _frexpl($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (+_frexp($x,$e));
 return (+$0);
}
function _wctomb($s,$wc) {
 $s = $s|0;
 $wc = $wc|0;
 var $$0 = 0, $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 if ($0) {
  $$0 = 0;
 } else {
  $1 = (_wcrtomb($s,$wc,0)|0);
  $$0 = $1;
 }
 return ($$0|0);
}
function _wcrtomb($s,$wc,$st) {
 $s = $s|0;
 $wc = $wc|0;
 $st = $st|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 if ($0) {
  $$0 = 1;
  return ($$0|0);
 }
 $1 = ($wc>>>0)<(128);
 if ($1) {
  $2 = $wc&255;
  HEAP8[$s>>0] = $2;
  $$0 = 1;
  return ($$0|0);
 }
 $3 = ($wc>>>0)<(2048);
 if ($3) {
  $4 = $wc >>> 6;
  $5 = $4 | 192;
  $6 = $5&255;
  $7 = ((($s)) + 1|0);
  HEAP8[$s>>0] = $6;
  $8 = $wc & 63;
  $9 = $8 | 128;
  $10 = $9&255;
  HEAP8[$7>>0] = $10;
  $$0 = 2;
  return ($$0|0);
 }
 $11 = ($wc>>>0)<(55296);
 $12 = $wc & -8192;
 $13 = ($12|0)==(57344);
 $or$cond = $11 | $13;
 if ($or$cond) {
  $14 = $wc >>> 12;
  $15 = $14 | 224;
  $16 = $15&255;
  $17 = ((($s)) + 1|0);
  HEAP8[$s>>0] = $16;
  $18 = $wc >>> 6;
  $19 = $18 & 63;
  $20 = $19 | 128;
  $21 = $20&255;
  $22 = ((($s)) + 2|0);
  HEAP8[$17>>0] = $21;
  $23 = $wc & 63;
  $24 = $23 | 128;
  $25 = $24&255;
  HEAP8[$22>>0] = $25;
  $$0 = 3;
  return ($$0|0);
 }
 $26 = (($wc) + -65536)|0;
 $27 = ($26>>>0)<(1048576);
 if ($27) {
  $28 = $wc >>> 18;
  $29 = $28 | 240;
  $30 = $29&255;
  $31 = ((($s)) + 1|0);
  HEAP8[$s>>0] = $30;
  $32 = $wc >>> 12;
  $33 = $32 & 63;
  $34 = $33 | 128;
  $35 = $34&255;
  $36 = ((($s)) + 2|0);
  HEAP8[$31>>0] = $35;
  $37 = $wc >>> 6;
  $38 = $37 & 63;
  $39 = $38 | 128;
  $40 = $39&255;
  $41 = ((($s)) + 3|0);
  HEAP8[$36>>0] = $40;
  $42 = $wc & 63;
  $43 = $42 | 128;
  $44 = $43&255;
  HEAP8[$41>>0] = $44;
  $$0 = 4;
  return ($$0|0);
 } else {
  $45 = (___errno_location()|0);
  HEAP32[$45>>2] = 84;
  $$0 = -1;
  return ($$0|0);
 }
 return (0)|0;
}
function ___towrite($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 74|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = (($2) + 255)|0;
 $4 = $3 | $2;
 $5 = $4&255;
 HEAP8[$0>>0] = $5;
 $6 = HEAP32[$f>>2]|0;
 $7 = $6 & 8;
 $8 = ($7|0)==(0);
 if ($8) {
  $10 = ((($f)) + 8|0);
  HEAP32[$10>>2] = 0;
  $11 = ((($f)) + 4|0);
  HEAP32[$11>>2] = 0;
  $12 = ((($f)) + 44|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ((($f)) + 28|0);
  HEAP32[$14>>2] = $13;
  $15 = ((($f)) + 20|0);
  HEAP32[$15>>2] = $13;
  $16 = $13;
  $17 = ((($f)) + 48|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + ($18)|0);
  $20 = ((($f)) + 16|0);
  HEAP32[$20>>2] = $19;
  $$0 = 0;
  return ($$0|0);
 } else {
  $9 = $6 | 32;
  HEAP32[$f>>2] = $9;
  $$0 = -1;
  return ($$0|0);
 }
 return (0)|0;
}
function ___fwritex($s,$l,$f) {
 $s = $s|0;
 $l = $l|0;
 $f = $f|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $$pre = 0, $$pre6 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i$0 = 0, $i$0$lcssa10 = 0;
 var $i$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 do {
  if ($2) {
   $3 = (___towrite($f)|0);
   $4 = ($3|0)==(0);
   if ($4) {
    $$pre = HEAP32[$0>>2]|0;
    $7 = $$pre;
    break;
   } else {
    $$0 = 0;
    return ($$0|0);
   }
  } else {
   $7 = $1;
  }
 } while(0);
 $5 = ((($f)) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $8 = $7;
 $9 = $6;
 $10 = (($8) - ($9))|0;
 $11 = ($10>>>0)<($l>>>0);
 if ($11) {
  $12 = ((($f)) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (FUNCTION_TABLE_iiii[$13 & 1]($f,$s,$l)|0);
  $$0 = $14;
  return ($$0|0);
 }
 $15 = ((($f)) + 75|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)>(-1);
 L11: do {
  if ($17) {
   $i$0 = $l;
   while(1) {
    $18 = ($i$0|0)==(0);
    if ($18) {
     $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
     break L11;
    }
    $19 = (($i$0) + -1)|0;
    $20 = (($s) + ($19)|0);
    $21 = HEAP8[$20>>0]|0;
    $22 = ($21<<24>>24)==(10);
    if ($22) {
     $i$0$lcssa10 = $i$0;
     break;
    } else {
     $i$0 = $19;
    }
   }
   $23 = ((($f)) + 36|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (FUNCTION_TABLE_iiii[$24 & 1]($f,$s,$i$0$lcssa10)|0);
   $26 = ($25>>>0)<($i$0$lcssa10>>>0);
   if ($26) {
    $$0 = $i$0$lcssa10;
    return ($$0|0);
   } else {
    $27 = (($s) + ($i$0$lcssa10)|0);
    $28 = (($l) - ($i$0$lcssa10))|0;
    $$pre6 = HEAP32[$5>>2]|0;
    $$01 = $28;$$02 = $27;$29 = $$pre6;$i$1 = $i$0$lcssa10;
    break;
   }
  } else {
   $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
  }
 } while(0);
 _memcpy(($29|0),($$02|0),($$01|0))|0;
 $30 = HEAP32[$5>>2]|0;
 $31 = (($30) + ($$01)|0);
 HEAP32[$5>>2] = $31;
 $32 = (($i$1) + ($$01))|0;
 $$0 = $32;
 return ($$0|0);
}
function _sprintf($s,$fmt,$varargs) {
 $s = $s|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsprintf($s,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _MUSL_vfprintf($f,$fmt,$ap) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$ = 0, $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ap2 = 0, $internal_buf = 0, $nl_arg = 0, $nl_type = 0, $ret$1 = 0, $vacopy_currentptr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ap2 = sp + 120|0;
 $nl_type = sp + 80|0;
 $nl_arg = sp;
 $internal_buf = sp + 136|0;
 dest=$nl_type; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $vacopy_currentptr = HEAP32[$ap>>2]|0;
 HEAP32[$ap2>>2] = $vacopy_currentptr;
 $0 = (_printf_core(0,$fmt,$ap2,$nl_arg,$nl_type)|0);
 $1 = ($0|0)<(0);
 if ($1) {
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 $2 = ((($f)) + 48|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $6 = ((($f)) + 44|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$6>>2] = $internal_buf;
  $8 = ((($f)) + 28|0);
  HEAP32[$8>>2] = $internal_buf;
  $9 = ((($f)) + 20|0);
  HEAP32[$9>>2] = $internal_buf;
  HEAP32[$2>>2] = 80;
  $10 = ((($internal_buf)) + 80|0);
  $11 = ((($f)) + 16|0);
  HEAP32[$11>>2] = $10;
  $12 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
  $13 = ($7|0)==(0|0);
  if ($13) {
   $ret$1 = $12;
  } else {
   $14 = ((($f)) + 36|0);
   $15 = HEAP32[$14>>2]|0;
   (FUNCTION_TABLE_iiii[$15 & 1]($f,0,0)|0);
   $16 = HEAP32[$9>>2]|0;
   $17 = ($16|0)==(0|0);
   $$ = $17 ? -1 : $12;
   HEAP32[$6>>2] = $7;
   HEAP32[$2>>2] = 0;
   HEAP32[$11>>2] = 0;
   HEAP32[$8>>2] = 0;
   HEAP32[$9>>2] = 0;
   $ret$1 = $$;
  }
 } else {
  $5 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
  $ret$1 = $5;
 }
 $$0 = $ret$1;
 STACKTOP = sp;return ($$0|0);
}
function _vsnprintf($s,$n,$fmt,$ap) {
 $s = $s|0;
 $n = $n|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$$02 = 0, $$0 = 0, $$01 = 0, $$02 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $f = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $b = sp + 112|0;
 $f = sp;
 dest=$f; src=3512; stop=dest+112|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $0 = (($n) + -1)|0;
 $1 = ($0>>>0)>(2147483646);
 if ($1) {
  $2 = ($n|0)==(0);
  if ($2) {
   $$01 = $b;$$02 = 1;
  } else {
   $3 = (___errno_location()|0);
   HEAP32[$3>>2] = 75;
   $$0 = -1;
   STACKTOP = sp;return ($$0|0);
  }
 } else {
  $$01 = $s;$$02 = $n;
 }
 $4 = $$01;
 $5 = (-2 - ($4))|0;
 $6 = ($$02>>>0)>($5>>>0);
 $$$02 = $6 ? $5 : $$02;
 $7 = ((($f)) + 48|0);
 HEAP32[$7>>2] = $$$02;
 $8 = ((($f)) + 20|0);
 HEAP32[$8>>2] = $$01;
 $9 = ((($f)) + 44|0);
 HEAP32[$9>>2] = $$01;
 $10 = (($$01) + ($$$02)|0);
 $11 = ((($f)) + 16|0);
 HEAP32[$11>>2] = $10;
 $12 = ((($f)) + 28|0);
 HEAP32[$12>>2] = $10;
 $13 = (_MUSL_vfprintf($f,$fmt,$ap)|0);
 $14 = ($$$02|0)==(0);
 if ($14) {
  $$0 = $13;
  STACKTOP = sp;return ($$0|0);
 }
 $15 = HEAP32[$8>>2]|0;
 $16 = HEAP32[$11>>2]|0;
 $17 = ($15|0)==($16|0);
 $18 = $17 << 31 >> 31;
 $19 = (($15) + ($18)|0);
 HEAP8[$19>>0] = 0;
 $$0 = $13;
 STACKTOP = sp;return ($$0|0);
}
function _vsprintf($s,$fmt,$ap) {
 $s = $s|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_vsnprintf($s,2147483647,$fmt,$ap)|0);
 return ($0|0);
}
function _memchr($src,$c,$n) {
 $src = $src|0;
 $c = $c|0;
 $n = $n|0;
 var $$0$lcssa = 0, $$0$lcssa44 = 0, $$019 = 0, $$1$lcssa = 0, $$110 = 0, $$110$lcssa = 0, $$24 = 0, $$3 = 0, $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond18 = 0, $s$0$lcssa = 0, $s$0$lcssa43 = 0, $s$020 = 0, $s$15 = 0, $s$2 = 0, $w$0$lcssa = 0, $w$011 = 0, $w$011$lcssa = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $c & 255;
 $1 = $src;
 $2 = $1 & 3;
 $3 = ($2|0)!=(0);
 $4 = ($n|0)!=(0);
 $or$cond18 = $4 & $3;
 L1: do {
  if ($or$cond18) {
   $5 = $c&255;
   $$019 = $n;$s$020 = $src;
   while(1) {
    $6 = HEAP8[$s$020>>0]|0;
    $7 = ($6<<24>>24)==($5<<24>>24);
    if ($7) {
     $$0$lcssa44 = $$019;$s$0$lcssa43 = $s$020;
     label = 6;
     break L1;
    }
    $8 = ((($s$020)) + 1|0);
    $9 = (($$019) + -1)|0;
    $10 = $8;
    $11 = $10 & 3;
    $12 = ($11|0)!=(0);
    $13 = ($9|0)!=(0);
    $or$cond = $13 & $12;
    if ($or$cond) {
     $$019 = $9;$s$020 = $8;
    } else {
     $$0$lcssa = $9;$$lcssa = $13;$s$0$lcssa = $8;
     label = 5;
     break;
    }
   }
  } else {
   $$0$lcssa = $n;$$lcssa = $4;$s$0$lcssa = $src;
   label = 5;
  }
 } while(0);
 if ((label|0) == 5) {
  if ($$lcssa) {
   $$0$lcssa44 = $$0$lcssa;$s$0$lcssa43 = $s$0$lcssa;
   label = 6;
  } else {
   $$3 = 0;$s$2 = $s$0$lcssa;
  }
 }
 L8: do {
  if ((label|0) == 6) {
   $14 = HEAP8[$s$0$lcssa43>>0]|0;
   $15 = $c&255;
   $16 = ($14<<24>>24)==($15<<24>>24);
   if ($16) {
    $$3 = $$0$lcssa44;$s$2 = $s$0$lcssa43;
   } else {
    $17 = Math_imul($0, 16843009)|0;
    $18 = ($$0$lcssa44>>>0)>(3);
    L11: do {
     if ($18) {
      $$110 = $$0$lcssa44;$w$011 = $s$0$lcssa43;
      while(1) {
       $19 = HEAP32[$w$011>>2]|0;
       $20 = $19 ^ $17;
       $21 = (($20) + -16843009)|0;
       $22 = $20 & -2139062144;
       $23 = $22 ^ -2139062144;
       $24 = $23 & $21;
       $25 = ($24|0)==(0);
       if (!($25)) {
        $$110$lcssa = $$110;$w$011$lcssa = $w$011;
        break;
       }
       $26 = ((($w$011)) + 4|0);
       $27 = (($$110) + -4)|0;
       $28 = ($27>>>0)>(3);
       if ($28) {
        $$110 = $27;$w$011 = $26;
       } else {
        $$1$lcssa = $27;$w$0$lcssa = $26;
        label = 11;
        break L11;
       }
      }
      $$24 = $$110$lcssa;$s$15 = $w$011$lcssa;
     } else {
      $$1$lcssa = $$0$lcssa44;$w$0$lcssa = $s$0$lcssa43;
      label = 11;
     }
    } while(0);
    if ((label|0) == 11) {
     $29 = ($$1$lcssa|0)==(0);
     if ($29) {
      $$3 = 0;$s$2 = $w$0$lcssa;
      break;
     } else {
      $$24 = $$1$lcssa;$s$15 = $w$0$lcssa;
     }
    }
    while(1) {
     $30 = HEAP8[$s$15>>0]|0;
     $31 = ($30<<24>>24)==($15<<24>>24);
     if ($31) {
      $$3 = $$24;$s$2 = $s$15;
      break L8;
     }
     $32 = ((($s$15)) + 1|0);
     $33 = (($$24) + -1)|0;
     $34 = ($33|0)==(0);
     if ($34) {
      $$3 = 0;$s$2 = $32;
      break;
     } else {
      $$24 = $33;$s$15 = $32;
     }
    }
   }
  }
 } while(0);
 $35 = ($$3|0)!=(0);
 $36 = $35 ? $s$2 : 0;
 return ($36|0);
}
function _printf_core($f,$fmt,$ap,$nl_arg,$nl_type) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 $nl_arg = $nl_arg|0;
 $nl_type = $nl_type|0;
 var $$ = 0, $$$i = 0, $$0 = 0, $$0$i = 0, $$0$lcssa$i = 0, $$0$lcssa$i$i = 0, $$0$lcssa$i104$i = 0, $$0$lcssa$i128$i = 0, $$0$lcssa$i143$i = 0, $$0$lcssa$i39$i = 0, $$0$lcssa$i46 = 0, $$0$lcssa$i46$i = 0, $$0$lcssa$i48$i = 0, $$0$lcssa$i51 = 0, $$0$lcssa$i53 = 0, $$0$lcssa$i56$i = 0, $$0$lcssa$i61 = 0, $$0$lcssa$i63$i = 0, $$0$lcssa$i68 = 0, $$0$lcssa$i69$i = 0;
 var $$0$lcssa$i75 = 0, $$0$lcssa$i76$i = 0, $$0$lcssa$i84$i = 0, $$0$lcssa$i85 = 0, $$0$lcssa$i97$i = 0, $$01$i = 0, $$01$i$i = 0, $$01$i102$i = 0, $$01$i126$i = 0, $$01$i141$i = 0, $$01$i37$i = 0, $$01$i44 = 0, $$01$i44$i = 0, $$01$i54$i = 0, $$01$i59 = 0, $$01$i61$i = 0, $$01$i66 = 0, $$01$i67$i = 0, $$01$i73 = 0, $$01$i74$i = 0;
 var $$01$i83 = 0, $$01$i95$i = 0, $$01$lcssa$off0$i = 0, $$01$lcssa$off0$i$i = 0, $$01$lcssa$off0$i85$i = 0, $$012$i = 0, $$013$i = 0, $$03$i48 = 0, $$05$i = 0, $$05$i$i = 0, $$05$i79$i = 0, $$07$i = 0.0, $$1$i = 0.0, $$1$lcssa$i$i = 0, $$1$lcssa$i112$i = 0, $$114$i = 0, $$12$i = 0, $$12$i$i = 0, $$12$i110$i = 0, $$12$i119$i = 0;
 var $$12$i134$i = 0, $$12$i134$i$lcssa = 0, $$12$i87$i = 0, $$2$i = 0.0, $$2$us$i = 0.0, $$2$us$us$i = 0.0, $$20$i = 0.0, $$21$i = 0, $$210$$22$i = 0, $$210$$24$i = 0, $$210$i = 0, $$23$i = 0, $$24 = 0, $$25 = 0, $$3$i = 0.0, $$31$i = 0, $$311$i = 0, $$4$i = 0.0, $$412$lcssa$i = 0, $$412184$i = 0;
 var $$5196$i = 0, $$92 = 0, $$a$3$i = 0, $$a$3$us$i = 0, $$a$3$us303$i = 0, $$a$3$us304$i = 0, $$a$3305$i = 0, $$a$3306$i = 0, $$fl$4 = 0, $$l10n$0 = 0, $$lcssa106 = 0, $$lcssa275$i = 0, $$lcssa450 = 0, $$lcssa451 = 0, $$lcssa455 = 0, $$lcssa457 = 0, $$lcssa458 = 0, $$lcssa459 = 0, $$lcssa460 = 0, $$lcssa461 = 0;
 var $$lcssa463 = 0, $$lcssa464 = 0, $$lcssa470 = 0, $$lcssa474 = 0, $$lcssa476 = 0, $$lcssa479 = 0, $$lcssa480 = 0, $$lcssa483 = 0.0, $$lcssa484 = 0, $$lcssa487 = 0, $$lcssa491 = 0, $$mask$i = 0, $$mask$i38 = 0, $$mask1$i = 0, $$mask1$i37 = 0, $$neg151$i = 0, $$neg152$i = 0, $$p$$i = 0, $$p$5 = 0, $$p$i = 0;
 var $$pn$i = 0, $$pr$i = 0, $$pr146$i = 0, $$pre = 0, $$pre$i = 0, $$pre$phi302$iZ2D = 0, $$pre270 = 0, $$pre300$i = 0, $$pre301$i = 0, $$sum$i = 0, $$sum15$i = 0, $$sum16$i = 0, $$z$3$i = 0, $$z$4$i = 0, $$z$4$us$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0;
 var $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0;
 var $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0;
 var $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0;
 var $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0;
 var $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0, $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0;
 var $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0, $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0;
 var $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0, $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0;
 var $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0, $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0;
 var $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0.0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0, $116 = 0, $1160 = 0, $1161 = 0.0, $1162 = 0, $1163 = 0;
 var $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $1169$phi = 0, $117 = 0, $1170 = 0, $1170$phi = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0, $1178 = 0, $1179 = 0, $118 = 0;
 var $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0;
 var $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0;
 var $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0;
 var $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0;
 var $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0;
 var $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0;
 var $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0;
 var $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0.0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0.0, $253 = 0, $254 = 0, $255 = 0;
 var $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0;
 var $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0;
 var $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0;
 var $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0;
 var $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0;
 var $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0;
 var $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0;
 var $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0;
 var $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0;
 var $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0.0, $431 = 0, $432 = 0.0, $433 = 0, $434 = 0, $435 = 0;
 var $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0;
 var $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0.0;
 var $472 = 0.0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0.0, $488 = 0, $489 = 0, $49 = 0;
 var $490 = 0, $491 = 0.0, $492 = 0.0, $493 = 0.0, $494 = 0.0, $495 = 0.0, $496 = 0.0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0;
 var $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0;
 var $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0;
 var $544 = 0.0, $545 = 0.0, $546 = 0.0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0.0, $56 = 0, $560 = 0.0, $561 = 0.0;
 var $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0.0, $575 = 0.0, $576 = 0.0, $577 = 0, $578 = 0, $579 = 0, $58 = 0;
 var $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0;
 var $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0;
 var $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0.0, $632 = 0, $633 = 0;
 var $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0.0, $64 = 0, $640 = 0.0, $641 = 0.0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0;
 var $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0;
 var $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0;
 var $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0;
 var $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0;
 var $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0;
 var $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0;
 var $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0;
 var $779 = 0, $78 = 0, $780 = 0.0, $781 = 0.0, $782 = 0, $783 = 0.0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0;
 var $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0;
 var $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0;
 var $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0;
 var $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0;
 var $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0;
 var $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0;
 var $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0;
 var $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0;
 var $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0;
 var $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0;
 var $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0;
 var $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $a$0 = 0, $a$1 = 0, $a$1$lcssa$i = 0, $a$1253$i = 0, $a$2 = 0, $a$2$ph$i = 0, $a$3$lcssa$i = 0, $a$3240$i = 0, $a$3240$us$i = 0, $a$5$lcssa$i = 0, $a$5215$i = 0, $a$6$i = 0, $a$7$i = 0, $a$8$ph$i = 0, $arglist_current = 0;
 var $arglist_current11 = 0, $arglist_current14 = 0, $arglist_current17 = 0, $arglist_current2 = 0, $arglist_current20 = 0, $arglist_current23 = 0, $arglist_current26 = 0, $arglist_current29 = 0, $arglist_current32 = 0, $arglist_current35 = 0, $arglist_current38 = 0, $arglist_current41 = 0, $arglist_current44 = 0, $arglist_current47 = 0, $arglist_current5 = 0, $arglist_current50 = 0, $arglist_current53 = 0, $arglist_current56 = 0, $arglist_current59 = 0, $arglist_current62 = 0;
 var $arglist_current8 = 0, $arglist_next = 0, $arglist_next12 = 0, $arglist_next15 = 0, $arglist_next18 = 0, $arglist_next21 = 0, $arglist_next24 = 0, $arglist_next27 = 0, $arglist_next3 = 0, $arglist_next30 = 0, $arglist_next33 = 0, $arglist_next36 = 0, $arglist_next39 = 0, $arglist_next42 = 0, $arglist_next45 = 0, $arglist_next48 = 0, $arglist_next51 = 0, $arglist_next54 = 0, $arglist_next57 = 0, $arglist_next6 = 0;
 var $arglist_next60 = 0, $arglist_next63 = 0, $arglist_next9 = 0, $argpos$0 = 0, $big$i = 0, $buf = 0, $buf$i = 0, $carry$0246$i = 0, $carry3$0234$i = 0, $carry3$0234$us$i = 0, $cnt$0 = 0, $cnt$1 = 0, $cnt$1$lcssa = 0, $d$0$i = 0, $d$0245$i = 0, $d$0247$i = 0, $d$1233$i = 0, $d$1233$us$i = 0, $d$2$lcssa$i = 0, $d$2214$i = 0;
 var $d$3$i = 0, $d$4191$i = 0, $d$5183$i = 0, $d$6195$i = 0, $e$0229$i = 0, $e$1$i = 0, $e$2210$i = 0, $e$3$i = 0, $e$4$ph$i = 0, $e2$i = 0, $ebuf0$i = 0, $estr$0$i = 0, $estr$1$lcssa$i = 0, $estr$1$ph$i = 0, $estr$1201$i = 0, $estr$2$i = 0, $exitcond$i = 0, $expanded = 0, $expanded101 = 0, $expanded102 = 0;
 var $expanded103 = 0, $expanded105 = 0, $expanded106 = 0, $expanded108 = 0, $expanded109 = 0, $expanded110 = 0, $expanded112 = 0, $expanded113 = 0, $expanded115 = 0, $expanded116 = 0, $expanded117 = 0, $expanded119 = 0, $expanded120 = 0, $expanded122 = 0, $expanded123 = 0, $expanded124 = 0, $expanded126 = 0, $expanded127 = 0, $expanded129 = 0, $expanded130 = 0;
 var $expanded131 = 0, $expanded133 = 0, $expanded134 = 0, $expanded136 = 0, $expanded137 = 0, $expanded138 = 0, $expanded140 = 0, $expanded141 = 0, $expanded143 = 0, $expanded144 = 0, $expanded145 = 0, $expanded147 = 0, $expanded148 = 0, $expanded150 = 0, $expanded151 = 0, $expanded152 = 0, $expanded154 = 0, $expanded155 = 0, $expanded157 = 0, $expanded158 = 0;
 var $expanded159 = 0, $expanded161 = 0, $expanded162 = 0, $expanded164 = 0, $expanded165 = 0, $expanded166 = 0, $expanded168 = 0, $expanded169 = 0, $expanded171 = 0, $expanded172 = 0, $expanded173 = 0, $expanded175 = 0, $expanded176 = 0, $expanded178 = 0, $expanded179 = 0, $expanded180 = 0, $expanded182 = 0, $expanded183 = 0, $expanded185 = 0, $expanded186 = 0;
 var $expanded187 = 0, $expanded189 = 0, $expanded190 = 0, $expanded192 = 0, $expanded193 = 0, $expanded194 = 0, $expanded196 = 0, $expanded197 = 0, $expanded199 = 0, $expanded200 = 0, $expanded201 = 0, $expanded203 = 0, $expanded204 = 0, $expanded206 = 0, $expanded207 = 0, $expanded208 = 0, $expanded210 = 0, $expanded211 = 0, $expanded213 = 0, $expanded214 = 0;
 var $expanded215 = 0, $expanded64 = 0, $expanded66 = 0, $expanded67 = 0, $expanded68 = 0, $expanded70 = 0, $expanded71 = 0, $expanded73 = 0, $expanded74 = 0, $expanded75 = 0, $expanded77 = 0, $expanded78 = 0, $expanded80 = 0, $expanded81 = 0, $expanded82 = 0, $expanded84 = 0, $expanded85 = 0, $expanded87 = 0, $expanded88 = 0, $expanded89 = 0;
 var $expanded91 = 0, $expanded92 = 0, $expanded94 = 0, $expanded95 = 0, $expanded96 = 0, $expanded98 = 0, $expanded99 = 0, $fl$0110 = 0, $fl$0170 = 0, $fl$1 = 0, $fl$1$ = 0, $fl$3 = 0, $fl$4 = 0, $fl$6 = 0, $i$0$lcssa = 0, $i$0$lcssa273 = 0, $i$0175 = 0, $i$0228$i = 0, $i$03$i = 0, $i$03$i30 = 0;
 var $i$1$lcssa$i = 0, $i$1186 = 0, $i$1222$i = 0, $i$2162 = 0, $i$2162$lcssa = 0, $i$2209$i = 0, $i$3160 = 0, $i$3205$i = 0, $isdigit = 0, $isdigit$i = 0, $isdigit$i32 = 0, $isdigit10 = 0, $isdigit12 = 0, $isdigit2$i = 0, $isdigit2$i28 = 0, $isdigittmp = 0, $isdigittmp$ = 0, $isdigittmp$i = 0, $isdigittmp$i31 = 0, $isdigittmp1$i = 0;
 var $isdigittmp1$i27 = 0, $isdigittmp11 = 0, $isdigittmp4$i = 0, $isdigittmp4$i29 = 0, $isdigittmp9 = 0, $j$0$i = 0, $j$0221$i = 0, $j$0223$i = 0, $j$1206$i = 0, $j$2$i = 0, $l$0 = 0, $l$0$i = 0, $l$1$i = 0, $l$1174 = 0, $l$2 = 0, $l10n$0 = 0, $l10n$0$lcssa = 0, $l10n$0$phi = 0, $l10n$1 = 0, $l10n$2 = 0;
 var $l10n$3 = 0, $mb = 0, $notlhs$us$us$i = 0, $notrhs$i = 0, $or$cond = 0, $or$cond$i = 0, $or$cond$i$i = 0, $or$cond$i100$i = 0, $or$cond$i35$i = 0, $or$cond$i42$i = 0, $or$cond$i52$i = 0, $or$cond$i57 = 0, $or$cond$i59$i = 0, $or$cond$i64 = 0, $or$cond$i71 = 0, $or$cond$i72$i = 0, $or$cond$i79 = 0, $or$cond$i81 = 0, $or$cond$i93$i = 0, $or$cond15 = 0;
 var $or$cond19 = 0, $or$cond22 = 0, $or$cond29$i = 0, $or$cond332 = 0, $or$cond6$i = 0, $p$0 = 0, $p$1 = 0, $p$2 = 0, $p$2$ = 0, $p$3 = 0, $p$4272 = 0, $p$5 = 0, $pad$i = 0, $pl$0 = 0, $pl$0$i = 0, $pl$1 = 0, $pl$1$i = 0, $pl$2 = 0, $prefix$0 = 0, $prefix$0$$i = 0;
 var $prefix$0$i = 0, $prefix$1 = 0, $prefix$2 = 0, $r$0$a$8$i = 0, $re$1179$i = 0, $round$0178$i = 0.0, $round6$1$i = 0.0, $s$0$i = 0, $s$0$us$i = 0, $s$0$us$us$i = 0, $s$1$i = 0, $s$1$lcssa$i = 0, $s$1$us$i = 0, $s$1$us$us$i = 0, $s1$0$i = 0, $s7$0188$i = 0, $s7$1$i = 0, $s8$0$lcssa$i = 0, $s8$0180$i = 0, $s9$0$i = 0;
 var $s9$1192$i = 0, $s9$2$i = 0, $sext = 0, $sext93 = 0, $small$0$i = 0.0, $small$1$i = 0.0, $st$0 = 0, $st$0$lcssa456 = 0, $storemerge = 0, $storemerge13 = 0, $storemerge8108 = 0, $storemerge8169 = 0, $t$0 = 0, $t$1 = 0, $w$$i = 0, $w$0 = 0, $w$1 = 0, $w$2 = 0, $w$30$i = 0, $wc = 0;
 var $ws$0176 = 0, $ws$1187 = 0, $y$03$i = 0, $y$03$i$i = 0, $y$03$i109$i = 0, $y$03$i118$i = 0, $y$03$i133$i = 0, $y$03$i86$i = 0, $z$0$i = 0, $z$0$lcssa = 0, $z$0163 = 0, $z$1 = 0, $z$1$lcssa$i = 0, $z$1252$i = 0, $z$2 = 0, $z$2$i = 0, $z$2$i$lcssa = 0, $z$3$lcssa$i = 0, $z$3239$i = 0, $z$3239$us$i = 0;
 var $z$4$i = 0, $z$4$us$i = 0, $z$6$$i = 0, $z$6$i = 0, $z$6$i$lcssa = 0, $z$6$ph$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 864|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $big$i = sp + 16|0;
 $e2$i = sp + 8|0;
 $buf$i = sp + 836|0;
 $0 = $buf$i;
 $ebuf0$i = sp + 824|0;
 $pad$i = sp + 568|0;
 $buf = sp + 528|0;
 $wc = sp;
 $mb = sp + 520|0;
 $1 = ($f|0)!=(0|0);
 $2 = ((($buf)) + 40|0);
 $3 = $2;
 $4 = ((($buf)) + 39|0);
 $5 = ((($wc)) + 4|0);
 $6 = $wc;
 $7 = ((($ebuf0$i)) + 12|0);
 $8 = ((($ebuf0$i)) + 11|0);
 $9 = $7;
 $10 = (($9) - ($0))|0;
 $11 = (-2 - ($0))|0;
 $12 = (($9) + 2)|0;
 $13 = ((($big$i)) + 288|0);
 $14 = ((($buf$i)) + 9|0);
 $15 = $14;
 $16 = ((($buf$i)) + 8|0);
 $1169 = 0;$1170 = 0;$23 = $fmt;$cnt$0 = 0;$l$0 = 0;$l10n$0 = 0;
 L1: while(1) {
  $17 = ($cnt$0|0)>(-1);
  do {
   if ($17) {
    $18 = (2147483647 - ($cnt$0))|0;
    $19 = ($l$0|0)>($18|0);
    if ($19) {
     $20 = (___errno_location()|0);
     HEAP32[$20>>2] = 75;
     $cnt$1 = -1;
     break;
    } else {
     $21 = (($l$0) + ($cnt$0))|0;
     $cnt$1 = $21;
     break;
    }
   } else {
    $cnt$1 = $cnt$0;
   }
  } while(0);
  $22 = HEAP8[$23>>0]|0;
  $24 = ($22<<24>>24)==(0);
  if ($24) {
   $cnt$1$lcssa = $cnt$1;$l10n$0$lcssa = $l10n$0;
   label = 344;
   break;
  } else {
   $1171 = $22;$26 = $23;
  }
  while(1) {
   if ((($1171<<24>>24) == 0)) {
    $$lcssa106 = $26;$z$0$lcssa = $26;
    break;
   } else if ((($1171<<24>>24) == 37)) {
    $28 = $26;$z$0163 = $26;
    label = 9;
    break;
   }
   $25 = ((($26)) + 1|0);
   $$pre = HEAP8[$25>>0]|0;
   $1171 = $$pre;$26 = $25;
  }
  L12: do {
   if ((label|0) == 9) {
    while(1) {
     label = 0;
     $27 = ((($28)) + 1|0);
     $29 = HEAP8[$27>>0]|0;
     $30 = ($29<<24>>24)==(37);
     if (!($30)) {
      $$lcssa106 = $28;$z$0$lcssa = $z$0163;
      break L12;
     }
     $31 = ((($z$0163)) + 1|0);
     $32 = ((($28)) + 2|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = ($33<<24>>24)==(37);
     if ($34) {
      $28 = $32;$z$0163 = $31;
      label = 9;
     } else {
      $$lcssa106 = $32;$z$0$lcssa = $31;
      break;
     }
    }
   }
  } while(0);
  $35 = $z$0$lcssa;
  $36 = $23;
  $37 = (($35) - ($36))|0;
  if ($1) {
   (___fwritex($23,$37,$f)|0);
  }
  $38 = ($z$0$lcssa|0)==($23|0);
  if (!($38)) {
   $l10n$0$phi = $l10n$0;$1170$phi = $1170;$1169$phi = $1169;$23 = $$lcssa106;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$0$phi;$1170 = $1170$phi;$1169 = $1169$phi;
   continue;
  }
  $39 = ((($$lcssa106)) + 1|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40 << 24 >> 24;
  $isdigittmp = (($41) + -48)|0;
  $isdigit = ($isdigittmp>>>0)<(10);
  if ($isdigit) {
   $42 = ((($$lcssa106)) + 2|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = ($43<<24>>24)==(36);
   $45 = ((($$lcssa106)) + 3|0);
   $$92 = $44 ? $45 : $39;
   $$l10n$0 = $44 ? 1 : $l10n$0;
   $isdigittmp$ = $44 ? $isdigittmp : -1;
   $$pre270 = HEAP8[$$92>>0]|0;
   $47 = $$pre270;$argpos$0 = $isdigittmp$;$l10n$1 = $$l10n$0;$storemerge = $$92;
  } else {
   $47 = $40;$argpos$0 = -1;$l10n$1 = $l10n$0;$storemerge = $39;
  }
  $46 = $47 << 24 >> 24;
  $48 = $46 & -32;
  $49 = ($48|0)==(32);
  L24: do {
   if ($49) {
    $51 = $46;$56 = $47;$fl$0170 = 0;$storemerge8169 = $storemerge;
    while(1) {
     $50 = (($51) + -32)|0;
     $52 = 1 << $50;
     $53 = $52 & 75913;
     $54 = ($53|0)==(0);
     if ($54) {
      $65 = $56;$fl$0110 = $fl$0170;$storemerge8108 = $storemerge8169;
      break L24;
     }
     $55 = $56 << 24 >> 24;
     $57 = (($55) + -32)|0;
     $58 = 1 << $57;
     $59 = $58 | $fl$0170;
     $60 = ((($storemerge8169)) + 1|0);
     $61 = HEAP8[$60>>0]|0;
     $62 = $61 << 24 >> 24;
     $63 = $62 & -32;
     $64 = ($63|0)==(32);
     if ($64) {
      $51 = $62;$56 = $61;$fl$0170 = $59;$storemerge8169 = $60;
     } else {
      $65 = $61;$fl$0110 = $59;$storemerge8108 = $60;
      break;
     }
    }
   } else {
    $65 = $47;$fl$0110 = 0;$storemerge8108 = $storemerge;
   }
  } while(0);
  $66 = ($65<<24>>24)==(42);
  do {
   if ($66) {
    $67 = ((($storemerge8108)) + 1|0);
    $68 = HEAP8[$67>>0]|0;
    $69 = $68 << 24 >> 24;
    $isdigittmp11 = (($69) + -48)|0;
    $isdigit12 = ($isdigittmp11>>>0)<(10);
    if ($isdigit12) {
     $70 = ((($storemerge8108)) + 2|0);
     $71 = HEAP8[$70>>0]|0;
     $72 = ($71<<24>>24)==(36);
     if ($72) {
      $73 = (($nl_type) + ($isdigittmp11<<2)|0);
      HEAP32[$73>>2] = 10;
      $74 = HEAP8[$67>>0]|0;
      $75 = $74 << 24 >> 24;
      $76 = (($75) + -48)|0;
      $77 = (($nl_arg) + ($76<<3)|0);
      $78 = $77;
      $79 = $78;
      $80 = HEAP32[$79>>2]|0;
      $81 = (($78) + 4)|0;
      $82 = $81;
      $83 = HEAP32[$82>>2]|0;
      $84 = ((($storemerge8108)) + 3|0);
      $l10n$2 = 1;$storemerge13 = $84;$w$0 = $80;
     } else {
      label = 23;
     }
    } else {
     label = 23;
    }
    if ((label|0) == 23) {
     label = 0;
     $85 = ($l10n$1|0)==(0);
     if (!($85)) {
      $$0 = -1;
      label = 363;
      break L1;
     }
     if (!($1)) {
      $105 = $67;$fl$1 = $fl$0110;$l10n$3 = 0;$w$1 = 0;
      break;
     }
     $arglist_current = HEAP32[$ap>>2]|0;
     $86 = $arglist_current;
     $87 = ((0) + 4|0);
     $expanded64 = $87;
     $expanded = (($expanded64) - 1)|0;
     $88 = (($86) + ($expanded))|0;
     $89 = ((0) + 4|0);
     $expanded68 = $89;
     $expanded67 = (($expanded68) - 1)|0;
     $expanded66 = $expanded67 ^ -1;
     $90 = $88 & $expanded66;
     $91 = $90;
     $92 = HEAP32[$91>>2]|0;
     $arglist_next = ((($91)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next;
     $l10n$2 = 0;$storemerge13 = $67;$w$0 = $92;
    }
    $93 = ($w$0|0)<(0);
    if ($93) {
     $94 = $fl$0110 | 8192;
     $95 = (0 - ($w$0))|0;
     $105 = $storemerge13;$fl$1 = $94;$l10n$3 = $l10n$2;$w$1 = $95;
    } else {
     $105 = $storemerge13;$fl$1 = $fl$0110;$l10n$3 = $l10n$2;$w$1 = $w$0;
    }
   } else {
    $96 = $65 << 24 >> 24;
    $isdigittmp1$i = (($96) + -48)|0;
    $isdigit2$i = ($isdigittmp1$i>>>0)<(10);
    if ($isdigit2$i) {
     $100 = $storemerge8108;$i$03$i = 0;$isdigittmp4$i = $isdigittmp1$i;
     while(1) {
      $97 = ($i$03$i*10)|0;
      $98 = (($97) + ($isdigittmp4$i))|0;
      $99 = ((($100)) + 1|0);
      $101 = HEAP8[$99>>0]|0;
      $102 = $101 << 24 >> 24;
      $isdigittmp$i = (($102) + -48)|0;
      $isdigit$i = ($isdigittmp$i>>>0)<(10);
      if ($isdigit$i) {
       $100 = $99;$i$03$i = $98;$isdigittmp4$i = $isdigittmp$i;
      } else {
       $$lcssa450 = $98;$$lcssa451 = $99;
       break;
      }
     }
     $103 = ($$lcssa450|0)<(0);
     if ($103) {
      $$0 = -1;
      label = 363;
      break L1;
     } else {
      $105 = $$lcssa451;$fl$1 = $fl$0110;$l10n$3 = $l10n$1;$w$1 = $$lcssa450;
     }
    } else {
     $105 = $storemerge8108;$fl$1 = $fl$0110;$l10n$3 = $l10n$1;$w$1 = 0;
    }
   }
  } while(0);
  $104 = HEAP8[$105>>0]|0;
  $106 = ($104<<24>>24)==(46);
  L45: do {
   if ($106) {
    $107 = ((($105)) + 1|0);
    $108 = HEAP8[$107>>0]|0;
    $109 = ($108<<24>>24)==(42);
    if (!($109)) {
     $136 = $108 << 24 >> 24;
     $isdigittmp1$i27 = (($136) + -48)|0;
     $isdigit2$i28 = ($isdigittmp1$i27>>>0)<(10);
     if ($isdigit2$i28) {
      $140 = $107;$i$03$i30 = 0;$isdigittmp4$i29 = $isdigittmp1$i27;
     } else {
      $1172 = $107;$p$0 = 0;
      break;
     }
     while(1) {
      $137 = ($i$03$i30*10)|0;
      $138 = (($137) + ($isdigittmp4$i29))|0;
      $139 = ((($140)) + 1|0);
      $141 = HEAP8[$139>>0]|0;
      $142 = $141 << 24 >> 24;
      $isdigittmp$i31 = (($142) + -48)|0;
      $isdigit$i32 = ($isdigittmp$i31>>>0)<(10);
      if ($isdigit$i32) {
       $140 = $139;$i$03$i30 = $138;$isdigittmp4$i29 = $isdigittmp$i31;
      } else {
       $1172 = $139;$p$0 = $138;
       break L45;
      }
     }
    }
    $110 = ((($105)) + 2|0);
    $111 = HEAP8[$110>>0]|0;
    $112 = $111 << 24 >> 24;
    $isdigittmp9 = (($112) + -48)|0;
    $isdigit10 = ($isdigittmp9>>>0)<(10);
    if ($isdigit10) {
     $113 = ((($105)) + 3|0);
     $114 = HEAP8[$113>>0]|0;
     $115 = ($114<<24>>24)==(36);
     if ($115) {
      $116 = (($nl_type) + ($isdigittmp9<<2)|0);
      HEAP32[$116>>2] = 10;
      $117 = HEAP8[$110>>0]|0;
      $118 = $117 << 24 >> 24;
      $119 = (($118) + -48)|0;
      $120 = (($nl_arg) + ($119<<3)|0);
      $121 = $120;
      $122 = $121;
      $123 = HEAP32[$122>>2]|0;
      $124 = (($121) + 4)|0;
      $125 = $124;
      $126 = HEAP32[$125>>2]|0;
      $127 = ((($105)) + 4|0);
      $1172 = $127;$p$0 = $123;
      break;
     }
    }
    $128 = ($l10n$3|0)==(0);
    if (!($128)) {
     $$0 = -1;
     label = 363;
     break L1;
    }
    if ($1) {
     $arglist_current2 = HEAP32[$ap>>2]|0;
     $129 = $arglist_current2;
     $130 = ((0) + 4|0);
     $expanded71 = $130;
     $expanded70 = (($expanded71) - 1)|0;
     $131 = (($129) + ($expanded70))|0;
     $132 = ((0) + 4|0);
     $expanded75 = $132;
     $expanded74 = (($expanded75) - 1)|0;
     $expanded73 = $expanded74 ^ -1;
     $133 = $131 & $expanded73;
     $134 = $133;
     $135 = HEAP32[$134>>2]|0;
     $arglist_next3 = ((($134)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next3;
     $1172 = $110;$p$0 = $135;
    } else {
     $1172 = $110;$p$0 = 0;
    }
   } else {
    $1172 = $105;$p$0 = -1;
   }
  } while(0);
  $144 = $1172;$st$0 = 0;
  while(1) {
   $143 = HEAP8[$144>>0]|0;
   $145 = $143 << 24 >> 24;
   $146 = (($145) + -65)|0;
   $147 = ($146>>>0)>(57);
   if ($147) {
    $$0 = -1;
    label = 363;
    break L1;
   }
   $148 = ((($144)) + 1|0);
   $149 = ((3624 + (($st$0*58)|0)|0) + ($146)|0);
   $150 = HEAP8[$149>>0]|0;
   $151 = $150&255;
   $152 = (($151) + -1)|0;
   $153 = ($152>>>0)<(8);
   if ($153) {
    $144 = $148;$st$0 = $151;
   } else {
    $$lcssa455 = $144;$$lcssa457 = $148;$$lcssa458 = $150;$$lcssa459 = $151;$st$0$lcssa456 = $st$0;
    break;
   }
  }
  $154 = ($$lcssa458<<24>>24)==(0);
  if ($154) {
   $$0 = -1;
   label = 363;
   break;
  }
  $155 = ($$lcssa458<<24>>24)==(19);
  $156 = ($argpos$0|0)>(-1);
  L64: do {
   if ($155) {
    if ($156) {
     $$0 = -1;
     label = 363;
     break L1;
    } else {
     $1173 = $1169;$1174 = $1170;
     label = 62;
    }
   } else {
    if ($156) {
     $157 = (($nl_type) + ($argpos$0<<2)|0);
     HEAP32[$157>>2] = $$lcssa459;
     $158 = (($nl_arg) + ($argpos$0<<3)|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = ((($158)) + 4|0);
     $161 = HEAP32[$160>>2]|0;
     $1173 = $161;$1174 = $159;
     label = 62;
     break;
    }
    if (!($1)) {
     $$0 = 0;
     label = 363;
     break L1;
    }
    $162 = ($$lcssa458&255)>(20);
    if ($162) {
     $264 = $1170;$291 = $1169;
    } else {
     do {
      switch ($$lcssa459|0) {
      case 9:  {
       $arglist_current5 = HEAP32[$ap>>2]|0;
       $163 = $arglist_current5;
       $164 = ((0) + 4|0);
       $expanded78 = $164;
       $expanded77 = (($expanded78) - 1)|0;
       $165 = (($163) + ($expanded77))|0;
       $166 = ((0) + 4|0);
       $expanded82 = $166;
       $expanded81 = (($expanded82) - 1)|0;
       $expanded80 = $expanded81 ^ -1;
       $167 = $165 & $expanded80;
       $168 = $167;
       $169 = HEAP32[$168>>2]|0;
       $arglist_next6 = ((($168)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next6;
       $170 = $169;
       $264 = $170;$291 = $1169;
       break L64;
       break;
      }
      case 18:  {
       $arglist_current32 = HEAP32[$ap>>2]|0;
       $246 = $arglist_current32;
       $247 = ((0) + 8|0);
       $expanded141 = $247;
       $expanded140 = (($expanded141) - 1)|0;
       $248 = (($246) + ($expanded140))|0;
       $249 = ((0) + 8|0);
       $expanded145 = $249;
       $expanded144 = (($expanded145) - 1)|0;
       $expanded143 = $expanded144 ^ -1;
       $250 = $248 & $expanded143;
       $251 = $250;
       $252 = +HEAPF64[$251>>3];
       $arglist_next33 = ((($251)) + 8|0);
       HEAP32[$ap>>2] = $arglist_next33;
       HEAPF64[tempDoublePtr>>3] = $252;$253 = HEAP32[tempDoublePtr>>2]|0;
       $254 = HEAP32[tempDoublePtr+4>>2]|0;
       $264 = $253;$291 = $254;
       break L64;
       break;
      }
      case 14:  {
       $arglist_current20 = HEAP32[$ap>>2]|0;
       $211 = $arglist_current20;
       $212 = ((0) + 4|0);
       $expanded113 = $212;
       $expanded112 = (($expanded113) - 1)|0;
       $213 = (($211) + ($expanded112))|0;
       $214 = ((0) + 4|0);
       $expanded117 = $214;
       $expanded116 = (($expanded117) - 1)|0;
       $expanded115 = $expanded116 ^ -1;
       $215 = $213 & $expanded115;
       $216 = $215;
       $217 = HEAP32[$216>>2]|0;
       $arglist_next21 = ((($216)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next21;
       $$mask1$i37 = $217 & 65535;
       $264 = $$mask1$i37;$291 = 0;
       break L64;
       break;
      }
      case 10:  {
       $arglist_current8 = HEAP32[$ap>>2]|0;
       $171 = $arglist_current8;
       $172 = ((0) + 4|0);
       $expanded85 = $172;
       $expanded84 = (($expanded85) - 1)|0;
       $173 = (($171) + ($expanded84))|0;
       $174 = ((0) + 4|0);
       $expanded89 = $174;
       $expanded88 = (($expanded89) - 1)|0;
       $expanded87 = $expanded88 ^ -1;
       $175 = $173 & $expanded87;
       $176 = $175;
       $177 = HEAP32[$176>>2]|0;
       $arglist_next9 = ((($176)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next9;
       $178 = ($177|0)<(0);
       $179 = $178 << 31 >> 31;
       $264 = $177;$291 = $179;
       break L64;
       break;
      }
      case 15:  {
       $arglist_current23 = HEAP32[$ap>>2]|0;
       $218 = $arglist_current23;
       $219 = ((0) + 4|0);
       $expanded120 = $219;
       $expanded119 = (($expanded120) - 1)|0;
       $220 = (($218) + ($expanded119))|0;
       $221 = ((0) + 4|0);
       $expanded124 = $221;
       $expanded123 = (($expanded124) - 1)|0;
       $expanded122 = $expanded123 ^ -1;
       $222 = $220 & $expanded122;
       $223 = $222;
       $224 = HEAP32[$223>>2]|0;
       $arglist_next24 = ((($223)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next24;
       $225 = $224&255;
       $226 = $225 << 24 >> 24;
       $227 = ($226|0)<(0);
       $228 = $227 << 31 >> 31;
       $sext = $224 << 24;
       $229 = $sext >> 24;
       $264 = $229;$291 = $228;
       break L64;
       break;
      }
      case 12:  {
       $arglist_current14 = HEAP32[$ap>>2]|0;
       $187 = $arglist_current14;
       $188 = ((0) + 8|0);
       $expanded99 = $188;
       $expanded98 = (($expanded99) - 1)|0;
       $189 = (($187) + ($expanded98))|0;
       $190 = ((0) + 8|0);
       $expanded103 = $190;
       $expanded102 = (($expanded103) - 1)|0;
       $expanded101 = $expanded102 ^ -1;
       $191 = $189 & $expanded101;
       $192 = $191;
       $193 = $192;
       $194 = $193;
       $195 = HEAP32[$194>>2]|0;
       $196 = (($193) + 4)|0;
       $197 = $196;
       $198 = HEAP32[$197>>2]|0;
       $arglist_next15 = ((($192)) + 8|0);
       HEAP32[$ap>>2] = $arglist_next15;
       $264 = $195;$291 = $198;
       break L64;
       break;
      }
      case 16:  {
       $arglist_current26 = HEAP32[$ap>>2]|0;
       $230 = $arglist_current26;
       $231 = ((0) + 4|0);
       $expanded127 = $231;
       $expanded126 = (($expanded127) - 1)|0;
       $232 = (($230) + ($expanded126))|0;
       $233 = ((0) + 4|0);
       $expanded131 = $233;
       $expanded130 = (($expanded131) - 1)|0;
       $expanded129 = $expanded130 ^ -1;
       $234 = $232 & $expanded129;
       $235 = $234;
       $236 = HEAP32[$235>>2]|0;
       $arglist_next27 = ((($235)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next27;
       $$mask$i38 = $236 & 255;
       $264 = $$mask$i38;$291 = 0;
       break L64;
       break;
      }
      case 13:  {
       $arglist_current17 = HEAP32[$ap>>2]|0;
       $199 = $arglist_current17;
       $200 = ((0) + 4|0);
       $expanded106 = $200;
       $expanded105 = (($expanded106) - 1)|0;
       $201 = (($199) + ($expanded105))|0;
       $202 = ((0) + 4|0);
       $expanded110 = $202;
       $expanded109 = (($expanded110) - 1)|0;
       $expanded108 = $expanded109 ^ -1;
       $203 = $201 & $expanded108;
       $204 = $203;
       $205 = HEAP32[$204>>2]|0;
       $arglist_next18 = ((($204)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next18;
       $206 = $205&65535;
       $207 = $206 << 16 >> 16;
       $208 = ($207|0)<(0);
       $209 = $208 << 31 >> 31;
       $sext93 = $205 << 16;
       $210 = $sext93 >> 16;
       $264 = $210;$291 = $209;
       break L64;
       break;
      }
      case 11:  {
       $arglist_current11 = HEAP32[$ap>>2]|0;
       $180 = $arglist_current11;
       $181 = ((0) + 4|0);
       $expanded92 = $181;
       $expanded91 = (($expanded92) - 1)|0;
       $182 = (($180) + ($expanded91))|0;
       $183 = ((0) + 4|0);
       $expanded96 = $183;
       $expanded95 = (($expanded96) - 1)|0;
       $expanded94 = $expanded95 ^ -1;
       $184 = $182 & $expanded94;
       $185 = $184;
       $186 = HEAP32[$185>>2]|0;
       $arglist_next12 = ((($185)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next12;
       $264 = $186;$291 = 0;
       break L64;
       break;
      }
      case 17:  {
       $arglist_current29 = HEAP32[$ap>>2]|0;
       $237 = $arglist_current29;
       $238 = ((0) + 8|0);
       $expanded134 = $238;
       $expanded133 = (($expanded134) - 1)|0;
       $239 = (($237) + ($expanded133))|0;
       $240 = ((0) + 8|0);
       $expanded138 = $240;
       $expanded137 = (($expanded138) - 1)|0;
       $expanded136 = $expanded137 ^ -1;
       $241 = $239 & $expanded136;
       $242 = $241;
       $243 = +HEAPF64[$242>>3];
       $arglist_next30 = ((($242)) + 8|0);
       HEAP32[$ap>>2] = $arglist_next30;
       HEAPF64[tempDoublePtr>>3] = $243;$244 = HEAP32[tempDoublePtr>>2]|0;
       $245 = HEAP32[tempDoublePtr+4>>2]|0;
       $264 = $244;$291 = $245;
       break L64;
       break;
      }
      default: {
       $264 = $1170;$291 = $1169;
       break L64;
      }
      }
     } while(0);
    }
   }
  } while(0);
  if ((label|0) == 62) {
   label = 0;
   if ($1) {
    $264 = $1174;$291 = $1173;
   } else {
    $1169 = $1173;$1170 = $1174;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
    continue;
   }
  }
  $255 = HEAP8[$$lcssa455>>0]|0;
  $256 = $255 << 24 >> 24;
  $257 = ($st$0$lcssa456|0)!=(0);
  $258 = $256 & 15;
  $259 = ($258|0)==(3);
  $or$cond15 = $257 & $259;
  $260 = $256 & -33;
  $t$0 = $or$cond15 ? $260 : $256;
  $261 = $fl$1 & 8192;
  $262 = ($261|0)==(0);
  $263 = $fl$1 & -65537;
  $fl$1$ = $262 ? $fl$1 : $263;
  L86: do {
   switch ($t$0|0) {
   case 88: case 120:  {
    $fl$3 = $fl$1$;$p$1 = $p$0;$t$1 = $t$0;
    label = 73;
    break;
   }
   case 110:  {
    switch ($st$0$lcssa456|0) {
    case 7:  {
     $279 = ($cnt$1|0)<(0);
     $280 = $279 << 31 >> 31;
     $281 = $264;
     $282 = $281;
     $283 = $282;
     HEAP32[$283>>2] = $cnt$1;
     $284 = (($282) + 4)|0;
     $285 = $284;
     HEAP32[$285>>2] = $280;
     $1169 = $291;$1170 = $264;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 2:  {
     $267 = ($cnt$1|0)<(0);
     $268 = $267 << 31 >> 31;
     $269 = $264;
     $270 = $269;
     $271 = $270;
     HEAP32[$271>>2] = $cnt$1;
     $272 = (($270) + 4)|0;
     $273 = $272;
     HEAP32[$273>>2] = $268;
     $1169 = $291;$1170 = $264;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 3:  {
     $274 = $cnt$1&65535;
     $275 = $264;
     HEAP16[$275>>1] = $274;
     $1169 = $291;$1170 = $264;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 0:  {
     $265 = $264;
     HEAP32[$265>>2] = $cnt$1;
     $1169 = $291;$1170 = $264;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 1:  {
     $266 = $264;
     HEAP32[$266>>2] = $cnt$1;
     $1169 = $291;$1170 = $264;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 4:  {
     $276 = $cnt$1&255;
     $277 = $264;
     HEAP8[$277>>0] = $276;
     $1169 = $291;$1170 = $264;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 6:  {
     $278 = $264;
     HEAP32[$278>>2] = $cnt$1;
     $1169 = $291;$1170 = $264;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    default: {
     $1169 = $291;$1170 = $264;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
    }
    }
    break;
   }
   case 105: case 100:  {
    $329 = ($291|0)<(0);
    if ($329) {
     $330 = (_i64Subtract(0,0,($264|0),($291|0))|0);
     $331 = tempRet0;
     $336 = $331;$338 = $330;$pl$0 = 1;$prefix$0 = 4104;
     label = 84;
     break L86;
    }
    $332 = $fl$1$ & 2048;
    $333 = ($332|0)==(0);
    if ($333) {
     $334 = $fl$1$ & 1;
     $335 = ($334|0)==(0);
     $$ = $335 ? 4104 : (4106);
     $336 = $291;$338 = $264;$pl$0 = $334;$prefix$0 = $$;
     label = 84;
    } else {
     $336 = $291;$338 = $264;$pl$0 = 1;$prefix$0 = (4105);
     label = 84;
    }
    break;
   }
   case 115:  {
    $382 = $264;
    $383 = ($264|0)!=(0);
    $384 = $383 ? $382 : 4120;
    $a$1 = $384;
    label = 94;
    break;
   }
   case 67:  {
    HEAP32[$wc>>2] = $264;
    HEAP32[$5>>2] = 0;
    $1179 = $wc;$1180 = $6;$p$4272 = -1;
    label = 97;
    break;
   }
   case 83:  {
    $391 = $264;
    $392 = ($p$0|0)==(0);
    if ($392) {
     $1181 = $264;$1182 = $391;$i$0$lcssa273 = 0;
     label = 102;
    } else {
     $1179 = $391;$1180 = $264;$p$4272 = $p$0;
     label = 97;
    }
    break;
   }
   case 99:  {
    $378 = $264&255;
    HEAP8[$4>>0] = $378;
    $1175 = $291;$1176 = $264;$a$2 = $4;$fl$6 = $263;$p$5 = 1;$pl$2 = 0;$prefix$2 = 4104;$z$2 = $2;
    break;
   }
   case 109:  {
    $379 = (___errno_location()|0);
    $380 = HEAP32[$379>>2]|0;
    $381 = (_strerror(($380|0))|0);
    $a$1 = $381;
    label = 94;
    break;
   }
   case 112:  {
    $286 = ($p$0>>>0)>(8);
    $287 = $286 ? $p$0 : 8;
    $288 = $fl$1$ | 8;
    $fl$3 = $288;$p$1 = $287;$t$1 = 120;
    label = 73;
    break;
   }
   case 117:  {
    $336 = $291;$338 = $264;$pl$0 = 0;$prefix$0 = 4104;
    label = 84;
    break;
   }
   case 111:  {
    $312 = ($264|0)==(0);
    $313 = ($291|0)==(0);
    $314 = $312 & $313;
    if ($314) {
     $$0$lcssa$i51 = $2;
    } else {
     $$03$i48 = $2;$316 = $264;$320 = $291;
     while(1) {
      $315 = $316 & 7;
      $317 = $315 | 48;
      $318 = $317&255;
      $319 = ((($$03$i48)) + -1|0);
      HEAP8[$319>>0] = $318;
      $321 = (_bitshift64Lshr(($316|0),($320|0),3)|0);
      $322 = tempRet0;
      $323 = ($321|0)==(0);
      $324 = ($322|0)==(0);
      $325 = $323 & $324;
      if ($325) {
       $$0$lcssa$i51 = $319;
       break;
      } else {
       $$03$i48 = $319;$316 = $321;$320 = $322;
      }
     }
    }
    $326 = $fl$1$ & 8;
    $327 = ($326|0)==(0);
    $or$cond19 = $327 | $314;
    $$24 = $or$cond19 ? 4104 : (4109);
    $328 = $or$cond19&1;
    $$25 = $328 ^ 1;
    $366 = $264;$368 = $291;$a$0 = $$0$lcssa$i51;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $$25;$prefix$1 = $$24;
    label = 89;
    break;
   }
   case 65: case 71: case 70: case 69: case 97: case 103: case 102: case 101:  {
    HEAP32[tempDoublePtr>>2] = $264;HEAP32[tempDoublePtr+4>>2] = $291;$430 = +HEAPF64[tempDoublePtr>>3];
    HEAP32[$e2$i>>2] = 0;
    $431 = ($291|0)<(0);
    if ($431) {
     $432 = -$430;
     $$07$i = $432;$pl$0$i = 1;$prefix$0$i = 4128;
    } else {
     $433 = $fl$1$ & 2048;
     $434 = ($433|0)==(0);
     if ($434) {
      $435 = $fl$1$ & 1;
      $436 = ($435|0)==(0);
      $$$i = $436 ? (4129) : (4134);
      $$07$i = $430;$pl$0$i = $435;$prefix$0$i = $$$i;
     } else {
      $$07$i = $430;$pl$0$i = 1;$prefix$0$i = (4131);
     }
    }
    HEAPF64[tempDoublePtr>>3] = $$07$i;$437 = HEAP32[tempDoublePtr>>2]|0;
    $438 = HEAP32[tempDoublePtr+4>>2]|0;
    $439 = $438 & 2146435072;
    $440 = ($439>>>0)<(2146435072);
    $441 = (0)<(0);
    $442 = ($439|0)==(2146435072);
    $443 = $442 & $441;
    $444 = $440 | $443;
    do {
     if ($444) {
      $471 = (+_frexpl($$07$i,$e2$i));
      $472 = $471 * 2.0;
      $473 = $472 != 0.0;
      if ($473) {
       $474 = HEAP32[$e2$i>>2]|0;
       $475 = (($474) + -1)|0;
       HEAP32[$e2$i>>2] = $475;
      }
      $476 = $t$0 | 32;
      $477 = ($476|0)==(97);
      if ($477) {
       $478 = $t$0 & 32;
       $479 = ($478|0)==(0);
       $480 = ((($prefix$0$i)) + 9|0);
       $prefix$0$$i = $479 ? $prefix$0$i : $480;
       $481 = $pl$0$i | 2;
       $482 = ($p$0>>>0)>(11);
       $483 = (12 - ($p$0))|0;
       $484 = ($483|0)==(0);
       $485 = $482 | $484;
       do {
        if ($485) {
         $$1$i = $472;
        } else {
         $re$1179$i = $483;$round$0178$i = 8.0;
         while(1) {
          $486 = (($re$1179$i) + -1)|0;
          $487 = $round$0178$i * 16.0;
          $488 = ($486|0)==(0);
          if ($488) {
           $$lcssa483 = $487;
           break;
          } else {
           $re$1179$i = $486;$round$0178$i = $487;
          }
         }
         $489 = HEAP8[$prefix$0$$i>>0]|0;
         $490 = ($489<<24>>24)==(45);
         if ($490) {
          $491 = -$472;
          $492 = $491 - $$lcssa483;
          $493 = $$lcssa483 + $492;
          $494 = -$493;
          $$1$i = $494;
          break;
         } else {
          $495 = $472 + $$lcssa483;
          $496 = $495 - $$lcssa483;
          $$1$i = $496;
          break;
         }
        }
       } while(0);
       $497 = HEAP32[$e2$i>>2]|0;
       $498 = ($497|0)<(0);
       $499 = (0 - ($497))|0;
       $500 = $498 ? $499 : $497;
       $501 = ($500|0)<(0);
       if ($501) {
        $502 = ($500|0)<(0);
        $503 = $502 << 31 >> 31;
        $$05$i$i = $7;$504 = $500;$505 = $503;
        while(1) {
         $506 = (___uremdi3(($504|0),($505|0),10,0)|0);
         $507 = tempRet0;
         $508 = $506 | 48;
         $509 = $508&255;
         $510 = ((($$05$i$i)) + -1|0);
         HEAP8[$510>>0] = $509;
         $511 = (___udivdi3(($504|0),($505|0),10,0)|0);
         $512 = tempRet0;
         $513 = ($505>>>0)>(9);
         $514 = ($504>>>0)>(4294967295);
         $515 = ($505|0)==(9);
         $516 = $515 & $514;
         $517 = $513 | $516;
         if ($517) {
          $$05$i$i = $510;$504 = $511;$505 = $512;
         } else {
          $$lcssa484 = $510;$1183 = $511;$1184 = $512;
          break;
         }
        }
        $$0$lcssa$i48$i = $$lcssa484;$$01$lcssa$off0$i$i = $1183;
       } else {
        $$0$lcssa$i48$i = $7;$$01$lcssa$off0$i$i = $500;
       }
       $518 = ($$01$lcssa$off0$i$i|0)==(0);
       if ($518) {
        $$1$lcssa$i$i = $$0$lcssa$i48$i;
       } else {
        $$12$i$i = $$0$lcssa$i48$i;$y$03$i$i = $$01$lcssa$off0$i$i;
        while(1) {
         $519 = (($y$03$i$i>>>0) % 10)&-1;
         $520 = $519 | 48;
         $521 = $520&255;
         $522 = ((($$12$i$i)) + -1|0);
         HEAP8[$522>>0] = $521;
         $523 = (($y$03$i$i>>>0) / 10)&-1;
         $524 = ($y$03$i$i>>>0)<(10);
         if ($524) {
          $$1$lcssa$i$i = $522;
          break;
         } else {
          $$12$i$i = $522;$y$03$i$i = $523;
         }
        }
       }
       $525 = ($$1$lcssa$i$i|0)==($7|0);
       if ($525) {
        HEAP8[$8>>0] = 48;
        $estr$0$i = $8;
       } else {
        $estr$0$i = $$1$lcssa$i$i;
       }
       $526 = HEAP32[$e2$i>>2]|0;
       $527 = $526 >> 31;
       $528 = $527 & 2;
       $529 = (($528) + 43)|0;
       $530 = $529&255;
       $531 = ((($estr$0$i)) + -1|0);
       HEAP8[$531>>0] = $530;
       $532 = (($t$0) + 15)|0;
       $533 = $532&255;
       $534 = ((($estr$0$i)) + -2|0);
       HEAP8[$534>>0] = $533;
       $535 = $fl$1$ & 8;
       $536 = ($535|0)==(0);
       if ($536) {
        $notrhs$i = ($p$0|0)<(1);
        if ($notrhs$i) {
         $$2$us$us$i = $$1$i;$s$0$us$us$i = $buf$i;
         while(1) {
          $537 = (~~(($$2$us$us$i)));
          $538 = (4088 + ($537)|0);
          $539 = HEAP8[$538>>0]|0;
          $540 = $539&255;
          $541 = $540 | $478;
          $542 = $541&255;
          $543 = ((($s$0$us$us$i)) + 1|0);
          HEAP8[$s$0$us$us$i>>0] = $542;
          $544 = (+($537|0));
          $545 = $$2$us$us$i - $544;
          $546 = $545 * 16.0;
          $547 = $543;
          $548 = (($547) - ($0))|0;
          $549 = ($548|0)!=(1);
          $notlhs$us$us$i = $546 == 0.0;
          $or$cond$i79 = $549 | $notlhs$us$us$i;
          if ($or$cond$i79) {
           $s$1$us$us$i = $543;
          } else {
           $550 = ((($s$0$us$us$i)) + 2|0);
           HEAP8[$543>>0] = 46;
           $s$1$us$us$i = $550;
          }
          $551 = $546 != 0.0;
          if ($551) {
           $$2$us$us$i = $546;$s$0$us$us$i = $s$1$us$us$i;
          } else {
           $s$1$lcssa$i = $s$1$us$us$i;
           break;
          }
         }
        } else {
         $$2$us$i = $$1$i;$s$0$us$i = $buf$i;
         while(1) {
          $552 = (~~(($$2$us$i)));
          $553 = (4088 + ($552)|0);
          $554 = HEAP8[$553>>0]|0;
          $555 = $554&255;
          $556 = $555 | $478;
          $557 = $556&255;
          $558 = ((($s$0$us$i)) + 1|0);
          HEAP8[$s$0$us$i>>0] = $557;
          $559 = (+($552|0));
          $560 = $$2$us$i - $559;
          $561 = $560 * 16.0;
          $562 = $558;
          $563 = (($562) - ($0))|0;
          $564 = ($563|0)==(1);
          if ($564) {
           $565 = ((($s$0$us$i)) + 2|0);
           HEAP8[$558>>0] = 46;
           $s$1$us$i = $565;
          } else {
           $s$1$us$i = $558;
          }
          $566 = $561 != 0.0;
          if ($566) {
           $$2$us$i = $561;$s$0$us$i = $s$1$us$i;
          } else {
           $s$1$lcssa$i = $s$1$us$i;
           break;
          }
         }
        }
       } else {
        $$2$i = $$1$i;$s$0$i = $buf$i;
        while(1) {
         $567 = (~~(($$2$i)));
         $568 = (4088 + ($567)|0);
         $569 = HEAP8[$568>>0]|0;
         $570 = $569&255;
         $571 = $570 | $478;
         $572 = $571&255;
         $573 = ((($s$0$i)) + 1|0);
         HEAP8[$s$0$i>>0] = $572;
         $574 = (+($567|0));
         $575 = $$2$i - $574;
         $576 = $575 * 16.0;
         $577 = $573;
         $578 = (($577) - ($0))|0;
         $579 = ($578|0)==(1);
         if ($579) {
          $580 = ((($s$0$i)) + 2|0);
          HEAP8[$573>>0] = 46;
          $s$1$i = $580;
         } else {
          $s$1$i = $573;
         }
         $581 = $576 != 0.0;
         if ($581) {
          $$2$i = $576;$s$0$i = $s$1$i;
         } else {
          $s$1$lcssa$i = $s$1$i;
          break;
         }
        }
       }
       $582 = ($p$0|0)!=(0);
       $$pre300$i = $s$1$lcssa$i;
       $583 = (($11) + ($$pre300$i))|0;
       $584 = ($583|0)<($p$0|0);
       $or$cond332 = $582 & $584;
       $585 = $534;
       $586 = (($12) + ($p$0))|0;
       $587 = (($586) - ($585))|0;
       $588 = $534;
       $589 = (($10) - ($588))|0;
       $590 = (($589) + ($$pre300$i))|0;
       $l$0$i = $or$cond332 ? $587 : $590;
       $591 = (($l$0$i) + ($481))|0;
       $592 = $fl$1$ & 73728;
       $593 = ($592|0)==(0);
       $594 = ($w$1|0)>($591|0);
       $or$cond$i52$i = $593 & $594;
       if ($or$cond$i52$i) {
        $595 = (($w$1) - ($591))|0;
        $596 = ($595>>>0)>(256);
        $597 = $596 ? 256 : $595;
        _memset(($pad$i|0),32,($597|0))|0;
        $598 = ($595>>>0)>(255);
        if ($598) {
         $$01$i54$i = $595;
         while(1) {
          (___fwritex($pad$i,256,$f)|0);
          $599 = (($$01$i54$i) + -256)|0;
          $600 = ($599>>>0)>(255);
          if ($600) {
           $$01$i54$i = $599;
          } else {
           break;
          }
         }
         $601 = $595 & 255;
         $$0$lcssa$i56$i = $601;
        } else {
         $$0$lcssa$i56$i = $595;
        }
        (___fwritex($pad$i,$$0$lcssa$i56$i,$f)|0);
       }
       (___fwritex($prefix$0$$i,$481,$f)|0);
       $602 = ($592|0)==(65536);
       $or$cond$i59$i = $602 & $594;
       if ($or$cond$i59$i) {
        $603 = (($w$1) - ($591))|0;
        $604 = ($603>>>0)>(256);
        $605 = $604 ? 256 : $603;
        _memset(($pad$i|0),48,($605|0))|0;
        $606 = ($603>>>0)>(255);
        if ($606) {
         $$01$i61$i = $603;
         while(1) {
          (___fwritex($pad$i,256,$f)|0);
          $607 = (($$01$i61$i) + -256)|0;
          $608 = ($607>>>0)>(255);
          if ($608) {
           $$01$i61$i = $607;
          } else {
           break;
          }
         }
         $609 = $603 & 255;
         $$0$lcssa$i63$i = $609;
        } else {
         $$0$lcssa$i63$i = $603;
        }
        (___fwritex($pad$i,$$0$lcssa$i63$i,$f)|0);
       }
       $610 = (($$pre300$i) - ($0))|0;
       (___fwritex($buf$i,$610,$f)|0);
       $611 = $534;
       $612 = (($9) - ($611))|0;
       $613 = (($l$0$i) - ($612))|0;
       $614 = (($613) - ($610))|0;
       $615 = ($614|0)>(0);
       if ($615) {
        $616 = ($614>>>0)>(256);
        $617 = $616 ? 256 : $614;
        _memset(($pad$i|0),48,($617|0))|0;
        $618 = ($614>>>0)>(255);
        if ($618) {
         $$01$i67$i = $614;
         while(1) {
          (___fwritex($pad$i,256,$f)|0);
          $619 = (($$01$i67$i) + -256)|0;
          $620 = ($619>>>0)>(255);
          if ($620) {
           $$01$i67$i = $619;
          } else {
           break;
          }
         }
         $621 = $614 & 255;
         $$0$lcssa$i69$i = $621;
        } else {
         $$0$lcssa$i69$i = $614;
        }
        (___fwritex($pad$i,$$0$lcssa$i69$i,$f)|0);
       }
       (___fwritex($534,$612,$f)|0);
       $622 = ($592|0)==(8192);
       $or$cond$i72$i = $622 & $594;
       if ($or$cond$i72$i) {
        $623 = (($w$1) - ($591))|0;
        $624 = ($623>>>0)>(256);
        $625 = $624 ? 256 : $623;
        _memset(($pad$i|0),32,($625|0))|0;
        $626 = ($623>>>0)>(255);
        if ($626) {
         $$01$i74$i = $623;
         while(1) {
          (___fwritex($pad$i,256,$f)|0);
          $627 = (($$01$i74$i) + -256)|0;
          $628 = ($627>>>0)>(255);
          if ($628) {
           $$01$i74$i = $627;
          } else {
           break;
          }
         }
         $629 = $623 & 255;
         $$0$lcssa$i76$i = $629;
        } else {
         $$0$lcssa$i76$i = $623;
        }
        (___fwritex($pad$i,$$0$lcssa$i76$i,$f)|0);
       }
       $w$$i = $594 ? $w$1 : $591;
       $$0$i = $w$$i;
       break;
      }
      $630 = ($p$0|0)<(0);
      $$p$i = $630 ? 6 : $p$0;
      if ($473) {
       $631 = $472 * 268435456.0;
       $632 = HEAP32[$e2$i>>2]|0;
       $633 = (($632) + -28)|0;
       HEAP32[$e2$i>>2] = $633;
       $$3$i = $631;$634 = $633;
      } else {
       $$pre$i = HEAP32[$e2$i>>2]|0;
       $$3$i = $472;$634 = $$pre$i;
      }
      $635 = ($634|0)<(0);
      $$31$i = $635 ? $big$i : $13;
      $636 = $$31$i;
      $$4$i = $$3$i;$z$0$i = $$31$i;
      while(1) {
       $637 = (~~(($$4$i))>>>0);
       HEAP32[$z$0$i>>2] = $637;
       $638 = ((($z$0$i)) + 4|0);
       $639 = (+($637>>>0));
       $640 = $$4$i - $639;
       $641 = $640 * 1.0E+9;
       $642 = $641 != 0.0;
       if ($642) {
        $$4$i = $641;$z$0$i = $638;
       } else {
        $$lcssa460 = $638;
        break;
       }
      }
      $$pr$i = HEAP32[$e2$i>>2]|0;
      $643 = ($$pr$i|0)>(0);
      if ($643) {
       $644 = $$pr$i;$a$1253$i = $$31$i;$z$1252$i = $$lcssa460;
       while(1) {
        $645 = ($644|0)>(29);
        $646 = $645 ? 29 : $644;
        $d$0245$i = ((($z$1252$i)) + -4|0);
        $647 = ($d$0245$i>>>0)<($a$1253$i>>>0);
        do {
         if ($647) {
          $a$2$ph$i = $a$1253$i;
         } else {
          $carry$0246$i = 0;$d$0247$i = $d$0245$i;
          while(1) {
           $648 = HEAP32[$d$0247$i>>2]|0;
           $649 = (_bitshift64Shl(($648|0),0,($646|0))|0);
           $650 = tempRet0;
           $651 = (_i64Add(($649|0),($650|0),($carry$0246$i|0),0)|0);
           $652 = tempRet0;
           $653 = (___uremdi3(($651|0),($652|0),1000000000,0)|0);
           $654 = tempRet0;
           HEAP32[$d$0247$i>>2] = $653;
           $655 = (___udivdi3(($651|0),($652|0),1000000000,0)|0);
           $656 = tempRet0;
           $d$0$i = ((($d$0247$i)) + -4|0);
           $657 = ($d$0$i>>>0)<($a$1253$i>>>0);
           if ($657) {
            $$lcssa461 = $655;
            break;
           } else {
            $carry$0246$i = $655;$d$0247$i = $d$0$i;
           }
          }
          $658 = ($$lcssa461|0)==(0);
          if ($658) {
           $a$2$ph$i = $a$1253$i;
           break;
          }
          $659 = ((($a$1253$i)) + -4|0);
          HEAP32[$659>>2] = $$lcssa461;
          $a$2$ph$i = $659;
         }
        } while(0);
        $z$2$i = $z$1252$i;
        while(1) {
         $660 = ($z$2$i>>>0)>($a$2$ph$i>>>0);
         if (!($660)) {
          $z$2$i$lcssa = $z$2$i;
          break;
         }
         $661 = ((($z$2$i)) + -4|0);
         $662 = HEAP32[$661>>2]|0;
         $663 = ($662|0)==(0);
         if ($663) {
          $z$2$i = $661;
         } else {
          $z$2$i$lcssa = $z$2$i;
          break;
         }
        }
        $664 = HEAP32[$e2$i>>2]|0;
        $665 = (($664) - ($646))|0;
        HEAP32[$e2$i>>2] = $665;
        $666 = ($665|0)>(0);
        if ($666) {
         $644 = $665;$a$1253$i = $a$2$ph$i;$z$1252$i = $z$2$i$lcssa;
        } else {
         $$pr146$i = $665;$a$1$lcssa$i = $a$2$ph$i;$z$1$lcssa$i = $z$2$i$lcssa;
         break;
        }
       }
      } else {
       $$pr146$i = $$pr$i;$a$1$lcssa$i = $$31$i;$z$1$lcssa$i = $$lcssa460;
      }
      $667 = ($$pr146$i|0)<(0);
      L220: do {
       if ($667) {
        $668 = (($$p$i) + 25)|0;
        $669 = (($668|0) / 9)&-1;
        $670 = (($669) + 1)|0;
        $671 = ($476|0)==(102);
        if (!($671)) {
         $704 = $$pr146$i;$a$3240$i = $a$1$lcssa$i;$z$3239$i = $z$1$lcssa$i;
         while(1) {
          $703 = (0 - ($704))|0;
          $705 = ($703|0)>(9);
          $706 = $705 ? 9 : $703;
          $707 = ($a$3240$i>>>0)<($z$3239$i>>>0);
          do {
           if ($707) {
            $711 = 1 << $706;
            $712 = (($711) + -1)|0;
            $713 = 1000000000 >>> $706;
            $carry3$0234$i = 0;$d$1233$i = $a$3240$i;
            while(1) {
             $714 = HEAP32[$d$1233$i>>2]|0;
             $715 = $714 & $712;
             $716 = $714 >>> $706;
             $717 = (($716) + ($carry3$0234$i))|0;
             HEAP32[$d$1233$i>>2] = $717;
             $718 = Math_imul($715, $713)|0;
             $719 = ((($d$1233$i)) + 4|0);
             $720 = ($719>>>0)<($z$3239$i>>>0);
             if ($720) {
              $carry3$0234$i = $718;$d$1233$i = $719;
             } else {
              $$lcssa463 = $718;
              break;
             }
            }
            $721 = HEAP32[$a$3240$i>>2]|0;
            $722 = ($721|0)==(0);
            $723 = ((($a$3240$i)) + 4|0);
            $$a$3$i = $722 ? $723 : $a$3240$i;
            $724 = ($$lcssa463|0)==(0);
            if ($724) {
             $$a$3306$i = $$a$3$i;$z$4$i = $z$3239$i;
             break;
            }
            $725 = ((($z$3239$i)) + 4|0);
            HEAP32[$z$3239$i>>2] = $$lcssa463;
            $$a$3306$i = $$a$3$i;$z$4$i = $725;
           } else {
            $708 = HEAP32[$a$3240$i>>2]|0;
            $709 = ($708|0)==(0);
            $710 = ((($a$3240$i)) + 4|0);
            $$a$3305$i = $709 ? $710 : $a$3240$i;
            $$a$3306$i = $$a$3305$i;$z$4$i = $z$3239$i;
           }
          } while(0);
          $726 = $z$4$i;
          $727 = $$a$3306$i;
          $728 = (($726) - ($727))|0;
          $729 = $728 >> 2;
          $730 = ($729|0)>($670|0);
          $731 = (($$a$3306$i) + ($670<<2)|0);
          $$z$4$i = $730 ? $731 : $z$4$i;
          $732 = HEAP32[$e2$i>>2]|0;
          $733 = (($732) + ($706))|0;
          HEAP32[$e2$i>>2] = $733;
          $734 = ($733|0)<(0);
          if ($734) {
           $704 = $733;$a$3240$i = $$a$3306$i;$z$3239$i = $$z$4$i;
          } else {
           $a$3$lcssa$i = $$a$3306$i;$z$3$lcssa$i = $$z$4$i;
           break L220;
          }
         }
        }
        $672 = (($$31$i) + ($670<<2)|0);
        $674 = $$pr146$i;$a$3240$us$i = $a$1$lcssa$i;$z$3239$us$i = $z$1$lcssa$i;
        while(1) {
         $673 = (0 - ($674))|0;
         $675 = ($673|0)>(9);
         $676 = $675 ? 9 : $673;
         $677 = ($a$3240$us$i>>>0)<($z$3239$us$i>>>0);
         do {
          if ($677) {
           $702 = 1 << $676;
           $695 = (($702) + -1)|0;
           $699 = 1000000000 >>> $676;
           $carry3$0234$us$i = 0;$d$1233$us$i = $a$3240$us$i;
           while(1) {
            $693 = HEAP32[$d$1233$us$i>>2]|0;
            $694 = $693 & $695;
            $696 = $693 >>> $676;
            $697 = (($696) + ($carry3$0234$us$i))|0;
            HEAP32[$d$1233$us$i>>2] = $697;
            $698 = Math_imul($694, $699)|0;
            $700 = ((($d$1233$us$i)) + 4|0);
            $701 = ($700>>>0)<($z$3239$us$i>>>0);
            if ($701) {
             $carry3$0234$us$i = $698;$d$1233$us$i = $700;
            } else {
             $$lcssa464 = $698;
             break;
            }
           }
           $681 = HEAP32[$a$3240$us$i>>2]|0;
           $682 = ($681|0)==(0);
           $683 = ((($a$3240$us$i)) + 4|0);
           $$a$3$us$i = $682 ? $683 : $a$3240$us$i;
           $684 = ($$lcssa464|0)==(0);
           if ($684) {
            $$a$3$us304$i = $$a$3$us$i;$z$4$us$i = $z$3239$us$i;
            break;
           }
           $685 = ((($z$3239$us$i)) + 4|0);
           HEAP32[$z$3239$us$i>>2] = $$lcssa464;
           $$a$3$us304$i = $$a$3$us$i;$z$4$us$i = $685;
          } else {
           $678 = HEAP32[$a$3240$us$i>>2]|0;
           $679 = ($678|0)==(0);
           $680 = ((($a$3240$us$i)) + 4|0);
           $$a$3$us303$i = $679 ? $680 : $a$3240$us$i;
           $$a$3$us304$i = $$a$3$us303$i;$z$4$us$i = $z$3239$us$i;
          }
         } while(0);
         $686 = $z$4$us$i;
         $687 = (($686) - ($636))|0;
         $688 = $687 >> 2;
         $689 = ($688|0)>($670|0);
         $$z$4$us$i = $689 ? $672 : $z$4$us$i;
         $690 = HEAP32[$e2$i>>2]|0;
         $691 = (($690) + ($676))|0;
         HEAP32[$e2$i>>2] = $691;
         $692 = ($691|0)<(0);
         if ($692) {
          $674 = $691;$a$3240$us$i = $$a$3$us304$i;$z$3239$us$i = $$z$4$us$i;
         } else {
          $a$3$lcssa$i = $$a$3$us304$i;$z$3$lcssa$i = $$z$4$us$i;
          break;
         }
        }
       } else {
        $a$3$lcssa$i = $a$1$lcssa$i;$z$3$lcssa$i = $z$1$lcssa$i;
       }
      } while(0);
      $735 = ($a$3$lcssa$i>>>0)<($z$3$lcssa$i>>>0);
      do {
       if ($735) {
        $736 = $a$3$lcssa$i;
        $737 = (($636) - ($736))|0;
        $738 = $737 >> 2;
        $739 = ($738*9)|0;
        $740 = HEAP32[$a$3$lcssa$i>>2]|0;
        $741 = ($740>>>0)<(10);
        if ($741) {
         $e$1$i = $739;
         break;
        } else {
         $e$0229$i = $739;$i$0228$i = 10;
        }
        while(1) {
         $742 = ($i$0228$i*10)|0;
         $743 = (($e$0229$i) + 1)|0;
         $744 = ($740>>>0)<($742>>>0);
         if ($744) {
          $e$1$i = $743;
          break;
         } else {
          $e$0229$i = $743;$i$0228$i = $742;
         }
        }
       } else {
        $e$1$i = 0;
       }
      } while(0);
      $745 = ($476|0)!=(102);
      $746 = $745 ? $e$1$i : 0;
      $747 = (($$p$i) - ($746))|0;
      $748 = ($476|0)==(103);
      $749 = ($$p$i|0)!=(0);
      $750 = $749 & $748;
      $$neg151$i = $750 << 31 >> 31;
      $751 = (($747) + ($$neg151$i))|0;
      $752 = $z$3$lcssa$i;
      $753 = (($752) - ($636))|0;
      $754 = $753 >> 2;
      $755 = ($754*9)|0;
      $756 = (($755) + -9)|0;
      $757 = ($751|0)<($756|0);
      if ($757) {
       $758 = (($751) + 9216)|0;
       $759 = (($758|0) / 9)&-1;
       $$sum$i = (($759) + -1023)|0;
       $760 = (($$31$i) + ($$sum$i<<2)|0);
       $761 = (($758|0) % 9)&-1;
       $j$0221$i = (($761) + 1)|0;
       $762 = ($j$0221$i|0)<(9);
       if ($762) {
        $i$1222$i = 10;$j$0223$i = $j$0221$i;
        while(1) {
         $763 = ($i$1222$i*10)|0;
         $j$0$i = (($j$0223$i) + 1)|0;
         $exitcond$i = ($j$0$i|0)==(9);
         if ($exitcond$i) {
          $i$1$lcssa$i = $763;
          break;
         } else {
          $i$1222$i = $763;$j$0223$i = $j$0$i;
         }
        }
       } else {
        $i$1$lcssa$i = 10;
       }
       $764 = HEAP32[$760>>2]|0;
       $765 = (($764>>>0) % ($i$1$lcssa$i>>>0))&-1;
       $766 = ($765|0)==(0);
       if ($766) {
        $$sum15$i = (($759) + -1022)|0;
        $767 = (($$31$i) + ($$sum15$i<<2)|0);
        $768 = ($767|0)==($z$3$lcssa$i|0);
        if ($768) {
         $a$7$i = $a$3$lcssa$i;$d$3$i = $760;$e$3$i = $e$1$i;
        } else {
         label = 221;
        }
       } else {
        label = 221;
       }
       do {
        if ((label|0) == 221) {
         label = 0;
         $769 = (($764>>>0) / ($i$1$lcssa$i>>>0))&-1;
         $770 = $769 & 1;
         $771 = ($770|0)==(0);
         $$20$i = $771 ? 9007199254740992.0 : 9007199254740994.0;
         $772 = (($i$1$lcssa$i|0) / 2)&-1;
         $773 = ($765>>>0)<($772>>>0);
         do {
          if ($773) {
           $small$0$i = 0.5;
          } else {
           $774 = ($765|0)==($772|0);
           if ($774) {
            $$sum16$i = (($759) + -1022)|0;
            $775 = (($$31$i) + ($$sum16$i<<2)|0);
            $776 = ($775|0)==($z$3$lcssa$i|0);
            if ($776) {
             $small$0$i = 1.0;
             break;
            }
           }
           $small$0$i = 1.5;
          }
         } while(0);
         $777 = ($pl$0$i|0)==(0);
         do {
          if ($777) {
           $round6$1$i = $$20$i;$small$1$i = $small$0$i;
          } else {
           $778 = HEAP8[$prefix$0$i>>0]|0;
           $779 = ($778<<24>>24)==(45);
           if (!($779)) {
            $round6$1$i = $$20$i;$small$1$i = $small$0$i;
            break;
           }
           $780 = -$$20$i;
           $781 = -$small$0$i;
           $round6$1$i = $780;$small$1$i = $781;
          }
         } while(0);
         $782 = (($764) - ($765))|0;
         HEAP32[$760>>2] = $782;
         $783 = $round6$1$i + $small$1$i;
         $784 = $783 != $round6$1$i;
         if (!($784)) {
          $a$7$i = $a$3$lcssa$i;$d$3$i = $760;$e$3$i = $e$1$i;
          break;
         }
         $785 = (($782) + ($i$1$lcssa$i))|0;
         HEAP32[$760>>2] = $785;
         $786 = ($785>>>0)>(999999999);
         if ($786) {
          $a$5215$i = $a$3$lcssa$i;$d$2214$i = $760;
          while(1) {
           $787 = ((($d$2214$i)) + -4|0);
           HEAP32[$d$2214$i>>2] = 0;
           $788 = ($787>>>0)<($a$5215$i>>>0);
           if ($788) {
            $789 = ((($a$5215$i)) + -4|0);
            HEAP32[$789>>2] = 0;
            $a$6$i = $789;
           } else {
            $a$6$i = $a$5215$i;
           }
           $790 = HEAP32[$787>>2]|0;
           $791 = (($790) + 1)|0;
           HEAP32[$787>>2] = $791;
           $792 = ($791>>>0)>(999999999);
           if ($792) {
            $a$5215$i = $a$6$i;$d$2214$i = $787;
           } else {
            $a$5$lcssa$i = $a$6$i;$d$2$lcssa$i = $787;
            break;
           }
          }
         } else {
          $a$5$lcssa$i = $a$3$lcssa$i;$d$2$lcssa$i = $760;
         }
         $793 = $a$5$lcssa$i;
         $794 = (($636) - ($793))|0;
         $795 = $794 >> 2;
         $796 = ($795*9)|0;
         $797 = HEAP32[$a$5$lcssa$i>>2]|0;
         $798 = ($797>>>0)<(10);
         if ($798) {
          $a$7$i = $a$5$lcssa$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $796;
          break;
         } else {
          $e$2210$i = $796;$i$2209$i = 10;
         }
         while(1) {
          $799 = ($i$2209$i*10)|0;
          $800 = (($e$2210$i) + 1)|0;
          $801 = ($797>>>0)<($799>>>0);
          if ($801) {
           $a$7$i = $a$5$lcssa$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $800;
           break;
          } else {
           $e$2210$i = $800;$i$2209$i = $799;
          }
         }
        }
       } while(0);
       $802 = ((($d$3$i)) + 4|0);
       $803 = ($z$3$lcssa$i>>>0)>($802>>>0);
       $$z$3$i = $803 ? $802 : $z$3$lcssa$i;
       $a$8$ph$i = $a$7$i;$e$4$ph$i = $e$3$i;$z$6$ph$i = $$z$3$i;
      } else {
       $a$8$ph$i = $a$3$lcssa$i;$e$4$ph$i = $e$1$i;$z$6$ph$i = $z$3$lcssa$i;
      }
      $804 = (0 - ($e$4$ph$i))|0;
      $z$6$i = $z$6$ph$i;
      while(1) {
       $805 = ($z$6$i>>>0)>($a$8$ph$i>>>0);
       if (!($805)) {
        $$lcssa275$i = 0;$z$6$i$lcssa = $z$6$i;
        break;
       }
       $806 = ((($z$6$i)) + -4|0);
       $807 = HEAP32[$806>>2]|0;
       $808 = ($807|0)==(0);
       if ($808) {
        $z$6$i = $806;
       } else {
        $$lcssa275$i = 1;$z$6$i$lcssa = $z$6$i;
        break;
       }
      }
      do {
       if ($748) {
        $809 = $749&1;
        $810 = $809 ^ 1;
        $$p$$i = (($810) + ($$p$i))|0;
        $811 = ($$p$$i|0)>($e$4$ph$i|0);
        $812 = ($e$4$ph$i|0)>(-5);
        $or$cond6$i = $811 & $812;
        if ($or$cond6$i) {
         $813 = (($t$0) + -1)|0;
         $$neg152$i = (($$p$$i) + -1)|0;
         $814 = (($$neg152$i) - ($e$4$ph$i))|0;
         $$013$i = $813;$$210$i = $814;
        } else {
         $815 = (($t$0) + -2)|0;
         $816 = (($$p$$i) + -1)|0;
         $$013$i = $815;$$210$i = $816;
        }
        $817 = $fl$1$ & 8;
        $818 = ($817|0)==(0);
        if (!($818)) {
         $$114$i = $$013$i;$$311$i = $$210$i;$$pre$phi302$iZ2D = $817;
         break;
        }
        do {
         if ($$lcssa275$i) {
          $819 = ((($z$6$i$lcssa)) + -4|0);
          $820 = HEAP32[$819>>2]|0;
          $821 = ($820|0)==(0);
          if ($821) {
           $j$2$i = 9;
           break;
          }
          $822 = (($820>>>0) % 10)&-1;
          $823 = ($822|0)==(0);
          if ($823) {
           $i$3205$i = 10;$j$1206$i = 0;
          } else {
           $j$2$i = 0;
           break;
          }
          while(1) {
           $824 = ($i$3205$i*10)|0;
           $825 = (($j$1206$i) + 1)|0;
           $826 = (($820>>>0) % ($824>>>0))&-1;
           $827 = ($826|0)==(0);
           if ($827) {
            $i$3205$i = $824;$j$1206$i = $825;
           } else {
            $j$2$i = $825;
            break;
           }
          }
         } else {
          $j$2$i = 9;
         }
        } while(0);
        $828 = $$013$i | 32;
        $829 = ($828|0)==(102);
        $830 = $z$6$i$lcssa;
        $831 = (($830) - ($636))|0;
        $832 = $831 >> 2;
        $833 = ($832*9)|0;
        $834 = (($833) + -9)|0;
        if ($829) {
         $835 = (($834) - ($j$2$i))|0;
         $836 = ($835|0)<(0);
         $$21$i = $836 ? 0 : $835;
         $837 = ($$210$i|0)<($$21$i|0);
         $$210$$22$i = $837 ? $$210$i : $$21$i;
         $$114$i = $$013$i;$$311$i = $$210$$22$i;$$pre$phi302$iZ2D = 0;
         break;
        } else {
         $838 = (($834) + ($e$4$ph$i))|0;
         $839 = (($838) - ($j$2$i))|0;
         $840 = ($839|0)<(0);
         $$23$i = $840 ? 0 : $839;
         $841 = ($$210$i|0)<($$23$i|0);
         $$210$$24$i = $841 ? $$210$i : $$23$i;
         $$114$i = $$013$i;$$311$i = $$210$$24$i;$$pre$phi302$iZ2D = 0;
         break;
        }
       } else {
        $$pre301$i = $fl$1$ & 8;
        $$114$i = $t$0;$$311$i = $$p$i;$$pre$phi302$iZ2D = $$pre301$i;
       }
      } while(0);
      $842 = $$311$i | $$pre$phi302$iZ2D;
      $843 = ($842|0)!=(0);
      $844 = $843&1;
      $845 = $$114$i | 32;
      $846 = ($845|0)==(102);
      if ($846) {
       $847 = ($e$4$ph$i|0)>(0);
       $848 = $847 ? $e$4$ph$i : 0;
       $$pn$i = $848;$estr$2$i = 0;
      } else {
       $849 = ($e$4$ph$i|0)<(0);
       $850 = $849 ? $804 : $e$4$ph$i;
       $851 = ($850|0)<(0);
       if ($851) {
        $852 = ($850|0)<(0);
        $853 = $852 << 31 >> 31;
        $$05$i79$i = $7;$854 = $850;$855 = $853;
        while(1) {
         $856 = (___uremdi3(($854|0),($855|0),10,0)|0);
         $857 = tempRet0;
         $858 = $856 | 48;
         $859 = $858&255;
         $860 = ((($$05$i79$i)) + -1|0);
         HEAP8[$860>>0] = $859;
         $861 = (___udivdi3(($854|0),($855|0),10,0)|0);
         $862 = tempRet0;
         $863 = ($855>>>0)>(9);
         $864 = ($854>>>0)>(4294967295);
         $865 = ($855|0)==(9);
         $866 = $865 & $864;
         $867 = $863 | $866;
         if ($867) {
          $$05$i79$i = $860;$854 = $861;$855 = $862;
         } else {
          $$lcssa470 = $860;$1185 = $861;$1186 = $862;
          break;
         }
        }
        $$0$lcssa$i84$i = $$lcssa470;$$01$lcssa$off0$i85$i = $1185;
       } else {
        $$0$lcssa$i84$i = $7;$$01$lcssa$off0$i85$i = $850;
       }
       $868 = ($$01$lcssa$off0$i85$i|0)==(0);
       if ($868) {
        $estr$1$ph$i = $$0$lcssa$i84$i;
       } else {
        $$12$i87$i = $$0$lcssa$i84$i;$y$03$i86$i = $$01$lcssa$off0$i85$i;
        while(1) {
         $869 = (($y$03$i86$i>>>0) % 10)&-1;
         $870 = $869 | 48;
         $871 = $870&255;
         $872 = ((($$12$i87$i)) + -1|0);
         HEAP8[$872>>0] = $871;
         $873 = (($y$03$i86$i>>>0) / 10)&-1;
         $874 = ($y$03$i86$i>>>0)<(10);
         if ($874) {
          $estr$1$ph$i = $872;
          break;
         } else {
          $$12$i87$i = $872;$y$03$i86$i = $873;
         }
        }
       }
       $875 = $estr$1$ph$i;
       $876 = (($9) - ($875))|0;
       $877 = ($876|0)<(2);
       if ($877) {
        $estr$1201$i = $estr$1$ph$i;
        while(1) {
         $878 = ((($estr$1201$i)) + -1|0);
         HEAP8[$878>>0] = 48;
         $879 = $878;
         $880 = (($9) - ($879))|0;
         $881 = ($880|0)<(2);
         if ($881) {
          $estr$1201$i = $878;
         } else {
          $estr$1$lcssa$i = $878;
          break;
         }
        }
       } else {
        $estr$1$lcssa$i = $estr$1$ph$i;
       }
       $882 = $e$4$ph$i >> 31;
       $883 = $882 & 2;
       $884 = (($883) + 43)|0;
       $885 = $884&255;
       $886 = ((($estr$1$lcssa$i)) + -1|0);
       HEAP8[$886>>0] = $885;
       $887 = $$114$i&255;
       $888 = ((($estr$1$lcssa$i)) + -2|0);
       HEAP8[$888>>0] = $887;
       $889 = $888;
       $890 = (($9) - ($889))|0;
       $$pn$i = $890;$estr$2$i = $888;
      }
      $891 = (($pl$0$i) + 1)|0;
      $892 = (($891) + ($$311$i))|0;
      $l$1$i = (($892) + ($844))|0;
      $893 = (($l$1$i) + ($$pn$i))|0;
      $894 = $fl$1$ & 73728;
      $895 = ($894|0)==(0);
      $896 = ($w$1|0)>($893|0);
      $or$cond$i93$i = $895 & $896;
      if ($or$cond$i93$i) {
       $897 = (($w$1) - ($893))|0;
       $898 = ($897>>>0)>(256);
       $899 = $898 ? 256 : $897;
       _memset(($pad$i|0),32,($899|0))|0;
       $900 = ($897>>>0)>(255);
       if ($900) {
        $$01$i95$i = $897;
        while(1) {
         (___fwritex($pad$i,256,$f)|0);
         $901 = (($$01$i95$i) + -256)|0;
         $902 = ($901>>>0)>(255);
         if ($902) {
          $$01$i95$i = $901;
         } else {
          break;
         }
        }
        $903 = $897 & 255;
        $$0$lcssa$i97$i = $903;
       } else {
        $$0$lcssa$i97$i = $897;
       }
       (___fwritex($pad$i,$$0$lcssa$i97$i,$f)|0);
      }
      (___fwritex($prefix$0$i,$pl$0$i,$f)|0);
      $904 = ($894|0)==(65536);
      $or$cond$i100$i = $904 & $896;
      if ($or$cond$i100$i) {
       $905 = (($w$1) - ($893))|0;
       $906 = ($905>>>0)>(256);
       $907 = $906 ? 256 : $905;
       _memset(($pad$i|0),48,($907|0))|0;
       $908 = ($905>>>0)>(255);
       if ($908) {
        $$01$i102$i = $905;
        while(1) {
         (___fwritex($pad$i,256,$f)|0);
         $909 = (($$01$i102$i) + -256)|0;
         $910 = ($909>>>0)>(255);
         if ($910) {
          $$01$i102$i = $909;
         } else {
          break;
         }
        }
        $911 = $905 & 255;
        $$0$lcssa$i104$i = $911;
       } else {
        $$0$lcssa$i104$i = $905;
       }
       (___fwritex($pad$i,$$0$lcssa$i104$i,$f)|0);
      }
      if ($846) {
       $912 = ($a$8$ph$i>>>0)>($$31$i>>>0);
       $r$0$a$8$i = $912 ? $$31$i : $a$8$ph$i;
       $d$4191$i = $r$0$a$8$i;
       while(1) {
        $913 = HEAP32[$d$4191$i>>2]|0;
        $914 = ($913|0)==(0);
        if ($914) {
         $$1$lcssa$i112$i = $14;
        } else {
         $$12$i110$i = $14;$y$03$i109$i = $913;
         while(1) {
          $915 = (($y$03$i109$i>>>0) % 10)&-1;
          $916 = $915 | 48;
          $917 = $916&255;
          $918 = ((($$12$i110$i)) + -1|0);
          HEAP8[$918>>0] = $917;
          $919 = (($y$03$i109$i>>>0) / 10)&-1;
          $920 = ($y$03$i109$i>>>0)<(10);
          if ($920) {
           $$1$lcssa$i112$i = $918;
           break;
          } else {
           $$12$i110$i = $918;$y$03$i109$i = $919;
          }
         }
        }
        $921 = ($d$4191$i|0)==($r$0$a$8$i|0);
        do {
         if ($921) {
          $925 = ($$1$lcssa$i112$i|0)==($14|0);
          if (!($925)) {
           $s7$1$i = $$1$lcssa$i112$i;
           break;
          }
          HEAP8[$16>>0] = 48;
          $s7$1$i = $16;
         } else {
          $922 = ($$1$lcssa$i112$i>>>0)>($buf$i>>>0);
          if ($922) {
           $s7$0188$i = $$1$lcssa$i112$i;
          } else {
           $s7$1$i = $$1$lcssa$i112$i;
           break;
          }
          while(1) {
           $923 = ((($s7$0188$i)) + -1|0);
           HEAP8[$923>>0] = 48;
           $924 = ($923>>>0)>($buf$i>>>0);
           if ($924) {
            $s7$0188$i = $923;
           } else {
            $s7$1$i = $923;
            break;
           }
          }
         }
        } while(0);
        $926 = $s7$1$i;
        $927 = (($15) - ($926))|0;
        (___fwritex($s7$1$i,$927,$f)|0);
        $928 = ((($d$4191$i)) + 4|0);
        $929 = ($928>>>0)>($$31$i>>>0);
        if ($929) {
         $$lcssa479 = $928;
         break;
        } else {
         $d$4191$i = $928;
        }
       }
       $930 = ($842|0)==(0);
       if (!($930)) {
        (___fwritex(4184,1,$f)|0);
       }
       $931 = ($$lcssa479>>>0)<($z$6$i$lcssa>>>0);
       $932 = ($$311$i|0)>(0);
       $933 = $932 & $931;
       if ($933) {
        $$412184$i = $$311$i;$d$5183$i = $$lcssa479;
        while(1) {
         $934 = HEAP32[$d$5183$i>>2]|0;
         $935 = ($934|0)==(0);
         if ($935) {
          $s8$0180$i = $14;
          label = 289;
         } else {
          $$12$i119$i = $14;$y$03$i118$i = $934;
          while(1) {
           $936 = (($y$03$i118$i>>>0) % 10)&-1;
           $937 = $936 | 48;
           $938 = $937&255;
           $939 = ((($$12$i119$i)) + -1|0);
           HEAP8[$939>>0] = $938;
           $940 = (($y$03$i118$i>>>0) / 10)&-1;
           $941 = ($y$03$i118$i>>>0)<(10);
           if ($941) {
            $$lcssa480 = $939;
            break;
           } else {
            $$12$i119$i = $939;$y$03$i118$i = $940;
           }
          }
          $942 = ($$lcssa480>>>0)>($buf$i>>>0);
          if ($942) {
           $s8$0180$i = $$lcssa480;
           label = 289;
          } else {
           $s8$0$lcssa$i = $$lcssa480;
          }
         }
         if ((label|0) == 289) {
          while(1) {
           label = 0;
           $943 = ((($s8$0180$i)) + -1|0);
           HEAP8[$943>>0] = 48;
           $944 = ($943>>>0)>($buf$i>>>0);
           if ($944) {
            $s8$0180$i = $943;
            label = 289;
           } else {
            $s8$0$lcssa$i = $943;
            break;
           }
          }
         }
         $945 = ($$412184$i|0)>(9);
         $946 = $945 ? 9 : $$412184$i;
         (___fwritex($s8$0$lcssa$i,$946,$f)|0);
         $947 = ((($d$5183$i)) + 4|0);
         $948 = (($$412184$i) + -9)|0;
         $949 = ($947>>>0)<($z$6$i$lcssa>>>0);
         $950 = $945 & $949;
         if ($950) {
          $$412184$i = $948;$d$5183$i = $947;
         } else {
          $$412$lcssa$i = $948;
          break;
         }
        }
       } else {
        $$412$lcssa$i = $$311$i;
       }
       $951 = ($$412$lcssa$i|0)>(0);
       if ($951) {
        $952 = ($$412$lcssa$i>>>0)>(256);
        $953 = $952 ? 256 : $$412$lcssa$i;
        _memset(($pad$i|0),48,($953|0))|0;
        $954 = ($$412$lcssa$i>>>0)>(255);
        if ($954) {
         $$01$i126$i = $$412$lcssa$i;
         while(1) {
          (___fwritex($pad$i,256,$f)|0);
          $955 = (($$01$i126$i) + -256)|0;
          $956 = ($955>>>0)>(255);
          if ($956) {
           $$01$i126$i = $955;
          } else {
           break;
          }
         }
         $957 = $$412$lcssa$i & 255;
         $$0$lcssa$i128$i = $957;
        } else {
         $$0$lcssa$i128$i = $$412$lcssa$i;
        }
        (___fwritex($pad$i,$$0$lcssa$i128$i,$f)|0);
       }
      } else {
       $958 = ((($a$8$ph$i)) + 4|0);
       $z$6$$i = $$lcssa275$i ? $z$6$i$lcssa : $958;
       $959 = ($$311$i|0)>(-1);
       do {
        if ($959) {
         $960 = ($$pre$phi302$iZ2D|0)==(0);
         $$5196$i = $$311$i;$d$6195$i = $a$8$ph$i;
         while(1) {
          $961 = HEAP32[$d$6195$i>>2]|0;
          $962 = ($961|0)==(0);
          if ($962) {
           label = 303;
          } else {
           $$12$i134$i = $14;$y$03$i133$i = $961;
           while(1) {
            $963 = (($y$03$i133$i>>>0) % 10)&-1;
            $964 = $963 | 48;
            $965 = $964&255;
            $966 = ((($$12$i134$i)) + -1|0);
            HEAP8[$966>>0] = $965;
            $967 = (($y$03$i133$i>>>0) / 10)&-1;
            $968 = ($y$03$i133$i>>>0)<(10);
            if ($968) {
             $$12$i134$i$lcssa = $$12$i134$i;$$lcssa474 = $966;
             break;
            } else {
             $$12$i134$i = $966;$y$03$i133$i = $967;
            }
           }
           $969 = ($$lcssa474|0)==($14|0);
           if ($969) {
            label = 303;
           } else {
            $1187 = $$12$i134$i$lcssa;$s9$0$i = $$lcssa474;
           }
          }
          if ((label|0) == 303) {
           label = 0;
           HEAP8[$16>>0] = 48;
           $1187 = $14;$s9$0$i = $16;
          }
          $970 = ($d$6195$i|0)==($a$8$ph$i|0);
          do {
           if ($970) {
            (___fwritex($s9$0$i,1,$f)|0);
            $974 = ($$5196$i|0)<(1);
            $or$cond29$i = $960 & $974;
            if ($or$cond29$i) {
             $s9$2$i = $1187;
             break;
            }
            (___fwritex(4184,1,$f)|0);
            $s9$2$i = $1187;
           } else {
            $971 = ($s9$0$i>>>0)>($buf$i>>>0);
            if ($971) {
             $s9$1192$i = $s9$0$i;
            } else {
             $s9$2$i = $s9$0$i;
             break;
            }
            while(1) {
             $972 = ((($s9$1192$i)) + -1|0);
             HEAP8[$972>>0] = 48;
             $973 = ($972>>>0)>($buf$i>>>0);
             if ($973) {
              $s9$1192$i = $972;
             } else {
              $s9$2$i = $972;
              break;
             }
            }
           }
          } while(0);
          $975 = $s9$2$i;
          $976 = (($15) - ($975))|0;
          $977 = ($$5196$i|0)>($976|0);
          $978 = $977 ? $976 : $$5196$i;
          (___fwritex($s9$2$i,$978,$f)|0);
          $979 = (($$5196$i) - ($976))|0;
          $980 = ((($d$6195$i)) + 4|0);
          $981 = ($980>>>0)<($z$6$$i>>>0);
          $982 = ($979|0)>(-1);
          $983 = $981 & $982;
          if ($983) {
           $$5196$i = $979;$d$6195$i = $980;
          } else {
           $$lcssa476 = $979;
           break;
          }
         }
         $984 = ($$lcssa476|0)>(0);
         if (!($984)) {
          break;
         }
         $985 = ($$lcssa476>>>0)>(256);
         $986 = $985 ? 256 : $$lcssa476;
         _memset(($pad$i|0),48,($986|0))|0;
         $987 = ($$lcssa476>>>0)>(255);
         if ($987) {
          $$01$i141$i = $$lcssa476;
          while(1) {
           (___fwritex($pad$i,256,$f)|0);
           $988 = (($$01$i141$i) + -256)|0;
           $989 = ($988>>>0)>(255);
           if ($989) {
            $$01$i141$i = $988;
           } else {
            break;
           }
          }
          $990 = $$lcssa476 & 255;
          $$0$lcssa$i143$i = $990;
         } else {
          $$0$lcssa$i143$i = $$lcssa476;
         }
         (___fwritex($pad$i,$$0$lcssa$i143$i,$f)|0);
        } else {
        }
       } while(0);
       $991 = $estr$2$i;
       $992 = (($9) - ($991))|0;
       (___fwritex($estr$2$i,$992,$f)|0);
      }
      $993 = ($894|0)==(8192);
      $or$cond$i$i = $993 & $896;
      if ($or$cond$i$i) {
       $994 = (($w$1) - ($893))|0;
       $995 = ($994>>>0)>(256);
       $996 = $995 ? 256 : $994;
       _memset(($pad$i|0),32,($996|0))|0;
       $997 = ($994>>>0)>(255);
       if ($997) {
        $$01$i$i = $994;
        while(1) {
         (___fwritex($pad$i,256,$f)|0);
         $998 = (($$01$i$i) + -256)|0;
         $999 = ($998>>>0)>(255);
         if ($999) {
          $$01$i$i = $998;
         } else {
          break;
         }
        }
        $1000 = $994 & 255;
        $$0$lcssa$i$i = $1000;
       } else {
        $$0$lcssa$i$i = $994;
       }
       (___fwritex($pad$i,$$0$lcssa$i$i,$f)|0);
      }
      $w$30$i = $896 ? $w$1 : $893;
      $$0$i = $w$30$i;
     } else {
      $445 = $t$0 & 32;
      $446 = ($445|0)!=(0);
      $447 = $446 ? 4152 : 4160;
      $448 = ($$07$i != $$07$i) | (0.0 != 0.0);
      $449 = $446 ? 4168 : 4176;
      $pl$1$i = $448 ? 0 : $pl$0$i;
      $s1$0$i = $448 ? $449 : $447;
      $450 = (($pl$1$i) + 3)|0;
      $451 = $fl$1$ & 8192;
      $452 = ($451|0)==(0);
      $453 = ($w$1|0)>($450|0);
      $or$cond$i35$i = $452 & $453;
      if ($or$cond$i35$i) {
       $454 = (($w$1) - ($450))|0;
       $455 = ($454>>>0)>(256);
       $456 = $455 ? 256 : $454;
       _memset(($pad$i|0),32,($456|0))|0;
       $457 = ($454>>>0)>(255);
       if ($457) {
        $$01$i37$i = $454;
        while(1) {
         (___fwritex($pad$i,256,$f)|0);
         $458 = (($$01$i37$i) + -256)|0;
         $459 = ($458>>>0)>(255);
         if ($459) {
          $$01$i37$i = $458;
         } else {
          break;
         }
        }
        $460 = $454 & 255;
        $$0$lcssa$i39$i = $460;
       } else {
        $$0$lcssa$i39$i = $454;
       }
       (___fwritex($pad$i,$$0$lcssa$i39$i,$f)|0);
      }
      (___fwritex($prefix$0$i,$pl$1$i,$f)|0);
      (___fwritex($s1$0$i,3,$f)|0);
      $461 = $fl$1$ & 73728;
      $462 = ($461|0)==(8192);
      $or$cond$i42$i = $462 & $453;
      if ($or$cond$i42$i) {
       $463 = (($w$1) - ($450))|0;
       $464 = ($463>>>0)>(256);
       $465 = $464 ? 256 : $463;
       _memset(($pad$i|0),32,($465|0))|0;
       $466 = ($463>>>0)>(255);
       if ($466) {
        $$01$i44$i = $463;
        while(1) {
         (___fwritex($pad$i,256,$f)|0);
         $467 = (($$01$i44$i) + -256)|0;
         $468 = ($467>>>0)>(255);
         if ($468) {
          $$01$i44$i = $467;
         } else {
          break;
         }
        }
        $469 = $463 & 255;
        $$0$lcssa$i46$i = $469;
       } else {
        $$0$lcssa$i46$i = $463;
       }
       (___fwritex($pad$i,$$0$lcssa$i46$i,$f)|0);
      }
      $470 = $453 ? $w$1 : $450;
      $$0$i = $470;
     }
    } while(0);
    $1169 = $291;$1170 = $264;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $$0$i;$l10n$0 = $l10n$3;
    continue L1;
    break;
   }
   default: {
    $1175 = $291;$1176 = $264;$a$2 = $23;$fl$6 = $fl$1$;$p$5 = $p$0;$pl$2 = 0;$prefix$2 = 4104;$z$2 = $2;
   }
   }
  } while(0);
  if ((label|0) == 73) {
   label = 0;
   $289 = $t$1 & 32;
   $290 = ($264|0)==(0);
   $292 = ($291|0)==(0);
   $293 = $290 & $292;
   if ($293) {
    $366 = $264;$368 = $291;$a$0 = $2;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 4104;
    label = 89;
   } else {
    $$012$i = $2;$295 = $264;$302 = $291;
    while(1) {
     $294 = $295 & 15;
     $296 = (4088 + ($294)|0);
     $297 = HEAP8[$296>>0]|0;
     $298 = $297&255;
     $299 = $298 | $289;
     $300 = $299&255;
     $301 = ((($$012$i)) + -1|0);
     HEAP8[$301>>0] = $300;
     $303 = (_bitshift64Lshr(($295|0),($302|0),4)|0);
     $304 = tempRet0;
     $305 = ($303|0)==(0);
     $306 = ($304|0)==(0);
     $307 = $305 & $306;
     if ($307) {
      $$lcssa491 = $301;
      break;
     } else {
      $$012$i = $301;$295 = $303;$302 = $304;
     }
    }
    $308 = $fl$3 & 8;
    $309 = ($308|0)==(0);
    if ($309) {
     $366 = $264;$368 = $291;$a$0 = $$lcssa491;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 4104;
     label = 89;
    } else {
     $310 = $t$1 >> 4;
     $311 = (4104 + ($310)|0);
     $366 = $264;$368 = $291;$a$0 = $$lcssa491;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 2;$prefix$1 = $311;
     label = 89;
    }
   }
  }
  else if ((label|0) == 84) {
   label = 0;
   $337 = ($336>>>0)>(0);
   $339 = ($338>>>0)>(4294967295);
   $340 = ($336|0)==(0);
   $341 = $340 & $339;
   $342 = $337 | $341;
   if ($342) {
    $$05$i = $2;$343 = $338;$344 = $336;
    while(1) {
     $345 = (___uremdi3(($343|0),($344|0),10,0)|0);
     $346 = tempRet0;
     $347 = $345 | 48;
     $348 = $347&255;
     $349 = ((($$05$i)) + -1|0);
     HEAP8[$349>>0] = $348;
     $350 = (___udivdi3(($343|0),($344|0),10,0)|0);
     $351 = tempRet0;
     $352 = ($344>>>0)>(9);
     $353 = ($343>>>0)>(4294967295);
     $354 = ($344|0)==(9);
     $355 = $354 & $353;
     $356 = $352 | $355;
     if ($356) {
      $$05$i = $349;$343 = $350;$344 = $351;
     } else {
      $$lcssa487 = $349;$1177 = $350;$1178 = $351;
      break;
     }
    }
    $$0$lcssa$i53 = $$lcssa487;$$01$lcssa$off0$i = $1177;
   } else {
    $$0$lcssa$i53 = $2;$$01$lcssa$off0$i = $338;
   }
   $357 = ($$01$lcssa$off0$i|0)==(0);
   if ($357) {
    $366 = $338;$368 = $336;$a$0 = $$0$lcssa$i53;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $pl$0;$prefix$1 = $prefix$0;
    label = 89;
   } else {
    $$12$i = $$0$lcssa$i53;$y$03$i = $$01$lcssa$off0$i;
    while(1) {
     $358 = (($y$03$i>>>0) % 10)&-1;
     $359 = $358 | 48;
     $360 = $359&255;
     $361 = ((($$12$i)) + -1|0);
     HEAP8[$361>>0] = $360;
     $362 = (($y$03$i>>>0) / 10)&-1;
     $363 = ($y$03$i>>>0)<(10);
     if ($363) {
      $366 = $338;$368 = $336;$a$0 = $361;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $pl$0;$prefix$1 = $prefix$0;
      label = 89;
      break;
     } else {
      $$12$i = $361;$y$03$i = $362;
     }
    }
   }
  }
  else if ((label|0) == 94) {
   label = 0;
   $385 = (_memchr($a$1,0,$p$0)|0);
   $386 = ($385|0)==(0|0);
   $387 = $385;
   $388 = $a$1;
   $389 = (($387) - ($388))|0;
   $390 = (($a$1) + ($p$0)|0);
   $z$1 = $386 ? $390 : $385;
   $p$3 = $386 ? $p$0 : $389;
   $1175 = $291;$1176 = $264;$a$2 = $a$1;$fl$6 = $263;$p$5 = $p$3;$pl$2 = 0;$prefix$2 = 4104;$z$2 = $z$1;
  }
  else if ((label|0) == 97) {
   label = 0;
   $i$0175 = 0;$l$1174 = 0;$ws$0176 = $1179;
   while(1) {
    $393 = HEAP32[$ws$0176>>2]|0;
    $394 = ($393|0)==(0);
    if ($394) {
     $i$0$lcssa = $i$0175;$l$2 = $l$1174;
     break;
    }
    $395 = (_wctomb($mb,$393)|0);
    $396 = ($395|0)<(0);
    $397 = (($p$4272) - ($i$0175))|0;
    $398 = ($395>>>0)>($397>>>0);
    $or$cond22 = $396 | $398;
    if ($or$cond22) {
     $i$0$lcssa = $i$0175;$l$2 = $395;
     break;
    }
    $399 = ((($ws$0176)) + 4|0);
    $400 = (($395) + ($i$0175))|0;
    $401 = ($p$4272>>>0)>($400>>>0);
    if ($401) {
     $i$0175 = $400;$l$1174 = $395;$ws$0176 = $399;
    } else {
     $i$0$lcssa = $400;$l$2 = $395;
     break;
    }
   }
   $402 = ($l$2|0)<(0);
   if ($402) {
    $$0 = -1;
    label = 363;
    break;
   } else {
    $1181 = $1180;$1182 = $1179;$i$0$lcssa273 = $i$0$lcssa;
    label = 102;
   }
  }
  if ((label|0) == 89) {
   label = 0;
   $364 = ($p$2|0)>(-1);
   $365 = $fl$4 & -65537;
   $$fl$4 = $364 ? $365 : $fl$4;
   $367 = ($366|0)!=(0);
   $369 = ($368|0)!=(0);
   $370 = $367 | $369;
   $371 = ($p$2|0)!=(0);
   $or$cond = $370 | $371;
   if ($or$cond) {
    $372 = $a$0;
    $373 = (($3) - ($372))|0;
    $374 = $370&1;
    $375 = $374 ^ 1;
    $376 = (($375) + ($373))|0;
    $377 = ($p$2|0)>($376|0);
    $p$2$ = $377 ? $p$2 : $376;
    $1175 = $368;$1176 = $366;$a$2 = $a$0;$fl$6 = $$fl$4;$p$5 = $p$2$;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $2;
   } else {
    $1175 = $368;$1176 = $366;$a$2 = $2;$fl$6 = $$fl$4;$p$5 = 0;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $2;
   }
  }
  else if ((label|0) == 102) {
   label = 0;
   $403 = $fl$1$ & 73728;
   $404 = ($403|0)==(0);
   $405 = ($w$1|0)>($i$0$lcssa273|0);
   $or$cond$i64 = $404 & $405;
   if ($or$cond$i64) {
    $406 = (($w$1) - ($i$0$lcssa273))|0;
    $407 = ($406>>>0)>(256);
    $408 = $407 ? 256 : $406;
    _memset(($pad$i|0),32,($408|0))|0;
    $409 = ($406>>>0)>(255);
    if ($409) {
     $$01$i66 = $406;
     while(1) {
      (___fwritex($pad$i,256,$f)|0);
      $410 = (($$01$i66) + -256)|0;
      $411 = ($410>>>0)>(255);
      if ($411) {
       $$01$i66 = $410;
      } else {
       break;
      }
     }
     $412 = $406 & 255;
     $$0$lcssa$i68 = $412;
    } else {
     $$0$lcssa$i68 = $406;
    }
    (___fwritex($pad$i,$$0$lcssa$i68,$f)|0);
   }
   $413 = ($i$0$lcssa273|0)==(0);
   L465: do {
    if (!($413)) {
     $i$1186 = 0;$ws$1187 = $1182;
     while(1) {
      $414 = HEAP32[$ws$1187>>2]|0;
      $415 = ($414|0)==(0);
      if ($415) {
       break L465;
      }
      $416 = (_wctomb($mb,$414)|0);
      $417 = (($416) + ($i$1186))|0;
      $418 = ($417|0)>($i$0$lcssa273|0);
      if ($418) {
       break L465;
      }
      $419 = ((($ws$1187)) + 4|0);
      (___fwritex($mb,$416,$f)|0);
      $420 = ($417>>>0)<($i$0$lcssa273>>>0);
      if ($420) {
       $i$1186 = $417;$ws$1187 = $419;
      } else {
       break;
      }
     }
    }
   } while(0);
   $421 = ($403|0)==(8192);
   $or$cond$i71 = $421 & $405;
   if ($or$cond$i71) {
    $422 = (($w$1) - ($i$0$lcssa273))|0;
    $423 = ($422>>>0)>(256);
    $424 = $423 ? 256 : $422;
    _memset(($pad$i|0),32,($424|0))|0;
    $425 = ($422>>>0)>(255);
    if ($425) {
     $$01$i73 = $422;
     while(1) {
      (___fwritex($pad$i,256,$f)|0);
      $426 = (($$01$i73) + -256)|0;
      $427 = ($426>>>0)>(255);
      if ($427) {
       $$01$i73 = $426;
      } else {
       break;
      }
     }
     $428 = $422 & 255;
     $$0$lcssa$i75 = $428;
    } else {
     $$0$lcssa$i75 = $422;
    }
    (___fwritex($pad$i,$$0$lcssa$i75,$f)|0);
   }
   $429 = $405 ? $w$1 : $i$0$lcssa273;
   $1169 = $291;$1170 = $1181;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $429;$l10n$0 = $l10n$3;
   continue;
  }
  $1001 = $z$2;
  $1002 = $a$2;
  $1003 = (($1001) - ($1002))|0;
  $1004 = ($p$5|0)<($1003|0);
  $$p$5 = $1004 ? $1003 : $p$5;
  $1005 = (($pl$2) + ($$p$5))|0;
  $1006 = ($w$1|0)<($1005|0);
  $w$2 = $1006 ? $1005 : $w$1;
  $1007 = $fl$6 & 73728;
  $1008 = ($1007|0)==(0);
  $1009 = ($w$2|0)>($1005|0);
  $or$cond$i81 = $1008 & $1009;
  if ($or$cond$i81) {
   $1010 = (($w$2) - ($1005))|0;
   $1011 = ($1010>>>0)>(256);
   $1012 = $1011 ? 256 : $1010;
   _memset(($pad$i|0),32,($1012|0))|0;
   $1013 = ($1010>>>0)>(255);
   if ($1013) {
    $$01$i83 = $1010;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $1014 = (($$01$i83) + -256)|0;
     $1015 = ($1014>>>0)>(255);
     if ($1015) {
      $$01$i83 = $1014;
     } else {
      break;
     }
    }
    $1016 = $1010 & 255;
    $$0$lcssa$i85 = $1016;
   } else {
    $$0$lcssa$i85 = $1010;
   }
   (___fwritex($pad$i,$$0$lcssa$i85,$f)|0);
  }
  (___fwritex($prefix$2,$pl$2,$f)|0);
  $1017 = ($1007|0)==(65536);
  $or$cond$i57 = $1017 & $1009;
  if ($or$cond$i57) {
   $1018 = (($w$2) - ($1005))|0;
   $1019 = ($1018>>>0)>(256);
   $1020 = $1019 ? 256 : $1018;
   _memset(($pad$i|0),48,($1020|0))|0;
   $1021 = ($1018>>>0)>(255);
   if ($1021) {
    $$01$i59 = $1018;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $1022 = (($$01$i59) + -256)|0;
     $1023 = ($1022>>>0)>(255);
     if ($1023) {
      $$01$i59 = $1022;
     } else {
      break;
     }
    }
    $1024 = $1018 & 255;
    $$0$lcssa$i61 = $1024;
   } else {
    $$0$lcssa$i61 = $1018;
   }
   (___fwritex($pad$i,$$0$lcssa$i61,$f)|0);
  }
  $1025 = ($$p$5|0)>($1003|0);
  if ($1025) {
   $1026 = (($$p$5) - ($1003))|0;
   $1027 = ($1026>>>0)>(256);
   $1028 = $1027 ? 256 : $1026;
   _memset(($pad$i|0),48,($1028|0))|0;
   $1029 = ($1026>>>0)>(255);
   if ($1029) {
    $$01$i44 = $1026;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $1030 = (($$01$i44) + -256)|0;
     $1031 = ($1030>>>0)>(255);
     if ($1031) {
      $$01$i44 = $1030;
     } else {
      break;
     }
    }
    $1032 = $1026 & 255;
    $$0$lcssa$i46 = $1032;
   } else {
    $$0$lcssa$i46 = $1026;
   }
   (___fwritex($pad$i,$$0$lcssa$i46,$f)|0);
  }
  (___fwritex($a$2,$1003,$f)|0);
  $1033 = ($1007|0)==(8192);
  $or$cond$i = $1033 & $1009;
  if ($or$cond$i) {
   $1034 = (($w$2) - ($1005))|0;
   $1035 = ($1034>>>0)>(256);
   $1036 = $1035 ? 256 : $1034;
   _memset(($pad$i|0),32,($1036|0))|0;
   $1037 = ($1034>>>0)>(255);
   if ($1037) {
    $$01$i = $1034;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $1038 = (($$01$i) + -256)|0;
     $1039 = ($1038>>>0)>(255);
     if ($1039) {
      $$01$i = $1038;
     } else {
      break;
     }
    }
    $1040 = $1034 & 255;
    $$0$lcssa$i = $1040;
   } else {
    $$0$lcssa$i = $1034;
   }
   (___fwritex($pad$i,$$0$lcssa$i,$f)|0);
  }
  $1169 = $1175;$1170 = $1176;$23 = $$lcssa457;$cnt$0 = $cnt$1;$l$0 = $w$2;$l10n$0 = $l10n$3;
 }
 if ((label|0) == 344) {
  $1041 = ($f|0)==(0|0);
  if (!($1041)) {
   $$0 = $cnt$1$lcssa;
   STACKTOP = sp;return ($$0|0);
  }
  $1042 = ($l10n$0$lcssa|0)==(0);
  if ($1042) {
   $$0 = 0;
   STACKTOP = sp;return ($$0|0);
  } else {
   $i$2162 = 1;
  }
  while(1) {
   $1043 = (($nl_type) + ($i$2162<<2)|0);
   $1044 = HEAP32[$1043>>2]|0;
   $1045 = ($1044|0)==(0);
   if ($1045) {
    $i$2162$lcssa = $i$2162;
    break;
   }
   $1047 = (($nl_arg) + ($i$2162<<3)|0);
   $1048 = ($1044>>>0)>(20);
   L522: do {
    if (!($1048)) {
     do {
      switch ($1044|0) {
      case 9:  {
       $arglist_current35 = HEAP32[$ap>>2]|0;
       $1049 = $arglist_current35;
       $1050 = ((0) + 4|0);
       $expanded148 = $1050;
       $expanded147 = (($expanded148) - 1)|0;
       $1051 = (($1049) + ($expanded147))|0;
       $1052 = ((0) + 4|0);
       $expanded152 = $1052;
       $expanded151 = (($expanded152) - 1)|0;
       $expanded150 = $expanded151 ^ -1;
       $1053 = $1051 & $expanded150;
       $1054 = $1053;
       $1055 = HEAP32[$1054>>2]|0;
       $arglist_next36 = ((($1054)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next36;
       HEAP32[$1047>>2] = $1055;
       break L522;
       break;
      }
      case 10:  {
       $arglist_current38 = HEAP32[$ap>>2]|0;
       $1056 = $arglist_current38;
       $1057 = ((0) + 4|0);
       $expanded155 = $1057;
       $expanded154 = (($expanded155) - 1)|0;
       $1058 = (($1056) + ($expanded154))|0;
       $1059 = ((0) + 4|0);
       $expanded159 = $1059;
       $expanded158 = (($expanded159) - 1)|0;
       $expanded157 = $expanded158 ^ -1;
       $1060 = $1058 & $expanded157;
       $1061 = $1060;
       $1062 = HEAP32[$1061>>2]|0;
       $arglist_next39 = ((($1061)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next39;
       $1063 = ($1062|0)<(0);
       $1064 = $1063 << 31 >> 31;
       $1065 = $1047;
       $1066 = $1065;
       HEAP32[$1066>>2] = $1062;
       $1067 = (($1065) + 4)|0;
       $1068 = $1067;
       HEAP32[$1068>>2] = $1064;
       break L522;
       break;
      }
      case 11:  {
       $arglist_current41 = HEAP32[$ap>>2]|0;
       $1069 = $arglist_current41;
       $1070 = ((0) + 4|0);
       $expanded162 = $1070;
       $expanded161 = (($expanded162) - 1)|0;
       $1071 = (($1069) + ($expanded161))|0;
       $1072 = ((0) + 4|0);
       $expanded166 = $1072;
       $expanded165 = (($expanded166) - 1)|0;
       $expanded164 = $expanded165 ^ -1;
       $1073 = $1071 & $expanded164;
       $1074 = $1073;
       $1075 = HEAP32[$1074>>2]|0;
       $arglist_next42 = ((($1074)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next42;
       $1076 = $1047;
       $1077 = $1076;
       HEAP32[$1077>>2] = $1075;
       $1078 = (($1076) + 4)|0;
       $1079 = $1078;
       HEAP32[$1079>>2] = 0;
       break L522;
       break;
      }
      case 12:  {
       $arglist_current44 = HEAP32[$ap>>2]|0;
       $1080 = $arglist_current44;
       $1081 = ((0) + 8|0);
       $expanded169 = $1081;
       $expanded168 = (($expanded169) - 1)|0;
       $1082 = (($1080) + ($expanded168))|0;
       $1083 = ((0) + 8|0);
       $expanded173 = $1083;
       $expanded172 = (($expanded173) - 1)|0;
       $expanded171 = $expanded172 ^ -1;
       $1084 = $1082 & $expanded171;
       $1085 = $1084;
       $1086 = $1085;
       $1087 = $1086;
       $1088 = HEAP32[$1087>>2]|0;
       $1089 = (($1086) + 4)|0;
       $1090 = $1089;
       $1091 = HEAP32[$1090>>2]|0;
       $arglist_next45 = ((($1085)) + 8|0);
       HEAP32[$ap>>2] = $arglist_next45;
       $1092 = $1047;
       $1093 = $1092;
       HEAP32[$1093>>2] = $1088;
       $1094 = (($1092) + 4)|0;
       $1095 = $1094;
       HEAP32[$1095>>2] = $1091;
       break L522;
       break;
      }
      case 13:  {
       $arglist_current47 = HEAP32[$ap>>2]|0;
       $1096 = $arglist_current47;
       $1097 = ((0) + 4|0);
       $expanded176 = $1097;
       $expanded175 = (($expanded176) - 1)|0;
       $1098 = (($1096) + ($expanded175))|0;
       $1099 = ((0) + 4|0);
       $expanded180 = $1099;
       $expanded179 = (($expanded180) - 1)|0;
       $expanded178 = $expanded179 ^ -1;
       $1100 = $1098 & $expanded178;
       $1101 = $1100;
       $1102 = HEAP32[$1101>>2]|0;
       $arglist_next48 = ((($1101)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next48;
       $1103 = $1102&65535;
       $1104 = $1103 << 16 >> 16;
       $1105 = ($1104|0)<(0);
       $1106 = $1105 << 31 >> 31;
       $1107 = $1047;
       $1108 = $1107;
       HEAP32[$1108>>2] = $1104;
       $1109 = (($1107) + 4)|0;
       $1110 = $1109;
       HEAP32[$1110>>2] = $1106;
       break L522;
       break;
      }
      case 14:  {
       $arglist_current50 = HEAP32[$ap>>2]|0;
       $1111 = $arglist_current50;
       $1112 = ((0) + 4|0);
       $expanded183 = $1112;
       $expanded182 = (($expanded183) - 1)|0;
       $1113 = (($1111) + ($expanded182))|0;
       $1114 = ((0) + 4|0);
       $expanded187 = $1114;
       $expanded186 = (($expanded187) - 1)|0;
       $expanded185 = $expanded186 ^ -1;
       $1115 = $1113 & $expanded185;
       $1116 = $1115;
       $1117 = HEAP32[$1116>>2]|0;
       $arglist_next51 = ((($1116)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next51;
       $$mask1$i = $1117 & 65535;
       $1118 = $1047;
       $1119 = $1118;
       HEAP32[$1119>>2] = $$mask1$i;
       $1120 = (($1118) + 4)|0;
       $1121 = $1120;
       HEAP32[$1121>>2] = 0;
       break L522;
       break;
      }
      case 15:  {
       $arglist_current53 = HEAP32[$ap>>2]|0;
       $1122 = $arglist_current53;
       $1123 = ((0) + 4|0);
       $expanded190 = $1123;
       $expanded189 = (($expanded190) - 1)|0;
       $1124 = (($1122) + ($expanded189))|0;
       $1125 = ((0) + 4|0);
       $expanded194 = $1125;
       $expanded193 = (($expanded194) - 1)|0;
       $expanded192 = $expanded193 ^ -1;
       $1126 = $1124 & $expanded192;
       $1127 = $1126;
       $1128 = HEAP32[$1127>>2]|0;
       $arglist_next54 = ((($1127)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next54;
       $1129 = $1128&255;
       $1130 = $1129 << 24 >> 24;
       $1131 = ($1130|0)<(0);
       $1132 = $1131 << 31 >> 31;
       $1133 = $1047;
       $1134 = $1133;
       HEAP32[$1134>>2] = $1130;
       $1135 = (($1133) + 4)|0;
       $1136 = $1135;
       HEAP32[$1136>>2] = $1132;
       break L522;
       break;
      }
      case 16:  {
       $arglist_current56 = HEAP32[$ap>>2]|0;
       $1137 = $arglist_current56;
       $1138 = ((0) + 4|0);
       $expanded197 = $1138;
       $expanded196 = (($expanded197) - 1)|0;
       $1139 = (($1137) + ($expanded196))|0;
       $1140 = ((0) + 4|0);
       $expanded201 = $1140;
       $expanded200 = (($expanded201) - 1)|0;
       $expanded199 = $expanded200 ^ -1;
       $1141 = $1139 & $expanded199;
       $1142 = $1141;
       $1143 = HEAP32[$1142>>2]|0;
       $arglist_next57 = ((($1142)) + 4|0);
       HEAP32[$ap>>2] = $arglist_next57;
       $$mask$i = $1143 & 255;
       $1144 = $1047;
       $1145 = $1144;
       HEAP32[$1145>>2] = $$mask$i;
       $1146 = (($1144) + 4)|0;
       $1147 = $1146;
       HEAP32[$1147>>2] = 0;
       break L522;
       break;
      }
      case 17:  {
       $arglist_current59 = HEAP32[$ap>>2]|0;
       $1148 = $arglist_current59;
       $1149 = ((0) + 8|0);
       $expanded204 = $1149;
       $expanded203 = (($expanded204) - 1)|0;
       $1150 = (($1148) + ($expanded203))|0;
       $1151 = ((0) + 8|0);
       $expanded208 = $1151;
       $expanded207 = (($expanded208) - 1)|0;
       $expanded206 = $expanded207 ^ -1;
       $1152 = $1150 & $expanded206;
       $1153 = $1152;
       $1154 = +HEAPF64[$1153>>3];
       $arglist_next60 = ((($1153)) + 8|0);
       HEAP32[$ap>>2] = $arglist_next60;
       HEAPF64[$1047>>3] = $1154;
       break L522;
       break;
      }
      case 18:  {
       $arglist_current62 = HEAP32[$ap>>2]|0;
       $1155 = $arglist_current62;
       $1156 = ((0) + 8|0);
       $expanded211 = $1156;
       $expanded210 = (($expanded211) - 1)|0;
       $1157 = (($1155) + ($expanded210))|0;
       $1158 = ((0) + 8|0);
       $expanded215 = $1158;
       $expanded214 = (($expanded215) - 1)|0;
       $expanded213 = $expanded214 ^ -1;
       $1159 = $1157 & $expanded213;
       $1160 = $1159;
       $1161 = +HEAPF64[$1160>>3];
       $arglist_next63 = ((($1160)) + 8|0);
       HEAP32[$ap>>2] = $arglist_next63;
       HEAPF64[$1047>>3] = $1161;
       break L522;
       break;
      }
      default: {
       break L522;
      }
      }
     } while(0);
    }
   } while(0);
   $1162 = (($i$2162) + 1)|0;
   $1163 = ($1162|0)<(10);
   if ($1163) {
    $i$2162 = $1162;
   } else {
    $$0 = 1;
    label = 363;
    break;
   }
  }
  if ((label|0) == 363) {
   STACKTOP = sp;return ($$0|0);
  }
  $1046 = ($i$2162$lcssa|0)<(10);
  if ($1046) {
   $i$3160 = $i$2162$lcssa;
  } else {
   $$0 = 1;
   STACKTOP = sp;return ($$0|0);
  }
  while(1) {
   $1166 = (($nl_type) + ($i$3160<<2)|0);
   $1167 = HEAP32[$1166>>2]|0;
   $1168 = ($1167|0)==(0);
   $1164 = (($i$3160) + 1)|0;
   if (!($1168)) {
    $$0 = -1;
    label = 363;
    break;
   }
   $1165 = ($1164|0)<(10);
   if ($1165) {
    $i$3160 = $1164;
   } else {
    $$0 = 1;
    label = 363;
    break;
   }
  }
  if ((label|0) == 363) {
   STACKTOP = sp;return ($$0|0);
  }
 }
 else if ((label|0) == 363) {
  STACKTOP = sp;return ($$0|0);
 }
 return (0)|0;
}
function _sn_write($f,$s,$l) {
 $f = $f|0;
 $s = $s|0;
 $l = $l|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($f)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $1;
 $5 = $3;
 $6 = (($4) - ($5))|0;
 $7 = ($6>>>0)>($l>>>0);
 $l$ = $7 ? $l : $6;
 _memcpy(($3|0),($s|0),($l$|0))|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + ($l$)|0);
 HEAP32[$2>>2] = $9;
 return ($l|0);
}
function _malloc($bytes) {
 $bytes = $bytes|0;
 var $$3$i = 0, $$lcssa = 0, $$lcssa211 = 0, $$lcssa215 = 0, $$lcssa216 = 0, $$lcssa217 = 0, $$lcssa219 = 0, $$lcssa222 = 0, $$lcssa224 = 0, $$lcssa226 = 0, $$lcssa228 = 0, $$lcssa230 = 0, $$lcssa232 = 0, $$pre = 0, $$pre$i = 0, $$pre$i$i = 0, $$pre$i22$i = 0, $$pre$i25 = 0, $$pre$phi$i$iZ2D = 0, $$pre$phi$i23$iZ2D = 0;
 var $$pre$phi$i26Z2D = 0, $$pre$phi$iZ2D = 0, $$pre$phi58$i$iZ2D = 0, $$pre$phiZ2D = 0, $$pre105 = 0, $$pre106 = 0, $$pre14$i$i = 0, $$pre43$i = 0, $$pre56$i$i = 0, $$pre57$i$i = 0, $$pre8$i = 0, $$rsize$0$i = 0, $$rsize$3$i = 0, $$sum = 0, $$sum$i$i = 0, $$sum$i$i$i = 0, $$sum$i13$i = 0, $$sum$i14$i = 0, $$sum$i17$i = 0, $$sum$i19$i = 0;
 var $$sum$i2334 = 0, $$sum$i32 = 0, $$sum$i35 = 0, $$sum1 = 0, $$sum1$i = 0, $$sum1$i$i = 0, $$sum1$i15$i = 0, $$sum1$i20$i = 0, $$sum1$i24 = 0, $$sum10 = 0, $$sum10$i = 0, $$sum10$i$i = 0, $$sum11$i = 0, $$sum11$i$i = 0, $$sum1112 = 0, $$sum112$i = 0, $$sum113$i = 0, $$sum114$i = 0, $$sum115$i = 0, $$sum116$i = 0;
 var $$sum117$i = 0, $$sum118$i = 0, $$sum119$i = 0, $$sum12$i = 0, $$sum12$i$i = 0, $$sum120$i = 0, $$sum121$i = 0, $$sum122$i = 0, $$sum123$i = 0, $$sum124$i = 0, $$sum125$i = 0, $$sum13$i = 0, $$sum13$i$i = 0, $$sum14$i$i = 0, $$sum15$i = 0, $$sum15$i$i = 0, $$sum16$i = 0, $$sum16$i$i = 0, $$sum17$i = 0, $$sum17$i$i = 0;
 var $$sum18$i = 0, $$sum1819$i$i = 0, $$sum2 = 0, $$sum2$i = 0, $$sum2$i$i = 0, $$sum2$i$i$i = 0, $$sum2$i16$i = 0, $$sum2$i18$i = 0, $$sum2$i21$i = 0, $$sum20$i$i = 0, $$sum21$i$i = 0, $$sum22$i$i = 0, $$sum23$i$i = 0, $$sum24$i$i = 0, $$sum25$i$i = 0, $$sum27$i$i = 0, $$sum28$i$i = 0, $$sum29$i$i = 0, $$sum3$i = 0, $$sum3$i27 = 0;
 var $$sum30$i$i = 0, $$sum3132$i$i = 0, $$sum34$i$i = 0, $$sum3536$i$i = 0, $$sum3738$i$i = 0, $$sum39$i$i = 0, $$sum4 = 0, $$sum4$i = 0, $$sum4$i$i = 0, $$sum4$i28 = 0, $$sum40$i$i = 0, $$sum41$i$i = 0, $$sum42$i$i = 0, $$sum5$i = 0, $$sum5$i$i = 0, $$sum56 = 0, $$sum6$i = 0, $$sum67$i$i = 0, $$sum7$i = 0, $$sum8$i = 0;
 var $$sum9 = 0, $$sum9$i = 0, $$sum9$i$i = 0, $$tsize$1$i = 0, $$v$0$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0;
 var $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0;
 var $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0;
 var $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0;
 var $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0;
 var $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0;
 var $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0;
 var $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0;
 var $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0;
 var $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0;
 var $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0;
 var $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0;
 var $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0;
 var $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0;
 var $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0;
 var $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0;
 var $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0;
 var $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0;
 var $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0;
 var $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0;
 var $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0;
 var $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0;
 var $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0;
 var $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0;
 var $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0;
 var $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0;
 var $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0;
 var $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0;
 var $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0;
 var $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0;
 var $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0;
 var $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0;
 var $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0;
 var $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0;
 var $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0;
 var $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0;
 var $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0;
 var $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0;
 var $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0;
 var $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0;
 var $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0;
 var $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0;
 var $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0;
 var $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0;
 var $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0;
 var $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0;
 var $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0;
 var $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0;
 var $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0;
 var $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0;
 var $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0;
 var $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0;
 var $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0;
 var $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $F$0$i$i = 0, $F1$0$i = 0, $F4$0 = 0, $F4$0$i$i = 0;
 var $F5$0$i = 0, $I1$0$i$i = 0, $I7$0$i = 0, $I7$0$i$i = 0, $K12$029$i = 0, $K2$07$i$i = 0, $K8$051$i$i = 0, $R$0$i = 0, $R$0$i$i = 0, $R$0$i$i$lcssa = 0, $R$0$i$lcssa = 0, $R$0$i18 = 0, $R$0$i18$lcssa = 0, $R$1$i = 0, $R$1$i$i = 0, $R$1$i20 = 0, $RP$0$i = 0, $RP$0$i$i = 0, $RP$0$i$i$lcssa = 0, $RP$0$i$lcssa = 0;
 var $RP$0$i17 = 0, $RP$0$i17$lcssa = 0, $T$0$lcssa$i = 0, $T$0$lcssa$i$i = 0, $T$0$lcssa$i25$i = 0, $T$028$i = 0, $T$028$i$lcssa = 0, $T$050$i$i = 0, $T$050$i$i$lcssa = 0, $T$06$i$i = 0, $T$06$i$i$lcssa = 0, $br$0$ph$i = 0, $cond$i = 0, $cond$i$i = 0, $cond$i21 = 0, $exitcond$i$i = 0, $i$02$i$i = 0, $idx$0$i = 0, $mem$0 = 0, $nb$0 = 0;
 var $not$$i = 0, $not$$i$i = 0, $not$$i26$i = 0, $oldfirst$0$i$i = 0, $or$cond$i = 0, $or$cond$i30 = 0, $or$cond1$i = 0, $or$cond19$i = 0, $or$cond2$i = 0, $or$cond3$i = 0, $or$cond5$i = 0, $or$cond57$i = 0, $or$cond6$i = 0, $or$cond8$i = 0, $or$cond9$i = 0, $qsize$0$i$i = 0, $rsize$0$i = 0, $rsize$0$i$lcssa = 0, $rsize$0$i15 = 0, $rsize$1$i = 0;
 var $rsize$2$i = 0, $rsize$3$lcssa$i = 0, $rsize$331$i = 0, $rst$0$i = 0, $rst$1$i = 0, $sizebits$0$i = 0, $sp$0$i$i = 0, $sp$0$i$i$i = 0, $sp$084$i = 0, $sp$084$i$lcssa = 0, $sp$183$i = 0, $sp$183$i$lcssa = 0, $ssize$0$$i = 0, $ssize$0$i = 0, $ssize$1$ph$i = 0, $ssize$2$i = 0, $t$0$i = 0, $t$0$i14 = 0, $t$1$i = 0, $t$2$ph$i = 0;
 var $t$2$v$3$i = 0, $t$230$i = 0, $tbase$255$i = 0, $tsize$0$ph$i = 0, $tsize$0323944$i = 0, $tsize$1$i = 0, $tsize$254$i = 0, $v$0$i = 0, $v$0$i$lcssa = 0, $v$0$i16 = 0, $v$1$i = 0, $v$2$i = 0, $v$3$lcssa$i = 0, $v$3$ph$i = 0, $v$332$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($bytes>>>0)<(245);
 do {
  if ($0) {
   $1 = ($bytes>>>0)<(11);
   $2 = (($bytes) + 11)|0;
   $3 = $2 & -8;
   $4 = $1 ? 16 : $3;
   $5 = $4 >>> 3;
   $6 = HEAP32[4192>>2]|0;
   $7 = $6 >>> $5;
   $8 = $7 & 3;
   $9 = ($8|0)==(0);
   if (!($9)) {
    $10 = $7 & 1;
    $11 = $10 ^ 1;
    $12 = (($11) + ($5))|0;
    $13 = $12 << 1;
    $14 = (4232 + ($13<<2)|0);
    $$sum10 = (($13) + 2)|0;
    $15 = (4232 + ($$sum10<<2)|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ((($16)) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($14|0)==($18|0);
    do {
     if ($19) {
      $20 = 1 << $12;
      $21 = $20 ^ -1;
      $22 = $6 & $21;
      HEAP32[4192>>2] = $22;
     } else {
      $23 = HEAP32[(4208)>>2]|0;
      $24 = ($18>>>0)<($23>>>0);
      if ($24) {
       _abort();
       // unreachable;
      }
      $25 = ((($18)) + 12|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==($16|0);
      if ($27) {
       HEAP32[$25>>2] = $14;
       HEAP32[$15>>2] = $18;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $28 = $12 << 3;
    $29 = $28 | 3;
    $30 = ((($16)) + 4|0);
    HEAP32[$30>>2] = $29;
    $$sum1112 = $28 | 4;
    $31 = (($16) + ($$sum1112)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 1;
    HEAP32[$31>>2] = $33;
    $mem$0 = $17;
    return ($mem$0|0);
   }
   $34 = HEAP32[(4200)>>2]|0;
   $35 = ($4>>>0)>($34>>>0);
   if ($35) {
    $36 = ($7|0)==(0);
    if (!($36)) {
     $37 = $7 << $5;
     $38 = 2 << $5;
     $39 = (0 - ($38))|0;
     $40 = $38 | $39;
     $41 = $37 & $40;
     $42 = (0 - ($41))|0;
     $43 = $41 & $42;
     $44 = (($43) + -1)|0;
     $45 = $44 >>> 12;
     $46 = $45 & 16;
     $47 = $44 >>> $46;
     $48 = $47 >>> 5;
     $49 = $48 & 8;
     $50 = $49 | $46;
     $51 = $47 >>> $49;
     $52 = $51 >>> 2;
     $53 = $52 & 4;
     $54 = $50 | $53;
     $55 = $51 >>> $53;
     $56 = $55 >>> 1;
     $57 = $56 & 2;
     $58 = $54 | $57;
     $59 = $55 >>> $57;
     $60 = $59 >>> 1;
     $61 = $60 & 1;
     $62 = $58 | $61;
     $63 = $59 >>> $61;
     $64 = (($62) + ($63))|0;
     $65 = $64 << 1;
     $66 = (4232 + ($65<<2)|0);
     $$sum4 = (($65) + 2)|0;
     $67 = (4232 + ($$sum4<<2)|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = ((($68)) + 8|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($66|0)==($70|0);
     do {
      if ($71) {
       $72 = 1 << $64;
       $73 = $72 ^ -1;
       $74 = $6 & $73;
       HEAP32[4192>>2] = $74;
       $88 = $34;
      } else {
       $75 = HEAP32[(4208)>>2]|0;
       $76 = ($70>>>0)<($75>>>0);
       if ($76) {
        _abort();
        // unreachable;
       }
       $77 = ((($70)) + 12|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = ($78|0)==($68|0);
       if ($79) {
        HEAP32[$77>>2] = $66;
        HEAP32[$67>>2] = $70;
        $$pre = HEAP32[(4200)>>2]|0;
        $88 = $$pre;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $80 = $64 << 3;
     $81 = (($80) - ($4))|0;
     $82 = $4 | 3;
     $83 = ((($68)) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = (($68) + ($4)|0);
     $85 = $81 | 1;
     $$sum56 = $4 | 4;
     $86 = (($68) + ($$sum56)|0);
     HEAP32[$86>>2] = $85;
     $87 = (($68) + ($80)|0);
     HEAP32[$87>>2] = $81;
     $89 = ($88|0)==(0);
     if (!($89)) {
      $90 = HEAP32[(4212)>>2]|0;
      $91 = $88 >>> 3;
      $92 = $91 << 1;
      $93 = (4232 + ($92<<2)|0);
      $94 = HEAP32[4192>>2]|0;
      $95 = 1 << $91;
      $96 = $94 & $95;
      $97 = ($96|0)==(0);
      if ($97) {
       $98 = $94 | $95;
       HEAP32[4192>>2] = $98;
       $$pre105 = (($92) + 2)|0;
       $$pre106 = (4232 + ($$pre105<<2)|0);
       $$pre$phiZ2D = $$pre106;$F4$0 = $93;
      } else {
       $$sum9 = (($92) + 2)|0;
       $99 = (4232 + ($$sum9<<2)|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = HEAP32[(4208)>>2]|0;
       $102 = ($100>>>0)<($101>>>0);
       if ($102) {
        _abort();
        // unreachable;
       } else {
        $$pre$phiZ2D = $99;$F4$0 = $100;
       }
      }
      HEAP32[$$pre$phiZ2D>>2] = $90;
      $103 = ((($F4$0)) + 12|0);
      HEAP32[$103>>2] = $90;
      $104 = ((($90)) + 8|0);
      HEAP32[$104>>2] = $F4$0;
      $105 = ((($90)) + 12|0);
      HEAP32[$105>>2] = $93;
     }
     HEAP32[(4200)>>2] = $81;
     HEAP32[(4212)>>2] = $84;
     $mem$0 = $69;
     return ($mem$0|0);
    }
    $106 = HEAP32[(4196)>>2]|0;
    $107 = ($106|0)==(0);
    if ($107) {
     $nb$0 = $4;
    } else {
     $108 = (0 - ($106))|0;
     $109 = $106 & $108;
     $110 = (($109) + -1)|0;
     $111 = $110 >>> 12;
     $112 = $111 & 16;
     $113 = $110 >>> $112;
     $114 = $113 >>> 5;
     $115 = $114 & 8;
     $116 = $115 | $112;
     $117 = $113 >>> $115;
     $118 = $117 >>> 2;
     $119 = $118 & 4;
     $120 = $116 | $119;
     $121 = $117 >>> $119;
     $122 = $121 >>> 1;
     $123 = $122 & 2;
     $124 = $120 | $123;
     $125 = $121 >>> $123;
     $126 = $125 >>> 1;
     $127 = $126 & 1;
     $128 = $124 | $127;
     $129 = $125 >>> $127;
     $130 = (($128) + ($129))|0;
     $131 = (4496 + ($130<<2)|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = ((($132)) + 4|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = $134 & -8;
     $136 = (($135) - ($4))|0;
     $rsize$0$i = $136;$t$0$i = $132;$v$0$i = $132;
     while(1) {
      $137 = ((($t$0$i)) + 16|0);
      $138 = HEAP32[$137>>2]|0;
      $139 = ($138|0)==(0|0);
      if ($139) {
       $140 = ((($t$0$i)) + 20|0);
       $141 = HEAP32[$140>>2]|0;
       $142 = ($141|0)==(0|0);
       if ($142) {
        $rsize$0$i$lcssa = $rsize$0$i;$v$0$i$lcssa = $v$0$i;
        break;
       } else {
        $144 = $141;
       }
      } else {
       $144 = $138;
      }
      $143 = ((($144)) + 4|0);
      $145 = HEAP32[$143>>2]|0;
      $146 = $145 & -8;
      $147 = (($146) - ($4))|0;
      $148 = ($147>>>0)<($rsize$0$i>>>0);
      $$rsize$0$i = $148 ? $147 : $rsize$0$i;
      $$v$0$i = $148 ? $144 : $v$0$i;
      $rsize$0$i = $$rsize$0$i;$t$0$i = $144;$v$0$i = $$v$0$i;
     }
     $149 = HEAP32[(4208)>>2]|0;
     $150 = ($v$0$i$lcssa>>>0)<($149>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($v$0$i$lcssa) + ($4)|0);
     $152 = ($v$0$i$lcssa>>>0)<($151>>>0);
     if (!($152)) {
      _abort();
      // unreachable;
     }
     $153 = ((($v$0$i$lcssa)) + 24|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = ((($v$0$i$lcssa)) + 12|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($156|0)==($v$0$i$lcssa|0);
     do {
      if ($157) {
       $167 = ((($v$0$i$lcssa)) + 20|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = ($168|0)==(0|0);
       if ($169) {
        $170 = ((($v$0$i$lcssa)) + 16|0);
        $171 = HEAP32[$170>>2]|0;
        $172 = ($171|0)==(0|0);
        if ($172) {
         $R$1$i = 0;
         break;
        } else {
         $R$0$i = $171;$RP$0$i = $170;
        }
       } else {
        $R$0$i = $168;$RP$0$i = $167;
       }
       while(1) {
        $173 = ((($R$0$i)) + 20|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = ($174|0)==(0|0);
        if (!($175)) {
         $R$0$i = $174;$RP$0$i = $173;
         continue;
        }
        $176 = ((($R$0$i)) + 16|0);
        $177 = HEAP32[$176>>2]|0;
        $178 = ($177|0)==(0|0);
        if ($178) {
         $R$0$i$lcssa = $R$0$i;$RP$0$i$lcssa = $RP$0$i;
         break;
        } else {
         $R$0$i = $177;$RP$0$i = $176;
        }
       }
       $179 = ($RP$0$i$lcssa>>>0)<($149>>>0);
       if ($179) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$RP$0$i$lcssa>>2] = 0;
        $R$1$i = $R$0$i$lcssa;
        break;
       }
      } else {
       $158 = ((($v$0$i$lcssa)) + 8|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = ($159>>>0)<($149>>>0);
       if ($160) {
        _abort();
        // unreachable;
       }
       $161 = ((($159)) + 12|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = ($162|0)==($v$0$i$lcssa|0);
       if (!($163)) {
        _abort();
        // unreachable;
       }
       $164 = ((($156)) + 8|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = ($165|0)==($v$0$i$lcssa|0);
       if ($166) {
        HEAP32[$161>>2] = $156;
        HEAP32[$164>>2] = $159;
        $R$1$i = $156;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $180 = ($154|0)==(0|0);
     do {
      if (!($180)) {
       $181 = ((($v$0$i$lcssa)) + 28|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = (4496 + ($182<<2)|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($v$0$i$lcssa|0)==($184|0);
       if ($185) {
        HEAP32[$183>>2] = $R$1$i;
        $cond$i = ($R$1$i|0)==(0|0);
        if ($cond$i) {
         $186 = 1 << $182;
         $187 = $186 ^ -1;
         $188 = HEAP32[(4196)>>2]|0;
         $189 = $188 & $187;
         HEAP32[(4196)>>2] = $189;
         break;
        }
       } else {
        $190 = HEAP32[(4208)>>2]|0;
        $191 = ($154>>>0)<($190>>>0);
        if ($191) {
         _abort();
         // unreachable;
        }
        $192 = ((($154)) + 16|0);
        $193 = HEAP32[$192>>2]|0;
        $194 = ($193|0)==($v$0$i$lcssa|0);
        if ($194) {
         HEAP32[$192>>2] = $R$1$i;
        } else {
         $195 = ((($154)) + 20|0);
         HEAP32[$195>>2] = $R$1$i;
        }
        $196 = ($R$1$i|0)==(0|0);
        if ($196) {
         break;
        }
       }
       $197 = HEAP32[(4208)>>2]|0;
       $198 = ($R$1$i>>>0)<($197>>>0);
       if ($198) {
        _abort();
        // unreachable;
       }
       $199 = ((($R$1$i)) + 24|0);
       HEAP32[$199>>2] = $154;
       $200 = ((($v$0$i$lcssa)) + 16|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)==(0|0);
       do {
        if (!($202)) {
         $203 = ($201>>>0)<($197>>>0);
         if ($203) {
          _abort();
          // unreachable;
         } else {
          $204 = ((($R$1$i)) + 16|0);
          HEAP32[$204>>2] = $201;
          $205 = ((($201)) + 24|0);
          HEAP32[$205>>2] = $R$1$i;
          break;
         }
        }
       } while(0);
       $206 = ((($v$0$i$lcssa)) + 20|0);
       $207 = HEAP32[$206>>2]|0;
       $208 = ($207|0)==(0|0);
       if (!($208)) {
        $209 = HEAP32[(4208)>>2]|0;
        $210 = ($207>>>0)<($209>>>0);
        if ($210) {
         _abort();
         // unreachable;
        } else {
         $211 = ((($R$1$i)) + 20|0);
         HEAP32[$211>>2] = $207;
         $212 = ((($207)) + 24|0);
         HEAP32[$212>>2] = $R$1$i;
         break;
        }
       }
      }
     } while(0);
     $213 = ($rsize$0$i$lcssa>>>0)<(16);
     if ($213) {
      $214 = (($rsize$0$i$lcssa) + ($4))|0;
      $215 = $214 | 3;
      $216 = ((($v$0$i$lcssa)) + 4|0);
      HEAP32[$216>>2] = $215;
      $$sum4$i = (($214) + 4)|0;
      $217 = (($v$0$i$lcssa) + ($$sum4$i)|0);
      $218 = HEAP32[$217>>2]|0;
      $219 = $218 | 1;
      HEAP32[$217>>2] = $219;
     } else {
      $220 = $4 | 3;
      $221 = ((($v$0$i$lcssa)) + 4|0);
      HEAP32[$221>>2] = $220;
      $222 = $rsize$0$i$lcssa | 1;
      $$sum$i35 = $4 | 4;
      $223 = (($v$0$i$lcssa) + ($$sum$i35)|0);
      HEAP32[$223>>2] = $222;
      $$sum1$i = (($rsize$0$i$lcssa) + ($4))|0;
      $224 = (($v$0$i$lcssa) + ($$sum1$i)|0);
      HEAP32[$224>>2] = $rsize$0$i$lcssa;
      $225 = HEAP32[(4200)>>2]|0;
      $226 = ($225|0)==(0);
      if (!($226)) {
       $227 = HEAP32[(4212)>>2]|0;
       $228 = $225 >>> 3;
       $229 = $228 << 1;
       $230 = (4232 + ($229<<2)|0);
       $231 = HEAP32[4192>>2]|0;
       $232 = 1 << $228;
       $233 = $231 & $232;
       $234 = ($233|0)==(0);
       if ($234) {
        $235 = $231 | $232;
        HEAP32[4192>>2] = $235;
        $$pre$i = (($229) + 2)|0;
        $$pre8$i = (4232 + ($$pre$i<<2)|0);
        $$pre$phi$iZ2D = $$pre8$i;$F1$0$i = $230;
       } else {
        $$sum3$i = (($229) + 2)|0;
        $236 = (4232 + ($$sum3$i<<2)|0);
        $237 = HEAP32[$236>>2]|0;
        $238 = HEAP32[(4208)>>2]|0;
        $239 = ($237>>>0)<($238>>>0);
        if ($239) {
         _abort();
         // unreachable;
        } else {
         $$pre$phi$iZ2D = $236;$F1$0$i = $237;
        }
       }
       HEAP32[$$pre$phi$iZ2D>>2] = $227;
       $240 = ((($F1$0$i)) + 12|0);
       HEAP32[$240>>2] = $227;
       $241 = ((($227)) + 8|0);
       HEAP32[$241>>2] = $F1$0$i;
       $242 = ((($227)) + 12|0);
       HEAP32[$242>>2] = $230;
      }
      HEAP32[(4200)>>2] = $rsize$0$i$lcssa;
      HEAP32[(4212)>>2] = $151;
     }
     $243 = ((($v$0$i$lcssa)) + 8|0);
     $mem$0 = $243;
     return ($mem$0|0);
    }
   } else {
    $nb$0 = $4;
   }
  } else {
   $244 = ($bytes>>>0)>(4294967231);
   if ($244) {
    $nb$0 = -1;
   } else {
    $245 = (($bytes) + 11)|0;
    $246 = $245 & -8;
    $247 = HEAP32[(4196)>>2]|0;
    $248 = ($247|0)==(0);
    if ($248) {
     $nb$0 = $246;
    } else {
     $249 = (0 - ($246))|0;
     $250 = $245 >>> 8;
     $251 = ($250|0)==(0);
     if ($251) {
      $idx$0$i = 0;
     } else {
      $252 = ($246>>>0)>(16777215);
      if ($252) {
       $idx$0$i = 31;
      } else {
       $253 = (($250) + 1048320)|0;
       $254 = $253 >>> 16;
       $255 = $254 & 8;
       $256 = $250 << $255;
       $257 = (($256) + 520192)|0;
       $258 = $257 >>> 16;
       $259 = $258 & 4;
       $260 = $259 | $255;
       $261 = $256 << $259;
       $262 = (($261) + 245760)|0;
       $263 = $262 >>> 16;
       $264 = $263 & 2;
       $265 = $260 | $264;
       $266 = (14 - ($265))|0;
       $267 = $261 << $264;
       $268 = $267 >>> 15;
       $269 = (($266) + ($268))|0;
       $270 = $269 << 1;
       $271 = (($269) + 7)|0;
       $272 = $246 >>> $271;
       $273 = $272 & 1;
       $274 = $273 | $270;
       $idx$0$i = $274;
      }
     }
     $275 = (4496 + ($idx$0$i<<2)|0);
     $276 = HEAP32[$275>>2]|0;
     $277 = ($276|0)==(0|0);
     L9: do {
      if ($277) {
       $rsize$2$i = $249;$t$1$i = 0;$v$2$i = 0;
       label = 86;
      } else {
       $278 = ($idx$0$i|0)==(31);
       $279 = $idx$0$i >>> 1;
       $280 = (25 - ($279))|0;
       $281 = $278 ? 0 : $280;
       $282 = $246 << $281;
       $rsize$0$i15 = $249;$rst$0$i = 0;$sizebits$0$i = $282;$t$0$i14 = $276;$v$0$i16 = 0;
       while(1) {
        $283 = ((($t$0$i14)) + 4|0);
        $284 = HEAP32[$283>>2]|0;
        $285 = $284 & -8;
        $286 = (($285) - ($246))|0;
        $287 = ($286>>>0)<($rsize$0$i15>>>0);
        if ($287) {
         $288 = ($285|0)==($246|0);
         if ($288) {
          $rsize$331$i = $286;$t$230$i = $t$0$i14;$v$332$i = $t$0$i14;
          label = 90;
          break L9;
         } else {
          $rsize$1$i = $286;$v$1$i = $t$0$i14;
         }
        } else {
         $rsize$1$i = $rsize$0$i15;$v$1$i = $v$0$i16;
        }
        $289 = ((($t$0$i14)) + 20|0);
        $290 = HEAP32[$289>>2]|0;
        $291 = $sizebits$0$i >>> 31;
        $292 = (((($t$0$i14)) + 16|0) + ($291<<2)|0);
        $293 = HEAP32[$292>>2]|0;
        $294 = ($290|0)==(0|0);
        $295 = ($290|0)==($293|0);
        $or$cond19$i = $294 | $295;
        $rst$1$i = $or$cond19$i ? $rst$0$i : $290;
        $296 = ($293|0)==(0|0);
        $297 = $sizebits$0$i << 1;
        if ($296) {
         $rsize$2$i = $rsize$1$i;$t$1$i = $rst$1$i;$v$2$i = $v$1$i;
         label = 86;
         break;
        } else {
         $rsize$0$i15 = $rsize$1$i;$rst$0$i = $rst$1$i;$sizebits$0$i = $297;$t$0$i14 = $293;$v$0$i16 = $v$1$i;
        }
       }
      }
     } while(0);
     if ((label|0) == 86) {
      $298 = ($t$1$i|0)==(0|0);
      $299 = ($v$2$i|0)==(0|0);
      $or$cond$i = $298 & $299;
      if ($or$cond$i) {
       $300 = 2 << $idx$0$i;
       $301 = (0 - ($300))|0;
       $302 = $300 | $301;
       $303 = $247 & $302;
       $304 = ($303|0)==(0);
       if ($304) {
        $nb$0 = $246;
        break;
       }
       $305 = (0 - ($303))|0;
       $306 = $303 & $305;
       $307 = (($306) + -1)|0;
       $308 = $307 >>> 12;
       $309 = $308 & 16;
       $310 = $307 >>> $309;
       $311 = $310 >>> 5;
       $312 = $311 & 8;
       $313 = $312 | $309;
       $314 = $310 >>> $312;
       $315 = $314 >>> 2;
       $316 = $315 & 4;
       $317 = $313 | $316;
       $318 = $314 >>> $316;
       $319 = $318 >>> 1;
       $320 = $319 & 2;
       $321 = $317 | $320;
       $322 = $318 >>> $320;
       $323 = $322 >>> 1;
       $324 = $323 & 1;
       $325 = $321 | $324;
       $326 = $322 >>> $324;
       $327 = (($325) + ($326))|0;
       $328 = (4496 + ($327<<2)|0);
       $329 = HEAP32[$328>>2]|0;
       $t$2$ph$i = $329;$v$3$ph$i = 0;
      } else {
       $t$2$ph$i = $t$1$i;$v$3$ph$i = $v$2$i;
      }
      $330 = ($t$2$ph$i|0)==(0|0);
      if ($330) {
       $rsize$3$lcssa$i = $rsize$2$i;$v$3$lcssa$i = $v$3$ph$i;
      } else {
       $rsize$331$i = $rsize$2$i;$t$230$i = $t$2$ph$i;$v$332$i = $v$3$ph$i;
       label = 90;
      }
     }
     if ((label|0) == 90) {
      while(1) {
       label = 0;
       $331 = ((($t$230$i)) + 4|0);
       $332 = HEAP32[$331>>2]|0;
       $333 = $332 & -8;
       $334 = (($333) - ($246))|0;
       $335 = ($334>>>0)<($rsize$331$i>>>0);
       $$rsize$3$i = $335 ? $334 : $rsize$331$i;
       $t$2$v$3$i = $335 ? $t$230$i : $v$332$i;
       $336 = ((($t$230$i)) + 16|0);
       $337 = HEAP32[$336>>2]|0;
       $338 = ($337|0)==(0|0);
       if (!($338)) {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $337;$v$332$i = $t$2$v$3$i;
        label = 90;
        continue;
       }
       $339 = ((($t$230$i)) + 20|0);
       $340 = HEAP32[$339>>2]|0;
       $341 = ($340|0)==(0|0);
       if ($341) {
        $rsize$3$lcssa$i = $$rsize$3$i;$v$3$lcssa$i = $t$2$v$3$i;
        break;
       } else {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $340;$v$332$i = $t$2$v$3$i;
        label = 90;
       }
      }
     }
     $342 = ($v$3$lcssa$i|0)==(0|0);
     if ($342) {
      $nb$0 = $246;
     } else {
      $343 = HEAP32[(4200)>>2]|0;
      $344 = (($343) - ($246))|0;
      $345 = ($rsize$3$lcssa$i>>>0)<($344>>>0);
      if ($345) {
       $346 = HEAP32[(4208)>>2]|0;
       $347 = ($v$3$lcssa$i>>>0)<($346>>>0);
       if ($347) {
        _abort();
        // unreachable;
       }
       $348 = (($v$3$lcssa$i) + ($246)|0);
       $349 = ($v$3$lcssa$i>>>0)<($348>>>0);
       if (!($349)) {
        _abort();
        // unreachable;
       }
       $350 = ((($v$3$lcssa$i)) + 24|0);
       $351 = HEAP32[$350>>2]|0;
       $352 = ((($v$3$lcssa$i)) + 12|0);
       $353 = HEAP32[$352>>2]|0;
       $354 = ($353|0)==($v$3$lcssa$i|0);
       do {
        if ($354) {
         $364 = ((($v$3$lcssa$i)) + 20|0);
         $365 = HEAP32[$364>>2]|0;
         $366 = ($365|0)==(0|0);
         if ($366) {
          $367 = ((($v$3$lcssa$i)) + 16|0);
          $368 = HEAP32[$367>>2]|0;
          $369 = ($368|0)==(0|0);
          if ($369) {
           $R$1$i20 = 0;
           break;
          } else {
           $R$0$i18 = $368;$RP$0$i17 = $367;
          }
         } else {
          $R$0$i18 = $365;$RP$0$i17 = $364;
         }
         while(1) {
          $370 = ((($R$0$i18)) + 20|0);
          $371 = HEAP32[$370>>2]|0;
          $372 = ($371|0)==(0|0);
          if (!($372)) {
           $R$0$i18 = $371;$RP$0$i17 = $370;
           continue;
          }
          $373 = ((($R$0$i18)) + 16|0);
          $374 = HEAP32[$373>>2]|0;
          $375 = ($374|0)==(0|0);
          if ($375) {
           $R$0$i18$lcssa = $R$0$i18;$RP$0$i17$lcssa = $RP$0$i17;
           break;
          } else {
           $R$0$i18 = $374;$RP$0$i17 = $373;
          }
         }
         $376 = ($RP$0$i17$lcssa>>>0)<($346>>>0);
         if ($376) {
          _abort();
          // unreachable;
         } else {
          HEAP32[$RP$0$i17$lcssa>>2] = 0;
          $R$1$i20 = $R$0$i18$lcssa;
          break;
         }
        } else {
         $355 = ((($v$3$lcssa$i)) + 8|0);
         $356 = HEAP32[$355>>2]|0;
         $357 = ($356>>>0)<($346>>>0);
         if ($357) {
          _abort();
          // unreachable;
         }
         $358 = ((($356)) + 12|0);
         $359 = HEAP32[$358>>2]|0;
         $360 = ($359|0)==($v$3$lcssa$i|0);
         if (!($360)) {
          _abort();
          // unreachable;
         }
         $361 = ((($353)) + 8|0);
         $362 = HEAP32[$361>>2]|0;
         $363 = ($362|0)==($v$3$lcssa$i|0);
         if ($363) {
          HEAP32[$358>>2] = $353;
          HEAP32[$361>>2] = $356;
          $R$1$i20 = $353;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $377 = ($351|0)==(0|0);
       do {
        if (!($377)) {
         $378 = ((($v$3$lcssa$i)) + 28|0);
         $379 = HEAP32[$378>>2]|0;
         $380 = (4496 + ($379<<2)|0);
         $381 = HEAP32[$380>>2]|0;
         $382 = ($v$3$lcssa$i|0)==($381|0);
         if ($382) {
          HEAP32[$380>>2] = $R$1$i20;
          $cond$i21 = ($R$1$i20|0)==(0|0);
          if ($cond$i21) {
           $383 = 1 << $379;
           $384 = $383 ^ -1;
           $385 = HEAP32[(4196)>>2]|0;
           $386 = $385 & $384;
           HEAP32[(4196)>>2] = $386;
           break;
          }
         } else {
          $387 = HEAP32[(4208)>>2]|0;
          $388 = ($351>>>0)<($387>>>0);
          if ($388) {
           _abort();
           // unreachable;
          }
          $389 = ((($351)) + 16|0);
          $390 = HEAP32[$389>>2]|0;
          $391 = ($390|0)==($v$3$lcssa$i|0);
          if ($391) {
           HEAP32[$389>>2] = $R$1$i20;
          } else {
           $392 = ((($351)) + 20|0);
           HEAP32[$392>>2] = $R$1$i20;
          }
          $393 = ($R$1$i20|0)==(0|0);
          if ($393) {
           break;
          }
         }
         $394 = HEAP32[(4208)>>2]|0;
         $395 = ($R$1$i20>>>0)<($394>>>0);
         if ($395) {
          _abort();
          // unreachable;
         }
         $396 = ((($R$1$i20)) + 24|0);
         HEAP32[$396>>2] = $351;
         $397 = ((($v$3$lcssa$i)) + 16|0);
         $398 = HEAP32[$397>>2]|0;
         $399 = ($398|0)==(0|0);
         do {
          if (!($399)) {
           $400 = ($398>>>0)<($394>>>0);
           if ($400) {
            _abort();
            // unreachable;
           } else {
            $401 = ((($R$1$i20)) + 16|0);
            HEAP32[$401>>2] = $398;
            $402 = ((($398)) + 24|0);
            HEAP32[$402>>2] = $R$1$i20;
            break;
           }
          }
         } while(0);
         $403 = ((($v$3$lcssa$i)) + 20|0);
         $404 = HEAP32[$403>>2]|0;
         $405 = ($404|0)==(0|0);
         if (!($405)) {
          $406 = HEAP32[(4208)>>2]|0;
          $407 = ($404>>>0)<($406>>>0);
          if ($407) {
           _abort();
           // unreachable;
          } else {
           $408 = ((($R$1$i20)) + 20|0);
           HEAP32[$408>>2] = $404;
           $409 = ((($404)) + 24|0);
           HEAP32[$409>>2] = $R$1$i20;
           break;
          }
         }
        }
       } while(0);
       $410 = ($rsize$3$lcssa$i>>>0)<(16);
       L85: do {
        if ($410) {
         $411 = (($rsize$3$lcssa$i) + ($246))|0;
         $412 = $411 | 3;
         $413 = ((($v$3$lcssa$i)) + 4|0);
         HEAP32[$413>>2] = $412;
         $$sum18$i = (($411) + 4)|0;
         $414 = (($v$3$lcssa$i) + ($$sum18$i)|0);
         $415 = HEAP32[$414>>2]|0;
         $416 = $415 | 1;
         HEAP32[$414>>2] = $416;
        } else {
         $417 = $246 | 3;
         $418 = ((($v$3$lcssa$i)) + 4|0);
         HEAP32[$418>>2] = $417;
         $419 = $rsize$3$lcssa$i | 1;
         $$sum$i2334 = $246 | 4;
         $420 = (($v$3$lcssa$i) + ($$sum$i2334)|0);
         HEAP32[$420>>2] = $419;
         $$sum1$i24 = (($rsize$3$lcssa$i) + ($246))|0;
         $421 = (($v$3$lcssa$i) + ($$sum1$i24)|0);
         HEAP32[$421>>2] = $rsize$3$lcssa$i;
         $422 = $rsize$3$lcssa$i >>> 3;
         $423 = ($rsize$3$lcssa$i>>>0)<(256);
         if ($423) {
          $424 = $422 << 1;
          $425 = (4232 + ($424<<2)|0);
          $426 = HEAP32[4192>>2]|0;
          $427 = 1 << $422;
          $428 = $426 & $427;
          $429 = ($428|0)==(0);
          if ($429) {
           $430 = $426 | $427;
           HEAP32[4192>>2] = $430;
           $$pre$i25 = (($424) + 2)|0;
           $$pre43$i = (4232 + ($$pre$i25<<2)|0);
           $$pre$phi$i26Z2D = $$pre43$i;$F5$0$i = $425;
          } else {
           $$sum17$i = (($424) + 2)|0;
           $431 = (4232 + ($$sum17$i<<2)|0);
           $432 = HEAP32[$431>>2]|0;
           $433 = HEAP32[(4208)>>2]|0;
           $434 = ($432>>>0)<($433>>>0);
           if ($434) {
            _abort();
            // unreachable;
           } else {
            $$pre$phi$i26Z2D = $431;$F5$0$i = $432;
           }
          }
          HEAP32[$$pre$phi$i26Z2D>>2] = $348;
          $435 = ((($F5$0$i)) + 12|0);
          HEAP32[$435>>2] = $348;
          $$sum15$i = (($246) + 8)|0;
          $436 = (($v$3$lcssa$i) + ($$sum15$i)|0);
          HEAP32[$436>>2] = $F5$0$i;
          $$sum16$i = (($246) + 12)|0;
          $437 = (($v$3$lcssa$i) + ($$sum16$i)|0);
          HEAP32[$437>>2] = $425;
          break;
         }
         $438 = $rsize$3$lcssa$i >>> 8;
         $439 = ($438|0)==(0);
         if ($439) {
          $I7$0$i = 0;
         } else {
          $440 = ($rsize$3$lcssa$i>>>0)>(16777215);
          if ($440) {
           $I7$0$i = 31;
          } else {
           $441 = (($438) + 1048320)|0;
           $442 = $441 >>> 16;
           $443 = $442 & 8;
           $444 = $438 << $443;
           $445 = (($444) + 520192)|0;
           $446 = $445 >>> 16;
           $447 = $446 & 4;
           $448 = $447 | $443;
           $449 = $444 << $447;
           $450 = (($449) + 245760)|0;
           $451 = $450 >>> 16;
           $452 = $451 & 2;
           $453 = $448 | $452;
           $454 = (14 - ($453))|0;
           $455 = $449 << $452;
           $456 = $455 >>> 15;
           $457 = (($454) + ($456))|0;
           $458 = $457 << 1;
           $459 = (($457) + 7)|0;
           $460 = $rsize$3$lcssa$i >>> $459;
           $461 = $460 & 1;
           $462 = $461 | $458;
           $I7$0$i = $462;
          }
         }
         $463 = (4496 + ($I7$0$i<<2)|0);
         $$sum2$i = (($246) + 28)|0;
         $464 = (($v$3$lcssa$i) + ($$sum2$i)|0);
         HEAP32[$464>>2] = $I7$0$i;
         $$sum3$i27 = (($246) + 16)|0;
         $465 = (($v$3$lcssa$i) + ($$sum3$i27)|0);
         $$sum4$i28 = (($246) + 20)|0;
         $466 = (($v$3$lcssa$i) + ($$sum4$i28)|0);
         HEAP32[$466>>2] = 0;
         HEAP32[$465>>2] = 0;
         $467 = HEAP32[(4196)>>2]|0;
         $468 = 1 << $I7$0$i;
         $469 = $467 & $468;
         $470 = ($469|0)==(0);
         if ($470) {
          $471 = $467 | $468;
          HEAP32[(4196)>>2] = $471;
          HEAP32[$463>>2] = $348;
          $$sum5$i = (($246) + 24)|0;
          $472 = (($v$3$lcssa$i) + ($$sum5$i)|0);
          HEAP32[$472>>2] = $463;
          $$sum6$i = (($246) + 12)|0;
          $473 = (($v$3$lcssa$i) + ($$sum6$i)|0);
          HEAP32[$473>>2] = $348;
          $$sum7$i = (($246) + 8)|0;
          $474 = (($v$3$lcssa$i) + ($$sum7$i)|0);
          HEAP32[$474>>2] = $348;
          break;
         }
         $475 = HEAP32[$463>>2]|0;
         $476 = ((($475)) + 4|0);
         $477 = HEAP32[$476>>2]|0;
         $478 = $477 & -8;
         $479 = ($478|0)==($rsize$3$lcssa$i|0);
         L103: do {
          if ($479) {
           $T$0$lcssa$i = $475;
          } else {
           $480 = ($I7$0$i|0)==(31);
           $481 = $I7$0$i >>> 1;
           $482 = (25 - ($481))|0;
           $483 = $480 ? 0 : $482;
           $484 = $rsize$3$lcssa$i << $483;
           $K12$029$i = $484;$T$028$i = $475;
           while(1) {
            $491 = $K12$029$i >>> 31;
            $492 = (((($T$028$i)) + 16|0) + ($491<<2)|0);
            $487 = HEAP32[$492>>2]|0;
            $493 = ($487|0)==(0|0);
            if ($493) {
             $$lcssa232 = $492;$T$028$i$lcssa = $T$028$i;
             break;
            }
            $485 = $K12$029$i << 1;
            $486 = ((($487)) + 4|0);
            $488 = HEAP32[$486>>2]|0;
            $489 = $488 & -8;
            $490 = ($489|0)==($rsize$3$lcssa$i|0);
            if ($490) {
             $T$0$lcssa$i = $487;
             break L103;
            } else {
             $K12$029$i = $485;$T$028$i = $487;
            }
           }
           $494 = HEAP32[(4208)>>2]|0;
           $495 = ($$lcssa232>>>0)<($494>>>0);
           if ($495) {
            _abort();
            // unreachable;
           } else {
            HEAP32[$$lcssa232>>2] = $348;
            $$sum11$i = (($246) + 24)|0;
            $496 = (($v$3$lcssa$i) + ($$sum11$i)|0);
            HEAP32[$496>>2] = $T$028$i$lcssa;
            $$sum12$i = (($246) + 12)|0;
            $497 = (($v$3$lcssa$i) + ($$sum12$i)|0);
            HEAP32[$497>>2] = $348;
            $$sum13$i = (($246) + 8)|0;
            $498 = (($v$3$lcssa$i) + ($$sum13$i)|0);
            HEAP32[$498>>2] = $348;
            break L85;
           }
          }
         } while(0);
         $499 = ((($T$0$lcssa$i)) + 8|0);
         $500 = HEAP32[$499>>2]|0;
         $501 = HEAP32[(4208)>>2]|0;
         $502 = ($500>>>0)>=($501>>>0);
         $not$$i = ($T$0$lcssa$i>>>0)>=($501>>>0);
         $503 = $502 & $not$$i;
         if ($503) {
          $504 = ((($500)) + 12|0);
          HEAP32[$504>>2] = $348;
          HEAP32[$499>>2] = $348;
          $$sum8$i = (($246) + 8)|0;
          $505 = (($v$3$lcssa$i) + ($$sum8$i)|0);
          HEAP32[$505>>2] = $500;
          $$sum9$i = (($246) + 12)|0;
          $506 = (($v$3$lcssa$i) + ($$sum9$i)|0);
          HEAP32[$506>>2] = $T$0$lcssa$i;
          $$sum10$i = (($246) + 24)|0;
          $507 = (($v$3$lcssa$i) + ($$sum10$i)|0);
          HEAP32[$507>>2] = 0;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $508 = ((($v$3$lcssa$i)) + 8|0);
       $mem$0 = $508;
       return ($mem$0|0);
      } else {
       $nb$0 = $246;
      }
     }
    }
   }
  }
 } while(0);
 $509 = HEAP32[(4200)>>2]|0;
 $510 = ($509>>>0)<($nb$0>>>0);
 if (!($510)) {
  $511 = (($509) - ($nb$0))|0;
  $512 = HEAP32[(4212)>>2]|0;
  $513 = ($511>>>0)>(15);
  if ($513) {
   $514 = (($512) + ($nb$0)|0);
   HEAP32[(4212)>>2] = $514;
   HEAP32[(4200)>>2] = $511;
   $515 = $511 | 1;
   $$sum2 = (($nb$0) + 4)|0;
   $516 = (($512) + ($$sum2)|0);
   HEAP32[$516>>2] = $515;
   $517 = (($512) + ($509)|0);
   HEAP32[$517>>2] = $511;
   $518 = $nb$0 | 3;
   $519 = ((($512)) + 4|0);
   HEAP32[$519>>2] = $518;
  } else {
   HEAP32[(4200)>>2] = 0;
   HEAP32[(4212)>>2] = 0;
   $520 = $509 | 3;
   $521 = ((($512)) + 4|0);
   HEAP32[$521>>2] = $520;
   $$sum1 = (($509) + 4)|0;
   $522 = (($512) + ($$sum1)|0);
   $523 = HEAP32[$522>>2]|0;
   $524 = $523 | 1;
   HEAP32[$522>>2] = $524;
  }
  $525 = ((($512)) + 8|0);
  $mem$0 = $525;
  return ($mem$0|0);
 }
 $526 = HEAP32[(4204)>>2]|0;
 $527 = ($526>>>0)>($nb$0>>>0);
 if ($527) {
  $528 = (($526) - ($nb$0))|0;
  HEAP32[(4204)>>2] = $528;
  $529 = HEAP32[(4216)>>2]|0;
  $530 = (($529) + ($nb$0)|0);
  HEAP32[(4216)>>2] = $530;
  $531 = $528 | 1;
  $$sum = (($nb$0) + 4)|0;
  $532 = (($529) + ($$sum)|0);
  HEAP32[$532>>2] = $531;
  $533 = $nb$0 | 3;
  $534 = ((($529)) + 4|0);
  HEAP32[$534>>2] = $533;
  $535 = ((($529)) + 8|0);
  $mem$0 = $535;
  return ($mem$0|0);
 }
 $536 = HEAP32[4664>>2]|0;
 $537 = ($536|0)==(0);
 do {
  if ($537) {
   $538 = (_sysconf(30)|0);
   $539 = (($538) + -1)|0;
   $540 = $539 & $538;
   $541 = ($540|0)==(0);
   if ($541) {
    HEAP32[(4672)>>2] = $538;
    HEAP32[(4668)>>2] = $538;
    HEAP32[(4676)>>2] = -1;
    HEAP32[(4680)>>2] = -1;
    HEAP32[(4684)>>2] = 0;
    HEAP32[(4636)>>2] = 0;
    $542 = (_time((0|0))|0);
    $543 = $542 & -16;
    $544 = $543 ^ 1431655768;
    HEAP32[4664>>2] = $544;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $545 = (($nb$0) + 48)|0;
 $546 = HEAP32[(4672)>>2]|0;
 $547 = (($nb$0) + 47)|0;
 $548 = (($546) + ($547))|0;
 $549 = (0 - ($546))|0;
 $550 = $548 & $549;
 $551 = ($550>>>0)>($nb$0>>>0);
 if (!($551)) {
  $mem$0 = 0;
  return ($mem$0|0);
 }
 $552 = HEAP32[(4632)>>2]|0;
 $553 = ($552|0)==(0);
 if (!($553)) {
  $554 = HEAP32[(4624)>>2]|0;
  $555 = (($554) + ($550))|0;
  $556 = ($555>>>0)<=($554>>>0);
  $557 = ($555>>>0)>($552>>>0);
  $or$cond1$i = $556 | $557;
  if ($or$cond1$i) {
   $mem$0 = 0;
   return ($mem$0|0);
  }
 }
 $558 = HEAP32[(4636)>>2]|0;
 $559 = $558 & 4;
 $560 = ($559|0)==(0);
 L258: do {
  if ($560) {
   $561 = HEAP32[(4216)>>2]|0;
   $562 = ($561|0)==(0|0);
   L260: do {
    if ($562) {
     label = 174;
    } else {
     $sp$0$i$i = (4640);
     while(1) {
      $563 = HEAP32[$sp$0$i$i>>2]|0;
      $564 = ($563>>>0)>($561>>>0);
      if (!($564)) {
       $565 = ((($sp$0$i$i)) + 4|0);
       $566 = HEAP32[$565>>2]|0;
       $567 = (($563) + ($566)|0);
       $568 = ($567>>>0)>($561>>>0);
       if ($568) {
        $$lcssa228 = $sp$0$i$i;$$lcssa230 = $565;
        break;
       }
      }
      $569 = ((($sp$0$i$i)) + 8|0);
      $570 = HEAP32[$569>>2]|0;
      $571 = ($570|0)==(0|0);
      if ($571) {
       label = 174;
       break L260;
      } else {
       $sp$0$i$i = $570;
      }
     }
     $594 = HEAP32[(4204)>>2]|0;
     $595 = (($548) - ($594))|0;
     $596 = $595 & $549;
     $597 = ($596>>>0)<(2147483647);
     if ($597) {
      $598 = (_sbrk(($596|0))|0);
      $599 = HEAP32[$$lcssa228>>2]|0;
      $600 = HEAP32[$$lcssa230>>2]|0;
      $601 = (($599) + ($600)|0);
      $602 = ($598|0)==($601|0);
      $$3$i = $602 ? $596 : 0;
      if ($602) {
       $603 = ($598|0)==((-1)|0);
       if ($603) {
        $tsize$0323944$i = $$3$i;
       } else {
        $tbase$255$i = $598;$tsize$254$i = $$3$i;
        label = 194;
        break L258;
       }
      } else {
       $br$0$ph$i = $598;$ssize$1$ph$i = $596;$tsize$0$ph$i = $$3$i;
       label = 184;
      }
     } else {
      $tsize$0323944$i = 0;
     }
    }
   } while(0);
   do {
    if ((label|0) == 174) {
     $572 = (_sbrk(0)|0);
     $573 = ($572|0)==((-1)|0);
     if ($573) {
      $tsize$0323944$i = 0;
     } else {
      $574 = $572;
      $575 = HEAP32[(4668)>>2]|0;
      $576 = (($575) + -1)|0;
      $577 = $576 & $574;
      $578 = ($577|0)==(0);
      if ($578) {
       $ssize$0$i = $550;
      } else {
       $579 = (($576) + ($574))|0;
       $580 = (0 - ($575))|0;
       $581 = $579 & $580;
       $582 = (($550) - ($574))|0;
       $583 = (($582) + ($581))|0;
       $ssize$0$i = $583;
      }
      $584 = HEAP32[(4624)>>2]|0;
      $585 = (($584) + ($ssize$0$i))|0;
      $586 = ($ssize$0$i>>>0)>($nb$0>>>0);
      $587 = ($ssize$0$i>>>0)<(2147483647);
      $or$cond$i30 = $586 & $587;
      if ($or$cond$i30) {
       $588 = HEAP32[(4632)>>2]|0;
       $589 = ($588|0)==(0);
       if (!($589)) {
        $590 = ($585>>>0)<=($584>>>0);
        $591 = ($585>>>0)>($588>>>0);
        $or$cond2$i = $590 | $591;
        if ($or$cond2$i) {
         $tsize$0323944$i = 0;
         break;
        }
       }
       $592 = (_sbrk(($ssize$0$i|0))|0);
       $593 = ($592|0)==($572|0);
       $ssize$0$$i = $593 ? $ssize$0$i : 0;
       if ($593) {
        $tbase$255$i = $572;$tsize$254$i = $ssize$0$$i;
        label = 194;
        break L258;
       } else {
        $br$0$ph$i = $592;$ssize$1$ph$i = $ssize$0$i;$tsize$0$ph$i = $ssize$0$$i;
        label = 184;
       }
      } else {
       $tsize$0323944$i = 0;
      }
     }
    }
   } while(0);
   L280: do {
    if ((label|0) == 184) {
     $604 = (0 - ($ssize$1$ph$i))|0;
     $605 = ($br$0$ph$i|0)!=((-1)|0);
     $606 = ($ssize$1$ph$i>>>0)<(2147483647);
     $or$cond5$i = $606 & $605;
     $607 = ($545>>>0)>($ssize$1$ph$i>>>0);
     $or$cond6$i = $607 & $or$cond5$i;
     do {
      if ($or$cond6$i) {
       $608 = HEAP32[(4672)>>2]|0;
       $609 = (($547) - ($ssize$1$ph$i))|0;
       $610 = (($609) + ($608))|0;
       $611 = (0 - ($608))|0;
       $612 = $610 & $611;
       $613 = ($612>>>0)<(2147483647);
       if ($613) {
        $614 = (_sbrk(($612|0))|0);
        $615 = ($614|0)==((-1)|0);
        if ($615) {
         (_sbrk(($604|0))|0);
         $tsize$0323944$i = $tsize$0$ph$i;
         break L280;
        } else {
         $616 = (($612) + ($ssize$1$ph$i))|0;
         $ssize$2$i = $616;
         break;
        }
       } else {
        $ssize$2$i = $ssize$1$ph$i;
       }
      } else {
       $ssize$2$i = $ssize$1$ph$i;
      }
     } while(0);
     $617 = ($br$0$ph$i|0)==((-1)|0);
     if ($617) {
      $tsize$0323944$i = $tsize$0$ph$i;
     } else {
      $tbase$255$i = $br$0$ph$i;$tsize$254$i = $ssize$2$i;
      label = 194;
      break L258;
     }
    }
   } while(0);
   $618 = HEAP32[(4636)>>2]|0;
   $619 = $618 | 4;
   HEAP32[(4636)>>2] = $619;
   $tsize$1$i = $tsize$0323944$i;
   label = 191;
  } else {
   $tsize$1$i = 0;
   label = 191;
  }
 } while(0);
 if ((label|0) == 191) {
  $620 = ($550>>>0)<(2147483647);
  if ($620) {
   $621 = (_sbrk(($550|0))|0);
   $622 = (_sbrk(0)|0);
   $623 = ($621|0)!=((-1)|0);
   $624 = ($622|0)!=((-1)|0);
   $or$cond3$i = $623 & $624;
   $625 = ($621>>>0)<($622>>>0);
   $or$cond8$i = $625 & $or$cond3$i;
   if ($or$cond8$i) {
    $626 = $622;
    $627 = $621;
    $628 = (($626) - ($627))|0;
    $629 = (($nb$0) + 40)|0;
    $630 = ($628>>>0)>($629>>>0);
    $$tsize$1$i = $630 ? $628 : $tsize$1$i;
    if ($630) {
     $tbase$255$i = $621;$tsize$254$i = $$tsize$1$i;
     label = 194;
    }
   }
  }
 }
 if ((label|0) == 194) {
  $631 = HEAP32[(4624)>>2]|0;
  $632 = (($631) + ($tsize$254$i))|0;
  HEAP32[(4624)>>2] = $632;
  $633 = HEAP32[(4628)>>2]|0;
  $634 = ($632>>>0)>($633>>>0);
  if ($634) {
   HEAP32[(4628)>>2] = $632;
  }
  $635 = HEAP32[(4216)>>2]|0;
  $636 = ($635|0)==(0|0);
  L299: do {
   if ($636) {
    $637 = HEAP32[(4208)>>2]|0;
    $638 = ($637|0)==(0|0);
    $639 = ($tbase$255$i>>>0)<($637>>>0);
    $or$cond9$i = $638 | $639;
    if ($or$cond9$i) {
     HEAP32[(4208)>>2] = $tbase$255$i;
    }
    HEAP32[(4640)>>2] = $tbase$255$i;
    HEAP32[(4644)>>2] = $tsize$254$i;
    HEAP32[(4652)>>2] = 0;
    $640 = HEAP32[4664>>2]|0;
    HEAP32[(4228)>>2] = $640;
    HEAP32[(4224)>>2] = -1;
    $i$02$i$i = 0;
    while(1) {
     $641 = $i$02$i$i << 1;
     $642 = (4232 + ($641<<2)|0);
     $$sum$i$i = (($641) + 3)|0;
     $643 = (4232 + ($$sum$i$i<<2)|0);
     HEAP32[$643>>2] = $642;
     $$sum1$i$i = (($641) + 2)|0;
     $644 = (4232 + ($$sum1$i$i<<2)|0);
     HEAP32[$644>>2] = $642;
     $645 = (($i$02$i$i) + 1)|0;
     $exitcond$i$i = ($645|0)==(32);
     if ($exitcond$i$i) {
      break;
     } else {
      $i$02$i$i = $645;
     }
    }
    $646 = (($tsize$254$i) + -40)|0;
    $647 = ((($tbase$255$i)) + 8|0);
    $648 = $647;
    $649 = $648 & 7;
    $650 = ($649|0)==(0);
    $651 = (0 - ($648))|0;
    $652 = $651 & 7;
    $653 = $650 ? 0 : $652;
    $654 = (($tbase$255$i) + ($653)|0);
    $655 = (($646) - ($653))|0;
    HEAP32[(4216)>>2] = $654;
    HEAP32[(4204)>>2] = $655;
    $656 = $655 | 1;
    $$sum$i13$i = (($653) + 4)|0;
    $657 = (($tbase$255$i) + ($$sum$i13$i)|0);
    HEAP32[$657>>2] = $656;
    $$sum2$i$i = (($tsize$254$i) + -36)|0;
    $658 = (($tbase$255$i) + ($$sum2$i$i)|0);
    HEAP32[$658>>2] = 40;
    $659 = HEAP32[(4680)>>2]|0;
    HEAP32[(4220)>>2] = $659;
   } else {
    $sp$084$i = (4640);
    while(1) {
     $660 = HEAP32[$sp$084$i>>2]|0;
     $661 = ((($sp$084$i)) + 4|0);
     $662 = HEAP32[$661>>2]|0;
     $663 = (($660) + ($662)|0);
     $664 = ($tbase$255$i|0)==($663|0);
     if ($664) {
      $$lcssa222 = $660;$$lcssa224 = $661;$$lcssa226 = $662;$sp$084$i$lcssa = $sp$084$i;
      label = 204;
      break;
     }
     $665 = ((($sp$084$i)) + 8|0);
     $666 = HEAP32[$665>>2]|0;
     $667 = ($666|0)==(0|0);
     if ($667) {
      break;
     } else {
      $sp$084$i = $666;
     }
    }
    if ((label|0) == 204) {
     $668 = ((($sp$084$i$lcssa)) + 12|0);
     $669 = HEAP32[$668>>2]|0;
     $670 = $669 & 8;
     $671 = ($670|0)==(0);
     if ($671) {
      $672 = ($635>>>0)>=($$lcssa222>>>0);
      $673 = ($635>>>0)<($tbase$255$i>>>0);
      $or$cond57$i = $673 & $672;
      if ($or$cond57$i) {
       $674 = (($$lcssa226) + ($tsize$254$i))|0;
       HEAP32[$$lcssa224>>2] = $674;
       $675 = HEAP32[(4204)>>2]|0;
       $676 = (($675) + ($tsize$254$i))|0;
       $677 = ((($635)) + 8|0);
       $678 = $677;
       $679 = $678 & 7;
       $680 = ($679|0)==(0);
       $681 = (0 - ($678))|0;
       $682 = $681 & 7;
       $683 = $680 ? 0 : $682;
       $684 = (($635) + ($683)|0);
       $685 = (($676) - ($683))|0;
       HEAP32[(4216)>>2] = $684;
       HEAP32[(4204)>>2] = $685;
       $686 = $685 | 1;
       $$sum$i17$i = (($683) + 4)|0;
       $687 = (($635) + ($$sum$i17$i)|0);
       HEAP32[$687>>2] = $686;
       $$sum2$i18$i = (($676) + 4)|0;
       $688 = (($635) + ($$sum2$i18$i)|0);
       HEAP32[$688>>2] = 40;
       $689 = HEAP32[(4680)>>2]|0;
       HEAP32[(4220)>>2] = $689;
       break;
      }
     }
    }
    $690 = HEAP32[(4208)>>2]|0;
    $691 = ($tbase$255$i>>>0)<($690>>>0);
    if ($691) {
     HEAP32[(4208)>>2] = $tbase$255$i;
     $755 = $tbase$255$i;
    } else {
     $755 = $690;
    }
    $692 = (($tbase$255$i) + ($tsize$254$i)|0);
    $sp$183$i = (4640);
    while(1) {
     $693 = HEAP32[$sp$183$i>>2]|0;
     $694 = ($693|0)==($692|0);
     if ($694) {
      $$lcssa219 = $sp$183$i;$sp$183$i$lcssa = $sp$183$i;
      label = 212;
      break;
     }
     $695 = ((($sp$183$i)) + 8|0);
     $696 = HEAP32[$695>>2]|0;
     $697 = ($696|0)==(0|0);
     if ($697) {
      $sp$0$i$i$i = (4640);
      break;
     } else {
      $sp$183$i = $696;
     }
    }
    if ((label|0) == 212) {
     $698 = ((($sp$183$i$lcssa)) + 12|0);
     $699 = HEAP32[$698>>2]|0;
     $700 = $699 & 8;
     $701 = ($700|0)==(0);
     if ($701) {
      HEAP32[$$lcssa219>>2] = $tbase$255$i;
      $702 = ((($sp$183$i$lcssa)) + 4|0);
      $703 = HEAP32[$702>>2]|0;
      $704 = (($703) + ($tsize$254$i))|0;
      HEAP32[$702>>2] = $704;
      $705 = ((($tbase$255$i)) + 8|0);
      $706 = $705;
      $707 = $706 & 7;
      $708 = ($707|0)==(0);
      $709 = (0 - ($706))|0;
      $710 = $709 & 7;
      $711 = $708 ? 0 : $710;
      $712 = (($tbase$255$i) + ($711)|0);
      $$sum112$i = (($tsize$254$i) + 8)|0;
      $713 = (($tbase$255$i) + ($$sum112$i)|0);
      $714 = $713;
      $715 = $714 & 7;
      $716 = ($715|0)==(0);
      $717 = (0 - ($714))|0;
      $718 = $717 & 7;
      $719 = $716 ? 0 : $718;
      $$sum113$i = (($719) + ($tsize$254$i))|0;
      $720 = (($tbase$255$i) + ($$sum113$i)|0);
      $721 = $720;
      $722 = $712;
      $723 = (($721) - ($722))|0;
      $$sum$i19$i = (($711) + ($nb$0))|0;
      $724 = (($tbase$255$i) + ($$sum$i19$i)|0);
      $725 = (($723) - ($nb$0))|0;
      $726 = $nb$0 | 3;
      $$sum1$i20$i = (($711) + 4)|0;
      $727 = (($tbase$255$i) + ($$sum1$i20$i)|0);
      HEAP32[$727>>2] = $726;
      $728 = ($720|0)==($635|0);
      L324: do {
       if ($728) {
        $729 = HEAP32[(4204)>>2]|0;
        $730 = (($729) + ($725))|0;
        HEAP32[(4204)>>2] = $730;
        HEAP32[(4216)>>2] = $724;
        $731 = $730 | 1;
        $$sum42$i$i = (($$sum$i19$i) + 4)|0;
        $732 = (($tbase$255$i) + ($$sum42$i$i)|0);
        HEAP32[$732>>2] = $731;
       } else {
        $733 = HEAP32[(4212)>>2]|0;
        $734 = ($720|0)==($733|0);
        if ($734) {
         $735 = HEAP32[(4200)>>2]|0;
         $736 = (($735) + ($725))|0;
         HEAP32[(4200)>>2] = $736;
         HEAP32[(4212)>>2] = $724;
         $737 = $736 | 1;
         $$sum40$i$i = (($$sum$i19$i) + 4)|0;
         $738 = (($tbase$255$i) + ($$sum40$i$i)|0);
         HEAP32[$738>>2] = $737;
         $$sum41$i$i = (($736) + ($$sum$i19$i))|0;
         $739 = (($tbase$255$i) + ($$sum41$i$i)|0);
         HEAP32[$739>>2] = $736;
         break;
        }
        $$sum2$i21$i = (($tsize$254$i) + 4)|0;
        $$sum114$i = (($$sum2$i21$i) + ($719))|0;
        $740 = (($tbase$255$i) + ($$sum114$i)|0);
        $741 = HEAP32[$740>>2]|0;
        $742 = $741 & 3;
        $743 = ($742|0)==(1);
        if ($743) {
         $744 = $741 & -8;
         $745 = $741 >>> 3;
         $746 = ($741>>>0)<(256);
         L332: do {
          if ($746) {
           $$sum3738$i$i = $719 | 8;
           $$sum124$i = (($$sum3738$i$i) + ($tsize$254$i))|0;
           $747 = (($tbase$255$i) + ($$sum124$i)|0);
           $748 = HEAP32[$747>>2]|0;
           $$sum39$i$i = (($tsize$254$i) + 12)|0;
           $$sum125$i = (($$sum39$i$i) + ($719))|0;
           $749 = (($tbase$255$i) + ($$sum125$i)|0);
           $750 = HEAP32[$749>>2]|0;
           $751 = $745 << 1;
           $752 = (4232 + ($751<<2)|0);
           $753 = ($748|0)==($752|0);
           do {
            if (!($753)) {
             $754 = ($748>>>0)<($755>>>0);
             if ($754) {
              _abort();
              // unreachable;
             }
             $756 = ((($748)) + 12|0);
             $757 = HEAP32[$756>>2]|0;
             $758 = ($757|0)==($720|0);
             if ($758) {
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $759 = ($750|0)==($748|0);
           if ($759) {
            $760 = 1 << $745;
            $761 = $760 ^ -1;
            $762 = HEAP32[4192>>2]|0;
            $763 = $762 & $761;
            HEAP32[4192>>2] = $763;
            break;
           }
           $764 = ($750|0)==($752|0);
           do {
            if ($764) {
             $$pre57$i$i = ((($750)) + 8|0);
             $$pre$phi58$i$iZ2D = $$pre57$i$i;
            } else {
             $765 = ($750>>>0)<($755>>>0);
             if ($765) {
              _abort();
              // unreachable;
             }
             $766 = ((($750)) + 8|0);
             $767 = HEAP32[$766>>2]|0;
             $768 = ($767|0)==($720|0);
             if ($768) {
              $$pre$phi58$i$iZ2D = $766;
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $769 = ((($748)) + 12|0);
           HEAP32[$769>>2] = $750;
           HEAP32[$$pre$phi58$i$iZ2D>>2] = $748;
          } else {
           $$sum34$i$i = $719 | 24;
           $$sum115$i = (($$sum34$i$i) + ($tsize$254$i))|0;
           $770 = (($tbase$255$i) + ($$sum115$i)|0);
           $771 = HEAP32[$770>>2]|0;
           $$sum5$i$i = (($tsize$254$i) + 12)|0;
           $$sum116$i = (($$sum5$i$i) + ($719))|0;
           $772 = (($tbase$255$i) + ($$sum116$i)|0);
           $773 = HEAP32[$772>>2]|0;
           $774 = ($773|0)==($720|0);
           do {
            if ($774) {
             $$sum67$i$i = $719 | 16;
             $$sum122$i = (($$sum2$i21$i) + ($$sum67$i$i))|0;
             $784 = (($tbase$255$i) + ($$sum122$i)|0);
             $785 = HEAP32[$784>>2]|0;
             $786 = ($785|0)==(0|0);
             if ($786) {
              $$sum123$i = (($$sum67$i$i) + ($tsize$254$i))|0;
              $787 = (($tbase$255$i) + ($$sum123$i)|0);
              $788 = HEAP32[$787>>2]|0;
              $789 = ($788|0)==(0|0);
              if ($789) {
               $R$1$i$i = 0;
               break;
              } else {
               $R$0$i$i = $788;$RP$0$i$i = $787;
              }
             } else {
              $R$0$i$i = $785;$RP$0$i$i = $784;
             }
             while(1) {
              $790 = ((($R$0$i$i)) + 20|0);
              $791 = HEAP32[$790>>2]|0;
              $792 = ($791|0)==(0|0);
              if (!($792)) {
               $R$0$i$i = $791;$RP$0$i$i = $790;
               continue;
              }
              $793 = ((($R$0$i$i)) + 16|0);
              $794 = HEAP32[$793>>2]|0;
              $795 = ($794|0)==(0|0);
              if ($795) {
               $R$0$i$i$lcssa = $R$0$i$i;$RP$0$i$i$lcssa = $RP$0$i$i;
               break;
              } else {
               $R$0$i$i = $794;$RP$0$i$i = $793;
              }
             }
             $796 = ($RP$0$i$i$lcssa>>>0)<($755>>>0);
             if ($796) {
              _abort();
              // unreachable;
             } else {
              HEAP32[$RP$0$i$i$lcssa>>2] = 0;
              $R$1$i$i = $R$0$i$i$lcssa;
              break;
             }
            } else {
             $$sum3536$i$i = $719 | 8;
             $$sum117$i = (($$sum3536$i$i) + ($tsize$254$i))|0;
             $775 = (($tbase$255$i) + ($$sum117$i)|0);
             $776 = HEAP32[$775>>2]|0;
             $777 = ($776>>>0)<($755>>>0);
             if ($777) {
              _abort();
              // unreachable;
             }
             $778 = ((($776)) + 12|0);
             $779 = HEAP32[$778>>2]|0;
             $780 = ($779|0)==($720|0);
             if (!($780)) {
              _abort();
              // unreachable;
             }
             $781 = ((($773)) + 8|0);
             $782 = HEAP32[$781>>2]|0;
             $783 = ($782|0)==($720|0);
             if ($783) {
              HEAP32[$778>>2] = $773;
              HEAP32[$781>>2] = $776;
              $R$1$i$i = $773;
              break;
             } else {
              _abort();
              // unreachable;
             }
            }
           } while(0);
           $797 = ($771|0)==(0|0);
           if ($797) {
            break;
           }
           $$sum30$i$i = (($tsize$254$i) + 28)|0;
           $$sum118$i = (($$sum30$i$i) + ($719))|0;
           $798 = (($tbase$255$i) + ($$sum118$i)|0);
           $799 = HEAP32[$798>>2]|0;
           $800 = (4496 + ($799<<2)|0);
           $801 = HEAP32[$800>>2]|0;
           $802 = ($720|0)==($801|0);
           do {
            if ($802) {
             HEAP32[$800>>2] = $R$1$i$i;
             $cond$i$i = ($R$1$i$i|0)==(0|0);
             if (!($cond$i$i)) {
              break;
             }
             $803 = 1 << $799;
             $804 = $803 ^ -1;
             $805 = HEAP32[(4196)>>2]|0;
             $806 = $805 & $804;
             HEAP32[(4196)>>2] = $806;
             break L332;
            } else {
             $807 = HEAP32[(4208)>>2]|0;
             $808 = ($771>>>0)<($807>>>0);
             if ($808) {
              _abort();
              // unreachable;
             }
             $809 = ((($771)) + 16|0);
             $810 = HEAP32[$809>>2]|0;
             $811 = ($810|0)==($720|0);
             if ($811) {
              HEAP32[$809>>2] = $R$1$i$i;
             } else {
              $812 = ((($771)) + 20|0);
              HEAP32[$812>>2] = $R$1$i$i;
             }
             $813 = ($R$1$i$i|0)==(0|0);
             if ($813) {
              break L332;
             }
            }
           } while(0);
           $814 = HEAP32[(4208)>>2]|0;
           $815 = ($R$1$i$i>>>0)<($814>>>0);
           if ($815) {
            _abort();
            // unreachable;
           }
           $816 = ((($R$1$i$i)) + 24|0);
           HEAP32[$816>>2] = $771;
           $$sum3132$i$i = $719 | 16;
           $$sum119$i = (($$sum3132$i$i) + ($tsize$254$i))|0;
           $817 = (($tbase$255$i) + ($$sum119$i)|0);
           $818 = HEAP32[$817>>2]|0;
           $819 = ($818|0)==(0|0);
           do {
            if (!($819)) {
             $820 = ($818>>>0)<($814>>>0);
             if ($820) {
              _abort();
              // unreachable;
             } else {
              $821 = ((($R$1$i$i)) + 16|0);
              HEAP32[$821>>2] = $818;
              $822 = ((($818)) + 24|0);
              HEAP32[$822>>2] = $R$1$i$i;
              break;
             }
            }
           } while(0);
           $$sum120$i = (($$sum2$i21$i) + ($$sum3132$i$i))|0;
           $823 = (($tbase$255$i) + ($$sum120$i)|0);
           $824 = HEAP32[$823>>2]|0;
           $825 = ($824|0)==(0|0);
           if ($825) {
            break;
           }
           $826 = HEAP32[(4208)>>2]|0;
           $827 = ($824>>>0)<($826>>>0);
           if ($827) {
            _abort();
            // unreachable;
           } else {
            $828 = ((($R$1$i$i)) + 20|0);
            HEAP32[$828>>2] = $824;
            $829 = ((($824)) + 24|0);
            HEAP32[$829>>2] = $R$1$i$i;
            break;
           }
          }
         } while(0);
         $$sum9$i$i = $744 | $719;
         $$sum121$i = (($$sum9$i$i) + ($tsize$254$i))|0;
         $830 = (($tbase$255$i) + ($$sum121$i)|0);
         $831 = (($744) + ($725))|0;
         $oldfirst$0$i$i = $830;$qsize$0$i$i = $831;
        } else {
         $oldfirst$0$i$i = $720;$qsize$0$i$i = $725;
        }
        $832 = ((($oldfirst$0$i$i)) + 4|0);
        $833 = HEAP32[$832>>2]|0;
        $834 = $833 & -2;
        HEAP32[$832>>2] = $834;
        $835 = $qsize$0$i$i | 1;
        $$sum10$i$i = (($$sum$i19$i) + 4)|0;
        $836 = (($tbase$255$i) + ($$sum10$i$i)|0);
        HEAP32[$836>>2] = $835;
        $$sum11$i$i = (($qsize$0$i$i) + ($$sum$i19$i))|0;
        $837 = (($tbase$255$i) + ($$sum11$i$i)|0);
        HEAP32[$837>>2] = $qsize$0$i$i;
        $838 = $qsize$0$i$i >>> 3;
        $839 = ($qsize$0$i$i>>>0)<(256);
        if ($839) {
         $840 = $838 << 1;
         $841 = (4232 + ($840<<2)|0);
         $842 = HEAP32[4192>>2]|0;
         $843 = 1 << $838;
         $844 = $842 & $843;
         $845 = ($844|0)==(0);
         do {
          if ($845) {
           $846 = $842 | $843;
           HEAP32[4192>>2] = $846;
           $$pre$i22$i = (($840) + 2)|0;
           $$pre56$i$i = (4232 + ($$pre$i22$i<<2)|0);
           $$pre$phi$i23$iZ2D = $$pre56$i$i;$F4$0$i$i = $841;
          } else {
           $$sum29$i$i = (($840) + 2)|0;
           $847 = (4232 + ($$sum29$i$i<<2)|0);
           $848 = HEAP32[$847>>2]|0;
           $849 = HEAP32[(4208)>>2]|0;
           $850 = ($848>>>0)<($849>>>0);
           if (!($850)) {
            $$pre$phi$i23$iZ2D = $847;$F4$0$i$i = $848;
            break;
           }
           _abort();
           // unreachable;
          }
         } while(0);
         HEAP32[$$pre$phi$i23$iZ2D>>2] = $724;
         $851 = ((($F4$0$i$i)) + 12|0);
         HEAP32[$851>>2] = $724;
         $$sum27$i$i = (($$sum$i19$i) + 8)|0;
         $852 = (($tbase$255$i) + ($$sum27$i$i)|0);
         HEAP32[$852>>2] = $F4$0$i$i;
         $$sum28$i$i = (($$sum$i19$i) + 12)|0;
         $853 = (($tbase$255$i) + ($$sum28$i$i)|0);
         HEAP32[$853>>2] = $841;
         break;
        }
        $854 = $qsize$0$i$i >>> 8;
        $855 = ($854|0)==(0);
        do {
         if ($855) {
          $I7$0$i$i = 0;
         } else {
          $856 = ($qsize$0$i$i>>>0)>(16777215);
          if ($856) {
           $I7$0$i$i = 31;
           break;
          }
          $857 = (($854) + 1048320)|0;
          $858 = $857 >>> 16;
          $859 = $858 & 8;
          $860 = $854 << $859;
          $861 = (($860) + 520192)|0;
          $862 = $861 >>> 16;
          $863 = $862 & 4;
          $864 = $863 | $859;
          $865 = $860 << $863;
          $866 = (($865) + 245760)|0;
          $867 = $866 >>> 16;
          $868 = $867 & 2;
          $869 = $864 | $868;
          $870 = (14 - ($869))|0;
          $871 = $865 << $868;
          $872 = $871 >>> 15;
          $873 = (($870) + ($872))|0;
          $874 = $873 << 1;
          $875 = (($873) + 7)|0;
          $876 = $qsize$0$i$i >>> $875;
          $877 = $876 & 1;
          $878 = $877 | $874;
          $I7$0$i$i = $878;
         }
        } while(0);
        $879 = (4496 + ($I7$0$i$i<<2)|0);
        $$sum12$i$i = (($$sum$i19$i) + 28)|0;
        $880 = (($tbase$255$i) + ($$sum12$i$i)|0);
        HEAP32[$880>>2] = $I7$0$i$i;
        $$sum13$i$i = (($$sum$i19$i) + 16)|0;
        $881 = (($tbase$255$i) + ($$sum13$i$i)|0);
        $$sum14$i$i = (($$sum$i19$i) + 20)|0;
        $882 = (($tbase$255$i) + ($$sum14$i$i)|0);
        HEAP32[$882>>2] = 0;
        HEAP32[$881>>2] = 0;
        $883 = HEAP32[(4196)>>2]|0;
        $884 = 1 << $I7$0$i$i;
        $885 = $883 & $884;
        $886 = ($885|0)==(0);
        if ($886) {
         $887 = $883 | $884;
         HEAP32[(4196)>>2] = $887;
         HEAP32[$879>>2] = $724;
         $$sum15$i$i = (($$sum$i19$i) + 24)|0;
         $888 = (($tbase$255$i) + ($$sum15$i$i)|0);
         HEAP32[$888>>2] = $879;
         $$sum16$i$i = (($$sum$i19$i) + 12)|0;
         $889 = (($tbase$255$i) + ($$sum16$i$i)|0);
         HEAP32[$889>>2] = $724;
         $$sum17$i$i = (($$sum$i19$i) + 8)|0;
         $890 = (($tbase$255$i) + ($$sum17$i$i)|0);
         HEAP32[$890>>2] = $724;
         break;
        }
        $891 = HEAP32[$879>>2]|0;
        $892 = ((($891)) + 4|0);
        $893 = HEAP32[$892>>2]|0;
        $894 = $893 & -8;
        $895 = ($894|0)==($qsize$0$i$i|0);
        L418: do {
         if ($895) {
          $T$0$lcssa$i25$i = $891;
         } else {
          $896 = ($I7$0$i$i|0)==(31);
          $897 = $I7$0$i$i >>> 1;
          $898 = (25 - ($897))|0;
          $899 = $896 ? 0 : $898;
          $900 = $qsize$0$i$i << $899;
          $K8$051$i$i = $900;$T$050$i$i = $891;
          while(1) {
           $907 = $K8$051$i$i >>> 31;
           $908 = (((($T$050$i$i)) + 16|0) + ($907<<2)|0);
           $903 = HEAP32[$908>>2]|0;
           $909 = ($903|0)==(0|0);
           if ($909) {
            $$lcssa = $908;$T$050$i$i$lcssa = $T$050$i$i;
            break;
           }
           $901 = $K8$051$i$i << 1;
           $902 = ((($903)) + 4|0);
           $904 = HEAP32[$902>>2]|0;
           $905 = $904 & -8;
           $906 = ($905|0)==($qsize$0$i$i|0);
           if ($906) {
            $T$0$lcssa$i25$i = $903;
            break L418;
           } else {
            $K8$051$i$i = $901;$T$050$i$i = $903;
           }
          }
          $910 = HEAP32[(4208)>>2]|0;
          $911 = ($$lcssa>>>0)<($910>>>0);
          if ($911) {
           _abort();
           // unreachable;
          } else {
           HEAP32[$$lcssa>>2] = $724;
           $$sum23$i$i = (($$sum$i19$i) + 24)|0;
           $912 = (($tbase$255$i) + ($$sum23$i$i)|0);
           HEAP32[$912>>2] = $T$050$i$i$lcssa;
           $$sum24$i$i = (($$sum$i19$i) + 12)|0;
           $913 = (($tbase$255$i) + ($$sum24$i$i)|0);
           HEAP32[$913>>2] = $724;
           $$sum25$i$i = (($$sum$i19$i) + 8)|0;
           $914 = (($tbase$255$i) + ($$sum25$i$i)|0);
           HEAP32[$914>>2] = $724;
           break L324;
          }
         }
        } while(0);
        $915 = ((($T$0$lcssa$i25$i)) + 8|0);
        $916 = HEAP32[$915>>2]|0;
        $917 = HEAP32[(4208)>>2]|0;
        $918 = ($916>>>0)>=($917>>>0);
        $not$$i26$i = ($T$0$lcssa$i25$i>>>0)>=($917>>>0);
        $919 = $918 & $not$$i26$i;
        if ($919) {
         $920 = ((($916)) + 12|0);
         HEAP32[$920>>2] = $724;
         HEAP32[$915>>2] = $724;
         $$sum20$i$i = (($$sum$i19$i) + 8)|0;
         $921 = (($tbase$255$i) + ($$sum20$i$i)|0);
         HEAP32[$921>>2] = $916;
         $$sum21$i$i = (($$sum$i19$i) + 12)|0;
         $922 = (($tbase$255$i) + ($$sum21$i$i)|0);
         HEAP32[$922>>2] = $T$0$lcssa$i25$i;
         $$sum22$i$i = (($$sum$i19$i) + 24)|0;
         $923 = (($tbase$255$i) + ($$sum22$i$i)|0);
         HEAP32[$923>>2] = 0;
         break;
        } else {
         _abort();
         // unreachable;
        }
       }
      } while(0);
      $$sum1819$i$i = $711 | 8;
      $924 = (($tbase$255$i) + ($$sum1819$i$i)|0);
      $mem$0 = $924;
      return ($mem$0|0);
     } else {
      $sp$0$i$i$i = (4640);
     }
    }
    while(1) {
     $925 = HEAP32[$sp$0$i$i$i>>2]|0;
     $926 = ($925>>>0)>($635>>>0);
     if (!($926)) {
      $927 = ((($sp$0$i$i$i)) + 4|0);
      $928 = HEAP32[$927>>2]|0;
      $929 = (($925) + ($928)|0);
      $930 = ($929>>>0)>($635>>>0);
      if ($930) {
       $$lcssa215 = $925;$$lcssa216 = $928;$$lcssa217 = $929;
       break;
      }
     }
     $931 = ((($sp$0$i$i$i)) + 8|0);
     $932 = HEAP32[$931>>2]|0;
     $sp$0$i$i$i = $932;
    }
    $$sum$i14$i = (($$lcssa216) + -47)|0;
    $$sum1$i15$i = (($$lcssa216) + -39)|0;
    $933 = (($$lcssa215) + ($$sum1$i15$i)|0);
    $934 = $933;
    $935 = $934 & 7;
    $936 = ($935|0)==(0);
    $937 = (0 - ($934))|0;
    $938 = $937 & 7;
    $939 = $936 ? 0 : $938;
    $$sum2$i16$i = (($$sum$i14$i) + ($939))|0;
    $940 = (($$lcssa215) + ($$sum2$i16$i)|0);
    $941 = ((($635)) + 16|0);
    $942 = ($940>>>0)<($941>>>0);
    $943 = $942 ? $635 : $940;
    $944 = ((($943)) + 8|0);
    $945 = (($tsize$254$i) + -40)|0;
    $946 = ((($tbase$255$i)) + 8|0);
    $947 = $946;
    $948 = $947 & 7;
    $949 = ($948|0)==(0);
    $950 = (0 - ($947))|0;
    $951 = $950 & 7;
    $952 = $949 ? 0 : $951;
    $953 = (($tbase$255$i) + ($952)|0);
    $954 = (($945) - ($952))|0;
    HEAP32[(4216)>>2] = $953;
    HEAP32[(4204)>>2] = $954;
    $955 = $954 | 1;
    $$sum$i$i$i = (($952) + 4)|0;
    $956 = (($tbase$255$i) + ($$sum$i$i$i)|0);
    HEAP32[$956>>2] = $955;
    $$sum2$i$i$i = (($tsize$254$i) + -36)|0;
    $957 = (($tbase$255$i) + ($$sum2$i$i$i)|0);
    HEAP32[$957>>2] = 40;
    $958 = HEAP32[(4680)>>2]|0;
    HEAP32[(4220)>>2] = $958;
    $959 = ((($943)) + 4|0);
    HEAP32[$959>>2] = 27;
    ;HEAP32[$944>>2]=HEAP32[(4640)>>2]|0;HEAP32[$944+4>>2]=HEAP32[(4640)+4>>2]|0;HEAP32[$944+8>>2]=HEAP32[(4640)+8>>2]|0;HEAP32[$944+12>>2]=HEAP32[(4640)+12>>2]|0;
    HEAP32[(4640)>>2] = $tbase$255$i;
    HEAP32[(4644)>>2] = $tsize$254$i;
    HEAP32[(4652)>>2] = 0;
    HEAP32[(4648)>>2] = $944;
    $960 = ((($943)) + 28|0);
    HEAP32[$960>>2] = 7;
    $961 = ((($943)) + 32|0);
    $962 = ($961>>>0)<($$lcssa217>>>0);
    if ($962) {
     $964 = $960;
     while(1) {
      $963 = ((($964)) + 4|0);
      HEAP32[$963>>2] = 7;
      $965 = ((($964)) + 8|0);
      $966 = ($965>>>0)<($$lcssa217>>>0);
      if ($966) {
       $964 = $963;
      } else {
       break;
      }
     }
    }
    $967 = ($943|0)==($635|0);
    if (!($967)) {
     $968 = $943;
     $969 = $635;
     $970 = (($968) - ($969))|0;
     $971 = HEAP32[$959>>2]|0;
     $972 = $971 & -2;
     HEAP32[$959>>2] = $972;
     $973 = $970 | 1;
     $974 = ((($635)) + 4|0);
     HEAP32[$974>>2] = $973;
     HEAP32[$943>>2] = $970;
     $975 = $970 >>> 3;
     $976 = ($970>>>0)<(256);
     if ($976) {
      $977 = $975 << 1;
      $978 = (4232 + ($977<<2)|0);
      $979 = HEAP32[4192>>2]|0;
      $980 = 1 << $975;
      $981 = $979 & $980;
      $982 = ($981|0)==(0);
      if ($982) {
       $983 = $979 | $980;
       HEAP32[4192>>2] = $983;
       $$pre$i$i = (($977) + 2)|0;
       $$pre14$i$i = (4232 + ($$pre$i$i<<2)|0);
       $$pre$phi$i$iZ2D = $$pre14$i$i;$F$0$i$i = $978;
      } else {
       $$sum4$i$i = (($977) + 2)|0;
       $984 = (4232 + ($$sum4$i$i<<2)|0);
       $985 = HEAP32[$984>>2]|0;
       $986 = HEAP32[(4208)>>2]|0;
       $987 = ($985>>>0)<($986>>>0);
       if ($987) {
        _abort();
        // unreachable;
       } else {
        $$pre$phi$i$iZ2D = $984;$F$0$i$i = $985;
       }
      }
      HEAP32[$$pre$phi$i$iZ2D>>2] = $635;
      $988 = ((($F$0$i$i)) + 12|0);
      HEAP32[$988>>2] = $635;
      $989 = ((($635)) + 8|0);
      HEAP32[$989>>2] = $F$0$i$i;
      $990 = ((($635)) + 12|0);
      HEAP32[$990>>2] = $978;
      break;
     }
     $991 = $970 >>> 8;
     $992 = ($991|0)==(0);
     if ($992) {
      $I1$0$i$i = 0;
     } else {
      $993 = ($970>>>0)>(16777215);
      if ($993) {
       $I1$0$i$i = 31;
      } else {
       $994 = (($991) + 1048320)|0;
       $995 = $994 >>> 16;
       $996 = $995 & 8;
       $997 = $991 << $996;
       $998 = (($997) + 520192)|0;
       $999 = $998 >>> 16;
       $1000 = $999 & 4;
       $1001 = $1000 | $996;
       $1002 = $997 << $1000;
       $1003 = (($1002) + 245760)|0;
       $1004 = $1003 >>> 16;
       $1005 = $1004 & 2;
       $1006 = $1001 | $1005;
       $1007 = (14 - ($1006))|0;
       $1008 = $1002 << $1005;
       $1009 = $1008 >>> 15;
       $1010 = (($1007) + ($1009))|0;
       $1011 = $1010 << 1;
       $1012 = (($1010) + 7)|0;
       $1013 = $970 >>> $1012;
       $1014 = $1013 & 1;
       $1015 = $1014 | $1011;
       $I1$0$i$i = $1015;
      }
     }
     $1016 = (4496 + ($I1$0$i$i<<2)|0);
     $1017 = ((($635)) + 28|0);
     HEAP32[$1017>>2] = $I1$0$i$i;
     $1018 = ((($635)) + 20|0);
     HEAP32[$1018>>2] = 0;
     HEAP32[$941>>2] = 0;
     $1019 = HEAP32[(4196)>>2]|0;
     $1020 = 1 << $I1$0$i$i;
     $1021 = $1019 & $1020;
     $1022 = ($1021|0)==(0);
     if ($1022) {
      $1023 = $1019 | $1020;
      HEAP32[(4196)>>2] = $1023;
      HEAP32[$1016>>2] = $635;
      $1024 = ((($635)) + 24|0);
      HEAP32[$1024>>2] = $1016;
      $1025 = ((($635)) + 12|0);
      HEAP32[$1025>>2] = $635;
      $1026 = ((($635)) + 8|0);
      HEAP32[$1026>>2] = $635;
      break;
     }
     $1027 = HEAP32[$1016>>2]|0;
     $1028 = ((($1027)) + 4|0);
     $1029 = HEAP32[$1028>>2]|0;
     $1030 = $1029 & -8;
     $1031 = ($1030|0)==($970|0);
     L459: do {
      if ($1031) {
       $T$0$lcssa$i$i = $1027;
      } else {
       $1032 = ($I1$0$i$i|0)==(31);
       $1033 = $I1$0$i$i >>> 1;
       $1034 = (25 - ($1033))|0;
       $1035 = $1032 ? 0 : $1034;
       $1036 = $970 << $1035;
       $K2$07$i$i = $1036;$T$06$i$i = $1027;
       while(1) {
        $1043 = $K2$07$i$i >>> 31;
        $1044 = (((($T$06$i$i)) + 16|0) + ($1043<<2)|0);
        $1039 = HEAP32[$1044>>2]|0;
        $1045 = ($1039|0)==(0|0);
        if ($1045) {
         $$lcssa211 = $1044;$T$06$i$i$lcssa = $T$06$i$i;
         break;
        }
        $1037 = $K2$07$i$i << 1;
        $1038 = ((($1039)) + 4|0);
        $1040 = HEAP32[$1038>>2]|0;
        $1041 = $1040 & -8;
        $1042 = ($1041|0)==($970|0);
        if ($1042) {
         $T$0$lcssa$i$i = $1039;
         break L459;
        } else {
         $K2$07$i$i = $1037;$T$06$i$i = $1039;
        }
       }
       $1046 = HEAP32[(4208)>>2]|0;
       $1047 = ($$lcssa211>>>0)<($1046>>>0);
       if ($1047) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$$lcssa211>>2] = $635;
        $1048 = ((($635)) + 24|0);
        HEAP32[$1048>>2] = $T$06$i$i$lcssa;
        $1049 = ((($635)) + 12|0);
        HEAP32[$1049>>2] = $635;
        $1050 = ((($635)) + 8|0);
        HEAP32[$1050>>2] = $635;
        break L299;
       }
      }
     } while(0);
     $1051 = ((($T$0$lcssa$i$i)) + 8|0);
     $1052 = HEAP32[$1051>>2]|0;
     $1053 = HEAP32[(4208)>>2]|0;
     $1054 = ($1052>>>0)>=($1053>>>0);
     $not$$i$i = ($T$0$lcssa$i$i>>>0)>=($1053>>>0);
     $1055 = $1054 & $not$$i$i;
     if ($1055) {
      $1056 = ((($1052)) + 12|0);
      HEAP32[$1056>>2] = $635;
      HEAP32[$1051>>2] = $635;
      $1057 = ((($635)) + 8|0);
      HEAP32[$1057>>2] = $1052;
      $1058 = ((($635)) + 12|0);
      HEAP32[$1058>>2] = $T$0$lcssa$i$i;
      $1059 = ((($635)) + 24|0);
      HEAP32[$1059>>2] = 0;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   }
  } while(0);
  $1060 = HEAP32[(4204)>>2]|0;
  $1061 = ($1060>>>0)>($nb$0>>>0);
  if ($1061) {
   $1062 = (($1060) - ($nb$0))|0;
   HEAP32[(4204)>>2] = $1062;
   $1063 = HEAP32[(4216)>>2]|0;
   $1064 = (($1063) + ($nb$0)|0);
   HEAP32[(4216)>>2] = $1064;
   $1065 = $1062 | 1;
   $$sum$i32 = (($nb$0) + 4)|0;
   $1066 = (($1063) + ($$sum$i32)|0);
   HEAP32[$1066>>2] = $1065;
   $1067 = $nb$0 | 3;
   $1068 = ((($1063)) + 4|0);
   HEAP32[$1068>>2] = $1067;
   $1069 = ((($1063)) + 8|0);
   $mem$0 = $1069;
   return ($mem$0|0);
  }
 }
 $1070 = (___errno_location()|0);
 HEAP32[$1070>>2] = 12;
 $mem$0 = 0;
 return ($mem$0|0);
}
function _free($mem) {
 $mem = $mem|0;
 var $$lcssa = 0, $$pre = 0, $$pre$phi59Z2D = 0, $$pre$phi61Z2D = 0, $$pre$phiZ2D = 0, $$pre57 = 0, $$pre58 = 0, $$pre60 = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum1718 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0;
 var $$sum25 = 0, $$sum26 = 0, $$sum27 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0, $$sum31 = 0, $$sum5 = 0, $$sum67 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0;
 var $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0;
 var $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0;
 var $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0;
 var $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0;
 var $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0;
 var $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0;
 var $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0;
 var $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0;
 var $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0;
 var $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0;
 var $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0;
 var $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0;
 var $321 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I18$0 = 0, $K19$052 = 0, $R$0 = 0, $R$0$lcssa = 0, $R$1 = 0;
 var $R7$0 = 0, $R7$0$lcssa = 0, $R7$1 = 0, $RP$0 = 0, $RP$0$lcssa = 0, $RP9$0 = 0, $RP9$0$lcssa = 0, $T$0$lcssa = 0, $T$051 = 0, $T$051$lcssa = 0, $cond = 0, $cond47 = 0, $not$ = 0, $p$0 = 0, $psize$0 = 0, $psize$1 = 0, $sp$0$i = 0, $sp$0$in$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($mem|0)==(0|0);
 if ($0) {
  return;
 }
 $1 = ((($mem)) + -8|0);
 $2 = HEAP32[(4208)>>2]|0;
 $3 = ($1>>>0)<($2>>>0);
 if ($3) {
  _abort();
  // unreachable;
 }
 $4 = ((($mem)) + -4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 3;
 $7 = ($6|0)==(1);
 if ($7) {
  _abort();
  // unreachable;
 }
 $8 = $5 & -8;
 $$sum = (($8) + -8)|0;
 $9 = (($mem) + ($$sum)|0);
 $10 = $5 & 1;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ($6|0)==(0);
   if ($13) {
    return;
   }
   $$sum2 = (-8 - ($12))|0;
   $14 = (($mem) + ($$sum2)|0);
   $15 = (($12) + ($8))|0;
   $16 = ($14>>>0)<($2>>>0);
   if ($16) {
    _abort();
    // unreachable;
   }
   $17 = HEAP32[(4212)>>2]|0;
   $18 = ($14|0)==($17|0);
   if ($18) {
    $$sum3 = (($8) + -4)|0;
    $103 = (($mem) + ($$sum3)|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = $104 & 3;
    $106 = ($105|0)==(3);
    if (!($106)) {
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    HEAP32[(4200)>>2] = $15;
    $107 = $104 & -2;
    HEAP32[$103>>2] = $107;
    $108 = $15 | 1;
    $$sum20 = (($$sum2) + 4)|0;
    $109 = (($mem) + ($$sum20)|0);
    HEAP32[$109>>2] = $108;
    HEAP32[$9>>2] = $15;
    return;
   }
   $19 = $12 >>> 3;
   $20 = ($12>>>0)<(256);
   if ($20) {
    $$sum30 = (($$sum2) + 8)|0;
    $21 = (($mem) + ($$sum30)|0);
    $22 = HEAP32[$21>>2]|0;
    $$sum31 = (($$sum2) + 12)|0;
    $23 = (($mem) + ($$sum31)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $19 << 1;
    $26 = (4232 + ($25<<2)|0);
    $27 = ($22|0)==($26|0);
    if (!($27)) {
     $28 = ($22>>>0)<($2>>>0);
     if ($28) {
      _abort();
      // unreachable;
     }
     $29 = ((($22)) + 12|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($30|0)==($14|0);
     if (!($31)) {
      _abort();
      // unreachable;
     }
    }
    $32 = ($24|0)==($22|0);
    if ($32) {
     $33 = 1 << $19;
     $34 = $33 ^ -1;
     $35 = HEAP32[4192>>2]|0;
     $36 = $35 & $34;
     HEAP32[4192>>2] = $36;
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    $37 = ($24|0)==($26|0);
    if ($37) {
     $$pre60 = ((($24)) + 8|0);
     $$pre$phi61Z2D = $$pre60;
    } else {
     $38 = ($24>>>0)<($2>>>0);
     if ($38) {
      _abort();
      // unreachable;
     }
     $39 = ((($24)) + 8|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($40|0)==($14|0);
     if ($41) {
      $$pre$phi61Z2D = $39;
     } else {
      _abort();
      // unreachable;
     }
    }
    $42 = ((($22)) + 12|0);
    HEAP32[$42>>2] = $24;
    HEAP32[$$pre$phi61Z2D>>2] = $22;
    $p$0 = $14;$psize$0 = $15;
    break;
   }
   $$sum22 = (($$sum2) + 24)|0;
   $43 = (($mem) + ($$sum22)|0);
   $44 = HEAP32[$43>>2]|0;
   $$sum23 = (($$sum2) + 12)|0;
   $45 = (($mem) + ($$sum23)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==($14|0);
   do {
    if ($47) {
     $$sum25 = (($$sum2) + 20)|0;
     $57 = (($mem) + ($$sum25)|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      $$sum24 = (($$sum2) + 16)|0;
      $60 = (($mem) + ($$sum24)|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)==(0|0);
      if ($62) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $61;$RP$0 = $60;
      }
     } else {
      $R$0 = $58;$RP$0 = $57;
     }
     while(1) {
      $63 = ((($R$0)) + 20|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = ($64|0)==(0|0);
      if (!($65)) {
       $R$0 = $64;$RP$0 = $63;
       continue;
      }
      $66 = ((($R$0)) + 16|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = ($67|0)==(0|0);
      if ($68) {
       $R$0$lcssa = $R$0;$RP$0$lcssa = $RP$0;
       break;
      } else {
       $R$0 = $67;$RP$0 = $66;
      }
     }
     $69 = ($RP$0$lcssa>>>0)<($2>>>0);
     if ($69) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0$lcssa>>2] = 0;
      $R$1 = $R$0$lcssa;
      break;
     }
    } else {
     $$sum29 = (($$sum2) + 8)|0;
     $48 = (($mem) + ($$sum29)|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49>>>0)<($2>>>0);
     if ($50) {
      _abort();
      // unreachable;
     }
     $51 = ((($49)) + 12|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ($52|0)==($14|0);
     if (!($53)) {
      _abort();
      // unreachable;
     }
     $54 = ((($46)) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)==($14|0);
     if ($56) {
      HEAP32[$51>>2] = $46;
      HEAP32[$54>>2] = $49;
      $R$1 = $46;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $70 = ($44|0)==(0|0);
   if ($70) {
    $p$0 = $14;$psize$0 = $15;
   } else {
    $$sum26 = (($$sum2) + 28)|0;
    $71 = (($mem) + ($$sum26)|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (4496 + ($72<<2)|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($14|0)==($74|0);
    if ($75) {
     HEAP32[$73>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $76 = 1 << $72;
      $77 = $76 ^ -1;
      $78 = HEAP32[(4196)>>2]|0;
      $79 = $78 & $77;
      HEAP32[(4196)>>2] = $79;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    } else {
     $80 = HEAP32[(4208)>>2]|0;
     $81 = ($44>>>0)<($80>>>0);
     if ($81) {
      _abort();
      // unreachable;
     }
     $82 = ((($44)) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)==($14|0);
     if ($84) {
      HEAP32[$82>>2] = $R$1;
     } else {
      $85 = ((($44)) + 20|0);
      HEAP32[$85>>2] = $R$1;
     }
     $86 = ($R$1|0)==(0|0);
     if ($86) {
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
    $87 = HEAP32[(4208)>>2]|0;
    $88 = ($R$1>>>0)<($87>>>0);
    if ($88) {
     _abort();
     // unreachable;
    }
    $89 = ((($R$1)) + 24|0);
    HEAP32[$89>>2] = $44;
    $$sum27 = (($$sum2) + 16)|0;
    $90 = (($mem) + ($$sum27)|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    do {
     if (!($92)) {
      $93 = ($91>>>0)<($87>>>0);
      if ($93) {
       _abort();
       // unreachable;
      } else {
       $94 = ((($R$1)) + 16|0);
       HEAP32[$94>>2] = $91;
       $95 = ((($91)) + 24|0);
       HEAP32[$95>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum28 = (($$sum2) + 20)|0;
    $96 = (($mem) + ($$sum28)|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = ($97|0)==(0|0);
    if ($98) {
     $p$0 = $14;$psize$0 = $15;
    } else {
     $99 = HEAP32[(4208)>>2]|0;
     $100 = ($97>>>0)<($99>>>0);
     if ($100) {
      _abort();
      // unreachable;
     } else {
      $101 = ((($R$1)) + 20|0);
      HEAP32[$101>>2] = $97;
      $102 = ((($97)) + 24|0);
      HEAP32[$102>>2] = $R$1;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
   }
  } else {
   $p$0 = $1;$psize$0 = $8;
  }
 } while(0);
 $110 = ($p$0>>>0)<($9>>>0);
 if (!($110)) {
  _abort();
  // unreachable;
 }
 $$sum19 = (($8) + -4)|0;
 $111 = (($mem) + ($$sum19)|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = $112 & 1;
 $114 = ($113|0)==(0);
 if ($114) {
  _abort();
  // unreachable;
 }
 $115 = $112 & 2;
 $116 = ($115|0)==(0);
 if ($116) {
  $117 = HEAP32[(4216)>>2]|0;
  $118 = ($9|0)==($117|0);
  if ($118) {
   $119 = HEAP32[(4204)>>2]|0;
   $120 = (($119) + ($psize$0))|0;
   HEAP32[(4204)>>2] = $120;
   HEAP32[(4216)>>2] = $p$0;
   $121 = $120 | 1;
   $122 = ((($p$0)) + 4|0);
   HEAP32[$122>>2] = $121;
   $123 = HEAP32[(4212)>>2]|0;
   $124 = ($p$0|0)==($123|0);
   if (!($124)) {
    return;
   }
   HEAP32[(4212)>>2] = 0;
   HEAP32[(4200)>>2] = 0;
   return;
  }
  $125 = HEAP32[(4212)>>2]|0;
  $126 = ($9|0)==($125|0);
  if ($126) {
   $127 = HEAP32[(4200)>>2]|0;
   $128 = (($127) + ($psize$0))|0;
   HEAP32[(4200)>>2] = $128;
   HEAP32[(4212)>>2] = $p$0;
   $129 = $128 | 1;
   $130 = ((($p$0)) + 4|0);
   HEAP32[$130>>2] = $129;
   $131 = (($p$0) + ($128)|0);
   HEAP32[$131>>2] = $128;
   return;
  }
  $132 = $112 & -8;
  $133 = (($132) + ($psize$0))|0;
  $134 = $112 >>> 3;
  $135 = ($112>>>0)<(256);
  do {
   if ($135) {
    $136 = (($mem) + ($8)|0);
    $137 = HEAP32[$136>>2]|0;
    $$sum1718 = $8 | 4;
    $138 = (($mem) + ($$sum1718)|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = $134 << 1;
    $141 = (4232 + ($140<<2)|0);
    $142 = ($137|0)==($141|0);
    if (!($142)) {
     $143 = HEAP32[(4208)>>2]|0;
     $144 = ($137>>>0)<($143>>>0);
     if ($144) {
      _abort();
      // unreachable;
     }
     $145 = ((($137)) + 12|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = ($146|0)==($9|0);
     if (!($147)) {
      _abort();
      // unreachable;
     }
    }
    $148 = ($139|0)==($137|0);
    if ($148) {
     $149 = 1 << $134;
     $150 = $149 ^ -1;
     $151 = HEAP32[4192>>2]|0;
     $152 = $151 & $150;
     HEAP32[4192>>2] = $152;
     break;
    }
    $153 = ($139|0)==($141|0);
    if ($153) {
     $$pre58 = ((($139)) + 8|0);
     $$pre$phi59Z2D = $$pre58;
    } else {
     $154 = HEAP32[(4208)>>2]|0;
     $155 = ($139>>>0)<($154>>>0);
     if ($155) {
      _abort();
      // unreachable;
     }
     $156 = ((($139)) + 8|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = ($157|0)==($9|0);
     if ($158) {
      $$pre$phi59Z2D = $156;
     } else {
      _abort();
      // unreachable;
     }
    }
    $159 = ((($137)) + 12|0);
    HEAP32[$159>>2] = $139;
    HEAP32[$$pre$phi59Z2D>>2] = $137;
   } else {
    $$sum5 = (($8) + 16)|0;
    $160 = (($mem) + ($$sum5)|0);
    $161 = HEAP32[$160>>2]|0;
    $$sum67 = $8 | 4;
    $162 = (($mem) + ($$sum67)|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = ($163|0)==($9|0);
    do {
     if ($164) {
      $$sum9 = (($8) + 12)|0;
      $175 = (($mem) + ($$sum9)|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = ($176|0)==(0|0);
      if ($177) {
       $$sum8 = (($8) + 8)|0;
       $178 = (($mem) + ($$sum8)|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = ($179|0)==(0|0);
       if ($180) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $179;$RP9$0 = $178;
       }
      } else {
       $R7$0 = $176;$RP9$0 = $175;
      }
      while(1) {
       $181 = ((($R7$0)) + 20|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = ($182|0)==(0|0);
       if (!($183)) {
        $R7$0 = $182;$RP9$0 = $181;
        continue;
       }
       $184 = ((($R7$0)) + 16|0);
       $185 = HEAP32[$184>>2]|0;
       $186 = ($185|0)==(0|0);
       if ($186) {
        $R7$0$lcssa = $R7$0;$RP9$0$lcssa = $RP9$0;
        break;
       } else {
        $R7$0 = $185;$RP9$0 = $184;
       }
      }
      $187 = HEAP32[(4208)>>2]|0;
      $188 = ($RP9$0$lcssa>>>0)<($187>>>0);
      if ($188) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0$lcssa>>2] = 0;
       $R7$1 = $R7$0$lcssa;
       break;
      }
     } else {
      $165 = (($mem) + ($8)|0);
      $166 = HEAP32[$165>>2]|0;
      $167 = HEAP32[(4208)>>2]|0;
      $168 = ($166>>>0)<($167>>>0);
      if ($168) {
       _abort();
       // unreachable;
      }
      $169 = ((($166)) + 12|0);
      $170 = HEAP32[$169>>2]|0;
      $171 = ($170|0)==($9|0);
      if (!($171)) {
       _abort();
       // unreachable;
      }
      $172 = ((($163)) + 8|0);
      $173 = HEAP32[$172>>2]|0;
      $174 = ($173|0)==($9|0);
      if ($174) {
       HEAP32[$169>>2] = $163;
       HEAP32[$172>>2] = $166;
       $R7$1 = $163;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $189 = ($161|0)==(0|0);
    if (!($189)) {
     $$sum12 = (($8) + 20)|0;
     $190 = (($mem) + ($$sum12)|0);
     $191 = HEAP32[$190>>2]|0;
     $192 = (4496 + ($191<<2)|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ($9|0)==($193|0);
     if ($194) {
      HEAP32[$192>>2] = $R7$1;
      $cond47 = ($R7$1|0)==(0|0);
      if ($cond47) {
       $195 = 1 << $191;
       $196 = $195 ^ -1;
       $197 = HEAP32[(4196)>>2]|0;
       $198 = $197 & $196;
       HEAP32[(4196)>>2] = $198;
       break;
      }
     } else {
      $199 = HEAP32[(4208)>>2]|0;
      $200 = ($161>>>0)<($199>>>0);
      if ($200) {
       _abort();
       // unreachable;
      }
      $201 = ((($161)) + 16|0);
      $202 = HEAP32[$201>>2]|0;
      $203 = ($202|0)==($9|0);
      if ($203) {
       HEAP32[$201>>2] = $R7$1;
      } else {
       $204 = ((($161)) + 20|0);
       HEAP32[$204>>2] = $R7$1;
      }
      $205 = ($R7$1|0)==(0|0);
      if ($205) {
       break;
      }
     }
     $206 = HEAP32[(4208)>>2]|0;
     $207 = ($R7$1>>>0)<($206>>>0);
     if ($207) {
      _abort();
      // unreachable;
     }
     $208 = ((($R7$1)) + 24|0);
     HEAP32[$208>>2] = $161;
     $$sum13 = (($8) + 8)|0;
     $209 = (($mem) + ($$sum13)|0);
     $210 = HEAP32[$209>>2]|0;
     $211 = ($210|0)==(0|0);
     do {
      if (!($211)) {
       $212 = ($210>>>0)<($206>>>0);
       if ($212) {
        _abort();
        // unreachable;
       } else {
        $213 = ((($R7$1)) + 16|0);
        HEAP32[$213>>2] = $210;
        $214 = ((($210)) + 24|0);
        HEAP32[$214>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum14 = (($8) + 12)|0;
     $215 = (($mem) + ($$sum14)|0);
     $216 = HEAP32[$215>>2]|0;
     $217 = ($216|0)==(0|0);
     if (!($217)) {
      $218 = HEAP32[(4208)>>2]|0;
      $219 = ($216>>>0)<($218>>>0);
      if ($219) {
       _abort();
       // unreachable;
      } else {
       $220 = ((($R7$1)) + 20|0);
       HEAP32[$220>>2] = $216;
       $221 = ((($216)) + 24|0);
       HEAP32[$221>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $222 = $133 | 1;
  $223 = ((($p$0)) + 4|0);
  HEAP32[$223>>2] = $222;
  $224 = (($p$0) + ($133)|0);
  HEAP32[$224>>2] = $133;
  $225 = HEAP32[(4212)>>2]|0;
  $226 = ($p$0|0)==($225|0);
  if ($226) {
   HEAP32[(4200)>>2] = $133;
   return;
  } else {
   $psize$1 = $133;
  }
 } else {
  $227 = $112 & -2;
  HEAP32[$111>>2] = $227;
  $228 = $psize$0 | 1;
  $229 = ((($p$0)) + 4|0);
  HEAP32[$229>>2] = $228;
  $230 = (($p$0) + ($psize$0)|0);
  HEAP32[$230>>2] = $psize$0;
  $psize$1 = $psize$0;
 }
 $231 = $psize$1 >>> 3;
 $232 = ($psize$1>>>0)<(256);
 if ($232) {
  $233 = $231 << 1;
  $234 = (4232 + ($233<<2)|0);
  $235 = HEAP32[4192>>2]|0;
  $236 = 1 << $231;
  $237 = $235 & $236;
  $238 = ($237|0)==(0);
  if ($238) {
   $239 = $235 | $236;
   HEAP32[4192>>2] = $239;
   $$pre = (($233) + 2)|0;
   $$pre57 = (4232 + ($$pre<<2)|0);
   $$pre$phiZ2D = $$pre57;$F16$0 = $234;
  } else {
   $$sum11 = (($233) + 2)|0;
   $240 = (4232 + ($$sum11<<2)|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = HEAP32[(4208)>>2]|0;
   $243 = ($241>>>0)<($242>>>0);
   if ($243) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $240;$F16$0 = $241;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $p$0;
  $244 = ((($F16$0)) + 12|0);
  HEAP32[$244>>2] = $p$0;
  $245 = ((($p$0)) + 8|0);
  HEAP32[$245>>2] = $F16$0;
  $246 = ((($p$0)) + 12|0);
  HEAP32[$246>>2] = $234;
  return;
 }
 $247 = $psize$1 >>> 8;
 $248 = ($247|0)==(0);
 if ($248) {
  $I18$0 = 0;
 } else {
  $249 = ($psize$1>>>0)>(16777215);
  if ($249) {
   $I18$0 = 31;
  } else {
   $250 = (($247) + 1048320)|0;
   $251 = $250 >>> 16;
   $252 = $251 & 8;
   $253 = $247 << $252;
   $254 = (($253) + 520192)|0;
   $255 = $254 >>> 16;
   $256 = $255 & 4;
   $257 = $256 | $252;
   $258 = $253 << $256;
   $259 = (($258) + 245760)|0;
   $260 = $259 >>> 16;
   $261 = $260 & 2;
   $262 = $257 | $261;
   $263 = (14 - ($262))|0;
   $264 = $258 << $261;
   $265 = $264 >>> 15;
   $266 = (($263) + ($265))|0;
   $267 = $266 << 1;
   $268 = (($266) + 7)|0;
   $269 = $psize$1 >>> $268;
   $270 = $269 & 1;
   $271 = $270 | $267;
   $I18$0 = $271;
  }
 }
 $272 = (4496 + ($I18$0<<2)|0);
 $273 = ((($p$0)) + 28|0);
 HEAP32[$273>>2] = $I18$0;
 $274 = ((($p$0)) + 16|0);
 $275 = ((($p$0)) + 20|0);
 HEAP32[$275>>2] = 0;
 HEAP32[$274>>2] = 0;
 $276 = HEAP32[(4196)>>2]|0;
 $277 = 1 << $I18$0;
 $278 = $276 & $277;
 $279 = ($278|0)==(0);
 L199: do {
  if ($279) {
   $280 = $276 | $277;
   HEAP32[(4196)>>2] = $280;
   HEAP32[$272>>2] = $p$0;
   $281 = ((($p$0)) + 24|0);
   HEAP32[$281>>2] = $272;
   $282 = ((($p$0)) + 12|0);
   HEAP32[$282>>2] = $p$0;
   $283 = ((($p$0)) + 8|0);
   HEAP32[$283>>2] = $p$0;
  } else {
   $284 = HEAP32[$272>>2]|0;
   $285 = ((($284)) + 4|0);
   $286 = HEAP32[$285>>2]|0;
   $287 = $286 & -8;
   $288 = ($287|0)==($psize$1|0);
   L202: do {
    if ($288) {
     $T$0$lcssa = $284;
    } else {
     $289 = ($I18$0|0)==(31);
     $290 = $I18$0 >>> 1;
     $291 = (25 - ($290))|0;
     $292 = $289 ? 0 : $291;
     $293 = $psize$1 << $292;
     $K19$052 = $293;$T$051 = $284;
     while(1) {
      $300 = $K19$052 >>> 31;
      $301 = (((($T$051)) + 16|0) + ($300<<2)|0);
      $296 = HEAP32[$301>>2]|0;
      $302 = ($296|0)==(0|0);
      if ($302) {
       $$lcssa = $301;$T$051$lcssa = $T$051;
       break;
      }
      $294 = $K19$052 << 1;
      $295 = ((($296)) + 4|0);
      $297 = HEAP32[$295>>2]|0;
      $298 = $297 & -8;
      $299 = ($298|0)==($psize$1|0);
      if ($299) {
       $T$0$lcssa = $296;
       break L202;
      } else {
       $K19$052 = $294;$T$051 = $296;
      }
     }
     $303 = HEAP32[(4208)>>2]|0;
     $304 = ($$lcssa>>>0)<($303>>>0);
     if ($304) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$$lcssa>>2] = $p$0;
      $305 = ((($p$0)) + 24|0);
      HEAP32[$305>>2] = $T$051$lcssa;
      $306 = ((($p$0)) + 12|0);
      HEAP32[$306>>2] = $p$0;
      $307 = ((($p$0)) + 8|0);
      HEAP32[$307>>2] = $p$0;
      break L199;
     }
    }
   } while(0);
   $308 = ((($T$0$lcssa)) + 8|0);
   $309 = HEAP32[$308>>2]|0;
   $310 = HEAP32[(4208)>>2]|0;
   $311 = ($309>>>0)>=($310>>>0);
   $not$ = ($T$0$lcssa>>>0)>=($310>>>0);
   $312 = $311 & $not$;
   if ($312) {
    $313 = ((($309)) + 12|0);
    HEAP32[$313>>2] = $p$0;
    HEAP32[$308>>2] = $p$0;
    $314 = ((($p$0)) + 8|0);
    HEAP32[$314>>2] = $309;
    $315 = ((($p$0)) + 12|0);
    HEAP32[$315>>2] = $T$0$lcssa;
    $316 = ((($p$0)) + 24|0);
    HEAP32[$316>>2] = 0;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $317 = HEAP32[(4224)>>2]|0;
 $318 = (($317) + -1)|0;
 HEAP32[(4224)>>2] = $318;
 $319 = ($318|0)==(0);
 if ($319) {
  $sp$0$in$i = (4648);
 } else {
  return;
 }
 while(1) {
  $sp$0$i = HEAP32[$sp$0$in$i>>2]|0;
  $320 = ($sp$0$i|0)==(0|0);
  $321 = ((($sp$0$i)) + 8|0);
  if ($320) {
   break;
  } else {
   $sp$0$in$i = $321;
  }
 }
 HEAP32[(4224)>>2] = -1;
 return;
}
function runPostSets() {

}
function _i64Subtract(a, b, c, d) {
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a - c)>>>0;
    h = (b - d)>>>0;
    h = (b - d - (((c>>>0) > (a>>>0))|0))>>>0; // Borrow one from high word to low word on underflow.
    return ((tempRet0 = h,l|0)|0);
}
function _i64Add(a, b, c, d) {
    /*
      x = a + b*2^32
      y = c + d*2^32
      result = l + h*2^32
    */
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a + c)>>>0;
    h = (b + d + (((l>>>0) < (a>>>0))|0))>>>0; // Add carry from low word to high word on overflow.
    return ((tempRet0 = h,l|0)|0);
}
function _memset(ptr, value, num) {
    ptr = ptr|0; value = value|0; num = num|0;
    var stop = 0, value4 = 0, stop4 = 0, unaligned = 0;
    stop = (ptr + num)|0;
    if ((num|0) >= 20) {
      // This is unaligned, but quite large, so work hard to get to aligned settings
      value = value & 0xff;
      unaligned = ptr & 3;
      value4 = value | (value << 8) | (value << 16) | (value << 24);
      stop4 = stop & ~3;
      if (unaligned) {
        unaligned = (ptr + 4 - unaligned)|0;
        while ((ptr|0) < (unaligned|0)) { // no need to check for stop, since we have large num
          HEAP8[((ptr)>>0)]=value;
          ptr = (ptr+1)|0;
        }
      }
      while ((ptr|0) < (stop4|0)) {
        HEAP32[((ptr)>>2)]=value4;
        ptr = (ptr+4)|0;
      }
    }
    while ((ptr|0) < (stop|0)) {
      HEAP8[((ptr)>>0)]=value;
      ptr = (ptr+1)|0;
    }
    return (ptr-num)|0;
}
function _bitshift64Lshr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >>> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = 0;
    return (high >>> (bits - 32))|0;
}
function _bitshift64Shl(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = (high << bits) | ((low&(ander << (32 - bits))) >>> (32 - bits));
      return low << bits;
    }
    tempRet0 = low << (bits - 32);
    return 0;
}
function _strlen(ptr) {
    ptr = ptr|0;
    var curr = 0;
    curr = ptr;
    while (((HEAP8[((curr)>>0)])|0)) {
      curr = (curr + 1)|0;
    }
    return (curr - ptr)|0;
}
function _memcpy(dest, src, num) {
    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if ((num|0) >= 4096) return _emscripten_memcpy_big(dest|0, src|0, num|0)|0;
    ret = dest|0;
    if ((dest&3) == (src&3)) {
      while (dest & 3) {
        if ((num|0) == 0) return ret|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
        dest = (dest+1)|0;
        src = (src+1)|0;
        num = (num-1)|0;
      }
      while ((num|0) >= 4) {
        HEAP32[((dest)>>2)]=((HEAP32[((src)>>2)])|0);
        dest = (dest+4)|0;
        src = (src+4)|0;
        num = (num-4)|0;
      }
    }
    while ((num|0) > 0) {
      HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      dest = (dest+1)|0;
      src = (src+1)|0;
      num = (num-1)|0;
    }
    return ret|0;
}
function _bitshift64Ashr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = (high|0) < 0 ? -1 : 0;
    return (high >> (bits - 32))|0;
  }
function _llvm_cttz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((cttz_i8)+(x & 0xff))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((cttz_i8)+(x >>> 24))>>0)])|0) + 24)|0;
  }

// ======== compiled code from system/lib/compiler-rt , see readme therein
function ___muldsi3($a, $b) {
  $a = $a | 0;
  $b = $b | 0;
  var $1 = 0, $2 = 0, $3 = 0, $6 = 0, $8 = 0, $11 = 0, $12 = 0;
  $1 = $a & 65535;
  $2 = $b & 65535;
  $3 = Math_imul($2, $1) | 0;
  $6 = $a >>> 16;
  $8 = ($3 >>> 16) + (Math_imul($2, $6) | 0) | 0;
  $11 = $b >>> 16;
  $12 = Math_imul($11, $1) | 0;
  return (tempRet0 = (($8 >>> 16) + (Math_imul($11, $6) | 0) | 0) + ((($8 & 65535) + $12 | 0) >>> 16) | 0, 0 | ($8 + $12 << 16 | $3 & 65535)) | 0;
}
function ___divdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $7$0 = 0, $7$1 = 0, $8$0 = 0, $10$0 = 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  $7$0 = $2$0 ^ $1$0;
  $7$1 = $2$1 ^ $1$1;
  $8$0 = ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, 0) | 0;
  $10$0 = _i64Subtract($8$0 ^ $7$0, tempRet0 ^ $7$1, $7$0, $7$1) | 0;
  return $10$0 | 0;
}
function ___remdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $10$0 = 0, $10$1 = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, $rem) | 0;
  $10$0 = _i64Subtract(HEAP32[$rem >> 2] ^ $1$0, HEAP32[$rem + 4 >> 2] ^ $1$1, $1$0, $1$1) | 0;
  $10$1 = tempRet0;
  STACKTOP = __stackBase__;
  return (tempRet0 = $10$1, $10$0) | 0;
}
function ___muldi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $x_sroa_0_0_extract_trunc = 0, $y_sroa_0_0_extract_trunc = 0, $1$0 = 0, $1$1 = 0, $2 = 0;
  $x_sroa_0_0_extract_trunc = $a$0;
  $y_sroa_0_0_extract_trunc = $b$0;
  $1$0 = ___muldsi3($x_sroa_0_0_extract_trunc, $y_sroa_0_0_extract_trunc) | 0;
  $1$1 = tempRet0;
  $2 = Math_imul($a$1, $y_sroa_0_0_extract_trunc) | 0;
  return (tempRet0 = ((Math_imul($b$1, $x_sroa_0_0_extract_trunc) | 0) + $2 | 0) + $1$1 | $1$1 & 0, 0 | $1$0 & -1) | 0;
}
function ___udivdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0;
  $1$0 = ___udivmoddi4($a$0, $a$1, $b$0, $b$1, 0) | 0;
  return $1$0 | 0;
}
function ___uremdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) | 0;
  STACKTOP = __stackBase__;
  return (tempRet0 = HEAP32[$rem + 4 >> 2] | 0, HEAP32[$rem >> 2] | 0) | 0;
}
function ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  $rem = $rem | 0;
  var $n_sroa_0_0_extract_trunc = 0, $n_sroa_1_4_extract_shift$0 = 0, $n_sroa_1_4_extract_trunc = 0, $d_sroa_0_0_extract_trunc = 0, $d_sroa_1_4_extract_shift$0 = 0, $d_sroa_1_4_extract_trunc = 0, $4 = 0, $17 = 0, $37 = 0, $49 = 0, $51 = 0, $57 = 0, $58 = 0, $66 = 0, $78 = 0, $86 = 0, $88 = 0, $89 = 0, $91 = 0, $92 = 0, $95 = 0, $105 = 0, $117 = 0, $119 = 0, $125 = 0, $126 = 0, $130 = 0, $q_sroa_1_1_ph = 0, $q_sroa_0_1_ph = 0, $r_sroa_1_1_ph = 0, $r_sroa_0_1_ph = 0, $sr_1_ph = 0, $d_sroa_0_0_insert_insert99$0 = 0, $d_sroa_0_0_insert_insert99$1 = 0, $137$0 = 0, $137$1 = 0, $carry_0203 = 0, $sr_1202 = 0, $r_sroa_0_1201 = 0, $r_sroa_1_1200 = 0, $q_sroa_0_1199 = 0, $q_sroa_1_1198 = 0, $147 = 0, $149 = 0, $r_sroa_0_0_insert_insert42$0 = 0, $r_sroa_0_0_insert_insert42$1 = 0, $150$1 = 0, $151$0 = 0, $152 = 0, $154$0 = 0, $r_sroa_0_0_extract_trunc = 0, $r_sroa_1_4_extract_trunc = 0, $155 = 0, $carry_0_lcssa$0 = 0, $carry_0_lcssa$1 = 0, $r_sroa_0_1_lcssa = 0, $r_sroa_1_1_lcssa = 0, $q_sroa_0_1_lcssa = 0, $q_sroa_1_1_lcssa = 0, $q_sroa_0_0_insert_ext75$0 = 0, $q_sroa_0_0_insert_ext75$1 = 0, $q_sroa_0_0_insert_insert77$1 = 0, $_0$0 = 0, $_0$1 = 0;
  $n_sroa_0_0_extract_trunc = $a$0;
  $n_sroa_1_4_extract_shift$0 = $a$1;
  $n_sroa_1_4_extract_trunc = $n_sroa_1_4_extract_shift$0;
  $d_sroa_0_0_extract_trunc = $b$0;
  $d_sroa_1_4_extract_shift$0 = $b$1;
  $d_sroa_1_4_extract_trunc = $d_sroa_1_4_extract_shift$0;
  if (($n_sroa_1_4_extract_trunc | 0) == 0) {
    $4 = ($rem | 0) != 0;
    if (($d_sroa_1_4_extract_trunc | 0) == 0) {
      if ($4) {
        HEAP32[$rem >> 2] = ($n_sroa_0_0_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
        HEAP32[$rem + 4 >> 2] = 0;
      }
      $_0$1 = 0;
      $_0$0 = ($n_sroa_0_0_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$4) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    }
  }
  $17 = ($d_sroa_1_4_extract_trunc | 0) == 0;
  do {
    if (($d_sroa_0_0_extract_trunc | 0) == 0) {
      if ($17) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
          HEAP32[$rem + 4 >> 2] = 0;
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      if (($n_sroa_0_0_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0;
          HEAP32[$rem + 4 >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_1_4_extract_trunc >>> 0);
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_1_4_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $37 = $d_sroa_1_4_extract_trunc - 1 | 0;
      if (($37 & $d_sroa_1_4_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0 | $a$0 & -1;
          HEAP32[$rem + 4 >> 2] = $37 & $n_sroa_1_4_extract_trunc | $a$1 & 0;
        }
        $_0$1 = 0;
        $_0$0 = $n_sroa_1_4_extract_trunc >>> ((_llvm_cttz_i32($d_sroa_1_4_extract_trunc | 0) | 0) >>> 0);
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $49 = Math_clz32($d_sroa_1_4_extract_trunc | 0) | 0;
      $51 = $49 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
      if ($51 >>> 0 <= 30) {
        $57 = $51 + 1 | 0;
        $58 = 31 - $51 | 0;
        $sr_1_ph = $57;
        $r_sroa_0_1_ph = $n_sroa_1_4_extract_trunc << $58 | $n_sroa_0_0_extract_trunc >>> ($57 >>> 0);
        $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($57 >>> 0);
        $q_sroa_0_1_ph = 0;
        $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $58;
        break;
      }
      if (($rem | 0) == 0) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = 0 | $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$17) {
        $117 = Math_clz32($d_sroa_1_4_extract_trunc | 0) | 0;
        $119 = $117 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        if ($119 >>> 0 <= 31) {
          $125 = $119 + 1 | 0;
          $126 = 31 - $119 | 0;
          $130 = $119 - 31 >> 31;
          $sr_1_ph = $125;
          $r_sroa_0_1_ph = $n_sroa_0_0_extract_trunc >>> ($125 >>> 0) & $130 | $n_sroa_1_4_extract_trunc << $126;
          $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($125 >>> 0) & $130;
          $q_sroa_0_1_ph = 0;
          $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $126;
          break;
        }
        if (($rem | 0) == 0) {
          $_0$1 = 0;
          $_0$0 = 0;
          return (tempRet0 = $_0$1, $_0$0) | 0;
        }
        HEAP32[$rem >> 2] = 0 | $a$0 & -1;
        HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $66 = $d_sroa_0_0_extract_trunc - 1 | 0;
      if (($66 & $d_sroa_0_0_extract_trunc | 0) != 0) {
        $86 = (Math_clz32($d_sroa_0_0_extract_trunc | 0) | 0) + 33 | 0;
        $88 = $86 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        $89 = 64 - $88 | 0;
        $91 = 32 - $88 | 0;
        $92 = $91 >> 31;
        $95 = $88 - 32 | 0;
        $105 = $95 >> 31;
        $sr_1_ph = $88;
        $r_sroa_0_1_ph = $91 - 1 >> 31 & $n_sroa_1_4_extract_trunc >>> ($95 >>> 0) | ($n_sroa_1_4_extract_trunc << $91 | $n_sroa_0_0_extract_trunc >>> ($88 >>> 0)) & $105;
        $r_sroa_1_1_ph = $105 & $n_sroa_1_4_extract_trunc >>> ($88 >>> 0);
        $q_sroa_0_1_ph = $n_sroa_0_0_extract_trunc << $89 & $92;
        $q_sroa_1_1_ph = ($n_sroa_1_4_extract_trunc << $89 | $n_sroa_0_0_extract_trunc >>> ($95 >>> 0)) & $92 | $n_sroa_0_0_extract_trunc << $91 & $88 - 33 >> 31;
        break;
      }
      if (($rem | 0) != 0) {
        HEAP32[$rem >> 2] = $66 & $n_sroa_0_0_extract_trunc;
        HEAP32[$rem + 4 >> 2] = 0;
      }
      if (($d_sroa_0_0_extract_trunc | 0) == 1) {
        $_0$1 = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$0 = 0 | $a$0 & -1;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      } else {
        $78 = _llvm_cttz_i32($d_sroa_0_0_extract_trunc | 0) | 0;
        $_0$1 = 0 | $n_sroa_1_4_extract_trunc >>> ($78 >>> 0);
        $_0$0 = $n_sroa_1_4_extract_trunc << 32 - $78 | $n_sroa_0_0_extract_trunc >>> ($78 >>> 0) | 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
    }
  } while (0);
  if (($sr_1_ph | 0) == 0) {
    $q_sroa_1_1_lcssa = $q_sroa_1_1_ph;
    $q_sroa_0_1_lcssa = $q_sroa_0_1_ph;
    $r_sroa_1_1_lcssa = $r_sroa_1_1_ph;
    $r_sroa_0_1_lcssa = $r_sroa_0_1_ph;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = 0;
  } else {
    $d_sroa_0_0_insert_insert99$0 = 0 | $b$0 & -1;
    $d_sroa_0_0_insert_insert99$1 = $d_sroa_1_4_extract_shift$0 | $b$1 & 0;
    $137$0 = _i64Add($d_sroa_0_0_insert_insert99$0 | 0, $d_sroa_0_0_insert_insert99$1 | 0, -1, -1) | 0;
    $137$1 = tempRet0;
    $q_sroa_1_1198 = $q_sroa_1_1_ph;
    $q_sroa_0_1199 = $q_sroa_0_1_ph;
    $r_sroa_1_1200 = $r_sroa_1_1_ph;
    $r_sroa_0_1201 = $r_sroa_0_1_ph;
    $sr_1202 = $sr_1_ph;
    $carry_0203 = 0;
    while (1) {
      $147 = $q_sroa_0_1199 >>> 31 | $q_sroa_1_1198 << 1;
      $149 = $carry_0203 | $q_sroa_0_1199 << 1;
      $r_sroa_0_0_insert_insert42$0 = 0 | ($r_sroa_0_1201 << 1 | $q_sroa_1_1198 >>> 31);
      $r_sroa_0_0_insert_insert42$1 = $r_sroa_0_1201 >>> 31 | $r_sroa_1_1200 << 1 | 0;
      _i64Subtract($137$0, $137$1, $r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1) | 0;
      $150$1 = tempRet0;
      $151$0 = $150$1 >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1;
      $152 = $151$0 & 1;
      $154$0 = _i64Subtract($r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1, $151$0 & $d_sroa_0_0_insert_insert99$0, ((($150$1 | 0) < 0 ? -1 : 0) >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1) & $d_sroa_0_0_insert_insert99$1) | 0;
      $r_sroa_0_0_extract_trunc = $154$0;
      $r_sroa_1_4_extract_trunc = tempRet0;
      $155 = $sr_1202 - 1 | 0;
      if (($155 | 0) == 0) {
        break;
      } else {
        $q_sroa_1_1198 = $147;
        $q_sroa_0_1199 = $149;
        $r_sroa_1_1200 = $r_sroa_1_4_extract_trunc;
        $r_sroa_0_1201 = $r_sroa_0_0_extract_trunc;
        $sr_1202 = $155;
        $carry_0203 = $152;
      }
    }
    $q_sroa_1_1_lcssa = $147;
    $q_sroa_0_1_lcssa = $149;
    $r_sroa_1_1_lcssa = $r_sroa_1_4_extract_trunc;
    $r_sroa_0_1_lcssa = $r_sroa_0_0_extract_trunc;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = $152;
  }
  $q_sroa_0_0_insert_ext75$0 = $q_sroa_0_1_lcssa;
  $q_sroa_0_0_insert_ext75$1 = 0;
  $q_sroa_0_0_insert_insert77$1 = $q_sroa_1_1_lcssa | $q_sroa_0_0_insert_ext75$1;
  if (($rem | 0) != 0) {
    HEAP32[$rem >> 2] = 0 | $r_sroa_0_1_lcssa;
    HEAP32[$rem + 4 >> 2] = $r_sroa_1_1_lcssa | 0;
  }
  $_0$1 = (0 | $q_sroa_0_0_insert_ext75$0) >>> 31 | $q_sroa_0_0_insert_insert77$1 << 1 | ($q_sroa_0_0_insert_ext75$1 << 1 | $q_sroa_0_0_insert_ext75$0 >>> 31) & 0 | $carry_0_lcssa$1;
  $_0$0 = ($q_sroa_0_0_insert_ext75$0 << 1 | 0 >>> 31) & -2 | $carry_0_lcssa$0;
  return (tempRet0 = $_0$1, $_0$0) | 0;
}
// =======================================================================



  
function dynCall_iiii(index,a1,a2,a3) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0;
  return FUNCTION_TABLE_iiii[index&1](a1|0,a2|0,a3|0)|0;
}

function b0(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; nullFunc_iiii(0);return 0; }

// EMSCRIPTEN_END_FUNCS
var FUNCTION_TABLE_iiii = [b0,_sn_write];

  return { _i64Subtract: _i64Subtract, _free: _free, _i64Add: _i64Add, _dlinsol: _dlinsol, _strlen: _strlen, _zlinsol: _zlinsol, _memset: _memset, _malloc: _malloc, _memcpy: _memcpy, _bitshift64Lshr: _bitshift64Lshr, _bitshift64Shl: _bitshift64Shl, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, establishStackSpace: establishStackSpace, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_iiii: dynCall_iiii };
})
// EMSCRIPTEN_END_ASM
(Module.asmGlobalArg, Module.asmLibraryArg, buffer);
var real__i64Subtract = asm["_i64Subtract"]; asm["_i64Subtract"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__i64Subtract.apply(null, arguments);
};

var real__free = asm["_free"]; asm["_free"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__free.apply(null, arguments);
};

var real__i64Add = asm["_i64Add"]; asm["_i64Add"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__i64Add.apply(null, arguments);
};

var real__dlinsol = asm["_dlinsol"]; asm["_dlinsol"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__dlinsol.apply(null, arguments);
};

var real__strlen = asm["_strlen"]; asm["_strlen"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__strlen.apply(null, arguments);
};

var real__zlinsol = asm["_zlinsol"]; asm["_zlinsol"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__zlinsol.apply(null, arguments);
};

var real__malloc = asm["_malloc"]; asm["_malloc"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__malloc.apply(null, arguments);
};

var real__bitshift64Lshr = asm["_bitshift64Lshr"]; asm["_bitshift64Lshr"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__bitshift64Lshr.apply(null, arguments);
};

var real__bitshift64Shl = asm["_bitshift64Shl"]; asm["_bitshift64Shl"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__bitshift64Shl.apply(null, arguments);
};
var _i64Subtract = Module["_i64Subtract"] = asm["_i64Subtract"];
var _free = Module["_free"] = asm["_free"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var _i64Add = Module["_i64Add"] = asm["_i64Add"];
var _dlinsol = Module["_dlinsol"] = asm["_dlinsol"];
var _strlen = Module["_strlen"] = asm["_strlen"];
var _zlinsol = Module["_zlinsol"] = asm["_zlinsol"];
var _memset = Module["_memset"] = asm["_memset"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _bitshift64Lshr = Module["_bitshift64Lshr"] = asm["_bitshift64Lshr"];
var _bitshift64Shl = Module["_bitshift64Shl"] = asm["_bitshift64Shl"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
;

Runtime.stackAlloc = asm['stackAlloc'];
Runtime.stackSave = asm['stackSave'];
Runtime.stackRestore = asm['stackRestore'];
Runtime.establishStackSpace = asm['establishStackSpace'];

Runtime.setTempRet0 = asm['setTempRet0'];
Runtime.getTempRet0 = asm['getTempRet0'];


// TODO: strip out parts of this we do not need

//======= begin closure i64 code =======

// Copyright 2009 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Defines a Long class for representing a 64-bit two's-complement
 * integer value, which faithfully simulates the behavior of a Java "long". This
 * implementation is derived from LongLib in GWT.
 *
 */

var i64Math = (function() { // Emscripten wrapper
  var goog = { math: {} };


  /**
   * Constructs a 64-bit two's-complement integer, given its low and high 32-bit
   * values as *signed* integers.  See the from* functions below for more
   * convenient ways of constructing Longs.
   *
   * The internal representation of a long is the two given signed, 32-bit values.
   * We use 32-bit pieces because these are the size of integers on which
   * Javascript performs bit-operations.  For operations like addition and
   * multiplication, we split each number into 16-bit pieces, which can easily be
   * multiplied within Javascript's floating-point representation without overflow
   * or change in sign.
   *
   * In the algorithms below, we frequently reduce the negative case to the
   * positive case by negating the input(s) and then post-processing the result.
   * Note that we must ALWAYS check specially whether those values are MIN_VALUE
   * (-2^63) because -MIN_VALUE == MIN_VALUE (since 2^63 cannot be represented as
   * a positive number, it overflows back into a negative).  Not handling this
   * case would often result in infinite recursion.
   *
   * @param {number} low  The low (signed) 32 bits of the long.
   * @param {number} high  The high (signed) 32 bits of the long.
   * @constructor
   */
  goog.math.Long = function(low, high) {
    /**
     * @type {number}
     * @private
     */
    this.low_ = low | 0;  // force into 32 signed bits.

    /**
     * @type {number}
     * @private
     */
    this.high_ = high | 0;  // force into 32 signed bits.
  };


  // NOTE: Common constant values ZERO, ONE, NEG_ONE, etc. are defined below the
  // from* methods on which they depend.


  /**
   * A cache of the Long representations of small integer values.
   * @type {!Object}
   * @private
   */
  goog.math.Long.IntCache_ = {};


  /**
   * Returns a Long representing the given (32-bit) integer value.
   * @param {number} value The 32-bit integer in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromInt = function(value) {
    if (-128 <= value && value < 128) {
      var cachedObj = goog.math.Long.IntCache_[value];
      if (cachedObj) {
        return cachedObj;
      }
    }

    var obj = new goog.math.Long(value | 0, value < 0 ? -1 : 0);
    if (-128 <= value && value < 128) {
      goog.math.Long.IntCache_[value] = obj;
    }
    return obj;
  };


  /**
   * Returns a Long representing the given value, provided that it is a finite
   * number.  Otherwise, zero is returned.
   * @param {number} value The number in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromNumber = function(value) {
    if (isNaN(value) || !isFinite(value)) {
      return goog.math.Long.ZERO;
    } else if (value <= -goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MIN_VALUE;
    } else if (value + 1 >= goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MAX_VALUE;
    } else if (value < 0) {
      return goog.math.Long.fromNumber(-value).negate();
    } else {
      return new goog.math.Long(
          (value % goog.math.Long.TWO_PWR_32_DBL_) | 0,
          (value / goog.math.Long.TWO_PWR_32_DBL_) | 0);
    }
  };


  /**
   * Returns a Long representing the 64-bit integer that comes by concatenating
   * the given high and low bits.  Each is assumed to use 32 bits.
   * @param {number} lowBits The low 32-bits.
   * @param {number} highBits The high 32-bits.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromBits = function(lowBits, highBits) {
    return new goog.math.Long(lowBits, highBits);
  };


  /**
   * Returns a Long representation of the given string, written using the given
   * radix.
   * @param {string} str The textual representation of the Long.
   * @param {number=} opt_radix The radix in which the text is written.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromString = function(str, opt_radix) {
    if (str.length == 0) {
      throw Error('number format error: empty string');
    }

    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (str.charAt(0) == '-') {
      return goog.math.Long.fromString(str.substring(1), radix).negate();
    } else if (str.indexOf('-') >= 0) {
      throw Error('number format error: interior "-" character: ' + str);
    }

    // Do several (8) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 8));

    var result = goog.math.Long.ZERO;
    for (var i = 0; i < str.length; i += 8) {
      var size = Math.min(8, str.length - i);
      var value = parseInt(str.substring(i, i + size), radix);
      if (size < 8) {
        var power = goog.math.Long.fromNumber(Math.pow(radix, size));
        result = result.multiply(power).add(goog.math.Long.fromNumber(value));
      } else {
        result = result.multiply(radixToPower);
        result = result.add(goog.math.Long.fromNumber(value));
      }
    }
    return result;
  };


  // NOTE: the compiler should inline these constant values below and then remove
  // these variables, so there should be no runtime penalty for these.


  /**
   * Number used repeated below in calculations.  This must appear before the
   * first call to any from* function below.
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_16_DBL_ = 1 << 16;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_24_DBL_ = 1 << 24;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_32_DBL_ =
      goog.math.Long.TWO_PWR_16_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_31_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ / 2;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_48_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_64_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_32_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_63_DBL_ =
      goog.math.Long.TWO_PWR_64_DBL_ / 2;


  /** @type {!goog.math.Long} */
  goog.math.Long.ZERO = goog.math.Long.fromInt(0);


  /** @type {!goog.math.Long} */
  goog.math.Long.ONE = goog.math.Long.fromInt(1);


  /** @type {!goog.math.Long} */
  goog.math.Long.NEG_ONE = goog.math.Long.fromInt(-1);


  /** @type {!goog.math.Long} */
  goog.math.Long.MAX_VALUE =
      goog.math.Long.fromBits(0xFFFFFFFF | 0, 0x7FFFFFFF | 0);


  /** @type {!goog.math.Long} */
  goog.math.Long.MIN_VALUE = goog.math.Long.fromBits(0, 0x80000000 | 0);


  /**
   * @type {!goog.math.Long}
   * @private
   */
  goog.math.Long.TWO_PWR_24_ = goog.math.Long.fromInt(1 << 24);


  /** @return {number} The value, assuming it is a 32-bit integer. */
  goog.math.Long.prototype.toInt = function() {
    return this.low_;
  };


  /** @return {number} The closest floating-point representation to this value. */
  goog.math.Long.prototype.toNumber = function() {
    return this.high_ * goog.math.Long.TWO_PWR_32_DBL_ +
           this.getLowBitsUnsigned();
  };


  /**
   * @param {number=} opt_radix The radix in which the text should be written.
   * @return {string} The textual representation of this value.
   */
  goog.math.Long.prototype.toString = function(opt_radix) {
    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (this.isZero()) {
      return '0';
    }

    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        // We need to change the Long value before it can be negated, so we remove
        // the bottom-most digit in this base and then recurse to do the rest.
        var radixLong = goog.math.Long.fromNumber(radix);
        var div = this.div(radixLong);
        var rem = div.multiply(radixLong).subtract(this);
        return div.toString(radix) + rem.toInt().toString(radix);
      } else {
        return '-' + this.negate().toString(radix);
      }
    }

    // Do several (6) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 6));

    var rem = this;
    var result = '';
    while (true) {
      var remDiv = rem.div(radixToPower);
      var intval = rem.subtract(remDiv.multiply(radixToPower)).toInt();
      var digits = intval.toString(radix);

      rem = remDiv;
      if (rem.isZero()) {
        return digits + result;
      } else {
        while (digits.length < 6) {
          digits = '0' + digits;
        }
        result = '' + digits + result;
      }
    }
  };


  /** @return {number} The high 32-bits as a signed value. */
  goog.math.Long.prototype.getHighBits = function() {
    return this.high_;
  };


  /** @return {number} The low 32-bits as a signed value. */
  goog.math.Long.prototype.getLowBits = function() {
    return this.low_;
  };


  /** @return {number} The low 32-bits as an unsigned value. */
  goog.math.Long.prototype.getLowBitsUnsigned = function() {
    return (this.low_ >= 0) ?
        this.low_ : goog.math.Long.TWO_PWR_32_DBL_ + this.low_;
  };


  /**
   * @return {number} Returns the number of bits needed to represent the absolute
   *     value of this Long.
   */
  goog.math.Long.prototype.getNumBitsAbs = function() {
    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        return 64;
      } else {
        return this.negate().getNumBitsAbs();
      }
    } else {
      var val = this.high_ != 0 ? this.high_ : this.low_;
      for (var bit = 31; bit > 0; bit--) {
        if ((val & (1 << bit)) != 0) {
          break;
        }
      }
      return this.high_ != 0 ? bit + 33 : bit + 1;
    }
  };


  /** @return {boolean} Whether this value is zero. */
  goog.math.Long.prototype.isZero = function() {
    return this.high_ == 0 && this.low_ == 0;
  };


  /** @return {boolean} Whether this value is negative. */
  goog.math.Long.prototype.isNegative = function() {
    return this.high_ < 0;
  };


  /** @return {boolean} Whether this value is odd. */
  goog.math.Long.prototype.isOdd = function() {
    return (this.low_ & 1) == 1;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long equals the other.
   */
  goog.math.Long.prototype.equals = function(other) {
    return (this.high_ == other.high_) && (this.low_ == other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long does not equal the other.
   */
  goog.math.Long.prototype.notEquals = function(other) {
    return (this.high_ != other.high_) || (this.low_ != other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than the other.
   */
  goog.math.Long.prototype.lessThan = function(other) {
    return this.compare(other) < 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than or equal to the other.
   */
  goog.math.Long.prototype.lessThanOrEqual = function(other) {
    return this.compare(other) <= 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than the other.
   */
  goog.math.Long.prototype.greaterThan = function(other) {
    return this.compare(other) > 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than or equal to the other.
   */
  goog.math.Long.prototype.greaterThanOrEqual = function(other) {
    return this.compare(other) >= 0;
  };


  /**
   * Compares this Long with the given one.
   * @param {goog.math.Long} other Long to compare against.
   * @return {number} 0 if they are the same, 1 if the this is greater, and -1
   *     if the given one is greater.
   */
  goog.math.Long.prototype.compare = function(other) {
    if (this.equals(other)) {
      return 0;
    }

    var thisNeg = this.isNegative();
    var otherNeg = other.isNegative();
    if (thisNeg && !otherNeg) {
      return -1;
    }
    if (!thisNeg && otherNeg) {
      return 1;
    }

    // at this point, the signs are the same, so subtraction will not overflow
    if (this.subtract(other).isNegative()) {
      return -1;
    } else {
      return 1;
    }
  };


  /** @return {!goog.math.Long} The negation of this value. */
  goog.math.Long.prototype.negate = function() {
    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.MIN_VALUE;
    } else {
      return this.not().add(goog.math.Long.ONE);
    }
  };


  /**
   * Returns the sum of this and the given Long.
   * @param {goog.math.Long} other Long to add to this one.
   * @return {!goog.math.Long} The sum of this and the given Long.
   */
  goog.math.Long.prototype.add = function(other) {
    // Divide each number into 4 chunks of 16 bits, and then sum the chunks.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 + b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 + b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 + b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 + b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns the difference of this and the given Long.
   * @param {goog.math.Long} other Long to subtract from this.
   * @return {!goog.math.Long} The difference of this and the given Long.
   */
  goog.math.Long.prototype.subtract = function(other) {
    return this.add(other.negate());
  };


  /**
   * Returns the product of this and the given long.
   * @param {goog.math.Long} other Long to multiply with this.
   * @return {!goog.math.Long} The product of this and the other.
   */
  goog.math.Long.prototype.multiply = function(other) {
    if (this.isZero()) {
      return goog.math.Long.ZERO;
    } else if (other.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return other.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return this.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().multiply(other.negate());
      } else {
        return this.negate().multiply(other).negate();
      }
    } else if (other.isNegative()) {
      return this.multiply(other.negate()).negate();
    }

    // If both longs are small, use float multiplication
    if (this.lessThan(goog.math.Long.TWO_PWR_24_) &&
        other.lessThan(goog.math.Long.TWO_PWR_24_)) {
      return goog.math.Long.fromNumber(this.toNumber() * other.toNumber());
    }

    // Divide each long into 4 chunks of 16 bits, and then add up 4x4 products.
    // We can skip products that would overflow.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 * b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 * b00;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c16 += a00 * b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 * b00;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a16 * b16;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a00 * b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 * b00 + a32 * b16 + a16 * b32 + a00 * b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns this Long divided by the given one.
   * @param {goog.math.Long} other Long by which to divide.
   * @return {!goog.math.Long} This Long divided by the given one.
   */
  goog.math.Long.prototype.div = function(other) {
    if (other.isZero()) {
      throw Error('division by zero');
    } else if (this.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      if (other.equals(goog.math.Long.ONE) ||
          other.equals(goog.math.Long.NEG_ONE)) {
        return goog.math.Long.MIN_VALUE;  // recall that -MIN_VALUE == MIN_VALUE
      } else if (other.equals(goog.math.Long.MIN_VALUE)) {
        return goog.math.Long.ONE;
      } else {
        // At this point, we have |other| >= 2, so |this/other| < |MIN_VALUE|.
        var halfThis = this.shiftRight(1);
        var approx = halfThis.div(other).shiftLeft(1);
        if (approx.equals(goog.math.Long.ZERO)) {
          return other.isNegative() ? goog.math.Long.ONE : goog.math.Long.NEG_ONE;
        } else {
          var rem = this.subtract(other.multiply(approx));
          var result = approx.add(rem.div(other));
          return result;
        }
      }
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().div(other.negate());
      } else {
        return this.negate().div(other).negate();
      }
    } else if (other.isNegative()) {
      return this.div(other.negate()).negate();
    }

    // Repeat the following until the remainder is less than other:  find a
    // floating-point that approximates remainder / other *from below*, add this
    // into the result, and subtract it from the remainder.  It is critical that
    // the approximate value is less than or equal to the real value so that the
    // remainder never becomes negative.
    var res = goog.math.Long.ZERO;
    var rem = this;
    while (rem.greaterThanOrEqual(other)) {
      // Approximate the result of division. This may be a little greater or
      // smaller than the actual value.
      var approx = Math.max(1, Math.floor(rem.toNumber() / other.toNumber()));

      // We will tweak the approximate result by changing it in the 48-th digit or
      // the smallest non-fractional digit, whichever is larger.
      var log2 = Math.ceil(Math.log(approx) / Math.LN2);
      var delta = (log2 <= 48) ? 1 : Math.pow(2, log2 - 48);

      // Decrease the approximation until it is smaller than the remainder.  Note
      // that if it is too large, the product overflows and is negative.
      var approxRes = goog.math.Long.fromNumber(approx);
      var approxRem = approxRes.multiply(other);
      while (approxRem.isNegative() || approxRem.greaterThan(rem)) {
        approx -= delta;
        approxRes = goog.math.Long.fromNumber(approx);
        approxRem = approxRes.multiply(other);
      }

      // We know the answer can't be zero... and actually, zero would cause
      // infinite recursion since we would make no progress.
      if (approxRes.isZero()) {
        approxRes = goog.math.Long.ONE;
      }

      res = res.add(approxRes);
      rem = rem.subtract(approxRem);
    }
    return res;
  };


  /**
   * Returns this Long modulo the given one.
   * @param {goog.math.Long} other Long by which to mod.
   * @return {!goog.math.Long} This Long modulo the given one.
   */
  goog.math.Long.prototype.modulo = function(other) {
    return this.subtract(this.div(other).multiply(other));
  };


  /** @return {!goog.math.Long} The bitwise-NOT of this value. */
  goog.math.Long.prototype.not = function() {
    return goog.math.Long.fromBits(~this.low_, ~this.high_);
  };


  /**
   * Returns the bitwise-AND of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to AND.
   * @return {!goog.math.Long} The bitwise-AND of this and the other.
   */
  goog.math.Long.prototype.and = function(other) {
    return goog.math.Long.fromBits(this.low_ & other.low_,
                                   this.high_ & other.high_);
  };


  /**
   * Returns the bitwise-OR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to OR.
   * @return {!goog.math.Long} The bitwise-OR of this and the other.
   */
  goog.math.Long.prototype.or = function(other) {
    return goog.math.Long.fromBits(this.low_ | other.low_,
                                   this.high_ | other.high_);
  };


  /**
   * Returns the bitwise-XOR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to XOR.
   * @return {!goog.math.Long} The bitwise-XOR of this and the other.
   */
  goog.math.Long.prototype.xor = function(other) {
    return goog.math.Long.fromBits(this.low_ ^ other.low_,
                                   this.high_ ^ other.high_);
  };


  /**
   * Returns this Long with bits shifted to the left by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the left by the given amount.
   */
  goog.math.Long.prototype.shiftLeft = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var low = this.low_;
      if (numBits < 32) {
        var high = this.high_;
        return goog.math.Long.fromBits(
            low << numBits,
            (high << numBits) | (low >>> (32 - numBits)));
      } else {
        return goog.math.Long.fromBits(0, low << (numBits - 32));
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount.
   */
  goog.math.Long.prototype.shiftRight = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >> numBits);
      } else {
        return goog.math.Long.fromBits(
            high >> (numBits - 32),
            high >= 0 ? 0 : -1);
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount, with
   * the new top bits matching the current sign bit.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount, with
   *     zeros placed into the new leading bits.
   */
  goog.math.Long.prototype.shiftRightUnsigned = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >>> numBits);
      } else if (numBits == 32) {
        return goog.math.Long.fromBits(high, 0);
      } else {
        return goog.math.Long.fromBits(high >>> (numBits - 32), 0);
      }
    }
  };

  //======= begin jsbn =======

  var navigator = { appName: 'Modern Browser' }; // polyfill a little

  // Copyright (c) 2005  Tom Wu
  // All Rights Reserved.
  // http://www-cs-students.stanford.edu/~tjw/jsbn/

  /*
   * Copyright (c) 2003-2005  Tom Wu
   * All Rights Reserved.
   *
   * Permission is hereby granted, free of charge, to any person obtaining
   * a copy of this software and associated documentation files (the
   * "Software"), to deal in the Software without restriction, including
   * without limitation the rights to use, copy, modify, merge, publish,
   * distribute, sublicense, and/or sell copies of the Software, and to
   * permit persons to whom the Software is furnished to do so, subject to
   * the following conditions:
   *
   * The above copyright notice and this permission notice shall be
   * included in all copies or substantial portions of the Software.
   *
   * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
   * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
   * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
   *
   * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
   * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
   * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
   * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
   * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   *
   * In addition, the following condition applies:
   *
   * All redistributions must retain an intact copy of this copyright notice
   * and disclaimer.
   */

  // Basic JavaScript BN library - subset useful for RSA encryption.

  // Bits per digit
  var dbits;

  // JavaScript engine analysis
  var canary = 0xdeadbeefcafe;
  var j_lm = ((canary&0xffffff)==0xefcafe);

  // (public) Constructor
  function BigInteger(a,b,c) {
    if(a != null)
      if("number" == typeof a) this.fromNumber(a,b,c);
      else if(b == null && "string" != typeof a) this.fromString(a,256);
      else this.fromString(a,b);
  }

  // return new, unset BigInteger
  function nbi() { return new BigInteger(null); }

  // am: Compute w_j += (x*this_i), propagate carries,
  // c is initial carry, returns final carry.
  // c < 3*dvalue, x < 2*dvalue, this_i < dvalue
  // We need to select the fastest one that works in this environment.

  // am1: use a single mult and divide to get the high bits,
  // max digit bits should be 26 because
  // max internal value = 2*dvalue^2-2*dvalue (< 2^53)
  function am1(i,x,w,j,c,n) {
    while(--n >= 0) {
      var v = x*this[i++]+w[j]+c;
      c = Math.floor(v/0x4000000);
      w[j++] = v&0x3ffffff;
    }
    return c;
  }
  // am2 avoids a big mult-and-extract completely.
  // Max digit bits should be <= 30 because we do bitwise ops
  // on values up to 2*hdvalue^2-hdvalue-1 (< 2^31)
  function am2(i,x,w,j,c,n) {
    var xl = x&0x7fff, xh = x>>15;
    while(--n >= 0) {
      var l = this[i]&0x7fff;
      var h = this[i++]>>15;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x7fff)<<15)+w[j]+(c&0x3fffffff);
      c = (l>>>30)+(m>>>15)+xh*h+(c>>>30);
      w[j++] = l&0x3fffffff;
    }
    return c;
  }
  // Alternately, set max digit bits to 28 since some
  // browsers slow down when dealing with 32-bit numbers.
  function am3(i,x,w,j,c,n) {
    var xl = x&0x3fff, xh = x>>14;
    while(--n >= 0) {
      var l = this[i]&0x3fff;
      var h = this[i++]>>14;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x3fff)<<14)+w[j]+c;
      c = (l>>28)+(m>>14)+xh*h;
      w[j++] = l&0xfffffff;
    }
    return c;
  }
  if(j_lm && (navigator.appName == "Microsoft Internet Explorer")) {
    BigInteger.prototype.am = am2;
    dbits = 30;
  }
  else if(j_lm && (navigator.appName != "Netscape")) {
    BigInteger.prototype.am = am1;
    dbits = 26;
  }
  else { // Mozilla/Netscape seems to prefer am3
    BigInteger.prototype.am = am3;
    dbits = 28;
  }

  BigInteger.prototype.DB = dbits;
  BigInteger.prototype.DM = ((1<<dbits)-1);
  BigInteger.prototype.DV = (1<<dbits);

  var BI_FP = 52;
  BigInteger.prototype.FV = Math.pow(2,BI_FP);
  BigInteger.prototype.F1 = BI_FP-dbits;
  BigInteger.prototype.F2 = 2*dbits-BI_FP;

  // Digit conversions
  var BI_RM = "0123456789abcdefghijklmnopqrstuvwxyz";
  var BI_RC = new Array();
  var rr,vv;
  rr = "0".charCodeAt(0);
  for(vv = 0; vv <= 9; ++vv) BI_RC[rr++] = vv;
  rr = "a".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;
  rr = "A".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;

  function int2char(n) { return BI_RM.charAt(n); }
  function intAt(s,i) {
    var c = BI_RC[s.charCodeAt(i)];
    return (c==null)?-1:c;
  }

  // (protected) copy this to r
  function bnpCopyTo(r) {
    for(var i = this.t-1; i >= 0; --i) r[i] = this[i];
    r.t = this.t;
    r.s = this.s;
  }

  // (protected) set from integer value x, -DV <= x < DV
  function bnpFromInt(x) {
    this.t = 1;
    this.s = (x<0)?-1:0;
    if(x > 0) this[0] = x;
    else if(x < -1) this[0] = x+DV;
    else this.t = 0;
  }

  // return bigint initialized to value
  function nbv(i) { var r = nbi(); r.fromInt(i); return r; }

  // (protected) set from string and radix
  function bnpFromString(s,b) {
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 256) k = 8; // byte array
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else { this.fromRadix(s,b); return; }
    this.t = 0;
    this.s = 0;
    var i = s.length, mi = false, sh = 0;
    while(--i >= 0) {
      var x = (k==8)?s[i]&0xff:intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-") mi = true;
        continue;
      }
      mi = false;
      if(sh == 0)
        this[this.t++] = x;
      else if(sh+k > this.DB) {
        this[this.t-1] |= (x&((1<<(this.DB-sh))-1))<<sh;
        this[this.t++] = (x>>(this.DB-sh));
      }
      else
        this[this.t-1] |= x<<sh;
      sh += k;
      if(sh >= this.DB) sh -= this.DB;
    }
    if(k == 8 && (s[0]&0x80) != 0) {
      this.s = -1;
      if(sh > 0) this[this.t-1] |= ((1<<(this.DB-sh))-1)<<sh;
    }
    this.clamp();
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) clamp off excess high words
  function bnpClamp() {
    var c = this.s&this.DM;
    while(this.t > 0 && this[this.t-1] == c) --this.t;
  }

  // (public) return string representation in given radix
  function bnToString(b) {
    if(this.s < 0) return "-"+this.negate().toString(b);
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else return this.toRadix(b);
    var km = (1<<k)-1, d, m = false, r = "", i = this.t;
    var p = this.DB-(i*this.DB)%k;
    if(i-- > 0) {
      if(p < this.DB && (d = this[i]>>p) > 0) { m = true; r = int2char(d); }
      while(i >= 0) {
        if(p < k) {
          d = (this[i]&((1<<p)-1))<<(k-p);
          d |= this[--i]>>(p+=this.DB-k);
        }
        else {
          d = (this[i]>>(p-=k))&km;
          if(p <= 0) { p += this.DB; --i; }
        }
        if(d > 0) m = true;
        if(m) r += int2char(d);
      }
    }
    return m?r:"0";
  }

  // (public) -this
  function bnNegate() { var r = nbi(); BigInteger.ZERO.subTo(this,r); return r; }

  // (public) |this|
  function bnAbs() { return (this.s<0)?this.negate():this; }

  // (public) return + if this > a, - if this < a, 0 if equal
  function bnCompareTo(a) {
    var r = this.s-a.s;
    if(r != 0) return r;
    var i = this.t;
    r = i-a.t;
    if(r != 0) return (this.s<0)?-r:r;
    while(--i >= 0) if((r=this[i]-a[i]) != 0) return r;
    return 0;
  }

  // returns bit length of the integer x
  function nbits(x) {
    var r = 1, t;
    if((t=x>>>16) != 0) { x = t; r += 16; }
    if((t=x>>8) != 0) { x = t; r += 8; }
    if((t=x>>4) != 0) { x = t; r += 4; }
    if((t=x>>2) != 0) { x = t; r += 2; }
    if((t=x>>1) != 0) { x = t; r += 1; }
    return r;
  }

  // (public) return the number of bits in "this"
  function bnBitLength() {
    if(this.t <= 0) return 0;
    return this.DB*(this.t-1)+nbits(this[this.t-1]^(this.s&this.DM));
  }

  // (protected) r = this << n*DB
  function bnpDLShiftTo(n,r) {
    var i;
    for(i = this.t-1; i >= 0; --i) r[i+n] = this[i];
    for(i = n-1; i >= 0; --i) r[i] = 0;
    r.t = this.t+n;
    r.s = this.s;
  }

  // (protected) r = this >> n*DB
  function bnpDRShiftTo(n,r) {
    for(var i = n; i < this.t; ++i) r[i-n] = this[i];
    r.t = Math.max(this.t-n,0);
    r.s = this.s;
  }

  // (protected) r = this << n
  function bnpLShiftTo(n,r) {
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<cbs)-1;
    var ds = Math.floor(n/this.DB), c = (this.s<<bs)&this.DM, i;
    for(i = this.t-1; i >= 0; --i) {
      r[i+ds+1] = (this[i]>>cbs)|c;
      c = (this[i]&bm)<<bs;
    }
    for(i = ds-1; i >= 0; --i) r[i] = 0;
    r[ds] = c;
    r.t = this.t+ds+1;
    r.s = this.s;
    r.clamp();
  }

  // (protected) r = this >> n
  function bnpRShiftTo(n,r) {
    r.s = this.s;
    var ds = Math.floor(n/this.DB);
    if(ds >= this.t) { r.t = 0; return; }
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<bs)-1;
    r[0] = this[ds]>>bs;
    for(var i = ds+1; i < this.t; ++i) {
      r[i-ds-1] |= (this[i]&bm)<<cbs;
      r[i-ds] = this[i]>>bs;
    }
    if(bs > 0) r[this.t-ds-1] |= (this.s&bm)<<cbs;
    r.t = this.t-ds;
    r.clamp();
  }

  // (protected) r = this - a
  function bnpSubTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]-a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c -= a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c -= a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c -= a.s;
    }
    r.s = (c<0)?-1:0;
    if(c < -1) r[i++] = this.DV+c;
    else if(c > 0) r[i++] = c;
    r.t = i;
    r.clamp();
  }

  // (protected) r = this * a, r != this,a (HAC 14.12)
  // "this" should be the larger one if appropriate.
  function bnpMultiplyTo(a,r) {
    var x = this.abs(), y = a.abs();
    var i = x.t;
    r.t = i+y.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < y.t; ++i) r[i+x.t] = x.am(0,y[i],r,i,0,x.t);
    r.s = 0;
    r.clamp();
    if(this.s != a.s) BigInteger.ZERO.subTo(r,r);
  }

  // (protected) r = this^2, r != this (HAC 14.16)
  function bnpSquareTo(r) {
    var x = this.abs();
    var i = r.t = 2*x.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < x.t-1; ++i) {
      var c = x.am(i,x[i],r,2*i,0,1);
      if((r[i+x.t]+=x.am(i+1,2*x[i],r,2*i+1,c,x.t-i-1)) >= x.DV) {
        r[i+x.t] -= x.DV;
        r[i+x.t+1] = 1;
      }
    }
    if(r.t > 0) r[r.t-1] += x.am(i,x[i],r,2*i,0,1);
    r.s = 0;
    r.clamp();
  }

  // (protected) divide this by m, quotient and remainder to q, r (HAC 14.20)
  // r != q, this != m.  q or r may be null.
  function bnpDivRemTo(m,q,r) {
    var pm = m.abs();
    if(pm.t <= 0) return;
    var pt = this.abs();
    if(pt.t < pm.t) {
      if(q != null) q.fromInt(0);
      if(r != null) this.copyTo(r);
      return;
    }
    if(r == null) r = nbi();
    var y = nbi(), ts = this.s, ms = m.s;
    var nsh = this.DB-nbits(pm[pm.t-1]);	// normalize modulus
    if(nsh > 0) { pm.lShiftTo(nsh,y); pt.lShiftTo(nsh,r); }
    else { pm.copyTo(y); pt.copyTo(r); }
    var ys = y.t;
    var y0 = y[ys-1];
    if(y0 == 0) return;
    var yt = y0*(1<<this.F1)+((ys>1)?y[ys-2]>>this.F2:0);
    var d1 = this.FV/yt, d2 = (1<<this.F1)/yt, e = 1<<this.F2;
    var i = r.t, j = i-ys, t = (q==null)?nbi():q;
    y.dlShiftTo(j,t);
    if(r.compareTo(t) >= 0) {
      r[r.t++] = 1;
      r.subTo(t,r);
    }
    BigInteger.ONE.dlShiftTo(ys,t);
    t.subTo(y,y);	// "negative" y so we can replace sub with am later
    while(y.t < ys) y[y.t++] = 0;
    while(--j >= 0) {
      // Estimate quotient digit
      var qd = (r[--i]==y0)?this.DM:Math.floor(r[i]*d1+(r[i-1]+e)*d2);
      if((r[i]+=y.am(0,qd,r,j,0,ys)) < qd) {	// Try it out
        y.dlShiftTo(j,t);
        r.subTo(t,r);
        while(r[i] < --qd) r.subTo(t,r);
      }
    }
    if(q != null) {
      r.drShiftTo(ys,q);
      if(ts != ms) BigInteger.ZERO.subTo(q,q);
    }
    r.t = ys;
    r.clamp();
    if(nsh > 0) r.rShiftTo(nsh,r);	// Denormalize remainder
    if(ts < 0) BigInteger.ZERO.subTo(r,r);
  }

  // (public) this mod a
  function bnMod(a) {
    var r = nbi();
    this.abs().divRemTo(a,null,r);
    if(this.s < 0 && r.compareTo(BigInteger.ZERO) > 0) a.subTo(r,r);
    return r;
  }

  // Modular reduction using "classic" algorithm
  function Classic(m) { this.m = m; }
  function cConvert(x) {
    if(x.s < 0 || x.compareTo(this.m) >= 0) return x.mod(this.m);
    else return x;
  }
  function cRevert(x) { return x; }
  function cReduce(x) { x.divRemTo(this.m,null,x); }
  function cMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }
  function cSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  Classic.prototype.convert = cConvert;
  Classic.prototype.revert = cRevert;
  Classic.prototype.reduce = cReduce;
  Classic.prototype.mulTo = cMulTo;
  Classic.prototype.sqrTo = cSqrTo;

  // (protected) return "-1/this % 2^DB"; useful for Mont. reduction
  // justification:
  //         xy == 1 (mod m)
  //         xy =  1+km
  //   xy(2-xy) = (1+km)(1-km)
  // x[y(2-xy)] = 1-k^2m^2
  // x[y(2-xy)] == 1 (mod m^2)
  // if y is 1/x mod m, then y(2-xy) is 1/x mod m^2
  // should reduce x and y(2-xy) by m^2 at each step to keep size bounded.
  // JS multiply "overflows" differently from C/C++, so care is needed here.
  function bnpInvDigit() {
    if(this.t < 1) return 0;
    var x = this[0];
    if((x&1) == 0) return 0;
    var y = x&3;		// y == 1/x mod 2^2
    y = (y*(2-(x&0xf)*y))&0xf;	// y == 1/x mod 2^4
    y = (y*(2-(x&0xff)*y))&0xff;	// y == 1/x mod 2^8
    y = (y*(2-(((x&0xffff)*y)&0xffff)))&0xffff;	// y == 1/x mod 2^16
    // last step - calculate inverse mod DV directly;
    // assumes 16 < DB <= 32 and assumes ability to handle 48-bit ints
    y = (y*(2-x*y%this.DV))%this.DV;		// y == 1/x mod 2^dbits
    // we really want the negative inverse, and -DV < y < DV
    return (y>0)?this.DV-y:-y;
  }

  // Montgomery reduction
  function Montgomery(m) {
    this.m = m;
    this.mp = m.invDigit();
    this.mpl = this.mp&0x7fff;
    this.mph = this.mp>>15;
    this.um = (1<<(m.DB-15))-1;
    this.mt2 = 2*m.t;
  }

  // xR mod m
  function montConvert(x) {
    var r = nbi();
    x.abs().dlShiftTo(this.m.t,r);
    r.divRemTo(this.m,null,r);
    if(x.s < 0 && r.compareTo(BigInteger.ZERO) > 0) this.m.subTo(r,r);
    return r;
  }

  // x/R mod m
  function montRevert(x) {
    var r = nbi();
    x.copyTo(r);
    this.reduce(r);
    return r;
  }

  // x = x/R mod m (HAC 14.32)
  function montReduce(x) {
    while(x.t <= this.mt2)	// pad x so am has enough room later
      x[x.t++] = 0;
    for(var i = 0; i < this.m.t; ++i) {
      // faster way of calculating u0 = x[i]*mp mod DV
      var j = x[i]&0x7fff;
      var u0 = (j*this.mpl+(((j*this.mph+(x[i]>>15)*this.mpl)&this.um)<<15))&x.DM;
      // use am to combine the multiply-shift-add into one call
      j = i+this.m.t;
      x[j] += this.m.am(0,u0,x,i,0,this.m.t);
      // propagate carry
      while(x[j] >= x.DV) { x[j] -= x.DV; x[++j]++; }
    }
    x.clamp();
    x.drShiftTo(this.m.t,x);
    if(x.compareTo(this.m) >= 0) x.subTo(this.m,x);
  }

  // r = "x^2/R mod m"; x != r
  function montSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  // r = "xy/R mod m"; x,y != r
  function montMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }

  Montgomery.prototype.convert = montConvert;
  Montgomery.prototype.revert = montRevert;
  Montgomery.prototype.reduce = montReduce;
  Montgomery.prototype.mulTo = montMulTo;
  Montgomery.prototype.sqrTo = montSqrTo;

  // (protected) true iff this is even
  function bnpIsEven() { return ((this.t>0)?(this[0]&1):this.s) == 0; }

  // (protected) this^e, e < 2^32, doing sqr and mul with "r" (HAC 14.79)
  function bnpExp(e,z) {
    if(e > 0xffffffff || e < 1) return BigInteger.ONE;
    var r = nbi(), r2 = nbi(), g = z.convert(this), i = nbits(e)-1;
    g.copyTo(r);
    while(--i >= 0) {
      z.sqrTo(r,r2);
      if((e&(1<<i)) > 0) z.mulTo(r2,g,r);
      else { var t = r; r = r2; r2 = t; }
    }
    return z.revert(r);
  }

  // (public) this^e % m, 0 <= e < 2^32
  function bnModPowInt(e,m) {
    var z;
    if(e < 256 || m.isEven()) z = new Classic(m); else z = new Montgomery(m);
    return this.exp(e,z);
  }

  // protected
  BigInteger.prototype.copyTo = bnpCopyTo;
  BigInteger.prototype.fromInt = bnpFromInt;
  BigInteger.prototype.fromString = bnpFromString;
  BigInteger.prototype.clamp = bnpClamp;
  BigInteger.prototype.dlShiftTo = bnpDLShiftTo;
  BigInteger.prototype.drShiftTo = bnpDRShiftTo;
  BigInteger.prototype.lShiftTo = bnpLShiftTo;
  BigInteger.prototype.rShiftTo = bnpRShiftTo;
  BigInteger.prototype.subTo = bnpSubTo;
  BigInteger.prototype.multiplyTo = bnpMultiplyTo;
  BigInteger.prototype.squareTo = bnpSquareTo;
  BigInteger.prototype.divRemTo = bnpDivRemTo;
  BigInteger.prototype.invDigit = bnpInvDigit;
  BigInteger.prototype.isEven = bnpIsEven;
  BigInteger.prototype.exp = bnpExp;

  // public
  BigInteger.prototype.toString = bnToString;
  BigInteger.prototype.negate = bnNegate;
  BigInteger.prototype.abs = bnAbs;
  BigInteger.prototype.compareTo = bnCompareTo;
  BigInteger.prototype.bitLength = bnBitLength;
  BigInteger.prototype.mod = bnMod;
  BigInteger.prototype.modPowInt = bnModPowInt;

  // "constants"
  BigInteger.ZERO = nbv(0);
  BigInteger.ONE = nbv(1);

  // jsbn2 stuff

  // (protected) convert from radix string
  function bnpFromRadix(s,b) {
    this.fromInt(0);
    if(b == null) b = 10;
    var cs = this.chunkSize(b);
    var d = Math.pow(b,cs), mi = false, j = 0, w = 0;
    for(var i = 0; i < s.length; ++i) {
      var x = intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-" && this.signum() == 0) mi = true;
        continue;
      }
      w = b*w+x;
      if(++j >= cs) {
        this.dMultiply(d);
        this.dAddOffset(w,0);
        j = 0;
        w = 0;
      }
    }
    if(j > 0) {
      this.dMultiply(Math.pow(b,j));
      this.dAddOffset(w,0);
    }
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) return x s.t. r^x < DV
  function bnpChunkSize(r) { return Math.floor(Math.LN2*this.DB/Math.log(r)); }

  // (public) 0 if this == 0, 1 if this > 0
  function bnSigNum() {
    if(this.s < 0) return -1;
    else if(this.t <= 0 || (this.t == 1 && this[0] <= 0)) return 0;
    else return 1;
  }

  // (protected) this *= n, this >= 0, 1 < n < DV
  function bnpDMultiply(n) {
    this[this.t] = this.am(0,n-1,this,0,0,this.t);
    ++this.t;
    this.clamp();
  }

  // (protected) this += n << w words, this >= 0
  function bnpDAddOffset(n,w) {
    if(n == 0) return;
    while(this.t <= w) this[this.t++] = 0;
    this[w] += n;
    while(this[w] >= this.DV) {
      this[w] -= this.DV;
      if(++w >= this.t) this[this.t++] = 0;
      ++this[w];
    }
  }

  // (protected) convert to radix string
  function bnpToRadix(b) {
    if(b == null) b = 10;
    if(this.signum() == 0 || b < 2 || b > 36) return "0";
    var cs = this.chunkSize(b);
    var a = Math.pow(b,cs);
    var d = nbv(a), y = nbi(), z = nbi(), r = "";
    this.divRemTo(d,y,z);
    while(y.signum() > 0) {
      r = (a+z.intValue()).toString(b).substr(1) + r;
      y.divRemTo(d,y,z);
    }
    return z.intValue().toString(b) + r;
  }

  // (public) return value as integer
  function bnIntValue() {
    if(this.s < 0) {
      if(this.t == 1) return this[0]-this.DV;
      else if(this.t == 0) return -1;
    }
    else if(this.t == 1) return this[0];
    else if(this.t == 0) return 0;
    // assumes 16 < DB < 32
    return ((this[1]&((1<<(32-this.DB))-1))<<this.DB)|this[0];
  }

  // (protected) r = this + a
  function bnpAddTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]+a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c += a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c += a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += a.s;
    }
    r.s = (c<0)?-1:0;
    if(c > 0) r[i++] = c;
    else if(c < -1) r[i++] = this.DV+c;
    r.t = i;
    r.clamp();
  }

  BigInteger.prototype.fromRadix = bnpFromRadix;
  BigInteger.prototype.chunkSize = bnpChunkSize;
  BigInteger.prototype.signum = bnSigNum;
  BigInteger.prototype.dMultiply = bnpDMultiply;
  BigInteger.prototype.dAddOffset = bnpDAddOffset;
  BigInteger.prototype.toRadix = bnpToRadix;
  BigInteger.prototype.intValue = bnIntValue;
  BigInteger.prototype.addTo = bnpAddTo;

  //======= end jsbn =======

  // Emscripten wrapper
  var Wrapper = {
    abs: function(l, h) {
      var x = new goog.math.Long(l, h);
      var ret;
      if (x.isNegative()) {
        ret = x.negate();
      } else {
        ret = x;
      }
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
    },
    ensureTemps: function() {
      if (Wrapper.ensuredTemps) return;
      Wrapper.ensuredTemps = true;
      Wrapper.two32 = new BigInteger();
      Wrapper.two32.fromString('4294967296', 10);
      Wrapper.two64 = new BigInteger();
      Wrapper.two64.fromString('18446744073709551616', 10);
      Wrapper.temp1 = new BigInteger();
      Wrapper.temp2 = new BigInteger();
    },
    lh2bignum: function(l, h) {
      var a = new BigInteger();
      a.fromString(h.toString(), 10);
      var b = new BigInteger();
      a.multiplyTo(Wrapper.two32, b);
      var c = new BigInteger();
      c.fromString(l.toString(), 10);
      var d = new BigInteger();
      c.addTo(b, d);
      return d;
    },
    stringify: function(l, h, unsigned) {
      var ret = new goog.math.Long(l, h).toString();
      if (unsigned && ret[0] == '-') {
        // unsign slowly using jsbn bignums
        Wrapper.ensureTemps();
        var bignum = new BigInteger();
        bignum.fromString(ret, 10);
        ret = new BigInteger();
        Wrapper.two64.addTo(bignum, ret);
        ret = ret.toString(10);
      }
      return ret;
    },
    fromString: function(str, base, min, max, unsigned) {
      Wrapper.ensureTemps();
      var bignum = new BigInteger();
      bignum.fromString(str, base);
      var bigmin = new BigInteger();
      bigmin.fromString(min, 10);
      var bigmax = new BigInteger();
      bigmax.fromString(max, 10);
      if (unsigned && bignum.compareTo(BigInteger.ZERO) < 0) {
        var temp = new BigInteger();
        bignum.addTo(Wrapper.two64, temp);
        bignum = temp;
      }
      var error = false;
      if (bignum.compareTo(bigmin) < 0) {
        bignum = bigmin;
        error = true;
      } else if (bignum.compareTo(bigmax) > 0) {
        bignum = bigmax;
        error = true;
      }
      var ret = goog.math.Long.fromString(bignum.toString()); // min-max checks should have clamped this to a range goog.math.Long can handle well
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
      if (error) throw 'range error';
    }
  };
  return Wrapper;
})();

//======= end closure i64 code =======



// === Auto-generated postamble setup entry stuff ===


function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun']) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString(Module['thisProgram']), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);

  initialStackTop = STACKTOP;

  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    exit(ret, /* implicit = */ true);
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    Module.printErr('run() called, but dependencies remain, so not running');
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    if (ABORT) return; 

    ensureInitRuntime();

    preMain();

    if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
      Module.printErr('pre-main prep time: ' + (Date.now() - preloadStartTime) + ' ms');
    }

    if (Module['onRuntimeInitialized']) Module['onRuntimeInitialized']();

    if (Module['_main'] && shouldRunNow) Module['callMain'](args);

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status, implicit) {
  if (implicit && Module['noExitRuntime']) {
    Module.printErr('exit(' + status + ') implicitly called by end of main(), but noExitRuntime, so not exiting the runtime (you can use emscripten_force_exit, if you want to force a true shutdown)');
    return;
  }

  if (Module['noExitRuntime']) {
    Module.printErr('exit(' + status + ') called, but noExitRuntime, so halting execution but not exiting the runtime or preventing further async execution (you can use emscripten_force_exit, if you want to force a true shutdown)');
  } else {

    ABORT = true;
    EXITSTATUS = status;
    STACKTOP = initialStackTop;

    exitRuntime();

    if (Module['onExit']) Module['onExit'](status);
  }

  if (ENVIRONMENT_IS_NODE) {
    // Work around a node.js bug where stdout buffer is not flushed at process exit:
    // Instead of process.exit() directly, wait for stdout flush event.
    // See https://github.com/joyent/node/issues/1669 and https://github.com/kripken/emscripten/issues/2582
    // Workaround is based on https://github.com/RReverser/acorn/commit/50ab143cecc9ed71a2d66f78b4aec3bb2e9844f6
    process['stdout']['once']('drain', function () {
      process['exit'](status);
    });
    console.log(' '); // Make sure to print something to force the drain event to occur, in case the stdout buffer was empty.
    // Work around another node bug where sometimes 'drain' is never fired - make another effort
    // to emit the exit status, after a significant delay (if node hasn't fired drain by then, give up)
    setTimeout(function() {
      process['exit'](status);
    }, 500);
  } else
  if (ENVIRONMENT_IS_SHELL && typeof quit === 'function') {
    quit(status);
  }
  // if we reach here, we must throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

var abortDecorators = [];

function abort(what) {
  if (what !== undefined) {
    Module.print(what);
    Module.printErr(what);
    what = JSON.stringify(what)
  } else {
    what = '';
  }

  ABORT = true;
  EXITSTATUS = 1;

  var extra = '';

  var output = 'abort(' + what + ') at ' + stackTrace() + extra;
  if (abortDecorators) {
    abortDecorators.forEach(function(decorator) {
      output = decorator(output, what);
    });
  }
  throw output;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}


run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



// TypedData gets converted to ArrayBuffer when called from Dart.
function setTypedData(arrayBuffer, offset) {
  var bytes = new Uint8Array(arrayBuffer);
  Module['HEAPU8'].set(bytes, offset);
}
Module['setTypedData'] = setTypedData;

function getTypedData(ptr, nBytes) {
  var bufView = new Uint8Array(Module['buffer'], ptr, nBytes);
  return bufView;
}
Module['getTypedData'] = getTypedData;

if (this.document === undefined) {
  // Web Worker mode
  
  // Incoming message = [ handle, [ command, args... ] ]
  // Outgoing message = [ handle, output ]
  // Outgoing message = [ handle, [ "error", error ] ]

  // handle is a random Number (assigned by the host) to match 
  // request and response.

  var s = 'number';
  var argTypes13 = [s, s, s, s, s, s, s, s, s, s, s, s, s];

  function copy_buf(/*TypedData*/ dat) {
    var ptr = Module['_malloc'](dat.length*dat.BYTES_PER_ELEMENT);
    var bytes = new Uint8Array(dat.buffer);
    Module['HEAPU8'].set(bytes, ptr);
    return ptr;
  }

  this.onmessage = function (event) {
    if (!Array.isArray(event["data"])) {
      postMessage("internal error: expected array");
      return;
    }

    var handle = event["data"][0];
    var payload = event["data"][1];
    if (handle == undefined && payload == undefined) {
      postMessage("internal error: no payload");
      return;
    }

    var iopt, n, nnz, nrhs, p_values, p_rowind, p_colptr, p_b, ldb,
      pp_factors, permc_spec, trans, p_info;
    var result;
    
    var i32 = Int32Array.BYTES_PER_ELEMENT; // pointers are 32-bit
    
    var p_info = Module['_malloc'](i32);
    Module['setValue'](p_info, 0, 'i32');

    var command = payload[0];
    switch (command) {
      case "dlinsol.factor":
      case "zlinsol.factor":
        var _values = payload[1];
        var _rowind = payload[2];
        var _colptr = payload[3];
        permc_spec = payload[4];

        var values = new Float64Array(_values);
        var rowind = new Int32Array(_rowind);
        var colptr = new Int32Array(_colptr);

        iopt = 1;
        n = colptr.length - 1;
        nnz = values.length;
        nrhs = 0;

        p_values = copy_buf(values);
        p_rowind = copy_buf(rowind);
        p_colptr = copy_buf(colptr);

        p_b = 0; // NULL
        ldb = 0;
        trans = 0;

        pp_factors = Module['_malloc'](i32);
        break;
      case "dlinsol.solve":
      case "zlinsol.solve":
        pp_factors = payload[1];
        n = payload[2];
        nrhs = payload[3];
        var _rhs = payload[4];
        ldb = payload[5];
        trans = payload[6];

        rhs = new Float64Array(_rhs);

        iopt = 2;
        nnz = 0;
        p_values = 0;
        p_rowind = 0;
        p_colptr = 0;

        p_b = copy_buf(rhs);
        if (ldb == undefined) {
          ldb = n;
        }

        permc_spec = 0;
        break;
      case "dlinsol.free":
      case "zlinsol.free":
        pp_factors = payload[1];

        iopt = 3;
        n = 0;
        nnz = 0;
        nrhs = 0;
        p_values = 0;
        p_rowind = 0;
        p_colptr = 0;
        p_b = 0;
        ldb = 0;
        permc_spec = 0;
        trans = 0;
        break;
      default:
        result = ["error", "invalid command: " + command];
    }

    switch (command) {
      case "dlinsol.factor":
      case "dlinsol.solve":
      case "dlinsol.free":
    	Module.ccall('dlinsol', null, argTypes13,
          [iopt, n, nnz, nrhs, p_values, p_rowind, p_colptr, p_b, ldb,
           pp_factors, permc_spec, trans, p_info]);
        break;
      case "zlinsol.factor":
      case "zlinsol.solve":
      case "zlinsol.free":
    	Module.ccall('zlinsol', null, argTypes13,
          [iopt, n, nnz, nrhs, p_values, p_rowind, p_colptr, p_b, ldb,
           pp_factors, permc_spec, trans, p_info]);
        break;
    }

    switch (command) {
      case "dlinsol.factor":
      case "zlinsol.factor":
        Module['_free'](p_values);
        Module['_free'](p_rowind);
        Module['_free'](p_colptr);
        result = pp_factors;
        break;
      case "dlinsol.solve":
      case "zlinsol.solve":
        var bview = new Float64Array(Module['buffer'], p_b, rhs.length);
        rhs.set(bview);
        result = rhs;
        Module['_free'](p_b);
        break;
      case "dlinsol.free":
      case "zlinsol.free":
        result = null;
    }

    var info = Module['getValue'](p_info, 'i32');
    Module['_free'](p_info);
    if (info != 0) {
      result = ["error", info];
    }

    var msg = [handle, result];
    this.postMessage(msg);
  }
}

